
/*********************************************************************
 *  made by  : Kees Lemmens
 *  version  : 0.1
 *  date     : May 1999
 * 
 *  class name  : ViewCanvas
 *  function    : Contains all routines to show the mesh in the canvas
 * 
 *  modified : July 1999 KL
 * Complete rewrite of scrolling, scaling and zooming routines.
 * 
 * modified : April 2000 KL 
 * Changed paint() routine into double buffering for performance reasons
 * 
 *********************************************************************/

import java.io.File;
import java.io.IOException;
import java.io.StreamTokenizer;
import java.awt.*;

final class PlotItem
{
   Vertex start;
   JSepRecord   record;
   Scaler s;
   ColorTables  coltab;
   RecordReader reader;
   
   public PlotItem(ColorTables coltab,RecordReader reader,Scaler s)
   {
      this.coltab = coltab;
      this.reader = reader;
      this.s      = s;
      start = new Vertex();
   }
   
   public void parseMove(JSepRecord record)
     throws IOException
   {
      start.x = record.xcrd;
      start.y = record.ycrd;
   }
   
   public void drawLine(Graphics g, JSepRecord nrecord)
   {
      record = nrecord;
      int x1 = s.xVal(start.x);
      int y1 = s.yVal(start.y);
      int x2 = s.xVal(record.xcrd);
      int y2 = s.yVal(record.ycrd);
      
      if(x1 < -32000 || y1 < -32000 ||
	 x2 > 32000 || y2 > 32000) 
	return;
      coltab.setColor(g,record.nchar);
      g.drawLine(x1,y1,x2,y2);
//      g.drawLine(s.xVal(start.x),
//	s.yVal(start.y),
//	s.xVal(record.xcrd),
//	s.yVal(record.ycrd));
      start.x = record.xcrd;
      start.y = record.ycrd;
   }
   
   public String toString()
   {
      return ("Line :" + "start= " + start + ", end= ("
	      + record.xcrd + "," + record.ycrd
	      + "), colornr=" + record.nchar);
   }
}

final class TextItem
{
   float oldheight;
   JSepRecord record;
   float angle, newheight;
   String string;
   int color,fontSize;
   ColorTables  coltab;
   RecordReader reader;
   Scaler s;
   static String fontName="Dialog";
   static int fontStyle=Font.PLAIN;
   
   public TextItem(ColorTables ncoltab,RecordReader nreader,Scaler s)
   {
      coltab    = ncoltab;
      reader    = nreader;
      this.s    = s;
      resetHeight(); // force a new font 
   }
   
   public void draw(Graphics g, JSepRecord nrecord)
     throws IOException
   {
      record = nrecord; // remember for toString
      
      angle     = record.angle;
      newheight = record.height;
      color     = record.nchar;
      if(color < 0) color += 256; // fix for signed byte
      color    /= 100;
      
      string = reader.readString();
      
      coltab.setColor(g,color);
      if(newheight != oldheight) // saves time if font does not change often
      {
	 if(newheight < 0.4)
	   fontSize = 10;
	 else if(newheight < 0.8) 
	   fontSize = 12;
	 else
	   fontSize = 18;
	 g.setFont(new Font(fontName,fontStyle,fontSize));
	 oldheight = newheight;
	 
	 if(Debug.LEVEL>=2) Debug.print("Textheight for '" + 
					string + "' : " + newheight);
      }
      g.drawString(string,s.xVal(record.xcrd),
		   s.yVal(record.ycrd));
   }
   
   public void resetHeight()
   {
      oldheight = -1;
   }
   
   public String toString()
   {
      return ("Textstring :" + string);
   }
}

final class PolygonItem
{
   int nrVertices,color;
   Polygon pol;
   ColorTables  coltab;
   RecordReader reader;
   Scaler s;
   
   public PolygonItem(ColorTables ncoltab,RecordReader nreader,Scaler ns)
   {
      coltab = ncoltab;
      reader = nreader;
      s      = ns;
   }
   
   public void draw(Graphics g, JSepRecord nrecord)
     throws IOException
   {
      int xv[],yv[];
      Vertex[] vertices;

      color      = (int)nrecord.xcrd;
      nrVertices =      nrecord.nchar;

      xv = new int[nrVertices];
      yv = new int[nrVertices];

//    vertices = new Vertex[nrVertices];
      vertices = reader.readVertices(nrVertices);
      
      for(int n=0;n < nrVertices;n++)
      {  
	 // vertices[n] = reader.readVertex();
	 xv[n] = s.xVal(vertices[n].x);
	 yv[n] = s.yVal(vertices[n].y);
      }
      pol = new Polygon(xv,yv,nrVertices);
      
      coltab.setColor(g,color);
      g.fillPolygon(pol); 
   }   
   
   public String toString()
   {
      return ("Polygon nr of vertices=" + nrVertices);
   }
}

final class ViewCanvas extends Canvas
{
   JSepRecord record;
   JsepView tool;
   RecordReader reader;
   ScrollCanvas viewport;
   ViewMenu mview;
   Scaler s;
   int xOffset   =  40;
   int yOffset   =  40;
   PlotItem    plot;
   TextItem    text;
   PolygonItem poly;
   Vertex  start,end;
   ColorTables coltab;
   PerfTimer   timer;
   boolean initialized = false;
   
   public static final int
     START   = 1,
     PLOT    = 2,
     MOVE    = 3,
     EMPTY   = 4,
     CLOSE   = 5,
     TEXT    = 6,
     POLYGON = 7,
     SYMBOL  = 8;
   
   public ViewCanvas(JsepView tool,Scaler scaler) // constructor
   {
      super();
      this.tool = tool;
      this.reader = (new RecordReader()).setReaderType(tool.recordType);
      this.mview  = tool.header.mview; // need View Menu states
      this.s      = scaler;
      
      coltab = new ColorTables();
      
      coltab.BuildColorTables();
      setBackground(Color.white);
      
      plot   = new PlotItem(coltab,reader,s);
      text   = new TextItem(coltab,reader,s);
      poly   = new PolygonItem(coltab,reader,s);
      start  = new Vertex(0.0,0.0);
      end    = new Vertex();

      s.setOriginOffset(xOffset,yOffset);
   }
   
   public void setNormalTable()
   {
      coltab.fixedTable = false;
      coltab.colorset   = 0;
   }
   
   public void setFixedTable(int nr)
   {
      coltab.fixedTable = true;
      coltab.colorset   = nr;
   }
   
   public int getFixedTable()
   {
      if(coltab.fixedTable == true)
	return coltab.colorset;
      else return -1;
   }
   
   public void openPlotSelector()
   {
      FileDialog sel = new FileDialog(tool.getParentFrame(tool));
      
      sel.show();
      if(sel.getFile() != null)
      {
	 String plotfile = sel.getDirectory() +
	   File.separator + sel.getFile();
	 // platform independent / (thanks Bill for inventing \ !) 
	 tool.header.setCurrentName(plotfile);
	 tool.current=0;
	 tool.filelist[0] = plotfile; // filelist.length == 0 ? : fixed
      }
      this.repaint();
   }
   
   private void initPlot(JSepRecord record)
   {
      
      start.x=start.y=0;
      end.x = ((record.xcrd == 1) ? 21 : 29) + 4; // A3 or A4
      end.y =   record.ycrd + 2;
      
      s.setDataRange(start,end);
      if(initialized == false)
      {
	 initialized = true;
	 s.setScreenSize(getSize());
      }
   }

   // This uses double buffering to avoid very slow drawing of fonts that
   // occurred since version JDK 1.2. It draws on an offscreen Image and
   // copies the whole image to the screen afterwards.
   
   private Dimension od;
   private Graphics og;
   private Image osImage;
   
   public void paint(Graphics g) 
   {
      String name = tool.getCurrent();

      timer.setStartTime(); // no instance : static method
      
      if(name == null)
	return;

      Dimension d = getSize();
      if(osImage == null || d.width != od.width || d.height != od.height)
      {
	 osImage = createImage(d.width, d.height);
	 od = d;
	 og = osImage.getGraphics();
      }
      og.clearRect(0, 0, d.width, d.height);

      try
      {
	 if(Debug.LEVEL>=1)
	   Debug.print("Reading tokens from file " + name);
	 
	 reader.openFile(name);
	 if(reader.status < 0) 
	   return;
	 text.resetHeight(); // force a new font

	 scan :
	 while(true)
	 {
	    record = reader.readRecord();
	    switch(reader.status)
	    {  
	     case StreamTokenizer.TT_EOF:
	       break scan;
	     case StreamTokenizer.TT_WORD:
	       System.err.println("Error: invalid token ! ");
	       break;
	       
	     default:  //  StreamTokenizer.TT_NUMBER:
	       if(Debug.LEVEL>=2) Debug.print("Token : " + record.type);
	       if(Debug.LEVEL>=2) Debug.print(record.toString());
	       switch(record.type)
	       {
		case START   :
		  initPlot(record);
		  break;
		case MOVE    :
		  plot.parseMove(record);
		  break;
		case PLOT    :
		  plot.drawLine(og, record);
		  break;
		case EMPTY   :
		case SYMBOL  :
		  break;
		case CLOSE   :
		  break scan;
		case TEXT    :
		  text.draw(og, record);
		  break;
		case POLYGON :
		  poly.draw(og, record);
		  break;
	       }
	    }
	 }
	 reader.close();
	 drawAxes(og,start,end); // put them on top of the rest !
      }
      catch (NumberFormatException n)
      {
	 System.err.println("Badly formatted number in plot :" + n);
      }
      catch (IOException i)
      {
	 System.err.println("IO Exception : " + i);
      }
      g.drawImage(osImage, 0, 0, null);
      timer.showElapsedTime("paint");
   }
   
   private void drawAxes(Graphics g,Vertex start, Vertex end)
   {
      Color linecolor = Color.lightGray;
      Color textcolor = Color.gray;
      double x,y;
      
      g.setColor(linecolor);
      g.drawLine(s.xVal(-xOffset), s.yVal(0), s.xVal(end.x), s.yVal(0));
      
      for(y=start.y; y<=end.y; y+=(end.y - start.y)/5)
      {
	 g.setColor(textcolor);
	 g.drawString(String.valueOf((int)y), s.xVal(0) - 30, s.yVal(y) - 1);
      }
      
      g.setColor(linecolor);
      g.drawLine(s.xVal(0), s.yVal(start.y), s.xVal(0), s.yVal(end.y));
      
      for(x=start.x; x < end.x; x+=(end.x - start.x)/5)
      {
	 g.setColor(textcolor);
	 g.drawString(String.valueOf((int)x),s.xVal(x) - 20, s.yVal(0) + 10);
      }
   }
}
