
/*********************************************************************
 *  made by : Kees Lemmens
 *  class name  : ToolCanvas
 * 
 * Note that the floating point coordinates are assumed to have the origin
 * at the lowerleft corner of the canvas, while the integer coordinates on
 * the screen all have the origin at the topleft corner ! 
 *
 * And believe me : writing a piece of software like this (including
 * zooming and scrolling, data offsets, screen offsets and reversal of the
 * y-axis) is not as easy as it might seem ... (at least for me ;-)
 * 
 *********************************************************************/

import java.awt.*;

final class Scaler
{
   static final float inset = 0.95f;      // keep some space (use Insets ?)
   static final float zoomStep = 1.5f;    // for zoomIn and zoomOut
   static final int CLIPMIN     = -32767; // minimum for clipping area
   static final int CLIPMAX     =  32767; // maximum for clipping area   
   
   ScrollCanvas viewport;
   // data :
   Vertex min,max;
   double dw,dh;
   // screen :
   Point soff;
   Dimension sdim;
   Vertex scale;
   // zoomarea :
   Vertex zmf;
   Point zmul;
   Point zoff;
   
   Point toff; // precomputed total offset: saves time
   
   boolean  fixedRatio;
   
   public Scaler(ScrollCanvas viewport) // constructor
   {
      this.viewport = viewport;
      min   = new Vertex(0,0);
      max   = new Vertex(1,1);
      soff  = new Point(0,0);
      sdim  = new Dimension(300,300);
      scale = new Vertex(1,1);
      zmf   = new Vertex(1,1);
      zmul  = new Point();
      zoff  = new Point(0,0);
      toff  = new Point();
   }
   
   public void setDataRange(Vertex min,Vertex max)
   {
      this.min.x = min.x;      this.max.x = max.x;
      this.min.y = min.y;      this.max.y = max.y;
      setNewScale();
   }
   
   public void setScreenSize(Dimension dim)
   {
      sdim.width  = dim.width;
      sdim.height = dim.height;
      setNewScale();
      resetZoom();
   }
   
   public void setOriginOffset(int sx,int sy)
   {
      soff.x = sx; soff.y = sy;
      // x: added      , shift to right
      // y: subtracted , shift upwards !!
      setNewScale();
      resetZoom();
   }
   
   public void setParameters(Vertex min,Vertex max,Dimension dim)
   {
      setDataRange(min,max);
      setScreenSize(dim);
      setNewScale();
   }
   
   private void setNewScale()
   {
      scale.x = (sdim.width - soff.x)/(max.x - min.x) * inset;
      scale.y = (sdim.height- soff.y)/(max.y - min.y) * inset;
      
      checkFixedAspectRatio();
      
      if(Debug.LEVEL >= 1)
	Debug.print("Scalefactor :" + scale + "\n" +
		    "ScreenOffset:" + soff);
   }
   
   public boolean getFixedRatioState()
   {
      return fixedRatio;
   }
   
   public void toggleFixedRatio()
   {
      fixedRatio = !fixedRatio;
      checkFixedAspectRatio();
   }
   
   private void checkFixedAspectRatio()      
   {
      if(fixedRatio)
      {
	 // Force a fixed aspect ratio :
	 if( scale.x > scale.y ) scale.x = scale.y;
	 else                    scale.y = scale.x;
	 if( zmf.x > zmf.y )     zmf.x   = zmf.y;
	 else                    zmf.y   = zmf.x;

	 setZoomOffset(zoff.x,zoff.y);
	 viewport.setVisibleAmount(zmf.x,zmf.y);
      }
   }
   
   public void setZoomOffset(int zx,int zy)
   {
      zoff.x = zx; // remember for stacked zooms
      zoff.y = zy;
      // precomputed total offset makes coordinate conversion faster
      toff.x = (int)(-min.x * scale.x + soff.x - zoff.x);
      toff.y = (int)( min.y * scale.y - soff.y - zoff.y); 

      if(Debug.LEVEL >= 2) 
	Debug.print("Total offset :" + toff.x + "," + toff.y);
   }
   
   public void resetZoom()
   {
      zmf.x  =  zmf.y = 1;
      zoff.x = zoff.y = 0;

      setZoomOffset(zoff.x,zoff.y);
      viewport.setVisibleAmount(zmf.x,zmf.y);
   }
   
   public void setZoom(Point zll,Point zur)
   {
      // Note: zll and zur are the bounding box coordinates of the zoom area

      // Be careful to use OLD zoomfactor/offset to scale new bounding box !

      zoff.x += (int)(zll.x/zmf.x);
      zoff.y += (int)(zur.y/zmf.y);

      zmf.x = (sdim.width  - soff.x)/(float)(zur.x - zll.x) * zmf.x;
      zmf.y = (sdim.height - soff.y)/(float)(zll.y - zur.y) * zmf.y;
      
      checkFixedAspectRatio();
      
      viewport.setVisibleAmount(zmf.x,zmf.y);
      viewport.setScrollPosition(zoff.x,zoff.y);
      setZoomOffset(zoff.x,zoff.y);
      
      if(Debug.LEVEL >= 1) 
	Debug.print("Zoomfactor :" + zmf + "\n" +
		    "Zoomoffset :" + zoff.x + "," + zoff.y);
   }
   
   public void zoomIn()
   {
      zmf.x *= zoomStep;
      zmf.y *= zoomStep;
      
      viewport.setVisibleAmount(zmf.x,zmf.y);

      if(Debug.LEVEL >= 1) 
	Debug.print("Zoomfactor :" + zmf + "\n" +
		    "Zoomoffset :" + zoff.x + "," + zoff.y);
   }
   
   public void zoomOut()
   {
      if(zmf.x <= 1 && zmf.y <= 1)
      {
	 resetZoom();
	 return;
      }
      
      zmf.x /= zoomStep;
      zmf.y /= zoomStep;

      viewport.setVisibleAmount(zmf.x,zmf.y);
      
      if(Debug.LEVEL >= 1) 
	Debug.print("Zoomfactor :" + zmf + "\n" +
		    "Zoomoffset :" + zoff.x + "," + zoff.y);
   }
   
   public int xVal(double x)
   {
      return (int)((              x*scale.x + toff.x)*zmf.x);
   }
   
   public int yVal(double y)
   {
      return (int)((sdim.height - y*scale.y + toff.y)*zmf.y);
   }
}
