      recursive subroutine chsort2darr1 ( iarray, narray, ndim, iseq )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 01-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Sort part of a ndim-d array in increasing sequence recursively
!     First first entry, next second entry (if first are equal) etc
! **********************************************************************
!
!                       KEYWORDS
!
!     sort
!     2d
!     array
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer narray, ndim, iarray(narray,ndim), iseq

!     iarray        i/o   Two-dimensional array with integer values to be sorted
!                         At output the array is sorted in increasing
!                         sequence, first for the components (.,1)
!                         and in case these are equal for (.,2) etc
!     iseq           i    Present component to be sorted (1<=iseq<=ndim)
!                         If iseq>1, the components 1... iseq-1 are equal
!     narray         i    First dimension of iarray
!     ndim           i    Second dimension of iarray
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iperm(narray), ihelp(narray), i, icount, jcount, ientry, &
              error
      integer, allocatable :: icopy(:,:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     icopy          Number of volume to be copied
!     icount         Counter
!     ientry         value of entry in iarray
!     ihelp          integer work array for copying
!     iperm          used to store permutation array
!     jcount         Counter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT         Sort integer array for increasing sequence
!     CHSORT2DARR1   Sort part of a ndim-d array in increasing sequence
!                    recursively
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ2        Print an integer two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'chsort2darr1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from chsort2darr1'
         call prinin2 ( iarray, narray, ndim, 'iarray/in' )
         write(irefwr,1) 'iseq, ndim', iseq, ndim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Sort array, result in iperm

      call chsort ( iarray(1,iseq), iperm, narray )
      if ( debug ) call prinin ( iperm, narray, 'iperm' )

!     --- Rearrange array iarray

      do i = 1, ndim
         ihelp(1:narray) = iarray(iperm(1:narray),i)
         iarray(:,i) = ihelp
      end do  ! i = 1, ndim
      if ( debug ) call prinin2 ( iarray, narray, ndim, 'iarray/1' )

!     --- Check for equal iseq entry

      icount = 1
      do while ( icount<=narray .and. iseq<ndim )

         jcount = icount
         ientry = iarray(icount,iseq)  ! first value of entry
         if ( debug ) &
            write(irefwr,1) 'icount, jcount, ientry, narray', &
                             icount, jcount, ientry, narray

         do while ( jcount<narray )

!        --- Next entry has same first entry

            if ( iarray(jcount+1,iseq)==ientry ) then
               jcount = jcount+1
            else
               exit
            end if  ! ( intarray(jcount+1,iseq)==ientry )

         end do  ! while ( jcount<narray )

         if ( jcount>icount ) then

!        --- jcount>icount, hence the first entries icount to jcount are equal
!            Sort with respect to next entry

            allocate ( icopy(jcount-icount+1,ndim), stat = error )
            if ( error/=0 ) &
               call eralloc ( error, (jcount-icount+1)*ndim, 'icopy' )
            if ( ierror/=0 ) go to 1000

            icopy(1:jcount-icount+1,:) = iarray(icount:jcount,:)
            call chsort2darr1 ( icopy, jcount-icount+1, ndim, &
                                iseq+1 )
            iarray(icount:jcount,:) = icopy(1:jcount-icount+1,:)

            deallocate ( icopy, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'icopy' )
            if ( ierror/=0 ) go to 1000

         end if  ! ( jcount>icount )

         icount = jcount+1  ! raise icount

      end do  ! while ( icount<=narray .and. iseq<ndim )

1000  call erclos ( 'chsort2darr1' )
      if ( debug ) then

!     --- Debug information

         call prinin2 ( iarray, narray, ndim, 'iarray/out' )
         write(irefwr,*) 'End chsort2darr1'

      end if  ! ( debug )

      end
