      subroutine er0063 ( indprf, indprh, indprp, kprobf, kprobh, &
                          kprobp, npoint, npboun, nbound, nusol, nphys, &
                          idegfd, iwork, igathr, nusout, nbnout, kmesh, &
                          kprob, kmeshc, iskip, ioutpoints )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  2.2  date 09-03-2013 Correction declaration
!        version  2.1  date 20-07-2001 Correction iskip=0 and idegfd=0
!        version  2.0  date 02-01-2000 Extra parameters iskip, ioutpoints
!                                      skip element groups
!
!   copyright (c) 1989-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine is used to fill the arrays iwork and igathr with all
!     positions in the solution array that correspond to degree of freedom
!     idegfd (per point)
!     Array IWORK is filled as follows:
!        IWORK(i)>0, gives start position of unknown in point i
!        IWORK(i) = 0, unknown is not available in point i
!        IWORK(i)<0, point is second periodical point
!                      -IWORK(i) refers to first point
!     Array IGATHR contains the same information as IWORK but compressed
!     such that all non-positive information is deleted. Hence:
!     periodical points are counted for only once, and if nbound>0, the
!     essential boundary conditions are placed in the rear
!     In combination with subroutines ER0061 and ER0062 this subroutine is
!     used to store the unknowns of a large vector into a smaller one for
!     computational purposes and to place it back into the large vector
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     gather
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer indprf, indprh, indprp, npoint, nusol, nbound, npboun, &
              nphys, idegfd, kprobf(npoint+1), kprobh(nusol), &
              kprobp(npoint,nphys), iwork(npoint), igathr(*), &
              nusout, nbnout, kmesh(*), kprob(*), kmeshc(*), iskip, &
              ioutpoints(*)

!     idegfd         i   Actual degree of freedom per point
!     igathr         o   Integer array containing the indices of the positions
!                        in usol corresponding to IDEGFD
!                        If NBOUND>0 the prescribed degrees of freedom are
!                        positioned at the rear. Periodical boundary conditions
!                        are counted for only once
!     indprf         i   Indication if KPROB part f has been filled (>0) or
!                        not (=0)
!     indprh         i   Indication if KPROB part h has been filled (>0) or
!                        not (=0)
!     indprp         i   Indication if KPROB part p has been filled (>0) or
!                        not (=0)
!     ioutpoints     i   Output array of length npoint
!                        If ioutpoints(i)=0, node i must be skipped, else
!                        ioutpoints(i)>0, node i must not be skipped
!     iskip          i   If>0, element groups must be skipped
!     iwork          o   Work array of length NPOINT to store the degrees of
!                        freedom per point.  See description
!     kmesh          i   Standard SEPRAN array containing information of the
!                        mesh
!     kmeshc         i   SEPRAN array KMESH part c, contains the node numbers of
!                        the elements
!     kprob          i   Standard SEPRAN array containing information of the
!                        problem definition. See the Programmer's Guide 13.2 for
!                        a complete description
!     kprobf         i   SEPRAN array KPROB part f containing the number of
!                        unkowns per point
!     kprobh         i   SEPRAN array KPROB part h containing the positions of
!                        the renumbered degrees of freedom in USOL
!     kprobp         i   SEPRAN array KPROB part p containing the positions of
!                        the unkowns per point
!     nbnout         i   Number of prescribed boundary conditions in IGATHR
!     nbound         i   Number of prescribed boundary conditions
!     npboun         i   Number of periodical boundary conditions
!     nphys          i   Number of physical unknowns, or maximal degrees of
!                        freedom per point
!     npoint         i   Number of nodal points
!     nusol          i   Number of degrees of freedom in a solution vector
!     nusout         i   Number of degrees of freedom in IGATHR
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nunkp, inpelm, nelgrp, kelma, kelmg, ikprbc, ikelmc, &
              ielgrp, itype, inelem, iielem, inodp1, inodp2, ih, j, &
              jdegfd
      logical presen

!     i        Counting variable
!     ielgrp   Element group number
!     ih       help variable to store a constant
!     iielem   Relative element number in element group
!     ikelmc   Address in KMESH part c
!     ikprbc   Starting address of KPROB part c
!     inelem   Number of elements in element group ielgrp
!     inodp1   First node in element for periodical boundary conditions
!     inodp2   Second node in element for periodical boundary conditions
!     inpelm   Number of nodes in the element
!     itype    Type number of element
!     j        Counting variable
!     jdegfd   Maximum (1, idegfd)
!     kelma    Starting address of KMESH part a
!     kelmg    Starting address of KMESH part g
!     nelgrp   Number of element groups
!     nunkp    number of unknowns in a point
!     presen   Check if the degree of freedom is present in at least one point
!              (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     218   Degree of freedom does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
!     First fill iwork with the positions of the unknown in all nodes
!     If ( periodical boundary conditions present )
!        Set second point of set of periodical boundary conditions negative
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
!     --- Fill array IWORK, except for periodical boundary conditions

      jdegfd  = max(idegfd,1)
      if ( iskip>0 .and. idegfd==0 ) then

!     --- idegfd = 0, only possible in combination with skip element group

         nusout = 0
         nbnout = 0
         if ( indprp>0 ) then

!        --- indprp>0, i.e. copy iwork from KPROBP

            do i = 1, npoint
               if ( ioutpoints(i)>0 ) then
                  do j = 1, nphys
                     ih = kprobp(i,j)
                     if ( ih>0 ) then
                        if ( nbound>0 ) then
                           if ( kprobh(ih)>nusol-nbound-npboun .and. &
                             nbound>0 ) then
                              nbnout = nbnout + 1
                              igathr(nusol+1-nbnout) = ih
                           else
                              nusout = nusout + 1
                              igathr(nusout) = ih
                           end if
                        else
                           nusout = nusout + 1
                           igathr(nusout) = ih
                        end if
                     end if
                  end do
               end if
            end do

         else if ( indprf>0 ) then

!        --- indprf>0, i.e. no renumbering but the number of degrees of
!                             freedom is variable

            do i = 1, npoint
               if ( ioutpoints(i)>0 ) then
                  nunkp = kprobf(i+1) - kprobf(i)
                  do j = 1, nunkp
                     ih = kprobf(i)+j
                     if ( ih>0 ) then
                        if ( nbound>0 ) then
                           if ( kprobh(ih)>nusol-nbound-npboun .and. &
                             nbound>0 ) then
                              nbnout = nbnout + 1
                              igathr(nusol+1-nbnout) = ih
                           else
                              nusout = nusout + 1
                              igathr(nusout) = ih
                           end if
                        else
                           nusout = nusout + 1
                           igathr(nusout) = ih
                        end if
                     end if
                  end do
               end if
            end do

         else

!        --- Constant number of degrees of freedom (nphys) and no renumbering

            do i = 1, npoint
               if ( ioutpoints(i)>0 ) then
                  do j = 1, nphys
                     ih = (i-1)*nphys+j
                     if ( ih>0 ) then
                        if ( nbound>0 ) then
                           if ( kprobh(ih)>nusol-nbound-npboun .and. &
                             nbound>0 ) then
                              nbnout = nbnout + 1
                              igathr(nusol+1-nbnout) = ih
                           else
                              nusout = nusout + 1
                              igathr(nusout) = ih
                           end if
                        else
                           nusout = nusout + 1
                           igathr(nusout) = ih
                        end if
                     end if
                  end do
               end if
            end do

         end if
         if ( nbnout>0 ) then

!        --- Boundary conditions

            do i = 1, nbnout
               igathr(nusout+i) = igathr(nusol-nbnout+i)
            end do
            nusout = nusout + nbnout

         end if
         go to 1000

      else if ( indprp>0 ) then

!     --- indprp>0, i.e. copy iwork from KPROBP

         do i = 1, npoint
            iwork(i) = kprobp(i,jdegfd)
         end do
         if ( jdegfd>nphys ) then
            call eropen ( 'er0063' )
            call errint ( jdegfd, 1 )
            call errint ( nphys, 2 )
            call errsub ( 218, 2, 0, 0 )
            call erclos ( 'er0063' )
         end if

      else if ( indprf>0 ) then

!     --- indprf>0, i.e. no renumbering but the number of degrees of freedom
!                          is variable

         presen = .false.
         do i = 1, npoint
            nunkp = kprobf(i+1) - kprobf(i)
            if ( jdegfd<=nunkp ) then
               iwork(i) = kprobf(i) + jdegfd
               presen = .true.
            else
               iwork(i) = 0
            end if
         end do
         if ( .not. presen ) then
            call eropen ( 'er0063' )
            call errint ( jdegfd, 1 )
            call errint ( nunkp, 2 )
            call errsub ( 218, 2, 0, 0 )
            call erclos ( 'er0063' )
         end if

      else

!     --- Constant number of degrees of freedom (nphys) and no renumbering

         do i = 1, npoint
            iwork(i) = nphys*(i-1) + jdegfd
         end do
         if ( jdegfd>nphys ) then
            call eropen ( 'er0063' )
            call errint ( jdegfd, 1 )
            call errint ( nphys, 2 )
            call errsub ( 218, 2, 0, 0 )
            call erclos ( 'er0063' )
         end if

      end if
      if ( ierror/=0 ) go to 1000
      if ( iskip>0 ) then

!     --- Element groups must be skipped

         do i = 1, npoint
            if ( ioutpoints(i)==0 ) iwork(i) = 0
         end do

      end if

      if ( npboun>0 ) then

!     --- Periodical boundary conditions
!         Set second point equal to - first one in iwork if iwork(i)#0

         inpelm = kmesh( 4)
         nelgrp = kmesh( 5)
         kelma  = kmesh(15)
         kelmg  = kmesh(21)
         ikprbc = kprob(16)
         ikelmc = 0

!        --- Loop over element groups

         do ielgrp = 1, nelgrp
            itype  = kprob(ikprbc+ielgrp-1)
            inelem = kmesh(kelmg+ielgrp-1)
            if ( kelma/=0 ) inpelm = kmesh(kelma+ielgrp) - &
                                       kmesh(kelma+ielgrp-1)
            if ( itype/=-1 ) then

!           --- itype # -1, not a periodical boundary condition

               ikelmc = ikelmc + inelem * inpelm

            else

!           --- itype = -1,  periodical boundary condition
!               Loop over elements

               do iielem = 1, inelem
                  inodp1 = kmeshc(ikelmc+1)
                  inodp2 = kmeshc(ikelmc+2)
                  ikelmc = ikelmc + 2
                  if ( iwork(inodp2)>0 ) iwork(inodp2) = - inodp1
               end do

            end if

         end do

      end if

!     --- Reorder iwork and store into IGATHR

      nbnout = 0
      if ( indprf==0 .and. nbound==0 .and. iskip==0 ) then

!     --- indprf=0, nbound=0, i.e. constant number of degrees of freedom
!                             per point and no boundary conditions

         do i = 1, npoint
            igathr(i) = iwork(i)
         end do
         nusout = npoint

      else if ( indprh==0 ) then

!     --- indprf#0 and nbound=0, i.e. no boundary conditions

         nusout = 0
         do i = 1, npoint
            if ( iwork(i)>0 ) then
               nusout = nusout + 1
               igathr(nusout) = iwork(i)
            end if
         end do

      else

!     --- General situation,  nbound>0

         nusout = 0
         do i = 1, npoint
            if ( iwork(kprobh(i))>nusol-nbound-npboun ) then
               igathr(npoint-nbnout) = iwork(i)
               nbnout = nbnout + 1
            else if ( iwork(kprobh(i))>0 ) then
               nusout = nusout + 1
               igathr(nusout) = iwork(i)
            end if
         end do

         do i = 1, nbnout
            igathr(nusout+i) = igathr(npoint-nbnout+i)
         end do
         nusout = nusout + nbnout

      end if

1000  end
