      subroutine eroutcurvnod ( kmesh, kmeshm, kmeshq, iouterboun, &
                                nnodes, nodes )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  2.0  date 27-09-2010 Make work arrays local
!        version  1.0  date 21-02-2007
!
!   copyright (c) 2007-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all nodes in the outer curves and store in array nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     curve
!     outer_boundary
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kmeshm(*), kmeshq(*), iouterboun(*), &
              nnodes, nodes(*)

!     iouterboun     i    Either array ioutercurvs or ioutersurfs
!                         Contains outer curves or surfaces in the following way
!                         Pos. 1  nparts  Number of parts the outer boundary
!                                         consists of
!                         Pos. 2-1+nparts Starting addresses of each part
!                         Per part: (first address: ioutercurvs(1+i)
!                         Pos. 1: nouter  Number of curves in part
!                         followed by the curve numbers (provided with a sign)
!                         These curve numbers are consecutive, which means
!                         the end of one curve is the start of the new one
!                         The part itself is closed
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshq         i    KMESH part q, contains the subcurve numbers of
!                         each curve
!     nnodes         o    Number of nodes in array nodes
!     nodes          o    Contains the node numbers of the outer boundary
!                         at output
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npoint, nparts, j, istart, ncurves, icurnr, istep, &
              npointscurve, inelem, k, inodp, ipart, ncurvs, error
      integer, allocatable :: iwork(:), iwork1(:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     icurnr         Curve sequence number
!     inelem         Number of elements in a curve
!     inodp          Nodal point number
!     ipart          Sequence number of part
!     istart         First position used at a part
!     istep          Input parameter in er0026
!     iwork          integer work array of length npoint
!                    Is filled by subroutine er0026:
!                    array containing the nodal point numbers of the
!                    curves subsequently
!                    If point does not belong to a node, the address of
!                    the point with respect to curves provided with a
!                    minus sign is stored
!     iwork1         integer work array of length npoint
!                    iwork1 is to mark points that have been stored
!     j              Counting variable
!     k              Counting variable
!     ncurves        Number of curves in part
!     ncurvs         Number of curves in mesh
!     nparts         number of parts the boundary consists of
!     npoint         Number of nodal points
!     npointscurve   number of points along a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0026         fill nodal points of curves icrv1 to icrv2 in array iwork
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'eroutcurvnod' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eroutcurvnod'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npoint = kmesh(8)
      ncurvs = kmesh(11)

      allocate ( iwork(npoint), iwork1(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork1

      iwork1 = 0

      nparts = iouterboun(1)    ! number of parts in the outer boundary
      nnodes = 0
      istep = 1

!     --- For all parts do

      do ipart = 1, nparts

         istart = iouterboun(ipart+1)
         ncurves = iouterboun(istart)

         do j = 1, ncurves

            icurnr = abs(iouterboun(istart+j))
            call er0026 ( icurnr, icurnr, ncurvs, istep, &
                          npointscurve, inelem, kmeshm, iwork, &
                         .false., kmesh, kmeshq )

            do k = 1, npointscurve

               inodp = iwork(k)
               if ( inodp>0 ) then

!              --- Node number is positive

                  if ( iwork1(inodp)==0 ) then

!                 --- Node number has not been used yet
!                     add to array and mark as used

                     nnodes = nnodes+1
                     nodes(nnodes) = inodp
                     iwork1(inodp) = 1

                  end if  ! ( iwork1(inodp)==0 )

               end if  ! ( inodp>0 )

            end do  ! k = 1, npointscurve

         end do  ! k = 1, ncurves

      end do  ! ipart = 1, nparts

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'eroutcurvnod' )
      if ( debug ) then

!     --- Debug information

         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,*) 'End eroutcurvnod'

      end if  ! ( debug )

      end
