      subroutine ini068 ( ibuffr, isol, kprob, indprf, indprh, indprp, &
                          nunkp, nphys )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 28-09-2012 Jump in case ierror#0
!        version  1.5  date 27-12-2011 Extra debug
!        version  1.4  date 11-08-2004 ini060 --> ini060text
!
!   copyright (c) 1997-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  Memory management subroutine which simplifies the computation of the starting
!  addresses of an existing SEPRAN solution array and KPROB parts f, h and p
!     If these arrays exist, it is checked if they are positioned in
!     in array IBUFFR (Blank common) or not.
!     If one or more of these arrays are not available
!     these array are read from file 3 and space is reserved in array IBUFFR.
!     The priority number is made equal to 10.
!     if no space is available in array IBUFFR, INI068 tries to
!     create space in the following ways:
!       -  by writing low priority arrays to backing storage
!       -  if this is not sufficient by compressing
!          array ibuffr after writing of all arrays with priority
!          smaller than 10 to backing storage
!     INI068 is exactly the same as ini075, but with the parameter nunkp as
!     extra output parameter
!     If array KPROB part f (or its corresponding counterpart) does exist, &
!     INDPRF gets the correct index number referring to array INFOR in
!     common block CARRAY.
!     Afterwards the starting address of KPROB part f is given by
!     INFOR(1,INDFOR)
!     In the same way INDPRH and INDPRP refer to KPROB parts h and p
!     respectively
!     INI068 is identical to subroutine INI055 as described in the Programmer's
!     Guide, except that the parameter name is not longer used.
!     The name of the calling subroutine is extracted from common blocks
!     CMESSC and CMESSG, so it is supposed that the subroutine EROPEN has been
!     used
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(5), kprob(*), indprf, indprh, indprp, nphys, nunkp, &
              ibuffr(*)

!     ibuffr i/o Integer buffer array in which all large data is stored
!     indprf  o  At output INDPRF contains information concerning the KPROB
!                part f corresponding to ISOL.
!                If no KPROB part f corresponds to ISOL, INDPRF = 0, otherwise
!                (INDPRF>0) the starting address of KPROB part f is given
!                by INFOR(1,INDPRF)
!     indprh  o  At output INDPRH contains information concerning the KPROB
!                part h corresponding to ISOL.
!                If no KPROB part h corresponds to ISOL, INDPRH = 0, otherwise
!                (INDPRH>0) the starting address of KPROB part h is given
!                by INFOR(1,INDPRH)
!     indprp  o  At output INDPRP contains information concerning the KPROB
!                part p corresponding to ISOL.
!                If no KPROB part p corresponds to ISOL, INDPRP = 0, otherwise
!                (INDPRP>0) the starting address of KPROB part p is given
!                by INFOR(1,INDPRP)
!     isol    i  The standard SEPRAN solution array or array of special
!                structure of type 115 from which the starting positions
!                must be computed and the relevant information found.
!                Other types of arrays are not allowed
!     kprob   i  Standard SEPRAN array containing information of the problems
!                to be solved.
!     nphys   o  At output NPHYS contains the maximal number of degrees of
!                freedom per point or the number of physical variables if
!                defined
!                If ISOL is a vector of special structure and the number of
!                degrees of freedom per point is not constant then NPHYS = 0
!     nunkp   o  At output NUNKP contains the maximal number of degrees of
!                freedom per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, itypvc, iprob, ipkprb, ivec, kpivec
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Loop variable
!     ipkprb         Starting address of part of KPROB with respect to
!                    a complete problem in array KPROB (minus one)
!     iprob          Problem number
!     itypvc         Type number of solution vector (110=solution vector, &
!                    115=vector of special structure)
!     ivec           Indication of the structure of a vector of special structure
!     kpivec         Starting address of information concerning arrays of
!                    special structure in KPROB
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniprb

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIPRB         Get value of ipkprb as function of iprob
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     206   Input vector with illegal sequence number ivec
!     744   Only arrays of type 110 and 115 are permitted
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute problem number (IPROB), type of solution vector (ITYPVC)
!     and for a vector of special structure the vector number (IVEC)
!     Read solution array in IBUFFR (INI070)
!     If ( ISOL is of type 110 ) then
!        if ( KPROB part f has been filled ) then
!           Set INDPRF and read KPROB part f in IBUFFR
!        if ( KPROB part h has been filled ) then
!           Set INDPRH and read KPROB part h in IBUFFR
!        if ( KPROB part p has been filled ) then
!           Set INDPRP and read KPROB part p in IBUFFR
!     else if ( ISOL is of type 115 ) then
!        if ( KPROB part f has been filled ) then
!           Set INDPRF and read KPROB part f in IBUFFR
!     else if ( ISOL is of type 116 or 126 ) then
!        Compute nphys
!     else
!        wrong type of vector found: error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini068' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini068'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      indprf = 0
      indprh = 0
      indprp = 0
      itypvc = isol(2)
      if ( debug ) write(irefwr,1) 'itypvc', itypvc

!     --- Put ISOL into IBUFFR

      call ini060text ( ibuffr, isol(1), 'usol' )
      if ( ierror/=0 ) go to 1000

!     --- Compute IPROB and offset for array KPROB

      iprob  = 1 + isol(4)/1000
      ipkprb = iniprb ( iprob, kprob )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,1) 'iprob', iprob

      if ( itypvc==110 ) then

!     --- ISOL is of type 110:  solution vector

         nphys = kprob(ipkprb+33)
         nunkp = kprob(ipkprb+4)
         indprf = kprob(ipkprb+19)
         indprh = kprob(ipkprb+21)
         indprp = kprob(ipkprb+35)
         if ( indprf>0 ) &
            call ini060text ( ibuffr, -indprf, 'kprobf' )
         if ( indprh>0 ) &
            call ini060text ( ibuffr, -indprh, 'kprobh' )
         if ( indprp>0 ) &
            call ini060text ( ibuffr, -indprp, 'kprobp' )

      else if ( itypvc==115 ) then

!     --- ISOL is of type 115:  array of special structure

         ivec   = mod(isol(4),1000)
         if ( ivec<1 .or. ivec>kprob(ipkprb+6) ) then

!        --- ivec out of range

            call errint ( ivec, 1 )
            call errint ( kprob(ipkprb+6), 2 )
            call errsub ( 206, 2, 0, 0 )
            ivec = 1

         end if
         kpivec = kprob(ipkprb+18)-1+kprob(ipkprb+39)*kprob(ipkprb+6)+ &
                  ivec + ipkprb
         nphys  = kprob(kpivec)
         if ( nphys<0 ) then

!        --- nphys<0, constant number of degrees of freedom per point

            nphys = -nphys

         else

!        --- nphys>0, number of degrees of freedom per point not constant

            indprf = nphys
            kpivec = kprob(ipkprb+18)-1+(ivec-1)*kprob(ipkprb+39) + &
                     ipkprb
            nphys  = 0
            do i = 1, kprob(ipkprb+39)
               nphys = max ( nphys, kprob(kpivec+i) )
            end do
            call ini060text ( ibuffr, -indprf, 'kprobf' )
            if ( ierror/=0 ) go to 1000

         end if
         nunkp = nphys

      else if ( itypvc==116 .or. itypvc==126 ) then

!     --- ISOL is of type 116:  array of special structure defined per element

         nphys = mod(isol(4),1000)
         nunkp = nphys

      else if ( itypvc==127 ) then

!     --- ISOL is of type 127:  array defined per face

         ivec = 0
         nphys = isol(4)
         nunkp = nphys

      else if ( itypvc==801 ) then

!     --- ISOL is of type 801:  array of special structure related to
!         electrodes

         ivec = mod(isol(4),1000)
         ivec = (ivec*(ivec-1))/2
         nphys = isol(5)/ivec
         nunkp = nphys

      else

!     --- wrong type of vector

         call errint ( itypvc, 1 )
         call errsub ( 744, 1, 0, 0 )

      end if

1000  call erclos ( 'ini068' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ini068'

      end if  ! ( debug )

      end
