      subroutine ini069 ( ibuffr, ipstar, length, nbufol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.9  date 15-02-2013 Debug statements
!        version  1.8  date 21-11-2012 Long integers
!        version  1.7  date 28-09-2012 Remove instop
!
!   copyright (c) 1997-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reserve temporarily LENGTH words at the end of array IBUFFR
!     The starting position is equal to IPSTAR and NBUFFR is set equal
!     to IPSTAR-1
!     NBUFOL gets the old value of NBUFFR, so that at the end NBUFFR can
!     be reset. If the last positions of IBUFFR are already occupied, the
!     corresponding arrays are written to file 3 and in the case these
!     arrays have priority 10 or larger, thet are read in new positions
!     in IBUFFR
!     INI069 is identical to subroutine INI079 as described in the Programmer's
!     Guide, except that IBUFFR is used as extra parameter
!     The name of the calling subroutine is extracted from common blocks
!     CMESSC and CMESSG, so it is supposed that the subroutine EROPEN has been
!     used
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/carrayf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cmessgf90'
      include 'SPcommon/cmesscf90'
      include 'SPcommon/cdebugf90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer (kind=8) ipstar, nbufol
      integer ibuffr(*), length

!    ibuffr    i/o   Integer buffer array in which all large data is stored
!    ipstar     o    Starting position in array IBUFFR from which space
!                    is available
!    length     i    Length of the space required in words
!    nbufol     o    Value of NBUFFR at input
!                    This value may be used to reset NBUFFR in a later stage
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipold, ipnew, ibfrsv
      integer index, indtmp, lenloc, iextra, i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              General loop variable
!     ibfrsv         help variable to save or to compute ibfree
!     iextra         Extra space needed at both sides of the array for control
!                    integers
!     index          Reference number of an array in INFOR
!     indtmp         Reference number of a temporarily created array in INFOR
!     ipnew          Starting address of new array in IBUFFR
!     ipold          Starting address of old array in IBUFFR
!     lenloc         Length of local array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRINTLONG     Put long integer in error message
!     ERRSUB         Error messages
!     INI066         delete temporary array
!     INI081         Create permanent space in array IBUFFR
!     INICHK1        Check IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     17  Length of array IBUFFR too small
!   1765  length of array less than one
!   1816  Too much subsequent calls of ini069
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( idebug>0 ) then
!        Check IBUFFR
!     if ( length<=0 or length>nbuffr ) then
!        Error
!     ipstar := nbuffr-length
!     nbuffr := ipstar-1
!     if ( ibfree>=ipstar ) then
!        move all arrays with end address>ipstar to first part of IBUFFR
!        decrease ibfree accordingly
!     if ( idebug>0 ) then
!        Set control integers
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini069' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini069'
         write(irefwr,1) 'nbuffr, length', nbuffr, length
  1      format ( a, 1x, (10i12) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( idebug==0 ) then
         iextra = 0
      else
         iextra = intlen
         ifirws = 0
         call inichk1(ibuffr)
      end if
      if ( length<=0 ) then

!     --- length<=0, error

         call errint ( length, 1 )
         call errchr ( 'end_of_buffer', 1 )
         call errsub ( 1765, 1, 0, 1 )

      end if

      nbufol = nbuffr
      if ( length>nbuffr ) then
         call errint ( length, 1 )
         call errintlong ( nbuffr, 2 )
         call errsub ( 17, 2, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000
      ipstar = nbuffr - length + 1 - iextra
      if ( intlen==2 .and. mod(ipstar,2_8)==0 ) ipstar = ipstar - 1
      nbuffr = ipstar - 1 - iextra
      ibfree = min ( ibfree, nbuffr+1 )

      if ( ibfree>=ipstar-1-iextra ) then

!     --- ibfree>=ipstar
!         Move all arrays with starting address>nbuffr to positions in
!         the first part
!         Decrease IBFREE accordingly

         ibfrsv = ibfree
         do index = 1, iinfor
            if ( infor(index)>0 .and. infor(index) + &
                 infor1(1,index)+iextra>=nbuffr ) then

!           --- array "INDEX" has end address larger than NBUFFR
!               Move to new positions
!               First a new permanent array is created

               indtmp = 0
               ibfrsv = min ( ibfrsv, infor(index) )
               lenloc = infor1(1,index)
               call ini081 ( ibuffr, indtmp, lenloc, ' ' )
               if ( ierror/=0 ) go to 1000
               ipnew  = infor(indtmp)
               ipold  = infor(index)
               ibfrsv = max ( ibfrsv, ipnew+lenloc+2*iextra )
               if ( intlen==2 .and. mod(ibfrsv,2_8)==0 ) &
                  ibfrsv = ibfrsv - 1

!              --- Check if this array is in-core
!                  If so move contents of old array to this new one

               if ( ipold>0 ) then

!              --- old array is still in-core, which means that there was
!                  space enough
!                  Copy array to new positions

                  if ( ipold>ipnew ) then

!                 --- Copy forwards

                      do i = 0, lenloc-1
                         ibuffr(ipnew+i) = ibuffr(ipold+i)
                      end do

                  else if ( ipold<ipnew ) then

!                 --- Copy backwards

                      do i = lenloc, 1, -1
                         ibuffr(ipnew+lenloc-i) = &
                            ibuffr(ipold+lenloc-i)
                      end do
                  end if
                  infor(index) = ipnew

!                 --- Delete array indtmp, but use old positions

                  infor(indtmp) = ipold
                  if ( idebug>0 ) then
                     ibuffr(ipold-1) = indtmp
                     ibuffr(ipold+lenloc) = indtmp
                     ibuffr(ipnew-1) = index
                     ibuffr(ipnew+lenloc) = index
                  end if

               end if
               call ini066 ( ibuffr, indtmp )

            end if

         end do
         if ( ierror/=0 ) go to 1000

         ibfree = ibfrsv

      end if

      narbuf = narbuf + 1
      if ( idebug>0 ) then

!     --- set control integers

         if ( narbuf>25 ) then

!        --- error 1816: maximum number of calls of ini069 overwritten

            call errsub ( 1816, 0, 0, 0 )

         end if
         if ( ierror/=0 ) go to 1000
         iarbuf ( narbuf,1 ) = ipstar-1
         iarbuf ( narbuf,2 ) = ipstar+length
         ibuffr ( ipstar-1 ) = 1600+narbuf
         ibuffr ( ipstar+length ) = 1600+narbuf

      end if

      if ( ioutp>2 ) then
         write(irefwr,*) ' Create temporary array ', narbuf, &
                         ' with starting address', ipstar, &
                         ' and length ', length, &
                         ' in subroutine ', &
                         namsub(levsub-1)//':ini069'
      end if

1000  call erclos ( 'ini069' )

      if ( debug) then

!     --- Debug information

         write(irefwr,*) 'End ini069'

      end if

      end
