      subroutine iniactmk ( ibuffr, ichoice, ipos, intar, intarex, &
                            mmindex )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.12 date 24-02-2012 Debug statements
!        version  1.11 date 26-01-2009 extension with kmesh aa part j
!        version  1.10 date 27-04-2004 Replace part of body by subroutine
!
!   copyright (c) 1998-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    General memory management subroutine that is used to activate
!    a permanent array. If this array does not exist
!    the new array is created if possible. If it is not possible to create
!    the new array an error message is given.
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), ichoice, ipos, intar(*), intarex(*), mmindex

!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     ichoice    i    Choice parameter indicating which array must be activated
!                     or created.
!                     Possible values:
!                     1: Standard sub array of KMESH
!                     2: Standard sub array of KMESH part aa
!                     3: Standard sub array of KMESH part ab
!                     4: Standard sub array of KMESH part y
!                     5: Standard sub array of KMESH part ad
!     intar     i/o   Contains the main array from which a part must be
!                     created or activated.
!                     Which array it concerns depends on ICHOICE
!     intarex    i    Extra array that may be necessary for the creation of
!                     an array.
!                     Depends on ICHOICE
!     ipos       i    Indicates which position in the main array defines
!                     the array to be activated or created
!     mmindex    o    contains the memory management sequence number of the
!                     array that is activated or created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jchoice, kelmaa, kelmab, kelmy, kelmad, length
      double precision dummy(1)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     dummy      Dummy parameter
!     jchoice    Defines the actual array that must be created or activated
!                 Possible values:
!                 1:  kmesh part AB: part ELGRPS
!                 2:  kmesh part AA: part NMESHE
!                 3:  kmesh part d
!                 4:  kmesh part AA: parts NMESHC and D
!                 5:  kmesh part AA: part NMESHH
!                 6:  kmesh part AA: part NMESHL
!                 7:  kmesh part AA: part NMESHA and B
!                 8:  kmesh part AA: part NMESHF
!                 9:  kmesh part AA: part NMESHG
!                10:  kmesh part AB: part ELSCURVS
!                11:  kmesh part AB: part ELSSURFS
!                12:  kmesh part AB: part INFVOLMS
!                13:  kmesh part AB: part INFSURFS
!                14:  kmesh part AB: part INFCURVS
!                15:  kmesh part AB: part INFUSPNT
!                16:  kmesh part AB: part ELSUSPNT
!                17:  kmesh part Y: part face
!                18:  kmesh part Y: parts lengthf and areafv
!                19:  kmesh part Y: parts iconcell and iconface
!                20:  kmesh part e
!                21:  kmesh part AD: parts ipointobst
!                22:  kmesh part AD: parts ielemobst
!                23:  kmesh part AD: parts icrossobst,rcrossobst
!                24:  kmesh part ae
!                25:  kmesh part af
!                26:  kmesh part AD: part obstfaces
!                27:  kmesh part AA: part NMESHK
!                28:  kmesh part o
!                29:  kmesh part AH: outer curves/surfaces
!                30:  kmesh part AA: part NMESH J
!     kelmaa     Starting position of KMESH part AA in array KMESH
!     kelmab     Starting position of KMESH part AB in array KMESH
!     kelmad     Starting position of KMESH part AD in array KMESH
!     kelmy      Starting position of KMESH part Y in array KMESH
!     length     Help parameter to indicate the length of arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIACTM0       Actual body of subroutine INIACTMK
!     PRINLG         Print and checks length of user arrays
!     TOPFILLKMESHAA Fill first part of array KMESH part aa
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1212   Position 3 of array has not been filled
!    1685   Value of ipos corresponds to part of array that is not in IBUFFR
!    1686   The combination ichoice, ipos has not yet been implemented
!    1687   Incorrect value of ichoice
!    1690   INTARR is wrong type
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check parameters
!     Compute jchoice
!     Initialize kmesh part aa or ab depending on ichoice
!     Perform actual actions
!     set mmindex
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniactmk' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniactmk'
         write(irefwr,1) 'ichoice, ipos', ichoice, ipos
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ichoice<=5 ) then

!     --- ichoice<=4, array intar must be of type kmesh

         if ( intar(2)/=100 ) then

!        --- Array intar of wrong type

            call errint ( 100, 1 )
            call errint ( intar(2), 2 )
            call errint ( ichoice, 3 )
            call errchr ( 'KMESH', 1 )
            call errsub ( 1690, 3, 0, 1 )
            go to 1000

         end if
         if ( intar(3)<=50 ) then

!        --- Position 3 of intar has not been filled

            call errint ( intar(3), 1 )
            call errchr ( 'KMESH', 1 )
            call errsub ( 1212, 1, 0, 1 )
            go to 1000

         end if

      end if
      select case ( ichoice )

         case(1)

!        --- Standard subarray of KMESH (i.e. parts a-x,z, iinput, rinput)

         select case ( ipos )

            case(18)

!           --- Part d

               jchoice = 3

            case(19)

!           --- Part e

               jchoice = 20

            case(29)

!           --- Part o

               jchoice = 28

            case(52)

!           --- Part ad

               jchoice = 21

            case(58)

!           --- Part ah

               jchoice = 29

            case(55, 56)

!           --- Part ah

               jchoice = ipos-31

            case ( -100:16, 20, 21, 36:44, 47:51, 53, 54, 57, 59:100)

!           --- ipos does not correspond to a standard position in KMESH that is
!               stored in IBUFFR

               call errint ( ipos, 1 )
               call errchr ( 'standard position in KMESH', 1 )
               call errsub ( 1685, 1, 0, 1 )
               go to 1000

            case default

               if ( intar(ipos)/=0 ) then

!              --- Activate array

                  call ini060text ( ibuffr, intar(ipos), 'kmesh_ipos' )
                  go to 900

               else

!              --- Possiblity has not yet been implemented

                  call errint ( ichoice, 1 )
                  call errint ( ipos, 2 )
                  call errsub ( 1686, 2, 0, 0 )
                  go to 1000

               end if

            end select  ! case ( ipos )

         case(2)

!        --- Subarray of KMESH part aa

            select case ( ipos )

               case(-100:30, 43:1000 )

!              --- ipos does not correspond to part aa in KMESH that is
!                  stored in IBUFFR

                  call errint ( ipos, 1 )
                  call errchr ( 'part aa in KMESH', 1 )
                  call errsub ( 1685, 1, 0, 1 )
                  go to 1000

               case(31, 32)

!              --- Parts nmesha and b

                  jchoice = 7

               case(33, 34)

!              --- Parts nmeshc and d

                  jchoice = 4

               case(35)

!              --- Part nmeshe

                  jchoice = 2

               case(36)

!              --- Part nmeshf

                  jchoice = 8

               case(37)

!              --- Part nmeshg

                  jchoice = 9

               case(38)

!              --- Part nmeshh

                  jchoice = 5

               case(39)

!              --- Part nmeshi

                  jchoice = 31

               case(40)

!              --- Part nmeshj

                  jchoice = 30

               case(41)

!              --- Part nmeshk

                  jchoice = 27

               case(42)

!              --- Part nmeshl

                  jchoice = 6

               case default

!              --- Possiblity has not yet been implemented

                  call errint ( ichoice, 1 )
                  call errint ( ipos, 2 )
                  call errsub ( 1686, 2, 0, 0 )
                  go to 1000

            end select  ! case ( ipos )

         case(3)

!        --- Subarray of KMESH part ab

            select case ( ipos )

               case(16:1000 )

!              --- ipos does not correspond to part ab in KMESH that is
!                  stored in IBUFFR

                  call errint ( ipos, 1 )
                  call errchr ( 'part ab in KMESH', 1 )
                  call errsub ( 1685, 1, 0, 1 )
                  go to 1000

               case(1)

!              --- Part elgrps

                  jchoice = 1

               case(2)

!              --- Part elgrps

                  jchoice = 12

               case(3)

!              --- Part elgrps

                  jchoice = 13

               case(4)

!              --- Part elgrps

                  jchoice = 14

               case(5)

!              --- Part elgrps

                  jchoice = 15

               case(10)

!              --- Part elssurfs

                  jchoice = 11

               case(11)

!              --- Part elscurvs

                  jchoice = 10

               case(12)

!              --- Part elscurvs

                  jchoice = 16

               case default

!              --- Possiblity has not yet been implemented

                  call errint ( ichoice, 1 )
                  call errint ( ipos, 2 )
                  call errsub ( 1686, 2, 0, 0 )
                  go to 1000

            end select  ! case ( ipos )

         case(4)

!        --- Subarray of KMESH part y

            select case ( ipos )

               case(1:20, 26:1000 )

!              --- ipos does not correspond to part y in KMESH that is
!                  stored in IBUFFR

                  call errint ( ipos, 1 )
                  call errchr ( 'part y in KMESH', 1 )
                  call errsub ( 1685, 1, 0, 1 )
                  go to 1000

               case(21)

!              --- Part face

                  jchoice = 17

               case(22, 23)

!              --- Parts iconcell and iconface

                  jchoice = 19

               case(24, 25)

!              --- Parts lengthf and areafv

                  jchoice = 18

               case default

!              --- Incorrect value of ichoice

                  call errint ( ichoice, 1 )
                  call errint ( ipos, 2 )
                  call errsub ( 1686, 2, 0, 0 )
                  go to 1000

            end select  ! case ( ipos )

         case(5)

!        --- Subarray of KMESH part ad

            select case ( ipos )

               case(-100:0, 5:8, 10:1000 )

!              --- ipos does not correspond to part ad in KMESH that is
!                  stored in IBUFFR

                  call errint ( ipos, 1 )
                  call errchr ( 'part ad in KMESH', 1 )
                  call errsub ( 1685, 1, 0, 1 )
                  go to 1000

               case(1)

!              --- Part ipointobst

                  jchoice = 21

               case(2)

!              --- Part ielemobst

                  jchoice = 22

               case(3, 4)

!              --- Part icrossobst,rcrossobst

                  jchoice = 23

               case(9)

!              --- Part obstfaces

                  jchoice = 26

               case default

!              --- Incorrect value of ichocie

                  call errint ( ichoice, 1 )
                  call errint ( 1, 2 )
                  call errint ( 4, 3 )
                  call errsub ( 1687, 3, 0, 0 )

            end select  ! case ( ipos )

      end select  ! case ( ichoice )

      if ( (ichoice>=2 .and. ichoice<=5 .or. jchoice==23) .and. &
           intar(48)==0 ) then

!     --- kmesh aa has not been created, create and initialize

         call topfillkmeshaa ( intar )

      end if

      if ( ichoice==3 .and. intar(49)==0 ) then

!     --- kmesh ab has not been created, create and initialize

         length = intar(3)
         intar(49) = length+1
         length = length+20
         call prinlg ( 'kmesh', length, dummy, intar )
         kelmab = intar(49)-1
         intar(kelmab+1:kelmab+20) = 0

      end if

      if ( ichoice==4 .and. intar(50)==0 ) then

!     --- kmesh y has not been created, create and initialize

         length = intar(3)
         intar(50) = length+1
         length = length+30
         call prinlg ( 'kmesh', length, dummy, intar )
         kelmy = intar(50)-1
         intar(kelmy+1:kelmy+30) = 0

      end if

!     --- Fill arrays recursively if needed

      call iniactm0 ( ibuffr, ibuffr, jchoice, intar, intarex )

!     --- Set mmindex

900   if ( ichoice==1 ) then
         mmindex = intar(ipos)
      else if ( ichoice==2 ) then
         kelmaa = intar(48)-1
         mmindex = intar(kelmaa+ipos)
      else if ( ichoice==3 ) then
         kelmab = intar(49)-1
         mmindex = intar(kelmab+ipos)
      else if ( ichoice==4 ) then
         kelmy = intar(50)-1
         mmindex = intar(kelmy+ipos)
      else if ( ichoice==5 ) then
         kelmad = intar(52)-1
         mmindex = intar(kelmad+ipos)
      end if

1000  call erclos ( 'iniactmk' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'mmindex', mmindex
         write(irefwr,*) 'End iniactmk'

      end if  ! ( debug )

      end
