      subroutine inimultadd2nbc ( ibuffr, buffer, iu1, iu2, &
                                  a1, a2, iures, kprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 23-07-2007
!
!   copyright (c) 2007-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute a1 u1 + a2 u2 and store in ures
!     ures may be one of the two vectors
!     The vectors must be sepran solution vectors with the same length
!     The part corresponding to boundary conditions is not filled
!     If the vector is new, it is filled with zeros
! **********************************************************************
!
!                       KEYWORDS
!
!     multiplication
!     linear_combination
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iu1(5), iu2(5), iures(5), ibuffr(*), &
              kprob(*)
      double precision a1, a2, buffer(*)

!     a1             i    multiplication constant
!     a2             i    multiplication constant
!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iu1            i    First vector
!     iu2            i    Second vector
!     iures         i/o   Result vector
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipu1, ipu2, ipures, ipkprh
      integer nusol, nbound, npboun, ipkprb, iprob, nprob, indprh
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     indprh         indication if kprob part h has been filled (>0)
!                    or not (0)
!     ipkprb         Starting address of array KPROB for actual problem
!                    number - 1
!     ipkprh         starting address of array kprob part h
!     iprob          Actual problem number
!     ipu1           Starting address of array u1
!     ipu2           Starting address of array u2
!     ipures         Starting address of array ures
!     nbound         Number of essential boundary conditions
!     npboun         Number of periodical boundary conditions
!     nprob          Number of various problems
!     nusol          Total number of unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext, inigettext
      integer iniprb

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI080EXTVEC   Create new sepran double precision array of type isol
!                    Clear this array
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INIPRB         Get value of ipkprb as function of iprob
!     PRMULTADD2     Compute a1 u1 + a2 u2 and store in ures
!     PRMULTADD2NB   Compute a1 u1 + a2 u2 and store in ures
!                    Only for free degrees of freedom
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     699   Problem number incorrect
!    2887   The length of two input vectors is not the same
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inimultadd2nbc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inimultadd2nbc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays iu1/2

      call ini060text ( ibuffr, iu1(1), 'iu1' )
      call ini060text ( ibuffr, iu2(1), 'iu2' )

!     --- Get length of arrays

      nusol = iu1(5)
      if ( iu2(5)/=nusol ) then

!     --- error: arrays do not have the same length

         call errchr ( 'iu1', 1 )
         call errchr ( 'iu2', 2 )
         call errint ( iu1(5), 1 )
         call errint ( iu2(5), 2 )
         call errsub ( 2887, 2, 0, 2 )
         go to 1000

      end if  ! ( iu2(5)/=nusol )

!     --- Check iprob

      iprob  = iu1(4) / 1000 + 1      ! problem number
      nprob  = max ( 1, kprob(40) )   ! number of problems
      if ( iprob<1 .or. iprob>nprob )  then
         call errint ( iprob, 1 )
         call errint ( nprob, 2 )
         call errsub ( 699, 2, 0, 0)
         go to 1000
      end if
      ipkprb = iniprb ( iprob, kprob ) ! starting address in kprob minus 1
      nbound = kprob(ipkprb+10)   ! number of essential bc's
      npboun = kprob(ipkprb+34)   ! number of periodical bc's
      indprh = kprob(ipkprb+50)   ! mm number of kprob part h
      if ( indprh>0 ) call ini060text ( ibuffr, indprh, 'kprobh' )

!     --- Create or activate result array

      if ( iures(1)==0 ) then

!     --- iures(1) = 0, hence new array
!         Create and clear

         call ini080extvec ( ibuffr, buffer, iures, nusol, 0, iprob, &
                             'iures' )

      else

!     --- iures(1)>0, hence existing array
!         activate

         call ini060text ( ibuffr, iures(1), 'iures' )

      end if  ! ( iures(1)==0 )

!     --- Compute starting addresses

      ipu1 = inidgttext ( iu1(1), 'iu1' )
      ipu2 = inidgttext ( iu2(1), 'iu2' )
      ipures = inidgttext ( iures(1), 'iures' )
      if ( nusol+nbound==0 ) then

!     --- No boundary conditions, so standard case

         call prmultadd2 ( nusol, buffer(ipu1), buffer(ipu2), &
                           a1, a2, buffer(ipures) )

      else

!     --- There are boundary conditions, jump over corresponding
!         positions

         ipkprh = inigettext ( indprh, 'kprobh' )+nusol
         nusol = nusol-nbound-npboun

         call prmultadd2nb ( nusol, buffer(ipu1), buffer(ipu2), &
                             a1, a2, buffer(ipures), ibuffr(ipkprh) )

      end if  ! ( nusol+nbound==0 )

1000  call erclos ( 'inimultadd2nbc' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inimultadd2nbc'

      end if  ! ( debug )

      end
