      subroutine inineighborset ( nodes, nnodes, npoint, kmeshe, nset, &
                                  neighset )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all neighbors of a set of nodes and store in array neighbors
!
! **********************************************************************
!
!                       KEYWORDS
!
!     neighbour
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(nnodes), npoint, kmeshe(*), nset, &
              neighset(nset)

!     kmeshe         i    STANDARD SEPRAN array containing information on
!                         nodal points connected to nodal points
!     neighset       o    Set of neighbors
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Set of node numbers from which we want the
!                         neighbors
!     npoint         i    Number of nodal points
!     nset          i/o   at output: Number of neighbors of nodes
!                         at input: declared length of setneigh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer error, lenneigh, i, neigh, nneigh, node, j
      integer, allocatable :: iwork(:), setneigh(:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     iwork          integer work array tp mark nodes
!                    iwork(i) = 0, node i does not belong to neighset
!                            >0, node i is stored in neighset
!     j              Counting variable
!     lenneigh       Declared length of array neighset
!     neigh          neighbor of node
!     nneigh         Number of neighbors of node
!     node           Node number
!     setneigh       Set of neighbors of node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     ININEIGHBORS   Find all neighbors of a node and store in array neighbors
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inineighborset' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inineighborset'
         call prinin ( nodes, nnodes, 'nodes' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), setneigh(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear iwork

      iwork = 0

      lenneigh = nset  ! declared length
      nset = 0

!     --- For each node in nodes do
!           Put unmarked neighbors in neighset
!           Mark these neighbors

      do i = 1, nnodes

         node = nodes(i)  ! node in array nodes

!        --- Store neighbors of node in setneigh

         nneigh = npoint
         call inineighbors ( node, npoint, kmeshe, nneigh, setneigh )
         if ( ierror/=0 ) go to 1000

         do j = 1, nneigh

            neigh = setneigh(j)
            if ( iwork(neigh)==0 ) then

!           --- Unmarked node found: mark and store

               iwork(neigh) = 1
               nset = nset+1

               call inichkarlen ( nset, lenneigh, 'neighset' )
               if ( ierror/=0 ) go to 1000

               neighset(nset) = neigh

            end if  ! ( iwork(neigh)==0 )

         end do  ! i = 1, nneigh

      end do  ! i = 1, nnodes

      deallocate ( iwork, setneigh, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

1000  call erclos ( 'inineighborset' )
      if ( debug ) then

!     --- Debug information

         call prinin ( neighset, nset, 'neighset' )
         write(irefwr,*) 'End inineighborset'

      end if  ! ( debug )

      end
