      subroutine inistrnew1 ( kprob, iprob, itypvc, ivec, number, &
                              kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 10-02-2015 Minor change in comment
!        version  1.3  date 03-12-2013 Extra debug
!        version  1.2  date 18-01-2012 extra debug
!
!   copyright (c) 2007-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the structure of the vector with exactly NUMBER unknowns per point
!
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     structure
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprob(*), iprob, itypvc, ivec, number, kmesh(*)

!     iprob          i    Actual problem number
!     itypvc         o    Indication of the type of solution vector:
!                         110: solution vector
!                         115, 116: vectors of special structure
!                         If 0, no vector found
!     ivec           o    Sequence number of vector of special structure
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     number         i    The required number of unknowns per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipkprb, nunkp, kproba, kprobe, kelgrp, ndtvec, i, jprob, &
              nprob
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ipkprb         Starting address of array KPROB for actual problem
!                    number - 1
!     jprob          Problem number
!     kelgrp         sum over all standard elements of number of points in
!                    standard elements
!     kproba         starting address in kprob part a
!     kprobe         starting address in kprob part e
!     ndtvec         Number of vectors of special structure
!     nprob          Number of various problems
!     nunkp          Maximum number of unknowns in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniprb

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIPRB         Get value of ipkprb as function of iprob
!     INISTRELGRP    Check if vector of special structure has a constant number
!                    of unknowns per point for all non-empty elements
!     INISTRSLELGRP  Check if solution vector has a constant number of number
!                    unknowns per point for all non-empty elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inistrnew1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inistrnew1'
         write(irefwr,1) 'itypvc, number', itypvc, number
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ipkprb = iniprb ( iprob, kprob )
      if ( ierror/=0 ) go to 1000

      nunkp  = kprob(ipkprb+4)
      kproba = kprob(ipkprb+14)
      ivec = 0

      if ( debug ) write(irefwr,1) 'nunkp, kproba', nunkp, kproba

      if ( itypvc==110 .and. nunkp==number ) then

!     --- nunkp = number, vector is of type solution vector

         if ( kproba==0 ) then

!        --- Number of unknowns per point is constant

            itypvc = 110
            go to 1000

         else

!        --- Number of unknowns per point is not constant
!            Check if solution vector is a candidate

            call inistrslelgrp ( kmesh, kprob(ipkprb+1), number, &
                                 itypvc )

!           --- if itypvc = 110 the solution vector is found
!               in this case or in case of an error: jump

            if ( ierror/=0 .or. itypvc==110 ) go to 1000

         end if  ! ( kproba==0 )

      end if  ! ( itypvc==110 .and. nunkp==number )

!     --- vector is not of type solution vector
!         Check if there is a vector of special structure with NUMBER
!         unknowns per point

      ivec = 0
      ndtvec = kprob(ipkprb+6)
      kprobe = kprob(ipkprb+18)
      kelgrp = kprob(ipkprb+39)
      itypvc = 115

      if ( debug ) &
         write(irefwr,1) 'ndtvec, kprobe, kelgrp', &
                          ndtvec, kprobe, kelgrp

      do i = 1, ndtvec

         nunkp = -kprob(ipkprb+kprobe-1+kelgrp*ndtvec+i)

         if ( nunkp==number ) then

!        --- Vector of special structure found

            itypvc = 115
            ivec = i
            exit

         end if

      end do  ! i = 1, ndtvec

      if ( debug ) write(irefwr,1) 'ivec/1', ivec

      if ( ivec==0 ) then

!     --- No vector with a constant number of degrees of freedom has been found
!         Check if there is a vector with a number degrees of freedom

         itypvc = 115

         do i = 1, ndtvec

!        --- Check if vector of type i is the one with number degrees of freedom
!            per point

            call inistrelgrp ( kmesh, kprob(ipkprb+1), number, ivec, i )
            if ( ivec>0 ) then

!           --- ivec>0, vector found

               go to 1000

            end if  ! ( ivec>0 )

         end do  ! i = 1, ndtvec

      end if  ! ( ivec==0 )

      if ( ivec==0 ) then

!     --- No vector found, check if there is a vector for a different problem

         jprob = 1
         nprob = kprob(40)
         do while ( jprob<=nprob .and. jprob/=iprob )

            ipkprb = iniprb ( jprob, kprob )
            ndtvec = kprob(ipkprb+6)

            do i = 1, ndtvec

!           --- Check if vector of type i is the one with number degrees of
!               freedom per point

               call inistrelgrp ( kmesh, kprob(ipkprb+1), number, ivec, &
                                  i )
               if ( ivec>0 ) then

!              --- ivec>0, vector found

                  iprob = jprob
                  go to 1000

               end if  ! ( ivec>0 )

            end do  ! i = 1, ndtvec

            jprob = jprob+1  ! raise jprob

         end do  ! while ( jprob<=nprob .and. jprob/=iprob )

      end if  ! ( ivec==0 )

1000  call erclos ( 'inistrnew1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'itypvc, ivec', itypvc, ivec
         write(irefwr,*) 'End inistrnew1'

      end if  ! ( debug )

      end
