      subroutine inistrslelgrp ( kmesh, kprob, number, itypvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 10-02-2015 Extension with vectors of special
!                                      structure
!        version  1.1  date 03-12-2013 Do not skip itype = 0
!        version  1.0  date 09-12-2005
!
!   copyright (c) 2005-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if solution vector has a constant number of number unknowns
!     per point for all non-empty elements
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     structure
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kprob(*), number, itypvc

!     itypvc         o    Indicates if the vector required is solution vector
!                         110: solution vector
!                           0: not solution vector
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     number         i    The required number of unknowns per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, nelgrp, kelma, kproba, ielgrp, itype, kprobc, &
              inunkp, jnunkp, i, ikelm
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ielgrp         element group number
!     ikelm          Last position used for element group
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in point
!     itype          type number of element
!     jnunkp         Number of degrees of freedom in point
!     kelma          Indicates if KMESH part a has been filled (>0)
!                    or not (0)
!                    If kelma>0, kelma is the starting address of KMESH
!                    part a in KMESH
!     kproba         starting address in kprob part a
!     kprobc         Starting address of KPROB part c in KPROB
!     nelgrp         Number of element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     itypvc = 0
!     For all element groups do
!        inunkp := number of degrees of freedom per point in solution vector
!        if ( inunkp not constant ) then
!           error
!              return
!        else if ( inunkp/=number ) then
!           Vector is not solution vector
!              return
!     itypvc = 110
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inistrslelgrp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inistrslelgrp'
         write(irefwr,1) 'number', number
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Extract constants

      inpelm = kmesh(4)   ! number of points per element
      nelgrp = kmesh(5)   ! number of element groups
      kelma = kmesh(15)   ! start of kmesh part a
      kproba = kprob(14)  ! start of kprob part a
      kprobc = kprob(16)  ! start of kprob part c
      if ( debug ) &
         write(irefwr,1) 'inpelm, nelgrp, kelma, kproba, kprobc', &
                          inpelm, nelgrp, kelma, kproba, kprobc

!     --- For all element groups do

      do ielgrp = 1, nelgrp

!     --- inunkp := number of degrees of freedom per point in solution vector

         itype = kprob(kprobc+ielgrp-1)  ! type number of element
         if ( itype<0 ) go to 200       ! skip elements with type number<1
         if ( kelma==0) then

!        --- kelma = 0, constant number of nodes per element

            ikelm = (ielgrp-1)*inpelm

         else

!        --- kelma>0, variable number of nodes per element

            ikelm = kmesh(kelma+ielgrp-1)
            inpelm = kmesh(kelma+ielgrp)-ikelm

         end if
         inunkp = kprob(ikelm+kproba)  ! number of unknowns in node 1
         do i = 2, inpelm
            jnunkp = kprob(ikelm+i+kproba-1)  ! number of unknowns in node i
            if ( jnunkp/=inunkp ) inunkp = -1  ! mark inunkp if not constant
         end do  ! i = 2, inpelm

         if ( debug ) &
          write(irefwr,1) 'ielgrp, itype, ikelm, inpelm, inunkp', &
                           ielgrp, itype, ikelm, inpelm, inunkp

         if ( inunkp/=number ) then

!        --- inunkp # number, so solution vector is not the one we are looking
!            for: jump

            itypvc = 0
            go to 1000

         end if  ! ( inunkp==-1 )

!        --- ivec := jvec

         itypvc = 110  ! apparantly vector found

200   end do  ! ielgrp = 1, nelgrp

1000  call erclos ( 'inistrslelgrp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'itypvc', itypvc
         write(irefwr,*) 'End inistrslelgrp'

      end if

      end
