      subroutine intx0218 ( icode, code, ncode, itype, table, codemn, &
                            nmcode, tabmin, tabmax, tabout, nout, jump, &
                            iposut, ihelp, finished, readnx, minusg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.7  date 10-11-2014 Check tabout
!        version  1.6  date 17-03-2012 Use intx02curves
!        version  1.5  date 05-09-2008 correction scalars
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret part of input record for subroutine intx02 for the
!     possibilities itype = 14, 18 and 31
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icode, ncode(*), itype(*), nmcode(*), &
              nout, jump, iposut, ihelp(*), minusg
      character (len=*) code(*), codemn(*)
      double precision table(*), tabmin(*), tabmax(*), tabout(nout)
      logical finished, readnx

!     code           i    Table with keywords
!     codemn         i    Table with main keywords
!     finished       o    Indicates if the loop has been finished (true) or not
!                         (false)
!     icode          i    Return code with respect to subroutine INTX01
!                         Indicates position in array code or code1
!     ihelp          o    Help array to store number of appearances of keywords
!                         that may be used more than once or number of numbers
!                         read
!     iposut        i/o   Position in array tabout
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     jump           o    if 0 no error has been found, if 1 array tabout
!                         is too small
!     minusg        i/o   Indicates if a minus sign has been found (-1)
!                         or not (1)
!     ncode          i    Table containing the number of significant characters
!                         in the keyword table code
!     nmcode         i    Table containing the number of significant characters
!                         in the main keyword table codemn, including those of
!                         the extra texts
!     nout           i    Number of entries in tabout
!     readnx         i    Indicates if the next item has not been read (true)
!                         or has been (false)
!     table         i/o   Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabmin        i/o   Contains minimum values allowed in table
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ih, ivalrd, num, numkey, ispeclsav, numchar
      logical debug, readnext
      character (len=80) subkey, keyword

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ih             Help variable to store a constant temporarily
!     ispeclsav      Help parameter to save the value of ispecl
!     ivalrd         Integer value read
!     keyword        contains the keyword without number
!     num            Number of numbers read between brackets
!     numchar        Number of characters in keyword
!     numkey         Number of significant characters in sub keyword
!     readnext       If true the next item must be read
!     subkey         Sub (second) keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTX02CURVES   Read part ( Ci, Cj, ... Cl ) in input record
!     READNAMEVEC    Read name of vector in common cpost5
!     READSPLITKEYW  check whether the keyword contains numbers
!                    If so split the keyword into a part without number and
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1461   incorrect keyword found after keyword
!    1467   keyword has been found more than once
!    1468   Array tabout too small
!    1914   Not a second keyword found after keyword = &
! **********************************************************************
!
!                       PSEUDO CODE
!
!    itype(i) =14, special situation of keyword followed
!                  by number, followed by same keyword followed
!                  by number and so on, for example:
!                  p1, p3, p5, p8, p2
!                  The keywords may be preceded by a minus sign, &
!                  hence c1, c3, -c5, c8 etc., in which the
!                  sequence number gets the minus sign
!                  input:  code(i) keyword
!                          tabmin(i) maximum number of integers
!                          tabmax(i) maximum value of integers
!                  output: table(i) = iposut
!                          tabmax(i) = m (number of integers read)
!                          tabout(iposut to iposut+m-1) = values
!    itype(i) =18, special situation of keyword followed by
!                  followed by a series of the same keywords
!                  followed by sequence numbers
!                  hence we have the situation keyword followed
!                  by possibility 14
!                  input:  code(i) keyword
!                          table(i) -iposmain
!                          codemn(iposmain) special sub keyword
!                          tabmin(i) maximum number of numbers
!                                    that may be read
!                          tabmax(i) maximum value of |numbers|
!                  output: table(i)  iposut
!                          tabout(iposut to iposut+m-1) values
!                          tabmax(i) m
!    itype(i) =31, special situation of keyword followed by
!                  possibly an integer followed by
!                  followed by a series of the same keywords
!                  followed by sequence numbers
!                  hence we have the situation keyword
!                  possibly followed by an integer followed
!                  by possibility 18
!                  input:  code(i) keyword
!                          table(i) -iposmain
!                          codemn(iposmain) special sub keyword
!                          tabmin(i) maximum number of numbers
!                                    that may be read
!                          tabmax(i) maximum value of |numbers|
!                  output: table(i)  iposut
!                          tabout(iposut to iposut+m-1) values
!                          tabmax(i) m
!                          tabmin(i) integer after keyword
!                                    if not found: 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0218' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0218'
         write(irefwr,1) 'icode, itype(icode)', icode, itype(icode)
         write(irefwr,1) 'lastitem/1, ityprd', lastitem, ityprd
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jump = 0
      readnext = .true.
      ispeclsav = ispecl
      if ( itype(icode)==18 .or. itype(icode)==31 ) then

!     --- the keyword must be followed by the next keyword
!         possibly provided with minus sign
!         In the case 31 it may be followed first by an integer

         ih = nint(-table(icode))
         if ( ih<0 ) then
            call errchr ( code(icode), 1 )
            call errsub ( 1467, 0, 0, 1 )
            call errcrd
            go to 1000
         end if
         subkey = codemn(ih)
         numkey = nmcode(ih)

         call asknxt
         if ( debug ) then
            write(irefwr,1) 'ityprd/1, lastitem', ityprd, lastitem
         end if  ! ( debug )

         if ( ityprd==4 ) then

!        --- ityprd = 4, include digits if necessary

            lastitem = lastitembef
            ispecl = 9
            call asknxt
            ispecl = ispeclsav
            if ( debug ) write(irefwr,*) 'string ', string(1:nchar)

         end if  ! ( ityprd==4 )

         ivalrd = 0
         if ( itype(icode)==31 .and. ityprd==2 .or. ityprd==11 ) then

!        --- integer found

            ivalrd = nint(realrd)
            if ( debug ) write(irefwr,1) 'ivalrd', ivalrd
            call asknxt
            if ( debug ) then
               write(irefwr,1) 'ityprd/2, lastitem', ityprd, lastitem
               if ( ityprd==4 ) &
                  write(irefwr,*) 'string ', string(1:nchar)
            end if  ! ( debug )

         end if
         if ( ityprd==6 ) then

!        --- minus sign found

            minusg = -1
            ispecl = 9
            call asknxt
            ispecl = ispeclsav
            if ( debug ) then
               write(irefwr,*) 'minus sign found'
               write(irefwr,1) 'ityprd/3, lastitem', ityprd, lastitem
               if ( ityprd==4 ) &
                  write(irefwr,*) 'string ', string(1:nchar)
            end if  ! ( debug )

         end if

         if ( debug ) &
            write(irefwr,*) 'subkey(1:numkey) ', subkey(1:numkey)

         if ( subkey(1:numkey)=='V' ) then

!        --- Special case, second keyword is V, so it may also be
!            followed by a vectorname

            if ( ityprd==4 ) then

               if ( string(1:numkey)/=subkey(1:numkey) ) then

!              --- String found is not V, check if it is a vectorname

                  call readnamevec
                  if ( debug ) &
                     write(irefwr,1) 'ityprd/3a, lastitem', &
                                      ityprd, lastitem
                  if ( ityprd==2 ) then

!                 --- Vectorname found

                     readnext = .false.
                     if ( debug ) write(irefwr,1) 'realrd', nint(realrd)

                  else

!                 --- No vectorname found

                     call errchr ( code(icode), 1 )
                     call errchr ( string(1:nchar), 2 )
                     call errchr ( subkey(1:numkey), 3 )
                     call errsub ( 1461, 0, 0, 3 )
                     call errcrd
                     finished = .true.

                  end if  ! ( ityprd==2 )

               end if  ! ( string(1:numkey)/=subkey(1:numkey) )

            else if ( ityprd==2 ) then

!           --- Number found

               readnext = .false.

            else

!           --- Not a keyword found

               call errchr ( code(icode), 1 )
               call errchr ( subkey(1:numkey), 2 )
               call errsub ( 1914, 0, 0, 2 )
               call errcrd
               finished = .true.

            end if  ! ( ityprd==4 ) then

         else if ( ityprd/=4 ) then

!        --- Not a keyword found

            call errchr ( code(icode), 1 )
            call errchr ( subkey(1:numkey), 2 )
            call errsub ( 1914, 0, 0, 2 )
            call errcrd
            finished = .true.

         else if ( string(1:numkey)/=subkey(1:numkey) ) then

!        --- incorrect keyword found

            call errchr ( code(icode), 1 )
            call errchr ( string(1:nchar), 2 )
            call errchr ( subkey(1:numkey), 3 )
            call errsub ( 1461, 0, 0, 3 )
            call errcrd
            finished = .true.

         end if
         if ( readnext .and. .not. finished ) then

!        --- Check if string contains numbers, use name only

            call readsplitkeyw ( numchar, keyword )
            lastitem = lastitembef+1   ! Only string is used
            nchar = numchar

         end if  ! ( readnext .and. .not. finished )

      else

!     --- icode = 14, set subkey

         subkey = code(icode)
         numkey = ncode(icode)

      end if

      if ( iposut>nout ) then
         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )
         go to 1000
      end if
      table(icode) = iposut+1

!     --- Read part (Ci, Cj, ... ) and store in tabout

      if ( iposut>nout ) then
         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )
         go to 1000
      end if  ! ( iposut>nout )

      call intx02curves ( code, tabmax, icode, finished, readnext, &
                          ispeclsav, tabout, nout, minusg, subkey, &
                          numkey, jump, iposut, ncode(icode), tabmin, &
                          readnx, num )

      if ( itype(icode)==31 ) tabmin(icode) = ivalrd

      ihelp(icode) = num
      tabmax(icode) = num

1000  call erclos ( 'intx0218' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         write(irefwr,*) 'End intx0218'

      end if

      end
