      subroutine mshsortel3d ( kmesh, coor, kmeshj, kmeshk, &
                               kmeshac, rmeshac, lenkmeshac )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 22-08-2012 Remove iwork form parameter list
!        version  1.2  date 29-02-2012 Use subroutine prflgridnod3d
!        version  1.1  date 16-03-2001 Allow nsplit=0
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh part ac with information of in which part of the
!     region an element is positioned
!     Three-dimensional grid
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     sort
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cmcdprf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kmeshj(*), kmeshk(*), kmeshac(*), lenkmeshac
      double precision coor(3,*), rmeshac(6,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshac        o    Integer array containing the information of the
!                         subdivision of the mesh in cells
!                         kmeshac is organized as follows:
!                         Part 1: length (ndim+2) x nsplit
!                            For each discontinuous part ndim+2 positions
!                            Pos. 1 number of nodes in part
!                            pos. 2-ndim+1: nx,ny,nz i.e. number of cells in
!                                           each direction
!                            pos. ndim+2: starting address w.r.t. part 2
!                         For each discontinuous part we have a separate
!                         part 2 and part 3
!                         Part 2: length 1+nx ny nz starting address of
!                            nodes in part 3 for each cell
!                            So in 2d the number of nodes in cell (i,j) is
!                            kmeshac_2(k+1)-kmeshac_2(k), with k=(i-1)*ny+j
!                         Part 3: length nnodes contains the node numbers
!                            in the sequence defined by part 2
!     kmeshj         i    KMESH part j, contains the new numbering of the nodes
!                         kmeshj(1) contains the first node according to the new
!                         numbering; kmeshj(2) the second one and so on
!     kmeshk         i    KMESH part k, contains information about the levels
!                         and sub parts of the mesh
!     lenkmeshac     o    Length of array kmeshac
!     rmeshac        o    Real array of length 2 x ndim x nsplit
!                         contains xmin, xmax, ymin, ymax, zmin zmax
!                         for each subpart
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, nx, nz, ny, nsplit, ifirstnode, ilastnode, &
              isplit, kelmj, nlevel, nnodes, error, npoint
      integer, allocatable :: iwork(:)
      double precision x, y, z, xmin, xmax, ymin, ymax, zmin, zmax
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     ifirstnode     First node in a part
!     ilastnode      Last node in a part
!     isplit         Sequence number of discontinuous part
!     istart         First position used at a part
!     iwork          integer work array to store nodes in a set
!     kelmj          Indication if KMESH part j has been filled (>0)
!                    or not (0)
!                    If KELMJ>0, the value gives the infor entry f KMESH
!                    part j in KMESH
!     nlevel         Number of levels stored in levels
!     nnodes         Number of nodes
!     npoint         Number of nodal points
!     nsplit         Number of parts into which the mesh has been splitted
!     nx             number of points in x direction
!     ny             number of points in y direction
!     nz             number of points in z direction
!     x              x co-ordinate of point
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     y              y co-ordinate of point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     z              z co-ordinate of point
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRFLGRIDNOD3D  Fill information of a set of nodes in rectangular 3d grid
!     PRINKMESHAC    Print contents of kmesh part AC
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear pointer array
!     nsplit := kmesh(44)
!     For isplit := 1 (1) nsplit do
!        Compute xmin, xmax, ymin, ymax for region
!        Store all nodes of part into iwork(1,.)
!        nx := min(100,sqrt(number of nodes))
!        dx := (xmax-xmin)/dx
!        For all nodes in part do
!           Compute i,j index for node and store in iwork(2-3,.)
!           Adapt Pointer array pos k = (i-1)*nx+j
!        Accumulate pointer array
!        For all nodes in part do
!           Fill node in second part of kmeshac
!           Adapt pointer array
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsortel3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsortel3d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nlevel = kmesh(7)
      npoint = kmesh(8)
      kelmj  = kmesh(24)
      nsplit = max(1,kmesh(44))
      ifirstnode = 0
      istart = 5*nsplit

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear kmeshac part 1

      kmeshac(1:istart) = 0
      istart = istart+1

!     --- For isplit := 1 (1) nsplit do

      do isplit = 1, nsplit

!     --- iwork(1,.) := all nodes in region isplit

         if ( nsplit==1 ) then

!        --- The region consists of one part only

            nnodes = kmesh(8)
            do i = 1, nnodes
               iwork(i) = i
            end do

         else

!        --- The region consists of more parts

            ilastnode = kmeshk(nlevel+isplit)
            nnodes = ilastnode-ifirstnode
            if ( kelmj==0 ) then

!           --- No renumbering of nodes

               do i = 1, nnodes
                  iwork(i) = ifirstnode+i
               end do

            else

!           --- Nodes are renumbered

               do i = 1, nnodes
                  iwork(i) = kmeshj(ifirstnode+i)
               end do

            end if
            ifirstnode = ilastnode

         end if

!        --- Compute xmin, xmax, ymin, ymax, zmin and zmax

         xmin = rinfin
         ymin = rinfin
         zmin = rinfin
         xmax =-rinfin
         ymax =-rinfin
         zmax =-rinfin
         do i = 1, nnodes
            x = coor(1,iwork(i))
            y = coor(2,iwork(i))
            z = coor(3,iwork(i))
            xmin = min(xmin,x)
            ymin = min(ymin,y)
            zmin = min(zmin,z)
            xmax = max(xmax,x)
            ymax = max(ymax,y)
            zmax = max(zmax,z)
         end do
         rmeshac(1,isplit) = xmin
         rmeshac(2,isplit) = xmax
         rmeshac(3,isplit) = ymin
         rmeshac(4,isplit) = ymax
         rmeshac(5,isplit) = zmin
         rmeshac(6,isplit) = zmax

!        --- nx := min(100,sqrt(number of nodes))
!            dx := (xmax-xmin)/dx

         x = nnodes
         nx = max(1,nint(x**(1d0/3d0)))
         nx = min(50,nx)
         ny = nx
         nz = nx
         kmeshac((isplit-1)*5+1) = nnodes
         kmeshac((isplit-1)*5+2) = nx
         kmeshac((isplit-1)*5+3) = ny
         kmeshac((isplit-1)*5+4) = nz
         kmeshac((isplit-1)*5+5) = istart

!        --- Fill information of a set of nodes in rectangular 3d grid

         call prflgridnod3d ( kmeshac, nx, ny, nz, nnodes, coor, &
                              iwork, istart, xmin, xmax, ymin, ymax, &
                              zmin, zmax )
         istart = istart+nx*ny*nz+1+nnodes

      end do
      lenkmeshac = istart-1
      if ( debug ) then

!     --- debug is true, print kmesh part ac

         call prinkmeshac ( 3, nsplit, kmeshac, rmeshac )

      end if

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'mshsortel3d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsortel3d'

      end if  ! ( debug )

      end
