      subroutine prcopsepvec2 ( ibuffr, buffer, isol, kprob, kmesh, &
                                idegfd, func, nnodes, nodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy one degree of freedom of sepran vector in user vector
!     Only nodes in array nodes are used
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
!     sepran_vector
!     user_vector
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: kmesh(*), kprob(*), idegfd, isol(5), &
                             nnodes, nodes(nnodes)
      integer, intent(inout) :: ibuffr(*)
      double precision, intent(out) :: func(nnodes)
      double precision, intent(inout) :: buffer(*)

!     buffer        i/o   Buffer array
!     func           o    Contains the function values
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     idegfd         i    Degree of freedom sequence number
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: indprf, indprp, nunkp, nphys, npoint
      integer (kind=8) :: ipusol, ipkprp
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     ipkprp         Starting address of array KPROB part p
!     ipusol         Starting address of solution array
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     npoint         Number of nodal points
!     nunkp          Maximum number of unknowns in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) :: inidgttext, inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI061         compute pointers to degrees of freedom of vectors
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     PRCOPSEPVEC3   Copy one degree of freedom of sepran vector in user vector
!     PRCOPYVEC2US   Copy part of sepran vector to user vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopsepvec2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopsepvec2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initializations

      npoint = kmesh(8)

!     --- Activate array isol and related arrays

      call ini061 ( ibuffr, isol, kprob, indprf, indprp, nunkp, nphys )

!     --- Compute starting addresses

      ipusol = inidgttext ( isol(1), 'usol' )

      if ( indprf==0 ) then

!     --- indprf = 0, constant number of degrees per point

         call prcopyvec2us ( buffer(ipusol), func, nodes, nnodes, nunkp, &
                             idegfd )

      else

!     --- indprf>0, variable number of degrees per point

         ipkprp = inigettext ( indprp, 'kprobp' )
         call prcopsepvec3 ( func, buffer(ipusol), ibuffr(ipkprp), &
                             npoint, idegfd, nodes, nnodes )

      end if  ! ( indprf==0 )

1000  call erclos ( 'prcopsepvec2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopsepvec2'

      end if  ! ( debug )

      end
