      subroutine prfillnodsurf1 ( kmesh, isrf1, isrf2, kmeshn, nodes, &
                                  nnodes, kmesht, kmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for set of surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), isrf1, isrf2, kmeshn(*), nodes(*), &
              nnodes, kmesht(*), kmeshc(*)

!     isrf1          i    First surface number
!     isrf2          i    Last surface number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     kmesht         i    kmesh part t, contains element numbers of elements
!                         with highest dimensionality
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsurfs, nsurfaces, i, error
      integer, allocatable :: surfaces(:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     nsurfaces      Number of surfaces in array surfaces
!     nsurfs         number of surfaces
!     surfaces       Contains the surface numbers to be used for filling nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSURFSNOD1    Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!     ERSURFSNOD2    Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     155   first surface number has incorrect value
!     156   last surface number has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillnodsurf1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillnodsurf1'
         write(irefwr,1) 'isrf1, isrf2', isrf1, isrf2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nsurfs = kmesh(12)

      if ( isrf1<=0 .or. isrf1>nsurfs ) then
         call errint ( isrf1, 1 )
         call errint ( nsurfs, 2 )
         call errsub ( 155, 2, 0, 0 )
         isrf1 = 1
      end if

      if ( isrf2<isrf1 .or. isrf2>nsurfs ) then
         call errint ( isrf2, 1 )
         call errint ( isrf1, 2 )
         call errint ( nsurfs, 3 )
         call errsub ( 156, 3, 0, 0 )
         isrf2 = isrf1
      end if

      nsurfaces = isrf2-isrf1+1

      allocate ( surfaces(nsurfaces), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nsurfaces, 'surfaces' )
      if ( ierror/=0 ) go to 1000

      do i = 1, nsurfaces
         surfaces(i) = isrf1+i-1
      end do  ! i = 1, nsurfaces

      if ( debug ) call prinin ( surfaces, nsurfaces, 'surfaces' )

      nnodes = 0

      if ( kmesh(28)>0 ) then

!     --- kmeshn has been filled

         call ersurfsnod1 ( nsurfaces, surfaces, kmeshn, kmesh, &
                            nodes, nnodes )

      else

!     --- kmeshn has not been filled

         call ersurfsnod2 ( nsurfaces, surfaces, kmesht, kmesh, &
                            nodes, nnodes, kmeshc )

      end if  ! ( kmesh(28)>0 )

      deallocate ( surfaces, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'surfaces' )

1000  call erclos ( 'prfillnodsurf1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillnodsurf1'

      end if  ! ( debug )

      end
