      subroutine prfillnodsurf3 ( kmesh, kmeshn, nodes, nnodes, numnod, &
                                  ihelp, iouterboun )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for all outer surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kmeshn(*), nodes(*), &
              nnodes, numnod, ihelp(*), iouterboun(*)

!     ihelp          i    contains the realtive node numbers per element
!                         that should be used in case numnod>0
!     iouterboun     i    Either array ioutercurvs or ioutersurfs
!                         Contains outer curves or surfaces in the following way
!                         Pos. 1  nparts  Number of parts the outer boundary
!                                         consists of
!                         Pos. 2-1+nparts Starting addresses of each part
!                         Per part: (first address: ioutercurvs(1+i)
!                         Pos. 1: nouter  Number of curves in part
!                         followed by the curve numbers (provided with a sign)
!                         These curve numbers are consecutive, which means
!                         the end of one curve is the start of the new one
!                         The part itself is closed
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
!     numnod         i    If>0, only numnod nodes per element must be used
!                         In that case ihelp is used
!                         If 0 all nodes are used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsurfaces, error, ipart, istart, isurnr, j, nboun, nparts
      integer, allocatable :: surfaces(:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     ipart          Sequence number of part
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     j              Counting variable
!     nboun          Number of boundary points at an element
!     nparts         number of parts the boundary consists of
!     nsurfaces      Number of surfaces in array surfaces
!     surfaces       Contains the surface numbers to be used for filling nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERSURFSNOD3    Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!                    It is assumed that kmesh part n has been filled
!                    Only a restricted number of nodes per element is used
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillnodsurf3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillnodsurf3'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nsurfaces = 0

      nparts = iouterboun(1)
      if ( debug ) write(irefwr,1) 'nparts', nparts

      do ipart = 1, nparts

         istart = iouterboun(ipart+1)
         nboun = iouterboun(istart)
         nsurfaces = nsurfaces+nboun

      end do  ! ipart = 1, nparts

      allocate ( surfaces(nsurfaces), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nsurfaces, 'surfaces' )
      if ( ierror/=0 ) go to 1000

      nsurfaces = 0
      do ipart = 1, nparts

         istart = iouterboun(ipart+1)
         nboun = iouterboun(istart)
         do j = 1, nboun
            isurnr = abs(iouterboun(istart+j))
            nsurfaces = nsurfaces+1
            surfaces(nsurfaces) = isurnr
         end do  ! j = 1, nboun

      end do  ! ipart = 1, nparts

      if ( debug ) call prinin ( surfaces, nsurfaces, 'surfaces' )

      call ersurfsnod3 ( nsurfaces, surfaces, kmeshn, kmesh, &
                         nodes, nnodes, numnod, ihelp )

      deallocate ( surfaces, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'surfaces' )

1000  call erclos ( 'prfillnodsurf3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillnodsurf3'

      end if  ! ( debug )

      end
