      subroutine prschurmethod ( ibuffr, buffer, qmat, gmat, intmat,
     +                           ru, inpsol, qmatpre, dmat, projvc,
     +                           rmatpre, rinsol, u, normrhsd,
     +                           infmatvec, ninpsol1, nrinsol1,
     +                           intmatpre, kprob, cmat )
! ======================================================================
!
!        programmer    Mehfooz ur Rehman
!        version  2.1  date 20-08-2009 Better termination criterion
!        version  2.0  date 28-04-2009 Extra parameter cmat
!        version  1.1  date 22-12-2008 New subroutine prschuriter
!        version  1.0  date 07-11-2008
!
!   copyright (c) 2008-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve a system of linear equations of special form using
!     the schur complement method
!     The method is accelerated by the GCR process
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     matrix
!     real
!     krylov
!     conjugate_gradients
!     schur_method
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cinout'
      include 'SPcommon/cconst'
      include 'SPcommon/citerinfo'
      include 'SPcommon/cdctim'
      include 'SPcommon/cmcdpr'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intmat(*), inpsol(*), infmatvec(*), ibuffr(*),
     +        ninpsol1, nrinsol1, intmatpre(*), kprob(*)
      double precision qmat(*), gmat(*), ru(*), dmat(*), buffer(*),
     +                 qmatpre(*), projvc(*), rmatpre(*),
     +                 rinsol(*), u(*), normrhsd, cmat(*)

!     buffer        i/o   Buffer array
!     cmat           i    Matrix C (Pressure part of large matrix)
!     dmat           i    Matrix D (divergence matrix)
!     gmat           i    Matrix G (Gradient matrix)
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     infmatvec      i    Array to indicate information about the matrix-vector
!                         multiplication and other solver related quantities
!                         See subroutine prsimple for a description
!     inpsol         i    Integer input array for subroutine solvel
!                         See read03 for a description
!     intmat         i    First part of matrix structure in case of Schur
!                         storage.
!                         The description can be found in incommatsimple
!                         as intmatnew
!     intmatpre      i    See intmat, now with respect to preconditioner
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     ninpsol1       i    Length of array inpsol per (sub)equation
!     normrhsd       i    Norm of right-hand side vector
!     nrinsol1       i    Length of array rinsol per (sub)equation
!     projvc         i    Double precision array containing the projection
!                         vectors.
!                         See subroutine prpjspar4 for a description
!     qmat           i    Matrix Q
!     qmatpre        i    Preconditioning of matrix Q
!     rinsol         i    Contains information concerning the linear solver
!                         See the Programmers Guide for a description
!     rmatpre        i    Preconditioning of matrix R
!     ru            i/o   Residual of system of equations
!     u             i/o   Solution vector
!                         v during the iteration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, i, nusol, ireler, iprint, iter, nu, np,
     +        maxiter, itimesav, niteruacc, niterpacc,
     +        niteracc, niterutot, niterptot, error,
     +        iterations(4), ipinnercontr, ipirowd, ipicold
      double precision, allocatable ::  rua(:), un(:), rucc(:),
     +                                  diagpmm(:), ucop(:)
      double precision norm, eps, normu, normp, timebefore, rferr
      logical debug, matlab, converged, scaling

!     converged      If true the process is converged
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     diagpmm        Work array to store the diagonal of the pressure
!                    scaling matrix to check the accuracy
!     eps            Local accuracy
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     ipicold        Starting address of column numbers of D matrix
!     ipinnercontr   Starting address of array innercontr
!     ipirowd        Starting address of number of non-zero entries per row
!                    of D matrix (accumulated)
!     iprint         Print level for information
!     ireler         Defines how the error is measured
!     iter           Iteration sequence number
!     iterations     ?
!     itimesav       Help parameter to save the parameter itime
!     k              Counting variable
!     matlab         If true matlab input is generated
!     maxiter        Maximum number of iterations
!     niteracc       Accumulated number of GCR iterations
!     niterpacc      Accumulated number of iterations to solve the p-equations
!     niterptot      Total number of p iterations in all GCR steps
!     niteruacc      Accumulated number of iterations to solve the u-equations
!     niterutot      Total number of u iterations in all GCR steps
!     norm           norm of residual
!     normp          norm of p-vector
!     normu          norm of u-vector
!     np             Number of pressure unknowns
!     nu             Number of velocity unknowns
!     nusol          Total number of unknowns
!     rferr          Maximal accuracy possible for this matrix, due to
!                    round-off
!     rua            Actual residual at input
!     rucc           Work array to store ru temporarily
!     scaling        If true the residual is scaled before computing the
!                    termination criterion
!     timebefore     Time at entrance of subroutine
!     ucop           Copy of start vector
!     un             Update of u, also used as help vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision prpardnrm2

!     ASKCLK         print elapsed time
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRINRL         Print 1d real vector
!     PRPARDNRM2     Parallel Blas routine to compute the 2-norm of a vector
!     PRSCHURITER    schur complement iteration as part of some outer loop
!     PRSCHURSCALR   Compute the scaling matrix for the residual
!                    for the PMM approach
!     PRSIMPLEMATVEC Compute the matrix vector multiplication A x = y for the
!                    simple algorithm where the matrix A is stored in 3 parts
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
!     677   The maximal accuracy with respect to the condition of the
!           matrix is reached
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The system of equations to be solved reads:
!
!     | Q    G | |u|  = |f_u|
!     | D    C | |p|    |f_p|
!
!     input:  Q, G, D, f_u, f_p
!     output: du, dp
!
!     In the Schur method this system is rewritten as:
!
!     | Q      0    | | I  Q^-1G | |u|  = |f_u|
!     | D  C-DQ^-1G | | 0    I   | |p|    |f_p|,
!
!     where the matrix -DQ^-1G is called the Schur complement matrix
!
!     Define    | u* |  =  | I  Q^-1G | |u|
!               | p  |     | 0    I   | |p|
!
!     Then the system can be solved in the following 3 steps:
!
!           Q u* = f
!      -DQ^-1G p = f_p = D u*
!              u = u* - Q^-1 G p
!
!     Here we solve each of the substeps inaccurately, with the following
!     algorithm
!
!     Start:  given u^0 and p^0
!             ru = f_u - Qu^0 - Gp^0;  r_p = f_p - Du^0
!
!     For k := 0 to maxiter do
!           Q u_f  = ru           inaccurately
!       -DQ^-1G p* = rp - D uf    inaccurately by GCR
!            Q ul  = G p*         inaccurately
!        u^k+1 = u^k + uf-ul
!        p^k+1 = p^k + p*
!        ru = f_u - Qu^k+1 - Gp^k+1;   r_p = f_p - Du^k+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data niteruacc /0/, niterpacc /0/, niteracc /0/
! ======================================================================
!
      call eropen ( 'prschurmethod' )
      call ersettime ( timebefore )

      nu = infmatvec(11)
      np = infmatvec(12)
      allocate(rua(nu+np), un(nu+np), rucc(nu+np), diagpmm(np),
     +         ucop(nu+np), stat = error)
      if ( error/= 0 ) then

!     --- error#0, so not enough space

         call errint ( 3*(nu+np),1 )
         call errint ( error, 2 )
         call errchr ( 'rua', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

      debug = .false.
      matlab = .false.
      scaling = .true.      ! the residual is scaled to get a better termination

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prschurmethod'
         write(irefwr,1) 'nu, np', nu, np
         call prinrl ( ru, nu+np, 'ru' )
         call prinrl ( rmatpre, np, 'rmatpre' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      converged = .false.
      itimesav = itime
      itime = 0
      if ( matlab ) then
         open ( unit=19, file = 'fu.dat' )
         do i = 1, nu
            write(19,10) ru(i)
 10         format ( e15.8 )
         end do
         close ( unit=19 )
         open ( unit=19, file = 'fp.dat' )
         do i = 1, np
            write(19,10) ru(nu+i)
         end do
         close ( unit=19 )
      end if
      ipinnercontr = infmatvec(4)     ! starting address innercontr

      nusol = nu+np
      norm = prpardnrm2 (  nusol, ru, 1, ibuffr(ipinnercontr) )
      normu = prpardnrm2 (  nu, ru, 1, ibuffr(ipinnercontr) )
      normp = prpardnrm2 (  np, ru(nu+1), 1, ibuffr(ipinnercontr+nu) )
      rferr = 1d3*normrhsd*epsmac

      if ( debug )
     +   write(irefwr,2) 'norm, normu', norm, normu

!     --- Check if accuracy has been reached

      ireler = inpsol(11)
      if ( ireler==10 ) ireler = 1
      eps = rinsol(1)

      maxiter = inpsol(5)
      if ( maxiter==0 ) maxiter = nusol
      if ( debug ) write(irefwr,*) 'ireler, eps', ireler, eps
      if ( ireler==-1 ) then

!     --- Relative error compared to rhs

         eps = eps * normrhsd

      else if ( ireler==1 ) then

!     --- Relative error compared to initial residual

         eps = eps * norm

      end if
      if ( ireler/=0 .and. norm<=eps ) then

!     --- Initial residual already small

         go to 1000

      end if
      iprint = inpsol(6)
      niterutot = 0
      niterptot = 0

      if ( scaling ) then

!     --- scaling, create scaling matrix for P part

         ipirowd  = intmat(2)
         ipicold  = ipirowd+np+1
         call prschurscalr ( diagpmm, np, dmat, qmat, intmat(ipirowd),
     +                       intmat(ipicold) )
         if ( ireler==1 ) eps = eps/norm   ! reset eps

!        --- Since we are using scaling the initial residual should be scaled
!            too
!            Compute scaled norm

         rucc = ru(1:nu+np)/qmat(1:nu)
         rucc(nu+1:nu+np) = ru(nu+1:nu+np)/diagpmm(1:np)
         norm = prpardnrm2 ( nu+np, rucc, 1, ibuffr(ipinnercontr) )

         if ( iprint>=2 ) then
            normu = prpardnrm2 (  nu, rucc, 1, ibuffr(ipinnercontr) )
            normp = prpardnrm2 (  np, rucc(nu+1), 1,
     +                            ibuffr(ipinnercontr+nu) )
         end if  ! ( iprint>=2 )

         if ( ireler==1 ) eps = eps*norm   ! new eps

      end if  ! ( scaling )

!     --- Print heading if iprint>0

      if ( iprint>0 ) then
             write ( irefwr, 100 )   ! simple
100         format(//' Output for the Schur Method'//)
      end if  ! ( iprint>0 )

      if ( iprint==1 ) then

!     --- iprint = 1,  Print information

         write ( irefwr, 110 ) norm
110      format ( ' The 2-norm of the initial residual', ss, 1pe17.2e2 )

      else if ( iprint>=2 ) then

!     --- iprint>=2,  Maximal output

         if ( .not. scaling )
     +      normp = prpardnrm2 ( np, ru(nu+1), 1,
     +                           ibuffr(ipinnercontr+nu) )
         write ( irefwr, 120 ) 0, norm, normu, normp, 0, 0,0,0
120      format ( '  Iter    residual  res_u     res_p      iter_u',
     +            '   iter_p, iter_u(in P),iter_u '/
     +            ' number   |r_k|_2   |ru_k|_2  |rp_k|_2' //
     +              i5, 3x, ss, 3(1pe10.2e2), 4i8 )

      end if  ! ( iprint==1 )

      iter = 0

!     --- rua is actual right hand side, un = contains update of u and p

      rua = ru(1:nu+np)
      ucop = u(1:nu+np)      ! start vector
      u(1:nu+np) = 0d0       ! make start vector zero

!     --- For k := 0 (1) maxiter-1 do

      do k = 0, maxiter-1

         iter = iter+1
         un = 0d0

!        --- Carry out the three steps in the algorithm
!            The result is stored in un

         call prschuriter ( ibuffr, nu, np, qmat, gmat,
     +                      dmat, intmat, ru, un, inpsol,
     +                      qmatpre, projvc, rmatpre,
     +                      rinsol, infmatvec, ninpsol1, nrinsol1,
     +                      intmatpre, kprob, iterations )

         if ( ierror/=0 ) go to 1000

!        --- Update u

         u(1:nu+np) = un + u(1:nu+np)
         if ( debug ) call prinrl ( un, nu+np, 'un' )

!        --- Multiply the complete matrix by u
!            The result is stored in un

         call prsimplematvec ( ibuffr, buffer, nu, np, qmat,
     +                         gmat, dmat, u, un, intmat, infmatvec,
     +                         cmat )

!        --- Update the residual

         ru(1:nu+np) = rua - un

         if ( scaling ) then

!        --- scaling = true, copy ru into rucc and adapt

            rucc = ru(1:nu+np)
            ru(1:nu) = ru(1:nu)/qmat(1:nu)
            ru(nu+1:nu+np) = ru(nu+1:nu+np)/diagpmm(1:np)

         end if  ! ( scaling )

!        --- Compute norms

         norm = prpardnrm2 ( nu+np, ru, 1, ibuffr(ipinnercontr) )
         normu = prpardnrm2 (  nu, ru, 1, ibuffr(ipinnercontr) )

         if ( ierror/=0 ) go to 1000

         if ( iprint>=2 ) then

            normp = prpardnrm2 (  np, ru(nu+1), 1,
     +                            ibuffr(ipinnercontr+nu) )
            write ( irefwr, 400 ) iter, norm, normu, normp,
     +             iterations(1), iterations(3),iterations(2),
     +             iterations(4)
400         format ( i5, 3x, ss, 3(1pe10.2e2), 4i8 )

         end if

         if ( scaling ) then

!        --- scaling = true, reset ru

            ru(1:nu+np) = rucc

         end if  ! ( scaling )

         niterutot = niterutot+iterations(1)+iterations(2)+iterations(4)
         niterptot = niterptot+iterations(3)

         converged = norm<=eps
         if ( normu<rferr .and. iter>3 ) then

!        --- maximum accuracy has been reached

            call erreal ( rferr, 1 )
            call errwar ( 677, 0, 1, 0 )
            converged = .true.

         end if  ! ( normu<rferr .and. iter>3 ) then

         if ( converged ) go to 500

      end do  ! k = 1, maxiter

!     --- If the subroutine reaches this stage, too much iterations have
!         been carried out

      call errint ( iter, 1 )
      if ( inpsol(22)==1) then
         inpsol(1) = iter
         call errwar ( 676, 1, 0, 0 )
      else
         call errsub ( 676, 1, 0, 0 )
      end if

500   niteracc = niteracc + iter
      niteruacc = niteruacc + niterutot
      niterpacc = niterpacc + niterptot
      u(1:nu+np) = ucop + u(1:nu+np)      ! add start vector to u

      if ( iprint>=1 ) then

!     --- iprint = 1  Print information

         write ( irefwr, 510 ) iter, norm, niterutot, niterptot,
     +                         niteracc, niteruacc, niterpacc
510      format('The number of executed iterations in the',
     +          ' Schur Method is', i5/
     +          'The 2-norm of the residual is:', ss,1pe12.2e2/
     +          'The number of u iterations is ', i6/
     +          'The number of p iterations is ', i6/
     +          'The accumulated number of GCR-iterations is', i8/
     +          'The accumulated number of u-iterations is', i8/
     +          'The accumulated number of p-iterations is', i8 )

      end if

!     --- Fill information in citerinfo
!         This is only used for performance checking

      niterout = niteracc
      niteruout = niteruacc
      niterpout = niterpacc
      call askclk
      timesubr = tim3-timebefore

1000  deallocate ( rua, un, rucc, diagpmm, ucop, stat = error )

      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'rua', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

      call erclos ( 'prschurmethod' )
      if ( debug ) write(irefwr,*) 'End prschurmethod'
      itime = itimesav

      end
