      subroutine prvecchecklen ( isol, iseqvec, vecs, numvecrhs, kmesh, &
                                 nunkp, completevec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if all vectors at right-hand side have the same length as
!     the vector on the left-hand side
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     vector
!     expression
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(5,*), iseqvec, vecs(*), numvecrhs, kmesh(*), nunkp
      logical completevec

!     completevec   i/o   If true the complete vector must be filled
!     iseqvec        i    Sequence number of left-hand side vector in isol
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     numvecrhs      i    Number of vectors in right-hand side
!                         Length of array vecs
!     nunkp          i    Maximum number of unknowns in a point
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, lenvec, length, ndim, npoint, iseqv
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     iseqv          Sequence number of vector in isol with respect to
!                    vector in vecs
!     length         Help parameter to indicate the length of arrays
!     lenvec         Length of left-hand side vector in expression
!     ndim           Dimension of the space.
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3163   Vector has wrong sequence number
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecchecklen' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecchecklen'
         write(irefwr,1) 'iseqvec, maxvc1', iseqvec, maxvc1
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lenvec = isol(5,iseqvec)  ! length of array iseqvec
      ndim = kmesh(6)           ! dimension of space
      npoint = kmesh(8)         ! number of nodes in mesh

      if ( debug ) &
         write(irefwr,1) 'lenvec, npoint, ndim', lenvec, npoint, ndim

!     --- Loop over all vectors of right-hand side

      do i = 1, numvecrhs

         iseqv = vecs(i)

         if ( debug ) write(irefwr,1) 'iseqv', iseqv
         if ( iseqv<=maxvc1 ) then

!        --- Standard sepran vector

            length = isol(5,iseqv)

         else if ( iseqv==maxvc1+1 ) then

!        --- Coordinates

            length = ndim*npoint

         else if ( iseqv<=maxvc1+1+ndim ) then

!        --- One coordinate

            length = npoint

         else

!        --- Unknown vector

            call errint ( iseqv, 1 )
            call errchr ( names(iseqvec), 1 )
            call errsub ( 3163, 1, 0, 1 )
            go to 1000

         end if

         if ( length/=lenvec .and. length*nunkp/=lenvec ) then

!        --- Not all vectors at the right-hand side have the same
!            length as the one on the right-hand side
!            Treat as not complete vector

            completevec = .false.
            go to 1000

         end if  ! ( length/=lenvec .and. length*nunkp/=lenvec )

      end do  ! i = 1, numvecrhs

1000  call erclos ( 'prvecchecklen' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prvecchecklen'

      end if  ! ( debug )

      end
