      subroutine prveccombinat7 ( ibuffr, buffer, isol, iseqvec, vecs, &
                                  numvecrhs, kmesh, kprob, idegfd, &
                                  istore, store, icount, numrealrhs, &
                                  nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate right-hand side of vector expression
!     Special case: a part of the real vector must be computed
!                   the number of unknowns per point is constant
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     expression
!     real
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isol(5,*), iseqvec, vecs(*), numvecrhs, &
              kmesh(*), kprob(*), istore(*), icount, numrealrhs, &
              idegfd, nnodes, nodes(nnodes)
      double precision buffer(*), store(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     icount         i    Computed length of array istore
!     idegfd         i    Degree of freedom sequence number
!     iseqvec        i    sequence number of vector in left-hand side with
!                         to isol
!     isol           i    Standard SEPRAN array containing information of the
!                         respect solution array
!     istore         i    Contains the expression as stored in common
!                         cexpression, where all scalars and reals
!                         have been computed and stored in store
!                         References to these values are given by
!                         their position in store plus 1000
!                         The values of the vectors are stored in
!                         array vectors
!                         References to these vectors are given by
!                         their position in vectors plus 10000
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Contains the node numbers that must be filled
!     numrealrhs     i    Number of reals stored in store
!     numvecrhs      i    Number of vectors in right-hand side
!                         Length of array vecs
!     store          i    Contains real information of the expression
!                         See subroutine READEXPR, array work
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: indprf, indprp, nunkp, nphys, lenvec, error, npoint
      double precision, allocatable :: outvec(:), vectors(:,:)
      integer (kind=8) :: ipvec, ipkprp
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     ipkprp         Starting address of array KPROB part p
!     ipvec          Starting address of output vector in buffer
!     lenvec         Length of left-hand side vector in expression
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     npoint         Number of nodal points
!     nunkp          Maximum number of unknowns in a point
!     outvec         Output vector; result of expression in the nodes of nodes
!     vectors        Array of size lenvec x numvecrhs to store the vectors
!                    of the right-hand side in the nodes required
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext, inigettext

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INI061         compute pointers to degrees of freedom of vectors
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     PRCOPYUS2VEC2  Copy user vector to sepran vector
!                    The user vector has a variable number of degrees of
!                    freedom per point
!     PREVALVECEXPR  Evaluate vector expression and fill actual values of
!                    vectors
!     PRINRL         Print 1d real vector
!     PRVECFILL1     Fill all right-hand side vectors in expression where
!                    either idegfd>0 or nunkp = 1
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3162   Option not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prveccombinat7' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prveccombinat7'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      npoint = kmesh(8)  ! number of nodes

!     --- activate array

      call ini061 ( ibuffr, isol(1,iseqvec), kprob, indprf, &
                    indprp, nunkp, nphys )

      if ( debug ) then
         write(irefwr,1) 'idegfd, nunkp', idegfd, nunkp
      end if  ! ( debug )

      if ( idegfd>0 .or. nunkp==1 ) then

!     --- Only one degree of freedom must be filled

         lenvec = nnodes       ! 1 degree of freedom per point

         allocate ( outvec(lenvec), vectors(lenvec,numvecrhs), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, lenvec, 'outvec' )
         if ( ierror/=0 ) go to 1000

!        --- Fill all right-hand side vectors

         call prvecfill1 ( ibuffr, buffer, isol, iseqvec, vecs, &
                           numvecrhs, kmesh, vectors, lenvec, nodes, &
                           nnodes, kprob, nunkp, idegfd )

!        --- Fill output vector by computing the expression

         call prevalvecexpr ( outvec, vectors, lenvec, &
                              numvecrhs, istore, store, icount, &
                              numrealrhs, names(iseqvec) )

         if ( debug ) call prinrl ( outvec, lenvec, 'outvec' )

!        --- Map output vector in isol
!            Find starting address

         ipvec = inidgttext ( isol(1,iseqvec), names(iseqvec) )
         ipkprp = inigettext ( indprp, 'kprobp' )

!        --- copy

         call prcopyus2vec2 ( buffer(ipvec), outvec, nnodes, &
                              max(1,idegfd), ibuffr(ipkprp), &
                              nodes, npoint )

      else

!     --- More degrees of freedom must be filled

         call errsub ( 3162, 0, 0, 0 )
         go to 1000

      end if  ! ( idegfd>0 )

      if ( allocated(outvec) ) deallocate ( outvec, stat = error )
      if ( allocated(vectors) ) deallocate ( vectors, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'outvec' )

1000  call erclos ( 'prveccombinat7' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prveccombinat7'

      end if  ! ( debug )

      end
