      subroutine prvecfill ( ibuffr, buffer, isol, iseqvec, vecs, &
                             numvecrhs, kmesh, vectors, length, nunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill all vectors of right-hand side of expression in array vectors
!     It is assumed that the complete vectors must be filled
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isol(5,*), iseqvec, vecs(*), numvecrhs, &
              kmesh(*), length, nunkp
      double precision buffer(*), vectors(length,numvecrhs)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iseqvec        i    sequence number of vector in left-hand side with
!                         to isol
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     length         i    Length of the vectors to be filled
!     numvecrhs      i    Number of vectors in right-hand side
!                         Length of array vecs
!     nunkp          i    Maximum number of unknowns in a point
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
!     vectors        o    Array of size length x numvecrhs to store the vectors
!                         of the right-hand side in the nodes required
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ndim, iseqv, npoint, lenvec, j
      integer (kind=8) ipvec
      logical debug, extradebug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     extradebug     If true extra debugging statements are carried out
!     i              Counting variable
!     ipvec          Starting address of output vector in buffer
!     iseqv          Sequence number of keyword V in array code1
!     j              Counting variable
!     lenvec         Length of left-hand side vector in expression
!     ndim           Dimension of the space.
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext

!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3163   Vector has wrong sequence number
!    3165   Not all vectors in the right-hand side of the vector expression
!           have the same length as the left-hand side
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecfill' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecfill'
         write(irefwr,1) 'length, nunkp', length, nunkp
         call prinin ( vecs, numvecrhs, 'vecs' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndim = kmesh(6)           ! dimension of space
      npoint = kmesh(8)         ! number of nodes

!     --- Loop over vectors

      do i = 1, numvecrhs

         iseqv = vecs(i)

         if ( debug ) write(irefwr,1) 'i, iseqv', i, iseqv
         if ( iseqv<=maxvc1 ) then

!        --- Standard sepran vector
!            activate array

            call ini060text ( ibuffr, isol(1,iseqv), names(iseqv) )

!           --- Find starting address

            ipvec = inidgttext ( isol(1,iseqv), names(iseqv) )
            lenvec = isol(5,iseqv)
            if ( debug ) &
               write(irefwr,1) 'length, lenvec', length, lenvec

            if ( lenvec==length ) then

!           --- Both arrays have same length: copy vector

               call dcopy ( length, buffer(ipvec), 1, vectors(1,i), 1 )

            else if ( lenvec*nunkp==length ) then

!           --- Array to be copied has one degree of freedom
!               Copy each component

               do j = 1, nunkp
                  call dcopy ( lenvec, buffer(ipvec), 1, vectors(j,i), &
                               nunkp )
               end do  ! j = 1, nunkp

            else

!           --- Error length not correct

               call errint ( lenvec, 1 )
               call errint ( i, 2 )
               call errint ( length, 3 )
               call errchr ( names(iseqvec), 1 )
               call errchr ( names(iseqv), 2 )
               call errsub ( 3165, 3, 0, 2 )

            end if  ! ( lenvec==length )

         else if ( iseqv==maxvc1+1 ) then

!        --- Coordinates
!            activate array

            call ini060text ( ibuffr, kmesh(23), 'coor' )

!           --- Find starting address

            ipvec = inidgttext ( kmesh(23), 'coor' )

!           --- Copy vector

            if ( length==ndim*npoint ) then

!           --- Coor has same length as vector

               call dcopy ( length, buffer(ipvec), 1, vectors(1,i), 1 )

            else

!           --- Error coor has different length as vector

               call errint ( ndim*npoint, 1 )
               call errint ( i, 2 )
               call errint ( length, 3 )
               call errchr ( names(iseqvec), 1 )
               call errchr ( 'coor', 2 )
               call errsub ( 3165, 3, 0, 2 )

            end if  ! ( length==ndim*npoint )

         else if ( iseqv<=maxvc1+1+ndim ) then

!        --- One of the coordinates
!            activate array

            iseqv = iseqv-maxvc1-1
            call ini060text ( ibuffr, kmesh(23), 'coor' )

!           --- Find starting address

            ipvec = inidgttext ( kmesh(23), 'coor' )+iseqv-1

!           --- Copy vector

            if ( npoint==length ) then

!           --- Both arrays have same length: copy vector

               call dcopy ( length, buffer(ipvec), ndim, vectors(1,i), &
                            1 )

            else if ( npoint*nunkp==length ) then

!           --- Array to be copied has one degree of freedom
!               Copy each component

               do j = 1, nunkp
                  call dcopy ( lenvec, buffer(ipvec), ndim, &
                               vectors(j,i), nunkp )
               end do  ! i = 1, nunkp

            else

!           --- Error length not correct

               call errint ( npoint, 1 )
               call errint ( i, 2 )
               call errint ( length, 3 )
               call errchr ( names(iseqvec), 1 )
               call errchr ( 'coor_x', 2 )
               call errsub ( 3165, 3, 0, 2 )

            end if  ! ( lenvec==length )

         else

!        --- Unknown vector

            call errint ( iseqv, 1 )
            call errchr ( names(iseqvec), 1 )
            call errsub ( 3163, 1, 0, 1 )
            go to 1000

         end if

      end do  ! i = 1, numvecrhs

1000  call erclos ( 'prvecfill' )
      if ( debug ) then

!     --- Debug information

         if ( extradebug ) then
            call prinrl2 ( vectors, length, numvecrhs, 'vectors' )
         else

            do i = 1, numvecrhs
               call prinrl ( vectors(1,i), 5, 'vectors_i' )
            end do  ! i = 1, numvecrhs

         end if  ! ( extradebug )
         write(irefwr,*) 'End prvecfill'

      end if  ! ( debug )

      end
