      subroutine prvecfillc ( ibuffr, buffer, isol, iseqvec, vecs, &
                              numvecrhs, kmesh, vectors, length )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill all vectors of right-hand side of expression in array vectors
!     It is assumed that the complete vectors must be filled
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isol(5,*), iseqvec, vecs(*), numvecrhs, &
              kmesh(*), length
      double precision buffer(*)
      double complex vectors(length,numvecrhs)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iseqvec        i    sequence number of vector in left-hand side with
!                         to isol
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     length         i    Length of the vectors to be filled
!     numvecrhs      i    Number of vectors in right-hand side
!                         Length of array vecs
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
!     vectors        o    Array of size length x numvecrhs to store the vectors
!                         of the right-hand side in the nodes required
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ndim, iseqv
      integer (kind=8) ipvec
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ipvec          Starting address of output vector in buffer
!     iseqv          Sequence number of keyword V in array code1
!     ndim           Dimension of the space.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext

!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3163   Vector has wrong sequence number
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecfillc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecfillc'
         call prinin ( vecs, numvecrhs, 'vecs' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndim = kmesh(6)           ! dimension of space

      vectors = 0d0

!     --- Loop over vectors

      do i = 1, numvecrhs

         iseqv = vecs(i)
         if ( iseqv<=maxvc1 ) then

!        --- Standard sepran vector
!            activate array

            call ini060text ( ibuffr, isol(1,iseqv), names(iseqv) )

!           --- Find starting address

            ipvec = inidgttext ( isol(1,iseqv), names(iseqv) )

!           --- Copy vector

            if ( mod(isol(3,iseqv),10)==1 ) then

!           --- Complex vector

               call dcopy ( 2*length, buffer(ipvec), 1, vectors(1,i), &
                            1 )

            else

!           --- Real vector, skip imaginary part

               call dcopy ( length, buffer(ipvec), 1, vectors(1,i), 2 )

            end if  ! ( mod(isol(3,iseqv),10)==1 )

         else if ( iseqv==maxvc1+1 ) then

!        --- Coordinates
!            activate array

            call ini060text ( ibuffr, kmesh(23), 'coor' )

!           --- Find starting address

            ipvec = inidgttext ( kmesh(23), 'coor' )

!           --- Copy vector

            call dcopy ( length, buffer(ipvec), 1, vectors(1,i), 2 )

         else if ( iseqv<maxvc1+1+ndim ) then

!        --- One of the coordinates
!            activate array

            iseqv = iseqv-maxvc1-1
            call ini060text ( ibuffr, kmesh(23), 'coor' )

!           --- Find starting address

            ipvec = inidgttext ( kmesh(23), 'coor' )+iseqv-1

!           --- Copy vector

            call dcopy ( length, buffer(ipvec), ndim, vectors(1,i), 2 )

         else

!        --- Unknown vector

            call errint ( iseqv, 1 )
            call errchr ( names(iseqvec), 1 )
            call errsub ( 3163, 1, 0, 1 )
            go to 1000

         end if

      end do  ! i = 1, numvecrhs

1000  call erclos ( 'prvecfillc' )
      if ( debug ) then

!     --- Debug information

         call prinrl2 ( vectors, 2*length, numvecrhs, 'vectors' )
         write(irefwr,*) 'End prvecfillc'

      end if  ! ( debug )

      end
