      subroutine readexpress ( openfound, closefound, scalpermitted, &
                               bool, vecpermitted, vecfound )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 12-02-2015 Extra parameter vecfound
!        version  2.0  date 10-01-2015 Replace expression by real if possible
!        version  1.5  date 10-04-2014 New call to readstrfunc
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret part of input as number or expression
!     Store information in istore/store
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cseprdf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/cexpressionf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(in) :: scalpermitted, bool, vecpermitted
      logical, intent(inout) :: openfound, closefound
      logical, intent(out) :: vecfound

!     bool           i    If true the subroutine is called by readboolexpr
!     closefound    i/o   If true a closing bracket has been found, which
!                         corresponds to a closing bracket read earlier
!                         At input closefound must be false
!                         closefound is set to true if openfound is true
!                         and exactly one close bracket extra has been found
!     openfound     i/o   Indicates if an openings bracket is found before
!                         reading the number or expression
!                         openfound must be false for the first call of this
!                         subroutine
!     scalpermitted  i    If true the expression may contain scalars, &
!                         if false not
!     vecfound       o    If true the expression contains vectors
!     vecpermitted   i    If true the expression may contain vectors, &
!                         if false not
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icount, iwork(2), jcount, ispeclsav, numfun, &
                 minusg, iopen, i, numchar, istore(1000), lenstore, &
                 wcount, numexpresssv, lastrealexprsv, icheck, kmesh(9)
      double precision :: store(1000)
      character (len=32) :: keyword
      logical :: debug, finished

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              General loop variable
!     icheck         Indication if expression contains scalars (>100000000)
!     icount         Counter
!     iopen          Number of open brackets minus number of close brackets
!                    found
!     ispeclsav      Help parameter to save the value of ispecl
!     istore         Contains information of the expression
!                    See subroutine READEXPR, array iwork
!     iwork          integer work array
!     jcount         Counter for array istore
!     keyword        contains the keyword without number
!     kmesh          Dummy
!     lastrealexprsv Help parameter to save the value of lastrealexpr
!     lenstore       At input: declared length of array istore
!                    At output number of items stored in istore
!     minusg         Indicates if a minus sign has been found (-1) or
!                    not (1)
!     numchar        Number of characters in keyword
!     numexpresssv   Help parameter to save the value of numexpress
!     numfun         Sequence number of function
!     store          Contains real information of the expression
!                    See subroutine READEXPR, array work
!     wcount         Number of items stored in store
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PREVALEXPR     Evaluate expression and fill actual values of scalars
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
!     READCONSTANT   read the name of a constant behind the \ sign and
!                    substitute value
!     READEXPRESSION Read and evaluate expression between two backslashes
!     READEXPRSTORE  Store the information as stored in istore/store into the
!                    commons cexpression and cseprd
!     READNAME       Read name following %-sign
!     READNAME1      read the name of a constant behind the $ or % sign
!     READNAMEVEC    Read name of vector in common cpost5
!     READSPLITKEYW  check whether the keyword contains numbers
!                    If so split the keyword into a part without number and
!     READSTRCONST   Check if name read corresponds to one of the integer or
!                    real constants
!     READSTRFUNC    Check if name read corresponds to one of the known
!                    functions
!     READSTRSCALS   Check if name read corresponds to one of the scalars
!     READWR         Read keyword
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2799   Name found following the % is not a scalar name
!    2943   More opening than closing brackets found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     icount := 0
!     finished := false
!     while ( not finished .and. no items left ) do
!        Read next item
!        if ( item = operator or number or bracket ) then
!           store item
!           icount := icount+1
!        else if ( item = \ ) then
!           read item following \ and store
!           icount := icount+1
!        else if ( item = string ) then
!           if ( string corresponds to constant or scalar ) then
!              store item
!              icount := icount+1
!           if ( string corresponds to function ) then
!              store item
!              icount := icount+1
!           else
!              finished := true
!        else
!           finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readexpress' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readexpress'
         write(irefwr,*) 'scalpermitted', scalpermitted
         write(irefwr,*) 'openfound', openfound
         write(irefwr,1) 'lastitem', lastitem
         call prinin ( typeitems, nitems, 'typeitems' )
         istore = 0
  1      format ( a, 1x, (10i10) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- icount := 0
!         finished := false

      icount = 1
      istore(icount) = 1
      finished = .false.
      ispeclsav = ispecl
      iopen = 0
      wcount = 1
      lenstore = 1000

      kmesh = 0
      lastrealexprsv = lastrealexpr
      numexpresssv = numexpress

      vecfound = .false.

!     --- Skip first item if it is a equals sign

      if ( typeitems(lastitem+1)==13 ) lastitem = lastitem+1

!     --- while ( not finished .and. no items left ) do

      do while ( .not. finished .and. lastitem<nitems )

!     --- Read next item

         lastitem = lastitem + 1
         if ( debug ) then
            write(irefwr,1) 'lastitem, icount, typeitems(lastitem)', &
                             lastitem, icount, typeitems(lastitem)
         end if  ! ( debug )
         select case ( typeitems(lastitem) )

            case ( 1, 2 )

!           --- integer or real found

               if ( istore(icount)>1000 ) then

!              --- Number preceded by number, stop

                  lastitem = lastitem-1
                  finished = .true.

               else

!              --- Standard case

                  icount = icount+1
                  istore(icount) = 1000+wcount
                  store(wcount) = realitems(lastitem)

                  if ( debug ) then
                     write(irefwr,1) 'icount, wcount, lastitem', &
                                      icount, wcount, lastitem
                     write(irefwr,2) 'store(wcount)', store(wcount)
                  end if  ! ( debug )
                  wcount = wcount+1

               end if  ! ( istore(icount)>1000 )

            case ( 5 )

!           --- open bracket found

               if ( icount>1 .and. istore(icount)>1000 ) then

!              --- Combination number ( found, hence end of expression

                  finished = .true.
                  lastitem = lastitem-1

               else

!              --- Standard case

                  icount = icount+2
                  istore(icount-1) = -3
                  istore(icount) = 1
                  iopen = iopen+1

               end if  ! ( lastitem<nitems .and....
               if ( debug ) then
                  write(irefwr,1)'open found, iopen =', iopen
                  call prininlarge ( istore, icount, 'istore' )
               end if  ! ( debug )

            case ( 6 )

!           --- close bracket found

               icount = icount+1
               istore(icount) = -4
               iopen = iopen-1
               if ( debug ) &
                  write(irefwr,1) 'close found: iopen, lastitem', &
                                   iopen, lastitem
               if ( iopen==-1 .and. openfound ) then

!              --- Closing bracket corresponding to first open bracket

                  closefound = .true.
                  if ( lastitem>=nitems ) then

!                 --- end of loop

                     finished = .true.

                  end if  ! ( lastitem>=nitems )

               else if ( iopen<0 ) then

!              --- No open bracket corresponding to this close bracket

                  lastitem = lastitem-1
                  icount = icount-1
                  finished = .true.

               end if  ! ( iopen<0 )
               if ( lastitem<nitems .and. &
                   (typeitems(lastitem+1)==5 .or. &
                    typeitems(lastitem+1)==10 .or. &
                    typeitems(lastitem+1)>=1 .and. &
                    typeitems(lastitem+1)<=3 .or. &
                    typeitems(lastitem+1)>=26 .and. &
                    typeitems(lastitem+1)<=29) ) then

!              --- Combination ) ( found, hence end of expression
!                  Also if followed by number or , &

                  finished = .true.

               end if  ! ( lastitem<nitems .and....

            case ( 7 )

!           --- * found

               icount = icount+1
               istore(icount) = 2

            case ( 8 )

!           --- + found

               if ( istore(icount)/=1 ) icount = icount+1
               istore(icount) = 1

            case ( 9 )

!           --- $ found

               ispecl = 12  ! do not check minus sign
               call readname1 ( minusg )
               ispecl = 0
               call readstrconst
               if ( ityprd==2 .or. ityprd==3 ) then

!              --- integer or real found

                  icount = icount+1
                  istore(icount) = 1000+wcount
                  store(wcount ) = realrd       ! value
                  wcount = wcount+1

               end if

            case ( 10 )

!           --- % found, hence scalar

               ispecl = 10   ! store sequence number of scalar in
                             ! index_rdarr(1)
               call readname
               ispecl = 0
               if ( ityprd==2 ) then

!              --- Vector found

                  icount = icount+1
                  istore(icount) = 1000+wcount
                  store(wcount) = realrd
                  wcount = wcount+1

               else if ( ityprd/=14 ) then

!              --- Name found is not a scalar

                  call errchr ( string(1:nchar), 1 )
                  call errsub ( 2799, 0, 0, 1 )
                  call errcrd
                  go to 1000

               else

!              --- Scalar found

                  icount = icount+1
                  istore(icount) = index_rdarr(1)+100000000
                  if ( debug ) then
                     write(irefwr,*) 'scalarname ', string(1:nchar)
                     write(irefwr,1) 'scalar_number', index_rdarr(1)
                     write(irefwr,1) 'istore(icount)', istore(icount)
                  end if  ! ( debug )

               end if  ! ( ityprd==2 )

            case ( 11 )

!           --- - found

               if ( istore(icount)/=1 ) icount = icount+1
               istore(icount) = -1
               if ( debug ) write(irefwr,1) 'icount', icount

            case ( 12 )

!           --- / found

               icount = icount+1
               istore(icount) = -2

            case ( 14 )

!           --- \ found

               icount = icount+1
               jcount = 0
               call readconstant ( iwork, jcount )
               if ( debug ) &
                  write(irefwr,1) 'ityprd, iwork(jcount)', &
                                   ityprd, iwork(jcount)
               if ( ityprd==2 ) then

!              --- ityprd = 2, hence constant

                  istore(icount) = 1000+wcount
                  if ( istore(icount-1)==-1 ) then

!                 --- Prior value = -1, multiply realrd by -1
!                     The reason is that readconstant also takes this
!                     minus sign into account

                     store(wcount ) = -realrd

                  else

!                 --- Standard case

                     store(wcount ) = realrd

                  end if  ! ( istore(icount)==-1 )
                  wcount = wcount+1

               else

!              --- ityprd # 2, hence function

                  istore(icount) = iwork(jcount)

               end if

            case ( 17 )

!           --- { found, read expression and make it real or integer

               call readexpression ( cardim(1:lastp) )
               if ( ierror/=0 ) go to 1000

               icount = icount+1
               istore(icount) = 1000+wcount
               store(wcount ) = realrd       ! value
               wcount = wcount+1

            case ( 19 )

!           --- ^ found

               icount = icount+1
               istore(icount) = 3

            case ( 3 )

!           --- string found, interpret

               if ( istore(icount)>1000 ) then

!              --- String preceded by number, finished
!                  lastitem must be decreased except when called by readboolexpr

                  if ( .not. bool ) lastitem = lastitem-1
                  finished = .true.
                  go to 200

               end if  ! ( istore(icount)>1000 )
               ispecl = 9
               call readwr
               ispecl = 0

               if ( debug ) then

!              --- Debug information

                  write(irefwr,*) 'case 3'
                  write(irefwr,1) 'nchar, lastitem', nchar, lastitem
                  write(irefwr,*) 'string ', string(1:nchar)

               end if  ! ( debug )

               if ( string(1:nchar)=='V' .and. &
                    typeitems(lastitem+1)==10 ) then

!              --- V% found, also  finished

                  lastitem = lastitembef
                  finished = .true.
                  go to 200

               end if  ! ( string(1:nchar)=='V' .and.... )

               if ( lastitem<nitems-1 .and. typeitems(lastitem+1)==10 ) &
                  then

!              --- Next item is %, so this item should be S or scalar

                  if ( nchar==1 .and. string(1:nchar)=='S' .or. &
                       nchar>=6 .and. string(1:6)=='SCALAR' ) then

!                 --- Skip S%

                     lastitem = lastitem+2
                     ispecl = 9
                     call readwr
                     ispecl = 0

                     if ( debug ) then

!                    --- Debug information

                        write(irefwr,1) 'nchar, lastitem', &
                                         nchar, lastitem
                        write(irefwr,*) 'string ', string(1:nchar)

                     end if  ! ( debug )

                  end if  ! ( nchar==1 .and. string(1:nchar)=='S' ... )

               end if  ! ( lastitem<nitems-1 .and. typeitems(lastitem+1)==10 )

               if ( nchar<=32 ) then

!              --- nchar<=32, hence this may be a constant, function or
!                  variable
!                  First check if it is a constant

                  ispecl = 0
                  call readstrconst
                  if ( debug ) &
                     write(irefwr,1) 'ityprd, lastitem/2', &
                                      ityprd, lastitem
                  if ( ityprd==2 .or. ityprd==3 ) then

!                 --- integer or real found

                     icount = icount+1
                     istore(icount) = 1000+wcount
                     store(wcount ) = realrd       ! value
                     wcount = wcount+1

                  end if

                  if ( scalpermitted .and. ityprd==4 ) then

!                 --- not constant found, check if it is a variable

                     ispecl = 10
                     call readstrscals
                     if ( debug ) &
                        write(irefwr,1) 'ityprd, lastitem/3', &
                                         ityprd, lastitem
                     ispecl = 0
                     if ( ityprd==14 ) then

!                    --- scalar found

                        icount = icount+1
                        istore(icount) = index_rdarr(1)+100000000
                        if ( debug ) &
                           write(irefwr,1) 'istore(icount)', &
                                            istore(icount)

                     end if  ! ( ityprd==14 )

                  end if  ! ( ityprd==4 )

                  if ( vecpermitted .and. ityprd==4 ) then

!                 --- not scalar found, check if it is a vector

                     ispecl = 10
                     call readnamevec
                     ispecl = 0
                     if ( debug ) then
                        write(irefwr,1) 'ityprd, index_rdarr(1)', &
                                         ityprd, index_rdarr(1)
                        write(irefwr,2) 'realrd', realrd
                     end if  ! ( debug )
                     if ( ityprd==2 ) then

!                    --- vector found

                        icount = icount+1
                        istore(icount) = index_rdarr(1)+150000000
                        if ( debug ) &
                           write(irefwr,1) 'istore(icount)', &
                                            istore(icount)
                        vecfound = .true.

                     end if  ! ( ityprd==14 )

                  end if  ! ( ityprd==4 )

                  if ( debug ) &
                     write(irefwr,1) 'ityprd, lastitem/4', &
                                      ityprd, lastitem
                  if ( ityprd==4 ) then

!                 --- not constant or variable found, check if it is a function

                     if ( lastitem<nitems .and. &
                          typeitems(lastitem+1)==5 ) then

!                    --- Next item = open bracket (

                        if ( debug ) write(irefwr,*) 'open bracket'
                        call readstrfunc ( numfun, string, nchar )
                        if ( numfun>0 ) then

!                       --- function found

                           icount = icount+1
                           istore(icount) = numfun

                        else

!                       --- no function found

                          finished = .true.
                          lastitem = lastitembef

                        end if  ! ( numfun>0 ) then

                     else

!                    --- no function found
!                        Check if it is S followed by number

                       call readsplitkeyw ( numchar, keyword )

                       if ( debug ) write(irefwr,1) 'numchar', numchar

                       if ( scalpermitted .and. numchar==1 .and. &
                            keyword(1:1)=='S' ) then

!                      --- keyword is S, check if followed by number

                          if ( debug ) write(irefwr,*) 'keyword S'
                          lastitem = lastitembef+1
                          if ( typeitems(lastitem+1)==1 .or. &
                               typeitems(lastitem+1)==9 ) then

!                         --- Next item is integer number

                             call asknxt
                             if ( debug ) then
                                write(irefwr,*) 'keyword Si'
                                write(irefwr,1) 'ityprd', ityprd
                                write(irefwr,2) 'realrd', realrd
                             end if  ! ( debug )

                             icount = icount+1
                             istore(icount) = nint(realrd)+100000000
                             if ( debug ) &
                                write(irefwr,1) 'icount, istore', &
                                                 icount, istore(icount)

                          else

!                         --- keyword is not S followed by number

                             finished = .true.
                             lastitem = lastitembef

                          end if  ! ( typeitems(lastitem+1)==1 )

                       else

!                      --- keyword is not S

                          finished = .true.
                          lastitem = lastitembef

                       end if  ! ( numchar==1 .and. keyword(1:1)=='S' )

                     end if  ! ( lastitem<nitems .and.... )

                  else if ( ityprd<2 .or. ityprd>3 .and. &
                            ityprd/=14 .and. ityprd/=18 ) then

!                 --- not constant, variable or function found
!                     end of expression

                     finished = .true.
                     lastitem = lastitembef
                     if ( debug ) &
                        write(irefwr,1) 'ityprd, lastitem/3', &
                                         ityprd, lastitem

                  end if  ! ( ityprd==4 )

               else

!              --- nchar>32, hence other type of string

                  finished = .true.

               end if  ! ( nchar<=32 )

            case default

!           --- Other type of function found
!               finished := true

            if ( icount>1 ) finished = .true.

         end select  ! case ( typeitems(lastitem) )

!        --- Check array length

200      call inichkarlen ( icount, lenstore, 'istore' )
         if ( ierror/=0 ) go to 1000

      end do  ! while ( .not. finished .and. lastitem<=nitems )
      if ( debug ) &
         write(irefwr,1) 'icount, istore', &
                          icount, istore(1:icount)

!     --- Check if last item read is an operator
!         If so remove from istore and decrease lastitem

      if ( icount>1 .and. &
           (istore(icount)>=1 .and. istore(icount)<=3 .or. &
           istore(icount)==-2 .or. istore(icount)==-1) ) then

!     --- operator found, decrease icount and lastitem

         icount = icount-1
         lastitem = lastitem-1

      end if  ! ( istore(icount)>=1 .and. istore(icount)<=3 .or. .. )

      ispecl = ispeclsav
      lenstore = icount

      if ( debug ) then
         write(irefwr,1) 'iopen, lenstore', iopen, lenstore
         call prininlarge ( istore, lenstore, 'istore' )
      end if  ! ( debug )

      if ( iopen==1 ) then

!     --- There is exactly one open bracket more than close brackets

         if ( lenstore==2 ) then

!        --- lenstore = 2, only an open bracket, no number

            lenstore = 0   ! no number read
            openfound = .true.

         else if ( istore(2)==-3 ) then

!        --- starts with open bracket, remove this bracket
!            by shifting all positions one to the left

            if ( istore(3)==1 .or. istore(3)==-1 ) then

!           --- Special case, bracket followed by + or - &
!               shift 2 positions

               lenstore = lenstore-2
               do i = 1, lenstore
                  istore(i) = istore(i+2)
               end do  ! i = 2, lenstore

            else

!           --- Standard case

               lenstore = lenstore-1
               do i = 2, lenstore
                  istore(i) = istore(i+1)
               end do  ! i = 2, lenstore

            end if  ! ( istore(3)==1 .or. istore(3)==-1 )
            openfound = .true.

         else

!        --- Does not start with open bracket, hence error

            call errsub ( 2943, 0, 0, 0 )
            call errcrd

         end if  ! ( lenstore==2 )

      else if ( iopen>1 ) then

!     --- Too many open brackets

         call errsub ( 2943, 0, 0, 0 )

      end if  ! ( iopen==1 )

      if ( debug ) call prinin ( istore, lenstore, 'istore' )
      if ( lenstore==5 ) then

!     --- istore contains 5 integers, check if this is the following
!         situation + ( + number )

         if ( istore(1)==1 .and. istore(2)==-3 .and. istore(5)==-4 ) &
           then

!        --- Situation found, remove superfluous things

            istore(1) = istore(3)
            istore(2) = istore(4)
            lenstore = 2
            closefound = .true.
            openfound = .true.

         end if  ! ( istore(1)==1 .and. istore(2)==-3 .and. istore(5)==-4 )

      end if  ! ( lenstore==4 )

!     --- Store information from istore and store into cexpression

      call readexprstore ( istore, store, lenstore, wcount-1 )

      if ( debug ) then
         call prininlarge ( istore, lenstore, 'istore' )
         call prinrl ( store, wcount-1, 'store' )
      end if  ! ( debug )

      icheck = maxval(istore(1:lenstore))

      if ( debug ) write(irefwr,1) 'icheck', icheck

      if ( ityprd==19 .and. icheck<100000000 ) then

!     --- No scalars found, compute expression and remove from cexpression

         call prevalexpr ( numexpress, realrd, kmesh, cardim(1:lastp) )
         numexpress = numexpresssv
         lastrealexpr = lastrealexprsv
         ityprd = 3

      end if  ! ( icheck<100000000 )

1000  call erclos ( 'readexpress' )
      if ( debug ) then

!     --- Debug information

         if ( lenstore>0 ) &
            call prininlarge ( istore, lenstore, 'istore' )
         if ( wcount>1 ) call prinrl ( store, wcount-1, 'store' )
         write(irefwr,*) 'vecfound', vecfound
         write(irefwr,1) 'lastitem, iopen, ityprd', &
                          lastitem, iopen, ityprd
         write(irefwr,2) 'realrd', realrd
         write(irefwr,*) 'End readexpress'

      end if  ! ( debug )

      end
