      subroutine readfile801 ( filename, vector, nelectrodes, ndegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 20-06-2013 New definition file ref numbers
!        version  2.0  date 27-12-2011 Remove work arrays
!        version  1.0  date 06-11-1999
!
!   copyright (c) 1998-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read capacity numbers and values from file and store in array vector
!     Real case.
!     At this moment only ndegfd degrees of freedom per electrode is permitted
! **********************************************************************
!
!                       KEYWORDS
!
!     read_file
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) filename
      integer ndegfd, nelectrodes
      double precision vector(ndegfd,*)

!     filename       i    Name of file
!     iwork          o    integer work array of length
!                         nelectrodes (nelectrodes-1)
!                         The electrode numbers to be read are stored in iwork
!     ndegfd         i    Number of degrees of freedom per point
!     nelectrodes    i    Number of electrodes
!     vector        i/o   Solution array or array of structure of type 801
!                         in which the values to be read are stored
!     work           o    real work array of length
!                         nelectrodes (nelectrodes-1)/2*NDEGFD
!                         The values to be read are stored in work
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nlines, k, ielectrode, idum, nelem, i, ih, jelectrode, &
              error
      integer, allocatable :: iwork(:)
      double precision, allocatable :: work(:,:)

!     error          Return value of allocate or deallocate
!     i              Counting variable
!     idum           General dummy variable
!     ielectrode     Electrode sequence number
!     ih             Help variable to store a constant temporarily
!     jelectrode     Electrode sequence number
!     k              Counting variable
!     nelem          Number of elements in vector
!     nlines         Number of lines that are read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READFILE       Read information from an ASCII file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2134   Wrong number of elements found
!    2135   Electrode number out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!     set constants
!     Read numbers from file
!     Fill values in vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readfile801' )

!     --- set constants

      nelem = (nelectrodes*(nelectrodes-1))/2

      allocate ( iwork(2*nelem), work(ndegfd,nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem*(ndegfd+2), 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Read numbers from file

      nlines = ndegfd
      call readfile ( filename, .false., nlines, &
                      idum, iwork, work, 4 )
      if ( ierror/=0 ) go to 1000
      if ( nlines>nelem ) then

!     --- Too many numbers in file

         call errint ( nlines, 1 )
         call errint ( nelem, 2 )
         call errchr ( filename, 1)
         call errsub ( 2134, 2, 0, 1)
         go to 1000

      end if

!     --- Fill values in vector

      do k = 1, nlines
         ielectrode = iwork(k)
         jelectrode = iwork(k+nlines)
         if ( ielectrode<=0 .or. ielectrode>nelectrodes ) then
            call errint ( 1, 1 )
            call errint ( k+1, 2 )
            call errint ( ielectrode, 3 )
            call errint ( nelectrodes, 4 )
            call errchr ( filename, 1)
            call errsub ( 2135, 4, 0, 1 )
            go to 1000
         end if
         if ( jelectrode<=0 .or. jelectrode>nelectrodes ) then
            call errint ( 1, 1 )
            call errint ( k+1, 2 )
            call errint ( jelectrode, 3 )
            call errint ( nelectrodes, 4 )
            call errchr ( filename, 1)
            call errsub ( 2135, 4, 0, 1 )
            go to 1000
         end if
         ih = max(ielectrode,jelectrode)
         ielectrode = min(ielectrode,jelectrode)
         jelectrode = ih
         ih = 0
         do i = 1, ielectrode-1
            ih = ih+nelectrodes-i
         end do
         ih = ih+jelectrode-ielectrode
         vector(1:ndegfd,ih) = work(1:ndegfd,k)

      end do

      deallocate ( iwork, work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'readfile801' )
      end
