      subroutine readscderiv ( istart, inpsep, tabout, icode,
     +                         nout, strout, nprob, numvec, numscl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 02-03-2011 Input of ivec
!        version  1.5  date 13-11-2010 error 1488 --> errkeywexcl
!        version  1.4  date 02-05-2008 New definition of scalars and vectors
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   DERIVATIVES
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons1'
      include 'SPcommon/cuscons'
      include 'SPcommon/cusname'
      include 'SPcommon/cconst'
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cscalvec'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), icode, nout, nprob,
     +        numvec, numscl
      double precision tabout(*)
      character (len=*) strout(*)

!     icode          i    Indicates which of the options in readsc is found
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         1: ICODE
!                         2: Sequence number corresponding to input for
!                            coefficients
!                            If iseqcf = -1, the next one will be used
!                         3: IPROB   Problem sequence number
!                         4: IVECTOR Sequence number output vector
!                                    (first vector)
!                            If ivector = -1, the same value as iprob
!                            will be used
!                            If iseqdr = -1, the next one will be used
!                            If ivector = -2, no output vector is used
!                         5: ISEQDR Sequence number derivatives
!                            if -10: no block is read
!                         6: ISCALAR Sequence number scalar, is only used if
!                            boundary integrals are computed.
!                            Otherwise iscalar should be 0
!                            If iscalar>0, ivector must be -2
!                         7: ICHELD Choice parameter
!                         8: ISEQINPVEC Sequence number input vector
!                         9: IVEC Type of output vector
!     istart        i/o   Next free position in inpsep
!     nout           i    Number of entries in tabout
!     nprob          i    Number of various problems
!     numscl         i    Number of scalars read
!     numvec         i    Number of vectors stored in isol
!     strout        i/o   Contains series of strings
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nncode
      parameter ( nncode=9 )
      integer ncode(nncode), itype(nncode), ncodmx(nncode),
     +        ivec, iretur, iscal, iprev
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 maxtable
      character (len=12) code(nncode)
      logical debug

!     code           Table with keywords
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iprev          Previous value of istart
!     iretur         Return value of subroutine INTX02
!                    Possible values:
!                    -2:  end of file has been reached
!                    -1:  error has been found
!                     0:  normal return
!     iscal          Sequence number of scalar
!     itype          Indicates the type of action corresponding to keywords
!                    in code
!     ivec           Sequence number of vector with respect to array ISOL
!     maxtable       Maximum value found in table
!     ncode          Table containing the number of significant letters in
!                    code
!     ncodmx         Table containing the number of maximum characters in
!                    the keyword table code
!     nncode         Number of entries in code
!     table          Contains values of items read corresponding to code
!     tabmax         Maximum value of items read corresponding to code
!     tabmin         Minimum value of items read corresponding to code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     INTX02         Interpret a record containing keywords followed by numbers
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!       DERIVATIVES, options (all in one line)
!           where options may have the following contents
!               SEQ_COEF = s         (Default none)
!               SEQ_DERIV = s        (Default the next one)
!               PROBLEM = s          (Default the next one)
!               VECTOR = s           (Default nprob+1)
!               SCALAR = s           (Default 0)
!               ICHELD = s           (Default 1)
!               INPUT_VECTOR = v
!           Remark: SCALAR and VECTOR are mutually exclusive
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1506   Both VECTOR and SCALAR given in DERIVATIVES
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'SEQ_COEF', 'PROBLEM', 'VECTOR', 'SEQ_DERIV',
     +             'SCALAR', 'V', 'ICHELD', 'INPUT_VECTOR', 'IVEC' /
      data ncode  / 6, 4, 6, 6, 1, 1, 3, 4, 4 /
      data ncodmx / 8, 7, 6, 9, 6, 1, 6,12, 4 /

!     code   Table with sub keywords corresponding to keyword DERIVATIVES
!             Values:
!             1:  SEQ_COEF
!             2:  PROBLEM
!             3:  VECTOR
!             4:  SEQ_DERIV
!             5:  SCALAR
!             6:  V
!             7:  ICHELD
!             8:  INPUT_VECTOR
!             9:  IVEC
! ======================================================================
!
      call eropen ( 'readscderiv' )
      debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscderiv'
         iprev = istart

      end if
      if ( ierror/=0 ) go to 1000

      itype = 0
      itype(3) = 38  ! VECTOR
      itype(5) = 3   ! SCALAR
      itype(6) = 38  ! V
      itype(8) = 38  ! INPUT_VECTOR

      table = -1d0
      table(1) = 0d0 ! SEQ_COEF
      tabmin = 1d0

      tabmax(1) = 25d0
      tabmax(2) = nprob
      tabmax(3) = dble(maxvectors)
      tabmax(4) = 100d0
      tabmax(5) = dble(maxscalars)
      tabmax(6) = tabmax(3)
      tabmax(7) = 1d3
      tabmax(8) = tabmax(3)
      tabmax(9) = 10d0

      iretur = 0
      call intx02 ( code, ncode, nncode, itype, table, code,
     +              ncode, 0, iretur, tabmin, tabmax, tabout,
     +              nout, strout, 1, ncodmx )
      if ( ierror/=0 ) go to 1000
      if ( table(3)>0d0 .and. table(6)>0d0 )
     +   call errkeywexcl ( code, ncodmx, 3, 6 )

      table(3) = max(table(3),table(6))
      inpsep(istart) = icode
      inpsep(istart+1) = nint(table(1))
      inpsep(istart+2) = nint(table(2))
      inpsep(istart+4) = nint(table(4))
      ivec  = nint(table(3))
      iscal = mod(nint(table(5)),100000)
      if ( ivec==-1 ) then

!     --- Vector not given

         if ( iscal==-1 ) then
            ivec = nprob+1
         else
            ivec = -2
         end if

      else

!     --- Vector has been given, so scalar is not allowed

         if ( iscal>0 ) then
            call errsub ( 1506, 0, 0, 0 )
            call errcrd
         else
            iscal = 0
         end if
      end if
      inpsep(istart+3) = ivec
      inpsep(istart+5) = iscal

      maxtable = maxval(table(7:8))

      if ( inpsep(istart+4)<0 .and. maxtable>=0d0 ) then

!     --- No sequence number found

         inpsep(istart+4) = -10                    ! sequence number
         inpsep(istart+6) = max(1,nint(table(7)))  ! icheld
         inpsep(istart+7) = max(1,nint(table(8)))  ! input vector
         inpsep(istart+8) = max(0,nint(table(9)))  ! ivec
         istart = istart + 9

      else

!     --- Sequence number found

         istart = istart + 6

      end if  ! ( inpsep(istart+4)<0 .and. maxtable>=0d0 )

      numvec = min ( max(numvec,ivec), maxvectors )
      if ( ivec>0 .and. ivec<=maxvectors ) ivectors(ivec) = 1
      numscl = min ( max(numscl,iscal), maxscalars )
      if ( iscal>0 .and. iscal<=maxscalars ) iscalars(iscal) = 1

       print *,'ivec ', ivec
1000  call erclos ( 'readscderiv' )
      if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(iprev), istart-iprev, 'inpsep' )
         write(irefwr,*) 'End readscderiv'

      end if

      end
