      subroutine readsrin1 ( inumbr, maxnum, inum, positive, maxval, &
                            code )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-04-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine readsrin1 is used to read a series of integer numbers
!   The numbers may be alternated with texts, which are skipped
!   The reading of numbers is stopped if
!    a ) sign is found
!    an end of record is found
!    maxnum numbers have been read
!   If a minus sign followed by text followed by a number is read, &
!   the number is made negative
!   After reading the numbers, icolum points to the next position to be read
!   If maxnum<0, texts are used as separater and no numbers are also allowed
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cseprdf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxnum, inum, inumbr(*), maxval
      logical positive
      character *(*) code

!     code      i  String containing the corresponding keyword
!     inum      o  Number of numbers that have been actually read
!     inumbr    o  Array to be filled with the numbers
!     maxnum    i  |maxnum| defines the maximum number of numbers to be read
!                  If maxnum>0, texts are skipped
!                  If maxnum<0, texts are used as separater
!     maxval    i  Maximum value allowed for the integer read
!     positive  i  If true all numbers must be positive
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ready, minsgn
      integer i, kcolsv, ival

!    i        General loop variable
!    ival     Integer value read
!    kcolsv   Help parameter to save the old value of kcolum
!    minsgn   Indicates if a minus sign has been found (true) or not (false)
!    ready    Indicates if a while loop must be terminated (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT    Read next item from the standard input file
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put character in error message
!     ERRCRD    Prints the last record read and underlines the errorneous part
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1870   Integer number read out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Clear array dnumbr
!   ready  := false
!   inum   := 0
!   ispecl := 4
!   minsgn := false
!   while ( not ready ) do
!      Read next item (ASKNXT)
!      if ( ityprd = 2 or 3 ) then
!         inum := inum+1
!         if ( minsgn ) then realrd := -realrd
!         minsgn := false
!      else if ( ityprd = 6 ) then
!         minsgn := true
!      else if ( ityprd = 7 or 0 or 1 ) then
!         ready := true
!      if ( inum = maxnum ) then
!         ready := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Make some initializations

      call eropen ( 'readsrin1' )

!     --- Clear array dnumbr

      do i = 1, abs(maxnum)
         inumbr(i) = 0
      end do

      ready  = .false.
      inum   = 0
      ispecl = 4
      minsgn = .false.
      kcolsv = kcolum

200   if ( .not. ready ) then

!     --- While loop
!         Read next item

         call asknxt
         if ( ierror/=0 ) go to 1000

         if ( ityprd==2 .or. ityprd==3 ) then

!        --- ityprd = 2 or 3, number found

            inum = inum + 1
            if ( minsgn .and. realrd>0 ) realrd = - realrd
            minsgn = .false.
            inumbr(inum) = nint(realrd)
            ival = nint(realrd)
            if ( positive ) then

!           --- only positive integers

               if ( ival<1 .or. ival>maxval ) then

!              --- Error integer out of range

                  call errint ( ival, 1 )
                  call errint ( 1, 2 )
                  call errint ( maxval, 3 )
                  call errchr ( code, 1 )
                  call errsub ( 1870, 3, 0, 1 )
                  call errcrd

               end if

            else

!           --- positive and negative integers

               if ( abs(ival)>maxval ) then

!              --- Error integer out of range

                  call errint ( ival, 1 )
                  call errint ( -maxval, 2 )
                  call errint ( maxval, 3 )
                  call errchr ( code, 1 )
                  call errsub ( 1870, 3, 0, 1 )
                  call errcrd

               end if

            end if

         else if ( ityprd==6 ) then

!        --- ityprd = 6, minus sign found

            minsgn = .true.

         else if ( ityprd<=1 .or. ityprd==7 .or. &
                   maxnum<0 .and. ityprd==4 ) then

!        --- ityprd = 0, 1 or 7, i.e. end of file, end of record or
!            ) sign found

            ready = .true.

         end if

         if ( inum>=abs(maxnum) ) then

!        --- inum = maxnum, stop reading

            ready = .true.

         end if

         go to 200

      end if
      ispecl = 0
      kcolum = min ( kcolum, kcolsv )

1000  call erclos ( 'readsrin1' )

      end
