      subroutine msh607 ( xcub, ycub, zcub, xmini, coor, kbound, nbound, &
                          ja )
! ======================================================================
!
!        programmer    niek praagman
!        version  5.0  date 03-09-2010 New release
!        version  4.2  date 25-03-2009 No replacement of node cub
!                                      and replace outside node in a more
!                                      random manner (ws)
!
!   copyright (c) 1987-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine whether point (xcub,ycub,zcub) belongs to
!     the volume given by kbound.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbound(3,*), nbound, ja
      double precision xcub, ycub, zcub, xmini, coor(3,*)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ja         o    if point belongs to volume ( ja = 1 ) else ( ja = 0 )
!     kbound     i    boundary element array
!     nbound     i    number of boundary elements
!     xcub       i    x-coordinate of point considered
!     xmini      i    smallest x-coordinate of all nodes
!     ycub       i    y-coordinate of point considered
!     zcub       i    z-coordinate of point considered
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, isnij
      double precision t, x1, x2, x3, xleft, xmin, y1, y2, y3, yleft, &
                       ymax, ymin, z1, z2, z3, zleft, zmax, zmin, eps

!     eps        accuracy parameter
!     i          Counting variable
!     i1         First node boundary triangle
!     i2         Second node boundary triangle
!     i3         Third node boundary triangle
!     isnij      Indicator for times of intersection
!     t          Parameter: 0<t<1 point belongs to line segment
!                considered
!     x1         x-coordinate node 1
!     x2         x-coordinate node 2
!     x3         x-coordinate node 3
!     xleft      x-coordinate most left point
!     xmin       minimum of x-values of triangle nodes
!     y1         y-coordinate node 1
!     y2         y-coordinate node 2
!     y3         y-coordinate node 3
!     yleft      y-coordinate most left point
!     ymax       maximum of y-values of triangle nodes
!     ymin       minimum of y-values of triangle nodes
!     z1         z-coordinate node 1
!     z2         z-coordinate node 2
!     z3         z-coordinate node 3
!     zleft      z-coordinate most left point
!     zmax       maximum of z-values of triangle nodes
!     zmin       minimum of z-values of triangle nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSH610    Determine intersection of triangle and line
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     none
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all triangles of boundary
!     Check for each whether it has an intersection point with the
!     line from the point considered to the "most left point"
!     Check number of times that there is an intersection: if this number
!     is odd: point belongs to volume
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set accuracy

      eps = 100 * epsmac

!     --- Set most left point

      xleft = xmini -  1.00d0
      yleft = ycub  -  1.23d-6
      zleft = zcub  -  1.45d-6

!     --- Determine number of intersections:

      isnij = 0
      ja    = 0

!     --- Run through all boundary triangles:

      do i=1, nbound
          i1 = kbound( 1, i )
          i2 = kbound( 2, i )
          i3 = kbound( 3, i )

          x1 = coor( 1, i1 )
          y1 = coor( 2, i1 )
          z1 = coor( 3, i1 )

          x2 = coor( 1, i2 )
          y2 = coor( 2, i2 )
          z2 = coor( 3, i2 )

          x3 = coor( 1, i3 )
          y3 = coor( 2, i3 )
          z3 = coor( 3, i3 )

          xmin = min(x1,x2,x3)
          xmin = xmin - eps * abs( xmin )
          ymin = min(y1,y2,y3)
          ymin = ymin - eps * abs( ymin )
          ymax = max(y1,y2,y3)
          ymax = ymax + eps * abs( ymax )
          zmin = min(z1,z2,z3)
          zmin = zmin - eps * abs( zmin )
          zmax = max(z1,z2,z3)
          zmax = zmax + eps * abs( zmax )

          if ( xcub >xmin .and. &
               ycub >ymin .and. &
               zcub >zmin .and. &
               yleft<ymax .and. &
               zleft<zmax ) then

!         --- Potential point

             call msh610(xcub,ycub,zcub,xleft,yleft,zleft,x1,y1,z1, &
                         x2,y2,z2,x3,y3,z3,t)

!            --- Belongs point to triangle?

             if ( t<=1d0 ) isnij = isnij + 1
          end if

      end do

      if ( (isnij - (isnij/2) * 2)/=0 ) ja = 1

      end
