      subroutine msh641 ( nbound, jnew, nipnt, fix_pnt, &
                          curvenumbers, coorcurvs, surfnumbers  )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  6.1  date 20-11-2013 Remove mshg3d
!        version  6.0  date 08-09-2010 New release
!        version  5.2  date 20-08-2010 Dynamic allocation helparrays
!
!   copyright (c) 1991-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to make a mesh of tetrahedrons for a volume which is
!     given by the quadratic 7-node triangles of KBOUND. First a linear
!     mesh is created using KBOND out of KBOUND,and the linear sub- &
!     routine mshphil3d. Middle points on the lines and
!     centrepoints in the faces of the tetrahedrons are placed afterwards.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nbound, nipnt, curvenumbers(*), &
              surfnumbers(*)
      double precision fix_pnt(*), coorcurvs(3,*)
      logical jnew

!     coorcurvs      i    Contains the coordinates of the extra curves
!     curvenumbers   i    Contains the extra curve numbers
!     fix_pnt        i    Real array of size 4 x nextrapnts containing the
!                         coordinates and coarseness of the fixed points
!                         in the mesh
!     jnew           i    indication whether the mesh is new (true) or old
!     nbound         i    number of boundary elements
!     nipnt               number of points in boundary ; only needed if
!                         jnew = 'false'
!     surfnumbers    i    Array of size nextrasrfs containing the surface
!                         sequence numbers of each extra surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, i5, i6, i7, iel, nextra, npnt
      integer, allocatable :: istart(:), ibuur(:), kbond(:)
      double precision coorsav(12*nbound)

!     coorsav        array of length 12*nbound to store the coordinates
!                    of the boundary nodes
!     i              Counting variable
!     i1             Node number
!     i2             Node number
!     i3             Node number
!     i4             Node number
!     i5             Node number
!     i6             Node number
!     i7             Node number
!     ibuur          helparray with neighbour nodes for all nodes
!     iel            Counter
!     istart         helparray starting points neighbour array
!     kbond          helparray for linear boundary triangles
!     nextra         Counter
!     npnt           help to store starting number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH622         determine the neighbours of each node with node
!                    number larger than its own node number
!     MSH625         fill the boundary points and neighbours in ibuur
!     MSH626         create new internal mid points
!     MSH628         refill array kmeshc. Linear elements are changed
!                    in quadratic elements
!     MSH642         create and place the centerpoints in tetrahedrons
!                    when MESHTP = 15
!     MSHCOPYBOUNCR  Copy boundary coordinates into temporary array coorsav
!     MSHPHIL3D      Fill elements and nodes for shape 11
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('msh641')

!     --- Copy coordinates of boundary into coorsav

      call mshcopybouncr ( kbndptlc, nbound, coor3d, coorsav, 7 )

!     --- Start to fill array KBOND and call mshphil3d

      nextra = 0

!     --- Allocate space for helparray kbond:

      allocate( kbond( 3 * nbound ), stat = error)
      if ( error/= 0 ) call eralloc ( error, 3*nbound, 'kbond' )
      if ( ierror/=0 ) go to 1000

!     --- Determine contents kbond:

      do i = 1 , nbound
          i1 = kbndptlc(7*i-6)
          i2 = kbndptlc(7*i-4)
          i3 = kbndptlc(7*i-2)
          i4 = kbndptlc(7*i-5)
          i5 = kbndptlc(7*i-3)
          i6 = kbndptlc(7*i-1)
          i7 = kbndptlc(7*i  )

          kbond(3*i-2) = i1
          kbond(3*i-1) = i2
          kbond(3*i  ) = i3

          if  ( .not. jnew ) then
             if ( i1>nextra ) nextra = i1
             if ( i2>nextra ) nextra = i2
             if ( i3>nextra ) nextra = i3
             if ( i4>nextra ) nextra = i4
             if ( i5>nextra ) nextra = i5
             if ( i6>nextra ) nextra = i6
             if ( i7>nextra ) nextra = i7
          end if
      end do

!     --- Divide region given by KBOND in linear triangles

      if ( jnew ) then
           nipnt = npointlc
      else
           nipnt = nextra

           do iel = 1 , nelemlc
               i1 = kmeshc(14*iel - 13)
               i2 = kmeshc(14*iel - 11)
               i3 = kmeshc(14*iel -  9)
               i4 = kmeshc(14*iel -  4)

               kmeshc(4*iel-3) = i1
               kmeshc(4*iel-2) = i2
               kmeshc(4*iel-1) = i3
               kmeshc(4*iel  ) = i4

               if ( i1>nextra ) nextra = i1
               if ( i2>nextra ) nextra = i2
               if ( i3>nextra ) nextra = i3
               if ( i4>nextra ) nextra = i4
           end do
      end if

      call mshphil3d ( kbond, nbound, kmeshc, &
                       jnew, nipnt, fix_pnt, &
                       numnodextcurvs, curvenumbers, coorcurvs, &
                       surfnumbers  )
      if ( ierror/=0 ) go to 1000

!     --- Determine all neighbours for each point

      if ( .not. jnew ) npointlc = nipnt

!     --- Allocate space for arrays istart and ibuur

      allocate( istart(npointlc), ibuur(10*npointlc), stat = error )
      if ( error/=0 ) call eralloc ( error, 11*npointlc, 'istart' )
      if ( ierror/=0 ) go to 1000

      call msh622( kmeshc, nelemlc, istart, ibuur, npointlc )
      npnt = npointlc

!     --- Place "middle" points : (first old points)

      call msh625( kbndptlc, nbound, istart, ibuur, 7 )

!     --- New points

      if ( .not. jnew ) npointlc = nextra

!     --- Compute coordinates new points:

      call msh626( istart, ibuur, npointlc, coor3d )

!     --- Refill kmeshc

      call msh628( kmeshc, nelemlc, istart, ibuur )

!     --- ibuur array not needed anymore:

      deallocate( ibuur, stat = error )
      if ( ierror/=0 ) call erdealloc ( error, 'ibuur' )
      if ( ierror/=0 ) go to 1000

!     --- Refill coordinates of boundary in coor3d

      call mshcopybouncr ( kbndptlc, nbound, coorsav, coor3d, 7 )

!     --- Finally place centerpoints in faces of all sides of the tetrahedrons
!         include 'SPcomm

      call msh642 ( kmeshc, nelemlc, coor3d, npnt, npointlc, istart, &
                    kbndptlc, nbound )

!     --- Finally deallocate helparrays

      deallocate( istart, kbond, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'istart' )

1000  call erclos('msh641')

      end
