      subroutine msh649 ( kelem , nelem , kstap , kstapl, coor  , &
                          npoint, nipnt , itri  , icube , nx    , &
                          ny    , xstart, ystart, zstart, dismax )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  3.0  date 08-09-2010 New release
!        version  2.0  date 25-01-2005 Simplified, new norms (NP)
!        version  1.0  date 21-04-1998
!
!   copyright (c) 1998-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Renumber kelem, kstap, coor so that only used nodes occur in
!     kelem, kstapl, coor and icube
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none

!     None
! **********************************************************************
!
!                       COMMON BLOCKS
!
!     None
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, kelem(4,*), kstap, kstapl(3,*), npoint, nx, ny, &
              itri(*), icube(*), nipnt
      double precision coor(3,*), xstart, ystart, zstart, dismax

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     dismax     i    max distamce of nodes in boundary
!     icube      i    indicator for each node in which block it is situated
!     itri            work array of length npoint for detecting that
!                     all work is done
!     kelem     i/o   new KMESH part c with respect to the surface elements
!                     all elements have the same number of nodes
!     kstap     i/o   number of triangles constituting the actual
!                     boundary : at output this variable should be 0
!     kstapl    i/o   helparray for actual boundary of region, &
!                     at input containing all boundary triangles, &
!                     and during the process the actual boundary hence
!                     at output empty
!     nelem      i    number of elements
!     nipnt      i    number of initial points
!     npoint    i/o   number of nodal points
!     nx         i    number of points in x direction
!     ny         i    number of points in y direction
!     xstart     i    x-coordinate starting value
!     ystart     i    y-coordinate starting value
!     zstart     i    z-coordinate starting value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ktal, nc, n1, n2, n3, ielem, i1, i2, i3, i4

!     i          loop variable
!     i1         1th node of actual element
!     i2         2th node of actual element
!     i3         3th node of actual element
!     i4         4th node of actual element
!     ielem      element number
!     ktal       number of used nodes
!     n1         number block in x-direction
!     n2         number block in y-dircetion
!     n3         number block in z-direction
!     nc         composed number of block
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     None
! **********************************************************************
!
!                       I/O
!
!     None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize array itri to zero
!     Run through all elements. If node in element
!         fill array itri in according position
!     Run through all boundary triangles and do the same
!         for all nodes in the triangles
!     All nodes of interest have 1 in array itri
!     Rearrange array coor : cancel all nodes that are not
!         in either the elements or the boundary.
!         If itri(inode) = 0 then
!            skip node, &
!         else if itri(inode) = 1 then
!            add node and
!            place new node number in itri
!     Set new number of nodes
!     Renumber kelem with respect to new node numbers
!     Renumber kstapl with respect to new node numbers
!     Clear array itri again and refill according to the new boundary
!     Adjust icube values for the new ordering
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     None
! ======================================================================
!
!     --- Initialize array itri to zero

      do i = 1 , npoint
         itri(i) = 0
      end do

!     --- Run through all elements. If node in element
!         fill array itri in according position

      do i = 1 , nelem

!     --- Find nodes of element

         i1 = kelem(1,i)
         i2 = kelem(2,i)
         i3 = kelem(3,i)
         i4 = kelem(4,i)

!        --- Mark nodes as in mesh:

         itri(i1) = 1
         itri(i2) = 1
         itri(i3) = 1
         itri(i4) = 1
      end do

!     --- Run through all boundary triangles and do the same
!         for all nodes in the triangles

      do i = 1 , kstap

!     --- Find nodes in triangle

         i1 = kstapl(1,i)
         i2 = kstapl(2,i)
         i3 = kstapl(3,i)

!        --- Mark all nodes as in mesh

         itri(i1) = 1
         itri(i2) = 1
         itri(i3) = 1
      end do

!     --- Now all entries j with itri(j)==0 indicate a node that is
!         skipped

      ktal = 0

!     --- Rearrange array coor : cancel all nodes that are not
!         in either the elements or the adjusted boundary.
!         If itri(inode) = 0 then
!            skip node, &
!         else if itri(inode) = 1 then add node and
!            place new node number in itri.

      do i = 1, npoint

!     --- Consider nodepoint i:

         if ( itri(i)==1 .or. i<= nipnt ) then

!        --- Node exists:

            ktal         = ktal + 1
            coor(1,ktal) = coor(1,i)
            coor(2,ktal) = coor(2,i)
            coor(3,ktal) = coor(3,i)
            itri(i)      = ktal
         end if
      end do

!     --- Set new number of nodes:

      npoint = ktal

!     --- Renumber kelem

      do ielem = 1 , nelem

!     --- Old nodenumbers

         i1 = kelem(1,ielem)
         i2 = kelem(2,ielem)
         i3 = kelem(3,ielem)
         i4 = kelem(4,ielem)

!        --- Find corresponding nodenumber:

         kelem(1,ielem) = itri(i1)
         kelem(2,ielem) = itri(i2)
         kelem(3,ielem) = itri(i3)
         kelem(4,ielem) = itri(i4)
      end do

!     --- Renumber kstapl in the same way:

      do i = 1 , kstap

!     --- Old nodenumbers

         i1 = kstapl(1,i)
         i2 = kstapl(2,i)
         i3 = kstapl(3,i)

!        --- Find new numbers

         kstapl(1,i) = itri(i1)
         kstapl(2,i) = itri(i2)
         kstapl(3,i) = itri(i3)
      end do

!     --- Clear array itri again:

      do i = 1, npoint
         itri(i) = 0
      end do

!     --- Refill array itri for the boundary

      do i = 1, kstap
         i1 = kstapl(1,i)
         i2 = kstapl(2,i)
         i3 = kstapl(3,i)
         itri(i1) = itri(i1) + 1
         itri(i2) = itri(i2) + 1
         itri(i3) = itri(i3) + 1
      end do

!     --- Finally adjust icube values

      do i = nipnt + 1 , npoint

!     --- Determine cube for nodepoint i:

          n1 = int ( (coor(1,i)-xstart) / dismax )
          n2 = int ( (coor(2,i)-ystart) / dismax )
          n3 = int ( (coor(3,i)-zstart) / dismax )
          nc = 1 + n1 + n2*nx + n3*nx*ny
          icube(i) = nc
      end do

      end
