      subroutine msh651 ( i1, i2, i3, i4, kmeshc, nelem, &
                          coor, npoint, numelm )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.0  date 08-09-2010 New release
!        version  1.0  date 19-10-2006
!
!   copyright (c) 2006-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check intersections for proposed tetrahedron and all tetrahedrons
!     created already. Especially used for debugging!!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     3d
!     tetrahedron
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer i1, i2, i3, i4, nelem, kmeshc(4,nelem), npoint, numelm
      double precision coor(3,npoint)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     i1             i    new vertex of element considered
!     i2             i    new vertex of element considered
!     i3             i    new vertex of element considered
!     i4             i    new vertex of element considered
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nelem          i    Number of elements
!     npoint         i    Number of nodal points
!     numelm         o    indicator which element is wrong
!                          0: all elements are disjunct
!                         >0: element with number numelm belongs also to other
!                          element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dist, disth, &
                       xa, ya, za, xb, yb, zb,  t, xinf, &
                       x1, x2, x3, x4, y1, y2, y3, y4, &
                       z1, z2, z3, z4
      integer jel, j1, j2, j3, j4, ja

!     dist        representative distance
!     disth       helpdistance
!     j1          helpvariable node loop
!     j2          helpvariable node loop
!     j3          helpvariable node loop
!     j4          helpvariable node loop
!     ja          indicator whether point is inside (1) element or not
!     jel         loopvariable for element to be considered
!     t           helpvariable
!     x1          x-coordinate
!     x2          x-coordinate
!     x3          x-coordinate
!     x4          x-coordinate
!     xa          x-coordinate
!     xb          x-coordinate
!     xinf        x value 'infinity'
!     y1          y-coordinate
!     y2          y-coordinate
!     y3          y-coordinate
!     y4          y-coordinate
!     ya          y-coordinate
!     yb          y-coordinate
!     z1          z-coordinate
!     z2          z-coordinate
!     z3          z-coordinate
!     z4          z-coordinate
!     za          z-coordinate
!     zb          z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH610         Determine intersection of triangle and line
! **********************************************************************
!
!                       I/O
!
!     None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine barycenter of new element
!     Run through all elements (loop jel) and check that
!       barycenter new element does not belong to element jel
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     None
! ======================================================================
!
      call eropen ( 'msh651' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh651'
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set the two variables xinf and numelm:

      xinf   = -1000.10d0
      numelm = 0

!     --- Determine coordinates barycenter new element:

      xb = ( coor( 1, i1 ) + coor( 1, i2 ) + &
             coor( 1, i3 ) + coor( 1, i4 ) ) / 4d0
      yb = ( coor( 2, i1 ) + coor( 2, i2 ) + &
             coor( 2, i3 ) + coor( 2, i4 ) ) / 4d0
      zb = ( coor( 3, i1 ) + coor( 3, i2 ) + &
             coor( 3, i3 ) + coor( 3, i4 ) ) / 4d0

      x1 = coor(1,i1)
      y1 = coor(2,i1)
      z1 = coor(3,i1)

      dist = (x1-xb)*(x1-xb)+(y1-yb)*(y1-yb)+(z1-zb)*(z1-zb)
      dist = 10 * dist

      xa =  xinf
      ya =  yb
      za =  zb

!     --- Run through all elements and check whether (xb,yb,zb) is inside:

      do jel = 1, nelem
         j1 = kmeshc( 1, jel )
         j2 = kmeshc( 2, jel )
         j3 = kmeshc( 3, jel )
         j4 = kmeshc( 4, jel )

         x1 = coor( 1, j1 )
         y1 = coor( 2, j1 )
         z1 = coor( 3, j1 )

         x2 = (x1-xb)*(x1-xb)+(y1-yb)*(y1-yb)+(z1-zb)*(z1-zb)

         if ( x2<dist ) then

!        --- Check:

            x2 = coor( 1, j2 )
            y2 = coor( 2, j2 )
            z2 = coor( 3, j2 )

            x3 = coor( 1, j3 )
            y3 = coor( 2, j3 )
            z3 = coor( 3, j3 )

            x4 = coor( 1, j4 )
            y4 = coor( 2, j4 )
            z4 = coor( 3, j4 )

!           --- Check common points line a - b with the four faces

            t = - 1000d0
            ja = 0

!           --- Face 1 - 2 - 3

            call msh610(xa,ya,za,xb,yb,zb,x1,y1,z1,x2,y2,z2, &
                        x3,y3,z3,t)
            if ( t<1d0 ) ja = ja + 1

!           --- Face 1 - 2 - 4

            call msh610(xa,ya,za,xb,yb,zb,x1,y1,z1,x2,y2,z2, &
                        x4,y4,z4,t)
            if ( t<1d0 ) ja = ja + 1

!           --- Face 1 - 3 - 4

            call msh610(xa,ya,za,xb,yb,zb,x1,y1,z1,x3,y3,z3, &
                        x4,y4,z4,t)
            if ( t<1d0 ) ja = ja + 1

!           --- Face 2 - 3 - 4

            call msh610(xa,ya,za,xb,yb,zb,x2,y2,z2,x3,y3,z3, &
                        x4,y4,z4,t)
            if ( t<1d0 ) ja = ja + 1

!           --- Set element number:

            if ( ja==1 ) numelm = jel

            if ( ja==1 .and. debug ) then
               write(irefwr,*) '651: Error for ',i1,i2,i3,i4
               write(irefwr,*) 'number of elements is ',nelem
               write(irefwr,*) 'barycenter = ',xb,yb,zb

               write(irefwr,*) i1,'crd',coor(1,i1),coor(2,i1),coor(3,i1)
               write(irefwr,*) i2,'crd',coor(1,i2),coor(2,i2),coor(3,i2)
               write(irefwr,*) i3,'crd',coor(1,i3),coor(2,i3),coor(3,i3)
               write(irefwr,*) i4,'crd',coor(1,i4),coor(2,i4),coor(3,i4)

               write(irefwr,*) 'is inside element: ',j1,j2,j3,j4
               write(irefwr,*) j1,'crd',coor(1,j1),coor(2,j1),coor(3,j1)
               write(irefwr,*) j2,'crd',coor(1,j2),coor(2,j2),coor(3,j2)
               write(irefwr,*) j3,'crd',coor(1,j3),coor(2,j3),coor(3,j3)
               write(irefwr,*) j4,'crd',coor(1,j4),coor(2,j4),coor(3,j4)
            end if
         end if !end general check
      end do
      if ( numelm>0 ) go to 1000

!     --- Next check all barycenters with respect to the new element:
!         (Nodes new element i1, i2, i3, i4):

      x1 = coor( 1, i1 )
      y1 = coor( 2, i1 )
      z1 = coor( 3, i1 )

      x2 = coor( 1, i2 )
      y2 = coor( 2, i2 )
      z2 = coor( 3, i2 )

      x3 = coor( 1, i3 )
      y3 = coor( 2, i3 )
      z3 = coor( 3, i3 )

      x4 = coor( 1, i4 )
      y4 = coor( 2, i4 )
      z4 = coor( 3, i4 )

!     --- Run through all elements created already:

      do jel = 1, nelem
         j1 = kmeshc( 1, jel )
         j2 = kmeshc( 2, jel )
         j3 = kmeshc( 3, jel )
         j4 = kmeshc( 4, jel )

!        --- Determine coordinates barycenter:

         xb = ( coor( 1, j1 ) + coor( 1, j2 ) + &
                coor( 1, j3 ) + coor( 1, j4 ) ) / 4d0
         yb = ( coor( 2, j1 ) + coor( 2, j2 ) + &
                coor( 2, j3 ) + coor( 2, j4 ) ) / 4d0
         zb = ( coor( 3, j1 ) + coor( 3, j2 ) + &
                coor( 3, j3 ) + coor( 3, j4 ) ) / 4d0

         disth = (xb-x1)*(xb-x1)+(yb-y1)*(yb-y1)+(zb-z1)*(zb-z1)

         if ( disth<dist ) then

!        --- Check further:

            xa =  xinf
            ya =  yb
            za =  zb

!           --- check if (xb,yb,zb) is inside:
!               Check common points line a - b with the four faces

            t = - 1000d0

            ja = 0

!           --- Face 1 - 2 - 3

            call msh610(xa,ya,za,xb,yb,zb,x1,y1,z1,x2,y2,z2, &
                        x3,y3,z3,t)
            if ( t<1d0 ) ja = ja + 1

!           --- Face 1 - 2 - 4

            call msh610(xa,ya,za,xb,yb,zb,x1,y1,z1,x2,y2,z2, &
                        x4,y4,z4,t)
            if ( t<1d0 ) ja = ja + 1

!           --- Face 1 - 3 - 4

            call msh610(xa,ya,za,xb,yb,zb,x1,y1,z1,x3,y3,z3, &
                        x4,y4,z4,t)
            if ( t<1d0 ) ja = ja + 1

!           --- Face 2 - 3 - 4

            call msh610(xa,ya,za,xb,yb,zb,x2,y2,z2,x3,y3,z3, &
                        x4,y4,z4,t)
            if ( t<1d0 ) ja = ja + 1

!           --- Set element number:

            if ( ja==1 ) then
               numelm = jel
               if ( debug ) &
                  write(irefwr,*) '651: Error for element: ',j1,j2,j3,j4
            end if
         end if !end General check
      end do

1000  call erclos ( 'msh651' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh651'
      end if  ! ( debug )

      end
