      subroutine msh670 ( kstap , kstapl, coor , npoint, nelem , kelem, &
                          klaar , line  , nlind, nipnt , xstart, &
                          xn    , yn    , zn, ivolnr, xminvec )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 09-09-2010 New release
!        version  1.0  date 13-04-2010
!
!   copyright (c) 2010-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine MSH670 constructs a grid of linear tetrahedrons if only
!     the boundary triangles and the nodes in a 3-D domain are given. Use
!     is made of a special TESSELATION property, the ratio's of the elements
!     constructed and of the so-called advancing front method.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     tesselation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kelem(4,*), kstapl(3,*), kstap, nelem, nipnt, &
              npoint, klaar(*), nlind, line( nlind), ivolnr
      double precision coor(3,*), xstart, xn, yn, zn, xminvec(4)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ivolnr     i    Volume sequence number
!     kelem     i/o   array containing the nodes of the tetrahedrons
!                     length of this 2d-array at output : (4,nelem)
!     klaar           work array of length npoint for detecting that
!                     all work is done
!     kstap     i/o   number of triangles constituting the actual
!                     boundary : at output this variable should be 0
!     kstapl     i    helparray for actual boundary of region, &
!                     at input containing all boundary triangles, &
!                     and during the process the actual boundary hence
!                     at output empty
!     line            workarray of length nlind to store boundary lines
!     nelem     i/o   number of tetrahedron elements created
!     nipnt      i    total number of initial points in boundary (important
!                     for quadratic case)
!     nlind      i    declared length array line
!     npoint     i    total number of points in mesh.
!     xminvec    i    Contains the vector xmint, ymint, zmint and tran
!     xn         i    x coordinate
!     xstart     i    helpvalue
!     yn         i    y coordinate
!     zn         i    z coordinate
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, i1, i2, i3, i4, icheck, &
              ja, jpoint, numelm, nwrel, &
              ichange, k1, k2, k3
      double precision d1, d2, d3, e1, e2, e3, dx, dy, dz, dis, &
                       x1, x2, x3, y1, y2, y3, z1, z2, z3, &
                       xm, ym, zm, ratib, ratio, refvol, &
                       t, th, volume, volumt

!     d1             distance
!     d2             distance
!     d3             distance
!     dis            distance
!     dx             x-distance
!     dy             y-distance
!     dz             z-distance
!     e1             first component of inward pointing normal
!     e2             second component of inward pointing normal
!     e3             third component of inward pointing normal
!     i              loop-variable
!     i1             first node of triangle
!     i2             second node of triangle
!     i3             third node of triangle
!     i4             ?
!     ichange
!     icheck         indicator for visibility
!     j              loop-variable
!     ja             indicator
!     jpoint         node considered as possible candidate for tetrahedron
!     k1             first node of triangle
!     k2             second node of triangle
!     k3             third node of triangle
!     numelm         temporary number element with wrong barycentre
!     nwrel          Threshold value for printing elements
!     ratib          best ratio found so far
!     ratio          (radius inner sphere / radius outer sphere) * 3
!     refvol         reference volume to compute the accuracy
!     t              actual time
!     th             ?
!     volume
!     volumt         total volume of all tets
!     x1             x-coordinate first point triangle
!     x2             x-coordinate second point triangle
!     x3             x-coordinate third point triangle
!     xm             x-coordinate of barycentre of triangle
!     y1             y-coordinate first point triangle
!     y2             y-coordinate second point triangle
!     y3             y-coordinate third point triangle
!     ym             y-coordinate of barycentre of triangle
!     z1             z-coordinate first point triangle
!     z2             z-coordinate second point triangle
!     z3             z-coordinate third point triangle
!     zm             z-coordinate of barycentre of triangle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
!     MSH3DQ         plot one surface in 3-D (no hidden lines)
!     MSH479         check whether point is visible
!     MSH480         check whether boundary does not intersect itself
!     MSH607         check whether point is inside or outside volume
!     MSH609         Determine centre of circumcircle of triangle
!     MSH610         Determine intersection of triangle and line
!     MSH612         fill element in array and adjust
!     MSH632         compute volume of tetrahedron
!     MSH637         Check boundary triangles
!     MSH652         Check if all tetrahedrons of kmeshc are disjoint
!     MSH661         Compute ratio of inner and outer sphere of tetrahedron
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2913   Error with barycenters
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Start to check input boundary triangles      msh480
!     Clear array KLAAR
!     Fill array KLAAR
!     Compute reference value xm
!     Check whether the points are inside on the boundary or outside
!     Set actual number of triangles in boundary array
!     While boundary not empty , search for first boundary triangle
!        Check first whether tetrahedron can be formed with other
!        boundary triangles, compute the special neighbours
!        if point possible ready, jump over next loop
!        Run through all possible points and find first allowed point
!            Jump out of loop as soon as possible
!        No point visible : Error, according to the theory this
!        situation should never occur
!           Reorder and start again with next triangle
!        Point found
!        Determine inward pointing normal and centre of circumcircle
!        Determine radius of circumsphere if point jpoint is chosen
!           Check whether other points are
!              - inside the sphere
!              - give pos volume
!              - are visible
!           hence are "better", go on with this point
!        Create tetrahedron for triangle and point found
!        Fill element and adjust array kstapl and kstap
!        Go for next triangle
!     End while
!     Do some final checks
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msh670' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh670'
  1      format ( a, 1x, (10i6) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,*) 'At start of msh670: array coor'
         write(irefwr,*) 'For each node number its coordinates'
         do i=1, npoint
            write(irefwr, 2 ) i,(coor(j,i),j=1,3)
         end do
 2       format ( i5, 1x, (3f12.3) )
         write(irefwr,*) 'Coordinates array coor ok?'

         write(irefwr,*) 'Start msh670 kstapl, nodes boundary triangles'
         do i=1, kstap
            write(irefwr,*) (kstapl(j,i),j=1,3)
         end do
         write(irefwr,*) 'Boundary array kstapl ok?'
      end if

      nwrel   = (nelem/100 + 1)*100
      ichange = 0

!     --- Start by checking the local boundary triangles
!         (No intersections are allowed! Check for all lines that there
!          are no intersection points with all these triangles)

      call msh480 ( kstapl, kstap, line, nlind, coor, ivolnr, xminvec, &
                    0 )

      if ( ierror/=0 ) go to 1000

!     --- Compute reference value for volume using the distances of the first
!         triangles given

      i1 = min ( kstap, 25 )

      refvol = 0d0

      do i = 1, i1

         k1 = kstapl( 1, i1 )
         k2 = kstapl( 2, i1 )
         k3 = kstapl( 3, i1 )

         x1 = coor( 1, k1 )
         y1 = coor( 2, k1 )
         z1 = coor( 3, k1 )

         x2 = coor( 1, k2 )
         y2 = coor( 2, k2 )
         z2 = coor( 3, k2 )

         x3 = coor( 1, k3 )
         y3 = coor( 2, k3 )
         z3 = coor( 3, k3 )

         d1 = (x2-x1) * (x2-x1) + (y2-y1)*(y2-y1) + (z2-z1)*(z2-z1)
         d2 = (x3-x2) * (x3-x2) + (y3-y2)*(y3-y2) + (z3-z2)*(z3-z2)
         d3 = (x1-x3) * (x1-x3) + (y1-y3)*(y1-y3) + (z1-z3)*(z1-z3)

         d1 = sqrt( d1 )
         d2 = sqrt( d2 )
         d3 = sqrt( d3 )

         refvol = refvol + d1 * d2 * d3

      end do

      refvol = refvol / i1

!     --- Fill array KLAAR for all nodes:
!         value = 0 means node is in an element and not in boundary, &
!         value>0 means node is in, at least 3, boundary triangles
!         value<0 means node is not yet used, hence in the internal
!         Clear array KLAAR, and start assuming all nodes not yet used
!         At start: either in interior i.e. klaar(i) = -1
!                   or in boundary i.e. klaar(i)>0
!         If in a situation with already elements then fill array KLAAR
!         for these elements

!         --- Initialize array klaar
!         (first nipnt nodes are in the starting boundary):

      do i = 1, nipnt
         klaar(i) = 0
      end do

!     --- Assume that all other points are inside, give indication -1:

      do i = nipnt+1, npoint
         klaar(i) = -1
      end do

!     --- set used in elements

      do i= 1,nelem
         do j=1,4
            i1 = kelem(j,i)
            klaar(i1)=0
         end do
      end do

!     --- set used in actual boundary

      do i = 1 , kstap
         do j=1,3
            i1 = kstapl( j,i )
            if (klaar(i1)==-1) klaar(i1)=0
            klaar( i1 ) = klaar( i1 ) + 1
         end do
      end do

      if ( debug ) then
         write(irefwr,*) 'Coordinates of all nodes in volume'
         do i = 1, npoint
            write(irefwr,'(2i10,3f10.4)') i,klaar(i),(coor(j,i),j=1,3)
         end do
         write(irefwr,*) 'end coordinates nodes. Ok?'
      end if

!     --- As long as the actual number of triangles in boundary is
!         not zero find a new tetrahedron:

200   if ( kstap/=0 ) then

!     --- Check kstap: should always be even

         if ( nelem==-13394 ) then
            write(irefwr,*) 'Nu moet het gebeuren, kstap =',kstap
         end if
         if ( mod(kstap,2)==1 ) then
            write(irefwr,*) 'Error in msh670, kstap=',kstap
            write(irefwr,*) 'Internal error, inform SEPRA'
            write(irefwr,*) 'nelem = ',nelem
            write(irefwr,*) 'execution terminated'
            call instop
         end if

         if ( nelem>0 .and. ichange==0 ) then

!        --- Check element created

            i1 = kelem( 1, nelem )
            i2 = kelem( 2, nelem )
            i3 = kelem( 3, nelem )
            i4 = kelem( 4, nelem )
            call msh632( coor, i1, i2, i3, i4, volume )
            if ( volume<0 ) then
               write(irefwr,*) 'elemnt',nelem,'volume=',volume
               write(irefwr,*) 'nodes',i1,i2,i3,i4
            end if
         end if

!        --- Get boundary triangle and find "best" tetahedron
!            Determine nodes of boundary triangle to be considered

         i1 = kstapl( 1,1 )
         i2 = kstapl( 2,1 )
         i3 = kstapl( 3,1 )

!        --- Run through all nodes and try to find a node that is visible and
!            that has the "best" ratio and volume combination:

         ratib  = 0d0
         ratio  = 0d0
         volume = rinfin/2d0
         jpoint = 0

         if ( debug ) then
            write(irefwr,*) 'Create new tetrahedron for basis'
            write(irefwr,'(3i10)') i1,i2,i3

            if ( nelem==-1 ) then
               write(irefwr,*) 'The actual situation, nelem=',nelem
               write(irefwr,*) 'contents of array klaar'
               do i=1, npoint
                  if ( klaar(i)==0 ) then
                     write(irefwr,*) 'Node',i,'is outside'
                  else if ( klaar(i)<0 ) then
                     write(irefwr,*) 'Node',i,'is internal'
                  else if ( klaar(i)>0 ) then
                     write(irefwr,*) 'Node',i,'in actual bound',klaar(i)
                  end if
               end do
            end if
         end if

!        --- Make loop through all nodes. In order to complete the
!            tetrahedron with the "best" choice, take of all the visible
!            nodes that one that gives the best ratio:

         do i = 1, npoint

!        --- Node i allowed?
!            If node i is node of basistriangle then i is no candidate, skip:

            if ( i==i1 .or. i==i2 .or. i==i3 ) goto 215

!           --- If node i is outside present region it is no candidate:

            if ( klaar(i)==0 ) goto 215

!           --- If the volume is not positive, i is no candidate, skip:

            call msh632( coor, i1, i2, i3, i, volume )
            if ( volume<=0d0 ) goto 215

!           --- If the ratio of the new tet is too small skip:

            call msh661( coor, i1, i2, i3, i, ratio  )
            if ( ratio<=ratib ) goto 215

!           --- First tests are passed: i is a serious candidate

!               Check visibility by checking
!               1) no boundary lines pass through the four constituting
!                  triangles, i.e. faces, of the tetrahedron
!               2) none of the edges of the new tetrahedron intersect
!                  one of the kstap boundary triangles

            call msh479( kstapl, kstap, line, nlind, coor, &
                         i, klaar, npoint, icheck )

!           --- If i is not visible, then no candidate hence skip:

            if ( icheck/=0 ) goto 215

!           --- Node i is visible and ratio is computed

!               --- All checks passed: use Candidate:

            ratib  = ratio

!           --- Set value jpoint

            jpoint = i
215      end do

         if ( jpoint>0 ) then
            ichange = 0
            if ( debug ) then
               call msh661( coor, i1, i2, i3, jpoint, ratio  )
               numelm = nelem+1
               write(irefwr,216) &
                 'New',numelm,'Nodes',i1,i2,i3,jpoint,'ratio=',ratio
216            format ( a,(i8),1x,a,(4i8),1x,a,(f10.3) )
            end if
         else if ( jpoint==0 ) then

!        --- No node found for triangle i1,i2,i3
!            Change kstapl:

            ichange = ichange + 1
            if ( ichange>kstap+1 ) then
               ja = 0

!              --- Determine startpoint line from triangle i1, i2, i3

               call msh609(coor,i1,i2,i3,e1,e2,e3,xm,ym,zm)

               x1 = coor(1,i1)
               x2 = coor(1,i2)
               x3 = coor(1,i3)

               y1 = coor(2,i1)
               y2 = coor(2,i2)
               y3 = coor(2,i3)

               z1 = coor(3,i1)
               z2 = coor(3,i2)
               z3 = coor(3,i3)

               dx   = x1 - x2
               dy   = y1 - y2
               dz   = z1 - z2

               dis  = sqrt(dx*dx + dy*dy + dz*dz)

!              --- Barycentre of triangle:

               xm = ( x1 + x2 + x3 ) / 3d0
               ym = ( y1 + y2 + y3 ) / 3d0
               zm = ( z1 + z2 + z3 ) / 3d0

!              --- Find "endpoint" of helpline

               xn = xm + dis * e1
               yn = ym + dis * e2
               zn = zm + dis * e3

!              --- Find "startpoint" of helpline

               xm = xm + 1d-8 * dis * e1
               ym = ym + 1d-8 * dis * e2
               zm = zm + 1d-8 * dis * e3

!              --- Startpoint should be inside:

               call msh607 ( xm, ym, zm, -1000d0, coor, kstapl, kstap, &
                             ja )

!              --- Check whether new point is inside:

               if ( debug .or. 1<0 ) then
                  write(irefwr,*) 'Startpnt =',xm,ym,zm
                  write(irefwr,*) 'Endpoint =',xn,yn,zn
               end if

               th = 1000d0

!              --- Run through the boundary triangles

               do i = 2, kstap

                  i1 = kstapl ( 1, i )
                  i2 = kstapl ( 2, i )
                  i3 = kstapl ( 3, i )
                  if (debug) write(irefwr,*) 'BndTri',i,'Nodes',i1,i2,i3

                  x1 = coor(1,i1)
                  x2 = coor(1,i2)
                  x3 = coor(1,i3)

                  y1 = coor(2,i1)
                  y2 = coor(2,i2)
                  y3 = coor(2,i3)

                  z1 = coor(3,i1)
                  z2 = coor(3,i2)
                  z3 = coor(3,i3)

!                 --- Common point with line?

                  call msh610(xn, yn, zn, xm, ym, zm, x1, y1, z1, &
                              x2, y2, z2, x3, y3, z3, t)

                  if ( t>0 .and. t<th ) th = t
               end do

               if ( th<=0 .or. th>1d0 ) then
                  write(irefwr,*) 'Error in msh670, stop'
                  write(irefwr,*) 'even weer terug!'
                  goto 1000
                  !call instop
               end if

!              --- Place new point "halfway"

               xn = xm + th/2d0 * dis * e1
               yn = ym + th/2d0 * dis * e2
               zn = zm + th/2d0 * dis * e3

!              --- Check this new point:

               call msh607(xn,yn,zn,xstart,coor,kstapl,kstap,ja)

!              --- If inside

               if ( ja==1 ) then
                  write(irefwr,*) 'Nieuw punt gevonden, bereken opnieuw'
                  write(irefwr,*) 'Wat te doen?'
                  write(irefwr,*) 'Crds',npoint+1,xn,yn,zn
                  !goto 1000
                  npoint = npoint + 1
                  coor( 1, npoint ) = xn
                  coor( 2, npoint ) = yn
                  coor( 3, npoint ) = zn
                  klaar(npoint) = -1
               end if
            end if

!           --- Replace triangle

            i1 = kstapl(1,1)
            i2 = kstapl(2,1)
            i3 = kstapl(3,1)

!           --- Run through rest:

            do i = 2, kstap
               kstapl( 1, i-1 ) = kstapl ( 1, i )
               kstapl( 2, i-1 ) = kstapl ( 2, i )
               kstapl( 3, i-1 ) = kstapl ( 3, i )
            end do
            kstapl ( 1, kstap ) = i1
            kstapl ( 2, kstap ) = i2
            kstapl ( 3, kstap ) = i3

!           --- Next triangle

            goto 200
         end if

!        --- Create tetrahedron for triangle i1-i2-i3 and point jpoint

         klaar(i1) = klaar(i1) - 1
         klaar(i2) = klaar(i2) - 1
         klaar(i3) = klaar(i3) - 1

         if ( klaar(jpoint)==-1 ) klaar(jpoint) = 0

!        --- Fill element in array and adjust array kstapl and kstap

         call msh612( kelem, nelem, i1, i2, i3, jpoint, &
                      kstapl, kstap, klaar )

!        --- Check the boundary

         if ( debug ) then
            write(irefwr,*) 'Check msh637,nelem =',nelem,'kstap =',kstap
            if ( kstap>0 ) &
               call msh637 ( kstapl, kstap, line, line(kstap+11) )
            write(irefwr,*) 'msh637 passed'

!           --- Plot the boundary:

            call msh3dq(1,coor,kstapl,npoint,15D0,1D0,3,kstap)

!           --- Boundary plotted:

            write(irefwr,*) 'Boundary plotted, stop!'
            call instop
         end if

         if ( debug .and. nelem==-3 ) then

!        --- Temporary check:

            numelm = 0
            write(irefwr,*) 'Barycenter check msh652, nelem=',nelem
            call msh652 (kelem, nelem, coor, npoint, numelm)
            if ( numelm>0 ) then
               write(irefwr,*) 'In 652 wrong element numelm=',numelm
               call instop
            end if
         end if

         if ( ioutp>=1 .and. nelem>=nwrel .and. debug ) then
            write(irefwr,1)'Number of elements', nelem
            write(irefwr,1)'Number of remaining boundary triangles', &
                            kstap
            write(irefwr,1)'Number of nodes', npoint
            nwrel = nwrel + 100
         end if

!        --- Go for next triangle

         goto 200
      end if

1000  call erclos( 'msh670' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug from end msh670'
         write(irefwr,*) 'kstap = ',kstap
         do i=1, kstap
            i1 = kstapl(1,i)
            i2 = kstapl(2,i)
            i3 = kstapl(3,i)
            write(irefwr,*) 'triangle',i1,i2,i3
         end do

         write(irefwr,*) 'Compute volume at end of 670'
         volumt = 0d0

         do i=1, nelem
            i1 = kelem(1,i)
            i2 = kelem(2,i)
            i3 = kelem(3,i)
            i4 = kelem(4,i)
            call msh632( coor, i1, i2, i3, i4, volume )
            if ( volume<0 ) then
               write(irefwr,*) 'error in element',i1,i2,i3,i4
               write(irefwr,*) 'volume = ',volume
            end if
            volumt = volumt + volume
         end do
         write(irefwr,*) 'Total volume = ',volumt/6d0

!        --- Eventually check all barycenters:

         if ( 0<1 ) then

!        --- Check all elements

            i1 = 0
            call msh652( kelem, nelem, coor, npoint, i1 )
            write(irefwr,*) 'After call to msh652'
            write(irefwr,*) 'Problems in element',i1
            if (i1>0) then
               call errint ( i1, 1 )
               call errsub ( 2913, 1, 0, 0 )
            end if
         end if
         write(irefwr,*) 'End debug info msh670'
      end if  ! ( debug )

      if ( kstap>0 ) then
         write(irefwr,*) 'End msh670, not yet ready'
         do i=1,npoint
            if ( klaar(i)/=0 ) then
               write(irefwr,*) i,'in bound, value',klaar(i)
            end if
         end do
      end if

      end
