      subroutine sepactivate ( mmindex, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if an already existing array is positioned in array IBUFFR or not
!     If the array is available in IBUFFR, then the priority is set equal
!     to 10, and the subroutine returns
!     When the array is only available on backing storage, and not in IBUFFR
!     then the array is read from file 3 and space is reserved in array IBUFFR
!     if no space is available in array IBUFFR, sepactivate tries to
!     create space in the following ways:
!       -  by writing low priority arrays to backing storage
!       -  if this is not sufficient by compressing
!          array ibuffr after writing of all arrays with priority
!          smaller than 10 to backing storage
!     ini060 replaces the old subroutine INI001 (for existing arrays)
!     ini060 is identical to subroutine INI050 as described in the Programmer's
!     Guide, except that the parameter name is not longer used.
!     The name of the calling subroutine is extracted from common blocks
!     CMESSC and CMESSG, so it is supposed that the subroutine EROPEN has been
!     used
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuf
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cdebugf90'
      include 'SPcommon/cmessgf90'
      include 'SPcommon/cmesscf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mmindex
      character (len=*) text

!     mmindex        i    The absolute value of mmindex refers to array INFOR in
!                         common block CARRAY.
!                         This means that the positions (i,|mmindex|) (i=1,2,3) in
!                         INFOR contain all the information about the storage
!                         of the array
!                         If mmindex is positive then it is supposed that the
!                         array is changed in the calling subroutine, otherwise
!                         (mmindex<0) the arrays remains unchanged
!     text           i    String variable containing the name of the vector to
!                         be activated
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipstar, i
      integer indfor, length, number, iextra

!     i         Help variable to store new length
!     iextra    Extra space needed at both sides of the array for control
!               integers
!     indfor    Help parameter to store the record number from which the
!               array has been stored in file 3
!     ipstar    computed starting position in array IBUFFR
!     length    length of the array in words
!     number    Absolute value of mmindex
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI049         Create space in IBUFFR by writing low priority arrays to
!                    file 3, or to compress IBUFFR
!     INICHK1        Check IBUFFR
!     PRINBF         Print and check length of array IBUFFR
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
!     If necessary arrays are written to and/or read from file 3 (iref3)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     802 the array from which the start address must be found
!         does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( idebug>0 ) then
!        Set iextra = extra space needed
!        check all control integers
!     If ( mmindex = 0 or mmindex>IINFOR ) then
!         Error: Array does not exist;  STOP
!     Set priority
!     If ( array on file 3 ) then
!         If ( enough space in IBUFFR after IBFREE ) then
!             IPSTAR = IBFREE
!             Raise IBFREE
!         else
!             Create space by writing low priority arrays to file 3
!             or compressing IBUFFR
!         Read array from file 3
!     Set INBACK ( |mmindex| ) negative if nessary
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepactivate' )
      if ( ierror/=0 ) go to 1000

      if ( idebug==0 ) then
         iextra = 0
      else
         iextra = intlen
         ifirws = 0
         call inichk1 ( ibuffr )
      end if
      number = abs (mmindex)
      if ( mmindex==0 .or. number>iinfor ) then

!     --- mmindex<0: error

         call errint ( mmindex,  1 )
         call errint ( number, 2 )
         call errint ( iinfor+1, 3 )
         call errchr ( text, 1 )
         call errsub ( 802, 3, 0, 1 )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set priority 10 in INFOR

      infor1(2,number) = max(infor1(2,number),10)
      length = infor1 ( 1,number )
      ipstar = infor ( number )

      if ( ipstar<0 ) then

!     --- ipstar<0  array in file 3

         indfor = -ipstar
         i = ibfree+length+iextra*2-1
         if ( i>0 .and. i<=3*nbuffr/4 ) then

!        --- ibfree+length-1<=nbuffr:  Read array in IBUFFR
!                                        compute ipstar and raise ibfree
!            For a 32-bits computer, ibfree must always be odd

            infor ( number ) = ibfree
            ipstar = ibfree
            ibfree = ibfree+length+iextra*2
            if ( intlen==2 .and. mod ( ibfree, 2_8) ==0 ) ibfree = ibfree+1

         else

!        --- try to create free space by writing low priority arrays
!            to backing storage   ( file 3 ) or compressing ibuffr

            call ini049 ( ibuffr, length, ipstar )
            if ( ierror/=0 ) go to 1000
            infor(number) = ipstar

         end if
         if ( ierror/=0 ) go to 1000

         call prinbf ( ibfree-1 )
         if ( ierror/=0 ) go to 1000
         call readfl ( indfor, 1, length+2*iextra, 1, 3, &
                       ibuffr(ipstar-iextra) )
         if ( ierror/=0 ) go to 1000

      end if

      if ( mmindex>0 .and. inback (number)>0 ) &
         inback ( number ) = -inback ( number )
      if ( ioutp>2 ) then
         write(irefwr,*) ' Activate array ', namarr(abs(mmindex)), &
                         ' with sequence number ', abs(mmindex), &
                         ' and starting address ', ipstar, &
                         ' in subroutine ', &
                           namsub(levsub-1)//':sepactivate'
      end if

1000  call erclos ( 'sepactivate' )

      end
