      subroutine findxy ( coor, x, y, z, npoint, ipoint, ndim, iprint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 13-05-2010 Debug information
!        version  1.2  date 28-06-2004 first dimension added
!        version  1.1  date 28-03-1996 Sander Gielen: third dimension added
!        version  1.0  date 21-10-1990
!
!   copyright (c) 1990-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   find nearest node to point (x,y,z) and store point number in ipoint
!
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     2d
!     3d
!     find
!     find_node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, ipoint, ndim, iprint
      double precision coor(ndim,*), x, y, z

!    coor      i   array containing the co-ordinates of the nodal points
!    ipoint    o   Nodal point number of node nearest to (x,y,z)
!    iprint    i   Print level, 0 is no printing
!    ndim      i   Dimension of space
!    npoint    i   Number of nodal points
!    x         i   x-co-ordinate of point, for which ipoint must be found
!    y         i   y-co-ordinate of point, for which ipoint must be found
!    z         i   z-co-ordinate of point, for which ipoint must be found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision dx, dy, dz, dist, distx, disty, distz, d

!     d              help variable to store the length of a vector
!     dist           Distance between two points
!     distx          Distance in x-direction
!     disty          Distance in y-direction
!     distz          Distance in z-direction
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'findxy' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from findxy'
         write(irefwr,2) 'x, y, z', x, y, z
         call prinrl1 ( coor, npoint, ndim, 'coor' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ipoint = 1

      select case ( ndim )

      case (1)

!     --- ndim = 1

         dist = abs(coor(1,1)-x)
         do i = 2, npoint
            d = abs(coor(1,i)-x)
            if ( d<dist ) then
               dist = d
               ipoint = i
            end if
         end do

         if ( iprint>2 ) then
            write(irefwr,100) x,ipoint,coor(1,ipoint)
         end if  ! ( iprint>2 )

100      format( ' Nearest nodal point to the point (', (ss,1pe12.3e2), &
              ') is node ', i5/' with co-ordinate (', (ss,1pe12.3e2), &
              ')' )

      case (2)

!     --- ndim = 2

         distx = coor(1,1)-x
         disty = coor(2,1)-y
         dist = distx**2+disty**2
         if ( debug ) &
            write(irefwr,2) 'dist, distx, disty', dist, distx, disty

         do i = 2,npoint

            dx = coor(1,i)-x
            dy = coor(2,i)-y
            d = dx**2+dy**2
            if(d<dist) then
               dist = d
               ipoint = i
            end if

            if ( debug ) then
               write(irefwr,1) 'i, ipoint', i, ipoint
               write(irefwr,2) 'dist, d, dx, dy', dist, d, dx, dy
            end if  ! ( debug )

         end do

         if ( iprint>2 ) then
            write(irefwr,110) x,y,ipoint,coor(1:2,ipoint)
         end if ! ( iprint>2 )

110      format( ' Nearest nodal point to the point (', 2(ss,1pe12.3e2), &
              ') is node ', i5/' with co-ordinates (', 2(ss,1pe12.3e2), &
              ')' )

      case (3)

!     --- ndim = 3

         distx = coor(1,1)-x
         disty = coor(2,1)-y
         distz = coor(3,1)-z
         dist = distx**2+disty**2+distz**2
         do i = 2,npoint
            dx = coor(1,i)-x
            dy = coor(2,i)-y
            dz = coor(3,i)-z
            d = dx**2+dy**2+dz**2
            if(d<dist) then
               dist = d
               ipoint = i
            end if
         end do
         if ( iprint>2 ) then
            write(irefwr,210) x,y,z,ipoint, &
               coor(1:3,ipoint)
         end if ! ( iprint>2 )
210      format( ' Nearest nodal point to the point (', 3(ss,1pe12.3e2), &
              ') is node ', i5/' with co-ordinates (', 3(ss,1pe12.3e2), &
              ')' )

      end select  ! case ( ndim )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End findxy'

      end if  ! ( debug )
      call erclos ( 'findxy' )

      end
