      subroutine prextractunk ( isol, jdegfd, isolcp, newvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extract degree of freedom jdegfd from the vector corresponding
!     to isol and store in the vector corresponding to isolcp
!     if isol has only one unknown per point, isolcp is identical to isol
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
!     extract
!     extract_unknown
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: jdegfd, isol
      integer, intent(out) :: isolcp
      logical, intent(out) :: newvec

!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     isolcp         o    Standard SEPRAN array containing information of the
!                         new solution array with only one unknown per point
!     jdegfd         i    Sequence of degree of freedom to be extracted
!     newvec         o    If true isolcp is a new vector otherwise
!                         isolcp contains exactly isol, which means that
!                         removing the array corresponding to isolcp
!                         also implies removing the array corresponding to isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: itypvc, ivec

!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCOPSEPVEC   Copy one degree of freedom of sepran vector in user vector
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPCREASTRUC   find structure of vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prextractunk' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prextractunk'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First activate isol and compute information

      call sepactsolbf2 ( isol )

      if ( nunkpi>1 .and. jdegfd>0 ) then

!     --- Create new vector from old vector by extracting degree of freedom

         newvec = .true.

!        --- Find structure of vector with exactly 1 parameters per node

         itypvc = 110
         iprob  = ks(isol)%iprob  ! problem number
         call sepcreastruc ( 1, itypvc, ivec )
         if ( ierror/=0 ) go to 1000

!        --- Create new vector

         call sepcreasolbf1 ( isolcp, itypvc, ivec, 'isolcp', 0 )
         vectout => usoltemp

!        --- Copy degree of freedom

         call sepcopsepvec ( isol, jdegfd, vectout )

      else

!     --- Copy vector into isolcp

         call sepcopyvc ( isol, isolcp )
         newvec = .false.

      end if  ! ( nunkpi>1 .and. jdegfd>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prextractunk'

      end if  ! ( debug )
      call erclos ( 'prextractunk' )

      end subroutine prextractunk
