      subroutine prveccombinat7 ( isol, iseqvec, vecs, numvecrhs, idegfd, &
                                  istore, store, icount, numrealrhs, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate right-hand side of vector expression
!     Special case: a part of the real vector must be computed
!                   the number of unknowns per point is constant
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     expression
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), iseqvec, vecs(*), numvecrhs, &
              istore(*), icount, numrealrhs, &
              idegfd, nnodes
      double precision store(*)

!     icount         i    Computed length of array istore
!     idegfd         i    Degree of freedom sequence number
!     iseqvec        i    sequence number of vector in left-hand side with
!                         to isol
!     isol           i    Standard SEPRAN array containing information of the
!                         respect solution array
!     istore         i    Contains the expression as stored in common
!                         cexpression, where all scalars and reals
!                         have been computed and stored in store
!                         References to these values are given by
!                         their position in store plus 1000
!                         The values of the vectors are stored in
!                         array vectors
!                         References to these vectors are given by
!                         their position in vectors plus 10000
!     nnodes         i    Number of nodes in array nodes
!     numrealrhs     i    Number of reals stored in store
!     numvecrhs      i    Number of vectors in right-hand side
!                         Length of array vecs
!     store          i    Contains real information of the expression
!                         See subroutine READEXPR, array work
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenvec
      double precision, allocatable :: outvec(:), vectors(:,:)

!     lenvec         Length of left-hand side vector in expression
!     outvec         Output vector; result of expression in the nodes of nodes
!     vectors        Array of size lenvec x numvecrhs to store the vectors
!                    of the right-hand side in the nodes required
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRCOPYUS2VEC2  Copy user vector to sepran vector
!                    The user vector has a variable number of degrees of
!                    freedom per point
!     PREVALVECEXPR  Evaluate vector expression and fill actual values of
!                    vectors
!     PRINRL         Print 1d real vector
!     PRVECFILL1     Fill all right-hand side vectors in expression where
!                    either idegfd>0 or nunkpi = 1
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3162   Option not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prveccombinat7' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prveccombinat7'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- activate array

      call sepactsolbf2 ( isol(iseqvec) )

      if ( debug ) then
         write(irefwr,1) 'idegfd, nunkpi', idegfd, nunkpi
      end if  ! ( debug )

      if ( idegfd>0 .or. nunkpi==1 ) then

!     --- Only one degree of freedom must be filled

         lenvec = nnodes       ! 1 degree of freedom per point

         allocate ( outvec(lenvec), vectors(lenvec,numvecrhs), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, lenvec, 'outvec' )
         if ( ierror/=0 ) go to 1000

!        --- Fill all right-hand side vectors

         call prvecfill1 ( isol, iseqvec, vecs, numvecrhs, vectors, &
                           lenvec, nnodes, nunkpi, idegfd )

!        --- Fill output vector by computing the expression

         call prevalvecexpr ( outvec, vectors, lenvec, &
                              numvecrhs, istore, store, icount, &
                              numrealrhs, names(iseqvec) )

         if ( debug ) call prinrl ( outvec, lenvec, 'outvec' )

!        --- Map output vector in isol
!            Find starting address

         usoltemp => ks(isol(iseqvec))%sol

!        --- copy

         call prcopyus2vec2 ( usoltemp, outvec, nnodes, &
                              max(1,idegfd), nodes )

      else

!     --- More degrees of freedom must be filled

         call errsub ( 3162, 0, 0, 0 )
         go to 1000

      end if  ! ( idegfd>0 )

      if ( allocated(outvec) ) deallocate ( outvec, stat = error )
      if ( allocated(vectors) ) deallocate ( vectors, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'outvec' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prveccombinat7'

      end if  ! ( debug )
      call erclos ( 'prveccombinat7' )

      end subroutine prveccombinat7
