      subroutine prvecfill3 ( isol, iseqvec, vecs, &
                              numvecrhs, vectors, leng, idegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill all right-hand side vectors in expression where either
!     idegfd>0 or nunkp = 1 (whole region)
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), iseqvec, vecs(*), numvecrhs, &
              leng, idegfd
      double precision vectors(leng,numvecrhs)

!     idegfd         i    Degree of freedom sequence number
!     iseqvec        i    sequence number of vector in left-hand side with
!                         to isol
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     leng           i    Length of the vectors to be filled
!     numvecrhs      i    Number of vectors in right-hand side
!                         Length of array vecs
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
!     vectors        o    Array of size leng x numvecrhs to store the vectors
!                         of the right-hand side in the nodes required
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iseqv

!     i              Counting variable
!     iseqv          Sequence number of vector in isol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRCOPYVEC2US1  Copy part of sepran vector to user vector
!                    The user vector has 1 unknown per point and is defined
!                    in all nodes
!                    The sepran vector is defined in all nodes and has exactly
!                    nunkp degrees of freedom per point
!     PRINRL         Print 1d real vector
!     SEPCOPSEPVEC   Copy one degree of freedom of sepran vector in user vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3163   Vector has wrong sequence number
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecfill3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecfill3'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over vectors

      do i = 1, numvecrhs

         iseqv = vecs(i)     ! sequence number of vector

         if ( debug ) &
            write(irefwr,1) 'i, iseqv, idegfd', i, iseqv, idegfd

         if ( iseqv<=maxvc1 ) then

!        --- Standard sepran vector
!            activate array

            call  sepcopsepvec ( isol(iseqv), idegfd, vectors(1,i) )
            if ( ierror/=0 ) go to 1000

         else if ( iseqv==maxvc1+1 ) then

!        --- Coordinates

            call prcopyvec2us1 ( coor, vectors(1,i), npoint, ndim, 1 )

         else if ( iseqv<=maxvc1+1+ndim ) then

!        --- One of the coordinates

            iseqv = iseqv-maxvc1-1


!           --- Copy vector

            call prcopyvec2us1 ( coor, vectors(1,i), npoint, ndim, iseqv )

         else

!        --- Unknown vector

            call errint ( iseqv, 1 )
            call errchr ( names(iseqvec), 1 )
            call errsub ( 3163, 1, 0, 1 )
            go to 1000

         end if

      end do  ! i = 1, numvecrhs

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( vectors(1,1), npoint, 'vectors' )
         write(irefwr,*) 'End prvecfill3'

      end if  ! ( debug )
      call erclos ( 'prvecfill3' )

      end subroutine prvecfill3
