      subroutine prvecfillnods ( nodes, nnodes, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 14-01-2015 Extension with iplace
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes with the nodes that must be used in the creation
!     of a vector
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodes(*), nnodes, inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine ASKVEC
!     nnodes         o    Number of nodes in nodes
!     nodes          o    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nfilled, ncurves, istart, nparts, &
              ielgrp, i, iinput(2), iplace, nelgrplc
      integer, allocatable :: iwork(:), ipoint(:)

!     i              Counting variable
!     ielgrp         element group number
!     iinput         integer input array
!     iplace         Defines from which items nodes must be filled
!     ipoint         Dummy array of length nparts+1
!     istart         Starting address in inpsep with information of curves
!     iwork          integer work array of length npoint
!                    Is used to mark nodes on curves
!     ncurves        Number of curves that must be used to fill nodes
!     nelgrplc       Number of element groups
!     nfilled        Number of times array nodes has been filled
!     nparts         number of parts the boundary consists of (dummy)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRFILLNODELGRP Fill array nodes for element groups
!     PRININ         print 1d integer array
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecfillnods' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecfillnods'
         call prinin ( inpsep, 20, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), ipoint(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

      iwork(1:npoint) = 0

      nfilled = 0  ! nodes has not been filled

!     --- Make a list of all nodes that must be filled

      if ( inpsep(15)>0 ) then

!     --- inpsep(15)>0, ncurves must be filled

         iplace = inpsep(16)       ! type of entity (curves/surfaces)
         istart = inpsep(15)       ! starting address of curves in inpsep
         ncurves = inpsep(istart)  ! number of curves
         call sepfillnodes ( iplace, ncurves, inpsep(istart+1), nodes, &
                             nnodes, ipoint, nparts )

         iwork(nodes(1:nnodes)) = 1  ! all nodes in the curves are marked
         nfilled = nfilled+1  ! raise number of times nodes has been filled

      end if  ! ( inpsep(15)>0 )

      if ( inpsep(14)>0 ) then

!     --- inpsep(14)>0, skip some element groups
!         This can not be combined with curves, surfaces etc.

         istart = inpsep(14)      ! starting address of el groups in inpsep
         nelgrplc = inpsep(istart)  ! number of element groups

         if ( debug ) write(irefwr,1) 'nelgrplc', nelgrplc

         do ielgrp = 1, nelgrplc

            iinput(1) = inpsep(istart+ielgrp)
            iinput(2) = 0

            if ( debug ) write(irefwr,1) 'elgrp', iinput(1)
            call prfillnodelgrp ( iinput, nodes, nnodes )
            iwork(nodes(1:nnodes)) = 1  ! all nodes in the group are marked
            nfilled = nfilled+1  ! raise number of times nodes has been filled

            if ( debug ) &
               write(irefwr,1) 'nnodes, nfilled', nnodes, nfilled

         end do  ! ielgrp = 1, nelgrplc

      end if  ! ( inpsep(14)>0 )

      if ( nfilled>1 ) then

!     --- nodes has been filled more than once; refill

         nnodes = 0
         do i = 1, npoint
            if ( iwork(i)==1 ) then
               nnodes = nnodes+1
               nodes(nnodes) = i
            end if  ! ( iwork(i)==1 )
         end do  ! i = 1, npoint

      end if  ! ( nfilled>1 )

      deallocate ( iwork, ipoint, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,*) 'End prvecfillnods'

      end if  ! ( debug )
      call erclos ( 'prvecfillnods' )

      end
