      subroutine readfn ( ifunc, value, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.8  date 10-01-2015 New call to readreal
!        version  2.7  date 04-02-2009 Correction minus sign
!        version  2.6  date 10-03-2008 Replace readsr by readreal
!
!   copyright (c) 1993-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read rest of record of the type: number
!   or value = number, time_func = i   or  func = i
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ifunc
      double precision value(*)
      logical compl

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
!     ifunc          o    If>0, it indicates the value behind func
!                         if 0, it indicates that a number or value = number has
!                         been found
!     value          o    numbers read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer minsign
      logical funcfn, timefn, openfound

!     funcfn         Indicates if func has been found (true) or not (false)
!     minsign        Indicates if min sign is found (-1) or not (1)
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
!     timefn         Indicates if time_func has been found (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     READREAL       Read a real or real expression from the input file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!     898   Parameter in FUNC = k non-positive
!    1108   No number found after VALUE = &
!    1474   Keyword contains too much characters
!    1869   No number found after keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
!    read next item
!    func_found := false
!    time_func_found := false
!    if ( next item = text ) then
!       if ( text = 'FUNC' or 'TIME_FUNC' ) then
!          func_found := true
!          if ( text =  'TIME_FUNC' ) then
!             time_func_found := true
!       else if ( text # 'VALUE' ) then
!          error
!       read next item
!    if ( func_found ) then
!       if ( next item = integer ) then
!          ifunc = next item
!          if ( time_func_found ) then
!             ifunc =  -100-ifunc
!       else
!          error
!    else if ( next item = real ) then
!       value = next item
!    else if ( next item = scalar ) then
!       set ifunc to value of scalar
!    else
!       error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readfn' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readfn'
         write(irefwr,1) 'ityprd, lastitem/1', ityprd, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read next item

      if ( ityprd/=4 .or. &
           (string(1:5)/='VALUE' .and. string(1:4)/='FUNC' ) ) then

!     --- Standard case, read next item
!         In case FUNC or value found, jump

         if ( compl ) ispecl = 5
         call asknxt

      end if  ! ( ityprd==4 .and.... )

      ispecl = 0
      ifunc = 0
      funcfn = .false.
      timefn = .false.
      minsign = 1
      if ( ityprd==6 ) then

!     --- minus sign found

         minsign = -1
         call asknxt

      end if  ! ( ityprd==6 )
      if ( debug ) &
         write(irefwr,1) 'ityprd, minsign, lastitem/2', &
                          ityprd, minsign, lastitem

      if ( ityprd==4 ) then

!     --- text found, analyse

         if ( string(1:4)=='FUNC' .or. string(1:6)=='TIME_F' ) then

!        --- Func found

             if ( debug ) write(irefwr,*) 'func found'
             funcfn = .true.
             if ( string(1:6)=='TIME_F' ) then
                timefn = .true.
             else if ( nchar>4 ) then
                call errint ( nchar, 1 )
                call errint ( 4, 2 )
                call errchr ( string(1:nchar), 1 )
                call errchr ( 'FUNC', 2 )
                call errwar ( 1474, 2, 0, 1 )
                if ( nwarn<=mxwarn+1) call errcrd
             end if

         else if ( string(1:5)/='VALUE' ) then

!        --- Wrong text found, might be a constant
!            Set lastitem back to previous value

            lastitem = lastitembef

         end if
         if ( debug ) write(irefwr,*) 'string ', string(1:nchar)

         if ( compl ) then

!        --- Complex function

            ispecl = 5
            call asknxt
            ispecl = 0

         else

!        --- Real function, read value

            if ( debug ) write(irefwr,*) 'before readreal'
            openfound = .false.
            call readreal ( openfound, .false. )

         end if  ! ( compl )

      end if
      if ( ityprd<2 .or. ityprd>3 .and. ityprd/=8 .and. &
           ityprd/=11 .or. ityprd==20 ) then

!     --- Not a number found

         call errchr ( string(1:nchar), 1 )
         call errsub ( 1869, 0, 0, 1 )
         call errcrd

      end if
      if ( funcfn ) then

!     --- FUNC = i, found

         if ( ityprd/=2 ) then

!        --- Not an integer found

            call errchr ( string(1:nchar), 1 )
            call errsub ( 574, 0, 0, 1 )
            call errcrd

         else

            ifunc = nint(realrd)
            if ( ifunc<=0 .or. ifunc>20000 ) then

!           --- IFUNC out of range

               call errint ( ifunc, 1 )
               call errint ( 20000, 2 )
               call errsub ( 898, 2, 0, 0 )
               call errcrd

            end if
            if ( timefn ) ifunc = -100-ifunc

         end if

      else if ( ityprd/=11 ) then

!     --- value found

         if ( compl .and. ityprd==8 ) then

!        --- ( found, i.e. 2 numbers

            openfound = .true.
            call readreal ( openfound, .false. )
            if ( ityprd==2 .or. ityprd==3 ) then

!           --- Number found, store in value(1)

               value(1) = minsign*realrd
               value(2) = 0d0

!              --- Check if next value is present, if so read

               openfound = .true.
               call readreal ( openfound, .false. )
               if ( ityprd==2 .or. ityprd==3 ) value(2) = realrd

            else

!           --- No value found

               call errsub ( 1108, 0, 0, 0 )
               call errcrd

            end if  ! ( ityprd==2 .or. ityprd==3 )

         else if ( ityprd==2 .or. ityprd==3 ) then

!        --- one number found

            value(1) = minsign*realrd
            if ( compl ) value(2) = 0d0

         end if

      else

!     --- ityprd=11, scalar found

         ifunc = minsign*nint(realrd)

      end if

1000  call erclos ( 'readfn' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ifunc', ifunc
         write(irefwr,2) 'value', value(1)
         write(irefwr,*) 'End readfn'

      end if  ! ( debug )
      end
