      subroutine sepactsolbf2 ( isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-10-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Activate solution array or array of special structure
!     Also activate corresponding sub arrays
!     Old subroutine sepactsolbuf
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isol

!     isol    i  The standard SEPRAN solution array or array of special
!                structure of type 115 from which the starting positions
!                must be computed and the relevant information found.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, itypvc, ivec, iproblc, leng, iseqkpsav

!     i              Loop variable
!     iproblc        Local value of iprob
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     leng           Help variable to compute the length of some arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     206   Input vector with illegal sequence number ivec
!     744   Only arrays of type 110 and 115 are permitted
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute problem number (iproblc), type of solution vector (ITYPVC)
!     and for a vector of special structure the vector number (IVEC)
!     Read solution array in IBUFFR (INI070)
!     If ( ISOL is of type 110 ) then
!        if ( KPROB part f has been filled ) then
!           Set indprfi and read KPROB part f in IBUFFR
!        if ( KPROB part p has been filled ) then
!           Set indprpi and read KPROB part p in IBUFFR
!     else if ( ISOL is of type 115 ) then
!        if ( KPROB part f has been filled ) then
!           Set indprfi and read KPROB part f in IBUFFR
!     else if ( ISOL is of type 116 or 126 ) then
!        Compute nphysi
!     else
!        wrong type of vector found: error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepactsolbf2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepactsolbf2'
         call ertrace
         write(irefwr,1) 'isol', isol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkpsav = iseqkp

      indprfi = 0
      indprpi = 0
      itypvc = ks(isol)%typevec

!     --- Compute iproblc and offset for array KPROB

      iproblc  = iprob
      iprob  = ks(isol)%iprob
      iseqkp = ks(isol)%iseqkp
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      indprpi = 0
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'iprob, iproblc, kprob, iseqkp', &
                          iprob, iproblc, kprob, iseqkp

      select case ( itypvc )

      case ( 110 )

!     --- ISOL is of type 110:  solution vector

         nunkpi = kprobloc(4)
         nusoli = kprobloc(5)
         nphysi = max(nunkpi,kprobloc(33))
         indprfi = kprobloc(19)
         indprpi = kprobloc(35)
         if ( indprpi>0 ) kprobpi => kprobpart%kprobp
         if ( indprfi>0 ) kprobfi => kprobpart%kprobf

         if ( debug ) &
            write(irefwr,1) 'nunkpi, nphysi, indprpi', nunkpi, nphysi, indprpi

      case ( 115 )

!     --- ISOL is of type 115:  array of special structure

         ivec   = ks(isol)%ivec
         if ( debug ) write(irefwr,1) 'ivec', ivec
         if ( ivec<1 .or. ivec>kprobloc(6) ) then

!        --- ivec out of range

            call errint ( ivec, 1 )
            call errint ( kprobloc(6), 2 )
            call errsub ( 206, 2, 0, 0 )
            ivec = 1

         end if
         ndtvec = kprobloc(6)
         indpre = kprobloc(18)
         kprobe => kprobpart%kprobe
         kprobe2 => kprobpart%kprobe2
         kprobe3 => kprobpart%kprobe3
         nphysi  = kprobe2(ivec)
         if ( nphysi<0 ) then

!        --- nphysi<0, constant number of degrees of freedom per point

            nphysi = -nphysi
            nusoli = npoint*nphysi

         else

!        --- nphysi>0, number of degrees of freedom per point not constant

            indprfi = nphysi
            nphysi = maxval(kprobe(:,ivec))
            kprobfi => kprobe3(ivec)%arr

         end if
         nunkpi = nphysi

      case ( 116, 122, 126, 129, 136, 801 )

!     --- ISOL is of type 116:  array of special structure defined per element

         nphysi = ks(isol)%nphys
         nunkpi = nphysi
         nusoli = nunkpi*nelem

      case default

!     --- wrong type of vector

         call errint ( itypvc, 1 )
         call errsub ( 744, 1, 0, 0 )

      end select  ! case ( itypvc )

!     --- Make relation with kprobfi and kprobpi

      if ( iproblc>0 .and. iproblc<maxprob ) iprob  = iproblc
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc

      if ( indprfi>0 ) nusoli = kprobfi(size(kprobfi))

      iseqkp = iseqkpsav

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nphysi, nusoli', nphysi, nusoli
         write(irefwr,*) 'End sepactsolbf2'

      end if  ! ( debug )
      call erclos ( 'sepactsolbf2' )

      end subroutine sepactsolbf2
