      subroutine sepbuild
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 17-12-2015 New call to incommat
!        version  1.0  date 31-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     the large matrix and vector are computed
!     the effect of boundary conditions is taken into account
! **********************************************************************
!
!                       KEYWORDS
!
!     diagonal_matrix
!     mass_matrix
!     right_hand_side
!     stiffness_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulematstr
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulekprob
      use sepmoduleintmat
      use sepmoduleoldvecs
      use sepmodulesmf
      use sepmodulebuild
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: timebefore(2)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRININ         print 1d integer array
!     SEPBUILDACT    Actual building of matrix and right-hand side
!     SEPBUILDSTART  Perform some tests and fill intmat with defaults in case
!                    intmat has not been filled.
!     SEPPRINTMAT    Print matrix
!     SEPPRINTV      Print solution vector or vector of special structure
!     TOADDRHSPAR    Add parts of right-hand-side at interface to lower blocks
!     TOBUILD        Perform those actions in build that are not carried out
!                    by TO0050
!     TOREORDC       Map array of solution type from matrix sequence to vector
!                    sequence. Complex version
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Perform some tests
!    Compute starting address of kprob
!    If ( there are unknowns ) then
!       Fill array iinbld with defaults
!       Perform some tests
!       Call actual building subroutine
!       Compute indprh
!       If ( there are global unknowns ) then
!          Build part of matrix and right-hand side corresponding to global
!          unknowns
!       renumber right-hand side
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbuild' )
      debug = .false.
      if ( itime>0 ) call ersettime ( timebefore )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbuild'
         write(irefwr,*) 'parallel ', parallel
         call prinin ( iinbld, iinbld(1), 'iinbld' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Perform some tests and fill intmat with defaults in case
!         intmat has not been filled
!         Compute iprob and ipkprb
!         Fill some modules like sepmodulekmesh and sepmodulekprob

      if ( debug ) write(irefwr,*) 'before sepbuildstart'
      call sepbuildstart
      if ( ierror/=0 ) go to 1000

!     --- If there are no unknowns the matrix must not be filled and
!         everything is skipped

      if ( nusol-nbound-npboun==0 ) go to 1000

!     --- Call actual building subroutine
!         From now on the modules are filled and used

      if ( debug ) write(irefwr,*) 'before sepbuildact'

      if ( imas>0 .or. iinbld(2)/=4 ) call sepbuildact
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'after sepbuildact'

!     --- Rest of building of matrix

      bufchanged = .true.
      call tobuild ( iinbld, skipelgrps )
      if ( debug ) write(irefwr,*) 'after tobuild'

      if ( iinbld(2)==6 ) then

!     --- iinbld(2) = 6, order solution array back to old sequence

         if ( compl ) then

!        --- complex case

            call toreordc ( kprobr, usol, nusol )

         else

!        --- real case

            usol = usol(kprobr(1:nusol))

         end if

      end if  ! ( iinbld(2)==6 )
      if ( debug ) write(irefwr,*) 'after torenumsolback'

!     --- If parallel case, dd storage
!         Add right-hand side from interface unknowns to lower blocks

      if ( parallel .and. typecoupl>0 ) call toaddrhspar

      if ( allocated(rhsds) ) deallocate ( rhsds, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'rhsds' )
      if ( allocated(istold) ) deallocate ( istold, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'istold' )
      if ( allocated(istold1) ) deallocate ( istold1, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'istold1' )
      if ( allocated(oldsols) ) deallocate ( oldsols, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'oldsols' )
      if ( allocated(indoldsols) ) deallocate ( indoldsols, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'indoldsols' )

      deallocate ( skipelgrps )

      if ( fillkmeshw ) then

!     --- Remove nintprop and realprop

         nintprop = 0
         nrealprop = 0
         kmeshw(1:2) = 0

      end if  ! ( fillkmeshw )

1000  if ( debug ) then

!     --- debug = true, print matrix and rhsd

         if ( SMF%irhsd(1)>0 ) call sepprintv ( SMF%irhsd, 'rhs' )
         if ( stiffmat(1)>0 ) call sepprintmat ( stiffmat )
         if ( massmat>0 ) then
            write(irefwr,*) 'mass matrix'
            call sepprintmat ( massmat )
         end if  ! ( massmat>0 )
         write(irefwr,*) 'End sepbuild'

      end if
      bufchanged = .true.
      call erclmn ( 'sepbuild', timebefore )

      end subroutine sepbuild
