      subroutine sepcomnor ( norm, weight, icurvs, normaldr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the averaged normals in all points on the surfaces isrf1 to isrf2
!     Old subroutine ercomnor
! **********************************************************************
!
!                       KEYWORDS
!
!     average
!     normal_vector
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision norm(3,npoint), weight(npoint)
      integer :: icurvs(*), normaldr

!     icurvs         i    Integer array containing the surface numbers
!                         Pos. 1 indicates how array icurvs is filled.
!                         Possible values:
!                          0   only one surface is used. The
!                              surface number is stored in icurvs(2)
!                         -1   The solution is used along the
!                              surfaces icrv1 to icrv2. icrv1 is stored in
!                              icurvs(2) and icrv2 in icurvs(3).
!                         n>0  The surface numbers are stored
!                              in icurvs(2) to icurvs(n+1).
!                              That sequence is used.
!     norm           o    Array of length ndim * npoint containing the normal in
!                         the nodal points
!     normaldr       i    If 1 the outward directed normal is required, if -1
!                         the inward directed normal
!     weight              Help array of length npoint to store the weights
!                         for the averaging
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurnr, i, k, nodp, ichoice, isrf1, isrf2
      double precision h

!     h              Help variable to compute the length of a vector
!     i              Counting variable
!     ichoice        Choice parameter
!     isrf1          First surface number
!     isrf2          Last surface number
!     isurnr         Surface sequence number
!     k              Counting variable
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     SEPCOMNOR1     ?Body of subroutine SEPCOMNOR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2337   Element incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear arrays norms and weights
!     For isurnr := isrf1 (1) isrf2
!        For all elements ielem in surface isurnr
!           For all nodes nodp in ielem do
!              Compute the normal in the point using the right-handed screw
!              direction and the direction of the elements
!              Add normal to array norms
!              Add 1 to array weights
!     Divide all entries in norms by corresponding weights
!     Normalize the entries in norms
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcomnor' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcomnor'
         call prinin ( icurvs, 3, 'icurvs' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays norms and weights

      weight = 0d0
      norm = 0d0

!     --- For isurnr := isrf1 (1) isrf2

      ichoice = icurvs(1)
      if ( ichoice>=0 ) then

!     --- ichoice>0;  ichoice surfaces

         do k = 1, max(1,ichoice)

            isurnr = icurvs(k+1)
            if ( kmeshnglob(isurnr)>0 ) &
               call sepcomnor1 ( norm, weight, normaldr, isurnr )

         end do

      else

!     --- ichoice<0;  surfaces icrv1 to icrv2

         isrf1 = icurvs(2)
         isrf2 = icurvs(3)
         isurnr = isrf1

         do isurnr = isrf1, isrf2

            if ( kmeshnglob(isurnr)>0 ) &
               call sepcomnor1 ( norm, weight,normaldr, isurnr )

         end do
         isurnr = isrf1

      end if

!     --- Divide all entries in norms by corresponding weights
!         Normalize the entries in norms

      do nodp = 1, npoint

         if ( weight(nodp)>0d0 ) then

!        --- Point is in surface

            h = 1d0/weight(nodp)
            do k = 1, 3
               norm(k,nodp) = norm(k,nodp)*h
            end do

            h = sqrt(norm(1,nodp)**2+norm(2,nodp)**2+norm(3,nodp)**2)

            if ( h>0d0 ) then
               h = 1d0/h
            else
               call errint ( nodp, 1 )
               call errint ( 0, 2 )
               call errint ( isurnr, 3 )
               call errwar ( 2337, 3, 0, 0 )
               h = 1d0
            end if

            do k = 1, 3
               norm(k,nodp) = norm(k,nodp)*h
            end do

         end if

      end do

1000  call erclos ( 'sepcomnor' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcomnor'

      end if

      end
