      subroutine sepcrmatrix ( matr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     create a new array in realmt
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulematr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: matr

!     matr         i/o   Defines matrix information by integer sequence number
!                        If it already exists the old info is removed
!                        New matr info is created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lencols, ipstart, icompl
      type (globmat), pointer :: S
      type (intmatrix), pointer :: IM
      type (rmatrix), pointer :: Ar
      type (cmatrix), pointer :: Ac

!     Ac             Refers to realmt(matr) (complex matrix)
!     Ar             Refers to realmt(matr) (real matrix)
!     IM             Refers to intmt(intmat)
!     S              pointer to realmt(iseqmt)
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     ipstart        Starting address in array
!     lencols        length of the part columns of the matrix structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCRMATPART1  create a new array in realmt without allocation of space
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcrmatrix' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcrmatrix'
         write(irefwr,1) 'lastmatrix, nummat, matr', &
                          lastmatrix, nummat, matr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcrmatpart1 ( matr )

      IM => intmt(intmat)

      S => realmt(matr)

      length = 0
      icompl = 1
      if ( associated(IM%colsip) ) &
         length = length+size(IM%colsip)*2+size(IM%colspp)
      if ( associated(IM%colsic) ) length = length+size(IM%colsic)
      if ( associated(IM%colsout) ) length = length+size(IM%colsout)
      if ( associated(IM%colsfi) ) length = length+2*size(IM%colsfi)

      if ( S%complexmat ) icompl = 2

      if ( IM%simple ) then

!     --- Simple

         nrusolvel   = IM%nu
         nrusolpress = IM%np
         lencols = size(IM%colsQ)

!        --- Address simple matrices
!            First Q-matrix

         ipstart = lencols+size(IM%colsG)+size(IM%colsPM)*2+&
                   nrusolpress+nrusolvel

         if ( .not. IM%Gsymmetric ) ipstart = ipstart+size(IM%colsG)
         length = length+ipstart

         allocate ( S%S_ii(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'S%S_ii' )

         S%S_Q => S%S_ii(1:lencols)   ! Q-matrix
         ipstart = lencols

         S%S_DIV => S%S_ii(ipstart+1:ipstart+size(IM%colsG))   ! D-matrix
         ipstart = ipstart+size(IM%colsG)

         if ( .not. IM%Gsymmetric ) then
            S%S_G => S%S_ii(ipstart+1:ipstart+size(IM%colsG))   ! G-matrix
            ipstart = ipstart+size(IM%colsG)
         end if  ! ( .not. IM%Gsymmetric )

         S%S_C => S%S_ii(ipstart+1:ipstart+size(IM%colsPM))   ! C-matrix
         ipstart = ipstart+size(IM%colsPM)

         S%S_PM => S%S_ii(ipstart+1:ipstart+size(IM%colsPM))  ! PM-matrix
         ipstart = ipstart+size(IM%colsPM)

         S%S_PMD => S%S_ii(ipstart+1:ipstart+nrusolpress)  ! diagonal PM mat
         ipstart = ipstart+nrusolpress

         S%S_UD => S%S_ii(ipstart+1:ipstart+nrusolvel)  ! diagonal vel mat
         ipstart = ipstart+nrusolvel

      else if ( IM%nosepmat ) then

!     --- No sepran matrix

         allocate ( S%S_ii(max(1,length)) )
         ipstart = 0

      else

!     --- Standard case, not simple

         if ( IM%complexm .and. IM%direct ) then
            ipstart = IM%diag(IM%nrusol)
            if ( .not. IM%symmetric ) &
               ipstart = ipstart+nrusol-IM%colsii(IM%nrusol)
         else
            ipstart = IM%rowsii(IM%nrusol+1)
         end if  ! ( IM%direct )
         length = (length+ipstart)*icompl
         allocate ( S%S_ii(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'S%S_ii' )

         ipstart = ipstart*icompl

      end if  ! ( IM%simple )

      nullify ( S%S_pi, S%S_ip, S%S_pp, S%S_ic, S%matinv, S%Dlapshift  )
      nullify ( S%S_if, S%S_fi, S%S_iout )

      if ( associated(IM%colsip) ) then

!     --- Essential boundary conditions

         length = size(IM%colsip)*icompl
         S%S_ip => S%S_ii(ipstart+1:ipstart+length)
         ipstart =ipstart+length
         S%S_pi => S%S_ii(ipstart+1:ipstart+length)
         ipstart =ipstart+length
         length = size(IM%colspp)*icompl
         S%S_pp => S%S_ii(ipstart+1:ipstart+length)
         ipstart =ipstart+length

      end if  ! ( associated(IM%colsip) )

      if ( associated(IM%colsic) ) then

!     --- Periodical boundary conditions

         length = size(IM%colsic)*icompl
         S%S_ic => S%S_ii(ipstart+1:ipstart+length)
         ipstart =ipstart+length

      end if  ! ( associated(IM%colsic) )

      if ( associated(IM%colsfi) ) then

!     --- There are interface unknowns

         length = size(IM%colsfi)*icompl
         S%S_if => S%S_ii(ipstart+1:ipstart+length)
         ipstart =ipstart+length*icompl
         S%S_fi => S%S_ii(ipstart+1:ipstart+length)
         ipstart =ipstart+length

      end if  ! ( associated(IM%colsfi) )

      if ( associated(IM%colsout) )  then
         length = size(IM%colsout)*icompl
         S%S_iout => S%S_ii(ipstart+1:ipstart+length)
      end if  ! ( associated(IM%colsout) )

      S%decomp = .true.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'matr, lastmatrix, matr', &
                          matr, lastmatrix, matr
         write(irefwr,*) 'End sepcrmatrix'

      end if  ! ( debug )
      call erclos ( 'sepcrmatrix' )

      end subroutine sepcrmatrix
