      subroutine sepcurvenodes ( icurvs, ipoint, inodes, nparts, kmeshmloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all nodes of one curve in array inodes and gaps in
!     these curves in array ipoint
!     Old subroutine ercurvenodes
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurvs(*), ipoint(*), inodes(*), nparts, kmeshmloc(*)

!     icurvs         i    Integer array containing the curve or surface
!                         numbers that must be printed.
!                         Pos. 1 indicates how array icurvs is filled.
!                         Possible values:
!                          0   only one curve is used. The curve is stored in
!                              icurvs(2)
!                         -1   The  curves icrv1 to icrv2 are used.
!                              icrv1 is stored in icurvs(2) and icrv2
!                              in icurvs(3).
!                         -2   See -1, however, the sequence is reversed.
!                              This means that the curves are stored from
!                              the end point of icrv2 to the initial point
!                              of icrv2, then the end point of icrv2-1 to the
!                              begin point of icrv2-1 and so on.
!                         n>0  The curve numbers numbers are stored
!                              in icurvs(2) to icurvs(n+1).
!                              That sequence is used.
!                         Remark: a negative curve number means that the curve
!                              is printed from end point to initial point, hence
!                              in reversed order.
!     inodes         o    Array containing the nodal point numbers along curve.
!                         The nodes along the curves are ordered in
!                         the sequence indicated by icurvs. Common nodes in
!                         consecutive curves are counted for only once.
!                         If a gap occurs, i.e. if curves are no longer
!                         consecutive, the new curve is considered as start of
!                         a new part. Starting addresses of all parts are stored
!                         in array ipoint.
!     ipoint         o    Array of length nparts+1 containing the starting
!                         addresses in array inodes of the various parts
!                         the boundary consists of.
!                         ipoint(1) = 1, &
!                         number of nodes at part i = ipoint(i+1)-ipoint(i)
!     kmeshmloc      i    Local array KMESH part m
!                         This is either kmeshm or kmeshmglob
!     nparts        i/o   number of parts the curves are subdivided in
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsubcv, istore(ncurvs), istar3, numpoints, k, icrv1, &
              jcrv1, jdirec, ifirst, i1, i2, ilast, i, j, numnodes

!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     icrv1          First curve number
!     ifirst         First curve (surface) number read
!     ilast          Last curve (surface) number read
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istore         work array of length ncurvs to store all subcurves
!     j              Counting variable
!     jcrv1          abs(icrv1)
!     jdirec         Direction parameter
!     k              Counting variable
!     nsubcv         Number of subcurves of a curve
!     numnodes       Number of points along curve
!     numpoints      Number of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPSTORECURVS  Fill array istore with (sub) curve numbers of the curves
!                    stored in array icurvs using the parameter ichois
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     139   curve number does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcurvenodes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcurvenodes'
         write(irefwr,1) 'nparts', nparts
  1      format ( a, 1x, (10i6) )
         call prinin ( icurvs(2), max(1,icurvs(1)), 'icurvs' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First store all (sub) curves in array istore

      call sepstorecurvs ( icurvs(1),nsubcv, istore, icurvs(2) )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nsubcv, ncurvs', nsubcv, ncurvs
         call prinin ( istore, nsubcv, 'istore' )

      end if

!     --- fill ipoint and inodes

      istar3 = kmeshmloc(2)-1
      ipoint(1) = 1
      numpoints = 1
      do k = 1, nsubcv
         icrv1 = istore(k)
         jcrv1 = abs(icrv1)
         if ( jcrv1==0 .or. jcrv1>ncurvs ) then
            call errint ( jcrv1, 1 )
            call errint ( ncurvs, 2 )
            call errsub ( 139, 2, 0, 0 )
            jcrv1 = 1
         end if
         if ( icrv1>0 ) then
            jdirec = 1
            i1 = kmeshmloc(4+jcrv1)
            i2 = kmeshmloc(5+jcrv1)-1
            numnodes = i2-i1
         else
            jdirec = -1
            i1 = kmeshmloc(5+jcrv1)-1
            i2 = kmeshmloc(4+jcrv1)
            numnodes = i1-i2
         end if

         if ( debug ) then

!        --- Debug information

            write(irefwr,1) 'k, icrv1, i1, i2, jdirec', &
                             k, icrv1, i1, i2, jdirec
            write(irefwr,1) 'numpoints, nparts, numnodes', &
                             numpoints, nparts, numnodes

         end if  ! ( debug )

         if ( numnodes>0 ) then

!        --- numnodes>0, hence curve is not empty

            ifirst = kmeshmloc(istar3+i1)
            if ( numpoints>1 ) then

!           --- At least second curve

               ilast = inodes(numpoints)
               if ( ifirst/=ilast ) then

!              --- gap found, raise nparts

                  nparts = nparts+1
                  numpoints = numpoints+1
                  ipoint(nparts) = numpoints

               end if

            end if

!           --- fill array inodes

            do j = i1, i2, jdirec
               inodes(numpoints+jdirec*(j-i1)) = kmeshmloc(istar3+j)
            end do
            numpoints = numpoints+(i2-i1)*jdirec
            if ( debug) &
               write(irefwr,1) 'numpoints, nparts', numpoints, nparts

         end if  ! ( i2/=i1 )

      end do

!     --- Check if the curves had been filled by subroutine mesh
!         If so all positions in inodes must correspond to a nodal point
!         number

      if ( numpoints==1 ) numpoints = 0
      do i = 1, numpoints
         if ( inodes(i)<=0 ) inodes(i) = -i
      end do
      ipoint(nparts+1) = numpoints+1
      if ( numpoints==0 ) nparts = 0

1000  call erclos ( 'sepcurvenodes' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nparts', nparts
         call prinin ( ipoint, nparts+1, 'ipoint' )
         call prinin ( inodes, numpoints, 'inodes' )
         write(irefwr,*) 'End sepcurvenodes'

      end if

      end subroutine sepcurvenodes
