      subroutine sepoutcurvnod ( nnodes, nodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all nodes in the outer curves and store in array nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     curve
!     outer_boundary
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(*)

!     nnodes         o    Number of nodes in array nodes
!     nodes          o    Contains the node numbers of the outer boundary
!                         at output
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nparts, j, istart, ncurves, icurnr, istep, &
                 npointscurve, k, inodp, ipart
      integer, allocatable :: iwork(:), iwork1(:)

!     icurnr         Curve sequence number
!     inodp          Nodal point number
!     ipart          Sequence number of part
!     istart         First position used at a part
!     istep          Input parameter in er0026
!     iwork          integer work array of length npoint
!                    Is filled by subroutine er0026:
!                    array containing the nodal point numbers of the
!                    curves subsequently
!                    If point does not belong to a node, the address of
!                    the point with respect to curves provided with a
!                    minus sign is stored
!     iwork1         integer work array of length npoint
!                    iwork1 is to mark points that have been stored
!     j              Counting variable
!     k              Counting variable
!     ncurves        Number of curves in part
!     nparts         number of parts the boundary consists of
!     npointscurve   number of points along a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPER0026      fill nodal points of curves icrv1 to icrv2 in array iwork
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepoutcurvnod' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepoutcurvnod'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), iwork1(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork1

      iwork1 = 0

      nparts = outerboun(1)    ! number of parts in the outer boundary
      nnodes = 0
      istep = 1

!     --- For all parts do

      do ipart = 1, nparts

         istart = outerboun(ipart+1)
         ncurves = outerboun(istart)

         do j = 1, ncurves

            icurnr = abs(outerboun(istart+j))
            call seper0026 ( icurnr, icurnr, istep, &
                               npointscurve, iwork, .false., kmeshm )

            do k = 1, npointscurve

               inodp = iwork(k)
               if ( inodp>0 ) then

!              --- Node number is positive

                  if ( iwork1(inodp)==0 ) then

!                 --- Node number has not been used yet
!                     add to array and mark as used

                     nnodes = nnodes+1
                     nodes(nnodes) = inodp
                     iwork1(inodp) = 1

                  end if  ! ( iwork1(inodp)==0 )

               end if  ! ( inodp>0 )

            end do  ! k = 1, npointscurve

         end do  ! k = 1, ncurves

      end do  ! ipart = 1, nparts

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,*) 'End sepoutcurvnod'

      end if  ! ( debug )
      call erclos ( 'sepoutcurvnod' )

      end subroutine sepoutcurvnod
