      subroutine sepsurnod ( ifirst, ilast, nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all nodes in the surfaces ifirst to ilast in array nodes
!     The number of nodes in nodes is stored in nnodes
!     At this moment the surfaces must be part of kmeshn
!     Old subroutine ersurnod
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ifirst, ilast, nodes(*), nnodes

!     ifirst         i    First surface number
!     ilast          i    Last surface number
!     nnodes        i/o   Number of nodes in nodes
!                         At input declared length of array nodes
!     nodes          o    Contains the node numbers in the surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isurnr, istart, inpelm, inode, lennodes
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     inode          Node sequence number
!     inpelm         number of points per element
!     istart         First position of a surface in kmeshn minus 1
!     isurnr         Surface sequence number
!     iwork          integer work array of length npoint
!                    iwork(i) = 0, means node i has not yet been filled
!                    iwork(i) = 1, means node i has been filled
!     lennodes       Length of array nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array iwork
!    nnodes := 0
!    For isurnr := ifirst (1) ilast do
!       inelem := number of elements in isurnr
!       inpelm := number of nodes in elements in isurnr
!       istart := starting address of surface in kmeshc
!       For all nodes inode in surface do
!          if ( iwork(inode)=0 ) then
!             iwork(inode):=1
!             nnodes := nnodes+1
!             nodes(nnodes) := inode
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsurnod' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsurnod'

      end if
      if ( ierror/=0 ) go to 1000
      lennodes = nnodes

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

      nnodes = 0
      istart = 2*nsurfs
      do isurnr = 1, ifirst-1
         istart = istart+kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
      end do

!     --- Loop over surfaces

      do isurnr = ifirst, ilast

         inelem = kmeshn(isurnr+nsurfs)
         inpelm = kmeshn(isurnr)

!        --- For all nodes inode in surface

         do i = istart+1, inelem*inpelm+istart
            inode = kmeshn(i)
            if ( inode>0 ) then

!           --- Node number positive

               if ( iwork(inode)==0 ) then

!              --- iwork(inode)=0

                  iwork(inode)=1
                  nnodes = nnodes+1
                  call inichkarlen ( nnodes, lennodes, 'nodes' )
                  if ( ierror/=0 ) go to 1000
                  nodes(nnodes) = inode

               end if  ! ( iwork(inode)==0 )

            end if  ! ( inode>0 ) then

         end do  ! i = istart+1, inelem*inpelm+istart

         istart = istart+inelem*inpelm

      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsurnod'

      end if
      call erclos ( 'sepsurnod' )

      end
