      subroutine topqu0 ( ichois, number, iskip, iskpar, iresul, compl, &
                          ifill, special, ilevel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.14 date 20-01-2015 Use subroutines
!        version  3.13 date 11-09-2013 New call to topqu8
!        version  3.12 date 15-11-2012 Long integers
!
!   copyright (c) 1993-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check if a mesh must be subdivided into sub elements
!    If necessary perform the subdivision and construct an output sol. array
!    The following tasks are performed:
!    If there are volume elements check whether a surface grid or a volume
!    grid is needed
!    Construct either a volume grid or construct a grid consisting of the
!    outer surface only
!    If required make a new grid consisting of the vertices of this
!    grid only
!    If required subdivide the original grid or the surface grid into
!    triangular or tetrahedral elements based on the
!    presence of the required degree of freedom.
!    Furthermore store the required degree of freedom in an output vector
!    Finally (if required, i.e. ifill=1) fill array ihelp. In ihelp it is
!    stored which new elements (elements ihelp(i)+1 up to ihelp(i+1)
!    originated from the old i'th element group. If properties for group i
!    have been given, for example the porosity, these properties can be used
!    for the new elements utilizing array ihelp
!    Remark: both output meshes consist of one type of elements only!
!    Besides that some arrays are checked (KMESH, KPROB, ISOL)
!    and some arrays are activated (COOR, new grids, new solution array,
!    old solution array, KPROB parts f/h/p), depending on ichois
!    Warning:  subroutine TOPQU0 uses memory management subroutines
!              This means that all pointers must be recomputed after
!              this subroutine
!    If more types of elements must be used, subroutine PL0013 must be
!    incorporated
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulework
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ichois, number(*), &
                 iresul(5,*), iskip, iskpar(*), ifill, special, ilevel
      logical compl

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
!     ichois         i   Choice parameter. With this parameter it is defined
!                        which actions will be performed. Possible values:
!                        1:  Create a submesh consisting of 3-point triangles
!                            only
!                            Create also a copy of the solution array containing
!                            number(1) unknowns in all points of the new grid
!                            In this case arrays KPROB and ISOL are also used
!                            If the solution vector is complex only the real or
!                            imaginary part is used
!                        2:  Create a submesh consisting of 3-point triangles
!                            only
!                            Create also a copy of the solution array containing
!                            one unknown only in all points of the new grid
!                            In this case array KPROB and ISOL is also used
!                            If the solution vector is complex the copied
!                            solution vector is also complex
!                        3:  Create a submesh consisting of the surface elements
!                            only of each element only the vertices are stored, &
!                            hence the final grid consists of a grid of 3-point
!                            triangles or 4-point quadrilaterals only
!                            This possibility only uses KMESH
!                            It has only been implemented for 3D
!                        4:  Combination of 1 and 3, hence possibly 2 meshes may
!                            be created
!                        5:  Combination of 2 and 3, hence possibly 2 meshes may
!                            be created
!                        6:  Create a submesh consisting of 4-point tetrahedrons
!                            only
!                            Create also a copy of the solution array containing
!                            number(1) unknowns in all points of the new grid
!                        7:  See 6, but now a choice between tetrahedra and
!                            hexahedrons is made
!     ifill          i   Indicator whether help array iwork1 should be filled
!                        (1) or not (0)
!     ilevel         i   Defines the level of elements to be plotted
!     iresul         o   Output array of length 5,1 if ichois=2 and otherwise of
!                        length 5,3
!                        The array is filled as follows:
!                        Pos .,1 corresponds to the "vertex" mesh array
!                                (ichois=2,3)
!                        Pos .,2 corresponds to the "triangle" mesh array
!                                (ichois=1,3)
!                        Pos .,3 corresponds to the solution array (ichois=1,3)
!                        Pos. 1,1  INDVER  sequence number with respect to
!                                          INFOR for the "vertex" mesh array
!                        Pos. 2,1  NPELM   number of points per element for
!                                          this mesh
!                        Pos. 3,1  NELEM   number of points per element for
!                                          this mesh
!                        Pos. 4,1  NEWVER  indicates if the mesh is new (1) or
!                                          is in fact the standard mesh (0)
!                        Pos. 5,1  INDLIN  sequence number with respect to INFOR
!                                          for the "line-elements" array
!                        Pos. 1,2  INDTRI  sequence number with respect to INFOR
!                                          for the "triangle" mesh array
!                        Pos. 2,2  NPELM   number of points per element for
!                                          this mesh
!                        Pos. 3,2  NELEM   number of points per element for
!                                          this mesh
!                        Pos. 4,2  NEWTRI  indicates if the mesh is new (1) or
!                                          is in fact the standard mesh (0) or
!                                          the mesh created by the "vertex" mesh
!                        Pos. 5,2  nphysi   Max. number of degrees of freedom
!                        Pos. 1,3  INDSOL  sequence number with respect to INFOR
!                                          for the new solution array
!                        Pos. 2,3  NEWSOL  indicates if the solution array is
!                                          new (1) or is the standard
!                                          solution array (0)
!                        Pos. 3,3  INDPRF  sequence number of KPROBF
!                        Pos. 4,3  indprhi  sequence number of KPROBH
!                        Pos. 5,3  INPPRP  sequence number of KPROBP
!     iskip          i   Indicates if element groups must be skipped (>0) or
!                        not (0)
!     iskpar         i   array of length NELGRP that is only used if ISKIP>0
!                        It indicates if element group ielgrp must be skipped
!                        iskpar(ielgrp) = 0 or not iskpar(ielgrp) = 1
!     number         i   Array indicating which unknowns have to be considered
!                        Number(1) = total number of components
!                        Number(2) first component to be considered
!                        Number(1+number(1)) last component
!                        The number-th unknown in each nodal point of the array
!                        is considered. If the array is complex and ichois =2
!                        or 5
!                        number(2)=2*i-1 corresponds to the real part of array i
!                        and number(2)=2*i to the imaginary part of array i
!     special       i/o  Standard situation: special=0 at input
!                        If special=2 and the unknown is present in the
!                        vertices, special is set to 0, else if the unknown is
!                        only present in the midside points, special
!                        remains unchanged
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical volume, subtri, subver, vertex, outcmp
      integer i,  idegfd, ivec, centroid, nomber, neleml, nelemlc
      integer, allocatable :: itrian(:)

!     centroid       If 1 a 9 point element has been found of which the
!                    only 8 points have been filled
!     i              Counting variable
!     idegfd         Degree of freedom corresponding to number
!                    In the real case idegfd is number in the complex case it
!                    corresponds to the degree of freedom number refers to
!     itrian         work array of length nelgrp, see topqu2
!     ivec           Sequence number of vector of special structure
!     neleml         Number of "line"-elements
!     nelemlc        Local number of elements
!     nomber         Highest number of unknown to be considered in the nodes
!     outcmp         Indicates if the output vector may be complex (true) or not
!     subtri         Indicates if the mesh must be subdivided into triangles and
!                    also the solution copied (true) or not
!     subver         Indicates if a "vertex" mesh must be created (true) or not
!     vertex         indicates if only the vertices of the surface elements
!                    have to be filled (true) or all points (false)
!     volume         Indication if the mesh contains volume elements (true) or
!                    not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCONC         Fill ninth point of element of shape 6 (real case)
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSVNM         Fill ninth point of element of shape 6 (complex case)
!     INIAC0         Activate arrays ISOL, KPROB parts F, H and P, COOR
!                    Compute compl, nphysi, ivec. Check NUMBER
!     SEPCRSOLINFO   Create a new vector in ks
!     SEPQU7         Compute length of array "KMESHC", check if mesh contains
!                    volumes
!     TOPQ12         Fill help array with lengths of different element subgroups
!     TOPQU4         Compute vertex mesh
!     TOPQU5         Fill solution array with one unknown (complex case)
!     TOPQU6         Fill solution array with one unknown (complex case)
!     TOPSUBDIVMESH  subdivide volume into tetrahedrons or hexahedrons
!     TOPSURFMESH    construct a mesh consisting of surface elements only
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM # 2
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Perform some standard checks
!   Clear array IRESUL
!   Activate standard arrays
!   Check if the mesh contains volume elements
!   If ( volume elements are available and ichois<6 ) then
!      Create mesh of surface elements only
!      If ( vertex mesh must be created ) then
!         Create vertex mesh
!      If ( triangle mesh must be created ) then
!         Create triangle mesh
!   else
!      If ( vertex mesh must be created ) then
!         Create vertex mesh
!      If ( triangle/tetrahedral mesh must be created ) then
!         Create triangle mesh
!   If ( triangle/tetrahedral mesh must be created ) then
!      Copy solution
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topqu0' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topqu0'
         call ertrace
         write(irefwr,1) 'ichois, nelem, npoint, ndim', &
                          ichois, nelem, npoint, ndim
         write(irefwr,1) 'inelem, nelgrp', inelem, nelgrp
1        format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Check arrays KPROB, ISOL and compute IPKPRB

      if ( ichois<=2 .or. ichois>=4 ) then
         subtri = .true.
      else
         subtri = .false.
      end if
      if ( ichois>=3 .and. ichois<6 ) then
         subver = .true.
      else
         subver = .false.
      end if
      vertex = subver .and. .not. subtri
      centroid = 0
      nelemlc = nelem
      if ( debug ) write(irefwr,*) 'subtri, subver, vertex ', &
                                    subtri, subver, vertex

  !    if ( subtri ) call inich0 ( ipkprb )  ! Check all arrays
      if ( ndim<2 .or. ndim>3 ) then
         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 3, 3 )
         call errsub ( 419, 3, 0, 0 )
         go to 1000
      end if

      allocate ( itrian(nelgrp), stat = error )
      if ( error/=0 ) call eralloc ( error, nelgrp, 'itrian' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array IRESUL

      iresul(:,1) = 0
      if ( subtri ) then
         do i = 1, 5
            iresul(i,2) = 0
            iresul(i,3) = 0
         end do
         if ( ichois<=5 ) then
            iresul(2,2) = 3
         else
            iresul(2,2) = 4
         end if
      end if

      if ( debug ) write(irefwr,*) 'subtri', subtri

!     --- Activate standard arrays

      if ( subtri ) then

!     --- Solution array is used, check whether the variables are
!         available
!         compute nomber, which is the highest unknown sequentially

         nomber = 0

         do i = 1 , number(1)
            if ( number(i+1)>nomber ) nomber = number(i+1)
         end do

         call iniac0 ( compl, nomber, idegfd, ivec )
         if ( ichois==2 .or. ichois==5 ) then
            outcmp = compl
         else
            outcmp = .false.
         end if
         iresul(3,3) = indprfi
         iresul(4,3) = 0
         iresul(5,3) = indprpi
         iresul(5,2) = nphysi

      end if

!     --- In the case of 3D it is checked whether the grid contains
!         volume elements

      if ( ndim==3 ) then
         call sepqu7 ( volume, vertex, neleml )
         if ( debug ) write(irefwr,1) 'length, neleml', length, neleml
      else
         volume = .false.
      end if
      if ( ierror/=0 ) go to 1000
      if ( debug ) then
         write(irefwr,*) 'volume ', volume
         write(irefwr,1) 'nelem/1', nelemlc
      end if  ! ( debug )

      if ( volume .and. ichois<6 ) then

!     --- Volume elements found, construct a mesh consisting of surface
!         elements only
!         First create all faces of the elements
!         Next find all volume elements corresponding to the faces
!         Finally store all faces that correspond to only one volume element

         call topsurfmesh ( vertex, iskip, &
                            iskpar, ilevel, nelemlc, subver, iresul, &
                            subtri, idegfd, itrian, &
                            centroid, special, neleml, isolut )

      else

!     --- If ichois = 6 then volume has to be subdivided into tetrahedrons
!         if ichois = 6, 7 then volume has to be subdivided into tetrahedrons
!         or hexahedrons
!         else situation with surface elements only, check if mesh must be
!         subdivided into triangles

         call topsubdivmesh ( subver, ichois, neleml, iresul, subtri, &
                              idegfd, itrian, iskip, iskpar, &
                              special, isolut, centroid )

      end if  ! ( volume .and. ichois<6 )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,*) 'subtri ', subtri
         write(irefwr,1) 'nelem/2', nelemlc
      end if  ! ( debug )
      if ( subtri ) then

!     --- Store the required degree of freedom into array solutn

         if ( iresul(4,2)==0 .and. iresul(4,1)==0 .and. indprfi==0 &
              .and. indprpi==0 .and. (.not. compl &
              .or. compl .and. outcmp) .and. nphysi==1 .and. &
              centroid==0 ) then
            iresul(1,3) = isolut
            usolt => ks(isolut)%sol
            if ( debug ) write(irefwr,*) 'use isolut'

         else

            if ( debug ) write(irefwr,*) 'new array'
            if ( outcmp ) then
               length = 2*npoint
            else
               length = number(1)*npoint
            end if
            call sepcrsolinfo ( iresul(1,3), length, 0 )
            usolt => ks(iresul(1,3))%sol
            iresul(2,3) = 1
            usol => ks(isolut)%sol
            if ( ierror/=0 ) go to 1000
            if ( outcmp ) then

!           --- Complex version, complex output

               call topqu5 ( 1, number(2), usol, usolt )
               if ( ierror/=0 ) go to 1000
               if ( centroid==1 ) then
                  call ersvnm ( usolt, number(1) )
               end if

            else if ( compl ) then

!           --- Complex version, real output

               call topqu6 ( 1, number(2), usol, usolt )
               if ( ierror/=0 ) go to 1000
               if ( centroid==1 ) then
                  call erconc ( usolt, number(1) )
               end if

            else

!           --- Copy number(1) components, starting from number(2)

               call topqu4 ( number(1), number(2), usolt )
               if ( ierror/=0 ) go to 1000
               if ( centroid==1 ) then
                  call erconc ( usolt, number(1) )
               end if

            end if

         end if

      end if

!     --- If needed fill help array for original element groups

      if ( ifill==1 ) then

!     --- Fill help array for group properties

         call topq12( itrian, iwork1 )

      end if

      deallocate ( itrian, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'itrian' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topqu0'

      end if
      call erclos ( 'topqu0' )

      end subroutine topqu0
