      subroutine topqu2 ( number, lensub, itrian, nelemt, &
                          neleml, special, centroid, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.7  date 10-03-2006 Extension with interface elements
!        version  6.6  date 16-02-2005 Correction connection elements
!        version  6.5  date 07-02-2005 Debug statements
!
!   copyright (c) 1988-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if a mesh consists of linear triangles (2D) or linear tetra- &
!     hedrons (3D) only, or that a subdivision into triangles or tetra- &
!     hedrons must be made depending on the unknowns considered in number
!     If so, the length of the local array KMESHC, containing the new
!     mesh is computed.
!     TOPQU2 is used as help subroutine for several output subroutines
!     like INTP2D, PLCN00, PLTRAC
!     It must be used in co-operation with subroutine TOPQU3
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     mesh
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, number, lensub, nelemt, &
              itrian, neleml, special, centroid

!     centroid  o   If 1 a 9 point element has been found of which the
!                   only 8 points have been filled
!     inpelm         number of points per element
!     itrian    o   Output array of length NELGRP in which information of the
!                   various element groups is stored with respect to the
!                   subdivision into linear triangles.
!                   ITRIAN ( ielgrp ) corresponds to the ielgrp th element
!                   group and its value has the following meaning:
!                   -3:   (extra) Subdivision quadratic line element
!                   -2:   Linear line element
!                   -1:   Skip the element group
!                    0:   No subdivision required
!                    1:   Subdivision of a quadrilateral with 4 or 5 points into
!                         2 triangles
!                    2:   Subdivision of a triangle with 6 or 7 points into 4
!                         triangles
!                    3:   Subdivision of a triangle with 6 or 7 points into 1
!                         triangle
!                    4:   Subdivision of a quadrilateral with 9 points into 8
!                         triangles
!                    5:   Subdivision of a quadrilateral with 9 points into 8
!                         triangles. The centroid must be computed by
!                         interpolation
!                    6:   Subdivision of a quadrilateral with 9 points into 2
!                         triangles.
!                    7:   Subdivision of a tetrahedron with 10, 14 or 15 points
!                         into 8 tetrahedrons
!                    8:   Subdivision of a tetrahedron with 10, 14 or 15 points
!                         into 1 tetrahedron
!                    9:   Subdivision of a hexahedron with 8 points into 6
!                         tetrahedrons
!                   10:   Subdivision of a hexahedron with 27 points into 48
!                         tetrahedrons
!                   11:   Subdivision of a hexahedron with 27 points into 6
!                         tetrahedrons
!                   12:   Subdivision of a hexahedron with 27 points into 6
!                         hexahedrons
!                   13:   Subdivision of a hexahedron with 27 points into 1
!                         hexahedron
!     lensub    o   If zero the original mesh is used, otherwise lensub>0
!                   gives the length of the KMESH part c
!     neleml    o   Number of "line"elements in the new mesh
!     nelemt    o   Number of elements in the new mesh
!     number    i   The number-th unknown in each nodal point of the array
!                   is considered.
!     special  i/o  Standard situation: special=0 at input
!                   If special=2 and the unknown is present in the vertices,
!                   special is set to 0, else if the unknown is only present
!                   in the midside points, special remains unchanged
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inodp, ikelmc, ivrtri(4), &
              imdtri(6), in, ielem, jkelmc, iskip
      logical subdiv, presen, check, finished

!     check          Logical indicating if a unknown is present in all points of
!                    an element (true) or not (false)
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     ielem          Element number
!     ikelmc         position in kmesh part c
!     imdtri         nodenumbers of midpoints for quadratic triangles and
!                    tetrahedrons
!     in             helppointer
!     inodp          Nodal point number
!     iskip          Indication if element groups must be skipped a-priori (>0)
!                    or not (0)
!     ivrtri         Vertices numbers for quadratic tetrahedron
!     jkelmc         position in kmesh part c
!     presen         Logical parameter indicating if the required unknowns are
!                    present in the mesh (true) or not (false)
!     subdiv         Logical parameter indicating if the mesh must be subdivided
!                    (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     701   NUMBER has wrong value.
!    1707   The unknown NUMBER is not present in one of the nodes of the
!           solution array
! **********************************************************************
!
!                       PSEUDO CODE
!
!       ITER (for all element groups)
!          Check if element must be skipped
!          If not consider shape of element
!          If element is quadratic line then
!              subdivide in two linear lines
!          else if element is linear line
!              do not subdivide
!          else if element is linear triangle then
!              no special treatment
!          else if element is bilinear quadrilateral then
!              subdivide into 2 triangles
!          else if element is quadratic triangle then
!             If unknown is present in all nodes then
!                subdivide into 4 triangles
!             else if unknown is present in all vertices then
!                subdivide into 1 triangle
!             else
!                error
!             end if
!          else if element is quadratic quadrilateral then
!             If unknown is present in all boundary nodes then
!                if unknown is not present in central point interpolate
!                subdivide into 8 triangles
!             else if unknown is present in all vertices then
!                subdivide into 2 triangles
!             else
!                error
!             end if
!          else if element is linear tetrahedron then
!             no special treatment
!          else if element is one of the quadratic tetrahedrons then
!             if unknown is present in all 10 boundary points then
!                subdivide into 8 tetrahedrons
!             else if unknown is only present in the four vertices then
!                subdivide into 1 tetrahedron
!             else
!                error
!             end if
!          else if element is trilinear hexahedron then
!             subdivide into 6 tetrahedrons
!          else if element is triquadratic hexahedron then
!             if unknown is present in all vertices and "mid" nodes then
!                subdivide in 48 tetrahedrons
!             else if unknown is present in all vertices then
!                subdivide into 6 tetrahedrons
!             else
!                error
!             end if
!          else
!             element not recognized, error !
!          end if
!       end iteration
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ivrtri / 1, 3, 5, 10 /
      data imdtri / 2, 4, 6, 7, 8, 9 /
! ======================================================================
!
!     --- Fill some constants

      call eropen ( 'topqu2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topqu2'
         write(irefwr,1) 'number, nphysi, ishape', number, nphysi, ishape
         write(irefwr,1) 'indprfi, indprpi', indprfi, indprpi
         write(irefwr,1) 'nelgrp', nelgrp
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Initialize new constants

      subdiv = .false.
      presen = .false.
      centroid = 0
      lensub = 0
      nelemt = 0
      neleml = 0

!     --- Check number and idegfd

      if ( number<=0 .or. number>nphysi ) then

!     --- number>nphysi   Error

         call errint ( number, 1 )
         call errint ( nphysi, 2 )
         call errsub ( 701, 2, 0, 0 )
         go to 1000

      end if

!     --- Loop over element groups

      ikelmc = 0

      check = .true.

!     --- Check elements

      if ( ishape==3 ) then

!     --- Linear triangular element
!         Check whether unknown is present in vertices

         if ( indprpi>0 ) then

!        --- indprpi>0, use array kprobpi

            do i = 1, 3
               if ( kprobpi(kmeshc(ikelmc+i),number)==0 ) &
                 check = .false.
            end do

            if ( debug ) write(irefwr,*) 'check', check

            if ( .not. check .and. nelgrp==1 ) then

!           --- In the first element we did not have the present
!               unknown, but it may be available in a subset of the elements
!               because we are considering a surface mesh originally
!               arising from perhaps a number of element groups

                jkelmc = ikelmc
                finished = .false.
                ielem = 0
                do while ( .not. finished )

                   check = .true.
                   do i = 1, 3
                      if ( kprobpi(kmeshc(jkelmc+i),number)==0 ) &
                        check = .false.
                   end do
                   jkelmc = jkelmc+3
                   ielem = ielem+1
                   finished = check .or. ielem>inelem

                end do  ! while ( .not. finished )

            end if

            if ( debug ) write(irefwr,*) 'check', check

         else if ( indprfi>0 ) then

!        --- indprfi>0, use array kprobfi

            do i = 1, 3
               inodp = kmeshc(ikelmc+i)
               if ( kprobfi(inodp+1)-kprobfi(inodp)<number) &
                 check = .false.
            end do

         end if

         if ( check ) then

!        --- Unknown is present in vertices

            itrian = 0
            nelemt = nelemt + inelem
            lensub = lensub + inelem * 3
            presen = .true.
            if ( ishape==10 ) subdiv = .true.

         else

!        --- Skip this group

            itrian = -1
            subdiv = .true.

         end if

      else if ( ishape==4 ) then

!     --- Quadratic triangular element
!         Check if unknown is present in all nodes or in the vertices only

         if ( indprpi>0 ) then

!        --- indprpi>0, use array kprobpi

            do i = 1, 3
               in = ivrtri(i)
               if ( kprobpi(kmeshc(ikelmc+in),number)==0 ) &
                 check = .false.
            end do

         else if ( indprfi>0 ) then

!        --- indprfi>0, use array kprobfi

            do i = 1, 3
               in = ivrtri(i)
               inodp = kmeshc(ikelmc+in)
               if ( kprobfi(inodp+1)-kprobfi(inodp)<number) &
                 check = .false.
            end do

         end if

         if ( check ) then

!        --- Unknown is present in vertices, check mid points

            presen = .true.
            if ( indprpi>0 ) then

!           --- indprpi>0, use array kprobpi

               do i = 1, 3
                  in = imdtri(i)
                  if ( kprobpi(kmeshc(ikelmc+in),number)==0 ) &
                    check = .false.
               end do

            else if ( indprfi>0 ) then

!           --- indprfi>0, use array kprobfi

               do i = 1, 3
                  in = imdtri(i)
                  inodp = kmeshc(ikelmc+in)
                  if ( kprobfi(inodp+1)-kprobfi(inodp)<number) &
                    check = .false.
               end do

            end if

            if ( check ) then

!           --- Unknown is present in all 6 points

               itrian = 2
               nelemt = nelemt + inelem * 4
               lensub = lensub + inelem * 12
               subdiv = .true.

            else

!           --- Unknown is present in vertices only

               itrian = 3
               nelemt = nelemt + inelem
               lensub = lensub + inelem * 3
               subdiv = .true.

            end if

         else

!        --- Unknown is not present in vertices, skip

            itrian = -1
            subdiv = .true.

         end if

      else if ( ishape==5 ) then

!     --- Bilinear quadrilateral element
!         Check if unknown is present in vertices

         if ( indprpi>0 ) then

!        --- indprpi>0, use array kprobpi

            do i = 1, 4
               if ( kprobpi(kmeshc(ikelmc+i),number)==0 ) &
                 check = .false.
            end do

         else if ( indprfi>0 ) then

!        --- indprfi>0, use array kprobfi

            do i = 1, 4
               inodp = kmeshc(ikelmc+i)
               if ( kprobfi(inodp+1)-kprobfi(inodp)<number) &
                 check = .false.
            end do

         end if

         if ( check ) then

!        --- Unknown is present in all vertices

            itrian = 1
            nelemt = nelemt + inelem * 2
            lensub = lensub + inelem * 6
            subdiv = .true.
            presen = .true.

         else

            itrian = -1
            subdiv = .true.

         end if

      else if ( ishape==6 ) then

!     --- Biquadratic quadrilateral element
!         Check if unknown is present in all nodes or in the vertices only

         if ( indprpi>0 ) then

!        --- indprpi>0, use array kprobpi

            do i = 1, 8, 2
               if ( kprobpi(kmeshc(ikelmc+i),number)==0 ) &
                 check = .false.
            end do

         else if ( indprfi>0 ) then

!        --- indprfi>0, use array kprobfi

            do i = 1, 8, 2
               inodp = kmeshc(ikelmc+i)
               if ( kprobfi(inodp+1)-kprobfi(inodp)<number) &
                 check = .false.
            end do

         end if

         if ( check ) then

!        --- Unknown is present in vertices, check mid points

            presen = .true.
            if ( indprpi>0 ) then

!           --- indprpi>0, use array kprobpi

               do i = 2, 8, 2
                  if ( kprobpi(kmeshc(ikelmc+i),number)==0 ) &
                    check = .false.
               end do

            else if ( indprfi>0 ) then

!           --- indprfi>0, use array kprobfi

               do i = 2, 8, 2
                  inodp = kmeshc(ikelmc+i)
                  if ( kprobfi(inodp+1)-kprobfi(inodp)<number) &
                    check = .false.
               end do

            end if

            if ( check ) then

!           --- Unknown is present in all 8 points of the boundary
!               Check last point (centroid)

               if ( indprpi>0 ) then

!              --- indprpi>0, use array kprobpi

                  if ( kprobpi(kmeshc(ikelmc+9),number)==0 ) &
                    check = .false.

               else if ( indprfi>0 ) then

!              --- indprfi>0, use array kprobfi

                  inodp = kmeshc(ikelmc+9)
                  if ( kprobfi(inodp+1)-kprobfi(inodp)<number) &
                    check = .false.

               end if
               if ( .not. check ) then
                  itrian = 5
                  centroid = 1
               else
                  itrian = 4
               end if

               nelemt = nelemt + inelem * 8
               lensub = lensub + inelem * 24
               subdiv = .true.

            else

!           --- Unknown is present in vertices only

               itrian = 6
               nelemt = nelemt + inelem * 2
               lensub = lensub + inelem * 6
               subdiv = .true.

            end if

         else

!        --- Unknown is not present in vertices, skip

            itrian = -1
            subdiv = .true.

         end if

      end if

      if ( debug ) write(irefwr,1) 'lensub', lensub

!     --- If only line elements than set presen .TRUE.

      if ( nelemt==0 .and. neleml>0 ) presen = .true.
      if ( special==2 .and. presen ) special = 0

      if ( .not. subdiv ) lensub = 0
      if ( special==0 .and. .not. presen ) then
         call errint ( number, 1 )
         call errsub ( 1707, 1, 0, 0 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topqu2'

      end if
      call erclos ( 'topqu2' )

      end
