      subroutine derivatives ( ivectr, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 13-01-2015 Extra parameter istart
!        version  2.3  date 24-12-2014 New definition clinsl
!        version  2.2  date 10-10-2012 Use user and iuser
!
!   copyright (c) 2010-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     computes derived quantities of an input vector, either in the nodal points
!     or in the elements
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulesmf
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ivectr, inpsep(*)

!     inpsep         i    See subroutine readscderiv
!     ivectr         i    Sequence number of output vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istartnum, leniuser, lenuser
      integer, allocatable :: icurvs(:), iskpel(:)

!     icurvs         dummy
!     iskpel         dummy
!     istartnum      Starting address of a series of numbers in INPSEP
!     leniuser       Length of array iuser
!     lenuser        Length of array user
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLSMF1    Fill part of input for subroutine SEPBUILD in SMF
!     TO0100         Performs the actual computation of the derivatives
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'derivatives' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from derivatives'
         write(irefwr,1) 'istartmn', istartmn
         call prinin ( inpsep, 15, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepfillkmesh ( kmesh )

      if ( debug ) write(irefwr,1) 'nelgrp', nelgrp

!     --- Fill array iinder

      iinder = 0
      iinder(1) = 3   ! call by derivs
      iinder(2) = inpsep(10) ! type output vector
      iinder(4) = inpsep(6)  ! icheld
      iinder(5) = inpsep(9)  ! ix
      iinder(6) = 1     ! jdegfd
      iinder(7) = inpsep(8)   ! ivec
      iinder(8) = numvecmn   ! numvec

      iinder(10:11) = inpsep(7)  ! input vector
      iinder(12) = inpsep(14)    ! skip elements
      iinder(16) = inpsep(11)    ! itypeboun
      iinder(15) = inpsep(12)    ! number of curves
      if ( debug ) call prinin ( iinder, 20, 'iinder' )

      iprob = ks(isolut2d(inpsep(7)))%iprob
      if ( debug ) write(irefwr,1) 'iprob', iprob

      allocate ( icurvs(max(1,inpsep(12))), iskpel(nelgrp), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, inpsep(12)+nelgrp, 'icurvs' )
      if ( ierror/=0 ) go to 1000
      istartnum = 15
      if ( inpsep(12)>0) then
         istartnum = inpsep(13)
         icurvs(1:inpsep(12)) = inpsep(istartnum:istartnum+inpsep(12)-1)
         istartnum = istartnum+inpsep(12)
      end if  ! ( inpsep(12)>0)

      if ( inpsep(14)>0 ) then
         istartnum = inpsep(14)
         iskpel(1:nelgrp) = inpsep(istartnum:istartnum+nelgrp-1)
         istartnum = istartnum+nelgrp
      end if  ! ( inpsep(14)>0 ) then

      if ( debug ) call prinin ( icurvs, inpsep(12), 'icurvs' )

!     --- Fill arrays iuser and user if possible

      if ( inpsep(1)>0 ) then

!     --- Sequence number of coefficient given

         isubr = 3
         iprob = inpsep(2)
         if ( isolut2d(ivectr)>0 .and. iprob<=0 ) then
            iprob = ks(isolut2d(ivectr))%iprob
         else
            iprob = max(1,iprob)
         end if  ! ( isolut2d(ivectr)>0 .and. iprob<=0 )
         call erlins ( isubr, iprob, inpsep(1) )

      end if  ! ( inpsep(1)>0 )

      if ( associated ( iuser1(iprob)%iuser ) ) then

!     --- iuser and user exist

         leniuser = size(iuser1(iprob)%iuser)
         lenuser = size(iuser1(iprob)%user)
         if ( debug ) &
            write(irefwr,1) 'leniuser, lenuser', leniuser, lenuser

!        --- Copy arrays IUSER and USER into iuser and user

         iuser => iuser1(iprob)%iuser
         user => iuser1(iprob)%user
         if ( debug ) then
            call prinin ( iuser, leniuser, 'iuser' )
            call prinrl ( user, lenuser, 'user' )
         end if  ! ( debug )

      end if  ! ( associated ( iuser1(iprob) ) )

!     --- call actual subroutine

      call sepfillsmf1 ( isolut2d, maxvec )
      SMF%ioutvc => isolut2d(ivectr)
      if ( debug ) write(irefwr,*) 'before to0100'
      call to0100 ( iskpel, icurvs )
      numvecmn = max(numvecmn,ivectr)
      if ( debug ) write(irefwr,*) 'after to0100'

      istartmn = istartmn+istartnum  ! raise istartmn (see readscderiv)

      deallocate ( icurvs, iskpel, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'icurvs' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istartmn', istartmn
         write(irefwr,*) 'End derivatives'

      end if  ! ( debug )
      call erclos ( 'derivatives' )

      end subroutine derivatives
