      subroutine ercompvector ( isol, iinput, maxsols )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 22-11-2012 Long integers
!        version  1.2  date 30-08-2006 New definition of cpost5
!        version  1.1  date 09-12-2005 Call to sepcreastruc
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the normal stress vector along a set of curves and store
!     in a solution vector
!     All components outside the set of curves are set to 0
! **********************************************************************
!
!                       KEYWORDS
!
!     stress
!     normal_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), iinput(*), maxsols

!     iinput         i    integer input array,
!                         see ercompvector1 for a description
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
!     maxsols        i    Maximum number of vectors in isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqstress, iseqnormstress, idim, itypvc, ivec

!     idim           dimension of the space
!     iseqnormstress Sequence number of array normstressin isol
!     iseqstress     Sequence number of array stress in isol
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOMPVECTOR1  Extract parameters from array iinput and check them
!     ERCOMPVECTOR2  Store inner product of normal and tensor in all nodes of a
!                    set of curves into solution vector defined per node
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEP2DVEC       Associate 2d array usoltemp2d to 1d vector
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPCREASTRUC   find structure of vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1059   number of degrees of freedom incorrect
!    1653   Number of unknowns not constant in node
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get parameters
!     Activate and create arrays
!     Compute starting addresses
!     Clear solution array
!     Compute the normals along the curves
!     Compute the inner product of stress tensor and normals and store in usol
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercompvector' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercompvector'
         write(irefwr,1) 'maxsols', maxsols
         call prinin ( iinput, 5, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      call ercompvector1 ( iinput, iseqstress, iseqnormstress, &
                           idim, maxsols )
      if ( ierror/=0 ) go to 1000
      if ( debug ) &
         write(irefwr,1) 'iseqstress, iseqnormstress, idim', &
                          iseqstress, iseqnormstress, idim

!     --- solution vector and corresponding arrays

      call sepactsolbf2 ( isol(iseqstress) )
      iprob  = ks(isol(iseqstress))%iprob
      if ( nphysi/=6 ) then

!     --- nphys # 6

         call errint ( nphys, 1 )
         call errint ( 6, 2 )
         call errchr ( names(iseqstress), 1 )
         call errsub ( 1059, 2, 0, 1)

      end if  ! ( nphys/=6 )

      if ( indprfi/=0 ) then

!     --- indprf # 0, not implemented

         call errsub ( 1653, 0, 0, 0 )

      end if  ! ( indprfi/=0 )
      if ( ierror/=0 ) go to 1000

!     --- Find structure of vector with exactly idim parameters per node

      itypvc = 110
      call sepcreastruc ( idim, itypvc, ivec )
      if ( ierror/=0 ) go to 1000

!     --- Create normal stress vector

      call sepcreasolbf1 ( isol(iseqnormstress), itypvc, ivec, &
                           names(iseqnormstress), 0 )
      if ( ierror/=0 ) go to 1000

!     --- Clear normal stress array

      call sep2dvec ( usoltemp, 2, size(usoltemp)/2 )

      normalstress => usoltemp2d
      normalstress = 0d0

!     --- Compute starting addresses

      usoltemp => ks(isol(iseqstress))%sol
      call sep2dvec ( usoltemp, 6, size(usoltemp)/6 )
      stress => usoltemp2d

!     --- Compute the inner product of stress tensor and normals and store in
!         usol

      call ercompvector2 ( ncurvs, icurvsmsh )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercompvector'

      end if
      call erclos ( 'ercompvector' )

      end subroutine ercompvector
