      subroutine erfilquc ( vector, nodes, n, kdeg, iposmax, &
                            rincre, kfun )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 15-05-1999 Extra parameter kfun
!        version  1.0  date 20-02-1998
!
!   copyright (c) 1998-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array vector with a quadratic function in the nodal points stored
!     in array NODES corresponding to a series of curves
!     Complex case
!     The vector is only filled for the degrees of freedom required
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     real
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, nodes(n), kdeg, iposmax, kfun
      double precision rincre(*)
      double complex vector(*)

!     iposmax    i    Position of maximum in array rincre
!     kdeg       i    Indication which degrees of freedom must be filled
!                     Possibilities:
!                     >0:   Only the KDEGth degree of freedom
!     kfun       i    If kfun = 9001 a fully quadratic profile is created, &
!                     otherwise kfun = 9002 only half of it, with the maximum
!                     in the end point
!     n          i    Number of nodes stored in array NODES
!     nodes      i    Contains the node numbers that must be filled
!     rincre     i    Double precision array containing the values of the
!                     constant if KFUN < 0
!     vector    i/o   Array of length NUSOL to be (partly) filled by this
!                     subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node, inusol, inunkp
      double precision vmax, xmin, x,  ymin, y, zmin, z, s, smax

!     i          Counting variable
!     inunkp     Number of degrees of freedom in point
!     inusol     Position of unknown in array USOL
!     node       Node number
!     s          Length of curve from initial point to present point
!     smax       Length of curve
!     vmax       Maximal value of the function
!     x          x co-ordinate of point
!     xmin       x co-ordinate of first point
!     y          y-coordinate of point
!     ymin       y co-ordinate of first point
!     z          z co-ordinate of point
!     zmin       z co-ordinate of first point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     183   idegfd incorrect
!    1131   Degree of freedom required is larger than number of unknowns
!           per point
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The quadratic function along a curve from smin to smax is given by:
!     u = 4 vmax/ smax^2 (s-smin) (s-smax)
!     In case of a half quadratic function it is sufficient to double smax
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( kdeg.gt.0 ) then

!     --- kdeg > 0, only degree of freedom kdeg

         if ( kdeg.gt.nphys ) then
            call eropen ( 'erfilquc' )
            call errint ( kdeg, 1 )
            call errint ( nphys, 2 )
            call errsub ( 1131, 2, 0, 0 )
            call erclos ( 'erfilquc' )
            go to 1000
         end if

      else

!     --- kdeg <= 0, not allowed

         call eropen ( 'erfilquc' )
         call errint ( kdeg, 1 )
         call errint ( nphys, 2 )
         call errsub ( 183, 2, 0, 0 )
         call erclos ( 'erfilquc' )
         go to 1000

      end if

!     --- Compute vmax

      vmax = rincre(iposmax)

!     --- Compute length of the interval

      node   = nodes(1)
      xmin = coor(1,node)
      ymin = coor(2,node)
      if ( ndim.eq.3 ) then
         zmin = coor(3,node)
      else
         z    = 0d0
         zmin = 0d0
      end if
      smax = 0d0
      do i = 2, n
         node   = nodes(i)
         x = coor(1,node)
         y = coor(2,node)
         if ( ndim.eq.3 ) z = coor(3,node)
         smax = smax + sqrt((x-xmin)**2+(y-ymin)**2+(z-zmin)**2)
         xmin = x
         ymin = y
         zmin = z
      end do
      if ( kfun.eq.9002 ) smax = 2d0*smax
      vmax = 4d0*vmax/smax**2

!     --- Distinguish between the various cases

      if ( indprpo.gt.0 ) then

!     --- indprpo>0, degrees of freedom are stored in kprobpo
!         Fill quadratic profile

         node   = nodes(1)
         inusol = kprobpo(node,kdeg)
         if ( inusol.gt.0 ) vector(inusol) = ( 0d0, 0d0 )
         xmin = coor(1,node)
         ymin = coor(2,node)
         if ( ndim.eq.3 ) zmin = coor(3,node)
         s = 0d0
         do i = 2, n-1
            node   = nodes(i)
            inusol = kprobpo(node,kdeg)
            x = coor(1,node)
            y = coor(2,node)
            if ( ndim.eq.3 ) z = coor(3,node)
            s = s + sqrt((x-xmin)**2+(y-ymin)**2+(z-zmin)**2)
            if ( inusol.gt.0 ) vector(inusol) = vmax*s*(smax-s)
            xmin = x
            ymin = y
            zmin = z
         end do
         node   = nodes(n)
         inusol = kprobpo(node,kdeg)
         if ( inusol.gt.0 ) then
            if ( kfun.eq.9001 ) then
               vector(inusol) = ( 0d0, 0d0 )
            else
               vector(inusol) = vmax
            end if
         end if

      else if ( indprfo.gt.0 ) then

!     --- indprfo>0, Non-constant number of degrees of freedom per point
!          Loop over nodes

         node   = nodes(1)
         inusol = kprobfo(node)
         inunkp = kprobfo(node+1) - inusol
         if ( kdeg.le.inunkp ) vector(inusol) = ( 0d0, 0d0 )
         xmin = coor(1,node)
         ymin = coor(2,node)
         if ( ndim.eq.3 ) zmin = coor(3,node)
         s = 0d0
         do i = 2, n-1
            node   = nodes(i)
            inusol = kprobfo(node)
            inunkp = kprobfo(node+1) - inusol
            x = coor(1,node)
            y = coor(2,node)
            if ( ndim.eq.3 ) z = coor(3,node)
            s = s + sqrt((x-xmin)**2+(y-ymin)**2+(z-zmin)**2)
            if ( kdeg.le.inunkp ) vector(inusol) = vmax*s*(smax-s)
            xmin = x
            ymin = y
            zmin = z
         end do
         node   = nodes(n)
         inusol = kprobfo(node)
         inunkp = kprobfo(node+1) - inusol
         if ( kdeg.le.inunkp ) then
            if ( kfun.eq.9001 ) then
               vector(inusol) = ( 0d0, 0d0 )
            else
               vector(inusol) = vmax
            end if
         end if

      else

!     --- indprfo=0, Constant number of degrees of freedom per point
!         Loop over nodes

         node   = nodes(1)
         inusol = (node-1) * nphys + kdeg
         vector(inusol) = ( 0d0, 0d0 )
         xmin = coor(1,node)
         ymin = coor(2,node)
         if ( ndim.eq.3 ) zmin = coor(3,node)
         s = 0d0
         do i = 2, n-1
            node   = nodes(i)
            inusol = (node-1) * nphys + kdeg
            x = coor(1,node)
            y = coor(2,node)
            if ( ndim.eq.3 ) z = coor(3,node)
            s = s + sqrt((x-xmin)**2+(y-ymin)**2+(z-zmin)**2)
            vector(inusol) = vmax*s*(smax-s)
            xmin = x
            ymin = y
            zmin = z
         end do
         node   = nodes(n)
         inusol = (node-1) * nphys + kdeg
         if ( kfun.eq.9001 ) then
            vector(inusol) = ( 0d0, 0d0 )
         else
            vector(inusol) = vmax
         end if

      end if

1000  end
