      subroutine integsinput ( inpcop, nparm, iread, &
                               nelgrp, numvec, iseqvec, inpvol, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.2  date 09-03-2013 Correction declaration
!        version  1.1  date 22-22-2012 Extra debug
!
!   copyright (c) 2011-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read or copy input for subroutine integsbf into inpcop
!
! **********************************************************************
!
!                       KEYWORDS
!
!     integral
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nparm, nelgrp, inpcop(nparm+nelgrp), iread, &
              numvec, iseqvec, inpvol(*), ihelp

!     ihelp          o    starting address of array IHELP in inpcop
!     inpcop         o    Copy of integer input array extended with defaults
!     inpvol         i    iinput array, see READIN
!     iread          i    Defines how the input must be read
!                         Possible values:
!                         0   The input is read from the standard input file
!                         >0  The input as read from the standard input file by
!                             subroutine SEPSTN is used. The value of iread
!                             defines the sequence number
!                         -1  No input is read, all information from array
!                             INPVOL is used
!     iseqvec        i    Starting address of sequence numbers of vectors
!     nelgrp         i    Number of element groups
!     nparm          i    Length of first part of inpcop
!     numvec         i    Number of vectors stored in isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lniinvol, iseqnr, maxcal, ipos

!                    they are not
!     ipos           Position of input arrays in sepmoduleinput
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     lniinvol       Length of array IINVOL
!     maxcal         Maximum number of coefficient inputs read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     INICHINPSOL    Adapt input array with respect to variables
!     PRININ         print 1d integer array
!     READIN         Read the input for the subroutine INTEGR
!     SEPGETINPFINF  Extract infor entries of integer and real input arrays from
!                    ISEPIN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'integsinput' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from integsinput'
         call ertrace
         write(irefwr,1) 'iread, nelgrp, numvec', iread, nelgrp, numvec
         write(irefwr,1) 'nparm', nparm
         call prinin ( inpvol, max(inpvol(1),nparm), 'inpvol' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array inpcop

      lniinvol = 0
      inpcop(1:nparm+nelgrp) = 0
      inpcop(2:3) = 1
      inpcop(7) = 1
      inpcop(8) = 1

      if ( iread==-1 ) then

!     --- iread = -1, fill inpcop from inpvol

         lniinvol = min(inpvol(1),nparm)
         inpcop(1:lniinvol) = inpvol(1:lniinvol)
         if ( inpcop(5)>0 ) then
            inpcop(8:7+nelgrp) = inpvol(lniinvol+1:lniinvol+nelgrp)
            lniinvol = nparm+nelgrp
         end if ! ( inpcop(5)>0 ) then

      else if ( iread==0 ) then

!     --- iread = 0, fill inpcop by reading from the standard input file

         iseqnr = 1
         inpcop(1) = nparm+nelgrp
         call readin ( inpcop(2), 1, iseqnr, nelgrp )
         lniinvol = nparm+nelgrp

      else

!     --- iread>0, fill inpcop by copying from IBUFFR

         iseqnr = iread
         call sepgetinpfinf ( 13, iseqnr, ipos, maxcal )
         if ( debug ) &
            write(irefwr,1) 'iseqnr, ipos, nelgrp', &
                             iseqnr, ipos, nelgrp

         inpcop(1) = nparm+nelgrp
         lniinvol = size( in(ipos)%iinput )
         inpcop(2:1+lniinvol) = in(ipos)%iinput

      end if

!     --- Adapt variables

      if ( debug ) then
         call prinin ( inpcop(2), lniinvol-1, 'inpvol' )
      end if  ! ( debug )

      call inichinpsol ( inpcop(2), lniinvol-1 )
      inpcop(6) = max(numvec,inpcop(6))
      inpcop(7) = inpcop(7)-1+iseqvec
      if ( inpcop(5)<0 ) then

!     --- inpcop(5)<0, level set number in position 8

         ihelp = 8

      else

!     --- inpcop(5)>=0, set ihelp

         ihelp = inpcop(5)+1

      end if  ! ( inpcop(5)<0 )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpcop, 7, 'inpcop' )
         write(irefwr,*) 'End integsinput'

      end if  ! ( debug )
      call erclos ( 'integsinput' )

      end
