      subroutine linprbnew ( iinsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 01-09-2016 New parameter list
!        version  1.2  date 30-01-2016 New call to to0090
!        version  1.1  date 29-05-2014 Remove time routine
!
!   copyright (c) 2014-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     constructs the matrix and right-hand side for a linear problem and
!     solves it
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulecadapt
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinsol(*)

!     iinsol         i    Contains information concerning the linear solver
!                         See the Programmers Guide for a description
!                  With this array the user may indicate which actions
!                  he wants to be performed by this subroutine
!                  IINSOL must be filled as follows:
!                  Pos. 1: Last entry number that has been filled by the
!                          user. If 0 or 1 only defaults are used
!                       2: IREAD Defines how the input must be read
!                                Possible values:
!                                0:  All SEPRAN input has been read by
!                                    subroutine SEPSTN until END_OF_SEPRAN_INPUT
!                                    or end of file has been found
!                                    This input is used
!                                1:  The input is read as described for LINSOL
!                                2:  The last input read as described for LINSOL
!                                    is used (equivalent with LINSTM ICTIME=1)
!                          Default value: 0
!                       3: ISEQCF, This parameter is only used if IREAD=0
!                          It indicates the sequence number of the input
!                          for the coefficients to be used
!                          Default value: 1
!                       4: ISEQSL, This parameter is only used if IREAD=0
!                          It indicates the sequence number of the input
!                          for the linear solver to be used
!                          Default value: 1
!                       5: IPROB, Problem sequence number
!                          Default value: 1
!                       6: IVECTR, sequence number of solution vector with
!                          respect to array isolut
!                          Default value: iprob
!                       7: maxvc1, Maximum number of vectors stored in isolut
!                          Default value: 1
!                       8: IDEF_COR   indicates if defect correction should be
!                                     used
!                          0: no defect correction
!                          1: defect correction
!                          Default value: 0
!                       9: LINEAR_SUB indicates if quadratic elements should be
!                          treated as a cluster of linear subelements
!                          0: no subelements
!                          1: subelements
!                          Default value: 0
!                      10: IVEC_REACT indicates if reaction forces must be
!                          computed (>0) or not (0)
!                          If >0, IVEC_REACT defines the sequence number of the
!                          vector in which the reaction forces must be stored
!                          Default value: 0
!                      11: IVEC_FEM is only used in case of a spectral mesh
!                          It indicates if also the fem solution vector
!                          must be stored besides the sem solution vector
!                          if>0, IVEC_FEM defines the sequence number
!                          of the fem solution vector
!                          if<0, it has a special meaning
!                          In this case isolutut(1,-IVEC_FEM) contains a
!                          right-hand side vector that must be added to an
!                          existing rhs vector
!                          Default value: 0
!                      12: Indicates if a FEM preconditioning must be applied
!                          in case of a spectral mesh
!                          Possible values:
!                          0: no fem preconditioning
!                          1: fem preconditioning
!                          Default value: 0
!                      13: KEEP indicates if the matrix and right-hand side
!                          must be kept, destroyed or reused
!                          Possible values:
!                          0: new and destroy
!                          1: new and keep
!                          2: old and destroy
!                          3: old and keep
!                      14: IVEC_REACT_SP indicates if reaction forces must be
!                          computed (>0) or not (0)
!                          The difference with 10 is that maver is called with 9
!                          instead of 10. This implies that the vector must have
!                          been filled before and moreover that matrix times
!                          solution without the effect of essential boundary
!                          conditions are stored. The rest of the vector is
!                          unchanged
!                          If >0, IVEC_REACT_SP defines the sequence number of
!                          the vector in which the reaction forces must be
!                          stored
!                      15: Is only used if iinsol(10)>0, hence if reaction
!                          forces are computed
!                          If 0, no action is carried out
!                          Possibility 1 is meant for the special case of
!                          slip boundary conditions where a part of the
!                          slip is stored in a boundary matrix
!                          This part multiplied by the solution must be
!                          subtracted from the reaction force to get the
!                          real reaction force
!                      16: KEEP_MAT indicates if the matrix
!                          must be kept, destroyed or reused
!                          Possible values:
!                          0: new and destroy
!                          1: new and keep
!                          2: old and destroy
!                          3: old and keep
!                      17: KEEP_RHS indicates if the right-hand side
!                          must be kept, destroyed or reused
!                          Possible values:
!                          0: new and destroy
!                          1: new and keep
!                          2: old and destroy
!                          3: old and keep
!                      18: iseq_test_force sequence number of test force
!                          vector. If 0 no test force is computed
!                      19: iseq_rhs sequence number of right-hand side
!                          vector. If 0 the standard vector is used
!                      20: iseq_intmat1d, sequence number of intmat1d entry
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ictime, inpsol(19), icode

!                    they are not
!     icode          input parameter for ercofs
!                    Might be changed by ercofs
!     ictime         Indication if information must be read from standard
!                    input file ( 0 ) or that information read before is
!                    used ( 1 )
!     inpsol         Copy of array iinsol extended with defaults
!                    inpsol must be filled as follows:
!                    See subroutine TO0089 for a description
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOFS         Read input concerning the coefficients
!     EROPEN         Produces concatenated name of local subroutine
!     LINPRBINP      Fill array inpsol with information stored in iinsol and
!                    extended with defaults
!     PRININ         print 1d integer array
!     TO0089         Build system of equations and solves linear system of
!                    equations
!     TO0090         Adaptive refinement of the meshes, building of the new
!                    contact structure, new system of equations, solution of the
!                    system
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'linprbnew' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from linprbnew'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icode = 0

      if ( iinsol(2)==1 ) call ercofs ( icode )  ! Read coefficients

!     --- Fill array inpsol with information stored in iinsol and
!         extended with defaults

      call linprbinp ( iinsol, inpsol, ictime )
      if ( debug ) call prinin ( inpsol, 19, 'inpsol' )

!     --- Build system of equations and solve linear system

      call to0089 ( ictime, inpsol )
      if ( ierror/=0 ) go to 1000

!     --- the common /cadapt/ has been filled according to the data file

      if ( iadapt==1 ) then

!     --- the keyword ADAPTIVE was found in the data section SOLVE ... END,
!         call the adaptive solver to do the job

         inplrf(8) = 1
         call to0090 ( inpsol )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End linprbnew'

      end if  ! ( debug )
      call erclos ( 'linprbnew' )

      end subroutine linprbnew
