      subroutine pravs0 ( iref, coor, ndim, npoint, nelem, npelm, &
                          kmeshc, usol, nunkp, iseq, time )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.8  date 03-02-2014 Use name of vector
!        version  1.7  date 11-07-2005 Use subroutine pravs3
!        version  1.6  date 27-08-1997 New format (i6-->i8)
!
!   copyright (c) 1993-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write solution array or array of special structure to file sepavs_xxx.yyy
!     This possibility is meant for a SEPRAN-AVS interface
!     The array should already have been mapped on a triangular mesh or a
!     mesh consisting of tetrahedrons only
!     It is assumed that the solution vector contains only scalars in each
!     point or vectors
!     There are no points to be skipped
! **********************************************************************
!
!                       KEYWORDS
!
!     avs
!     output
!     vector
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iref, ndim, npoint, nelem, nunkp, npelm, iseq, &
              kmeshc(npelm,nelem)
      double precision coor(ndim,npoint), usol(nunkp,npoint), time

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     iref    i    File reference number
!     iseq    i    Sequence number of vector
!     kmeshc  i    kmesh part c, contains the nodal points corresponding to the
!                  elements
!     ndim    i    Dimension of the space.
!     nelem   i    Number of elements
!     npelm   i    Number of points per element
!     npoint  i    Number of nodal points
!     nunkp   i    Maximum number of unknowns in a point
!     time    i    Actual time
!     usol    i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i       Counting variable
!     j       Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PRAVS3         Write mesh information to file iref in AVS format
! **********************************************************************
!
!                       I/O
!
!    This subroutine writes the co-ordinates, the topology and the
!    solution to the file indicated by iref.
!    This file is in standard formatted AVS format
!    Shape of the file:                 meaning
!    #   SEPRAN - AVS OUTPUT FILE       comments
!    #
!    #   time level time
!    #
!    npoint, nelem, ndim, 0, 0          1. dimensions
!    1     x1 y1 z1                     2. node-id and co-ordinates
!    2     x2 y2 z2
!          .
!          .
!          .
!    1  1  cel-type i1 i2 i3 ...        3. node connectivity list
!    2  1  cel-type i1 i2 i3 ...
!          .
!          .
!          .
!    1  nunkp                           4. number of components of vec
!    vector iseq,                       5. component label, units label (empty)
!    1  u1,1 u1,2 u1,3                  6. data values
!    2  u2,1 u2,2 u2,3
!          .
!          .
!          .
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Write comments to file
!   Write dimension parameters to file
!   Write co-ordinates to file
!   Write topology to file
!   Write information of solution to file
!   Write solution to file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Write comments to file

      write(iref,100) time
100   format ( '#   SEPRAN - AVS OUTPUT FILE'/ &
               '#'/ &
               '#    time level ', f12.3/ &
               '#' )

!     --- Write dimension parameters to file

      write(iref,110) npoint, nelem, nunkp
110   format ( 3i8, ' 0 0' )

!     --- Write mesh information to file

      call pravs3 ( iref, coor, ndim, npoint, nelem, npelm, &
                    kmeshc )

!     --- Write information of solution to file

      write(iref,400) nunkp
400   format ( '1', i3 )
      length = len_trim(names(iseq))
      write(iref,410) names(iseq)(1:length)
410   format ( a, ',' )

!     --- Write solution to file

      if ( nunkp==1 ) then

!     --- nunkp = 1, scalar

         write(iref,420) (i, usol(1,i), i=1,npoint )
420      format ( i8, e12.4 )

      else if ( nunkp==2 ) then

!     --- nunkp = 2, 2D-vector

         write(iref,430) ( i, (usol(j,i), j=1,2), i=1,npoint )
430      format ( i8, 2e12.4, ' 0' )

      else if ( nunkp==3 ) then

!     --- nunkp = 3, 3D-vector

         write(iref,440) ( i, (usol(j,i), j=1,3), i=1,npoint )
440      format ( i8, 3e12.4 )

      else if ( nunkp==4 ) then

!     --- nunkp = 4, 4D-vector

         write(iref,450) ( i, (usol(j,i), j=1,4), i=1,npoint )
450      format ( i8, 4e12.4 )

      else if ( nunkp==5 ) then

!     --- nunkp = 5, 5D-vector

         write(iref,460) ( i, (usol(j,i), j=1,5), i=1,npoint )
460      format ( i8, 5e12.4 )

      else if ( nunkp==6 ) then

!     --- nunkp = 6, 6D-vector

         write(iref,470) ( i, (usol(j,i), j=1,6), i=1,npoint )
470      format ( i8, 6e12.4 )

      else if ( nunkp==7 ) then

!     --- nunkp = 7, 7D-vector

         write(iref,480) ( i, (usol(j,i), j=1,7), i=1,npoint )
480      format ( i8, 7e12.4 )

      else if ( nunkp==8 ) then

!     --- nunkp = 8, 8D-vector

         write(iref,490) ( i, (usol(j,i), j=1,8), i=1,npoint )
490      format ( i8, 8e12.4 )

      else if ( nunkp==9 ) then

!     --- nunkp = 9, 9D-vector

         write(iref,500) ( i, (usol(j,i), j=1,9), i=1,npoint )
500      format ( i8, 9e12.4 )

      else if ( nunkp==10 ) then

!     --- nunkp = 10, 10D-vector

         write(iref,510) ( i, (usol(j,i), j=1,10), i=1,npoint )
510      format ( i8, 10e12.4 )

      end if

      end
