      subroutine prfillfuncx ( nodes, nnodes, funcx, coor, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array funcx with arc length along set of nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
!     sepran_vector
!     user_vector
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nnodes,  nodes(nnodes), ndim
      double precision, intent(in) :: coor(ndim,*)
      double precision, intent(out) :: funcx(nnodes)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     funcx          o    Contains the x-coordinates
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, dir
      double precision :: s, dx(ndim), xmin, xmax, ymin, ymax, zmin, &
                          zmax, ds
      logical :: xconst, yconst, zconst

!     dir            Defines the direction of the curve
!                    (0 general, 1: x, 2: y, 3: z)
!     ds             length of vector in s-direction
!     dx             length of vector in x-direction
!     i              Counting variable
!     s              length parameter along curve
!     xconst         if true the x-coordinate is constant
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     yconst         if true the y-coordinate is constant
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     zconst         if true the z-coordinate is constant
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillfuncx' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillfuncx'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if one (or two) of the coordinate directions is constant

      dir = 0
      if ( ndim==2 ) then

!     --- ndim  = 2, two-dimensional case

         xmin = minval(coor(1,nodes(1:nnodes)))
         xmax = maxval(coor(1,nodes(1:nnodes)))
         ymin = minval(coor(2,nodes(1:nnodes)))
         ymax = maxval(coor(2,nodes(1:nnodes)))

         if ( debug ) &
           write(irefwr,2) 'xmin, xmax, ymin, ymax', &
                            xmin, xmax, ymin, ymax

         if ( abs(xmax-xmin)<= sqreps*(abs(xmin)+abs(xmax)) ) then

!        --- x = constant

            dir = 2
            s = xmin

         else if ( abs(ymax-ymin)<= sqreps*(abs(ymin)+abs(ymax)) ) then

!        --- y = constant

            dir = 1
            s = ymin

         end if  ! ( abs(xmax-xmin)<= sqreps*(abs(xmin)+abs(xmax)) )

      else if ( ndim==3 ) then

!     --- ndim  = 3, three-dimensional case

         xmin = minval(coor(1,nodes(1:nnodes)))
         xmax = maxval(coor(1,nodes(1:nnodes)))
         ymin = minval(coor(2,nodes(1:nnodes)))
         ymax = maxval(coor(2,nodes(1:nnodes)))
         zmin = minval(coor(3,nodes(1:nnodes)))
         zmax = maxval(coor(3,nodes(1:nnodes)))

         if ( debug ) &
           write(irefwr,2) 'xmin, xmax, ymin, ymax, zmin, zmax', &
                            xmin, xmax, ymin, ymax, zmin, zmax

         xconst = abs(xmax-xmin)<= sqreps*(abs(xmin)+abs(xmax))
         yconst = abs(ymax-ymin)<= sqreps*(abs(ymin)+abs(ymax))
         zconst = abs(zmax-zmin)<= sqreps*(abs(zmin)+abs(zmax))

         if ( debug ) &
            write(irefwr,*) 'xconst, yconst, zconst', &
                             xconst, yconst, zconst

         if ( xconst .and. yconst ) then

!        --- x and y = constant

            dir = 3
            s = zmin

         else if ( xconst .and. zconst ) then

!        --- x and z = constant

            dir = 2
            s = ymin

         else if ( yconst .and. zconst ) then

!        --- y and z = constant

            dir = 1
            s = xmin

         end if  ! ( xconst .and. yconst )

      end if  ! ( ndim==2 )

      if ( debug ) write(irefwr,1) 'dir', dir

      if ( dir==0 ) then

!     --- dir = 0, no direction constant, use curve length

         s = 0d0
         funcx(1) = s
         do i = 2, nnodes
            dx = coor(:,nodes(i))-coor(:,nodes(i-1))
            s = s + sqrt(dot_product(dx,dx))
            funcx(i) = s
         end do  ! i = 1, nnodes-1

      else

!     --- dir >0, x/y/z-direction

         funcx(1) = s
         do i = 2, nnodes
            ds = coor(dir,nodes(i))-coor(dir,nodes(i-1))
            s = s + ds
            funcx(i) = s
         end do  ! i = 1, nnodes-1

      end if  ! ( dir==0 )

1000  call erclos ( 'prfillfuncx' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillfuncx'

      end if  ! ( debug )

      end
