      subroutine printersect2d ( inpsep, rinsep, &
                                 icross, numsubcurvs, numnodes, numveclc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Intersect a 2d mesh by a straight line and store the values
!     in array icross of type 119
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     vector
!     2d
!     mesh
!     line
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      integer, intent(out) :: icross(*), numsubcurvs, numnodes, numveclc
      double precision, intent(in) :: rinsep(*)

!     icross         o    Integer SEPRAN array of type 119 containing
!                         the information of the intersections
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotint
!     numnodes       o    Number of nodal points in intersection
!     numsubcurvs    o    Number of parts the intersection consists of
!                         At this moment only 1 is allowed
!     numveclc         o    Number of vectors stored in isolut
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscplotint
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: idegfd, ichois, iskip, ivalue, i, ihelp(1), number
      double precision :: values(3)

!     i              Counting variable
!     ichois         Choice parameter for subroutine intp2dbf
!     idegfd         Degree of freedom sequence number
!     ihelp          dummy
!     iskip          Indicates if element groups must be skipped (>0)
!                    or not (0)
!     ivalue         dummy
!     number         The number-th unknown in each nodal point of the array
!                    is considered. If the array is complex number=2*i-1
!                    corresponds to the real part of array i and
!                    number=2*i to the imaginary part of array i
!     values         Contains the values defining the intersection line
!                    See subroutine intp2dbf
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: prevalexpreal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PREVALEXPREAL  Evaluate real, scalar or real expression
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPINTP2D      Interpolate 2D region with straight line
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1438   Intersection consists of sub parts
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printersect2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printersect2d'
         call prinin ( inpsep, 7, 'inpsep' )
         call prinin ( isolut, 5, 'isolut' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      idegfd = inpsep(2)
      do i = 1, 3
         values(i) =  prevalexpreal ( inpsep(2+i), rinsep, 'origin' )
      end do  ! i = 1, 3

      if ( debug ) call prinrl ( values, 3, 'values' )

!     --- Compute nunkpi and nphysi

      call sepactsolbf2 ( isolut )

      call inichkarlen ( nphysi, 20, 'icross' )
      if ( ierror/=0 ) go to 1000

      ichois = 1
      iskip = 0

      if ( idegfd>0 .or. nphysi==1 ) then

!     --- Only 1 degree of freedom

         number = max(1,idegfd)
         icross(1) = 0
         call sepintp2d ( ichois, ivalue, values, number,icross, &
                          iskip, ihelp )
         numveclc = 1

      else

!     --- More degrees of freedom

         icross(1:nphysi) = 0
         do number = 1, nphysi

            call sepintp2d ( ichois, ivalue, values, number, &
                             icross(number), iskip, ihelp )

         end do  ! number = 1, nphysi
         numveclc = nphysi

      end if  ! ( idegfd>0 .or. nphysi==1 )

      numsubcurvs = ks(icross(1))%n   ! number of subcurves
      isoltemp => ks(icross(1))%isol
      numnodes = sum(isoltemp(1:numsubcurvs)) ! number of nodes

      if ( numsubcurvs>1 ) then

!     --- More than one subparts

         call errint ( numsubcurvs, 1 )
         call erreal ( values(1), 1 )
         call erreal ( values(2), 2 )
         call erreal ( values(3), 3 )
         call errsub ( 1438, 1, 3, 0 )
         go to 1000

      end if  ! ( numsubcurvs>1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printersect2d'

      end if  ! ( debug )
      call erclos ( 'printersect2d' )

      end subroutine printersect2d
