      subroutine printsectmesh ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 06-03-2015 Extra Debug
!        version  1.0  date 01-10-2012
!
!   copyright (c) 2012-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Intersect 3d mesh by plane and store the result as 2d mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     plane
!     mesh
!     3d
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscintscmsh for a description
!     rinsep         i    Array containing the real input for this subroutine
!                         See inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer leninpsep
      parameter ( leninpsep=2 )
      integer meshout, ichoicearr(3), i, &
              inpcop(leninpsep), iplane, iposrinsep
      double precision plane(4), values(4)

!                    they are not
!     i              Counting variable
!     ichoicearr     ?
!     inpcop         Copy of integer input array extended with defaults
!     iplane         Defines direction of plane
!                    1:  plane x = constant
!                    2:  plane y = constant
!                    3:  plane z = constant
!                    4:  general plane
!     iposrinsep     Position in rinsep
!     leninpsep      Length of array inpsep
!     meshout        Sequence number of obstacle mesh to be used
!     plane          Array of length 4 containing the parameters a, b, c
!                    and d describing the plane: ax=by+cz+d = 0
!                    plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d
!     values         Double precision array with the same contents as plane
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHINPSOL    Adapt input array with respect to variables
!     PRININ         print 1d integer array
!     SEPCLEAR       clear array map
!     SEPINTERSECPLA Intersect 3d mesh by plane
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2478   Mesh sequence number out of range
!    2836   No intersection of surface with plane found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printsectmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printsectmesh'
         call prinin ( meshseqs, 5, 'meshseqs' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Replace scalars by actual values

      inpcop(1:leninpsep) = inpsep(1:leninpsep)
      call inichinpsol ( inpsep, leninpsep )

      if ( debug ) call prinin ( inpsep, leninpsep, 'inpsep' )

      if ( inpsep(1)<=0 ) then

!     --- inpsep(1)<=0, hence default mesh number

         meshout = meshnr+1

      else

!     --- inpsep(1)>0, hence mesh number from inpsep(1)

         meshout  = meshseqs(inpsep(1))

      end if  ! ( inpsep(1)<0 )

      if ( meshout>maxmesh ) then

!     --- mesh number too large

         call errint ( meshout, 1 )
         call errint ( maxmesh, 2 )
         call errsub ( 2478, 2, 0, 0 )
         go to 1000

      end if  ! ( meshout>maxmesh )

      iposrinsep = inpsep(2)
      if ( debug ) &
         write(irefwr,1) 'meshout, iposrinsep', meshout, iposrinsep

!     --- delete all arrays corresponding to map(.,meshout)

      do i = 1, maxmesh
         if ( i/=meshnr ) then
            if ( associated(map(i,meshout)%imap)) then
               deallocate ( map(i,meshout)%imap, map(i,meshout)%rmap )
               call sepclear ( map(i,meshout) )
            end if  ! ( associated(map(i,meshout)%imap))
         end if  ! ( i/=meshnr )
         if ( associated(map(meshout,i)%imap)) then
            deallocate ( map(meshout,i)%imap, map(meshout,i)%rmap )
            call sepclear ( map(meshout,i) )
         end if  ! ( associated(map(meshout,i)%imap))
      end do  ! i = 1, maxmesh

!     --- Compute the intersections of the obstacle with the basis mesh

      iplane = 4
      ichoicearr(1) = 4
      ichoicearr(2:3) = 0

      plane(1:4) = rinsep(iposrinsep:iposrinsep+3)
      values = plane

      call sepintersecpla ( meshnr, meshout, plane, &
                            map(meshnr,meshout), ichoicearr, &
                            values, iplane )

      if ( ierror==2836 ) then
         call errint ( iplane, 1 )
         call erreal ( plane(4), 1 )
         call errsub ( 2836, 1, 1, 0 )
         ierror = 0
      end if  ! ( ierror==2836 )

      if ( ierror/=0 ) go to 1000

!     --- reset inpsep

      inpcop(1:leninpsep) = inpcop(1:leninpsep)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'meshout, meshnr', meshout, meshnr
         write(irefwr,*) 'End printsectmesh'

      end if  ! ( debug )
      call erclos ( 'printsectmesh' )

      istartmn = istartmn+3

      end subroutine printsectmesh
