      subroutine prlevcopsol ( meshorig, meshobst, map1d, iseqlevel, &
                               inpsep, numinterpol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-04-2013 Loop in case of series
!        version  1.0  date 22-11-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solutions from old mesh onto new mesh if possible
!     Special case for levelset
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     level_set
!     copy
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshorig, meshobst, &
              inpsep(*), numinterpol, iseqlevel
      type (mapping) :: map1d

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readsclevmesh
!     iseqlevel      i    Sequence number of level set input block
!     map1d         i/o   contains information of the mapping
!     meshobst       i    Sequence number of obstacle mesh to be used
!     meshorig       i    Sequence number of original mesh to be used
!     numinterpol    o    number of vectors to be interpolated
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iseq, j, numarrays

!     i              Counting variable
!     iseq           Sequence number
!     j              Counting variable
!     numarrays      Number of indexed vectors in vector array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
!     PRCOPSOL       Interpolate a series of solution vectors from one mesh
!                    to the other
!     PROBSTCOPSOL   Copy solutions from old mesh onto new mesh if possible
!     PRSETLEVSETZER Set levelset function equal to zero at interface
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2951   Sequence number of vectors out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prlevcopsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prlevcopsol'
         write(irefwr,1) 'iseqlevel', iseqlevel
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numinterpol = inpsep(5)

      if ( debug ) write(irefwr,1) 'numinterpol', numinterpol

      if ( numinterpol==0 .and. meshorig/=meshobst ) then

!     --- numinterpol = 0, interpolate all vectors

         call probstcopsol ( meshorig, meshobst, map1d )

      else if ( meshorig/=meshobst ) then

!     --- numinterpol#0, interpolate some vectors
!         First interpolate phi

         iseq = iseqlevel
         call prcopsol ( meshorig, meshobst, map1d, iseq, .false. )

!        --- make phi equal to zero for points at interface

         kprob => kprob2d(meshobst)
         call prsetlevsetzer ( kmesh2d(meshobst), &
                               isolut3d(iseq,meshobst), 1 )

!        --- interpolate rest

         do i = 1, max(0,numinterpol)

            iseq = inpsep(9+i)
            if ( debug ) write(irefwr,1) 'i, iseq', i, iseq
            call inireducevec ( iseq, iseq, numarrays )

            if ( iseq<=0 .or. iseq>numvecmn ) then

!           --- error: iseq out of range

               call errint ( iseq, 1 )
               call errint ( numvecmn, 2 )
               call errsub ( 2951, 2, 0, 0 )
               go to 1000

            end if  ! ( iseq<=0 .or. iseq>numvecmn )

            if ( debug ) write(irefwr,1) 'iseq', iseq

!           --- Loop over all subarrays

            do j = 0, numarrays-1

               call prcopsol ( meshorig, meshobst, map1d, iseq+j, .false. )

            end do  ! j = 0, numarrays-1

         end do  ! i = 1, numinterpol

      end if  ! ( numinterpol==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prlevcopsol'

      end if  ! ( debug )
      call erclos ( 'prlevcopsol' )

      end subroutine prlevcopsol
