      subroutine prplotcol ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Driving subroutine for colored level plot
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     color
!     driving_subroutine
!     plot
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulesymm
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulemnsub
      use sepmoduleplot
      use sepmoduletexts
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      double precision, intent(in) :: rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotcn
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: lencontln=46
      integer :: plotboun, ikader, ktimes, ncntlnus, ncntln, &
                 idegfd, lentxt, jindcl, indcolus(1), &
                 ishift, ivecseq, numarrays, itypsc, ityplg, jmaxsave, &
                 indisol, icompt, itext, itypvc
      integer, allocatable :: skipelgrps(:,:), indcol(:)
      double precision, allocatable :: contln(:)
      double precision :: plotfm, yfact, contlnus(lencontln), umin, umax
      character (len=120) textloc
      logical :: bouncurv
      integer, pointer :: isol

!     bouncurv       if true the 2D boundaries are plotted using icurvs/curves
!                    otherwise using kmeshm and coor
!     contln         Contains the contour levels
!     contlnus       User array of length ncntlnus + 5 (if used) in which the
!                    user is supposed to have stored the contour levels
!                    from position 6
!                    they are not
!     icompt         indicates how the contour levels must be
!                    computed from NCNTLN in the case NCNTLN<=0
!                    0:  The minimum and maximum level are computed
!                    1:  The minimum level is given in contln(6), the
!                        maximum level must be computed
!                    2:  The maximum level is given in contln(7), the
!                        minimum level must be computed
!                    3:  The minimum level is given in contln(6), the
!                        maximum level in contln(7)
!     idegfd         Degree of freedom sequence number
!     ikader         Help variable to store jkader temporarily
!     indcol         Contains the colors of the levels
!     indcolus       This array is only used when isubr=2 and jsmoot>0
!                    If JSMOOT=1 it is a variable length array of length:
!                    number of contour levels plus 5
!                    indcol must be filled by the user with the colors
!                    of the various contour levels in increasing sequence.
!                    indcol must be filled from position 6
!                    If JSMOOT=2 it is a variable length array of length 6
!                    In INDCOL(6) the user must store the minimum color
!                    required
!                    If JSMOOT=3 it is a variable length array of length 7
!                    The minimum color must be stored in INDCOL(6) the
!                    maximum one in INDCOL(7)
!                    If the range of the colors is less than the available
!                    range of colors the color numbering is repeated
!     indisol        memory management reference to solution array
!     ishift         Defines which of the two kmesh arrays should be used
!     isol           Standard SEPRAN array containing information of the
!                    solution array
!     itext          Text sequence number
!     ityplg         Choice parameter for type of legenda
!                    Possible values:
!                    1:  The legenda are not plotted
!                    2:  The colors are plotted vertical including all values
!                    3:  The colors are plotted horizontal only minimum
!                        and maximum values
!     itypsc         Indicates if scales must be plotted (1) or not (2)
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivecseq        Sequence number of vector to be considered with
!                    respect to the solution array (isolut2d or isolut2dut)
!     jindcl         indicates how array indcol must be filled for routine
!                    plotc2.
!                    Possible values:
!                    0: The colors with numbers 1 to 1+ncntln are used
!                    1: The user defines the color numbers explicitly
!                    2: The user defines the minimum color
!                    3: The user defines the minimum and maximum color
!                    See indcol
!     jmaxsave       Help variable to save the value of jmax
!     ktimes         mod(itimes,10)
!     lencontln      Declared length of array contln
!     lentxt         length of text parameter, with trailing blanks deleted
!     ncntln         number of contour levels
!     ncntlnus       Parameter to compute the number of contour levels
!     numarrays      Number of indexed vectors in vector array
!     plotboun       Choice parameter consisting of 3 parts: iinner, number, &
!                    isymbl according to:
!                    plotboun = iinner + 10 * number + 100 * isymbl
!                    with:
!                    iinner   Information of how the curves must be plotted
!                      0,1    only the outer boundaries are plotted
!                      2      only the inner boundaries are plotted
!                      3      all boundaries are plotted
!                      4      no boundaries are plotted
!                    number   Indication if the curves must be provided with
!                             a curve number (1) or not
!                    isymbl   Indicates if the nodes along the curves must
!                              be marked with a cross (1) or not (=0)
!     plotfm         Plotting factor. When PLOTFM<0, then the length of
!                    the plot is computed by - (xmax-xmin) * PLOTFM
!                    and the width by - (ymax-ymin) * YFACT * PLOTFM
!                    else the length of the plot is equal to PLOTFM cm
!                    When by some mistake the user has given | PLOTFM | a
!                    value smaller than 1 cm, then the length is chosen
!                    equal to the default length stored in common block
!                    consta
!     skipelgrps     array of size 2 x nelgrp containing information
!                    for the subdivision of the elements in triangles
!                    (1,i) mult, i.e. number of triangles each element
!                          of element group i must be subdivided
!                    (2,i) typesubdiv, defines special subdivisions
!     textloc        String to store a local text
!     umax           largest value of function to be plotted
!     umin           smallest value of function to be plotted
!     yfact          Multiplication for the y-co-ordinates. With this
!                    parameter the user can influence the scale in
!                    y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLFILLLEVELS   Fill array with contour levels
!     PLLGCL         Plot legenda in colored level case
!     PLMORE         User subroutine to plot extra information in the plot
!     PLOTCHECKREG   Compute number of sub-arrays and ivecseq
!     PLOTCOLCONT    Actual filled colored level plot
!     PLOTFILLCOLORS Fill array with sequence numbers of colors
!     PLTOPBOTTEXT   Plot text at top or bottom of plot if required
!     PRCOMPLEVELS   Compute levels for contour plots and store in array contln
!     PRCOMPNCNTLN   Compute number of contour lines
!     PRINRL         Print 1d real vector
!     PROPENPLOT     Open plot and plot boundary (2d)
!     PRPLOTCN       Plot contours
!     SEPCOPSOLSYMM  Copy solution from original mesh to extended
!                    (axi-symmetric) mesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     TOPSUBDIV2D    Subdivide a 2d region in linear triangles if necessary
!     TOPSUBINT2D    Interpolate solution from original mesh to triangular mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prplotcol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prplotcol'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jmaxsave = jmax  ! save value of jmax

!     --- Compute number of sub arrays and ivecseq

      call inireducevec ( inpsep(1), ivecseq, numarrays )

      itypvc = ks(isolut2d(ivecseq))%typevec
      if ( itypvc/=110 .and. itypvc/=115 .or. &
           ndim/=2 ) then

!     --- This case has not yet been implemented

         call prplotcn ( inpsep, rinsep )
         go to 1000

      end if  ! ( itypvc/=110 .and. itypvc/=115 ... )

      if ( inpsep(13)==1 ) then

!     --- Use extended mesh and extend solution to new mesh

         call sepcopsolsymm ( ivecseq )
         isol => isolsymm
         kprob => kprobsymm

      else

!     --- Use original mesh

         isol => isolut2d(ivecseq)

      end if  ! ( inpsep(13)==1 )

!     --- Compute number of arrays and ivecseq
!         Compute region to be plotted

      call plotcheckreg ( inpsep, rinsep )
      if ( ierror/=0 ) go to 1000

!     --- Set ktimes and other parameters

      ktimes = mod(jtimes,10)
      bouncurv = .false.
      plotfm = 15d0
      yfact = 1d0
      plotboun = 0

      if ( kelmu/=0 ) then
         ishift = 1
         if ( inpsep(13)==1 ) then
            kmesh => kmesh2d(meshsymm+ishift)
         else
            kmesh => kmesh2d(meshnr+ishift)
         end if  ! ( inpsep(13)==1 ) then
      else
         ishift = 0
      end if

      call sepfillkmesh(kmesh)

      if ( debug ) write(irefwr,1) 'ishift', ishift

      idegfd = inpsep(3)  ! degree of freedom

!     --- Fill array contlnus and ncntlnus

      call prcomplevels ( contlnus, ncntlnus, icompt, lencontln, &
                          inpsep, rinsep, 2 )
      if ( ierror/=0 ) go to 1000

!     --- Compute number of contour levels

      call prcompncntln ( ncntlnus, ncntln, 2, inpsep )

      if ( debug ) write(irefwr,1) 'ncntlnus, ncntln', ncntlnus, ncntln

      allocate ( contln(ncntln), skipelgrps(2,nelgrp), indcol(ncntln+1), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 2*ncntln+2*nelgrp, 'contln' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array indcol

      jindcl = 0
      call plotfillcolors ( indcol, indcolus, ncntln, jindcl )

!     --- Create new mesh if necessary

      call topsubdiv2d ( isol, idegfd, skipelgrps )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'after topsubdiv2d'

      do iseqarray = 1, numarrays

         if ( iseqarray>1 ) then

            if ( inpsep(13)==1 ) then

!           --- Use extended mesh and extend solution to new mesh

               call sepcopsolsymm ( ivecseq )

            else

               isol => isolut2d(ivecseq)

            end if  ! ( inpsep(13)==1 )

         end if  ! ( iseqarray>1 )

!     --- Open plot and plot boundary if necessary

         call propenplot ( bouncurv, plotfm, yfact, ikader, plotboun )
         if ( ierror/=0 ) go to 1000

         if ( debug ) write(irefwr,*) 'after propenplot'

!        --- Interpolate solution if necessary

         indisol = 0  ! reference to copy of usol
         call topsubint2d ( isol, indisol, idegfd, &
                            skipelgrps, umin, umax )
         if ( ierror/=0 ) go to 1000
         if ( debug ) write(irefwr,*) 'after topsubint2d'

         if ( debug ) write(irefwr,2) 'umin, umax', umin, umax

!        --- Fill array contln with contour levels

         call plfilllevels ( icompt, umin, umax, contlnus, ncntlnus, &
                             contln, ncntln )
         if ( ierror/=0 ) go to 1000

         if ( debug ) call prinrl ( contln, ncntln, 'contln' )

!        --- actual contour plot

         call plotcolcont ( contln, ncntln, indcol )
         if ( ierror/=0 ) go to 1000

         if ( ktimes<=1 ) then

!        --- plot legenda

            itypsc = 1
            ityplg = 2
            call pllgcl ( contln, ncntln, indcol, itypsc, ityplg )

!           --- Fill text

            if ( inpsep(8)>0 ) then

!           --- inpsep(8)>0, hence text found

               textloc = texts(inpsep(8))  ! text
               lentxt = inpsep(9)  ! number of characters of text

            else

!           --- inpsep(8)=0, no text found, use default

               textloc = names(ivecseq)
               lentxt = len_trim(textloc)

            end if  ! ( inpsep(8)>0 )

!           --- Plot text at top and bottom if required

            itext = 1
            call pltopbottext ( itext, textloc(1:lentxt), rinsep(1) )

         end if  ! ( ktimes<=1 )
         if ( ierror/=0 ) go to 1000

!        --- Close plot if necessary

         if ( ktimes<1 .or. ktimes>2 ) then
            call plmore
            call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
         end if

         if ( iseqarray<numarrays ) ivecseq = ivecseq+1  ! raise ivecseq

         if ( indisol==0 ) deallocate ( usolt)

      end do  ! iseqarray = 1, numarrays

      iseqarray = 0

      kmesh => kmesh2d(meshnr)
      kprob => kprob2d(meshnr)

      if ( inpsep(2)>0 ) jmax = jmaxsave  ! reset jmax

!     --- remove temporary arrays isolut2dtmp and kmeshctmp

      if ( associated(kmeshctmp) ) deallocate(kmeshctmp)

!     --- deallocate temporary arrays

      deallocate ( contln, skipelgrps, indcol, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'contln' )

      kmesh => kmesh2d(meshnr)
      call sepfillkmesh ( kmesh )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prplotcol'

      end if  ! ( debug )
      call erclos ( 'prplotcol' )

      end subroutine prplotcol
