      subroutine prstoc1 ( inpsep, rinsep, &
                           iseqvec, iseqboun, iseqcoeff, iseqreac )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 24-07-2015 Jump in case of error
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 04-11-2009
!
!   copyright (c) 2009-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve 1d stochastic investment problem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     stochastic
!     investment
!     1d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulestoc
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), iseqvec(2), &
              iseqboun(2), iseqcoeff(2), iseqreac
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscstoc
!     iseqboun       i    Array containing sequence numbers of input for
!                         boundary conditions
!     iseqcoeff      i    Array containing sequence numbers of input for
!                         coefficients
!     iseqreac       i    Sequence number of reaction vector in array ISOL
!                         If <0 also the right-hand side contribution is added
!     iseqvec        i    Array containing sequence numbers of input for
!                         vectors
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscstoc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: maxusp=100
      integer maxiter, userpoint(maxusp+1), &
              iter, k, iprobw0, iprint, iprobw1, &
              numelgrpw0, numelgrpw1, &
              iseqw0, iseqw1, numcurw0, numcurw1, &
              numusp, isubr
      integer, allocatable :: elgrpw0(:), elgrpw1(:), &
                              curvesw0(:), curvesw1(:)
      double precision diff, eps, valw0, valw1, investment, dwdx, &
                       distance(3), coorloc(3), L

!     L
!     coorloc        Array with the coordinates of the user point
!     curvesw0       Contains the curves on which w0 is defined
!     curvesw1       Contains the curves on which w1 is defined
!                    they are not
!     diff           Help parameter to compute the difference of two
!                    parameters
!     distance       distance to be used to move interface
!     dwdx           derivative of function along axis in userpoint
!     elgrpw0        Contains the element groups for which w0 is defined
!     elgrpw1        Contains the element groups for which w1 is defined
!     eps            Local accuracy
!     investment     Defines the investment
!     iprint         If 1 information must be printed
!     iprobw0        problem number for w0
!     iprobw1        problem number for w1
!     iseqw0         sequence number of w0 in isolut2d
!     iseqw1         sequence number of w1 in isolut2d
!     isubr          Indicates the type of calling subroutine for er0034
!                    Possible values
!                    1:  CREATE
!                    2:  PRESDF/TM
!                    3:  CREATEN
!                    4:  presbc
!     iter           Iteration sequence number
!     k              Counting variable
!     maxiter        maximum number of iterations
!     maxusp         Dimension of array userpoint
!     numcurw0       Number of curves stored in array curvesw0
!     numcurw1       Number of curves stored in array curvesw1
!     numelgrpw0     Number of eleemnt groups stored in array elgrpw0
!     numelgrpw1     Number of eleemnt groups stored in array elgrpw1
!     numusp         Contains number of nodes between origin and interface
!     userpoint      Contains the user point numbers along the
!                    axis sorted in ascending order from position 2
!                    userpoint(1) contains the interface
!     valw0          Value of w0 in userpoint
!     valw1          Value of w1 in userpoint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision prvaluserp, prderuserp

!     ER0034         Performs the actual tasks for CREAT and PRESDF
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICURVUSERP   Find user point that is on the interface of two sets
!                    of curves
!     INICURVUSP     Find all user points along ix axis with coordinates
!                    smaller than that of userpoint and more than 0
!     INIELGRPCURVS  Find curve numbers corresponding to the element groups
!                    stored in ielgrps
!     INIELGRPPROB   Find all active element groups for problem iprob
!     MSHADAPCURV    Adapt coordinates of curves when coordinates of user points
!                    have been changed
!     MSHMVCOORUSP   Move coordinates of user point to new position
!     PRCOMPIDIR     Compute direction of axis (1=x, 2=y)
!     PRDERUSERP     Get the value of the derivative of the solution vector in
!                    a given user point in direction ix
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRSTOCREAC     Compute reaction force for stochastic investment
!     PRSTOCSOLVE    Solve linear system for stochastic investment problem
!     PRVALUSERP     Get the value of the solution vector in a given user point
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPPRINTV      Print solution vector or vector of special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
!    3040   No convergence reached in maxiter iterations
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill essential boundary conditions in vectors
!     iter := 0
!     diff := 1
!     While ( diff>eps and iter<maxiter ) do
!        w1 := solution linear system
!        Compute flux in user point
!        w0 := solution linear system
!        diff := value(w1)-value(w0) - investment
!        distance := -diff/dwdx
!        Move user point over distance
!        Adapt curves containing user point
!        diff := abs(diff)
!        iter := iter+1
!     recompute complete mesh and compute w0 and w1 once again
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstoc1' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstoc1'
         write(irefwr,1) 'iseqreac', iseqreac
         call prinin ( iseqvec, 2, 'iseqvec' )
         call prinin ( iseqboun, 2, 'iseqboun' )
         call prinin ( iseqcoeff, 2, 'iseqcoeff' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      maxiter = inpsep(2)    ! maximum number of iterations
      iprint = inpsep(6)     ! print level
      eps = rinsep(1)        ! accuracy
      investment = rinsep(2) ! investment
      if ( iprint>2 ) debug = .false.

      if ( debug ) then
         write(irefwr,1) 'maxiter, iprint', maxiter, iprint
         write(irefwr,2) 'eps, investment', eps, investment
      end if  ! ( debug )

!     --- allocate space

      allocate ( elgrpw0(nelgrp), elgrpw1(nelgrp), &
                 curvesw0(ncurvs), curvesw1(ncurvs), stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( 2*nelgrp+2*ncurvs, 1 )
         call errint ( error, 2 )
         call errchr ( 'elgrpw0', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/= 0 )

!     --- Fill essential boundary conditions in vectors

      do k = 1, 2
         isubr  = 6+100*(iseqvec(k)-1)
         call er0034 ( isubr, 0, isolut2d, iseqboun(k), isolut2d, 100 )
         numvecmn = max(numvecmn,isubr)
      end do  ! k = 1, 2
      if ( ierror/=0 ) go to 900

!     --- Find element groups corresponding to w0 and w1
!         Store in elgrpw0 and elgrpw1

      iseqw0 = iseqvec(2)
      iseqw1 = iseqvec(1)
      iprobw0 = ks(isolut2d(iseqw0))%iprob
      iprobw1 = ks(isolut2d(iseqw1))%iprob
      iprob = iprobw0
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      kprobc => kprobpart%kprobc
      call sepfillkprob

      call inielgrpprob ( numelgrpw0, elgrpw0 )
      iprob = iprobw1
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      call sepfillkprob
      call inielgrpprob ( numelgrpw1, elgrpw1 )

      if ( debug ) then
         call prinin ( isolut2d(iseqw1), 5, 'isol_w1' )
         call prinin ( isolut2d(iseqw0), 5, 'isol_w0' )
         write(irefwr,1) 'iseqw0, iseqw1', iseqw0, iseqw1
         write(irefwr,1) 'iprobw0, iprobw1', iprobw0, iprobw1
         call prinin ( elgrpw0, numelgrpw0, 'elgrps for w0' )
         call prinin ( elgrpw1, numelgrpw1, 'elgrps for w1' )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 900

!     --- Find curves corresponding to element groups


      call inielgrpcurvs ( iinputkm, elgrpw0, numelgrpw0, &
                           curvesw0, numcurw0 )
      call inielgrpcurvs ( iinputkm, elgrpw1, numelgrpw1, &
                           curvesw1, numcurw1 )

      if ( debug ) then
         call prinin ( curvesw0, numcurw0, 'curves for w0' )
         call prinin ( curvesw1, numcurw1, 'curves for w1' )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 900

!     --- Find userpoint on the interface

      call inicurvuserp ( curvesw0, numcurw0, curvesw1, numcurw1, &
                          userpoint(1), curveusrp )
      if ( debug ) write(irefwr,1) 'userpoint', userpoint(1)
      if ( ierror/=0 ) go to 900

!     --- Compute ix, i.e. axis (1 = x, 2 = y)

      call prcompidir ( isolut2d(iseqw1), ixstoc )
      if ( debug ) write(irefwr,1) 'ix', ixstoc
      if ( ierror/=0 ) go to 900

!     --- Set sigma and mu in common cstoc

      if ( ndim>1 ) then

!     --- ndim = 2, sigma and mu must be set

         if ( ixstoc==1 ) then

!        --- ix = 1, x-direction

            sigma = sigmax
            mu = mux

         else

!        --- ix = 2, y-direction

            sigma = sigmay
            mu = muy

         end if  ! ( ixstoc==1 )

      end if  ! ( ndim>1 )
      if ( debug ) write(irefwr,2) 'sigma, mu', sigma, mu

!     --- Find all user points on the ix axis with ix-coordinate
!         smaller than userpoint but more than 0
!         Store in userpoint from position 2

      call inicurvusp ( userpoint, maxusp, numusp, 1 )
      if ( debug ) call prinin ( userpoint, numusp+1, 'userpoint' )
      if ( ierror/=0 ) go to 900

!     --- iter := 0
!         diff := 1

      iter = 0
      diff = 1d0
      if ( iprint>=2 ) write(irefwr,100)
100   format ( '  iter     diff           L        valw0        valw1' )

!     --- While ( diff>eps and iter<maxiter ) do

      do while ( diff>eps .and. iter<maxiter )

!     --- w1 := solution linear system

         call prstocsolve ( iseqcoeff(1), iprobw1, iseqw1 )
         if ( ierror/=0 ) go to 1000
         if ( debug ) &
            call sepprintv ( isolut2d(iseqw1), 'w1' )

!        --- Compute flux in user point

         call prstocreac ( iseqw1, iseqcoeff(1), iseqreac, &
                           elgrpw0, numelgrpw0, userpoint(1) )
         if ( ierror/=0 ) go to 1000
         numvecmn = max(numvecmn,iseqreac)
         if ( debug ) &
            call sepprintv ( isolut2d(iseqreac), 'flux' )

!        --- w0 := solution linear system

         call prstocsolve ( iseqcoeff(2), iprobw0, iseqw0 )
         if ( ierror/=0 ) go to 1000
         if ( debug ) &
            call sepprintv ( isolut2d(iseqw0), 'w0' )

!        --- diff := value(w1)-value(w0) - investment

         valw1 = prvaluserp ( isolut2d(iseqw1), userpoint(1) )
         valw0 = prvaluserp ( isolut2d(iseqw0), userpoint(1) )
         diff = valw1-valw0-investment
         if ( debug ) &
            write(irefwr,2) 'valw0, valw1, diff', valw0, valw1, diff

!        --- distance := -diff/dwdx
!            Since this 1d we actually use ix = 1

         isolut2d =>isolut2d(iseqw1:iseqw1)
         dwdx = prderuserp ( userpoint(1), 1 )
         if ( ierror/=0 ) go to 1000
         if ( debug ) write(irefwr,2) 'dwdx', dwdx

!        --- Move user point over distance

         distance = 0d0
         distance(ixstoc) = -diff/dwdx

         if ( debug ) call prinrl ( distance, ndim, 'distance' )

         call mshmvcoorusp ( distance, userpoint, numusp )
         if ( ierror/=0 ) go to 1000

!        --- Adapt curves containing user point

         call mshadapcurv ( curvesw1, numcurw1 )

!        --- diff := abs(diff)
!            iter := iter+1

         diff = abs(diff)
         iter = iter+1
         if (iprint>=2 ) then

!        --- iprint = 2, maximal output
!            First get L

            coorloc = coor(:,userpoint(1))
            if ( ierror/=0 ) go to 1000
            L = coorloc(ixstoc)
            write(irefwr,200) iter, diff, L, valw0, valw1
200         format(i6, 5d12.3)

         end if  ! (iprint>=2 )

      end do  ! while ( diff>eps .and. iter<maxiter )

      if ( diff>eps ) then

!     --- Error, accuracy not reached, but number of iterations too large

         call errint ( 1, 1 )
         call errint ( iter, 2 )
         call erreal ( diff, 1 )
         call errsub ( 3040, 2, 1, 0 )
         go to 900

      end if  ! ( diff>eps )

      if (iprint>=1 ) then

!     --- iprint = 1, standard output

         write(irefwr,300) iter, diff, L
300      format('Number of iterations is ',i4/ &
                'The error is ', d12.3/ &
                'The computed interface has coordinate', d15.6 )

      end if  ! (iprint>=1 )

900   deallocate ( elgrpw0, elgrpw1, curvesw0, curvesw1, &
                   stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'elgrpw0', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prstoc1'

      end if  ! ( debug )
      call erclos ( 'prstoc1' )

      end subroutine prstoc1
