      subroutine prtimpresscor1 ( iinpresscor, dtlc, method_tm, first_step )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.6  date 13-12-2017 New storage of intmat
!        version  2.5  date 07-08-2014 use seprmbufarr
!        version  2.4  date 28-05-2014 New call to sepsolvel
!
!   copyright (c) 2005-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute pressure in pressure correction step and correct
!     velocity
! **********************************************************************
!
!                       KEYWORDS
!
!     time_integration
!     navier_stokes_equation
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmoduleuser
      use sepmoduleintmat
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulemain
      use sepmodulesolve
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinpresscor(*), method_tm
      double precision dtlc
      logical first_step

!     dtlc           i    Time step
!     first_step     o    If true it concerns the first time step
!     iinpresscor    i    integer input array
!     method_tm      o    Defines the time integration method
!                          5:  Euler implicit
!                         13:  Second orderr Gear
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpder(4), iseqveloc, iread(2), iseqpress, i, &
              idummy(1), iq, iinbldlc(2), &
              iu1, irhsdbc, iinvec(2), ictime, iseqcf
      double precision dummy(1), uu1(1), rinvec(2), factor, mfac

!     dummy          Dummy parameter
!     factor         Multiplication factor
!     i              general loop variable
!     ictime         Dummy parameter
!     idummy         Dummy parameter
!     iinbldlc       User input array for subroutine BUILD
!     iinvec         User input array for subroutine MANVEC
!                    For a description see subroutine MANVEC
!     inpder         Input array for DERIVS
!     iq             Contains information concerning the q update vector
!     iread          Defines how the input must be read
!                    See subroutines that are called
!     irhsdbc        Standard SEPRAN array, containing information of the
!                    large vector due to essential boundary conditions
!     iseqcf         Sequence number corresponding to the coefficients
!     iseqpress      Sequence number of pressure in set of solution arrays
!     iseqveloc      Sequence number of of velocity vector in set of
!                    solution arrays
!     iu1            Input array for subroutine creavc
!     mfac           Multiplication factor
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
!     uu1            Input array for subroutine creavc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     PRTIMPRESSCOR2 Fill zero boundary conditions into vector Gq
!     PRTIMPRESSCOR4 Store the difference between the present pressure
!                    boundary conditions and the previous ones into q
!     SEPBUILD       Build matrix and right-hand side
!     SEPCOPINPSOLIN Copy input for subroutine seplinsol from sepmoduleinput
!     SEPCREAVC      Create vector
!     SEPDERIVS      Compute derivatives
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
!     SEPLINSOL      Body of linear solver
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMMATRIX    Remove array with sequence number iseq from realmt
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The prediction velocity u* has been computed
!     Now the pressure correction must be computed and the
!     velocity updated
!     To compute the pressure update we solve:
!     - Laplace q^n+1 = -div (u*) / dt
!
!     The velocity is corrected by:
!
!     u^n+1 = u* + dt * Grad(q^n+1)
!
!     The pressure is updated either by:
!     p^n+1 = p^n + q^n+1   (standard method)
!        or
!     p^n+1 = p^n + q^n+1 - nu Div(u*)  (Timmermans/Minev method)
!     with nu the viscosity
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtimpresscor1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtimpresscor1'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      intmat => intmat1d(1)

!     --- Compute right-hand side -div(u*)/dt

      iseqveloc = iinpresscor(1)   ! sequence number of velocity vector
      iseqpress = iinpresscor(2)   ! sequence number of pressure vector

      inpder(1) = 4           ! number of entries filled by this subroutine
      inpder(2) = 1           ! output is vector of special structure (115)
      inpder(3) = 0           ! real output vector
      inpder(4) = 54          ! icheld, special value for pressure correction

      iread(1) = -1           ! Use array inpder as input array
      iread(2) = iinpresscor(3)    ! seguence number input for coefficients
      iprob = ks(isolut2d(iseqveloc))%iprob
      ictime = 0

      irhsd(1) = 0  ! memory management sequence number of irhsd (new)

      if ( debug ) then
         write(irefwr,100) 'iseqveloc, iseqpress', iseqveloc, iseqpress
         call sepprintv ( isolut2d(iseqveloc), 'veloc' )
      end if  ! ( debug )

      call sepderivs ( inpder, irhsd, isolut2d(iseqveloc), iread, &
                       dummy, iprob, idummy )
      if ( ierror/=0 ) go to 1000

!     --- Multiply vector by -1/dt (Euler implicit) or -3/2dt (Gear)

      if ( first_step .or. method_tm==5 ) then

!     --- First time step or Euler implicit

         factor = -1d0/dtlc

      else

!     --- Next time step and second order Gear
!         First store 2 u^n - 1/2 u^n-1 in ihelp1

         factor = -1.5d0/dtlc

      end if  ! ( first_step .or. method_tm==5 )

      rhsd1 => ks(irhsd(1))%sol
      rhsd1 = rhsd1*factor

      if ( debug ) call sepprintv ( irhsd, '-Lu*/dt' )

      intmat => intmat1d(2)
      iprob = intmt(intmat)%iprob
      iq = 0  ! memory management sequence number of iq (new)
      if ( iinpresscor(12)>0 ) then

!     --- Put difference in pressure boundary conditions into q

         call prtimpresscor4 ( iq, iinpresscor(12), iprob, &
                               isolut2d(iseqpress) )

      else

!     --- Create vector with zeros (essential bcs)

         iu1 = 0    ! value is constant
         uu1(1) = 0d0   ! constant is 0
         call sepcreavc ( 0, 1, iprob, 0, iq, iu1, uu1 )

      end if  ! ( iinpresscor(12)>0 )
      if ( debug ) call sepprintv ( iq, 'q/bouncond' )
      if ( ierror/=0 ) go to 1000

!     --- Solve q^n+1 from Laplace equation
!         First fill coefficients if necessary

      iseqcf = iinpresscor(6)  ! sequence number of coefficients
      if ( iseqcf>0 ) then

!     --- iseqcf>0, coefficients are available

      if ( ioutp>=0 .and. itimloop<=1 .and. iseqcf>0 .and. &
           nitercom<=1 ) write ( irefwr, 110 ) iseqcf
110   format ( ' Fill coefficients with sequence number ', i3 )

      call erlins ( isubr, iprob, iseqcf )

      iuser => iuser1(iprob)%iuser
      user => iuser1(iprob)%user

      end if  ! ( iseqcf>0 )
      if ( ierror/=0 ) go to 1000

!     --- Next build matrix and
!         put the effect of the boundary conditions in irhsdbc

      iinbldlc(1) = 2
      iinbldlc(2) = 1
      irhsdbc = 0  ! memory management sequence number of irhsdbc (new)
      call sepfillsmf ( iinbldlc, irhsdbc, iq, isolut2d )
      call sepbuild
      if ( debug ) call sepprintv ( irhsdbc, 'rhsd/bouncond' )
      if ( ierror/=0 ) go to 1000

      if ( iinpresscor(12)>0 ) then

!     --- Add irhsdbc to irhsd

         rhsdbc => ks(irhsdbc)%sol
         rhsd1 = rhsd1 + rhsdbc
         if ( debug ) call sepprintv ( irhsd, 'rhsd' )

      end if  ! ( iinpresscor(12)>0 )
      if ( ierror/=0 ) go to 1000

!     --- Remove irhsdbc

      call seprmsolinfo ( irhsdbc )

!     --- Next solve

      iseqreac = 0         ! no reaction force
      iseqsol = 1          ! only one vetor in iq
      iread(1) = iinpresscor(7) ! sequence number input solver

!     --- irhsd is of type 115, make it the right type

      ks(irhsd(1))%typevec = 110       ! make rhsd of type solution vector
      ks(irhsd(1))%iprob = iprob       ! put correct problem number in irhsd
      if ( ierror/=0 ) go to 1000

!     --- Solve system of equations

      ireadinpsol = iread(1)
      iseqintmat = 2
      keepmat = 0
      iseqsolut = 1
      iseqmt = stiffmat(1)
      call sepcopinpsolin ( iread(1) )
      call seplinsol ( stiffmat, iq, irhsd )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call sepprintv ( iq, 'q^n+1' )

!     --- Remove stiffmat

      call seprmmatrix ( stiffmat )
      if ( ierror/=0 ) go to 1000

!     --- Correct velocity
!         First compute Gq

      iprob = ks(isolut2d(iseqveloc))%iprob
      inpder(1) = 4           ! number of entries filled by this subroutine
      inpder(2) = 0           ! output is of type solution vector
      inpder(3) = 0           ! real output vector
      inpder(4) = 55          ! icheld, special value for pressure correction
      iread(1) = -1           ! Use array inpder as input
      iread(2) = -1           ! seguence number input for coefficients
      ks(iq)%typevec = 115
      ks(iq)%iprob = iprob
      ks(iq)%ivec = 6
      call sepderivs ( inpder, irhsd, iq, iread, dummy, iprob, idummy )
      if ( ierror/=0 ) go to 1000

!     --- Make Gq equal to zero for all essential boundary conditions

      call prtimpresscor2 ( irhsd, iprob )
      rhsd1 => ks(irhsd(1))%sol

      if ( debug ) call sepprintv ( irhsd, 'G q' )
      if ( ierror/=0 ) go to 1000

!     --- Next correct the velocity
!         u^n+1 = u* - (dt/B_0) * Grad(q^n+1)

      mfac = 1d0/factor  ! mfac= dt/B_0
      velocity => ks(isolut2d(iseqveloc))%sol
      velocity = mfac*rhsd1 + velocity

      if ( debug ) call sepprintv ( isolut2d(iseqveloc), 'u^(n+1)' )
      if ( ierror/=0 ) go to 1000

!     --- Update pressure
!         p^n+1 = p^n + q^n+1   (standard method)

      qvec => ks(iq)%sol
      pressure => ks(isolut2d(iseqpress))%sol
      pressure = pressure+qvec
      if ( ierror/=0 ) go to 1000

      if ( debug ) call sepprintv ( isolut2d(iseqpress), 'p^(n+1)' )

!     --- Remove irhsd and iq

      call seprmsolinfo ( irhsd(1) )
      call seprmsolinfo ( iq )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtimpresscor1'

      end if
      call erclos ( 'prtimpresscor1' )

      end subroutine prtimpresscor1
