      subroutine prtransnorm2d ( kmeshm, normal, ncurvs, icurvs, vect, &
                                 normalvect, jsmooth, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform a vector consisting of two components into the normal and
!     tangential component along a set of curves
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshm(*), ncurvs, icurvs(*), jsmooth
      double precision normal(2,*), vect(2,*), normalvect(2,*), &
                       coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     icurvs         i    array containing the curve numbers to be treated
!     jsmooth        i    if 0, end points are not smoothed, if >0 they are
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     ncurvs         i    Number of curves in icurvs
!     normal         i    Contains the normal vectors in the same sequence as
!                         kmeshm
!     normalvect    i/o   Array of size 2 x npoint containing the normal vect
!                         vector along the set of curves and zero elsewhere
!     vect           i    Array of size 2 x npoint containing the vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurnr, istar3, jcurnr, k, k1, k2, nodp, nodp1, nodp2
      double precision nx, ny, dudx, du1dx, h, h1

!     du1dx          Derivative in element next to first or last element
!     dudx           Derivative in first or last element
!     h              length of first or last element
!     h1             length of element next to first or last element
!     i              Counting variable
!     icurnr         Curve sequence number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     nodp           Nodal point sequence number
!     nodp1          Node sequence number of point next to nodp
!     nodp2          Node sequence number of point next to nodp1
!     nx             x component of normal
!     ny             y component of normal
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     255   nodal point number in curve has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all curves do
!        For all nodes along the curve do
!           transform from Cartesian to direction coordinates
!        if ( jsmooth>0 ) then
!           if ( derivative in first element much larger than in second one )
!              replace value in first node by value in second node - h du/dx
!              with h distance and dudx the derivative in second element
!           if ( derivative in last element much larger than in previous one )
!              replace value in last node by value in previous node + h du/dx
!              with h distance and dudx the derivative in previous element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtransnorm2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtransnorm2d'
         write(irefwr,1) 'jsmooth', jsmooth
         call prinin ( icurvs, ncurvs, 'icurvs' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istar3 = kmeshm(2)-1

!     --- For all curves icurnr in icurvs do

      do i = 1, ncurvs

         jcurnr = icurvs(i)
         icurnr = abs(jcurnr)
         k1 = kmeshm(4+icurnr)
         k2 = kmeshm(5+icurnr)-1
         if ( debug ) write(irefwr,1) 'jcurnr, k1, k2 ', jcurnr, k1, k2

!        --- Find nodes and normals and values
!            store inner product

         do k = k1, k2

            nx = normal(1,k)
            ny = normal(2,k)
            if ( jcurnr<0 ) then

!           --- Curve number negative, use negative normal

               nx = -nx
               ny = -ny

            end if  ! ( icurnr<0 )
            nodp = kmeshm(istar3+k)
            if ( debug ) write(irefwr,*) 'nodp, nx, ny ', nodp, nx, ny
            if ( nodp<=0 ) then

!           --- Not permitted

               call errint ( icurnr, 1 )
               call errint ( nodp, 2 )
               call errint ( 1000000, 3 )
               call errwar ( 255, 3, 0, 0 )

            else

!           --- standard

              if ( debug ) call prinrl ( vect(1,nodp), 2, 'vector' )
              normalvect(1,nodp) = nx*vect(1,nodp)+ny*vect(2,nodp)
              normalvect(2,nodp) = ny*vect(1,nodp)-nx*vect(2,nodp)

            end if  ! ( nodp<=0 )

         end do  ! k = k1, k2

         if ( jsmooth>0 .and. k2-k1>3 ) then

!        --- jsmooth = 1, smooth initial and end point
!            First initial point

            nodp  = kmeshm(istar3+k1)    ! initial point
            nodp1 = kmeshm(istar3+k1+1)  ! next point
            nodp2 = kmeshm(istar3+k1+2)  ! following point
            h1 = sqrt((coor(1,nodp2)-coor(1,nodp1))**2+ &
                     (coor(2,nodp2)-coor(2,nodp1))**2)  ! distance nodp2-nodp1
            h  = sqrt((coor(1,nodp1)-coor(1,nodp))**2+ &
                     (coor(2,nodp1)-coor(2,nodp))**2)  ! distance nodp1-nodp

            if ( debug ) then
               write(irefwr,1) 'nodp, nodp1, nodp2', nodp, nodp1, nodp2
               write(irefwr,2) 'h1, h', h1, h
            end if  ! ( debug )

            do k = 1, 2

               dudx = (normalvect(k,nodp2)-normalvect(k,nodp1))/h1  !derivative
               du1dx = (normalvect(k,nodp1)-normalvect(k,nodp))/h

               if ( debug ) then
                  write(irefwr,2) 'dudx, du1dx', dudx, du1dx
                  write(irefwr,2) 'normalvect(k,nodp)/bef', &
                                   normalvect(k,nodp)
               end if  ! ( debug )

               if ( abs(du1dx)>2d0*abs(dudx) ) then

!              --- Derivative too large, replace value in nodp

                  normalvect(k,nodp) = normalvect(k,nodp1)-dudx*h

               end if  ! ( abs(du1dx)>2d0*abs(dudx) )
               if ( debug ) &
                  write(irefwr,2) 'normalvect(k,nodp)/aft', &
                                   normalvect(k,nodp)

            end do  ! k = 1, 2

!           --- Next end point

            nodp  = kmeshm(istar3+k2)    ! end point
            nodp1 = kmeshm(istar3+k2-1)  ! previous point
            nodp2 = kmeshm(istar3+k2-2)  ! next point
            h1 = sqrt((coor(1,nodp2)-coor(1,nodp1))**2+ &
                     (coor(2,nodp2)-coor(2,nodp1))**2)  ! distance nodp2-nodp1
            h  = sqrt((coor(1,nodp1)-coor(1,nodp))**2+ &
                     (coor(2,nodp1)-coor(2,nodp))**2)  ! distance nodp1-nodp

            do k = 1, 2

               dudx = -(normalvect(k,nodp2)-normalvect(k,nodp1))/h1  !derivative
               du1dx = -(normalvect(k,nodp1)-normalvect(k,nodp))/h
               if ( abs(du1dx)>2d0*abs(dudx) ) then

!              --- Derivative too large, replace value in nodp

                  normalvect(k,nodp) = normalvect(k,nodp1)+dudx*h

               end if  ! ( abs(du1dx)>2d0*abs(dudx) )

            end do  ! k = 1, 2

         end if  ! ( jsmooth>0 )

      end do  ! i = 1, ncurvs

1000  call erclos ( 'prtransnorm2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtransnorm2d'

      end if  ! ( debug )

      end
