      subroutine read00method ( table, jmethod, code, ncodmx, codemn, scheme )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.10 date 06-09-2014 Use mpistart
!        version  1.9  date 11-07-2014 Extension with petsc
!        version  1.8  date 29-03-2014 Extension with mumps
!
!   copyright (c) 2007-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Translate keywords found in input file into the parameter jmethod
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jmethod, ncodmx(*), scheme
      double precision table(*)
      character (len=*) code(*), codemn(*)

!     code           i    Table with keywords
!     codemn         i    Table with main keywords (in this case subsukeywords)
!     jmethod        o    Indicates type of storage scheme for the matrix
!                         See Programmers Guide for a description
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     scheme         o    Type of storage scheme as read form input file
!                         Possible values
!                          1. COMPACT
!                          2. PROFILE
!                          3. SCHUR_COMPLEMENT
!                          4. ROW_COMPACT
!                          5. VECTOR_COMPUTER
!                          6. AUGMENTED_LAGRANGE
!                          7: AMG_ROW_COMPACT
!                          8: SIMPLE
!                          9: MUMPS
!                         10: PETSC
!                         11: NESTED_DISSECTION
!                         12: MINIMUM_DEGREE
!                         20: DEFAULT
!     table          i    Contains the numbers read for the various keywords
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icompl, isymm, isymincomp, ishiftlap

!     icompl         If real then 0, otherwise 1
!     ishiftlap      If shifted laplace has been found 1, else 0
!     isymincomp     if incompressibility and gradient each others transpose, 0, &
!                    else 1
!     isymm          if symmetrical then 0 else 1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     PRINRL         Print 1d real vector
!     SEPPETSCSTART  Start petsc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     488   Two keywords are mutually exclusive
!    2764   Integer parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read00method' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read00method'
         call ertrace
         call prinrl ( table, 12, 'table' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      isymm = 1
      if ( table(2)>0d0 ) isymm = 0  ! SYMMETRIC
      icompl = nint(table(3))      ! COMPLEX
      isymincomp = nint(table(7))  ! INCOMPRESSIBILITY_UNSYM
      ishiftlap = nint(table(10))   ! SHIFTED_LAPLACE
      if ( debug ) &
         write(irefwr,1) 'isymm, icompl, isymincomp, ishiftlap', &
                          isymm, icompl, isymincomp, ishiftlap

!     --- check symmetry

      if ( isymm==0 .and. table(5)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 2, 5 )

!     --- check real or complex

      if ( icompl>0 .and. table(4)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 3, 4 )

!     --- check symmetry of incompressiblity

      if ( table(6)>0d0 .and. isymincomp>0 ) &
         call errkeywexcl ( code, ncodmx, 6, 7 )

!     --- Check type of storage scheme

      scheme = nint(table(1))
      if ( debug ) write(irefwr,1) 'scheme/1', scheme
      if ( scheme<=0 ) then

!     --- scheme not found

         if ( table(12)>0d0 ) then

!        --- mumps found

            scheme = 9  ! mumps

         else if ( table(11)>0d0 ) then

!        --- petsc found

            scheme = 10  ! petsc

            if ( .not. mpistart ) then

!           --- mpi has not started, start petsc and make mpistart true

               call seppetscstart
               mpistart = .true.

            end if  ! ( .not mpistart )

         else

!        --- Not mumps found

            scheme = 20  ! default is profile

         end if  ! ( table(12)>0d0 )

      end if  ! ( scheme<=0 )

      if ( debug ) write(irefwr,1) 'scheme/2', scheme
      select case ( scheme )

         case (1)

!        --- compact storage found

            jmethod = 5+isymm+2*icompl  ! 5 to 8 for compact storage
            if ( ishiftlap==1 ) jmethod = jmethod+19  ! shifted laplace

         case (2, 11, 12, 20)

!        --- profile storage   (Default)

            jmethod = 1+isymm+2*icompl  ! 1 to 4 for profile storage
            if ( scheme==20 ) then
               scheme = 0
            else if ( scheme>10 ) then
               scheme = scheme-10
            end if  ! ( scheme==20 ) then

         case (3, 6, 8)

!        --- simple, al or schur found

            if ( icompl>0 ) then

!           --- keywords al and complex are mutually exclusive

               call errchr ( code(3)(1:ncodmx(3)), 1 )
               call errchr ( codemn(6)(1:6), 2 )
               call errsub ( 488, 0, 0, 2 )

            end if  ! ( compl )
            jmethod = 13+isymm+2*isymincomp  ! 13 to 16 for simple
            if ( debug ) &
               write(irefwr,1) 'isymm, isymincomp, jmethod', &
                                isymm, isymincomp, jmethod

         case (4)

!        --- compact row storage found

            jmethod = 6+2*icompl  ! 6 or 8 for row storage

         case (9)

!        --- mumps storage found

            jmethod = 52
            if ( table(2)>0d0 ) then

!           --- symmetric found

               if ( table(13)>0d0 ) then

!              --- positive-definite found

                  jmethod = 54

               else

!              --- not positive-definite found

                  jmethod = 53

               end if  ! ( table(13)>0d0 )

            end if  ! ( table(2)>0d0 )

         case (10)

!        --- petsc storage found

            jmethod = 55

         case default

            call errchr ( 'scheme', 1 )
            call errint ( scheme, 1 )
            call errsub ( 2764, 1, 0, 1 )

      end select  ! case ( scheme )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'jmethod, scheme', jmethod, scheme
         write(irefwr,*) 'End read00method'

      end if  ! ( debug )
      call erclos ( 'read00method' )

      end
