      subroutine readadbn ( inpada, rinada, icurves, isubr, &
                            iseqnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 13-11-2010 error 1488 --> errkeywexcl
!        version  2.4  date 13-05-2009 Extension with surfaces
!        version  2.3  date 16-02-2009 Fortran 90
!
!   copyright (c) 1996-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input for the subroutine ADAPBOUN
!   The input is stored in array INPADA
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     adapt_boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpada(*), isubr, iseqnr, icurves(*)
      double precision rinada(*)

!     icurves        o    Integer array containing the curve numbers for the
!                         adaptation of the boundary. A negative curve
!                         number indicates that the curve must be computed in
!                         the reversed direction.
!                         The curves including their sign must form one
!                         contiguous curve
!                         The number of curves is stored in array inpada
!                         If iredist>100, also the curve numbers of
!                         which the distribution must be changed are stored
!      inpada        o    Integer array to be filled by the user, indicating
!                         how the boundary must be adapted.
!                         INPADA is only used if iread = -1
!                         INPADA must be filled as follows:
!                         Pos. 1   Position of last entry that has been filled
!                                  by the user. If inpada(1) = 0 or 1, &
!                                  only defaults are used.
!                         Pos. 2   ncurves, indicates the number of curves
!                                  in array icurves that must be used for the
!                                  adaptation of the mesh
!                                  Default value: 1
!                         Pos. 3   Not yet used. Must be set equal to 0
!                                  Default value: 0
!                         Pos. 4   iadapt Indicates how the boundary must be
!                                  adapted
!                                  Possible values:
!                                  1.  The curves are adapted by:
!                                      xnew(i) = xold(i) + alpha(i) n(i)
!                                      with xold(i) the old co-ordinates of the
!                                      i-th node and xnew(i) the new ones.
!                                      n(i) is the normal at the boundary of the
!                                      i-th node and alpha(i) the number-th
!                                      degree of freedom in this node
!                                      The normal is computed assuming a counter
!                                      clockwise direction of the boundary
!                                  2.  The curves are adapted by the user
!                                      provided function subroutine FUNCCR
!                                  3.  See 1, however, now alpha is computed
!                                      according to the 'film method'
!                                      See: Caswell and Viriyayuthakorn, &
!                                      Finite element simulation of die swell
!                                      for a Maxwell fluid, J. of Non-Newt.
!                                      Fluid Mech. Vol 12, 13-29(1983).
!                                      alpha(i) = factor*(psi(i)-psi(0))/|vt(i)|
!                                                       xi
!                                      psi(i)-psi(0) = /  v.n dx
!                                                     x0
!                                      The vector isol must be the velocity
!                                      vector with degrees of freedom number and
!                                      number+1 from isol. Cartesian coordinates
!                                      Starting flow rate integration along free
!                                      boundary with flow rate zero in first
!                                      point of curves.
!                                  4.  Other method according to:
!                                      xnew(i) = xold(i) + v(i) * factor
!                                      Time-dependent case if imult=1
!                                      (see inpada(11))
!                                      In the axisymmetric case we use:
!                                      (r,z)new(i) = (r,z)old(i) + &
!                                                     (vr,vz)(i) dt /(2 pi)
!                                  5.  See 1. Now alpha is computed according to
!                                      alpha=v.n * factor
!                                  6.  See 1. Now alpha is computed according to
!                                      alpha=(grad c).n * factor
!                                      It is assumed that grad c is stored in
!                                      a vector of special structure defined per
!                                      element
!                                  7.  See 6. The more accurate scheme described
!                                      in Segal, Vuik and Vermolen is used
!                                      Use omega, eps and maxiter as stored in
!                                      rinada(7), rinada(6) and inpada(18)
!                                  8.  Constant velocity i.e.
!                                      xnew(i) = xold(i) + u * factor
!                                      where u is a given velocity vector
!                                      Time-dependent case if imult=1
!                                      (see inpada(11))
!                                  9.  The curves are adapted by the user
!                                      provided function subroutine FUNCSOLCR
!                                  Default value: 1
!                         Pos. 5   idirec Indicates the direction of the
!                                  computation
!                                  If idirec = 1 the computation is performed
!                                  from first to last point, &
!                                  if idirec = -1 from last to first point
!                                  It is sometimes required to alternate between
!                                  1 and -1 during iteration.
!                                  This option is only used for iadapt = 3
!                                  Default value: 1
!                         Pos. 6   number, the number-th degree of freedom of
!                                  isol (and higher) are used for the
!                                  computation of alpha.
!                                  See iadapt.
!                                  Default value: 1
!                         Pos. 7   linear. Indicates if the elements along the
!                                  boundary must be treated as linear or as
!                                  quadratic elements.
!                                  Possible values:
!                                  1: linear
!                                  2: quadratic
!                                  Default value: to be computed by the
!                                  subroutine in the following way:
!                                  If all internal elements are quadratic, the
!                                  boundary elements are assumed quadratic, &
!                                  otherwise linear
!                         Pos. 8   jcart defines the type of co-ordinate system
!                                  to be used. This is of importance for
!                                  iadapt = 3, 4 and 5
!                                  Possible values:
!                                  1:  Cartesian co-ordinates
!                                  2:  Axi-symmetric co-ordinates
!                                  Default value: 1
!                         Pos. 9   iexlude_1: indicates which of the
!                                  co-ordinates of the first point of the first
!                                  curve must be adapted.
!                                  Possible values:
!                                  0  Both co-ordinates must be adapted
!                                  1  Only the second co-ordinate must be
!                                     adapted, the first one remains unchanged
!                                  2  Only the first co-ordinate must be adapted
!                                     the second one remains unchanged
!                                  3  Both co-ordinates remain unchanged
!                                  Default value: 0
!                         Pos. 10  iexlude_2: indicates which of the
!                                  co-ordinates of the last point of the last
!                                  curve must be adapted.
!                                  Possible values:
!                                  0  Both co-ordinates must be adapted
!                                  1  Only the second co-ordinate must be
!                                     adapted, the first one remains unchanged
!                                  2  Only the first co-ordinate must be adapted
!                                     the second one remains unchanged
!                                  3  Both co-ordinates remain unchanged
!                                  Default value: 0
!                         Pos. 11  imult: indicates how the value of factor
!                                  must be computed. (iadapt>2)
!                                  Possible values:
!                                  0  Multiplication factor is 1
!                                  1  Multiplication factor is dt from common
!                                     ctime
!                                  2  Multiplication factor is stored in
!                                     rinada(1)
!                                  3  Multiplication factor is dt*rinada(1)
!                                  Default value: 0
!                         Pos. 12  ithres: indicates if a threshold value is
!                                  used in Caswell's method and where to find it
!                                  Possible values:
!                                  0  Threshold value = 0
!                                  1  Threshold value is stored in rinada(2)
!                                  Default value: 0
!                         Pos. 13  iangle_1: Indicates if angle of freesurface
!                                  at first point is prescribed or not
!                                  Possible values:
!                                  0  angle is not given
!                                  1  angle is stored in rinada(3)
!                                 <0  The first point must be projected onto
!                                     the curve c |iangle_1|
!                                  Default value: 0
!                         Pos. 14  iangle_2: Indicates if angle of freesurface
!                                  at last point is prescribed or not
!                                  Possible values:
!                                  0  angle is not given
!                                  1  angle is stored in rinada(4)
!                                 <0  The last point must be projected onto
!                                     the curve c |iangle_2|
!                                  Default value: 0
!                         Pos. 15  redistribute nodes
!                                  Consists of two parts
!                                  iredist_type and ncurves_redist according to:
!                                  iredist = iredist_type + 100*ncurves_redist
!                                  Possible values:
!                                  iredist_type:
!                                  0  no node redistribution
!                                  1  nodes are redistributed, nelm constant
!                                  2  nodes are redistributed, nelm recomputed
!                                     according to coarseness
!                                  ncurves_redist: number of curves that
!                                  must be redistributed
!                                  The curve numbers are stored in icurves
!                                  positions ncurves+1 to ncurves+ncurves_redist
!                                  This option is only available in combination
!                                  with iredist = 1
!                                  Default value: 0
!                         Pos. 16  Plot_boun
!                                  Possible values:
!                                  0  do not plot new boundary
!                                  1  plot new boundary
!                                  2  plot new boundary, use xmin, xmax, ymin
!                                     and ymax from rinada(8-11)
!                                  3  plot new boundary, use
!                                     yfact from rinada(14)
!                                  4  plot new boundary, use xmin, xmax, ymin
!                                     and ymax from rinada(8-11) and
!                                     yfact from rinada(14)
!                                  Default value: 0
!                         Pos. 17  Numobst (Number of obstacles)
!                                  Each obstacle must be closed and consist of
!                                  one curve only
!                                  (This may be a curve of curves)
!                                  At most 10 obstacles may be used.
!                                  The obstacle numbers of the Numobst obstacles
!                                  are stored in array icurves positions:
!                                  ncurves+ncurves_redist+1 ... .+Numobst
!                                  Default value: 0
!                         Pos. 18  Maxiter, maximum number of iterations in
!                                  iteration process
!                                  Default value: 10
!                         Pos. 19  Prin_boun
!                                  Possible values:
!                                  0  do not plot new boundary
!                                  1  print new boundary
!                                  2  print new boundary, use xmin, xmax, ymin
!                                     and ymax from rinada(8-11)
!                                  Default value: 10
!                         Pos. 20  check_direction
!                                  Possible values:
!                                 -1  the direction of boundary must be
!                                     negative relative to the outer normal
!                                  0  no checking
!                                  1  the adaptation of boundary must be
!                                     in the direction of the outer normal
!                                  Default value: 0
!     iseqnr        i/o   Sequence number with respect to the storage in array
!                         ISEPIN
!     isubr          i    Indicates by which subroutine READADBN is called
!                         Possible values:
!                         1:  ADAPBOUN,  read record with ADAPT_BOUNDARY
!                         2:  ERADAPBN,  do not read record with ADAPT_BOUNDARY
!     rinada         o    Contains real information concerning the adaptation of
!                         the boundary. See array inpada
!                         1:  multiplication factor
!                         2:  threshold value
!                         3:  angle_1
!                         4:  angle_2
!                         5:  alpha for splines
!                         6:  accuracy eps for non-linear iteration
!                         7:  relaxation parameter omega
!                         8-13:  plot region (xmin,xmax,ymin,ymax)
!                             positions 12,13 reserved for zmin, zmax
!                        14:  yfact
!                        15-17:  velocity u
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=31, nncod2=22
      integer ncode1(nncode1), ncode2(nncod2), itype1(nncode1), ncodmx1(nncode1)
      integer k, numcurves, nobst, m, idim
      double precision :: tabmin1(nncode1), tabmax1(nncode1)
      character (len=25) code1(nncode1), code2(nncod2)

!     code1           Table with keywords corresponding to options
!     code2          Table with the keywords following the subkeywords
!     idim           Refers to position in table corresponding to dimension
!     itype1         itype array for code1 (See intx02)
!     k              Counting variable
!     m              Number of numbers read
!     ncode1          Table containing the number of significant characters in
!                    the keyword table code1
!     ncode2         See ncode, now for table code2
!     ncodmx1         Table containing the number of maximum characters in
!                    the keyword table code1
!     nncod2         Number of entries in table code2
!     nncode1         Number of obstacles
!     nobst          Maximal length of array tabout
!     numcurves      Number of curves found
!     tabmax1         Table with maximum values of numbers corresponding to code1
!     tabmin1         Table with minimum values of numbers corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     INICHKCURVS    Check if curves are contiguous
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     SEPSETMAINEND  Fill keyword END as first main keyword and set nmain to 1
! **********************************************************************
!
!                       I/O
!
!     readadbn requires input from the standard input file
!     The following input is recognized:
!     ADAPT_BOUNDARY  [, SEQUENCE_NUMBER = s]      (mandatory):
!        start of input
!      optional records:                          Default:
!        curves = ( c1, c5, -c8, ... )            c1
!        surfaces = ( si, ... )                   s1
!        adaptation_method = a                    standard
!        direction = i                            1
!        number = n                               1
!        linear/quadratic                         see manual
!        coordinate_system = s                    Cartesian
!        exlude_begin = i                         0
!        exlude_end = i                           0
!        multiply = m                             factor
!        factor = f                               1
!        threshold_value = t                      no threshold
!        angle_1 = alpha1                         no angle given
!        angle_2 = alpha2                         no angle given
!        distribute_curves = (c1, c3, c6 )        not
!        redistribute_nodes                       not
!        change_number_of_elements                not
!        plot_boundary                            no plot
!        print_boundary                           no print
!        alpha_redistribute                       1
!        move_begin = ci                          0
!        move_end = ci                            0
!        obstacle (i1, i2, .. )                   no obstacles
!        accuracy = eps                           1d-2
!        omega = w                                0.5
!        maxiter = n                              10
!        region = ( xmin,xmax,ymin,ymax )         Complete region
!        yfact = y                                1
!        velocity=(u_1,u_2,u_3)                   (0,0,0)
!        check_direction = c                      none
!     END   (mandatory):  end of input
!     With respect to the subkeywords the following option are possible:
!      adaptation_method = standard           (iadapt=1)
!                          funccr             (iadapt=2)
!                          film_method        (iadapt=3)
!                          velocity           (iadapt=4)
!                          normal_velocity    (iadapt=5)
!                          normal_gradient    (iadapt=6)
!                          stefan             (iadapt=7)
!                          constant_velocity  (iadapt=8)
!                          funcsolcr          (iadapt=9)
!      coordinate_system = Cartesian
!                          Axi_symmetric
!      exlude_i = none                        (iexclude_i=1)
!                 first                       (iexclude_i=2)
!                 second                      (iexclude_i=3)
!                 both                        (iexclude_i=4)
!      multiply = none                        (mult=1)
!                 dt                          (mult=2)
!      check_direction = negative             (check_dir=-1)
!                        none                 (check_dir= 0)
!                        positive             (check_dir= 1)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     650   First record read by subroutine is not equal to ADAPT_BOUNDARY
!     665   Number of numbers found in REGION = ( ..., ...,    ) not 2,4 or 6
!    1588   Two keywords are mutually exclusive
!    1632   Number after keyword incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (derivatives)
!    Read rest of first record (sequence number, problem number )
!    Read the rest of the records by intx02 until END is found
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'CURVES', 'ADAPTATION_METHOD', 'DIRECTION', 'NUMBER', &
                  'LINEAR', 'QUADRATIC', 'COORDINATE_SYSTEM', &
                  'EXCLUDE_BEGIN', 'EXCLUDE_END', 'MULTIPLY', &
                  'FACTOR', 'THRESHOLD_VALUE', 'ANGLE', 'ANGLE', &
                  'REDISTRIBUTE_NODES', 'CHANGE_NUMBER_OF_ELEMENTS', &
                  'PLOT_BOUNDARY', 'DISTRIBUTE_CURVES', &
                  'ALPHA_REDISTRIBUTE', 'MOVE_BEGIN', &
                  'MOVE_END', 'OBSTACLE', 'ACCURACY', &
                  'OMEGA', 'MAXITER', 'REGION', 'YFACT', &
                  'VELOCITY', 'PRINT_BOUNDARY', 'CHECK_DIRECTION', &
                  'SURFACES' /
      data ncode1 /  3,  5,  5,  3,  3,   4,  4,  9,  9,  3, &
                    3,  5,  3,  3,  4,   4,  4,  6,  5,  8, &
                    8,  4,  4,  4,  4,  4,   3,  4,  7,  9, &
                    4 /
      data ncodmx1 /  6, 17,  9,  6,  6,   9, 17, 13, 11,  8, &
                     6, 15,  5,  5, 18,  25, 13, 17, 18, 10, &
                    10,  8,  8,  5,  7,  6,   5,  8, 14, 15, &
                     8 /
      data itype1 / 18, 5,  0, 0, 2,   2, 5, 5, 5, 5, &
                    1, 1, 10, 1, 2,   2, 2,18, 1,18, &
                   18,11,  1, 1, 0,  19, 1,19, 2, 5, &
                   18  /
      data tabmin1 / 0d0, 1d0, -1d0, 1d0, 0d0,   0d0, 7d0, 9d0, 9d0,13d0, &
                  -1d10,-1d10, 1d0,-7d0, 0d0,   0d0, 0d0, 0d0, 0d0, 0d0, &
                    0d0, 0d0,1d-10, 0d0, 1d0,   4d0, 0d0, 2d0, 0d0,20d0, &
                    0D0 /
      data tabmax1 / 0d0, 5d0,  1d0, 1d2, 0d0,   0d0, 8d0,12d0,12d0,14d0, &
                   1d10, 1d10, 2d0, 7d0, 0d0,   0d0, 0d0, 0d0, 1d0, 0d0, &
                    0d0, 0d0,  1d0, 1d0, 1d2,   1d5, 1d5, 3d0, 0d0,22d0, &
                    0D0 /
      data code2 / 'STANDARD', 'FUNCCR', 'FILM_METHOD', &
                   'VELOCITY', 'NORMAL_VELOCITY', 'NORMAL_GRADIENT', &
                   'STEFAN', 'CONSTANT_VELOCITY', 'FUNCSOLCR', &
                   'CARTESIAN', 'AXI_SYMMETRIC', &
                   'NONE', 'FIRST', 'SECOND', 'BOTH', &
                   'NONE', 'DT', 'C', 'NEGATIVE', 'NONE', 'POSITIVE', &
                   'S' /
      data ncode2 / 4, 6, 4, 4, 10, 10, 6, 5, 8, &
                    4, 3, 3, 3, 3,  4,  3, 2, 1, 5, &
                    4, 5, 1 /

!     code1    Table with keywords corresponding to options
!             Values:
!             1:  CURVES
!             2:  ADAPTATION_METHOD
!             3:  DIRECTION
!             4:  NUMBER
!             5:  LINEAR
!             6:  QUADRATIC
!             7:  COORDINATE_SYSTEM
!             8:  EXCLUDE_BEGIN
!             9:  EXCLUDE_END
!            10:  MULTIPLY
!            11:  FACTOR
!            12:  THRESHOLD_VALUE
!            13:  ANGLE
!            14:  ANGLE
!            15:  REDISTRIBUTE_NODES
!            16:  CHANGE_NUMBER_OF_ELEMENTS
!            17:  PLOT_BOUNDARY
!            18:  DISTRIBUTE_CURVES
!            19:  ALPHA_REDISTRIBUTE
!            20:  MOVE_BEGIN
!            21:  MOVE_END
!            22:  OBSTACLE
!            23:  ACCURACY
!            24:  OMEGA
!            25:  MAXITER
!            26:  REGION
!            27:  YFACT
!            28:  VELOCITY
!            29:  PRINT_BOUNDARY
!            30:  CHECK_DIRECTION
!            31:  SURFACES
!     code1   Table with the keywords in the ADAPT_BOUNDARY record
!             Values:
!             1: sequence
!     code2   Table with the keywords following the subkeywords
!             Values:
!             1: STANDARD
!             2: FUNCCR
!             3: FILM_METHOD
!             4: VELOCITY
!             5: NORMAL_VELOCITY
!             6: NORMAL_GRADIENT
!             7: STEFAN
!             8: CONSTANT_VELOCITY
!             9: FUNCSOLCR
!            10: CARTESIAN
!            11: AXI_SYMMETRIC
!            12: NONE
!            13: FIRST
!            14: SECOND
!            15: BOTH
!            16: NONE
!            17: DT
!            18: C
!            19: NEGATIVE
!            20: NONE
!            21: POSITIVE
!            22: S
! ======================================================================
!
      call eropen ( 'readadbn' )

      inpada(1:20) = 0
      inpada(2) = 1
      inpada(4) = 1
      inpada(5) = 1
      inpada(6) = 1
      inpada(8) = 1
      inpada(18) = 10

      rinada(1:17) = 0d0
      rinada(5) = 0.5d0
      rinada(6) = 1d-2
      rinada(7) = 0.5d0

      if ( isubr==1 ) then

!     --- Call by ADAPBN, read first record  (ADAPT_BOUNDARY)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'ADAPT_BOUNDARY', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         else if ( string(1:5)/='ADAPT' ) then
            call errchr ( 'ADAPT_BOUNDARY', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

      call sepreadseqnr ( iseqnr )    ! Read sequence number

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )
      call sepcopycodesub ( code2, nncod2, ncode2 )
      call sepsetmainend

      table(1:nncode) = -1d20
      tabmin(1) = ncurvs
      tabmax(1) = ncurvs
      tabmin(2) = 1d0
      tabmax(2) = 9d0
      tabmin(7) = tabmax(2)+1d0
      tabmax(7) = tabmin(7)+1d0
      tabmin(8) = tabmax(7)+1d0
      tabmax(8) = tabmin(8)+3d0
      tabmin(9) = tabmin(8)
      tabmax(9) = tabmax(8)
      tabmin(10) = tabmax(9)+1d0
      tabmax(10) = tabmin(10)+1d0
      iposut = nint(tabmax(10))+1
      table(1) = -iposut         ! refers to subkeyword C
      tabmin(18) = ncurvs
      tabmax(18) = ncurvs
      table(18) = -iposut
      table(20) = -iposut
      tabmin(20) = 1d0
      tabmax(20) = ncurvs
      table(21) = -iposut
      tabmin(21) = 1d0
      tabmax(21) = ncurvs
      tabmin(22) = numobst
      tabmax(22) = numobst
      tabmax(28) = ndim
      iposut = iposut+4         ! refers to subkeyword S
      table(31) = -iposut
      tabmin(31) = nsurfs
      tabmax(31) = nsurfs
      istop = 2
      iretur = 0
      call sepreadsetkeyw
      istop = 0
      numcurves = 0

      if ( ndim==2 ) then

!     --- ndim = 2, curves

         idim = 1
         if ( table(31)>0d0 ) call errsub ( 3016, 0, 0, 0 )

      else

!     --- ndim = 3, surfaces

         idim = 31
         if ( table(1)>0d0 ) call errsub ( 3017, 0, 0, 0 )

      end if  ! ( ndim==2 )

      if ( table(idim)>0 ) numcurves = nint(tabmax(idim))
      if ( numcurves>0 ) then

!     --- Keyword curves found

         icurves(1:numcurves) = nint(tabout(1:numcurves))
         inpada(2) = numcurves

!        --- Check if curves are contiguous

         if ( idim==1 ) call inichkcurvs ( icurves, numcurves, kmeshm )

      else

!     --- Keyword curves not found

         icurves(1) = 1

      end if

      if ( table(2)>0d0 ) inpada(4) = nint(table(2))
      if ( table(3)>-1d1 ) inpada(5) = nint(table(3))
      if ( inpada(5)==0 ) then
         call errint ( inpada(5), 1 )
         call errchr ( code(3), 1 )
         call errsub ( 1632, 1, 0, 1 )
      end if
      if ( table(4)>0d0 ) inpada(6) = nint(table(4))
      if ( table(5)>0d0 .and. table(6)>0d0 ) then
         call errkeywexcl ( code, ncodmx, 5, 6 )
      else if ( table(5)>0d0 ) then
         inpada(7) = 1
      else if ( table(6)>0d0 ) then
         inpada(7) = 2
      end if
      if ( table(7)>0d0 ) inpada(8) = nint(table(7))
      if ( table(8)>0d0 ) inpada(9) = nint(table(8))-1
      if ( table(9)>0d0 ) inpada(10) = nint(table(9))-1
      if ( table(10)>0d0 .and. table(11)>-1d19 ) then
         inpada(11) = 3
         rinada(1) = table(11)
      else if ( table(10)>0d0 ) then
         inpada(11) = nint(table(10))-1
      else if ( table(11)>-1d19 ) then
         inpada(11) = 2
         rinada(1) = table(11)
      end if
      if ( table(12)>-1d19 ) then
         inpada(12) = 1
         rinada(2) = table(12)
      end if
      if ( table(13)>-1d19 ) then
         inpada(13) = 1
         rinada(3) = table(13)
      end if
      if ( table(14)>-1d19 ) then
         inpada(14) = 1
         rinada(4) = table(14)
      end if
      if ( table(15)>0d0 .and. table(16)>0d0 ) then
         call errkeywexcl ( code, ncodmx, 15, 16 )
      else if ( table(15)>0d0 ) then
         inpada(15) = 1
      else if ( table(16)>0d0 ) then
         inpada(15) = 2
      end if
      if ( table(17)>0d0 ) inpada(16) = 1
      if ( table(29)>0d0 ) inpada(19) = 1
      if ( table(18)>0d0 ) then
         if ( inpada(15)==0 ) inpada(15) = 1
         numcurves = nint(tabmax(18))
         inpada(15) = inpada(15)+100*numcurves
         iposut = nint(table(18))
         do k = 1, numcurves
            icurves(k+inpada(2)) = nint(tabout(iposut+k-1))
         end do
      end if
      if ( table(19)>0d0 ) rinada(5) = table(19)
      if ( table(20)>0d0 ) then

!     --- Move begin is given

         if ( inpada(13)>0 ) then

!        --- Error already angle given

            call errchr ( 'ANGLE_1', 1 )
            call errchr ( code(20), 2 )
            call errsub ( 1588, 0, 0, 2 )

         else

            iposut = nint(table(20))
            inpada(13) = -nint(tabout(iposut))

         end if

      end if

      if ( table(21)>0d0 ) then

!     --- Move end is given

         if ( inpada(14)>0 ) then

!        --- Error already angle given

            call errchr ( 'ANGLE_2', 1 )
            call errchr ( code(21), 2 )
            call errsub ( 1588, 0, 0, 2 )

         else

            iposut = nint(table(21))
            inpada(14) = -nint(tabout(iposut))

         end if

      end if
      nobst = 0
      if ( table(22)>0d0 ) then

!     --- obstacles

         numcurves = inpada(2)+inpada(15)/100
         iposut = nint(table(22))
         if ( iposut>0 ) then
            nobst = nint(tabmax(22))
            do k = 1, nobst
               icurves(k+numcurves) = nint(tabout(iposut+k-1))
            end do
         end if

      end if

      inpada(17) =  nobst
      inpada(1) = 19

!     --- Input with respect to Stefan boundary conditions

      if ( table(23)>0d0 ) rinada(6) = table(23)
      if ( table(24)>0d0 ) rinada(7) = table(24)
      if ( table(25)>0d0 ) inpada(18) = nint(table(25))

!     --- Plot region

      if ( table(26)>0d0 ) then

         iposut = nint(table(26))
         m = nint(tabmax(26))
         if ( m/=4 ) then

!        --- m # 4

            call errint ( m, 1 )
            call errsub ( 665, 1, 0, 0 )
            go to 1000

         end if
         tabmax(26) = 4d0
         do k = 0, 3
            rinada(8+k) = tabout(iposut+k)
         end do
         if ( inpada(16)==1 ) inpada(16) = 2
         if ( inpada(19)>0 ) inpada(19) = inpada(19)+10

      end if
      if ( table(27)>0d0 ) then

!     --- yfact = y

         inpada(16) = inpada(16) + 2
         rinada(14) = table(27)

      end if
      if ( table(28)>0d0 ) then

!     --- velocity = (u_1,u_2,u_3)

         iposut = nint(table(28))
         m = nint(tabmax(28))
         do k = 0, m-1
            rinada(15+k) = tabout(iposut+k)
         end do

      end if
      if ( table(30)>0d0 ) inpada(20) = nint(table(30))-2

1000  call erclos ( 'readadbn' )

      end subroutine readadbn
