      subroutine readadms ( inpmsh, rinmsh, isubr, iseqnr, ireadbn, &
                            maxlen )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 09-03-2013 Correction length code
!        version  2.0  date 26-08-2011 Remove tabout from parameter list
!        version  1.2  date 14-08-1996 Correction position 1
!
!   copyright (c) 1996-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input for the subroutine ADAPMESH
!   The input is stored in array INPMSH
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     adapt_boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpmsh(*), isubr, iseqnr, ireadbn(*), maxlen
      double precision rinmsh(*)

!      inpmsh    o    Integer array to be filled by the user, indicating how the
!                     boundary must be adapted.
!                     INPMSH is only used if iread = -1
!                     INPMSH must be filled as follows:
!                     Pos. 1   Position of last entry that has been filled by
!                              the user. If inpmsh(1) = 0 or 1, only defaults
!                              are used.
!                              Default value: 0
!                     Pos. 2   NCALL: Number of calls of adapboun that must be
!                              used to update the complete boundary before
!                              updating the mesh.
!                              In this way it is possible to update parts of the
!                              boundary separately
!                              The positions iread(2) to iread(NCALL+1)
!                              correspond to the various inputs for each of
!                              these calls
!                              Default value: 1
!                     Pos. 3   CHANGE_TOPOLOGY: defines whether the topology
!                              must be adapted or not
!                              Possible values:
!                              0: The topology remains unchanged
!                              1: The topology is always adapted
!                              2: The topology is adapted if necessary
!                              Default value: 0
!                     Pos. 4   PLOT_MESH: defines whether the new mesh
!                              must be plotted or not
!                              Possible values:
!                              0: No plot
!                              1: Plot
!                              Default value: 0
!                     Pos. 5   INTERPOLATE_SOL: defines the number of solution
!                              vectors that must be interpolated
!                              Default value: 0
!                     Pos. 6   INDEX_SOL: defines the starting address in
!                              INPMSH of the vectors to be interpolated
!                              In positions INPMSH(INDEX_SOL) to
!                              INPMSH(INDEX_SOL+INTERPOLATE_SOL-1) the
!                              sequence numbers of the solution vectors
!                              to be interpolated are stored.
!                              If INDEX_SOL = 0, the vectors 1 to
!                              INTERPOLATE_SOL are interpolated
!                              If INDEX_SOL<0, the vectors
!                              -INDEX_SOL to -INDEX_SOL+INTERPOLATE_SOL-1
!                              are interpolated
!                              Default value: 0
!     ireadbn    o    Array of length 25 containing the sequence numbers
!                     with respect to adapt boundary
!     iseqnr    i/o   Sequence number with respect to the storage in array
!                     ISEPIN
!     isubr      i    Indicates by which subroutine READADBN is called
!                     Possible values:
!                     1:  ADAPMESH,  read record with ADAPT_MESH
!                     2:  ERADAPMS,  do not read record with ADAPT_MESH
!     maxlen     i    Maximum length of array inpmsh
!     rinmsh     o    See inpmsh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=4, nncod2=4
      integer ncode1(nncode1), ncode2(nncod2), itype1(nncode1), ncodmx1(nncode1)
      integer k, m, ifirst, lastmsh
      double precision :: tabmin1(nncode1), tabmax1(nncode1)
      character (len=20) code1(nncode1), code2(nncod2)
      logical sequent

!     code1       Table with keywords corresponding to options
!     code2      Table with the keywords following the subkeywords
!     ifirst     First array to be interpolated
!     itype1      itype array for code1  (See intx02)
!     k          Counting variable
!     lastmsh    Last position filled in array inpmsh
!     m          Number of arrays to be interpolated
!     ncode1      Table containing the number of significant characters in
!                the keyword table code1
!     ncode2     See ncode, now for table code2
!     ncodmx1     Table containing the number of maximum characters in
!                the keyword table code1
!     nncod2     Number of entries in table code2
!     nncode1     Number of entries in table code1
!     sequent    If true the arrays to be interpolated are subsequent arrays
!     tabmax1     Table with maximum values of numbers corresponding to code1
!     tabmin1     Table with minimum values of numbers corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     SEPSETMAINEND  Fill keyword END as first main keyword and set nmain to 1
! **********************************************************************
!
!                       I/O
!
!     readadms requires input from the standard input file
!     The following input is recognized:
!     ADAPT_MESH [, SEQUENCE_NUMBER = s]      (mandatory):
!        start of input
!      optional records:                               Default:
!        adapt_boundary = (i1, i2, i3,... )            1
!        change_topology                               not
!        plot_mesh                                     not
!        interpolate_solution = ( v1, v2, ... , vn )   not
!     END   (mandatory):  end of input
!     With respect to the subkeywords the following option are possible:
!      change_topology = not
!                        always
!                        dependent
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!     650   First record read by subroutine is not equal to ADAPT_BOUNDARY
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (derivatives)
!    Read rest of first record (sequence number, problem number )
!    Read the rest of the records by intx02 until END is found
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'ADAPT_BOUNDARY', 'CHANGE_TOPOLOGY', 'PLOT_MESH', &
                  'INTERPOLATE_SOL' /
      data ncode1 /  10, 10, 4, 5 /
      data ncodmx1 / 14, 15, 9,20 /
      data itype1 /  11,  5, 2,18 /
      data tabmin1 / 25d0, 1d0, 0d0, 0d0 /
      data tabmax1 / 25d0, 3d0, 0d0, 0d0 /
      data code2 / 'NOT', 'ALWAYS', 'DEPENDENT', 'V' /
      data ncode2 / 3, 4, 5, 1 /

!     code1    Table with keywords corresponding to options
!             Values:
!             1:  ADAPT_BOUNDARY
!             2:  CHANGE_TOPOLOGY
!             3:  PLOT_MESH
!             4:  INTERPOLATE_SOLUTION
!     code2   Table with the keywords following the subkeywords
!             Values:
!             1: NOT
!             2: ALWAYS
!             3: DEPENDENT
!             4: V
! ======================================================================
!
      call eropen ( 'readadms' )

      lastmsh = 6
      inpmsh(1:lastmsh) = 0
      inpmsh(1) = lastmsh
      inpmsh(2) = 1
      rinmsh(1) = 0d0
      ireadbn(1) = 1

      if ( isubr==1 ) then

!     --- Call by ADAPMS, read first record  (ADAPT_MESH)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'ADAPT_MESH', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         else if ( string(1:5)/='ADAPT' ) then
            call errchr ( 'ADAPT_MESH', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

      call sepreadseqnr ( iseqnr )    ! Read sequence number

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )
      call sepcopycodesub ( code2, nncod2, ncode2 )
      call sepsetmainend

      table(1:nncode) = -1d20
      tabmax(1) = 25d0
      table(4)  = -5d0
      tabmin(4) = nout
      tabmax(4) = 1d2
      istop = 2
      iretur = 0
      call sepreadsetkeyw
      istop = 0
      if ( table(1)>0d0 ) then
         inpmsh(2) = nint(tabmax(1))
         iposut = nint(table(1))
         do k = 1, inpmsh(2)
            ireadbn(k) = nint(tabout(iposut+k-1))
         end do
      end if
      if ( table(2)>0d0 ) inpmsh(3) = nint(table(2))-1
      if ( table(3)>0d0 ) inpmsh(4) = 1
      if ( table(4)>0d0 ) then

!     --- interpolate solution found

         iposut = nint(table(4))
         m = nint(tabmax(4))
         inpmsh(5) = m

!        --- Check the sequence numbers stored in tabout

         ifirst = nint(tabout(iposut))
         sequent = .true.
         do k = 1, m-1
            if ( nint(tabout(iposut+k))/=nint(tabout(iposut+k-1))-1 ) &
               sequent = .false.
         end do
         if ( sequent ) then
            if ( ifirst>1 ) inpmsh(6) = -ifirst
         else
            inpmsh(6) = lastmsh+1
            if ( lastmsh+m>maxlen ) then
               call errint ( lastmsh+m, 1 )
               call errint ( maxlen, 2 )
               call errchr ( 'inpmsh', 1 )
               call errsub ( 550, 2, 0, 1 )
               go to 1000
            end if
            do k = 1, m
               inpmsh(lastmsh+k) = nint(tabout(iposut+k-1))
            end do
            lastmsh = lastmsh+m
         end if
      end if
      inpmsh(1) = lastmsh

1000  call erclos ( 'readadms' )
      end subroutine readadms
