      subroutine readbearing ( iinput, rinput, isubr, iseqnr, niinp, &
                               nrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 07-03-2012 New call to readseqvector
!        version  1.1  date 12-03-2008 New way of defining scalars
!        version  1.0  date 04-09-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for bearings
!     The input is stored in array iinput
!     The input must be enclosed between BEARING and END.
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     mass_conservation
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niinp, iinput(*), iseqnr, isubr, nrinp
      double precision rinput(*)

!     iinput         o    The integer input read is stored in array IINPUT
!                         This array must be filled as follows:
!                          1: Sequence number of coefficients input
!                             Default value: 1
!                          2: Problem number
!                             Default value: 1
!                          3: Sequence number of pressure in solution array
!                             Default value: 1
!                          4: Sequence number of linear solver input
!                             Default value: 1
!                          5: Sequence number of reaction_force
!                             in solution array
!                             Default value: 2
!                          6: Maximum number of iterations
!                             Default value: 10
!                          7: Print_level
!                             Default value: 0
!                          8: ichoice, type of solution method (not yet used)
!                             1:   Kumar's mass conservation method
!                             Default value: 1
!                          9-10: Not yet used
!                             Default value: 0
!     iseqnr        i/o   Sequence number with respect to the storage in
!                         array ISEPIN
!     isubr          i    Indicates by which subroutine readpresscor is called
!                         Possible values:
!                         1:  ...      read record with
!                         2:  ERbearing,  do not read record with
!                                           NAVIER_STOKES
!     niinp          i    Length of integer input array
!     nrinp          i    Length of real input array
!     rinput         o    Real input array
!                         1:   Cavity pressure
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=9
      integer ncode2(nncode2), itype2(nncode2), ncodmx2(nncode2)
      double precision tabmn2(nncode2), tabmx2(nncode2)
      character (len=23) code2(nncode2)
      character (len=13) mainkeyw

!     code2          Table with keywords
!     itype2         Table containing information about the numbers behind
!                    the keywords in the table code2
!     mainkeyw       Name of main keyword for this block
!                    Only used for error messages
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     nncode2        Number of entries in code2
!     tabmn2         Contains minimum values allowed in table
!     tabmx2         Contains maximum values allowed in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     READSEQVECTOR  Put the value of the sequence number read in iinput
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEY1 Interpret a record containing keywords followed by data
!                    Set END as sole main keyword
! **********************************************************************
!
!                       I/O
!
!     readbearing requires input from the standard input file
!     The following input is recognized:
!     BEARING [ SEQUENCE_NUMBER = s] (mandatory):
!         start of input
!      optional records:
!           METHOD = m, (Default kumar)
!              m = kumar
!           SEQ_COEFFICIENTS (Default 1)
!           SEQ_PRESSURE (Default 1 or pressure)
!           SEQ_REACTION_FORCE (Default 2 or reac_force)
!           SEQ_SOLVE (Default 1)
!           MAX_ITER (Default 10)
!           PROBLEM_NUMBER (Default 1)
!           PRINT_LEVEL (Default 0)
!           P_CAVITY (Default 0)
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     650   First record read by subroutine is not equal to BEARING
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2   / 'SEQ_COEFFICIENTS', 'PROBLEM_NUMBER', &
                     'SEQ_PRESSURE', 'SEQ_SOLVE', 'SEQ_REACTION_FORCE', &
                     'MAX_ITER', 'PRINT_LEVEL', 'METHOD', 'P_CAVITY' /
      data ncode2  /  8, 4, 8, 7, 8,   6, 4, 4, 5 /
      data ncodmx2 / 16,14,12, 9,18,   8,11, 6, 8 /
      data itype2  /  0, 0,38, 0,38,   0, 0, 5, 1 /
      data tabmn2 / 1d0, 1d0, 1d0, 1d0, 1d0,  1d0, 0d0, 1d0, -1d10 /
      data tabmx2 / 1d1, 1d1, 1d2, 1d1, 1d2,  1d3, 3d0, 1d0,  1d10 /
      data mainkeyw / 'BEARING' /

!     code2    Table with main keywords
!             Values:
!             1:  SEQ_COEFFICIENTS
!             2:  PROBLEM_NUMBER
!             3:  SEQ_PRESSURE
!             4:  SEQ_SOLVE
!             5:  SEQ_REACTION_FORCE
!             6:  MAX_ITER
!             7:  PRINT_LEVEL
!             8:  METHOD
!             9:  P_CAVITY
! ======================================================================
!
      call eropen ( 'readbearing' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readbearing'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check sizes of arrays

      call inichkarlen ( 10, niinp, 'iinput' ) ! number of integers
      call inichkarlen ( 1, nrinp, 'rinput' )  ! number of reals
      if ( ierror/=0 ) go to 1000

      if ( isubr==1 ) then

!     --- Call by readbearingxx, read first record  (BEARING)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( mainkeyw, 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

!     --- Read sequence number

      call sepreadseqnr ( iseqnr )

!     --- Read data records

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopytype ( nncode2, itype2, tabmn2, tabmx2 )

      table(1:nncode) = -1d0
      table(1) = 1d0  ! SEQ_COEFFICIENTS
      table(2) = 1d0  ! PROBLEM_NUMBER
      table(4) = 1d0  ! SEQ_SOLVE
      table(6) = 1d1  ! MAX_ITER
      table(7) = 0d0  ! PRINT_LEVEL
      table(8) = 1d0  ! METHOD
      table(9) = 0d0  ! P_CAVITY
      codesub(1) = 'KUMAR'
      ncodesub(1) = 5

      iinput(8:niinp) = 1

      call sepreadsetkey1     ! interpret keywords

!     --- Fill arrays

      iinput(1) = nint(table(1))  ! SEQ_COEFFICIENTS
      iinput(2) = nint(table(2))  ! PROBLEM_NUMBER
      call readseqvector ( table(3), 'pressure', iinput(3), 1 )  ! seq_pressure
      iinput(4) = nint(table(4))  ! SEQ_SOLVE
      call readseqvector ( table(5), 'reac_force', iinput(5), 2 )  ! SEQ_REACTION
      iinput(6) = nint(table(6))  ! MAX_ITER
      iinput(7) = nint(table(7))  ! PRINT_LEVEL

      if ( table(8)>0d0 ) then

!     --- METHOD found, fill iinput(8)

         iinput(8) = nint(table(8))

      end if  ! ( table(8)>0d0 )

      rinput(1) = table(9)  !  P_CAVITY

1000  call erclos ( 'readbearing' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readbearing'

      end if  ! ( debug )

      end
