      subroutine readbi ( iinint, isubr, iseqnr, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 30-04-2014 Extra error message
!        version  3.1  date 09-03-2013 Correction length code
!        version  3.0  date 26-08-2011 Remove tabout from parameter list
!
!   copyright (c) 1994-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for the subroutine INTBND
!     The input is stored in array IININT
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinint(*), isubr, iseqnr, icurvs(*)

!     icurvs         o    Integer array containing the curve numbers for the
!                         computation of the boundary integral. A negative curve
!                         number indicates that the curve must be computed in
!                         the reversed direction.
!                         The curves including their sign must form one
!                         contiguous curve
!                         The number of curves is stored in array iintbn
!                         If the integration must be carried out over surfaces, &
!                         icurvs must contain the surface sequence numbers.
!     iinint         o    The integer input read is stored in array IININT
!                         Array IININT is filled as follows:
!                         1  ICHINT, choice parameter for the type of integral
!                                    Possibible values:
!                                    1          /  f u  ds
!                                    2          /  f u.n  ds
!                                    3          /  f u.t  ds
!                                    4          /  f n. sigma .n  ds
!                                    5          /  f n. sigma .t  ds
!                                    6          /  f n. sigma     ds
!                                    7          /  f u  n  ds
!                                    8          sum f(i) u(i)
!                                    9          /  f ds
!                                    The possibilities 3 to 6 have only been
!                                    implemented for NDIM = 2, i.e.
!                                    two-dimensional regions
!                            Default value: 1
!                         2  ICHFUN,  choice parameter for the computation of f
!                                    Possibible values:
!                                    0   f = 1
!                                    >0  f = func(ichfun, x, y, z) (real case)
!                                        or
!                                        f = cfunc(ichfun, x, y, z) (complex
!                                            case)
!                                        at this moment only the combination f#1
!                                        and ichint=1 is permitted
!                            Default value: 0
!                         3  IRULE, The numerical integration rule to be carried
!                                   out depends on IRULE.
!                                   Possibible values:
!                                    For curves only (i.e. 2D):
!                                    1  trapezoid rule cartesian co-ordinates
!                                    2  trapezoid rule axisymmetric co-ordinates
!                                    3  simpson's rule cartesian co-ordinates
!                                    4  simpson's rule axisymmetric co-ordinates
!                                    For surfaces only (i.e. 3D):
!                                    1  Newton cotes rule, i.e. integration rule
!                                       based on the type of approximation used
!                                       in the element
!                            Default value: 1
!                         4  IDIM    This parameter indicates what type of
!                                    boundary integral must be computed
!                                    Possible values:
!                                    1: curves
!                                    2: surfaces
!                                    3: contact
!                                    4: no_contact
!                                    5: obstacles
!                            Default value: NDIM-1
!                         5  NUMBER  This parameter indicates which unknown
!                                    (physical degree of freedom) must be used
!                                    from the (solution) vector u (IVECTR)
!                                    If ICHINT = 2, 3 or 7, it is supposed that
!                                    the degrees of freedom are equal to NUMBER, &
!                                    NUMBER+1 and NUMBER+2 (3D)
!                                    If ICHINT = 4, 5 or 6, it is supposed that
!                                    the degrees of freedom are equal to NUMBER, &
!                                    NUMBER+1 and NUMBER+2 (2D) or NUMBER to
!                                    NUMBER+5 (3D)
!                            Default value: 1
!                         6  NCURVES Indicates the number of curves or surfaces
!                                    in array ICURVS that must be used for the
!                                    integration
!                                    Default value: 1
!     iseqnr        i/o   Sequence number with respect to the storage in array
!                         ISEPIN
!     isubr          i    Indicates by which subroutine READBI is called
!                         Possible values:
!                         1:  INTBND,  read record with BOUNDARY_INTEGRAL
!                         2:  ERINTB,  do not read record with BOUNDARY_INTEGRAL
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=9, nncod2=3
      integer ncode1(nncode1), ncode2(nncod2), itype1(nncode1), ncodmx1(nncode1)
      integer k,  mcurvs, msurfs, mcont, mnocont, mobst
      double precision :: tabmin1(nncode1), tabmax1(nncode1)
      character (len=17) code1(nncode1), code2(nncod2)

!     code1          Table with keywords corresponding to options
!     code2          Table with the keywords following CURVES and SURFACES
!     itype1         itype array for code1  (See intx02)
!     k              Counting variable
!     mcont          Number of contacts found
!     mcurvs         Number of curves found
!     mnocont        Number of no-contacts found
!     mobst          Number of obstacles found
!     msurfs         Number of surfaces found
!     ncode1         Table containing the number of significant characters in
!                    the keyword table code1
!     ncode2         See ncode, now for table code2
!     ncodmx1        Table containing the number of maximum characters in
!                    the keyword table code1
!     nncod2         Number of entries in table code2
!     nncode1        Number of entries in table code1
!     tabmax1        Table with maximum values of numbers corresponding to code1
!     tabmin1        Table with minimum values of numbers corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     SEPSETMAINEND  Fill keyword END as first main keyword and set nmain to 1
! **********************************************************************
!
!                       I/O
!
!     readbi requires input from the standard input file
!     The following input is recognized:
!     BOUNDARY_INTEGRAL [, SEQUENCE_NUMBER = s]      (mandatory):
!        start of input
!      optional records:
!        ICHINT = s                               (Default 1)
!        ICHFUN = s                               (Default 0)
!        IRULE = s                                (Default 1)
!        CURVES ( C1, C2, ... )                   (Default none)
!        SURFACES ( S1, S2, ... )                 (Default none)
!        OBSTACLES ( O1, O2, ... )                (Default none)
!        DEGREE_OF_FREEDOM = d                    (Default 1)
!        CONTACT i                                (Default 0)
!        NO_CONTACT i                             (Default 0)
!     END   (mandatory):  end of input
!     The keywords CURVES and SURFACES are mutually exclusive, &
!     precisely one of them is necessary
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     650   First record read by subroutine is not equal to BOUNDARY_INTEGRAL
!    1511   Both keywords curves and surfaces found in one input block
!    3183   No definition of where the boundary integral must be computed
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (derivatives)
!    Read rest of first record (sequence number, problem number )
!    Read the rest of the records by intx02 until END is found
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'ICHINT', 'ICHFUN', 'IRULE', 'DEGREE_OF', &
                  'CURVES', 'SURFACES', 'CONTACT', 'NO_CONTACT', &
                  'OBSTACLES' /
      data ncode1 / 4, 4, 3, 2, 4, 4, 4, 7, 4 /
      data ncodmx1/ 6, 6, 5,17, 6, 8, 7,10, 9 /
      data itype1 / 0, 0, 0, 0,18,18, 0, 0,18 /
      data tabmin1 / 1d0, 0d0, 1d0, 1d0, 0d0, 0d0, 1d0, 1d0, 0d0 /
      data tabmax1 / 9d0, 1d3, 4d0, 1d2, 0d0, 0d0,10d0, 10d0, 0d0 /
      data code2 / 'C', 'S', 'O' /
      data ncode2 / 1, 1, 1 /

!     code1   Table with keywords corresponding to options
!             Values:
!             1:  ICHINT
!             2:  ICHFUN
!             3:  IRULE
!             4:  DEGREE_OF_FREEDOM
!             5:  CURVES
!             6:  SURFACES
!             7:  CONTACT
!             8:  NO_CONTACT
!             9:  OBSTACLES
!     code2   Table with the keywords following CURVES and SURFACES
!             Values:
!             1: C
!             2: S
!             3: O
! ======================================================================
!
      call eropen ( 'readbi' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readbi'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iinint(1:6) = 1
      iinint(2) = 0

      if ( isubr==1 ) then

!     --- Call by INTBND, read first record  (BOUNDARY_INTEGRAL)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'BOUNDARY_INTEGRAL', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         else if ( string(1:5)/='BOUND' ) then
            call errchr ( 'BOUNDARY_INTEGRAL', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

      call sepreadseqnr ( iseqnr )    ! Read sequence number

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )
      call sepcopycodesub ( code2, nncod2, ncode2 )
      call sepsetmainend

      table(1:nncode) = -1d20
      tabmin(5) = dble(nout)
      tabmin(6) = dble(nout)
      tabmin(9) = dble(nout)
      tabmax(5) = ncurvs
      tabmax(6) = nsurfs
      tabmax(9) = numobst
      table(5) = -1d0
      table(6) = -2d0
      table(9) = -3d0
      istop = 2
      iretur = 0
      call sepreadsetkeyw
      if ( debug ) then
         call prinrl ( table, nncode, 'table' )
         call prinrl ( tabmax, nncode, 'tabmax' )
      end if  ! ( debug )
      istop = 0
      if ( table(1)>0d0 )  iinint(1) = nint(table(1))
      if ( table(2)>0d0 )  iinint(2) = nint(table(2))
      if ( table(3)>0d0 )  iinint(3) = nint(table(3))
      if ( table(4)>0d0 )  iinint(5) = nint(table(4))

      mcurvs  = 0
      msurfs  = 0
      mcont   = 0
      mnocont = 0
      mobst   = 0
      if ( table(5)>0 ) mcurvs  = nint(tabmax(5))
      if ( table(6)>0 ) msurfs  = nint(tabmax(6))
      if ( table(7)>0 ) mcont   = nint(table(7))
      if ( table(8)>0 ) mnocont = nint(table(8))
      if ( table(9)>0 ) mobst = nint(table(9))

      if ( mcurvs>0 ) then

!     --- Keyword curves found

         if ( msurfs>0 .or. mcont>0 .or. mnocont>0 .or. &
              mobst>0 ) call errsub ( 1511, 0, 0, 0 )
         do k = 1, mcurvs
            icurvs(k) = nint(tabout(k))
         end do
         iinint(4) = 1
         iinint(6) = mcurvs

      else if ( msurfs>0 ) then

!     --- Keyword surfaces found

         if ( mcont>0 .or. mnocont>0 .or. mobst>0 ) &
            call errsub ( 1511, 0, 0, 0 )
         do k = 1, msurfs
            icurvs(k) = nint(tabout(k))
         end do
         iinint(4) = 2
         iinint(6) = msurfs

      else if ( mcont>0 ) then

!     --- Keyword contact found

         if ( mnocont>0 .or. mobst>0 ) call errsub ( 1511, 0, 0, 0 )
         iinint(4) = 3
         iinint(6) = mcont

      else if ( mnocont>0 ) then

!     --- Keyword no_contact found

         if ( mobst>0 ) call errsub ( 1511, 0, 0, 0 )
         iinint(4) = 4
         iinint(6) = mnocont

      else if ( mobst>0 ) then

!     --- Keyword obstacle found

         iinint(4) = 5
         iinint(6) = mobst

      else

!     --- Non of these keywords found

         call errsub ( 3183, 0, 0, 0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinint, 6, 'iinint' )
         write(irefwr,*) 'End readbi'

      end if
      call erclos ( 'readbi' )

      end subroutine readbi
