      subroutine readboolexpr ( inpsep, rinsep, leninpsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 12-02-2015 New call to readexpress
!        version  2.0  date 18-12-2013 Completely rewritten
!        version  1.0  date 06-02-2008
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read boolean expression between opening and closing brackets
!     Store result in array inpsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     boolean
!     boolean_expression
!     record
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), leninpsep, istrin
      double precision rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         Contents
!                         1:  Number of positions used for this expression
!                             length
!                         2:  Position that is filled later on with an
!                             address for jumping in case the expression
!                             is false
!                         Followed by length positions containing the expression
!                         Possible values in these positions
!                         1-9:   Boolean operator, see subroutine readboolop:
!                                1: operator is <
!                                2: operator is <= &
!                                3: operator is >
!                                4: operator is >= &
!                                5: operator is /= &
!                                6: operator is == &
!                                7: operator is AND
!                                8: operator is OR
!                                9: operator is NOT
!                         11:    Open bracket (
!                         12:    Close bracket (
!                         100:   end of expression
!                         <0  Starting address of information of a real
!                             expression according to the rules of
!                             readexpressnew
!                         >1000 and<10000 refers to position in rinsep
!                          +1000 where a real value is stored
!                        >10000 refers to position in scalars+10000, &
!                          where a scalar is stored
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     leninpsep      i    Length of array inpsep
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iopen, operator, icount, i, jcount, iopenbef
      character (len=3) operators(9)
      logical finished, expressfound, operatorfound, openfound, &
              closefound, vecfound

!     closefound     If true a closing bracket has been found, which
!                    corresponds to a closing bracket read earlier
!     expressfound   If true the previous part of the expression is an
!                    expression
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     icount         Length of array inpsep
!     iopen          Contains the number of open openings brackets
!                    Hence not yet closed
!     iopenbef       Value of iopen before call to readexpressnew
!     jcount         Length of array inpsep after the call of readboolcheck
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
!     operator       Code for boolean operator
!                    1:  <
!                    2:  <= &
!                    3:  >
!                    4:  >= &
!                    5:  /= &
!                    6:  == &
!                    7:  and
!                    8:  or
!                    9:  not
!     operatorfound  If true the previous part of the expression is a
!                    boolean operator
!     operators      Character array containing the
!                    logical operators recognized
!     vecfound       If true the expression contains vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     READBOOLCHECK  Check contents of array inpsep for boolean expression
!                    Reorder these contents
!     READBOOLOP     Check if next item is boolean operator and return
!                    with value if it is
!     READEXPRESS    Interprete part of input as number or expression
! **********************************************************************
!
!                       I/O
!
!     The subroutine interprets a boolean expression between brackets
!     and stores this expression into inpsep/rinsep
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2924   First item in boolean expression is not an openings bracket (
!    2926   NOT in a boolean expression is preceded by another operator
!    2927   operator not preceded by expression
!    2928   No closing brackets found
!    2929   Boolean operator found for closing bracket in boolean expression
!    2930   Not an expression found for closing bracket in boolean expression
!    2933   Open bracket ( in boolean expression preceded by number, &
!           expression or operator
!    2937   More opening than closing brackets found in boolean expression
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( first item is not ( ) then
!        error
!     iopen := 1
!     finished := false
!     expressfound := false
!     operatorfound := false
!     while ( not finished ) do
!        if ( next item = operator ) then
!           if ( operator = not ) then
!              if ( operatorfound ) then
!                 error
!              store operator and mark as operator
!              operatorfound := true, expressfound := false
!           else if ( not expressfound ) then
!              error: operator must be preceded by expression
!           else
!              store operator and mark as operator
!              operatorfound := true, expressfound := false
!        else if ( next item = ( ) then
!           store ( in inpsep
!           iopen := iopen+1
!        else if ( next item = ) ) then
!           store ) in inpsep
!           iopen := iopen-1
!           if ( operatorfound or not expressfound ) then
!              error
!        else
!           interpret information as expression and store
!           operatorfound := false, expressfound := true
!        if ( iopen<= 0 ) finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data operators / '<', '<=', '>', '>=', '/=', '==', 'AND', 'OR', &
                       'NOT' /
! ======================================================================
!
      call eropen ( 'readboolexpr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readboolexpr'
         write(irefwr,1) 'lastitem, leninpsep, istrin', &
                          lastitem, leninpsep, istrin
  1      format ( a, 1x, (10i6) )
         call prinin ( typeitems, nitems, 'typeitems' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( typeitems(lastitem)/=5 ) then

!     --- first item is not (
!         error

         call errsub ( 2924, 0, 0, 0 )
         call errcrd
         go to 1000

      end if  ! ( typeitems(lastitem)/=5 )

!     --- iopen := 1
!         finished := false
!         expressfound := false; operatorfound := false

      iopen = 1
      finished = .false.
      expressfound = .false.
      operatorfound = .false.
      icount = 1
      lastitem = lastitem+1  ! we started with a (, now the next item
      if ( debug ) &
         write(irefwr,1) 'icount, lastitem, iopen', &
                          icount, lastitem, iopen

!     --- while ( not finished ) do

      do while ( .not. finished )

         if ( debug ) write(irefwr,1) 'lastitem', lastitem
         if ( lastitem>nitems ) then

!        --- Input exhausted, whereas more input is expected

            call errint ( iopen, 1 )
            call errsub ( 2928, 1, 0, 0 )
            call errcrd
            go to 1000

         end if  ! ( lastitem>nitems )

!        --- Read next item and check if it is a boolean operator

         call readboolop ( operator )
         if ( debug ) then
            write(irefwr,*) 'after readboolop'
            write(irefwr,1) 'operator', operator
            write(irefwr,1) 'lastitem', lastitem
            write(irefwr,1) 'typeitems(lastitem)', typeitems(lastitem)
         end if  ! ( debug )
         if ( ierror/=0 ) go to 1000

         if ( operator>0 ) then

!        --- next item = operator

            if ( operator==9 ) then

!           --- operator = not

               if ( operatorfound ) then

!              --- error: operatorfound, hence not preceded by operator

                  call errsub ( 2926, 0, 0, 0 )
                  call errcrd
                  go to 1000

               else

!              --- store operator and mark as operator
!                  operatorfound := true, expressfound := false

                  icount = icount+1
                  inpsep(icount) = operator   ! operator number stored
                  operatorfound = .true.

               end if  ! ( operatorfound )

            else if ( .not. expressfound ) then

!           --- not expressfound
!               error: operator must be preceded by expression

               call errchr ( operators(operator), 1 )
               call errsub ( 2927, 0, 0, 1 )
               go to 1000

            else

!           --- store operator and mark as operator
!               operatorfound := true, expressfound := false

               icount = icount+1
               inpsep(icount) = operator   ! operator number stored
               operatorfound = .true.
               expressfound = .false.

            end if  ! ( operator==9 )
            if ( debug ) &
               call prininlarge ( inpsep, icount, 'inpsep' )

         else if ( typeitems(lastitem)==5 ) then

!        --- next item = (
!            store ( in inpsep
!            iopen := iopen+1

            icount = icount+1
            inpsep(icount) = 11   ! ( found
            lastitem = lastitem+1
            iopen = iopen+1
            if ( debug ) then
               write(irefwr,*) 'open bracket found'
               write(irefwr,1) 'icount, iopen', icount, iopen
               call prininlarge ( inpsep, icount, 'inpsep' )
            end if  ! ( debug )

            if ( expressfound ) then

!           --- error expression found before opening bracket

               call errsub ( 2933, 0, 0, 0 )
               call errcrd
               go to 1000

            end if  ! ( operatorfound )

         else if ( typeitems(lastitem)==6 ) then

!        --- next item = )
!            store ) in inpsep
!            iopen := iopen-1

            if ( debug ) write(irefwr,*) 'close bracket found'
            if ( inpsep(icount-1)==11 .and. &
                 (inpsep(icount)>1000 .or. inpsep(icount)<0) ) then

!           --- Situation of openings bracket followed by expression
!               followed by closing bracket. Remove brackets

               iopen = iopen-1
               inpsep(icount-1) = inpsep(icount)
               icount = icount-1
               if ( debug ) &
                  write(irefwr,1) 'icount, inpsep(icount), iopen', &
                                   icount, inpsep(icount), iopen

            else

!           --- Closing bracket

               icount = icount+1
               inpsep(icount) = 12   ! ) found
               iopen = iopen-1
               if ( debug ) &
                  write(irefwr,1) 'icount, inpsep(icount), iopen', &
                                   icount, inpsep(icount), iopen
               if ( operatorfound ) then

!              --- error operator found before closing bracket

                  call errsub ( 2929, 0, 0, 0 )
                  call errcrd
                  go to 1000

               end if  ! ( operatorfound )
               if ( .not. expressfound ) then

!              --- error not an expression found before closing bracket

                  call errsub ( 2930, 0, 0, 0 )
                  call errcrd
                  go to 1000

               end if  ! ( expressfound )bracketfound

            end if  ! ( inpsep(icount-1)==11 ....
            lastitem = lastitem+1

            if ( debug ) write(irefwr,1) 'lastitem/close', lastitem

         else

!        --- interpret information as expression and store
!            might also start with an openings bracket, as part of
!            a boolean expression

            if ( debug ) write(irefwr,*) 'expression'
            iopenbef = iopen
            openfound = .false.
            closefound = .false.
            lastitem = lastitem-1  ! is increased in readexpres
            call readexpress ( openfound, closefound, .true., .true., &
                               .false., vecfound )

            if ( debug ) write(irefwr,1) 'ityprd', ityprd

            if ( .not. closefound .and. lastitem<nitems ) then

!           --- not closefound and not end of card
!               Check if next item is )

               if ( typeitems(lastitem+1)==6 ) then

!              --- Lastitem = ), raise lastitem

                  lastitem = lastitem+1

               end if  ! ( typeitems(lastitem+1)==6 )

            end if  ! ( .not. closefound .and. lastitem<nitems )
            if ( ierror/=0 ) go to 1000

            icount = icount+1
            if ( ityprd==3 ) then

!           --- real found

               rinsep(istrin) = realrd
               inpsep(icount) = istrin+1000
               istrin = istrin + 1

            else

!           --- expression found

               inpsep(icount) = nint(realrd)

            end if  ! ( ityprd==3 )

            expressfound = .true.
            operatorfound = .false.

            if ( debug ) then
               write(irefwr,*) 'expression found'
               write(irefwr,*) 'closefound ', closefound
               write(irefwr,1) 'iopenbef, iopen, icount, lastitem', &
                                iopenbef, iopen, icount, lastitem
               call prininlarge ( inpsep, icount, 'inpsep' )
            end if  ! ( debug )

            if ( iopenbef>iopen ) then

!           --- Number of open brackets before was larger
!               This means that the end closing brackets correspond to
!               the boolean expression
!               Remove them from iwork and raise iopen

               do i = 1, iopenbef-iopen

!              --- error: no closing bracket, so incorrect number of
!                  brackets in expression

                     call errsub ( 2937, 0, 0, 0 )
                     call errcrd

               end do  ! i = 1, iopenbef-iopen
               iopen = iopenbef

            end if  ! ( iopenbef>iopen )

         end if  ! ( operator>0 )
         call inichkarlen ( icount, leninpsep, 'inpsep' )

         if ( iopen<=0 ) finished = .true.

         if ( debug ) then
            write(irefwr,*) 'expressfound', expressfound
            write(irefwr,*) 'operatorfound', operatorfound
            write(irefwr,*) 'finished', finished
         end if  ! ( debug )

      end do  ! while ( .not. finished )

!     --- remove last closing bracket

      if ( inpsep(icount)==12 ) icount = icount-1

!     --- Check

      call readboolcheck ( inpsep, operators, icount, jcount )
      if ( debug ) write(irefwr,1) 'jcount', jcount

      icount = jcount
      inpsep(1) = icount-1  ! Fill length of array

1000  if ( debug ) then

!     --- Debug information

         call prininlarge ( inpsep, icount, 'inpsep' )
         write(irefwr,*) 'End readboolexpr'

      end if  ! ( debug )
      call erclos ( 'readboolexpr' )

      end
