      subroutine readc0quad ( iwork, table, rincre, nreal, &
                              inforinpcre, code, ncodmx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 12-04-2012 Extension with kfill = 24
!        version  2.1  date 14-07-2011 Extension with kfill = 22
!        version  2.0  date 13-11-2010 Extra parameters
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information concerning boundary conditions of the type
!     (half)-quadratic into itypebc and rincre
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     read
!     quadratic
!     polynomial
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nreal, inforinpcre(*), iwork(*), ncodmx(*)
      double precision table(*), rincre(*)
      character (len=*) code(*)

!     code           i    Table with keywords
!     inforinpcre    i    Array of length of rincre that contains a 0 if the
!                         number stored in rincre is a real and a reference
!                         to scalars if it is a scalar
!     iwork         i/o   integer work array
!                         At input iwork(2) contains the position where
!                         boundary conditions are given
!                         2: along curves
!                         3: along surfaces
!                         At output iwork(1) contains the type of boundary
!                         condition
!                         Possibilities:
!                         9001:  Fill quadratic boundary condition along curve
!                         9002:  Fill half-quadratic boundary condition along
!                                curve
!                         9003:  Fill quadratic boundary condition with
!                                boundary layer along circular surface
!                         9004:  Fill quadratic boundary condition with
!                                boundary layer and temperature difference
!                                along circular surface
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     nreal          i    Number of interpolated values at the line of
!                         intersection
!     rincre         o    Double precision array containing the parameters
!                         corresponding to the boundary conditions
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision max_val, mean_val, band_width, dif_val
      integer itypebc, i, nread

!     band_width     boundary layer thickness
!     dif_val        difference between maximum value and value at the boundary
!     i              Counting variable
!     itypebc        Indication of how the degrees of freedom must be
!                    filled.
!                    Possibilities:
!                    9001:  Fill quadratic boundary condition along curve
!                    9002:  Fill half-quadratic boundary condition along
!                           curve
!                    9003:  Fill quadratic boundary condition with
!                           boundary layer along circular surface
!                    9004:  Fill quadratic boundary condition with
!                           boundary layer and temperature difference
!                           along circular surface
!     max_val        maximum value of the boundary condition
!     mean_val       mean value of the boundary condition
!     nread          Number of items read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1068   A quadratic profile may only be described along curves
!    2637   A quadratic function with given mean value has only been implemented
!           for surfaces.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readc0quad' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readc0quad'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Extract constants from table

      max_val = table(22)
      mean_val = table(34)
      band_width = table(35)
      dif_val = table(36)
      if ( table(21)>0d0 ) then

!     --- Quadratic found, set itypebc

         if ( max_val>-1d19 .or. mean_val<-1d19 ) then

!        --- max_val is given

            if ( table(25)>0d0 ) &
               call errkeywexcl ( code, ncodmx, 21, 25 )
            itypebc = 9001
            if ( max_val<-1d19 ) max_val = 1d0
            rincre(nreal+1) = max_val
            nread = 1

!           --- Check if only max_val is given

            if ( mean_val>-1d19 ) &
               call errkeywexcl ( code, ncodmx, 21, 34 )
            if ( band_width>-1d19 ) &
               call errkeywexcl ( code, ncodmx, 21, 35 )
            if ( dif_val>-1d19 ) &
               call errkeywexcl ( code, ncodmx, 21, 36 )

         else if ( mean_val>-1d19 ) then

!        --- mean_val is given

            if ( band_width<-1d19 ) band_width = 0d0
            rincre(nreal+1) = mean_val
            rincre(nreal+2) = band_width
            if ( dif_val>-1d19 ) then

!           --- DIFFERENCE found, fill values and set itypebc

               rincre(nreal+3) = dif_val
               nread = 3
               itypebc = 9004

            else

!           --- DIFFERENCE not found, set itypebc

               nread = 2
               itypebc = 9003

            end if  ! ( dif_val>-1d19 )

         end if  ! ( max_val>-1d19 )

      else if ( table(25)>0d0 ) then

!     --- Halfquadratic found, set itypebc

         itypebc = 9002
         nread = 1
         if ( max_val<-1d19 ) max_val = 1d0
         rincre(nreal+1) = max_val

!        --- Check if only max_val is given

         if ( mean_val>-1d19 ) &
            call errkeywexcl ( code, ncodmx, 25, 34 )
         if ( band_width>-1d19 ) &
            call errkeywexcl (  code, ncodmx, 25, 35 )
         if ( dif_val>-1d19 ) &
            call errkeywexcl (  code, ncodmx, 25, 36 )

      end if  ! ( table(21)>0d0 )

      if ( itypebc<9003 ) then

!     --- Standard case 9001 or 9002, only curves allowed

         if ( iwork(2)/=2 .and. iwork(2)/=22 ) then
            call errsub ( 1068, 0, 0, 0 )
            call errcrd
         end if

      else

!     --- Special case 9003 or 9004, only surfaces allowed

         if ( iwork(2)/=3 .and. iwork(2)/=24 ) then
            call errsub ( 2637, 0, 0, 0 )
            call errcrd
         end if

      end if  ! ( itypebc<9003 )

!     --- Set inforinpcre 0 for the moment, may be changed later on

      do i = 1, nread
         inforinpcre(nreal+i) = 0
      end do
      iwork(1) = itypebc

1000  call erclos ( 'readc0quad' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'itypebc ', itypebc
         write(irefwr,*) 'max_val, mean_val, band_width, dif_val ', &
                          max_val, mean_val, band_width, dif_val
         write(irefwr,*) 'End readc0quad'

      end if

      end
