      subroutine readeigv ( iinput, rinput, isubr, iseqnr, niinp, nrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-04-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  Read the input for the eigenvalue subroutine eigenval
!  The input is stored in the arrays iinput and rinput
!  The input must be enclosed between EIGENVALUES and END.
! **********************************************************************
!
!                       KEYWORDS
!
!     loop
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niinp, nrinp, iinput(*), iseqnr, isubr
      double precision rinput(*)

!    iinput  o   The integer input read is stored in array IINPUT
!                This array must be filled as follows:
!                 1:  IEIGVEC   If 1 the eigenvectors must be computed, &
!                               if 0 not
!                 2:  IPRINT    Defines the amount of output produced
!                               Possible values:
!                               0:  No information about the iteration
!                                   process is printed except in the case
!                                   of an error
!                               1:  The number of iterations performed is
!                                   printed
!                               2:  Some extra information about the iteration
!                                   process is printed for each iteration
!                 3:  ISEQ_COEF Defines the sequence number of the
!                               input for the coefficients
!                 4:  INITIAL   Defines how the initial vector must be filled
!                               0: ev(i) = i
!                               >0: random, the value is used to dfine the
!                                   random values
!                 5:  IMAS      If 0 the mass matrix is diagonal, if 1 it is
!                               a profile matrix
!                 6:  MAXITER   Maximum number of iterations
!                               ( when 0 a default value is used )
!                 7:  MAXINTV   Maximum number of e.v. intervals
!                               ( when 0 a default value is used )
!                 8:  MAXPNT    Maximum number of points x(*) used
!                               in P+R process to catch the eigenvalues
!                               ( when 0 a default value is used )
!                 9:  NEIGV     Number of eigenvalues to be computed
!                10:  IUSEUSR   This parameter indicates if the arrays IUSER
!                               and USER from the parameter list are used (1)
!                               or if the parameter ISEQFILCOF is used (0)
!                11:  NUMVEC    Maximum number of vectors stored in isol
!                12:  IVECTOR   First vector sequence number for the
!                               storage of the eigenvectors
!    iseqnr i/o  Sequence number with respect to the storage in array ISEPIN
!    isubr   i   Indicates by which subroutine readeigv is called
!                Possible values:
!                1:  ...      read record with EIGENVALUES
!                2:  EREIGV,  do not read record with EIGENVALUES
!    niinp  i/o  Length of integer input array
!                At input available length, at output computed length
!    nrinp  i/o  Length of real input array
!                At input available length, at output computed length
!    rinput  o   Contains the real input for the eigenvalue subroutine
!                At this moment he input is defined as follows
!                1:  accuracy eps
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=11, nncod2=4
      integer ncode1(nncode1), ncode2(nncod2), itype1(nncode1), ncodmx1(nncode1)
      integer k
      double precision :: tabmin1(nncode1), tabmax1(nncode1)
      character (len=21) :: code1(nncode1), code2(nncod2)

!     code1      Table with keywords corresponding to options
!     code2      Table with the keywords following the subkeywords
!     itype1     itype array for code1 (See intx02)
!     k          Counting variable
!     ncode1     See ncode, now for table code1
!     ncode2     See ncode1, now for table code2
!     ncodmx1    Table containing the number of maximum characters in
!                the keyword table code1
!     nncod2     Number of entries in table code2
!     nncode1    Number of entries in table code1
!     tabmax1    Table with maximum values of numbers corresponding to code1
!     tabmin1    Table with minimum values of numbers corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     SEPSETMAINEND  Fill keyword END as first main keyword and set nmain to 1
! **********************************************************************
!
!                       I/O
!
!     readeigv requires input from the standard input file
!     The following input is recognized:
!     EIGENVALUES [, SEQUENCE_NUMBER = s] (mandatory):
!         start of input
!      optional records:
!          eigenvectors            (Default no eigenvectors)
!          number_of_eigenvalues   (Default 1)
!          accuracy = eps          (Default 0)
!          print_level = p         (Default 0)
!          maxiter = m             (Default 0)
!          maxinterval = m         (Default 0)
!          maxpoints = m           (Default 0)
!          seq_coef = m            (Default 1)
!          mass_matrix = diagonal  (Default diagonal)
!                        consistent
!          start_vector = structured  (Default structured)
!                         random
!          randvalue = i           (Default 1)
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     329   Array to store the input too small
!     650   First record read by subroutine is not equal to EIGENVALUES
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize arrays and parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (EIGENVALUES)
!    Read rest of first record (sequence number)
!    Read the rest of the records by intx02 until END is found
!    set niinp and nrinp to final values
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'EIGENVECTORS', 'PRINT_LEVEL', 'SEQ_COEF', &
                  'START_VECTOR', 'MASS_MATRIX', 'MAXITER', &
                  'MAXINTERVAL', 'MAXPOINTS', 'NUMBER_OF_EIGENVALUES', &
                  'ACCURACY', 'RANDVALUE' /
      data ncode1  / 8, 3, 7, 5, 4, 5, 6, 6, 6, 3, 4 /
      data ncodmx1 /12,11, 8,12,11, 7,11, 9,21, 8, 9 /
      data itype1  / 2, 0, 0, 5, 5, 0, 0, 0, 0, 1, 0 /
      data tabmin1 / 0d0, 0d0, 1d0, 3d0, 1d0, 1d0, 1d0, 1d0, 1d0, 1d-15, &
                    1d0 /
      data tabmax1 / 0d0, 2d0, 1d1, 4d0, 2d0, 1d4, 1d2, 1d2, 1d3, 1d0, &
                    1d2 /
      data code2 / 'DIAGONAL', 'CONSISTENT', 'STRUCTURED', &
                   'RANDOM' /
      data ncode2 / 4, 4, 5, 6 /

!     code1    Table with keywords corresponding to options
!             Values:
!             1:  EIGENVECTORS
!             2:  PRINT_LEVEL
!             3:  SEQ_COEF
!             4:  START_VECTOR
!             5:  MASS_MATRIX
!             6:  MAXITER
!             7:  MAXINTERVAL
!             8:  MAXPOINTS
!             9:  NUMBER_OF_EIGENVALUES
!            10:  ACCURACY
!            11:  RANDVALUE
!     code2   Table with the keywords following the subkeywords
!             Values:
!             1:  DIAGONAL
!             2:  CONSISTENT
!             3:  STRUCTURED
!             4:  RANDOM
! ======================================================================
!
!     --- Initialize arrays and parameters

      call eropen ( 'readeigv' )

      if ( niinp<12 ) then

!     --- niinp too small

         call errint ( 12, 1 )
         call errint ( niinp, 2 )
         call errsub ( 329, 2, 0, 0 )
         go to 1000

      end if
      if ( nrinp<1 ) then

!     --- nrinp too small

         call errint ( 1, 1 )
         call errint ( nrinp, 2 )
         call errsub ( 329, 2, 0, 0 )

      end if
      iinput(1:niinp) = 0
      iinput(3) = 1
      iinput(9) = 1
      iinput(12) = 1
      rinput(1) = 1d-5

      if ( isubr==1 ) then

!     --- Call by xxxxx, read first record  (EIGENVALUES)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'EIGENVALUES', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         else if ( string(1:4)/='EIGV' ) then
            call errchr ( 'EIGENVALUES', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

      call sepreadseqnr (iseqnr  )   ! read sequence number
      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 ) ! copy code1 in code
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )
      call sepcopycodesub ( code2, nncod2, ncode2 )
      call sepsetmainend                  ! Fill END as main keyword

      table(1:nncode) = -1d20
      istop = 2
      iretur = 0
      call sepreadsetkeyw
      istop = 0
      if ( table(1)>0d0 ) iinput(1) = nint(table(1))
      if ( table(2)>0d0 ) iinput(4) = nint(table(2))
      if ( table(3)>0d0 ) iinput(2) = nint(table(3))
      if ( table(4)>0d0 ) rinput(1) = table(4)
      if ( table(5)>0d0 ) iinput(5) = nint(table(5))-1
      if ( table(6)>0d0 ) iinput(3) = nint(table(6))-1
      if ( table(7)>0d0 ) iinput(6) = nint(table(7))
      if ( table(8)>0d0 ) iinput(7) = nint(table(8))-1

1000  call erclos ( 'readeigv' )
      end subroutine readeigv
