      subroutine readenthalpy ( iinput, rinput, isubr, iseqnr, niinp, &
                                nrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 07-03-2012 New call to readseqvector
!        version  1.4  date 29-07-2010 delta_tmelt
!        version  1.3  date 17-05-2010 Default value refine
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for the subroutine prenthalpy
!     The input is stored in the arrays iinput and rinput
!     The input must be enclosed between ENTHALPY_INTEGRATION and END.
! **********************************************************************
!
!                       KEYWORDS
!
!     enthalpy
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulemushy
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niinp, nrinp, iinput(*), iseqnr, isubr
      double precision rinput(*)

!    iinput  o   The integer input read is stored in array IINPUT
!                This array must be filled as follows:
!                 1: Largest entry that has been filled by the user
!                    Default value: 8
!                 2: iseqenth:  sequence number of enthalpy vector in array isol
!                    Default value: 1 or vector with name enthalpy
!                 3: iseqtemp:  sequence number of temperature vector in array
!                               isol
!                    Default value: 2 or vector with name temperature
!                 4: iseqtime: sequence number of the time integration input
!                    Default value: 1
!                 5: iseqbc: sequence number of boundary conditions for
!                    temperature
!                    Default value: 1
!                 6: method_sol: type of solution method
!                    1:  overrelaxation
!                    2:  newton
!                    Default value: 1
!                 7: iseq_coef: sequence number for coefficients
!                    Default value: 1
!                 8: IPRINT: print level
!                    Default value: 0
!                 9: iseq_refine: sequence number for refine input
!                    Default value: 1
!    iseqnr i/o  Sequence number with respect to the storage in array ISEPIN
!    isubr   i   Indicates by which subroutine readenthalpy is called
!                Possible values:
!                1:  ...      read record with
!                2:  ERENTHALPY,  do not read record with ENTHALPY_INTEGRATION
!    niinp   i   Length of integer input array
!    nrinp   i   Length of real input array
!    rinput  o   Contains the real input for the enthalpy subroutine
!                At this moment he input is defined as follows
!                1: kappa_s
!                2: kappa_l
!                3: latent_heat L
!                4: capacity_s
!                5: capacity_l
!                6: rho
!                7: melt_temp
!                8: mu  rho min (c_s,c_l)  (ad-hoc)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=16
      integer ncode2(nncode2), itype2(nncode2), ncodmx2(nncode2)
      double precision :: tabmn2(nncode2), tabmx2(nncode2)
      character (len=23) code2(nncode2)

!     code2          Table with keywords
!     itype2         Table containing information about the numbers behind
!                    the keywords in the table code2
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     nncode2        Number of entries in code2
!     tabmn2         Contains minimum values allowed in table
!     tabmx2         Contains maximum values allowed in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision getconst

!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETCONST       Get the value of a constant in the input block constants
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READSEQVECTOR  Put the value of the sequence number read in iinput
!     READVALCONS    Put the value of the parameter read in rinput
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEY1 Interpret a record containing keywords followed by data
!                    Set END as sole main keyword
! **********************************************************************
!
!                       I/O
!
!     readenthalpy requires input from the standard input file
!     The following input is recognized:
!     ENTHALPY_INTEGRATION [, SEQUENCE_NUMBER = s] (mandatory):
!         start of input
!      optional records:
!          seq_enthalpy = i             (Default 1)
!          seq_temperature = i          (Default 2)
!          seq_time_integration = i     (Default 1)
!          seq_boundary_conditions = i  (Default 1)
!          seq_coefficients = i         (Default 1)
!          seq_refine = i               (Default 1)
!          kappa_s = k                  (Default 1)
!          kappa_l = k                  (Default kappa_s)
!          latent_heat = k              (Default 0)
!          capacity_s = k               (Default 1)
!          capacity_l = k               (Default capacity_s)
!          rho = k                      (Default 1)
!          delta_tmelt = k              (Default 0)
!          print_level = k              (Default 0)
!          solution_method = k          (Default overrelaxation)
!               with k = overrelaxation
!                        nedjar
!                        newton
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     329   Array to store the input too small
!     650   First record read by subroutine is not equal to ENTHALPY_INTEGRATION
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize arrays and parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (ENTHALPY_INTEGRATION)
!    Read rest of first record (sequence number)
!    Read the rest of the records by intx02 until END is found
!    set niinp and nrinp to final values
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2   / 'SEQ_ENTHALPY', 'SEQ_TEMPERATURE', &
                    'SEQ_TIME_INTEGRATION', 'SEQ_BOUNDARY_CONDITIONS', &
                    'KAPPA_S', 'KAPPA_L', 'LATENT_HEAT', 'CAPACITY_S', &
                    'CAPACITY_L', 'RHO', 'MELT_TEMP', &
                    'SOLUTION_METHOD', 'SEQ_COEFFICIENTS', &
                    'PRINT_LEVEL', 'SEQ_REFINE', 'DELTA_TMELT' /
      data ncode2  / 8, 8, 8, 8, 7,   7, 5,10,10, 3, &
                    4, 4, 8, 5, 7,   7 /
      data ncodmx2 /12,15,20,23, 7,   7,11,10,10, 3, &
                    9,15,16,11,10,  11 /
      data itype2  / 0, 0, 0, 0, 1,   1, 1, 1, 1, 1, &
                    1, 5, 0, 0, 1, 1 /
      data tabmn2 / 1d0, 1d0, 1d0, 1d0,1d-8, 1d-8, 1d0,1d-8,1d-8,1d-4, &
                    -1d2,1d0, 1d0, 0d0, 1d0, 0d0  /
      data tabmx2 / 1d2, 1d2, 1d2, 1d2, 1d8,  1d8,1d10, 1d8, 1d8, 1d4, &
                    1d4, 3d0, 1d2, 3d0, 1d1, 1d2 /

!     code2    Table with keywords corresponding to options
!             Values:
!             1:  SEQ_ENTHALPY
!             2:  SEQ_TEMPERATURE
!             3:  SEQ_TIME_INTEGRATION
!             4:  SEQ_BOUNDARY_CONDITIONS
!             5:  KAPPA_S
!             6:  KAPPA_L
!             7:  LATENT_HEAT
!             8:  CAPACITY_S
!             9:  CAPACITY_L
!            10:  RHO
!            11:  MELT_TEMP
!            12:  SOLUTION_METHOD
!            13:  SEQ_COEFFICIENTS
!            14:  PRINT_LEVEL
!            15:  SEQ_REFINE
!            16:  DELTA_TMELT
! ======================================================================
!
      call eropen ( 'readenthalpy' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readenthalpy'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize arrays and parameters

      if ( niinp<nncode2-7 ) then

!     --- niinp too small

         call errint ( nncode-6, 1 )
         call errint ( niinp, 2 )
         call errsub ( 329, 2, 0, 0 )
         go to 1000

      end if
      if ( nrinp<8 ) then

!     --- nrinp too small

         call errint ( 2, 1 )
         call errint ( nrinp, 2 )
         call errsub ( 329, 2, 0, 0 )

      end if
      iinput(1) = 9
      iinput(2:niinp) = 1
      iinput(3) = 2  ! default temperature

      if ( isubr==1 ) then

!     --- Call by readenthalpyxx, read first record  (ENTHALPY_INTEGRATION)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'ENTHALPY_INTEGRATION', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         else if ( string(1:8)/='ENTHALPY' ) then
            call errchr ( 'ENTHALPY_INTEGRATION', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

!     --- Read sequence number

      call sepreadseqnr ( iseqnr )

!     --- Read rest of record

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopytype ( nncode2, itype2, tabmn2, tabmx2 )
      codesub(1) = 'OVERRELAXATION'
      codesub(2) = 'NEDJAR'
      codesub(3) = 'NEWTON'
      ncodesub(1) = 4
      ncodesub(2:3) = 5

      table(1:nncode) = -1d20
      table(14) = 0d0
      table(15) = 1d0

      call sepreadsetkey1     ! interpret keywords

      if ( table(3)>0d0 ) iinput(4) = nint(table(3)) ! seq_time_integration
      if ( table(4)>0d0 ) iinput(5) = nint(table(4)) ! seq_boundary_condition

!     --- Set solution method

      if ( table(12)>0d0 ) iinput(6) = nint(table(12))

!     --- Sequence number coefficients

      if ( table(13)>0d0 ) iinput(7) = nint(table(13))
      iinput(8) = nint(table(14))   ! print level
      iinput(9) = nint(table(15))   ! seq_refine

!     --- Set vector sequence numbers
!         enthalpy vector only in case of overrelaxation or Nedjar

      if ( iinput(6)<3 ) &
          call readseqvector ( table(1), 'enthalpy', iinput(2), 1 )
      call readseqvector ( table(2), 'temperature', iinput(3), 2 )
      if ( iinput(6)==1 ) then

!     --- solution method = 1, Set physical parameters

         call readvalcons ( table(5), 'kappa_s', rinput(1) )
         call readvalcons ( table(6), 'kappa_l', rinput(2) )
         call readvalcons ( table(7), 'latent_heat', rinput(3) )
         call readvalcons ( table(8), 'capacity_s', rinput(4) )
         call readvalcons ( table(9), 'capacity_l', rinput(5) )
         call readvalcons ( table(10), 'rho', rinput(6) )
         call readvalcons ( table(11), 'melt_temp', rinput(7) )
         rinput(8) = rinput(6)*min(rinput(4),rinput(5))

      else if ( iinput(6)==3 ) then

         call readvalcons ( table(11), 'melt_temp', rinput(7) )
         if ( table(16)>0d0 ) then
            tdelta = table(16)
         else
            tdelta = getconst('delta_tmelt')
         end if  ! ( table(16)>0d0 )
         if ( tdelta>0d0 ) mushy = .true.

      end if  ! ( iinput(6)==1 )

1000  call erclos ( 'readenthalpy' )
      if ( debug ) then

!     --- Debug information

         call prinin ( iinput, niinp, 'iinput' )
         call prinrl ( rinput, nrinp, 'rinput' )
         write(irefwr,*) 'End readenthalpy'

      end if  ! ( debug )

      end
