      subroutine readmeshextra ( nelgrp, inpsep, leninpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read input found following the keyword EXTRA_MESH_INPUT and store
!     in array inpsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelgrp, inpsep(*), leninpsep

!     inpsep         o    The integer input read is stored in array INPSEP
!                         Organization of inpsep:
!                         Pos. 1 number of main keywords (presently 2)
!                         Pos. 2 nseq1: Number of input parts defining
!                                outer boundaries
!                         Pos. 3 istart1: Starting address part outer boundaries
!                         Pos. 4 nseq2: Number of input parts defining
!                                subdivisions
!                         Pos. 5 istart2: Starting address part subdivisions
!                         From istart1
!                         Pos. 1-nseq1: starting addresses for each
!                              outer boundary
!                         From these starting addresses:
!                         - Number of element groups used for this group
!                           followed by element group sequence numbers
!                         From istart2
!                         Pos. 1-nseq2: starting addresses for each
!                              subdivision
!                         From these starting addresses:
!                         - Type of subdivision (1=nodes_on_outer_boundary)
!                         - Sequence number outer boundary
!     leninpsep     i/o   Length of array inpsep
!                         At input: declared length
!                         At output: computed length
!     nelgrp         i    Number of element groups
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=3
      integer :: ncode2(nncode2), ncodmx2(nncode2), ireadseq1(nelgrp), &
              iinput1(leninpsep), istart1, iinput2(leninpsep), &
              istart2, ireadseq2(0:nelgrp), i, iseq, numgroups
      character (len=20) code2(nncode2)
      logical ::  finished

!     code2         Table with main keywords
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     iinput1        Contains the input read with respect to
!                    store_outer_boundary in the sequence as given by the user
!                    The storage is identical to inpsep from position istart1
!     iinput2        Contains the input read with respect to
!                    subdivide elements in the sequence as given by the user
!                    The storage is identical to inpsep from position istart2
!     ireadseq1      Contains starting addresses with respect to iinput1
!                    for each element group
!     ireadseq2      Array of size nelgrp+1 indicating if sequence numbers
!                    are free (0) or already in use (>0)
!                    ireadseq2(0) refers to the complete outer boundary
!     iseq           Sequence number
!     istart1        First position used at a part
!     istart2        First position used at a part
!     ncode2         Table containing the number of significant characters
!                    in the main keyword table code2, including those of
!                    the extra texts
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code
!     nncode2        Number of entries in code2
!     numgroups      Number of element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READMESHEXTRA1 Rearrange input as stored in iinput1 and iinput2 and put
!                    into array inpsep
!     READSTOUTER    Read the input following the keyword
!                    STORE_OUTER_BOUNDARY
!     READSUBELMS    Read the input following the keyword
!                    SUBDIVIDE_ELEMENT
!     SEPCOPYCODE    Copy code2 and related arrays into code
! **********************************************************************
!
!                       I/O
!
!     The following records are recognized
!     EXTRA_MESH_INPUT  (may have been read before)
!        store_outer_boundary iseq, elgrps = (i1, i2, ... ) (repeatedly)
!        subdivide_elements nodes_on_outer_boundary iseq (repeatedly)
!     END
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!     648   string could not be recognized
!    2492   The record read does not start with a keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( last record read is not EXTRA_MESH_INPUT ) then
!        read record and check if it is EXTRA_MESH_INPUT
!     While ( last record # END ) do
!        read next record and store input
!     Rearrange input
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'STORE_OUTER_BOUNDARY', 'SUBDIVIDE_ELEMENTS', &
                    'END' /
      data ncode2 /  9, 12, 3 /
      data ncodmx2 / 20, 18, 3 /
! ======================================================================
!
      call eropen ( 'readmeshextra' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readmeshextra'
         write(irefwr,*) 'ityprd ', ityprd
         write(irefwr,*) 'string ', string

      end if
      if ( ierror.ne.0 ) go to 1000

      if ( ityprd.ne.4 .or. string(1:8).ne.'EXTRA_ME' ) then

!     --- last record read is not EXTRA_MESH_INPUT
!         read record and check if it is EXTRA_MESH_INPUT

         ityprd = 1
         call asknxt
         if ( ityprd.ne.4 ) then

!        --- Not a keyword found

            call  errchr ( 'EXTRA_MESH_INPUT', 1 )
            call errsub ( 2492, 0, 0, 1 )
            go to 1000

         else if ( string(1:8).ne.'EXTRA_ME' ) then

!        --- wrong keyword found

            call errchr ( string(1:nchar), 1 )
            call errchr ( 'EXTRA_MESH_INPUT', 2 )
            call errsub ( 648, 0, 0, 2 )
            go to 1000

         end if

      else

!     --- Last record is EXTRA_MESH_INPUT

         ityprd = 1

      end if  ! ( ityprd.ne.4 ) then

!     --- Clear array ireadseq

      do i = 1, nelgrp
         ireadseq1(i) = 0
         ireadseq2(i) = 0
      end do
      ireadseq2(0) = 0

      finished = .false.
      istart1 = 1
      istart2 = 1

!     --- While ( last record # END ) do

      do while ( .not. finished )

!     --- read next record and store input

         call asknxt

         ierpri = 1
         call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
         icode = sepsubkeyw()
         if ( ierror.ne.0 ) go to 1000

!        --- Perform action as function of icode

         select case (icode)

            case(:0)

!           --- Error found, jump

               go to 1000

            case(1)

!           --- STORE_OUTER_BOUNDARY found

               call readstouter ( nelgrp, iinput1(istart1), ireadseq1 )
               if ( ierror.ne.0 ) go to 1000

               if ( istart1.gt.leninpsep ) then

!              --- Array length of array iinput1 too small

                  call errint ( istart1, 1 )
                  call errint ( leninpsep, 2 )
                  call errchr ( 'iinput1', 1 )
                  call errsub ( 550, 2, 0, 1 )
                  go to 1000

               end if
               iseq = iinput1(istart1)
               if ( iseq.gt.0 ) then

!              --- iseq > 0

                  numgroups = iinput1(istart1+1)
                  ireadseq1(iseq) = istart1
                  istart1 = istart1+2+numgroups

               end if  ! ( iseq.gt.0 )

            case(2)

!           --- SUBDIVIDE_ELEMENT found

               call readsubelms ( nelgrp, iinput2(istart2), ireadseq2 )
               if ( ierror.ne.0 ) go to 1000

               if ( istart2.gt.leninpsep ) then

!              --- Array length of array iinput2 too small

                  call errint ( istart2, 1 )
                  call errint ( leninpsep, 2 )
                  call errchr ( 'iinput2', 1 )
                  call errsub ( 550, 2, 0, 1 )
                  go to 1000

               end if
               iseq = iinput2(istart2+1)
               if ( iseq.ge.0 ) then

!              --- iseq >= 0

                  ireadseq2(iseq) = istart2
                  istart2 = istart2+2

               end if  ! ( iseq.gt.0 )

            case(3)

!           --- END found, finished

               finished = .true.

         end select  ! case (icode)

      end do  ! while ( .not. finished )

      if ( debug ) then

!     --- Debug information

         call prinin ( ireadseq1, nelgrp, 'ireadseq1' )
         call prinin ( ireadseq2(0), nelgrp+1, 'ireadseq2' )
         if ( istart1.gt.1 ) &
            call prinin ( iinput1, istart1-1, 'iinput1' )
         if ( istart2.gt.1 ) &
            call prinin ( iinput2, istart2-1, 'iinput2' )

      end if  ! ( debug )

!     --- Rearrange input

      call readmeshextra1 ( inpsep, leninpsep, iinput1, ireadseq1, &
                            iinput2, ireadseq2, nelgrp )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep, leninpsep, 'inpsep' )
         write(irefwr,*) 'End readmeshextra'

      end if
      call erclos ( 'readmeshextra' )

      end
