      subroutine readnavstokes ( iinput, rinput, isubr, iseqnr, niinp, &
                                 nrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 07-03-2012 New call to readseqvector
!        version  1.1  date 06-03-2006 Allow larger values of iseqvel
!        version  1.0  date 30-11-2005
!
!   copyright (c) 2005-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for navier-stokes
!     The input is stored in array iinput
!     The input must be enclosed between NAVIER_STOKES and END.
! **********************************************************************
!
!                       KEYWORDS
!
!     navier_stokes_equation
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niinp, iinput(*), iseqnr, isubr, nrinp
      double precision rinput(*)

!     iinput         o    The integer input read is stored in array IINPUT
!                         This array must be filled as follows:
!                          1: ichoice, type of solution method
!                             1:  standard
!                             2:  pressure correction
!                             3:  standard time integration, convection through
!                                 explicit substepping
!                             Default value: 1
!                          2: ireadtm: sequence number of input for
!                                      time integration
!                             Default value: 1
!                          3: ivectr, sequence number of velocity vector
!                             Default value: 1 or if available %velocity
!                          4: nsubsteps, number of substeps, see iinput(1)
!                             Default value: 1
!                          5: ireadprescor, sequence number of input for
!                                           pressure correction
!                             Default value: 1
!                          6-10: Not yet used
!                             Default value: 0
!     iseqnr        i/o   Sequence number with respect to the storage in
!                         array ISEPIN
!     isubr          i    Indicates by which subroutine readpresscor is called
!                         Possible values:
!                         1:  ...      read record with
!                         2:  ERNAVSTOKES,  do not read record with
!                                           NAVIER_STOKES
!     niinp          i    Length of integer input array
!     nrinp          i    Length of real input array
!     rinput         o    Real input array
!                         At this moment not actually used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=5
      integer ncode2(nncode2), itype2(nncode2), ncodmx2(nncode2)
      integer k
      double precision :: tabmn2(nncode2), tabmx2(nncode2)
      character (len=23) code2(nncode2)
      character (len=13) mainkeyw

!     code2          Table with keywords
!     itype2         Table containing information about the numbers behind
!                    the keywords in the table code2
!     k              Counting variable
!     mainkeyw       Name of main keyword for this block
!                    Only used for error messages
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     nncode2        Number of entries in code2
!     tabmn2         Contains minimum values allowed in table
!     tabmx2         Contains maximum values allowed in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINRL         Print 1d real vector
!     READSEQVECTOR  Put the value of the sequence number read in iinput
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEY1 Interpret a record containing keywords followed by data
!                    Set END as sole main keyword
! **********************************************************************
!
!                       I/O
!
!     readnavstokes requires input from the standard input file
!     The following input is recognized:
!     NAVIER_STOKES [ SEQUENCE_NUMBER = s] (mandatory):
!         start of input
!      optional records:
!           METHOD = m, (Default standard)
!              m = standard
!                  pressure_correction
!                  convection_substepping
!           SEQ_TIME_INTEGRATION (Default 1)
!           SEQ_VELOCITY (Default 1 or %velocity)
!           SEQ_PRESSURE_CORRECTION (Default 1)
!           NUMBER_OF_SUBSTEPS (Default 1)
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     650   First record read by subroutine is not equal to NAVIER_STOKES
!    1588   Two keywords are mutually exclusive
!    2789   keyword is ignored
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize arrays and parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (PRESSURE_CORRECTION)
!    Read rest of first record (sequence number)
!    Read the rest of the records by intx02 until END is found
!    set niinp and nrinp to final values
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2   / 'METHOD', 'SEQ_TIME_INTEGRATION', 'SEQ_VELOCITY', &
                    'NUMBER_OF_SUBSTEPS', 'SEQ_PRESSURE_CORRECTION' /
      data ncode2  /  4, 7, 7,12,15 /
      data ncodmx2 /  6,20,12,18,23 /
      data itype2  /  5, 0, 0, 0, 0 /
      data tabmn2 / 1d0, 1d0, 1d0, 1d0, 1d0 /
      data tabmx2 / 3d0, 1d1, 1d3, 1d2, 1d1 /
      data mainkeyw / 'NAVIER_STOKES' /

!     code2    Table with keywords corresponding to options
!             Values:
!              1:  METHOD
!              2:  SEQ_TIME_INTEGRATION
!              3:  SEQ_VELOCITY
!              4:  NUMBER_OF_SUBSTEPS
!              5:  SEQ_PRESSURE_CORRECTION
! ======================================================================
!
      call eropen ( 'readnavstokes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readnavstokes'

      end if
      if ( ierror/=0 ) go to 1000

      call inichkarlen ( nncode2, niinp, 'iinput' )
      call inichkarlen ( 1, nrinp, 'rinput' )
      if ( ierror/=0 ) go to 1000

!     --- fill iinput with default values

      rinput(1) = 0d0

      if ( isubr==1 ) then

!     --- Call by readnavstokesxx, read first record  (NAVIER_STOKES)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( mainkeyw, 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

!     --- Read sequence number

      call sepreadseqnr ( iseqnr )

!     --- Read rest of record

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopytype ( nncode2, itype2, tabmn2, tabmx2 )

      table(1:nncode) = -1d0

      codesub(1) = 'STANDARD'
      codesub(2) = 'PRESSURE_CORRECTION'
      codesub(3) = 'CONVECTION_SUBSTEPPING'
      ncodesub(1:3) = 4

      table(1:nncode) = -1d0
      iinput(1:niinp) = 1

      call sepreadsetkey1     ! interpret keywords
      if ( debug ) call prinrl ( table, nncode, 'table' )

      if ( table(1)>0d0 ) then

!     --- METHOD found, fill iinput(1)

         iinput(1) = nint(table(1))
         do k = 4, 5

            if ( table(k)>0 .and. iinput(1)==1 ) then

!           --- keywords mutually exclusive

               call errchr ( &
                  code(1)(1:ncodmx(1))//' = '//codesub(1)(1:8), 1 )
               call errchr ( code(k)(1:ncodmx(k)), 2 )
               call errsub ( 1588, 0, 0, 2 )

            end if

         end do !  k = 4, 5
         if ( iinput(1)==3 .and. table(5)>0d0 ) then

!        --- keywords mutually exclusive

            call errchr ( code(1)(1:ncodmx(1))//' = '//codesub(3)(1:22), &
                          1 )
            call errchr ( code(5)(1:ncodmx(5)), 2 )
            call errsub ( 1588, 0, 0, 2 )

         end if

      else if ( table(5)>0d0 ) then

!     --- Default: pressure correction

         iinput(1) = 2

      else if ( table(4)>0d0 ) then

!     --- Default: convection substepping

         iinput(1) = 3

      else

!     --- Default: standard

         iinput(1) = 1

      end if  ! ( table(1)>0d0 )

      do k = 2, 4

         if ( table(k)>0d0 ) then

!        --- seq_time integration (2) or seq_velocity (3) or
!            number of substeps (4) found
!            not used in case of pressure correction

            iinput(k) = nint(table(k))
            if ( iinput(1)==2 ) then

!           --- pressure correction, give warning

               call errchr ( code(k)(1:ncodmx(k)), 1 )
               call errchr ( mainkeyw, 2 )
               call errchr ( codesub(2)(1:19), 3 )
               call errwar ( 2789, 0, 0, 3 )

            end if

         else if ( k==3 ) then

!        --- Default: 1 or velocity

            call readseqvector ( table(3), 'velocity', iinput(3), 1 )

         else

!        --- Default: 1

            iinput(k) = 1

         end if  ! ( table(k)>0d0 )

      end do  ! k = 2, 4

      if ( table(5)>0d0 ) iinput(5) = nint(table(5))

1000  call erclos ( 'readnavstokes' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readnavstokes'

      end if

      end
