      subroutine readnl ( iinsol, rinsol, isubr, iseqnr, iprob, nprob, &
                          niinp, nrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.24 date 10-01-2015 New call to readreal
!        version  1.23 date 09-03-2013 Correction declaration
!        version  1.22 date 09-03-2013 Correction length code
!
!   copyright (c) 1993-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for the non linear solver NLNSOL
!     The input is stored in the arrays iinsol and rinsol
!     The input must be enclosed between NONLINEAR_EQUATIONS and END.
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niinp, nrinp, iinsol(*), iseqnr, iprob, nprob, isubr
      double precision rinsol(*)

!     iinsol         o    The integer input read is stored in array INPSOL
!                         Array IINSOL must be filled as follows:
!                         Pos. 1-25: General information for all problems
!                             26-25+NCOPEQ: Starting addresses of the specific
!                                    information for each of the coupled
!                                    equations
!                                    If 0, no special information
!                             26+NCOPEQ...  Specific information of the coupled
!                                    equations
!                         Pos. 1-25: General information
!                          1: NCOPEQ    Number of coupled equations to be solved
!                                        bythis subroutine
!                                       For each iteration all equations are
!                                       solved in the natural sequence, where
!                                       the output of the preceding solutions
!                                       may be used as input for the new ones
!                          2: MAXITER   Maximum number of iterations allowed
!                          3: IPRINT    Defines the amount of output produced
!                                       Possible values:
!                                      -1:  All output information is suppressed
!                                           even the message of no-convergence
!                                       0:  No information about the iteration
!                                           process is printed except in the
!                                           case of an error
!                                       1:  The number of iterations performed
!                                           is printed
!                                       2:  Some extra information about the
!                                           iteration process is printed for
!                                           each iteration
!                                       3:  See 2, in this case also the ratio
!                                           of two succeeding iterations is
!                                           printed
!                          4: IRETURN   Indicates if the subroutine must stop if
!                                       no convergence has been found (0) or
!                                       that control must be given back to the
!                                       main program (1)
!                          5: MINITER   Minimum number of iterations allowed
!                          6: ICRITER   Type of criterion to be used for the
!                                       termination.
!                                       ICRITER consists of two parts
!                                       ICRIT and IRESCRIT
!                                       according to: ICRIT + 100*IRESCRIT
!                                       Possible values for ICRIT
!                                       0:  Absolute difference between two
!                                           succeeding iterations
!                                       1:  Relative difference between two
!                                           succeeding iterations
!                                       2:  Absolute value of right-hand side
!                                       3:  Absolute value of right-hand side
!                                           divided by solution
!                                       4:  Absolute value of energy
!                                           increment (u,f)
!                                       5:  Absolute value of energy increment
!                                           (u,f) divided by solution
!                                       6:  Absolute value of residual
!                                       7:  Absolute value of residual
!                                           divided by solution
!                                       IRESCRIT  Extra type of criterion based
!                                       on residual. To reach convergence both
!                                       the standard criterion as well as the
!                                       extra criterion must be satisfied.
!                                       Possible values:
!                                       0: No residual criterion
!                                       1: Absolute residual criterion
!                                       2: Relative residual criterion, i.e.
!                                          based on ||residual|| / ||solution||
!                                       The corresponding parameter eps is
!                                       stored in rinsol(3)
!                          7: ISEQSOL   Sequence number of linear solver to be
!                                       used
!                          8: IVECTR    Sequence number of first vector
!                                       Unless stated otherwise the vector
!                                       numbers are given by
!                                       IVECTR, IVECTR+1, ... , IVECTR+NCOPEQ-1
!                          9: CALLUSR   Indicates if the user subroutine
!                                       funcnonlin1 must be called in each
!                                       step (1) or not (0)
!                         10: MAXVEC    Maximum number of vectors stored in ISOL
!                         11: ISEQOUT   Sequence number of intermediate output
!                                       to sepcomp.out
!                                       If 0, no output is written
!                         13: METHOD_NL Indicates type of iteration method to be
!                                       applied
!                                       Possible values:
!                                       0: Standard method A^k x^(k+1) = f^k
!                                       1: Newton method: dx = 0 at boundary
!                                                         A^k dx = f^k
!                                                         x^(k+1) x^k + dx
!                                       2: Incremental Newton method:
!                                                 dx^1 contains bc for increment
!                                                 A^k dx^k = f^k
!                                                 xtotal^(k+1) = xtotal^k+dx^k
!                                                 dx^k = 0 at boundary
!                                         In this case we need 2 vectors:
!                                         ivectr: containing dx
!                                         ivec_tot: containing the total
!                                                   solution
!                         14: IVEC_TOT  Sequence number of total solution in
!                                       ISOL in case of METHOD_NL = 2
!                         15: IUSEUSR   This parameter indicates if the arrays
!                                       IUSER and USER from the parameter list
!                                       are used (1) or if the parameter
!                                       ISEQFILCOF is used (0)
!                         16: ICOPYOLD  If icopyold>0, the solution in the
!                                       last iteration before the final is
!                                       stored in array ISOL
!                                       The sequence numbers
!                                       ICOPYOLD ...ICOPYOLD+NCOPEQ-1
!                                       in ISOL are used to store the NCOPEQ
!                                       solution vectors
!                                       The user must be carefull to create ISOL
!                                       with sufficient length. No check is
!                                       possible
!                         17: LIMIT_SOLUTION  Limit the solution to minimum and
!                                       maximum values.
!                                       Posible values:
!                                       0:  no limiting
!                                       1:  only the end result
!                         18: ISUBDIVIDE  indicates if quadratic elements must
!                                       be treated as a conglomerate of
!                                       linear elements (1)
!                                       or as quadratic elements itself (0)
!                         25: NITER     Output parameter: indicates the number
!                                       of iterations performed
!                         Pos. 26+NCOPEQ + ... specific information about the
!                         various equations.
!                         For each of the equations the following information is
!                         required:
!                          1: ICRITER     Local type of criterion
!                          2: IPOSEPS     Position of local epsilon in array
!                                         RINSOL (if 0 the global eps is used)
!                          3: IPOSOMEG    Position of local omega in array
!                                         RINSOL (if 0 the global omega is used)
!                          4: ISEQSOLVE   Sequence number of linear solver call
!                                         to be used. If 0 the global one is
!                                         used
!                          5: ISEQFILCOF  Sequence number of FILCOF call to be
!                                         used in the first iteration.
!                                         If 0, FILCOF is not called
!                          6: IVECTR      Sequence number of vector to be solved
!                                         If 0, the corresponding global value
!                                         is used
!                          7: NUMCHANCOF  Maximum number of calls of CHANCOF
!                                         during the iterations.
!                                         If 0, change coeff is not called
!                          8: ISTARTCHAN  Starting address of information about
!                                         the calls to CHANCOF minus one
!                          9: NUMPRESBC   Maximum number of calls to PRESBC
!                                         during the iterations.
!                                         If 0, PRESBC is not called
!                         10: ISTARTPRES  Starting address of information about
!                                         the calls to PRESBCF minus one
!                         11: METHOD_NL   See global value
!                                         if 0 the global value is used
!                         12: IVECTOT     See global value
!                                         if 0 the global value is used
!                         13: IPOSRELEPS  Position of local residual epsilon in
!                                         array RINSOL
!                                         (if 0 the global eps is used)
!                         14: LIMIT_SOLUTION  Limit the solution to minimum and
!                                         maximum values.
!                                         Posible values:
!                                         0:  no limiting
!                                         1:  only the end result
!                                           (if 0 the global value is used)
!                         15: ISUBDIVIDE  indicates if quadratic elements must
!                                        be treated as a conglomerate of
!                                        linear elements (1)
!                                        or as quadratic elements itself (0)
!                         16-25: Not yet used, set to 0
!                         ISTARTCHAN + &
!                                     2*j-1:  iteration number for call j
!                                     2*j  :  sequence number for call j
!                         ISTARTPRES + &
!                                     2*j-1:  iteration number for call j
!                                     2*j  :  sequence number for call j
!     iprob         i/o   Problem number
!     iseqnr        i/o   Sequence number with respect to the storage in array
!                         ISEPIN
!     isubr          i    Indicates by which subroutine READNL is called
!                         Possible values:
!                         1:  ...      read record with NONLINEAR_EQUATIONS
!                         2:  ERNLNS,  do not read record with
!                                      NONLINEAR_EQUATIONS
!     niinp         i/o   Length of integer input array
!                         At input available length, at output computed length
!     nprob          i    Number of problems
!     nrinp         i/o   Length of real input array
!                         At input available length, at output computed length
!     rinsol         o    Contains the real input for the solver
!                         At this moment the input is defined as follows
!                         1:  Global accuracy eps
!                         2:  Global relaxation parameter omega
!                         3:  Accuracy with respect to residual
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode5=7, nhelp=100
      integer ncode5(nncode5), ncodmxmn(nncode5)
      integer k, ihelp(nhelp), iposrl, numcop, &
              istart, numchn, iter, iseq, j, iprev
      character (len=27) code5(nncode5)
      logical finished, readkw, openfound

!     code5          Table with the main keywords (1-4) or the keywords after
!                    equation (5-7)
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     ihelp          Work array of length 100
!                    In ihelp the starting addresses of the various problems
!                    are stored temporarily
!     iposrl         Position in array rinsol
!     iprev          Previous iteration number
!     iseq           Sequence number of change coefficients
!     istart         Starting address in array iinsol
!     iter           Iteration sequence number
!     j              Counting variable
!     k              Counting variable
!     ncode5         Table containing the number of significant characters in
!                    the keyword table code5
!     ncodmxmn       Table containing the number of maximum characters in
!                    the keyword table code5
!     nhelp          Declared length of array help
!     nncode5        Number of entries in code5
!     numchn         Number of calls of change coefficients
!     numcop         number of coupled equations
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
!     readkw         Indicates if the next keyword must be read (true) or not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READNLEQ       Read input for subroutine readnl with respect to the part
!                    equations
!     READNLGLOB     Read global options for non-linear equations
!     READREAL       Read a real or real expression from the input file
!     SEPREADPROBSEQ Read sequence number iseqnr and problem number iprob
! **********************************************************************
!
!                       I/O
!
!     READNL requires input from the standard input file
!
!     The following input is recognized:
!
!     NONLINEAR_EQUATIONS [, SEQUENCE_NUMBER = s][, PROBLEM = p] (mandatory):
!         start of input
!
!      optional records:
!
!       NUMBER_OF_COUPLED_EQUATIONS = n    (Default nprob)
!       GLOBAL_OPTIONS, options
!          where options must be on the same line or next lines if the
!          record is closed with //
!          The following options are available:
!          maxiter = m      (Default 20)
!          miniter = m      (Default 2)
!          accuracy = eps   (Default 1d-3)
!          print_level = p  (Default 0)
!          relaxation = w   (Default 0)
!          criterion = c    (Default abs)
!          lin_solver = i   (Default value 1)
!          at_error = e     (Default stop)
!          output = i       (Default 0)
!          seqtotal_vector = i   (Default 1)
!          iteration_method = m (Default standard)
!          abs_residual_accuracy = eps (Default: not)
!          rel_residual_accuracy = eps (Default: not)
!          call_func = i     (Default: 0)
!          limit_solution
!          linear_subelement
!                      Possible values for c:   absolute
!                                               relative
!                                               rhs_absolute
!                                               rhs_relative
!                                               energy_absolute
!                                               energy_relative
!                      Possible values for e:   stop
!                                               return/resume
!                      Possible values for m:   standard
!                                               newton
!                                               incremental_newton
!       EQUATION i   Information about the i-th equation, followed by
!                    records with information about this i-th equation
!            LOCAL_OPTIONS, options
!                where options must be on the same line or next lines if the
!                record is closed with //
!                The following options are available:
!                accuracy = eps   (Default global)
!                relaxation = w   (Default global)
!                criterion = c    (Default global)
!                lin_solver = i   (Default value global)
!                iteration_method = m  (Default global)
!                seqtotal_vector = i   (Default global)
!                abs_residual_accuracy = eps (Default: not)
!                rel_residual_accuracy = eps (Default: not)
!                limit_solution
!                linear_subelement
!            FILL_COEFFICIENTS = s  (Default value 0)
!            CHANGE_COEFFICIENTS followed by
!                AT_ITERATION j, SEQUENCE_NUMBER = s (repeatedly)
!
!     END   (mandatory):  end of input
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     329   Array to store the input too small
!     574   Not an integer number found after the keyword
!    1885   First record read by subroutine is not equal to NONLINEAR_EQUATIONS
!    1895   NUMCOP out of range
!    1896   IEQ out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize arrays and parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyze the first word (nonlinear_equations)
!    Read rest of first record (sequence number, problem number )
!    finished := false
!    istart := 26
!    read_keyword := true
!    While ( not finished ) do
!       if ( read_keyword ) then
!          Read next record
!       read_keyword := true
!       Analyze first item
!       If ( item # keyword ) then
!          error
!       else
!          analyze keyword
!          if ( keyword = END ) then
!             finished := true
!          else if ( keyword = NUMBER_OF_COUPLED_EQUATIONS ) then
!             Read n and fill in iinsol(1)
!          else if ( keyword = GLOBAL) then
!             analyze rest of record and fill result in iinsol
!          else if ( keyword = EQUATION ) then
!             read i
!             if ( i>100 ) then
!                error
!             ihelp(i) := istart
!             finished := false
!             numchancof := 0
!             while ( not finished ) do
!                if ( read_keyword ) then
!                   Read next record
!                read_keyword := true
!                Analyze next item
!                if ( item # keyword ) then
!                   error
!                else
!                   analyze keyword
!                   If ( keyword = main keyword ) then
!                      finished = true
!                      read_keyword := false
!                      iinsol(istart+5) := numchancof
!                      istart := istart+12+2*numchancof
!                   else if ( keyword = LOCAL ) then
!                      analyze rest of record and fill result in iinsol
!                   else if ( keyword = FILL_COEFFICIENTS ) then
!                      read sequence number s and store in iinsol
!                   else if ( keyword = CHANGE_COEFFICIENTS ) then
!                      subloop := true
!                      while ( subloop ) do
!                         read next record
!                         analyze next item
!                         if ( item = keyword AT_ITERATION ) then
!                            numchancof := numchancof + 1
!                            read j and s and store in iinsol
!                         else
!                            subloop := false
!                            read_keyword := false
!    Put ihelp in between iinsol
!    set niinp and nrinp to final values
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code5 / 'NUMBER_OF_COUPLED_EQUATIONS', 'GLOBAL_OPTIONS', &
                  'EQUATION', 'END', 'LOCAL_OPTIONS', &
                  'FILL_COEFFICIENTS', 'CHANGE_COEFFICIENTS' /
      data ncode5 / 4, 4, 3, 3, 3, 3, 3 /
      data ncodmxmn/27,14, 9, 3,13,17,19 /

!     code5  Table with the main keywords (1-4) or the keywords after
!             equation (5-7)
!             Values:
!             1:  number_of_coupled_equations
!             2:  global_options
!             3:  equations
!             4:  end
!             5:  local_options
!             6:  fill_coefficients
!             7:  change_coefficients
! ======================================================================
!
      call eropen ( 'readnl' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readnl'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncodemain = nncode5
      codemain(1:nncode5) = code5
      ncodemain(1:nncode5) = ncode5
      ncodmxmain(1:nncode5) = ncodmxmn

!     --- Initialize arrays and parameters

      iinsol(1:25) = 0
      iinsol(2) = 20
      iinsol(5) =  2
      if ( isubr==2 ) iinsol(7) =  1
      iinsol(8) =  1
      rinsol(1) =  1d-3
      rinsol(2:3) =  0d0
      iposrl    =  3
      numcop    = nhelp
      ihelp = 0
      openfound = .false.   ! No open bracket found

      if ( isubr==1 ) then

!     --- Call by TO0095, read first record  (NONLINEAR_EQUATIONS)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errsub ( 1885, 0, 0, 0 )
            call errcrd
            go to 1000
         else if ( string(1:4)/='NONL' ) then
            call errsub ( 1885, 0, 0, 0 )
            call errcrd
            go to 1000
         end if

      end if

!     --- Read rest of record

      call sepreadprobseq ( iprob, iseqnr, nprob )
      if ( ierror/=0 ) go to 1000

      istart = 26
      finished = .false.
      readkw = .true.

!     --- While loop finished

      do while ( .not. finished )

!     --- Read next record

         if ( readkw ) then
            ityprd = 1
            call asknxt
         end if
         readkw = .true.
         ierpri = 1
         nncodemain =  4
         icode = sepmainkeyw()
         nncodemain = nncode5
         if ( icode==0 ) go to 1000

         select case (icode)

!        --- keyword = NUMBER_OF_COUPLED_EQUATIONS

         case(1)

            call readreal ( openfound, .false. )
            if ( ityprd/=2 ) then
               call errchr ( code5(icode), 1 )
               call errsub ( 574, 0, 0, 1 )
               call errcrd
            else
               numcop = nint(realrd)
               if ( numcop<1 .or. numcop>nhelp ) then
                  call errint ( numcop, 1 )
                  call errint ( 100, 2 )
                  call errsub ( 1895, 2, 0, 0 )
               end if
               iinsol(1) = numcop
            end if

         case(2)

!        --- keyword = GLOBAL

            call readnlglob ( iinsol, rinsol, isubr )

         case(3)

!        --- keyword = EQUATION

            call readnleq ( iinsol, rinsol, istart, iposrl, numcop, ihelp, &
                            readkw, niinp, numchn, openfound )

         case(4)

!        --- END found

            finished = .true.

         end select  ! case (icode)
         if ( ierror/=0 ) go to 1000

      end do  ! while ( .not. finished )

      if ( iposrl>nrinp ) then

!     --- nrinp too small

         call errint ( iposrl, 1 )
         call errint ( nrinp, 2 )
         call errsub ( 329, 2, 0, 0 )

      end if

!     --- Move iinsol to final positions

      if ( iinsol(1)==0 ) numcop = nprob
      iinsol(1) = numcop
      if ( istart+numcop-1>niinp ) then

!     --- niinp too small

         call errint ( istart, 1 )
         call errint ( niinp, 2 )
         call errsub ( 329, 2, 0, 0 )
         go to 1000

      end if
      do k = istart, 26, -1
         iinsol(k+numcop) = iinsol(k)
      end do
      do k = 1, numcop
         if ( ihelp(k)>0 ) then
            iinsol(25+k) = ihelp(k) + numcop
         else
            iinsol(25+k) = 0
         end if
      end do

!     --- Check if equation numbers are smaller than numcop

      do k = numcop+1, nhelp
         if ( ihelp(k)/=0 ) then
            call errint ( k, 1 )
            call errint ( numcop, 1 )
            call errsub ( 1896, 2, 0, 0 )
         end if
      end do

!     --- Put iteration numbers in increasing sequence

      do k = 1, numcop

         if ( ihelp(k)>0 ) then

!        --- ihelp(k)>0

            istart = ihelp(k) + numcop + 6
            numchn = iinsol(istart)
            if ( numchn>0 ) iinsol(istart+1) = istart + 18
            istart = istart + 18
            finished = .false.
930         if ( .not. finished ) then

               iprev = 0
               finished = .true.
               do j = 1, numchn

                  iter = iinsol(istart+2*j-1)
                  if ( iter<iprev ) then

!                 --- interchange iter and iprev

                     iseq = iinsol(istart+2*j)
                     iinsol(istart+2*j-1) = iinsol(istart+2*j-3)
                     iinsol(istart+2*j  ) = iinsol(istart+2*j-2)
                     iinsol(istart+2*j-3) = iter
                     iinsol(istart+2*j-2) = iseq
                     finished = .false.

                  else

                     iprev = iter

                  end if

               end do
               go to 930

            end if
            istart = istart + 2*numchn

         end if

      end do

!     --- set niinp and nrinp to final values

      niinp = max(istart, 25+numcop)
      nrinp = iposrl

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinsol, niinp, 'iinsol' )
         call prinrl ( rinsol, nrinp, 'rinsol' )
         write(irefwr,*) 'End readnl'

      end if
      call erclos ( 'readnl' )

      end subroutine readnl
