      subroutine readrefine ( iinput, rinput, isubr, iseqnr, niinp, &
                              nrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 20-09-2006 Correction sequence number
!        version  1.1  date 23-05-2005 Set defaults
!        version  1.0  date 15-03-2005
!
!   copyright (c) 2005-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for refinement
!     The input is stored in array iinput
!     The input must be enclosed between REFINE and END.
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niinp, iinput(*), iseqnr, isubr, nrinp
      double precision rinput(*)

!     iinput         o    The integer input read is stored in array IINPUT
!                         This array must be filled as follows:
!                          1: times: number of refinements
!                             Default value: 1
!                          2: mesh_in:   defines sequence number of input mesh
!                             Default value: 1
!                          3: mesh_out:   defines sequence number of output mesh
!                             Default value: mesh_in
!                          4: type_refine:  defines type of refinement
!                             0: global refinement
!                             1: local refinement defined by type_criterion
!                             Default value: 0
!                          5: TYPE_CRITERION, defines criterion to be used for
!                             refinement
!                             1:  MARKED_ELEMENTS,a marker array of length nelem
!                                 is used to indicate if an element must be
!                                 refined (1) or not (0)
!                                 The sequence nuber of this array must be
!                                 stored in iseq_refarray
!                             Default value: 1
!                          6: ISEQ_REFARRAY: sequence number of the marker array
!                             Default value: 1
!                          7: IPRINT: print level
!                             Default value: 0
!     iseqnr        i/o   Sequence number with respect to the storage in
!                         array ISEPIN
!     isubr          i    Indicates by which subroutine readrefine is called
!                         Possible values:
!                         1:  ...      read record with
!                         2:  ERENTHALPY,  do not read record with REFINE
!     niinp          i    Length of integer input array
!     nrinp          i    Length of real input array
!     rinput         o    Real input array
!                         At this moment not actually used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=7
      integer ncode2(nncode2), itype2(nncode2), ncodmx2(nncode2)
      double precision tabmn2(nncode2), tabmx2(nncode2)
      character (len=23) code2(nncode2)

!     code2          Table with keywords
!     itype2         Table containing information about the numbers behind
!                    the keywords in the table code2
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     nncode2        Number of entries in code2
!     tabmn2         Minimum value of items read corresponding to code2
!     tabmx2         Maximum value of items read corresponding to code2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEY1 Interpret a record containing keywords followed by data
!                    Set END as sole main keyword
! **********************************************************************
!
!                       I/O
!
!     readrefine requires input from the standard input file
!     The following input is recognized:
!     REFINE [ SEQUENCE_NUMBER = s] (mandatory):
!         start of input
!      optional records:
!           TIMES = 2, Number of refinements (Default 1)
!           MESH_IN (Default 1)
!           MESH_OUT (Default MESH_IN)
!           LOCAL_REFINEMENT
!           TYPE_CRITERION = t
!              with t = MARKED_ELEMENTS
!           ISEQ_REFARRAY = iseq
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     329   Array to store the input too small
!     650   First record read by subroutine is not equal to REFINE
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize arrays and parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (REFINE)
!    Read rest of first record (sequence number)
!    Read the rest of the records by intx02 until END is found
!    set niinp and nrinp to final values
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2   / 'TIMES', 'MESH_IN', 'MESH_OUT', 'LOCAL_REFINEMENT', &
                    'TYPE_CRITERION', 'ISEQ_REFARRAY', 'PRINT_LEVEL' /
      data ncode2  /  4, 7, 7, 9, 8,   8,  5 /
      data ncodmx2 /  5, 7, 8,16,14,  13, 11 /
      data itype2  /  0, 0, 0, 2, 5,   0,  0 /
      data tabmn2 / 1d0, 1d0, 1d0, 0d0, 1d0, 1d0, 0d0 /
      data tabmx2 / 5d0, 5d0, 5d0, 0d0, 1d0, 1d2, 2d0 /

!     code2    Table with keywords corresponding to options
!             Values:
!             1:  TIMES
!             2:  MESH_IN
!             3:  MESH_OUT
!             4:  LOCAL_REFINEMENT
!             5:  TYPE_CRITERION
!             6:  ISEQ_REFARRAY
!             7:  PRINT_LEVEL
! ======================================================================
!
!     --- Initialize arrays and parameters

      call eropen ( 'readrefine' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readrefine'

      end if
      if ( ierror/=0 ) go to 1000

      if ( niinp<nncode2 ) then

!     --- niinp too small

         call errint ( nncode2, 1 )
         call errint ( niinp, 2 )
         call errsub ( 329, 2, 0, 0 )
         go to 1000

      end if
      call inichkarlen ( 6, niinp, 'iinput' )
      call inichkarlen ( 1, nrinp, 'rinput' )
      if ( ierror/=0 ) go to 1000

!     --- fill iinput with default values

      rinput(1) = 0d0
      iinput(1:niinp) = 1
      iinput(3) = -1  ! Temporary to indicate not filled

      if ( isubr==1 ) then

!     --- Call by readrefinexx, read first record  (REFINE)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'REFINE', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

!     --- Read sequence number

      call sepreadseqnr ( iseqnr )

!     --- Read data records

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopytype ( nncode2, itype2, tabmn2, tabmx2 )

      table(1:nncode) = -1d0
      table(1) = 1d0
      table(5) = 1d0
      table(6) = 1d0
      table(7) = 0d0

      codesub(1) = 'MARKED_ELEMENTS'
      ncodesub(1) = 5

      call sepreadsetkey1     ! interpret keywords
      if ( debug ) call prinrl ( table, nncode, 'table' )

      iinput(1) = nint(table(1))    ! times
      iinput(2) = nint(table(2))    ! mesh_in
      if ( table(3)>0 ) then            ! mesh_out
         iinput(3) = nint(table(3))
      else
         iinput(3) = iinput(3)
      end if
      iinput(4) = nint(table(4))   ! local_refinement
      iinput(5) = nint(table(5))   ! type_criterion
      iinput(6) = nint(table(6))   ! iseq_refarray
      iinput(7) = nint(table(7))   ! print level

1000  call erclos ( 'readrefine' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readrefine'

      end if

      end
