      subroutine readscintegral ( istart, inpsep, iscalinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.11 date 28-12-2014 Extension with COORDINATE_SYSTEM
!        version  1.10 date 02-01-2014 Correction vectors
!        version  1.9  date 19-12-2013 Use type 39 for scalars
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following the keyword
!     INTEGRAL
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), iscalinp

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                          2: Sequence number coefficients
!                          3: ISEQNR Sequence number integrals
!                          4: IVECTOR  Sequence number input vector
!                          5: ISCALAR1 Sequence number scalar number of integral
!                          6: Scalar number of min integral per element
!                          7: Scalar number of max integral per element
!                          8: ICHELI
!                          9: ICHOICE for level set (See inpvol(5) INTEGSBF)
!                         10: Level set number
!                         11: Degree of freedom
!                         12: Integration rule
!                         13: coordinate system
!                         14: Start address element groups (at this moment 15)
!                         15: Number of element groups
!                         16 ... Element group numbers
!     iscalinp       i    Scalar number read before
!                         If 0, no scalar number has been read
!     istart        i/o   Next free position in inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=14
      integer ncode1(nncode1), ncodmx1(nncode1), &
              ivec, iscal, i, ncomps
      integer :: numelgrp
      character (len=20) code1(nncode1)

!     code1          Table with keywords
!     i              Counting variable
!     iscal          Sequence number of scalar
!     ivec           Sequence number of vector with respect to array ISOL
!     ncode1         Table containing the number of significant letters in
!                    code1
!     ncodmx1        Table containing the number of maximum characters in
!                    the keyword table code1
!     ncomps         Number of components
!     nncode1        Number of entries in code1
!     numelgrp       Number of element groups stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getdimens

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHKDIM      Check dimension of array or scalar
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRWAR         Print warnings
!     GETDIMENS      Return with dimension of constant or scalar corresponding
!                    to iseq
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
!     READSEQINT     Put the value of the integer variable read in iinput
!     READSEQINTGR   Put the value of the integer variable read in iinput
!                    If no sequence number is found, use default value
!     READSEQSCAL    Put the value of the sequence number read in iinput
!                    If no sequence number is found, use default
!     READSEQSTRING  Put the value of the integer variable read in iinput
!                    If no sequence number is found, check if the variable is
!                    known as string
!                    If no sequence number is found, use default
!     READSEQVEC     Put the value of the sequence number read in iinput
!                    If no sequence number is found, use default value
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       INTEGRAL, options (all in one line)
!           where options may have the following contents
!               SEQ_COEF = s         (Default none)
!               SEQ_INTEGRAL = s     (Default the next one)
!               VECTOR = s           (Default 1)
!               SCALAR = s           (Default 1)
!               SCAL_MIN = s         (Default 0)
!               SCAL_MAX = s         (Default 0)
!               ICHELI = i           (Default 0)
!               ACTIVE_LEVEL_SET i
!               NON_ACTIVE_LEVEL_SET i
!               DEGFD i              (Default 0)
!               COORDINATE_SYSTEM = CARTESIAN/AXI_SYMMETRIC/POLAR
!                                    (Default cartesian)
!               ELGRP = (i1, i2, ... )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1886   Vector has not yet been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'SEQ_COEF', 'SEQ_INTEGRAL', 'VECTOR', &
                  'SCAL_MIN', 'SCAL_MAX', 'SCALAR', 'V', 'ICHELI', &
                  'ACTIVE_LEVEL_SET', 'NON_ACTIVE_LEVEL_SET', 'DEGFD', &
                  'COORDINATE_SYSTEM', 'INTEGRATION_RULE', 'ELGRP' /
      data ncode1  / 6, 7, 6, 7, 7,   5, 1, 3, 5, 8, &
                    3, 4, 3, 3 /
      data ncodmx1 / 8,12, 6, 8, 8,   6, 1, 6,16,20, &
                    5,17,16, 5 /

!     code   Table with sub keywords corresponding to keyword INTEGRAL
!             Values:
!             1:  SEQ_COEF
!             2:  SEQ_INTEGRAL
!             3:  VECTOR
!             4:  SCAL_MIN
!             5:  SCAL_MAX
!             6:  SCALAR
!             7:  V
!             8:  ICHELI
!             9:  ACTIVE_LEVEL_SET
!            10:  NON_ACTIVE_LEVEL_SET
!            11:  DEGFD
!            12:  COORDINATE_SYSTEM
!            13:  INTEGRATION_RULE
!            14:  ELGRP
! ======================================================================
!
      call eropen ( 'readscintegral' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscintegral'
         write(irefwr,1) 'istart', istart
         write(irefwr,1) 'iscalinp, lastitem', iscalinp, lastitem
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i8) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(1) = icode
      inpsep(2:14) = 0

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopycodesub ( codecsys, ncodecsys, nscodecsys )


      table(1:nncode) = -1d0

      itype(1:nncode) = 0
      itype(3) = 38    ! VECTOR
      itype(4:6) = 39  ! SCALAR
      itype(7) = 38    ! V
      itype(14) = 11   ! ELGRP

      tabmin(1:nncode) = 1d0

      tabmax(1) = 25d0
      tabmax(2) = 25d0   ! sequence_number
      tabmax(3) = dble(maxvec)
      tabmax(4) = dble(maxcls)
      tabmax(5) = dble(maxcls)
      tabmax(6) = dble(maxcls)
      tabmax(7) = tabmax(3)
      tabmax(8) = 1d3     ! ICHELI
      tabmax(9:10) = 1d0  ! LEVELSET
      tabmax(11) = 10d0   ! Degfd
      tabmin(12) = 1d0    ! COORDINATE_SYSTEM
      tabmax(12) = tabmin(12)+2d0
      tabmin(13) = 0d0    ! INTEGRATION_RULE
      tabmax(13) = 4d0
      tabmin(14) = nelgrp
      tabmax(14) = tabmin(14)

      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinrl ( table, nncode, 'table' )

!     --- Check which keywords are mutually exclusive

      if ( table(3)>0d0 .and. table(7)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 3, 7 )

      if ( table(9)>0d0 .and. table(10)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 9, 10 )

      table(3) = max(table(3),table(7))
      icode = 1
      call readseqint ( inpsep(2), 0 )
      icode = 2
      call readseqintgr ( inpsep(3), -1, 'sequence_number' )
      call readseqvec ( table(3), inpsep(4), 0, 'vector' )

      if ( debug ) then
         write(irefwr,2) 'table(3)', table(3)
         write(irefwr,1) 'inpsep(4)', inpsep(4)
      end if  ! ( debug )

      if ( iscalinp==0 ) then

!     --- Standard case, scalar has not been read before

         call readseqscal ( table, inpsep(5), 1, code, 3, 6, ncodmx )

      else

!     --- Scalar number has been read before

         inpsep(5) = iscalinp
         iscal = mod(inpsep(5),5000)

         if ( debug ) write(irefwr,1) 'iscalinp, iscal', iscalinp, iscal

         if ( iscal<=maxints+maxreals ) then

!        --- Scalar number given, raise with maxints+maxreals

            inpsep(5) = inpsep(5)+maxints+maxreals   ! correction

         end if  ! ( iscal<=maxints+maxreals )

      end if  ! ( iscal==0 )
      call readseqscal ( table, inpsep(6), 0, code, 3, 4, ncodmx )
      call readseqscal ( table, inpsep(7), 0, code, 3, 5, ncodmx )
      call readseqscal ( table, inpsep(8), 0, code, 0, 8, ncodmx )
      if ( table(9)>0d0 ) then

!     --- Active part of level set

         inpsep(9) = -1              ! ichoice
         inpsep(10) = nint(table(9))  ! level set

      else if ( table(10)>0d0 ) then

!     --- Active part of level set

         inpsep(9) = -2              ! ichoice
         inpsep(10) = nint(table(10)) ! level set

      end if  ! ( table(9)>0d0 ) then

      call readseqscal ( table, inpsep(11), 1, code, 0, 11, ncodmx )
      icode = 13
      call readseqint ( inpsep(12), 0 )
      call readseqstring ( table, code, inpsep(13), 12, 0, ncodmx, 1 )

      if ( debug ) call prininlarge ( inpsep, 14, 'inpsep' )

      ivec = inpsep(4)

      if ( ivec>0 .and. ivec<200000000 ) then

         ivec = mod(ivec,5000)   ! correction for indexed vectors
         if ( ivectors(ivec)==0 ) then

!        --- vector does not exist

            call errint ( ivec, 1 )
            call errchr ( names(ivec), 1 )
            call errwar ( 1886, 1, 0, 1 )
            call errcrd

         end if

      end if
      do i = 5, 7
         iscal = mod(inpsep(i),100000)
         if ( iscal>0 ) then
            if ( iscal<=maxcls ) iscalars(iscal) = 1
         end if  ! ( iscal>0 )
      end do

!     --- Element groups

      numelgrp = max(0,nint(table(14)))
      if ( numelgrp>0 ) then

!     --- Element groups found

         iposut = numelgrp
         numelgrp = nint(tabmax(14))
         inpsep(14) = 15
         inpsep(15) = numelgrp
         inpsep(16:15+numelgrp) = nint(tabout(iposut:iposut+numelgrp-1))
         numelgrp = numelgrp+1

      end if  ! ( numelgrp>0 )

!     --- Check dimensions

      ncomps = getdimens ( inpsep(5) )

      if ( debug ) write(irefwr,1) 'ncomps', ncomps

      call errchkdim ( ncomps, inpsep(4), 2, 'vector' )
      if ( inpsep(6)>0 ) &
         call errchkdim ( ncomps, inpsep(6), 1, 'scal_min' )
      if ( inpsep(7)>0 ) &
         call errchkdim ( ncomps, inpsep(7), 1, 'scal_max' )

      istart = istart + 14+numelgrp  ! raise istart

1000  if ( debug ) then

!     --- Debug information

         call prininlarge ( inpsep, 14+numelgrp, 'inpsep' )
         write(irefwr,1) 'istart', istart
         write(irefwr,*) 'End readscintegral'

      end if  ! ( debug )
      call erclos ( 'readscintegral' )

      end subroutine readscintegral
