      subroutine readsclevmesh ( istart, inpsep, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 15-12-2017 New storage of intmat
!        version  2.0  date 05-11-2013 Extra parameters
!        version  1.5  date 10-04-2013 Change itype for vector
!
!   copyright (c) 2004-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following the keywords
!     MAKE_LEVELSET_MESH, REMOVE_LEVELSET_MESH
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*), istrin
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                          0: icode
!                          1: MESH_ORIG Sequence number of mesh to be adapted
!                          2: MESH_SUBDIV Sequence number of adapted mesh
!                          3: IREAD_commat parameter iread for commat
!                          4: SEQ_LEVELSET Sequence number of levelset vector
!                             in array solut
!                          5: m   number of vectors to be interpolated
!                          6: shift, defines type of shifting of nodes
!                             Possible values
!                             -2:  nodes may be shifted
!                             -1:  special case for Etels program
!                              0:  no shift allowed
!                              1:  shift accordinag to Raoul
!                          7: reference to eps_move
!                          8: keep_part
!                              0: remove everything
!                              1: keep parts for correction interpolation
!                          9: reference to eps_min
!                         10..9+m:  sequence numbers of these vectors
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     rinsep        i/o   Array containing the real input for this subroutine
!                         See inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=11
      integer ncode1(nncode1), ncodmx1(nncode1), m, iposvec
      character (len=16) code1(nncode1)

!     code1          Table with keywords
!     iposvec        Start position of vector numbers in inpsep
!     m              number of items read
!     ncode1         Table containing the number of significant letters in
!                    code1
!     ncodmx1        Table containing the number of maximum characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READSEQVAR     Put the value of the variable read in rinput
!     READSEQVECTOR  Put the value of the sequence number read in inpsep
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       MAKE_LEVELSET_MESH, options (all in one line)
!           where options may have the following contents
!           MESH_ORIG (Default 1)
!           MESH_SUBDIVIDE (Default 2)
!           LEVELSET_VECTOR = i (Default phi or 2)
!           SEQ_STRUCTURE (Default -1)
!           INTERPOLATE ( Vecx, Vecy, ... )
!           NO_INTERPOLATION
!           EPS_MOVE = e  (Default ACCURACY_OBSTACLE)
!           EPS_MIN = e  (Default 0.01 * ACCURACY_OBSTACLE)
!           Present mesh becomes MESH_SUBDIV
!       REMOVE_LEVELSET_MESH, options (all in one line)
!           where options may have the same contents as above
!           extra
!           KEEP_PART
!
!           Present mesh becomes MESH_ORIG
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     996   mesh_in and mesh_out are equal
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'MESH_ORIG', 'MESH_SUBDIVIDE', 'SEQ_STRUCTURE', &
                  'LEVELSET_VECTOR', 'INTERPOLATE', &
                  'NO_INTERPOLATION', 'SHIFT', 'NO_SHIFT', 'KEEP_PART', &
                  'EPS_MOVE', 'EPS_MIN' /
      data ncode1 /  7, 7, 7, 6, 5,    6, 4, 7, 4, 6, &
                    6 /
      data ncodmx1 / 9,14,13,15,11,   16, 5, 8, 9, 8, &
                    7 /

!     code1   Table with sub keywords corresponding to keyword LEVELSET_MESH
!             Values:
!             1:   MESH_ORIG
!             2:   MESH_SUBDIV
!             3:   SEQ_STRUCTURE
!             4:   LEVELSET_VECTOR
!             5:   INTERPOLATE
!             6:   NO_INTERPOLATION
!             7:   SHIFT
!             8:   NO_SHIFT
!             9:   KEEP_PART
!            10:   EPS_MOVE
!            11:   EPS_MIN
! ======================================================================
!
      call eropen ( 'readsclevmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsclevmesh'
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- MAKE_LEVELSET_MESH or REMOVE_LEVELSET_MESH

      inpsep(0) = icode-2

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )

      iposvec = 10

      table(1:nncode) = -1d0
      itype(1:nncode) = 0
      tabmin(1:nncode) = 1d0
      tabmax(1:nncode) = 5d0

      table(1) = 1d0     ! MESH_ORIG

      table(2) = 2d0     ! MESH_SUBDIV

      tabmin(3) = -1d0    ! SEQ_STRUCTURE
      tabmax(3) = 1d2

      tabmax(4) = dble(maxvec)     ! LEVELSET_VECTOR

      itype(5) = 40      ! INTERPOLATE
      tabmin(5) = 10d0
      tabmax(5) = 1d2

      itype(6:9) = 2     ! NO_INTERPOLATION, SHIFT, NO_SHIFT, KEEP_PART
      table(9) = 0d0     ! KEEP_PART

      itype(10:11) = 1   ! EPS_MOVE, EPS_MIN
      tabmin(10:11) = 0d0
      tabmax(10:11) = 0.5d0

      iretur = 0
      nmain = 0
      call sepreadsetkeyw

!     --- Check which keywords are mutually exclusive

      if ( table(6)>0d0 .and. table(7)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 6, 7 )

      inpsep(1:3) = nint(table(1:3))
      inpsep(2) = nint(table(2))
      if ( inpsep(1)==inpsep(2) ) then

!     --- mesh_orig and mesh_subdiv are the same

         call errint ( inpsep(1), 1 )
         call errsub ( 996, 1, 0, 0 )

      end if

      call readseqvector ( table(4), 'phi', inpsep(4), 2 ) ! LEVELSET_VEC
      iposut = nint(table(5))

      if ( iposut>0 ) then

!     --- interpolate found

         m = nint(tabmax(5))
         inpsep(iposvec:iposvec+m-1) = &
            nint(tabout(iposut:iposut+m-1))

      else

!     --- not interpolate found

         m = 0

      end if  ! ( iposut>0 )

      if ( table(6)>0d0 ) then

!     --- no_interpolation found

         if ( m>0 ) call errkeywexcl ( code, ncodmx, 5, 6 )
         m = -1

      end if  ! ( table(6)>0d0 )

      inpsep(5) = m  ! number of vectors to be interpolated

      if ( table(7)>0d0 .or. table(8)<=0d0 ) then
         inpsep(6) = -2  ! shift
      else if ( table(8)>0d0 ) then
         inpsep(6) = -1  ! no_shift
      end if  ! ( table(7)>0d0 .or. table(8)<=0d0 )

      icode = 10
      call readseqvar ( 'EPS_MOVE', rinsep, inpsep(7), &
                        istrin, accuracy_obstacle )
      inpsep(8) = nint(table(9))  ! keep_part
      icode = 11
      call readseqvar ( 'EPS_MIN', rinsep, inpsep(9), &
                        istrin, 1d-2*accuracy_obstacle )

!     --- raise istart
!         if this is changed also change prmshlevms and prrmlevmesh

      istart = istart+10+max(m,0)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart/end', istart
         call prinin ( inpsep(0), 10+max(m,0), 'inpsep' )
         write(irefwr,*) 'End readsclevmesh'

      end if  ! ( debug )
      call erclos ( 'readsclevmesh' )

      end subroutine readsclevmesh
