      subroutine readscmakedist ( inpsep, istart, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 29-04-2013 inpsep from position 1
!        version  1.3  date 31-07-2012 Extra diffusion and smoothing
!        version  1.2  date 11-07-2012 Extension with typenorm
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keywords after the keyword MAKE_DISTANCE_FUNCTION
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), istrin
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine readsc
!                         positions 1-10 are used:
!                          1:  ICODE
!                          2:  iupwind (0=upwind, 1=no upwind)
!                          3:  maxiter
!                          4:  print_level (0=none, 2=max)
!                          5:  method
!                          6:  seq_solve
!                          7:  vector sequence number
!                          8:  SEQDT Sequence number delta t in real input array
!                          9:  seq_phibarrier
!                         10:  seq_eps
!                         11:  at_error (0,1=stop, 2,3=resume)
!                         12:  type_norm (1,2=distance, 3,4=difference)
!                         13:  seq_diffusion Sequence number diffusion in rinput
!                         14:  Number of steps that are carried out with
!                              sign(phi)=0
!                         15:  Distance method
!                              1: solve ||grad phi|| = 1 by pseudo time integr.
!                              2: compute "exact distance" for 2 layers
!                         16:  seq_h
!     istart        i/o   Position in inpsep
!                         See subroutine readsc
!     istrin        i/o   Next free position in rinsep
!     rinsep         o    Array containing the real input for this subroutine
!                         Contents:
!                         1:   dt
!                         2:   eps
!                         3:   diffusion term
!                         4:   h (step size)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=16, nnmcode=10
      integer ncode1(nncode1), ncodmx1(nncode1), nmcode(nnmcode)
      character (len=22) code1(nncode1), codemn(nnmcode)

!     code1          Table with keywords
!     codemn         Table with main keywords and extra texts
!     ncode1         Table containing the number of significant characters
!                    in the keyword table code1
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code1
!     nmcode         Table containing the number of significant characters
!                    in the main keyword table codemn, including those of
!                    the extra texts
!     nncode1        Number of entries in code1
!     nnmcode        Length of array nmcode
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READSEQVAR     Put the value of the variable read in rinput
!     READSEQVECTOR  Put the value of the sequence number read in iinput
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       MAKE_DISTANCE_FUNCTION V1, options
!           where options may have the following contents
!           time_step = dt       default: dtpseudo
!           upwind / no_upwind   default: upwind
!           accuracy = eps       default: eps or 1e-2
!           maxiter = m          default: 100
!           print_level = p      default: 0
!           integration_method = m    default: euler_implicit
!           seq_solve = s        default: 1
!           phi_barrier = p      default: epsdist or 1
!           at_error = e         default: stop
!              e = stop/resume/return
!           type_norm = e        default: two_distance
!              e = distance/max_distance/difference/max_difference
!           diffusion = e        default: 0
!           number_smoothing_steps = i  default: 0
!           distance_method = pseudo_time/distance  default: distance
!           step_size = h        default: h
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'UPWIND', 'NO_UPWIND', 'V', 'TIME_STEP', 'ACCURACY', &
                  'MAXITER', 'PRINT_LEVEL', 'INTEGRATION_METHOD', &
                  'SEQ_SOLVE', 'PHI_BARRIER', 'AT_ERROR', 'TYPE_NORM', &
                  'DIFFUSION', 'NUMBER_SMOOTHING_STEPS', &
                  'DISTANCE_METHOD', 'STEP_SIZE' /
      data ncode1  /  3, 5, 1, 4, 3,   4, 4, 6, 7, 7, &
                     4, 7, 4, 4, 6,   4 /
      data ncodmx1 /  6, 9, 1, 9, 8,   7,11,18, 9,11, &
                     8, 9, 9,22,15,   9 /
      data codemn /  'EULER_IMPLICIT', 'STOP', 'RESUME', 'RETURN', &
                     'DISTANCE', 'MAX_DISTANCE', 'DIFFERENCE', &
                     'MAX_DIFFERENCE', 'PSEUDO_TIME', 'DISTANCE' /
      data nmcode / 9, 3, 3, 3, 4,  7, 4, 7, 4, 4 /

!     code1      Table containing the subkeywords corresponding to this
!               subroutine. Contents:
!                1:  UPWIND
!                2:  NO_UPWIND
!                3:  V
!                4:  TIME_STEP
!                5:  ACCURACY
!                6:  MAXITER
!                7:  PRINT_LEVEL
!                8:  INTEGRATION_METHOD
!                9:  SEQ_SOLVE
!               10:  PHI_BARRIER
!               11:  AT_ERROR
!               12:  TYPE_NORM
!               13:  DIFFUSION
!               14:  NUMBER_SMOOTH_STEPS
!               15:  DISTANCE_METHOD
!               16:  STEP_SIZE
!     codemn    Table containing the subsubkeywords corresponding to this
!               subroutine. Contents:
!                1:  EULER_IMPLICIT
!                2:  STOP
!                3:  RESUME
!                4:  RETURN
!                5:  DISTANCE
!                6:  MAX_DISTANCE
!                7:  DIFFERENCE
!                8:  MAX_DIFFERENCE
!                9:  PSEUDO_TIME
!               10:  DISTANCE
! ======================================================================
!
      call eropen ( 'readscmakedist' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscmakedist'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(1) = icode  ! icode (MAKE_DISTANCE_FUNCTION)

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopycodesub ( codemn, nnmcode, nmcode )

!     --- Input for subroutine intx02

      itype(1:nncode) = 0
      itype(1:2) = 2 ! upwind/no_upwind
      itype(3) = 38  ! vector
      itype(4:5) = 1 ! time_step, accuracy
      itype(8) = 5   ! integration_method
      itype(10) = 1  ! phi_barrier
      itype(11) = 5  ! at_error
      itype(12) = 5  ! at_error
      itype(13) = 1  ! diffusion
      itype(15) = 5  ! distance_method
      itype(16) = 1  ! step_size

      tabmin(1:nncode) = 1d0
      tabmin(4:5) = 1d-15 ! time_step, accuracy
      tabmin(7) = 0d0     ! print_level
      tabmin(8) = 1d0     ! integration_method
      tabmin(11) = 2d0    ! at_error
      tabmin(12) = 5d0    ! type_norm
      tabmin(13:14) = 0d0 ! diffusion, smoothing
      tabmin(15) = 9d0    ! distance_method
      tabmin(16) = 0d0    ! step_size

      tabmax(1:nncode) = 1d2
      tabmax(4:5) = 1d0  ! time_step, accuracy
      tabmax(7) = 2d0    ! print_level
      tabmax(8) = 1d0    ! integration_method
      tabmax(9) = 1d1    ! seq_solve
      tabmax(11) = 4d0   ! at_error
      tabmax(12) = 8d0   ! type_norm
      tabmax(15) = 10d0  ! distance_method

      table(1:nncode) = 0d0
      table(5) = 1d-2    ! ACCURACY
      table(6) = 1d2     ! MAXITER
      table(12) = 4d0    ! typenorm
      table(15) = 2d0    ! distance_method
      table(16) = -1d0   ! step_size

      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( debug ) then
         call prinrl ( table, nncode, 'table' )
         call prinrl ( tabout, 1, 'tabout' )
         call prinin ( infotable, nncode, 'infotable' )
      end if  ! ( debug )

!     --- upwind/no_upwind

      if ( table(1)>0d0 .and. table(2)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 1, 2 )  ! keywords mutually exclusive

      if ( table(2)>0d0 ) then
         inpsep(2) = 1  ! no upwind
      else
         inpsep(2) = 0  ! upwind
      end if  ! ( table(2)>0d0 )

      call readseqvector ( table(3), 'phi', inpsep(7), 2 )  ! vector
      icode = 4
      call readseqvar ( 'dtpseudo', rinsep, inpsep(8), &
                        istrin, 1d-2 )  ! time_step
      icode = 5
      call readseqvar ( 'eps', rinsep, inpsep(10), &
                        istrin, 1d-2 )  ! accuracy
      icode = 10
      call readseqvar ( 'epsdist', rinsep, inpsep(9), &
                        istrin, 1d0 )  ! phi_barrier
      icode = 13
      call readseqvar ( 'epsilon', rinsep, inpsep(13), &
                        istrin, 0d0 )  ! diffusion
      inpsep(3) = nint(table(6))    ! MAXITER
      inpsep(4) = nint(table(7))    ! PRINT_LEVEL
      inpsep(5) = 5                 ! METHOD (EULER_IMPLICIT)
      inpsep(6) = nint(table(9))    ! SEQ_SOLVE
      inpsep(11) = nint(table(11))  ! at_error
      inpsep(12) = nint(table(12))  ! type_norm
      inpsep(14) = nint(table(14))  ! number of smoothing steps
      inpsep(15) = nint(table(15))  ! type_distance
      icode = 16
      call readseqvar ( 'h', rinsep, inpsep(16), &
                        istrin, 0d0 )  ! step_size

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep, 16, 'inpsep' )
         call prinrl ( rinsep(istrin-4), 5, 'rinsep' )
         write(irefwr,*) 'End readscmakedist'

      end if  ! ( debug )
      call erclos ( 'readscmakedist' )

      istart = istart+16  ! raise istart

      end subroutine readscmakedist
