      subroutine readscplotcn ( istart, inpsep, istrin, rinsep, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.8  date 14-11-2013 Shift of inpsep
!        version  1.7  date 21-11-2012 Extension with icolor
!        version  1.6  date 09-11-2012 Extension with levels
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for the contour plot with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     contour_level
!     read
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*), istrin, ndim
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine readsc
!                         positions 1-12 are used:
!                          0: ICODE
!                          1: IVECTOR
!                          2: IREGION
!                             If IREGION#0, 4 positions in rinsep are used
!                          3: IDEGFD
!                          4: NLEVEL
!                          5: MINLEVEL
!                             If MINLEVEL#0, 1 extra position in rinsep is used
!                          6: MAXLEVEL
!                             If MAXLEVEL#0, 1 extra position in rinsep is used
!                          7: IYFACT
!                             If IYFACT#0, 1 extra position in rinsep is used
!                          8: TEXT, sequence number in text array
!                          9: NTEXT, number of characters in text
!                         10: NINNER: plot inner boundaries
!                             0: not
!                             1: plot
!                             +10:  plot zero level set boundary
!                         11: Starting address of levels in rinsep
!                         12: ICOLOR: colour of contour lines
!                         13: IAXI: if 1 the axi-symmetric mesh is used
!                                   and the solution extended to this mesh
!     istart        i/o   Position in inpsep
!                         See subroutine readsc
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     ndim           i    Dimension of the space.
!     rinsep        i/o   Array containing the real input for this subroutine
!                         See subroutine readsc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=12
      integer ncode1(nncode1), ncodmx1(nncode1)
      character (len=20) code1(nncode1)
      integer i, iseqv, istrinsav, j, ncntln

!     code1          Table with keywords with respect to plot contour
!     i              Counting variable
!     iseqv          Sequence number of keyword V in array code
!     istrinsav      Help variable to store istrin
!     j              Counting variable
!     ncntln         Parameter to compute the number of contour levels
!     ncode1         Table containing the number of significant letters in
!                    code1
!     ncodmx1        Table containing the number of maximum characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCLUS  Check if two keywords are mutually exclusive
!     INTXSTORETEXT  Fill reference to text and number of characters in inpsep
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       PLOT_CONTOUR V1, options
!       PLOT_COLOURED_LEVELS V1, options
!           where options may have the following contents
!           region = (xmin,xmax,ymin,ymax,zmin,zmax)
!           degfd=i
!           nlevels = n
!           minlevel = m1
!           maxlevel = m2
!           yfact = y
!           text = '..'
!           zero_level_set
!           levels = (l1, l2, ... )
!           color = j
!           axi_symmetric
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'REGION', 'YFACT', 'DEGFD', 'NLEVELS', 'MINLEVEL', &
                   'MAXLEVEL', 'TEXT', 'V', 'ZERO_LEVEL_SET', 'LEVELS', &
                   'COLOR', 'AXI_SYMMETRIC' /
      data ncode1  /  3, 3, 3, 3, 4,   4, 4, 1, 4, 3, &
                      3, 3 /
      data ncodmx1 /  6, 5, 5, 7, 8,   8, 4, 1,14, 6, &
                      6,13 /

!     code   Table with sub keywords corresponding to keyword PLOT_CONTOUR
!             Values:
!             1:  REGION
!             2:  YFACT
!             3:  DEGFD
!             4:  NLEVELS
!             5:  MINLEVEL
!             6:  MAXLEVEL
!             7:  TEXT
!             8:  V
!             9:  ZERO_LEVEL_SET
!            10:  LEVELS
!            11:  COLOR
!            12:  AXI_SYMMETRIC
! ======================================================================
!
      call eropen ( 'readscplotcn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscplotcn'
         istrinsav = istrin
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      inpsep(0) = icode

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )

      iseqv = 8

!     --- Fill arrays

      itype(1:nncode) = 0
      itype(1) = 19  !  REGION
      itype(2) = 1   !  YFACT
      itype(5:6) = 1 !  MINLEVEL, MAXLEVEL
      itype(7) = 6   !  TEXT
      itype(9) = 2   !  ZERO_LEVEL_SET
      itype(10) = 19 !  LEVELS
      itype(12) = 2  !  AXI_SYMMETRIC

      tabmin(1:nncode) = 0d0
      tabmin(1) = 2d0*ndim   !  REGION
      tabmin(3) = 1d0   !  DEGFD
      tabmin(4) = 1d0   !  NLEVELS
      tabmin(5:6) = -1d15 !  MINLEVEL, MAXLEVEL
      tabmin(8) = 1d0   !  V
      tabmin(10) = 41d0   !  LEVELS
      tabmin(11) = 1d0   !  color

      tabmax(1:nncode) = 1d2
      tabmax(1) = 1d5   !  REGION
      tabmax(5:6) = 1d15 !  MINLEVEL, MAXLEVEL
      tabmax(8) = 1d2   !  V
      tabmax(10) = 1d20   !  LEVELS
      tabmax(11) = 1d2   !  color

      table(1:nncode) = 0d0
      table(5) = -1d20    ! minlevel
      table(6) = -1d20    ! maxlevel
      table(iseqv) = 1d0

      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( debug ) call prinrl ( table, nncode, 'table' )

      if ( ierror/=0 ) go to 1000

!     --- Check on keywords that are mutually exclusive

      do i = 4, 6
         do j = 9, 10
            call errkeywexclus ( table, code, ncodmx, i, j )
         end do  ! j = 9, 10
      end do  ! i = 4, 6
      call errkeywexclus ( table, code, ncodmx, 9, 10 )

      inpsep(1:12) = 0
      if ( table(1)>0 ) then
         inpsep(2) = istrin
         iposut = nint(table(1))
         i = 2*ndim-1
         rinsep(istrin:istrin+i) = tabout(iposut:iposut+i)
         istrin = istrin+2*ndim
      else
         inpsep(2) = 0
      end if
      if ( table(2)>0 ) then
         inpsep(7) = istrin
         rinsep(istrin) = table(2)
         istrin = istrin+1
      else
         inpsep(7) = 0
      end if
      if ( table(3)>0d0 ) then
         inpsep(3) = nint(table(3))
      else
         inpsep(3) = 1
      end if
      if ( table(4)>0d0 ) then
         inpsep(4) = nint(table(4))
      else
         inpsep(4) = 0
      end if
      if ( table(5)>-1d19 ) then
         inpsep(5) = istrin
         rinsep(istrin) = table(5)
         istrin = istrin+1
      else
         inpsep(5) = 0
      end if
      if ( table(6)>-1d19 ) then
         inpsep(6) = istrin
         rinsep(istrin) = table(6)
         istrin = istrin+1
      else
         inpsep(6) = 0
      end if

      if ( table(10)>0d0 ) then

!     --- levels found

         iposut = nint(table(10))  ! starting address in tabout
         ncntln = nint(tabmax(10)) ! number of levels
         inpsep(4) = ncntln
         inpsep(11) = istrin  ! starting address in rinsep
         rinsep(istrin:istrin+ncntln-1) = &
            tabout(iposut:iposut+ncntln-1)
         istrin = istrin+ncntln

      end if  ! ( table(10)>0d0 )

!     --- text

      call intxstoretext ( inpsep(8), table(7), tabmax(7), &
                           strout )

      inpsep(1) = nint(table(iseqv))  ! sequence number of vector

      inpsep(10) = 0   ! ninner
      if ( table(9)>0d0 ) inpsep(10) = inpsep(10)+10 !zero_level
      inpsep(12) = nint(table(11))
      inpsep(13) = nint(table(12))
      istart = istart+14

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart, istrinsav, istrin', &
                          istart, istrinsav, istrin
         call prinin ( inpsep(1), 13, 'inpsep' )
         call prinrl ( rinsep(istrinsav), istrin-istrinsav, 'rinsep' )
         write(irefwr,*) 'End readscplotcn'

      end if
      call erclos ( 'readscplotcn' )

      end
