      subroutine readscpsi ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-03-2012 New call to readseqvector
!        version  1.0  date 16-02-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following the keyword
!     COMPUTE_PSI_FILM
!     with respect to subroutine readsc
!     The input is stored in array inpsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     boundary
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istart

!     inpsep         o    The integer input read is stored in array INPSEP
!                         INPSEP is filled as follows
!                          1: IPSI, sequence number of stream function in isol
!                          2: IVEL, sequence number of velocity in isol
!                          3: NUMPOINTS, number of points given
!                          4: NUMCURVS, number of curves given
!                          5: NUMSURFS, number of surfaces given
!                          Followed by point numbers, curve numbers, surface
!                                      numbers
!     istart        i/o   Next position in array inpsep
!                         The parameter is raised in this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=4
      integer ncode1(nncode1), itype1(nncode1), ncodmx1(nncode1), &
              numcurvs, numsurfs, istart1, ivec
      double precision :: tabmin1(nncode1), tabmax1(nncode1)
      character (len=10) code1(nncode1)

!     code1          Table with keywords
!     istart1        First position used at a part
!     itype1         Table containing information about the numbers behind
!                    the keywords in the table code1
!     ivec           Sequence number of vector
!     ncode1         Table containing the number of significant characters
!                    in the keyword table code1
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
!     numcurvs       Number of curves
!     numsurfs       number of surfaces
!     tabmax1        Contains maximum values allowed in table
!     tabmin1        Contains minimum values allowed in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READSEQVECTOR  Put the value of the sequence number read in iinput
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       COMPUTE_PSI_FILM, options
!           where options may have the following contents
!           VECTOR = ...
!           VEL_VECTOR = ...
!           SURFACES = (Si, Sj, .. )
!           CURVES = (Ci, Cj, .. )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1886   Vector has not yet been filled
!    2999   No psi vector found after the keyword COMPUTE_PSI_FILM
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'VECTOR','VEL_VECTOR', 'CURVES', 'SURFACES' /
      data ncode1  / 6,  6, 4, 4 /
      data ncodmx1 / 6, 10, 6, 8 /
      data itype1  / 38,38,18,18 /
      data tabmin1 / 1d0, 1d0, 1d2, 1d2 /
      data tabmax1 / 0d0, 0d0, 1d4, 1d4 /

!     code   Table with sub keywords corresponding to keyword COMPUTE_PSI_FILM
!             Values:
!             1:  VECTOR
!             2:  VEL_VECTOR
!             3:  CURVES
!             4:  SURFACES
! ======================================================================
!
      call eropen ( 'readscpsi' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscpsi'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(istart) = icode      ! standard first item

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )

      codesub(1) = 'C'
      codesub(2) = 'S'
      ncodesub(1:2) = 1

!     --- Clear table

      table(1:nncode) = -1d0
      table(3) = -1d0     ! refers to C in code1
      table(4) = -2d0     ! refers to S in code1

!     --- Clear inpsep

      inpsep(istart+2) = 1   ! default value velocity vector

      tabmax(1) = dble(maxvec)  ! maximum number of vectors
      tabmax(2) = dble(maxvec)  ! maximum number of vectors
      tabmax(3) = ncurvs        ! number of curves
      tabmax(4) = nsurfs        ! number of surfaces

      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinrl ( tabout, nout, 'tabout' )

!     --- psi vector

      call readseqvector ( table(1), 'psi', ivec, 2 )

      if ( ivec>0 ) then

!     --- psi vector found

         inpsep(istart+1) = ivec

      else

!     --- not psi vector found

         call errsub ( 2999, 0, 0, 0 )

      end if  ! ( ivec>0 )

!     --- velocity vector

      call readseqvector ( table(2), 'velocity', ivec, 1 )

      if ( ivec>0 ) then

!     --- velocity vector found

         inpsep(istart+2) = ivec

         if ( ivectors(ivec)==0 ) then

!        --- vector does not exist

            call errint ( ivec, 1 )
            call errchr ( names(ivec), 1 )
            call errwar ( 1886, 1, 0, 1 )
            call errcrd

         end if

      end if  ! ( ivec>0 )

      istart1 = istart+6          ! start of curve numbers
      inpsep(istart+3) = 0        ! number of points (future use)

!     --- Curves

      iposut = nint(table(3))
      numcurvs = nint(tabmax(3))
      inpsep(istart+4) =  numcurvs ! number of curves
      if ( numcurvs>0 ) &
         inpsep(istart1:istart1+numcurvs-1) = &
         nint(tabout(iposut:iposut+numcurvs-1))
      istart1 = istart1+numcurvs

!     --- Surfaces

      iposut = nint(table(4))
      numsurfs = nint(tabmax(4))
      inpsep(istart+5) = numsurfs ! number of surfaces
      if ( numsurfs>0 ) &
         inpsep(istart1:istart1+numsurfs-1) = &
         nint(tabout(iposut:iposut+numsurfs-1))
      if ( debug ) &
         call prinin ( inpsep(istart), istart1+numsurfs-istart+1, &
                       'inpsep' )

      istart = istart1+numsurfs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readscpsi'

      end if  ! ( debug )
      call erclos ( 'readscpsi' )

      end subroutine readscpsi
