      subroutine sepcopsolsymm ( ivecseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solution from original mesh to extended (axi-symmetric) mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     mesh
!     2d
!     symmetric
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulesymm
      use sepmodulemain
      use sepmodulemnsub
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ivecseq

!     ivecseq        i    Sequence number of vector to be considered with
!                         respect to the solution array (isol or isolut)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: isol
      integer :: ivec, itypvc, indprfsymm, indprpsymm, nunkpsymm
      integer :: iseqkm1, npointsav

!     indprfsymm     indprf for extended mesh (dummy)
!     indprpsymm     indprp for extended mesh (dummy)
!     iseqkm1        Sequence number with respect to km of first mesh
!     isol           Standard SEPRAN array containing information of the
!                    solution array
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     npointsav      Help parameter to save the old value of npoint
!     nunkpsymm      nunkp for extended mesh (dummy)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCOPSOLSYMM1 Copy solution from original mesh to extended
!                    (axi-symmetric) mesh
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3219   symmetric mesh has not been constructed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopsolsymm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopsolsymm'
         write(irefwr,1) 'meshsymm, meshnr', meshsymm, meshnr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( meshsymm<=0 .or. meshsymm>maxmesh .or. meshsymm==meshnr ) then

!     --- meshsymm is incorrect, probably not extend_axi_symmetric_mesh
!         used

         call errint ( meshsymm, 1 )
         call errint ( meshnr, 2 )
         call errsub ( 3219, 2, 0, 0 )
         go to 1000

      end if  ! ( meshsymm<=0 .or. meshsymm>maxmesh .or. meshsymm==meshnr )
      iseqkm1 = iseqkm
      kmesh => kmesh2d(meshsymm)
      call sepfillkmesh(kmesh)

!     --- Activate solution array and related arrays

      isol => isolut2d(ivecseq)
      call sepactsolbf2 ( isol )
      kprobf => kprobfi

!     --- Create new solution array

      itypvc = ks(isol)%typevec
      ivec = ks(isol)%ivec
      iprob = ks(isol)%iprob
      isolsymm => isolut3d(ivecseq,meshsymm)
      kprob => kprob2d(meshsymm)
      kprobsymm => kprob
      iseqkp = kprob
      call sepcreasolbf1 ( isolsymm, itypvc, ivec, 'isolsymm', &
                           ks(isol)%compl )
      indprfsymm = indprfo
      indprpsymm = indprpo
      nunkpsymm = nunkpo
      kprob => kprob2d(meshnr)

      if ( indprf>0 ) then

!     --- indprf > 0, activate kprobf and kprobfsymm

         kprobfsymm => kprobfo

      end if  ! ( indprf>0 )

      usol => ks(isol)%sol
      nusol = size(usol)
      usolsymm => ks(isolsymm)%sol

      npointsav = npoint
      npoint = km(iseqkm1)%kmesh(8)
      call sepcopsolsymm1
      npoint = npointsav

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopsolsymm'

      end if  ! ( debug )
      call erclos ( 'sepcopsolsymm' )

      end subroutine sepcopsolsymm
