      subroutine sepderivs ( inpder, ioutvc, inpvec, &
                             iread, output, jprob, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.11 date 24-12-2014 New definition clinsl
!        version  1.10 date 17-08-2013 Set jprob
!        version  1.9  date 10-10-2012 Extension of leniinder
!
!   copyright (c) 1998-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   computes derived quantities of an input vector, either in the nodal points
!   or in the elements
!   Depending on the parameters in iread input is given by inpder and iuser/user
!   or read from the standard input file
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpder(*), iread(*), jprob, icurvs(*)
      integer, target :: inpvec(*), ioutvc
      double precision output(*)

!     icurvs   i  Integer array containing the curve numbers for the
!                 computation of the boundary integral. A negative curve
!                 number indicates that the curve must be computed in
!                 the reversed direction.
!                 The curves including their sign must form one contiguous curve
!                 The number of curves is stored in array inpcop
!                 icurvs is only used if iread(1) = -1
!     inpder  i   Integer array in which the user indicates what computations
!                 must be performed by subroutine DERIV
!                 inpder is only used if iread(1) = -1 or iread(1)>0
!                 If iread(1)>0,inpder(1) defines the scalar sequence number.
!                 This number must be >0 if an integral must be computed
!                 inpder(2) defines numvec
!                 If iread(1) = -1 then
!                 the entries of INPDER must be filled (by the user) as follows
!                 If the user sets a value equal to 0, the default value is used
!                 1:  Number of entries filled by the user
!                 2-20:  See subroutine readdr  iinpcop(1:19)
!     inpvec  i   Contains information concerning the input vector
!     ioutvc  o   Contains information of the output vector
!     iread   i   Array iread defines if parameters should be read or
!                 are given through the parameter list.
!                 iread(1) refers to inpder
!                 iread(2) refers to iuser/user
!                 iread(1) indicates if array INPDER is used
!                 or that input is read from the standard INPUT file
!                 Possible values:
!                 0   The input is read from the standard input file
!                 >0  The input as read from the standard input file by
!                     subroutine SEPSTN is used. The value of iread
!                     defines the sequence number
!                 -1  No input is read, all information from array INPDER
!                     is used
!                 iread(2) indicates if the arrays IUSER/USER are used
!                 or that input is read from the standard INPUT file
!                 Possible values:
!                 0   The input is read from the standard input file
!                 >0  The input as read from the standard input file by
!                     subroutine SEPSTN is used. The value of iread
!                     defines the sequence number of the coefficients
!                 -1  No input is read, all information from arrays IUSER/USER
!                     is used
!     jprob   i   Actual problem number
!     output  o   Array containing the integrals that are computed if ICHOIS>10
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqnr, iskip, itypbn, iscalr, leninp, lenrnp
      double precision rinput(1)
      integer, allocatable :: inpcop(:)

!                    they are not
!     inpcop         Copy of integer input array extended with defaults
!     iscalr         Scalar sequence number
!     iseqnr         Sequence number with respect to the storage in array ISEPIN
!     iskip          Indicates if element groups must be skipped (>0) or not (0)
!     itypbn         itype_boundary, see inpder(13)
!     leninp         Computed length of integer input array
!     leniuser       Length of array iuser
!     lenrnp         Computed length of real input array
!     lenuser        Length of array user
!     rinput         Dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     IINPUTSEQ      Read contents of input block with sequence number iseqnr
!                    corresponding to main keyword name
!     PRININ         print 1d integer array
!     READDR         Read the input for the subroutine DERIVS
!     SEPFILLSMF1    Fill part of input for subroutine SEPBUILD in SMF
!     TO0100         Performs the actual computation of the derivatives
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1507   ISCALAR>0 and ITYPE_BOUN = 0
!    1508   ISCALAR = 0 and ITYPE_BOUN>0
!    1734   Array inpcop too small to store array IELHLP
! **********************************************************************
!
!                       PSEUDO CODE
!
!   initialize array inpcop
!   If ( iread(1)=-1 ) then
!      Fill array inpcop by copying from array inpder
!   else if ( iread(1)=0 ) then
!      Read array inpcop from the standard input file
!   else (iread(1)>0)
!      Copy array inpcop from IBUFFR
!   if ( iread(2)=0 ) then
!      Read arrays iuser and user from the standard input file
!   else (iread(2)>0)
!      Copy array iuser and user from IBUFFR
!   Compute derivatives
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderivs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderivs'
         call ertrace
         call prinin ( iread, 2, 'iread' )
         write(irefwr,1) 'jprob', jprob
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      isolut2d => inpvec(1:maxvec)
      isolut => isolut2d(1)
      iprob = jprob

!     --- Allocate array inpcop

      allocate ( inpcop(leniinder+nelgrp+101), stat = error )
      if ( error/=0 ) call eralloc ( error, leniinder+nelgrp+100, 'inpcop' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array inpcop

      inpcop = 0
      inpcop(1) = 3
      inpcop(4:7) = 1
      inpcop(10:11) = 1

      if ( iread(1)==-1 ) then

!     --- iread(1) = -1, fill inpcop from inpder

         leninp = min(inpder(1),leniinder)
         inpcop(2:leninp) = inpder(2:leninp)
         if ( inpder(12)>0 ) &
            inpcop(leniinder+1:leniinder+nelgrp) = &
               inpder(leniinder+1:leniinder+nelgrp)
         leninp = inpcop(15)
         if ( leninp>0 ) &
            inpcop(leniinder+nelgrp+1:leniinder+nelgrp+leninp) = &
            icurvs(1:leninp)

      else if ( iread(1)==0 ) then

!     --- iread(1) = 0, fill inpcop by reading from the standard input file

         iseqnr = 1
         inpcop(1) = leniinder+nelgrp
         call readdr ( inpcop(2), 1, iseqnr, iprob, nprob )

      else

!     --- iread(1)>0, fill inpcop by copying from IBUFFR

         iseqnr = iread(1)
         iscalr = inpder(1)
         leninp = leniinder+nelgrp+100
         lenrnp = 1
         call iinputseq ( 11, -iseqnr, inpcop(2), &
                          rinput, leninp, lenrnp, 'derivatives' )
         inpcop(1) = leniinder+nelgrp+100
         itypbn = inpcop(13)
         inpcop(8) = inpder(2)
         if ( iscalr>0 .and. itypbn==0 ) then

!        --- iscalr>0, only integral should be computed

            call errwar ( 1507, 0, 0, 0 )
            iscalr = 0

         else if ( iscalr==0 .and. itypbn>0 ) then

!        --- iscalr = 0, no integral should be computed

            call errsub ( 1508, 0, 0, 0 )

         end if
         if ( iscalr>0 ) inpcop(2) = -1

      end if

      if ( inpcop(10)>0 ) then

!     --- inpcop(10)>0, hence input vector given
!         Set iprob

         iprob = ks(inpvec(inpcop(10)))%iprob

      end if  ! ( inpcop(10)>0 )

      if ( debug ) then
         call prinin ( inpcop, leniinder, 'inpder' )
         write(irefwr,1) 'iprob', iprob
      end if  ! ( debug )

!     --- Compute starting address of array IELHLP

      iskip = inpcop(12)
      itypbn = inpcop(13)
      if ( iskip>0 .and. iread(1)==-1 ) then
         if ( inpder(1)<iskip+nelgrp-1 ) then

!        --- inpder(1) too small to store IELHLP

            call errint ( inpder(1), 1 )
            call errint ( nelgrp+iskip, 2 )
            call errint ( iskip, 3 )
            call errint ( iskip, 4 )
            call errint ( iskip, 5 )
            call errint ( nelgrp, 6 )
            call errsub ( 1734, 6, 0, 0 )

         end if

      else if ( iskip==0 ) then

         iskip = 1

      end if
      inpcop(1) = 3
      SMF%ioutvc => ioutvc
      iinder = inpcop(1:leniinder)

      if ( iread(2)==-1 ) then

!     --- iread(2)=-1, the user has filled iuser and user himself

         call sepfillsmf1 ( isolut2d, inpcop(7) )
         call to0100 ( inpcop(1+leniinder), &
                       inpcop(1+leniinder+nelgrp) )

      else

!     --- iread(2)>=0, iuser and user must be read or have been read
!                      they are stored in IBUFFR

         isubr = 3
         if ( iread(2)/=0 ) call erlins ( isubr, iprob, iread(2) )

         if ( associated(iuser1(iprob)%iuser) ) then

!        --- iuser1(iprob)>0, IUSER and USER are filled

            iuser => iuser1(iprob)%iuser
            user => iuser1(iprob)%user

         end if  ! ( associated(iuser1(iprob)%iuser) )

         call sepfillsmf1 ( isolut2d, maxvec )
         call to0100 ( inpcop(1+leniinder), &
                       inpcop(1+leniinder+nelgrp) )

      end if

      if ( meshnr<1 .or. meshnr>maxmesh ) meshnr = 1
      isolut2d => isolut3d(:,meshnr)

      if ( meshnr<1 .or. meshnr>maxmesh ) meshnr = 1
      isolut2d => isolut3d(:,meshnr)
      kmesh => kmesh2d(meshnr)
      kprob => kprob2d(meshnr)

      output(1) = valintegral

      deallocate ( inpcop, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'inpcop' )
      nullify ( iuser, user )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepderivs'

      end if
      call erclos ( 'sepderivs' )

      end subroutine sepderivs
