      subroutine sepmanmap ( inpsep, ivec2, ivec1, iinvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Map vector of one structure onto a vector of another structure
!
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     map
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*), ivec1, iinvec(*)
      integer, intent(inout) :: ivec2

!     iinvec         i    User input array for subroutine MANVEC
!                         For a description see subroutine MANVECBF
!     inpsep         i    The integer input read is stored in array INPSEP
!     ivec1          i    sepran input vector
!     ivec2          i    sepran output vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iskip, ivec, itypvc, i, idummy, &
                 inunkp, jdegfd, idegfd(3), jskip
      integer, allocatable :: iskippoints(:)
      double precision, pointer :: vec1(:), vec2(:)
      logical :: compli, clearvec

!     clearvec       If true the vector must be cleared
!     compli         Indication if IVEC1 corresponds to a complex array
!                    (true) or to a real one (false)
!     i              Loop variable
!     idegfd         Degree of freedom sequence number
!     idummy         Dummy parameter
!     inunkp         Number of degrees of freedom in point
!     iskip          Indicates if element groups must be skipped (>0)
!                    or not (0)
!     iskippoints    If iskippoints(i)=0, node i must be skipped, else
!                    iskippoints(i)>0, node i must not be skipped
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     jdegfd         Degree of freedom
!     jskip          Skip element groups (>0) or not (0) or volumes (<0)
!     vec1           input vector
!     vec2           output vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     ERSKIPELGRP    Fill array ioutpoints for all points that are not lying in
!                    element groups that must be skipped
!     PRININ         print 1d integer array
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPMANMAP1     Copy degree of freedom jdegfd from vec1 into vec2
!     TO0082         Map one vector onto another (real case)
!     TO0087         Map one vector onto another (complex case)
!     TOMAPTOVEC116  Map vector defined per node onto a vector defined per
!                    element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     386   complex quantities are not permitted in this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmanmap' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmanmap'
         call prinin ( inpsep, 14, 'inpsep' )
         call prinin ( ivec1, 5, 'ivec1' )
         call prinin ( ivec2, 5, 'ivec2' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get some parameters

      idegfd(1) =  inpsep(5)
      ivec = inpsep(7)
      iskip = inpsep(14)
      if ( ivec==0 ) then
         itypvc = 110
      else if ( ivec<0 ) then
         itypvc = 116
         ivec = -ivec
      else
         itypvc = 115
      end if

!     --- Check type of ivec1

      if ( ks(ivec1)%compl==0 ) then
         compli = .false.
      else
         compli = .true.
      end if

!     --- Activate arrays corresponding to ivec2

      call sepactsolbf2 ( ivec1 )

!     --- Reserve space for the output vector and activate the
!         corresponding arrays KPROBF and KPROBP

      clearvec = ivec2<=0 .or. ivec2>lastsol
      iprob = max(1,inpsep(2))
      call sepcreasolbf1 ( ivec2, itypvc, ivec, 'vector', ks(ivec1)%compl )
      if ( debug ) call prinin ( ivec2, 5, 'ivec2' )

      vec1 => ks(ivec1)%sol
      vec2 => ks(ivec2)%sol

      if ( clearvec ) vec2 = 0d0

      if ( iskip>0 ) then

!     --- iskip>0, create and fill array iskippoints

         allocate ( iskippoints(npoint), stat = error )
         if ( error/=0 ) call eralloc ( error, npoint, 'iskippoints' )
         if ( ierror/=0 ) go to 1000

         jskip = iinvec(7)
         call erskipelgrp ( iinvec(jskip), iskippoints )

      end if  ! ( iskip>0 )

      if ( compli ) then

!     --- Complex case

         if (  ks(ivec2)%nusol/=ks(ivec1)%nusol ) then

!        --- New vector other length than old one
!            clear this vector first

            vec2 = 0d0

         end if
         if ( itypvc==116 ) then

!        --- itypvc = 116, array defined per element
!            Not yet implemented for type 116

            call errsub ( 386, 0, 0, 0 )
            go to 1000

         else

!        --- itypvc = 110 or 115, standard arrays

            call to0087 ( vec1, vec2, indprfi, &
                          kprobfi, indprfo, kprobfo, npoint, nphysi )

         end if  ! ( ivec==116 )

      else

!     --- Real case

         if (  ks(ivec2)%nusol/=ks(ivec1)%nusol ) then

!        --- New vector other length than old one
!            clear this vector first

            vec2 = 0d0

         end if

         if ( debug ) then
            write(irefwr,1) 'itypvc, indprfo, indprf', itypvc, indprfo, indprfi
            write(irefwr,1) 'iskip', iskip
         end if  ! ( debug )

         if ( itypvc==116 ) then

!        --- itypvc = 116, array defined per element

            call tomaptovec116 ( vec1, vec2, nphysi, &
                                 ivec, idegfd(1), indprfi )

         else

!        --- itypvc = 110 or 115, standard arrays


            if ( iskip>0 ) then

!           --- Both vectors have the same structure; iksip>0

               call to0082 ( vec1, vec2, idegfd(1), iskip, iskippoints )

            else if ( indprfo/=indprfi .or. indprpo/=indprpi ) then

!           --- Both vectors have different structure
!               Copy component-wise

               jdegfd = max(1,idegfd(1))
               call sepmanmap1 ( jdegfd, vec1, vec2 )

            else

!           --- Both vectors have the same structure; iksip==0

               call to0082 ( vec1, vec2, idegfd(1), iskip, idummy )

            end if  ! ( indfpo/=0 .and. indfpi==0 ... )

         end if  ! ( ivec==116 )

      end if

      if ( iskip>0 ) then

!     --- iskip>0, deallocate iskippoints

         deallocate ( iskippoints, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iskippoints' )

      end if  ! ( iskip>0 ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmanmap'

      end if  ! ( debug )
      call erclos ( 'sepmanmap' )

      end subroutine sepmanmap
