      subroutine to0089reac ( iinsol, ivectr, iinbldlc, ivecreac, ivecfem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 17-07-2015 Initialize idummy
!        version  1.3  date 25-11-2012 Long integers
!        version  1.2  date 23-03-2007 Extra debug
!
!   copyright (c) 2005-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the reaction forces in the linear system of equations
!     Filling is done depending on iinsol
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     right_hand_side
!     solve
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulesmf
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinsol(*), ivectr, iinbldlc(*), &
              ivecreac, ivecfem

!     iinbldlc       i    User input array for subroutine BUILD
!     iinsol         i    Contains information concerning the linear solver
!                         See subroutine solvelbf
!     ivecfem        i    Sequence number of FEM solution vector
!     ivecreac       i    Sequence number of the reaction force vector with
!                         respect to array isol (if >0)
!     ivectr         i    Sequence number of the solution vector with respect to
!                         array isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinvec(2)
      double precision rinvec(2)

!     iinvec         User input array for subroutine MANVEC
!                    For a description see subroutine MANVEC
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIDAXPY       Overwrite double precision dy with double precision
!                    da*dx + dy
!     PRININ1        print 2d integer array
!     SEPBUILD       Build matrix and right-hand side
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
!     SEPLOCTRN      Compute (back-)transformation in case of local transform
!     SEPMANVEC      Vector manipulation subroutine
!     SEPMAVER       perform matrix vector multiplication
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0089reac' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0089reac'
         write(irefwr,1) 'ivecreac, iinsol(10), iinsol(13), iinsol(14)', &
                          ivecreac, iinsol(10), iinsol(13), iinsol(14)
         write(irefwr,1) 'ivectr', ivectr
         call prinin1 ( isolut2d, max(ivectr,ivecreac), 5, 'isolut2d' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( iinsol(13)>0 ) then

!     --- Compute reaction force by matrix vector multiplication: 9

         call sepmaver ( stiffmat, isolut2d(ivectr), &
                         isolut2d(iinsol(13)), 9 )

      end if
      if ( ierror/=0 ) go to 1000

      if ( iinsol(14)>0 .and. iinsol(10)>0 ) then

!     --- Subtract the contribution of the boundary matrix (i.e. sigma nt)
!         from the local vector
!         First transform the solution vector in a local coordinate system

         call seploctrn ( 1, isolut2d(ivectr) )

!        --- Next matrix vector multiplication

         iinbldlc(2) = 6   ! Perform matrix vector multiplication for boundary
                           ! elements, use only element matrices
         iinbldlc(6) = -2  ! Use boundary elements only
         call sepfillsmf ( iinbldlc, irhsd, isolut2d(ivectr), &
                           isolut2d(ivectr) )
         call sepbuild

!        --- The solution vector and the result vector are transformed
!            back to the global (Cartesian) system
!            This must be done since the reaction force is always in
!            the global system

         call seploctrn ( 2, irhsd )
         call seploctrn ( 2, isolut2d(ivectr) )

!        --- Reset iinbld

         iinbldlc(6) = 0
         iinbldlc(2) = 1

!        --- Subtract this vector from the load vector

         iinvec(1) = 2
         iinvec(2) = 27
         rinvec(1) = 1d0
         rinvec(2) = -1d0
         call sepmanvec ( iinvec, rinvec, isolut2d(ivecreac), &
                          irhsd, isolut2d(ivecreac) )

      end if  ! ( iinsol(14)>0 )

      if ( ivecfem<0 ) then

!     --- Special case: ivecfem<0, add extra rhsd to reaction force vector
!         Since the extra rhsd corresponds to the local coordinate system
!         and the reaction force to the global one we have to transform
!         the extra rhsd

         call seploctrn ( 2, isolut2d(-ivecfem) )
         rhsdsp => ks(isolut2d(-ivecfem))%sol
         rhsd => ks(isolut2d(ivecreac))%sol
         rhsd = rhsdsp + rhsd

      end if  ! ( ivecfem<0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0089reac'

      end if
      call erclos ( 'to0089reac' )

      end subroutine to0089reac
