      subroutine toconct ( inpconct, changed )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 09-11-2012 New call to er0080
!        version  3.0  date 27-09-2010 Make work arrays local
!        version  2.5  date 21-07-2001 Revision
!
!   copyright (c) 1997-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute whether surfaces or curves make contact and if so compute
!     the points and elements in the contact surface
! **********************************************************************
!
!                       KEYWORDS
!
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpconct(*)
      logical changed

!     changed   o     If true the contact points have been changed
!     inpconct   i    In this array it is indicated how the contact surface
!                     must be computed
!                     Contents
!                     1: sequence_number of contact
!                     2: type of contact
!                        1: curves
!                        2: surfaces
!                     3: contact method
!                        1: contact is made if contact_distance<=0
!                     4: contact disable method
!                        1: contact is disabled if CONTACT_FORCE<=0
!                     5: numveclc
!                     6: sequence number of contact surface or curve
!                     7: sequence number of vector defining contact distance
!                     8: sequence number of vector defining contact force
!                     9: -
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iconct, iconctyp, iconctmt, numveclc, j, &
              iconctsr, iseqdist, itypvc, iseqforce, &
              idegfd, i, icondismt, nnodes, idum
      integer, pointer :: kprobloc1(:)
      double precision, pointer :: force(:), distance(:)
      logical first

!     distance       Contains the distance in the nodes of the contact surface
!     first          If true it is the first time the contact is filled
!     force      i   Contains the force component in the nodes of the contact
!                    surface
!     i              Counting variable
!     iconct         Sequence number of contact
!     iconctmt       Method to define contact
!                    1: contact is made if dist<0
!     iconctsr       Surface number where contact is made
!     iconctyp       Type of contacts possible
!                    1: curves
!                    2: surfaces
!     icondismt      Method to disable contact
!                    1: contact is disabled if force<0
!     idegfd         Degree of freedom in VECTOR(IVEC) that must be used
!     idum           General dummy variable
!     iseqdist       sequence number of distance vector in isolut2d
!     iseqforce      sequence number of force vector in isolut2d
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     j              Counting variable
!     kprobloc1      local array kprobloc
!     nnodes         Number of nodes in isolut2dnw
!     numveclc       Number of vectors stored in isolut2d
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSRNOD2       Store all nodes in the surfaces ifirst to ilast in array
!                    nodes
!     PRININ         print 1d integer array
!     SEPACTSOLBF1   compute pointers to degrees of freedom of vectors
!     SEPPRINKPROB   Print array KPROB
!     SEPSTORENEWSOL Find all degrees of freedom from the solution array or
!                    array of special structure as indicated by array inodes and
!                    store in output array. Real case
!     SEPSURNOD      Store all nodes in the surfaces ifirst to ilast in array
!                    nodes
!     TOCNCT22       Compute the elements in the contact surface
!     TOCONCT1       Compute the nodes in the contact surface
!     TOCONCT2       Compute the elements in the contact surface
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1054   Parameter in inpconct out of range
!    1055   Array in solution vector has not been filled
!    1059   number of degrees of freedom incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check input
!     Compute the nodes in the contact surface
!     Compute the distance in these points
!     Find all points in the contact surface that make contact
!     Find the contact elements
!     Store the information about the contact elements in KPROB
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toconct' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toconct'
         call prinin ( inpconct, 9, 'inpconct' )
  1      format ( a, 1x, (10i6) )

      end if

!     --- Check input

      iconct = inpconct(1)
      if ( iconct<1 .or. iconct>min(10,nsurfs) ) then

!     --- iconct out of range

         call errint ( 1, 1 )
         call errint ( iconct, 2 )
         call errint ( 1, 3 )
         call errint ( 10, 4 )
         call errchr ( 'conctact_nr', 1 )
         call errsub ( 1054, 4, 0, 1 )

      end if
      iconctyp = inpconct(2)
      if ( iconctyp<2 .or. iconctyp>2 ) then

!     --- iconctyp out of range

         call errint ( 2, 1 )
         call errint ( iconctyp, 2 )
         call errint ( 2, 3 )
         call errint ( 2, 4 )
         call errchr ( 'conctact_type', 1 )
         call errsub ( 1054, 4, 0, 1 )

      end if
      iconctmt = inpconct(3)
      if ( iconctmt<1 .or. iconctmt>1 ) then

!     --- iconctmt out of range

         call errint ( 3, 1 )
         call errint ( iconctmt, 2 )
         call errint ( 1, 3 )
         call errint ( 1, 4 )
         call errchr ( 'conctact_method', 1 )
         call errsub ( 1054, 4, 0, 1 )

      end if
      icondismt = inpconct(4)
      if ( icondismt<1 .or. icondismt>1 ) then

!     --- icondismt out of range

         call errint ( 4, 1 )
         call errint ( icondismt, 2 )
         call errint ( 1, 3 )
         call errint ( 1, 4 )
         call errchr ( 'conctact_disable_method', 1 )
         call errsub ( 1054, 4, 0, 1 )

      end if
      numveclc = inpconct(5)
      if ( numveclc<1 .or. numveclc>100 ) then

!     --- numveclc out of range

         call errint ( 5, 1 )
         call errint ( numveclc, 2 )
         call errint ( 1, 3 )
         call errint ( 100, 4 )
         call errchr ( 'number of vectors', 1 )
         call errsub ( 1054, 4, 0, 1 )

      end if
      iconctsr = inpconct(6)
      if ( iconctsr<1 .or. iconctsr>nsurfs ) then

!     --- iconctsr out of range

         call errint ( 6, 1 )
         call errint ( iconctsr, 2 )
         call errint ( 1, 3 )
         call errint ( nsurfs, 4 )
         call errchr ( 'conctact_surface', 1 )
         call errsub ( 1054, 4, 0, 1 )

      end if
      iseqdist = inpconct(7)
      if ( iseqdist<1 .or. iseqdist>numveclc ) then

!     --- iseqdist out of range

         call errint ( 7, 1 )
         call errint ( iseqdist, 2 )
         call errint ( 1, 3 )
         call errint ( numveclc, 4 )
         call errchr ( 'vector nr contact distance', 1 )
         call errsub ( 1054, 4, 0, 1 )
         iseqdist = 1

      end if
      itypvc = ks(isolut2d(iseqdist))%typevec
      if ( itypvc/=110 .and. itypvc/=115 ) then
         call errint ( iseqdist, 1 )
         call errint ( itypvc, 2 )
         call errchr ( 'vector nr contact distance', 1 )
         call errsub ( 1055, 2, 0, 1 )
      end if
      iseqforce = inpconct(8)
      if ( iseqforce<1 .or. iseqforce>numveclc ) then

!     --- iseqforce out of range

         call errint ( 8, 1 )
         call errint ( iseqforce, 2 )
         call errint ( 1, 3 )
         call errint ( numveclc, 4 )
         call errchr ( 'vector nr contact force', 1 )
         call errsub ( 1054, 4, 0, 1 )
         iseqforce = 1

      end if
      itypvc = ks(isolut2d(iseqforce))%typevec
      if ( itypvc/=110 .and. itypvc/=115 ) then
         call errint ( iseqforce, 1 )
         call errint ( itypvc, 2 )
         call errchr ( 'vector nr contact force', 1 )
         call errsub ( 1055, 2, 0, 1 )
      end if

      if ( debug ) then

!     --- Debug information

          write(irefwr,*) 'npoint, nsurfs, iconct, iconctyp, iconctmt ', &
                           npoint, nsurfs, iconct, iconctyp, iconctmt
          write(irefwr,*) 'icondismt, numveclc, iconctsr, iseqdist ', &
                           icondismt, numveclc, iconctsr, iseqdist
          write(irefwr,*) 'itypvc, iseqforce, iprob, iseqforce ', &
                           itypvc, iseqforce, iprob, iseqforce

      end if

!     --- First check if kprobloc(46) has already been filled

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(1)
      kprobloc => kprobpart%kprobloc
      if ( debug ) write(irefwr,1) 'kprobloc(46), npoint', kprobloc(46), npoint
      if ( kprobloc(46)==0 ) then

!     --- First time contact is made, define kprobv and kprobw

         if ( associated(kp(iseqkp)%kprobsub(1)%kprobvp1) ) &
            deallocate ( kp(iseqkp)%kprobsub(1)%kprobvp1, &
                         kp(iseqkp)%kprobsub(1)%kprobvp2 )

         allocate ( kp(iseqkp)%kprobsub(1)%kprobvp1(nsurfs,3), &
                    kp(iseqkp)%kprobsub(1)%kprobvp2(npoint) )
         kprobvp1 => kprobpart%kprobvp1
         kprobvp2 => kprobpart%kprobvp2
         kprobvp1 = 0
         kprobvp2 = 0
         kprobloc(46) = nsurfs

!        --- no volumes, then 2D-contact in 2D-field: use kmesh C

         if ( nvolms==0 ) then
            length = size ( kmeshc )+4*nsurfs
         else
            length = size ( kmeshn )+2*nsurfs
         end if

         if ( associated(kp(iseqkp)%kprobsub(1)%kprobwp1) ) &
            deallocate ( kp(iseqkp)%kprobsub(1)%kprobwp1, &
                         kp(iseqkp)%kprobsub(1)%kprobwp2 )

         allocate ( kp(iseqkp)%kprobsub(1)%kprobwp1(nsurfs,4), &
                    kp(iseqkp)%kprobsub(1)%kprobwp2(length) )
         kprobwp1 => kprobpart%kprobwp1
         kprobwp2 => kprobpart%kprobwp2
         kprobwp1 = 0
         kprobwp2 = 0
         kprobloc(47) = nsurfs

!        --- Clear arrays

         kprobloc(48) = 0
         kprobloc(49) = 0
         first = .true.

      else

!     --- Activate arrays

         first = .false.

      end if

      do iprob = 1, nprob
         kprobloc1 => kp(iseqkp)%kprobsub(iprob)%kprobloc
         kprobloc1(46:49) = kprobloc(46:49)
      end do  ! iprob = 1, nprob

!     --- no volumes, then 2D-contact in 2D-field

      if ( nvolms==0 ) then

         inelem = nelem

      else

         inelem = 0
         do i = 0, nsurfs-1
            inelem = max(inelem,kmeshn(nsurfs+1+i))
         end do

      end if

      allocate ( nodes(npoint), force(npoint), distance(npoint), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, 3*npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting addresses

      forcevec => ks(isolut2d(iseqforce))%sol
      distvec => ks(isolut2d(iseqdist))%sol

!     --- Compute the nodes in the contact surface

      if ( nvolms==0 ) then

!     --- no volumes, then 2D-contact in 2D-field

         call ersrnod2 ( iconctsr, iconctsr, nodes, nnodes )

      else

!     --- no volumes, then contact in 3D-field

         nnodes = npoint
         call sepsurnod ( iconctsr, iconctsr, nodes, nnodes )

      end if

!     --- Compute the force in these points

      idegfd = 1
      call sepactsolbf1 ( isolut2d(iseqforce) )
      if ( nphysi>1 ) then

!     --- More than one degree of freedom in force vector

         call errint ( nphysi, 1 )
         call errint ( 1, 2 )
         call errchr ( 'contact force', 1 )
         call errsub ( 1059, 2, 0, 1 )

      end if

      usol => forcevec
      unew => force
      call sepstorenewsol ( idegfd, 1, nodes, nnodes )

!     --- Compute the distance in these points

      call sepactsolbf1 ( isolut2d(iseqdist) )
      if ( nphysi>1 ) then

!     --- More than one degree of freedom in distance vector

         call errint ( nphysi, 1 )
         call errint ( 1, 2 )
         call errchr ( 'contact distance', 1 )
         call errsub ( 1059, 2, 0, 1 )

      end if
      usol => distvec
      unew => distance
      call sepstorenewsol ( idegfd, 1, nodes, nnodes )

!     --- Find all points in the contact surface that make contact

      call toconct1 ( inpconct, force, distance, &
                      nodes, nnodes, changed )
      if ( first ) changed = .true.

      deallocate ( nodes, force, distance, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

!     --- Find the contact elements

      if ( changed ) then

!     --- The contact points have been changed

         if ( nvolms==0 ) then

!        --- no volumes, then 2D-contact in 2D-field

            call tocnct22 ( inpconct )

         else

            call toconct2 ( inpconct )

         end if

         kprobloc(49) = 1

      else

!     --- The contact points have not been changed

         kprobloc(49) = 0

      end if
      kprobloc(48) = max(kprobloc(48), inpconct(1))
      maxcontc = kprobloc(48)

!     --- Copy contact information for all problems

      do i = 2, kprobloc(40)
         kprobloc1 => kp(iseqkp)%kprobsub(i)%kprobloc
         kprobloc1(46:49) = kprobloc(46:49)
      end do

1000  if ( debug ) then

!     --- Debug information

         call sepprinkprob ( 26 )
         write(irefwr,*) 'changed ', changed
         write(irefwr,*) 'End toconct'

      end if
      call erclos ( 'toconct' )

      end subroutine toconct
