      subroutine msh002sb ( iinput, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 02-09-2010 Remove intarmsh
!        version  2.0  date 09-08-1999 Extra parameter ipointcurv
!        version  1.2  date 30-05-1999 Correction for spcurve
!
!   copyright (c) 1998-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check if there are any double curves, with only 2 user points on the
!    curves. If so make the number of elements along these curves at
!    least equal to 2
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), icurvs(*)

!     icurvs        i/o   array containing the number of points in the curves
!                         accumulated
!     iinput        i/o   integer input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurve, ifirst, ilast, iinp, isum, j, jcurnr, &
              nelm, nintcrv, i, jcurve, kcurve, n, ncrvs
      integer, allocatable :: iwork(:,:)
      logical found

!     found      If true there are curves with only one element on it
!     i          Counting variable
!     icurve     Type of curve
!     ifirst     First user point on curve
!     iinp       last position used in array iinput
!     ilast      Last user point on curve
!     isum       number of nodes already found
!     iwork      integer work array of length 4*ncurvs
!                In iwork(1,i) the first user point of a curve is
!                stored,
!                in iwork(2,i) the last user point provided the number
!                of elements along the curve is 1
!                in iwork(3-4,i) interface curve numbers coupled to
!                curve i are stored
!                In case of a curve to be copied in iwork(1,i) the
!                curve sequence number of the curve is stored with a
!                minus sign
!                If a double curve has been found, for the second we
!                set iwork(1,i) equal to -2*ncurvs
!     j          Counting variable
!     jcurnr     |icurnr| i.e. absolute value of curve sequence number
!     jcurve     Curve number
!     kcurve     abs(jcurve)
!     n          number of elements on a curve
!     ncrvs      number of curves the curve consists of
!     nelm       number of elements along a curve
!     nintcrv    number of interface curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Store initial point and end point of curves in array iwork(1-2,i)
!    For all curves that have more than one element on it make iwork(1-2,i) = 0
!    found := false
!    For icurnr := 1 (1) ncurvs do
!       If ( iwork(1,1)#0 ) then
!          For jcurnr := icurnr+1 (1) ncurvs do
!             if ( begin and end point of icurnr and jcurnr equal ) then
!                make number of elements equal to 2
!       if ( found and curve is curve of curves ) then
!          adapt icurvs if necessary
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh002sb' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh002sb'
         write(irefwr,1) 'ncurvs', ncurvs
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(4,ncurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, 4*ncurvs, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Store initial point and end point of curves in array iwork(1-2,i)
!         For all curves that have more than one element on it,
!         make iwork(1-2,i) = 0

      isum = 0
      found = .false.
      do icurnr = 1, ncurvs
         iinp = ipointcurv(icurnr)
         icurve = iinput(iinp)
         nelm = icurvs(icurnr)-isum-1
         if ( icurve==0 ) then

!        --- empty curve, skip

            iinp = iinp + 1
            iwork(1,icurnr) = 0
            iwork(2,icurnr) = 0

         else

!        --- non-empty curve

            if ( nelm/=1 ) then

!           --- Number of elements more than one, skip this curve

               iwork(1,icurnr) = 0
               iwork(2,icurnr) = 0

            else

!           --- Number of elements equal to one, store begin and end point
!               for this curve

               if ( icurve<=3 .or. icurve==5 .or. icurve==6 .or. &
                    icurve==8 .or. icurve==9 .or. icurve==14 ) then
                  iwork(1,icurnr) = iinput(iinp+4)
                  iwork(2,icurnr) = iinput(iinp+5)
                  found = .true.
               else if ( icurve==4 ) then
                  iwork(1,icurnr) = iinput(iinp+5)
                  iwork(2,icurnr) = iinput(iinp+6)
                  found = .true.
               else if ( icurve==10 .or. icurve==11 .or. &
                         icurve==13 ) then

!              --- curve must be copied from other curve, store that curve
!                  number

                  jcurnr = iinput(iinp+4)
                  iwork(1,icurnr) = -jcurnr
                  iwork(2,icurnr) = 0

               end if

            end if
            isum = isum+nelm+1

         end if

      end do
      if ( .not. found ) go to 1000

      do j = 1, ncurvs
         iwork(3,j) = 0
         iwork(4,j) = 0
      end do
      if ( iinp11>0 ) then

!     --- iinp11>0, there are interface elements
!         Store for each curve its neighbouring curve(s) in iwork(3-4,.)

         nintcrv = iinput(iinp11+1)
         iinp = iinp11+3
         do j = 1, nintcrv
            ifirst = abs(iinput(iinp+2))
            ilast  = abs(iinput(iinp+3))
            iinp = iinp+4
            if ( iwork(3,ifirst)==0 ) then
               iwork(3,ifirst) = ilast
            else
               iwork(4,ifirst) = ilast
            end if
            if ( iwork(3,ilast)==0 ) then
               iwork(3,ilast) = ifirst
            else
               iwork(4,ilast) = ifirst
            end if
         end do

      end if

!     --- For icurnr := 1 (1) ncurvs do

      found = .false.
      do icurnr = 1, ncurvs
         iinp = ipointcurv(icurnr)
         icurve = iinput(iinp)
         if ( iwork(1,icurnr)>0 ) then

!        --- For jcurnr := icurnr+1 (1) ncurvs do

            ifirst = iwork(1,icurnr)
            ilast = iwork(2,icurnr)

            do jcurnr = icurnr+1, ncurvs
               if ( iwork(1,jcurnr)==ifirst .and. &
                    iwork(2,jcurnr)==ilast .or. &
                    iwork(2,jcurnr)==ifirst .and. &
                    iwork(1,jcurnr)==ilast ) then

!              --- begin and end point of icurnr and jcurnr equal
!                  make number of elements equal to 2

                  if ( iinput(iinp+2)==1 ) then

!                 --- Increasing is allowed only once

                     iinput(iinp+2) = 2
                     do j = icurnr, ncurvs
                        icurvs(j) = icurvs(j)+1
                     end do
                     found = .true.

                  end if

!                 --- mark jcurnr since he has to be doubled too

                  iwork(1,jcurnr) = -2*ncurvs

!                 --- Check if there is an interface that also must be doubled

                  do j = 3, 4
                     if ( iwork(j,icurnr)>0 ) &
                        iwork(1,iwork(j,icurnr)) = -2*ncurvs
                  end do

               end if

            end do

         else if ( iwork(1,icurnr)<-ncurvs ) then

!        --- This curve is a doubled curve with one element
!            Increase the number of elements

            if ( iinput(iinp+2)==1 ) then

!           --- Increasing is allowed only once

               iinput(iinp+2) = 2
               do j = icurnr, ncurvs
                  icurvs(j) = icurvs(j)+1
               end do

            end if

         else if ( iwork(1,icurnr)<0 ) then

!        --- Curve must be copied from other curve

            jcurnr = -iwork(1,icurnr)
            if ( jcurnr>1 ) then
               nelm = icurvs(jcurnr)-icurvs(jcurnr-1)
            else
               nelm = icurvs(jcurnr)
            end if
            if ( nelm==3 ) then
               iinput(iinp+2) = 2
               do j = icurnr, ncurvs
                  icurvs(j) = icurvs(j)+1
               end do
            end if
         end if
         if ( found .and. icurve==7 .or. icurve==14 ) then

!        --- Curve of curves, if one of the subcurves is doubled, adapt this
!            curve also
!            Find the number of subcurves

            nelm = 1
            if ( icurve==7 ) then
               ncrvs = iinput(iinp+3)
            else
               ncrvs = iinput(iinp+6)
            end if

!           --- For all subcurves do

            do i = 1, ncrvs

               if ( icurve==7 ) then
                  jcurve = iinput(iinp+i+3)
               else
                  jcurve = iinput(iinp+i+6)
               end if
               kcurve = abs(jcurve)

!              --- detect number of elements for subcurve

               if ( kcurve==1 ) then
                  n = icurvs(1)
               else
                  n = icurvs(kcurve) - icurvs(kcurve-1)
               end if
               nelm = nelm+n-1

          end do

!         --- Set n equal to the new number of elements minus the old
!             number of elements

            n = nelm-icurvs(icurnr)+icurvs(icurnr-1)
            if ( n>0 ) then

!           --- n>0, hence some of the subcurves have been extended with nodes
!               adapt icurvs

               do j = icurnr, ncurvs
                  icurvs(j) = icurvs(j)+n
               end do

            end if

         end if

      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh002sb'

      end if  ! ( debug )
      call erclos ( 'msh002sb' )

      end subroutine msh002sb
