      subroutine msh014 ( iinput, inpold, ipoint, ncurvs, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version 10.1  date 10-02-2015 Extension with circ with extra user pnts
!        version 10.0  date 21-11-2014 New parameter list
!        version  9.9  date 15-08-2009 Extension with ellips
!
!   copyright (c) 1982-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill array iinput with information of curves by rearranging the
!    information in INPOLD
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), inpold(*), ipoint(*), ncurvs, ndim

!     iinput   o   new array iinput
!     inpold   i   Array IINPUT at input, this array must be rearranged
!     ipoint   i   pointer array, see mshrd1
!     ncurvs   i   number of curves
!     ndim     i   dimension of space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iinp, icurve, n, j, jinp, nwarnlc, ncrvs, jcurve, kcurve, &
              kinp, k, ic, ishape

!     i              Counting variable
!     ic             Curve number
!     icurve         Curve number
!     iinp           Position in array INPOLD
!     ishape         Shape number of elements along curve
!     j              Counting variable
!     jcurve         Subcurve number
!     jinp           Position in array IINPUT
!     k              Counting variable
!     kcurve         |jcurve|
!     kinp           Position in array IINPUT
!     n              Number of numbers stored for a specific curve
!     ncrvs          number of curves a curve consists of
!     nwarnlc          Number of warnings
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     MSH046         Raise iinp as function of icurve
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     124   code for generation of curves (icurve) incorrect
!     143   Curve has not been defined
!    1031   Shape numbers are not constant along curve of type spcurve
!    1032   Sub curve of spline curve is composite curve
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh014' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh014'
         write(irefwr,1) 'ncurvs', ncurvs
         call prinin ( ipoint, ncurvs, 'ipoint' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      jinp = 15
      nwarnlc = 0
      do i = 1, ncurvs
         iinp = ipoint(i)

         if ( debug ) write(irefwr,1) 'i, iinp', i, iinp
         if ( iinp==0 ) then

!        --- curve i has not been defined

            nwarnlc = nwarnlc + 1
            call errint ( i, nwarnlc )
            iinput(jinp+1) = 0
            jinp = jinp + 1

         else

!        --- curve i has been defined

            icurve = inpold(iinp)
            if ( debug ) write(irefwr,1) 'icurve, iinp', icurve, iinp
            select case ( icurve )

               case ( 0, 1, 8, 18, 19, 20 )

!              --- icurve = 1, 8, 18, 19, 20

                  n = 6

               case ( 2 )

!              --- icurve = 2

                  n = 4+inpold(iinp+2)+1

               case ( 3, 9, 15, 22 )

!              --- icurve = 3, 5, 9, 15, 22

                  n = 7

               case ( 5 )

!              --- icurve = 5

                  n = 5+max(2,inpold(iinp+3))

               case ( 4, 12, 21 )

!              --- icurve =  4, 12: splines
!                  icurve = 21    : framecurve

                  n = 6+inpold(iinp+5)
                  if ( icurve>4 ) n = n+1

               case ( 6, 16, 23 )

!              --- icurve = 6, 16, 23

                  n = 8

               case ( 7 )

!              --- icurve = 7

                  n = 4+inpold(iinp+3)

               case ( 10, 11, 13 )

!              --- icurve = 10, 11, 13

                  n = 5+inpold(iinp+3)

               case ( 14 )

!              --- icurve = 14, spline of curves
!                  Fill ishape and check previous curves

                  ncrvs = inpold(iinp+6)
                  n = 7+ncrvs
                  do j = 1, ncrvs
                     jcurve = inpold(iinp+j+6)
                     kcurve = abs(jcurve)
                     kinp = 1
                     do k = 1, kcurve-1
                        ic = inpold(kinp)
                        call msh046 ( inpold, kinp, ic )
                     end do
                     ic = inpold(kinp)
                     if ( ic==7 .or. ic==14 ) then

!                    --- Subcurve is composite curve

                        call errint ( j, 1 )
                        call errint ( i, 2 )
                        call errint ( jcurve, 3 )
                        call errsub ( 1032, 3, 0, 0 )

                     end if

                     if ( j==1 ) then

!                    --- first subcurve

                        ishape = inpold(kinp+1)
                        if ( ic==4 .or. ic==12 ) then

!                       --- first subcurve is of type spline

                           inpold(iinp+4) = inpold(kinp+6)

                        else if ( ic==10 .or. ic==11 .or. &
                                  ic==13 ) then

!                       --- first subcurve is of type copy

                           inpold(iinp+4) = inpold(kinp+5)

                        else

                           inpold(iinp+4) = inpold(kinp+4)

                        end if

                     else

!                    --- next subcurve

                        if ( ishape/=inpold(kinp+1) ) then

!                       --- shape number not constant along different parts of
!                           composite curve

                           call errint ( i, 1 )
                           call errint ( ishape, 2 )
                           call errint ( j, 3 )
                           call errint ( jcurve, 4 )
                           call errint ( inpold(kinp+1), 5 )
                           call errsub ( 1031, 5, 0, 0 )

                        end if

                     end if

                     inpold(iinp+1) = ishape
                     if ( ic==4 .or. ic==12 ) then

!                    --- last subcurve is of type spline

                        inpold(iinp+5) = inpold(kinp+6+inpold(kinp+5))

                     else if ( ic==10 .or. ic==11 .or. &
                               ic==13 ) then

!                    --- last subcurve is of type copy

                        inpold(iinp+5) = inpold(kinp+6)

                     else

                        inpold(iinp+5) = inpold(kinp+5)

                     end if

                  end do

               case ( 17 )

!              --- icurve = 17

                  n = 4+ndim

               case ( 24 )

!              --- icurve = 24

                  n = 5+max(2,inpold(iinp+3))

               case default

!              --- Curve type not recognized

                  call errint ( icurve, 1 )
                  call errint ( 23, 2 )
                  call errsub ( 124, 2, 0, 0 )
                  go to 1000

            end select  ! case ( icurve )

            iinp = iinp-1
            iinput(jinp+1:jinp+n) = inpold(iinp+1:iinp+n)
            jinp = jinp+n

         end if

      end do

      iinput(3) = ncurvs
      if ( nwarnlc>0 ) call errwar ( 143, nwarnlc, 0, 0 )

1000  call erclos ( 'msh014' )

      if ( debug ) then

!     --- Debug information

         call prinin ( iinput, jinp, 'iinput' )
         write(irefwr,*) 'End msh014'

      end if

      end
