      subroutine msh017 ( kbndpt, mapar, iinput, isubsf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.2  date 12-03-2015 Use subroutines
!        version  8.1  date 30-11-2014 Change isurf 23 in 99
!        version  8.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1984-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill the arrays mapar and kbndpt with the global numbers of the nodal
!    points for a surface
!    Parts of kmesh m and kmesh l corresponding to this surface are also filled
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), mapar(*), kbndpt(*), isubsf

!     iinput     i    integer input array
!     isubsf     i    actual (sub)surface number
!     kbndpt    i/o   array containing at input the local nodal point numbers of
!                     the boundary
!                     At output the global numbers are stored in kbndpt
!     mapar      o    Integer array of length npoint containing the global nodal
!                     point numbers at output
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istar3, istype, ndefsf, istart, iknpt, &
              jknpt, kknpt, indkbndpt, indfor, iprec, kinp, &
              iposiinpcrv, iposiinppnt
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     iknpt          Nodal point number
!     indfor         First indkbndpt of a closed curve
!     indkbndpt      Next position in kbndpt
!     iposiinpcrv    Starting address in array iinput with respect to the
!                    extra user curves (internal curves)
!     iposiinppnt    Starting address in array iinput with respect to the
!                    extra user user points (internal points)
!     iprec          Surface number of original surface to be copied
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         Refers to position in KBNDPT
!     istype         Type of surface generator used (istype=10: pipe surface)
!     iwork          Integer work array of length ncurvs
!                    In iwork for each curve that already has been treated the
!                    starting address of that curve in KBNDPT is stored
!                    provided with a sign indicating in which direction
!                    the curve is used
!     jknpt          Nodal point number
!     kinp           Position in iinput
!     kknpt          Nodal point number
!     ndefsf         Number of curves in surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH017EXT      Fill the arrays kmeshm, kmeshl and kbndpt partly for
!                    internal curves
!     MSH017KBN      Fill the arrays kmeshm, kmeshl and kbndpt partly for
!                    external curves
!     MSH017KMSM     Adapt arrays kmeshm, kmeshl and kbndpt for external curves
!     MSH017KMSMEXT  Adapt arrays kmeshm, kmeshl and kbndpt for internal curves
!     PR0008         Print information of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Clear MAPAR and IWORK
!      Refer boundary points in MAPAR to the corresponding positions in KBNDPT
!      with a negative address
!      Refer double points in KBNDPT to their first appearance in KBNDPT with
!      negative values.
!      Fill part of KBNDPT (prescribed points and double curves)
!      Fill array MAPAR
!      Fill KBNDPT for negative values
!      Fill arrays KMESHM and KMESHL for the surface
!      Scheme:
!      MAPAR(1:NPOINT) := 0;   IWORK(1:NCURVS) := 0
!      Set references:
!      For i := 1 (1) NBNDPT
!         check:
!         SEL  MAPAR(KBNDPT(i)) = 0
!                   MAPAR(KBNDPT(i)) = -i
!                   KBNDPT(i) = 0
!         ALT  MAPAR(KBNDPT(i)) # 0
!                   KBNDPT(i) = MAPAR(KBNDPT(i))
!         END check
!      END for
!      Fill part of KBNDPT ( precribed points and double curves):
!      For CURVE := 1 (1) Number of curves in surface
!          For SUBCURVE := 1 (1) number of subcurves in curve
!              Check:
!              SEL subcurve has been used before
!                     Verify that other direction has been used.
!                     Otherwise error.
!                     Refer all internal points of the curve to positions in
!                     KBNDPT corresponding to first appearance of the curve
!                     (negative address)
!                     Refer begin and end point of the curve to position in
!                     KBNDPT corresponding to their first appearance, except if
!                     the point refer to the present position
!              ALT subcurve has not been used before
!                     Set starting address of curve in IWORK with direction
!                     For all nodes NODE in subcurve
!                         Fill KBNDPT if NODE has already been filled in KMESH
!                         part m or l
!                     END for
!              END check
!          END for subcurve
!      END for curve
!      Fill array MAPAR:
!      For i := 1 (1) NPOINT
!          Check:
!          SEL MAPAR(i) = 0
!                 put INODP into MAPAR(i);  raise INODP
!          ALT MAPAR(i) # 0  (i.e. refers to KBNDPT)
!                 test:
!                 SEL KBNDPT(-MAPAR(i)) = 0
!                        Put INODP in KBNDPT and MAPAR;  raise INODP
!                 ALT KBNDPT(-MAPAR(i))>0
!                        Put node in MAPAR
!                 ALT KBNDPT(-MAPAR(i))<0
!                        check preceding KBNDPT:
!                        SEL KBNDPT(-KBNDPT(-MAPAR(i))) = 0
!                               Put INODP in KBNDPT (twice) and MAPAR;
!                               raise INODP
!                        ALT KBNDPT(-KBNDPT(-MAPAR(i)))>0
!                               Put node in KBNDPT and MAPAR
!                        END check preceding KBNDPT
!                 END test
!          END check
!      END for i
!      Fill KBNDPT for negative values
!      Fill KMESHL and KMESHM
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh017' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh017'
  1      format ( a, 1x, (10i6) )

      end if
      iprec  = 0
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'isubsf', isubsf
         call prinin ( iinput, 10, 'iinput' )
         write(irefwr,1) 'nuspnt, ncurvs, nsurfs, nbndpt, npoint', &
                          nuspnt, ncurvs, nsurfs, nbndpt, npoint
         call prinin ( kbndpt, nbndpt, 'kbndpt/1' )
         call pr0008 ( kmeshm, ncurvs )

      end if

      allocate ( iwork(ncurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, ncurvs, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- clear mapar and iwork

      mapar(1:npoint) = 0
      iwork(1:ncurvs) = 0

!     --- initialize mapar

      do i = 1, nbndpt
         if ( mapar(kbndpt(i))==0 ) then

!        --- mapar(kbndpt(i)) = 0, i.e. node has not been filled in mapar
!                                      Set reference in MAPAR, clear KBNDPT

            mapar(kbndpt(i)) = -i
            kbndpt (i) = 0

         else

!        --- mapar(kbndpt(i)) # 0, i.e. node has been filled in mapar
!                                     Set reference in KBNDPT

            kbndpt(i) = mapar(kbndpt(i))

         end if

      end do

      if ( debug ) call prinin ( kbndpt, nbndpt, 'kbndpt/2' )

!     --- fill kbndpt for prescribed unknowns

      istar3  = kmeshm(2)-1
      istype  = iinput(1)
      if ( debug ) write(irefwr,1) 'istype', istype
      if ( istype==8 .or. istype==9 .or. istype==14 ) then
         iprec = kmeshs(nsurfs+5+(isubsf-1)*8)
         istype = kmeshs(nsurfs+2+(iprec-1)*8)
      end if
      ndefsf  = mod(iinput(3),10000)
      if ( debug ) write(irefwr,1) 'istype, iprec, ndefsf', &
                                    istype, iprec, ndefsf

      if ( istype==22 .or. istype==99 ) then

!     --- For type 99, we have to fill the internal nodes into
!         and curves
!         Compute position of userpoints and curves in iinput

         kinp = ndefsf+3
         nextrapnts = iinput(kinp+1)
         nextracrvs = iinput(kinp+2)
         iposiinppnt = kinp+3
         iposiinpcrv = iposiinppnt+nextrapnts

      else

         nextrapnts = 0
         nextracrvs = 0
         iposiinppnt = 1
         iposiinpcrv = 1

      end if  ! ( itype==99 )

!     --- In the case of a pipe surface the number of curves may be three or
!         four
!         If there are three curves internally four curves are assumed with
!         the third and fourth curve identical
!         Therefore we set ndefsf equal to 4 for a pipe surface and in the case
!         of three curves we repeat the last one

      if ( istype==10 ) ndefsf = 4

      call msh017kbn ( ndefsf, istype, iinput, kelmq, kmeshq, &
                       kmeshm, kmeshl, kbndpt, indkbndpt, isubsf, &
                       iwork, istart, istar3, indfor )

      if ( debug ) then
         call prinin ( kbndpt, nbndpt, 'kbndpt/3' )
         call prinin ( mapar, npoint, 'mapar/1' )
         call prinin ( kmeshl, nuspnt, 'kmeshl' )
      end if  ! ( debug )

      if ( nextracrvs>0 ) then

!     --- There are extra curves, fill arrays kmeshm, kmeshl and kbndpt

         call msh017ext ( nextracrvs, iinput(iposiinpcrv), kmeshm, &
                          kmeshl, kbndpt, indkbndpt, iwork, istar3 )

         if ( debug ) call prinin ( kbndpt, nbndpt, 'kbndpt/4' )

      end if  ! ( nextracrvs>0 )

!     --- fill array mapar

      do i = 1, npoint
         iknpt = mapar(i)
         if ( iknpt==0 ) then

!        --- iknpt = 0  new point

            mapar(i) = inodp
            inodp = inodp+1

         else

!        --- iknpt<0  position in kbndpt

            iknpt = -iknpt
            jknpt = kbndpt(iknpt)
            if ( jknpt==0 ) then

!           --- jknpt = 0

               kbndpt(iknpt) = inodp
               mapar(i) = inodp
               inodp = inodp+1

            else if ( jknpt>0 ) then

!           --- jknpt>0

               mapar(i) = jknpt

            else

!           --- jknpt<0   double boundary

               kknpt = kbndpt(-jknpt)
               if ( kknpt>0 ) then

!              --- kbndpt(-jknpt)>0

                  kbndpt(iknpt) = kknpt
                  mapar(i) = kknpt

               else

!              --- kbndpt(-jknpt) = 0

                  kbndpt(-jknpt) = inodp
                  kbndpt(iknpt) = inodp
                  mapar(i) = inodp
                  inodp = inodp+1

               end if

            end if

         end if

      end do

      if ( debug ) then
         call prinin ( kbndpt, nbndpt, 'kbndpt/5' )
         call prinin ( mapar, npoint, 'mapar/2' )
      end if  ! ( debug )

!     --- Fill KBNDPT for negative values

      do i = 1, nbndpt
         if ( kbndpt(i)<0 ) kbndpt(i) = kbndpt(-kbndpt(i))
      end do
      if ( debug ) call prinin ( kbndpt, nbndpt, 'kbndpt/6' )

!     --- Fill KMESHL for negative values

      do i = 1, nuspnt
         if ( kmeshl(i)<0 ) kmeshl(i) = kbndpt(-kmeshl(i))
      end do

!     --- fill array kmeshl and kmeshm

      call msh017kmsm ( indkbndpt, iinput, kelmq, kmeshq, kmeshl, &
                        kmeshm, istar3, istype, kbndpt, indfor )
      if ( debug ) call prinin ( kbndpt, nbndpt, 'kbndpt/7' )

      if ( nextracrvs>0 ) then

!     --- There are extra curves, fill arrays kmeshm, kmeshl and kbndpt

         call msh017kmsmext ( indkbndpt, iinput(iposiinpcrv), kmeshl, &
                              kmeshm, istar3, kbndpt, nextracrvs )

      end if  ! ( nextracrvs>0 )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then
         call prinin ( kbndpt, nbndpt, 'kbndpt' )
         call pr0008 ( kmeshm, ncurvs )
         write(irefwr,*) 'End msh017'
      end if  ! ( debug )
      call erclos ( 'msh017' )

      end
