      subroutine msh017ext ( nextracrvs, iinput, kmeshm, kmeshl, &
                             kbndpt, indkbndpt, iwork, istar3 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the arrays kmeshm, kmeshl and kbndpt partly for internal curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     internal_curve
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nextracrvs, iinput(*), istar3
      integer, intent(inout) :: kmeshm(*), kmeshl(*), kbndpt(*), &
                                iwork(*), indkbndpt

!     iinput         i    integer input array
!                         Only the part with respect to internal curves
!     indkbndpt     i/o   Next position in kbndpt
!     istar3         i    Starting address of KMESHM part 3 in KMESH part M
!     iwork         i/o   integer work array
!                         See subroutine msh017
!     kbndpt        i/o   array containing the nodal point numbers of the
!                         boundary
!     kmeshl        i/o   KMESH part l, contains nodal point numbers of the
!                         user points
!     kmeshm        i/o   KMESH part m, contains nodal point numbers of the
!                         curves
!     nextracrvs     i    Number of extra (internal) curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, icurv, k1, k2, ih, &
                 idirec, ilastp, iknpt, ifirsp, jknpt, k

!     i              Counting variable
!     icurv          Curve number with direction
!     idirec         Defines in which direction the curves must be
!                    considered
!                    idirec = 1, from low to high, idirec = -1: reversed
!                    order
!     ifirsp         Indicates first point of curve
!     ih             Help variable to store a constant temporarily
!     iknpt          Nodal point number
!     ilastp         Indicates last point of curve
!     jknpt          Nodal point number
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh017ext' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh017ext'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ifirsp = 0
      ilastp = 0

!     --- Loop over standard curves

      do i = 1, nextracrvs

         icurv = iinput(i)
         if ( debug ) write(irefwr,1) 'i, icurv', i, icurv

         k1 = kmeshm(4+icurv)
         k2 = kmeshm(5+icurv)-1

         ih = indkbndpt-k1
         idirec = 1

!        --- Store first and last point in ifirsp resp. ilastp

         ilastp = kmeshm(istar3+k2)
         ifirsp = kmeshm(istar3+k1)

         if ( debug ) &
            write(irefwr,1) 'ifirsp, ilastp, idirec, iwork(icurv)', &
                             ifirsp, ilastp, idirec, iwork(icurv)

         iwork(icurv) = indkbndpt

         do k = k1, k2

            iknpt = kmeshm(istar3+k)

            if ( debug ) write(irefwr,1) 'k, iknpt', k, iknpt
            if ( iknpt<0 ) then

!           --- iknpt<0  check whether kmeshl has been filled

              iknpt = -iknpt
              jknpt = kmeshl(iknpt)

              if ( debug ) write(irefwr,1) 'jknpt, ih, k', jknpt, ih, k
              if ( jknpt==0 ) then

!             --- jknpt = 0, mark kmeshl as used

                 kmeshl(iknpt) = -ih-k*idirec

              else if ( jknpt<0 ) then

!             --- jknpt<0, kmeshl has been used
!                            mark point as double

                 kbndpt(ih+k*idirec) = jknpt

              else

!             --- jknpt>0  fill kmesh part m

                 kmeshm(istar3+k) = jknpt
                 iknpt = jknpt
                 kbndpt(ih+k*idirec) = iknpt

              end if

            else if ( iknpt>0 ) then

!           --- iknpt>0

              kbndpt(ih+k*idirec) = iknpt

            end if

         end do

!        --- Check if negative values of ifirsp and ilastp
!            i.e. values corresponding to user points that are not yet
!            initialized, should be made positive

         if ( ifirsp<0 ) then
            if ( kmeshl(-ifirsp)>0 ) ifirsp = kmeshl(-ifirsp)
         end if
         if ( ilastp<0 ) then
            if ( kmeshl(-ilastp)>0 ) ilastp = kmeshl(-ilastp)
         end if

         indkbndpt = indkbndpt+k2-k1+1

      end do  ! i = 1, nextracrvs

1000  call erclos ( 'msh017ext' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh017ext'

      end if  ! ( debug )

      end subroutine msh017ext
