      subroutine msh017kbn ( ndefsf, istype, iinput, ikkmsq, kmeshq, &
                             kmeshm, kmeshl, kbndpt, indkbndpt, isubsf, &
                             iwork, istart, istar3, indfor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the arrays kmeshm, kmeshl and kbndpt partly for external curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ndefsf, istype, iinput(*), ikkmsq, &
                             kmeshq(*), isubsf, istar3
      integer, intent(inout) :: kmeshm(*), kmeshl(*), kbndpt(*), &
                                iwork(*), istart, indfor
      integer, intent(out) :: indkbndpt

!     iinput         i    integer input array
!     ikkmsq         i    Indication if array kmesh part q has been filled (>0)
!                         or not ( = 0)
!     indfor        i/o   First index of a closed curve
!     indkbndpt      o    Next position in kbndpt
!     istar3         i    Starting address of KMESHM part 3 in KMESH part M
!     istart        i/o   Refers to position in KBNDPT
!     istype         i    Type of surface generator used (istype=10:
!                         pipe surface)
!     isubsf         i    Sequence number of subsurface
!     iwork         i/o   integer work array
!                         See subroutine msh017
!     kbndpt        i/o   array containing the nodal point numbers of the
!                         boundary
!     kmeshl        i/o   KMESH part l, contains nodal point numbers of the
!                         user points
!     kmeshm        i/o   KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshq         i    KMESH part q, contains the subcurve numbers of
!                         each curve
!     ndefsf         i    Number of curves in surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, kcurv, lcurv, nsubcv, ic, icurv, jcurv, k1, k2, ih, &
                 idirec, ilastp, nk, iknpt, ifirsp, jknpt, k
      logical :: loopfound

!     i              Counting variable
!     ic             Relative subcurve number
!     icurv          Curve number with direction
!     idirec         Defines in which direction the curves must be
!                    considered
!                    idirec = 1, from low to high, idirec = -1: reversed
!                    order
!     ifirsp         Indicates first point of curve
!     ih             Help variable to store a constant temporarily
!     iknpt          Nodal point number
!     ilastp         Indicates last point of curve
!     jcurv          Absolute value of curve number
!     jknpt          Nodal point number
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     kcurv          Curve number with direction
!     lcurv          |kcurv|
!     loopfound      If true end point found that is the same as initial point
!     nk             Number of points along a (sub)curve
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      53   curve number has been used more than once in a submesh, with the
!           same sign
!     801   Third and fourth curve of pipe surface are identical but have
!           opposite sign.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh017kbn' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh017kbn'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ifirsp = 0
      ilastp = 0
      loopfound = .false.
      indkbndpt = 1
      nsubcv = 0

!     --- Loop over standard curves

      do i = 1, ndefsf

         if ( istype/=10 .or. iinput(3)/=3 .or. i/=4 ) kcurv = iinput(3+i)
         lcurv = abs(kcurv)
         if ( ikkmsq/=0 ) nsubcv = kmeshq(lcurv+1) - kmeshq(lcurv)
         if ( debug ) &
            write(irefwr,1) 'i, kcurv, nsubcv', i, kcurv, nsubcv

!        --- Loop over subcurves

         do ic = 1, max(1, nsubcv)
            if ( nsubcv==0 ) then
               icurv = kcurv
            else
               if ( kcurv>0 ) then
                  icurv = kmeshq( kmeshq(lcurv)+ic )
               else
                  icurv = -kmeshq( kmeshq(lcurv+1)-ic+1 )
               end if
            end if
            jcurv = abs(icurv)
            k1 = kmeshm(4+jcurv)
            k2 = kmeshm(5+jcurv)-1
            if ( debug ) write(irefwr,1) 'ic, icurv', ic, icurv
            if ( icurv>0 ) then

!           --- icurve>0  copy forwards

              ih = indkbndpt-k1
              idirec = 1

            else

!           --- icurve<0  copy backwards

              ih = indkbndpt+k2
              idirec = -1

            end if

!           --- Store first and last point in ifirsp resp. ilastp

            if ( debug ) then
               write(irefwr,1) 'i, ifirsp', i, ifirsp
               write(irefwr,*) 'loopfound ', loopfound
            end if  ! ( debug )
            if ( loopfound .or. ifirsp==0 ) then

!           --- ifirsp has not yet been filled

               if ( idirec==1 ) then

!              --- idirec = 1, standard sequence (ifirsp corresponds to k1)

                  ifirsp = kmeshm(istar3+k1)

               else

!              --- idirec =-1, reversed sequence (ifirsp corresponds to k2)

                  ifirsp = kmeshm(istar3+k2)

               end if
               if ( loopfound .and. ifirsp/=ilastp ) then
                  kbndpt(indkbndpt) = -indfor
                  indkbndpt = indkbndpt+1
                  ih = ih + 1
               end if
               if ( i<=ndefsf ) indfor = indkbndpt
               loopfound = .false.

            end if

            if ( idirec==1 ) then

!           --- idirec = 1, standard sequence (ilastp corresponds to k2)

               ilastp = kmeshm(istar3+k2)

            else

!           --- idirec =-1, reversed sequence (ilastp corresponds to k1)

               ilastp = kmeshm(istar3+k1)

            end if
            if ( debug ) &
               write(irefwr,1) 'ifirsp, ilastp, idirec, iwork(jcurv)', &
                                ifirsp, ilastp, idirec, iwork(jcurv)

            if ( iwork(jcurv)/=0 ) then

!           --- iwork(jcurv) # 0

              if ( istype==10 ) then

!             --- istype = 10, pipe surface, last curve
!                              In this case the direction of both curves
!                              must be identical

                 if ( sign(1,idirec)/=sign(1,iwork(jcurv)) ) then
                    call errint ( isubsf, 1 )
                    call errint ( jcurv, 2 )
                    call errsub ( 801, 2, 0, 0)
                 end if
                 nk = k2-k1+1
                 istart = abs(iwork(jcurv))
                 do k = 1, nk
                    if ( (k==1 .or. k==nk) .and. &
                        indkbndpt+k-1==istart+k-1 ) goto 140
                    kbndpt(indkbndpt+k-1) = -istart-k+1
140              end do

              else

!             --- istype # 10, other type of surface
!                              In this case the direction of both curves
!                              must be opposite

                 if ( sign(1,idirec)==sign(1,iwork(jcurv)) ) then
                    call errint( isubsf, 1 )
                    call errint( jcurv, 2 )
                    call errsub ( 53, 2, 0, 0)
                 end if
                 nk = k2-k1+1
                 istart = abs(iwork(jcurv))+nk
                 do k = 1, nk
                    if ( (k==1 .or. k==nk) .and. &
                        indkbndpt+k-1==istart-k ) goto 150
                    kbndpt(indkbndpt+k-1) = -istart+k
150              end do

              end if

            else

!           --- iwork(jcurv) = 0

              iwork(jcurv) = idirec*indkbndpt

              do k = k1, k2
                 iknpt = kmeshm(istar3+k)

                 if ( debug ) write(irefwr,1) 'k, iknpt', k, iknpt
                 if ( iknpt<0 ) then

!                --- iknpt<0  check whether kmeshl has been filled

                   iknpt = -iknpt
                   jknpt = kmeshl(iknpt)
                   if ( jknpt==0 ) then

!                  --- jknpt = 0, mark kmeshl as used

                      kmeshl(iknpt) = -ih-k*idirec

                   else if ( jknpt<0 ) then

!                  --- jknpt<0, kmeshl has been used
!                                 mark point as double

                      kbndpt(ih+k*idirec) = jknpt

                   else

!                  --- jknpt>0  fill kmesh part m

                      kmeshm(istar3+k) = jknpt
                      iknpt = jknpt
                      kbndpt(ih+k*idirec) = iknpt

                   end if

                 else if ( iknpt>0 ) then

!                --- iknpt>0

                   kbndpt(ih+k*idirec) = iknpt

                 end if

              end do

            end if

!           --- Check if negative values of ifirsp and ilastp
!               i.e. values corresponding to user points that are not yet
!               initialized, should be made positive

            if ( ifirsp<0 ) then
               if ( kmeshl(-ifirsp)>0 ) ifirsp = kmeshl(-ifirsp)
            end if
            if ( ilastp<0 ) then
               if ( kmeshl(-ilastp)>0 ) ilastp = kmeshl(-ilastp)
            end if

!           --- Check if end of closed boundary is reached, if so raise
!               indkbndpt

            if ( istype==10 ) then

!           --- istype = 10, i.e. pipe surface, special treatment

              if ( ic==max(1,nsubcv) ) then

!             --- Last curve of a set, indicate that begin point and end point
!                 are equal

                 if ( i<=2 .and. (iinput(3)==3 .or. &
                    iinput(4)==iinput(7)) ) &
                      kbndpt(indkbndpt+k2-k1) = -indfor
                 ifirsp = 0
                 indkbndpt = indkbndpt+1

               end if

            else if ( (istype==3 .or. istype==19 .or. istype==22 .or. &
                       istype==99) .and. ilastp==ifirsp ) then

!           --- starting point of new boundary is equal to first point, &
!               this means that end of closed boundary has been found
!               This situation can only occur for istype = 3 (triangle)

               loopfound = .true.
               if ( i==ndefsf .and. ic==max(1,nsubcv) ) then
                  kbndpt(indkbndpt+k2-k1) = -indfor
                  indkbndpt  = indkbndpt + 1
                  ifirsp = 0
               end if

            end if

            indkbndpt = indkbndpt+k2-k1

         end do

      end do

1000  call erclos ( 'msh017kbn' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh017kbn'

      end if  ! ( debug )

      end subroutine msh017kbn
