      subroutine msh017kmsmext ( indkbndpt, iinput, kmeshl, kmeshm, &
                                 istar3, kbndpt, nextracrvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt arrays kmeshm, kmeshl and kbndpt for internal curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     internal_curve
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinput(*), istar3, nextracrvs, kbndpt(*)
      integer, intent(inout) :: kmeshl(*), kmeshm(*), indkbndpt

!     iinput         i    integer input array
!                         Only the part with respect to internal curves
!     indkbndpt     i/o   Next position in kbndpt
!     istar3         i    Starting address of KMESHM part 3 in KMESH part M
!     kbndpt         i    array containing the nodal point numbers of the
!                         boundary
!     kmeshl        i/o   KMESH part l, contains nodal point numbers of the
!                         user points
!     kmeshm        i/o   KMESH part m, contains nodal point numbers of the
!                         curves
!     nextracrvs     i    Number of extra (internal) curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ifirsp, ilastp, i, icurv, k1, k2, ih, idirec, &
                 iknpt, jknpt, k

!     i              Counting variable
!     icurv          Curve number with direction
!     idirec         Defines in which direction the curves must be
!                    considered
!                    idirec = 1, from low to high, idirec = -1: reversed
!                    order
!     ifirsp         Indicates first point of curve
!     ih             Help variable to store a constant temporarily
!     iknpt          Nodal point number
!     ilastp         Indicates last point of curve
!     jknpt          Nodal point number
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     617   User point corresponds to two different nodal point numbers.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh017kmsmext' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh017kmsmext'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nextracrvs

         icurv = iinput(i)
         if ( debug ) write(irefwr,1) 'i, icurv', i, icurv

         k1 = kmeshm(4+icurv)
         k2 = kmeshm(5+icurv)-1

         ih = indkbndpt
         idirec = 1
         ifirsp = kmeshm(istar3+k1)
         if ( ifirsp<0 ) ifirsp = kmeshl(-ifirsp)
         ilastp = kmeshm(istar3+k2)
         if ( ilastp<0 ) ilastp = kmeshl(-ilastp)

         if ( debug ) &
            write(irefwr,1) 'k1, k2, ih, ifirsp, ilastp', &
                             k1, k2, ih, ifirsp, ilastp

         do k = k1, k2
            iknpt = kbndpt(ih)
            ih = ih+idirec
            jknpt = kmeshm(istar3+k)

            if ( debug ) &
               write(irefwr,1) 'k, iknpt, ih, jknpt', &
                                k, iknpt, ih, jknpt

            if ( jknpt<0 ) then

!           --- jknpt<0  fill kmesh l and kmeshm

               if ( kmeshl(-jknpt)==0 ) then
                  kmeshl(-jknpt) = iknpt
               else if ( kmeshl(-jknpt)/= iknpt ) then
                  call errint( -jknpt, 1 )
                  call errint( iknpt, 2 )
                  call errint( kmeshl(-jknpt), 3 )
                  call errwar ( 617, 3, 0, 0 )
               end if
               kmeshm(istar3+k) = iknpt

            else if ( jknpt==0 ) then

!           --- jknpt = 0  fill kmeshm

               kmeshm(istar3+k) = iknpt

            end if

         end do

         indkbndpt = indkbndpt+k2-k1+1

      end do

1000  call erclos ( 'msh017kmsmext' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh017kmsmext'

      end if  ! ( debug )

      end subroutine msh017kmsmext
