      subroutine msh030circ ( rinput, nuspnt, iinput, ndim, iinp, np, &
                              x1, x2, sleng, curstr, icurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 10-02-2015 Use subroutine msh030arcleng
!        version  2.0  date 11-12-2014 Remove parameter irstr
!        version  1.0  date 31-07-2012
!
!   copyright (c) 2012-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     determine number of elements to be created along
!     circle with given coarseness
! **********************************************************************
!
!                       KEYWORDS
!
!     create_point
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nuspnt, iinput(*), ndim, iinp, np, icurnr
      double precision rinput(ndim,nuspnt), x1(ndim), x2(ndim), sleng, &
                       curstr(*)

!     curstr         o    in this array some geometrical information concerning
!                         the curves is stored
!                         for each curve 10 positions are available
!                         storage:  pos. 1  length of curve
!                         pos. 2-5: phi, r, xz, yz if ndim = 2 and
!                         pos. 2-6: phi, r, xz, yz, zz if ndim = 3
!     icurnr         i    Curve sequence number
!     iinp           i    starting position of curve input in array iinput
!     iinput         i    array containing the user input concerning the
!                         curve from position iinp
!     ndim           i    Dimension of the space.
!     np             o    Defines whether the number of points is odd (2) or
!                         even (3)
!     nuspnt         i    Number of user points
!     rinput         i    array containing on the first nuspnt*ndim positions
!                         coordinates , on position nuspnt+1 the unit edge
!                         length and on the next nuspnt positions the defined
!                         coarseness of the user points
!     sleng          o    length of the curve
!     x1             i    coordinates of first point
!     x2             i    coordinates of second point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j3
      double precision xt(ndim), dx(ndim), v(ndim), phi, r, xz(ndim), &
                       w(ndim), alpha, c2
      logical coinciding

!     alpha          help parameter
!     c2             Help constant (dx . dx)
!     coinciding     if true initial and end point coincide
!     dx             x2-x1
!     j3             Index of center point
!     phi            angle
!     r              distance to centroid
!     v              x1+x2
!     w              v/2 + xt
!     xt             coordinates of centroid
!     xz             work array to store coordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH030ARCLENG  Compute length of arc (part of circle)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     524   Coinciding initial and end point of arc in 3-D.
!    1783   User point number out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh030circ' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh030circ'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      j3 = iinput(iinp+6)
      if ( j3==0 .or. abs(j3)>nuspnt ) then
         call errint ( j3, 1 )
         call errint ( nuspnt, 2 )
         call errint ( icurnr, 3 )
         call errsub ( 1783, 3, 0, 0 )
      end if

      np = iinput(iinp+7)
      if ( debug ) write(irefwr,1) 'j3', j3

      xt = rinput(:,abs(j3))
      if ( debug ) write(irefwr,2) 'xt', xt
      dx = x1-x2
      if ( debug ) write(irefwr,2) 'dx', dx
      v = x1+x2

      coinciding = dot_product(dx,dx)<=sqreps*dot_product(v,v)

      if ( coinciding ) then

!     --- initial and end point are coinciding

         if ( debug ) write(irefwr,*) 'coinciding'
         if ( ndim==2 ) then

!        --- ndim = 2

            phi   = 2d0*pi
            r     = sqrt(dot_product(xt-x1,xt-x1))
            sleng = r*phi
            if ( j3<0 ) phi = -phi
            xz = xt

         else

!        --- ndim = 3: error

            call errint ( icurnr, 1 )
            call errsub ( 524, 1, 0, 0 )
            go to 1000

         end if  ! ( ndim==2 )

      else

!     --- initial and end point not coinciding

         if ( debug ) write(irefwr,*) 'not coinciding'

         c2 = dot_product(dx,dx)
         w = 0.5d0*v-xt
         alpha = dot_product(w,dx)/c2
         xz = xt+alpha*dx
         w = xz-x1
         r = sqrt(dot_product(w,w))

         if ( debug ) write(irefwr,2) 'c2, alpha, r', c2, alpha, r

         call msh030arcleng ( ndim, c2, r, phi, sleng, j3, &
                              x1, x2, xz, dx )

      end if  ! ( coinciding )

      curstr(2) = phi
      curstr(3) = r
      curstr(4:3+ndim) = xz(1:ndim)

1000  call erclos ( 'msh030circ' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh030circ'

      end if  ! ( debug )

      end
