      subroutine msh030prof ( rincor, iinput, iinp, icurnr, ndim, &
                              x1, x2, sleng, edgemn, np  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-03-2013 Correction declaration x
!        version  1.0  date 31-07-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     determine number of elements to be created along
!     profile with given coarseness
! **********************************************************************
!
!                       KEYWORDS
!
!     create_point
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, icurnr, ndim, np
      double precision rincor(2,*), x1(ndim), x2(ndim), sleng, edgemn

!     edgemn         i    Mean length of edges
!     icurnr         i    Curve sequence number
!     iinp           i    starting position of curve input in array iinput
!     iinput         i    array containing the user input concerning the
!                         curve from position iinp
!     ndim           i    Dimension of the space.
!     np             o    Defines whether the number of points is odd (2) or
!                         even (3)
!     rincor         i    Part of array rinput refering to coarseness
!     sleng          o    length of the curve
!     x1             i    coordinates of first point
!     x2             i    coordinates of second point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncheck, kmax, k
      double precision t0, t1, dt, x(ndim), xp(ndim), t, v(ndim), const, &
                       c1
      parameter ( const = 1.008930411365d0 )

!     c1             Inverse of const
!     const          constant to be used for correction of naca0012 formulas
!     dt             Step size of parameter t
!     k              Counting variable
!     kmax           Number of steps of parameter t
!     ncheck         Checking parameter
!     t              parameter t along curve
!     t0             Initial value of t
!     t1             End value of t
!     v              differnce between two vector
!     x              contains co-ordinates of point
!     xp             contains co-ordinates of prior point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision naca0012
      naca0012(t) = 0.6d0*(.2969d0*sqrt(t) +t * ( - 0.126d0 + t* &
                    (-0.3516d0 + t*(0.2843d0 - 0.1015d0*t))))

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     NACA0012       Defines formula for NACA0012 profile
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     423   initial and end parameter for curve generation are the same
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh030prof' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh030prof'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      t0 = rincor(1,icurnr)
      t1 = rincor(2,icurnr)
      np = iinput(iinp+7)
      c1 = 1d0/const

      dt = 0.1d0*(t1-t0)
      x  = 0d0
      ncheck = 10
      kmax = 10

      if ( debug ) then
         write(irefwr,1) 'np', np
         write(irefwr,2) 't0, t1, c1, dt', t0, t1, c1, dt
      end if  ! ( debug )

!     --- Loop

200   dt = 0.1d0*dt
      kmax = kmax*10
      if ( abs(dt)<=0d0 ) then

!     --- Error dt<=0

         call errint ( icurnr, 1 )
         call errsub ( 423, 1, 0, 0 )

      else

!     --- Standard case

         xp = x1
         sleng = 0d0
         do k = 1, kmax-1

            t = t0+k*dt
            x(2) = naca0012(t)*c1
            x(1) = t*c1
            v = x-xp
            sleng = sleng+sqrt(dot_product(v,v))
            xp = x

         end do  ! k = 1, kmax-1

         v = x2-xp
         sleng = sleng+sqrt(dot_product(v,v))

         if ( sleng/edgemn>ncheck) then

            ncheck = 10*ncheck
            goto 200

         end if

      end if  ! ( abs(dt)<=0d0 )

1000  call erclos ( 'msh030prof' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh030prof'

      end if  ! ( debug )

      end
