      subroutine msh038 ( iinput, lengc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 14-10-2002 Extra parameters kmeshq
!        version  3.1  date 07-08-1995 Extension with shape>0
!        version  3.0  date 20-10-1993 Extension for exclude boundary
!                                      of surfaces, new parameter list)
!
!   copyright (c) 1985-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    This subroutine checks the input for connected elements. It computes the
!    required length of these elements with respect to KMESH part c
!    MSH038 is closely related to MSH041, if one of them is changed, the other
!    must be changed also
! **********************************************************************
!
!                       KEYWORDS
!
!     connect
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), lengc

!     iinput         i    integer input array containing information of the
!                         connected elements
!     lengc         i/o   computed length of kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipoint, icurvs, isurfs, iinp, i, ielgrp, k, icrvn, &
              ileft, iright, istar3, idir1, idir2, n, j, &
              inpelm, ilefst, irigst, iexclu, nsubcv, &
              iwork(npoint+ncurvs)

!     i              Counting variable
!     icrvn          Curve sequence number
!     icurvs
!     idir1          ?
!     idir2          ?
!     ielgrp         element group number
!     iexclu         Indicates if begin and end points of curves must be
!                    excluded or not. Possible values in case of curves:
!                    0:  Do not exclude one of these points
!                    1:  Exclude begin point
!                    2:  Exclude end point
!                    3:  Exclude begin and end point
!                    And in case of surfaces:
!                    0:  Do not exclude boundaries
!                    1:  Exclude all external boundaries
!     iinp           last position used in array iinput
!     ilefst         Starting point of "left" surface in KMESH part n
!     ileft          User point, curve of surface number of "left" item to be
!                    connected
!     inpelm         number of points per element
!     ipoint         ?
!     iright         User point, curve of surface number of "right" item to be
!                    connected
!     irigst         Starting point of "right" surface in KMESH part n
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     isurfs         ?
!     iwork          Work array of length npoint+ncurvs
!                    This work array is only used in the case of surfaces
!                    to be connected. The first npoint positions are used
!                    to indicate if a nodal point has already been
!                    stored (>0) or not (0) or is excluded (<0)
!                    In this way double elements are avoided
!                    The positions npoint+1 to npoint+ncurvs are used to
!                    store the outer boundary.
!                    Inner curves are deleted from kmeshs
!     j              Counting variable
!     k              Counting variable
!     n              Number of elements created by mesh connect or twice this
!                    number in the case of surfaces
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH033         Fill array iwork for outer boundaries
!     SEPPRINKMESHM  Print the contents of array kmeshm
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     320   The number of user points connected is incorrect
!     321   The number of curves connected is incorrect
!     322   The number of surfaces connected is incorrect
!     324   user point number for connected element is incorrect
!     325   curve number for connected element is incorrect
!     326   surface number for connected element is incorrect
!     327   2 curves for connected element have different numbers of nodal
!           points.
!     558   The parameter l for the definition of essential boundary conditions
!           or connected elements has an incorrect value
!    1325   The left and right surface for connected elements are not
!           topological equivalent
!    1326   The left and right surface for connected elements are not
!           topological equivalent
!    1327   The left and right surface for connected elements have the same
!           surface number.
!    1437   Exclude parameter for surfaces incorrect
!    1442   excluded nodal point not in surface
!    1804   User point is linked with itself
!    1805   Curve is linked with itself
!    1806   Surface is linked with itself
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh038' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh038'
         write(irefwr,1)  'nuspnt, ncurvs, nsurfs, lengc, npoint', &
                           nuspnt, ncurvs, nsurfs, lengc, npoint
         call sepprinkmeshm
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ipoint = iinput(1)
      icurvs = iinput(2)
      isurfs = iinput(3)
      if ( ipoint<0 .or. ipoint>nuspnt ) then
         call errint(ipoint,1)
         call errint(nuspnt,2)
         call errsub ( 320, 2, 0, 0 )
      end if
      if ( icurvs<0 .or. icurvs>ncurvs ) then
         call errint(icurvs,1)
         call errint(ncurvs,2)
         call errsub ( 321, 2, 0, 0 )
      end if
      if ( isurfs<0 .or. isurfs>nsurfs ) then
         call errint(isurfs,1)
         call errint(nsurfs,2)
         call errsub ( 322, 2, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000
      iinp = 4
      do i = 1, ipoint
         ielgrp = iinput(iinp)
         ileft = iinput(iinp+1)
         if ( ileft<=0 .or. ileft>nuspnt ) then
            call errint(ileft,1)
            call errint(nuspnt,2)
            call errint(ielgrp,3)
            call errint(i,4)
            call errsub ( 324, 4, 0, 0 )
         end if
         iright = iinput(iinp+2)
         if ( iright<=0 .or. iright>nuspnt) then
            call errint(iright,1)
            call errint(nuspnt,2)
            call errint(ielgrp,3)
            call errint(i,4)
            call errsub ( 324, 4, 0, 0 )
         end if
         if ( ileft==iright ) then
            call errint ( ileft, 1 )
            call errsub ( 1804, 1, 0, 0 )
         end if
         lengc = lengc+2
         iinp = iinp+3
      end do

      istar3 = kmeshm(2)-1

!     --- Store curves to be connected into istoreleft and istoreright

      do i = 1, icurvs
         ielgrp = iinput(iinp)
         iexclu = iinput(iinp+1)/100
         ishape = iinput(iinp+1)-100*iexclu

         if ( debug ) &
            write(irefwr,1) 'i, ielgrp, iexclu, ishape', &
                             i, ielgrp, iexclu, ishape
         if ( iexclu<0 .or. iexclu>3 .or. &
              ishape>0 .and. iexclu>0 .or. &
              ishape>2 ) then

!        --- error 558: iexclu out of range

            call errint ( iexclu, 1 )
            call errint ( iinput(iinp+1), 2 )
            call errsub ( 558, 2, 0, 0 )

         end if
         ileft = iinput(iinp+2)
         idir1 = sign(1, ileft)
         ileft = abs(ileft)
         if ( ileft==0 .or. ileft>ncurvs ) then
            call errint(idir1*ileft,1)
            call errint(ncurvs,2)
            call errint(ielgrp,3)
            call errint(i,4)
            call errsub ( 325, 4, 0, 0 )
         end if
         iright = iinput(iinp+3)
         idir2 = sign(1,iright)
         iright = abs(iright)

         if ( debug ) &
            write(irefwr,1) 'ileft, iright, idir1, idir2', &
                             ileft, iright, idir1, idir2
         if ( iright==0 .or. iright>ncurvs ) then
            call errint(idir2*iright,1)
            call errint(ncurvs,2)
            call errint(ielgrp,3)
            call errint(i,4)
            call errsub ( 325, 4, 0, 0 )
         end if
         if ( ileft==iright ) then
            call errint ( ileft, 1 )
            call errsub ( 1805, 1, 0, 0 )
         end if
         nsubcv = 1
         if ( kelmq>0 ) nsubcv = kmeshq(ileft+1)-kmeshq(ileft)
         nsubcv = max(1,nsubcv)
         n = kmeshm(ileft+5)-kmeshm(ileft+4)+1-nsubcv

         nsubcv = 1
         if ( kelmq>0 ) nsubcv = kmeshq(iright+1)-kmeshq(iright)
         nsubcv = max(1,nsubcv)

         if ( n/=(kmeshm(iright+5)-kmeshm(iright+4)+1-nsubcv) ) then
            call errint(ileft,1)
            call errint(n,2)
            call errint(iright,3)
            call errint(kmeshm(iright+5)-kmeshm(iright+4)+1-nsubcv,4)
            call errint(ielgrp,5)
            call errint(i,6)
            call errsub ( 327, 6, 0, 0 )
         end if
         if ( ileft==iright ) then
            call errint ( ileft, 1 )
            call errsub ( 1806, 1, 0, 0 )
         end if
         if ( ierror/=0 ) go to 1000
         if ( ishape==0 ) then

!        --- ishape = 0, points are connected

            if ( kmeshm(istar3+kmeshm(ileft+4))==kmeshm(istar3-1+ &
                 kmeshm(ileft+5)) .and. kmeshm(istar3+kmeshm(iright+4)) &
                ==kmeshm(istar3-1+kmeshm(iright+5))) n = n-1
            if ( iexclu==1 .or. iexclu==2 ) then

!           --- iexclu = 1 or 2, skip initial point or end point

               n = n - 1

            else if ( iexclu==3 ) then

!           --- iexclu = 3 skip initial point and end point

               n = n - 2

            end if

         else

!        --- ishape > 0, elements are connected

            n = (n-1)/ishape
            n = n*(ishape+1)

         end if
         lengc = lengc + 2 * n

         iinp = iinp+4

      end do

!     --- Compute surface connections

      do i = 1, isurfs
         ielgrp = iinput(iinp)
         ishape = iinput(iinp+1)
         iexclu = iinput(iinp+4)
         if ( iexclu<-1 .or. iexclu>1500 ) then

!        --- error 558: iexclu out of range

            call errint ( iexclu, 1 )
            call errint ( -1, 2 )
            call errint ( 1500, 3 )
            call errsub ( 1437, 3, 0, 0 )

         end if
         ileft  = iinput(iinp+2)
         if ( ileft<=0 .or. ileft>nsurfs ) then
            call errint(ileft,1)
            call errint(nsurfs,2)
            call errint(ielgrp,3)
            call errint(i,4)
            call errsub ( 326, 4, 0, 0 )
         end if
         iright = iinput(iinp+3)
         if ( iright<=0 .or. iright>nsurfs ) then
            call errint(iright,1)
            call errint(nsurfs,2)
            call errint(ielgrp,3)
            call errint(i,4)
            call errsub ( 326, 4, 0, 0 )
         end if
         if ( ileft==iright ) then

!        --- Error 1327:  Left and right surface are identical

            call errint(ileft,1)
            call errint(ielgrp,2)
            call errint(i,3)
            call errsub ( 1327, 3, 0, 0 )
         end if

!        --- Check if left and right surface are topological identical

         inpelm = kmeshn(ileft)
         inelem = kmeshn(ileft+nsurfs)
         if ( inpelm/=kmeshn(iright) ) then

!        --- Error 1325:  Number of nodes per element not the same for both
!                         surfaces

            call errint(inpelm,1)
            call errint(kmeshn(iright),2)
            call errint(ielgrp,3)
            call errint(i,4)
            call errsub ( 1325, 4, 0, 0 )

         end if

         if ( inelem/=kmeshn(iright+nsurfs) ) then

!        --- Error 1326:  Number of elements not the same for both
!                         surfaces

            call errint(inelem,1)
            call errint(kmeshn(iright+nsurfs),2)
            call errint(ielgrp,3)
            call errint(i,4)
            call errsub ( 1326, 4, 0, 0 )
         end if
         if ( ierror/=0 ) go to 1000

         if ( ishape==0 ) then

!        --- ishape = 0, points are connected
!            Compute starting addresses in array KMESHN

            ilefst = 2*nsurfs
            do j = 1, ileft-1
               ilefst = ilefst + kmeshn(j)*kmeshn(j+nsurfs)
            end do

            irigst = 2*nsurfs
            do j = 1, iright-1
               irigst = irigst + kmeshn(j)*kmeshn(j+nsurfs)
            end do

!           --- Clear array iwork
!               Array IWORK is used as work array
!               It is used to avoid double elements
!               If IWORK(i) = 0, node i has not been found in the left surface
!               yet
!               If IWORK(i) > 0, the node has been used and elements containing
!               this node as starting point are skipped

            do j = 1, npoint
               iwork(j) =0
            end do

            if ( iexclu==-1 ) then

!           --- iexclu = -1, the complete outer boundary must be skipped

               call msh033 ( kmeshm, kmeshs, ncurvs, nsurfs, &
                             ileft, iwork, npoint )

            else if ( iexclu>0 ) then

!           --- iexclu > 0, skip given curves
!               Set iwork(ipoint) for curves

                do j = 1, iexclu
                   icrvn = iinput(iinp+4+j)
                   do k = kmeshm(4+icrvn), kmeshm(5+icrvn)-1
                      iwork(kmeshm(istar3+k)) = -1
                   end do
               end do

            end if

!           --- Loop over all elements in the surface

            n = 0
            do j = 1, inpelm*inelem
               ipoint = kmeshn(ilefst+j)
               if ( iwork(ipoint)<0 ) then

!              --- iwork(ipoint) < 0, the node is excluded

                   iwork(ipoint) = 1

               else if ( iwork(ipoint)==0 ) then

!              --- iwork(ipoint) = 0, the node has not been used yet

                       iwork(ipoint) = 1
                       n = n + 2

               end if
            end do

            do j = 1, npoint

!           --- check if surface contains all excluded nodal points

               if ( iwork(j)<0 ) then
                  call errint ( j, 1 )
                  call errint ( ileft, 2 )
                  call errsub ( 1442, 2, 0, 0 )
               end if

            end do

         else

!        --- ishape > 0, elements are connected

            n = inpelm*inelem*2

         end if

         lengc = lengc+n
         iinp = iinp+5+max(0,iexclu)

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh038'

      end if  ! ( debug )
      call erclos ( 'msh038' )

      end subroutine msh038
