      subroutine msh053 ( iinput, rinput, iinp, iposc, n, icurnr )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  6.0  date 14-10-2002 Extra parameters kmesh, kmeshq
!        version  5.2  date 06-03-1999 Correction accuracy check
!        version  5.1  date 28-01-1997 for all curves
!
!   copyright (c) 1987-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute co-ordinates of curve by translating and rotating the co-
!     ordinates of curve iprec. The translation distance is given by the
!     difference between the first user points on both curves and the
!     rotation angle is defined by the first three user points
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     rotation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, iposc, n, icurnr
      double precision rinput(ndim,*)

!     icurnr         i    Curve sequence number
!     iinp           i    starting position in array iinput with respect
!                         to new curve
!     iinput         i    new array iinput
!                         The first 5 positions have been skipped
!     iposc          i    starting position of co-ordinates in curve
!     n              i    number of points in curve
!     rinput        i/o   new array rinput
!                         The first 5 positions have been skipped
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iprec, ipc1, ipc2, k, npointlc, nuser, iposnp, icurve, &
              jposc, istrht, ipos, j, iuserp, itype, ic, &
              nparts, nsubcv, isubcv
      double precision xold(ndim), t1(ndim), t2(ndim), t3(ndim), xnew(ndim), &
              ta1(ndim), ta2(ndim), ta3(ndim), sum, angle, xyzmax, &
              alpha, beta, gamma, axp(ndim), eps, e1, &
              x1(ndim), x2(ndim), x3(ndim), dist, distlc
      logical errfnd, zero

!     alpha          local coordinate
!     angle          angle between two vectors
!     axp            help arrays to store the coordinates
!     beta           local coordinate
!     dist           distance
!     distlc         local distance
!     e1             local parameter to check the distance between two points
!     eps            accuracy
!     errfnd         error indicator
!     gamma          local coordinate
!     i              loop variable
!     ic             Relative subcurve number
!     icurve         type of local curve
!     ipc1           first pointer for curve
!     ipc2           second pointer for curve
!     ipos           local position
!     iposnp         local position for type of curve to be copied
!     iprec          pointer
!     istrht         indicator whether curve is straight line
!     isubcv         Subcurve sequence number with direction
!     itype          indicator which type of line the original curve is
!     iuserp         node number of user point
!     j              loop variable
!     jposc          position
!     k              local variable
!     nparts         number of points along subcurve
!     npointlc         number of points to be considered
!     nsubcv         Number of subcurves of a curve
!     nuser          number of user points
!     sum            local sum variable
!     t1             first basis vector in old system
!     t2             second basis vector in old system
!     t3             third basis vector in old system
!     ta1            first basis vector in new system
!     ta2            second basis vector in new system
!     ta3            third basis vector in new system
!     x1             first local coordinate vector
!     x2             second local coordinate vector
!     x3             third local coordinate vector
!     xnew           origin in new local coordinate system
!     xold           origin in old local coordinate system
!     xyzmax         max absolute value of the coordinates
!     zero           indicator whether extra userpoints have to be checked
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH046         Raise iinp as function of icurve
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     523   Co-ordinates of user point on curve with TRANSLATE or ROTATE not
!           the same as the corresponding user point on the curve to be copied
!     918   Number of user points on curve < ndim
!     919   Number of user points on curve more than user points on
!           original curve
!     920   No basis can be found for the rotation
!    1772   Negative user point number found, not the end point
!    2368   Curve to be copied is not coniguous
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine local basis for points on curve to be rotated
!     Make basis orthonormal, using Gramm-Schmidt
!     Compute basis in new space, i.e. for the new points as given by the
!     user
!     Make new basis orthonormal, using Gramm-Schmidt
!     Rotate all co-ordinates of old curve into new curve
!     Copy or check co-ordinates of user points on the new curve
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh053' )

      eps = 10 * epsmac

!     --- Set value of istrht (zero means no straight line)

      istrht = 0

!     --- Find starting positions of curve iprec in array iinput (kinp)
!         and array curvesmsh (jposc)
!         iprec is curve to be copied

      nsubcv = 0
      iprec = iinput(iinp)

      if ( kelmq>0 ) nsubcv = kmeshq(iprec+1)-kmeshq(iprec)

      if ( iprec==1 ) then
         jposc = 0
         n = icurvsmsh(iprec)
      else
         jposc = icurvsmsh(iprec-1)
         n = icurvsmsh(iprec) - jposc
      end if

!     --- Correct n if there are more subcurves due to double points

      if ( nsubcv>1 ) n = n-nsubcv+1

!     --- Characteristics of curve iprec
!         nuser is number of user points
!         ipc1 is first user point in curve iprec

      nuser = curveusrp(iprec+1)-curveusrp(iprec)

!     --- Determine the type of the curve to be copied

      iposnp = 16

      do i = 1 , iprec-1

          icurve = iinput( iposnp )
          call msh046( iinput, iposnp, icurve )

      end do

      itype = iinput( iposnp )

!     --- nuser should at least be ndim

      if ( nuser<ndim .and. itype/=1 .and. itype.ne.5 ) then

           call errint ( iprec, 1 )
           call errint ( nuser, 2 )
           call errint ( ndim, 3 )
           call errsub ( 918, 3, 0, 0 )

      end if

      ipc1 = curveusrp(curveusrp(iprec)+1)

!     --- Curve icurnr:  first point: ipc2
!                      npointlc: number of prescribed user points
!                              on second curve

      ipc2 = iinput(iinp+1)
      npointlc = iinput(iinp-1)

      if ( npointlc>nuser ) then

           call errint ( icurnr, 1 )
           call errint ( npointlc, 2 )
           call errint ( iprec, 3 )
           call errint ( nuser, 4 )
           call errsub ( 919, 4, 0, 0 )

      end if

      if ( npointlc<ndim .and. itype/=1 .and. itype.ne.5 ) then

           call errint ( icurnr, 1 )
           call errint ( npointlc, 2 )
           call errint ( ndim, 3 )
           call errsub ( 918, 3, 0, 0 )

      end if

      if ( ierror/=0 ) go to 1000

!     --- Determine local basis for old points

      xold = rinput(:,ipc1)

!     --- Base vector T1

      if ( npointlc==2 .and. iinput(iinp+2)<0 ) then
         ipc1 = curveusrp(curveusrp(iprec)+nuser)
      else
         ipc1 = curveusrp(curveusrp(iprec)+2)
      end if

      t1 = rinput(:,ipc1) - xold

!     --- Normalize

      sum = sqrt( dot_product(t1,t1) )
      t1 = t1/sum

!     --- Base vector T2

      if ( ndim==3 ) then

         do i = 3, nuser

            if ( npointlc==i .and. iinput(iinp+i)<0 ) then
               ipc1 = curveusrp(curveusrp(iprec)+nuser)
            else
               ipc1 = curveusrp(curveusrp(iprec)+i)
            end if

            t2 = rinput(:,ipc1)-xold

!           --- Normalize

            sum = sqrt(dot_product(t2,t2))
            t2 = t2/sum

            angle = dot_product(t1,t2)

            if ( abs(angle)<1d0-100*eps ) goto 150

         end do

!        --- Angle never large enough , original curve is straight line

         istrht = 1

      else

         if ( abs(t1(1))<eps ) then

              t2(1) = 1d0
              t2(2) = 0d0

         else

              t2(1) = - t1(2) / t1(1)
              t2(2) =   1d0

         end if

         sum   = t1(1)*t2(2) - t1(2)*t2(1)

         if ( sum<0 ) then

              t2(1) = -t2(1)
              t2(2) = -t2(2)

         end if

         sum  = sqrt( t2(1)*t2(1) + t2(2)*t2(2) )

         t2(1) = t2(1) / sum
         t2(2) = t2(2) / sum

      end if

150   if ( istrht==0 ) then

!     --- Make basis orthonormal if not straight line
!         (Use Gramm-Schmidt process)

         if ( ndim==3 ) then

           t2 = t2 - angle * t1

!          --- Normalize

           sum = sqrt( dot_product(t2,t2) )

           t2 = t2/ sum

!          --- Compute third vector of orthogonal system

           t3(1) = t1(2)*t2(3) - t2(2)*t1(3)
           t3(2) = t1(3)*t2(1) - t2(3)*t1(1)
           t3(3) = t1(1)*t2(2) - t2(1)*t1(2)

         else

!        --- ndim = 2 : third vector is zero

           t3 = 0

         end if

      else

           t2 = 0d0
           t3 = 0d0

      end if

!     --- Compute basis in new plane

      xnew = rinput(:,ipc2)

!     --- Determine vector Ta1

      ipc2 = iinput(iinp+2)
      if ( ipc2<0 ) then

         if ( 2/=npointlc ) then

!        --- negative point in rotate found, which is not the end point

            call errint ( icurnr, 1 )
            call errint ( ipc2, 2 )
            call errsub ( 1772, 2, 0, 0 )

         end if
         ipc2 = - ipc2

      end if

      ta1 = rinput(:,ipc2)-xnew

!     --- Normalize

      sum = sqrt(dot_product(ta1,ta1))
      ta1 = ta1/sum

!     --- Determine vector Ta2

      if ( ndim==3 ) then

         do i = 3 , npointlc

            ipc2 = iinput(iinp+i)
            if ( ipc2<0 ) then
               if ( i/=npointlc ) then

!              --- negative point in rotate found, which is not the end
!                  point

                  call errint ( icurnr, 1 )
                  call errint ( ipc2, 2 )
                  call errsub ( 1772, 2, 0, 0 )

               end if
               ipc2 = - ipc2

            end if

            ta2 = rinput(:,ipc2)-xnew

!           --- Normalize

            sum = sqrt(dot_product(ta2,ta2))
            ta2 = ta2/sum

            angle = dot_product(ta1,ta2)

            if ( abs(angle)<1d0-100*eps ) goto 250

         end do

!        --- Angle never large enough , error ?

         if ( istrht==0 ) then

            call errint ( icurnr, 1)
            call errsub ( 920, 1, 0, 0 )
            go to 1000

         end if

      else

         if ( abs(ta1(1))<eps ) then

              ta2(1) = 1d0
              ta2(2) = 0d0

         else

              ta2(1) = - ta1(2) / ta1(1)
              ta2(2) =   1d0

         end if

         sum = ta1(1)*ta2(2) - ta1(2)*ta2(1)

         if ( sum<0 ) then

            ta2(1) = -ta2(1)
            ta2(2) = -ta2(2)

         end if

         sum = sqrt( ta2(1)*ta2(1)+ta2(2)*ta2(2) )

         ta2(1) = ta2(1) / sum
         ta2(2) = ta2(2) / sum

      end if

250   if ( istrht==0 ) then

!     --- Make basis orthonormal
!         (Use Gramm-Schmidt process)

         if ( ndim==3 ) then

           ta2 = ta2 - angle * ta1

           sum = sqrt( dot_product(ta2,ta2) )

           ta2 = ta2/ sum

!          --- Compute third basis vector

           ta3(1) = ta1(2)*ta2(3) - ta2(2)*ta1(3)
           ta3(2) = ta1(3)*ta2(1) - ta2(3)*ta1(1)
           ta3(3) = ta1(1)*ta2(2) - ta2(1)*ta1(2)

         else

!        --- ndim = 2 : third vector is zero

           ta3 = 0d0

         end if

      else if ( istrht==1 ) then

           ta2 = 0d0
           ta3 = 0d0

      end if

!     --- Rotate all co-ordinates of curve iprec into curve icurnr
!         ( make exception for case of straight line )

      ipos = iposc-1
      nsubcv = max(1,nsubcv)

!     --- For all subcurves do

      do ic = 1, nsubcv

         if ( nsubcv>1 ) then

!        --- There are more subcurves,
!            find subcurve isubcv and number of nodes along isubcv

            isubcv = abs(kmeshq(kmeshq(iprec)+ic))
            if ( abs(isubcv)==1 ) then
               nparts = icurvsmsh(isubcv)
            else
               nparts = icurvsmsh(isubcv)-icurvsmsh(isubcv-1)
            end if

         else

!        --- No subcurves

            nparts = n

         end if

         do i = 1, nparts

             axp = curvesmsh(:,jposc+i)-xold

!            --- Compute coefficients of this node in old system

             if ( istrht==1 ) then

                alpha = sqrt(dot_product(axp,axp))
                beta  = 0d0
                gamma = 0d0

             else

                alpha = dot_product(axp,t1)
                beta  = dot_product(axp,t2)
                gamma = dot_product(axp,t3)

             end if

!            --- New coordinates, use coefficients in new system

             curvesmsh(:,ipos+i) = xnew + alpha * ta1 + &
                                     beta * ta2  + gamma * ta3

         end do
         if ( ic>1 ) then

!        --- not the first subcurve, check if first points is equal to
!            last one in previous subcurve
!            Store begin coordinates in x1

            errfnd = .false.
            x1 = curvesmsh(:,jposc+1)
            do j = 1, ndim
               if ( abs(x2(j)-x1(j))>&
                    sqreps*(abs(x1(j))+abs(x2(j))) ) errfnd = .true.
            end do
            if ( errfnd ) then

!           --- The curve is not contiguous

               call errint ( iprec, 1 )
               call errint ( icurnr, 2 )
               call errint ( ic, 3 )
               call errsub ( 2368, 3, 0, 0 )
               call prinrl ( x1, ndim, 'first node' )
               call prinrl ( x2, ndim, 'second node' )

            end if

         end if

         if ( nsubcv>1 ) then

!        --- Store end coordinates in x2

            x2 = curvesmsh(:,jposc+nparts)

         end if

!        --- Raise pointers, take double point into account

         jposc = jposc+nparts
         ipos = ipos+nparts-1

      end do

!     --- Compute the local accuracy for checking

      ipos = iposc-1
      xyzmax = 0d0
      do k = 2, min(npointlc,nuser)
         ipc1 = curveusrp(curveusrp(iprec)+k)
         ipc2 = iinput(iinp+k)
         if ( ipc2<0 ) then
            if ( k/=npointlc ) then

!           --- negative point in rotate found, which is not the end point

               call errint ( icurnr, 1 )
               call errint ( ipc2, 2 )
               call errsub ( 1772, 2, 0, 0 )

            else

               ipc1 = curveusrp(curveusrp(iprec)+nuser)

            end if
            ipc2 = - ipc2

         end if
         x1 = rinput(:,ipc1)
         xyzmax = max(xyzmax,maxval(abs(x1)))

      end do

      if ( ierror/=0 ) go to 1000

!     --- Compute local accuracy

      e1 = 1d-5*xyzmax
      if ( e1<epsmac ) e1 = sqreps

!     --- Copy or check co-ordinates of user points on the curve icurnr

      do k = 2, min(npointlc,nuser)
         ipc1 = curveusrp(curveusrp(iprec)+k)
         ipc2 = iinput(iinp+k)
         if ( ipc2<0 ) then
            ipc1 = curveusrp(curveusrp(iprec)+nuser)
            ipc2 = - ipc2
         end if

!        --- Put co-ordinates of user points in x1 resp. x2

         x1 = rinput(:,ipc1)
         x2 = rinput(:,ipc2)

!        --- Check whether x2 equals zero

         zero = .true.
         do i = 1, ndim
            if ( x2(i)<0d0 .or.x2(i)>0d0 ) then
               zero = .false.
               goto 340
            end if
         end do

340      axp = x1-xold

!        --- Compute coefficients user point

         if ( istrht==1 ) then

              alpha = sqrt( dot_product(axp,axp) )
              beta  = 0d0
              gamma = 0d0

         else if ( istrht==0 ) then

              alpha = dot_product(axp,t1)
              beta  = dot_product(axp,t2)
              gamma = dot_product(axp,t3)

         end if

         x3 = xnew+alpha*ta1+beta*ta2+gamma*ta3

         if ( k==2 ) then

!        --- k = 2, correct curves in order to prevent error message
!            First find which node is closest to the second user point

            dist = rinfin
            do i = 2, n
               distlc = 0d0
               do j = 1, ndim
                  distlc = distlc+ ( x3(j)-curvesmsh(j,ipos+i) ) ** 2
               end do  ! j = 1, ndim
               if ( distlc<dist ) then
                  dist = distlc
                  iuserp = i
               end if
            end do
            curvesmsh(:,ipos+iuserp) = x2

         else if ( zero ) then

!        --- zero = true    copy co-ordinates

            rinput(:,ipc2) = x3

         else

!        --- zero = false   check co-ordinates

            errfnd = .false.

            do i = 1, ndim
               if ( abs(x2(i)-x3(i))>e1 ) errfnd = .true.
            end do

            if ( errfnd ) then

!           --- Error 523:  Incorrect co-ordinates

                call errint ( k, 1 )
                call errint ( icurnr, 2 )
                do i = 1, 3
                   call erreal ( x2(i), i )
                   call erreal ( x3(i), i+3 )
                end do
                call errsub ( 523, 2, 6, 0 )

            end if

         end if

      end do

1000  call erclos ( 'msh053' )

      end
