      subroutine msh058 ( iinput, ielgeg, iinner )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.2  date 10-02-2015 Extra debug
!        version  4.1  date 02-09-2013 Debug statements
!        version  4.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1990-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine is used to fill the part of KMESH part F that corresponds
!     to line elements, surface elements and volume elements
!     Furthermore the positions connected to intermediate points are
!     filled if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), ielgeg(*), iinner(*)

!     ielgeg     i    array with number of nodes for each element shape
!     iinner     i    Number of inner curves in mesh
!     iinput    i/o   Standard integer input array for subroutine MESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, melgrp, iinp, ielgrp, ifirst, ilast, jinp, &
              j, inner, ivolm

!     i              Counting variable
!     ielgrp         element group number
!     ifirst         First curve (surface) number
!     iinp           last position used in array iinput
!     ilast          Last curve (surface) number
!     inner          number of internal points in element
!     ivolm          Type of volume with respect to mesh generator
!     j              Counting variable
!     jinp           last position used in array iinput
!     melgrp         Local value of nelgrp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
!     MSH050         Raise iinp as function of ivolm
!     MSH058CONN     Fill array kmesh part f for connection elements
!     MSH058INTERF   Fill array kmesh part f for interface elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     122   elements are not created in the right sequence
!     123   New element has the same element group number as preceding element, &
!           but its shape number differs.
!     155   first surface number has incorrect value
!     156   last surface number has incorrect value
!     188   element group consists of elements with different shape numbers.
!     296   first volume number has incorrect value
!     297   last volume number has incorrect value
!     323   element group number for connected element is incorrect
!    1168   Shape number of surface element incorrect
!    1169   Shape number of volume element incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh058' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh058'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Since nelgrp is already in use, we define locally melgrp

      nelgrp = nelgrp-nelgrpdum
      kmeshf(1:nelgrp) = 0
      melgrp = 0
      jnodp = 1
      jboun = 1

      if ( debug ) write(irefwr,1) 'nlines', nlines

      if ( nlines>0 ) then

!     --- nlines>0   line elements

         iinp = iinp2
         do i = 1, nlines
            ishape = iinput(iinp)
            ielgrp = iinput(iinp+1)

            if ( debug ) &
               write(irefwr,1) 'i, ishape, ielgrp', i, ishape, ielgrp
            if ( ielgrp<=melgrp ) then

!           --- ielgrp<= melgrp   checks

              if ( ielgrp<melgrp ) then
                 call errint ( ielgrp, 1 )
                 call errint ( melgrp, 2 )
                 call errsub ( 122, 2, 0, 0)
                 go to 1000
              end if
              if ( ishape/=-kmeshf(ielgrp)-10 ) then
                  call errint ( ishape, 1 )
                  call errint ( kmeshf(ielgrp), 2 )
                  call errint ( ielgrp, 3 )
                  call errsub ( 123, 3, 0, 0)
                  go to 1000
               end if
            else

!           --- ielgrp>melgrp,  new standard element

               melgrp = max(ielgrp,melgrp+1)
               if ( ielgrp/=melgrp ) then
                  call errint ( ielgrp, 1 )
                  call errint ( melgrp, 2 )
                  call errsub ( 122, 2, 0, 0)
                  go to 1000
               end if
               inpelm = ishape+1
               if ( ielgrp>1 .and. inpelm/=npelm ) jnodp = 0
               if ( inpelm>npelm ) npelm = inpelm
               kmeshf(ielgrp) = -ishape-10
            end if
            iinp = iinp+4
         end do
      end if

      if ( debug ) write(irefwr,1) 'nsurel, iinp4', nsurel, iinp4

      if ( nsurel>0 ) then

!     --- nsurel>0   surface elements

         iinp = iinp4
         do i = 1, nsurel
            ishape = iinput(iinp)
            ielgrp = iinput(iinp+1)
            ifirst = iinput(iinp+2)
            ilast  = iinput(iinp+3)
            if ( debug ) &
               write(irefwr,1) 'i, ishape, ielgrp', i, ishape, ielgrp
            if ( ishape/=0 .and. (ishape<=2 .or. ishape>10) ) then
               call errint ( ishape, 1 )
               call errint ( 8, 2 )
               call errsub ( 1168, 2, 0, 0)
               go to 1000
            end if

!           --- check shape number

            jinp = iinp3
            do j = 1, ilast
               isurf = iinput(jinp)
               if ( isurf==24 ) isurf = 7    ! compound surface
               if ( ishape==0 .and. j==ifirst ) then

!              --- ishape = 0, and first element group
!                  Fill correct value of ishape

                  ishape = iinput(jinp+1)
                  iinput(iinp) = ishape

               end if
               if ( isurf>0 .and. j>=ifirst .and. &
                    ishape/=iinput(jinp+1) ) then
                  call errint ( ielgrp, 1 )
                  call errint ( j, 2 )
                  call errint ( iinput(jinp+1), 3 )
                  call errint ( ishape, 4 )
                  call errsub ( 188, 4, 0, 0)
                  go to 1000
               end if
               call msh049 ( iinput, jinp, isurf )
            end do
            if ( ifirst<1 .or. ifirst>nsurfs ) then
               call errint ( ifirst, 1 )
               call errint ( nsurfs, 2 )
               call errsub ( 155, 2, 0, 0)
               go to 1000
            end if
            if ( ilast<ifirst .or. ilast>nsurfs ) then
               call errint ( ilast, 1 )
               call errint ( ifirst, 2 )
               call errint ( nsurfs, 3 )
               call errsub ( 156, 3, 0, 0)
               go to 1000
            end if
            if ( ielgrp<=melgrp ) then

!           --- ielgrp<= melgrp   checks

               if ( ielgrp<melgrp ) then
                  call errint ( ielgrp, 1 )
                  call errint ( melgrp, 2 )
                  call errsub ( 122, 2, 0, 0)
                  go to 1000
               end if
               if ( i>1 .and. ishape/=kmeshf(ielgrp) ) then
                  call errint ( ishape, 1 )
                  call errint ( kmeshf(ielgrp), 2 )
                  call errint ( ielgrp, 3 )
                  call errsub ( 123, 3, 0, 0)
                  go to 1000
               end if
            else

!           --- ielgrp>melgrp,  new standard element

               melgrp = max(melgrp+1,ielgrp)
               if ( ielgrp/=melgrp ) then
                  call errint ( ielgrp, 1 )
                  call errint ( melgrp, 2 )
                  call errsub ( 122, 2, 0, 0)
                  go to 1000
               end if
               inpelm = ielgeg(ishape)
               if ( ielgrp>1 .and. inpelm/=npelm ) jnodp = 0
               inner = iinner(ishape)
               if ( inner>0 ) jboun = 0
               if ( inpelm>npelm ) npelm = inpelm
               kmeshf(ielgrp) = ishape
            end if
            iinp = iinp+4
         end do
      end if

      if ( nvolel>0 ) then

!     --- nvolel>0 volume elements

         iinp = iinp6
         do i = 1, nvolel
            ishape = iinput(iinp)
            ielgrp = iinput(iinp+1)
            ifirst = iinput(iinp+2)
            ilast = iinput(iinp+3)
            if ( ishape<0 .or. ishape>=1 .and. ishape<=10 .or. &
                 ishape>18 .and. ishape<=20 .or. ishape>23 ) then

!           --- shape number out of range

               call errint ( ishape, 1 )
               call errint ( ifirst, 2 )
               call errsub ( 1169, 2, 0, 0)
               go to 1000

            end if

!           --- check shape number

            jinp = iinp5
            do j = 1, ilast

               ivolm = iinput(jinp)
               if ( ishape==0 .and. j==ifirst ) then

!              --- ishape = 0, and first element group
!                  Fill correct value of ishape

                  ishape = iinput(jinp+1)
                  iinput(iinp) = ishape

               end if
               if ( ivolm>0 .and. j>=ifirst .and. &
                    ishape/=iinput(jinp+1) ) then
                  call errint ( ielgrp, 1 )
                  call errint ( j, 2 )
                  call errint ( iinput(jinp+1), 3 )
                  call errint ( ishape, 4 )
                  call errsub ( 188, 4, 0, 0)
                  go to 1000
               end if
               call msh050(iinput, jinp, ivolm)

            end do

            if ( ifirst<1 .or. ifirst>nvolms ) then
               call errint ( ifirst, 1 )
               call errint ( nvolms, 2 )
               call errsub ( 296, 2, 0, 0)
               go to 1000
            end if
            if ( ilast<ifirst .or. ilast>nvolms ) then
               call errint ( ilast, 1 )
               call errint ( ifirst, 2 )
               call errint ( nvolms, 3 )
               call errsub ( 297, 3, 0, 0)
               go to 1000
            end if
            if ( ielgrp<=melgrp ) then

!           --- ielgrp<= melgrp   checks

              if ( ielgrp<melgrp ) then
                 call errint ( ielgrp, 1 )
                 call errint ( melgrp, 2 )
                 call errsub ( 122, 2, 0, 0)
                 go to 1000
              end if
              if ( i>1 .and. ishape/=kmeshf(ielgrp) ) then
                 call errint ( ishape, 1 )
                 call errint ( kmeshf(ielgrp), 2 )
                 call errint ( ielgrp, 3 )
                 call errsub ( 123, 3, 0, 0)
                 go to 1000
              end if

            else

!           --- ielgrp>melgrp,  new standard element

               melgrp = max(melgrp+1,ielgrp)
               if ( ielgrp/=melgrp ) then
                  call errint ( ielgrp, 1 )
                  call errint ( melgrp, 2 )
                  call errsub ( 122, 2, 0, 0)
               end if
               inpelm = ielgeg(ishape)
               if ( ielgrp>1 .and. inpelm/=npelm ) jnodp = 0
               if ( inpelm>npelm ) npelm = inpelm
               kmeshf(ielgrp) = ishape

            end if
            iinp = iinp+4

         end do

      end if

      if ( iinput(12)/=0 ) then

!     --- iinput(12) # 0     connected elements

         call msh058conn ( iinput, iinp, ielgeg )

      end if

      if ( iinp11/=0 ) then

!     --- iinp11 # 0     interface elements

         call msh058interf ( iinput, iinp, ielgeg )

      end if

      if ( iinp15/=0 ) then

!     --- iinp15 # 0     dummy element groups

         iinp = iinp15+1
         do i = 1, nelgrpdum
            ielgrp = iinput(iinp)
            nelgrp = nelgrp+1
            if ( ielgrp/=nelgrp ) then

!           --- ielgrp # nelgrp

               call errint ( ielgrp, 1 )
               call errint ( nelgrp, 2 )
               call errsub ( 323, 2, 0, 0 )
               go to 1000

            end if
            inpelm = iinput(iinp+1)
            if ( inpelm/=npelm) jnodp = 0
            npelm = max(inpelm,npelm)
            kmeshf(ielgrp) = 100+inpelm
            iinp = iinp+2
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh058'

      end if  ! ( debug )
      call erclos ( 'msh058' )

      end
