      subroutine msh065 ( icurves, ncurves, icurvar, icurvsl, &
                          ncurvsl, second )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-09-2002 Correction for compound curves
!                                      consisting of 1 curve
!        version  1.0  date 15-08-1996
!
!   copyright (c) 1996-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Copy contents of array icurves into temporary array icurvsl
!   Compound curves are splitted into single curves
!   Array icurvar is updated with respect to all curves that must be
!   changed
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     curve
!     adapt_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurves, icurves(ncurves), icurvar(*), &
              icurvsl(*), ncurvsl
      logical second

!     icurvar   i/o   Array of length ncurvs that contains information of
!                     the curves that must be changed. icurvar(i) mus be
!                     filled as follows:
!                     0  curve i is not yet adapted
!                     1  curve i has been adapted and is a single curve
!     icurves    i    Integer array containing the curve numbers that
!                     must be adapted. It is assumed that these curves form
!                     one continuous curve.
!     icurvsl    o    Array of length ncurvs that is filled with all
!                     stand alone curves that must be adapted in the
!                     correct sequence and provided with a sign
!     ncurves    i    Length of array icurves
!     ncurvsl    o    Number of curves stored in icurvsl
!     second     i    If true this is the second call in a row and only
!                     icurvsl and ncurvsl must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, icurnr, isubcur, jcurnr, jsubcur
      logical compound

!     compound       If true the curve is a compound curve
!     i              Counting variable
!     icurnr         Curve sequence number
!     isubcur        Subcurve sequence number
!     jcurnr         |icurnr|
!     jsubcur        |isubcur|
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1641   Curve has been used more than once to be updated
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ncurvsl = 0
!     For i := 1 (1) ncurves do
!        icurnr := icurves(i)
!        if ( icurnr = compound curve ) then
!           For all subcurves isubcur of icurnr do
!              if ( icurvar(isubcur)=1 ) then
!                 error curve has already been updated
!              icurvar(isubcur) := 1
!              ncurvsl := ncurvsl+1
!              icurvsl := isubcur
!        else
!           if ( icurvar(icurnr)=1 ) then
!              error curve has already been updated
!           icurvar(icurnr) := 1
!           ncurvsl := ncurvsl+1
!           icurvsl := icurnr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh065' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information in msh065'
      end if
      ncurvsl = 0
      do i = 1, ncurves
         icurnr = icurves(i)
         jcurnr = abs(icurnr)
         compound = .false.
         if ( kelmq>0 ) then
            if ( kmeshq(jcurnr+1)-kmeshq(jcurnr)>0 ) &
                compound = .true.
         end if
         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'i, icurnr, jcurnr, compound ', &
                             i, icurnr, jcurnr, compound
         end if
         if ( compound ) then

!        --- Curve is a composite curve

            if ( icurnr>0 ) then

!           --- positive direction

               do k = kmeshq(jcurnr)+1, kmeshq(jcurnr+1)
                  isubcur = kmeshq(k)
                  jsubcur = abs(isubcur)
                  if ( .not. second .and. icurvar(jsubcur)==1 ) then

!                 --- Curve has already been used

                     call eropen ( 'msh065' )
                     call errint ( jsubcur, 1 )
                     call errsub ( 1641, 1, 0, 0 )
                     call erclos ( 'msh065' )
                  else

!                 --- Curve has not yet been used

                     icurvar(jsubcur) = 1
                     ncurvsl = ncurvsl + 1
                     icurvsl(ncurvsl) = isubcur
                  end if
               end do
            else

!           --- negative direction

               do k = kmeshq(jcurnr+1), kmeshq(jcurnr)+1,-1
                  isubcur = kmeshq(k)
                  jsubcur = abs(isubcur)
                  if ( .not. second .and. icurvar(jsubcur)==1 ) then

!                 --- Curve has already been used

                     call eropen ( 'msh065' )
                     call errint ( jsubcur, 1 )
                     call errsub ( 1641, 1, 0, 0 )
                     call erclos ( 'msh065' )
                  else

!                 --- Curve has not yet been used

                     icurvar(jsubcur) = 1
                     ncurvsl = ncurvsl + 1
                     icurvsl(ncurvsl) = -isubcur
                  end if
               end do
            end if
         else

!        --- Curve is a stand alone curve

            if ( .not. second .and. icurvar(jcurnr)==1 ) then

!           --- Curve has already been used

               call eropen ( 'msh065' )
               call errint ( icurnr, 1 )
               call errsub ( 1641, 1, 0, 0 )
               call erclos ( 'msh065' )
            else

!           --- Curve has not yet been used

               icurvar(jcurnr) = 1
               ncurvsl = ncurvsl + 1
               icurvsl(ncurvsl) = icurnr
            end if
         end if
      end do
      call erclos ( 'msh065' )
      end
