      subroutine mshcurcline ( ndim, curves, iinput, rinput, tau, &
                               icurnr, numuserp, irelcurves )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array curves for a line with more than 2 user points
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     curve
!     point_distribution
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), ndim, icurnr, irelcurves(*), numuserp
      double precision curves(ndim,*), rinput(ndim,*), tau(*)

!     curves         o    array containing the coordinates of the curves
!     icurnr         i    Curve sequence number
!     iinput         i    integer input array   see subroutine mesh
!     irelcurves     i    Contains integer information of the curves
!                         See subroutine mshcrtcurv
!     ndim           i    Dimension of the space.
!     numuserp       i    Number of user points along curve
!     rinput         i    Real input array
!     tau            i    Work array of length n to store the relative distances
!                         between points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: n, ntot, istartrel, i, i1, i2, isegment
      double precision x1(ndim), x2(ndim), dx(ndim), eps, dlen(ndim)

!     dlen           |x1| + |x2|
!     dx             distance vector
!     eps            Local accuracy
!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     isegment       Sequence number of segment
!     istartrel      Starting position in array irelcurves
!     n              number of nodes along curve segment
!     ntot           Accumulated number of nodes
!     x1             coordinates of first point in segment
!     x2             coordinates of last point in segment
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     470   initial and end point of line have the same co-ordinates
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcurcline' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurcline'
         write(irefwr,1) 'numuserp-1', numuserp-1
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istartrel = irelcurves(icurnr)

!     --- Loop over the segments

      n = 0
      do isegment = 1, numuserp-1

         ntot = irelcurves(istartrel+isegment)+1

         i1 = iinput(isegment)  ! first user point
         i2 = iinput(isegment+1)  ! next user point

         x1 = rinput(1:ndim,i1)
         x2 = rinput(1:ndim,i2)
         dx = x2-x1

         dlen = abs(x2)+abs(x1)

         eps = epsmac*dot_product(dlen,dlen)

         if ( dot_product(dx,dx)<eps ) then
            call errint ( icurnr, 1 )
            call erreal ( sqrt(dx), 1 )
            call errsub ( 470, 1, 1, 0 )
         end if  ! ( dot_product(dx,dx)<eps )

         do i = 1, ndim
            curves(i,n+1:ntot) = x1(i)+tau(n+1:ntot)*dx(i)
         end do  ! i = 1, ndim
         n = ntot

      end do  ! isegment = 2, numuserp

1000  call erclos ( 'mshcurcline' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcurcline'

      end if  ! ( debug )

      end subroutine mshcurcline
