      subroutine mshcurcr ( iinput, rinput, curstr, &
                            icurve, iinp, irnp, &
                            iposc, icurnr, inpncv, jcoars, &
                            ipointcurv, irelcurves, irnp1, istartrel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  9.1  date 10-02-2015 Extension with ccircle
!        version  9.0  date 11-12-2014 Extra parameters
!        version  8.0  date 07-11-2009 New parameter list
!
!   copyright (c) 1996-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array curvesmsh for one curve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, irnp, icurve, &
              icurnr, inpncv, jcoars, ipointcurv(*), iposc, &
              irelcurves(*), irnp1, istartrel
      double precision rinput(*), curstr(*)

!     curstr         i    array of length 10 * ncurvs to store some
!                         geometrical information of the curves
!     icurnr         i    Curve sequence number
!     icurve         i    Type of curve
!     iinp          i/o   last position used in array iinput
!     iinput        i/o   standard integer input array for subroutine mesh
!                         it is supposed that the first five positions have been
!                         deleted
!     inpncv         o    number of points along curve
!     ipointcurv     i    array of length ncurvs with the starting
!                         addresses of the curves in iinput
!     iposc          i    last position used in array curves
!     irelcurves      o    Array with the same length as the first part of
!                         iinput (until surfaces)
!                         Relative node numbers of the user points in
!                         framecurv are filled at the same positions as
!                         the user points in iinput
!     irnp           i    position in array rinput
!     irnp1          i    first pos. rinput definition of coordinates of points
!     istartrel     i/o   next free position in irelcurves
!     jcoars         i    If 1 coarseness is given, if 0 not
!     rinput        i/o   standard real input array for subroutine mesh
!                         it is supposed that the first five positions have been
!                         deleted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelm, iratio, isubd, nnodes, irnp2, icount, jposc, istartcur
      double precision factor, sleng
      integer, allocatable :: ihelp(:)
      double precision, allocatable :: tau(:)
      save icount

!     factor         Multiplication factor
!     icount         counter
!     ihelp          integer work array of length n+2, with n maximum
!                    number of nodes along curve
!     iratio         Ratio as defined in input
!     irnp2          first pos. in rinput definition of factors
!     istartcur      starting position of in array curstr
!     isubd          Defines the type of subdivision of the nodes in the element
!                    in case of higher order elements
!     jposc          ?
!     nelm           number of elements along a curve
!     nnodes         Number of nodes
!     sleng          length of the curve
!     tau            work array containing the relative distribution of the
!                    coordinates on a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH004         Fill array tau
!     MSH005         Fill array curves in case of straight line
!     MSH006         Fill array curves in case of circle segment
!     MSH007         fill part of array curves   user defined curves
!     MSH031         Fill array curves in case of given coarseness
!     MSH032         Fill array curves in case of parameter function
!     MSH042         Copy coordinates for curves of curves
!     MSH052         Fill array curves for translated curve
!     MSH053         Fill array curves for rotated curve
!     MSHC01         Fill array curves with splines
!     MSHCURPROF     fill array curves, in case of a profile
!     MSHCURSPHERE   Construct 4 generating curves from circle to upper or
!                    lower point of sphere
!     MSHELLARC      fill part of array curves for an ellips segment
!     MSHOWNCURVE    Call user subroutine won_curve to create curve
!     MSHPARTCIRC    Subdivide a circle into 4 parts, each approximately 90
!                    degrees
!     MSHSP1         Fill array curves for reflected curve
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     124   code for generation of curves (icurve) incorrect
!     126   code for distance of elements (iratio) incorrect
!    1799   No coarseness defined
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data icount /0/
! ======================================================================
!
      call eropen ( 'mshcurcr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurcr'
         write(irefwr,1) 'icurnr, iinp, icurve', icurnr, iinp, icurve
         call prinin ( irelcurves, 10, 'irelcurves' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      irnp2 = (ndim+jcoars)*nuspnt+2*jcoars+irnp1
      ishape = iinput(iinp+1)
      nelm   = iinput(iinp+2)
      iratio = iinput(iinp+3)
      isubd  = 0
      iinp   = iinp+4
      jposc = iposc

      if ( debug ) &
         write(irefwr,1) 'ishape, nelm, iratio', ishape, nelm, iratio

      istartcur = max(1,irelcurves(ncurvs+icurnr))

      if ( debug ) write(irefwr,1) 'istartcur', istartcur

      if ( icurve>24 ) then
         call errint ( icurve, 1 )
         call errint ( 23, 2 )
         call errsub ( 124, 2, 0, 0 )
         go to 1000
      end if

!     --- Allocate arrays

      nnodes = ishape*nelm+1
      allocate ( tau(nnodes), ihelp(nnodes+2), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*nnodes+2, 'tau' )
      if ( error/=0 ) go to 1000

      if ( icurve==5 .or. icurve==6 .or. icurve==9 .or. &
           icurve==12 .or. icurve==16 .or. icurve==21 .or. &
           icurve==23 .or. icurve==24 ) then

!     --- icurve = 5, 6, 9, 12 or 16    Coarseness

         if ( jcoars==0 ) then

!        --- No coarseness defined

            call errint ( icurnr, 1 )
            call errsub ( 1799, 1, 0, 0 )
            go to 1000

         end if

         call msh031 ( icurnr, ncurvs, iinput, iinp, &
                       curvesmsh(1,iposc), rinput(irnp1), nelm, &
                       ishape, nuspnt, inpncv, ndim, &
                       icurve, curstr, tau, isubd, &
                       rinput(irnp2+ncurvs+2*icurnr-2), &
                       curvefacts(jposc), irelcurves, istartrel )

      else if ( icurve==7 .or. icurve==14 ) then

!     --- icurve = 7, 14 curves of curves

         call msh042 ( iinput, iinp, curvesmsh, iposc, ndim, inpncv, &
                       icurvsmsh, icurve, icurnr, tau, ihelp, &
                       ncurvs, rinput, irnp2 )

      else if ( icurve==10 ) then

!     --- icurve  =  10  translate

         call msh052 ( iinput, rinput(irnp1), iinp, iposc, &
                       inpncv, icurnr )

      else if ( icurve==11 ) then

!     --- icurve  =  11  rotate

         call msh053 ( iinput, rinput(irnp1), iinp, iposc, &
                       inpncv, icurnr )

      else if ( icurve==13 ) then

!     --- icurve  =  13  reflect

         call mshsp1 ( iinput, rinput(irnp1), iinp, &
                       inpncv, icurnr, ipointcurv )

      else if ( icurve==18 ) then

!     --- icurve  =  18:  4 parts of a circle
!         The number of elements in the parts is detected in this part

         if ( icount==0 ) then

!        --- icount = 0, first of 4, all 4 curves are filled

            call mshpartcirc ( iinput, ipointcurv, iinp, rinput(irnp1), &
                               curvesmsh, ishape, icurnr, icurvsmsh, inpncv )
            icount = 1

         else

!        --- icount>0, next of 4

            icount = icount+1
            if ( icount==4 ) icount = 0
            iinp = iinp+4

         end if

      else if ( icurve==19 ) then

!     --- icurve  =  19:  4 half arcs from circle to upper or lower point of
!                         shpere
!         The number of elements in the parts is detected in this part

         if ( icount==0 ) then

!        --- icount = 0, first of 4, all 4 curves are filled

            call mshcursphere ( iinput, ipointcurv, iinp, rinput(irnp1), &
                                curvesmsh, ishape, icurnr, icurvsmsh, inpncv, &
                                tau, nuspnt )
            icount = 1

         else

!        --- icount>0, next of 4

            icount = icount+1
            if ( icount==4 ) icount = 0
            iinp = iinp+5

         end if

      else

         if ( icurve/=20 ) then

!        --- icurve#20, check factor and ratio

            factor = rinput(irnp+icurnr)
            if ( iratio>6 .or. iratio==3 .or. iratio==4 .or. &
                 iratio<-3 .or. icurve/=8 .and. iratio<0 ) then
               call errint ( iratio, 1 )
               call errint ( icurnr, 2 )
               if ( icurve==1 ) then
                  call errchr ( 'line',1 )
               else if ( icurve==2 ) then
                  call errchr ( 'user',1 )
               else if ( icurve==3 ) then
                  call errchr ( 'arc',1 )
               else if ( icurve==4 ) then
                  call errchr ( 'spline',1 )
               else if ( icurve==8 ) then
                  call errchr ( 'parameter',1 )
               else if ( icurve==22 ) then
                  call errchr ( 'ell_arc',1 )
               end if
               call errsub ( 126, 2, 0, 1)
               go to 1000
            end if

         else

!        --- icurve=20

            factor = 1d0

         end if  ! ( icurve/=20 )
         if ( icurve/=2 ) call msh004 ( tau, ishape, nelm, inpncv, &
               iratio, factor, isubd )

         if ( icurve==1 ) then

!        --- icurve  =  1   straight line

            call msh005 ( ndim, curvesmsh(1,iposc), iinput(iinp), &
                          rinput(irnp1), tau, inpncv, icurnr )
         else if ( icurve==2 ) then

!        --- icurve  =  2  user defined  curve

            call msh007 ( ndim, curvesmsh(1,iposc), iinput(iinp), &
                          rinput(irnp1), inpncv, nuspnt, nelm, ishape, &
                          icurnr )

         else if ( icurve==3 .or. icurve==17 ) then

!        --- icurve  =  3  arc (circle segment)

            call msh006 ( .false., ndim, curvesmsh(1,iposc), &
                          iinput(iinp), rinput(irnp1), tau, inpncv, &
                          nuspnt, curstr(istartcur), icurnr, icurve )

         else if ( icurve==4 ) then

!        --- icurve  =  4 spline

            call mshc01 ( iinput(iinp), rinput(irnp1), &
                          curvesmsh(1,iposc), ndim, inpncv, tau, &
                          rinput(irnp2+ncurvs+2*icurnr-2), icurnr )

         else if ( icurve==8) then

!        --- icurve  =  8  parameter function

            call msh032 ( .false., ndim, curvesmsh(1,iposc), &
                          iinput(iinp), rinput(irnp1), tau, inpncv, &
                          nuspnt, sleng, icurnr, ncurvs, jcoars, &
                          iratio, curvefacts(jposc) )

         else if ( icurve==15 ) then

!        --- icurve  =  15  profile

            call mshcurprof ( .false., curvesmsh(1,iposc), iinput(iinp), &
                              rinput(irnp1), tau, inpncv, nuspnt, sleng, &
                              icurnr, ncurvs, jcoars, iratio, &
                              curvefacts(jposc) )

         else if ( icurve==20 ) then

!        --- icurve  =  20  own_curve

            call mshowncurve ( ndim, curvesmsh(1,iposc), iinput(iinp-1), &
                               rinput(irnp1), inpncv, nuspnt, icurnr )

         else if ( icurve==22 ) then

!        --- icurve  = 22  ell_arc (ellips segment)

            call mshellarc ( .false., ndim, curvesmsh(1,iposc), &
                             iinput(iinp), rinput(irnp1), tau, inpncv, &
                             nuspnt, curstr(istartcur) )

         end if

      end if

      deallocate ( tau, ihelp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'tau' )

1000  call erclos ( 'mshcurcr' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcurcr'

      end if  ! ( debug )

      end subroutine mshcurcr
