      subroutine mshcurprof ( coarse, curves, iinput, rinput, tau, n, &
                              nuspnt, sleng, icurnr, ncurvs, jcoars, &
                              iratio, curvefacts )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 23-09-2009 Extension with maxratio
!        version  2.0  date 22-08-2001 Extra parameter curvefacts
!        version  1.0  date 16-06-1999
!
!   copyright (c) 1999-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array curves, in case of a profile
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     mesh_generation
!     profile
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical coarse
      integer iinput(*), n, nuspnt, icurnr, jcoars, iratio, ncurvs
      double precision curves(2,*), rinput(*), tau(*), sleng, &
                       curvefacts(*)

!     coarse         i    indication whether sleng has been computed before
!                         (true) or not (false)
!     curvefacts     o    Array containing the factors t along the curves
!     curves         o    array that must be filled with the coordinates
!                         of the curves
!     icurnr         i    curve number
!     iinput         i    standard input array for subroutine mesh
!                         it is supposed that the information for the curve
!                         is stored from position 1
!     iratio         i    indication of the parameter ratio in the input records
!                         Is only used for the function param
!     jcoars         i    indication whether coarseness is used (1) or not (0)
!     n              i    number of points on curve
!     ncurvs         i    Number of curves in mesh
!     nuspnt         i    Number of user points
!     rinput         i    standard input arrays for subroutine mesh
!                         it is supposed that the first five
!                         positions have been deleted
!     sleng         i/o   length of curve (only when coarse is true)
!     tau            i    array containing the relative arc lengths for
!                         the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j1, j2, k, itel, i, j, iprofile, ih
      double precision x1, x2, y1, y2, y, x, dt, xp, t0, t1, &
                       yp, t, sprec, tprec, si, s, tp, sp, const, c1
      parameter ( const = 1.008930411365d0 )

!     c1             Inverse of const
!     const          constant to be used for correction of naca0012 formulas
!     dt             Step along arc
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     iprofile       Defines the type of profile
!                    Possible values:
!                    1: upper part of naca0012
!                    2: lower part of naca0012
!     itel           counter
!     j              Counting variable
!     j1             Sequence number of first user point on profile
!     j2             Sequence number of last user point on profile
!     k              Counting variable
!     s              Arc length
!     si
!     sp
!     sprec          previous value of s
!     t              Parameter along the curve
!     t0             Starting value of t
!     t1             End value of t
!     tp
!     tprec          previous value of t
!     x              x co-ordinate of point
!     x1             x-coordinate of first point in curve
!     x2             x-coordinate of last point in curve
!     xp             x-coordinate of previous point
!     y              y-coordinate read from the SEPRAN neutral plot file
!     y1             y-coordinate of first point in curve
!     y2             y-coordinate of last point in curve
!     yp             y-coordinate of previous point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision naca0012
      naca0012(t) = 0.6d0*(.2969d0*sqrt(t) +t * ( - 0.126d0 + t* &
                    (-0.3516d0 + t*(0.2843d0 - 0.1015d0*t))))

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     NACA0012       Defines formula for NACA0012 profile
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1197   Parameter out of range
!    1198   t0>=t1
!    1722   User point number out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcurprof' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurprof'
         write(irefwr,1) 'n', n
         call prinrl ( tau, n, 'tau' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if

      c1 = 1d0/const
      iprofile = iinput(1)
      j1 = iinput(2)
      j2 = iinput(3)
      if ( j1<=0 .or. j1>nuspnt ) then
         call errint ( j1, 1 )
         call errint ( nuspnt, 2 )
         call errint ( icurnr, 3 )
         call errsub ( 1722, 3, 0, 0 )
      end if
      if ( j2<=0 .or. j2>nuspnt ) then
         call errint ( j2, 1 )
         call errint ( nuspnt, 2 )
         call errint ( icurnr, 3 )
         call errsub ( 1722, 3, 0, 0 )
      end if
      x1 = rinput(2*(j1-1)+1)
      x2 = rinput(2*(j2-1)+1)
      y1 = rinput(2*(j1-1)+2)
      y2 = rinput(2*(j2-1)+2)
      if ( jcoars==1 ) then
         ih = nuspnt*3+2+ncurvs  !  change due to extra maxratio
      else
         ih = nuspnt*2+ncurvs
      end if
      t0 = rinput(ih+icurnr*2-1)
      t1 = rinput(ih+icurnr*2)
      if ( debug ) write(irefwr,2) 't0, t1', t0, t1
      if ( t0<0d0 .or. t0>1d0 ) then

!     --- t0 out of range

         call erreal ( t0, 1 )
         call errchr ( 't0', 1 )
         call errint ( iprofile, 1 )
         call errsub ( 1197, 1, 1, 1 )

      end if
      if ( t1<0d0 .or. t1>1d0 ) then

!     --- t1 out of range

         call erreal ( t1, 1 )
         call errchr ( 't1', 1 )
         call errint ( iprofile, 1 )
         call errsub ( 1197, 1, 1, 1 )

      end if
      if ( t0>=t1 ) then

!     --- t0>=t1

         call erreal ( t0, 1 )
         call erreal ( t1, 1 )
         call errint ( iprofile, 1 )
         call errsub ( 1198, 1, 2, 0 )

      end if
      if (  iratio>=0 .and. .not. coarse ) then

!     --- not coarse   compute length of curve

         dt = 0.1d0*(t1-t0)/n
         xp = 0d0
         yp = 0d0
         sleng = 0d0
         do k = 1, 10*n
            t = t0+k*dt
            y = naca0012(t)*c1
            x = t*c1
            sleng = sleng+sqrt((x-xp)**2+(y-yp)**2)
            yp = y
            xp = x
         end do
         sleng = sleng+sqrt((x2-xp)**2+(y2-yp)**2)

      end if

!     --- store first and last point in curves

      curves(1,1) = x1
      curves(1,n) = x2
      curves(2,1) = y1
      curves(2,n) = y2
      curvefacts(1) = t0
      curvefacts(n) = t1

!     --- find rest of co-ordinates

      if ( n==2 ) go to 1000

      if ( iratio>=0 ) then

!     --- iratio>0,  standard situation

         itel = 2
         sprec = 0d0
         tprec = t0
         si = tau(2)*sleng
         xp = x1
         yp = y1
         do i = 1, n
            dt = (t1-t0)*0.1d0*(tau(itel)-tau(itel-1))
            s = sprec
            t = tprec
            do j = 1, 1000*n
               tp = t
               sp = s
               t = min(t+dt, 1d0)
               y = naca0012(t)*c1
               x = t*c1
               s = s+sqrt((x-xp)**2+(y-yp)**2)
               xp = x
               yp = y
               if ( s>si ) then

!              --- point found between tp and t ( in fact sp and s )
!                  search for corresponding t value by linear interpolation

                  tprec = t
                  sprec = s
270               t = ((s-si)*tp+(si-sp)*t)/(s-sp)
                  y = naca0012(t)*c1
                  if ( iprofile==2 ) y = -y
                  x = t*c1
                  curves(1,itel) = x1+x*(x2-x1)+y/sleng*(y1-y2)
                  curves(2,itel) = y1+x*(y2-y1)+y/sleng*(x2-x1)
                  curvefacts(itel) = t
                  itel = itel+1
                  if ( itel>=n ) go to 1000
                  si = tau(itel)*sleng
                  if ( si<s ) then
                     goto 270
                  else
                     goto 350
                  end if
               end if
            end do
350      end do

      else

!     --- iratio<0, the distribution of t is completely determined by tau

         dt = t1 - t0
         do i = 2, n-1
            t = tau(i) * dt
            y = naca0012(t)*c1
            if ( iprofile==2 ) y = -y
            x = t*c1
            curves (1,i) = x1+x*(x2-x1)+y/sleng*(y1-y2)
            curves (2,i) = y1+x*(y2-y1)+y/sleng*(x2-x1)
            curvefacts(i) = t
         end do

      end if
1000  call erclos ( 'mshcurprof' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( curves, n, 2, 'curves' )
         call prinrl ( curvefacts, n, 'curvefacts' )
         write(irefwr,*) 'End mshcurprof'

      end if

      end
