      subroutine mshcut01 ( curves, curvesnw, lencurvs, coorln, icutcrnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 12-01-2009 New call to mshsl3
!        version  2.3  date 14-10-2002 New storage composite curves
!        version  2.2  date 28-12-1997 Check on kmeshz>=2
!                                      Correction work(.) in call to mshsl3
!
!   copyright (c) 1996-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Cut free boundary with given line, and remove all points below
!   that line. The cut curve is reset to the new cutting line
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lencurvs, icutcrnr
      double precision coorln(2,2), curves(2,*), curvesnw(2,*)

!     coorln     i    Array containing the co-ordinates of two points
!                     at the cutting line
!                     coorln(1,i) = x_i;  coorln(2,i) = y_i;
!     curves     i    array containing the old coordinates of the curves
!     curvesnw   o    array containing the new coordinates of the curves
!                     The length might be larger than that of curves
!                     Overestimate: 5*length curves
!     icutcrnr   i    Contains curve number corresponding to cut line, i.e.
!                     the curve icutcrnr is reset to the cut line
!     lencurvs  i/o   At input: declared length of curvesnw
!                     at output: actual length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, icount, icurnrin(2,3), icurve, iinp, &
              ipos, isub, isubcv, isum, isumold, isumsave, &
              jsubcv, n, nelm, nh, nold, nsubcv
      double precision t1, t2, h, x1, x2, y1, y2, xd, yd, n1, n2, &
                       xbound(2,2), x, y, beta, alpha, alphasav, a
      logical found
      integer, allocatable :: icurvs(:), iwork1(:)
      double precision, allocatable :: work(:)

!     a          Help variable to store quantity
!     alpha      Parameter alpha in computation of intersection point
!     alphasav   Help variable to store the value of alpha temporarily
!     beta       Parameter beta in computation of intersection point
!     found      If true an intersection point has been found
!     h          Help variable to compute the norm of a vector
!     i          Counting variable
!     i1         Help variable to store an integer quantity temporarily
!     i2         Help variable to store an integer quantity temporarily
!     icount     number of intersection points with cut line
!     icurnrin   Array that is filled as follows:
!                icurnrin(i,.) refers to i-th intersection point
!                icurnrin(.,1) contains curve sequence number of curve that
!                              intersects cut line
!                icurnrin(.,2) contains position in array icurvs close to
!                              intersection point
!                icurnrin(.,3) indicates if intersection point is above (1)
!                              or below cut line (-1)
!     icurve     Type of curve
!     icurvs     array containing the number of points in the old curves
!                accumulated
!     iinp       last position used in array iinputkm
!     ipos       position used in array curvesnw with respect to subcurve
!     isub       Loop parameter
!     isubcv     Subcurve sequence number with direction
!     isum       Number of nodes in the previous old curves
!     isumold    Number of nodes in the previous new curves
!     isumsave   Help variable to store the value of isumold
!     iwork1     integer work array of length 2+length curves
!     jsubcv     |isubcv|
!     n          Parameter for mesh generation
!     n1         First component of normal to cutting line
!     n2         Second component of normal to cutting line
!     nelm       number of elements along a curve
!     nh         number of nodes along a subcurve
!     nold       Number of nodes along the old curve
!     nsubcv     Number of subcurves
!     t1         First component to compute the tangential vector
!     t2         Second component to compute the tangential vector
!     work       Work array of length 16*number of points in the curves
!     x          x co-ordinate of point
!     x1         x-coordinate of first point in curve
!     x2         x-coordinate of last point in curve
!     xbound     Contains the co-ordinates of the intersection points
!                xbound(1,i) x-co-ordinate
!                xbound(2,i) x-co-ordinate
!     xd         x-co-ordinate of point below cutting line
!     y          y-coordinate
!     y1         y-coordinate of first point in curve
!     y2         y-coordinate of last point in curve
!     yd         y-co-ordinate of point below cutting line
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision interspnt
      interspnt ( x1, y1, t1, t2, xd, yd, n1, n2 ) = &
                ( -t2*(x1-xd)+t1*(y1-yd) ) / ( -n1*t2+t1*n2 )

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
!     INTERSPNT      Computes intersection of two lines
!     MSH046         Raise iinp as function of icurve
!     MSHSL2         Compute nelm for a free boundary
!     MSHSL3         Compute coordinates of a curve for a free boundary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    1004   Composite curves have not yet been implemented for this subroutine
!    1648   More than 2 free curves intersect cut line
!    1649   cut line intersects free curve more than once
!    1650   cutting line is not a straight line
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Find tangential and normal vector along cutting line
!    The normal is defined downwards
!    For all free curves do
!       If ( there is an intersection with the cutting line ) then
!          store co-ordinates of cross-section (at most 2)
!          Mark curve
!    For all curves do
!       if ( curve is not a free curve ) then
!          Copy old co-ordinates to new ones
!       else if ( curve is cutting line ) then
!          Compute new co-ordinates and number of nodes
!       else if ( curve is cut by cutting line ) then
!          Compute new co-ordinates and number of nodes
!       else
!          Check if curve above cutting line
!          Copy old co-ordinates to new ones
!    In order to find the down ward direction of the normal we proceed as
!    follows
!    The cutline is the line x_1 + beta t
!    Define the point x_D = ( (x1+x2)/2, (y1+y2)/2-1000 ) below the cutting line
!    The normal through x_D is the line x_D + alpha n
!    The intersection of cutting line and normal through x_D satisfies:
!    x_D + alpha n = x_1 + beta t
!    If alpha<0, the normal is downwards, otherwise the sign is reversed
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcut01' )

!     --- Find tangential and normal vector along cutting line
!         The normal is defined downwards

      x1 = coorln(1,1)
      x2 = coorln(1,2)
      y1 = coorln(2,1)
      y2 = coorln(2,2)
      t1 = x2 - x1
      t2 = y2 - y1
      h = sqrt(t1**2+t2**2)
      t1 = t1/h
      t2 = t2/h
      n1 = t2
      n2 =-t1
      ndim = 2

!     --- Check downward direction by defining a point (x_D, y_D) below the
!         cutting line and computing the intersection of the normal through
!         this point with the cutting line  (x_D,y_D) + alpha n = x_1 + beta t
!         If alpha>0, reverse sign

      xd = 0.5d0*(x1+x2)
      yd = 0.5d0*(y1+y2)-1d3
      alpha = interspnt ( x1, y1, t1, t2, xd, yd, n1, n2 )
      if ( alpha>0d0 ) then
         n1 = -n1
         n2 = -n2
      end if

!     --- For all free curves do

      allocate ( icurvs(0:ncurvs), iwork1(2*lencrv), work(16*lencrv), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, ncurvs+lencrv, 'icurvs' )
      if ( ierror/=0 ) go to 1000

      icurvs(0) = 0
      icurvs(1:ncurvs) = icurvsmsh(1:ncurvs)

      isum = 0
      icount = 0
      nsubcv = 0
      do icurnr = 1, ncurvs
         if ( kmeshz(icurnr)>=2 ) then

!        --- Free curve found
!            Check if there is an intersection with the cutting line
!            If ( there is an intersection with the cutting line ) then
!            store co-ordinates of cross-section (at most 2)
!            Mark curve

            if ( kelmq>0 ) &
               nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
            if ( nsubcv>1 ) then

!           --- Number of subcurves>1 not yet implemented

               call errsub ( 1004, 0, 0, 0 )
               go to 1000

            end if
            found = .false.
            do i = isum+1, icurvs(icurnr)-1
               alpha = interspnt ( x1, y1, t1, t2, &
                                   curves(1,i), curves(2,i), &
                                   curves(1,i+1)-curves(1,i), &
                                   curves(2,i+1)-curves(2,i) )
               if ( alpha>-1d-5 .and. alpha<1d0+1d-5 ) then

!              --- Candidate for intersection found

                  x = curves(1,i) + alpha*(curves(1,i+1)-curves(1,i))
                  y = curves(2,i) + alpha*(curves(2,i+1)-curves(2,i))
                  if ( .not. found ) then

!                 --- First intersection point

                     icount = icount + 1
                     if ( icount>2 ) then

!                    --- Too many intersection points

                        call errint ( icurnrin(1,1), 1 )
                        call errint ( icurnrin(2,1), 2 )
                        call errint ( icurnr, 3 )
                        call erreal ( xbound(1,1), 1 )
                        call erreal ( xbound(2,1), 2 )
                        call erreal ( xbound(1,2), 3 )
                        call erreal ( xbound(2,2), 4 )
                        call erreal ( x, 5 )
                        call erreal ( y, 6 )
                        call errsub ( 1648, 3, 6, 0 )
                        go to 1000
                     end if
                     xbound(1,icount) = x
                     xbound(2,icount) = y
                     icurnrin(icount,1) = icurnr
                     icurnrin(icount,2) = i

!                    --- Check if curve starts below or above the cutting
!                        line

                     i1 = icurvs(icurnr-1)
                     i2 = icurvs(icurnr)
                     beta = interspnt ( x1, y1, t1, t2, &
                                        curves(1,i1+1), curves(2,i1+1), &
                                        n1, n2 )
                     if ( beta<0d0 ) then

!                    --- Start of curve below cutting line

                        icurnrin(icount,3) = -1
                        if ( abs(1d0-alpha)>=1d-2 ) &
                           icurnrin(icount,2) = icurnrin(icount,2)+1

                     else

!                    --- Start of curve above cutting line

                        icurnrin(icount,3) = 1
                        if ( abs(alpha)<=1d-2 ) &
                           icurnrin(icount,2) = icurnrin(icount,2)+1

                     end if
                     alphasav = alpha

                  else

!                 --- Second intersection point, both should be almost equal

                     if ( alphasav<1d0-1d-2 .or. alpha>1d-2 ) then

!                    --- Error: previous intersection point not the same
!                               as present one

                        call errint ( icurnr, 1 )
                        call erreal ( xbound(1,icount), 1 )
                        call erreal ( xbound(2,icount), 2 )
                        call erreal ( x, 3 )
                        call erreal ( y, 4 )
                        call errsub ( 1649, 1, 4, 0 )
                        go to 1000
                     else if ( alpha>1d0-alphasav ) then
                        xbound(1,icount) = x
                        xbound(2,icount) = y
                        icurnrin(icount,2) = i
                        icurnrin(icount,3) = 1
                     end if
                     alphasav = 0d0

                  end if

               end if

            end do

         end if
         isum = icurvs(icurnr)

      end do

      isumold = 0
      isum = 0
      iinp  = iinp1
      icount = 1

!     --- Loop over curves

      do icurnr = 1, ncurvs

!     --- Get icurve from iinputkm

         icurve = iinputkm(iinp)
         nold = icurvs(icurnr)-isumold
         ipos = isumold
         if ( kelmq>0 ) then
            nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
         else
            nsubcv = 1
         end if

         if ( nsubcv>1 ) then

!        --- Curve contains sub curves
!            Copy information from sub curves

            n = 0
            do isub = 1, nsubcv
               isubcv = kmeshq(kmeshq(icurnr)+isub)
               jsubcv = abs(isubcv)
               if ( jsubcv==1 ) then
                  nh = icurvsmsh(1)
                  ipos = 0
               else
                  ipos = icurvsmsh(jsubcv-1)
                  nh = icurvsmsh(jsubcv)-ipos
               end if
               if ( isubcv>0 ) then
                  do i = 1, nh
                     curvesnw(1,isum+i+n) = curvesnw(1,ipos+i)
                     curvesnw(2,isum+i+n) = curvesnw(2,ipos+i)
                  end do
               else
                  do i = 1, nh
                     curvesnw(1,isum+i+n) = curvesnw(1,ipos+nh+1-i)
                     curvesnw(2,isum+i+n) = curvesnw(2,ipos+nh+1-i)
                  end do
               end if
               n = n+nh
            end do
            if ( n>0 ) n = n+1

         else if ( icurve==0 ) then

!        --- empty curve

            n = 0

         else if ( icurnr==icutcrnr ) then

!        --- Cut line found

            if ( icurve/=1 .and. icurve/=5 ) then

!           --- Cut line is not a straight line

               call errint ( icurnr, 1 )
               call errsub ( 1650, 1, 0, 0 )
               go to 1000

            end if

!           --- Compute number of nodes along curve
!               In order to fake subroutine mshsl2
!               icurvs(icurnr) is set equal to 2 and icurvs(icurnr-1) equal to 0
!               Instead of curves we use xbound

            i1 = icurvs(icurnr-1)
            i2 = icurvs(icurnr)
            icurvs(icurnr) = 2
            icurvs(icurnr-1) = 0
            call mshsl2 ( icurnr, icurve, xbound, iinp, iinputkm, &
                          rinputkm(3), nelm, icurvs, elsize )

!           --- Reset icurvs

            icurvs(icurnr-1) = i1
            icurvs(icurnr) = i2
            ishape = iinputkm(iinp+1)
            iinputkm(iinp+2) = nelm
            n = nelm*ishape+1

!           --- Fill array curvesnw with straight line
!               Check if the direction of line through the cutting points is
!               the original direction. If not reverse

            a = t1*(xbound(1,2)-xbound(1,1)) + &
                t2*(xbound(2,2)-xbound(2,1))

            if ( a<0d0 ) then
               work(1) = xbound(1,2)
               work(2) = xbound(2,2)
               work(3) = xbound(1,1)
               work(4) = xbound(2,1)
               call mshsl3 ( work, 2, curvesnw(1,isum+1), iinp, &
                             iinputkm, rinputkm(3), work(5), &
                             icurve, iwork1, ndim, nuspnt, icurnr )
            else
               call mshsl3 ( xbound, 2, curvesnw(1,isum+1), iinp, &
                             iinputkm, rinputkm(3), work, &
                             icurve, iwork1, ndim, nuspnt, icurnr )
            end if

         else if ( icurnr==icurnrin(icount,1) ) then

!        --- Part of free boundary, intersected by cutting line found
!            Compute number of nodes along curve
!            First find which points are above the cutting line

            i1 = icurvs(icurnr-1)
            i2 = icurvs(icurnr)
            i = icurnrin(icount,2)
            curves(1,i) = xbound(1,icount)
            curves(2,i) = xbound(2,icount)
            if ( icurnrin(icount,3)<0 ) then

!           --- Start of curve below cutting line
!               Only the points from the cutting line must be used

               icurvs(icurnr-1) = i-1
               nh = i2-i+1
               isumsave = isumold
               isumold = i-1

            else

!           --- Start of curve above cutting line
!               Only the points until the cutting line must be used

               icurvs(icurnr) = i
               nh = i-i1

            end if
            call mshsl2 ( icurnr, icurve, curves, iinp, iinputkm, &
                          rinputkm(3), nelm, icurvs, elsize )

!           --- Reset icurvs

            icurvs(icurnr-1) = i1
            icurvs(icurnr) = i2
            ishape = iinputkm(iinp+1)
            iinputkm(iinp+2) = nelm
            n = nelm*ishape+1

!           --- Fill array curvesnw with spline

            call mshsl3 ( curves(1,isumold+1), nh, curvesnw(1,isum+1), &
                          iinp, iinputkm, rinputkm(3), work, &
                          icurve, iwork1, ndim, nuspnt, icurnr )
            iinputkm(iinp) = abs(icurve)
            if ( icurnrin(icount,3)<0 ) isumold = isumsave
            icount = icount+1

         else

!        --- Other type of curve
!            Copy co-ordinates from curves to curvesnw

            n = nold
            do i = 1, n
               curvesnw(1,isum+i) = curves(1,isumold+i)
               curvesnw(2,isum+i) = curves(2,isumold+i)
            end do

         end if

!        --- Raise parameters

         if ( isum>lencurvs ) then
            call errint ( 2*isum, 1 )
            call errint ( lencurvs, 2 )
            call errchr ( 'curvesnw', 1 )
            call errsub ( 550, 2, 0, 1 )
         end if
         isum = isum + n
         isumold = isumold + nold
         call msh046 ( iinputkm, iinp, icurve )
         icurvsmsh(icurnr) = isum

      end do
      lencurvs = isum

      deallocate ( icurvs, iwork1, work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'icurvs' )

1000  call erclos ( 'mshcut01' )
      if ( ierror/=0 ) call instop

      end
