      subroutine mshfinddisj ( irenum, isplit )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if mesh consists of disjoint parts
!     If so renumber nodes per part
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     renumbering
!     disjoint
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: irenum(npoint), isplit(npoint)

!     irenum         o    Array in which the new node numbers are stored
!     isplit         o    Contains the number of nodes per disjoint set
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node, icount, mset

!     i              Counting variable
!     icount         Counter
!     mset           number of nodal points in ineigh
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     SEPNEXTLEVEL   Find neighbors corresponding to a set of nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfinddisj' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfinddisj'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( marknodes(npoint), nextlevel(npoint), iwork(npoint), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, 3*npoint, 'nextlevel' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array marknodes

      marknodes = 0
      nsplit = 1
      node = 1
      icount = 1
      isplit(1) = 1

      do while ( node<=npoint )

!     --- node<=npoint

         iwork(1) = node
         if ( debug ) &
            write(irefwr,1) 'node, npoint, nsplit',node, npoint, nsplit
         nlevel = 1
         do while ( nlevel>0 )

!        --- nlevel>0
!            Find all neighbors neigh of node with marknodes(neigh) = 0

            lastlevel => iwork(1:nlevel)
            call sepnextlevel ( -1, mset )

!           --- Store these nodes in iwork
!               nlevel = number of neighbours stored in iwork
!               Set marknodes equal to 1 for the nodes in iwork

            nlevel = mset
            iwork(1:nlevel) = nextlevel(1:nlevel)
            marknodes(nextlevel(1:nlevel)) = -nsplit

            if ( debug ) call prinin ( iwork, nlevel, 'iwork' )
            icount = icount+nlevel

         end do ! while ( nlevel>0 )

         if ( debug ) write(irefwr,1) 'icount, npoint', icount, npoint

         if ( icount<npoint ) then

!        --- icount<npoint, hence there are nodes left
!            The mesh consists of disjoint parts

            nsplit = nsplit+1
            call inichkarlen ( nsplit, npoint, 'isplit' )
            if ( ierror/=0 ) go to 1000
            isplit(nsplit) = icount

         end if  ! ( icount<npoint )
         node = icount

      end do  ! while ( node<=npoint )
      if ( debug ) call prinin ( marknodes, npoint, 'marknodes' )

      if ( nsplit>1 ) then

!     --- nsplit>1, fill array irenum

         do i = 1, npoint
            nlevel = -marknodes(i)
            irenum(i) = isplit(nlevel)
            isplit(nlevel) = isplit(nlevel)+1
         end do  ! i = 1, npoint
         isplit(1:nsplit) = isplit(1:nsplit)-1

         if ( debug ) then
            call prinin ( isplit, nsplit, 'isplit' )
            call prinin ( irenum, npoint, 'irenum' )
         end if  ! ( debug )

      end if  ! ( nsplit>1 )

      deallocate ( nextlevel, iwork, marknodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nextlevel' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfinddisj'

      end if  ! ( debug )
      call erclos ( 'mshfinddisj' )

      end
