      subroutine mshg10 ( x1, y1, x2, y2, xi, yi, xj, yj, iresul )
! ======================================================================
!
!        programmer    Sander de Snoo
!        version  1.0  date 30-12-1995
!
!   copyright (c) 1985-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine whether lines  1 - 2  and  I - J have an intersection
!     point
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     2d
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x1, y1, x2, y2, xi, yi, xj, yj
      integer iresul

!     iresul     o    indicator of iresul
!                     iresul = -2 : lines in each others safety zone
!                     iresul = -1 : lines are overlapping
!                     iresul =  0 : there is an intersection point
!                     iresul =  1 : there is no intersection point
!                     iresul =  2 : lines are on one line, no overlap
!                     iresul =  3 : lines are parallel
!     x1         i    x-coordinate point 1
!     x2         i    x-coordinate point 2
!     xi         i    x-coordinate point i
!     xj         i    x-coordinate point j
!     y1         i    y-coordinate point 1
!     y2         i    y-coordinate point 2
!     yi         i    y-coordinate point i
!     yj         i    y-coordinate point j
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision D,h,n1,ni,d12,dij,d1i,d1j,d2i,d2j,eps, &
                       x12,y12,xij,yij

!     d
!     d12
!     d1i
!     d1j
!     d2i
!     d2j
!     dij
!     eps        Local accuracy
!     h          Help variable to store a constant temporarily
!     n1
!     ni         Upper boundary in the computation for the loop variable i
!     x12
!     xij
!     y12
!     yij
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Calculate surfaces of
!         D :(1-2),(i-j)
!         n1:(1-2),(1-i)
!         ni:(i-j),(1-i)
!       intersectionpoint is at n1/D on line 1-2 and ni/D on i-j
!         intersection if: 0 <= n1/D <= 1 and 0 <= n2/D <= 1
!       (take care of truncation errors in D and compare without division)
!       if there is no intersection:
!         check if one of the end points of the lines is in the
!         'safety' zone of the other line.
!       (safety zone = area of distance h along the line, &
!         cut at 30 degrees at end points;
!         h = 1/4*min(length 1-2, length i-j)
!       note: the safety zone is to prevent the dividing line of
!             a concave area to pass close to parts of the border.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      x12 = x2-x1
      xij = xj-xi
      y12 = y2-y1
      yij = yj-yi

      D  = (x12)*(yij)-(y12)*(xij)
      n1 = (x12)*(y1-yi)-(y12)*(x1-xi)
      ni = (xij)*(y1-yi)-(yij)*(x1-xi)

      if (D.le.0) then
        D=-D
        n1=-n1
        ni=-ni
      end if

      eps = 5* epsmac * ( (abs(x1)+abs(x2)) * (abs(yi)+abs(yj)) + &
                          (abs(xi)+abs(xj)) * (abs(y1)+abs(y2)) )

      if ( D.lt.eps ) then

!     --- parallel or overlapped

        if ((abs(n1).lt.eps).and.(abs(ni).lt.eps)) then

!       --- lines on one line

          d12=(x12)**2+(y12)**2
          dij=(xij)**2+(yij)**2
          d1i=(x1-xi)**2+(y1-yi)**2
          d1j=(x1-xj)**2+(y1-yj)**2
          d2i=(x2-xi)**2+(y2-yi)**2
          d2j=(x2-xj)**2+(y2-yj)**2
          if ( ((d1i.gt.d12).and.(d1j.gt.d12)).or. &
               ((d2i.gt.d12).and.(d2j.gt.d12)).or. &
               ((d1i.gt.dij).and.(d2i.gt.dij)).or. &
               ((d1j.gt.dij).and.(d2j.gt.dij)) ) then
            iresul = 2
          else

!         --- there is partial overlap.

            iresul = -1
          end if
        else

!       --- parallel

          iresul = 3
        end if
      else
        if ( (n1.ge.0).and.(n1.le.D).and.(ni.ge.0).and.(ni.le.D) ) then
          iresul = 0
        else
          iresul = 1
        end if
      end if

      if (iresul.gt.0) then

!     --- check safety zone

        d12=sqrt((x12)**2+(y12)**2)
        dij=sqrt((xij)**2+(yij)**2)
        h = 0.25 * min(d12,dij)

        if ( (n1.ge.-h*d12).and.(n1.le.D+h*d12).and. &
             (ni.ge.-h*dij).and.(ni.le.D+h*dij) ) then

!       --- 1-2 and i-j are in each others area of width h along line
!           check 20 degrees cut off at end

          d1i=sqrt((x1-xi)**2+(y1-yi)**2)
          d1j=sqrt((x1-xj)**2+(y1-yj)**2)
          d2i=sqrt((x2-xi)**2+(y2-yi)**2)
          d2j=sqrt((x2-xj)**2+(y2-yj)**2)

          if ( (n1.le.0) .and. &
               ( (xi-x1)*(x12)+(yi-y1)*(y12).gt. 0.940*d12*d1i ) .and. &
               ( (xi-x2)*(-x12)+(yi-y2)*(-y12).gt. 0.940*d12*d2i ) )then

!         --- point i in zone of 1-2

            iresul = -2
            goto 100
          end if

          if ( (n1.ge.D) .and. &
               ( (xj-x1)*(x12)+(yj-y1)*(y12).gt. 0.940*d12*d1j ) .and. &
               ( (xj-x2)*(-x12)+(yj-y2)*(-y12).gt. 0.940*d12*d2j ) )then

!         --- point j in zone of 1-2

            iresul = -2
            goto 100
          end if

          if ( (ni.le.0) .and. &
               ( (x1-xi)*(xij)+(y1-yi)*(yij).gt. 0.940*dij*d1i ) .and. &
               ( (x1-xj)*(-xij)+(y1-yj)*(-yij).gt. 0.940*dij*d1j ) )then

!         --- point 1 in zone of i-j

            iresul = -2
            goto 100
          end if

          if ( (ni.ge.D) .and. &
               ( (x2-xi)*(xij)+(y2-yi)*(yij).gt. 0.940*dij*d2i ) .and. &
               ( (x2-xj)*(-xij)+(y2-yj)*(-yij).gt. 0.940*dij*d2j ) )then

!         --- point 2 in zone of i-j

            iresul = -2
            goto 100
          end if

        end if

      end if

100   continue

      end
