      subroutine msho50 ( jnew, bcord, kbndpt, irand, rinput, jdiag )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  6.2  date 29-05-2014 Extension of timebefore
!        version  6.1  date 15-09-2010 New call to mshcoons3drect
!        version  6.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1990-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a surface-mesh in 3-D space. Surface is formed by four lines
!     given in BCORD. Use the COONS method and MSHZDE or MSHER1.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     coons
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical jnew
      double precision bcord(*), rinput(*)
      integer jdiag, kbndpt(*), irand

!     bcord          i    coordinates of the boundary points
!     irand          i    helpvariable for number of points on
!                         boundary of element - 1
!     jdiag          i    If 1 all diagonals in the rectangles are directed in
!                         one direction if rectangle is called, if 2 they are
!                         ordered in alternating directions
!     jnew           i    indicator whether mesh is new or not
!     kbndpt         o    node numbers of boundary points
!     rinput         i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xtemp(3)
      integer nipnt, i, j, ipon, n1, n2, n3, n4, ne1, ne2, &
              ne3, ne4
      double precision, allocatable :: work(:), bcor2(:)
      double precision :: timebefore(2)

!     bcor2          help array for local 2-D boundary
!     i              loop variable
!     ipon           local node number
!     j              loop variable, node number
!     n1             number of points along first curve
!     n2             number of points along second curve
!     n3             number of points along third curve
!     n4             number of points along fourth curve
!     ne1            number of elements on the first boundary
!     ne2            number of elements on the second boundary
!     ne3            number of elements on the third boundary
!     ne4            number of elements on the fourth boundary
!     nipnt          number of boundary points
!     work           help array for 2-D meshgeneration
!     xtemp          temporary array to store nodes of end point of boundary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSHCOONS3DQUAD Create a coons surface-mesh in 3-D space in the
!                    case that opposite sides do not have the smae number of
!                    nodes using quadrilateral
!     MSHCOONS3DRECT Create a coons surface-mesh in 3-D space in the
!                    case that opposite sides have the smae number of nodes
!                    using rectangle
!     MSHMAPSUR3D    Map surface in 3D onto prescribed type of surface
!     MSHO51         Map boundary points on 2-d curve with length one
!     PRINRL1        Print 2d real vector
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Place boundary points on the four boundaries of the unit-square
!     Call routine (MSHZDE or MSHER1) to make a structured
!     grid over the unit-square. Map all points back, using the COONS
!     formula's.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msho50' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msho50'

      end if
      if ( itime>0 ) call ersettime ( timebefore )

!     --- Set number of boundary points

      ne1 = nx
      ne2 = ny
      ne3 = nz
      ne4 = nc
      n1 = ne1 * irand + 1
      n2 = ne2 * irand + 1
      n3 = ne3 * irand + 1
      n4 = ne4 * irand + 1
      nipnt = n1 + n2 + n3 + n4 - 4
      if ( debug ) &
         write(irefwr,*) 'icentre, ne1, ne2, ne3, ne4, irand, mapping3d ', &
                          icentre, ne1, ne2, ne3, ne4, irand, mapping3d

      ipon = max(n1,n2,n3,n4)
      allocate ( work(ipon), bcor2(2*(nipnt+1)), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, ipon+2*2*(nipnt+1), 'work' )
      if ( ierror/=0 ) go to 1000

!     --- Set extra (starting-)point at end of bcord

      j = 3 * nipnt
      xtemp(1) = bcord(j+1)
      xtemp(2) = bcord(j+2)
      xtemp(3) = bcord(j+3)
      bcord( j+1 ) = bcord(1)
      bcord( j+2 ) = bcord(2)
      bcord( j+3 ) = bcord(3)

!     --- Map boundary points to two-dimensional plane
!           ( compute relative coordinates of points on curves )

      call msho51(bcord, n1, work)
      ipon = 0
      do i = 1 , n1 - 1
          ipon = ipon + 1
          bcor2(2*ipon-1) = work(i)
          bcor2(2*ipon  ) =  0
      end do
      j = 3 * ( n1 - 1 ) + 1
      call msho51(bcord(j), n2, work)
      do i = 1 , n2 - 1
          ipon = ipon + 1
          bcor2(2*ipon-1) = 1d0
          bcor2(2*ipon  ) = work(i)
      end do
      j = 3 * ( n1 + n2 - 2 ) + 1
      call msho51(bcord(j), n3, work)
      do i = 1 , n3 - 1
          ipon = ipon + 1
          bcor2(2*ipon-1) = 1d0 - work(i)
          bcor2(2*ipon  ) = 1d0
      end do
      j = 3 * ( n1 + n2 + n3 - 3 ) + 1
      call msho51(bcord(j), n4, work)
      do i = 1 , n4 - 1
          ipon = ipon + 1
          bcor2(2*ipon-1) = 0d0
          bcor2(2*ipon  ) = 1d0 - work(i)
      end do
      ipon = ipon + 1
      bcor2(2*ipon-1) = bcor2(1)
      bcor2(2*ipon  ) = bcor2(2)
      if ( debug ) call prinrl1 ( bcor2, ipon, 2, 'bcor2' )

!     --- Compute points and elements in surface

      if ( ne1==ne3 .and. ne2==ne4 ) then

!     --- Simple situation use MSHER1

         call mshcoons3drect ( bcord, kbndpt, irand, bcor2, &
                               jnew, nipnt, jdiag, xtemp )

      else

!     --- Complicated situation use MSHZDE

         call mshcoons3dquad ( bcord, kbndpt, bcor2, jnew, &
                               nipnt, xtemp, irand, ipon )

      end if
      if ( ierror/=0 ) go to 1000

      if ( mapping3d>0 ) then

!     --- mapping3d>0, map on 3d surface that is given exactly

         call mshmapsur3d ( bcord, coor2d, nipnt, npointlc, mapping3d, &
                            isurnr, icentre, rinput )

      end if

      deallocate ( work, bcor2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )

      if ( itime==1 ) call printtime(' Coons-3D')
1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( coor2d, npointlc, 3, 'coor2d' )
         write(irefwr,*) 'End msho50'

      end if
      call erclos ( 'msho50' )

      end
