      subroutine msho60 ( new, kmeshcloc, kbndpt, bcord, &
                          ne1, ne2, ne3, ne4, umin, umax, vmin, vmax, &
                          irand )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.1  date 29-05-2014 Extension of timebefore
!        version  5.0  date 02-09-2010 Remove intarmsh
!        version  4.0  date 27-10-2003 Extra parameter intarmsh
!
!   copyright (c) 1993-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a surface-mesh in 3-D space using the subroutine FUNCSF
!     that describes the surface
! **********************************************************************
!
!                       KEYWORDS
!

!     3d
!     mesh_generation
!     parameter_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical new
      double precision bcord(*), umin, umax, vmin, vmax
      integer kmeshcloc(inpelm,*), kbndpt(*), ne1, ne2, ne3, ne4, &
              irand

!     bcord          i    coordinates of the boundary points
!     irand          i    number of points in one element side -1
!     kbndpt         o    node numbers of boundary points in local numbering
!     kmeshcloc      o    element array
!     ne1            i    number of elements along first side
!     ne2            i    number of elements along second side
!     ne3            i    number of elements along third side
!     ne4            i    number of elements along fourth side
!     new            i    indicator whether mesh is new or not
!     umax           i    max value first local coordinate 3d surface
!     umin           i    min value first local coordinate 3d surface
!     vmax           i    max value second local coordinate 3d surface
!     vmin           i    min value second local coordinate 3d surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, n1, n2, n3, n4, &
              nipnt, ipon, itim, mpoint, melem, icurve
      double precision coorh(3)
      double precision, allocatable :: work(:), bcor2(:), coortmp(:,:)
      logical trian
      double precision :: timebefore(2)

!     bcor2          help array to store the boundary of the 2-D help mesh
!     coorh          help array to determine coordinates in surface
!     coortmp        temporary 2d coordinates array
!     i              loop variable
!     icurve         helpvariable for MSHZDE
!     ipon           help-nodenumber
!     itim           helpvariable to store itime temporary
!     j              sequence number
!     melem          helpvariable for number of elements in MSHZDE
!     mpoint         helpvariable for number of points in MSHZDE
!     n1             number of points along first side
!     n2             number of points along second side
!     n3             number of points along third side
!     n4             number of points along fourth side
!     nipnt          number of initial points
!     trian          indicator whether triangles (true) or quadrilaterals
!                    (false) have to be created
!     work           help array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     FUNCSF         User routine which gives the 3-D coordinates of a (u,v)
!                    point in the surface
!     MSHER1         Submesh generator RECTANGLE
!     MSHO62         Compute the local distribution of nodes along the line
!     MSHO63         Compare two points that should be the same
!     MSHZDE         Submesh generator QUADRILATERAL
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Place the given boundary points on the four boundaries of the
!     unit-square. Call standard 2-D routine (MSHZDE or MSHER1) to make
!     a structured grid over the unit-square. Map all points back, using
!     the routine FUNCSF.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msho60' )
      if ( itime>0 ) call ersettime ( timebefore )

!     --- Set the number of boundary points on the four constituting
!         curves

      n1 = ne1 * irand + 1
      n2 = ne2 * irand + 1
      n3 = ne3 * irand + 1
      n4 = ne4 * irand + 1

      nipnt = nbndpt - 1

      ipon = max(n1,n2,n3,n4)
      length = size(coor2d,2)
      allocate ( work(ipon), bcor2(2*(nipnt+1)), coortmp(2,length), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, ipon+2*2*(nipnt+1)+2*length, 'work' )
      if ( ierror/=0 ) go to 1000

!     --- Set extra (starting-)point at end of bcord

      j = 3 * nipnt

      bcord( j+1 ) = bcord( 1 )
      bcord( j+2 ) = bcord( 2 )
      bcord( j+3 ) = bcord( 3 )

!     --- Check whether the four corner points in bcord coincide with
!         the points computed by FUNCSF.(Parameter representation of
!         surface)

      work( 1) = umin
      work( 2) = vmin
      call funcsf( isurnr, work, coorh, 1 )

      call msho63( isurnr, coorh, bcord, 1 )

      work( 1) = umax
      work( 2) = vmin
      call funcsf( isurnr, work, coorh, 1 )

      j = 3 * ( n1 - 1 ) + 1
      call msho63( isurnr, coorh, bcord(j), 2 )

      work( 1) = umax
      work( 2) = vmax
      call funcsf( isurnr, work, coorh, 1 )

      j = 3 * ( n1 + n2 - 2 ) + 1
      call msho63( isurnr, coorh, bcord(j), 3 )

      work( 1) = umin
      work( 2) = vmax
      call funcsf( isurnr, work, coorh, 1 )

      j = 3 * ( n1 + n2 + n3 - 3 ) + 1
      call msho63( isurnr, coorh, bcord(j), 4 )

!     --- Map boundarypoints to two-dimensional plane
!         ( compute relative coordinates of points on curves and check that
!           points on the curves are in ( or at least near to) the surface )

!         Run through first boundary curve and determine for each
!         point of bcord the "nearest" value from the surface

      call msho62( isurnr, 1, n1, umin, umax, vmin, bcord, work )

      ipon = 0

      do i = 1 , n1 - 1
          ipon = ipon + 1
          bcor2( 2*ipon-1 ) = work(i)
          bcor2( 2*ipon   ) =  0d0
      end do

      j =  n1 - 1

!     --- Run through second boundary curve and determine for each
!         point of bcord the "nearest" value from the surface

      call msho62( isurnr, 2, n2, vmin, vmax, umax, bcord(3*j+1), &
                   work )

      do i = 1 , n2 - 1
          ipon = ipon + 1
          bcor2( 2*ipon-1 ) = 1d0
          bcor2( 2*ipon   ) = work(i)
      end do

      j = n1 + n2 - 2

!     --- Run through third boundary curve and determine for each
!         point of bcord the "nearest" value from the surface

      call msho62( isurnr, 1, n3, umax, umin, vmax, bcord(3*j+1), &
                   work )

      do i = 1 , n3 - 1
          ipon = ipon + 1
          bcor2( 2*ipon-1 ) = 1d0 - work(i)
          bcor2( 2*ipon   ) = 1d0
      end do

      j = n1 + n2 + n3 - 3

!     --- Run through fourth boundary curve and determine for each
!         point of bcord the "nearest" value from the surface

      call msho62( isurnr, 2, n4, vmax, vmin, umin, bcord(3*j+1), &
                   work )

      do i = 1 , n4 - 1
          ipon = ipon + 1
          bcor2( 2*ipon-1 ) = 0d0
          bcor2( 2*ipon   ) = 1d0 - work(i)
      end do

      ipon = ipon + 1
      bcor2( 2*ipon-1 ) = bcor2(1)
      bcor2( 2*ipon   ) = bcor2(2)

!     --- Compute points and elements in surface

      if ( n1==n3 .and. n2==n4 ) then

!     --- Simple situation use MSHER1

         itim  = itime
         itime = 0

         call msher1( new, irand, kbndpt, bcor2, &
                      ne1, ne2, coortmp, kmeshcloc, 1 )

           itime = itim

!          --- Compute npointlc and nelemlc

           npointlc = n1 * n2
           nelemlc  = ne1 * ne2 * mtrian

           if ( ishape==7 ) npointlc = npointlc + nelemlc

      else

!     --- Complicated situation use MSHZDE

           itim  = itime
           itime = 0

           iblend = 2

           trian  = mtrian==2

           icurve = 0
           mpoint = npointlc
           melem  = nelemlc

           call mshzde ( new, trian, inpelm, icurve, iblend, &
                         ne1, ne2, ne3, ne4, ipon, bcor2, mpoint, &
                         melem, kbndpt, npointlc, nelemlc, coortmp, kmeshcloc )

           itime = itim

      end if

      deallocate ( work, bcor2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )

      allocate ( work(2*npointlc), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npointlc, 'work' )
      if ( ierror/=0 ) go to 1000

!     --- Map points back to rectangle as prescribed by user

      do i = 1 , npointlc

          work( 2*i-1 ) = umin + coortmp(1,i) * ( umax - umin )
          work( 2*i   ) = vmin + coortmp(2,i) * ( vmax - vmin )

      end do

!     --- Map new internal points to 3-D, call user routine

      call funcsf( isurnr, work, coor2d, npointlc )

!     --- All points and elements in new surface are determined

      if ( itime==1 ) call printtime( 'PARSURF')

      deallocate ( work, coortmp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )

1000  call erclos ( 'msho60' )

      end
