      subroutine msho62 ( isurf, ichois, np, tstart, tend, const, &
                          bcord, work )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.2  date 19-10-1993 Stop if point is not near
!                                      enough to line)
!        version  1.1  date 11-10-1993 New call to FUNCSF
!        version  1.0  date 01-10-1993
!
!   copyright (c) 1993-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine the local parameter values of a curve in a plane
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     point_distribution
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision bcord(*), work(*), tstart, tend, const
      integer isurf, ichois, np

!     bcord     i    coordinates of the boundary points on curve considered
!     const     i    value of constant parameter in curve on 3d surface
!     ichois    i    indicator whether first coordinate is running (1) or
!                    second coordinate (2)
!     isurf     i    surface number
!     np        i    number of points on curve as given in bcord
!     tend      i    max value running parameter in curve on 3d surface
!     tstart    i    min value running parameter in curve on 3d surface
!     work      o    array with local parameter values along the curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nstep
      double precision coorh(3), disold, disnew, dt, dis1, dis2, &
                       dx, dy, dz, refdis, cooruv(2)

!     coorh    helparray to determine coordinates in surface
!     cooruv   helparray to submit the local 2-d coordinates
!     dis1     local distance for first extra point
!     dis2     local distance for second extra point
!     disnew   new distance
!     disold   old distance
!     dt       stepsize running parameter
!     dx       x-distance
!     dy       y-distance
!     dz       z-distance
!     i        loop variable
!     j        sequence number
!     nstep    number of steps to be taken along curve
!     refdis   reference distance to determine whether curve point does
!              also belong to surface or not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   Deconcatenate name subroutine from string
!     EROPEN   Concatenate name of subroutine to string of names
!     ERRINT   Place integer value in error message array
!     ERRSUB   Submit error message
!     FUNCSF   User routine which gives the 3-D coordinates of a (u,v) point
!              in the surface
!     INSTOP   Stop SEPRAN execution
!     MSHO64   Determine Euclidian distance of two points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     957    Most probably point on curve is not in surface. Stop
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Start with first point of both BCORD and surface.
!     Run through all (NSTEP) points of surface and decide whether
!     point is nearest to BCORD point. Store, if so, parameter in
!     parameter array work.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msho62' )

!     --- Compute reference distance

      dx = bcord(1) - bcord(4)
      dy = bcord(2) - bcord(5)
      dz = bcord(3) - bcord(6)

      refdis = sqrt( dx*dx + dy*dy + dz*dz ) / 20d0

      nstep = 100 * ( np - 1 )

      dt = ( tend - tstart ) / nstep

      i = 1
      j = 1

      work(1) = 0d0

      if ( ichois.eq.1 ) then

           cooruv(1) = tstart
           cooruv(2) = const

      else if ( ichois.eq.2 ) then

           cooruv(1) = const
           cooruv(2) = tstart

      end if

      call funcsf( isurf, cooruv, coorh, 1 )
      call msho64( coorh, bcord(3*j+1), disold )

100   continue

!     --- Determine distance point j with point i + 1

      if ( ichois.eq.1 ) then

           cooruv(1) = tstart + i * dt
           cooruv(2) = const

      else if ( ichois.eq.2 ) then

           cooruv(1) = const
           cooruv(2) = tstart + i * dt

      end if

      call funcsf( isurf, cooruv, coorh, 1 )
      call msho64( coorh, bcord(3*j+1), disnew )

      if ( disnew.ge.disold ) then

!     --- Check which situation

           if ( ichois.eq.1 ) then

                cooruv(1) = tstart + (i-0.5) * dt
                cooruv(2) = const

           else if ( ichois.eq.2 ) then

                cooruv(1) = const
                cooruv(2) = tstart + (i-0.5) * dt

           end if

           call funcsf( isurf, cooruv, coorh, 1 )
           call msho64( coorh, bcord(3*j+1), dis1 )

           if ( ichois.eq.1 ) then

                cooruv(1) = tstart + (i-1.5) * dt
                cooruv(2) = const

           else if ( ichois.eq.2 ) then

                cooruv(1) = const
                cooruv(2) = tstart + (i-1.5) * dt

           end if

           call funcsf( isurf, cooruv, coorh, 1 )
           call msho64( coorh, bcord(3*j+1), dis2 )

           j = j + 1

           if ( disold.gt.refdis .and. &
                dis1  .gt.refdis .and. &
                dis2  .gt.refdis ) then

!          --- Problems in surface, curve point not in surface

                call errint( isurf, 1 )
                call errsub ( 957, 1, 0, 0 )
                call instop

           end if

           if ( dis1.lt.disold ) then

                work(j) = (i-0.5) * dt / ( tend - tstart )

           else if ( dis2.lt.disold ) then

                work(j) = (i-1.5) * dt / ( tend - tstart )

           else

                work(j) = (i-1.0) * dt / ( tend - tstart )

           end if

           disold = rinfin

!          --- Compute new reference distance

           if ( j.lt.np ) then

              dx = bcord(3 * j - 2) - bcord(3 * j + 1)
              dy = bcord(3 * j - 1) - bcord(3 * j + 2)
              dz = bcord(3 * j    ) - bcord(3 * j + 3)

              refdis = sqrt( dx*dx + dy*dy + dz*dz ) / 20d0

           end if

      else

           i = i + 1
           disold = disnew

      end if

      if ( j.lt.np ) goto 100

      call erclos ( 'msho62' )

      end
