      subroutine mshob3dmarkpt ( iobst, nx, ny, nz, &
                                 iobgrid, ipointobstlc, &
                                 inpelm, leniobgrid )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-11-2012 Remove input parameter iwork
!        version  2.3  date 11-01-2004 New definition ipointobst
!        version  2.2  date 27-09-2002 Extra call to mshobstmarkpt1
!
!   copyright (c) 2001-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all nodal points in the mesh with information if they are
!     within an obstacle or outside an obstacle
!     Store this information in array ipointobst
!     3d version
!     Remark:  if this subroutine is changed also change MSHOB3DPOINT
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobst, nx, ny, nz, &
              iobgrid(0:nx-1,0:ny-1,0:nz-1,*), leniobgrid, &
              ipointobstlc(*), inpelm

!     inpelm         i    number of points per element in obstacle
!     iobgrid        i    In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin,
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     iobst          i    Obstacle sequence number
!     ipointobstlc   o    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!     leniobgrid     i    Declared length of last dimension of iobgrid
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, numpntobst, numbndobst
      integer, allocatable :: iwork(:)
      double precision xmin, xmax, ymin, ymax, zmin, zmax, x, y, z, &
                       dx, dy, dz, eps, &
                       xminorig, yminorig, zminorig
      logical onobstacle, inside

!     dx             step in x-direction corresponding to xminorig
!     dy             step in y-direction corresponding to yminorig
!     dz             step in z-direction corresponding to zminorig
!     eps            Accuracy
!                    Points that are at a distance less than eps of the
!                    obstacle are defined as part of the obstacle
!     inode          Node sequence number
!     inside         if true the node is inside the obstacle
!     iwork          Work array of length npoint to store the boundary
!                    nodes temporarily
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numpntobst     Number of nodal points inside the obstacle
!     onobstacle     If true the point is positioned on the obstacle
!     x              x co-ordinate of point
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     xminorig       Minimum x-value to compute ipcell, jpcell
!     y              y co-ordinate of point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     yminorig       Minimum y-value to compute ipcell, jpcell
!     z              z co-ordinate of point
!     zmax           Maximum z-value
!     zmin           Minimum z-value
!     zminorig       Minimum z-value to compute ipcell, jpcell, kpcell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSHOB3DMARKPT1 case of open obstacle lying in x, y or z direction
!     MSHOB3DPNT     Check if point with coordinates x,y,z is inside or
!                    outside the obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2254   inpelm # 3
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Extract xmin,xmax,ymin,ymax, zmin, zmax from obstacle array
!    numpntobst := 0
!    numbndobst := 0
!    For inode := 1 (1) npoint do
!       (x,y,z) := coordinates of node
!       If ( (x,y,z) inside (xmin,xmax;ymin,ymax;zmin,zmax) )
!          icell := (i,j,k) index of obstacle cell in which (x,y,z) is lying
!          compute inside and onobstacle
!          Store node if inside
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshob3dmarkpt' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshob3dmarkpt'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( inpelm/=3 ) then

!     --- Only triangles have been implemented

         call errsub ( 2254, 0, 0, 0 )
         go to 1000

      end if

!     --- initialize parameters

      numpntobst = 0
      numbndobst = 0

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Extract xmin,xmax,ymin,ymax, zmin, zmax from obstacle array

      xminorig = rmeshx(1,iobst)
      xmax     = rmeshx(2,iobst)
      yminorig = rmeshx(3,iobst)
      ymax     = rmeshx(4,iobst)
      zminorig = rmeshx(5,iobst)
      zmax     = rmeshx(6,iobst)
      dx   = rmeshx(7,iobst)
      dy   = rmeshx(8,iobst)
      dz   = rmeshx(9,iobst)

      eps = min(dx,dy,dz)*0.01d0
      xmin = xminorig-eps
      xmax = xmax    +eps
      ymin = yminorig-eps
      ymax = ymax    +eps
      zmin = zminorig-eps
      zmax = zmax    +eps

      if ( debug ) then

!     --- debug is true, print info

         write(irefwr,*) 'Data with respect to obstacle'
         write(irefwr,*) 'xmin, xmax, ymin, ymax, zmin, zmax ', &
                          xmin, xmax, ymin, ymax, zmin, zmax
         write(irefwr,*) 'accuracy ', eps
         write(irefwr,*) 'nx, ny, nz, dx, dy, dz ', &
                          nx, ny, nz, dx, dy, dz

      end if

!     --- For inode := 1 (1) npoint do

      do inode = 1, npoint

!     --- (x,y) := coordinates of node

         x = coor(1,inode)
         y = coor(2,inode)
         z = coor(3,inode)
         if ( x>=xmin .and. x<=xmax .and. &
              y>=ymin .and. y<=ymax .and. &
              z>=zmin .and. z<=zmax ) then

!        --- (x,y,z) inside (xmin,xmax;ymin,ymax;zmin,zmax)
!            icell := (i,j,k) index of obstacle cell in which (x,y,z) is lying

            call mshob3dpnt ( nx, ny, nz, iobgrid, &
                              kmeshcobst, inpelm, coorobst, leniobgrid, &
                              x, y, z, inside, onobstacle, xminorig, &
                              yminorig, zminorig, dx, dy, dz, eps )
            if ( onobstacle ) then

!           --- Point on obstacle

               numbndobst = numbndobst+1
               iwork(numbndobst) = inode

            else if ( inside ) then

!           --- Point in obstacle

               numpntobst = numpntobst+1
               ipointobstlc(numpntobst+4) = inode

            end if

         end if

      end do

      if ( debug ) &
         write(irefwr,1) 'iobst, numpntobst, numbndobst', &
                          iobst, numpntobst, numbndobst

!     --- Fill four positions of ipointobstlc
!         At this moment position 2 is set to zero, it is filled in subroutine
!         mshobstmarkel

      ipointobstlc(1) = numpntobst
      ipointobstlc(2) = 0
      ipointobstlc(3) = numbndobst
      ipointobstlc(4) = 0

      ipointobstlc(numpntobst+5:numpntobst+4+numbndobst) = &
         iwork(1:numbndobst)

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

      if ( numpntobst==0 .and. numbndobst==0 .and. iobst==1 ) then

        call mshob3dmarkpt1 ( iobst, ipointobstlc, inpelm )

      end if


1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshob3dmarkpt'

      end if  ! ( debug )
      call erclos ( 'mshob3dmarkpt' )

      end subroutine mshob3dmarkpt
