      subroutine mshobsfl3xc ( nx, ny, nz, iobgrid, ielem, y, z, &
                               i1, i2, i3, inpelm, ymin, zmin, dy, dz, &
                               leniobgrid, leniobgridact )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 14-11-2001 Extra parameters leniobgrid,leniobgridact
!        version  1.0  date 15-06-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iobgrid for all edges of an obstacle element in case
!     the variation in i-direction of all elements is zero
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz, iobgrid(0:nx-1,0:ny-1,0:nz-1,*), ielem, &
              i1(*), i2(*), i3(*), inpelm, leniobgrid, leniobgridact
      double precision y(*), z(*), ymin, zmin, dy, dz

!     dy             i    length of vector in y-direction
!     dz             i    length of vector in z-direction
!     i1             i    Array containing the "i" values of the vertices
!     i2             i    Array containing the "j" values of the vertices
!     i3             i    Array containing the "k" values of the vertices
!     ielem          i    Element number
!     inpelm         i    number of points per element
!     iobgrid       i/o   In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     leniobgrid     i    Declared length of last dimension of iobgrid
!     leniobgridact i/o   Actual length of last dimension of iobgrid
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     y              i    Contains the y-coordinates of the vertices
!     ymin           i    Minimum y-value
!     z              i    Contains the z-coordinates of the vertices
!     zmin           i    Minimum z-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision beta, betaf, betal, gamma, gammaf, gammal, &
                       ratio
      integer i, j, k, l, kprev, lf, ll

!     beta           Factor in y-direction
!     betaf          first multiplication factor
!     betal          last multiplication factor
!     gamma          Factor in z-direction
!     gammaf         first multiplication factor
!     gammal         last multiplication factor
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     kprev          Previous value of k
!     l              General loop variable
!     lf             index of first node on edge
!     ll             index of first node on edge
!     ratio          z-length/y-length in units dz/dy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSFL1      Fills array iobsgrid for one cell
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all edges do
!        Find information of one edge
!        If ( variation in y-direction is 0 ) then
!           Fill iobgrid for all z cells
!        else If ( variation in z-direction is 0 ) then
!           Fill iobgrid for all y cells
!        else ( variation in y nor z-direction is 0 ) then
!           Fill iobgrid for all cells in z-direction
!           Fill iobgrid for all cells in y-direction
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsfl3xc' )

!     --- First consider all edges

      do l = 1, inpelm

!     --- Mark all cells at the intersection line as boundary cells

         if ( i2(l).eq.i2(l+1) ) then

!        --- i2(l) = i2(l+1)

            if ( abs(i3(l+1)-i3(l)).gt.1 ) then

!           --- i3(l) # i3(l+1)

               do k = min(i3(l),i3(l+1)), max(i3(l),i3(l+1))
                  call mshobsfl1 ( nx, ny, nz, iobgrid, i1(1), i2(l), k, &
                                   ielem, leniobgrid, leniobgridact )
               end do

            end if

         else

!        --- i2(l) # i2(l+1)

            if ( i3(l).eq.i3(l+1) .and. abs(i2(l+1)-i2(l)).gt.1 ) then

!           --- i3(l) = i3(l+1) and i2(l) # i2(l+1)

               do j = min(i2(l),i2(l+1)), max(i2(l),i2(l+1))
                  call mshobsfl1 ( nx, ny, nz, iobgrid, i1(1), j, i3(l), &
                                   ielem, leniobgrid, leniobgridact )
               end do

            else if ( i3(l).ne.i3(l+1) ) then

!           --- i3(l) # i3(l+1) and i2(l) # i2(l+1)
!               Compute all cross sections of the edge with the grid
!               in the j-direction
!               Start with the end node of edge for which i1 is minimal
!               end stop at the other end node

               if ( i2(l+1).gt.i2(l) ) then
                  lf = l
                  ll = l+1
               else
                  lf = l+1
                  ll = l
               end if

               kprev = i3(lf)
               betaf = (y(lf)-ymin)/dy
               betal = (y(ll)-ymin)/dy
               gammaf = (z(lf)-zmin)/dz
               gammal = (z(ll)-zmin)/dz
               ratio = (gammal-gammaf)/(betal-betaf)

               do j = i2(lf),i2(ll)
                  if( j.eq.i2(ll)) then

!                 --- the last point has to be treated differently

                     gamma = gammal

                  else

                     beta = j+1
                     gamma = gammaf +(beta-betaf) * ratio

                  end if

                  k = min(int(max(gamma,0d0)),nz-1)
                  do i = min(k,kprev), max(k,kprev)
                     call mshobsfl1 ( nx, ny, nz, iobgrid, i1(1), j, &
                                      i, ielem, leniobgrid, &
                                      leniobgridact )
                  end do
                  kprev = k

               end do

            end if

         end if

      end do
      call erclos ( 'mshobsfl3xc' )
      end
