      subroutine mshobsflinside ( iobgrid, nx, ny, iobgrid1, numcells, &
                                  xmin, ymin, ymax, iwork, n, &
                                  leniwork, dy, closed, dx, curvescp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill cells inside and outside the obstacle
!     Closed obstacle
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, iobgrid(0:nx-1,0:ny-1), iobgrid1(*), &
              leniwork, iwork(2,leniwork), n, numcells(2,n-1)
      double precision ymin, dy, dx, xmin, ymax, &
                       curvescp(2,*)
      logical closed

!     closed         i    If true the obstacle is closed
!     curvescp       i    Contains the coordinates of the obstacle curve
!     dx             i    step size of grid in x-direction
!     dy             i    step size of grid in y-direction
!     iobgrid       i/o   At input filled with the number of curve parts that
!                         intersect cell i,j
!                         At output position i,j contains:
!                         -1:  cell inside obstacle
!                          0:  cell outside obstacle
!                         >0:  cell contains part of the obstacle
!                         The value contains a starting address in iobgrid1
!     iobgrid1       o    Must be used in connection with iobgrid
!                         iobgrid contains the starting addresses for each cell
!                         i,j
!                         From this starting address iobgrid1 contains
!                         the number of curve parts of the obstacle
!                         intersecting the cell followed by the sequence
!                         numbers of these curve parts
!     iwork          i    Contains the cell numbers (i,j) in the sequence
!                         of the curve parts
!                         Must be used in connection with numcells
!     leniwork       i    Declared length of array iwork
!     n              i    number of nodes in curvescp
!     numcells       i    numcells(1,k) contains the number of cells curvepart
!                         k intersects
!                         numcells(1,k) contains the starting address of
!                         the cell numbers in iwork
!     nx             i    number of points of grid in x direction
!     ny             i    number of points of grid in y direction
!     xmin           i    Minimum x-value in grid
!     ymax           i    Maximum y-value in grid
!     ymin           i    Minimum y-value in grid
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxparts
      parameter (maxparts=100)
      integer ipiwork, i, nparts, icurbef, j, icurpart, ipart, &
              istorecurves(maxparts), iseq(maxparts), icount, &
              iobgridnw(0:nx-1,0:ny-1)
      double precision x1, x2, y1, y2, xlow, xhigh, xmid, ymid, ylow, &
                       xmin12, xmax12, ymin12, ymax12, beta1, beta2, &
                       storecurves(maxparts), yhigh, ymidbef, eps
      logical outside

!     beta1          parameter beta1 to define a line
!     beta2          parameter beta2 to define a line
!     eps            Local accuracy
!     i              Counting variable
!     icount         Counter
!     icurbef        Previous curve part used
!     icurpart       Sequence number of curve part
!     iobgridnw      Temporary array to store the output array iobgrid
!     ipart          Sequence number of part
!     ipiwork        Starting address in array iwork
!     iseq           Array containing the sorted sequence of storecurves
!     istorecurves   Contains the curve parts that cross one column in the grid
!     j              Counting variable
!     maxparts       Declared length of arrays istorecurves and iseq
!     nparts         number of parts that are stored in storecurves
!     outside        If true the cell is outside the obstacle
!     storecurves    Contains the y-coordinates of the curve parts that
!                    cross one column in the grid (cross section point)
!     x1             x-coordinate of first point in curve part
!     x2             x-coordinate of last point in curve part
!     xhigh          x-coordinate of right-hand side of cell
!     xlow           x-coordinate of left-hand side of cell
!     xmax12         max (x1,x2)
!     xmid           x-coordinate of middle of cell (cross section)
!     xmin12         min (x1,x2)
!     y1             y-coordinate of first point in curve part
!     y2             y-coordinate of last point in curve part
!     yhigh          y-coordinate of right-hand side of cell
!     ylow           y-coordinate of left-hand side of cell
!     ymax12         max (y1,y2)
!     ymid           y-coordinate of middle of cell (cross section)
!     ymidbef        Previous cross section
!     ymin12         min (y1,y2)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHOBSFLCURV   Store part of boundary in column into array istorecurves
!     MSHOBSFLEDGE2  Compute intersection of part of obstacle with x = ....
!                    Part parallel to y-axis
!     MSHOBSFLINSX   Compute intersection of part of obstacle with x = ....
!                    Part not parallel to y-axis
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     PRSORT         Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2295   Number of cross sections is odd
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsflinside' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsflinside'
         call prinin1 ( iobgrid, ny, nx, 'iobgrid' )

  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      eps = 0.001d0*dx

!     --- For each column in the grid do

      iobgridnw = 0
      ipiwork = 0
      numcells(1,:) = 0

      do i = 0, nx-1

!     --- Store all parts that are possibly in this column into array
!         istorecurves(1:nparts)

         nparts = 0
         icurbef = 0
         do j = 0, ny-1

            if ( iobgrid(i,j)>0 ) then

!           --- boundary part found

               call mshobsflcurv ( iobgrid, nx, ny, i, j, iobgrid1, &
                                   icurbef, nparts, istorecurves, &
                                   maxparts )
               if ( ierror/=0 ) go to 1000

            end if  ! ( iobgrid(i,j)>0 )

         end do  ! j = 0, ny-1

         if ( debug ) then
            write(irefwr,1) 'icolumn', i
            call prinin ( istorecurves, nparts, 'istorecurves' )
         end if  ! ( debug )

         xlow = xmin+i*dx
         xhigh = xlow+dx
         xmid = 0.5d0*(xhigh+xlow)
         if ( debug ) &
            write(irefwr,2) 'xlow, xhigh, xmid', xlow, xhigh, xmid

!        --- For each curve part do

         icount = 0
         do ipart = 1, nparts

!        --- Find (x1,y1), (x2,y2)

            icurpart = istorecurves(ipart)
            x1 = curvescp(1,icurpart)
            y1 = curvescp(2,icurpart)
            x2 = curvescp(1,icurpart+1)
            y2 = curvescp(2,icurpart+1)
            xmin12 = min(x1,x2)
            xmax12 = max(x1,x2)
            ymin12 = min(y1,y2)
            ymax12 = max(y1,y2)

            if ( debug ) then
               write(irefwr,1) 'ipart, icurpart', ipart, icurpart
               write(irefwr,2) 'x1, y1, x2, y2', x1, y1, x2, y2
            end if  ! ( debug )

            if ( xhigh>=xmin12-eps .and. xlow<=xmax12+eps ) then

!           --- Part is in column
!               Compute intersections with lines x=xmin, x=xmid and x=xmax
!               Fill iobgrid for elements that are crossed by boundary
!               Store ymid in storecurves(ipart)

               if ( xmax12-xmin12>1d-3*dx ) then

!              --- The line is not parallel to the y-axis
!                  Compute intersection with x = ....

                  call mshobsflinsx ( xlow, xmid, xhigh, x1, x2, y1, &
                                      y2, eps, xmin12, xmax12, ymin12, &
                                      ymax12, icount, storecurves, &
                                      ymin, iobgridnw, nx, ny, i, &
                                      numcells, icurpart, iwork, &
                                      leniwork, dy )

               else

!              --- The line is parallel to the y-axis
!                  This means that the complete part is in the column

                  beta1  = (y1-ymin)/dy
                  beta2  = (y2-ymin)/dy
                  call mshobsfledge2 ( iobgridnw, nx, ny, i, numcells, &
                                       icurpart, iwork, beta1, beta2 )

               end if  ! ( xmax12-xmin12>1d-3*dx )

            end if  ! ( xmin12>xlow-eps .and. xmax12<xhigh+eps ...

         end do

         if ( debug ) then

            call prinin1 ( numcells, n-1, 2, 'numcells' )
            call prinin1 ( iwork(1,1), ipiwork, 2, 'iwork' )
            call prinin1 ( iobgridnw, ny, nx, 'iobgridnw' )
            write(irefwr,1) 'icount', icount
            call prinrl ( storecurves, icount, 'storecurves' )

         end if  ! ( debug )

!        --- Sort storecurves in increasing sequence

         nparts = icount
         call prsort ( storecurves, iseq, nparts )

         if ( debug ) call prinin ( iseq, nparts, 'iseq' )

!        --- For each row in column do
!               Colour row as internal or external depending on number of
!               cross sections with line xmid

         outside = .true.
         ymid = storecurves(iseq(1))
         ymidbef = ymid
         icount = 1
         do j = 0, ny-1
            ylow = ymin+j*dy
            yhigh = ylow+dy
            if ( iobgridnw(i,j)/=0 ) then

!           --- iobgridnw(i,j)>0, boundary cell found

210            if ( ymid<=yhigh+eps ) then

!              --- Cross-section with boundary found

                  outside = .not. outside
220               if ( icount<nparts ) then

!                 --- icount<nparts, so not the last cross section found

                     icount = icount+1
                     ymid = storecurves(iseq(icount))

!                    --- if the new cross section is equal to the old one, go
!                        to the next one

                     if ( ymid<=ymidbef+eps ) go to 220
                     ymidbef = ymid
                     go to 210

                  else

!                 --- icount = nparts, so the last cross section found

                     ymid = ymax+1d2
                     if ( .not. outside .and. closed ) then

!                    --- error something wrong with the cross sections

                        call errint ( i, 1 )
                        call errsub ( 2295, 1, 0, 0 )
                        go to 1000

                     end if
                     go to 300

                  end if

               end if

            else if ( .not. outside ) then

!           --- rectangle is inside, mark by -1

               iobgridnw(i,j) = -1

            end if

         end do

300   end do

      iobgrid = iobgridnw

1000  call erclos ( 'mshobsflinside' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobsflinside'

      end if  ! ( debug )

      end
