      subroutine mshobsfliobgrd ( iobgrid, nx, ny, iwork, numcells, n, &
                                  iobgrid1, leniobgrid1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill iobgrid from information stored in iobgrid, iwork and numcells
!     Also fill array iobgrid1
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, iwork(2,*), nx, ny, leniobgrid1, &
              iobgrid(0:nx-1,0:ny-1), numcells(2,n-1), &
              iobgrid1(leniobgrid1)

!     iobgrid       i/o   position i,j contains:
!                         -1:  cell inside obstacle
!                          0:  cell outside obstacle
!                         >0:  cell contains part of the obstacle
!                         The value contains a starting address in iobgrid1
!     iobgrid1       o    Must be used in connection with iobgrid
!                         iobgrid contains the starting addresses for each cell
!                         i,j
!                         From this starting address iobgrid1 contains
!                         the number of curve parts of the obstacle
!                         intersecting the cell followed by the sequence
!                         numbers of these curve parts
!     iwork          i    Contains the cell numbers (i,j) in the sequence
!                         of the curve parts
!                         Must be used in connection with numcells
!     leniobgrid1   i/o   At input declared length of iobgrid1
!                         At output computed length
!     n              i    number of nodes in onstacle curve
!     numcells       i    numcells(1,k) contains the number of cells curvepart
!                         k intersects
!                         numcells(1,k) contains the starting address of
!                         the cell numbers in iwork
!     nx             i    number of points in x direction of grid
!     ny             i    number of points in y direction of grid
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, i, j, numedge, iedge, numcell, istartcell, &
              i1, j1
      integer, allocatable :: iobgridcop(:,:)

!     i              Counting variable
!     i1             Index of point with respect to rectangular grid
!     iedge          curve part sequence number
!     iobgridcop     Temporary copy of iobgrid
!                    Is used to create new iobgrid
!     istart         Starting address in iobgrid1
!     istartcell     starting address in iwork
!     j              Counting variable
!     j1             Index of point with respect to rectangular grid
!     numcell        number of cells edge belongs to
!     numedge        number of edges in cell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINIOBGRID    Print the contents of array iobgrid
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsfliobgrd' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsfliobgrd'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Make a temporary copy of iobgrid

      allocate ( iobgridcop(0:nx-1,0:ny-1), stat = error )
      if ( error/=0 ) call eralloc ( error, nx*ny, 'iobgridcop' )
      if ( ierror/=0 ) go to 1000

      iobgridcop = iobgrid

      iobgrid1 = 0

!     --- First store starting addresses in iobgrid

      istart = 1

      do j = 0, ny-1

         do i = 0, nx-1

            if ( iobgrid(i,j)>0 ) then

!           --- There are edges in cell i,j

               numedge = iobgrid(i,j)      ! number of edges in cell
               iobgrid(i,j) = istart       ! starting address
               iobgrid1(istart) = numedge  ! store number of edges
               istart = istart+numedge+1   ! raise istart

            end if  ! ( iobgrid(i,j)>0 )

         end do  ! i = 0, nc-1

      end do  ! j = 0, ny-1

      call inichkarlen ( istart-1, leniobgrid1, 'iobgrid1' )
      if ( ierror/=0 ) go to 1000

      leniobgrid1 = istart-1

!     --- Copy starting addresses

      iobgridcop = iobgrid

!     --- Loop over all edges

      do iedge = 1, n-1

         numcell = numcells(1,iedge)     ! number of cells edge belongs to
         istartcell = numcells(2,iedge)  ! starting address in iwork

         do i = 1, numcell

            i1 = iwork(1,istartcell)  ! i-index cell
            j1 = iwork(2,istartcell)  ! j-index cell
            istartcell = istartcell+1
            istart = iobgridcop(i1,j1) ! starting address in iobgrid1
            if ( debug ) &
               write(irefwr,1) 'iedge, i, i1, j1, istart', &
                                iedge, i, i1, j1, istart
            istart = istart+1         ! raise istart
            iobgrid1(istart) = iedge  ! store edge
            iobgridcop(i1,j1) = istart ! store new starting address

         end do  ! i = 1, numcell

      end do  ! iedge = 1, n-1

      deallocate ( iobgridcop, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iobgridcop' )

1000  if ( debug ) then

!     --- Debug information

         call priniobgrid ( iobgrid(0,0), nx, ny, iobgrid1 )
         write(irefwr,*) 'End mshobsfliobgrd'

      end if  ! ( debug )
      call erclos ( 'mshobsfliobgrd' )

      end
