      subroutine mshobsin ( iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 02-09-2010 Remove intarmsh
!        version  3.0  date 04-12-2002 Extra parameter ipointcurv
!        version  2.1  date 27-09-2002 Adaptation for open obstacles
!
!   copyright (c) 1997-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information in kmesh part x concerning obstacles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)

!     iinput         i    integer input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nx1, ny1, iobst, n, istart, &
              numsurfs, jstart, iinp
      double precision xmin, xmax, ymin, ymax, dx, dy, x, y, &
                       unit, deltax
      logical closed
      integer, allocatable :: iwork(:)

!     closed         If true the boundary is closed
!     deltax         max unit length of element
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     i              Counting variable
!     iinp           last position used in array iinput
!     iobst          Obstacle sequence number
!     istart         First position used at a part
!     iwork          integer work array of length ncurvs
!                    iwork(i) = 0, means curve i does not belong to
!                                  the present obstacle
!                    iwork(i) = 1, means curve i does belong to
!                                  the present obstacle
!     j              Counting variable
!     jstart         First position used at a part
!     n              number of basis functions for each component
!     numsurfs       number of surfaces
!     nx1            number of points in x direction
!     ny1            number of points in y direction
!     unit           unit length
!     x              x co-ordinate of point
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     y              y-coordinate read from the SEPRAN neutral plot file
!     ymax           Maximum y-value
!     ymin           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1049   Obstacle is not closed
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Clear arrays kmeshx and rmeshx
!   jstart := 1
!   For all obstacles do
!      Compute xmin, xmax, ymin, ymax
!      Check if obstacle is closed or open
!      unit := minimum length in elsize
!      dx := xmax-xmin; dy := ymax-ymin
!      xmax := xmax+0.005*dx  xmin := xmin-0.005*dx
!      ymax := ymax+0.005*dy  ymin := ymin-0.005*dy
!      nx1 := max(5,4*(xmax-xmin)/unit+1)
!      ny1 := max(5,4*(ymax-ymin)/unit+1)
!      dx := (xmax-xmin)/nx1;  dy := (ymax-ymin)/ny1
!      Fill information in arrays kmeshx
!      Fill iwork with curve numbers corresponding to obstacle
!      numsurfs := 0
!      For all surfaces isurnr do
!         For all curves surrounding surface isurnr, as long as no obstacle
!            curve on surface has been found do
!            if ( curve on obstacle ) then
!               raise numsurfs
!               put surface number in kmeshxsur
!      if ( numsurfs>0 ) then
!         kmeshxsur(jstart) := numsurfs
!         raise jstart
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsin' )

      allocate ( iwork(ncurvs), km(iseqkm)%kmeshx(6,numobst), &
                 km(iseqkm)%rmeshx(6,numobst), &
                 kmeshxsur(numobst*(nsurfs+1)), &
                 km(iseqkm)%kmobst(numobst), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, ncurvs+numobst*12+numobst*(nsurfs+1), 'iwork' )
      if ( ierror/=0 ) go to 1000

      kmeshx => km(iseqkm)%kmeshx
      rmeshx => km(iseqkm)%rmeshx

!     --- unit := minimum length in elsize

      unit = rinfin
      do i = 1, ncurvs
         x = elsize(1,i)
         if ( x>0d0 ) unit = min(unit,x)
         if ( x>0d0 ) deltax = max(unit,x)
         x = elsize(2,i)
         if ( x>0d0 ) unit = min(unit,x)
         if ( x>0d0 ) deltax = max(unit,x)
      end do

!     --- Clear arrays kmeshx and rmeshx

      kmeshx = 0
      rmeshx = 0d0

!     --- For all obstacles

      jstart = 1
      do iobst = 1, numobst

!     --- Compute xmin, xmax, ymin, ymax

         icurnr = iinput(iinp14+2*iobst)
         iinp = ipointcurv(icurnr)
         ishape = iinput(iinp+1)
         if ( icurnr>0 ) then

!        --- icurnr>0, hence existing obstacle

            if ( icurnr==1 ) then

!           --- icurnr = 1, first curve

               n = icurvsmsh(icurnr)
               istart = 0

            else

!           --- icurnr>1

               n = icurvsmsh(icurnr)-icurvsmsh(icurnr-1)
               istart = icurvsmsh(icurnr-1)

            end if
            xmin = rinfin
            xmax =-rinfin
            ymin = rinfin
            ymax =-rinfin
            do i = 1, n
               x    = curvesmsh(1,istart+i)
               y    = curvesmsh(2,istart+i)
               xmin = min(xmin,x)
               xmax = max(xmax,x)
               ymin = min(ymin,y)
               ymax = max(ymax,y)
            end do

!           --- dx := xmax-xmin; dy := ymax-ymin

            if ( abs(xmax-xmin)<1d-10)  then

!           --- xmax = xmin

               deltax=deltax*0.5d0
               xmin=xmin-deltax
               xmax=xmax+deltax

            end if

            if ( abs(ymax-ymin)<1d-10 ) then

!           --- ymax = ymin

               deltax=deltax*0.5d0
               ymin=ymin-deltax
               ymax=ymax+deltax

            end if

            dx = xmax-xmin
            dy = ymax-ymin

!           --- Check if obstacle is closed or open

            closed = .true.
            if ( abs(curvesmsh(1,istart+1)-curvesmsh(1,istart+n))> &
                 sqreps*dx .or. &
                 abs(curvesmsh(2,istart+1)-curvesmsh(2,istart+n))> &
                 sqreps*dy ) then

!           --- curve is not closed

               call errint ( iobst, 1 )
               call errwar ( 1049, 1, 0, 0 )
               closed = .false.

            end if

!           --- xmax := xmax+0.005*dx  xmin := xmin-0.005*dx
!               ymax := ymax+0.005*dy  ymin := ymin-0.005*dy
!               nx1 := max(5,4*(xmax-xmin)/unit+1)
!               ny1 := max(5,4*(ymax-ymin)/unit+1)
!               dx := (xmax-xmin)/nx1;  dy := (ymax-ymin)/ny1

            xmax = xmax+0.005d0*dx
            xmin = xmin-0.005d0*dx
            ymax = ymax+0.005d0*dy
            ymin = ymin-0.005d0*dy
            dx   = xmax-xmin
            dy   = ymax-ymin
            nx1   = min(max(5,int(1.1d0*dx/unit+1d0)),100)
            ny1   = min(max(5,int(1.1d0*dy/unit+1d0)),100)
            dx   = dx/nx1
            dy   = dy/ny1

!           --- Fill information in arrays kmeshx

            rmeshx(1,iobst) = xmin
            rmeshx(2,iobst) = xmax
            rmeshx(3,iobst) = ymin
            rmeshx(4,iobst) = ymax
            rmeshx(5,iobst) = dx
            rmeshx(6,iobst) = dy
            if ( closed ) then
               kmeshx(1,iobst) = 1
            else
               kmeshx(1,iobst) = -1
            end if
            if ( ishape==2 ) kmeshx(1,iobst) = 3*kmeshx(1,iobst)
            kmeshx(2,iobst) = icurnr
            kmeshx(3,iobst) = nx1
            kmeshx(4,iobst) = ny1

!           --- Clear array iwork

            do i = 1, ncurvs
               iwork(i) = 0
            end do

!           --- Fill array iwork
!               iwork(i) = 0, curve not on obstacle
!               iwork(i) = 1, curve on obstacle

            if ( kelmq/=0 ) then

!           --- kelmq/=0, hence there are subcurves

               if ( kmeshq(icurnr+1)-kmeshq(icurnr)>0 ) then

!              --- there are subcurves, fill iwork

                  do i = kmeshq(icurnr)+1, kmeshq(icurnr+1)
                     iwork(abs(kmeshq(i))) = 1
                  end do

               else

!              --- no subcurves, set iwork

                  iwork(icurnr) = 1

               end if

            else

!           --- kelmq=0, hence there are no subcurves, set iwork

               iwork(icurnr) = 1

            end if

!           --- Find all surfaces corresponding to obstacle and store in
!               kmeshxsur

            numsurfs = 0
            do isurnr = 1, nsurfs

               do i = kmeshs(isurnr)+1, kmeshs(isurnr+1)

                  icurnr = abs(kmeshs(i))
                  if ( iwork(icurnr)==1 ) then

!                 --- surface contains curve at obstacle

                     numsurfs = numsurfs+1
                     kmeshxsur(jstart+numsurfs) = isurnr
                     go to 100

                  end if

               end do

100         end do
            if ( numsurfs>0 ) then

!           --- numsurfs>0, surfaces have been found
!               Fill numsurfs in kmeshxsur and raise jstart

               kmeshx(6,iobst) = jstart
               kmeshxsur(jstart) = numsurfs
               jstart = jstart+numsurfs+1

            end if

         end if

      end do
      allocate ( km(iseqkm)%kmeshxsur(jstart-1) )
      km(iseqkm)%kmeshxsur = kmeshxsur(1:jstart-1)

      deallocate ( iwork, kmeshxsur, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

      kmeshxsur => km(iseqkm)%kmeshxsur

1000  call erclos ( 'mshobsin' )
      end
