      subroutine mshobst ( rinput, nx, ny, iobst, lengcv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 02-10-2012 Extra parameter iobgrid1
!        version  4.1  date 12-02-2000 Change error message
!        version  4.0  date 03-08-1999 Extra parameter elsize
!
!   copyright (c) 1996-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Check if free boundary intersects obstacles and if so, adapt the free
!   boundary to the obstacle
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, iobst, lengcv
      double precision rinput(2,*)

!     iobst          i    Obstacle sequence number
!     lengcv         i    length of curves
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     rinput         i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxcell
      parameter (maxcell=300)
      integer i, i1, icurnr, jstep, iuserp, ipart, ncurpart, &
              jcount, k, ielgrp, &
              icurnrobs, istartobs, istart, iend, j, j1, n, &
              index1, ind, icell(maxcell), istartiobgr, &
              idirec, ncell, nstep, istoreindex(30), ibnchk(lengcv)
      double precision dx, dy, t1, eps, u1, u2, x, xmax, xmin, beta, &
                       xs, y, ymax, ys, ymin, alpha2, beta2, &
                       work1(2,maxcell), h, xnew, ynew, dist, epsmesh
      logical quadrat, found

!     alpha2         Parameter to define a line
!     beta           A curve through the points (x1,y1) and (x2,y2) is
!                    represented by (x1,y1) + beta (t1,t2) with
!                    t1 = x2-x1;  t2 = y2-y1
!     beta2          Parameter to define a line
!     dist           Distance between two points
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     eps            Local accuracy
!     epsmesh        Accuracy based on the element sizes in the mesh
!     found          If true already a possibility has been found
!     h              Length of part of curve
!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     ibnchk         Work array of length of all curves to be used to
!                    check whether a boundary cell has been marked (1) or
!                    not (0)
!     icell          Array containing the boundary cells that are used to
!                    check whether a point is at an obstacle
!     icurnr         Curve sequence number
!     icurnrobs      Curve sequence number of obstacle
!     idirec         Defines in which direction the curves must be considered
!                    idirec = 1, from low to high, idirec = -1: reversed order
!     ielgrp         element group number
!     iend           Last position used at a part
!     ind            Indicates the index of a cell at the boundary
!     index1         Boundary cell index number of first cell
!     ipart          Sequence number of element in obstacle curve
!     istart         First position used at a part
!     istartiobgr    Starting address in array iobgrid2d
!     istartobs      Starting address in array icurvs of obstacle curve
!     istoreindex    Work array of length 30 to store the positions of the
!                    parts in the obstacle curve where the intersection
!                    point is found
!     iuserp         User point sequence number
!     j              Counting variable
!     j1             Index of point with respect to cell
!     jcount         Counter
!     jstep          Step in loop
!     k              Counting variable
!     maxcell        Maximum cells that may be used at the boundary
!     n              number of points at the boundary
!     ncell          number of cells that may be used at the boundary
!     ncurpart       Number of elements obstacle grid consists of
!                    internal cell we are trying to find the boundary
!     nstep          ?
!     quadrat        If true the internal elements are quadratic, otherwise they
!                    are linear
!     t1             First component to compute the tangential vector
!     u1             First component to compute the tangential vector
!     u2             Second component to compute the tangential vector
!     work1          Contains the coordinates of intersection points at the
!                    boundary of the obstacle
!     x              x co-ordinate of point
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     xnew           New x-coordinate
!     xs             x co-ordinate of point
!     y              y-coordinate
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     ynew           New y-coordinate
!     ys             y-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1052   No boundary of obstacle
!    1381   Too few space available in array
! **********************************************************************
!
!                       PSEUDO CODE
!
!     initialize parameters
!     Compute xmin, xmax, ymin, ymax
!     For all curves do
!        if ( curve at free boundary ) then
!           istart, iend := start and end point of curve in icurvs
!           For all vertex points along free curve do
!              if ( point not yet at obstacle ) then
!                 (x,y) := new coordinates of point
!                 if ( (x,y) in box around obstacle ) then
!                    compute (i,j) index of point
!                    if ( iobgrid2d(i,j)#0 ) then
!                       Store three subsequent active parts of obstacle
!                       Compute intersection with line x,y|old-x,y|new
!                       if ( intersection point between new and old point ) then
!                          move curvesmsh to intersection point
!                          Mark point as obstacle point
!          If ( quadratic elements are used ) then
!             Put mid point between two other points
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initialize parameters

      icurnrobs = kmeshx(2,iobst)
      nx        = kmeshx(3,iobst)
      ny        = kmeshx(4,iobst)
      dx        = rmeshx(5,iobst)
      dy        = rmeshx(6,iobst)
      if ( kmeshx(1,iobst)>0 ) then
         idirec = 1
      else
         idirec =-1
      end if

!     --- Compute xmin, xmax, ymin, ymax

      if ( icurnrobs==1 ) then

!     --- icurnrobs = 1, first curve

         n = icurvsmsh(icurnrobs)
         istartobs = 0

      else

!     --- icurnrobs>1

         n = icurvsmsh(icurnrobs)-icurvsmsh(icurnrobs-1)
         istartobs = icurvsmsh(icurnrobs-1)

      end if
      xmin = rinfin
      ymin = rinfin
      do i = 1, n
         ibnchk(i) = 0
      end do
      do i = 1, n
         x    = curvesmsh(1,istartobs+i)
         y    = curvesmsh(2,istartobs+i)
         xmin = min(xmin,x)
         ymin = min(ymin,y)
      end do
      xmin = xmin-0.005d0*dx/1.01d0
      ymin = ymin-0.005d0*dy/1.01d0
      xmax = xmin+dx*nx
      ymax = ymin+dy*ny
      eps = 1d-4*max((xmax-xmin),(ymax-ymin))

!     --- Compute mesh accuracy defined 1/3 of the minimum element length
!         along the free surface. Store in epsmesh

      epsmesh = rinfin
      do icurnr = 1, ncurvs
         if ( kmeshz(icurnr)==-1 .or. kmeshz(icurnr)==2 ) then
            epsmesh = min(epsmesh, elsize(1,icurnr),elsize(2,icurnr))
         end if
      end do
      epsmesh = min(1d2*eps, 0.333d0*epsmesh)

!     --- quadratic := internal elements are quadratic

      quadrat = .false.
      do ielgrp = 1, nelgrp
         ishape = kmeshf(ielgrp)
         if ( ishape==4 .or. ishape==6 .or. ishape==7 ) quadrat = .true.
      end do
      if ( quadrat ) then
         jstep = 2
      else
         jstep = 1
      end if

!     --- For all curves do

      do icurnr = 1, ncurvs
         if ( kmeshz(icurnr)==-1 .or. kmeshz(icurnr)==2 ) then

!        --- curve is free surface curve
!            istart, iend := start and end point of curve in icurvsmsh

            if ( icurnr==1 ) then
               istart = 1
            else
               istart = icurvsmsh(icurnr-1)+1
            end if
            iend = icurvsmsh(icurnr)

!           --- For all vertex points along free curve do

            do k = istart, iend, jstep

!           --- Check if point of curvesmsh in box around obstacle
!               (x,y) := new coordinates of point

               x = curvesmsh(1,k)
               y = curvesmsh(2,k)
               if ( x>xmin .and. x<xmax .and. &
                    y>ymin .and. y<ymax ) then

!              --- Point in box found
!                  compute (i,j) index of point

                  ncell = 0
                  jcount = 0
                  alpha2 = (x-xmin)/dx
                  beta2  = (y-ymin)/dy
                  i1 = nint(alpha2)
                  j1 = nint(beta2)
                  i1 = min(max(i1,0),nx-1)
                  j1 = min(max(j1,0),ny-1)

                  istartiobgr = iobgrid2d(i1,j1)

                  if ( istartiobgr>0 ) then

!                 --- New point is in cell that contains boundary
!                     Store three subsequent active parts of obstacle

                     ncurpart = iobgrid2d1(istartiobgr)

                     call inichkarlen ( ncurpart, maxcell, 'icell' )
                     if ( ierror/=0 ) go to 1000

                     do ipart = 1, ncurpart

                        index1 = iobgrid2d1(istartiobgr+ipart)
                        icell(ncell+2) = index1
                        if ( index1==1 ) then
                           icell(ncell+1) = n-1
                        else
                           icell(ncell+1) = index1-1
                        end if
                        if ( index1==n-1 ) then
                           icell(ncell+3) = 1
                        else
                           icell(ncell+3) = index1+1
                        end if
                        ncell = ncell+3

                     end do  ! ipart = 1, ncurpart

                  else if ( istartiobgr<0 ) then

!                 --- Point inside region, difficult situation
!                     find closest boundary cells

                     nstep = 1
                     found = .false.
100                  if ( nstep>max(nx-1,ny-1) ) then
                        call eropen ( 'mshobst' )
                        call errsub ( 1052, 0, 0, 0 )
                        call erclos ( 'mshobst' )
                        go to 1000
                     end if
                     do i = max(0,i1-nstep),min(nx-1,i1+nstep)
                        do j = max(0,j1-nstep),min(ny-1,j1+nstep)

                           istartiobgr = iobgrid2d(i,j)
                           if ( istartiobgr>0 ) then

!                          --- New point is in cell that contains boundary
!                              Store three subsequent active parts of
!                              obstacle

                              ncurpart = iobgrid2d1(istartiobgr)

                              call inichkarlen ( ncell+ncurpart*3, &
                                                 maxcell, 'icell' )
                              if ( ierror/=0 ) go to 1000

                              do ipart = 1, ncurpart

                                 index1 = iobgrid2d1(istartiobgr+ipart)

                                 found = .true.
                                 if ( ibnchk(index1)==0 ) then
                                    ibnchk(index1) = 1
                                    ncell = ncell+1
                                    icell(ncell) = index1
                                    if ( index1==1 .and. &
                                         ibnchk(n-1)==0) then
                                       ncell = ncell+1
                                       icell(ncell) = n-1
                                       ibnchk(n-1) = 1
                                    else if ( ibnchk(index1-1)==0 ) then
                                       ncell = ncell+1
                                       icell(ncell) = index1-1
                                       ibnchk(index1-1) = 1
                                    end if
                                    if ( index1==n-1 .and. &
                                         ibnchk(1)==0) then
                                       ncell = ncell+1
                                       icell(ncell) = 1
                                       ibnchk(1) = 1
                                    else if ( ibnchk(index1+1)==0 ) then
                                       ncell = ncell+1
                                       icell(ncell) = index1+1
                                       ibnchk(index1+1) = 1
                                    end if

                                 end if

                              end do  ! ipart = 1, ncurpart

                           end if

                        end do

                     end do

                     if ( .not. found ) then
                        nstep = nstep+1
                        go to 100
                     end if
                     if ( jcount>maxcell) then
                        call eropen ( 'mshobst' )
                        call errint ( jcount,1 )
                        call errint ( maxcell,2 )
                        call errsub ( 1381, 2, 0, 0 )
                        call erclos ( 'mshobst' )
                        go to 1000
                     end if

                  end if

!                 --- Compute all intersection points of the line through
!                     (x,y) perpendicular to the obstacle and the segments
!                     in the obstacle
!                     The boundary curve is defined by:
!                     (xs,ys) + alpha (u1,u2), with u the tangent
!                     The line through (x,y) with the direction normal to u:
!                     (x,y) + beta (u2,-u1)
!                     The value of beta defines the point at the boundary

                  do i = 1, ncell

                     ind = icell(i)
                     ibnchk(ind) = 0
                     xs = curvesmsh(1,istartobs+ind)
                     ys = curvesmsh(2,istartobs+ind)
                     u1 = curvesmsh(1,istartobs+ind+1)-xs
                     u2 = curvesmsh(2,istartobs+ind+1)-ys
                     beta = (u2*(xs-x)-u1*(ys-y))/(u1**2+u2**2)
                     xnew = x+beta*u2
                     ynew = y-beta*u1
                     dist = sqrt((xnew-x)**2+(ynew-y)**2)

                     if ( abs(beta)<eps ) then

!                    --- Point close to obstacle, do not change but mark
!                        as obstacle point

                        ichangeobst = 1
                        icurvsobst2d(1,k) = iobst
                        icurvsobst2d(2,k) = ind
                        go to 200

                     else if ( beta*idirec>0 .or. dist<epsmesh ) then

!                    --- beta * idirec>0, (x,y) is internal point
!                        store boundary point
!                        or the distance is less than 1/3 of the local
!                        element length. Point is also put on obstacle

                        jcount = jcount+1
                        work1(1,jcount) = x+beta*u2
                        work1(2,jcount) = y-beta*u1
                        istoreindex(jcount) = ind

                     end if

                  end do
                  if ( jcount>1 ) then

!                 --- jcount>1, find point with shortest distance

                     h = (x-work1(1,1))**2+(y-work1(2,1))**2
                     i1 = 1
                     do j = 2, jcount
                        t1 = (x-work1(1,j))**2+(y-work1(2,j))**2
                        if ( t1<h ) then
                           h = t1
                           i1 = j
                        end if
                     end do
                     jcount = i1

                  end if
                  if ( jcount>0 ) then

!                 --- intersection point found

                     curvesmsh(1,k) = work1(1,jcount)
                     curvesmsh(2,k) = work1(2,jcount)

!                    --- Mark point as obstacle point

                     if ( icurvsobst2d(1,k)==0 ) then
                        ichangeobst = 1
                        icurvsobst2d(1,k) = iobst
                        icurvsobst2d(2,k) = istoreindex(jcount)
                     end if
                     iuserp = 0
                     if ( k==istart ) then
                        if ( iobscurv(icurnr)==0 ) then
                           iobscurv(icurnr) = 3
                        else if ( iobscurv(icurnr)==4 ) then
                           iobscurv(icurnr) = 2
                        end if
                        iuserp = curveusrp(curveusrp(icurnr)+1)
                     else if ( k==iend ) then
                        if ( iobscurv(icurnr)==0 ) then
                           iobscurv(icurnr) = 4
                        else if ( iobscurv(icurnr)==3 ) then
                           iobscurv(icurnr) = 2
                        end if
                        iuserp = curveusrp(curveusrp(icurnr+1))
                     else
                        iobscurv(icurnr) = 1
                     end if
                     if ( iuserp>0 ) then
                        rinput(1,iuserp) = curvesmsh(1,k)
                        rinput(2,iuserp) = curvesmsh(2,k)
                     end if

                  else if ( icurvsobst2d(1,k)==iobst ) then
                     ichangeobst = 1
                     icurvsobst2d(1,k) = 0
                     icurvsobst2d(2,k) = 0
                  end if

               end if

200         end do  ! k = istart, iend, jstep

            if ( quadrat ) then

!           --- quadratic elements, make mid point between end points

               do k = istart+1, iend-1, jstep
                  if ( icurvsobst2d(1,k-1)==iobst .and. &
                       icurvsobst2d(1,k+1)==iobst ) then
                     icurvsobst2d(1,k) = iobst
                     icurvsobst2d(2,k) = icurvsobst2d(2,k-1)
                     curvesmsh(1,k) = &
                        0.5d0*(curvesmsh(1,k-1)+curvesmsh(1,k+1))
                     curvesmsh(2,k) = &
                        0.5d0*(curvesmsh(2,k-1)+curvesmsh(2,k+1))
                  end if
               end do

            end if

         end if

      end do

1000  end
