      subroutine mshobstac1 ( iwork, icurvs, icurvobs, nspecialpnts, &
                              specialpoints, istnewcurves, newcurves, &
                              newcurvesstrt, nextracurves, icurnr, &
                              iobst, icurvusp, specialusp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-08-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Split curve on free surface that is part of obstacle into several
!    curves and store the starting and end points of the part on the obstacle
!    as special points
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(2,*), icurvs(*), icurvobs(2,*), nspecialpnts, &
              specialpoints(7,*), istnewcurves(*), newcurves(*), &
              newcurvesstrt, nextracurves, icurnr, iobst, icurvusp(*), &
              specialusp(*)

!     icurnr         i    Curve sequence number
!     icurvobs       i    Array indicating for each point at the boundary
!                         whether it is positioned at an obstacle (>0) or not
!                         (0)
!                         icurvobs must have been filled before
!                         If icurvobs(1,k) = j (j>0), j denotes the sequence
!                         number of the obstacle
!     icurvs         i    array containing the number of points in the curves
!                         accumulated
!     iobst          i    Obstacle sequence number
!     istnewcurves  i/o   Array of length ncurvs containing the starting
!                         addresses of all curves in array newcurves
!                         if istnewcurves(i)=0, no information of the curve is
!                         stored, hence the curve is not changed
!                         if istnewcurves(i)>0, the value is the starting
!                         address of curve i
!     iwork               integer work array of length at most 2 times number of
!                         nodes on the curve
!                         iwork(1,i) contains nparts positions each with the
!                         relative starting address of a new part with respect
!                         to the curve
!                         iwork(2,i) contains the sequence number of the
!                         corresponding special node
!     newcurves     i/o   Contains information about the new curves to be
!                         created by splitting old curves
!                         The starting addresses of the information per curve is
!                         stored in array istnewcurves
!                         For each curve that is stored in newcurves 2*nparts+2
!                         positions are used, where nparts is the number of
!                         parts the curve is subdivided into
!                         The first position contains nparts, the following
!                         2*nparts positions the couples relative starting
!                         addresses of the parts in the curve and corresponding
!                         special points
!     newcurvesstrt i/o   Defines the next free position in newcurves
!     nextracurves  i/o   Number of extra curves to be created
!     nspecialpnts  i/o   Number of special points
!     specialpoints i/o   This array of size 7 x nspecialpnts contains
!                         information about the special points on the curve
!                         specialpoint(1,i) contains the position in curves
!                                           of special point i
!                         specialpoint(2,i) contains the curve number in the
!                                           free surface connected to point i
!                         specialpoint(3,i) contains the corresponding special
!                                           point on the the curve
!                                           The space between is obstacle
!                                           If 0 the point is stand alone
!                         specialpoint(4,i) contains the corresponding special
!                                           point on the the second curve
!                                           If 0 there is no second curve
!                         specialpoint(5,i) contains the second curve number in
!                                           the free surface connected to point
!                         specialpoint(6,i) contains the corresponding special
!                                           point on the the curve
!                         specialpoint(7,i) contains the corresponding user
!                                           point number
!                                           if 0 no user point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparts, istart, inpncv, i, inode, iuserp1, iuserp2, &
              ispecial
      logical typenode1, obstacle

!     i              Counting variable
!     inode          Node sequence number
!     inpncv         Number of points in curve
!     istart         First position used at a part
!     nparts         number of parts the boundary consists of
!     obstacle       if true an obstacle has been found
!     typenode1      if true the first node is on an obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     First split the curve into parts on the obstacle and parts not on
!     the obstacle:
!
!     nparts := 1
!     iwork(1,1) := 1;
!     obstacle := node 1 on obstacle
!     type_node_1 := obstacle
!     for inode := 1 (1) number_of nodes along curve do
!        if ( not obstacle and inode on obstacle ) then
!           nparts := nparts+1
!           iwork(1,nparts) := inode
!           obstacle := true
!        else if ( obstacle and inode not on obstacle ) then
!           nparts := nparts+1
!           iwork(1,nparts) := inode
!           obstacle := false
!
!     Next store the information of iwork in the arrays newcurves and
!     specialpoints
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstac1' )

!     --- Store the 2 user points of the curves in iuserp1/2

      istart = icurvusp(icurnr)
      inpncv = icurvusp(icurnr+1)-istart
      iuserp1 = icurvusp(istart+1)
      iuserp2 = icurvusp(istart+inpncv)

!     --- First split the curve into parts on the obstacle and parts not on
!         the obstacle:

      nparts = 1
      iwork(1,1) = 1

!     --- obstacle := node 1 on obstacle
!         type_node_1 := obstacle

      if ( icurnr.eq.1 ) then

!     --- First curve

         istart = 0
         inpncv = icurvs(1)

      else

!     --- Not the first curve

         istart = icurvs(icurnr-1)
         inpncv = icurvs(icurnr)-istart

      end if
      obstacle = icurvobs(1,istart+1).eq.iobst
      typenode1 = obstacle

!     --- for inode := 1 (1) number_of nodes along curve do

      do inode = 1, inpncv

         if ( .not. obstacle .and. icurvobs(1,istart+inode).eq.iobst ) then

!        --- not obstacle and inode on obstacle

            nparts = nparts+1
            iwork(1,nparts) = inode
            iwork(2,nparts) = 0
            obstacle = .true.

         else if ( obstacle .and. icurvobs(1,istart+inode).ne.iobst ) then

!        --- obstacle and inode not on obstacle

            nparts = nparts+1
            iwork(1,nparts) = inode-1
            iwork(2,nparts) = 0
            obstacle = .false.

         end if

      end do

!     --- Next store the information of iwork in the arrays newcurves and
!         specialpoints

      if ( nparts.eq.1 .or. nparts.eq.2 .and. iwork(1,2).eq.inpncv ) then

!     --- Either the curve consists of one part, or the curve consists of
!         two parts but the second starts in the end point
!         In this situation, the curve is not splitted
!         Only a special point may be created

         if ( typenode1 ) then

!        --- First point is obstacle point
!            The complete curve is on the obstacle

            if ( specialusp(iuserp1).eq.0 ) then

!           --- First point not already marked as special point

               nspecialpnts = nspecialpnts+1
               specialpoints(1,nspecialpnts) = istart+1
               iwork(2,1) = nspecialpnts
               specialpoints(2,nspecialpnts) = icurnr
               if ( specialusp(iuserp2).eq.0 ) then
                  specialpoints(3,nspecialpnts) = nspecialpnts+1
                  if ( nparts.eq.2 ) iwork(2,2) = nspecialpnts+1
               else
                  specialpoints(3,nspecialpnts) = specialusp(iuserp2)
                  if ( nparts.eq.2 ) iwork(2,2) = specialusp(iuserp2)
               end if
               specialpoints(4,nspecialpnts) = 0
               specialpoints(5,nspecialpnts) = 0
               specialpoints(6,nspecialpnts) = 0
               specialpoints(7,nspecialpnts) = iuserp1
               specialusp(iuserp1) = nspecialpnts

            else

!           --- First point already marked as special point

               ispecial = specialusp(iuserp1)
               if ( specialpoints(4,ispecial).ne.0 ) then

!              --- Error more than 2 curves in special point

                  call errint ( icurnr, 1 )
                  call errint ( iuserp1, 2 )
                  call errsub ( 2102, 2, 0, 0 )

               end if
               iwork(2,1) = specialusp(iuserp1)
               specialpoints(4,ispecial) = istart+1
               specialpoints(5,ispecial) = icurnr
               if ( specialusp(iuserp2).eq.0 ) then
                  specialpoints(6,nspecialpnts) = nspecialpnts+1
                  if ( nparts.eq.2 ) iwork(2,2) = nspecialpnts+1
               else
                  specialpoints(6,nspecialpnts) = specialusp(iuserp2)
                  if ( nparts.eq.2 ) iwork(2,2) = specialusp(iuserp2)
               end if

            end if

            if ( specialusp(iuserp2).eq.0 ) then

!           --- Last point not already marked as special point

               nspecialpnts = nspecialpnts+1
               specialpoints(1,nspecialpnts) = istart+inpncv
               specialpoints(2,nspecialpnts) = icurnr
               if ( specialusp(iuserp1).eq.0 ) then
                  specialpoints(3,nspecialpnts) = nspecialpnts-1
               else
                  specialpoints(3,nspecialpnts) = specialusp(iuserp1)
               end if
               specialpoints(4,nspecialpnts) = 0
               specialpoints(5,nspecialpnts) = 0
               specialpoints(6,nspecialpnts) = 0
               specialpoints(7,nspecialpnts) = iuserp2
               specialusp(iuserp2) = nspecialpnts

            else

!           --- First point already marked as special point

               ispecial = specialusp(iuserp2)
               if ( specialpoints(4,ispecial).ne.0 ) then

!              --- Error more than 2 curves in special point

                  call errint ( icurnr, 1 )
                  call errint ( iuserp2, 2 )
                  call errsub ( 2102, 2, 0, 0 )

               end if
               specialpoints(4,ispecial) = istart+inpncv
               specialpoints(5,ispecial) = icurnr
               if ( specialusp(iuserp1).eq.0 ) then
                  specialpoints(6,nspecialpnts) = nspecialpnts-1
               else
                  specialpoints(6,nspecialpnts) = specialusp(iuserp1)
               end if

            end if
            istnewcurves(icurnr) = newcurvesstrt
            newcurves(newcurvesstrt) = -1
            newcurvesstrt = newcurvesstrt+1

         else

!        --- Last point is obstacle point

            if ( specialusp(iuserp2).eq.0 ) then

!           --- Last point not already marked as special point

               nspecialpnts = nspecialpnts+1
               specialpoints(1,nspecialpnts) = istart+inpncv
               iwork(2,1) = nspecialpnts
               specialpoints(2,nspecialpnts) = icurnr
               specialpoints(3,nspecialpnts+1) = 0
               specialpoints(4,nspecialpnts) = 0
               specialpoints(5,nspecialpnts) = 0
               specialpoints(6,nspecialpnts) = 0
               specialpoints(7,nspecialpnts) = iuserp2
               specialusp(iuserp2) = nspecialpnts

            else

!           --- Last point already marked as special point

               ispecial = specialusp(iuserp2)
               iwork(2,1) = ispecial
               if ( specialpoints(4,ispecial).ne.0 ) then

!              --- Error more than 2 curves in special point

                  call errint ( icurnr, 1 )
                  call errint ( iuserp2, 2 )
                  call errsub ( 2102, 2, 0, 0 )

               end if
               specialpoints(4,ispecial) = istart+inpncv
               specialpoints(5,ispecial) = icurnr

            end if

         end if

      else

!     --- The curve consists of more parts, it must be splitted

         if ( typenode1 ) then

!        --- First point is obstacle point

            if ( specialusp(iuserp1).eq.0 ) then

!           --- First point not already marked as special point

               nspecialpnts = nspecialpnts+1
               iwork(2,1) = nspecialpnts
               specialpoints(1,nspecialpnts) = istart+1
               specialpoints(2,nspecialpnts) = icurnr
               specialpoints(3,nspecialpnts) = nspecialpnts+1
               specialpoints(4,nspecialpnts) = 0
               specialpoints(5,nspecialpnts) = 0
               specialpoints(6,nspecialpnts) = 0
               specialpoints(7,nspecialpnts) = iuserp1
               specialusp(iuserp1) = nspecialpnts

            else

!           --- First point already marked as special point

               ispecial = specialusp(iuserp1)
               iwork(2,1) = ispecial
               if ( specialpoints(4,ispecial).ne.0 ) then

!              --- Error more than 2 curves in special point

                  call errint ( icurnr, 1 )
                  call errint ( iuserp1, 2 )
                  call errsub ( 2102, 2, 0, 0 )

               end if
               specialpoints(4,ispecial) = istart+1
               specialpoints(5,ispecial) = icurnr
               specialpoints(6,ispecial) = nspecialpnts

            end if

         end if

!        --- Fill other special points

         obstacle = typenode1
         do i = 2, nparts
            obstacle = .not. obstacle
            nspecialpnts = nspecialpnts+1
            iwork(2,i) = nspecialpnts
            if ( obstacle ) then

!           --- obstacle = true, start of new part on obstacle

               specialpoints(1,nspecialpnts) = istart+iwork(1,i)
               specialpoints(3,nspecialpnts) = 0

            else

!           --- obstacle = false, end of part on obstacle

               specialpoints(1,nspecialpnts) = istart+iwork(1,i)
               if ( i.ne.2 ) then

!              --- Standard end point of part

                  specialpoints(3,nspecialpnts) = nspecialpnts-1
                  specialpoints(3,nspecialpnts-1) = nspecialpnts

               else

!              --- End point of part starting in point 1

                  specialpoints(3,nspecialpnts) = specialusp(iuserp1)

               end if

            end if
            specialpoints(2,nspecialpnts) = icurnr
            specialpoints(4,nspecialpnts) = 0
            specialpoints(5,nspecialpnts) = 0
            specialpoints(6,nspecialpnts) = 0
            specialpoints(7,nspecialpnts) = 0

         end do

         if ( obstacle ) then

!        --- Last point is on obstacle, hence part ends in iuserp2

            if ( specialusp(iuserp2).eq.0 ) then

!           --- Last point not already marked as special point

               nspecialpnts = nspecialpnts+1
               specialpoints(1,nspecialpnts) = istart+inpncv
               specialpoints(2,nspecialpnts) = icurnr
               specialpoints(3,nspecialpnts+1) = nspecialpnts-1
               specialpoints(4,nspecialpnts) = 0
               specialpoints(5,nspecialpnts) = 0
               specialpoints(6,nspecialpnts) = 0
               specialpoints(7,nspecialpnts) = iuserp2
               specialusp(iuserp2) = nspecialpnts

            else

!           --- Last point already marked as special point

               ispecial = specialusp(iuserp2)
               if ( specialpoints(4,ispecial).ne.0 ) then

!              --- Error more than 2 curves in special point

                  call errint ( icurnr, 1 )
                  call errint ( iuserp2, 2 )
                  call errsub ( 2102, 2, 0, 0 )

               end if
               specialpoints(3,nspecialpnts) = ispecial
               specialpoints(4,ispecial) = istart+inpncv
               specialpoints(5,ispecial) = icurnr
               specialpoints(6,ispecial) = nspecialpnts

            end if

         end if

!        --- Fill information about curves

         istnewcurves(icurnr) = newcurvesstrt
         newcurves(newcurvesstrt) = nparts
         do i = 1, nparts
            newcurves(newcurvesstrt+2*i-1) = iwork(1,i)
            newcurves(newcurvesstrt+2*i  ) = iwork(2,i)
         end do
         newcurvesstrt = newcurvesstrt+nparts*2+1
         nextracurves = nextracurves+nparts

      end if

      call erclos ( 'mshobstac1' )

      end
