      subroutine mshobstchsurf ( kmeshcobst, nelem, npoint, inpelm, &
                                 closed )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-11-2001
!
!   copyright (c) 2001-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Check if surface of obstacle is closed and if all edges contain exactly
!      2 obstacle surface elements
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     3d
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcobst(*), nelem, npoint, inpelm
      logical closed

!     closed         o    If true the boundary is closed
!     inpelm         i    number of points per element
!     kmeshcobst     i    kmesh part c, contains the nodal points
!                         corresponding to the elements of the obstacle
!     nelem          i    Number of elements
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipiwork, ipkmeshdobst, ipknelobst, ipnmeshdobst, isum, nwork
      integer, allocatable :: iwork(:)

!     ipiwork        Starting address of array iwork in array iwork
!     ipkmeshdobst   Starting address of array kmeshdobst in array iwork
!     ipknelobst     Starting address of array knelobst in array iwork
!     ipnmeshdobst   Starting address of array nmeshdobst in array iwork
!     isum           Length of kmeshdobst part 2 (knelobst)
!     iwork          integer work array of length: 11*npoint+1+4*nelem
!                    iwork is subdivided as follows
!                    array        length
!                    kmesh d1     npoint+1
!                    kmesh d2     <10*npoint
!                    nmeshd       3 * nelem
!                    iwork        nelem
!     nwork          Declared length of array iwork
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     TOPEDGEOBST    Create edges and store edge sequence numbers in array
!                    nmeshdobst
!     TOPLM1OBST     fill kmesh part d1 with pointers to kmesh part d2 for an
!                    obstacle
!     TOPLM2OBST     fill second part of kmesh part d for an obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    1382   Value of NPELM is not yet implemented for this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill kmesh d1 with pointers to kmesh d2 for obstacle
!     fill second part of kmesh part d for an obstacle with
!        elements corresponding to each node
!     Perform actual checking by creating edges
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstchsurf' )
      if ( inpelm.ne.3 ) then

!     --- inpelm # 3, not yet implemented

         call errint ( inpelm, 1 )
         call errsub ( 1382, 1, 0, 0 )
         go to 1000

      end if

      nwork = 11*npoint+1+4*nelem

      allocate ( iwork(nwork), stat = error )
      if ( error/=0 ) call eralloc ( error, nwork, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Fill kmesh d1 with pointers to kmesh d2 for obstacle
!         Store in iwork(1-npoint+1)

      ipkmeshdobst = 1
      ipknelobst = npoint+2
      call toplm1obst ( nelem, npoint, iwork(ipkmeshdobst), kmeshcobst, &
                        isum )

!     --- fill second part of kmesh part d for an obstacle with
!         elements corresponding to each node
!         Store in iwork(npoint+2 .. npoint+1+isum)

      ipiwork = npoint+2+isum
      if ( ipiwork+npoint.gt.nwork ) then

!     --- iwork too small

         call errint ( ipiwork+npoint, 1 )
         call errint ( nwork, 2 )
         call errchr ( 'iwork', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if
      call toplm2obst ( nelem, npoint, iwork(ipiwork), &
                        iwork(ipkmeshdobst), iwork(ipknelobst), &
                        kmeshcobst )

!     --- Perform actual checking by creating edges

      ipnmeshdobst = ipiwork
      ipiwork = ipiwork+3*nelem
      if ( ipiwork+nelem.gt.nwork ) then

!     --- iwork too small

         call errint ( ipiwork+nelem, 1 )
         call errint ( nwork, 2 )
         call errchr ( 'iwork', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      call topedgeobst ( nelem, iwork(ipkmeshdobst), kmeshcobst, &
                         iwork(ipnmeshdobst), iwork(ipiwork), &
                         iwork(ipknelobst), closed )


      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'mshobstchsurf' )
      end
