      subroutine mshobstcross1 ( yp, ipcell, jpcell, rmeshx, nintersect, &
                                 iobgrid, nx, ny, curves, nnodes, xi, &
                                 iobgrid1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 02-10-2012 Extra parameter iobgrid1
!        version  2.0  date 07-01-2002 Extra parameter xi
!        version  1.1  date 31-01-2001 Adaptation for cross-sections at common
!                                      points of curve parts
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute number of cross sections of the line y=yp with the obstacle in
!    cell (i,j) and store the result in nintersect
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     node
!     obstacle
!     crossing
!     cross_section
!     crossing_point
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision yp, rmeshx(6), curves(2,*), xi(*)
      integer ipcell, jpcell, nx, ny, iobgrid(0:nx-1,0:ny-1), nnodes, &
              nintersect, iobgrid1(*)

!     curves         i    array containing the coordinates of the curves
!                         In this particular case only the coordinates of the
!                         obstacle must be used, hence array curves must be
!                         called with starting address corresponding to the
!                         specific obstacle
!     iobgrid        i    array of length nx x ny that is filled as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax
!                         The cell size is dx x dy
!                         Cell i,j refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy) x (xmin+idx,ymin+jdy)
!                         iobgrid(i,j) = 0: cell outside obstacle
!                         iobgrid(i,j) = k>0: boundary part k crosses cell
!                         iobgrid(i,j) = -1: cell completely in obstacle
!     iobgrid1       i    Must be used in connection with iobgrid
!                         iobgrid contains the starting addresses for each cell
!                         i,j
!                         From this starting address iobgrid1 contains
!                         the number of curve parts of the obstacle
!                         intersecting the cell followed by the sequence
!                         numbers of these curve parts
!     ipcell         i    i-index of cell(i,j)
!     jpcell         i    j-index of cell(i,j)
!     nintersect    i/o   Number of intersections found
!                         This number must be initialized before
!     nnodes         i    Number of nodes in the obstacle
!     nx             i    number of cells in x direction corresponding to
!                         obstacle
!     ny             i    number of cells in y direction corresponding to
!                         obstacle
!     rmeshx         i    Real part of kmesh part x (i = obstacle number)
!                         rmeshx(1,i) = xmin
!                         rmeshx(2,i) = xmax
!                         rmeshx(3,i) = ymin
!                         rmeshx(4,i) = ymax
!                         rmeshx(5,i) = dx
!                         rmeshx(6,i) = dy
!     xi            i/o   Array containing the x-coordinates of the intersection
!                         points in the cell
!     yp             i    y-coordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxcells
      parameter ( maxcells=30 )
      integer icell(maxcells), ncell, i, ind, l, istart
      double precision xmin, xmax, dx, xlow(2), tang(2), &
                       l2norm, alpha1, x, xprev
      logical doublepoint

!     alpha1         multiplication factor
!     doublepoint    if true an intersection at a common point of two
!                    curve parts has been found
!     dx             length of vector in x-direction
!     i              Counting variable
!     icell          Array containing the curve parts that must be considered
!                    for computing the cross sections
!     ind            indicates index in curve part
!     istart         Starting address
!     l              Counting variable
!     l2norm         L2-norm of vector
!     maxcells       Declared length of array icell
!     ncell          Number of curve parts that must be used to compute
!                    cross sections
!     tang           In this array the tangential vector is stored
!     x              x co-ordinate of point
!     xlow           Array containing the coordinates of the first point at
!                    a curve part
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     xprev          Previous x value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHFILLCELL    Fill array icell for one grid cell
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Extract xmin,xmax,ymin,ymax of cell(i,j) from obstacle array
!    ind1 := iobgrid(i,j)
!    icell(1) := ind1-1; icell(2) := ind1; icell(3) := ind1+1; ncell := 3
!    For i := 1 (1) ncell do
!       xlow := coordinates first node of curve part icell(i)
!       tang := tangential vector along curve part icell(i)
!       if ( tang(2)#0 ) then
!          alpha := (yp-xlow(2))/tang(2)
!          x := xlow(2)-alpha*tang(2)
!          if ( xmin<=x<=xmax ) then
!             nintersect := nintersect+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstcross1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstcross1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Extract xmin,xmax,ymin,ymax of cell(i,j) from obstacle array

      dx = rmeshx(5)
      xmin = rmeshx(1)+ipcell*dx
      xmax = xmin+dx

!     --- ind1 := iobgrid(i,j)
!         icell(1) := ind1-1; icell(2) := ind1; icell(3) := ind1+1; ncell := 3

      ncell = 0
      istart = iobgrid(ipcell,jpcell)
      call mshfillcell ( istart, iobgrid1, ncell, maxcells, &
                         icell, nnodes )
      if ( ierror/=0 ) go to 1000

!     --- For i := 1 (1) ncell do

      doublepoint = .false.
      do i = 1, ncell

!     --- xlow := coordinates first node of curve part icell(i)
!         tang := tangential vector along curve part icell(i)

         ind = icell(i)
         xlow(1) = curves(1,ind)
         xlow(2) = curves(2,ind)
         tang(1) = curves(1,ind+1)-xlow(1)
         tang(2) = curves(2,ind+1)-xlow(2)
         l2norm = sqrt(tang(1)**2 + tang(2)**2)

         if ( abs(tang(2))>sqreps*l2norm ) then

!        --- tang(2)#0
!            alpha := (yp-xlow(2))/tang(2)
!            x := xlow(2)-alpha*tang(2)

            alpha1 = (yp-xlow(2))/tang(2)
            x = xlow(1)+alpha1*tang(1)
            if ( doublepoint .and. &
                 ( alpha1>=-1d-6 .and. alpha1<=1d-6 .or. &
                   alpha1>=0.999999d0 .and. alpha1<=1.000001d0 ) ) then

!           --- Possiby double point, check x

               if ( abs(x-xprev)>1d-5 ) then

!              --- Points are different, raise nintersect

                  do l = 1, nintersect
                     if ( abs(xi(l)-x )<=1d-5 ) go to 200
                  end do
                  nintersect = nintersect+1
                  if ( nintersect>6 ) then

!                 --- xi too small

                     call errint ( nintersect, 1 )
                     call errint ( 6, 2 )
                     call errchr ( 'xi', 1 )
                     call errsub ( 550, 2, 0, 1 )
                     go to 1000

                  end if
                  xprev = x
                  xi(nintersect) = x
                  doublepoint = .false.

200            end if

            else if ( alpha1>=-accuracy_obstacle .and. &
                      alpha1<=1d0+accuracy_obstacle .and. &
                      x>=xmin .and. x<=xmax ) then
               nintersect = nintersect+1
               if ( alpha1>=-1d-6 .and. alpha1<=1d-6 .or. &
                    alpha1>=0.999999d0 .and. alpha1<=1.000001d0 ) then

!              --- Point at the end of a cell, mark as possibly double

                  doublepoint = .true.
                  xprev = x

               end if

            end if

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstcross1'

      end if  ! ( debug )
      call erclos ( 'mshobstcross1' )

      end
