      subroutine mshobstmarkeln ( ipointobstlc, ielemobstlc, kmeshcloc, &
                                  lenielemobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 11-10-2011 Fortran 90
!        version  1.2  date 08-11-2004 Clear iwork for each obstacle
!        version  1.1  date 11-01-2004 New definition ipointobstlc
!
!   copyright (c) 2001-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all elements in the mesh with information if they are
!     within an obstacle or outside an obstacle
!     Store this information in array ielemobst
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     element
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipointobstlc(*), ielemobstlc(*), kmeshcloc(*), &
              lenielemobst

!     ielemobstlc      o    Array containing information about the elements in an
!                         obstacle
!                         The first numobst positions contain the starting
!                         positions of each obstacle sequentially
!                         Per obstacle the first three positions contain
!                         1. The number of elements that are completely inside
!                            the obstacle
!                         2. The number of elements that are not completely
!                            inside the element but with all vertices inside
!                            the element
!                         3. The number of elements that are partly inside
!                            the obstacle
!                            elements that contain only boundary points of the
!                            obstacle are not considered
!                         These positions are followed by the element numbers
!                         in the sequence defined by the first three positions
!     ipointobstlc   i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!     kmeshcloc      i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     lenielemobst   o    Computed length of array ielemobst
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iielem, ielem, &
              ielgrp, ikelmc, iobst, istart, istartelem, &
              nbnpelm, ninpelm, numbndobst, numinnobst, &
              numinelmobst, numpntobst, numinonobst
      integer :: iwork(npoint)

!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     ikelmc         position in kmesh part c
!     iobst          Obstacle sequence number
!     istart         First position used at a part
!     istartelem     Starting address in element array
!     iwork          integer work array of length npoint
!                    In this array all nodes in or on the obstacle are
!                    marked as follows
!                    iwork(i) = 0: node i outside obstacle
!                    iwork(i) = 1: node i in obstacle
!                    iwork(i) = 2: node i in obstacle near boundary
!                    iwork(i) = 3: node i in obstacle on boundary
!     nbnpelm        Number of points in element that are on the boundary of
!                    the obstacle
!     ninpelm        Number of points in element that are inside
!                    the obstacle
!     numbndobst     Number of nodal points inside the obstacle
!     numinelmobst   Number of elements that are completely within the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numinonobst    Number of nodal points on the boundary of the obstacle
!                    which are only in elements completely inside the obstacle
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPRINIELOBST Print the contents of array ielemobst
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork
!     For iobst := 1 (1) numobst do
!        Fill array iwork(i) with a 1 if node i is in the obstacle
!           and with a 3 when it is on the boundary
!        For ielgrp := 1 (1) nelgrp do
!           inelem := number of elements in element group ielgrp
!           For iielem := 1 (1) inelem
!              ielem := element numbr
!              If (all nodes of element in obstacle iobst) then
!                fill element in ielemobst
!              else if (some nodes of element in obstacle iobst and all others
!                       not ) then
!                fill element in iwork1
!        Fill rest of ielemobst for this obstacle
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstmarkeln' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstmarkeln'
         call sepprinipobst ( ipointobstlc )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Make initializations

      inpelm = km(iseqkm)%kmesh(4)
      nelgrp = km(iseqkm)%kmesh(5)
      npoint = km(iseqkm)%kmesh(8)
      inelem = km(iseqkm)%kmesh(9)
      kelma = km(iseqkm)%kmesh(15)
      kelmg = km(iseqkm)%kmesh(21)
      numobst = km(iseqkm)%kmesh(42)

!     --- Clear array iwork

      iwork(1:npoint) = 0
      istartelem = numobst+1

!     --- Loop over obstacles

      do iobst = 1, numobst

!     --- Fill array iwork(i) with a 1 if node i is in the obstacle
!         and with a 3 when it is on the boundary

         istart = ipointobstlc(iobst)-1
         numpntobst = ipointobstlc(istart+1)
         numinnobst = ipointobstlc(istart+2)
         numbndobst = ipointobstlc(istart+3)
         numinonobst = ipointobstlc(istart+4)
         istart = istart+4
         iwork(ipointobstlc(istart+1:istart+numpntobst)) = 1
         istart = istart+numpntobst+numinnobst
         iwork(ipointobstlc(istart+1:istart+numbndobst)) = 3
         istart = istart+numbndobst
         iwork(ipointobstlc(istart+1:istart+numinonobst)) = 1
         numinelmobst = 0
         ielemobstlc(iobst) = istartelem

!        --- For ielgrp := 1 (1) nelgrp do

         ielem = 0
         ikelmc = 0
         do ielgrp = 1, nelgrp

!        --- inelem := number of elements in element group ielgrp

            if ( nelgrp>1 ) inelem = km(iseqkm)%kmeshg(ielgrp)
            if ( kelma/=0 ) inpelm = km(iseqkm)%kmesha(1+ielgrp)- &
                                       km(iseqkm)%kmesha(ielgrp)

!           --- For iielem := 1 (1) inelem

            do iielem = 1, inelem

!           --- ielem := element number
!               If (all nodes of element in obstacle iobst) then
!                 fill element in ielemobst
!               else if (some nodes of element in obstacle iobst and all others
!                        not ) then
!                 fill element in iwork1

               ielem = ielem+1
               nbnpelm = 0
               ninpelm = 0
               do i = 1, inpelm
                  inodp = kmeshcloc(ikelmc+i)
                  if ( iwork(inodp)==3 ) then

!                 --- Node on obstacle found

                     nbnpelm = nbnpelm+1

                  else if ( iwork(inodp)>0 ) then

!                 --- Node in obstacle found

                     ninpelm = ninpelm+1

                  end if

               end do

               if ( ninpelm+nbnpelm==inpelm ) then

!              --- all points in obstacle, hence element completely in obstacle

                  numinelmobst = numinelmobst+1
                  ielemobstlc(istartelem+2+numinelmobst) = ielem

               end if

               ikelmc = ikelmc+inpelm

            end do

         end do

!        --- Fill rest of ielemobst for this obstacle

         ielemobstlc(istartelem) = numinelmobst  ! elements inside the obstacle
         ielemobstlc(istartelem+1) = 0           ! elements outside the obstacle
                                               ! but all vertices in (not
                                               ! defined yet)
         ielemobstlc(istartelem+2) = 0           ! elements partly inside the
                                               ! obstacle (none due to
                                               ! construction)
         istartelem = istartelem+3+numinelmobst

!        --- Clear array iwork

         istart = ipointobstlc(iobst)-1
         istart = istart+4
         do i = 1, numpntobst
            iwork(ipointobstlc(istart+i)) = 0
         end do
         istart = istart+numpntobst+numinnobst
         do i = 1, numbndobst
            iwork(ipointobstlc(istart+i)) = 0
         end do
         istart = istart+numbndobst
         do i = 1, numinonobst
            iwork(ipointobstlc(istart+i)) = 0
         end do

      end do
      lenielemobst = istartelem-1

1000  if ( debug ) then

!     --- Debug information

         call sepprinielobst ( ielemobst )
         write(irefwr,*) 'End mshobstmarkeln'

      end if
      call erclos ( 'mshobstmarkeln' )

      end
