      subroutine mshrd6 ( iinput, ipoint, inpmax, iinp, isurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version 19.2  date 20-11-2014 Use mshrd6shape
!        version 19.1  date 22-05-2014 Extension with shape 22
!        version 19.0  date 16-07-2008 Extra parameter
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read cards with volumes until a card with mesh is found
!     fill array iinput with the information of the volumes
!     iinput must be filled from position 1
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmoduleread1
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipoint(maxvlm), &
              inpmax, iinput(inpmax), iinp, isurfs(nsurfs)

!     iinp           o    next free position in iinput
!     iinput         o    integer input array
!     inpmax         i    maximal space for iinput
!     ipoint         o    pointer array: contains the starting positions of the
!                         curves in array iinput
!     isurfs         i    array containing the number of points in the surfaces
!                         accumulated
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nsigvl=3, nvoltp=10
      character (len=3) codecl(nvoltp)
      integer i, inum,  nsur, j, kvolms, icopy, jinp

!     codecl         In this table the significant characters of the various
!                    types of volumes are stored.
!                    The following types of volumes are available:
!                     1:   brick
!                     2:   user
!                     3:   input
!                     4:   pipe
!                     5:   general
!                     6:   translate
!                     7:   rotate
!                     8:   reflect
!                     9:   channel
!                    10:   general with internal points
!                    11:   old_general
!     i              general loop variable
!     icopy          Number of volume to be copied
!     inum           Number of numbers of read
!     j              Counting variable
!     jinp           temporary position in array iinput
!     kvolms         Actual number of volumes found
!     nsigvl         Number of significant characters to distinguish between the
!                    various volumes types
!     nsur           Number of surfaces found in volume record
!     nvoltp         Number of different types of volumes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     MSHR02         Read information after the part Vi = BRICK ( ..
!     MSHRD6SHAPE    Read or set shape number of volume
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     142   Internal error, too few space for integers
!     286   Volume number has already been read before
!     288   volume number has incorrect value
!     291   number of surfaces read in volume record is incorrect
!     648   string could not be recognized
!     968   Wrong character in volume record
!     969   Number of volume to be copied not correct
!    1348   Record does not start with a character
!    1353   Not a text found after Vi
!    1354   The item after V in reading of volumes is not an integer number
!    1356   No surface numbers found in Volume record
!    1358   Variable has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codecl / 'BRI', 'USE', 'INP', 'PIP', 'GEN', 'TRA', &
                    'ROT', 'REF', 'CHA', 'OLD' /
! ======================================================================
!
      call eropen ( 'mshrd6' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd6'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( work(1500) )

!     --- initialize array ipoint

      ipoint(1:maxvlm) = 0

      nvolms = 0
      kvolms = 0
      iinp = 1

!     --- read next record

200   ityprd = 1
      call asknxt
      if ( iinp>=inpmax ) then

!     --- iinp>=  inpmax

          call errint ( inpmax, 1 )
          call errint ( iinp, 2 )
          call errsub ( 142, 2, 0, 0 )
          go to 1000

      end if
      if ( ityprd/=4 ) then

!     --- error 1348: Record does not start with a character

         call errsub ( 1348, 0, 0, 0 )
         call errcrd
         go to 200

      else

!     --- Detect the type of record
!         If a main keyword has been found, return to calling subroutine

         ierpri = 0
         icode = sepmainkeyw () - 1
         if ( icode>=0 ) go to 1000

      end if
      if ( string(1:1)/='V' ) then
         call errchr ( string(1:1), 1 )
         call errchr ( 'V', 2 )
         call errsub ( 648, 0, 0, 2 )
         call errcrd
         go to 200
      end if

!     --- detect the volume
!         first the volume number is read

      call asknxt
      kvolms = kvolms+1
      if ( ityprd/=2 ) then

!     --- Next item is not an integer

         call errsub ( 1354, 0, 0, 0 )
         call errcrd
         go to 200

      end if
      ivolnr  = nint( realrd )
      nvolms = max ( nvolms, ivolnr )
      if ( ivolnr<=0 .or. ivolnr>maxvlm ) then
         call errint ( ivolnr, 1 )
         call errint ( maxvlm, 2 )
         call errsub ( 288, 2, 0, 0 )
         call errcrd
      end if
      if ( ipoint(ivolnr)/=0 ) then

!     --- Surface number has already been used

         call errint ( ivolnr, 1 )
         call errwar ( 286, 1, 0, 0 )
         call errcrd

      end if

      ipoint(ivolnr) = iinp

!     --- detect type of volume

      call asknxt
      if ( ityprd/=4 ) then

!     --- Not a text found after Vi = &

         call errint ( ivolnr, 1 )
         call errsub ( 1353, 1, 0, 0 )
         call errcrd
         go to 200

      end if

      nncode = nvoltp
      code(1:nvoltp) = codecl
      ncodefix = nsigvl
      ierpri = 1
      ivol = sepkeyword3()
      if ( ivol==10 ) ivol = 11
      if ( debug ) write(irefwr,1) 'Type of volume generator', ivol
      if ( ivol==0 ) goto 200

      iinput(iinp) = ivol
      iinput(iinp+1) = 0
      iinput(iinp+2) = 0

!     --- Read rest of volume depending on ivol

      if ( ivol==6 .or. ivol==7 .or. ivol==8 ) then

!     --- ivol = 6, 7 or 8 (TRANSLATE, ROTATE or REFLECT )
!         Read volume to be "copied"

         call asknxt

         if ( ityprd==4 ) then

!        --- Text found, this must be V

            if ( nchar/=1 .or. string(1:1)/='V' ) then

!           --- Error 968: Not a V found

               call errchr ( string(1:1), 1 )
               call errint ( ivolnr, 1 )
               call errsub ( 968, 1, 0, 1 )
               call errcrd
               go to 200

            else

               call asknxt

            end if

         end if

         if ( ityprd/=2 ) then

!        --- Not an integer found

            call errchr ( string(1:1), 1 )
            call errint ( ivolnr, 1 )
            call errsub ( 968, 1, 0, 1 )
            call errcrd
            go to 200

         end if

         icopy = nint( realrd )

      else

!     --- Other volume type, determine shape number

         call mshrd6shape ( iinput, iinp )

      end if

!     --- Detect rest of record

      call mshr02 ( isurfs, nsurfs, ivol, ivolnr, nuspnt, ncurvs )
      if ( debug ) call prinrl ( work, 10, 'work' )

      nsur   = nint( work(1) )
      nmes   = nint( work(2) )
      mmes   = nint( work(3) )
      lmes   = nint( work(4) )
      nelem  = nint( work(5) )
      npoint = nint( work(6) )
      iorien = nint( work(7) )

!     --- In case of general with internal points, curves or surfaces, &
!         make ivol 10

      if ( ivol==5 .and. work(8)+work(9)+work(10)>0d0 ) then

!     --- internal points found, set ivol to 10

         ivol = 10
         iinput(iinp) = ivol

      end if  ! ( ivol==3 .and. work(23)>0d0 )
      if ( debug ) write(irefwr,1) 'Type of volume generator', ivol

!     --- Distinguish between the various volumes

      if ( nsur==0 ) then

!     --- Error 1356: No surfaces found in volume record

         call errint ( ivolnr, 1 )
         call errsub ( 1356, 1, 0, 0 )
         call errcrd
         go to 200

      end if

      iinput(iinp+2) = nsur
      do i = 1, nsur
         iinput(iinp+2+i) = nint( work(24+i) )
      end do
      iinp = iinp + 3 + nsur
      if ( debug ) write(irefwr,1) 'iinp following nsur', iinp

      if ( ivol==1 ) then

!     --- brick

         iinput(iinp) = nmes
         iinput(iinp+1) = mmes
         iinput(iinp+2) = lmes
         if ( iorien>0 ) then

!        --- orientation found

            iinp = iinp - 3 - nsur
            do i = 1, 6
               j = mod(iorien,10)-1
               iinput(iinp+2+7-i) = iinput(iinp+2+7-i)+j*nsurfs
               iorien = iorien/10
            end do
            iinp = iinp + 3 + nsur
         end if
         iinp = iinp+3
         if ( nmes<0 .or. nmes>10000 ) then

!        --- Error 1358:   N out of range

            call errint ( nmes, 1 )
            call errint ( 10000, 2 )
            call errint ( ivolnr, 3 )
            call errchr ( 'N', 1 )
            call errsub ( 1358, 3, 0, 1 )

         end if
         if ( mmes<0 .or. mmes>10000 ) then

!        --- Error 1358:   M out of range

            call errint ( mmes, 1 )
            call errint ( 10000, 2 )
            call errint ( ivolnr, 3 )
            call errchr ( 'M', 1 )
            call errsub ( 1358, 3, 0, 1 )

         end if
         if ( lmes<0 .or. lmes>10000 ) then

!        --- Error 1358:   L out of range

            call errint ( lmes, 1 )
            call errint ( 10000, 2 )
            call errint ( ivolnr, 3 )
            call errchr ( 'L', 1 )
            call errsub ( 1358, 3, 0, 1 )

         end if
         inum = 6

      else if ( ivol==2 ) then

!     --- user

         iinput(iinp) = nelem
         iinput(iinp+1) = npoint
         iinp = iinp+2
         inum = nsur

      else if ( ivol==4 .or. ivol==9 ) then

!     --- pipe or channel

         inum = 3

      else if ( ivol==5 .or. ivol==11 ) then

!     --- general

         inum = 1

      else if ( ivol==6 .or. ivol==7 .or. &
                ivol==8 ) then

!     --- translate, rotate or reflect

         iinput(iinp) = icopy
         iinp = iinp+1

         if ( icopy>=ivolnr .or. icopy<=0 ) then

!        --- Number of volume to be copied either too large or too small

            call errint ( icopy, 1 )
            call errint ( ivolnr, 2 )
            call errsub ( 969, 2, 0, 0 )
            call errcrd

         end if

         inum = nsur

      else if ( ivol==10 ) then

!     --- general with internal points, curves or surfaces
!         store in iinput

         inum = 1
         iinput(iinp) = nint(work(8))  ! number of internal points
         iinput(iinp+1) = nint(work(9))  ! number of internal curves
         iinput(iinp+2) = nint(work(10))  ! number of internal surfaces
         jinp = iinp+2
         do i = 1, iinput(iinp)
            iinput(jinp+i) = nint(work(1000+i))  ! internal points
         end do  ! i = 1, iinput(iinp)
         jinp = jinp+iinput(iinp)
         do i = 1, iinput(iinp+1)
            iinput(jinp+i) = nint(work(1100+i))  ! internal curves
         end do  ! i = 1, iinput(iinp)
         jinp = jinp+iinput(iinp+1)
         do i = 1, iinput(iinp+2)
            iinput(jinp+i) = nint(work(1200+i))  ! internal surfaces
         end do  ! i = 1, iinput(iinp)

         if ( debug ) then
            write(irefwr,1) 'number of internal points', iinput(iinp)
            write(irefwr,1) 'iinp', iinp
            call prinin ( iinput(iinp+3), iinput(iinp), &
                          'internal points' )
            write(irefwr,1) 'number of internal curves', iinput(iinp+1)
            call prinin ( iinput(iinp+3+iinput(iinp)), iinput(iinp+1), &
                          'internal curves' )
            write(irefwr,1) 'number of internal surfaces', &
                            iinput(iinp+2)
            call prinin ( iinput(iinp+3+iinput(iinp)+iinput(iinp+1)), &
                          iinput(iinp+2), 'internal surfaces' )
         end if  ! ( debug )

         iinp = iinp+3+iinput(iinp)+iinput(iinp+1)+iinput(iinp+2)  ! raise iinp
         if ( debug ) write(irefwr,1) 'iinp ivol = 10:', iinp

      end if

      if ( inum/=nsur ) then
         call errint ( nsur, 1 )
         call errint ( inum, 2 )
         call errsub ( 291, 2, 0, 0 )
         call errcrd
      end if

      goto 200

1000  iinp = iinp + nvolms - kvolms

      deallocate ( work )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nvolms, kvolms', nvolms, kvolms
         call prinin ( iinput, iinp-1, 'iinput' )
         write(irefwr,*) 'End mshrd6'

      end if
      call erclos ( 'mshrd6' )

      end subroutine mshrd6
