      subroutine mshrdcompsf ( coorlc, krenum, jsurnr, nsubsf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.3  date 04-02-2015 Minor adaptations
!        version  3.2  date 15-11-2012 Long integers
!        version  3.1  date 14-08-2008 Fortran 90
!
!   copyright (c) 2001-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information of subsurfaces in general coordinates
!     and topology array of composite surface
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
!     topology
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer krenum(*), jsurnr, nsubsf
      double precision coorlc(ndim,*)

!     coorlc         o    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     jsurnr         i    Surface sequence number
!     krenum              Work array of length npoint+inelem to store the
!                         local mapping of the subsurfaces
!     nsubsf         i    Number of subsurfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, jstart, isubsf, isur, index, lenkmsc, &
              npoloc, nelloc, i, ih, ikelmc, k, jsur
      integer, pointer :: iwork(:)
      double precision, pointer :: work(:,:)

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ikelmc         position in kmesh part c
!     index          record number for reading
!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
!     isur           surface number
!     iwork          integer work array of length inpelm*inelem to store the
!                    local topology array of the subsurfaces
!     jstart         starting position
!     jsur           surface number provided with sign
!     k              Counting variable
!     lenkmsc        Length of the local topology array
!     nelloc         Local number of elements
!     npoloc         Number of nodes in subsurface
!     work           Work array of length npoint to store the local
!                    coordinates of the subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all subsurfaces do
!        read local numbering and store in krenum
!        read local array coor
!        Put co-ordinates from local array coor into global coor
!        Read local array kmeshclc and store in iwork
!        Put local array kmeshclc into global kmeshclc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrdcompsf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrdcompsf'
         write(irefwr,1) 'jsurnr, ndim', jsurnr, ndim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      istart = kmeshr(jsurnr)
      jstart = kmeshr(nsurfs+1) - nsurfs - 1 + istart
      inelem = 0

!     --- Loop over subsurfaces

      do isubsf = 1, nsubsf

         jsur = kmeshr(istart+isubsf)
         isur = abs(jsur)

         index = kmeshr(jstart+isubsf)
         npoloc = kmeshh(1,isur+1)
         nelloc = kmeshh(2,isur+1)
         lenkmsc = inpelm*nelloc
         inelem = inelem+nelloc

         if ( debug ) &
            write(irefwr,1) 'isubsf, isur, jsur, npoloc, nelloc', &
                             isubsf, isur, jsur, npoloc, nelloc

         allocate ( work(ndim,npoloc), iwork(lenkmsc), stat = error )
         if ( error/=0 ) call eralloc ( error, ndim*npoloc+lenkmsc, 'work' )
         if ( ierror/=0 ) go to 1000

!        --- read local numbering and store in krenum

         call readfl ( index, 1, npoloc+nelloc, 1, 3, krenum )

         if ( debug ) then
            call prinin ( krenum, npoloc, 'nodes' )
            call prinin ( krenum(1+npoloc), nelloc, 'elements' )
         end if  ! ( debug )

!        --- Copy local array coor into work

         call dcopy ( npoloc*ndim, kmhsur(isur)%coorsur, 1, work, 1 )

!        --- Put co-ordinates from local array coor into global coorlc

         coorlc(1:ndim,krenum(1:npoloc)) = work(1:ndim,1:npoloc)

!        --- Copy local array kmeshclc into iwork

         iwork(1:lenkmsc) = kmhsur(isur)%kmeshcsur

!        --- Put local array kmeshclc into global kmeshclc

         do i = 1, nelloc
            ikelmc = ( krenum(npoloc+i)-1 ) * inpelm
            ih = ( i-1 ) * inpelm
            if ( jsur>0 ) then

!           --- jsur>0, standard case, positive direction

               do k  = 1, inpelm
                  kmeshclc(ikelmc+k) = krenum ( iwork(ih+k) )
               end do

            else

!           --- jsur<0, special case, reversed direction

               if ( ishape<=5 ) then

!              --- ishape = 3, 4, 5 all points must be reversed

                  kmeshclc(ikelmc+1) = krenum ( iwork(ih+1) )
                  do k  = 2, inpelm
                     kmeshclc(ikelmc+inpelm+2-k) = krenum ( iwork(ih+k) )
                  end do

               else

!              --- ishape>5 last point must be kept

                  kmeshclc(ikelmc+1) = krenum ( iwork(ih+1) )
                  do k  = 2, inpelm-1
                     kmeshclc(ikelmc+inpelm+1-k) = krenum ( iwork(ih+k) )
                  end do
                  k = inpelm
                  kmeshclc(ikelmc+k) = krenum ( iwork(ih+k) )

               end if

            end if

         end do

         deallocate ( work, iwork, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'work' )
         if ( ierror/=0 ) go to 1000

      end do

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshclc, inelem, inpelm, 'kmeshclc' )
         write(irefwr,*) 'End mshrdcompsf'

      end if  ! ( debug )
      call erclos ( 'mshrdcompsf' )

      end
