      subroutine mshreadsqrcha ( iinput, iinputloc, ipoint, lenipoint, &
                                 leng, lengin, isurnr, newcurve, &
                                 rinputloc, maxcrv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-03-2015 change leng
!        version  1.0  date 03-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Change contents of array iinput and ipoint for square
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: lenipoint, isurnr, maxcrv
      integer, intent(inout) :: iinput(*), iinputloc(*), lengin, &
                                ipoint(lenipoint,2), leng
      logical, intent(inout) :: newcurve
      double precision, intent(inout) :: rinputloc(maxcrv,3)

!     iinput        i/o   integer input array, see mshrea
!                         The positions 1 to lengin have been filled
!                         i.e. positions corresponding to points and curves
!     iinputloc     i/o   Local array iinput, contains the input for the surface
!                         At input for a square at output for rectangle
!     ipoint        i/o   pointer array: contains the starting positions of the
!                         curves (1) and surfaces (2) in array iinput
!     isurnr         i    Surface sequence number
!     leng          i/o   Length of surface part of iinput
!     lengin        i/o   Length of integer array iinput
!     lenipoint      i    Length of array ipoint
!     maxcrv         i    Maximal number of curves allowed
!     newcurve      i/o   if true new curves have been created
!     rinputloc     i/o   Contains the part of rinput corresponding to the
!                         curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ncurvs, numpoints, numcurves, ishape, shapecur, n, m, &
                 jcoars, icurnr, jcurnr, iinp, i

!     i              Counting variable
!     icurnr         Curve sequence number
!     iinp           last position used in array iinput
!     ishape         Shape number of elements
!     jcoars         If 1 coarseness is given, if 0 not
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     m              Parameter m in surface record
!     n              Parameter n in surface record
!     ncurvs         Number of curves in mesh
!     numcurves      Number of curves
!     numpoints      Number of user points
!     shapecur       shape number of curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     129   Incorrect curve number found
!    2060   No points given whereas the number of curves is not 4.
!    2531   Number of points given after the keyword square is not 4
!    2538   not coarseness given, nor n and m
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshreadsqrcha' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshreadsqrcha'
         write(irefwr,1) 'leng, lengin', leng, lengin
         call prinin ( iinputloc, 15, 'iinputloc' )
         call prinin ( iinput, lengin, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ncurvs = iinput(3)
      jcoars = iinput(11)

      if ( debug ) then
         call prinin ( ipoint(1,1), ncurvs, 'ipoint/cur' )
      end if  ! ( debug )

      ishape = iinputloc(2)
      numpoints = iinputloc(3)
      iinp = 4+numpoints
      numcurves = iinputloc(iinp)
      n = iinputloc(iinp+2)
      m = iinputloc(iinp+3)
      iinp = iinp+3

      if ( ishape==4 .or. ishape==6 .or. ishape==7 ) then
         shapecur = 2
      else
         shapecur = 1
      end if  ! ( ishape==4 .or. ishape==6 .or. ishape==7 )

      if ( debug ) then
         write(irefwr,1) 'ishape, numpoints, numcurves, n, m', &
                          ishape, numpoints, numcurves, n, m
         write(irefwr,1) 'shapecur, iinp', shapecur, iinp
      end if  ! ( debug )

      if ( numpoints==0 ) then

!     --- No points are given

         if ( (n==0 .or. m==0) .and. numcurves/=4 ) then

!        --- Error: if no points are given there must be 4 existing curves

            call errint ( numcurves, 1)
            call errint ( isurnr, 2 )
            call errsub ( 2060, 2, 0, 0 )
            call errcrd
            go to 1000

         end if  ! ( numcurves/=4 )

!        --- Check if all curves are existing

         do i = 1, numcurves

            icurnr = iinputloc(iinp+i)

            if ( icurnr==0 .or. abs(icurnr)>ncurvs ) then

!           --- Curve number out of range

               call errint ( icurnr, 1 )
               call errint ( ncurvs, 2 )
               call errint ( isurnr, 3 )
               call errsub ( 129, 3, 0, 0 )
               call errcrd
               go to 1000

            end if  ! ( icurnr==0 .or. icurnr>ncurvs )

         end do  ! i = 1, numcurves

      else if ( numpoints/=4 ) then

!     --- Number of points given not equal to 4

         call errint ( numpoints, 1)
         call errint ( isurnr, 2 )
         call errsub ( 2531, 2, 0, 0 )
         call errcrd
         go to 1000

      else

!     --- Number of points given equal to 4, create curves and redefine surface

         if ( debug ) write(irefwr,1) 'n, m', n, m

         if ( n==0 .or. m==0 ) then

!        --- N and M not given, check if all curves are given

            newcurve = .false.
            do i = 1, 4

               icurnr = abs(iinputloc(iinp+i))

               if ( debug ) &
                  write(irefwr,1) 'icurnr, ipoint(icurnr,1)', &
                                   icurnr, ipoint(icurnr,1)

               if ( ipoint(icurnr,1)==0 ) then
                  newcurve = .true.
                  exit
               end if  ! ( ipoint(icurnr,1)==0 )

            end do  ! i = 1, 4

            if ( debug ) write(irefwr,*) 'newcurve', newcurve

            if ( .not. newcurve ) go to 500

            if ( jcoars==0 ) then

!           --- Error n and m not given and jcoars = 0

               call errint ( isurnr, 1 )
               call errsub ( 2538, 1, 0, 0 )
               call errcrd
               go to 1000

            else

!           --- jcoarse>0, use cline for those curves not defined

               do i = 1, 4

                  icurnr = iinputloc(iinp+i)
                  jcurnr = abs(icurnr)

                  if ( debug ) &
                     write(irefwr,1) 'iinp, i, icurnr', iinp, i, icurnr

                  if ( ipoint(jcurnr,1)==0 ) then

!                 --- New curve number found

                     ipoint(jcurnr,1) = lengin+1
                     iinput(lengin+1) = 5        ! cline
                     iinput(lengin+2) = shapecur ! shape of line
                     iinput(lengin+3:lengin+7) = 0
                     rinputloc(jcurnr,1:3) = 1d0
                     rinputloc(jcurnr,2) = 0d0

!                    --- Fill user points

                     if ( icurnr>0 ) then

!                    --- Positive direction

                        iinput(lengin+5) = iinputloc(3+i)
                        if ( i<4 ) then
                           iinput(lengin+6) = iinputloc(4+i)
                        else
                           iinput(lengin+6) = iinputloc(4)
                        end if  ! ( i<4 )

                     else

!                    --- Negative direction

                        iinput(lengin+6) = iinputloc(3+i)
                        if ( i<4 ) then
                           iinput(lengin+5) = iinputloc(4+i)
                        else
                           iinput(lengin+5) = iinputloc(4)
                        end if  ! ( i<4 )

                     end if  ! ( icurnr>0 )
                     lengin = lengin+7
                     ncurvs = max(ncurvs, jcurnr)

                  end if  ! ( abs(icurnr)>ncurvs )

               end do  ! i = 1, 4

            end if  ! ( jcoars==0 )

         else

!        --- n and m are given, use line and rectangle

            if ( debug ) write(irefwr,1) 'Create lines'

            do i = 1, 4

               icurnr = iinputloc(iinp+i)
               jcurnr = abs(icurnr)

               if ( debug ) &
                  write(irefwr,1) 'icurnr, ipoint(jcurnr,1)', &
                                   icurnr, ipoint(jcurnr,1)

               if ( ipoint(jcurnr,1)==0 ) then

!              --- New curve number found

                  newcurve = .true.
                  ipoint(jcurnr,1) = lengin+1
                  iinput(lengin+1) = 1        ! cline
                  iinput(lengin+2) = shapecur ! shape of line

!                 --- Fill number of elements

                  if ( mod(i,2)==1 ) then
                     iinput(lengin+3) = n   ! horizontal
                  else
                     iinput(lengin+3) = m   ! vertical
                  end if  ! ( mod(i,2)==1 )
                  iinput(lengin+4) = 0
                  rinputloc(jcurnr,1:3) = 1d0
                  rinputloc(jcurnr,2) = 0d0

!                 --- Fill user points

                  if ( icurnr>0 ) then

!                 --- Positive direction

                     iinput(lengin+5) = iinputloc(3+i)
                     if ( i<4 ) then
                        iinput(lengin+6) = iinputloc(4+i)
                     else
                        iinput(lengin+6) = iinputloc(4)
                     end if  ! ( i<4 )

                  else

!                 --- Negative direction

                     iinput(lengin+6) = iinputloc(3+i)
                     if ( i<4 ) then
                        iinput(lengin+5) = iinputloc(4+i)
                     else
                        iinput(lengin+5) = iinputloc(4)
                     end if  ! ( i<4 )

                  end if  ! ( icurnr>0 )
                  lengin = lengin+6
                  ncurvs = max(ncurvs, jcurnr)

               end if  ! ( abs(icurnr)>ncurvs )

            end do  ! i = 1, 4

         end if  ! ( n==0 .or. m==0 )

      end if  ! ( numpoints==0)

500   iinput(3) = ncurvs

      iinputloc(1) = 1          ! rectangle
      iinputloc(3) = numcurves  ! number of curves
      iinputloc(4:3+numcurves) = iinputloc(iinp+1:iinp+numcurves) ! curves
      iinputloc(4+numcurves) = n
      iinputloc(5+numcurves) = m
      leng = 5+numcurves

1000  call erclos ( 'mshreadsqrcha' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshreadsqrcha'

      end if  ! ( debug )

      end subroutine mshreadsqrcha
