      subroutine mshreadsurcha ( iinput, iinputloc, ipoint, lenipoint, &
                                 leng, lengin, nsurfs, lengrl, rinput, &
                                 maxcrv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-03-2015 change leng
!        version  1.0  date 03-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Change contents of array iinput and ipoint due to special surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: lenipoint, lengin, nsurfs, maxcrv
      integer, intent(inout) :: iinput(*), iinputloc(*), leng, &
                                ipoint(lenipoint,2), lengrl
      double precision, intent(inout) :: rinput(*)

!     iinput        i/o   integer input array, see mshrea
!                         The positions 1 to lengin have been filled
!                         i.e. positions corresponding to points and curves
!     iinputloc     i/o   Local array iinput, contains the input for the surface
!                         At input for a square at output for rectangle
!     ipoint        i/o   pointer array: contains the starting positions of the
!                         curves (1) and surfaces (2) in array iinput
!     leng          i/o   Length of surface part of iinput
!     lengin        i/o   Length of integer array iinput
!     lengrl        i/o   Length of real array rinput
!     lenipoint      i    Length of array ipoint
!     maxcrv         i    Maximal number of curves allowed
!     nsurfs         i    number of surfaces
!     rinput        i/o   Real input array for mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ncurvs, isurnr, istart, ndim, leng1, lengrlus, jcoars, &
                 i, nuspnt
      integer, allocatable :: iwork(:)
      double precision :: rinputloc(maxcrv,3)
      logical :: newcurve, changed

!     changed        If true new curves have been created
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     istart         starting address in iinputloc
!     isurnr         Surface sequence number
!     iwork          integer work array
!     jcoars         If 1 coarseness is given, if 0 not
!     leng1          Length of local co-ordinates array
!     lengrlus       computed length of array rinput with respect to user points
!     ncurvs         Number of curves in mesh
!     ndim           Dimension of the space.
!     newcurve       if true new curves have been created
!     nuspnt         Number of user points
!     rinputloc      Local array rinput corresponding to curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH014         Rearrange information in IINPUT and RINPUT wrt curves
!     MSHREADSQRCHA  Change contents of array iinput and ipoint due to special
!                    surfaces
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshreadsurcha' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshreadsurcha'
         call prinin ( iinputloc, leng, 'iinputloc' )
         call prinin ( iinput, lengin, 'iinput' )
         call prinrl ( rinput, lengrl, 'rinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      ndim = iinput(1)
      nuspnt = iinput(2)
      ncurvs = iinput(3)
      jcoars = iinput(11)

      lengrlus = 2 + nuspnt * (ndim+jcoars)+2*jcoars  ! present length of rinput

      changed = .false.

      if ( debug ) then
         call prinin ( ipoint(1,1), ncurvs, 'ipoint/cur' )
         call prinin ( ipoint(1,2), nsurfs, 'ipoint/sur' )
      end if  ! ( debug )

!     --- Loop over surfaces

      do isurnr = 1, nsurfs

         istart = ipoint(isurnr,2)         ! starting address in iinputloc

         if ( iinputloc(istart)==23 ) then

!        --- square found, hence possible update of iinput and iinputloc
!            Copy information of curves in rinputloc

            rinputloc(1:ncurvs,1) = rinput(lengrlus+1:lengrlus+ncurvs)
            leng1 = lengrlus+ncurvs
            do i = 1, ncurvs
               rinputloc(i,2) = rinput(leng1+2*i-1)
               rinputloc(i,3) = rinput(leng1+2*i)
            end do  ! i = 1, ncurvs

            call mshreadsqrcha ( iinput, iinputloc(istart), ipoint, &
                                 lenipoint, leng, lengin, isurnr, &
                                 newcurve, rinputloc, maxcrv )
            changed = changed .or. newcurve
            if ( ierror/=0 ) go to 1000

         end if  ! ( iinputloc(istart)==23 )

      end do  ! isurnr = 1, nsurfs

      if ( changed ) then

!     --- Curves have been changed, rearrange

         ncurvs = iinput(3)
         allocate ( iwork(lengin), stat = error )
         if ( error/=0 ) call eralloc ( error, lengin, 'iwork' )
         if ( ierror/=0 ) go to 1000

!        --- Copy rinputloc back to rinput

         rinput(lengrlus+1:lengrlus+ncurvs) = rinputloc(1:ncurvs,1)
         leng1 = lengrlus+ncurvs
         do i = 1, ncurvs
            rinput(leng1+2*i-1) = rinputloc(i,2)
            rinput(leng1+2*i) = rinputloc(i,3)
         end do  ! i = 1, ncurvs

         if ( ipoint(1,1)==1 ) then
            istart = 16
         else
            istart = 1
         end if  ! ( ipoint(1,1)==1 )
         iwork(1:lengin-istart+1) = iinput(istart:lengin)

         call msh014 ( iinput, iwork, ipoint, ncurvs, ndim )

         lengrl = lengrl+3*ncurvs

         if ( debug ) write(irefwr,1) 'lengrl, ncurvs', lengrl, ncurvs

         deallocate ( iwork, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iwork' )

      end if  ! ( changed )

1000  call erclos ( 'mshreadsurcha' )
      if ( debug ) then

!     --- Debug information

         call prinin ( iinputloc, leng, 'iinputloc' )
         call prinin ( iinput, lengin, 'iinput' )
         call prinin ( ipoint(1,1), ncurvs, 'ipoint/cur' )
         call prinin ( ipoint(1,2), nsurfs, 'ipoint/sur' )
         write(irefwr,*) 'End mshreadsurcha'

      end if  ! ( debug )

      end subroutine mshreadsurcha
