      subroutine mshremsh ( isol, iread, iinput, isubr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 11-12-2017 New storage of intmat
!        version  4.0  date 12-11-2009 intarmsh in common cmesh
!        version  3.2  date 20-04-2005 New call to intmshm2
!
!   copyright (c) 1996-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Performs remeshing and all associated actions
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleintmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), iread, iinput(*), isubr

!     iinput     i    integer input array with the following contents:
!                     1:  LASTIINP  Position of last entry that has been filled
!                     2:  IPRINT    Defines the amount of output produced
!                                   Possible values:
!                                   0:  No special information is printed
!                     3:  ISEQMESH  Sequence number of adapt mesh to be used
!                     4:  IVECTR    Sequence number of vector
!                     5:  INTERPOLATE_SOL: defines the number of solution
!                                   vectors that must be interpolated
!                     6:  INDEX_SOL: defines the starting address in
!                                   IINPUT of the vectors to be interpolated
!                                   In positions IINPUT(INDEX_SOL) to
!                                   IINPUT(INDEX_SOL+INTERPOLATE_SOL-1) the
!                                   sequence numbers of the solution vectors
!                                   to be interpolated are stored.
!                                   If INDEX_SOL = 0, the vectors 1 to
!                                   INTERPOLATE_SOL are interpolated
!                                   If INDEX_SOL<0, the vectors
!                                   -INDEX_SOL to -INDEX_SOL+INTERPOLATE_SOL-1
!                                   are interpolated
!                     7:  IVEC_MESHVEL Sequence number of vector in which
!                                   the mesh velocity must be stored
!                                   If 0, no mesh velocity is stored
!                     8:  ICHECK_BOUN indication of how the boundary must be
!                                   checked
!                                   Possible values:
!                               0   No checking of the boundary, except in the
!                                   case that general is applied
!                                   If general is applied all boundaries of the
!                                   surfaces generated by general are checked
!                                   with respect to the general requirements
!                               1   The first and last element of each curve
!                                   in array icurves is compared with the
!                                   original length stored in array kmesh p
!                                   If 1/a<dx/dx_orig<a the curve is
!                                   accepted, otherwise icheck is set equal to 1
!                                   dx is the actual length of the element
!                                   dx_orig is the original length of the
!                                   element a must be stored in rinput(1)
!                                   As soon as one of the curves returns with
!                                   icheck = 1, checking is stopped
!                     9:  ICHECK_MESH indication of how the internal elements
!                                   must be checked. This is only applied if the
!                                   boundary check returns with icheck = 0
!                                   Possible values:
!                               0   No checking of the internal elements
!                               1   All angles of the elements are checked
!                                   If the smallest angle of an element <
!                                   rinput(2) or the largest angle of an
!                                   element>rinput(3)
!                                   icheck is set equal to 2 and
!                                   checking is stopped
!                     In fact IINPUT is the local array iinput as used in
!                     INSTFREE or ADAPMESH, hence not all positions make sense
!     iread      i    Defines how the parameters must be read
!                     Possible values:
!                     0   The input is read from the standard input file
!                     >0  The input as read from the standard input file by
!                         subroutine SEPSTM is used. The value of iread
!                         defines the sequence number
!     isol      i/o   Standard SEPRAN array containing information of the
!                     solution array at the present iteration
!     isubr      i    Indicates calling subroutine.
!                     Possible values:
!                     1:  INSTFREE
!                     2:  ADAPMESH
!                     3:  ADAPMESH with complete new mesh
!                         Standard update of coordinates is not possible
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isol2(5), iseqmsv, &
              indexsol, i, iseqlc, iinmaplc(2), ichoicemesh, &
              icurvs(1), iseqprob
      double precision rinput(5)
      logical interums
      integer, pointer :: kprob2, kmesh2
      type (mapping) :: maplc
      save isol2

!     i              Counting variable
!     ichoicemesh    Choice parameter for subroutine mesh
!     icurvs         Dummy
!     iinmaplc       Information array with respect to subroutine intpolms
!     indexsol       See iinput(6)
!     interums       If true the mesh velocity must be interpolated
!     iseqlc         Sequence number of vector in ISOL
!     iseqmsv        Sequence number of mesh velocity vector with respect to
!                    isol
!     iseqprob       Sequence number of problem input
!     isol2          Interpolated solution
!     kmesh2         Contains copy of old mesh
!     kprob2         Contains copy of old problem
!     maplc          Mapping array with respect to subroutine intpolms
!     rinput         Real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INCOMGEN       Compute matrix structure for free boundary problem
!     MSHCOPY        Copy important information of mesh
!     PRININ         print 1d integer array
!     PROBDFEXT      Reads and stores problem definition
!     PROCOPY        Copy important information of problem definition
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPINTPOLMS    Interpolate solution from one mesh to another
!     SEPMESHSUB     General mesh generator
!     SEPRMMESHINFO  Remove mesh information
!     SEPRMIBUF      delete temporary array from ibuffr
!     SEPRMRBUF      Remove permanent space in array BUFFER
! **********************************************************************
!
!                       I/O
!
!   Depending on the values in IREAD input may be read from the standard
!   input file by subroutine READFRST
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    If ( mesh_velocity required ) then
!       Store old co-ordinates
!    Adapt mesh
!    If ( mesh_convection required ) then
!       compute mesh velocity
!    If ( mesh or boundary must be checked ) then
!       Check the quality of the mesh
!    If ( remeshing required ) then
!       remesh and interpolate if necessary
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data isol2/5*0/
! ======================================================================
!
      call eropen ( 'mshremsh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshremsh'
         write(irefwr,1) 'iread, isubr, meshnr', iread, isubr, meshnr
         call prinin ( iinput, 7, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kprob2 => kprob2d(maxmesh+1)
      kmesh2 => kmesh2d(maxmesh+1)
      call sepclear ( maplc )

!     --- Fill arrays iinput and rinput

      ichoicemesh = 10
      if ( isubr==1 ) then
         iseqmsv = mod(iinput(7),1000)
      else
         iseqmsv = 0
         if ( isubr==3 ) ichoicemesh = 12
      end if

      if ( debug ) write(irefwr,1) 'iseqmsv, iinput(5)',iseqmsv, iinput(5)
      if ( iseqmsv>0 .or. iinput(5)>0 ) then

!     --- Solution must be interpolated
!         Copy mesh and problem

         call mshcopy ( kmesh, kmesh2 )
         call procopy ( kprob, kprob2 )

      end if
      if ( ierror/=0 ) go to 1000
      call sepmeshsub ( ichoicemesh, iinput, rinput )
      if ( ierror/=0 ) go to 1000
      iseqprob = 1

      call probdfext ( 3, iseqprob )
      if ( ierror/=0 ) go to 1000
      if ( iseqmsv>0 ) then
         interums = .true.
      else
         interums = .false.
      end if
      iinmaplc(1) = 2
      iinmaplc(2) = 1
      if ( iinput(5)>0 ) then

!     --- Solution must be interpolated

         indexsol = iinput(6)
         do i = 1, iinput(5)
            if ( indexsol==0 ) then
               iseqlc = i
            else if ( indexsol<0 ) then
               iseqlc = i-indexsol-1
            else
               iseqlc = iinput(i+indexsol-1)
            end if
            if ( iseqlc==iseqmsv ) interums = .false.
            call sepintpolms ( maxmesh+1, meshnr, iinmaplc, &
                               isol(iseqlc), isol2, maplc, icurvs )
            iinmaplc(2) = 2
            if ( ierror/=0 ) go to 1000
            call sepcopyvc ( isol2, isol(iseqlc) )
            if ( ierror/=0 ) go to 1000

         end do

      end if
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'interums ', interums
      if ( interums ) then

!     --- The mesh velocity must be interpolated

         call sepintpolms ( maxmesh+1, meshnr, iinmaplc, &
                            isol(iseqmsv), isol2, maplc, icurvs )
         if ( ierror/=0 ) go to 1000
         call sepcopyvc ( isol2, isol(iseqmsv) )
         if ( ierror/=0 ) go to 1000
      end if

      if ( iseqmsv>0 .or. iinput(5)>0 ) then

!     --- Solution must be interpolated
!         delete second mesh and problem

         call seprmmeshinfo ( kmesh2 )
         do i = 1, lastsol

            if ( solarr(i)>0 ) then
               if ( ks(i)%iseqkp==kprob2 ) ks(i)%iseqkp = kprob2d(meshnr)
            end if  ! ( solarr(i)>0 )

         end do  ! i = 1, lastsol
         call seprmprobinfo ( kprob2 )

      end if

      kmesh => kmesh2d(meshnr)
      kprob => kprob2d(meshnr)

      if ( iseqmsv>0 .or. iinput(5)>0 ) then

!     --- Problem has changed, mark as changed

         iseqkp = kprob
         kprobpart => kp(iseqkp)%kprobsub(1)
         kprobloc => kprobpart%kprobloc
         kprobloc(80) = 0  !changed

      end if  ! ( iseqmsv>0 .or. iinput(5)>0 )

      call sepfillkmesh(kmesh)

      call incomgen ( iread, intmat1d )
      if ( associated(maplc%imap) ) deallocate(maplc%imap, maplc%rmap)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshremsh'

      end if  ! ( debug )
      call erclos ( 'mshremsh' )

      end subroutine mshremsh
