      subroutine mshz16 ( a, b, mrec, iQ, qnew, &
         mpoint, npoint, Coor )
!
! **********************************************************************
!
!       programmer      Zdenek Vlasek
!       version 1.0     date 14-02-1990
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                  PURPOSE
!
!   To create one new meshpoint determined by its relative position
!   [a,b] in the  l o c a l  quadrilateral iQ.
!
!   Use the "blending" interpolation on the quadrilateral iQ, &
!   the relative position of the new point is given by [a,b], 0<=a,b<=1, &
!   the edges 1, 2, 3, 4 correspond to  b=0, a=1, b=1, a=0

!   If  qnew = .true.   then the  iQ  is new for this subroutine.
!   If  qnew = .false.  then the  iQ  is the same as in the
!                       previous call and some saved values can help
!                       to speed up.
!
!   Results: updated  npoint and Coor
!
! **********************************************************************
!
!                  INPUT / OUTPUT  PARAMETERS

      logical qnew
      integer mrec, iQ ( mrec ), mpoint, npoint
      double precision a, b, Coor ( 2, mpoint )

!     a, b     i    Reference coordinates of the point
!     Coor     io   Real array (2,mpoint).
!                   Physical coordinates of all meshpoints
!     iQ       i    Integer array of length mrec. This record describes
!                   boundary of the quadrilateral region ( see mshz21 ).
!                   It is needed during the initialization only, for
!                   nonnegative npoint it is ignored
!     mpoint   i    Upper bound for npoint
!     mrec     i    Upper bound for length of iQ
!     npoint   io   Current number of all meshpoints
!     qnew     i    .true.   The record  iQ  is new for this subroutine.
!                   .false.  The record  iQ  is the same as in the
!                            previous call
!
! **********************************************************************
!
!                  LOCAL VARIABLES

!--- saved local, they keep information on the original quadrilateral
!                 and on the last found boundary intervals
!                 ( usually the next [a,b] is close to previous [a,b], &
!                   which helps to accelerate the search )

      integer ib1, ib2, ib3, ib4, ie1, ie2, ie3, ie4
      save    ib1, ib2, ib3, ib4, ie1, ie2, ie3, ie4

      double precision x00, x10, x11, x01, y00, y10, y11, y01
      save             x00, x10, x11, x01, y00, y10, y11, y01

!--- local, not saved

      integer k
      double precision a1, b1, xi0, xi1, xj0, xj1, yi0, yi1, yj0, yj1

!     a1, b1      1 - a  resp.  1 - b
!     ib1, ib2, ib3, ib4   Pointers into iQ for the beginning
!                          of the s/e/n/west edge of iQ
!     ie1, ie2, ie3, ie4   Pointers into iQ for the end
!                          of the s/e/n/west edge of iQ
!     k           A global nodenumber
!     x00, y00    Physical coordinates corresponding to ref.coord. [0,0]
!     x01, y01    Physical coordinates corresponding to ref.coord. [0,1]
!     x11, y11    Physical coordinates corresponding to ref.coord. [1,1]
!     x10, y10    Physical coordinates corresponding to ref.coord. [1,0]
!     xi0, yi0    Physical coordinates corresponding to [a,0]
!     xi1, yi1    Physical coordinates corresponding to [a,1]
!     xj0, yj0    Physical coordinates corresponding to [0,b]
!     xj1, yj1    Physical coordinates corresponding to [1,b]
!
! **********************************************************************
!
!                  SUBROUTINES CALLED
!
!     mshz04   To display an error message and to stop the run
!     mshz17   Local (using the current boundary iQ ) interpolation
!
! **********************************************************************
!
!                  ERROR MESSAGES are self-explaining
!
! **********************************************************************
!
!                  METHOD
!
!     Interpolation by blending. Boundary points of iQ are mapped onto
!     the boundary of the reference rectangle  e q u i d i s t a n l y.
!
!     reference plane: a,b           physical plane: z=x+i*y
!     -------------------- ----------------------- &
!
!      0,1-----a,1-----1,1           z(k01)---zi1------z(k11)
!       |               |             |                 |
!      0,b     a,b     1,b           zj0      zab=?    zj1
!       |               |             |                 |
!      0,0-----a,0-----1,0           z(k00)---zi0------z(k10)
!
!     The sum of linear interpolation in a-direction and b-direction, &
!     minus the bilinear interpolation defined by mapping of corners
!
!     x(a,b) = combination of ( 1, a, b, a*a, a*b, b*b )
!     y(a,b) = combination of ( 1, a, b, a*a, a*b, b*b )

!     The coefficients depend   n o n l i n e a r i l y   on a,b
!         ( nonlinearity comes via  zi0, zi1, zj0, zj1 )
!
!     The interpolation is exact on the boundary and in the interior
!     is as smooth as the boundary mapping.
!
!     Algorithm:  a1 = 1 - a
!                 b1 = 1 - b
!
!                 zab = + z(j0) * a1
!                       + z(j1) * a
!                       + z(i0) * b1
!                       + z(i1) * b
!                       - ( z(k00) * b1 + z(k01) * b ) * a1
!                       - ( z(k10) * b1 + z(k11) * b ) * a
!
! **********************************************************************
!

      if ( qnew ) then

!       * pointers into  iQ  for begs and ends of edges

         ib1 = iQ ( 6 )
         ib2 = iQ ( 7 )
         ib3 = iQ ( 8 )
         ib4 = iQ ( 9 )

         ie1 = iQ ( 10 )
         ie2 = iQ ( 11 )
         ie3 = iQ ( 12 )
         ie4 = iQ ( 13 )

!       * pointers into  Coor  for corners

         k = iQ ( ib1 )
         x00 = Coor ( 1, k )
         y00 = Coor ( 2, k )

         k = iQ ( ib2 )
         x10 = Coor ( 1, k )
         y10 = Coor ( 2, k )

         k = iQ ( ib3 )
         x11 = Coor ( 1, k )
         y11 = Coor ( 2, k )

         k = iQ ( ib4 )
         x01 = Coor ( 1, k )
         y01 = Coor ( 2, k )

      end if

      a1 = 1d0 - a
!    * the equidistant linear interpolation on edge 1 and 3
      call mshz17 ( a,  ib1, ie1, mrec, iQ, npoint, Coor, xi0, yi0 )
      call mshz17 ( a1, ib3, ie3, mrec, iQ, npoint, Coor, xi1, yi1 )

      b1 = 1d0 - b
!    * the equidistant linear interpolation on edge 4 and 2
      call mshz17 ( b1, ib4, ie4, mrec, iQ, npoint, Coor, xj0, yj0 )
      call mshz17 ( b,  ib2, ie2, mrec, iQ, npoint, Coor, xj1, yj1 )

!--- blending, create one new point

      if ( npoint.ge.mpoint ) then
         call mshz04 ( 'mshz16: mpoint overflow' )
      else
         npoint = npoint + 1
      end if

      Coor ( 1, npoint ) = &
           xj0 * a1  +  xj1 * a  +  xi0 * b1  +  xi1 * b &
         - ( x00 * b1  +  x01 * b ) * a1 &
         - ( x10 * b1  +  x11 * b ) * a

      Coor ( 2, npoint ) = &
           yj0 * a1  +  yj1 * a  +  yi0 * b1  +  yi1 * b &
         - ( y00 * b1  +  y01 * b ) * a1 &
         - ( y10 * b1  +  y11 * b ) * a

      end
