      subroutine msh055sort
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 27-11-2012 Long integers
!        version  1.1  date 24-11-2004 Fortran 90
!        version  1.0  date 29-03-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Sort the outer boundary of compound curves in the case that this
!     outer boundary contains holes
!     The ordering is such that the outer normal is always directed in
!     the same way
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     sort
!     surface
!     compound_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurf, istart, idirec, jcurnr, &
              firstpnt, isurnr, icurnr, lastpnt, &
              nouter, nfirst, nextcurve, i, iwork(ncurvs)

!     firstpnt       First point of a curve
!     i              Counting variable
!     icurnr         Curve sequence number
!     idirec         Defines in which direction the curves must be
!                    considered
!                    idirec = 1, from low to high, idirec = -1: reversed
!                    order
!     istart         First position used at a part
!     isurf          Type of surface with respect to mesh generator
!     isurnr         Surface sequence number
!     iwork          integer work array of length ncurvs
!                    Is used to store the curves of a surface temporary
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     lastpnt        last point of a curve
!     nextcurve      Next curve sequence number to be used
!     nfirst         First sequence number of new set
!     nouter         Number of points outside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHDIRCRVSRF   Compute the direction of the outer boundary of a surface by
!                    checking the direction of the first two nodes on the
!                    boundary
!     MSHFIRLPNTCRV  Find first and last point of a curve using the information
!                    stored in kmeshm
!     PR0012         Print accumulated array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2334   Part of outer boundary not closed
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all surfaces isurnr do
!        if ( surface = compound and outer boundary has a hole ) then
!           idirection := ordering of first curve in outer boundary
!           nouter := number of outer boundaries
!           nfirst := number of outer boundaries in first set
!           if ( idirection<0 ) then
!              reverse direction first set
!           While ( there are still outer boundaries left ) do
!              Find last curve in next set
!              idirection := ordering of first curve in next set
!              if ( idirection<0 ) then
!                 reverse direction next set
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh055sort' )
      debug = .false.

!     --- For all surfaces isurnr do

      do isurnr = 1, nsurfs

         isurf = kmeshs2(1,isurnr)
         if ( isurf==24 ) isurf = 7    ! compound surface
         if ( isurf==7 .and. kmeshs2(5,isurnr)>0 ) then

!        --- surface = compound and outer boundary has a hole
!            idirection := ordering of first curve in outer boundary

            istart = kmeshs(isurnr)
            icurnr = kmeshs(istart+1)
            call mshdircrvsrf ( firstpnt, idirec, isurnr, icurnr )

!           --- nouter := number of outer boundaries
!               nfirst := number of outer boundaries in first set

            nouter = kmeshs2(3,isurnr)
            nfirst = kmeshs2(5,isurnr)

            if ( debug )  then

!           --- debugging statements

               write(irefwr,*) 'isurnr, isurf, icurnr, idirec ', &
                                isurnr, isurf, icurnr, idirec
               write(irefwr,*) 'nouter, nfirst ',nouter, nfirst

            end if

            if ( idirec<0 ) then

!           --- idirection<0
!               reverse direction first set

               do i = 1, nfirst
                  kmeshs(istart+i) = -kmeshs(istart+i)
               end do

            end if  ! idirection<0

            nextcurve = nfirst+1

!           --- While ( there are still outer boundaries left ) do

            do while ( nextcurve<=nouter )

!           --- Find last curve in next set

               icurnr = kmeshs(istart+nextcurve)

               if ( debug )  then

!              --- debugging statements

                  write(irefwr,*) 'nouter, nfirst, nextcurve, icurnr ', &
                                   nouter, nfirst, nextcurve, icurnr

               end if

               call mshfirlpntcrv ( icurnr, kmeshm, firstpnt, lastpnt )
               nfirst = nfirst+1
               do while ( firstpnt/=lastpnt .and. nfirst<nouter )
                  nfirst = nfirst+1
                  jcurnr = kmeshs(istart+nfirst)
                  call mshfirlpntcrv ( jcurnr, kmeshm, nextpnt, &
                                       lastpnt )
               end do
               if ( firstpnt/=lastpnt ) then

!              --- error, not a closed boundary found

                  call errint ( isurnr, 1 )
                  call errint ( icurnr, 2 )
                  call errsub ( 2334, 2, 0, 0 )

               end if

!              --- idirection := ordering of first curve in next set

               call mshdircrvsrf ( firstpnt, idirec, isurnr, icurnr )

               if ( debug )  then

!              --- debugging statements

                  write(irefwr,*) 'icurnr, idirec, nextcurve, nfirst ', &
                                   icurnr, idirec, nextcurve, nfirst

               end if

               if ( idirec<0 ) then

!              --- idirection<0
!                  reverse direction first set

                  do i = nextcurve, nfirst
                     iwork(nfirst-i+1) = -kmeshs(istart+i)
                  end do
                  do i = nextcurve, nfirst
                     kmeshs(istart+i) = iwork(i+1-nextcurve)
                  end do

               end if  ! idirection<0

               nextcurve = nfirst+1

            end do  ! nextcurve<=nouter

         end if   ! icompound surface

      end do   ! isurnr = 1, nsurfs

      if ( debug ) then

!     --- debug = true, perform debugging

         write(irefwr,*) 'Curves corresponding to surfaces'
         call pr0012 ( nsurfs, kmeshs, kmeshs )
         call prinin1 ( kmeshs(nsurfs+2), nsurfs, 8, &
                        'Rest of information per surface' )

      end if


      call erclos ( 'msh055sort' )
      end
