      subroutine msh056 ( inpada, rinada, &
                          isol, iinput, rinput, icurves, ncall, icall, &
                          changed, changeinput, numvecs, iseqvecs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  9.2  date 15-11-2012 Long integers
!        version  9.1  date 02-10-2012 Use mshobsflperm
!        version  9.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1993-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Body of subroutines ADAPBN and CHANBN
!     Driver subroutine for adaptation of boundaries
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulework
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpada(*), isol(*), iinput(*), &
              icurves(*), ncall, icall, numvecs, iseqvecs(*)
      double precision rinada(*), rinput(*)
      logical changed, changeinput

!     changed        o    If true the number of nodes along the boundary has
!                         been changed
!     changeinput    i    If true the input arrays iinput and rinput have been
!                         changed
!     icall          i    Calling sequence number. Once icall=ncall it is reset
!     icurves        i    Integer array containing the curve numbers that
!                         must be adapted. It is assumed that these curves form
!                         one continuous curve.
!     iinput         i    integer input array for subroutine mesh
!                         iinput must be filled exactly in the same way
!                         as for mesh
!     inpada         i    Integer array to be filled by the user, indicating how
!                         the boundary must be adapted.
!                         INPADA is only used if iread = -1
!                         INPADA must be filled as follows:
!                         Pos. 1   Position of last entry that has been filled
!                                  by the user. If inpada(1) = 0 or 1, only
!                                  defaults are used.
!                         Pos. 2   ncurves, indicates the number of curves
!                                  in array icurves that must be used for the
!                                  adaptation of the mesh
!                                  Default value: 1
!                         Pos. 3   Not yet used. Must be set equal to 0
!                                  Default value: 0
!                         Pos. 4   iadapt Indicates how the boundary must be
!                                  adapted
!                                  Possible values:
!                                  1.  The curves are adapted by:
!                                      xnew(i) = xold(i) + alpha(i) n(i)
!                                      with xold(i) the old co-ordinates of the
!                                      i-th node and xnew(i) the new ones.
!                                      n(i) is the normal at the boundary of the
!                                      i-th node and alpha(i) the number-th
!                                      degree of freedom in this node
!                                      The normal is computed assuming a counter
!                                      clockwise direction of the boundary
!                                  2.  The curves are adapted by the user
!                                      provided function subroutine FUNCCR
!                                  3.  See 1, however, now alpha is computed
!                                      according to the 'film method'
!                                      See: Caswell and Viriyayuthakorn, &
!                                      Finite element simulation of die swell
!                                      for a Maxwell fluid, J. of Non-Newt.
!                                      Fluid Mech. Vol 12, 13-29(1983).
!                                      alpha(i) = factor*(psi(i)-psi(0))/|vt(i)|
!                                                       xi
!                                      psi(i)-psi(0) = /  v.n dx
!                                                     x0
!                                      The vector isol must be the velocity
!                                      vector with degrees of freedom number
!                                      and number+1 from isol.
!                                      Cartesian coordinates.
!                                      Starting flow rate integration along free
!                                      boundary with flow rate zero in first
!                                      point of curves.
!                                  4.  Other method according to:
!                                      xnew(i) = xold(i) + v(i) * factor
!                                      Time-dependent case if imult=1 see
!                                      inpada(11)
!                                      In the axisymmetric case we use:
!                                      (r,z)new(i) = (r,z)old(i) + &
!                                                     (vr,vz)(i) dt /(2 pi r(i))
!                                  5.  See 1. Now alpha is computed according to
!                                      alpha=v.n * factor
!                                  6.  See 1. Now alpha is computed according to
!                                      alpha=(grad c).n * factor
!                                      It is assumed that grad c is stored in
!                                      a vector of special structure defined per
!                                      element
!                                  Default value: 1
!                         Pos. 5   idirec Indicates the direction of the
!                                  computation
!                                  If idirec = 1 the computation is performed
!                                  from first to last point, &
!                                  if idirec = -1 from last to first point
!                                  It is sometimes required to alternate between
!                                  1 and -1 during iteration.
!                                  This option is only used for iadapt = 3
!                                  Default value: 1
!                         Pos. 6   number, the number-th degree of freedom of
!                                  isol (and higher) are used for the
!                                  computation of alpha. See iadapt.
!                                  Default value: 1
!                         Pos. 7   linear. Indicates if the elements along the
!                                  boundary must be treated as linear or as
!                                  quadratic elements.
!                                  Possible values:
!                                  1: linear
!                                  2: quadratic
!                                  Default value: to be computed by the
!                                  subroutine in the following way:
!                                  If all internal elements are quadratic, the
!                                  boundary elements are assumed quadratic, &
!                                  otherwise linear
!                         Pos. 8   jcart defines the type of co-ordinate system
!                                  to be used. This is of importance for
!                                  iadapt = 3, 4 and 5
!                                  Possible values:
!                                  1:  Cartesian co-ordinates
!                                  2:  Axi-symmetric co-ordinates
!                                  Default value: 1
!                         Pos. 9   iexlude_1: indicates which of the
!                                  co-ordinates of the first point of the first
!                                  curve must be adapted.
!                                  Possible values:
!                                  0  Both co-ordinates must be adapted
!                                  1  Only the second co-ordinate must be
!                                     adapted, the first one remains unchanged
!                                  2  Only the first co-ordinate must be adapted
!                                     the second one remains unchanged
!                                  3  Both co-ordinates remain unchanged
!                                  Default value: 0
!                         Pos. 10  iexlude_2: indicates which of the
!                                  co-ordinates of the last point of the last
!                                  curve must be adapted.
!                                  Possible values:
!                                  0  Both co-ordinates must be adapted
!                                  1  Only the second co-ordinate must be
!                                     adapted, the first one remains unchanged
!                                  2  Only the first co-ordinate must be adapted
!                                     the second one remains unchanged
!                                  3  Both co-ordinates remain unchanged
!                                  Default value: 0
!                         Pos. 11  imult: indicates how the value of factor must
!                                  be computed. (iadapt>2)
!                                  Possible values:
!                                  0  Multiplication factor is 1
!                                  1  Multiplication factor is dt from common
!                                     ctime
!                                  2  Multiplication factor is stored in
!                                     rinada(1)
!                                  3  Multiplication factor is dt * rinada(1)
!                                  Default value: 0
!                         Pos. 12  ithres: indicates if a threshold value is
!                                  used in Caswell's method and where to find it
!                                  Possible values:
!                                  0  Threshold value = 0
!                                  1  Threshold value is stored in rinada(2)
!                                  Default value: 0
!                         Pos. 13  iangle_1: Indicates if angle of freesurface
!                                  at first point is prescribed or not
!                                  Possible values:
!                                  0  angle is not given
!                                  1  angle is stored in rinada(3)
!                                  Default value: 0
!                         Pos. 14  iangle_2: Indicates if angle of freesurface
!                                  at last point is prescribed or not
!                                  Possible values:
!                                  0  angle is not given
!                                  1  angle is stored in rinada(4)
!                                  Default value: 0
!                         Pos. 15  redistribute nodes
!                                  Consists of two parts
!                                  iredist_type and ncurves_redist according to:
!                                  iredist = iredist_type + 100*ncurves_redist
!                                  Possible values:
!                                  iredist_type:
!                                  0  no node redistribution
!                                  1  nodes are redistributed, nelm constant
!                                  2  nodes are redistributed, nelm recomputed
!                                     according to coarseness
!                                  ncurves_redist: number of curves that
!                                  must be redistributed
!                                  The curve numbers are stored in icurvs
!                                  positions ncurves+1 to ncurves+ncurves_redist
!                                  This option is only available in combination
!                                  with iredist = 1
!                                  Default value: 0
!                         Pos. 16  Plot_boun
!                                  Possible values:
!                                  0  do not plot new boundary
!                                  1  plot new boundary
!                                  Default value: 0
!                         Pos. 17  Numobst (Number of obstacles)
!                                  Each obstacle must be closed and consist of
!                                  one curve only
!                                  (This may be a curve of curves)
!                                  At most 10 obstacles may be used.
!                                  The curve numbers of the Numobst obstacles
!                                  are stored in array icurvs positions:
!                                  ncurves+ncurves_redist+1 ... .+Numobst
!                                  Default value: 0
!     iseqvecs       i    Contains sequence numbers of vectors with respect to
!                         isol
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     ncall          i    Number of subsequence calls to msh056
!                         Once the number of calls is equal to ncall the
!                         subroutine is finished and the correct number of calls
!                         is returned
!     numvecs        i    Number of vectors in iseqvecs
!     rinada         i    Contains real information concerning the adaptation of
!                         the boundary. See array inpada
!                         1:  multiplication factor
!                         2:  threshold value
!                         3:  angle_1
!                         4:  angle_2
!                         5:  alpha for splines
!     rinput         i    Real input array for subroutine mesh
!                         rinput must be filled exactly in the same way as
!                         for mesh
!                         At output new co-ordinates may have been filled
!                         in rinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl, checksurobst, obstfound, closed
      integer number, iadapt, lengcv, &
              ncurvsl, mcurves, ncurves, lencurvs, leniwork1, &
              i, lenicrvob, iobst, istarcur, &
              lengin, lengrl, nspecialpnts, &
              ivectr, istore2d(4,10), jcount, nbefor, maxphys, n, &
              mmcurvefacts, leniobgrid1
      integer, allocatable :: iobgrid(:), iobgrid1(:), icurvscp(:), &
                              icurvsl(:), ispinp(:), iellist(:), &
                              newcurves(:), curvinfo(:), specialpnt(:)
      double precision, allocatable :: anorm(:), curstr(:)

!     anorm          In this array the normal vectors are stored
!     checksurobst   If true there are surfaces connected to the obstacle
!     closed         If true the boundary is closed
!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     curstr         work array of length 10 * ncurvs  to store some
!                    geometrical information of the curves
!     curvinfo       Array of length ncurvs containing information about
!                    which curve is on a free surface and also
!                    part of an obstacle
!                    curvinfo(i) = 0: standard curve
!                    curvinfo(i) = iobst(>0): curve on free surface and
!                        also new part of obstacle iobst . The curve is not
!                        of type >= 10000
!                    curvinfo(i) = -1: curve on obstacle that is made
!                       active
!     i              Counting variable
!     iadapt         See inpada(4)
!     icurvscp       Copy of array icurvs
!     icurvsl        Array of length ncurvs that is filled with all
!                    stand alone curves that must be adapted in the
!                    correct sequence and provided with a sign
!     iellist        Array of length npoint-1 containing a
!                    list of elements that are adjacent to the boundary
!                    element iellist(i) contains both nodal points
!                    iwork(i) and iwork(i+1)
!     iobgrid        position i,j contains:
!                    -1:  cell inside obstacle
!                     0:  cell outside obstacle
!                    >0:  cell contains part of the obstacle
!                    The value contains a starting address in iobgrid1
!     iobgrid1       Must be used in connection with iobgrid
!                    iobgrid contains the starting addresses for each cell
!                    i,j
!                    From this starting address iobgrid1 contains
!                    the number of curve parts of the obstacle
!                    intersecting the cell followed by the sequence
!                    numbers of these curve parts
!     iobst          Obstacle sequence number
!     ipvect         Starting address of VECTOR in array BUFFER
!     ispinp         Array of length 2 + maximal number of points along
!                    one curve to be used as work array
!     istarcur       Starting address in array curves
!     istore2d       Array containing information over the old solution vectors
!                    Contents
!                    1,i:  indprf
!                    2,i:  indprh
!                    3,i:  indprp
!                    4,i:  nphys
!     ivectr         sequence number of first vector referred to in iseqvecs
!     jcount         Counter
!     lencurvs       Overestimate of the number of points along the new curves
!     lengcv         Overestimate of the number of points along the curves
!     lengin         Length of integer array iinput
!     lengrl         Length of real array rinput
!                    (in double precision variables)
!     lenicrvob      Length of array iobscurv
!     leniobgrid1    Length of array iobgrid1
!     leniwork1      Length of array iwork1
!     maxphys        Maximum number of unknowns per point
!     mcurves        Number of curves that must be redistributed
!     mmcurvefacts   memory management reference number of array CURVEFACTS
!     n              number of nodes on boundary
!     nbefor         number of oiginal nodes in boundary
!     ncurves        Number of curves that ust be adpated
!     ncurvsl        Number of stand-alone curves that must be adapted
!     newcurves      Contains information about the new curves to be
!                    created by splitting old curves
!                    The starting addresses of the information per curve is
!                    stored in array istnewcurves
!                    For each curve that is stored in newcurves nparts+1
!                    positions are used, where nparts is the number of
!                    parts the curve is subdivided into
!                    The first position contains nparts, the following
!                    nparts positions the relative starting addresses
!                    of the parts in the curve
!     nspecialpnts   Number of special points
!     number         The number-th unknown in each nodal point of the array
!                    is considered. If the array is complex number=2*i-1
!                    corresponds to the real part of array i and number=2*i
!                    to the imaginary part of array i
!     obstfound      If true an obstacle that is also part of a surface
!                    has become active
!     specialpnt     This array of size 3 x nspecialpnts contains
!                     information about the special points on the curve
!                     specialpnt(1,i) contains the position in curves
!                                       of special point i
!                     specialpnt(2,i) contains first curve number in the
!                                       free surface connected to point i
!                     specialpnt(3,i) contains second curve number in the
!                                       free surface connected to point i
!                                       If 0, there is only one curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILLVC       Copy the contents of array vector into array uoldcp for all
!                    points in array nodes and all degrees of freedom
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH047         recompute co-ordinates of curves
!     MSH047BN       Fill nodal point numbers and normals of boundary
!     MSH048         Adapt curves of which at least one user point has been
!                    changed
!     MSH065         Split compound curves into single curves
!     MSH066         Redistribute nodes along curves
!     MSH066A        Mark free curves that may not be redistributed
!     MSHCHECKOBST   Check if an obstacle that is also part of a surface
!                    has been made active
!     MSHCPCOM       Fill array curves in case of composite curves by copying
!                    from the single curves it consists of
!     MSHCURFR       Compute new arrays icurvs and curves in case the number of
!                    nodes at the boundary may be changed
!     MSHOBSFLPERM   Store array iobgrid as permanent array in ibuffr
!     MSHOBST        Restrict new boundary to region outside the obstacles
!     MSHOBSTACT     Detect which actions must be undertaken with respect to
!                    subdivision of curves and recreation of new surfaces in
!                    case of obstacles that have become active
!     MSHOBSTIINP    Create new array iinput and rinput with respect to free
!                    surface intersecting an obstacle
!     MSHSEQCURVS    Compute sequence in which curves must be created
!     PRINCURV       Print coordinates of curves
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPOBSFL       Fill array iobgrid with grid information of one obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      59   array ivectr corresponds to a complex vector
!     102   NUMBER has wrong value
!    1393   Not constant number of degrees of freedom not available in intmsh.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh056' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh056'
  1      format ( a, 1x, (10i6) )

      end if

      iadapt = inpada(4)
      number = inpada(6)
      if ( number==0 ) number = 1
      jcoars = iinput(11)
      ivectr = iseqvecs(1)

      iinp1 = 16
      changed = .false.
      changeinput = .false.

!     --- activate solution arrays

      maxphys = 0
      do i = 1, numvecs
         call sepactsolbf2 ( isol(iseqvecs(i)) )
         maxphys = max(maxphys,nphysi)
      end do

!     --- Create kmesh part z

      if ( icall==1 ) then
         if ( associated(km(iseqkm)%kmeshz) ) deallocate(km(iseqkm)%kmeshz)
         allocate(km(iseqkm)%kmeshz(ncurvs))
         kelmz = ncurvs
         kmeshz => km(iseqkm)%kmeshz
         kmeshz = 0
      end if

!     --- initialize part of the arrays

      if ( kelmd==0 ) call sepfillkmeshd

      if ( debug ) write(irefwr,1) 'inpada(17)', inpada(17)
      if ( inpada(17)>0 ) then

!     --- activate or create array icurvsobs if obstacles are present

         if ( kmeshh(1,1)==0 ) then
            lenicrvob = size ( curvesmsh,2 )
            if ( associated(km(iseqkm)%icurvsobst2d) ) &
               deallocate(km(iseqkm)%icurvsobst2d)
            allocate(km(iseqkm)%icurvsobst2d(2,2*lenicrvob))
            icurvsobst2d => km(iseqkm)%icurvsobst2d
            icurvsobst2d = 0
            kmeshh(1,1) = 2*lenicrvob
         end if
         if ( kmeshh(2,1)==0 ) then
            if ( associated(km(iseqkm)%iactobst) ) &
               deallocate(km(iseqkm)%iactobst)
            allocate(km(iseqkm)%iactobst(numobst))
            kmeshh(2,1) = numobst
            iactobst => km(iseqkm)%iactobst
            iactobst = 0
         end if
         if ( kmeshh(4,1)==0 ) then
            if ( associated(km(iseqkm)%iobscurv) ) &
               deallocate(km(iseqkm)%iobscurv)
            allocate(km(iseqkm)%iobscurv(ncurvs))
            iobscurv => km(iseqkm)%iobscurv
            iobscurv = 0
            kmeshh(4,1) = ncurvs
         end if
         lencurvs = size ( curvesmsh,2 )

!        --- Check if array iobgrid has been filled,
!            if not fill it.

         do iobst = 1, numobst

            if ( kmeshx(5,iobst)==0 ) then

!           --- Array has not yet been filled

               nx = kmeshx(3,iobst)
               ny = kmeshx(4,iobst)

               leniobgrid1 = 2*nx*ny
               allocate ( iobgrid(nx*ny), iobgrid1(leniobgrid1), &
                          stat = error )
               if ( error/=0 ) &
                  call eralloc ( error, nx*ny+leniobgrid1, 'iobgrid' )
               if ( ierror/=0 ) go to 1000

               call sepobsfl ( iobgrid, nx, ny, iobst, &
                               lencurvs, iobgrid1, leniobgrid1 )

!              --- Make array iobgrid permanent

               call mshobsflperm ( iobgrid, iobgrid1, &
                                   nx, ny, leniobgrid1, iobst )

               deallocate ( iobgrid, iobgrid1, stat = error )
               if ( error/=0 ) call erdealloc ( error, 'iobgridcop' )

            end if

         end do

      end if  ! ( inpada(17)>0 )
      call sepactsolbf2 ( isol(ivectr) )
      if ( number<1 .or. number>nunkpi ) then
         call errint ( number, 1 )
         call errint ( nunkpi, 2 )
         call errsub ( 102, 2, 0, 0)
      end if
      if ( iadapt>2 .and. iadapt<9 .and. number>nunkpi-1 ) then
         call errint ( number, 1 )
         call errint ( nunkpi-1, 2 )
         call errsub ( 102, 2, 0, 0)
      end if
      compl = ks(isol(ivectr))%compl==1
      if ( compl ) &
          call errsub ( 59, 0, 0, 0 )
      if ( ierror/=0 ) go to 1000

!     --- Reserve temporay space
!         At the end this array is deleted

      lengcv = (kmeshm(3)-kmeshm(2))
      lencurvs = size ( curvesmsh,2 )*5

      if ( debug ) &
         write(irefwr,1) 'icall, lengcv, lencurvs', icall, lengcv, lencurvs
      if ( icall==1 ) then

!     --- First call in a row, create array icurvar and clear this array

!        --- Detect sequence of curves and store in array iseqcurvs

         allocate ( ipointcurv(ncurvs), iseqcurvs(ncurvs), &
                    icurvar(ncurvs), stat = error )
         if ( error/=0 ) call eralloc ( error, ncurvs, 'ipointcurv' )
         if ( ierror/=0 ) go to 1000

         alloccurv = .true.

         call mshseqcurvs ( iinput )
         icurvar = 0

      end if

!     --- The following temporary arrays are created:
!          name     pointer    length
!         iworkcv   ipworkcv   ncurvs
!         ispinp    ipspinp    2*lengcv
!         iellist   ipielist   ncurvs+1+ncurvs*nuspnt
!         Temporary arrays to be created as permanent and deleted later on
!         coorol

      leniwork1 = max(npoint,nelem,lengcv)+ncurvs
      allocate ( anorm(2*lengcv), iwork1(leniwork1), icurvsl(ncurvs), &
                 ispinp(2*lengcv+ncurvs), coorol(2*lengcv), &
                 iellist(ncurvs+1+ncurvs*nuspnt), &
                 uold(maxphys*numvecs*lencurvs*ndim), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 2*lengcv+leniwork1+lencurvs+&
                        maxphys*numvecs*lencurvs*ndim , 'anorm' )
      if ( ierror/=0 ) go to 1000

!     --- Permanent arrays

      vectin => ks(isol(ivectr))%sol

      if ( ierror/=0 ) go to 1000

!     --- Fill single curves and check if curves are used only once

      call msh065 ( icurves, inpada(2), icurvar, &
                    icurvsl, ncurvsl, .false. )

!     --- Fill information with respect to boundary

      call msh047bn ( inpada, kprobfi, indprfi, ispinp, &
                      coorol, anorm, &
                      icurvsl, iwork1, ncurvsl, iellist, &
                      closed, nbefor, jcount, n )

      if ( debug ) write(irefwr,1) 'inpada(4)', inpada(4)
      if ( inpada(4)==9 ) then

!     --- Put old solution vectors in uoldcp

         do i = 1, numvecs
            vectin2 => ks(isol(iseqvecs(i)))%sol
            call sepactsolbf2 ( isol(iseqvecs(i)) )
            call erfillvc ( vectin2, ispinp, n, &
                            uold(1+(i-1)*n*maxphys) )
         end do

      end if
      call sepactsolbf2 ( isol(ivectr) )

!     --- Update boundary

      call msh047 ( inpada, rinada, nunkpi, kprobfi, indprfi, &
                    vectin, ispinp, lengcv, coorol, anorm, &
                    icurvsl, iwork1, rinput(3), &
                    ncurvsl, iellist, closed, nbefor, &
                    jcount, n, uold, maxphys, numvecs )

      if ( ierror/=0 ) go to 1000

      deallocate ( anorm, iwork1, iellist, uold, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'anorm' )
      if ( ierror/=0 ) go to 1000

      allocate ( curstr(10*lengcv), stat = error )
      if ( error/=0 ) call eralloc ( error, 10*lengcv, 'curstr' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'inpada(15)', inpada(15)
      if ( inpada(15)>0 ) then

!     --- inpada(15)>0, nodes must be redistributed

         mcurves = inpada(15)/100
         if ( mcurves>0 ) then

!        --- a restricted number of curves may be redistributed
!            Refill array icurvsl in this case

            ncurves = inpada(2)
            call msh065 ( icurves(ncurves+1), mcurves, icurvar, &
                          icurvsl, ncurvsl, .true. )

         end if
         call msh066 ( icurvsl, ncurvsl, inpada, iinput, &
                       rinput(3), ispinp, curstr, rinada, icurvar )

      else

!     --- Mark present curves as not to be redistributed

         call msh066a ( icurvsl, ncurvsl )

      end if

      deallocate ( icurvsl, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'icurvsl' )
      if ( ierror/=0 ) go to 1000

      if ( inpada(17)>0 ) then

!     --- inpada(17)>0, obstacles are defined
!         Restrict update to obstacle

         istarcur = inpada(2)+inpada(15)/100
         checksurobst = .false.
         do i = 1, inpada(17)
            iobst = icurves(istarcur+i)

!           --- Mark obstacle as possibly active

            iactobst(iobst) = 1
            nx       = kmeshx(3,iobst)
            ny       = kmeshx(4,iobst)
            if ( kmeshx(6,iobst)/=0 ) checksurobst = .true.
            iobgrid2d => kmobst(iobst)%iobgrid2d
            iobgrid2d1 => kmobst(iobst)%iobgrid2d1
            call mshobst ( rinput(3), nx, ny, iobst, lengcv )
         end do

      end if
      if ( ierror/=0 ) go to 1000

      if ( icall==ncall ) then

!     --- Last call
!         Update curves and rinput for all curves containing
!         user points that have been changed

         call msh048 ( inpada, iinput, rinput(3), &
                       curstr, lengcv, icurvar, ispinp )

         if ( kelmq/=0 ) then

!        --- kelmq/=0, hence there are composite curves
!                         copy coordinates of subcurves

            call mshcpcom ( curveusrp, .false. )
            if ( debug ) then

!           --- Debug information

               call princurv ( 0, icurvsmsh, curvesmsh, &
                               ncurvs, ndim, curveusrp, &
                               'curves after mshcpcom' )

            end if

         end if

         if ( mod(inpada(15),100)==2 ) then

!        --- Nodes may be redistributed, recompute icurvs and curves
!            First copy old array curves to coorol and icurvs to icurvscp
!            icurvscp is extended with an extra position containing a 0

            if ( associated(curvefacts) ) nullify(curvefacts)
            allocate ( icurvscp(ncurvs+1), curvefacts(lencurvs), &
                       stat = error )
            if ( error/=0 ) call eralloc ( error, ncurvs, 'icurvscp' )
            if ( ierror/=0 ) go to 1000

            lengrl = 16*lencurvs*ndim + 10*ncurvs
            icurvsobst2d = 0
            call dcopy ( lencurvs/5*ndim, curvesmsh, 1, coorol, 1 )

            nullify(curvesmsh)
            allocate ( curvesmsh(ndim,lencurvs) )
            icurvscp(2:1+ncurvs) = icurvsmsh
            icurvscp(1) = 0
            call mshcurfr ( icurvscp, coorol, &
                            lencurvs, iinput, rinput, changed )

!           --- Copy new arrays curves to old ones

            if ( associated(km(iseqkm)%curvesmsh) ) &
               deallocate(km(iseqkm)%curvesmsh)
            allocate(km(iseqkm)%curvesmsh(ndim,lencurvs))
            km(iseqkm)%curvesmsh = curvesmsh(:,1:lencurvs)
            deallocate(curvesmsh)
            curvesmsh => km(iseqkm)%curvesmsh

!           --- Copy new arrays curvefacts to old ones

            if ( associated(km(iseqkm)%curvefacts) ) &
               deallocate(km(iseqkm)%curvefacts)
            allocate(km(iseqkm)%curvefacts(lencurvs))
            km(iseqkm)%curvefacts = curvefacts(1:lencurvs)

            deallocate ( icurvscp, curvefacts, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'icurvscp' )
            if ( ierror/=0 ) go to 1000

            curvefacts => km(iseqkm)%curvefacts

            if ( inpada(17)>0 ) then

!           --- inpada(17)>0, obstacles are defined
!               Restrict update to obstacle

               length = lencurvs/2
               if ( associated(km(iseqkm)%icurvsobst2d) ) &
                  deallocate(km(iseqkm)%icurvsobst2d)
               allocate(km(iseqkm)%icurvsobst2d(2,length))
               icurvsobst2d => km(iseqkm)%icurvsobst2d
               icurvsobst2d = 0
               kmeshh(1,1) = length
               istarcur = inpada(2)+inpada(15)/100

               do iobst = 1, numobst
                  if ( iactobst(iobst)>0 ) then

!                 --- Active obstacle

                     nx       = kmeshx(3,iobst)
                     ny       = kmeshx(4,iobst)
                     iobgrid2d => kmobst(iobst)%iobgrid2d
                     iobgrid2d1 => kmobst(iobst)%iobgrid2d1
                     call mshobst ( rinput(3), nx, ny, iobst, lengcv )

                  end if

               end do

            end if

         end if
         if ( ichangeobst>0 .and. checksurobst ) then

!        --- ichangeobst>0, hence obstacle points have been changed

            allocate ( curvinfo(ncurvs) )

!           --- Check if there are points on the obstacle that do not
!               correspond to a curve of type 10000, but are also part of
!               a surface
!               If that is the case set obstfound = true which means that
!               new curves and surfaces must be created

            call mshcheckobst ( obstfound, iinput, &
                                ipointcurv, curvinfo )
            if ( obstfound ) then

!           --- Obstacle found that is made active
!               Create new user points, curves and surfaces

               lencurvs = icurvsmsh(ncurvs)
               allocate ( newcurves(2*lencurvs+ncurvs), &
                          specialpnt(7*lencurvs) )
               call mshobstact ( nspecialpnts, specialpnt, &
                                 newcurves, nextracrvs, &
                                 curvinfo, iinput, ipointcurv )
               call mshobstiinp ( iinput, rinput, nextracrvs, &
                                  specialpnt, curvinfo, newcurves, &
                                  nspecialpnts )
               changeinput = .true.
               changed = .true.
               deallocate(newcurves, specialpnt)
            end if
            deallocate ( curvinfo )


         end if

!        --- reset icall and clear curvar

         deallocate ( ipointcurv, iseqcurvs, icurvar, coorol, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'ipointcurv' )
         if ( ierror/=0 ) go to 1000

         alloccurv = .false.

      end if

      deallocate ( curstr, ispinp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'curstr' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) write(irefwr,*) 'End msh056'
      call erclos ( 'msh056' )

      end subroutine msh056
