      subroutine mshanalvolcur ( iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-11-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if surface surrounding general is closed by checking
!     the number of curves in the surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     volume
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)

!     iinput         i    integer input array, standard for subroutine MESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, isur, ndefsf, ncur, jstart, icur, kstart, iinp
      integer, allocatable :: iwork(:)

!     icur           Curve loop number
!     iinp           position in array iinput
!     istart         First position used at a part
!     isur           surface loop number
!     iwork          integer work array of length ncurvs
!                    iwork(i) contains the number of times a curve is
!                    present in the surrounding surface
!     jstart         starting position in kmeshs for curves in isurnr
!     kstart         starting position in kmeshs corresponding to isurnr
!     ncur           Number of curves surrounding a surface
!     ndefsf         Number of curves in surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH050         Raise iinp as function of ivolm
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3154   Number of surfaces surrounding general is not 1
!    3155   surfaces surrounding general is not closed
! **********************************************************************
!
!                       PSEUDO CODE
!
!     iwork(1:ncurvs) := 0
!     For all volumes do
!        if ( volume==general ) then
!           For all subsurfaces surrounding the surface do
!              For all curves icurnr surrounding the subsurface do
!                 iwork(|icurnr|) = iwork(|icurnr|)+1
!           Check iwork (all entries must be 2 or 0)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshanalvolcur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshanalvolcur'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'ncurvs, nsurfs, nvolms, iinp5', &
                          ncurvs, nsurfs, nvolms, iinp5

      allocate ( iwork(ncurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, ncurvs, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- For all volumes do

      iinp = iinp5
      do ivolnr = 1, nvolms

         ivol = iinput(iinp)  ! type of volume

         if ( debug ) write(irefwr,1) 'ivolnr, ivol', ivolnr, ivol

         if ( ivol==5 .or. ivol==10 ) then

!        --- volume==general

            ndefsf = iinput(iinp+2) ! number of surfaces that defines the boun

            if ( ndefsf/=1 ) then

!           --- error number of surfaces is not equal to 1

               call errint ( ndefsf, 1 )
               call errsub ( 3154, 1, 0, 0 )

            end if  ! ( ndefsf/=1 )

            isurnr = iinput(iinp+3) ! surface number
            iwork = 0

            if ( kelmr>0 ) then
               ndefsf = kmeshr(isurnr+1)-kmeshr(isurnr)
               istart = kmeshr(isurnr)
            else
               ndefsf = 1
               istart = 0
            end if

            if ( debug ) write(irefwr,1) 'ndefsf', ndefsf

            if ( ivol==10 ) then

!           --- There are internal surfaces, remove from the list of surfaces

               nextrapnts = iinput(iinp+4)
               nextracrvs = iinput(iinp+5)
               nextrasrfs = iinput(iinp+6)
               jstart = iinp+6+nextrapnts+nextracrvs

               if ( debug ) then
                  write(irefwr,1) 'nextrapnts, curves, surfaces', &
                             nextrapnts, nextracrvs, nextrasrfs
                  call prinin ( iinput(jstart+1), nextrasrfs, &
                                'extra surfaces' )
               end if  ! ( debug )

               do isur = 1, nextrasrfs
                  isurnr = iinput(jstart+isur)
                  ndefsf = ndefsf- &
                     max(1,(kmeshr(isurnr+1)-kmeshr(isurnr)))
               end do  ! isur = 1, nextrasrfs

            end if  ! ( ivol==10 )

            if ( debug ) &
               write(irefwr,1) 'ndefsf, isurnr, istart', &
                                ndefsf, isurnr, istart

!           --- For all subsurfaces surrounding the surface do

            do isur = 1, ndefsf

               if ( istart>0 ) isurnr = kmeshr(istart+isur)

               jstart = kmeshs(isurnr)  ! start of curves -1
               kstart = nsurfs+1+8*(isurnr-1)
               ncur = kmeshs(isurnr+1)-kmeshs(isurnr)  ! number of curves
               isurf = kmeshs(kstart+1)
               if ( isurf==7 .or. isurf==11 ) ncur = kmeshs(kstart+3)

               if ( debug ) then
                  write(irefwr,1) 'isur, isurnr, ncur, isurf', &
                                   isur, isurnr, ncur, isurf
                  call prinin ( kmeshs(jstart+1), ncur, 'curves' )
               end if  ! ( debug )

!              --- For all curves icurnr surrounding the subsurface do
!                    iwork(|icurnr|) = iwork(|icurnr|)+1

               do icur = 1, ncur

                  icurnr = abs(kmeshs(jstart+icur))
                  iwork(icurnr) = iwork(icurnr)+1

               end do  ! icur = 1, ncur

            end do  ! isur = 1, ndefsf

!           --- Check iwork (all entries must be 2 or 0)

            do icurnr = 1, ncurvs

               if ( iwork(icurnr)/=0 .and. iwork(icurnr)/=2 ) then

!              --- curve is in surface but not twice

                  call errint ( ivolnr, 1 )
                  call errint ( icurnr, 2 )
                  call errint ( iwork(icurnr), 3 )
                  call errsub ( 3155, 3, 0, 0 )

               end if  ! ( iwork(icurnr)/=0 .and. iwork(icurnr)/=2 )

            end do  ! i = 1, ncurvs

         end if  ! ( ivol==5 .or. ivol>=10 )

         call msh050 ( iinput, iinp, ivol )  ! raise iinp5

      end do  ! ivolnr = 1, nvolms

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshanalvolcur'

      end if  ! ( debug )
      call erclos ( 'mshanalvolcur' )

      end
