      subroutine mshcloselines ( x1, x2, x3, x4, threshold, distance )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the minimum distance between the edges x1-x2 and x3-x4, &
!     provided this distance is less than threshold
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     distance
!     line
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x1(2), x2(2), x3(2), x4(2), threshold, distance

!     distance       o    distance between the edges x1-x2 and x3-x4
!     threshold      i    Threshold value
!     x1             i    Contains the coordinates of the point x1
!     x2             i    Contains the coordinates of the point x2
!     x3             i    Contains the coordinates of the point x3
!     x4             i    Contains the coordinates of the point x4
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision len, xmin1, xmin2, ymin1, ymin2, xmax1, xmax2, &
                       ymax1, ymax2, n1(2), n2(2), xh1(2), xh2(2), &
                       fact1, fact2

!     fact1          multiplication factor
!     fact2          multiplication factor
!     len            length of vector
!     n1             normal on edge 1 (x1-x2)
!     n2             normal on edge 2 (x3-x4)
!     xh1            help array to store coordinates
!     xh2            help array to store coordinates
!     xmax1          maximum x-coordinate of first edge
!     xmax2          maximum x-coordinate of second edge
!     xmin1          maximum y-coordinate of first edge
!     xmin2          maximum y-coordinate of second edge
!     ymax1          maximum x-coordinate of third edge
!     ymax2          maximum x-coordinate of fourth edge
!     ymin1          maximum y-coordinate of third edge
!     ymin2          maximum y-coordinate of fourth edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSHCROSSLINE1  Find the intersection of the lines x1-x2 and x3-x4
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     distance := infinity
!     xmax1, ymin1 := minimum coordinates of edge x1-x2
!     xmin1, ymax1 := maximum coordinates of edge x1-x2
!     xmin2, ymin2 := minimum coordinates of edge x3-x4
!     xmin2, ymax2 := maximum coordinates of edge x3-x4
!     if ( rectangles around edges have distance more than threshold ) then
!        distance := infinity
!     n1 := normal on edge 1 (x1-x2);   n2 := normal on edge 2 (x3-x4)
!     if ( line (x3-threshold*n, x3+threshold*n ) intersects edge 1  ) then
!        distance := minimum ( distance, distance of x3 to edge 1 )
!     if ( line (x4-threshold*n, x4+threshold*n ) intersects edge 1  ) then
!        distance := minimum ( distance, distance of x4 to edge 1 )
!     if ( line (x1-threshold*n, x1+threshold*n ) intersects edge 2  ) then
!        distance := minimum ( distance, distance of x1 to edge 2 )
!     if ( line (x2-threshold*n, x2+threshold*n ) intersects edge 2  ) then
!        distance := minimum ( distance, distance of x2 to edge 2 )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- distance := infinity
!         xmax1, ymin1 := minimum coordinates of edge x1-x2
!         xmin1, ymax1 := maximum coordinates of edge x1-x2
!         xmin2, ymin2 := minimum coordinates of edge x3-x4
!         xmin2, ymax2 := maximum coordinates of edge x3-x4

      distance = rinfin
      xmin1 = min(x1(1),x2(1))
      xmax1 = max(x1(1),x2(1))
      ymin1 = min(x1(2),x2(2))
      ymax1 = max(x1(2),x2(2))
      xmin2 = min(x3(1),x4(1))
      xmax2 = max(x3(1),x4(1))
      ymin2 = min(x3(2),x4(2))
      ymax2 = max(x3(2),x4(2))

!     --- if ( rectangles around edges have distance more than threshold ) then
!            distance := infinity

      if ( xmax1.lt.xmin2-threshold .or. xmin1.gt.xmax2+threshold .or. &
           ymax1.lt.ymin2-threshold .or. ymin1.gt.ymax2+threshold ) &
         go to 1000

!     --- n1 := normal on edge 1 (x1-x2);   n2 := normal on edge 2 (x3-x4)

      n1(1) = x2(2)-x1(2)
      n1(2) = -(x2(1)-x1(1))
      len = 1d0/sqrt(n1(1)**2+n1(2)**2)
      n1(1) = len*n1(1)
      n1(2) = len*n1(2)

      n2(1) = x4(2)-x3(2)
      n2(2) = -(x4(1)-x3(1))
      len = 1d0/sqrt(n2(1)**2+n2(2)**2)
      n2(1) = len*n2(1)
      n2(2) = len*n2(2)

!     --- if ( line (x3-threshold*n, x3+threshold*n ) intersects edge 1  ) then
!            distance := minimum ( distance, distance of x3 to edge 1 )

      xh1(1) = x3(1)-threshold*n1(1)
      xh1(2) = x3(2)-threshold*n1(2)
      xh2(1) = x3(1)+threshold*n1(1)
      xh2(2) = x3(2)+threshold*n1(2)
      call mshcrossline1 ( x1, x2, xh1, xh2, fact1, fact2, sqreps )
      if ( fact2.gt.-sqreps .and. fact2.lt.1d0+sqreps ) then

!     --- intersection found, compute distance

         distance = min(distance,abs(fact1-0.5d0)*2d0*threshold)

      end if

!     --- if ( line (x4-threshold*n, x4+threshold*n ) intersects edge 1  ) then
!            distance := minimum ( distance, distance of x4 to edge 1 )

      xh1(1) = x4(1)-threshold*n1(1)
      xh1(2) = x4(2)-threshold*n1(2)
      xh2(1) = x4(1)+threshold*n1(1)
      xh2(2) = x4(2)+threshold*n1(2)
      call mshcrossline1 ( x1, x2, xh1, xh2, fact1, fact2, sqreps )
      if ( fact2.gt.-sqreps .and. fact2.lt.1d0+sqreps ) then

!     --- intersection found, compute distance

         distance = min(distance,abs(fact1-0.5d0)*2d0*threshold)

      end if

!     --- if ( line (x1-threshold*n, x1+threshold*n ) intersects edge 2  ) then
!            distance := minimum ( distance, distance of x1 to edge 2 )

      xh1(1) = x1(1)-threshold*n2(1)
      xh1(2) = x1(2)-threshold*n2(2)
      xh2(1) = x1(1)+threshold*n2(1)
      xh2(2) = x1(2)+threshold*n2(2)
      call mshcrossline1 ( x3, x4, xh1, xh2, fact1, fact2, sqreps )
      if ( fact2.gt.-sqreps .and. fact2.lt.1d0+sqreps ) then

!     --- intersection found, compute distance

         distance = min(distance,abs(fact1-0.5d0)*2d0*threshold)

      end if

!     --- if ( line (x2-threshold*n, x2+threshold*n ) intersects edge 2  ) then
!            distance := minimum ( distance, distance of x2 to edge 2 )

      xh1(1) = x2(1)-threshold*n2(1)
      xh1(2) = x2(2)-threshold*n2(2)
      xh2(1) = x2(1)+threshold*n2(1)
      xh2(2) = x2(2)+threshold*n2(2)
      call mshcrossline1 ( x3, x4, xh1, xh2, fact1, fact2, sqreps )
      if ( fact2.gt.-sqreps .and. fact2.lt.1d0+sqreps ) then

!     --- intersection found, compute distance

         distance = min(distance,abs(fact1-0.5d0)*2d0*threshold)

      end if

1000  end
