      subroutine mshcoons3drect ( bcord, kbndpt, irand, bcor2, &
                                  jnew, nipnt, jdiag, xtemp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 02-09-2010 Remove intarmsh
!        version  2.1  date 27-10-2003 New call to msher1
!        version  2.0  date 15-10-2003 Remove coor2 from parameter list
!
!   copyright (c) 2002-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a surface-mesh in 3-D space. Surface is formed by four lines
!     given in BCORD. Use the COONS method and MSHER1.
!     This is the special case that opposite boundaries have the same number
!     of nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     coons
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision bcord(3,*), bcor2(2,*), &
                       xtemp(3)
      integer kbndpt(*), irand, nipnt, jdiag
      logical jnew

!     bcor2          i    Contains local 2d coordinates of boundary
!     bcord          i    array containing the coordinates of the boundaries
!     irand          i    number of points on the side of an element - 1
!     jdiag          i    If 1 all diagonals in the rectangles are directed in
!                         one direction if rectangle is called, if 2 they are
!                         ordered in alternating directions
!     jnew           i    indication whether the mesh is new (true) or old
!     kbndpt         o    array containing the nodal point numbers of the
!                         boundary
!     nipnt          i    number of boundary points
!     xtemp          i    temporary array to store nodes of end point of
!                         boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, i, l, k, itim, jd, n1, n2, n3, n4, &
              ne1, ne2, ne3, ne4, iextra
      double precision xte, yte, zte, coef(8), u, &
                       w, x(8), y(8), z(8), coor2(2,npointlc)

!     coef           coefficients for Coons formula
!     coor2          Array to store the 2d coordinates of the mesh
!     i              Counting variable
!     iextra         Number of extra points per row due to centroids of
!                    triangles
!     itim           help variable to store itime temporarily
!     j              Counting variable
!     jd             nodal point number
!     k              Counting variable
!     l              General loop variable
!     n1             number of points along first curve
!     n2             number of points along second curve
!     n3             number of points along third curve
!     n4             number of points along fourth curve
!     ne1            number of elements on the first boundary
!     ne2            number of elements on the second boundary
!     ne3            number of elements on the third boundary
!     ne4            number of elements on the fourth boundary
!     u              local line parameter in first direction
!     w              local line parameter in second direction
!     x              x-coordinates
!     xte            temporary x-coordinate
!     y              y-coordinates
!     yte            temporary y-coordinate
!     z              z-coordinates
!     zte            temporary z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHD7          Places coorindates of centroids of 7-point triangles
!                    in array coor
!     MSHER1         Submesh generator RECTANGLE
!     MSHO52         Compute coordinates of specified node
!     MSHO53         Compute coordinates of 3d point related to 2d
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcoons3drect' )
      if ( ierror/=0 ) go to 1000

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcoons3drect'
         write(irefwr,*) 'isurnr, inpelm ', isurnr, inpelm
         write(irefwr,*) 'irand, mtrian, jdiag ', irand, mtrian, jdiag
         write(irefwr,*) 'xtemp ', xtemp
         write(irefwr,*) 'nipnt ', nipnt

      end if

      itim  = itime
      itime = 0
      ne1 = nx
      ne2 = ny
      ne3 = nz
      ne4 = nc
      n1 = ne1 * irand + 1
      n2 = ne2 * irand + 1
      n3 = ne3 * irand + 1
      n4 = ne4 * irand + 1
      if ( debug ) &
         write(irefwr,*) 'ne1, ne2, ne3, ne4, n1, n2, n3, n4 ', &
                          ne1, ne2, ne3, ne4, n1, n2, n3, n4

      if ( inpelm==7 ) then

!     --- 7 point elements, extra nodes per row of elements

         iextra = ne1

      else

!     --- standard elements, no extra nodes per row of elements

         iextra = 0

      end if

!     --- Create 2d mesh

      call msher1( jnew, irand, kbndpt, bcor2, &
                   ne1, ne2, coor2, kmeshclc, jdiag )
      itime = itim

!     --- Compute npointlc and nelemlc

      npointlc = n1*n2 + iextra*ne2*2
      nelemlc  = mtrian * ne1 * ne2

      if ( debug ) then

!     --- Debugging statements

         call prinrl1 ( coor2, npointlc, 2, 'coor2 in coons' )
         call prinin1 ( kmeshclc, nelemlc, inpelm, 'kmeshclc' )
         call prinin ( kbndpt, nipnt, 'kbndpt' )

      end if

!     --- Map new internal points from 2d to 3-D

      do i = 1, npointlc
         coor2d(1,i) = coor2(1,i)
         coor2d(2,i) = coor2(2,i)
         coor2d(3,i) = 0d0
      end do

!     --- Determine four corner points in 3-D

      j = 1
      call msho52( bcord, j, x(1), y(1), z(1) )
      j = n1 - 1  + 1
      call msho52( bcord, j, x(3), y(3), z(3) )
      j = n1 + n2 - 2  + 1
      call msho52( bcord, j, x(5), y(5), z(5) )
      j = n1 + n2 + n3 - 3 + 1
      call msho52( bcord, j, x(7), y(7), z(7) )

      if ( debug ) then

!     --- Debugging statements

         write(irefwr,*) 'vertex 1: ', x(1), y(1), z(1)
         write(irefwr,*) 'vertex 2: ', x(3), y(3), z(3)
         write(irefwr,*) 'vertex 3: ', x(5), y(5), z(5)
         write(irefwr,*) 'vertex 4: ', x(7), y(7), z(7)

      end if

      do l = 2, n2-1
         do k = 2, n1-1
            i = (l-1)*n1 + k + (l-1)*iextra

!           --- New point

            u     = coor2d(1,i)
            coef(1) = 1d0 - u
            coef(2) =       u
            w     = coor2d(2,i)
            coef(3) = 1d0 - w
            coef(4) =       w
            coef(5) = coef(1) * coef(3)
            coef(6) = coef(1) * coef(4)
            coef(7) = coef(2) * coef(3)
            coef(8) = coef(2) * coef(4)

!           --- Determine coordinates on lines for points u and w

            jd = 1
            call msho53 ( bcord(1,jd), bcor2(1,jd), u, &
                          x(2), y(2), z(2), n1 )
            jd = n1
            call msho53 ( bcord(1,jd), bcor2(1,jd), w, &
                          x(4), y(4), z(4), n2 )
            jd = n1 + n2 - 1
            call msho53 ( bcord(1,jd), bcor2(1,jd), 1d0-u, &
                          x(6), y(6), z(6), n3 )
            jd = n1 + n2 + n3 - 2
            call msho53 ( bcord(1,jd), bcor2(1,jd), 1d0-w, &
                          x(8), y(8), z(8), n4 )
            xte =   coef(1) * x(8) + coef(2) * x(4) + coef(3) * x(2)+ &
                    coef(4) * x(6) &
                 -( coef(5) * x(1) + coef(6) * x(7) + coef(7) * x(3)+ &
                    coef(8) * x(5) )
            yte =   coef(1) * y(8) + coef(2) * y(4) + coef(3) * y(2)+ &
                    coef(4) * y(6) &
                 -( coef(5) * y(1) + coef(6) * y(7) + coef(7) * y(3)+ &
                    coef(8) * y(5) )
            zte =   coef(1) * z(8) + coef(2) * z(4) + coef(3) * z(2)+ &
                    coef(4) * z(6) &
                 -( coef(5) * z(1) + coef(6) * z(7) + coef(7) * z(3)+ &
                    coef(8) * z(5) )
            coor2d(1,i) = xte
            coor2d(2,i) = yte
            coor2d(3,i) = zte

         end do

      end do

!     --- Fill BCORD in in COOR

      do i = 1, nipnt
          j = kbndpt(i)
          coor2d(1,j) = bcord(1,i)
          coor2d(2,j) = bcord(2,i)
          coor2d(3,j) = bcord(3,i)
      end do

      if ( inpelm==7 ) then

!     --- inpelm = 7, fill centroid

         call mshd7 ( kmeshclc, coor2d, nelemlc, 3 )

      end if

      if ( debug ) then

!     --- Debug information

         call prinrl1 ( coor2d, npointlc, 3, 'coor3d' )

      end if

      j = nipnt+1
      bcord(1,j) = xtemp(1)
      bcord(2,j) = xtemp(2)
      bcord(3,j) = xtemp(3)

1000  if ( debug ) write(irefwr,*) 'End mshcoons3drect'
      call erclos ( 'mshcoons3drect' )

      end
