      subroutine mshcrosscurve ( x1, x2, xobst, fact1 )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 26-11-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Determine intersection of straight edge with curved
!       obstacle edge
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     intersection
!     obstacle
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision fact1, x1(2), x2(2), xobst(2,3)

!     fact1          i    Defines the factor for the intersection point
!                         The intersection point on the face is defined by
!                         x1 + fact1 * (x2-x1)
!     x1             i    Coordinates of mesh node 1
!     x2             i    Coordinates of mesh node 2
!     xobst          i    Coordinates of 3 obstacle nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  i, k, iter, ispecial
      double precision sum1, sum2, xi, phi(9), dphi(9), delta1, &
                       bet, alph

!     alph           Help constant
!     bet            Help constant
!     delta1         Computed residual
!     dphi           Array of length 9 to store the values of dphi/dxi
!                    according to:
!                               k
!                    dphi /dxi(x ) = dpdxi(i,k)
!                        i
!     i              Counting variable
!     ispecial       Indicates if line is vertical line
!                    0    Standard case
!                    1    Special case
!     iter           Iteration sequence number
!     k              Counting variable
!     phi            array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                    phi_i(xg^k) = phi (i,k)
!                    array phi is only filled when gauss integration is
!                    applied
!     sum1           Parameter to compute a sum
!     sum2           Parameter to compute a sum
!     xi             Local x coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcrosscurve' )

!     --- if x1=x2

      ispecial=0
      if ( abs(x1(1)-x2(1)).le.0d0 ) then
         ispecial=1
         bet=x1(1)
      else
         alph= (x2(2)-x1(2))/(x2(1)-x1(1))
         bet=x1(2)-alph*x1(1)
      end if

!     --- find local coordinates of intersection point using
!         a newton raphson iteration scheme

      xi=5d-1

      iter=0
      delta1=1d0
      do while ( abs(delta1).gt.1d-13 .and. iter.le.8 .and. &
                 abs(delta1).lt.1d1 )
         iter = iter+1

!        --- compute some functions for local coordinate xi

         phi(1) =  2d0*(xi-0.5d0)*(xi-1d0)
         phi(2) = -4d0*xi*(xi-1d0)
         phi(3) =  2d0*xi*(xi-0.5d0)
         dphi(1) =  4d0*xi-3d0
         dphi(2) = -8d0*xi+4d0
         dphi(3) =  4d0*xi-1d0

!        --- compute error delta1

         if ( ispecial.ne.1 ) then
            sum1=0d0
            sum2=0d0
            do k=1,3
               sum1 = sum1 +dphi(k)*( xobst(2,k)-alph*xobst(1,k) )
               sum2 = sum2 + phi(k)*( xobst(2,k)-alph*xobst(1,k) )
            end do
            delta1=-(sum2-bet)/sum1
         else
            sum1=0d0
            sum2=0d0
            do k=1,3
               sum1 = sum1 +dphi(k)* xobst(1,k)
               sum2 = sum2 + phi(k)* xobst(1,k)
            end do
            delta1=-(sum2-bet)/sum1
         end if

!        --- update local coordinate

            xi=xi+delta1

      end do

!     --- compute some functions for local coordinate xi

      phi(1) =  2d0*(xi-0.5d0)*(xi-1d0)
      phi(2) = -4d0*xi*(xi-1d0)
      phi(3) =  2d0*xi*(xi-0.5d0)

!     --- compute coordinates of intersection point

      sum1=0d0
      sum2=0d0
      do i=1,3
         sum1 = sum1 + phi(i)*xobst(1,i)
         sum2 = sum2 + phi(i)*xobst(2,i)
      end do

!     --- compute new fact

      if ( abs(x2(1)-x1(1)).gt.1d-13 ) then
         fact1=(sum1-x1(1))/(x2(1)-x1(1))
      else
         fact1=(sum2-x1(2))/(x2(2)-x1(2))
      end if

      call erclos ( 'mshcrosscurve' )
      end
