      subroutine mshcrtcurv ( ichois, iinput, rinput, activatm, jnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 11-12-2014 Adaptation due to new curve routines
!        version  2.2  date 27-11-2012 Long integers
!        version  2.1  date 30-07-2012 New treatment of iinput/rinput
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute all curves and store information in corresponding arrays
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, iinput(*)
      double precision rinput(*)
      logical activatm, jnew

!     activatm       o    If true kmesh part m must be activated
!     ichois         i    Choice parameter JCHOIS in subroutine MESH
!                         Possible values
!                          0  standard method,  input from standard input file
!                             the mesh is created for the first time
!                          1  input is read from the arrays iinput and rinput
!                             the mesh is created for the first time
!                          2  input is read from the standard input file and
!                              storedin the arrays iinput and rinput
!                             the mesh is created for the first time
!                          3  input is read from the arrays iinput and rinput
!                             the co-ordinates of the existing mesh are changed
!                             the topological description remains unchanged
!                          4  input is read from the arrays iinput and rinput.
!                             the coordinates and the topological description of
!                             the mesh are changed, however the input as stored
!                             in iinput and rinput is the same as in a preceding
!                             call of mesh.
!                             The number of points and curves remains the same,
!                             as well as the number of elements along the curves
!                             The coordinates of the user points and the curves
!                             are changed by a call of subroutine chanbn and as
!                             a consequence the number elements in the surfaces
!                             may be changed.
!                             This possibility must be preceded by calls of
!                             subroutine mesh and subroutine chanbn
!                          5  See jchois = 4, however, the topological
!                             description remains unchanged
!                          6  Only the arrays iinput and rinput are filled
!                             The subroutine is returned without actually
!                             creating a mesh
!                          7  The input must be stored in the arrays IINPUT and
!                             RINPUT
!                             The subroutine creates the curves and stores the
!                             information of curves and user points via KMESH
!                             After this step the subroutine returns
!                             This possibility is meant for program SEPGEOM
!                         10  The boundary of the mesh has been changed by
!                             subroutine ADAPBN. Information with respect to
!                             this change has been stored in the arrays ICURVS,
!                             CURVES,IINPUT and RINPUT, which are all part
!                             of array KMESH
!                             The input/output parameters IINPUT and RINPUT are
!                             not used.
!                             The topological description of the mesh may be
!                             changed
!                         11  See ICHOIS=10, however, the topological
!                             description may not be changed
!                         12  See ICHOIS=10, however, also the description of
!                             the surfaces has been changed
!     iinput         i    integer input array, standard for subroutine MESH
!                         The starting address in MESH is 1
!     jnew           i    indication whether the mesh is new (true) or old
!     rinput         i    Real input array, standard for subroutine MESH
!                         The starting address in MESH is 1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lencur, lenrel, istartcur, istartrel, idummy
      integer, allocatable :: irelcurves(:)
      double precision, allocatable :: curstr(:)

!     curstr         Contains real information of the curves
!                    The starting positions per curve are given in
!                    irelcurves(ncurvs+1:2*ncurvs)
!                    For each type of curve the first position contains the
!                    length of the curve
!                    The other positions depend in the type of curve (icurve)
!                    icurve = 5 and more than 2 user points:
!                                 Local length of each subcurve
!                    icurve = 6:  pos. 2-5: phi, r, xz, yz if ndim = 2 and
!                                 pos. 2-6: phi, r, xz, yz, zz if ndim = 3
!                    icurve =23:  pos. 2-5: phi, a, b, t0
!     idummy         Dummy
!     irelcurves     Contains integer information of the curves
!                    Positions 1:ncurvs contain the starting addresses of
!                    each curve in this array
!                    Positions: ncurvs+1:2*ncurvs contain the starting addresses of
!                    each curve in curstr
!                    The next positions depend on the type of curve (icurve)
!                    icurve = 5 and more than 2 user points:
!                    for each user point the relative position in the curve (-1)
!                    Hence the first position is 0
!                    icurve = 21 (frame curve)
!                    Relative node numbers of the user points
!     istartcur      starting position in curstr
!     istartrel      starting position in irelcurves
!     lencur         length of array CURSTR
!     lenrel         Length of array irelcurves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH002         Fill array ICURVS
!     MSH013         initialize kmesh part m
!     MSHCHECKCURVES Check if the single curves do not intersect
!     MSHCRV         Fill array CURVES
!     PLOTCURV       Plot curves
!     PRINCURV       Print coordinates of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     fill array icurvs;  compute length of array curves
!     Fill array ICURVS
!     fill array curves;  write curves to file 3
!     Fill all curves
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcrtcurv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcrtcurv'
         write(irefwr,1) 'ichois', ichois
         write(irefwr,*) 'jnew ', jnew
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinin ( iinput, iinp2, 'iinput' )

!     --- Allocate array irelcurves
!         This array has the same length as iinput until part 2
!         It is used by framecurv

      lencur = 10*ncurvs
      lenrel = iinp2
      allocate ( irelcurves(lenrel), curstr(lencur), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, lenrel+lencur, 'irelcurves' )
      if ( ierror/=0 ) go to 1000

      irelcurves = 0
      istartcur = 1
      istartrel = 2*ncurvs+1

!     --- fill array icurvs;  compute length of array curves
!         Array CURSTR is put at the end of idummy

      if ( debug ) write(irefwr,1) 'ncurvs', ncurvs

!     --- Fill array ICURVS
!         Compute length of arrays CURVES (lencrv)
!         These arrays are used by MSHCRV

      if ( ichois>=10 .and. ichois<=12 ) then

!     --- The array icurvs and curves have already been filled

         lencrv = size(curvesmsh,2)

      else if ( ichois<4 .or. ichois>5 ) then

!     --- standard case

         if ( associated(km(iseqkm)%icurvsmsh) ) &
            deallocate(km(iseqkm)%icurvsmsh)
         allocate ( km(iseqkm)%icurvsmsh(2*ncurvs) )
         icurvsmsh => km(iseqkm)%icurvsmsh
         call msh002 ( iinput, icurvsmsh, rinput(3), jnew, &
                       curstr, irelcurves, istartcur, istartrel )
         if ( debug ) write(irefwr,1) 'lencrv, ndim',lencrv, ndim

         if ( associated(km(iseqkm)%curvesmsh) ) &
            deallocate(km(iseqkm)%curvesmsh)
         allocate(km(iseqkm)%curvesmsh(ndim,lencrv))
         if ( associated(km(iseqkm)%curvefacts) ) &
            deallocate(km(iseqkm)%curvefacts)
         allocate (km(iseqkm)%curvefacts(lencrv) )
         curvesmsh => km(iseqkm)%curvesmsh
         curvefacts => km(iseqkm)%curvefacts
         curvefacts = 0d0
         kmeshh(3,1) = 2*ncurvs
         kmeshh(5,1) = lencrv
         lencrv = size(curvesmsh,2)

      end if
      if ( debug ) call prinin ( icurvsmsh, ncurvs, 'icurvs' )

      if ( jnew  ) then

!     --- jnew = true  reserve space for kmesh part m at the end of idummy

         if ( km(iseqkm)%kmesh(27)>0 ) deallocate(km(iseqkm)%kmeshm)
         allocate ( km(iseqkm)%kmeshm(lencrv+2*ncurvs+5), stat = error )
         if ( error/=0 ) call eralloc ( error, lencrv+2*ncurvs+5, 'kmeshm' )
         if ( ierror/=0 ) go to 1000
         kmeshm => km(iseqkm)%kmeshm
         kelmm = lencrv+2*ncurvs+5
         km(iseqkm)%kmesh(27) = kelmm

         activatm = .false.

      else if ( ichois==10 .or. ichois==12 ) then

         if ( km(iseqkm)%kmesh(27)>0 ) deallocate(km(iseqkm)%kmeshm)
         allocate ( km(iseqkm)%kmeshm(lencrv+2*ncurvs+5), stat = error )
         kmeshm => km(iseqkm)%kmeshm
         kelmm = lencrv+2*ncurvs+5
         activatm = .true.

      else

!     --- jnew = false,  old array kmeshm

         activatm = .true.

      end if
      if ( ierror/=0 ) go to 900

!     --- fill array curves;  write curves to file 3
!         fill curves in kmesh part m and h

      if ( ichois<=3 .or. ichois==7 ) then

!     --- ichois<= 3  call mshcrv
!         Length of temporary arrays:
!         iwork:  ncurvs+lencrv integers
!         ihelp:  lentau+4 integers

         call mshcrv ( jnew, iinput, rinput, curstr, irelcurves, istartrel )

!        --- This part used to be:
!            write co-ordinates of points and coarseness to file 3
!            Now we set kmeshh(1) = 0 and kmeshh(2) at old length
!            The information can be retrieved from rinput.

         kmeshh(4,1) = 0

      else

!     --- ichois = 4, 5, 10, 11 or 12 use previously computed values

         if ( ichois==4 .or. ichois==10 .or. ichois==12 ) then

!        --- ichois = 4, 10, 12  new mesh with old boundary
!                                Make jnew true and refill kmesh part m

            jnew = .true.
            call msh013 ( iinput, irelcurves )

         end if

      end if
      if ( ierror/=0 ) go to 900

      if ( debug ) then

!     --- Debug = true, Print curves

         call princurv ( 0, icurvsmsh, curvesmsh, ncurvs, &
                         ndim, idummy, 'curves' )

         if ( .false. ) &
            call plotcurv ( icurvsmsh, curvesmsh, ncurvs, &
                            'curves' )

      end if

!     --- Check if curves intersect (2d only)

      call mshcheckcurves ( ipointcurv, iinput )

900   deallocate ( irelcurves, curstr, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'irelcurves' )

1000  if ( debug ) write(irefwr,*) 'End mshcrtcurv'
      call erclos ( 'mshcrtcurv' )

      end subroutine mshcrtcurv
