      subroutine mshcursplin ( iinput, iinp, irnp, icurnr, rinput, &
                               icurve, ndim, nuspnt, jcoars, iwork, &
                               tau, ispinp, iposc, curves, alpha )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 12-01-2009 New call to mshc01
!        version  1.0  date 20-09-2002
!
!   copyright (c) 2002-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Redistribute the coordinates along a curve without changing the number
!     of nodes along that curve
!     The curve is approximated by a spline and the nodes are redistributed
!     based on the given coarseness in the two end points
!     This subroutine is meant for the situation that at least one of the
!     end points has been moved, for example in a free surface computation
! **********************************************************************
!
!                       KEYWORDS
!
!     approximation
!     curve
!     free_boundary
!     free_surface
!     adapt_boundary
!     spline
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, irnp, icurnr, icurve, ndim, nuspnt, &
              jcoars, iwork (*), ispinp(*), iposc
      double precision rinput(*), tau(*), curves(ndim,*), alpha

!     alpha          i    factor with respect to the creation of splines, &
!                         See the users manual (use 0.5 as default)
!     curves        i/o   array containing the coordinates of the curves
!                         At input the coordinates to be redistributed
!                         At output the new coordinates
!     icurnr         i    Curve sequence number of the curve to be
!                         redistributed
!     icurve         i    Type of curve
!     iinp           i    first position used in array iinput with respect to
!                         this curve
!     iinput         i    integer input array corresponding to the mesh
!                         generation , see PG
!     iposc          i    first position in array curves corresponding to
!                         curve icurnr
!     irnp           i    position in array rinput, defines start of coarseness
!                         factors
!     ispinp              Work array to be used by subroutine mshc01 as
!                         input array iinput
!                         Length n+2, where n is the number of nodes along
!                         the curve
!     iwork          i    integer work array of length ncurvs+1+ncurvs*nuspnt
!                         output of subroutine msh051
!                         iwork contains the user points of the curves in the
!                         following way:
!                         iwork(1)=ncurvs+1
!                         number of user points in curve i: iwork(i+1)-iwork(i)
!                         user points in curve i are stored in iwork from
!                         position iwork(i)+1 to iwork(i+1)
!     jcoars         i    If 1 coarseness is given, if 0 not
!     ndim           i    Dimension of the space.
!     nuspnt         i    Number of user points
!     rinput         i    Real input array corresponding to the mesh
!                         generation , see PG
!     tau                 work array of length at most length curves / 2 to
!                         store relative arc lengths along curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ishape, nelm, iratio, n, i
      double precision factor

!     factor         Multiplication factor
!     i              Counting variable
!     iratio         Ratio as defined in input
!     ishape         Shape number of elements along the curve
!     n              number of nodes along the curve
!     nelm           number of elements along a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH004         Fill array tau
!     MSHC01         Fill array curves with splines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get parameters from iinput
!     If ( curve has given coarseness at input ) then
!        get coarseness
!     Fill array tau with subdivision along curve
!     create spline
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcursplin' )

!     --- Get parameters from iinput

      ishape   = iinput(iinp+1)
      nelm     = iinput(iinp+2)
      iratio   = iinput(iinp+3)
      if ( nelm<=1 ) go to 1000

      factor = rinput(irnp+icurnr)
      if ( icurve==5 .or. icurve==6 .or. icurve==9 .or. &
           icurve==12 ) then

!     --- compute factor

         factor = (rinput(ndim*nuspnt+jcoars+ &
                       iwork(abs(iwork(icurnr+1))))/ &
                   rinput(ndim*nuspnt+jcoars+ &
                       iwork(abs(iwork(icurnr))+1)))**(1d0/nelm)
         iratio = 2

      end if

!     --- Fill array tau with subdivision along curve

      call msh004 ( tau, ishape, nelm, n, iratio, factor, 0 )

!     --- ipispl = itype=2, n, + [1..n]
!         rinput = curves

      ispinp(1) = 32
      ispinp(2) = n
      do i = 1, n
         ispinp(2+i) = i
      end do

!     --- create spline

      call mshc01 ( ispinp, curves(1,iposc), curves(1,iposc), &
                    ndim, n, tau, alpha, icurnr )

1000  call erclos ( 'mshcursplin' )
      end
