      subroutine mshcurves ( curvstosurfs, lncurvstosurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 27-11-2012 Long integers
!        version  3.0  date 26-01-2009 Remove mmsurffaces
!        version  2.1  date 21-03-2005 Fortran 90
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create all curves that can be seen as intersection of more surfaces
!     This subroutine is meant for user created meshes
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer curvstosurfs(*), lncurvstosurfs

!     curvstosurfs   i    Array of size nsurfs x nsurfs
!                         curvstosurfs(i,j) contains the j-th curve
!                         corresponding to surface i
!                         if 0 no more curves can be found
!     lncurvstosurfs i    Declared length of second dimension of array
!                         curvstosurfs
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenipoint, ncurvsnew, lenkmeshmtmp
      integer, pointer :: ipoint(:), kmeshmtmp(:)

!     ipoint         Array of length ncurvsnew containing the starting
!                    address of the next curve in kmeshmtmp
!                    So in ipoint(1) the number of nodes for the first new
!                    curve in kmeshmtmp plus one is stored etc.
!     kmeshmtmp      Temporary array kmeshm in which the nodes of the new
!                    curves are stored
!     lenipoint      Length of array ipoint
!     lenkmeshmtmp   Length of array kmeshmtmp
!     ncurvsnew      Last curve sequence number used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCURVES1     Create new curves as intersection of surfaces, provided no
!                    curve exists already as intersection
!     MSHCURVES4     Merge old array kmeshm with new part and create a new array
!                    kmeshm
!     PR0008         Print information of curves
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHL  Fill array nmesh part l
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate arrays
!     Create arrays in temporary space
!     Set starting addresses
!     Create new curves and compute number of new curves
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcurves' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurves'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      if ( kelmm>0 ) then

!     --- kmeshm has been created before

         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'Old kmesh part m'
            call pr0008 ( kmeshm, ncurvs )

         end if

      end if


!     --- The next two arrays are made semi-permanent,
!         they are removed later on
!         kmeshmtmp     npoint
!         ipoint        nsurfs^2

      lenipoint = max(nsurfs**2+1,1000)
      lenkmeshmtmp = npoint

!     --- Arrays concerning edges and faces

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(31)==0 ) call sepfillnmesha
      if ( nmesh(33)==0 ) call sepfillnmeshc
      if ( nmesh(42)==0 ) call sepfillnmeshl

      allocate ( kmeshmtmp(lenkmeshmtmp), ipoint(lenipoint), stat = error )
      if ( error/=0 ) call eralloc ( error, lenkmeshmtmp, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Create new curves and compute number of new curves

      call mshcurves1 ( kmeshmtmp, ncurvsnew, lenkmeshmtmp, curvstosurfs, &
                        ipoint, lenipoint, lncurvstosurfs )
      if ( ierror/=0 ) go to 1000

!     --- Update kmeshm
!         The old array kmeshm is copied to the back of ibuffr
!         The new array kmeshm overwrites the old one

      call mshcurves4 ( ncurvsnew, kmeshmtmp, ipoint, lenkmeshmtmp )

!     --- Remove temporary arrays

      deallocate ( kmeshmtmp, ipoint, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call pr0008 ( kmeshm, ncurvs )
         write(irefwr,*) 'End mshcurves'

      end if
      call erclos ( 'mshcurves' )

      end subroutine mshcurves
