      subroutine mshkmeshrdir ( kmeshr, kmeshs, ncurvs, nsurfs, &
                                kmeshsp2, isurnr, kmeshsp3 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.6  date 04-02-2015 Extra debug
!        version  2.5  date 28-01-2009 correction surface without boundary
!        version  2.4  date 09-01-2008 Debug information
!
!   copyright (c) 2002-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the direction of the boundary of the elements of each subsurface
!     of a compound surface and compare it with the direction of the outer
!     boundary.
!     If the directions are opposite, then the elements have a different
!     direction. The position in kmeshr is provided with a minus sign
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     boundary
!     direction
!     orientation
!     surface
!     compound_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshr(*), nsurfs, ncurvs, kmeshs(*), kmeshsp2(8,*), &
              isurnr, kmeshsp3(*)

!     kmeshr        i/o   Kmesh part r, see the PG 13.1
!     kmeshs         i    Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     kmeshsp2       i    Second part of kmeshs, referring to surfaces
!     kmeshsp3       i    Third part of kmeshs, containing curve numbers
!     ncurvs         i    Number of curves in mesh
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istartsf, nsubsf, nouter, istart, iwork(ncurvs), &
              i, icurnr, jcurnr, icount, j, isubsf, ksign, k, &
              iwork1(nsurfs), iwork2(ncurvs), jcount, &
              lenkmeshscp
      logical found, loop

!     found          If true a new curve has been found
!     i              Counting variable
!     icount         Counter
!     icurnr         Curve sequence number
!     istart         First position used at a part
!     istartsf       Starting address of surface
!     isubsf         Sequence number of subsurface
!     isurnr         Surface sequence number
!     iwork          integer work array of length ncurvs
!                    iwork indicates for each curve whether it is
!                    in the boundary of a surface or not
!                    Possible values:
!                    -2: curve is in inner boundary of compound surface
!                        with negative sign with respect to the first
!                        subsurface that contains this curve
!                    -1: curve is in outer boundary of compound surface
!                        with negative sign
!                     0: curve not treated yet or not in surface
!                     1: curve is in outer boundary of compound surface
!                        with positive sign
!                     2: curve is in inner boundary of compound surface
!                        with positive sign with respect to the first
!                        subsurface that contains this curve
!     iwork1         integer work array of length nsurfs
!                    iwork1(i) indicates if surface i has been treated (1)
!                    or not (0)
!     iwork2         integer work array of length ncurvs
!                    Is used to store curve numbers of subsurface
!     j              Counting variable
!     jcount         Counter
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     k              Counting variable
!     ksign          Help parameter to compute sign of the Jacobian
!     lenkmeshscp    Length of kmeshs parts 1 + 2
!     loop           If true the loop may be carried out twice, otherwise
!                    it is the last time
!     nouter         Number of outer boundaries
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHKMESHRDIR1  Store the outer boundaries of a subsurface into array
!                    iwork2 provided with a sign
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2408   Internal error: too many loops in while loop
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Method:
!
!     iwork is set to +/- 1 for outer curves and +/- 2 for inner curves
!     The sign indicates the direction of the curve in the outer surface
!     The sign in case of an inner curve is defined by the corresponding
!     surface
!     In first instance only "outer" subsurfaces can be treated, but
!     once an outer surface is treated all its inner curves are also
!     known.
!     The direction is positive if a curve is in the outer boundary and has
!     the same sign as the outer boundary curve. If it is in the opposite
!     direction the surface has negative direction
!     If a curve is an inner curve the direction of the surface is positive
!     if the curve has opposite sign with respect to the inner curve
!
!
!     For each surface isurnr with subsurfaces do
!        Clear iwork for ncurvs positions
!        nouter := number of outer boundaries of compound surface
!        For all outer curves, mark iwork with the sign of the outer curve
!        icount := 0
!        nsubsf := number of subsurfaces of isurnr
!        Clear array iwork1 nsurfs positions
!        While ( icount<nsubsf ) do
!           For all subsurfaces isubsf do
!              if ( iwork1(isubsf) = 0 ) then
!                 icurnr := first curve (with sign) in outer boundary
!                           such that iwork(|icurnr|) # 0
!                 if ( icurnr # 0 ) then
!                    icount := icount+1
!                    iwork1(isubsf) := 1
!                    if ( |iwork(|icurnr|)| = 1 ) then
!                       ksign = sign(iwork(|icurnr|)) * sign(icurnr)
!                    else ( |iwork(|icurnr|)| = 2 )
!                       ksign = -sign(iwork(|icurnr|)) * sign(icurnr)
!                    if ( ksign = -1 ) then
!                       kmeshr("") := -isubsf
!                    For all curves icurnr (with sign) in outer boundary do
!                       if ( iwork(|icurnr|) = 0 ) then
!                          iwork(|icurnr|) := ksign*sign(icurnr)*2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshkmeshrdir' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshkmeshrdir'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- For each surface isurnr with subsurfaces do
!           nsubsf := number of subsurfaces of isurnr

      istartsf = kmeshr(isurnr)
      nsubsf = kmeshr(isurnr+1)-istartsf
      lenkmeshscp = 9*nsurfs+1

!     --- Clear iwork for ncurvs positions

      iwork = 0

!     --- icount := 0
!         Clear array iwork1 nsurfs positions

      icount = 0
      iwork1 = 0

!     --- nouter := number of outer boundaries of compound surface
!         For all outer curves, mark iwork with the sign of the outer
!         curve

      nouter = kmeshsp2(3,isurnr)
      if ( kmeshsp2(5,isurnr)>0 ) nouter = kmeshsp2(5,isurnr)
      istart = kmeshs(isurnr)-lenkmeshscp

      if ( debug ) write(irefwr,1) 'isurnr, nouter', isurnr, nouter
      if ( nouter>0 ) then

!     --- The surface has outer boundaries

         do i = 1, nouter
            icurnr = kmeshsp3(istart+i)
            iwork(abs(icurnr)) = sign(1,icurnr)
         end do

      else

!     --- The surface consists of a set of closed surfaces
!         without outer boundaries
!         Use first curve in surface as outer boundary

         isubsf = kmeshr(istartsf+1)
         istart = kmeshs(isubsf)-lenkmeshscp
         icurnr = kmeshsp3(istart+1)
         iwork(abs(icurnr)) = sign(2,icurnr)
         iwork1(isubsf) = 1
         icount = 1

         if ( debug ) then

!        --- debug statements

            write(irefwr,1) 'isurnr, isubsf, icurnr', &
                             isurnr, isubsf, icurnr

         end if

!        --- Store outer curves provided with correct sign in iwork2

         call mshkmeshrdir1 ( isubsf, kmeshsp2, iwork2, kmeshsp3, &
                              kmeshs(isubsf+1)-kmeshs(isubsf), &
                              istart, debug )

!        --- For all curves icurnr (with sign) in outer boundary of
!               surface isurnr do
!               if ( iwork(|icurnr|) = 0 ) then
!                  iwork(|icurnr|) := ksign*sign(icurnr)*2

         do k = 1, kmeshs(isubsf+1)-kmeshs(isubsf)
            icurnr = iwork2(k)
            jcurnr = abs(icurnr)
            if ( iwork(jcurnr)==0 ) &
               iwork(jcurnr) = sign(1,icurnr)*2
         end do

      end if
      if ( debug ) then

!     --- debug statements

         write(irefwr,1) 'isurnr, nouter, nsubsf', &
                          isurnr, nouter, nsubsf
         if ( nouter>0 ) then
            call prinin ( kmeshsp3(istart+1), nouter, &
                          'outer boundaries')
         else
            write(irefwr,1) 'First curve is', icurnr
         end if

      end if
      loop = .true.

!     --- While ( icount<nsubsf ) do

100   jcount = 0
      found = .true.
      do while ( found .and. icount<nsubsf .and. jcount<=nsurfs )

!     --- For all subsurfaces isubsf do

         jcount=jcount+1
         found = .false.

         if ( debug ) call prinin ( iwork, ncurvs, 'iwork' )
         do j = 1, nsubsf

            isubsf = abs(kmeshr(istartsf+j))
            if ( debug ) &
               write(irefwr,1) 'j, isubsf, iwork1(isubsf)', &
                                j, isubsf, iwork1(isubsf)

            if ( iwork1(isubsf)==0 ) then

!           --- iwork1(isubsf) = 0
!               icurnr := first curve (with sign) in outer boundary
!                         such that iwork(|icurnr|) # 0

               istart = kmeshs(isubsf)
               nouter = kmeshs(isubsf+1)-istart

               if ( debug ) write(irefwr,1) 'nouter', nouter

               if ( nouter==0 ) then
                  iwork1(isubsf) = -1  ! mark sub surface
                  go to 1000  ! jump in case no outer boundary
               end if  ! ( nouter==0 )

               istart = istart-lenkmeshscp

!              --- Store outer curves provided with correct sign in iwork2

               call mshkmeshrdir1 ( isubsf, kmeshsp2, iwork2, kmeshsp3, &
                                    nouter, istart, debug )
               if ( debug ) call prinin ( iwork2, nouter, 'iwork2' )

               do k = 1, nouter
                  icurnr = iwork2(k)
                  if ( iwork(abs(icurnr))/=0 ) go to 150
               end do

!              --- No curve has been found, jump
!                  i.e. the subsurface is completely inside the surface
!                  It has no boundary common with the outer boundary, nor
!                  has one of its boundaries treated before as
!                  inner boundary of an outer surface

               go to 200

!              --- icurnr # 0
!                  icount := icount+1
!                  iwork1(isubsf) := 1

150            icount = icount+1
               iwork1(isubsf) = 1
               found = .true.
               jcurnr = abs(icurnr)
               if ( debug ) write(irefwr,1) 'jcurnr', jcurnr
               if ( abs(iwork(jcurnr))==1 ) then

!              --- |iwork(|icurnr|)| = 1
!                  ksign = sign(iwork(|icurnr|)) * sign(icurnr)

                  ksign = sign(1,iwork(jcurnr)) * sign(1,icurnr)

               else

!              --- |iwork(|icurnr|)| = 2
!                  ksign = -sign(iwork(|icurnr|)) * sign(icurnr)

                  ksign = -sign(1,iwork(jcurnr)) * sign(1,icurnr)

               end if

!              --- if ( ksign = -1 ) then
!                     kmeshr("") := -isubsf

               if ( ksign==-1 ) kmeshr(istartsf+j) = -isubsf
               if ( debug ) then

!              --- debug statements

                  write(irefwr,1) 'isurnr, isubsf, ksign', &
                                   isurnr, isubsf, ksign
                  write(irefwr,1) 'istartsf, j',istartsf, j

               end if

!              --- For all curves icurnr (with sign) in outer boundary do
!                     if ( iwork(|icurnr|) = 0 ) then
!                        iwork(|icurnr|) := ksign*sign(icurnr)*2

               do k = 1, nouter
                  icurnr = iwork2(k)
                  jcurnr = abs(icurnr)
                  if ( iwork(jcurnr)==0 ) &
                     iwork(jcurnr) = ksign*sign(1,icurnr)*2
               end do

200         end if

         end do

      end do

      if ( loop .and. .not. found ) then

!     --- No curve found, debug if there are any outer curves left

         if ( nouter<kmeshsp2(3,isurnr) ) then

!        --- nouter<kmeshsp2(3,isurnr), there are still outer curves
!            left. This means that the compound surface consists
!            of disjoint surfaces
!            Use these curves as boundary curves and repeat process

            do i = nouter+1,kmeshsp2(3,isurnr)
               icurnr = kmeshsp3(istart+i)
               iwork(abs(icurnr)) = sign(1,icurnr)
            end do
            nouter = kmeshsp2(3,isurnr)
            loop = .false.
            go to 100

         end if

!        --- No outer curves found, hence a new closed part has been found
!            Find first free surface and repeat process

         do j = 1, nsubsf

            isubsf = abs(kmeshr(istartsf+j))
            if ( debug ) &
               write(irefwr,1) 'j, isubsf, iwork1(isubsf)', &
                                j, isubsf, iwork1(isubsf)
            if ( iwork1(isubsf)==0 ) then

!           --- iwork1(isubsf) = 0, new surface

               istart = kmeshs(isubsf)
               nouter = kmeshs(isubsf+1)-istart
               if ( nouter==0 ) go to 300  ! jump in case of no curves
               istart = istart-lenkmeshscp

!              --- Store outer curves provided with correct sign in iwork2

               call mshkmeshrdir1 ( isubsf, kmeshsp2, iwork2, kmeshsp3, &
                                    nouter, istart, debug )
               if ( debug ) write(irefwr,1) 'nouter/2', nouter
               do k = 1, nouter
                  icurnr = iwork2(k)
                  iwork(abs(icurnr)) = sign(1,icurnr)
               end do
               loop = .false.
               go to 100

            end if

300      end do

         call errint ( jcount, 1 )
         call errint ( nsurfs, 2 )
         call errsub ( 2408, 2, 0, 0 )

      end if

      if ( jcount>nsurfs ) then

!     --- too many iterations

          call errint ( jcount, 1 )
          call errint ( nsurfs, 2 )
          call errsub ( 2408, 2, 0, 0 )

      end if

1000  call erclos ( 'mshkmeshrdir' )
      if ( debug ) then

!     --- Debug information

         call prinin ( iwork, nsurfs, 'iwork' )
         write(irefwr,*) 'End mshkmeshrdir'

      end if  ! ( debug )

      end
