      subroutine mshlistgrp ( listgrp, iinput, ioutercurvs, iseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 30-03-2004 Extra parameters
!        version  1.0  date 26-01-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array listgrp with a 0 if an element group must be skipped and
!     a 1 if it has to be taken into account
!     Check array iinput
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     outer_boundary
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer listgrp(*), iinput(*), ioutercurvs(*), iseq

!     iinput         i    integer input array describing which element
!                         groups must be coupled to a sequence number
!                         iinput must be filled as follows:
!                         Pos. 1  numelgrps: number of element groups
!                              2-numelgrps+1  element group sequence numbers
!     ioutercurvs   i/o   kmesh part al
!                         Contains outer boundaries corresponding to a selected
!                         set of element groups
!                         Pos. 1 is check and raised
!     iseq           i    Sequence number to be used in kmeshal
!     listgrp        o    Array of size nelgrp indicating if an element
!                         group must be used (1) or not (0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, nseq, ngroups, i, iwork(nelgrp)

!     i              Counting variable
!     ielgrp         element group number
!     iwork          integer work array of length nelgrp
!                    if iwork(i) = 0 element group i corresponds to lower
!                    dimesnion, if iwork(i) = 1, not.
!     ngroups        Number of element groups stored in iinput
!     nseq           Sequence number used in array ioutercurvs
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1576   Value of integer parameter incorrect
!    2485   Element group sequence number out of range
!    2486   Sequence number for the storage of outer boundaries out of range
!    2487   Element group has been found more than once in array iinput
!    2489   Element group  corresponds to elements of lower dimension
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check iinput(1-2)
!     Check if mesh has volume elements
!     Fill iwork for all element groups with elements of highest dimension
!     Fill listgrp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshlistgrp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshlistgrp'
         call prinin ( iinput, iinput(1)+1, 'iinput' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Check iinput(1-2)

      nseq = ioutercurvs(1)
      if ( debug ) write(irefwr,*) 'iseq, nseq ', iseq, nseq
      if ( iseq/=nseq+1 ) then

!     --- iseq has wrong value

         call errint ( iseq, 1 )
         call errint ( nseq, 2 )
         call errsub ( 2486, 2, 0, 0 )

      else

!     --- fill position 1 of ioutercurvs

         ioutercurvs(1) = iseq

      end if

!     --- Check if mesh has volume elements
!         Fill iwork for all element groups with elements of highest dimension

      if ( nvolms>0 ) then

!     --- There are volume elements

         do ielgrp = 1, nelgrp

            ishape = kmeshf(ielgrp)
            if ( ishape>10 .and. ishape<50 ) then

!           --- ishape > 10, hence volume element

               iwork(ielgrp) = 1

            else

!           --- ishape <= 10, hence not a volume element

               iwork(ielgrp) = 0

            end if  ! ( ishape>10 )

         end do  ! ielgrp = 1, nelgrp

      else

!     --- There are no volume elements

         do ielgrp = 1, nelgrp

            ishape = kmeshf(ielgrp)
            if ( ishape>2 .and. ishape<10 ) then

!           --- 3 <= ishape <= 10, hence surface element

               iwork(ielgrp) = 1

            else

!           --- ishape <= 10, hence not a volume element

               iwork(ielgrp) = 0

            end if  ! ( ishape>10 )

         end do  ! ielgrp = 1, nelgrp

      end if  ! ( nvolms>0 )

!     --- Fill array listgrp for all element groups in list

      do i = 1, nelgrp
         listgrp(i) = 0
      end do

      ngroups = iinput(1)
      if ( ngroups<1 ) then

!     --- ngroups has wrong value

         call errint ( ngroups, 1 )
         call errint ( 2, 2 )
         call errchr ( 'ngroups', 1 )
         call errchr ( 'iinput', 2 )
         call errsub ( 1576, 2, 0, 2 )

      end if

      do i = 1, ngroups

         ielgrp = iinput(1+i)
         if ( ielgrp<1 .or. ielgrp>nelgrp ) then

!        --- element group out of range

            call errint ( ielgrp, 1 )
            call errint ( nelgrp, 2 )
            call errint ( i, 3 )
            call errint ( iseq, 4 )
            call errsub ( 2485, 4, 0, 0 )

         end if  ! ( ielgrp<1 .or. ielgrp>nelgrp )

         if ( iwork(ielgrp)==0 ) then

!        --- element group is of lower dimension

            call errint ( ielgrp, 1 )
            call errint ( iseq, 2 )
            call errsub ( 2489, 2, 0, 0 )

         end if !  ( iwork(ielgrp)=0 )

         if ( listgrp(ielgrp)/=0 ) then

!        --- element group found more than once

            call errint ( ielgrp, 1 )
            call errint ( iseq, 2 )
            call errwar ( 2487, 2, 0, 0 )

         end if !  ( listgrp(ielgrp)/=0 )
         listgrp(ielgrp) = 1

      end do  ! i = 1, ngroups

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshlistgrp'

      end if
      call erclos ( 'mshlistgrp' )

      end
