      subroutine mshobstms2d03 ( trianglenode, trianglenodenw, &
                                 iworkpoint, npointnew, ielemnew, &
                                 coornew, iwork, iedges, nmeshc, &
                                 icrossobst, rcrossobst, ielem, &
                                 ikelmcnew, nmap, nelem, edgemoved )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.6  date 14-11-2013 Extra debug
!        version  3.5  date 11-10-2011 Extension with 1d
!        version  3.4  date 20-07-2011 Extra debug
!
!   copyright (c) 2001-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a triangle if necessary into two, three or four subtriangles and
!     indicate which of these triangles is in or out the obstacle
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     triangle
!     subdivision
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer trianglenode(3), trianglenodenw(12), npointnew, &
              ielemnew, nelem, iwork(nelem,*), nmap, &
              iworkpoint(nmap,*), iedges(3,*), nmeshc(*), &
              icrossobst(*), ielem, ikelmcnew, edgemoved(*)
      double precision coornew(2,*), rcrossobst(*)

!     coornew       i/o   Array containing the new coordinates of the nodes
!     edgemoved      i    Array of size nedges indicating if nodes on an edge
!                         have been moved (1) or not (0)
!     icrossobst     i    array containing information about the cross section
!                         of obstacles with edges
!                         The storage of icrossobst is as follows
!                         Pos. 1 to numobst (number of obstacles):
!                               icrossobst(iobst) = istartobst (starting address
!                               of obstacle iobst in array
!                         Per obstacle from position istartobst:
!                         Number of cross sections, followed by 2 positions for
!                         each cross section consisting of the edge sequence
!                         number and a reference to rcrossobst
!                         This part is followed by the number of elements
!                         that is intersected by the obstacle followed by
!                         for each element: element number, number of
!                         intersections and sequence numbers of intersections
!     iedges        i/o   Work array of size nedges
!                         If iedges(i) = 0, no point has been created in edge i
!                         If iedges(i)>0, a point has been created in edge i, &
!                         with sequence number iedges(i)
!     ielem          i    Element number
!     ielemnew      i/o   Sequence number of element in new mesh
!     ikelmcnew     i/o   Starting address of next element in trianglenodenw
!     iwork          i    integer work array of length nelem*2
!                         iwork(ielem,1) contains the starting address of
!                         element ielem in array icrossobst
!                         If 0, the element is not present in icrossobst
!                         iwork(ielem,2) contains the corresponding obstacle
!                         sequence number iobst
!     iworkpoint    i/o   Work array of length npointnew*2
!                         This array is filled as follows:
!                         iworkpoint(i,1) = 0 if node i outside obstacle
!                         iworkpoint(i,1) = 1 if node i inside obstacle
!                         iworkpoint(i,1) = 2 if node i inside obstacle (near
!                                             boundary)
!                         iworkpoint(i,1) = 3 if node i on obstacle
!                         iworkpoint(i,2) contains the corresponding obstacle
!                         sequence number iobst
!     nelem          i    Number of elements
!     nmap           i    length of first parameter of imap and rmap
!     nmeshc         i    nmesh part c
!     npointnew     i/o   Sequence number of last generated node
!     rcrossobst     i    Real array corresponding to icrossobst
!                         For each cross section a real factor f is stored
!                         defining the intersection of the edge with the
!                         obstacle in the following way
!                         x_intersect = x1 + f * (x2-x1)
!                         where x1 and x2 are the first and last point of the
!                         edge respectively
!     trianglenode   i    Array of nodal point numbers of the corners of
!                         the original triangle.
!     trianglenodenw o    Array of nodal point numbers of the corners of
!                         the new subtriangles
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer edgenode(3,3), istart, icross, iobst, &
              ipstart, i, iedge, nextranodes, ireledge, istarticross, &
              node1, node2
      double precision eps, fact

!     edgenode       Array with all nodes on the crossed edge
!                    edgenode(1,1) first node on crossed edge
!                    edgenode(2,1) last node on crossed edge
!                    edgenode(3,1) new node on crossed edge
!     eps            Local accuracy
!     fact           interpolation factor
!     i              Counting variable
!     icross         position of iedge in array icrossobst
!     iedge          edge sequence number
!     iobst          Obstacle sequence number
!     ipstart        Starting address in array
!     ireledge       relative edge sequence number
!     istart         First position used at a part
!     istarticross   Starting address of obstacle in array icrossobst
!     nextranodes    Number of extra nodes in element
!     node1          First node of edge
!     node2          Last node of edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHELEMREF2D   Subdivide a triangle if necessary into two, three or
!                    four subtriangles
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     eps := 0.2
!     nextranodes = 0
!     do i = 1, number of intersections
!        iedge := edge which is intersected
!        fact := interpolation factor
!        if ( cross section with edge close to end point of edge ) then
!           mark corresponding node as on boundary
!        else ( i-th edge is intersected )
!           nextranodes := nextranodes+1
!           edgenode(1,i) := first point of edge
!           edgenode(2,i) := last point on edge
!           if( first time edge is intersected)
!              mark edge as intersected
!              introduce the intersection node
!              calculate its coordinates
!              mark it as boundary node
!              store interpolation data
!           end if
!           edgenode(3,i) := intersection node
!        end if
!     end do
!     subdivide triangle into nextranode+1 subtriangles
!     mark each subtriangle as inside or outside
!     update pointers ikelmcnew and ielemnew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms2d03' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms2d03'
         write(irefwr,1) 'ielem', ielem
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )
  4      format ( a, 1x, 2i6, d12.3)
         call prinin ( trianglenode, 3, 'trianglenode' )
         write(irefwr,1) 'iworkpoint', iworkpoint(trianglenode(1:3),1)

      end if
      if ( ierror/=0 ) go to 1000

!     --- set local accuracy eps

      eps = accuracy_obstacle

      if ( debug ) write(irefwr,2) 'eps', eps

!     --- nextranodes := 0

      nextranodes = 0

      istart = iwork(ielem,1)
      iobst = iwork(ielem,2)
      istarticross = icrossobst(iobst)
      if ( debug ) write(irefwr,1) 'istart, iobst, istarticross', &
          istart, iobst, istarticross

!     --- do i = 1,nintersect

      do i = 1, icrossobst(istart)

         ireledge = icrossobst(istart+i)
         icross  = istarticross+1+2*(ireledge-1)

!        --- iedge := edge which is intersected

         iedge = icrossobst(icross)

         if ( edgemoved(iedge)==0 ) then

!        --- nodes on edge have not been moved yet

            fact  = rcrossobst(icrossobst(icross+1))
            ipstart = nmeshc(iedge)
            if ( debug ) &
               write(irefwr,4) 'i, iedge, fact', i, iedge, fact
            node1 = nmeshc(ipstart+1)  ! first node of edge
            node2 = nmeshc(ipstart+2)  ! last node of edge
            if ( debug ) write(irefwr,4) 'node1, node2', node1, node2
            if ( fact>1d0-eps ) then

!           --- i-th cross section close to begin point of edge
!               mark node corresponding to begin point as on boundary

               iworkpoint(node2,1) = 3
               iworkpoint(node2,2) = iobst
               if ( debug ) write(irefwr,4) 'nodebegin', node2

            else if ( fact<eps ) then

!           --- i-th cross section close to end point of edge
!               mark corresponding node as on boundary

               iworkpoint(node1,1) = 3
               iworkpoint(node1,2) = iobst
               if ( debug ) write(irefwr,4) 'nodeend', node1

            else

!           --- i-th edge is intersected
!               increment nextranodes
!               edgenode(1,nextranodes) := first point of edge
!               edgenode(2,nextranodes) := last point on edge

               nextranodes = nextranodes+1
               edgenode(1,nextranodes) = node1
               edgenode(2,nextranodes) = node2

               if ( iedges(1,iedge)==0 ) then

!              --- New point
!                  npointnew := npointnew+1
!                  Fill coordinates of cross section in coornew(npointnew)
!                  mark node as boundary node

                  npointnew = npointnew+1
                  iedges(1,iedge) = npointnew
                  coornew(1,npointnew) = (1d0-fact)*coornew(1,node1) &
                                        +fact*coornew(1,node2)
                  coornew(2,npointnew) = (1d0-fact)*coornew(2,node1) &
                                        +fact*coornew(2,node2)
                  iworkpoint(npointnew,1) = 3
                  iworkpoint(npointnew,2) = iobst
                  imap(npointnew,1) = 2
                  imap(npointnew,2) = node1
                  imap(npointnew,3) = node2
                  rmap(npointnew,1) = 1d0-fact
                  rmap(npointnew,2) = fact
                  if ( debug ) write(irefwr,4) 'npointnew', npointnew

               end if  ! ( iedges(1,iedge)==0 )

!              --- edgenode(3,nextranodes) := cross section node

               edgenode(3,nextranodes) = iedges(1,iedge)

            end if  ! ( fact>1d0-eps )

         end if  ! ( edgemoved(iedge)==0 )

      end do  ! i = 1, icrossobst(istart)
      if ( debug ) then
         write(irefwr,1) 'nextranodes', nextranodes
         call prinin1 ( edgenode, nextranodes, 3, 'edgenode' )
      end if  ! ( debug )

!     --- actual subdivision

      call mshelemref2d ( nextranodes, trianglenode, &
                          trianglenodenw, edgenode, coornew, &
                          ikelmcnew, ielemnew )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstms2d03'

      end if

      call erclos ( 'mshobstms2d03' )

      end
