      subroutine mshouterfind2d ( ncurvs, kmeshm, ioutercurvs, &
                                  lenoutercurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 18-10-2002 New storage of composite curves
!        version  1.0  date 03-01-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Find the outer boundary of a two-dimensional region and store the
!    curve numbers provided with the correct sign sequentially in array
!    ioutercurvs
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     boundary
!     outer_boundary
!     sort
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvs, kmeshm(*), ioutercurvs(*), lenoutercurvs

!     ioutercurvs    o    Contains information about the curves in the outer
!                         boundary (2D) in the following way
!                         Pos. 1  nparts  Number of parts the outer boundary
!                                         consists of
!                         Pos. 2-1+nparts Starting addresses of each part
!                         Per part: (first address: ioutercurvs(1+i)
!                         Pos. 1: nouter  Number of curves in part
!                         followed by the curve numbers (provided with a sign)
!                         These curve numbers are consecutive, which means
!                         the end of one curve is the start of the new one
!                         The part itself is closed
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     lenoutercurvs i/o   Length of array ioutercurvs
!                         At input: estimated length
!                         At output: computed length
!     ncurvs         i    Number of curves in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iwork(ncurvs), iwork1(ncurvs), iwork2(ncurvs), ninner, &
              istar3, istar4, firstpnt, lastpnt, npart, istartcurve, &
              istart, nleft, istore(2*ncurvs), j, icurnr

!     firstpnt       First point of a curve
!     i              Counting variable
!     icurnr         Curve sequence number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istar4         Starting address of KMESHM part 4 in KMESH part M
!     istart         First position used at a part
!     istartcurve    Starting position in array istore
!     istore         Contains the last level set
!     iwork          integer work array of length ncurvs that is used as follows
!                    iwork(i) < 0: i is inner curve or curve that is not used
!                    iwork(i) = 0: i is outer curve that has not been treated
!                                  yet
!                    iwork(i) > 0: i is outer curve that has been treated
!     iwork1         integer work array of length ncurvs that contains
!                    the node number of the first point of the curve
!     iwork2         integer work array of length ncurvs that contains
!                    the node number of the last point of the curve
!     j              Counting variable
!     lastpnt        last point of a curve
!     ninner         Number of inner curves
!     nleft          Number of curves left
!     npart          Number of parts
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Warnings
!     PRININ         print 1d integer array
!     PRINOUTCRVS    Print the contents of array ioutercurvs
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2413   Internal error: nleft < 0
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork
!     Set iwork(i) equal to -1 for all internal curves
!     Set iwork(i) equal to -1 for all curves that have not been filled
!     Put initial and end point into iwork1-2 for all boundary curves
!     npart := 0
!     istartcurve := 1
!     While ( there are still outer curves in iwork ) do
!        istart := istartcurve+1
!        npart := npart+1
!        icurnr := first free outer curve in iwork
!        istore(istart) := icurnr
!        iwork(icurnr) := npart
!        firstpnt := starting node of icurnr
!        lastpnt := end node of icurnr
!        While ( lastpnt # firstpnt ) do
!           icurnr := +/-(curve that starts or end with lastpnt)
!           iwork(|icurnr|) := npart
!           istart := istart+1
!           istore(istart) := icurnr
!        istore(istartcurve) := istart-istartcurve
!        istartcurve := istart+1
!     Fill array ioutercurvs from array istore
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshouterfind2d' )
      debug = .false.

!     --- Clear array iwork

      do i = 1, ncurvs
         iwork(i) = 0
         iwork1(i) = 0
         iwork2(i) = 0
      end do

!     --- Set iwork(i) equal to -1 for all internal curves

      ninner = kmeshm(1)
      istar3 = kmeshm(2)-1
      istar4 = kmeshm(3)-1
      do i = 1, ninner
         iwork(abs(kmeshm(istar4+i))) = -1
      end do

!     --- Set iwork(i) equal to -1 for all curves that have not been filled
!         Put initial and end point into iwork1-2 for all boundary curves

      do i = 1, ncurvs
         firstpnt = kmeshm(istar3+kmeshm(4+i))
         lastpnt = kmeshm(istar3+kmeshm(5+i)-1)
         if ( iwork(i)==0 .and. (firstpnt<=0 .or. lastpnt.le.0) ) &
            then
            iwork(i) = -1
            ninner = ninner+1
         else
            iwork1(i) = firstpnt
            iwork2(i) = lastpnt
         end if
      end do

      if ( debug ) then

!     --- debug true, debug statements

         write(irefwr,*) 'In mshouterfind2d: '
         write(irefwr,*) 'ncurvs, ninner ', ncurvs, ninner
         call prinin(iwork1,ncurvs,'iwork1 (first points)')
         call prinin(iwork2,ncurvs,'iwork2 (last points)')
         call prinin(kmeshm(istar4+1),ninner,'inner curves')

      end if

!     --- npart := 0
!         istartcurve := 1

      npart = 0
      istartcurve = 1
      nleft = ncurvs-ninner

!     --- While ( there are still outer curves in iwork ) do

      do while ( nleft>0 )

!     --- istart := istartcurve+1
!         npart := npart+1

         istart = istartcurve+1
         npart = npart+1
         nleft = nleft-1

!        --- icurnr := first free outer curve in iwork

         do i = 1, ncurvs
            if ( iwork(i)==0 ) then

!           --- free outer curve found

               icurnr = i
               go to 100

            end if
         end do

!        --- istore(istart) := icurnr
!            iwork(icurnr) := npart
!            firstpnt := starting node of icurnr
!            lastpnt := end node of icurnr

100      istore(istart) = icurnr
         iwork(icurnr) = npart
         firstpnt = iwork1(icurnr)
         lastpnt = iwork2(icurnr)

         if ( debug ) then

!        --- debugging information

            write(irefwr,*) 'istart,icurnr,istartcurve,npart,firstpnt,', &
            'lastpnt ',istart,icurnr,istartcurve,npart,firstpnt,lastpnt

         end if

!        --- While ( lastpnt # firstpnt ) do

         do while ( lastpnt/=firstpnt )

!        --- icurnr := +/-(curve that starts or ends with lastpnt)

            do i = 1, ncurvs

               if ( iwork(i)==0 ) then

!              --- curve is free outer curve

                  if ( iwork1(i)==lastpnt ) then

!                 --- First point of curve i is equal to last point

                     icurnr = i
                     go to 200

                  else if ( iwork2(i)==lastpnt ) then

!                 --- Last point of curve i is equal to last point

                     icurnr = -i
                     go to 200

                  end if

               end if

            end do

!           --- iwork(|icurnr|) := npart
!               istart := istart+1
!               istore(istart) := icurnr
!               lastpnt := end node of icurnr

200         iwork(abs(icurnr)) = npart
            istart = istart+1
            istore(istart) = icurnr
            if ( icurnr>0 ) then
               lastpnt = iwork2(icurnr)
            else
               lastpnt = iwork1(-icurnr)
            end if
            nleft = nleft-1
            if ( debug ) then

!           --- debugging information

               write ( irefwr,* ) ' istart, icurnr, lastpnt, nleft ', &
                                    istart, icurnr, lastpnt, nleft
            end if
            if ( nleft<0 ) then

!           --- nleft < 0, internal error

               call errint ( nleft, 1 )
               call errwar ( 2413, 1, 0, 0 )
               lenoutercurvs = 0
               go to 1000

            end if

         end do

!        --- istore(istartcurve) := istart-istartcurve
!            istartcurve := istart+1

         istore(istartcurve) = istart-istartcurve
         istartcurve = istart+1
         if ( istart>2*ncurvs ) then

!        --- array istore too small

            call errint ( istart, 1 )
            call errint ( 2*ncurvs, 2 )
            call errchr ( 'istore', 1 )
            call errsub ( 550, 2, 0, 1 )
            lenoutercurvs = 0
            go to 1000

         end if

      end do

!     --- Fill array ioutercurvs from array istore

      ioutercurvs(1) = npart
      istart = npart+2
      istartcurve = 1
      do i = 1, npart

         ioutercurvs(1+i) = istart
         nleft = istore(istartcurve)
         ioutercurvs(istart) = nleft
         do j = 1, nleft
            ioutercurvs(istart+j) = istore(istartcurve+j)
         end do
         istartcurve = istartcurve+nleft+1
         istart = istart+nleft+1
         if ( istart>lenoutercurvs ) then

!        --- array ioutercurvs too small

            call errint ( istart, 1 )
            call errint ( lenoutercurvs, 2 )
            call errchr ( 'ioutercurvs', 1 )
            call errsub ( 550, 2, 0, 1 )
            lenoutercurvs = 0
            go to 1000

         end if

      end do

      if ( debug ) then

!     --- debugging information

         call prinoutcrvs ( ioutercurvs )

      end if
      lenoutercurvs = istart-1

1000  call erclos ( 'mshouterfind2d' )

      end
