      subroutine mshparkmeshloc ( nprocessors, kmeshlc, neighblocks )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.9  date 25-03-2015 Correction lennormloc = 0
!        version  2.8  date 15-11-2012 Long integers
!        version  2.7  date 18-02-2012 Extra debug
!
!   copyright (c) 2003-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmeshlc with the complete topology of all submeshes
!     that are used in the parallel computation
!     The subdivision of the mesh is stored in the arrays elempointers and
!     nodepointers
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     2d
!     3d
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nprocessors, kmeshlc(*), neighblocks(*)

!     kmeshlc       i/o   Subdivision of array KMESH
!     neighblocks    i    Array containing of each block all neighbor blocks
!                         in the following way:
!                         Pos. 1 to nprocesors starting addresses of each block
!                         For each block from start position:
!                         Number of neighbor blocks, followed by
!                         neighbor block sequence numbers
!     nprocessors    i    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iprocessor, nelemloc, npointloc, lenkmeshc, &
              lengthl, lengthm, lenkmesht, &
              lenneighblock, lennormloc, iseqkm1, iseqkm2
      integer, allocatable :: kmeshcloc(:), pointmap(:)
      double precision, allocatable :: normloc(:,:)
      integer, pointer :: elements(:), points(:)

!     elements       Array of size nelemloc containing the element
!                    numbers corresponding to the local mesh
!     iprocessor     Processor sequence number
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     kmeshcloc      Local array kmeshc containing the node numbers of one
!                    element
!     lengthl        Length of kmesh part l
!     lengthm        Length of kmesh part m
!     lenkmeshc      length of array kmeshc
!     lenkmesht      length of array kmesht
!     lenneighblock  Declared length of array neighblocks
!     lennormloc     Computed number of nodes in array normloc
!     nelemloc       Number of elements in local mesh
!     normloc        Contains the normal in the local nodes in the
!                    same sequence as in array nodes
!     npointloc      Number of points in local mesh
!     pointmap       Array of size npoint containing the local node numbers
!                    of the nodes in the global mesh
!                    if node i is not present in local mesh pointmap(i) = 0
!     points         Array of size npointloc containing the node
!                    numbers corresponding to the local mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIMAPINT      Map integer array corresponding to global mesh into
!                    integer array corresponding to local mesh
!     MSHPARCOOR     Fill array coorloc with the coordinates of the nodes by
!                    extracting from the global mesh
!     MSHPARINV      Compute the inverse mapping
!                    (i.e. mapping from global to local mesh)
!     MSHPARKMESHC   Fill local arrays kmeshc and kmeshg using global arrays
!                    kmeshc and kmeshg and the arrays elements and points
!     MSHPARKMESHT   Fill array kmesht for a local mesh from array kmesht of
!                    the global mesh using the information stored in array
!                    elements
!     MSHPARM        Make local array kmeshm from global array kmeshm using
!                    array maparinv
!     MSHPARN        Make local array kmeshn from global array kmeshn using
!                    array maparinv
!     MSHPARNCORR    Make a correction of kmeshn in order that surface elements
!                    belong to one submesh only
!     MSHPARNORMAL   Fill array normloc with normals on outer surfaces
!                    for one processor
!     MSHTRANSINFO   Fill array transportinfo with information of neighbors
!                    of one surface
!     PRININ         print 1d integer array
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array kmeshlc, fill positions 1
!     For all processors iprocessor do
!        Fill local array kmeshlc(iprocessor) in the following way:
!           Pos. 2, 4-6 Copy of kmesh
!                8, 9   Computed
!               10-16   Copy of kmesh
!               17+21   Filled locally ( kmeshc and g)
!               20      Copy of kmesh
!               23      Filled locally ( coor )
!               26      Filled locally ( kmeshl )
!               27      Filled locally ( kmeshm )
!               28      Filled locally ( kmeshn )
!               30-35   Copy of kmesh
!               36      Filled locally ( kmesh part t )
!               38      Copy of kmesh
!               40-42   Copy of kmesh
!               45-47   Copy of kmesh
!               53,54   Filled locally ( transportinfo, blockinfo )
!               57      Copy of kmesh
!               61      Filled locally ( normals )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparkmeshloc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparkmeshloc'
  1      format ( a, 1x, (10i10) )

      end if

!     --- Clear array kmeshlc, fill positions 1

      kmeshlc(1:nprocessors) = 0
      iseqkm1 = iseqkm

      allocate ( pointmap(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'pointmap' )
      if ( ierror/=0 ) go to 1000

!     --- For all processors iprocessor do

      do iprocessor = 1, nprocessors

!     --- Fill local array kmeshlc(iprocessor) in the following way:
!         Pos. 2, 4-6 Copy of kmesh

         call sepcrmeshinfo ( kmeshlc(iprocessor) )
         iseqkm2 = iseqkm
         km(iseqkm)%kmesh(4) = npelm
         km(iseqkm)%kmesh(5) = nelgrp
         km(iseqkm)%kmesh(6) = ndim

!        --- 8, 9   Computed

         points => kmhpar(iprocessor)%nodepointers
         elements => kmhpar(iprocessor)%elempointers
         nelemloc = size ( elements )
         npointloc = size ( points )
         km(iseqkm)%kmesh(8) = npointloc
         km(iseqkm)%kmesh(9) = nelemloc

         if ( debug ) &
            write(irefwr,1) 'npointloc, nelemloc',npointloc, nelemloc

!        --- 10-16   Copy of kmesh

         km(iseqkm)%kmesh(10:14) = km(iseqkm1)%kmesh(10:14)

!        --- Copy of kmesh parts a and b

         if ( kelma/=0 ) then

!        --- kelma>0, hence kmesh a exists

            if ( km(iseqkm)%kmesh(15)>0 ) deallocate(km(iseqkm)%kmesha)
            allocate ( km(iseqkm)%kmesha(nelgrp+1) )
            km(iseqkm)%kmesha = kmesha
            km(iseqkm)%kmesh(15) = nelgrp+1

         end if  ! ( kelma/=0 )

         if ( kelmb/=0 ) then

!        --- kelmb/=0, hence kmesh b exists

            if ( km(iseqkm)%kmesh(16)>0 ) deallocate(km(iseqkm)%kmeshb)
            allocate ( km(iseqkm)%kmeshb(nelgrp) )
            km(iseqkm)%kmeshb = kmeshb
            km(iseqkm)%kmesh(16) = nelgrp

         end if  ! ( kelmb/=0 )

!        --- 17+21   Filled locally ( kmeshc and g)

!        --- Create temporary space to store kmeshcloc and elemmap
!            Array          Length
!            kmeshcloc      same as kmeshc_glob
!            elemmap        nelem

         length = size ( kmeshc )+nelem

         allocate ( kmeshcloc(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'kmeshcloc' )
         if ( ierror/=0 ) go to 1000

!        --- Store inverse mapping in pointmap

         call mshparinv ( points, npointloc, pointmap, npoint )
         if ( ierror>0 ) go to 1000

!        --- Actual filling of kmeshc

         call mshparkmeshc ( kmeshcloc, lenkmeshc, nelemloc, &
                             elements, pointmap )
         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'iprocessor ', iprocessor
            if ( nelgrp>1 ) &
               call prinin ( km(iseqkm)%kmeshg, nelgrp, 'kmeshg' )

         end if

         if ( ierror>0 ) go to 1000

!        --- Make array kmeshcloc permanent

         km(iseqkm)%kmesh(17) = lenkmeshc
         if ( associated(km(iseqkm)%kmeshc) ) &
            deallocate(km(iseqkm)%kmeshc)
         allocate ( km(iseqkm)%kmeshc(lenkmeshc) )
         km(iseqkm)%kmeshc = kmeshcloc(1:lenkmeshc)

         deallocate ( kmeshcloc, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshcloc' )
         if ( ierror/=0 ) go to 1000

!        --- 20      Copy of kmesh

         if ( km(iseqkm)%kmesh(20)>0 )  deallocate(km(iseqkm)%kmeshf)
         allocate ( km(iseqkm)%kmeshf(nelgrp) )
         km(iseqkm)%kmeshf = kmeshf
         km(iseqkm)%kmesh(20) = nelgrp

!        --- Create local coor array

         length = npointloc*ndim
         if ( km(iseqkm)%kmesh(23)>0 ) deallocate(km(iseqkm)%coor)
         allocate ( km(iseqkm)%coor(ndim,npointloc) )
         coor => km(iseqkm1)%coor
         km(iseqkm)%kmesh(23) = length

!        --- Actual filling

         call mshparcoor ( coor, npointloc, &
                           km(iseqkm)%coor, ndim, points )
         if ( ierror>0 ) go to 1000

!        --- 26      Filled locally ( kmeshl )
!            Activate arrays

         length = size ( kmeshl )

!        --- prepare space for new kmeshlc part l

         if ( km(iseqkm)%kmesh(26)>0 ) deallocate(km(iseqkm)%kmeshl)
         allocate ( km(iseqkm)%kmeshl(length) )
         km(iseqkm)%kmesh(26) = length

!        --- fill kmeshlc part l

         call inimapint ( pointmap, km(iseqkm)%kmeshl, &
                          kmeshl, length )

!        --- 27      Filled locally ( kmeshm )
!            Activate arrays

         length = size ( kmeshm )

!        --- prepare space for new kmeshlc part m

         if ( km(iseqkm)%kmesh(27)>0 ) deallocate(km(iseqkm)%kmeshm)
         allocate ( km(iseqkm)%kmeshm(length) )
         km(iseqkm)%kmesh(27) = length

!        --- Actual filling

         call mshparm ( km(iseqkm)%kmeshm, pointmap, length )

!        --- 28      Filled locally ( kmeshn )

         if ( kelmn/=0 ) then

!        --- kelmn/=0, hence kmeshn has been filled

            length = size ( kmeshn )

!           --- prepare space for new kmeshlc part n

            if ( km(iseqkm)%kmesh(28)>0 ) deallocate(km(iseqkm)%kmeshn)
            allocate ( km(iseqkm)%kmeshn(length) )
            km(iseqkm)%kmesh(28) = length

!           --- Actual filling

            call mshparn ( pointmap )

!           --- Make elements completely zero if one node is zero

            if ( ndim==3 ) then

!           --- ndim = 3, correct surfaces

               call sepfillkmesh(kmeshlc(iprocessor))
               call mshparncorr
               call sepfillkmesh(kmesh)
               iseqkm = iseqkm2

            end if  ! ( ndim==3 )

         end if

!        --- 30-35   Copy of kmesh

         km(iseqkm)%kmesh(30:35) = km(iseqkm1)%kmesh(30:35)
         km(iseqkm)%elsize => km(iseqkm1)%elsize
         km(iseqkm)%kmeshq => km(iseqkm1)%kmeshq
         km(iseqkm)%kmeshr => km(iseqkm1)%kmeshr
         km(iseqkm)%kmeshs => km(iseqkm1)%kmeshs
         km(iseqkm)%iinputkm => km(iseqkm1)%iinputkm
         km(iseqkm)%rinputkm => km(iseqkm1)%rinputkm

!        --- 36      Filled locally ( kmesh part t )

         lenkmesht = size(km(iseqkm1)%kmesht,2)
         if ( km(iseqkm)%kmesh(36)>0 ) deallocate(km(iseqkm)%kmesht)
         allocate ( km(iseqkm)%kmesht(2,lenkmesht) )
         call mshparkmesht ( km(iseqkm)%kmesht, nelemloc, &
                             elements, lenkmesht )
         km(iseqkm)%kmesh(36) = lenkmesht

!        --- 38      Copy of kmesh
!            40-42   Copy of kmesh
!            45-47   Copy of kmesh
!            57      Copy of kmesh
!            58      Copy of kmesh

         km(iseqkm)%kmesh(38) = km(iseqkm1)%kmesh(38)
         km(iseqkm)%kmeshu => km(iseqkm1)%kmeshu
         km(iseqkm)%kmesh(40:42) = km(iseqkm1)%kmesh(40:42)
         km(iseqkm)%kmesh(44:47) = km(iseqkm1)%kmesh(44:47)
         km(iseqkm)%kmeshv => km(iseqkm1)%kmeshv
         km(iseqkm)%kmesh(46) = size(km(iseqkm1)%kmeshv)
         km(iseqkm)%kmeshx => km(iseqkm1)%kmeshx
         km(iseqkm)%rmeshx => km(iseqkm1)%rmeshx
         km(iseqkm)%kmesh(58) = km(iseqkm1)%kmesh(58)
         km(iseqkm)%outerboun => km(iseqkm1)%outerboun

!        --- 61     Normals

         if ( ndim==3 .and. mmnormal>0 ) then

!        --- ndim = 3 and array normals has been filled
!           --- define work space

            length = size ( normal,2 )
            allocate ( normloc(3,length), stat = error )
            if ( error/=0 ) call eralloc ( error, length, 'normloc' )
            if ( ierror/=0 ) go to 1000

!           --- get starting addresses

            call mshparnormal ( pointmap, normloc, lennormloc )

!           --- make array permanent

            if ( lennormloc>0 ) then
               length = lennormloc
               km(iseqkm)%kmesh(61) = length
               if ( associated(km(iseqkm)%normal) ) &
                  deallocate(km(iseqkm)%normal)
               allocate ( km(iseqkm)%normal(3,length) )
               km(iseqkm)%normal = normloc(1:3,1:length)
            end if  ! ( lennormloc>0 ) then

            deallocate ( normloc, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'normloc' )
            if ( ierror/=0 ) go to 1000

         end if  ! ( ndim==3 )

!        --- 53,54   Filled locally ( transportinfo, blockinfo )

         call mshtransinfo ( iprocessor, pointmap, nprocessors, &
                             neighblocks )

      end do

      deallocate ( pointmap, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'pointmap' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) write(irefwr,*) 'End mshparkmeshloc'
      call erclos ( 'mshparkmeshloc' )

      end subroutine mshparkmeshloc
