      subroutine mshparncorr
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 27-11-2012 Long integers
!        version  1.0  date 04-04-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a correction of kmeshn in order that surface elements
!     belong to one submesh only
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     2d
!     3d
!     parallel
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numfacesurf,  inpelm, icount, &
              isur, iloc, iend, istart, iseqkm2
      integer, allocatable :: list(:), istore(:,:), kmeshncopy(:,:), &
                              nodesurf(:)

!     icount         Counter
!     iend           Last position to be used in istore
!     iloc           Sequence number of local node in element
!     inpelm         number of points per element
!     iseqkm2        Sequence number with respect to km of second mesh
!     istart         Next position to be used in istore
!     istore         Contains the part of the set of faces to be considered
!     isur           surface number
!     kmeshncopy     Sorted copy of array kmeshn with all zero elements
!                    removed
!     list           List of element numbers in kmeshncopy with respect to
!                    kmeshn
!     nodesurf       Array of length npoint containing the number
!                    of times a node in a surface appears
!                    If a node is cleared the number is decreased
!     numfacesurf    Number of faces in istore to be considered
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHPARNCORR2   Make a list of faces that appear only once
!     MSHPARNCORR3   Make a sorted list of nodes in outer faces
!     MSHPARNCORR5   Make a list of elements in one surface that are not equal
!                    to zero
!                    The elements are copied in kmeshncopy in sorted sequence
!     MSHPARNCORR6   Remove those elements from kmeshn that are not faces
!     PRININ1        print 2d integer array
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHF  Fill array nmeshf
!     SEPFILLNMESHG  Fill array nmeshg
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2250   The number of nodes per element is not constant
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparncorr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparncorr'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm2 = iseqkm

      if ( km(iseqkm)%kmesh(15)/=0 ) then

!     --- kelma>0, not yet implemented

         call errsub ( 2250, 0, 0, 0 )
         go to 1000

      end if  ! ( kmesh(15)/=0 )

!     --- First we create the faces in the mesh nmesha

      call sepfillkmeshaa
      call sepfillnmesha

!     --- Next element sequence numbers of elements connected to
!         faces

      call sepfillnmeshf

!     --- Finally node numbers of faces

      call sepfillnmeshg
      if ( ierror/=0 ) go to 1000

!     --- Make a list of faces that appear only once
!         These belong to the surfaces

      npoint = km(iseqkm)%kmesh(8)
      nsurfs = km(iseqkm)%kmesh(12)
      nfaces = km(iseqkm)%nmesh(5)

      allocate ( list(nfaces), stat = error )
      if ( error/=0 ) call eralloc ( error, nfaces, 'list' )

      call mshparncorr2 ( nfaces, km(iseqkm)%nmeshf, list, numfacesurf )
      if ( ierror/=0 ) go to 1000

!     --- Make a list of the nodes in these faces, sorted in increasing order

      inpelm = km(iseqkm)%nmeshg(1+nfaces)

      allocate ( istore(inpelm,numfacesurf), nodesurf(npoint), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, numfacesurf*inpelm+npoint, 'istore' )

      call mshparncorr3 ( list, istore, numfacesurf, inpelm, &
                          km(iseqkm)%nmeshg )
      if ( ierror/=0 ) go to 1000

!     --- Make a list of elements in surfaces that are not equal to zero
!         These belong to the surfaces
!         The elements are copied in kmeshncopy in sorted sequence

      kmeshn => km(iseqkm)%kmeshn
      istart = 1+2*nsurfs

      do isur = 1, nsurfs

         inelem = kmeshn(nsurfs+isur)
         if ( kmeshn(isur)/=inpelm ) then
            call errint ( 1, 1 )
            call errint ( inpelm, 2 )
            call errint ( isur, 3 )
            call errint ( kmeshn(isur-1), 4 )
         end if  ! ( kmeshn(isur)/=inpelm )

         if ( debug ) &
            write(irefwr,1) 'isur, inpelm, inelem', isur, inpelm, inelem

         allocate ( kmeshncopy(inpelm,inelem), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, inelem*inpelm, 'kmeshncopy' )

         call mshparncorr5 ( kmeshn(istart), inpelm, list, &
                             kmeshncopy, icount, inelem, nodesurf, &
                             npoint )

!        --- Remove those elements from kmeshn that are not faces

         if ( icount>0 ) then

!        --- icount>0, hence there is something left

            iloc = 1
            call mshparncorr6 ( kmeshncopy, istore, iloc, &
                                inpelm, icount, list, numfacesurf, &
                                kmeshn(istart), iend, nodesurf )

            if ( debug ) &
               call prinin1 ( kmeshn(istart), inelem, inpelm, 'kmeshn' )

         end if  ! ( icount>0 ) then

         istart = istart+inpelm*inelem  ! raise istart

         deallocate ( kmeshncopy, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshncopy' )

      end do  ! isur = 1, nsurfs

      deallocate ( list, istore, nodesurf, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'list' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshparncorr'

      end if  ! ( debug )
      call erclos ( 'mshparncorr' )

      end
