      subroutine mshpavgrid ( rgridinfo, nx, ny, grid, nowbnd, &
                              maxbounds, nloop, gridinfo, crd, layer, &
                              lengridinfo, gridfilled, lengridfilled, &
                              numgridfilled )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-07-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Cover region by rectangular grid and store in each cell of the grid which
!     boundary nodes are present
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     grid
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, maxbounds, nloop, nowbnd(maxbounds,*), &
              grid(0:nx,0:ny), gridinfo(*), layer, lengridinfo, &
              gridfilled(2,*), lengridfilled, numgridfilled
      double precision crd(4,*), rgridinfo(4)

!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     grid          i/o   Work array of size at most 101 x 101 to store
!                         information about the boundary in relation with
!                         arrays gridinfo and rgridinfo
!                         The actual size is (0:nx, 0:ny) and (ix,iy) refers
!                         to the cell ( xmin+ix*dx, ymin+iy*dy) x
!                         ( xmin+(ix+1)*dx, ymin+(iy+1)*dy)
!                         if grid(ix,iy) = 0, no boundary node is stored in cell
!                         (ix,iy), if > 0 information about the boundary cells
!                         is stored in array grdinfo from position
!                         grid(ix,iy)
!     gridfilled    i/o   Work array of size 2 x lengridfilled, indicating which
!                         cells in array grid have value larger than zero
!                         gridfilled(1,i) contains ix and
!                         gridfilled(2,i) contains iy for a cell with
!                         grid(ix,iy) > 0
!     gridinfo      i/o   In this array the boundary points corresponding to a
!                         specific grid cell in grid are stored
!                         istart = grid(ix,iy)
!                         In gridinfo(istart) the number of boundary points of
!                         cell (ix,iy) are stored (if > 0). This is followed
!                         by 2 positions for each boundary point
!                         The pair consists of the loop sequence number in the
!                         boundary where loop refers to a closed loop and
!                         a sequence number in the loop
!     layer          i    Present layer sequence number
!     lengridfilled  i    Contains the declared length of array gridfilled
!     lengridinfo    i    Contains the declared length of array gridinfo
!     maxbounds      i    maximum loops in nowbnd in paver
!     nloop          i    number of boundaries in newbnds
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (maxbounds,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     numgridfilled i/o   Contains the number of gridpoints filled in array
!                         gridfilled
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     rgridinfo      i    Array of size 4 containing the following information
!                         about array grid
!                         1:  xmin
!                         2:  ymin
!                         3:  dx
!                         4:  dy
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iloop, numnodes, inode, nstored, ix, iy, istart
      double precision x, y, xmax, ymax, xmin, ymin, dx, dy

!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     i              Counting variable
!     iloop          Array of size 10 in which information of the loops
!                    is stored in the following way:
!                    iloop(i) = starting address of while loop in inpsep
!     inode          Node sequence number
!     istart         First position used at a part
!     ix             Cell index number
!     iy             Cell index number
!     nstored        number of nodes stored in a cell
!     numnodes       Number of nodal points
!     x              x co-ordinate of point
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     y              y co-ordinate of point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINGRIDINFO   ?
!     PRINNOWBNDCRD  Print the actual boundary during the creation of the mesh
!                    in a specific iteration
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2451   Node outside region
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extract parameters
!     Clear array grid
!     First detect number of nodes per cell
!     Fill first part of array gridinfo and set starting addresses in grid
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavgrid' )
      debug = .false.
      if ( layer.eq.100000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavgrid'
         write(irefwr,*) 'nloop ', nloop
         call prinnowbndcrd ( nowbnd, nloop, maxbounds, layer, crd )

      end if

!     --- Extract parameters

      xmin = rgridinfo(1)
      ymin = rgridinfo(2)
      dx = rgridinfo(3)
      dy = rgridinfo(4)
      xmax = xmin+dx*nx
      ymax = ymin+dy*ny

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'nx, ny ', nx, ny
         write(irefwr,*) 'dx, dy ', dx, dy
         write(irefwr,*) 'xmin, xmax, ymin, ymax ', &
                          xmin, xmax, ymin, ymax

      end if

!     --- Clear array grid

      do i = 1, numgridfilled
         ix = gridfilled(1,i)
         iy = gridfilled(2,i)
         grid(ix,iy) = 0
      end do  ! i = 1, numgridfilled

!     --- First detect number of nodes per cell

      numgridfilled = 0
      do iloop = 1, nloop

         numnodes = nowbnd(iloop,1)

         do i = 1, numnodes

            inode =  abs(nowbnd(iloop,1+i))
            x = crd(1,inode)
            y = crd(2,inode)

            if ( x.lt.xmin .or. x.gt.xmax .or. &
                 y.lt.ymin .or. y.gt.ymax ) then

!           --- Node outside original region

               call errint ( inode, 1 )
               call erreal ( x, 1 )
               call erreal ( y, 2 )
               call errint ( layer, 2 )
               call errint ( iloop, 3 )
               call errint ( i, 4 )
               call errsub ( 2451, 4, 2, 0 )
               go to 1000

            end if

            ix = nint((x-xmin)/dx)
            iy = nint((y-ymin)/dy)
            grid(ix,iy) = grid(ix,iy)+1
            if ( grid(ix,iy).eq.1 ) then

!           --- The first time gridfilled must be filled

               numgridfilled = numgridfilled+1
               gridfilled(1,numgridfilled) = ix
               gridfilled(2,numgridfilled) = iy

            end if

         end do

      end do  ! iloop = 1, nloop

      if ( numgridfilled.gt.lengridfilled ) then

!     --- Error gridfilled too large

         call errint ( numgridfilled, 1 )
         call errint ( lengridfilled, 2 )
         call errchr ( 'gridfilled', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Fill first part of array gridinfo and set starting addresses in
!         grid

      istart = 1
      do i = 1, numgridfilled
         ix = gridfilled(1,i)
         iy = gridfilled(2,i)
         numnodes = grid(ix,iy)
         grid(ix,iy) = istart
         istart = istart+1+numnodes*2
      end do  ! i = 1, numgridfilled

      if ( istart-1.gt.lengridinfo ) then

!     --- Error gridinfo too large

         call errint ( istart-1, 1 )
         call errint ( lengridinfo, 2 )
         call errchr ( 'gridinfo', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      do i = 1, istart-1
         gridinfo(i) = 0
      end do

!     --- Fill gridinfo

      do iloop = 1, nloop

         numnodes = nowbnd(iloop,1)
         do i = 1, numnodes
            inode =  abs(nowbnd(iloop,1+i))
            x = crd(1,inode)
            y = crd(2,inode)
            ix = nint((x-xmin)/dx)
            iy = nint((y-ymin)/dy)
            istart = grid(ix,iy)
            nstored = gridinfo(istart)
            gridinfo(istart) = nstored+1
            gridinfo(istart+nstored*2+1) = iloop
            gridinfo(istart+nstored*2+2) = i
         end do  ! i = 1, numnodes

      end do  ! iloop = 1, nloop

1000  call erclos ( 'mshpavgrid' )
      if ( debug ) then

!     --- Debug information

         call pringridinfo ( nx, ny, grid, gridinfo, nowbnd, &
                             maxbounds, ' ' )
         write(irefwr,*) 'End mshpavgrid'

      end if

      end
