      subroutine mshpavmnchelm ( allowed, nowbnd, maxbounds, crd, &
                                 newbnd, ibndnr, inode1, inode2, &
                                 inode, x1, x2, nloop )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if the line x1-x2 does not intersect one of the boundaries of the
!     mesh or is too close to the boundaries
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     check
!     intersection
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxbounds, nowbnd(maxbounds,*), newbnd(*), ibndnr, inode1, &
              inode2, nloop, inode
      double precision crd(4,*), x1(2), x2(2)
      logical allowed

!     allowed        o    If true the paver algorithm may be applied in this
!                         point, otherwise (false), the node is too close to
!                         a boundary
!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     ibndnr         i    i'th boundary in nowbnd
!     inode          i    Node sequence number
!     inode1         i    First node in edge
!     inode2         i    Second node in edge
!     maxbounds      i    Max number of loops in nowbnd
!     newbnd         i    temp array containing the boundary nodes for one loop
!     nloop          i    number of boundaries in nowbnd
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     x1             i    coordinates of inode1
!     x2             i    coordinates of inode2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iedge, numedges, jnode1, jnode2, jbndnr
      double precision x3(2), x4(2), fact1, fact2, h1, h, distance

!     distance       distance between two quantities
!     fact1          multiplication factor
!     fact2          multiplication factor
!     h              Help variable to store a constant temporarily
!     h1             Help variable to compute the element matrix
!     iedge          edge sequence number
!     jbndnr         jth boundary in nowbnd
!     jnode1         first node in edge
!     jnode2         second node in edge
!     numedges       number of edges
!     x3             coordinates of jnode1
!     x4             coordinates of jnode2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCLOSELINES  Find the minimum distance between the edges x1-x2 and
!                    x3-x4, provided this distance is less than threshold
!     MSHCROSSLINE1  Find the intersection of the lines x1-x2 and x3-x4
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     allowed := .true.
!     iedge := 0
!     while ( allowed and iedge < numedges on ibndnr )
!        iedge := iedge+1
!        if ( inode1 and inode2 not on edge ) then
!           if ( edge intersects x1-x2 ) then
!              allowed := false
!           else ( if edge too close to edge inode1-inode2 ) then
!              allowed := false
!     jbndnr := 0
!     while ( allowed and jbndnr < number of boundaries ) do
!        jbndnr := jbndnr+1
!        if ( jbndnr # ibndnr ) then
!           iedge := 0
!           while ( allowed and iedge < numedges on jbndnr ) do
!              iedge := iedge+1
!              if ( edge intersects x1-x2 ) then
!                 allowed := false
!              else ( if edge too close to edge inode1-inode2 ) then
!                 allowed := false
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmnchelm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmnchelm'
         write(irefwr,*) 'ibndnr, inode1, inode2 ', &
                          ibndnr, inode1, inode2

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- allowed := true ; iedge := 0

      allowed = .true.
      iedge = 0                 ! edge number
      numedges = newbnd(1)-1    ! number of edges along ibndnr
      h1 = max ( crd(3,inode1)*crd(4,inode1), &
                 crd(3,inode2)*crd(4,inode2) )
      if ( h1.le.0d0 ) allowed = .false.  ! both neighbours on old boundary

!     --- while ( allowed and iedge < numedges on ibndnr )

      do while ( allowed .and. iedge.lt.numedges )

!     --- iedge := iedge+1

         iedge = iedge+1
         jnode1 = newbnd(iedge+1)       !first node on edge
         if ( iedge.lt.numedges ) then
            jnode2 = newbnd(iedge+2)    !second node on edge
         else
            jnode2 = newbnd(2)
         end if
         if ( inode1.ne.jnode1 .and. inode1.ne.jnode2 .and. &
              inode2.ne.jnode1 .and. inode2.ne.jnode2 .and. &
              inode.ne.jnode1 .and. inode.ne.jnode2 .and. &
              jnode1.ne.0 .and. jnode2.ne.0 ) then

!        --- inode1 and inode2 not on edge

            x3(1) = crd(1,jnode1)       ! coordinates of jnode1
            x3(2) = crd(2,jnode1)
            x4(1) = crd(1,jnode2)       ! coordinates of jnode2
            x4(2) = crd(2,jnode2)

!           --- Check if lines intersect

            call mshcrossline1 ( x1, x2, x3, x4, fact1, fact2, sqreps )
            if ( fact2.ge.-sqreps .and. fact2.le.1d0+sqreps ) then

!           --- edge intersects x1-x2
!               allowed := false

               allowed = .false.
               if ( debug ) &
                  write(irefwr,*) 'jnode1, jnode2, fact1, fact2 ', &
                                   jnode1, jnode2, fact1, fact2

            else

!           --- edge does not intersect x1-x2
!               check if it is closer than h/2 to other edge

               h = 0.5d0 * max ( crd(3,jnode1)*crd(4,jnode1), &
                                 crd(3,jnode2)*crd(4,jnode2), h1 )

               call  mshcloselines ( x1, x2, x3, x4, h, distance )
               if ( distance.lt.h ) then

!              --- if edge too close to edge inode1-inode2
!                  allowed := false

                  allowed = .false.
                  if ( debug ) &
                     write(irefwr,*) 'jnode1, jnode2, h, distance ', &
                                      jnode1, jnode2, h, distance

               end if  ! ( distance.lt.h )

            end if  ! ( fact2.ge.-sqreps .and. fact2.le.1d0+sqreps )

         end if  ! ( inode1.ne.jnode1 .and. inode1.ne.jnode2 ... )

      end do  ! while ( allowed .and. iedge.lt.numedges )

!     --- jbndnr := 0

      jbndnr = 0

!     --- while ( allowed and jbndnr < number of boundaries ) do

      do while ( allowed .and. jbndnr.lt.nloop )

!     --- jbndnr := jbndnr+1

         jbndnr = jbndnr+1     ! other boundary number
         if ( jbndnr.ne.ibndnr ) then

!        --- jbndnr # ibndnr
!            iedge := 0

            iedge = 0
            numedges = nowbnd(jbndnr,1)-1    ! number of edges along ibndnr

!           --- do while ( allowed and iedge < numedges on jbndnr )

            do while ( allowed .and. iedge.lt.numedges )

!           --- iedge := iedge+1

               iedge = iedge+1
               jnode1 = nowbnd(jbndnr,iedge+1)       !first node on edge
               if ( iedge.lt.numedges ) then
                  jnode2 = nowbnd(jbndnr,iedge+2)    !second node on edge
               else
                  jnode2 = nowbnd(jbndnr,2)
               end if
               if ( inode1.ne.jnode1 .and. inode1.ne.jnode2 .and. &
                    inode2.ne.jnode1 .and. inode2.ne.jnode2 .and. &
                    inode.ne.jnode1 .and. inode.ne.jnode2 .and. &
                    jnode1.ne.0 .and. jnode2.ne.0 ) then

                  x3(1) = crd(1,jnode1)       ! coordinates of jnode1
                  x3(2) = crd(2,jnode1)
                  x4(1) = crd(1,jnode2)       ! coordinates of jnode2
                  x4(2) = crd(2,jnode2)

!                 --- Check if lines intersect

                  call mshcrossline1 ( x1, x2, x3, x4, fact1, fact2, &
                                       sqreps )
                  if ( fact2.ge.-sqreps .and. fact2.le.1d0+sqreps ) then

!                 --- edge intersects x1-x2
!                     allowed := false

                     allowed = .false.
                     if ( debug ) then
                        write(irefwr,*) 'jnode1, jnode2, jbndnr ', &
                                         jnode1, jnode2, jbndnr
                        write(irefwr,*) 'fact1, fact2 ', fact1, fact2
                     end if

                  else

!                 --- edge does not intersect x1-x2
!                     check if it is closer than h/2 to other edge

                     h = 0.5d0 * max ( crd(3,jnode1)*crd(4,jnode1), &
                                       crd(3,jnode2)*crd(4,jnode2), h1 )

                     call  mshcloselines ( x1, x2, x3, x4, h, distance )
                     if ( distance.lt.h ) then

!                    --- if edge too close to edge inode1-inode2
!                        allowed := false

                        allowed = .false.
                        if ( debug ) then
                           write(irefwr,*) 'jnode1, jnode2, jbndnr ', &
                                            jnode1, jnode2, jbndnr
                           write(irefwr,*) 'h, distance ', h, distance
                        end if

                     end if  ! ( distance.lt.h )

                  end if  ! ( fact2.ge.-sqreps .and. fact2.le.1d0+sqreps )

               end if  ! ( inode1.ne.jnode1 .and. inode1.ne.jnode2 ... )

            end do  ! while ( allowed .and. iedge.lt.numedges )

         end if  ! ( jbndnr.ne.ibndnr )

      end do  ! while ( allowed .and. jbndnr.lt.nloop )

1000  call erclos ( 'mshpavmnchelm' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavmnchelm'

      end if

      end
