      subroutine mshpavsizes ( nbndpt, crd, rgridinfo, nx, ny, &
                               nowbnd, imxlps, nloop )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-07-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute minimum and maximum x and y coordinates of region
!     Compute distances dx and dy for square grid covering region
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     grid
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nbndpt, nx, ny, imxlps, nloop, nowbnd(imxlps,*)
      double precision crd(4,nbndpt), rgridinfo(4)

!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     imxlps         i    maximum loops in nowbnd in paver
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     nloop          i    number of boundaries in nowbnd
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     nx             o    number of cells in x direction in array grid
!     ny             o    number of cells in y direction in array grid
!     rgridinfo      o    Array of size 4 containing the following information
!                         about array grid
!                         1:  xmin
!                         2:  ymin
!                         3:  dx
!                         4:  dy
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iloop, numnodes, inode
      double precision xmax, ymax, ds, x, y, xprev, yprev, eps, tmin, &
                       xmin, ymin, dx, dy

!     ds             Minimum distance between two points along a curve
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     eps            Local accuracy
!     i              Counting variable
!     iloop          Sequence number of closed boundary
!     inode          Node sequence number
!     numnodes       Number of nodal points along closed boundary
!     tmin           Minimum thickness
!     x              x co-ordinate of point
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     xprev          x co-ordinate of previous point
!     y              y co-ordinate of point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     yprev          y co-ordinate of previous point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2450   Distance between two nodes on the boundary is 0
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute minimum and maximum coordinates
!     Compute minimum positive thickness
!     Compute minimum distance between succesive nodes on the boundary parts
!     Adapt to minimum positive thickness
!     Compute nx, ny, dx and dy
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavsizes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavsizes'

      end if

!     --- Compute minimum and maximum coordinates
!         Compute minimum positive thickness

      xmin = rinfin
      xmax = -rinfin
      ymin = xmin
      ymax = xmax
      tmin = rinfin
      do i = 1, nbndpt
         xmin = min ( xmin, crd(1,i) )
         xmax = max ( xmax, crd(1,i) )
         ymin = min ( ymin, crd(2,i) )
         ymax = max ( ymax, crd(2,i) )
         if ( crd(3,i).gt.0d0 ) tmin = min(tmin,crd(3,i))
      end do  ! i = 1, nbndpt
      eps = 1d-5*max(xmax-xmin,ymax-ymin)
      xmin = xmin-eps
      xmax = xmax+eps
      ymin = ymin-eps
      ymax = ymax+eps

!     --- Compute minimum distance between succesive nodes on the boundary parts

      ds = rinfin
      do iloop = 1, nloop
         numnodes = nowbnd(iloop,1)
         inode = nowbnd(iloop,2)
         xprev = crd(1,inode)
         yprev = crd(2,inode)
         do i = 2, numnodes
            inode =  nowbnd(iloop,2+i)
            x = crd(1,inode)
            y = crd(2,inode)
            ds = min(ds,(x-xprev)**2+(y-yprev)**2)
            xprev = x
            yprev = y
         end do
      end do  ! iloop = 1, nloop

      if ( ds.le.0d0 ) then
         call errsub ( 2450, 0, 0, 0 )
         go to 1000
      end if
      ds = sqrt(ds)

!     --- Adapt to minimum positive thickness

      ds = nbndpt
      iloop = min(100,nint(sqrt(ds))+1)
      if ( debug ) write(irefwr,*) 'ds, tmin ', ds, tmin
      ds = min ( ds, tmin )

!     --- Compute nx, ny, dx and dy

      nx = min(iloop,nint((xmax-xmin)/ds)+1)
      ny = min(iloop,nint((ymax-ymin)/ds)+1)
      dx = (xmax-xmin)/nx
      dy = (ymax-ymin)/ny

!     --- Store indo in rgridinfo

      rgridinfo(1) = xmin
      rgridinfo(2) = ymin
      rgridinfo(3) = dx
      rgridinfo(4) = dy

1000  call erclos ( 'mshpavsizes' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'xmin, xmax, ymin, ymax ', &
                          xmin, xmax, ymin, ymax
         write(irefwr,*) 'tmin, ds ', tmin, ds
         write(irefwr,*) 'nx, ny, dx, dy ', nx, ny, dx, dy
         write(irefwr,*) 'End mshpavsizes'

      end if

      end
