      subroutine mshpipboun ( iinput, iposi, nsubsf, ipipetopol, &
                              ibottopol, ibotboun )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 04-02-2015 Extra debug
!        version  3.1  date 27-11-2012 Long integers
!        version  3.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 2001-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read boundary for the generation of pipe
!     Check if bottom and top are topologically equivalent
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     check
!     surface
!     pipe
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iposi, &
              nsubsf, ipipetopol(*), ibottopol(*), ibotboun(*)

!     ibotboun       o    Contains node numbers of boundary of bottom surface
!     ibottopol      o    Contains topology of bottom surface
!     iinput         i    integer input array    see subroutine mesh
!                         iinput contains only the part with respect to the
!                         definition of the volumes
!     ipipetopol     o    Contains topology of pipe surface
!     iposi          i    next free position of iinput
!     nsubsf         i    Number of subsurfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer leng, istart, jsur, &
              jstart, isubsf, isur, index, leng2, npoloc, nelloc, &
              ikelmc, ih1, i, k, &
              jsurnr, inpelmbot, inpelmsur, ishapebot
      integer, allocatable :: ilocnum(:), iloctopol(:), ilockmeshc(:)
      logical trian, prism

!     i              Counting variable
!     ih1            Help variable to store a constant temporarily
!     ikelmc         position in kmesh part c
!     ilockmeshc     Work array to store local kmeshc
!     ilocnum        Work array to store local numbering
!     iloctopol      Work array to store local topology
!     index          Position in array
!     inpelmbot      Number of nodes per element of bottom surface and
!                    top surface of pipe
!     inpelmsur      Number of nodes per element of pipe surface
!     ishapebot      Shape number of elements in bottom and top surface
!                    of pipe
!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
!     isur           surface number
!     jstart         starting position of curve i in kmeshm
!     jsur           surface number provided with sign
!     jsurnr         Surface sequence number
!     k              Counting variable
!     leng           Help parameter to indicate the length of arrays
!     leng2          length indicator elements
!     nelloc         Local number of elements
!     npoloc         Number of nodes in subsurface
!     prism          If true prismatic elements are used
!     trian          Indication whether triangles (true) or quadrilaterals
!                    are created (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHPIPPARMS    Compute some parameters for a pipe as function of the shape
!                    number of the elements
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2405   Top and bottom of pipe are not topologically equivalent
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read information from bottom surface
!     Read boundary of surface
!     Read kmeshc corresponding to top surface
!     Compare top and bottom
!     Read kmeshc corresponding to pipe surface
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpipboun' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpipboun'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute shape numbers of surface elements

      call mshpipparms ( ishape, inpelmbot, inpelmsur, trian, &
                         prism, inpelm, linear, ishapebot )
      if ( ierror/=0 ) go to 1000

!     --- Read information from bottom surface and store in ibottopol

      isurnr = mod(iinput(iposi+3)-1,nsurfs)+1
      jsurnr = isurnr

      if ( kelmr/=0 ) nsubsf = kmeshr(isurnr+1)-kmeshr(isurnr)

      if ( debug ) write(irefwr,1) 'bottom surface', isurnr, nsubsf

      if ( nsubsf==0 ) then

!     --- nsubsf = 0, No subsurfaces

         leng = size(kmhsur(isurnr)%kmeshcsur)
         ibottopol(1:leng) = kmhsur(isurnr)%kmeshcsur

      else

!     --- Surface consists of subsurfaces
!         Read information of subsurfaces and store in ibottopol

         istart = kmeshr(isurnr)
         jstart = kmeshr(nsurfs+1) - nsurfs-1 + istart
         do isubsf = 1, nsubsf
            jsur = kmeshr(istart+isubsf)
            isur = abs(jsur)
            index = kmeshr(jstart+isubsf)
            npoloc = kmeshh(1,isur+1)
            nelloc = kmeshh(2,isur+1)
            leng2 = inpelmbot*nelloc

            if ( debug ) write(irefwr,1) 'isubsf, isur', isubsf, isur

!           --- Read local numbering and store in array ilocnum of length
!               npoloc+nelloc

            allocate ( ilocnum(npoloc+nelloc), &
                       iloctopol(leng2), stat = error )
            if ( error/=0 ) then

!           --- error#0, so not enough space

               call errint ( npoloc+nelloc+leng2, 1 )
               call errint ( error, 2 )
               call errchr ( 'ilocnum', 1 )
               call errsub ( 2910, 2, 0, 1 )
               go to 1000

            end if  ! ( error/= 0 )

            call readfl ( index, 1, npoloc+nelloc, 1, 3, ilocnum )

            if ( debug ) then

!           --- debug = true, debug statements

               write(irefwr,*) 'Bottom surface: isur ',isur
               call prinin ( ilocnum, npoloc, &
                             'node numbers' )
               call prinin ( ilocnum(1+npoloc), nelloc, &
                             'element numbers' )

            end if

!           --- Read local array kmeshc and store in iloctopol

            iloctopol(1:leng2) = kmhsur(isur)%kmeshcsur(1:leng2)
            if ( debug ) then

!           --- debug = true, debug statements

               call prinin1 ( iloctopol, nelloc, inpelmbot, &
                              'Local array kmeshc-bottom' )

            end if

!           --- Put local array kmeshc into global kmeshc

            do i = 1, nelloc
               ikelmc = (ilocnum(npoloc+i)-1 ) * inpelmbot
               ih1 = ( i-1 ) * inpelmbot
               if ( jsur>0 ) then

!              --- jsur>0, standard case, positive direction

                  ibottopol(ikelmc+1:ikelmc+inpelmbot) = &
                     ilocnum(iloctopol(ih1+1:ih1+inpelmbot))

               else

!              --- jsur<0, special case, reversed direction

                  if ( ishapebot<=5 ) then

!                 --- ishapebot = 3, 4, 5 all points must be reversed
!                     start with first one

                     ibottopol(ikelmc+1) = ilocnum(iloctopol(ih1+1))
                     do k = 2, inpelmbot
                        ibottopol(ikelmc+inpelmbot+2-k) = &
                           ilocnum(iloctopol(ih1+k) )
                     end do

                  else

!                 --- ishapebot>5 last point must be kept

                     ibottopol(ikelmc+1) = ilocnum(iloctopol(ih1+1))
                     do k = 2, inpelmbot-1
                        ibottopol(ikelmc+inpelmbot+1-k) = &
                           ilocnum(iloctopol(ih1+k))
                     end do
                     k = inpelmbot
                     ibottopol(ikelmc+k) = ilocnum(iloctopol(ih1+k))

                  end if

               end if

            end do

            deallocate ( ilocnum, iloctopol, stat = error )
            if ( error/= 0 ) then

!           --- error#0, problem with deallocate

               call errint ( error, 1 )
               call errchr ( 'ilocnum', 1 )
               call errsub ( 2911, 1, 0, 1 )
               go to 1000

            end if  ! ( error/= 0 )

         end do  ! isubsf = 1, nsubsf

      end if

      if ( debug ) then

!     --- debug = true, debug statements

         call prinin1 ( ibottopol, leng/inpelmbot, inpelmbot, &
                        'Global array kmeshc-bottom' )

      end if

!     --- Read boundary of surface and store in ibotboun

      leng = kmeshh(4,isurnr+1)
      ibotboun(1:leng) = kmhsur(isurnr)%kbndptsur(1:leng)

!     --- Read kmeshc corresponding to top surface
!         Store temporarily in iloctopol
!         This information is used to check if bottom and top are
!         topologically equivalent

      isurnr = mod(iinput(iposi+4)-1,nsurfs)+1

      if ( kelmr/=0 ) nsubsf = kmeshr(isurnr+1)-kmeshr(isurnr)
      leng = kmeshh(2,isurnr+1)*kmeshh(6,isurnr+1)
      allocate ( iloctopol(leng), stat = error )

      if ( debug ) write(irefwr,1) 'top surface', isurnr, nsubsf

      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( leng, 1 )
         call errint ( error, 2 )
         call errchr ( 'iloctopol', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

      if ( nsubsf==0 ) then

!     --- nsubsf = 0, No subsurfaces

         iloctopol = kmhsur(isurnr)%kmeshcsur

      else

!     --- Surface consists of subsurfaces
!         Read information of subsurfaces and store in iloctopol

         istart = kmeshr(isurnr)
         jstart = kmeshr(nsurfs+1) - nsurfs-1 + istart
         do isubsf = 1, nsubsf
            jsur = kmeshr(istart+isubsf)
            isur = abs(jsur)
            index = kmeshr(jstart+isubsf)
            npoloc = kmeshh(1,isur+1)
            nelloc = kmeshh(2,isur+1)
            leng2 = inpelmbot*nelloc

            if ( debug ) write(irefwr,1) 'isubsf, jsur', isubsf, jsur

            allocate ( ilocnum(npoloc+nelloc), &
                       ilockmeshc(leng2), stat = error )
            if ( error/=0 ) then

!           --- error#0, so not enough space

               call errint ( npoloc+nelloc+leng2, 1 )
               call errint ( error, 2 )
               call errchr ( 'ilocnum', 1 )
               call errsub ( 2910, 2, 0, 1 )
               go to 1000

            end if  ! ( error/= 0 )

!           --- Read local numbering and store in array ilocnum

            call readfl ( index, 1, npoloc+nelloc, 1, 3, ilocnum )
            if ( debug ) then

!           --- debug = true, debug statements

               write(irefwr,*) 'Top surface: isur ',isur
               call prinin ( ilocnum, npoloc, 'node numbers' )
               call prinin ( ilocnum(1+npoloc), nelloc, &
                             'element numbers' )

            end if

!           --- Read local array kmeshc and store in ilockmeshc

            ilockmeshc(1:leng2) = kmhsur(isur)%kmeshcsur
            if ( debug ) then

!           --- debug = true, debug statements

               call prinin1 ( ilockmeshc, leng2/inpelmbot, inpelmbot, &
                              'Local array kmeshc-top' )

            end if

!           --- Put local array kmeshc into global kmeshc

            do i = 1, nelloc
               ikelmc = (ilocnum(npoloc+i)-1 ) * inpelmbot
               ih1 = ( i-1 ) * inpelmbot
               if ( jsur>0 ) then

!              --- jsur>0, standard case, positive direction

                  do k = 1, inpelmbot
                     iloctopol(ikelmc+k) = ilocnum(ilockmeshc(ih1+k))
                  end do

               else

!              --- jsur<0, special case, reversed direction

                  if ( ishapebot<=5 ) then

!                 --- ishapebot = 3, 4, 5 all points must be reversed

                     iloctopol(ikelmc+1) = ilocnum(ilockmeshc(ih1+1))
                     do k = 2, inpelmbot
                        iloctopol(ikelmc+inpelmbot+2-k) = &
                           ilocnum(ilockmeshc(ih1+k) )
                     end do

                  else

!                 --- ishapebot>5 last point must be kept

                     iloctopol(ikelmc+1) = ilocnum(ilockmeshc(ih1+1))
                     do k = 2, inpelmbot-1
                        iloctopol(ikelmc+inpelmbot+1-k) = &
                           ilocnum(ilockmeshc(ih1+k) )
                     end do
                     k = inpelmbot
                     iloctopol(ikelmc+k) = ilocnum(ilockmeshc(ih1+k))

                  end if

               end if

            end do

            deallocate ( ilocnum, ilockmeshc, stat = error )
            if ( error/= 0 ) then

!           --- error#0, problem with deallocate

               call errint ( error, 1 )
               call errchr ( 'ilocnum', 1 )
               call errsub ( 2911, 1, 0, 1 )
               go to 1000

            end if  ! ( error/= 0 )

         end do  ! isubsf = 1, nsubsf

         if ( debug ) &
            call prinin1 ( iloctopol, leng/inpelmbot, inpelmbot, &
                        'Global array kmeshc-top' )

      end if

!     --- Compare top and bottom surface

      do i = 1, leng
         if ( ibottopol(i)/=iloctopol(i) ) then

!        --- bottom and top surface are not topological equivalent

            ih1 = mod(i,inpelmbot)
            if ( ih1==0 ) ih1 = inpelmbot
            call errint ( isurnr, 1 )
            call errint ( jsurnr, 2 )
            call errint ( ivolnr, 3 )
            call errint ( ih1, 4 )
            call errint ( (i-1)/inpelmbot+1, 5 )
            call errint ( ibottopol(i), 6 )
            call errint ( iloctopol(i), 7 )
            call errsub ( 2405, 7, 0, 0 )
            call prinin1 ( ibottopol, leng/inpelmbot, inpelmbot, &
                          'bottom-elements')
            call prinin1 ( iloctopol, leng/inpelmbot, inpelmbot, &
                         'top-elements')
            go to 1000

         end if
      end do

      deallocate ( iloctopol, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'iloctopol', 1 )
         call errsub ( 2911, 1, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

!     --- Read kmeshc corresponding to pipe surface
!         Store in ipipetopol

      isurnr = mod(iinput(iposi+5)-1,nsurfs)+1
      if ( kelmr/=0 ) nsubsf = kmeshr(isurnr+1)-kmeshr(isurnr)

      if ( nsubsf==0 ) then

!     --- nsubsf = 0, No subsurfaces

         leng = size(kmhsur(isurnr)%kmeshcsur)
         ipipetopol(1:leng) = kmhsur(isurnr)%kmeshcsur

      else

!     --- Surface consists of subsurfaces
!         Read information of subsurfaces and store in ipipetopol

         istart = kmeshr(isurnr)
         jstart = kmeshr(nsurfs+1) - nsurfs-1 + istart
         do isubsf = 1, nsubsf
            jsur = kmeshr(istart+isubsf)
            isur = abs(jsur)
            index = kmeshr(jstart+isubsf)
            npoloc = kmeshh(1,isur+1)
            nelloc = kmeshh(2,isur+1)
            leng2 = inpelmsur*nelloc

            allocate ( ilocnum(npoloc+nelloc), &
                       ilockmeshc(leng2), stat = error )
            if ( error/=0 ) then

!           --- error#0, so not enough space

               call errint ( npoloc+nelloc+leng2, 1 )
               call errint ( error, 2 )
               call errchr ( 'ilocnum', 1 )
               call errsub ( 2910, 2, 0, 1 )
               go to 1000

            end if  ! ( error/= 0 )

!           --- Read local numbering and store in array iwork of length
!               npoloc

            call readfl ( index, 1, npoloc+nelloc, 1, 3, &
                          ilocnum )
            if ( debug ) then

!           --- debug = true, debug statements

               write(irefwr,*) 'Pipe surface: isur ',isur
               call prinin ( ilocnum, npoloc, &
                             'node numbers' )
               call prinin ( ilocnum(1+npoloc), nelloc, &
                             'element numbers' )

            end if

!           --- Read local array kmeshc and store in iwork

            ilockmeshc(1:leng2) = kmhsur(isur)%kmeshcsur
            if ( debug ) then

!           --- debug = true, debug statements

               call prinin1 ( ilockmeshc, nelloc, inpelmsur, &
                              'Local array kmeshc-pipesurface' )

            end if

!           --- Put local array kmeshc into global kmeshc

            do i = 1, nelloc
               ikelmc = (ilocnum(npoloc+i)-1 ) * inpelmsur
               ih1 = ( i-1 ) * inpelmsur
               do k = 1, inpelmsur
                  ipipetopol(ikelmc+k) = ilocnum(ilockmeshc(ih1+k) )
               end do
            end do

            deallocate ( ilocnum, ilockmeshc, stat = error )
            if ( error/= 0 ) then

!           --- error#0, problem with deallocate

               call errint ( error, 1 )
               call errchr ( 'ilocnum', 1 )
               call errsub ( 2911, 1, 0, 1 )
               go to 1000

            end if  ! ( error/= 0 )

         end do

      end if

      if ( debug ) then

!     --- debug = true, debug statements

         call prinin1 ( ipipetopol, leng/inpelmsur, inpelmsur, &
                        'Global array kmeshc-pipe surface' )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpipboun'

      end if  ! ( debug )
      call erclos ( 'mshpipboun' )

      end subroutine mshpipboun
