      subroutine mshpipquadtets ( npobot, nelbot, kmeshc, ikelm, iwork, &
                                  kmscbt, klp, nlp, kmscps, istep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 20-06-2013 Correction mid points
!        version  1.0  date 11-12-2002
!
!   copyright (c) 2002-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill bottom layer of elements in case of a pipe consisting of
!     quadratic tetrahedrons
!     Each wedge is subdivided into 8 tetrahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     element
!     topology
!     pipe
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istep, npobot, nelbot, kmeshc(10,*), ikelm, iwork(*), &
              kmscbt(*), klp(2,*), nlp, kmscps(*)

!     ikelm         i/o   Last position used in kmeshc
!     istep          i    Step
!     iwork               integer work array
!     klp            i    In this array the relative node numbers with respect
!                         to array kbndbt of first and last point of a
!                         consecutive part are stored. The number
!                         of parts the boundary consists of is nlp
!                         Hence klp(1,1) = 1, klp(2,1) denotes the end point of
!                         the first part of the boundary
!                         klp(1,2) contains the node number of the first point
!                         of the second part. This must be klp(2,1)+1
!                         klp(2,nlp) contains the length of kbndbt
!     kmeshc        i/o    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmscbt         i    Kmesh part c corresponding to bottom surface
!                         Contains the local nodal point numbers of the bottom
!                         surface
!     kmscps         i    Kmesh part c corresponding to pipe surface
!                         This array is required to check the direction of
!                         diagonals in the pipe surface
!     nelbot         i    Number of elements in bottom surface
!     nlp            i    Number of disjoint parts the boundary of the bottom
!                         surface consists of
!     npobot         i    Number of points of bottom surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mcentr, ikelbt, iielem, k, i1(3), i, j1, j2, i2(6), &
              iunder, iupper, ind(3), ielpsf, ikelsf, ilp, ihalf, m1, &
              n1, icount
      logical bound(3), small

!     bound          Indicates if boundary elements are present (true) or not
!                    (false)
!     i              Counting variable
!     i1             Contains 3 vertices of a bottom  triangle
!     i2             Contains 6 nodes of a bottom riangle
!     icount         Counter
!     ielpsf
!     ihalf
!     iielem         Relative element number in element group
!     ikelbt         Last position used in kmscbt
!     ikelsf
!     ilp            Counting variable
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3,4
!     iunder         Lower bound of array
!     iupper         Upper bound of array
!     j1             Lower index number along second direction of a face
!     j2             Upper index number along second direction of a face
!     k              Counting variable
!     m1
!     mcentr         Counter for centroids of wedges
!     n1
!     small          Parameter small indicates the direction:
!                    small=true:  from i1(1) to i1(2)+istep
!                    small=false: from i1(2) to i1(1)+istep
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind /2,3,1/
! ======================================================================
!
      call eropen ( 'mshpipquadtets' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpipquadtets'
         write(irefwr,1) 'npobot, nelbot, ikelm', npobot, nelbot, ikelm
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istep = 2*npobot+7*nelbot
      ihalf = npobot+3*nelbot
      mcentr = 2*npobot+3*nelbot
      m1 = npobot
      n1 = 2*npobot+4*nelbot
      ikelbt = 0
      icount = 1

      if ( debug ) &
         write(irefwr,1) 'istep, ihalf, mcentr, m1, n1', &
                          istep, ihalf, mcentr, m1, n1
      do iielem = 1,nelbot

!     --- Loop over elements in bottom cross-section

         mcentr = mcentr+1
         do k = 1, 6
            i2(k) = kmscbt(ikelbt+k)
         end do
         do k = 1, 3
            i1(k) = i2(2*k-1)
         end do
         ikelbt = ikelbt+6

         if ( debug ) then
            write(irefwr,1) 'iielem, mcentr, ikelbt', &
                             iielem, mcentr, ikelbt
            call prinin ( i2, 6, 'i2' )
            call prinin ( i1, 3, 'i1' )
         end if  ! ( debug )

!        --- mcentr contains the node number of the centroid point
!            i2 contains the node numbers of the corresponding triangle
!            in the bottom surface
!            i1 contains the vertex numbers of the corresponding triangle
!            in the bottom surface
!            m1 contains the starting number of the midpoints between
!            centroid and bottom surface
!            n1 contains the starting number of the midpoints between
!            centroid and top surface
!            Check if sides belong to outer boundary or not

         do k = 1, 3
            bound(k) = iwork(i1(k))>0 .and. iwork(i1(ind(k)))>0
         end do

         if ( debug ) write(irefwr,*) 'bound ', bound(1:3)

!        --- Check if nodes on a side are subsequent boundary nodes
!            Only then we have an actual boundary

         do k = 1,3
            if ( bound(k) ) then

!           --- bound(k) = true, hence side k contains two boundary points

               j1 = iwork( i1(k) )
               j2 = iwork( i1(ind(k)) )
               iunder = min ( j1,j2 )
               iupper = max ( j1,j2 )
               if ( iupper/=iunder+2 ) then
                  bound(k) = .false.
                  do ilp = 1, nlp
                     if (iunder==klp(1,ilp)-ilp+1 .and. &
                         iupper==klp(2,ilp)-ilp+1-1) then

!                    --- store loop for this element

                        bound(k) = .true.

                     end if
                  end do
               end if
            end if
         end do

         if ( debug ) write(irefwr,*) 'bound/2 ', bound(1:3)

!        --- Fill elements
!            All elements have the centroid point mcentr as end point

         do k = 1, 8
            kmeshc(10,k+icount-1) = mcentr
         end do

!        --- Under and upper element

         do k = 1, 3
            kmeshc(6+k,icount) = m1+ind(4-k)
            kmeshc(6+k,icount+1) = n1+k
         end do
         kmeshc(1,icount) = i2(1)
         kmeshc(1,icount+1) = i2(1)+istep
         do k = 2, 6
            kmeshc(k,icount) = i2(8-k)
            kmeshc(k,icount+1) = i2(k)+istep
         end do
         icount = icount+2

!        --- Side elements

         do i = 1,3

!        --- Test direction of diagonal
!            Parameter small indicates the direction:
!            small = true:  from i1(1) to i1(2)+istep
!            small = false: from i1(2) to i1(1)+istep

            if ( bound(i) ) then

!           --- bound(i) true, hence boundary i is outer boundary
!               in that case the sequence in kbndbt gives the boundary
!               direction
!               ielpsf is minimum node number at pipe surface
!               ikelsf is start adress in kmscps minus one

               j1 = iwork( i1(i) )
               j2 = iwork( i1(ind(i)) )
               iunder = min ( j1,j2 )
               iupper = max ( j1,j2 )
               if ( iupper==iunder+2 ) then
                  ielpsf = (j1+1)/2
               else if ( iupper==klp(2,ilp)-ilp+1-1 .and. &
                         iunder==klp(1,ilp)-ilp+1 ) then
                  ielpsf = (iupper+1)/2
                  j2 = iupper+2
               else
                  ielpsf = (j2+1)/2
                  j2 = j2+2
               end if
               ikelsf = 12 * (ielpsf-1)
               if ( kmscps(ikelsf+7)==j2 ) then
                  small = .true.
               else
                  small = .false.
               end if
            else

!           --- internal boundary; check smallest node

               small = i1(i)<i1(ind(i))

            end if

            if ( debug ) write(irefwr,*) 'small ', small
            if ( small ) then

!           --- small = true

               kmeshc(1,icount) = i2(2*i-1)
               kmeshc(2,icount) = i2(2*i)+ihalf
               kmeshc(3,icount) = i2(2*ind(i)-1)+istep
               kmeshc(4,icount) = i2(2*i)+istep
               kmeshc(5,icount) = i2(2*i-1)+istep
               kmeshc(6,icount) = i2(2*i-1)+ihalf
               kmeshc(7,icount) = m1+i
               kmeshc(8,icount) = n1+ind(i)
               kmeshc(9,icount) = n1+i

               kmeshc(1,icount+1) = i2(2*ind(i)-1)
               kmeshc(2,icount+1) = kmeshc(1,icount+1)+ihalf
               kmeshc(3,icount+1) = kmeshc(3,icount)
               kmeshc(4,icount+1) = kmeshc(2,icount)
               kmeshc(5,icount+1) = kmeshc(1,icount)
               kmeshc(6,icount+1) = i2(2*i)
               kmeshc(7,icount+1) = m1+ind(i)
               kmeshc(8,icount+1) = n1+ind(i)
               kmeshc(9,icount+1) = m1+i

            else

!           --- small = false

               kmeshc(1,icount) = i2(2*i-1)
               kmeshc(2,icount) = i2(2*i)
               kmeshc(3,icount) = i2(2*ind(i)-1)
               kmeshc(4,icount) = kmeshc(2,icount)+ihalf
               kmeshc(5,icount) = kmeshc(1,icount)+istep
               kmeshc(6,icount) = kmeshc(1,icount)+ihalf
               kmeshc(7,icount) = m1+i
               kmeshc(8,icount) = m1+ind(i)
               kmeshc(9,icount) = n1+i

               kmeshc(1,icount+1) = kmeshc(3,icount)
               kmeshc(2,icount+1) = kmeshc(3,icount)+ihalf
               kmeshc(3,icount+1) = kmeshc(3,icount)+istep
               kmeshc(4,icount+1) = kmeshc(2,icount)+istep
               kmeshc(5,icount+1) = kmeshc(5,icount)
               kmeshc(6,icount+1) = kmeshc(4,icount)
               kmeshc(7,icount+1) = m1+ind(i)
               kmeshc(8,icount+1) = n1+ind(i)
               kmeshc(9,icount+1) = n1+i
            end if
            icount = icount+2
         end do
         m1 = m1+3
         n1 = n1+3
      end do

      ikelm = ikelm+10*(icount-1)

1000  call erclos ( 'mshpipquadtets' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ikelm', ikelm
         call prinin1 ( kmeshc, ikelm/10, 10, 'kmeshc' )
         write(irefwr,*) 'End mshpipquadtets'

      end if  ! ( debug )

      end
