      subroutine mshpln ( coor, npoint, plane )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 09-09-2013 Use prsolmat3
!        version  1.1  date 12-03-1995 Remove reversing of sign
!        version  1.0  date 12-11-1994
!
!   copyright (c) 1994-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Approximate a set of co-ordinates by a plane using a least squares
!   approximation
! **********************************************************************
!
!                       KEYWORDS
!
!     approximation
!     least_squares
!     plane
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint
      double precision coor(3,npoint), plane(4)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     npoint  i    Number of nodal points
!     plane   o    Array of length 4 containing the parameters a, b, c and d
!                  describing the plane: ax+by+cz+d = 0
!                  plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d;
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ind(5), imax, iy, iz
      double precision s(3,3), f(3), cnorm, x1(3), x2(3), t1(3), &
                       t2(3), h, dist, sol(3)
      logical singular

!     cnorm   Help variable to compute a norm
!     dist    Help variable to compute a distance or an angle
!     f       right-hand-side vector of system of equations to be solved
!     h       Help variable to store a constant temporarily
!     i       Counting variable
!     imax    Index of position in array for computation of maximum
!     ind     Integer array containing the cyclic permutation of 1,2,3,4,5
!     iy      Sequence number after imax (cyclic) with respect to normal
!     iz      Sequence number after iy (cyclic) with respect to normal
!     j       Counting variable
!     s       matrix of system of equations to be solved
!     singular       if true the matrix is singular
!     sol            solution of system of equations
!     t1      First tangential vector
!     t2      Second tangential vector
!     x1      Contains coordinates of first point
!     x2      Contains coordinates of second point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     PRSOLMAT3      Solve system of 3 linear equations S x = f
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Method:
!    Define the plane by a1x + a2y + a3z + a4 = 0
!    Since a, b, c and d are not independent some division is necessary
!    in order to set one of the constants equal to one.
!    To that end we define P1 as point 1
!    P2 is the point with largest distance to P1
!    t1 is the line P1-P2
!    Find P3 such that the angle of the line P1-P3 with t1 is maximal
!    t2 is the line P1-P3
!    Then the first approximation of n is t1 x t2
!    Let imax be the index of the maximal component of n (in absolute value)
!    Then we set a(imax) = 1 and the plane can be written as
!    x(imax) = -a(imax+1)x(imax+1)-a(imax+2)x(imax+2)-a(4) (Cyclic)
!    The residual r  = x(imax) - a(imax+1)x(imax+1) - a(imax+2)x(imax+2) -a4
!                  i    i                  i           i
!    For shortness of notation we set:
!    x(imax) = x,  x(imax+1)=y and x(imax+2)=z
!    a(imax+1) = a,  a(imax+2)=b and a(4)=c
!    Hence:
!    The residual r  = -x - ay - bz -c
!                  i     i    i    i
!    Least squares approximation:
!    z r  = 0  y r  = 0  r  = 0   (summation over i)
!     i i       i i   i   i
!    Hence:   |  s11   s12   s13  |   | a |    | f1 |
!             |  s21   s22   s23  |   | b | =  | f2 |
!             |  s31   s32   s33  |   | c |    | f3 |
!    with s11 = sum y^2     s12 = sum y z    s13 = sum y
!                i   i             i   i i          i   i
!         s22 = sum z^2     s23 = sum z             s33 = n
!                i   i             i   i
!         s21 = s12  s31 = s13  s32 = s23
!         f1 = -sum x y      f2 = -sum x z       f3 = -sum x
!                i   i i            i   i               i   i
!    a, b and c are solved by Cramers rule
!    a(i) are normalized such that sum a(i)^2=1
!    Pseudo code:
!    Find vectors t1 and t2
!    Compute estimate of normal
!    Find largest component of normal
!    Compute matrix elements and right-hand side
!    Solve system of equations
!    Compute a, b, c and d
!    Normalize a, b c and d
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind / 1, 2, 3, 1, 2 /
! ======================================================================
!
!     --- Find vectors t1 and t2

      do 100 i = 1, 3
         x1(i) = coor(i,1)
100   continue
      dist = 0d0
      do 150 j = 2, npoint
         do 120 i = 1, 3
            x2(i) = coor(i,j)
120      continue
         h = (x2(1)-x1(1))**2+(x2(2)-x1(2))**2+(x2(3)-x1(3))**2
         if ( h>dist ) then
            dist = h
            imax = j
         end if
150   continue
      do 160 i = 1, 3
         t1(i) = coor(i,imax)-x1(i)
160   continue
      h = 1d0/sqrt(t1(1)**2+t1(2)**2+t1(3)**2)
      do 170 i = 1, 3
         t1(i) = t1(i)*h
170   continue
      dist = 1d2
      do 190 j = 2, npoint
         do 180 i = 1, 3
            t2(i) = coor(i,j)-x1(i)
180      continue
         h = sqrt(t2(1)**2+t2(2)**2+t2(3)**2)
         if ( h>0d0 ) then
            h = abs(t1(1)*t2(1)+t1(2)*t2(2)+t1(3)*t2(3))/h
            if ( h<dist ) then
               dist = h
               imax = j
            end if
         end if
190   continue
      do 200 i = 1, 3
         t2(i) = coor(i,imax)-x1(i)
200   continue
      h = 1d0/sqrt(t2(1)**2+t2(2)**2+t2(3)**2)
      do 210 i = 1, 3
         t2(i) = t2(i)*h
210   continue

!     --- n = t1 x t2

      call eloutr ( t1, t2, x2 )
      dist = 0d0
      do 220 i = 1, 3
         if ( abs(x2(i))>dist ) then
            dist = abs(x2(i))
            imax = i
         end if
220   continue

!     --- Compute matrix elements and right-hand side

      do 310 i = 1, 3
         do 300 j = 1, 3
            s(i,j) = 0d0
300      continue
         f(i) = 0d0
310   continue
      iy = ind(imax+1)
      iz = ind(imax+2)
      do 330 i = 1, npoint
         s(1,1) = s(1,1) + coor(iy,i)**2
         s(1,2) = s(1,2) + coor(iy,i)*coor(iz,i)
         s(1,3) = s(1,3) + coor(iy,i)
         s(2,2) = s(2,2) + coor(iz,i)**2
         s(2,3) = s(2,3) + coor(iz,i)
         f(1)   = f(1) - coor(imax,i)*coor(iy,i)
         f(2)   = f(2) - coor(imax,i)*coor(iz,i)
         f(3)   = f(3) - coor(imax,i)
330   continue
      s(2,1) = s(1,2)
      s(3,1) = s(1,3)
      s(3,2) = s(2,3)
      s(3,3) = npoint

!     --- Solve system of equations
!         Compute a, b and c

      call prsolmat3 ( s, f, sol, singular )
      plane(imax) = 1d0
      plane(iy)   = sol(1)
      plane(iz)   = sol(2)
      plane(4)    = sol(3)

!     --- Normalize a, b c and d

      cnorm = sqrt(plane(1)**2+plane(2)**2+plane(3)**2)
      do 350 i = 1, 4
         plane(i) = plane(i)/cnorm
350   continue
      end
