      subroutine mshr00 ( nuspnt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.11 date 10-01-2015 New call to readreal
!        version  2.10 date 08-12-2014 Minor change
!        version  2.9  date 05-03-2008 New definition of constants
!
!   copyright (c) 1990-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Interpret information in curve record after the part
!   Ci = line j ( ...    etc. or Ci = reflect j ( ...
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nuspnt

!     nuspnt         i    Number of user points
!     work          o    Array containing the output of this subroutine
!                         work is filled as follows
!                         1:    n  (number of user points)
!                         2:    nelm
!                         3:    iratio
!                         4:    factor
!                         5:    nodd
!                         6:    init
!                         7:    end
!                         8:    type
!                         9:    tang(1)
!                        10:    tang(2)
!                        11:    (= 1 if AXIS was found, =2 if RPLANE was found)
!                        12:    alpha
!                        13:    shape (1=upper_naca0012, 2=lower_naca0012)
!                        14:    nsubcv (at most 100)
!                        15:    ifunc
!                        16-24: Not yet used
!                        25-24+n:  User point numbers (At most 975)
!                        1001-1100: factors t for extra user points
!                        1101-1200: factors nelm for subcurves
!                        1201-1300: ratio for subcurves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nncodelc
      parameter ( nncodelc=16 )
      character (len=9) codelc(nncodelc)
      integer k, n, itang, nelm, ifac, iratio, ivaltype, iend, init, nodd, &
              ichoice, number, minsgn, ncodelc(nncodelc), ncodmxlc(nncodelc), &
              ishape, ifunc
      logical :: finished, openfound

!     codelc         Table with keywords
!     finished       Indicates if loop is finished (true) or not (false)
!     ichoice        Choice parameter, indicating what type of word has been
!                    found, It contains the entry in codelc found
!     iend           Indication if end has already been read (>0) or not (0)
!     ifac           Indication if the factor has already been filled (>0) or
!                    not (0)
!     ifunc          Indication if ifunc has already been read (>0) or not (0)
!     init           Indication if init has already been read (>0) or not (0)
!     iratio         Value of ratio read, or indication that ratio has been read
!                    already (>0)
!     ishape         Value of shape read, or indication that shape has been read
!                    already (>0)
!     itang          Number of tangents that have been read
!     ivaltype       Value of type read, or indication that type has been read
!                    already (>0)
!     k              Loop variable
!     minsgn         Sign that has been read. If no sign is read minsgn = 1, &
!                    if a - has been found minsgn = -1
!     n              Number of user points read
!     ncodelc        Table of significant characters in codelc
!     ncodmxlc       Table containing the number of maximum characters in
!                    the keyword table codelc
!     nelm           Number of elements along a curve
!     nncodelc       Length of the arrays codelc and ncodelc
!     nodd           Value of NODD or indication if nodd has already been read
!                    (>0)
!     number         User point sequence number
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READREAL       Read a real or real expression from the input file
!     SEPCOPYCODE    Copy code2 and related arrays into code
! **********************************************************************
!
!                       I/O
!
!   The following texts are recognized:
!   Pi   or number
!   nelm =
!   ratio =
!   factor =
!   nodd =
!   init =
!   end =
!   type =
!   tang =
!   axis =
!   rplane =
!   alpha =
!   shape = ishape
!   ishape may have one of the following values:
!
!   upper_naca0012
!   lower_naca0012
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     179   point number of user point is incorrect
!     574   Not an integer number found after the keyword
!    1146   ivaltype not allowed in spline record
!    1188   Tang found more than twice in a record
!    1190   subkeyword not recognized
!    1192   Number of user points read in curve record too large
!    1193   Value of ratio incorrect
!    1194   Value of nelm incorrect
!    1195   Wrong number following keyword
!    1196   Value of NODD incorrect
!    1338   Nelm not followed by integer number
!    1339   P not followed by integer number
!    1340   Ratio not followed by integer number
!    1341   Tang not followed by integer number
!    1342   ivaltype not followed by integer number
!    1343   nodd not followed by integer number
!    1378   Parameter has wrong value
!    1467   keyword has been found more than once
!    2555   Not a text found after the keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codelc / 'P', 'NELM', 'FACTOR', 'RATIO', 'TANG', 'TYPE', &
                    'NODD', 'END', 'INIT', 'AXIS', 'RPLANE', &
                    'ALPHA', 'SHAPE', 'EP', 'SUBCURVES', 'IFUNC' /
      data ncodelc / 1, 3, 3, 3, 4,   4, 3, 3, 3, 4, &
                     5, 3, 4, 2, 4, 4 /
      data ncodmxlc/ 1, 4, 6, 5, 4,   4, 4, 3, 4, 4, &
                     6, 5, 5, 2, 9, 5 /

!     codelc   Table containing the keywords recognized:
!              Possible values:
!               1:  point (P or number)
!               2:  nelm
!               3:  factor
!               4:  ratio
!               5:  tang
!               6:  type
!               7:  nodd
!               8:  end
!               9:  init
!              10:  axis
!              11:  rplane
!              12:  alpha
!              13:  shape
!              14:  ep
!              15:  subcurves
!              16:  ifunc
! ======================================================================
!
      call eropen ( 'mshr00' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshr00'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      n      = 0
      itang  = 0
      nelm   = 0
      ifac   = 0
      iratio = 0
      ivaltype  = 0
      iend   = 0
      init   = 0
      nodd   = 0
      ishape  = 0
      ifunc  = 0

!     --- Fill defaults

      work(1:1000) = 0d0
      work(3) = -100d0
      work(4) = 1d0
      work(7) = 1d0
      work(8) = 1d0
      work(12) = 1d0
      work(13) = 1d0
      minsgn = 1
      finished = .false.
      ispecl = 4
      readnx = .true.

      call sepcopycode ( codelc, nncodelc, ncodelc, ncodmxlc )

150   do while ( .not. finished )

!     --- While loop
!         Find next item

         if ( readnx ) call asknxt
         readnx = .true.
         if ( debug ) write(irefwr,1) 'ityprd/1', ityprd

!        --- Interpret item

         if ( ityprd==1 ) then

!        --- End of record found

            finished = .true.
            go to 1000

         else if ( ityprd==6 ) then

!        --- minus sign found

            minsgn = -1
            go to 150

         else if ( ityprd==7 ) then

!        --- ) sign found

            go to 1000

         end if
         if ( ityprd==2 .or. ityprd==3 ) then

!        --- Number found

            ichoice = 1

         else if ( ityprd==4 ) then

!        --- Text found, interprete

            ierpri = 1
            ichoice = sepsubkeyw ()
            if ( debug ) then
               write(irefwr,*) 'string ', string(1:nchar)
               write(irefwr,1) 'ichoice', ichoice
            end if  ! ( debug )

!           --- If text not recognized

            if ( ichoice==0 ) go to 1000

!           --- Read next symbol

            if ( ichoice==10 .or. ichoice==11 ) then

!           --- = expected: Read next symbol but do nothing

               call asknxt

            else if ( ichoice==13 ) then

!           --- text required: Read next symbol

               call asknxt

               if ( ityprd/=4 ) then

!              --- Error 2555: no text found

                  call errchr ( code(13), 1 )
                  call errsub ( 2555, 0, 0, 1 )
                  call errcrd
                  go to 1000

               end if  ! ( ityprd/=4 )

            else

!           --- Number or expression expected, no scalar

               openfound = .true.
               call readreal ( openfound, .false. )

            end if  ! ( ichoice==10 .or. ichoice==11 )

         end if
         if ( debug ) &
            write(irefwr,1) 'ityprd, ichoice', ityprd, ichoice

!        --- Type of record detected, correct number for sign

         realrd = realrd*minsgn
         if ( debug ) write(irefwr,2) 'realrd', realrd
         minsgn = 1

!        --- Fill the various possibilities:

         select case ( ichoice )

            case(:0, 17:)

!           --- This option can not occur

            finished = .true.

            case(1)

!           --- ichoice = 1, User point

               n = n + 1
               if ( ityprd/=2 ) then

!              --- Error 1339:  P not followed by integer number

                  call errsub ( 1339, 0, 0, 0 )
                  call errcrd

               else if ( n>975 ) then

!              --- Error 1192:  More than 975 points read, not possible

                  call errint ( n, 1 )
                  call errint ( 975, 2 )
                  call errsub ( 1192, 2, 0, 0 )
                  call errcrd
                  go to 1000

               end if

               work(24+n) = realrd

            case(2)

!           --- ichoice = 2, Number or nelm = number

               if ( nelm>0 ) then

!              --- Error 1467:, nelm has already been filled

                  call errchr ( 'nelm', 1 )
                  call errsub ( 1467, 0, 0, 1 )
                  call errcrd

               end if
               nelm     = nint(realrd)
               work(2) = nelm
               if ( ityprd/=2 ) then

!              --- Error 1338:  Nelm not followed by integer number

                  call errsub ( 1338, 0, 0, 0 )
                  call errcrd

               else if ( nelm<=0 .or. nelm>100000 ) then

!              --- Error 1194:  NELM has wrong value

                  call errint ( nelm, 1 )
                  call errint ( 100000, 2 )
                  call errsub ( 1194, 2, 0, 0 )
                  call errcrd

               end if

            case(3)

!           --- ichoice = 3, FACTOR

               if ( ifac>0 ) then

!              --- Error 1467: factor has already been filled

                  call errchr ( 'factor', 1 )
                  call errsub ( 1467, 0, 0, 1 )
                  call errcrd

               end if
               ifac     = 1
               work(4) = realrd

            case(4)

!           --- ichoice = 4, Ratio

               if ( iratio>0 ) then

!              --- Error 1467: ratio has already been filled

                  call errchr ( 'ratio', 1 )
                  call errsub ( 1467, 0, 0, 1 )
                  call errcrd

               end if
               iratio   = nint(realrd)
               work(3) = realrd
               if ( ityprd/=2 ) then

!              --- Error 1340:  Ratio not followed by integer number

                  call errsub ( 1340, 0, 0, 0 )
                  call errcrd

               else if ( iratio<-5 .or. iratio>8 ) then

!              --- Error 1193: Ratio out of range

                  call errint ( iratio, 1 )
                  call errint ( -5, 2 )
                  call errint ( 8, 3 )
                  call errsub ( 1193, 3, 0, 0 )
                  call errcrd

               end if
               iratio = 1

            case(5)

!           --- ichoice = 5, Tang

               if ( itang>2 ) then

!              --- Error 1188: All tangents have already been filled

                  call errsub ( 1188, 0, 0, 0 )
                  call errcrd
                  go to 1000

               end if
               itang  = itang + 1
               number = nint(realrd)
               if ( ityprd/=2 ) then

!              --- Error 1341:  Tang not followed by integer number

                  call errsub ( 1341, 0, 0, 0 )
                  call errcrd

               else if ( number<=0 .or. number>nuspnt ) then

!              --- Error 179: User point number <1 or>number of user points

                  call errint ( number, 1 )
                  call errint ( nuspnt, 2 )
                  call errint ( 0, 3 )
                  call errint ( 0, 4 )
                  call errsub ( 179, 4, 0, 0 )
                  call errcrd

               end if
               work(8+itang) = realrd

            case(6)

!           --- ichoice = 6, Type

               if ( ivaltype>0 ) then

!              --- Error 1467: Type has already been read

                  call errchr ( 'type', 1 )
                  call errsub ( 1467, 0, 0, 1 )
                  call errcrd

               end if
               ivaltype = nint(realrd)
               if ( ityprd/=2 ) then

!              --- Error 1342:  ivaltype not followed by integer number

                  call errsub ( 1342, 0, 0, 0 )
                  call errcrd

               else if ( ivaltype<1.or.ivaltype>4 ) then
                  call errint ( ivaltype, 1 )
                  call errsub ( 1146, 1, 0, 0 )
                  call errcrd
               end if
               work(8) = realrd
               ivaltype    = 1

            case(7)

!           --- ichoice = 7, NODD

               if ( nodd>0 ) then

!              --- Error 1467: NODD has already been read

                  call errchr ( 'NODD', 1 )
                  call errsub ( 1467, 0, 0, 1 )
                  call errcrd

               end if
               nodd = nint(realrd)
               if ( ityprd/=2 ) then

!              --- Error 1343:  nodd not followed by integer number

                  call errsub ( 1343, 0, 0, 0 )
                  call errcrd

               else if ( nodd<0.or.nodd>3 ) then
                  call errint ( nodd, 1 )
                  call errint ( 3, 2 )
                  call errsub ( 1196, 2, 0, 0 )
                  call errcrd
               end if
               work(5) = realrd
               nodd = 1

            case(8)

!           --- ichoice = 8, END

               if ( iend>0 ) then

!              --- Error 1467: END has already been read

                  call errchr ( 'END', 1 )
                  call errsub ( 1467, 0, 0, 1 )
                  call errcrd

               end if
               work(7) = realrd
               iend = 1

            case(9)

!           --- ichoice = 9, INIT

               if ( init>0 ) then

!              --- Error 1467: INIT has already been read

                  call errchr ( 'INIT', 1 )
                  call errsub ( 1467, 0, 0, 1 )
                  call errcrd

               end if
               work(6) = realrd
               init = 1

            case(10)

!           --- ichoice = 10, AXIS

               work(11) = 1d0

            case(11)

!           --- ichoice = 11, RPLANE

               work(11) = 2d0

            case(12)

!           --- ichoice = 12, ALPHA

               work(12) = realrd

            case(13)

!           --- ichoice = 13, ishape

               if ( ishape>0 ) then

!              --- Error 1467: ishape has already been read

                  call errchr ( 'shape', 1 )
                  call errsub ( 1467, 0, 0, 1 )
                  call errcrd

               end if

!              --- Interpret the text behind shape

               if ( string(1:4)=='UPPE' ) then

!              --- upper_naca0012 found

                  work(13) = 1d0

               else if ( string(1:4)=='LOWE' ) then

!              --- lower_naca0012 found

                  work(13) = 2d0

               else

!              --- text not recognized

                  call errchr ( string, 1 )
                  call errchr ( code(13), 2 )
                  call errchr ( 'upper_naca0012', 3 )
                  call errchr ( 'lower_naca0012', 4 )
                  call errsub ( 1190, 0, 0, 4 )
                  call errcrd

               end if
               call asknxt
               if ( ityprd/=2 ) then

!              --- no integer found after the keyword naca

                  call errchr ( 'naca', 1 )
                  call errsub ( 574, 0, 0, 1 )
                  call errcrd

               else

!              --- integer found, check if it is 0012

                  k = nint(realrd)
                  if ( k/=12 ) then

!                 --- The number found behind naca is incorrect

                     call errint ( k, 1 )
                     call errchr ( 'naca', 1 )
                     call errsub ( 1195, 1, 0, 1 )
                     call errcrd

                  end if

               end if
               ishape = 1

            case(14, 15)

!           --- ichoice = 14, 15 EP, SUBCURVES
!               no action

            case(16)

!           --- ichoice = 16, ifunc

               if ( ifunc>0 ) then

!              --- Error 1467: ifunc has already been filled

                  call errchr ( 'ifunc', 1 )
                  call errsub ( 1467, 0, 0, 1 )
                  call errcrd

               end if

               work(15) = realrd
               ifunc = nint(realrd)
               if ( ifunc<=0 .or. ifunc>1000 ) then

!              --- ifunc out of range

                  call errint ( ifunc, 1 )
                  call errint ( 1, 2 )
                  call errint ( 1000, 3 )
                  call errchr ( 'ifunc', 1 )
                  call errsub ( 1378, 3, 0, 1 )

               end if  ! ( ifunc<=0 .or. ifunc>1000 )
               ifunc = 1

         end select  ! case ( ichoice )

      end do  ! while ( .not. finished )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshr00'

      end if  ! ( debug )
      call erclos ( 'mshr00' )

      work(1) = n
      ispecl = 0

      end
