      subroutine mshrdparallel ( iparallel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 30-04-2015 extension with test
!        version  1.4  date 03-03-2015 extension with metis
!        version  1.3  date 28-03-2013 Extra debug
!
!   copyright (c) 2003-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read rest of input following the keyword parallel
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     mesh
!     read
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iparallel

!     iparallel      o    Indicates if the mesh must be subdivided for parallel
!                         processing or not.
!                         iparallel consists of 2 parts methodpar and
!                         numprocessors according to
!                         iparallel = methodpar + 100 * numprocessors
!                         with
!                         methodpar the type of subdivision to be used
!                         numprocessors the number of processors (or blocks)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncodemn=8
      integer nmcode(nncodemn), methodpar, &
              numprocessors, nsubdiv
      double precision :: h
      character (len=15) codemn(nncodemn)

!     codemn         Table with main keywords
!     h              Help variable to store a constant temporarily
!     methodpar      Defines the type of subdivision of the mesh to be made
!                    for parallel computing
!                    Possible values:
!                    1:  If the mesh contains volumes the number of
!                        subparts is equal to the number of volumes
!                        otherwise equal to the number of surfaces
!                    2:  the mesh is subdivided in layers
!                    3:  the mesh is subdivided in blocks
!                    4:  specially meant for testpurposes:
!                        the number of elements in each region is
!                        approximately the same
!                        elements are not renumbered and are subdivided
!                        in natural sequence
!                    5:  specially meant for testpurposes:
!                        a 3d block of equal number of elements (hexahedrons)
!                        in each direction is subdivided into subblocks of
!                        equal size ( block subdivision)
!                        The number of blocks must be a power of 3
!     nmcode         Table containing the number of significant characters
!                    in the main keyword table codemn, including those of
!                    the extra texts
!     nncodemn       Number of entries in codemn
!     nsubdiv        number of processors ^ (1/3)
!     numprocessors  Number of processors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!      parallel, method = x, num_processors = i
!      with
!         method = surfaces
!                  volumes
!                  layers
!                  blocks
!                  volumes
!                  elements_serial
!                  threed_blocks
!                  metis
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2649   nprocessors is not third power
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codemn / 'SURFACES', 'LAYERS', 'BLOCKS', 'VOLUMES', &
                    'ELEMENTS_SERIAL', 'THREED_BLOCKS', 'METIS', &
                    'TEST' /
      data nmcode / 3, 3, 3, 3,10, 9, 5, 4 /

!    code      In this table the significant characters of the various
!              texts after parallel are stored
!              The following types of texts are recognised:
!              1:   method
!              2:   num_processors
!    codemn    In this table the significant characters of the various
!              texts after method are stored
!              The following types of texts are recognised:
!              1:   surfaces
!              2:   layers
!              3:   blocks
!              4:   volumes
!              5:   elements_serial
!              6:   threed_blocks
!              7:   metis
!              8:   test
! ======================================================================
!
      call eropen ( 'mshrdparallel' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrdparallel'
  1      format ( a, 1x, (10i6) )

      end if

      nncode = 2
      code(1) = 'METHOD'
      code(2) = 'NUM_PROCESSORS'

      call sepcopycodesub ( codemn, nncodemn, nmcode )
      itype(1) = 5
      itype(2) = 0
      ncode(1) = 3
      ncode(2) = 7
      ncodmx(1) = 6
      ncodmx(2) = 14
      tabmin(1) = 1d0
      tabmin(2) = 2d0

      table(1) = 2d0
      table(2) = 8d0
      tabmax(1) = nncodemn
      tabmax(2) = 1024d0
      nmain = 0

      call sepreadsetkeyw

      if ( debug ) call prinrl ( table, nncode, 'table' )

      if ( nseqkeyw>2 ) then

!     --- array iseqkeyw too small

         call errint ( nseqkeyw, 1 )
         call errint ( 2, 2 )
         call errchr ( 'iseqkeyw', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if

      methodpar = nint(table(1))
      if ( methodpar==4 ) methodpar = 1
      if ( methodpar>4 ) methodpar = methodpar-1
      numprocessors = nint(table(2))
      if ( methodpar==1 ) numprocessors = 0

      if ( debug ) write(irefwr,1) 'methodpar', methodpar

      iparallel = methodpar + 100*numprocessors
      if ( methodpar==5 ) then

!     --- methodpar = 5, i.e. threed_blocks

         h = numprocessors
         nsubdiv = nint(h**(1d0/3d0))
         if ( nsubdiv**3/=numprocessors ) then

!        --- numprocessors is not third power

            call errint ( numprocessors, 1 )
            call errint ( nsubdiv, 2 )
            call errint ( nsubdiv**3, 3 )
            call errsub ( 2649, 3, 0, 0 )

         end if

      end if  ! ( methodpar==5 )

      call erclos ( 'mshrdparallel' )
      if ( debug ) write(irefwr,*) 'End mshrdparallel'

      end
