      subroutine mshrdparmsh1 ( lenblockinfo, lentransinfo, npoint_glob, &
                                numnodesloc, globnodesp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 30-01-2016 Use sepmodulecomio
!        version  3.0  date 01-05-2015 Extra parameters
!        version  2.0  date 26-09-2007 Extra parameters mappingpar, npoint
!
!   copyright (c) 2001-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information about global mesh from file meshoutput_par.000
!     Only used if parallel computing is applied
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     parallel
!     neighbor
!     read
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemach
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: npoint_glob
      integer, intent(out) :: numnodesloc(iacnodes), globnodesp(npoint_glob)
      integer, intent(inout) ::  lenblockinfo, lentransinfo

!     globnodesp     o    Array of size npoint_glob containing the numbering of
!                         the global nodes under the condition that
!                         first all nodes of the first cpu are numbered,
!                         then of the second one and so on.
!                         A global node has only one sequence number
!                         defined by the lowest cpu it is positioned on
!     lenblockinfo  i/o   Length of array blockinfo
!                         At input estimated length
!                         At output computed length
!     lentransinfo  i/o   Length of array transinfo
!                         At input estimated length
!                         At output computed length
!     npoint_glob    i    Number of nodes in global mesh
!     numnodesloc    i    Contains the number of nodes that correspond
!                         to a cpu accumulated
!                         Nodes that belong to a cpu with lower number are
!                         excluded
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, numneigh, idum, nblocks, istartinfo, lenblock, &
                 npointloc, lastps, iref, icount, locnode, nodp, num
      integer, allocatable :: maploc(:)
      character (len=80) text
      character (len=100) :: name10sav

!     i              Counting variable
!     icount         Counter
!     idum           General dummy variable
!     iref           File reference number
!     istartinfo     Starting address to put information in array
!                    infotabout
!     j              Counting variable
!     lastps         Last non-blank position in namefl
!     lenblock       Actual length of array blockinfo
!     locnode        Loop variable for local nodes
!     maploc         Local mapping array, gives global node numbers of
!                    local nodes
!     name10sav      Help variable to store value of name10
!     nblocks        Number of blocks
!     nodp           Nodal point sequence number
!     npointloc      Number of nodal points read
!     num            Number of numbers to be read
!     numneigh       number of neighbour blocks of a block
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIFIL         Open files
!     MSHREADTEXT    Read text from file and compare with text that is expected
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!     This subroutine reads the information of one block to the file
!     meshoutput_par.000
!
!     Structure of the file
!     Record 1: Text: //number of blocks
!     Record 2: number of blocks
!     For each block:
!        Record 1: Text: //block "block sequence number"
!        Record 2: Text: //number of neighbours
!        Record 3: number of neighbours
!        Record 4: Text: //blockinfo (first length)
!        Record 5: lenblockinfo (length of blockinfo)
!        Record 6 ... : array blockinfo
!        Next record: Text: //transportinfo (first length)
!        Next record: lentransinfo (length of transinfo)
!        Next records:  array transinfo
!        Next record: Text: //global node numbers (first number)
!        Next record: nnodes (number of nodes in block)
!        Next records:  global node numbers in block
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2271   Number of blocks larger than number of nodes
!    2272   Number of nodes larger than number of blocks
!    2893   Number of nodes read incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read number of blocks from file meshoutput_par.000
!     Check if nblocks<=iacnodes
!     Read and skip first inodenr-1 blocks
!     Read blockinfo
!     Read transinfo
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrdparmsh1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrdparmsh1'
         write(irefwr,1) 'npoint_glob', npoint_glob
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array globnodesp (1:npoint_glob)

      globnodesp = 0
      icount = 0

      allocate ( maploc(npoint_glob), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint_glob, 'maploc' )
      if ( ierror/=0 ) go to 1000

      name10sav = name10  ! store name10

!     --- Open file meshoutput_par.000

      lastps = index ( name10, ' ' )-1
      name10(lastps-2:lastps) = '000'
      if ( debug ) write(irefwr,*) 'name10 ', name10
      call inifil(10)
      if ( ierror/=0 ) go to 1000

!     --- Read text from file and compare with input text

      text = '//number of blocks'
      call mshreadtext ( iref10, text, name10 )
      if ( ierror/=0 ) go to 1000

!     --- Read number of blocks from file meshoutput_par.000

      if ( iref10>0 ) then

!     --- formatted file

         read(iref10,*) nblocks

      else

!     --- unformatted file

         read(-iref10) nblocks

      end if

!     --- Check if nblocks<=iacnodes

      if ( nblocks>iacnodes ) then

!     --- Error: number of blocks larger than number of nodes

         call errint ( nblocks, 1 )
         call errint ( iacnodes, 2 )
         call errsub ( 2271, 2, 0, 0 )
         go to 1000

      else if ( nblocks<iacnodes ) then

!     --- Error: number of blocks smaller than number of nodes

         call errint ( nblocks, 1 )
         call errint ( iacnodes, 2 )
         call errwar ( 2272, 2, 0, 0 )

      end if

!     --- Read all blocks

      do i = 1, iacnodes

         if ( iref10>0 ) then

!        --- formatted file

            read(iref10,100) text
100         format(a)
            text = '//number of neighbours'
            call mshreadtext ( iref10, text, name10 )
            if ( ierror/=0 ) go to 1000

            read(iref10,*) numneigh

            text = '//blockinfo (first length)'
            call mshreadtext ( iref10, text, name10 )
            if ( ierror/=0 ) go to 1000

            if ( i==inodenr ) then
               read(iref10,*) lenblock
               call inichkarlen ( lenblock, lenblockinfo, &
                                  'blockinfo' )
               if ( ierror/=0 ) go to 1000
               read(iref10,*) blockinfo(1:lenblock)
            else
               read(iref10,*) num
               read(iref10,*) (idum,j=1,num)
            end if  ! ( i==inodenr )

            text = '//transportinfo (first length)'
            call mshreadtext ( iref10, text, name10 )
            if ( ierror/=0 ) go to 1000

            if ( i==inodenr ) then
               read(iref10,*) istartinfo
               call inichkarlen ( istartinfo, lentransinfo, &
                                  'transinfo' )
               if ( ierror/=0 ) go to 1000
               read(iref10,*) transinfo(1:istartinfo)
            else
               read(iref10,*) num
               read(iref10,*) (idum,j=1,num)
            end if  ! ( i==inodenr )

            text = '//global node numbers (first number)'
            call mshreadtext ( iref10, text, name10 )
            if ( ierror/=0 ) go to 1000

            read(iref10,*) npointloc
            read(iref10,*) maploc(1:npointloc)

         else

!        --- unformatted file

            read(-iref10) text
            text = '//number of neighbours'
            call mshreadtext ( iref10, text, name10 )
            if ( ierror/=0 ) go to 1000

            read(-iref10) numneigh

            text = '//blockinfo (first length)'
            call mshreadtext ( iref10, text, name10 )
            if ( ierror/=0 ) go to 1000

            if ( i==inodenr ) then
               read(-iref10) lenblock
               call inichkarlen ( lenblock, lenblockinfo, &
                                  'blockinfo' )
               if ( ierror/=0 ) go to 1000
               read(-iref10) blockinfo(1:lenblock)
            else
               read(-iref10) num
               read(-iref10) idum
            end if  ! ( i==inodenr )

            text = '//transportinfo (first length)'
            call mshreadtext ( iref10, text, name10 )
            if ( ierror/=0 ) go to 1000

            if ( i==inodenr ) then
               read(-iref10) istartinfo
               call inichkarlen ( istartinfo, lentransinfo, &
                                  'transinfo' )
               if ( ierror/=0 ) go to 1000
               read(-iref10) transinfo(1:istartinfo)
            else
               read(-iref10) num
               read(-iref10) idum
            end if  ! ( i==inodenr )

            text = '//global node numbers (first number)'
            call mshreadtext ( iref10, text, name10 )
            if ( ierror/=0 ) go to 1000

            read(-iref10) npointloc
            read(-iref10) maploc(1:npointloc)

         end if


         if ( i==inodenr ) then

!        --- Present cpu, copy maploc into mappingpar

            if ( npointloc/=npoint ) then

!           --- Error: npointloc # npoint

               call errint ( npointloc, 1 )
               call errint ( npoint, 2 )
               call errint ( inodenr, 3 )
               call errsub ( 2893, 3, 0, 0 )
               go to 1000

            end if  ! ( npointloc/=npoint )

            mappingpar(1:npoint) = maploc(1:npoint)
            if ( associated(mappar(i)%arr) ) deallocate(mappar(i)%arr)
            allocate(mappar(i)%arr(npoint))
            mappar(i)%arr = maploc(1:npoint)
            blockinfo(1) = numneigh

         end if  ! ( i==inodenr )

         if ( inodenr==1 ) then

!        --- node nr 1 (host), special treatment

            if ( i==1 ) then
               if ( associated(mappar(0)%arr) ) deallocate(mappar(0)%arr)
               allocate(mappar(0)%arr(iacnodes))
               mappar(0)%arr(i) = npointloc
            else
               if ( associated(mappar(i)%arr) ) deallocate(mappar(i)%arr)
               allocate(mappar(i)%arr(npointloc))
               mappar(i)%arr = maploc(1:npointloc)
               mappar(0)%arr(i) = npointloc
            end if  ! ( i==1 )

         end if  ! ( inodenr==1 )

         if ( debug ) call prinin ( mappingpar, npoint, '' )

!        --- For locnode := 1 (1) npoint do

         do locnode = 1, npointloc

!        --- nodp = maploc(i)

            nodp = maploc(locnode)
            if ( globnodesp(nodp)==0 ) then

!           --- Node is new
!               icount := icount+1; globnodes(nodp):=icount

               icount = icount+1
               globnodesp(nodp) = icount

            end if  ! ( globnodes(nodp)==0 )

         end do  ! locnode = 1, npointloc
         numnodesloc(i) = icount

      end do  ! i = 1, iacnodes
      lentransinfo = istartinfo
      lenblockinfo = lenblock

!     --- Close file meshoutput_par.000

      iref = abs(iref10)
      close ( unit = iref )
      name10 = name10sav   ! reset name10

      deallocate ( maploc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'mapping' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'npoint, npoint_glob', npoint, npoint_glob
         call prinin ( numnodesloc, iacnodes, 'numnodesloc' )
         call prinin ( globnodesp, npoint_glob, 'globnodes' )
         write(irefwr,*) 'End mshrdparmsh1'

      end if  ! ( debug )
      call erclos ( 'mshrdparmsh1' )

      end subroutine mshrdparmsh1
