      subroutine mshrefkmeshc12 ( kmeshclc, kmeshcnew, nelprev, &
                                  nodenumbers, fillmap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-06-2014 Extra parameters
!        version  1.2  date 20-06-2013 Correction mid points
!        version  1.1  date 10-03-2005 Fortran 90
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshclc for one element group in case that
!     the complete mesh is refined
!     Elements of shape 12, i.e. quadratic tetrahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     quadratic
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshclc(10,*), kmeshcnew(10,*), nodenumbers(*), &
              nelprev
      logical fillmap

!     fillmap        i    If true array imap must be filled
!     kmeshclc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
!     nelprev        i    Number of elements stored in kmeshclc at the start
!                         of the subroutine
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next inelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielemnew, ielem, ipstart, iedge, edge(6,2), i, face(4,3), &
              iface, nrotat, ind(6), ivol, inode1, inode2

!     edge           Contains the edge node numbers on the edges (2 per edge)
!     face           Contains the face node numbers on the 4 faces (3 per face)
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielemnew       Sequence number of element in new mesh
!     iface          Face sequence number
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3
!     inode1         Node number
!     inode2         Node number
!     ipstart        Starting address in array
!     ivol           Contains the volume node number
!     nrotat         Number of rotations necessary to map both elements
!                    onto each other
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The quadratic tetrahedron has nodes 1 2 3 4 5 6   7 8 9  and 10
!     The edges are defines as:
!     e1:  1 2  3
!     e2:  3 8 10
!     e3: 10 7  1
!     e4:  1 6  5
!     e5:  3 4  5
!     e6: 10 9  5
!
!     Let the corresponding nodes be e1a, e1b, e2a, e2b, e3a, ... e6b
!
!     The faces as
!
!     f1:  1 2 3 8 10 7
!     f2:  3 4 5 9 10 8
!     f3:  5 6 1 7 10 9
!     f4:  1 6 5 4  3 2
!
!     Let the top be node 10, hence the bottom triangle = 1-2-3-4-5-6
!
!     The 8 elements become:
!
!     1 e1a 2 f4a 6 e4a    e3b f1a f3b    7
!     3 e5a 4 f4c 2 e1b    e2a f2a f1b    8
!     5 e4b 6 f4b 4 e5b    e6b f3a f2b    9
!     7 f1c 8 f2c 9 f3c    e3a e2b e6a   10
!
!     2 f4a 6 f3b 7 f1a    f1b  v1 f1c    8
!     8  v1 6 f4b 4 f2a    f1b f4a f4c    2
!     7 f3b 6 f3a 9 f3c    f1c  v1 f2c    8
!     6 f3a 9 f2c 8  v1    f4b f2b f2a    4
!
!     ielemnew := 0
!     For all elements ielem in old mesh do
!        ielemnew := ielemnew+1
!        fill first element
!        ielemnew := ielemnew+1
!        fill second element
!        ielemnew := ielemnew+1
!        fill third element
!        ielemnew := ielemnew+1
!        fill fourth element
!        ielemnew := ielemnew+1
!        fill fifth element
!        ielemnew := ielemnew+1
!        fill sixth element
!        ielemnew := ielemnew+1
!        fill seventh element
!        ielemnew := ielemnew+1
!        fill element eight
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind / 1, 2, 3, 1, 2, 3 /
! ======================================================================
!
      call eropen ( 'mshrefkmeshc12' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc12'
         write(irefwr,*) 'inelem, nelprev ', inelem, nelprev
         call prinin1 ( kmeshclc, inelem, 10, 'kmeshclc/surface' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- ielemnew := 0

      ielemnew = 0

!     --- For all elements ielem in old mesh do

      do ielem = 1, inelem

!     --- fill node numbers of edges

         ipstart = nmeshd(nelprev+ielem)
         do i = 1, 6
            iedge = nmeshd(ipstart+i)
            if ( iedge>0 ) then
               edge(i,1) = nodenumbers(iedge)       ! 3 edge numbers / a
               edge(i,2) = nodenumbers(iedge)+1     ! 3 edge numbers / b
            else  ! ( iedge<0 )
               edge(i,1) = nodenumbers(-iedge)+1    ! 3 edge numbers / a
               edge(i,2) = nodenumbers(-iedge)      ! 3 edge numbers / b
            end if  ! ( iedge>0 )
         end do

!        --- fill node numbers of faces

         ipstart = nmesha(nelprev+ielem)
         do i = 1, 4
            iface = nmesha(ipstart+i)
            nrotat =  nmesha(ipstart+4+i)

            if ( iface>0 ) then

!           --- Standard direction

               iface = nodenumbers(iface+nedges)
               face(i,ind(nrotat+1)) = iface
               face(i,ind(nrotat+2)) = iface+1
               face(i,ind(nrotat+3)) = iface+2

            else

!           --- Reversed direction

               iface = nodenumbers(-iface+nedges)
               face(i,ind(nrotat+2)) = iface
               face(i,ind(nrotat+1)) = iface+1
               face(i,ind(nrotat+3)) = iface+2

            end if

         end do

         ivol = nodenumbers(nedges+nfaces+nelprev+ielem)

         inode1 = kmeshclc(6,ielem)
         inode2 = kmeshclc(8,ielem)
         coor(1:3,ivol) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))

         if ( fillmap ) then
            imap(ivol,1) = 2
            imap(ivol,2) = inode1
            imap(ivol,3) = inode2
            rmap(ivol,1:2) = 0.5d0
         end if  ! ( fillmap )

!        --- ielemnew := ielemnew+1
!            fill first element
!            1 e1a 2 f4a 6 e4a    e3b f1a f3b    7

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshclc(1,ielem)
         kmeshcnew(2,ielemnew) = edge(1,1)
         kmeshcnew(3,ielemnew) = kmeshclc(2,ielem)
         kmeshcnew(4,ielemnew) = face(4,1)
         kmeshcnew(5,ielemnew) = kmeshclc(6,ielem)
         kmeshcnew(6,ielemnew) = edge(4,1)
         kmeshcnew(7,ielemnew) = edge(3,2)
         kmeshcnew(8,ielemnew) = face(1,1)
         kmeshcnew(9,ielemnew) = face(3,2)
         kmeshcnew(10,ielemnew) = kmeshclc(7,ielem)

!        --- ielemnew := ielemnew+1
!             fill second element
!             3 e5a 4 f4c 2 e1b    e2a f2a f1b    8

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshclc(3,ielem)
         kmeshcnew(2,ielemnew) = edge(5,1)
         kmeshcnew(3,ielemnew) = kmeshclc(4,ielem)
         kmeshcnew(4,ielemnew) = face(4,3)
         kmeshcnew(5,ielemnew) = kmeshclc(2,ielem)
         kmeshcnew(6,ielemnew) = edge(1,2)
         kmeshcnew(7,ielemnew) = edge(2,1)
         kmeshcnew(8,ielemnew) = face(2,1)
         kmeshcnew(9,ielemnew) = face(1,2)
         kmeshcnew(10,ielemnew) = kmeshclc(8,ielem)

!        --- ielemnew := ielemnew+1
!             fill third element
!             5 e4b 6 f4b 4 e5b    e6b f3a f2b    9

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshclc(5,ielem)
         kmeshcnew(2,ielemnew) = edge(4,2)
         kmeshcnew(3,ielemnew) = kmeshclc(6,ielem)
         kmeshcnew(4,ielemnew) = face(4,2)
         kmeshcnew(5,ielemnew) = kmeshclc(4,ielem)
         kmeshcnew(6,ielemnew) = edge(5,2)
         kmeshcnew(7,ielemnew) = edge(6,2)
         kmeshcnew(8,ielemnew) = face(3,1)
         kmeshcnew(9,ielemnew) = face(2,2)
         kmeshcnew(10,ielemnew) = kmeshclc(9,ielem)

!        --- ielemnew := ielemnew+1
!             fill fourth element
!             7 f1c 8 f2c 9 f3c    e3a e2b e6a   10

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshclc(7,ielem)
         kmeshcnew(2,ielemnew) = face(1,3)
         kmeshcnew(3,ielemnew) = kmeshclc(8,ielem)
         kmeshcnew(4,ielemnew) = face(2,3)
         kmeshcnew(5,ielemnew) = kmeshclc(9,ielem)
         kmeshcnew(6,ielemnew) = face(3,3)
         kmeshcnew(7,ielemnew) = edge(3,1)
         kmeshcnew(8,ielemnew) = edge(2,2)
         kmeshcnew(9,ielemnew) = edge(6,1)
         kmeshcnew(10,ielemnew) = kmeshclc(10,ielem)

!        --- ielemnew := ielemnew+1
!             fill fifth element
!             2 f4a 6 f3b 7 f1a    f1b  v1 f1c    8

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshclc(2,ielem)
         kmeshcnew(2,ielemnew) = face(4,1)
         kmeshcnew(3,ielemnew) = kmeshclc(6,ielem)
         kmeshcnew(4,ielemnew) = face(3,2)
         kmeshcnew(5,ielemnew) = kmeshclc(7,ielem)
         kmeshcnew(6,ielemnew) = face(1,1)
         kmeshcnew(7,ielemnew) = face(1,2)
         kmeshcnew(8,ielemnew) = ivol
         kmeshcnew(9,ielemnew) = face(1,3)
         kmeshcnew(10,ielemnew) = kmeshclc(8,ielem)

!        --- ielemnew := ielemnew+1
!             fill sixth element
!             8  v1 6 f4b 4 f2a    f1b f4a f4c    2

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshclc(8,ielem)
         kmeshcnew(2,ielemnew) = ivol
         kmeshcnew(3,ielemnew) = kmeshclc(6,ielem)
         kmeshcnew(4,ielemnew) = face(4,2)
         kmeshcnew(5,ielemnew) = kmeshclc(4,ielem)
         kmeshcnew(6,ielemnew) = face(2,1)
         kmeshcnew(7,ielemnew) = face(1,2)
         kmeshcnew(8,ielemnew) = face(4,1)
         kmeshcnew(9,ielemnew) = face(4,3)
         kmeshcnew(10,ielemnew) = kmeshclc(2,ielem)

!        --- ielemnew := ielemnew+1
!             fill seventh element
!             7 f3b 6 f3a 9 f3c    f1c  v1 f2c    8

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshclc(7,ielem)
         kmeshcnew(2,ielemnew) = face(3,2)
         kmeshcnew(3,ielemnew) = kmeshclc(6,ielem)
         kmeshcnew(4,ielemnew) = face(3,1)
         kmeshcnew(5,ielemnew) = kmeshclc(9,ielem)
         kmeshcnew(6,ielemnew) = face(3,3)
         kmeshcnew(7,ielemnew) = face(1,3)
         kmeshcnew(8,ielemnew) = ivol
         kmeshcnew(9,ielemnew) = face(2,3)
         kmeshcnew(10,ielemnew) = kmeshclc(8,ielem)

!        --- ielemnew := ielemnew+1
!             fill element eight
!             6 f3a 9 f2c 8  v1    f4b f2b f2a    4

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshclc(6,ielem)
         kmeshcnew(2,ielemnew) = face(3,1)
         kmeshcnew(3,ielemnew) = kmeshclc(9,ielem)
         kmeshcnew(4,ielemnew) = face(2,3)
         kmeshcnew(5,ielemnew) = kmeshclc(8,ielem)
         kmeshcnew(6,ielemnew) = ivol
         kmeshcnew(7,ielemnew) = face(4,2)
         kmeshcnew(8,ielemnew) = face(2,2)
         kmeshcnew(9,ielemnew) = face(2,1)
         kmeshcnew(10,ielemnew) = kmeshclc(4,ielem)

      end do  ! ielem = 1, inelem

1000  call erclos ( 'mshrefkmeshc12' )

      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshcnew, ielemnew, 10, 'kmeshcnew/surface' )
         write(irefwr,*) 'End mshrefkmeshc12'

      end if

      end
