      subroutine mshrefkmeshc22 ( kmeshclc, kmeshcnew, nelprev, &
                                  nodenumbers, fillmap  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-06-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshclc for one element group in case that
!     the complete mesh is refined
!     Elements of shape 22, i.e. quadratic prisms
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     quadratic
!     prism
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshclc(18,*), kmeshcnew(18,*), nodenumbers(*), &
              nelprev
      logical fillmap

!     fillmap        i    If true array imap must be filled
!     kmeshclc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
!     nelprev        i    Number of elements stored in kmeshclc at the start
!                         of the subroutine
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next inelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielemnew, ielem, ipstart, iedge, edge(9,2), i, face(5,8), &
              iface, nrotat, ind(8), ivol, ind1(6)
      logical extradebug

!     edge           Contains the edge node numbers on the 9 edges (2 per edge)
!     face           Contains the face node numbers on the 6 faces (8 per face)
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielemnew       Sequence number of element in new mesh
!     iface          Face sequence number
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3,4
!     ind1           Array to store the cyclic permutation of
!                    1 - 3
!     ipstart        Starting address in array
!     ivol           Contains the volume node number
!     nrotat         Number of rotations necessary to map both elements
!                    onto each other
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The quadratic prism has nodes
!      1  2  3    4  5  6
!      7  8  9   10 11 12
!     13 14 15   16 17 18
!     The edges are defined as:
!     e1:  1  2  3
!     e2:  3  4  5
!     e3:  5  6  7
!     e4: 13 14 15
!     e5: 15 16 17
!     e6: 17 18 13
!     e7:  1  7 13
!     e8:  2  8 14
!     e9:  3  9 15
!
!     Let the corresponding nodes be e1a, e1b, e2a, e2b, e3a, ... e9b
!
!     The faces as
!
!     f1:  1  2  3  4  5  6
!     f2: 13 14 15 16 17 18
!     f3:  1  2  3  9 15 14 13  7  8
!     f4:  3  4  5 11 17 16 15  9 10
!     f5:  5  6  1  7 13 18 17 11 12
!
!     The nodes on each face is positioned as follows:
!     a between 2 and 9
!     b between 8 and 9
!     c between 6 and 9
!     d between 8 and 9
!     The other 4 in the centers of the 4 quadrants:
!     e in  1 - 2 - 9 - 8
!     f in  2 - 3 - 9 - 2
!     g in  4 - 5 - 6 - 9
!     h in  6 - 7 - 8 - 9
!
!     We need  9 extra volume nodes
!     These are lying in the middle triangle
!     1 in 8-12
!     2 in 8-10
!     3 in 10-12
!     4 in f3a-f5a
!     5 in f4a-f3a
!     6 in f5a-f4a
!     7 in f5a-f4a
!     8 in f5a-f4a
!     9 in f5a-f4a
!
!     The 8 elements become:
!
!        1  e1a    2     f1a    6  e3b
!      e7a  f3e  f3a      v4  f5a  f5f
!        7  f3d    8      v1   12  f5b
!
!        3  e2a    4     f1b    2  e1b
!      e8a  f4e  f4a      v5  f3a  f3f
!        9  f4d   10      v2    8  f3b
!
!        5  e3a    6     f1c    4  e2b
!      e9a  f5e  f5a      v6  f4a  f4f
!       11  f5d   12      v3   10  f4b
!
!        2  f1b    4     f1c    6  f1a
!      f3a   v5  f4a      v6  f5a   v4
!        8   v2   10      v3   12   v1
!
!        7  f2d    8      v1   12  f5d
!      e7b  f3h  f3c      v7  f5c  f5g
!       13  e4a   14     f2a   18  e6b
!
!        9  f3d   10      v2    8  f3b
!      e8b  f4h  f4c      v8  f3c  f3g
!       15  e5a   16     f2b   14  e4b
!
!       11  f5d   12      v3   10  f4b
!      e9b  f5h  f5c      v9  f4c  f4g
!       17  e6a   18     f2c   16  e5b
!
!        8   v2   10      v3   12   v1
!      f3c   v8  f4c      v9  f5c   v7
!       14  f2b   16     f2c   18  f2a
!
!     ielemnew := 0
!     For all elements ielem in old mesh do
!        ielemnew := ielemnew+1
!        fill first element
!        ielemnew := ielemnew+1
!        fill second element
!        ielemnew := ielemnew+1
!        fill third element
!        ielemnew := ielemnew+1
!        fill fourth element
!        ielemnew := ielemnew+1
!        fill fifth element
!        ielemnew := ielemnew+1
!        fill sixth element
!        ielemnew := ielemnew+1
!        fill seventh element
!        ielemnew := ielemnew+1
!        fill element eight
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind / 1, 2, 3, 4, 1, 2, 3, 4 /
      data ind1 / 1, 2, 3, 1, 2, 3 /
! ======================================================================
!
      call eropen ( 'mshrefkmeshc22' )
      debug = .false.
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc22'
         write(irefwr,1) 'inelem, nfaces, nelprev', &
                          inelem, nfaces, nelprev
         if ( extradebug ) &
            call prinin1 ( kmeshclc, inelem, 18, 'kmeshclc_old' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- ielemnew := 0

      ielemnew = 0

!     --- For all elements ielem in old mesh do

      do ielem = 1, inelem

!     --- fill node numbers of edges

         ipstart = nmeshd(nelprev+ielem)
         do i = 1, 9
            iedge = nmeshd(ipstart+i)
            if ( iedge>0 ) then
               edge(i,1) = nodenumbers(iedge)       ! 9 edge numbers / a
               edge(i,2) = nodenumbers(iedge)+1     ! 9 edge numbers / b
            else  ! ( iedge<0 )
               edge(i,1) = nodenumbers(-iedge)+1    ! 9 edge numbers / a
               edge(i,2) = nodenumbers(-iedge)      ! 9 edge numbers / b
            end if  ! ( iedge>0 )
         end do

!        --- fill node numbers of faces

         ipstart = nmesha(nelprev+ielem)
         do i = 1, 2
            iface = nmesha(ipstart+i)
            nrotat =  nmesha(ipstart+5+i)

            if ( iface>0 ) then

!           --- Standard direction

               iface = nodenumbers(iface+nedges)
               face(i,ind1(nrotat+1)) = iface
               face(i,ind1(nrotat+2)) = iface+1
               face(i,ind1(nrotat+3)) = iface+2

            else

!           --- Reversed direction

               iface = nodenumbers(-iface+nedges)
               face(i,ind1(nrotat+2)) = iface
               face(i,ind1(nrotat+1)) = iface+1
               face(i,ind1(nrotat+3)) = iface+2

            end if

         end do

         do i = 3, 5
            iface = nmesha(ipstart+i)
            nrotat =  nmesha(ipstart+5+i)
            if ( iface>0 ) then

!           --- Standard direction

               iface = nodenumbers(iface+nedges)
               face(i,ind(nrotat+1)) = iface
               face(i,ind(nrotat+2)) = iface+1
               face(i,ind(nrotat+3)) = iface+2
               face(i,ind(nrotat+4)) = iface+3
               face(i,ind(nrotat+1)+4) = iface+4
               face(i,ind(nrotat+2)+4) = iface+5
               face(i,ind(nrotat+3)+4) = iface+6
               face(i,ind(nrotat+4)+4) = iface+7

            else

!           --- Reversed direction

               iface = nodenumbers(-iface+nedges)
               face(i,1) = iface-1+ind(nrotat+1)
               face(i,2) = iface-1+ind(nrotat+4)
               face(i,3) = iface-1+ind(nrotat+3)
               face(i,4) = iface-1+ind(nrotat+2)
               face(i,5) = iface+ind(nrotat+2)+3
               face(i,6) = iface+ind(nrotat+1)+3
               face(i,7) = iface+ind(nrotat+4)+3
               face(i,8) = iface+ind(nrotat+3)+3

            end if

         end do

!        --- Fill volume coordinates

         ivol = nodenumbers(nedges+nfaces+nelprev+ielem)-1
         if ( debug ) write(irefwr,1) 'ielem, ivol', ielem, ivol
         coor(1:3,ivol+1) = 0.5d0*(coor(1:3,kmeshclc(8,ielem))+ &
                                   coor(1:3,kmeshclc(12,ielem)))
         coor(1:3,ivol+2) = 0.5d0*(coor(1:3,kmeshclc(8,ielem))+ &
                                   coor(1:3,kmeshclc(10,ielem)))
         coor(1:3,ivol+3) = 0.5d0*(coor(1:3,kmeshclc(10,ielem))+ &
                                   coor(1:3,kmeshclc(12,ielem)))
         coor(1:3,ivol+4) = 0.5d0*(coor(1:3,face(3,1))+ &
                                   coor(1:3,face(5,1)))
         coor(1:3,ivol+5) = 0.5d0*(coor(1:3,face(3,1))+ &
                                   coor(1:3,face(4,1)))
         coor(1:3,ivol+6) = 0.5d0*(coor(1:3,face(4,1))+ &
                                   coor(1:3,face(5,1)))
         coor(1:3,ivol+7) = 0.5d0*(coor(1:3,face(3,3))+ &
                                   coor(1:3,face(5,3)))
         coor(1:3,ivol+8) = 0.5d0*(coor(1:3,face(3,3))+ &
                                   coor(1:3,face(4,3)))
         coor(1:3,ivol+9) = 0.5d0*(coor(1:3,face(4,3))+ &
                                   coor(1:3,face(5,3)))

!        --- ielemnew := ielemnew+1
!            fill first element
!            1  e1a    2     f1a    6  e3b
!            e7a  f3e  f3a      v4  f5a  f5f
!            7  f3d    8      v1   12  f5b

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(1,ielem)
         kmeshcnew( 2,ielemnew) = edge(1,1)
         kmeshcnew( 3,ielemnew) = kmeshclc(2,ielem)
         kmeshcnew( 4,ielemnew) = face(1,1)
         kmeshcnew( 5,ielemnew) = kmeshclc(6,ielem)
         kmeshcnew( 6,ielemnew) = edge(3,2)

         kmeshcnew( 7,ielemnew) = edge(7,1)
         kmeshcnew( 8,ielemnew) = face(3,5)
         kmeshcnew( 9,ielemnew) = face(3,1)
         kmeshcnew(10,ielemnew) = ivol+4
         kmeshcnew(11,ielemnew) = face(5,1)
         kmeshcnew(12,ielemnew) = face(5,6)

         kmeshcnew(13,ielemnew) = kmeshclc(7,ielem)
         kmeshcnew(14,ielemnew) = face(3,4)
         kmeshcnew(15,ielemnew) = kmeshclc(8,ielem)
         kmeshcnew(16,ielemnew) = ivol+1
         kmeshcnew(17,ielemnew) = kmeshclc(12,ielem)
         kmeshcnew(18,ielemnew) = face(5,2)

!        --- ielemnew := ielemnew+1
!             fill second element
!            3  e2a    4     f1b    2  e1b
!            e8a  f4e  f4a      v5  f3a  f3f
!            9  f4d   10      v2    8  f3b

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(3,ielem)
         kmeshcnew( 2,ielemnew) = edge(2,1)
         kmeshcnew( 3,ielemnew) = kmeshclc(4,ielem)
         kmeshcnew( 4,ielemnew) = face(1,3)
         kmeshcnew( 5,ielemnew) = kmeshclc(2,ielem)
         kmeshcnew( 6,ielemnew) = edge(1,2)

         kmeshcnew( 7,ielemnew) = edge(8,1)
         kmeshcnew( 8,ielemnew) = face(4,5)
         kmeshcnew( 9,ielemnew) = face(4,1)
         kmeshcnew(10,ielemnew) = ivol+5
         kmeshcnew(11,ielemnew) = face(3,1)
         kmeshcnew(12,ielemnew) = face(3,6)

         kmeshcnew(13,ielemnew) = kmeshclc(9,ielem)
         kmeshcnew(14,ielemnew) = face(4,4)
         kmeshcnew(15,ielemnew) = kmeshclc(10,ielem)
         kmeshcnew(16,ielemnew) = ivol+2
         kmeshcnew(17,ielemnew) = kmeshclc(8,ielem)
         kmeshcnew(18,ielemnew) = face(3,2)

!        --- ielemnew := ielemnew+1
!             fill third element
!            5  e3a    6     f1c    4  e2b
!            e9a  f5e  f5a      v6  f4a  f4f
!            11  f5d   12      v3   10  f4b

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(5,ielem)
         kmeshcnew( 2,ielemnew) = edge(3,1)
         kmeshcnew( 3,ielemnew) = kmeshclc(6,ielem)
         kmeshcnew( 4,ielemnew) = face(1,2)
         kmeshcnew( 5,ielemnew) = kmeshclc(4,ielem)
         kmeshcnew( 6,ielemnew) = edge(2,2)

         kmeshcnew( 7,ielemnew) = edge(9,1)
         kmeshcnew( 8,ielemnew) = face(5,5)
         kmeshcnew( 9,ielemnew) = face(5,1)
         kmeshcnew(10,ielemnew) = ivol+6
         kmeshcnew(11,ielemnew) = face(4,1)
         kmeshcnew(12,ielemnew) = face(4,6)

         kmeshcnew(13,ielemnew) = kmeshclc(11,ielem)
         kmeshcnew(14,ielemnew) = face(5,4)
         kmeshcnew(15,ielemnew) = kmeshclc(12,ielem)
         kmeshcnew(16,ielemnew) = ivol+3
         kmeshcnew(17,ielemnew) = kmeshclc(10,ielem)
         kmeshcnew(18,ielemnew) = face(4,2)

!        --- ielemnew := ielemnew+1
!             fill fourth element
!            2  f1b    4     f1c    6  f1a
!            f3a   v5  f4a      v6  f5a   v4
!            8   v2   10      v3   12   v1

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(2,ielem)
         kmeshcnew( 2,ielemnew) = face(1,3)
         kmeshcnew( 3,ielemnew) = kmeshclc(4,ielem)
         kmeshcnew( 4,ielemnew) = face(1,2)
         kmeshcnew( 5,ielemnew) = kmeshclc(6,ielem)
         kmeshcnew( 6,ielemnew) = face(1,1)

         kmeshcnew( 7,ielemnew) = face(3,1)
         kmeshcnew( 8,ielemnew) = ivol+5
         kmeshcnew( 9,ielemnew) = face(4,1)
         kmeshcnew(10,ielemnew) = ivol+6
         kmeshcnew(11,ielemnew) = face(5,1)
         kmeshcnew(12,ielemnew) = ivol+4

         kmeshcnew(13,ielemnew) = kmeshclc(8,ielem)
         kmeshcnew(14,ielemnew) = ivol+2
         kmeshcnew(15,ielemnew) = kmeshclc(10,ielem)
         kmeshcnew(16,ielemnew) = ivol+3
         kmeshcnew(17,ielemnew) = kmeshclc(12,ielem)
         kmeshcnew(18,ielemnew) = ivol+1

!        --- ielemnew := ielemnew+1
!             fill fifth element
!            7  f2d    8      v1   12  f5b
!            e7b  f3h  f3c      v7  f5c  f5g
!            13  e4a   14     f2a   18  e6b

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(7,ielem)
         kmeshcnew( 2,ielemnew) = face(3,4)
         kmeshcnew( 3,ielemnew) = kmeshclc(8,ielem)
         kmeshcnew( 4,ielemnew) = ivol+1
         kmeshcnew( 5,ielemnew) = kmeshclc(12,ielem)
         kmeshcnew( 6,ielemnew) = face(5,2)

         kmeshcnew( 7,ielemnew) = edge(7,2)
         kmeshcnew( 8,ielemnew) = face(3,8)
         kmeshcnew( 9,ielemnew) = face(3,3)
         kmeshcnew(10,ielemnew) = ivol+7
         kmeshcnew(11,ielemnew) = face(5,3)
         kmeshcnew(12,ielemnew) = face(5,7)

         kmeshcnew(13,ielemnew) = kmeshclc(13,ielem)
         kmeshcnew(14,ielemnew) = edge(4,1)
         kmeshcnew(15,ielemnew) = kmeshclc(14,ielem)
         kmeshcnew(16,ielemnew) = face(2,1)
         kmeshcnew(17,ielemnew) = kmeshclc(18,ielem)
         kmeshcnew(18,ielemnew) = edge(6,2)

!        --- ielemnew := ielemnew+1
!             fill sixth element
!            9  f4d   10      v2    8  f3b
!            e8b  f4h  f4c      v8  f3c  f3g
!            15  e5a   16     f2b   14  e4b

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(9,ielem)
         kmeshcnew( 2,ielemnew) = face(4,4)
         kmeshcnew( 3,ielemnew) = kmeshclc(10,ielem)
         kmeshcnew( 4,ielemnew) = ivol+2
         kmeshcnew( 5,ielemnew) = kmeshclc(8,ielem)
         kmeshcnew( 6,ielemnew) = face(3,2)

         kmeshcnew( 7,ielemnew) = edge(8,2)
         kmeshcnew( 8,ielemnew) = face(4,8)
         kmeshcnew( 9,ielemnew) = face(4,3)
         kmeshcnew(10,ielemnew) = ivol+8
         kmeshcnew(11,ielemnew) = face(3,3)
         kmeshcnew(12,ielemnew) = face(3,7)

         kmeshcnew(13,ielemnew) = kmeshclc(15,ielem)
         kmeshcnew(14,ielemnew) = edge(5,1)
         kmeshcnew(15,ielemnew) = kmeshclc(16,ielem)
         kmeshcnew(16,ielemnew) = face(2,2)
         kmeshcnew(17,ielemnew) = kmeshclc(14,ielem)
         kmeshcnew(18,ielemnew) = edge(4,2)

!        --- ielemnew := ielemnew+1
!             fill seventh element
!            11  f5d   12      v3   10  f4b
!            e9b  f5h  f5c      v9  f4c  f4g
!            17  e6a   18     f2c   16  e5b

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(11,ielem)
         kmeshcnew( 2,ielemnew) = face(5,4)
         kmeshcnew( 3,ielemnew) = kmeshclc(12,ielem)
         kmeshcnew( 4,ielemnew) = ivol+3
         kmeshcnew( 5,ielemnew) = kmeshclc(10,ielem)
         kmeshcnew( 6,ielemnew) = face(4,2)

         kmeshcnew( 7,ielemnew) = edge(9,2)
         kmeshcnew( 8,ielemnew) = face(5,8)
         kmeshcnew( 9,ielemnew) = face(5,3)
         kmeshcnew(10,ielemnew) = ivol+9
         kmeshcnew(11,ielemnew) = face(4,3)
         kmeshcnew(12,ielemnew) = face(4,7)

         kmeshcnew(13,ielemnew) = kmeshclc(17,ielem)
         kmeshcnew(14,ielemnew) = edge(6,1)
         kmeshcnew(15,ielemnew) = kmeshclc(18,ielem)
         kmeshcnew(16,ielemnew) = face(2,3)
         kmeshcnew(17,ielemnew) = kmeshclc(16,ielem)
         kmeshcnew(18,ielemnew) = edge(5,2)

!        --- ielemnew := ielemnew+1
!             fill element eight
!            8   v2   10      v3   12   v1
!            f3c   v8  f4c      v9  f5c   v7
!            14  f2b   16     f2c   18  f2a

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(8,ielem)
         kmeshcnew( 2,ielemnew) = ivol+2
         kmeshcnew( 3,ielemnew) = kmeshclc(10,ielem)
         kmeshcnew( 4,ielemnew) = ivol+3
         kmeshcnew( 5,ielemnew) = kmeshclc(12,ielem)
         kmeshcnew( 6,ielemnew) = ivol+1

         kmeshcnew( 7,ielemnew) = face(3,3)
         kmeshcnew( 8,ielemnew) = ivol+8
         kmeshcnew( 9,ielemnew) = face(4,3)
         kmeshcnew(10,ielemnew) = ivol+9
         kmeshcnew(11,ielemnew) = face(5,3)
         kmeshcnew(12,ielemnew) = ivol+7

         kmeshcnew(13,ielemnew) = kmeshclc(14,ielem)
         kmeshcnew(14,ielemnew) = face(2,2)
         kmeshcnew(15,ielemnew) = kmeshclc(16,ielem)
         kmeshcnew(16,ielemnew) = face(2,3)
         kmeshcnew(17,ielemnew) = kmeshclc(18,ielem)
         kmeshcnew(18,ielemnew) = face(2,1)

         if ( fillmap ) then
            imap(ivol+1:ivol+9,1) = 2
            imap(ivol+1,2) = kmeshclc(8,ielem)
            imap(ivol+1,3) = kmeshclc(12,ielem)
            imap(ivol+2,2) = kmeshclc(8,ielem)
            imap(ivol+2,3) = kmeshclc(10,ielem)
            imap(ivol+3,2) = kmeshclc(10,ielem)
            imap(ivol+3,3) = kmeshclc(12,ielem)
            imap(ivol+4,2) = face(3,1)
            imap(ivol+4,3) = face(5,1)
            imap(ivol+5,2) = face(3,1)
            imap(ivol+5,3) = face(4,1)
            imap(ivol+6,2) = face(4,1)
            imap(ivol+6,3) = face(5,1)
            imap(ivol+7,2) = face(3,3)
            imap(ivol+7,3) = face(5,3)
            imap(ivol+8,2) = face(3,3)
            imap(ivol+8,3) = face(4,3)
            imap(ivol+9,2) = face(4,3)
            imap(ivol+9,3) = face(5,3)
            rmap(ivol+1:ivol+9,1:2) = 0.5d0
         end if  ! ( fillmap )

      end do  ! ielem = 1, inelem

1000  call erclos ( 'mshrefkmeshc22' )

      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshcnew, ielemnew, 18, 'kmeshcnew/surface' )
         write(irefwr,*) 'End mshrefkmeshc22'

      end if

      end
