      subroutine mshrefkmeshcs6 ( kmeshcnew, nelem, nelprev, &
                                  nodenumbers, nmesha, nmeshb, nmeshg, &
                                  nedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 29-02-2012 abs(iface)
!        version  1.0  date 14-09-2003
!
!   copyright (c) 2003-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshc for one element group in case that
!     the complete mesh is refined
!     Elements of shape 6, i.e. bi-quadratic quadrilaterals
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     quadratic
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(9,*), nelem, nodenumbers(*), nelprev, &
              nmesha(*), nmeshb(*), nmeshg(*), nedges

!     kmeshcnew      o    New kmesh part c
!     nedges         i    number of edges
!     nelem          i    Number of elements
!     nelprev        i    Number of elements stored in kmeshc at the start
!                         of the subroutine
!     nmesha         i    nmesh part a
!     nmeshb         i    nmesh part b
!     nmeshg         i    nmesh part g
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielemnew, ielem, ipstart, iedge, edge(4,2), i, iface, &
              face(8)

!     edge           Contains the edge node numbers on the 4 edges (2 per edge)
!     face           Contains the face node numbers on the 8 faces
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielemnew       Sequence number of element in new mesh
!     iface          Face sequence number
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHREFKMESHC6E Refine one element in case of a complete refinement
!                    Element of shape 6, i.e. bi-quadratic quadrilaterals
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The bi-quadratic quadrilateral has 9 nodes according to
!
!            7      6      5
!            x------x------x
!            |             |
!            |             |
!            |      9      |
!         8  x      x      x  4
!            |             |
!            |             |
!            |             |
!            x------x------x
!           1       2      3
!
!
!     The eight face nodes are defined as follows
!     f_1 between 2 and 9
!     f_2 between 4 and 9
!     f_3 between 6 and 9
!     f_4 between 8 and 9
!     The other 4 in the centers of the 4 quadrants:
!     f_5 in  1 - 2 - 9 - 8
!     f_6 in  2 - 3 - 9 - 2
!     f_7 in  4 - 5 - 6 - 9
!     f_8 in  6 - 7 - 8 - 9
!
!     The edges are defines as: 1-3,  3-5,  5-7, 7-1
!     Let the corresponding nodes be e1, e2, e3 and e4
!
!            4     e3    3
!            x-----0-----x
!            |           |
!            |           |
!        e4  0     +     0  e2
!            |           |
!            |           |
!            x-----0-----x
!           1     e1     2
!
!
!     On each edge we have two extra nodes: e1a/b etc
!     Then the 4 new quadrilaterals become:
!     1 e1a 2 f_1 9 f_4 8 e4b f_5
!     3 e2a 4 f_2 9 f_1 2 e1b f_6
!     5 e3a 6 f_3 9 f_2 4 e2b f_7
!     7 e4a 8 f_4 9 f_3 6 e3b f_8
!
!     ielemnew := 0
!     For all elements ielem in old mesh do
!        iface := face_number of element
!        ielemnew := ielemnew+1
!        fill first element
!        ielemnew := ielemnew+1
!        fill second element
!        ielemnew := ielemnew+1
!        fill third element
!        ielemnew := ielemnew+1
!        fill fourth element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshcs6' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshcs6'
         write(irefwr,*) 'nelem, nelprev ', nelem, nelprev

      end if
      if ( ierror/=0 ) go to 1000

!     --- ielemnew := 0

      ielemnew = 0

!     --- For all elements ielem in old mesh do

      do ielem = 1, nelem

!     --- iface := face_number of element
!         ielemnew := ielemnew+1
!         fill first element
!         ielemnew := ielemnew+1
!         fill second element
!         ielemnew := ielemnew+1
!         fill third element
!         ielemnew := ielemnew+1
!         fill fourth element

         ipstart = nmesha(nelprev+ielem)
         iface = abs(nmesha(ipstart+1))
         ipstart = nmeshb(iface)
         do i = 1, 4
            iedge = nmeshb(ipstart+i)
            if ( iedge>0 ) then
               edge(i,1) = nodenumbers(iedge)       ! 4 edge numbers / a
               edge(i,2) = nodenumbers(iedge)+1     ! 4 edge numbers / b
            else  ! ( iedge<0 )
               edge(i,1) = nodenumbers(-iedge)+1    ! 4 edge numbers / a
               edge(i,2) = nodenumbers(-iedge)      ! 4 edge numbers / b
            end if  ! ( iedge>0 )
         end do
         ipstart = nmeshg(iface)
         iface = nodenumbers(iface+nedges)
         do i = 0, 7
            face(i+1) = iface+i
         end do

         call mshrefkmeshc6e ( nmeshg(ipstart+1), kmeshcnew, ielemnew, &
                               edge, face )

      end do  ! ielem = 1, nelem

1000  call erclos ( 'mshrefkmeshcs6' )

      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshcnew, ielemnew, 3, 'kmeshcnew/surface' )
         write(irefwr,*) 'End mshrefkmeshcs6'

      end if

      end
