      subroutine mshrefkmeshm1d ( kmeshd, kmeshm, kmeshmnew, nelem, &
                                  leng, nodenumbers, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-02-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy information of old curve as stored in kmesh part m into
!     new curve extended with intermediate point in case a mesh is refined
!     completely
!     This is the special 1D case when no edges are available
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     curve
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshd(*), kmeshm(*), kmeshmnew(*), nelem, &
              leng, nodenumbers(*), npoint

!     kmeshd         i    KMESH part d corresponding to old mesh
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshmnew      o    New array kmeshm
!     leng           i    Help parameter to indicate the length of arrays
!     nelem          i    Number of elements in old mesh
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
!     npoint         i    Number of nodal points in old mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iwork(nelem), inode1, inode2, inode, ielem, j, ipstart, &
              numelms

!     i              Counting variable
!     ielem          Element number
!     inode          Node sequence number
!     inode1         First node in element
!     inode2         Second node in element
!     ipstart        Starting address in array
!     iwork          integer work array of length nelem
!                    if iwork(i) = 1 node belongs to element i
!     j              Counting variable
!     numelms        Number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1500   No element found that contains both nodal points
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all sets of 2 nodes on curve do
!        ielem := common element for both nodes
!        determine new node number from nodenumbers
!        copy first node and new node to kmeshmnew
!     copy last node to kmeshmnew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshm1d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshm1d'
         call prinin ( kmeshm, leng, 'kmeshm/curve' )
         write(irefwr,*) 'nelem, npoint ', nelem, npoint

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Clear array iwork

      do i = 1, nelem
         iwork(i) = 0
      end do

!     --- For all sets of 2 nodes on curve do

      do i = 1, leng-1

!     --- ielem := common element for both nodes

         inode1 = kmeshm(i)           ! first node in set
         inode2 = kmeshm(i+1)         ! second node in set

!        --- set iwork 1 for all elements connected to node 1

         ipstart = kmeshd(inode1)+npoint+1
         numelms = kmeshd(inode1+1)-kmeshd(inode1)
         if ( debug ) &
            write(irefwr,*) 'inode1, inode2, numelms ', &
                             inode1, inode2, numelms
         do j = 1, numelms
            iwork(kmeshd(ipstart+j)) = 1
         end do

!        --- Find element that also corresponds to inode2

         ipstart = kmeshd(inode2)+npoint+1
         numelms = kmeshd(inode2+1)-kmeshd(inode2)

         ielem = 0
         do j = 1, numelms

            if ( iwork(kmeshd(ipstart+j)).eq.1 ) then

!           --- element found

               ielem = kmeshd(ipstart+j)
               go to 100

            end if

         end do

100      if ( ielem.eq.0 ) then

!        --- error: no element found

            call errint ( inode1, 1 )
            call errint ( inode2, 2 )
            call errsub ( 1500, 2, 0, 0 )
            go to 1000

         end if

!        --- determine new node number from nodenumbers
!            copy first node and new node to kmeshmnew

         inode = nodenumbers(ielem)
         kmeshmnew(2*i-1) = inode1
         kmeshmnew(2*i) = inode
         if ( debug ) write(irefwr,*) 'i ,inode, inode1, ielem ', &
            i ,inode, inode1, ielem

!        --- clear iwork

         ipstart = kmeshd(inode1)+npoint+1
         numelms = kmeshd(inode1+1)-kmeshd(inode1)
         do j = 1, numelms
            iwork(kmeshd(ipstart+j)) = 0
         end do

      end do  ! i = 1, leng-1

!     --- copy last node to kmeshmnew

      kmeshmnew(2*leng-1) = inode2

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kmeshmnew, 2*leng-1, 'kmeshmnew/curve' )
         write(irefwr,*) 'End mshrefkmeshm1d'

      end if
      call erclos ( 'mshrefkmeshm1d' )

      end
