      subroutine mshrefkmesht
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 02-09-2010 Remove work arrays
!        version  2.0  date 21-03-2007 Extra parameters iwork1/2
!        version  1.0  date 20-09-2003
!
!   copyright (c) 2003-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Fill array kmesht in case of a refinement of a complete mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, iseq
      integer, allocatable :: iwork1(:), iwork2(:)

!     i              Counting variable
!     iseq           Sequence number
!     istart         First position used at a part
!     iwork1         integer work array of length max(nsurfs,nvolms)
!                    to store the first entries of kmesht
!     iwork2         integer work array of length max(nsurfs,nvolms)
!                    to store the ordered sequence of iwork1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT         Sort integer array for increasing sequence
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmesht' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmesht'
         call ertrace

      end if
      if ( ierror/=0 ) go to 1000

      i = max(ncurvs, nsurfs, nvolms)

      allocate ( iwork1(i), iwork2(i), stat = error )
      if ( error/=0 ) &
         call eralloc (  error, 2*i, 'iwork1' )
      if ( ierror/=0 ) go to 1000

      if ( nsurfs==0 .and. nvolms==0 ) then

!     --- only line elements

         do i = 1, ncurvs
            kmesht(1,i) = kmesht(1,i)*2-1
            kmesht(2,i) = kmesht(2,i)*2
         end do
         if ( debug ) call prinin1 ( kmesht, ncurvs, 2, 'kmesht' )

      else if ( nvolms==0 ) then

!     --- surface elements
!         First compute number of line elements

         istart = 1
         do i = 1, nelgrp
            ishape = kmeshf(i)
            if ( ishape<3 ) istart = istart+kmeshg(i)
         end do

!        --- Sort kmesht for increasing first index
!            First index is stored in iwork1
!            In iwork2 the increasing sequence is stored

         do i = 1, nsurfs
            iwork1(i) = kmesht(1,i)
         end do  ! i = 1, nsurfs
         call chsort ( iwork1, iwork2, nsurfs )

         do i = 1, nsurfs

            iseq = iwork2(i)
            if ( kmesht(1,iseq)>0 ) then

!           --- Surface contains nodes

               inelem = (kmesht(2,iseq)-kmesht(1,iseq)+1)*4
               kmesht(1,iseq) = istart
               kmesht(2,iseq) = istart+inelem-1
               istart = istart+inelem

            end if  ! ( kmesht(1,iseq)>0 )

         end do

         if ( debug ) call prinin1 ( kmesht, nsurfs, 2, 'kmesht' )

      else

!     --- volume elements
!         First compute number of line and surface elements

         istart = 1
         do i = 1, nelgrp
            ishape = kmeshf(i)
            if ( ishape<11 ) istart = istart+kmeshg(i)
         end do

!        --- Sort kmesht for increasing first index
!            First index is stored in iwork1
!            In iwork2 the increasing sequence is stored

         do i = 1, nvolms
            iwork1(i) = kmesht(1,i)
         end do  ! i = 1, nsurfs
         call chsort ( iwork1, iwork2, nvolms )

         do i = 1, nvolms

            iseq = iwork2(i)
            if ( kmesht(1,iseq)>0 ) then

!           --- Volume contains nodes

               inelem = (kmesht(2,iseq)-kmesht(1,iseq)+1)*8
               kmesht(1,iseq) = istart
               kmesht(2,iseq) = istart+inelem-1
               istart = istart+inelem

            end if  ! ( kmesht(1,iseq)>0 )

         end do
         if ( debug ) call prinin1 ( kmesht, nvolms, 2, 'kmesht' )

      end if

      deallocate ( iwork1, iwork2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork1' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefkmesht'

      end if
      call erclos ( 'mshrefkmesht' )

      end
