      subroutine mshspcboun ( iinput, input, tau, &
                              factor, curves, rinput, coorh, &
                              kmeshmnew, iwork, lncurves, lenkmeshmnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 11-12-2014 Adaptation due to new curve routines
!        version  2.1  date 08-09-2010 Remove work arrays
!        version  2.0  date 07-11-2009 Remove some parameters
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coordinates on curves in case of a spectral mesh
!     The coordinates of the original curves have already been filled
!     The curvature of the mesh is taken into account
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     spectral
!     curve
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), input(*), iwork(*), &
              lncurves, lenkmeshmnew, kmeshmnew(lenkmeshmnew)
      double precision tau(*), factor(0:*), curves(ndim,lncurves), &
                       rinput(*), coorh(ndim,*)

!     coorh         i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     curves              work array to store the coordinates of the curves
!     factor         i    array with the distribution of nodes along a
!                         straight line from -1 to +1
!     iinput         i    integer input array related to the mesh
!     input          i    array in which is described where and how many points
!                         have to be placed
!     iwork          i    integer work array of length ncurvs
!                         if iwork(i)=0 curve i has been filled
!                         if iwork(i)=1 curve i has not been filled
!     kmeshmnew      o    New array kmeshm
!     lenkmeshmnew   i    length of array kmeshmnew
!     lncurves       i    Size of second dimension of array curves
!     rinput         i    Real input array related to the mesh
!     tau                 Work array of length n to store the relative distances
!                         between points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iinp, icurve, np, &
              itaust, itauen, inode, istar3, &
              iinpsav, iprec, istold, ileold, istrht, ipos, &
              kcurnr, ilenew, istkrh, istnew, isubd, j, nelm, &
              iprev, istartrel
      double precision ds, afst, xold(3), t1(3), som, t2(3), angle, &
                       t3(3), xnew(3), ta1(3), ta2(3), ta3(3), axp, &
                       ayp, azp, curstr(10*ncurvs), alpha, beta, gamma
      integer, allocatable :: irelcurves(:)

!     afst           distance
!     alpha          help coefficient first component
!     angle          angle between two vectors
!     axp            help coefficient
!     ayp            help coefficient
!     azp            help coefficient
!     beta           help coefficient second component
!     curstr         help variable for coarseness ; not (yet) used here
!     ds             distance
!     gamma          help coefficient third component
!     i              Counting variable
!     icurve         Type of curve
!     iinp           last position used in array iinput
!     iinpsav        Help variable to save the value of iinp
!     ilenew         length of new curve (number of nodes)
!     ileold         length of old curve (number of nodes)
!     inode          Node sequence number
!     ipos           pointer to local position
!     iprec          Curve number of curve to be copied etc.
!     iprev          Previous node number used
!     irelcurves     dummy
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istartrel      dummy
!     istkrh         start position in kmeshm
!     istnew         start position new curve
!     istold         start position old curve
!     istrht         indicator whether straight line (1) or not (0)
!     isubd          Defines the type of subdivision of the nodes in the
!                    element in case of higher order elements
!     itauen         last position of local piece to be considered
!     itaust         first position of local piece to be considered
!     j              Counting variable
!     kcurnr         loop variable curves
!     nelm           number of elements along a curve
!     np             total number of points in new line
!     som            Help parameter to compute a sum
!     t1             first base vector old curve
!     t2             second base vector old curve
!     t3             third base vector old curve
!     ta1            first base vector new curve
!     ta2            second base vector new curve
!     ta3            third base vector new curve
!     xnew           origin for new curve
!     xold           origin for old curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
!     MSH006         Fill array curves in case of circle segment
!     MSH031         Fill array curves in case of given coarseness
!     MSHSEQCURVS    Compute sequence in which curves must be created
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     994   Intermediate points not yet implemented for this type of curves
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Detect sequence of curves and store in array iseqcurvs
!     Check whether coordinates have to be adjusted
!     For all curves do
!        Adapt coordinates
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshspcboun' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshspcboun'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      niedge = input(1)
      isubd  = input(4)
      istartrel = 2*ncurvs+1

      allocate ( irelcurves(iinp3), stat = error )
      if ( error/=0 ) call eralloc ( error, iinp3, 'irelcurves' )
      if ( ierror/=0 ) go to 1000

      if ( .not. alloccurv ) then
         allocate ( ipointcurv(ncurvs), iseqcurvs(ncurvs), stat = error )
         if ( error/=0 ) call eralloc ( error, 2*ncurvs, 'irelcurves' )
         if ( ierror/=0 ) go to 1000
      end if  ! ( .not. alloccurv )

!     --- Detect sequence of curves and store in array iseqcurvs

      call mshseqcurvs ( iinput )

!     --- Check whether coordinates have to be adjusted

      istar3 = kmeshmnew(2)-1
      ishape = niedge+1

!     --- Loop over curves

      do kcurnr = 1, ncurvs

!     --- extract curve number and starting address in iinput

         icurnr = iseqcurvs(kcurnr)

!        --- skip curves that have not been filled

         if ( iwork(icurnr)==1 ) go to 500

         iinp = ipointcurv(kcurnr)

         icurve = iinput(iinp)
         istkrh = istar3+kmeshmnew(4+icurnr)
         nelm = kmeshm(icurnr+5)-kmeshm(icurnr+4)-1

!        --- Determine number of nodes on (new) curve

         np = iinput( iinp+2 ) * ( 1+niedge )+1
         if ( debug ) &
            write(irefwr,1) 'kcurnr, icurnr, icurve, nelm, np', &
                             kcurnr, icurnr, icurve, nelm, np

         if ( icurve==3 ) then

!        --- Arc special treatment
!            Start to fill array tau

            ds = 1d0 / ( ( np-1 ) * 1d0 )

            do i = 1, np

               tau(i) = (i-1) * ds

            end do

!           --- Local sub-division

            do i = 1, iinput( iinp+2 )

               itaust = 1+(i-1) * (niedge+1)
               itauen = 1+ i    * (niedge+1)

               afst   = tau( itauen )-tau( itaust )

               do j = 1, niedge

                  tau(itaust+j) = tau(itaust) + &
                          ( 1d0+factor(j) ) * afst*0.5d0

               end do

            end do

            call msh006 ( .false., ndim, curves, iinput(iinp+4), rinput, &
                           tau, np, nuspnt, curstr, icurnr, icurve )

!           --- Copy these coordinates into the correct places of the
!               coordinate array

            do i = 1, np

                inode = kmeshmnew(istkrh-1+i)
                coorh(1:ndim,inode) = curves(1:ndim,i)

            end do

         else if ( icurve==5 .or. icurve==6 .or. icurve==9 .or. &
                   icurve==12 ) then

!        --- icurve = 5, 6, 9 or 12    Coarseness

            jcoars=1
            irnp2 = (ndim+jcoars)*nuspnt+2*jcoars+3
            iinpsav = iinp
            if ( debug ) write(irefwr,1) 'irnp2, iinp', irnp2, iinp

            if ( debug ) write(irefwr,1) 'iinp, nelm', iinp, nelm

            iinp = iinpsav
            call msh031 ( icurnr, ncurvs, iinput(5), iinp, &
                          curves, rinput, nelm, &
                          ishape, nuspnt, np, ndim, &
                          icurve, curstr, tau, isubd, &
                          rinput(irnp2+ncurvs+2*icurnr-2), curvefacts, &
                          irelcurves, istartrel )
            iinp = iinpsav

            do i = 1, np

                inode = kmeshmnew(istkrh-1+i)
                coorh(1:ndim,inode) = curves(1:ndim,i)

            end do  ! i = 1, np

         else if ( icurve==10 .or. icurve==11 .or. icurve==13 ) &
            then

!        --- Translation, Rotation or Reflection
!            Determine number of curve that has to be rotated

            iprec = iinput(iinp+4)

!           --- Determine the starting address of the nodes on the original
!               curve to be rotated

            istold = kmeshm(4+iprec)+kmeshm(2)-1
            ileold = kmeshm(5+iprec)-kmeshm(4+iprec)

!           --- Set value of istrht (zero means no straight line)

            istrht = 0

!           --- Determine local basis for old points

            xold(3) = 0d0

            xold(1:ndim) = coorh(1:ndim,kmeshm(istold))

!           --- Base vector T1

            t1(3) = 0d0

            t1(1:ndim) = coorh(1:ndim,kmeshm(istold+1))-xold(1:ndim)

!           --- Normalize

            som = sqrt( t1(1)*t1(1)+t1(2)*t1(2)+t1(3)*t1(3) )

            t1(1:ndim) = t1(1:ndim)/som

!           --- Base vector T2

            iprev = kmeshm(istold+1)
            if ( ileold>2 ) then

               do i = 2, ileold-1

!              --- If two equal nodal points are found this node must be skipped
!                  This is the case if the curve to be copied is a curve of
!                  curves

                  if ( kmeshm(istold+i)==iprev ) go to 100
                  iprev = kmeshm(istold+i)
                  t2(3) = 0d0

                  t2(1:ndim) = coorh(1:ndim,kmeshm(istold+i))- &
                               xold(1:ndim)

!                 --- Normalize

                  som = sqrt(t2(1)*t2(1)+t2(2)*t2(2)+t2(3)*t2(3))

                  t2(1:ndim) = t2(1:ndim)/som

!                 --- Compute angle of two vectors

                  angle = t1(1)*t2(1)+t1(2)*t2(2)+t1(3)*t2(3)

                  if ( abs(angle)<0.98d0 ) goto 450

100               iprev = kmeshm(istold+i)

               end do

            end if

!           --- Angle never large enough , original curve is straight line

            istrht = 1

!           --- Make basis orthonormal if not straight line
!               (Use Gramm-Schmidt process)

450         if ( istrht==0 ) then

               t2(1) = t2(1)-angle * t1(1)
               t2(2) = t2(2)-angle * t1(2)
               t2(3) = t2(3)-angle * t1(3)

!              --- Normalize

               som = sqrt( t2(1)*t2(1)+t2(2)*t2(2)+t2(3)*t2(3) )

               t2(1) = t2(1) / som
               t2(2) = t2(2) / som
               t2(3) = t2(3) / som

!              --- Compute third vector of orthogonal system

               t3(1) = t1(2)*t2(3)-t2(2)*t1(3)
               t3(2) = t1(3)*t2(1)-t2(3)*t1(1)
               t3(3) = t1(1)*t2(2)-t2(1)*t1(2)

            else

               t2(1) = 0d0
               t2(2) = 0d0
               t2(3) = 0d0

               t3(1) = 0d0
               t3(2) = 0d0
               t3(3) = 0d0

            end if

!           --- Determine the starting address of the nodes on the (old) rotated
!               curve

            istold = kmeshm(4+icurnr)+kmeshm(2)-1
            ileold = kmeshm(5+icurnr)-kmeshm(4+icurnr)

!           --- Compute basis for rotated curve

            xnew(3) = 0d0

            xnew(1:ndim) = coorh(1:ndim,kmeshm(istold))

!           --- Determine vector Ta1

            ta1(3) = 0d0

            ta1(1:ndim) = coorh(1:ndim,kmeshm(istold+1))-xnew(1:ndim)

!           --- Normalize

            som = sqrt(ta1(1)*ta1(1)+ta1(2)*ta1(2)+ta1(3)*ta1(3))

            ta1(1:ndim) = ta1(1:ndim)/som

            if ( istrht==0 ) then

!           --- Determine vector Ta2

               do i = 2, ileold-1

                  ta2(3) = 0d0

                  ta2(1:ndim) = coorh(1:ndim,kmeshm(istold+i))- &
                                xnew(1:ndim)

!                 --- Normalize

                  som = sqrt(ta2(1)*ta2(1)+ta2(2)*ta2(2)+ta2(3)*ta2(3))

                  ta2(1:ndim) = ta2(1:ndim)/som

                  angle = ta1(1)*ta2(1)+ta1(2)*ta2(2)+ta1(3)*ta2(3)

                  if ( abs(angle)<0.98d0 ) goto 550

               end do

!              --- Angle never large enough , error ?
!                  Make basis orthonormal
!                  (Use Gramm-Schmidt process)

550            ta2(1) = ta2(1)-angle * ta1(1)
               ta2(2) = ta2(2)-angle * ta1(2)
               ta2(3) = ta2(3)-angle * ta1(3)

               som = sqrt(ta2(1)*ta2(1)+ta2(2)*ta2(2)+ta2(3)*ta2(3))

               ta2(1) = ta2(1) / som
               ta2(2) = ta2(2) / som
               ta2(3) = ta2(3) / som

!              --- Compute third basis vector

               ta3(1) = ta1(2)*ta2(3)-ta2(2)*ta1(3)
               ta3(2) = ta1(3)*ta2(1)-ta2(3)*ta1(1)
               ta3(3) = ta1(1)*ta2(2)-ta2(1)*ta1(2)

            else

!           --- Straight line

               ta2(1) = 0
               ta2(2) = 0
               ta2(3) = 0

               ta3(1) = 0
               ta3(2) = 0
               ta3(3) = 0

            end if

!           --- Finally rotate all co-ordinates
!               (nodes are given in array kmeshmnew)
!               of first curve iprec into present curve icurnr
!               Determine starting positions and length of curves

            istold = kmeshmnew(4+iprec)+kmeshmnew(2)-1

            istnew = kmeshmnew(4+icurnr)+kmeshmnew(2)-1
            ilenew = kmeshmnew(5+icurnr)-kmeshmnew(4+icurnr)
            iprev = 0

            do i = 1, ilenew

!           --- If two equal nodal points are found this node must be skipped
!               This is the case if the curve to be copied is a curve of
!               curves

               if ( kmeshmnew(istold+i-1)==iprev ) istold = istold+1
               iprev = kmeshmnew(istold+i-1)
               axp = coorh(1,kmeshmnew(istold+i-1))-xold(1)
               ayp = coorh(2,kmeshmnew(istold+i-1))-xold(2)
               if ( ndim==3 ) then
                  azp = coorh(3,kmeshmnew(istold+i-1))-xold(3)
               else
                  azp = 0d0
               end if

!              --- Compute coefficients of this node in old system

               if ( istrht==1 ) then

                  alpha = sqrt(axp*axp+ayp*ayp+azp*azp)
                  beta  = 0d0
                  gamma = 0d0

               else

                  alpha = axp*t1(1)+ayp*t1(2)+azp*t1(3)
                  beta  = axp*t2(1)+ayp*t2(2)+azp*t2(3)
                  gamma = axp*t3(1)+ayp*t3(2)+azp*t3(3)

               end if

!              --- New coordinates, use computed coefficients in new
!                  system

               ipos = kmeshmnew(istnew+i-1)

               coorh(1:ndim,ipos) = xnew(1:ndim)+alpha * ta1(1:ndim) + &
                                beta * ta2(1:ndim) +gamma * ta3(1:ndim)

            end do

         else if ( icurve==4 .or. icurve==6 .or. icurve==12 ) &
            then

!        --- Arc with coarseness or spline: not yet implemented

            call errwar ( 994, 0, 0, 0 )

         end if

500   end do

      deallocate ( irelcurves, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'irelcurves' )

      if ( .not. alloccurv ) then
         deallocate ( ipointcurv, iseqcurvs, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'ipointcurv' )
      end if  ! ( .not. alloccurv )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshspcboun'

      end if  ! ( debug )
      call erclos ( 'mshspcboun' )

      end
