      subroutine mshsrfcopy ( isurnr, iprec, kmeshs, kmeshscp, ncurvs, &
                              lenkmeshscp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 04-07-2006 Fortran 90
!        version  1.0  date 05-04-2002
!
!   copyright (c) 2002-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if kmeshs part 3 in surface to be copied is equal to kmeshcp
!     If not change direction in kmeshcp for present surface
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     direction
!     outer_boundary
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr, iprec, kmeshs(*), kmeshscp(*), ncurvs, &
              lenkmeshscp

!     iprec          i    Surface number of surface to be copied etc.
!     isurnr         i    Surface sequence number
!     kmeshs         i    Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     kmeshscp       i    Copy of kemshs part 3
!                         The curves may be reversed so that all boundary
!                         parts have curves pointing in the same direction
!     lenkmeshscp    i    Length of kmeshs parts 1 + 2 or 3
!     ncurvs         i    Number of curves in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(ncurvs), istart, numcrv, i, icurnr, jcurnr, iend, j
      logical same

!     i              Counting variable
!     icurnr         Curve sequence number
!     iend           Last position used at a part
!     istart         First position used at a part
!     iwork          integer work array
!     j              Counting variable
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     numcrv         number of parts where curves are given for generation
!                    of boundary elements
!     same           If true the arrays kmeshs and kmeshscp are the same
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2335   Inconsistency in boundary
!    2336   Number of curves in surface to be copied different from new
!           surface
! **********************************************************************
!
!                       PSEUDO CODE
!
!     same = kmeshs part 3 is kmeshscp for surface iprec
!     if ( not same ) then
!        Store new sequence for surface iprec in iwork
!        Copy surface surnr in new sequence
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsrfcopy' )
      debug = .false.

!     --- same = kmeshs part 3 is kmeshscp for surface iprec

      same = .true.
      istart = kmeshs(iprec)
      numcrv = kmeshs(iprec+1)-istart

      if ( debug ) then

!     --- debug statements

         write(irefwr,*) 'in mshsrfcopy: iprec, isurnr ',iprec, isurnr
         call prinin(kmeshs(istart+1),numcrv,'kmeshs')
         call prinin(kmeshscp(istart+1-lenkmeshscp),numcrv,'kmeshscp')

      end if

      do i = 1, numcrv

         if ( kmeshs(istart+i)/=kmeshscp(istart+i-lenkmeshscp) ) then

!        --- sequence is different

            same = .false.
            go to 100

         end if

      end do

100   if ( .not. same ) then

!     --- not same: reverse sequence in isurnr
!         Store new sequence for surface iprec in iwork

         i = 0
         do while ( i<numcrv )

!        --- Consider next curve in kmeshs part 3 and kmeshscp

            i = i+1
            icurnr = kmeshs(istart+i)
            jcurnr = kmeshscp(istart+i-lenkmeshscp)
            if ( icurnr==jcurnr ) then

!           --- Both curves are the same

               iwork(i) = i

            else

!           --- Both curves are different, hence there must be hole
!               Find the end of the closed boundary

               iend = 0
               do j = i, numcrv

                  jcurnr = kmeshscp(istart+j-lenkmeshscp)
                  if ( jcurnr==-icurnr ) then

!                 --- end of closed boundary found
!                     the boundary conists of i ... j

                     iend = j
                     go to 200
                  end if

               end do  ! j = i, numcrv

200            if ( iend==0 ) then

!              --- internal error

                  call errint ( iprec, 1 )
                  call errint ( icurnr, 2 )
                  call errsub ( 2335, 2, 0, 0 )
                  go to 1000

               else

!              --- fill iwork and raise i

                  do j = i, iend
                     iwork(j) = -(iend+i-j)
                  end do
                  i = iend

               end if

            end if  ! icurnr==jcurnr

         end do  ! while ( i<numcrv )

         if ( debug ) then

!        --- debug statements

            call prinin ( iwork, numcrv, 'iwork' )

         end if

!        --- Copy surface isurnr in new sequence

         istart = kmeshs(isurnr)
         if ( numcrv/=kmeshs(isurnr+1)-istart ) then

!        --- internal error: number of curves not the same in both surfaces

            call errint ( iprec, 1 )
            call errint ( isurnr, 2 )
            call errsub ( 2336, 2, 0, 0 )
            go to 1000

         end if
         do i = 1, numcrv
            if ( iwork(i)>0 ) then
               icurnr = kmeshs(istart+iwork(i))
            else
               icurnr = -kmeshs(istart-iwork(i))
            end if
            kmeshscp(istart+i-lenkmeshscp) = icurnr
         end do

         if ( debug ) then

!        --- debug statements

            call prinin(kmeshs(istart+1),numcrv,'kmeshs')
            call prinin(kmeshscp(istart+1-lenkmeshscp),numcrv, &
                        'kmeshscp')

         end if

      end if  ! ( .not. same )

1000  call erclos ( 'mshsrfcopy' )

      end
