      subroutine mshsrfholes ( nholes, kmeshs, kmeshscp, holeinfo, &
                               lenkmeshscp, ncurvs, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 04-02-2015 Correction in case of holes
!        version  1.1  date 04-07-2006 Debug statements
!        version  1.0  date 05-04-2002
!
!   copyright (c) 2002-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reverse the direction of boundaries in kmeshscp in case there are holes
!     and the orientation of the boundary part is different from the first one
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     direction
!     outer_boundary
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nholes, kmeshs(*), kmeshscp(*), holeinfo(2,0:nholes+1), &
              lenkmeshscp, ncurvs, isurnr

!     holeinfo       i    Integer array of size 2 x (nholes+2)
!                         Contains information about holes in surfaces and where
!                         a new part of the boundary starts
!                         Pos (1,i-1) contains the local sequence number of the
!                            first curve of part i provided with a sign
!                            If the sign is positive the boundary is
!                            created counter clockwise, otherwise clockwise
!                         Pos (2,i-1) contains the local sequence number of the
!                            first node of of part i
!                         Pos (1,nholes+1) contains the number of curves of the
!                         boundary
!     isurnr         i    Surface sequence number
!     kmeshs         i    Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     kmeshscp      i/o   Copy of kemshs part 3
!                         The curves may be reversed so that all boundary
!                         parts have curves pointing in the same direction
!     lenkmeshscp    i    Length of kmeshs parts 1 + 2 or 3
!     ncurvs         i    Number of curves in mesh
!     nholes         i    Number of holes in the boundary of a surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inext, iprev, istart, istore(ncurvs), isign, j, ishift

!     i              Counting variable
!     inext          Next curve sequence number
!     iprev          Previous curve sequence number
!     ishift         Shift
!     isign          Direction of curves in first part of the boundary
!     istart         First position used at a part
!     istore         Help array to be used for copying
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsrfholes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsrfholes'
         write(irefwr,1) 'nholes, isign', nholes, holeinfo(1,0)
         write(irefwr,1) 'isurnr', isurnr
         call prinin1 ( holeinfo(1,0), nholes+2, 2, 'holeinfo' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      isign = holeinfo(1,0)
      ishift = 0
      if ( isign==-1 ) ishift = 1

      if ( debug ) write(irefwr,1) 'isign, ishift', isign, ishift

      do i = 1, nholes

         if ( debug ) &
            write(irefwr,1) 'i, isign, holeinfo(1,i)', &
                             i, isign, holeinfo(1,i)

         if ( isign*holeinfo(1,i)<0 ) then

!        --- The curves iprev to -holeinfo(1,i)-1 in kmeshscp must
!            get the reversed sign and stored in reversed sequence

            iprev = abs(holeinfo(1,i-ishift))
            inext = abs(holeinfo(1,i+1-ishift))-1
            istart = kmeshs(isurnr)-lenkmeshscp

            if ( debug ) then
               write(irefwr,1) 'i, iprev, inext', i, iprev, inext
               call prinin ( kmeshscp(istart+iprev), inext-iprev+1, &
                             'kmeshscp')
            end if  ! ( debug )

            do j = iprev, inext
               istore(inext+1-j) = -kmeshscp(istart+j)
            end do
            do j = iprev, inext
               kmeshscp(istart+j) = istore(j+1-iprev)
            end do
            if ( debug ) &
               call prinin ( kmeshscp(istart+iprev), inext-iprev+1, &
                             'kmeshscp')

         end if
         iprev = abs(holeinfo(1,i))

      end do

1000  call erclos ( 'mshsrfholes' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsrfholes'

      end if  ! ( debug )

      end
