      subroutine mshtransinfo ( blocknr, pointmap, nprocessors, &
                                neighblocks )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.3  date 12-06-2015 New call to mshtransinfo1
!        version  3.2  date 15-11-2012 Long integers
!        version  3.1  date 29-04-2004 Extra parameter npoint
!
!   copyright (c) 2001-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays blockinfolc and transinfolc for one block in the parallel
!     environment
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     neighbor
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer blocknr, pointmap(*), nprocessors, neighblocks(*)

!     blocknr        i    Sequence number of present block
!     neighblocks    i    Array containing of each block all neighbor blocks
!                         in the following way:
!                         Pos. 1 to nprocesors starting addresses of each block
!                         For each block from start position:
!                         Number of neighbor blocks, followed by
!                         neighbor block sequence numbers
!     nprocessors    i    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
!     pointmap       i    Array of size npoint containing the local node numbers
!                         of the nodes in the global mesh
!                         if node i is not present in local mesh pointmap(i) = 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iblock, istartinfo, nneigh, npointloc, numneigh, &
              blockinfolc(2*nprocessors**2), npointloc1, &
              i, lentransinfolc, mmkmeshe, ineigh, &
              istart, j, k, markblocks(nprocessors), &
              istartblock, istartneigh, m_i, n_ij, lenblockinfolc
      integer, allocatable :: transportinfo(:)
      integer, allocatable :: iwork(:), neighbors(:), istore(:)
      integer, pointer :: points(:), points1(:)

!     blockinfolc      Array with global block information for surface
!                    number isurnr
!                    See Programmers Guide Section 24.2 for a description
!     i              Counting variable
!     iblock         Block sequence number
!     ineigh         neighbor of iblock
!     istart         First position used at a part
!     istartblock    Starting address in array blockinfolc
!     istartinfo     Starting address to array startinfo
!     istartneigh    Starting address of ineigh in array neighblocks
!     istore         Work array of length npoint to store the
!                    common neighbor nodal points of two neighbor blocks
!                    of blocknr
!     iwork          integer work array of length npoint that is filled
!                    in the following way:
!                    iwork(i) = -1:  point i is in block blocknr
!                    iwork(i) =  0:  point i is not a neighbor of blocknr
!                    iwork(i) =  1:  point i is a neighbor of blocknr
!     j              Counting variable
!     k              Counting variable
!     lenblockinfolc   Length of array blockinfolc
!     lentransinfolc   Length of array transinfolc
!     m_i            The number of neighbor blocks of block i that are also
!                    neighbors of the present block
!     markblocks     Indicates if a block is a neighbor of blocknr (>0) or
!                    not (0)
!     mmkmeshe       Memory management address of kmesh part e
!     n_ij           nodes in the interface of iblock and ineigh that are also
!                    neighbor nodes of blocknr
!     neighbors      Array of length npoint that marks all points in
!                    blocknr with a 1 and all other points with a 0
!     nneigh         Number of neighbors of node
!     npointloc      Number of points in local mesh
!     npointloc1     Number of points in neighbor mesh
!     numneigh       number of neighbors of a point
!     points         Array of size npointloc containing the node
!                    numbers corresponding to neighbor mesh
!     points1        Array of size npointloc containing the node
!                    numbers corresponding to mesh blocknr
!     transportinfo  Work array of size 4*npoint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHPARINV      Compute the inverse mapping
!                    (i.e. mapping from global to local mesh)
!     MSHTRANSINFO1  Fill array transportinfo for one specific neighbor block
!                    of one block
!     MSHTRANSINFO2  Compute all neigbour points of block blocknr and store
!                    in array iwork
!     MSHTRANSINFO3  Compute number of nodes n_ij on common boundary of ineigh
!                    and iblock with iwork(i) > 0 and store the nodes in array
!                    istore
!     PRINKMESHPAR   Print the contents of the parallel arrays blockinfo and
!                    transportinfo
!     SEPFILLKMESHE  Fill array kmesh part e
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2496   No neighbor blocks of block
! **********************************************************************
!
!                       PSEUDO CODE
!
!     numneigh := 0; istartinfo := 1
!     For all neighbor blocks iblock of blocknr do
!        Activate array points
!        fill transportinfo for block and raise istartinfo
!        fill array blockinfolc
!     Second part:
!     For all neighbor blocks iblock of blocknr do
!        For all common neighbors ineigh of iblock and blocknr do
!           Compute all neigbour points of block ineigh and store in array iwork
!           in the following way:
!              iwork(i) = -1:  point is in block ineigh
!              iwork(i) =  0:  point is not a neighbor of ineigh
!              iwork(i) =  1:  point is a neighbor of ineigh
!           Compute number of nodes n_ij on common boundary of iblock and
!           blocknr with iwork(i)>0
!           Store information in transportino and blockinfolc
!           Raise istartinfo
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshtransinfo' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshtransinfo'
  1      format ( a, 1x, (10i10) )

      end if

      allocate ( iwork(npoint), transportinfo(4*npoint), &
                 neighbors(npoint), istore(npoint),  stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

      lenblockinfolc = 2*nprocessors**2
      lentransinfolc = 4*npoint

!     --- numneigh := 0; istartinfo := 1

      numneigh = 0
      istartinfo = 1
      istartneigh = neighblocks(blocknr)
      nneigh = neighblocks(istartneigh)
      if ( nneigh==0 ) then

!     --- Block has no neighbor, not yet tested

         call errint ( blocknr, 1 )
         call errsub ( 2496, 1, 0, 0 )
         go to 1000

      end if
      markblocks = 0
      if ( 2*nneigh+1>lenblockinfolc ) then

!     --- Array blockinfolc too small

         call errint ( 2*nneigh+1, 1  )
         call errint ( lenblockinfolc, 2  )
         call errchr ( 'blockinfolc', 1  )
         call errsub ( 550, 2, 0, 1  )
         go to 1000

      end if

!     --- Clear array neighbors (1:npoint)
!         Set neighbors(i) = i for all points i in blocknr

      points1 => kmhpar(blocknr)%nodepointers
      nnodes = size (points1)  ! number of nodes in points
      if ( ierror/=0 ) go to 1000
      call mshparinv ( points1, nnodes, neighbors, npoint )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'blocknr, nnodes', blocknr, nnodes

!     --- For all neighbor blocks iblock of blocknr do

      do i = 1, nneigh

         iblock = neighblocks(istartneigh+i)
         markblocks(iblock) = i

!        --- iblock # blocknr

         points => kmhpar(iblock)%nodepointers
         npointloc = size ( points )
         npointloc1 = size ( points1 )
         if ( ierror/=0 ) go to 1000

         if ( debug ) &
            write(irefwr,1) 'i, iblock, npointloc, npointloc1', &
                             i, iblock, npointloc, npointloc1

!        --- fill transportinfo for block and raise istartinfo
!            fill array blockinfolc if possible

         call mshtransinfo1 ( pointmap, points, nnodes, &
                              transportinfo(istartinfo+1), npointloc, &
                              npoint, points1, npointloc1, &
                              iwork )
         if ( ierror/=0 ) go to 1000

         blockinfolc(2*i) = iblock
         blockinfolc(2*i+1) = istartinfo
         transportinfo(istartinfo) = nnodes
         istartinfo = istartinfo+nnodes+1
         if ( istartinfo>lentransinfolc ) then

!        --- Array istartinfo too small

            call errint ( istartinfo, 1  )
            call errint ( lentransinfolc, 2  )
            call errchr ( 'transportinfo', 1  )
            call errsub ( 550, 2, 0, 1  )
            go to 1000

         end if

      end do
      blockinfolc(1) = nneigh
      istartblock = 2*nneigh+2
      if ( nprocessors>2 ) then

!     --- More than 2 processors fill rest of information

         if ( kelme==0 ) call sepfillkmeshe
         if ( ierror/=0 ) go to 1000

         istartblock = 3*nneigh+2

!        --- For all neighbor blocks iblock of blocknr do

         do i = 1, nneigh

            iblock = neighblocks(istartneigh+i)
            blockinfolc(2*nneigh+i+1) = istartblock
            m_i = 0

!           --- For all common neighbors ineigh of iblock and blocknr do

            istart = neighblocks(iblock)
            numneigh = neighblocks(istart)
            do j = 1, numneigh

               ineigh = neighblocks(istart+j)
               if ( markblocks(ineigh)>0 ) then

!              --- Compute all neigbour points of block ineigh and store in
!                  array iwork in the following way:
!                     iwork(i) = -1:  point is in block ineigh
!                     iwork(i) =  0:  point is not a neighbor of ineigh
!                     iwork(i) =  1:  point is a neighbor of ineigh
!                     Compute starting addresses

                  points => kmhpar(ineigh)%nodepointers
                  npointloc = size ( points )
                  call mshtransinfo2 ( iwork, npoint, kmshe1, kmshe2, &
                                       npointloc, points )
                  if ( ierror/=0 ) go to 1000

!                 --- Compute number of nodes n_ij on common boundary of iblock
!                     and blocknr with iwork(i)>0

                  points => kmhpar(iblock)%nodepointers
                  nnodes = size ( points )
                  if ( ierror/=0 ) go to 1000

!                 --- Actual computation

                  call mshtransinfo3 ( iwork, neighbors, nnodes, &
                                       points, istore, n_ij, npoint )
                  if ( ierror/=0 ) go to 1000

!                 --- Store information in transportinfo and blockinfolc
!                     Raise istartinfo

                  if ( n_ij>0 ) then

!                 --- n_ij>0, there are specific neighbors

                     m_i = m_i+1
                     blockinfolc(istartblock+2*m_i-1) = ineigh
                     blockinfolc(istartblock+2*m_i) = istartinfo
                     transportinfo(istartinfo) = n_ij
                     do k = 1, n_ij
                        transportinfo(istartinfo+k) = istore(k)
                     end do  ! k = 1, n_ij
                     istartinfo = istartinfo+n_ij+1
                     if ( istartinfo>lentransinfolc ) then

!                    --- Array istartinfo too small

                        call errint ( istartinfo, 1  )
                        call errint ( lentransinfolc, 2  )
                        call errchr ( 'transportinfo', 1  )
                        call errsub ( 550, 2, 0, 1  )
                        go to 1000

                     end if

                  end if  ! ( n_ij>0 )

               end if  ! ( markblocks(ineigh)>0 )

            end do  !  j = 1, numneigh

            blockinfolc(istartblock) = m_i
            istartblock = istartblock+1+2*m_i
            if ( istartblock>lenblockinfolc ) then

!           --- Array blockinfolc too small

               call errint ( istartblock, 1  )
               call errint ( lenblockinfolc, 2  )
               call errchr ( 'blockinfolc', 1  )
               call errsub ( 550, 2, 0, 1  )
               go to 1000

            end if

         end do  ! i = 1, nneigh

      else

!     --- Only 2 blocks, there is no extra information

         blockinfolc(istartblock) = 0
         blockinfolc(istartblock+1) = 0
         istartblock = istartblock+2

      end if

!     --- Copy arrays blockinfolc and istartinfo to permanent positions

      length = istartinfo-1
      km(iseqkm)%kmesh(53) = length
      if ( associated(km(iseqkm)%transinfo) ) &
         deallocate(km(iseqkm)%transinfo)
      allocate ( km(iseqkm)%transinfo(length) )
      km(iseqkm)%transinfo = transportinfo(1:length)

      length = istartblock-1
      km(iseqkm)%kmesh(54) = length
      if ( associated(km(iseqkm)%blockinfo) ) &
         deallocate(km(iseqkm)%blockinfo)
      allocate ( km(iseqkm)%blockinfo(length) )
      km(iseqkm)%blockinfo = blockinfolc(1:length)

      if ( debug ) then

!     --- Debug information

         parallel = .true.
         inodenr = blocknr
         call prinkmeshpar ( km(iseqkm)%blockinfo, km(iseqkm)%transinfo )
         parallel = .false.

      end if

      deallocate ( iwork, transportinfo, neighbors, istore, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) write(irefwr,*) 'End mshtransinfo'
      call erclos ( 'mshtransinfo' )

      end subroutine mshtransinfo
