      subroutine mshtransinfo1 ( pointmap, points1, nneigh, &
                                 transportinfo, npointloc1, npoint, &
                                 points, npointloc, pointmap1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 12-06-2015 Change of parameter list
!        version  1.0  date 07-01-2003
!
!   copyright (c) 2003-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array transportinfo for one specific neighbor block of one block
!
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     neighbor
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: npoint, pointmap(npoint), npointloc, &
                             points(npointloc), npointloc1, &
                             points1(npointloc1)
      integer, intent(out) :: nneigh, transportinfo(*)
      integer, intent(inout) :: pointmap1(npoint)

!     nneigh         o    Number of neighbors stored in transportinfo
!     npoint         i    Number of nodal points
!     npointloc      i    Number of points in local mesh
!     npointloc1     i    Number of points in neighbor mesh
!     pointmap       i    Array of size npoint containing the local node numbers
!                         of the nodes in the global mesh
!                         if node i is not present in local mesh pointmap(i) = 0
!     pointmap1     i/o   Same as pointmap, but now for neighbor mesh
!     points         i    Array of size npointloc containing the node
!                         numbers corresponding to the local present mesh
!     points1        i    Array of size npointloc containing the node
!                         numbers corresponding to the local neighbor mesh
!     transportinfo  o    Array containing information of the transport of
!                         unknowns between neighboring blocks
!                         The starting address of each neighbor block is
!                         stored in array blockinfo
!                         For each neighbor block the following information
!                         is stored
!                         Pos. 1 number of nodes n on common side of both blocks
!                         Pos. 2 ... n+1    local node numbers in block i
!                         Pos. n+2 ... 2n+1 local node numbers in neighbor
!                                           block j
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iwork(npoint)

!     i              Counting variable
!     iwork          integer work array of length npoint
!                    iwork is used to store node numbers of neigbouring blocks
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPARINV      Compute the inverse mapping
!                    (i.e. mapping from global to local mesh)
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nneigh := 0
!     For all points in neighbor block do
!        if ( point also in pointmap ) then
!           raise nneigh
!           store node number w.r.t. block blocknr into transportinfo
!           store node number w.r.t. neighbor block in iwork
!     if ( nneigh>0 ) then
!        Copy iwork into transportinfo
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshtransinfo1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshtransinfo1'
         write(irefwr,1) 'npointloc, npointloc1', npointloc, npointloc1
         call prinin ( points, npointloc, 'points' )
         call prinin ( points1, npointloc1, 'points1' )
         call prinin ( pointmap, npoint, 'pointmap' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array pointmap1 with inverse of points1

      call mshparinv ( points1, npointloc1, pointmap1, npoint )

      if ( debug ) call prinin ( pointmap1, npoint, 'pointmap1' )

!     --- nneigh := 0

      nneigh = 0

!     --- For all points in mesh do

      do i = 1, npoint

         if ( pointmap(i)>0 .and. pointmap1(i)>0 ) then

!        --- point in both blocks
!            raise nneigh
!            store node number w.r.t. block blocknr into transinfo
!            store node number w.r.t. neighbor block in iwork

            nneigh = nneigh+1
            transportinfo(nneigh) = pointmap(i)
            iwork(nneigh) = pointmap1(i)

         end if

      end do

!     --- Copy iwork into transportinfo

      do i = 1, nneigh
         transportinfo(nneigh+i) = iwork(i)
      end do

1000  call erclos ( 'mshtransinfo1' )
      if ( debug ) then

!     --- Debug information

         call prinin ( transportinfo, 2*nneigh, 'transportinfo' )
         write(irefwr,*) 'End mshtransinfo1'

      end if  ! ( debug )

      end subroutine mshtransinfo1
