      subroutine readiinput ( iinput, rinput, lniinp, lnrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 30-07-2012 Extra parameters lniinp/lnrinp
!        version  1.0  date 17-01-2003
!
!   copyright (c) 2003-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read arrays iinput and rinput from an input file
!     Store the result in the arrays iinput and rinput
!     The input file must have the special structure as described in the i/o
!     part
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), lniinp, lnrinp
      double precision rinput(*)

!     iinput        i/o   integer input array
!                         At input the declared length must be stored
!                         in position 1 and pos. 3 must be 0
!                         At output the array read has ben stored from pos. 6
!     lniinp         i    Available space for array IINPUT
!     lnrinp         i    Available space for array RINPUT
!     rinput        i/o   Real input array
!                         At input the declared length must be stored
!                         in position 1 and pos. 3 must be 0
!                         At output the array read has ben stored from pos. 6
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer niinp, inum, nrinp

!     inum           Number of numbers read in a record
!     niinp          Length of integer input array
!     nrinp          Length of real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     READSR         Read a series of numbers from the input (texts are
!                    neglected)
!     READSRIN       Read a series of integer numbers from the input
! **********************************************************************
!
!                       I/O
!
!     The arrays iinput and rinput are read from the standard input file
!     The organization of the ASCII input file is as follows:
!     record 1:  # Array iinput      (followed by some text)
!     record 2:  niinput: Number of integers stored in iinput
!     Next records:  niinput integers which form iinput
!     Next record:    # Array rinput      (followed by some text)
!     Next record:    nrinput: Number of reals stored in rinput
!     Next records:   nrinput reals which form rinput
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1378   Parameter has wrong value
!    2395   Next item read is not an integer
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readiinput' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readiinput'

      end if
      if ( ierror/=0 ) go to 1000

!     --- First read tag line

      ityprd = 1
      call asknxt

!     --- Read first line

      if ( ityprd/=2 ) then

!     --- Not an integer found

         call errsub ( 2395, 0, 0, 0 )
         call errcrd

      end if
      niinp = nint(realrd)
      if ( niinp<=0 .or. niinp>10000 ) then

!     --- niinp out of range

         call errint ( niinp, 1 )
         call errint ( 1, 2 )
         call errint ( 10000, 3 )
         call errchr ( 'niinput', 1 )
         call errsub ( 1378, 3, 0, 1 )
         call errcrd
         go to 1000

      end if
      call inichkarlen ( niinp, lniinp, 'iinput' )
      if ( ierror/=0 ) go to 1000

!     --- Read numbers into iinput

      istop = 2
      call readsrin ( iinput, niinp, inum, .false., 1000000, &
                      'iinput' )
      istop = 0

!     --- Read first line of rinput

      ityprd = 1
      call asknxt
      if ( ityprd/=2 ) then

!     --- Not an integer found

         call errsub ( 2395, 0, 0, 0 )
         call errcrd

      end if
      nrinp = nint(realrd)
      if ( nrinp<=0 .or. nrinp>10000 ) then

!     --- niinp out of range

         call errint ( nrinp, 1 )
         call errint ( 1, 2 )
         call errint ( 10000, 3 )
         call errchr ( 'nrinput', 1 )
         call errsub ( 1378, 3, 0, 1 )
         call errcrd
         go to 1000

      end if
      call inichkarlen ( nrinp, lnrinp, 'rinput' )

!     --- Read numbers into rinput

      istop = 2
      call readsr ( rinput, nrinp, inum )
      istop = 0

1000  call erclos ( 'readiinput' )
      if ( debug ) write(irefwr,*) 'End readiinput'

      end
