      subroutine wriinp2msh
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 30-07-2012 iinput and rinput from position 6
!        version  2.0  date 14-09-2010 Remove work arrays
!        version  1.0  date 17-01-2003
!
!   copyright (c) 2003-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Translate an input file with the arrays iinput and rinput into
!     a sepran mesh input file and write to the standard output file
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     conversion
!     file
!     input
!     output
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lniinp, lnrinp
      integer, allocatable :: iinput(:)
      double precision, allocatable :: rinput(:)

!     iinput         integer input array
!     lniinp         Available space for array IINPUT
!     lnrinp         Available space for array RINPUT
!     rinput         Real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIDBG         read file sepran.dbg and set debug parameters
!     INIFIL         Open files
!     INITCB         Initializes SEPRAN machine-independent common blocks
!     INITMDBF       Set machine-dependent quantities
!     READIINPUT     Read arrays iinput and rinput from an input file
!     SEPSTART       Standard SEPRAN starting subroutine
!     STARTSEPRAN    Create buffer array and fill part of sepmodulebuf
!     WRITMSHI       Write arrays iinput and rinput in original format to
!                    a file
! **********************************************************************
!
!                       I/O
!
!     The arrays iinput and rinput are read from the standard input file
!     The organization of the ASCII input file is as follows:
!     record 1:  # Array iinput      (followed by some text)
!     record 2:  niinput: Number of integers stored in iinput
!     Next records:  niinput integers which form iinput
!     Next record:    # Array rinput      (followed by some text)
!     Next record:    nrinput: Number of reals stored in rinput
!     Next records:   nrinput reals which form rinput
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize common blocks
!     Open read and write files
!     Open error message file
!     Read debug file if it exists
!     Start the SEPRAN program
!     read input from standard input file
!     Translate to mesh format and write to file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'wriinp2msh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from wriinp2msh'

      end if
      if ( startcalled/=99991 ) call startsepran

!     --- Initialize common blocks

      call initcb
      mainsubr = 1
      call initmd

!     --- Open read and write files

      call inifil(5)
      call inifil(6)

!     --- Open error message file

      call inifil(4)

!     --- Standard SEPRAN start subroutine

      call sepstart ( 0, 0, -1, 0 )
      call eropen ( 'wriinp2msh' )

!     --- Reserve temporary space to store  iinput and rinput
!         Maximum length same as in main program (500000 integers)

      lniinp = 200000
      lnrinp = 100000

      allocate ( iinput(lniinp), rinput(lnrinp), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, lniinp+lnrinp, 'iinput' )
      if ( ierror/=0 ) go to 1000

!     --- read input from standard input file
!         Store arrays IINPUT and RINPUT in IBUFFR

      call readiinput ( iinput, rinput, lniinp, lnrinp )
      if ( ierror/=0 ) go to 1000

!     --- Translate to mesh format and write to file

      call writmshi ( iinput, rinput, irefwr )

      deallocate ( iinput, rinput, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iinput' )

1000  if ( debug ) write(irefwr,*) 'End wriinp2msh'
      call erclos ( 'wriinp2msh' )

      end subroutine wriinp2msh
