      subroutine msh043case4 ( iinput, nc, ivolnr, nx, ny, nz, npointlc, &
                               nelemlc, nbcrd, nbndpt, lenwrk, inpelmbot, &
                               inpelmsur )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 22-05-2014 Extension with shape 22
!        version  3.0  date 24-08-2010 Removal of lnwrk1
!        version  2.1  date 20-10-2008 Allow isurf = 24
!
!   copyright (c) 2005-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     initialize some parameters for mshvlm
!     Case 4: subroutine pipe or channel
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     pipe
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), nc, &
              ivolnr, nx, ny, nz, npointlc, nelemlc, &
              nbcrd, nbndpt, lenwrk, inpelmbot, inpelmsur

!     iinput         i    integer input array containing the information
!                         of volumes
!     inpelmbot      i    Number of nodes per element of bottom surface and
!                         top surface of pipe
!     inpelmsur      i    Number of nodes per element of pipe surface
!     ivolnr         i    Volume sequence number
!     lenwrk         o    Length of array iwork
!     nbcrd          o    number of nodal points in boundary of submesh
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     nc             o    number of points along the curves
!     nelemlc        o    Number of elements
!     npointlc       o    Number of nodal points
!     nx             o    number of points in x direction
!     ny             o    number of points in y direction
!     nz             o    number of points in z direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurnr, ns, isur, istart, nsubsf, ic, icurnr, &
              npoloc, nelloc, isubsf, isurf, jcurnr, lenext, &
              nbnd1, nbnd2, nel, nelz, np, npobot, nsurbt

!     ic             Relative subcurve number
!     icurnr         Curve sequence number
!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
!     isur           surface number
!     isurf          Type of surface with respect to mesh generator
!     isurnr         Surface sequence number
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     lenext         Extra length required for IWORK in case of surfaces of
!                    surfaces
!     nbnd1          number of nodes of boundary of surface
!     nbnd2          number of nodes of boundary of surface
!     nel            number of elements in the topology
!     nelloc         Local number of elements
!     nelz           Number of line elements in vertical direction
!     np
!     npobot         Number of points of bottom surface
!     npoloc         Number of nodes in subsurface
!     ns             Starting address of information of specific surface
!                    in array KMESHS
!     nsubsf         Number of subsurfaces
!     nsurbt         Number of elements in boundary of bottom surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     596   The top and bottom surfaces S1 and S2 of the pipe are not congruent
!     641   Third generating surface of PIPE is not of the type PIPE SURFACE
!     642   top or bottom surface of PIPE is of type PIPE SURFACE
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute the number of elements in bottom and top surfaces (ny), &
!     the number of points in these surfaces (nx), and
!     the number of elements in vertical direction (nz)
!     The generating surfaces are used for this information
!     The estimated length of array iwork is stored in leniwk
!     First surface  (bottom)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh043case4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh043case4'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      isurnr = iinput(4)
      nx = abs(kmeshh(1,isurnr+1))
      lenext = 0
      ny = kmeshh(2,isurnr+1)

      if ( debug ) then

!     --- Check = true, print some information

         write(irefwr,1) 'isurnr, nx, ny', isurnr, nx, ny

      end if

      if ( kelmr==0 ) then

!     --- kelmr = 0   No subsurfaces

         nbnd1 = kmeshh(4,isurnr+1)+1

!        --- Correct due to new definition of kbndpt

         nbnd1 = nbnd1-kmhsur(isurnr)%kbndptsur(nbnd1)-1

      else

!     --- kelmr>0, there are surfaces of subsurfaces
!         Find boundaries from kmeshs

         istart = kmeshr(isurnr)
         nsubsf = kmeshr(isurnr+1)-kmeshr(isurnr)
         do isubsf = 1, nsubsf
            isur = abs( kmeshr(istart+isubsf) )
            npoloc = kmeshh(1,isur+1)
            nelloc = kmeshh(2,isur+1)
            lenext = max ( lenext, nelloc+npoloc+nelloc*inpelmbot )
         end do

         istart = kmeshs(isurnr)
         ns = nsurfs+2+(isurnr-1)*8
         isurf = kmeshs(ns)
         if ( isurf==24 ) isurf = 7    ! compound surface

         if ( isurf==7 .or. isurf==11 ) then

!        --- isurf = 7 or 11  surfaces of surfaces

            nc = kmeshs(ns+2)

         else if ( isurf==10 .or. isurf==12 ) then

!        --- isurf = 10 or 12  pipe surface

            call errint ( ivolnr, 1 )
            call errsub ( 642, 1, 0, 0)

         else

           nc = kmeshs(isurnr+1) - istart

         end if
         nbnd1 = 0
         do ic = 1, nc
            icurnr = kmeshs(istart+ic)
            jcurnr = abs ( icurnr )
            nbnd1 = nbnd1 + kmeshm(5+jcurnr) - kmeshm(4+jcurnr)-1
         end do
         nbnd1 = nbnd1 + 1

      end if

!     --- Second surface  (top)

      isurnr = iinput(5)
      np = abs(kmeshh(1,isurnr+1))
      nel = kmeshh(2,isurnr+1)
      if ( debug ) &
         write(irefwr,1) 'top surface, np, nel', isurnr, np, nel
      if ( kelmr==0 ) then

!     --- kelmr = 0   No subsurfaces

         nbnd2 = kmeshh(4,isurnr+1)+1

!        --- Correct due to new definition of kbndpt

         nbnd2 = nbnd2-kmhsur(isurnr)%kbndptsur(nbnd2)-1

      else

!     --- kelmr>0, there are surfaces of subsurfaces
!         Find boundaries from kmeshs

         istart = kmeshs(isurnr)
         ns = nsurfs+2+(isurnr-1)*8
         isurf = kmeshs(ns)
         if ( isurf==7 .or. isurf==11 ) then

!        --- isurf = 7 or 11  surfaces of surfaces

            nc = kmeshs(ns+2)

         else if ( isurf==10 .or. isurf==12 ) then

!        --- isurf = 10 or 12  pipe surface

            call errint ( ivolnr, 1 )
            call errsub ( 642, 1, 0, 0)
         else

            nc = kmeshs(isurnr+1) - istart

         end if
         nbnd2 = 0
         do ic = 1, nc
            icurnr = kmeshs(istart+ic)
            jcurnr = abs ( icurnr )
            nbnd2 = nbnd2 + kmeshm(5+jcurnr) - kmeshm(4+jcurnr)-1
         end do
         nbnd2 = nbnd2 + 1

      end if

      if ( np/=nx .or. nel/=ny .or. nbnd1/=nbnd2 ) then
         call errint ( np, 1 )
         call errint ( nx, 2 )
         call errint ( nel, 3 )
         call errint ( ny, 4 )
         call errint ( nbnd1, 5 )
         call errint ( nbnd2, 6 )
         call errint ( ivolnr, 7 )
         call errsub ( 596, 7, 0, 0)
      end if

!     --- Pipe surface

      isurnr = iinput(6)
      np = abs(kmeshh(1,isurnr+1))
      nel = kmeshh(2,isurnr+1)

      ns = nsurfs + 2 + (isurnr-1)*8
      isurf = kmeshs(ns)
      if ( isurf/=10 .and. isurf/=12 ) then
         call errint ( isurnr, 1 )
         call errint ( ivolnr, 2 )
         call errsub ( 641, 2, 0, 0)
         go to 1000
      end if

      if ( debug ) then

!     --- Check = true, print some information

         write(irefwr,1) 'isurnr, np, nel, isurf', &
                          isurnr, np, nel, isurf

      end if

      if ( isurf==10 ) then

!     --- isurf = 10   No subsurfaces

         nbnd2 = kmeshh(4,isurnr+1)+1

!        --- Correct due to new definition of kbndpt

         nbnd2 = nbnd2-kmhsur(isurnr)%kbndptsur(nbnd2)-1

         nbnd2 = (nbnd2-2*nbnd1)/2

      else

!     --- isurf = 12  surface of pipesurfaces
!         Find boundaries from kmeshs

         istart = kmeshr(isurnr)
         nsubsf = kmeshr(isurnr+1)-kmeshr(isurnr)
         do isubsf = 1, nsubsf
            isur = abs( kmeshr(istart+isubsf) )
            npoloc = kmeshh(1,isur+1)
            nelloc = kmeshh(2,isur+1)
            lenext = max ( lenext, nelloc+npoloc+nelloc*inpelmsur )
         end do

         istart = kmeshs(isurnr)+kmeshs(ns+2)+kmeshs(ns+3)
         nc = kmeshs(ns+4)
         nbnd2 = 0
         do ic = 1, nc
            icurnr = kmeshs(istart+ic)
            jcurnr = abs ( icurnr )
            nbnd2 = nbnd2 + kmeshm(5+jcurnr) - kmeshm(4+jcurnr)-1
         end do
         nbnd2 = nbnd2 + 1
      end if
      nbcrd = np+2*nx

      if ( ishape==11 .or. ishape==13 .or. ishape>=17 .and. &
           ishape<=21 ) then

!     --- linear elements (shape 11, 13, 17-21)

         nz = nbnd2-1
         nc = nbnd1-1

      else

!     --- quadratic elements (shape 12, 14, 15, 22)

         nz = (nbnd2-1)/2
         nc = (nbnd1-1)/2

      end if

      if ( debug ) then

!     --- Check = true, print some information

         write(irefwr,1) 'nbnd1, nbnd2, nx, nbcrd, nz, nc', &
                          nbnd1, nbnd2, nx, nbcrd, nz, nc

      end if

      npointlc = max(np,nbnd2*nx)
      nbndpt = (2*ny+nel)*inpelmsur
      nelemlc = ny*nz
      if ( ishape/=13 .and. ishape/=14 .and. ishape/=17 .and. &
           ishape<20 ) then

!     --- tetrahedrons

         npointlc = npointlc+ny*nz
         if ( ishape==12 ) npointlc = npointlc+6*ny*nz
         nelemlc = nelemlc*8

      end if

      lenwrk = nx * 10

      if ( ishape==15 .or. ishape==16 ) then

!     --- ishape = 15 or 16, extra work space needed (see mshpipe)

         npobot = nx
         nsurbt = nc
         nelz = nz
         lenwrk = max(lenwrk, &
                 2*npointlc+6*npobot+3*(nsurbt+1)*(nelz+1))
         npointlc = npointlc+4*nelemlc

      end if  ! ( ishape==15 .or. ishape==16 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'npointlc', npointlc
         write(irefwr,*) 'End msh043case4'

      end if
      call erclos ( 'msh043case4' )

      end
