      subroutine msh075surfs ( isurfs, iinput, iinp, inodp, lengc, iselem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-03-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     checks the input for interface elements. It computes the
!     required length of these elements with respect to KMESH part c
!     Furthermore inodp is raised if necessary
!     Surfaces only
! **********************************************************************
!
!                       KEYWORDS
!
!     interface_element
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurfs, iinput(*), iinp, inodp, lengc, iselem

!     iinp           i    last position used in array iinput
!     iinput         i    integer input array containing information of the
!                         interface elements
!     inodp         i/o   Next nodal point number
!     iselem        i/o   Next free element number
!     isurfs         i    Number of parts where surface interface elements
!                         are given
!     lengc         i/o   computed length of kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielgrp, isurnr1, isurnr2, istart1, &
              istart2, j, inpelm, mmcoor, npoloc

!     i              Counting variable
!     ielgrp         element group number
!     inpelm         number of points per element
!     istart1        First position used in kmeshn for surface 1
!     istart2        First position used in kmeshn for surface 2
!     isurnr1        Surface sequence number
!     isurnr2        Surface sequence number
!     j              Counting variable
!     mmcoor         Memory management sequence number with respect to
!                    local array coor
!     npoloc         Number of nodes in surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH075SURFS1   Fill kmeshn with new node numbers in case of surface
!                    interface elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1002   Shape number out of range
!    2821   first surface number out of range in interface elements
!    2822   second surface number out of range in interface elements
!    2823   Both surfaces have the same sequence number
!    2824   Both surfaces do not have the same number of elements
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh075surfs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh075surfs'
         write(irefwr,1) 'isurfs', isurfs
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Get constants

      if ( debug ) write(irefwr,1) 'nsurfs', nsurfs

!     --- Loop over isurfs parts

      do i = 1, isurfs

         ielgrp = iinput(iinp)        ! element group
         ishape = iinput(iinp+1)      ! shape number
         if ( ishape<=70 .or. ishape>79 ) then

!        --- error 1002: ishape out of range

            call errint ( ishape, 1 )
            call errint ( 70, 2 )
            call errint ( 79, 3 )
            call errsub ( 1002, 3, 0, 0 )
            go to 1000

         end if
         isurnr1 = iinput(iinp+2)    ! first surface number
         isurnr2 = iinput(iinp+3)    ! second surface number
         if ( debug ) &
            write(irefwr,1) 'i, ielgrp, ishape, isurnr1, isurnr2', &
                             i, ielgrp, ishape, isurnr1, isurnr2
         if ( isurnr1<1 .or. isurnr1>nsurfs ) then

!        --- surface number out of range

            call errint ( isurnr1, 1 )
            call errint ( i, 2 )
            call errint ( nsurfs, 3 )
            call errint ( ielgrp, 4 )
            call errsub ( 2821, 4, 0, 0 )
            go to 1000

         end if  ! ( isurnr1<1 .or. isurnr1>nsurfs )
         if ( isurnr2<1 .or. isurnr2>nsurfs ) then

!        --- surface number out of range

            call errint ( isurnr2, 1 )
            call errint ( i, 2 )
            call errint ( nsurfs, 3 )
            call errint ( ielgrp, 4 )
            call errsub ( 2822, 4, 0, 0 )
            go to 1000

         end if  ! ( isurnr1<1 .or. isurnr1>nsurfs )
         if ( isurnr1==isurnr2 ) then

!        --- Surface numbers are the same

            call errint ( i, 1 )
            call errint ( isurnr2, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 2823, 3, 0, 0 )

         end if  ! ( isurnr1==isurnr2 )

!        --- Compute starting addresses of surfaces

         istart1 = 2*nsurfs          ! starting address surface 1
         istart2 = istart1           ! starting address surface 2
         do j = 1, isurnr1-1
            istart1 = istart1+kmeshn(j)*kmeshn(j+nsurfs)
         end do  ! j = 1, isurnr1-1
         do j = 1, isurnr2-1
            istart2 = istart2+kmeshn(j)*kmeshn(j+nsurfs)
         end do  ! j = 1, isurnr2-1
         inpelm = kmeshn(isurnr1)
         inelem = kmeshn(isurnr1+nsurfs)
         if ( debug ) &
            write(irefwr,1) 'istart1, istart2, inpelm, inelem', &
                             istart1, istart2, inpelm, inelem

         if ( inelem/=kmeshn(isurnr2+nsurfs) ) then

!        --- surfaces do not have the same number of elements

            call errint ( i, 1 )
            call errint ( isurnr1, 2 )
            call errint ( inelem, 3 )
            call errint ( isurnr2, 4 )
            call errint ( kmeshn(isurnr2+nsurfs), 5 )
            call errint ( ielgrp, 6 )
            call errsub ( 2824, 6, 0, 0 )

         end if  ! ( inelem/=kmeshn(isurnr2+nsurfs) )

!        --- Loop over all elements in the surfaces

         npoloc = abs(kmeshh(1,isurnr1+1))
         call msh075surfs1 ( kmeshl, kmeshm, kmeshn, inodp, istart1, &
                             inelem, inpelm, kmhsur(isurnr1)%kmeshcsur, npoloc )

         npoloc = abs(kmeshh(1,isurnr2+1))
         call msh075surfs1 ( kmeshl, kmeshm, kmeshn, inodp, istart2, &
                             inelem, inpelm, kmhsur(isurnr2)%kmeshcsur, npoloc )

         lengc = lengc + 2*inelem*inpelm    ! raise lengc
         iselem = iselem+inelem    ! raise iselem
         iinp = iinp+4    ! raise iinp

      end do  ! i = 1, isurfs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh075surfs'

      end if
      call erclos ( 'msh075surfs' )

      end
