      subroutine mshcompdispl20 ( coornew, inpada, rinada, nunkp, &
                                  maxphys, usol, uold, numvecs, &
                                  indprf, kprobf, markcurv, marknodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-12-2008
!
!   copyright (c) 2008-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute new coordinates of boundary (2d)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpada(*), nunkp, maxphys, numvecs, indprf, kprobf(*), &
              markcurv(*), marknodes(*)
      double precision coornew(2,npoint), rinada(*), &
                       usol(*), uold(*)

!     coornew        o    Array containing the new coordinates of the nodes
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     inpada         i    User input array for subroutine ADAPBOUN
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     markcurv       o    Array of size ncurvs that marks all curves that
!                         have been adapted in the moving boundary
!                         markcurv(icurnr) = 0: curve has not been moved
!                         markcurv(icurnr) = 1: curve has been moved
!     marknodes      o    Array of size npoint that marks all nodes that
!                         have been adapted in the moving boundary
!                         marknodes(node) = 0: node has not been moved
!                         marknodes(node) = 1: node has been moved
!     maxphys        i    Maximum number of unknowns per point
!     numvecs        i    Number of vectors in iseqvecs
!     nunkp          i    Maximum number of unknowns in a point
!     rinada         i    User input array for subroutine ADAPBOUN
!     uold           i    Array containing the old solution
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nadapt
      parameter ( nadapt=20 )
      integer ncurvsl, i, inodp, linear, n
      integer, allocatable :: icurvar(:), icurvsl(:), iwork(:)
      double precision , allocatable :: coorbn(:,:), anorm(:,:), &
                                        coorbnnew(:,:)
      logical closed

!     anorm          In this array the normal vector is stored
!                    anorm(j,i) contains the j th component of the normal
!                    vector in the i-th integration point
!     closed         If true the boundary is closed
!     coorbn         array containing the old co-ordinates of the boundary
!     coorbnnew      array containing the new co-ordinates of the boundary
!     i              Counting variable
!     icurvar        Array of length ncurvs that contains information of
!                    the curves that must be changed. icurvar(i) mus be
!                    filled as follows:
!                    0  curve i is not yet adapted
!                    1  curve i has been adapted and is a single curve
!     icurvsl        Array of length ncurvs that is filled with all
!                    stand alone curves that must be adapted in the
!                    correct sequence and provided with a sign
!     inodp          Nodal point number
!     iwork          integer work array
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     n              number of nodes in boundary
!     nadapt         Length of array inpada
!     ncurvsl        Number of stand-alone curves that must be adapted
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0047         Compute normal to coorol and store in array anorm
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH065         Split compound curves into single curves
!     MSHCOMPDISPL22 recompute co-ordinates of curves
!     PRINRL1        Print 2d real vector
!     SEPER0027      Fill nodal points of curves as indicated by ichois and
!                    icurvs in array iwork
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     385   boundary contains less than two points
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     coornew := coor
!     Store nodes of curves to be changed in iwork and coordinates in curves
!     Compute new coordinates of curves
!     Adapt coornew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcompdispl20' )

      allocate(icurvar(ncurvs), icurvsl(ncurvs), iwork(npoint), &
               stat = error)
      if ( error/= 0 ) then

!     --- error#0, so not enough space

         call errint ( 2*ncurvs+npoint, 1 )
         call errint ( error, 2 )
         call errchr ( 'icurvar', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcompdispl20'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      linear = inpada(7)

!     --- coornew := coor

      coornew = coor

!     --- Fill single curves and check if curves are used only once

      call msh065 ( inpada(nadapt+1), inpada(2), icurvar, &
                    icurvsl, ncurvsl, .false. )

!     --- mark curves in array markcurv

      do i = 1, ncurvsl
         markcurv(abs(icurvsl(i))) = 1
      end do  ! i = 1, ncurvsl

!     --- Store nodes of curves to be changed in iwork

      n = npoint
      call seper0027 ( ncurvsl, icurvsl, n, iwork )
      if ( n<2 ) then
         call errint ( n, 1 )
         call errsub ( 385, 1, 0, 0 )
         go to 1000
      end if
      if ( debug ) write(irefwr,1) 'n', n

      allocate(coorbn(2,n), coorbnnew(2,n), anorm(2,n), stat = error)
      if ( error/= 0 ) then

!     --- error#0, so not enough space

         call errint ( 6*n, 1 )
         call errint ( error, 2 )
         call errchr ( 'coorbn', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

!     --- mark nodes in array marknodes

      do i = 1, n
         marknodes(iwork(i)) = 1
      end do  ! i = 1, n

!     --- Fill array coorbn with coordinates of boundary

      do i = 1, n
         inodp = iwork(i)
         coorbn(1:2,i) = coor(1:2,inodp)
      end do
      if ( debug ) call prinrl1 ( coorbn, n, 2, 'coorbn' )

!     --- compute normal to coorol and store in array anorm
!         First component nx, second one ny

      call er0047 ( linear, coorbn, n, anorm )
      coorbnnew = coorbn
      if ( debug ) call prinrl1 ( anorm, n, 2, 'anorm' )

      if ( iwork(1)==iwork(n) ) then

!     --- First and last point of the curves are the same, hence the
!         curve is closed

         n = n-1
         closed = .true.

      else

!     --- not a closed curve

         closed = .false.

      end if
      if ( debug ) write(irefwr,*) 'closed ', closed

!     --- Compute new coordinates of curves

      call mshcompdispl22 ( inpada, rinada, nunkp, coor, &
                            kprobf, indprf, usol, iwork, coorbnnew, &
                            coorbn, anorm, kmeshm, &
                            closed, n, uold, maxphys, numvecs )
      if ( debug ) call prinrl1 ( coorbnnew, n, 2, 'coorbnnew' )

!     --- Adapt coornew

      do i = 1, n
         inodp = iwork(i)
         coornew(1:2,inodp) = coorbnnew(1:2,i)
      end do

1000  deallocate ( icurvar, icurvsl, iwork, coorbn, anorm, coorbnnew, &
                   stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'icurvar', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcompdispl20'

      end if  ! ( debug )
      call erclos ( 'mshcompdispl20' )

      end
