      subroutine mshcoorcentr7 ( coor, m1, m2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-06-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coordinates of centroid of 7 point triangle when the coordinates of
!     the vertices and midside points are given
!     Special case of rectangular 3d grid with brick orientation
! **********************************************************************
!
!                       KEYWORDS
!
!     coordinate
!     centre
!     mesh
!     mesh_generation
!     rectangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer m1, m2
      double precision coor(3,*)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points
!     m1             i    Number of elements in 1-direction
!     m2             i    Number of elements in 2-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n1, i, j, idim, shiftj, shifti1, shifti2, shifti3, &
              i1, i2, i3, i4, i5, i6, i7, i8, i9, i10, i11
      double precision oneninth, fourninth

!     fourninth      4/9
!     i              Counting variable
!     i1             Point 1 in rectangle
!     i10            Point 10 in rectangle
!     i11            Point 10 in rectangle
!     i2             Point 2 in rectangle
!     i3             Point 3 in rectangle
!     i4             Point 4 in rectangle
!     i5             Point 5 in rectangle
!     i6             Point 6 in rectangle
!     i7             Point 7 in rectangle
!     i8             Point 8 in rectangle
!     i9             Point 9 in rectangle
!     idim           Counting variable
!     j              Counting variable
!     n1             Number of nodes in 1-direction
!     oneninth       1/9
!     shifti1        first shift in i-direction
!     shifti2        second shift in i-direction
!     shifti3        third shift in i-direction
!     shiftj         shift in j-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The number of elements in 1 direction is m1 in 2 direction m2
!     The number of nodes along sides n1 resp n2
!
!     Numbering is performed for rows of elements (j=1(1)m2)
!     and per row for sets of 2 elements (i=1(1)m1)
!
!     The ordering for a set of 2 triangles is as follows
!               10
!      9 x-------x-------x11
!        x               x
!        x   x8          x
!      5 x       x6      x7
!        x           x4  x
!        x               x
!        x-------x-------x
!       1        2       3
!
!     with 1: shiftj+2*i-1            shiftj = 2(j-1)(n1+m1)
!          2: shiftj+2*i
!          3: shiftj+2*i+1
!          4: shiftj+i+n1
!          5: shiftj+shifti1+2*i-1    shifti1 = n1+m1
!          6: shiftj+shifti1+2*i
!          7: shiftj+shifti1+2*i+1
!          8: shiftj+i+shifti2        shifti2 = 2n1+m1
!          9: shiftj+shifti3+2*i-1    shifti3 = 2(n1+m1)
!         10: shiftj+shifti3+2*i
!         11: shiftj+shifti3+2*i+1
!
!     The coordinates of the centroid are determined by assuming a quadratic
!     polynomial, i.e.
!
!     x4 = 4/9 ( x2+x7+x6) - 1/9 (x1+x3+x11)
!     x8 = 4/9 ( x5+x6+x10) - 1/9 (x1+x9+x11)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcoorcentr7' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcoorcentr7'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- set constants

      n1 = 2*m1+1
      shifti1 = n1+m1
      shifti2 = shifti1+n1
      shifti3 = shifti2+m1
      oneninth = 1d0/9d0
      fourninth = 4d0*oneninth

!     --- For all rows of elements do

      do j = 1, m2

!     --- For all sets of 2 elements in row do

         shiftj = 2*(j-1)*shifti1
         do i = 1, m1

!        --- Fill node numbers of 11 points

            i1 = shiftj+2*i-1
            i2 = shiftj+2*i
            i3 = shiftj+2*i+1
            i4 = shiftj+i+n1
            i5 = shiftj+shifti1+2*i-1
            i6 = shiftj+shifti1+2*i
            i7 = shiftj+shifti1+2*i+1
            i8 = shiftj+i+shifti2
            i9 = shiftj+shifti3+2*i-1
            i10 = shiftj+shifti3+2*i
            i11 = shiftj+shifti3+2*i+1

!           --- x4 = 4/9 ( x2+x7+x6) - 1/9 (x1+x3+x11)
!               x8 = 4/9 ( x5+x6+x10) - 1/9 (x1+x9+x11)

            do idim = 1, 3

               coor(idim,i4) = fourninth* &
                 (coor(idim,i2)+coor(idim,i7)+coor(idim,i6)) - &
                 oneninth*(coor(idim,i1)+coor(idim,i3)+coor(idim,i11))
               coor(idim,i8) = fourninth* &
                 (coor(idim,i5)+coor(idim,i6)+coor(idim,i10)) - &
                 oneninth*(coor(idim,i1)+coor(idim,i9)+coor(idim,i11))

            end do  ! idim = 1, 3

         end do  ! i = 1, m1

      end do  ! j = 1, m2

1000  call erclos ( 'mshcoorcentr7' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcoorcentr7'

      end if

      end
