      subroutine mshfindadcurv ( markcurv, marknodes, numcurvs, adapcurvs, &
                                 coornw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all curves that have at least one node that has been moved
!     during a free boundary problem
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer markcurv(*), marknodes(*), numcurvs, adapcurvs(*)
      double precision  coornw(ndim,*)

!     adapcurvs      o    Array of size numcurvs containing the curves that
!                         must be adapted
!     coornw         i    Array containing the new co-ordinates
!     markcurv       i    Array of size ncurvs that marks all curves that
!                         have been adapted in the moving boundary
!                         markcurv(icurnr) = 0: curve has not been moved
!                         markcurv(icurnr) = 1: curve has been moved
!     marknodes      i    Array of size npoint that marks all nodes that
!                         have been adapted in the moving boundary
!                         marknodes(node) = 0: node has not been moved
!                         marknodes(node) = 1: node has been moved
!     numcurvs       o    Number of curves that must be adapted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istar3, k1, k2, nodp1, nodp2, icurnr, dir, k
      double precision dist
      logical changed, found

!     changed        If true the curve is a candidate for adpatation
!     dir            if 1 the positive direction of the curve must be used
!                    otherwise the negative direction
!     dist           Distance between two points (squared)
!     found          If true the curve must be adapted
!     icurnr         Curve sequence number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     nodp1          Node sequence number (first in curve)
!     nodp2          Node sequence number (last in curve)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     numcurvs := 0
!     For all single curves icurnr do
!        if ( curve is not changed before ) then
!           if ( one of the end points has been moved ) then
!              numcurvs := numcurvs+1;  adapcurvs(numcurvs) := icurnr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfindadcurv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfindadcurv'
         call prinin ( marknodes, npoint, 'marknodes' )
         call prinin ( markcurv, ncurvs, 'markcurv' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get constants

      istar3 = kmeshm(2)-1   ! starting address of node numbers

!     --- numcurvs := 0

      numcurvs = 0

!     --- For all single curves icurnr do

      do icurnr = 1, ncurvs

         changed = .false.  ! start value of changed
         dir = -1
         if ( kelmq>0 ) then

!        --- There are composite curves
!            Set changed true in case of compound curve

            if ( kmeshq(icurnr+1)-kmeshq(icurnr)>0 ) changed = .true.

         end if  ! ( kelmq>0 ) then

         changed = changed .or. markcurv(icurnr)>0  ! curve is changed
         if ( debug ) then
            write(irefwr,1) 'icurnr', icurnr
            write(irefwr,*) 'changed ', changed
         end if  ! ( debug )

         if ( .not. changed ) then

!        --- curve is not changed before
!            Find initial and end point

            k1 = kmeshm(4+icurnr)
            k2 = kmeshm(5+icurnr)-1
            nodp1 = kmeshm(istar3+k1)
            nodp2 = kmeshm(istar3+k2)
            found = nodp1>0 .and. nodp2>0 .and. k2>k1  ! curve contains nodes
            if ( debug ) write(irefwr,1) 'nodp1, nodp2', nodp1, nodp2
            if ( found ) then

!           --- Curve has been filled
!               Check if one of the nodes has been moved

               found = marknodes(nodp1)>0

               if ( found ) then

!              --- First node has been moved, check displacement

                  dist = 0d0
                  do k = 1, ndim
                     dist = dist+(coornw(k,nodp1)-coor(k,nodp1))**2
                  end do  ! k = 1, ndim
                  if ( dist<=epsmac ) found = .false. ! no real movement
                  if ( debug ) write(irefwr,2) 'dist/1', dist
                  if ( found ) dir = 1  ! first node is changed

               end if  ! ( found )
               if ( .not. found ) then

!              --- node 1 has not been moved, what about node 2?

                  found = marknodes(nodp2)>0
                  if ( found ) then

!                 --- First node has been moved, check displacement

                     dist = 0d0
                     do k = 1, ndim
                        dist = dist+(coornw(k,nodp2)-coor(k,nodp2))**2
                     end do  ! k = 1, ndim
                     if ( dist<=epsmac ) found = .false. ! no real movement
                     if ( debug ) write(irefwr,2) 'dist/2', dist

                  end if  ! ( found )

               end if  ! ( .not. found )

            end if  ! ( found )

            if ( found ) then

!           --- one of the end points has been moved
!               numcurvs := numcurvs+1;  adapcurvs(numcurvs) := icurnr

               numcurvs = numcurvs+1
               adapcurvs(numcurvs) = icurnr*dir
               markcurv(icurnr) = 1

            end if  ! ( found )

         end if  ! ( .not. changed )

      end do  ! icurnr = 1, ncurvs

1000  if ( debug ) then

!     --- Debug information

         call prinin ( adapcurvs, numcurvs, 'adapcurvs' )
         write(irefwr,*) 'End mshfindadcurv'

      end if  ! ( debug )
      call erclos ( 'mshfindadcurv' )

      end
