      subroutine mshlevelmarkpt ( ipointobst, npoint, levelset )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-07-2011 Remove iwork from parameter list
!        version  1.0  date 08-04-2004
!
!   copyright (c) 2004-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array ipointobst for the first time for the level set function
!     All nodes with phi>=0 are marked
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     node
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipointobst(*), npoint
      double precision levelset(*)

!     ipointobst     o    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!     levelset       i    Array of length npoint containing the value of the
!                         level set function in each node
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numlevpnt, numlevbnd
      double precision eps
      integer, allocatable :: iwork(:)

!     eps            Local accuracy
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     iwork          Work array of length npoint to store the boundary
!                    nodes temporarily
!     numlevbnd      Number of nodal points with level set = 0
!     numlevpnt      Number of nodal points with level set>0
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     eps := threshold value
!     numlevpnt := 0; numlevbnd := 0
!     for i := 1 (1) npoint do
!        if ( abs(levelset(i))<eps ) then
!           numlevbnd := numlevbnd+1
!           iwork(numlevbnd) := i
!        else if ( levelset(i)>0 ) then
!           numlevpnt := numlevpnt+1
!           ipointobst(4+numlevpnt) := i
!     fill rest of array ipointobst
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshlevelmarkpt' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshlevelmarkpt'
         call prinrl ( levelset, npoint, 'levelset' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- allocate work array

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- eps := threshold value
!         numlevpnt := 0; numlevbnd := 0

      eps = 1d-8
      numlevpnt = 0
      numlevbnd = 0

!     --- for i := 1 (1) npoint do

      do i = 1, npoint

         if ( abs(levelset(i))<eps ) then

!        --- abs(levelset(i))<eps, point exactly on boundary, must be
!            exceptional
!            numlevbnd := numlevbnd+1
!            iwork(numlevbnd) := i

            numlevbnd = numlevbnd+1
            iwork(numlevbnd) = i

         else if ( levelset(i)>0 ) then

!        --- levelset(i)>0
!            numlevpnt := numlevpnt+1
!            ipointobst(4+numlevpnt) := i

            numlevpnt = numlevpnt+1
            ipointobst(4+numlevpnt) = i

         end if  ! ( abs(levelset(i))<eps )

      end do  ! i = 1, npoint

!     --- Fill four positions of ipointobst
!         At this moment position 2 is set to zero, it is filled in subroutine
!         mshobstmarkel
!         In that subroutine points are moved from part 1 to part 2
!         Position 4 is also set to zero
!         In mshobstmarkbt we fill position 4 and part 4

      ipointobst(1) = numlevpnt   ! number of points with phi>0
      ipointobst(2) = 0
      ipointobst(3) = numlevbnd   ! number of points on boundary (phi=0)
      ipointobst(4) = 0
      ipointobst(numlevpnt+4+1:numlevpnt+4+numlevbnd) = &
         iwork(1:numlevbnd)

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'mshlevelmarkpt' )
      if ( debug ) then

!     --- Debug information

         call prinin ( ipointobst, 4, 'ipointobst-1' )
         call prinin ( ipointobst(5), numlevpnt, 'ipointobst-2' )
         call prinin ( ipointobst(5+numlevpnt), numlevbnd, &
                       'ipointobst-3' )
         write(irefwr,*) 'End mshlevelmarkpt'

      end if

      end
