      subroutine mshmarkedglen ( blockinfo, transportinfo, &
                                 refedges, nsubdiv, nmeshc, nmeshh, &
                                 lenstore, imark )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-03-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute length of array istore
!
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     refine
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer blockinfo(*), transportinfo(*), refedges(*), &
              nsubdiv, nmeshc(*), imark(*), lenstore, nmeshh(*)

!     blockinfo      i    Array with global block information for block
!                         number iblock
!                         See Programmers Guide Section 24.2 for a description
!     imark         i/o   Work array of length npoint
!                         if k = imark(inode)>0, k defines the sequence number
!                         of node inode in transinfo
!     lenstore       o    Computed length of array istore
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshh         i    nmesh part h, contains element numbers corresponding
!                         to edges
!     nsubdiv        i    Number of edges that are subdivided, hence number
!                         of extra nodes to be created
!     refedges       i    Contains the edge numbers of the edges that
!                         must be subdivided
!                         Length: nsubdiv
!     transportinfo  i    Array containing information of the transport of
!                         unknowns between neighboring blocks
!                         See Programmers Guide Section 24.2 for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numneigh, i, istartinfo, nneigh, j, &
              inode, ipstart, nnode_edge, inode1, inode2, &
              iedge, nstore, nelems

!     i              Counting variable
!     iedge          edge sequence number
!     inode          Node sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     ipstart        Starting address in array
!     istartinfo     Starting address in array transinfo
!     j              Counting variable
!     nelems         number of elements connected to edge
!     nneigh         Number of nodes on common side of two blocks
!     nnode_edge     number of nodes corresponding to edge
!     nstore         number of edges in istore
!     numneigh       number of neighbor blocks
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmarkedglen' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmarkedglen'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      lenstore = 0
      numneigh = blockinfo(1)  ! number of neighbors of block
      do i = 1, numneigh

         istartinfo = blockinfo(2*i+1)   ! starting address in transportinfo
         nneigh = transportinfo(istartinfo)  ! number of nodes on common side

!        --- For all nodes inode on common side of present block and iblock do
!              imark(inode) := sequence number in transportinfo

         do j = 1, nneigh
            inode = transportinfo(istartinfo+j)  ! node on common side
            imark(inode) = j
         end do  ! j = 1, nneigh

!        --- For all edges that must be refined do
!              If both nodes satisfy imark(inode)>0 then
!                 Store local node numbers in istore

         nstore = 0  ! number of edges in istore
         do j = 1, nsubdiv

            iedge = refedges(j)  ! edge sequence number
            ipstart = nmeshh(iedge)  ! starting address
            nelems = nmeshh(ipstart)  ! number of elements connected to edge
            if ( nelems==1 ) then

!           --- only one element connected to edge, &
!               hence edge may be on interface

               ipstart = nmeshc(iedge)  ! starting address
               nnode_edge = nmeshc(ipstart)  ! number of nodes on edge
               inode1 = nmeshc(ipstart+1)    ! first node
               inode2 = nmeshc(ipstart+nnode_edge) ! last node

               if ( imark(inode1)>0 .and. imark(inode2)>0 ) then

!              --- edge found, store nodes

                  nstore = nstore+1

               end if  ! ( imark(inode1)>0 .and. imark(inode1)>0 )

            end if  ! ( nelems==1 )

         end do  ! j = 1, nsubdiv

!        --- Send istore to neighbor

         if ( nstore==0 ) then

!        --- Nothing to be sent, but that gives problems since there is nothing
!            to receive in thate case
!            Therefore we send two zeros

             nstore = 1

         end if  ! ( nstore==0 )
         lenstore = lenstore+nstore

      end do  ! i = 1, numneigh

1000  call erclos ( 'mshmarkedglen' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshmarkedglen'

      end if  ! ( debug )

      end
