      subroutine mshnframecur ( cunit, nelm, icurnr, sleng )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.1  date 20-06-2013 New definition file ref numbers
!        version  1.0  date 06-05-2009
!
!   copyright (c) 2009-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    determine number of elements to be created along polygon- &
!    track of points with given coarsenesses and the Euclidian length of
!    the polygontrack
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     framecurve
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision cunit, sleng
      integer nelm, icurnr

!    cunit  i  overall coarseness as given by the user
!    icurnr i  curve number
!    nelm   o  number of elements to be created on curve
!    sleng  o  total length of all segments together
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenname, ios, iref
      character (len=13) filename
      integer, allocatable          :: helpusp(:)
      double precision, allocatable :: helppoints(:,:)
      integer i, iusp, j, nh, nnodes, ntype, ndegree
      double precision dist, dx, dy, dz, x1, y1, z1, coarse, srest
      logical check

!     check          Indicates if the file exist (true) or not
!     coarse         coarseness value (local)
!     dist           Euclidian distance
!     dx             dif distance in x
!     dy             dif distance in y
!     dz             dif distance in z
!     error          error indicator
!     filename       Name of file
!     helppoints     helpfile coordinates for points in framecurve file
!     helpusp        helpfile for userpoint numbers as given in framecurve file
!     i              Counting variable
!     ios            IOSTAT variable, returns from reading or writing
!     iref           file reference number
!     iusp           helpvariable to store userpointnumber
!     j              helpvariable to store integer value
!     lenname        Length of name
!     ndegree        degree of spline
!     nh             local number of elements to be created
!     nnodes         number of points that define the polygon
!     ntype          type of spline curve
!     srest          restdistance of previous part
!     x1             x-coordinate
!     y1             y-coordinate
!     z1             z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCHKINT       Check if integer is between two values
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     146   surface number has incorrect value
!    1669   file does not exist
!    2706   error in opening file
!    2910   Not enough space
! **********************************************************************
!
!                       PSEUDO CODE
!
!     construct filename
!     open file
!     read values
!     check the values as read from userfile and allocate helpfiles
!     determine length curve and compute number of elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshnframecur' )

!     --- Open file and determine number of nodes that define the
!         framecurve line and corresponding helpvalues

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshnframecur'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      iref = inisetnewref()  ! reference number of file

!     --- Create file name

      filename = 'framecurve_'
      if ( icurnr<0 .or. icurnr>=10000 ) then

!     --- Curve number larger than 9999, not yet implemented

         call errint ( icurnr, 1 )
         call errint ( 10000, 2 )
         call errsub ( 146, 2, 0, 0 )
         go to 1000

      else if ( icurnr<10 ) then

!     --- Curve number between 1 and 9

         lenname = 11
         write(filename(11:lenname),100) icurnr
100      format(i1)

      else if ( icurnr<100 ) then

!     --- Curve number between 10 and 99

         lenname = 12
         write(filename(11:lenname),110) icurnr
110      format(i2)

      else if ( icurnr<1000 ) then

!     --- Curve number between 100 and 999

         lenname = 13
         write(filename(11:lenname),120) icurnr
120      format(i3)

      else

!     --- Curve number between 1000 and 9999

         lenname = 14
         write(filename(11:lenname),130) icurnr
130      format(i4)

      end if  ! ( icurnr<0 .or. icurnr>=10000 )

!     --- Check if file exists

      inquire ( file=filename, exist = check )
      if ( .not. check ) then

!     --- Error: file does not exist

         call errchr ( filename, 1 )
         call errsub ( 1669, 0, 0, 1 )
         go to 1000

      end if  ! ( .not. fileexist )
      open ( unit = iref, file = filename, iostat = ios )
      if ( ios/=0 ) then

!     --- error in opening file

         call errint ( ios, 1 )
         call errchr ( filename, 1 )
         call errsub ( 2706, 1, 0, 1 )
         go to 1000

      end if

!     --- Read number of nodes, type of curve and degree of helpspline
!         and check whether values are ok

      read(iref,*) nnodes, ntype, ndegree

!     --- Check the values found

      call erchkint ( nnodes,  1, 10000, 'number of nodes' )
      call erchkint ( ntype,   0,     1, 'type of curve'   )
      call erchkint ( ndegree, 0,     0, 'degree of fitting curve')

!     --- Second step: determine all values to be used for the construction
!         of the curve

      allocate ( helpusp(nnodes),      stat = error )
      allocate ( helppoints(4,nnodes), stat = error )

      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( 4*nnodes, 1 )
         call errint (  error, 2 )
         call errchr ( 'helppoints', 1 )
         call errsub ( 2910, 2, 0, 1 )
         goto 1000

      end if  ! ( error/= 0 )

!     --- Run through file and read all helpnodes of framecurve

      do i=1, nnodes

         read(iref,*) j, x1, y1, z1, iusp, coarse

         if ( j/=i ) then
            write(irefwr,*) 'wrong order in file, stop'
            go to 1000
         end if

         helppoints(1,i) = x1
         helppoints(2,i) = y1
         helppoints(3,i) = z1
         helppoints(4,i) = coarse

         helpusp(i) = iusp

      end do

!     --- For debugging: check values read

      if ( debug ) then
         do i = 1 , nnodes
            write(irefwr,*) (helppoints(j,i),j=1,4),helpusp(i)
         end do
      end if

!     --- Determine total length of this curve and determine number
!         of elements to be placed on the total curve

      sleng  = 0d0
      srest  = 0d0

!     --- Find number of elements (initialize)

      nelm = 0

      do i = 1 , nnodes - 1

         x1 = helppoints(1,i)
         y1 = helppoints(2,i)
         z1 = helppoints(3,i)

         dx = helppoints(1,i+1) - x1
         dy = helppoints(2,i+1) - y1
         dz = helppoints(3,i+1) - z1

         dist = sqrt( dx*dx + dy*dy +dz*dz )

         sleng = sleng + dist

!        --- Avarage coarseness value:

         coarse = cunit * (helppoints(4,i) + helppoints(4,i+1))/2d0

!        --- Add and check number of elements on this part:

         if ( helpusp(i+1)==0 ) then

!        --- End node is local node: only helpnodes between:

             nh    = int((srest+dist)/coarse)
             nelm = nelm + nh

             srest = srest+dist - nh * coarse

         else if ( helpusp(i+1)/= 0 ) then

!        --- End node is userpoint: hence point has to be used:

            nh = int((srest+dist)/coarse+0.5d0)
            nelm = nelm + nh

            srest = 0

         end if

      end do

!     --- Number of elements and total length of polygon determined

!         --- Close file and deallocate helparrays:

      close ( unit = iref )
      call inirmref ( iref )
      if ( error==0 ) deallocate ( helpusp,    stat = error )
      if ( error==0 ) deallocate ( helppoints, stat = error )

      if ( error/=0 ) then

!     --- error#0, errors in closing and/or deallocating

         write(irefwr,*) 'Error in mshnframecur'
         write(irefwr,*) 'Stop execution'
         go to 1000

      end if  ! ( error/= 0 )

1000  call erclos ( 'mshnframecur' )
      if ( debug ) then

!     --- Debug information

      end if  ! ( debug )

      end
