      subroutine mshobstboun2d5 ( ibounelems, iworkedge, npointnew, &
                                  coornew, nx, ny, iwork, &
                                  nmap, lencurvs, iobst, nmapmax, npointmap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 02-10-2012 New call to mshobstpoint
!        version  1.1  date 11-07-2004 Do not set nmapmax
!        version  1.0  date 26-05-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new points in mesh by subdividing some edges of
!     elements that have all points on the boundary of obstacle iobst
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     subdivision
!     edge
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibounelems(*), iworkedge(*), nmapmax, npointnew, npointmap, &
              nx, ny, iwork(*), iobst, &
              nmap, lencurvs
      double precision coornew(2,*)

!     coornew        o    Array containing the new coordinates of the nodes
!     ibounelems     i    Array of length 3+numbounelms (<nelem) containing
!                         the element numbers of the elements that have all
!                         points on the boundary or on the obstacle
!                         Pos. 1 contains the number of elements that have all
!                                points on one obstacle
!                         Pos. 2 contains the number of elements that have all
!                                points on the boundary
!                         Pos. 3 contains the number of elements that have some
!                                points on the boundary and the others on an
!                                obstacle or on different obstacles
!                         The other positions contain the element numbers in the
!                         sequence: first all elements corresponding to pos. 1
!                         then to pos 2 and finally to pos 3
!     iobst          i    Obstacle sequence number
!     iwork          i    integer work array of length npoint
!                         iwork(i) is filled as follows (see pseudo code)
!                          0 for all points outside the obstacle and
!                            the boundaries
!                         -1 for all points inside the obstacle
!                          i for all points on the boundary of the obstacle i
!                         1001 for all points on the outer curves
!                              but not on an obstacle
!                         1002 for all points on the outer curves
!                              and on an obstacle
!     iworkedge      o    work array of length nedges, containing information
!                         whether an edge must be subdivided into two parts (>0)
!                         or not (0)
!                         if iworkedge(iedge)>0, the value refers to the node
!                         number of the new node on the edge
!     lencurvs       i    Overestimate of the number of points along the new
!                         curves
!     nmap          i/o   length of second parameter of imap and rmap
!                         At input: declared length
!                         At output: computed length
!     nmapmax       i/o   Actual value of nmap
!     npointmap      i    Declared value of npointnew
!     npointnew      o    Sequence number of last generated node
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, numobstelms, istart, i, istartedge, j, iedge, &
              istartnode, node1, node2, nedge_elem
      double precision xc, yc
      logical inside

!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     inside         indicator how many internal regions have to be
!                    considered
!     istart         First position used at a part
!     istartedge     Starting position of edge in array NMESHD
!     istartnode     ?
!     j              Counting variable
!     nedge_elem     number of edges corresponding to element
!     node1          Node of face
!     node2          Node of face
!     numobstelms    number of elements that have all points on the obstacle
!     xc             array with the coordinates of the centre point of the
!                    element
!     yc             y-coordinate of the midth-point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMAP2D   Fill arrays imap and rmap for a new point inode which is
!                    the mean value of two already existing nodes node1 and
!                    node2 stored in imap/rmap
!     MSHOBSTPOINT   Check if point with coordinates x,y is inside or outside
!                    the obstacle
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array iworkshape with the shape numbers of the elements
!     Clear array iworkedge
!     Copy old arrays imap and rmap into imap2 and rmap2
!     npointnew := npoint
!     For all elements ielem with all nodes on boundary do
!        For all edges iedge corresponding to ielem do
!           nneigh := number of triangles corresponding to iedge
!           if ( nneigh=2 ) then
!              create new point
!     For all elements ielem with all nodes on obstacle do
!        For all edges iedge corresponding to ielem do
!           compute coordinates of centroid of edge
!           if ( centroid outside obstacle ) then
!              create new point
!     For all elements ielem with some nodes on obstacle and all others on
!         boundary do
!        For all edges iedge corresponding to ielem do
!           if ( one endpoint on boundary and other on obstacle ) then
!              create new point
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstboun2d5' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstboun2d5'
         call prinin ( ibounelems, 10, 'ibounelems' )

      end if
      if ( ierror/=0 ) go to 1000

      numobstelms = ibounelems(1)

!     --- For all elements ielem with all nodes on boundary of obstacle iobst do

      istart = 3
      do i = 1, numobstelms
         ielem = ibounelems(istart+i)

!        --- First we have to check if the centroid of the element is inside
!            or outside the obstacle.
!            Only if the centroid is outside the obstacle we want to
!            create an extra node

         istartedge = nmeshd(ielem)
         nedge_elem = nmeshd(istartedge)  ! number of edges of element

!        --- The vertices are given as the first nodes of the edges

         xc = 0d0   ! coordinates of centroid
         yc = 0d0
         do j = 1, nedge_elem     ! loop over edges (only for 3 node triangles)
            iedge = abs(nmeshd(istartedge+j))  ! edge
            istartnode = nmeshc(iedge)
            node1 = nmeshc(istartnode+1)
            xc = xc+coor(1,node1)
            yc = yc+coor(2,node1)
         end do  ! j = 1, nedge_elem
         xc = xc/nedge_elem
         yc = yc/nedge_elem
         if ( iwork(node1)==iobst ) then

!        --- Check if centroid inside of outside the obstacle

            call mshobstpoint ( iobst, nx, ny, iobgrid2d, xc, yc, inside, &
                                lencurvs, iobgrid2d1 )

            if ( .not. inside ) then

!           --- Centroid of element outside the obstacle
!               For all edges iedge corresponding to ielem do

               do j = 1, nedge_elem
                  iedge = abs(nmeshd(istartedge+j))
                  if ( iworkedge(iedge)==0 ) then

!                 --- compute coordinates of centroid of edge

                     istartnode = nmeshc(iedge)
                     node1 = nmeshc(istartnode+1)
                     node2 = nmeshc(istartnode+2)
                     xc = 0.5d0*(coor(1,node1)+coor(1,node2))
                     yc = 0.5d0*(coor(2,node1)+coor(2,node2))

!                    --- if ( centroid outside obstacle ) then
!                           create new point

                     call mshobstpoint ( iobst, nx, ny, iobgrid2d, &
                                         xc, yc, inside, &
                                         lencurvs, iobgrid2d1 )
                     if ( .not. inside ) then

!                    --- Point is outside

                        npointnew = npointnew+1
                        coornew(1,npointnew) = xc
                        coornew(2,npointnew) = yc
                        iworkedge(iedge) = npointnew
                        call mshobstmap2d ( npoint, npointmap, &
                                            nmap, nmapmax, node1, node2, &
                                            npointnew )
                        if ( debug ) then

!                       --- Debug information

                           write(irefwr,*) 'Node ', npointnew, &
                              ' has been created between nodes ', &
                              node1, ' and ', node2, &
                              ' which are on the obstacle ', iobst

                        end if  ! ( debug )

                     end if  ! ( .not. inside )

                  end if  ! ( iworkedge(iedge)==0 )

               end do  ! j = 1, nedge_elem

            end if  ! ( .not. inside )

         end if  ! ( iwork(node1)==iobst )

      end do  ! i = 1, numobstelms

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstboun2d5'

      end if
      call erclos ( 'mshobstboun2d5' )

      end
