      subroutine mshobstms03 ( kmeshcnew, coornew, iedges, kmeshnew, &
                               ipointobstnew, ielemobstnew, &
                               iinput, nmap, iinputusr, ielmfaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 22-11-2011 Remove elsize
!        version  1.0  date 21-05-2004
!
!   copyright (c) 2004-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a new mesh from a fixed mesh with an obstacle crossing the mesh
!     Elements that are intersected are subdivided into several elements,
!     such that these new elements all belong to the obstacle or are
!     outside the obstacle
!     In this special subroutine points are moved to the obstacle
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(*), nmap, &
              iedges(3,*), ipointobstnew(*), ielemobstnew(*), &
              kmeshnew, iinput(*), iinputusr(*), &
              ielmfaces(4,*)
      double precision coornew(2,*)

!     coornew        o    Array containing the new coordinates of the nodes
!     iedges         o    Work array of size nedges
!                         If iedges(i) = 0, no point has been created in edge i
!                         If iedges(i)>0, a point has been created in edge i,
!                         with sequence number iedges(i)
!     ielemobstnew   o    New array ielemobst
!     ielmfaces      i    ?
!     iinput         o    Array to pass some constants to mshobstms
!                         Contents
!                         1:  length of new array kmeshc
!                         2:  length of new array ipointobst
!                         3:  length of new array ielemobst
!     iinputusr      i    Integer input array defining which actions must
!                         be undertaken
!                         At this moment iinputusr is under construction
!                         Contents
!                         Pos. 1  Last entry that is filled by the calling
!                                 program
!                         Pos. 2  shft: Defines whether nodes must be
!                                       shifted to the obstacle (1) or not (0)
!                                       Default value: 0
!                         Pos. 3  smth: Defines type of smoothing;
!                                       Laplacian smoothing (1), angle
!                                       optimization (2), both (3), none (0)
!                                       Only allowed if shft=1
!     ipointobstnew  o    New array ipointobst
!     kmeshcnew      o    New kmesh part c
!     kmeshnew      i/o   new array kmesh
!     nmap           i    length of first parameter of imap and rmap
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, ielem, ielgrp, inpelm, irelem, &
              ielemnew, npointnew, i, ikelmc, ikelmcnew, &
              istarticross, ilow, ielemtop, iseqkm2, &
              ielembot, isurnr, skipnod(npoint), smth, shifted(npoint), &
              fluidnd(7*npoint), inelem1, inelem2, j, kelmtobst
      integer, allocatable :: iwork(:,:), iworkpoint(:,:)
      double precision eps

!     eps            Local accuracy
!     fluidnd        Array that contains the nodal numbers of the coupling
!                    elements in case of fluid-structure computations
!     i              Counting variable
!     ielem          Element number
!     ielembot       Element sequence number of element with lowest number in
!                    surface isurnr
!     ielemnew       Sequence number of element in new mesh
!     ielemtop       Element sequence number of element with largest number in
!                    surface isurnr
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     ikelmcnew      Starting address of next element in kmeshcnew
!     ilow           help variable to compute smallest element number
!     inelem1        Number of coupling elements in case of fluid-structure
!                    computations
!     inelem2        Help variable for determination of number of elements
!     inpelm         number of points per element
!     iobst          Obstacle sequence number
!     irelem         Relative element sequence number with respect to
!                    element group
!     iseqkm2        Sequence number with respect to km of second mesh
!     istarticross   Starting address of obstacle in array icrossobst
!     isurnr         Surface sequence number
!     iwork          integer work array of length nelem*2
!                    iwork(ielem,1) contains the starting address of
!                    element ielem in array icrossobst
!                    If 0, the element is not present in icrossobst
!                    iwork(ielem,2) contains the corresponding obstacle
!                    sequence number iobst
!     iworkpoint     Work array of length npointnew*2
!                    This array is filled as follows:
!                    iworkpoint(i,1) = 0 if node i outside obstacle
!                    iworkpoint(i,1) = 1 if node i inside obstacle
!                    iworkpoint(i,1) = 2 if node i inside obstacle (near
!                                        boundary)
!                    iworkpoint(i,1) = 3 if node i on obstacle
!                    iworkpoint(i,2) contains the corresponding obstacle
!                    sequence number iobst
!     j              Counting variable
!     kelmtobst      kelmt with respect to kmeshobst
!     npointnew      Sequence number of last generated node
!     shifted        Array of npoint long in which the nodes are marked
!                    which are shifted onto the obstacle
!     skipnod        Array of npoint long in which the nodes are marked
!                    which are not allowed to be moved
!     smth           Parameter for Laplacian smoothing (1), angle
!                    optimization (2), both (3) or none (0)
!                    Is only allowed if shft=1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHADD2DNOD    Subdivide a triangle if necessary into two, three or four
!                    subtriangles and indicate which of these triangles
!                    is in or out the obstacl
!     MSHADD3DNOD    Subdivide a tetrahedron if necessary into subtetareders and
!                    indicate which of these subtetareders is in or out
!                    the obstacle
!     MSHCROBSTELM   Determine line elements for fluid-structure coupling
!     MSHCROBSTELM3D Determine coupling elements for fluid-structure coupling
!                    in 3d
!     MSHOBSTMS021   Clear array iwork (1:nelem) and set iwork(i) = starting
!                    address in iobstcross if element i intersects obstacle
!     MSHOBSTMS022   Copy part of kmesh to kmeshnew n case of obstacles
!     MSHOBSTMS023   Fill iworkpoint with information about nodal points in and
!                    outside obstacle and corresponding obstacle sequence
!                    numbers
!     MSHOBSTMS024   Adapt the arrays ipointobstnew and ielemobstnew for new
!                    mesh
!     MSHSHIFTNODES  Shift node of intersecting curve onto boundary if
!                    node is close (depending on eps) to boundary
!     SEPFINDLOWSUR  Find surface (curve) with lowest non-zero entry in kmesht
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize some parameters
!     Copy part of kmesh to kmeshnew
!     Copy coordinates from coor to coornew
!     Clear array iwork
!     Set iwork to 1 for elements stored in icrossobst
!     Fill iworkpoint with information about nodal points in and outside obst
!     Fill iworkelem with information about elements in and outside obstacle
!     npointnew := npoint
!     ielem := 0
!     ielemnew := 0
!     For ielgrp := 1 (1) nelgrp do
!        inelem := number of elements in group ielgrp
!        inpelm := number of nodes per element in group ielgrp
!        For irelem := 1 (1) inelem do
!           ielem := ielem+1
!           ielemnew := ielemnew+1
!           if ( ielem completely in obstacle or completely outside obstacle )
!              then
!              copy element from kmeshc to kmeshcnew
!           else ( ielem is intersected by obstacle )
!              create new elements
!              copy these elements to kmeshcnew
!              Raise ielemnew
!     adapt ipointobstnew and ielemobstnew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms03' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms03'
         write(irefwr,*) 'ipointobst_old'
         call sepprinipobst ( ipointobst )

      end if
      if ( ierror/=0 ) go to 1000

!     --- extract parameters from iinputusr

      smth = 0
      if ( iinputusr(1)>2 ) smth = iinputusr(3)

!     --- Initialize some parameters

      iobst = 1

      allocate ( iwork(nelem,2), iworkpoint(nmap,2), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*nelem+2*nmap, 'coornew' )
      if ( ierror/=0 ) go to 1000

!     --- Copy part of kmesh to kmeshnew

      call mshobstms022 ( kmeshnew, iseqkm2, coornew )
      inelem2 = 0

!     --- Fill first part of imap

      do i = 1, npoint
         imap(i,1) = 1
         imap(i,2) = 1
         imap(i,3) = 1
      end do

!     --- Clear array iwork and array iedges
!         Set iwork to starting address for elements stored in icrossobst

      call mshobstms021 ( iwork, nelem, numobst, icrossobst )

!     --- clear array ielmfaces

      if ( ndim==3 ) then

!     --- move points to boundary of obstacle
!         clear array ielmfaces

         do i = 1, nfaces
            do j = 1, 4
               ielmfaces(j,i) = 0
            end do
         end do

      end if  ! ( ndim==3 )

!     --- Fill iworkpoint with information about nodal points in and outside
!         obstacle and corresponding obstacle sequence numbers

      call mshobstms023 ( npoint, iworkpoint, nmap, ipointobst, numobst )

!     --- Clear array fluidnd

      do i = 1, 7*npoint
         fluidnd(i) = 0
      end do

!     --- npointnew := npoint
!         ielem := 0
!         ielemnew := 0

      npointnew = npoint
      ielem = 0
      ielemnew = 0
      istarticross = icrossobst(iobst)

!     --- For ielgrp := 1 (1) nelgrp do

      inelem = nelem
      inpelm = npelm
      ikelmc = 0
      ikelmcnew = 0

!     --- Find surface with lowest positive element number
!         This is necessary to fill kmesh part t

      call sepfindlowsur ( nsurfs, ielembot, ielemtop, isurnr )

      skipnod(1:npoint)=0

!     --- In case of an obstacle intersecting a tetrahedral mesh, a
!         inner surface of fluid nodes is found such that it coincides
!         with the obstacle by applying a combination of shifting nodes
!         and adding elements. For a high value of parameter eps
!         more nodes will be shifted.

!         --- parameter eps is the fraction of an edge length (0-0.5)

      eps=3d-1

!     --- shift nodes if the intersection of a fluid edge:
!            <eps*edgelength
!            >(1-eps)*edgelength

      call mshshiftnodes ( iworkpoint, coornew, iwork, &
                           istarticross, skipnod, eps, &
                           imap, smth, fluidnd, inelem1, shifted )
      if ( ierror/=0 ) go to 1000
      ikelmc = 0
      ielem = 0
      ielemnew = 0
      inelem2 = 0

      do ielgrp = 1, nelgrp

!     --- inelem := number of elements in group ielgrp
!         inpelm := number of nodes per element in group ielgrp

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)

         if ( ishape==109.or.ishape==134 ) inelem=inelem1

!        --- For irelem := 1 (1) inelem do

         do irelem = 1, inelem

!        --- ielem := ielem+1
!            ielemnew := ielemnew+1

            ielem = ielem+1
            ielemnew = ielemnew+1
            if ( ielem==ielembot ) then

!           --- Lowest element of surface, refill kmesht

               km(iseqkm2)%kmesht(1,isurnr) = ielemnew

            end if

            if ( ishape==109 ) then

!           --- create coupling elements between fluidmesh and obstacle

               call mshcrobstelm ( kmeshcnew(ikelmcnew+1), fluidnd, &
                                   irelem, coornew )

               km(iseqkm2)%kmeshg(ielgrp)=inelem1
               ikelmcnew = ikelmcnew+inpelm

            else if ( ishape==134 ) then

!           --- create coupling elements between fluidmesh and obstacle for
!               3D obstacles

               call mshcrobstelm3d ( kmeshcnew(ikelmcnew+1), &
                                     fluidnd, irelem, coornew )

               km(iseqkm2)%kmeshg(ielgrp)=inelem1
               ikelmcnew = ikelmcnew+inpelm

            else if ( iwork(ielem,1)==0 .or. &
                    (ishape/=3 .and. ishape/=11 .and. &
                     ishape/=16 .and. ishape/=7) ) then

!           --- if (ishape/=109) then

!               --- ielem completely in obstacle or completely outside obstacle
!                  copy element from kmeshc to kmeshcnew

               do i = 1, inpelm
                  kmeshcnew(ikelmcnew+i) = kmeshc(ikelmc+i)
               end do
               ikelmc = ikelmc+inpelm
               ikelmcnew = ikelmcnew+inpelm

            else

!           --- ielem is intersected by obstacle
!               create new elements
!               copy these elements to kmeshcnew
!               Raise ielemnew

               if ( ishape/=3 .and. ishape/=11 .and. &
                    ishape/=16 .and. ishape/=7 ) then

!              --- shape number not 3 or 11, not yet implemented

                  call errint ( ishape, 1 )
                  call errsub ( 800, 1, 0, 0 )
                  go to 1000

               end if
               if ( ndim==2 ) then

!              --- ndim = 2

                  call mshadd2dnod ( kmeshc(ikelmc+1), &
                                     kmeshcnew(ikelmcnew+1), &
                                     npointnew, ielemnew, &
                                     coornew, iwork, iedges, nmeshc, &
                                     icrossobst, rcrossobst, ielem, &
                                     ikelmcnew, istarticross, inpelm, &
                                     skipnod, &
                                     eps, fluidnd, inelem1, shifted )

               else if ( ndim==3 ) then

!              --- ndim = 3

                  call mshadd3dnod ( kmeshc(ikelmc+1), &
                                     kmeshcnew(ikelmcnew+1), &
                                     npointnew, ielemnew, &
                                     coornew, iwork, iedges, nmeshc, &
                                     icrossobst, rcrossobst, ielem, &
                                     ikelmcnew, istarticross, &
                                     skipnod, &
                                     eps, inpelm, ielmfaces, nmesha, &
                                     nmeshe, fluidnd, inelem1 )

               end if

               ikelmc = ikelmc+inpelm

            end if
            if ( ielem==ielemtop ) then

!           --- Top element of surface, refill kmesht

               km(iseqkm2)%kmesht(2,isurnr) = ielemnew

!              --- Raise ielembot and ielemtop and recompute isurnr

               ilow = nelem
               do i = 1, nsurfs
                  if ( kmesht(1,i)<ilow .and. &
                       kmesht(1,i)>ielemtop ) then

!                 --- new surface found

                     isurnr = i
                     ilow = km(iseqkm2)%kmesht(1,i)

                  end if
               end do
               ielembot = km(iseqkm2)%kmesht(1,isurnr)
               ielemtop = km(iseqkm2)%kmesht(2,isurnr)

            end if

         end do
         if ( kelmg>0 ) then

!        --- Raise kmesh part g

            km(iseqkm2)%kmeshg(ielgrp) = ielemnew-inelem2
            inelem2=ielemnew

         end if

      end do !end loop over element groups

!     --- adapt ipointobstnew and ielemobstnew

      call mshobstms024 ( nmap, iworkpoint, &
                          ipointobstnew, npointnew, kmeshnew, &
                          ielemnew, kmeshcnew, iinput, ielemobstnew )

!     --- Fill iinput

      iinput(1) = ikelmcnew

!     --- Remove temporary arrays

      deallocate ( iworkpoint, iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iworkpoint' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'ipointobst_new'
         call sepprinipobst ( ipointobstnew )
         write(irefwr,*) 'End mshobstms03'

      end if
      call erclos ( 'mshobstms03' )

      end
