      subroutine mshobstms1d ( meshorig, meshobst, maploc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-10-2012 New call to princrossobst
!        version  1.0  date 11-10-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a new mesh from a fixed mesh with an obstacle crossing the mesh
!     Elements that are intersected are subdivided into several elements,
!     such that these new elements all belong to the obstacle or are
!     outside the obstacle
!     1d case with shift
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
!     1d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshorig, meshobst
      type (mapping) :: maploc

!     maploc         o    contains information of the mapping
!     meshobst       i    Sequence number of obstacle mesh to be used
!     meshorig       i    Sequence number of original mesh to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mmielemobst, numbnelmobst, iinput(3), &
              mmicrossobst, numcross, npointnew, iseqkm1, &
              nelemnew, mmrcrossobst, mmipointobst, mmnmeshc
      integer, allocatable :: kmeshcnew(:), ipointobstnew(:), &
                              ielemobstnew(:)
      double precision, allocatable :: coornew(:)

!     coornew        Array containing the new coordinates of the nodes
!     ielemobstnew   New array ielemobst
!     iinput         integer input array
!     ipointobstnew  New array ipointobst
!     iseqkm1        Sequence number with respect to km of first mesh
!     kmeshcnew      New kmesh part c
!     mmicrossobst   memory management sequence number of array icrossobst
!     mmielemobst    memory management sequence number of array ielemobst
!     mmipointobst   memory management sequence number of array ipointobst
!     mmnmeshc       memory management sequence number of array nmeshc
!     mmrcrossobst   memory management sequence number of array rcrossobst
!     nelemnew       Number of elements in new mesh
!     npointnew      Sequence number of last generated node
!     numbnelmobst   Number of elements that are partly within the obstacle
!     numcross       Number of cross sections
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSHOBSTMS01    Create a new mesh from a fixed mesh with an obstacle
!                    crossing the mesh
!     MSHOBSTMS1D01  Create a new mesh from a fixed mesh with an obstacle
!                    crossing the mesh (1d)
!     MSHOBSTMS1D03  Make temporary elements corresponding to new 1d mesh
!                    permanent
!     SEPFILLKMAD3   Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFLIELEMOBST Fill or activate array kmesh ad  part 2: ielemobst
!     SEPFLPOINTOBST Fill array kmesh ad  part 1: ipointobst
!     SEPPRCROSSOBST Print the contents of the arrays icrossobst and rcrossobst
!     SEPPRINIELOBST Print the contents of array ielemobst
!     SEPPRINIPOBST  Print the contents of array ipointobst
!     SEPPRINKMESH   Print array KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3084   Only linear elements have been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms1d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms1d'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )

      if ( npelm>2 ) then

!     --- Error: at this moment only linear elements have been implemented

         call errsub ( 3084, 0, 0, 0 )

      end if  ! ( npelm>2 )
      if ( ierror/=0 ) go to 1000

      iseqkm1 = iseqkm

      call sepflpointobst
      call sepflielemobst
      if ( kmeshad(3)==0 .or. kmeshad(5)/=kmeshad(8) ) call sepfillkmad3
      if ( debug .and. nmesh(33)==0 ) call sepfillnmeshc

!     --- Compute some parameters

      call mshobstms01 ( ielemobst, numobst, numbnelmobst, &
                         icrossobst, numcross )
      nelemnew = nelem+numcross
      npointnew = npoint+numcross
      if ( debug ) &
         write(irefwr,1) 'numcross, nelemnew, npointnew', &
                          numcross, nelemnew, npointnew

!     --- allocate space for temporary arrays

      allocate ( kmeshcnew(nelemnew*2), coornew(npointnew), &
                 ipointobstnew(4*numobst+npointnew), &
                 ielemobstnew(6*numobst+nelemnew), &
                 imap(npointnew,3), rmap(npointnew,2), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 2*nelemnew+7*npointnew+10*numobst, &
                        'kmeshcnew' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call sepprcrossobst

!     --- Fill new mesh

      call mshobstms1d01 ( kmeshcnew, coornew, &
                           meshobst, ipointobstnew, ielemobstnew, &
                           iinput, npointnew )

!     --- Make new arrays permanent

      kmesh => kmesh2d(meshobst)
      call sepfillkmesh ( kmesh )

      call mshobstms1d03 ( coornew, kmeshcnew, npointnew, &
                           iinput, maploc, ipointobstnew, ielemobstnew )

      maploc%iseqkm1 = iseqkm1
      maploc%iseqkm2 = iseqkm

      if ( debug ) then

!     --- Debug information

         call sepprinipobst ( ipointobst )
         call sepprinielobst ( ielemobst )

      end if  ! ( debug )

      deallocate ( kmeshcnew, coornew, ipointobstnew, ielemobstnew, &
                   imap, rmap, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshcnew' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'New mesh'
         call sepprinkmesh ( 1 )
         call sepprinkmesh ( 6 )
         call sepprinkmesh ( 38 )
         write(irefwr,*) 'End mshobstms1d'

      end if  ! ( debug )
      call erclos ( 'mshobstms1d' )

      end
