      subroutine mshobstms3dq6c ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew, centroid, &
                                  midfacenodes, facenode, fluidnd )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 08-06-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a tetrahedron, with three crossed edges which form a curve
!     that is not closed, into four or five tetrahedrons. Quadratic case.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnode(15),tetnodenew(15,*), edgenode(5,6), isort(4), &
              ledgenode(2,3), nelemnew, midfacenodes(4), centroid(*), &
              facenode(4,4), fluidnd(*)

!     centroid       i    If true the function is available in the centroid
!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     facenode       i    ?
!     fluidnd        i    Array containing the fluidnodes that lie on the
!                         obstacle
!     isort          i    if isort(i)>isort(j) then tetnode(i) is at least on
!                         the same number of crossed edges as tetnode(j)
!     ledgenode      i    for i=1,2 and 1<j< number of crossed edges
!                         ledgenode(i,j) is the local tetnode of edgenode(i,j), &
!                         in other words:
!                         edgnode(i,j) = tetnode(ledgenode(i,j))
!     midfacenodes   i    ?
!     nelemnew       o    Number of elements in new mesh
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, idx(6), transtot(15,12), itran, trans(4), transi(4), &
              transtemp, transposj, tetnodet(15), facetrans(4), &
              edgenodet(2,6), facenodet(4,4), idx1(2)
      integer pedgenodet(4,4)
      logical oddpairintchan, edgenodetpos2, flip(4), debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     edgenodetpos2  one of the transformed edge nodes is at local position 2
!     facenodet      ?
!     facetrans      ?
!     flip           ?
!     i              Counting variable
!     idx            Array containing the indices of the non-vertices
!     idx1           Array for renumbering of the linear basis functions
!     itran          ?
!     j              Counting variable
!     oddpairintchan if true the transformation is an odd number of
!                    pair interchanges
!     pedgenodet     the local edge number = local edgenode number
!                    for the edge [local tetnode i, local tetnode j]
!     tetnodet       Array of global nodal point numbers of the corners of
!                    the transformed original tetrahedron
!     trans          Transformation array. trans(i) is the transformed
!                    local position of the i-th corner of the original
!                    tetrahedron.
!     transi         Inverse of trans
!     transposj      The transformed local edge node number of edge node j
!     transtemp      temporary storage
!     transtot       ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMS3DQ6D Subdivide a tetrahedron with four crossed edges which form
!                    a closed curve
!     MSHOBSTMS3DQ6E Subdivide a tetrahedron with four crossed edges which form
!                    a closed curve
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Define transformation to move node isort(3) to node 1
!                               and node isort(4) to node 2.
!
!     Transform tetrahedron tetnode to tetnodet and the position of the
!     intersection points edgenode(3,.) to edgenodet(.)
!
!     If intersection points at positions 1,3,5 transform tetrahedron such
!     that they get into positions 1,2,4
!
!     If node number of intersection point 4 < node number of i.p. 2, &
!     interchange these 2
!
!     Split tetrahedron into four or five subtetrahedrons, taking into account
!     the determinant of the transformation and the node numbers of the
!     intersection points.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data pedgenodet /0,1,3,4, &
                       1,0,2,5, &
                       3,2,0,6, &
                       4,5,6,0/
      data transtot / 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, &
                      3, 4, 5, 6, 1, 2, 8, 9, 7, 10, 11, 13, 14, 12, 15, &
                      5, 6, 1, 2, 3, 4, 9, 7, 8, 10, 11, 14, 12, 13, 15, &
                      1, 7, 10, 8, 3, 2, 6, 9, 4, 5, 12, 14, 13, 11, 15, &
                      3, 2, 1, 7, 10, 8, 4, 6, 9, 5, 12, 11, 14, 13, 15, &
                      10, 8, 3, 2, 1, 7, 9, 4, 6, 5, 12, 13, 11, 14, 15, &
                      1, 6, 5, 9, 10, 7, 2, 4, 8, 3, 14, 11, 13, 12, 15, &
                      5, 9, 10, 7, 1, 6, 4, 8, 2, 3, 14, 13, 12, 11, 15, &
                      10, 7, 1, 6, 5, 9, 8, 2, 4, 3, 14, 12, 11, 13, 15, &
                      3, 8, 10, 9, 5, 4, 2, 7, 6, 1, 13, 12, 14, 11, 15, &
                      5, 4, 3, 8, 10, 9, 6, 2, 7, 1, 13, 11, 12, 14, 15, &
                      10, 9, 5, 4, 3, 8, 7, 6, 2, 1, 13, 14, 11, 12, 15/
      data idx / 1,3,1,1,3,10 /
      data idx1 / 2,1 /
! ======================================================================
!
      call eropen ( 'mshobstms3dq6c' )
      debug = .false.

!     --- Initialize

      oddpairintchan = .false.
      do i = 1,4
         trans(i) = i
      end do

!     --- move node isort(3) to node 1

      if( isort(3).ne.trans(1) ) then
         transtemp = trans(1)
         trans(1) = isort(3)
         trans(isort(3)) = transtemp
         oddpairintchan = .not.oddpairintchan
      end if

!     --- move node isort(4) to 2

      if (isort(4).ne.trans(2) ) then
         transtemp = trans(2)
         trans(2) = isort(4)
         trans(trans(isort(4))) = transtemp
         oddpairintchan = .not.oddpairintchan
      end if

      if( oddpairintchan ) then
         transtemp = trans(3)
         trans(3) = trans(4)
         trans(4) = transtemp
      end if

!     --- transi := inverse of trans

      do i = 1,4
         transi(trans(i)) = i
      end do

!     --- Transform tetrahedron tetnode to tetnodet

      if (trans(4).eq.4) then
         if (trans(1).eq.1) itran=1
         if (trans(1).eq.2) itran=2
         if (trans(1).eq.3) itran=3
      else if (trans(4).eq.3) then
         if (trans(1).eq.1) itran=4
         if (trans(1).eq.2) itran=5
         if (trans(1).eq.4) itran=6
      else if (trans(4).eq.2) then
         if (trans(1).eq.1) itran=7
         if (trans(1).eq.3) itran=8
         if (trans(1).eq.4) itran=9
      else if (trans(4).eq.1) then
         if (trans(1).eq.2) itran=10
         if (trans(1).eq.3) itran=11
         if (trans(1).eq.4) itran=12
      end if

      do i = 1,15
         tetnodet(i) = tetnode(transtot(i,itran))
      end do
      if (debug) then
         call prinrl ( tetnodet, 15, 'tetnodet' )
         call prinrl ( tetnode, 15, 'tetnode' )
      end if

!     --- Transform the intersection points to their transformed positions

      do i=1,4
        flip(i)=.false.
      end do

      edgenodetpos2 = .false.
      do j = 1,3

         transposj = pedgenodet(transi(ledgenode(1,j)), &
                                transi(ledgenode(2,j)))

         if (transposj.eq.2) edgenodetpos2 = .true.

         if (edgenode(1,j).eq.tetnodet(idx(transposj))) then
            edgenodet(1,transposj) = edgenode(3,j)
            edgenodet(2,transposj) = edgenode(5,j)
         else
            edgenodet(1,transposj) = edgenode(5,j)
            edgenodet(2,transposj) = edgenode(3,j)
            if (transposj.eq.2.or.transposj.eq.5) flip(3)=.true.
            if (transposj.eq.3.or.transposj.eq.4) flip(4)=.true.
         end if

      end do

      do j=1,4
         do i=1,4
            if (midfacenodes(i).eq.tetnodet(10+j)) facetrans(j)=i
         end do
      end do

!     --- Transform faces

      do j=1,4
         do i=1,2
            if (.not.flip(j)) facenodet(i,j)=facenode(i,facetrans(j))
            if (flip(j)) facenodet(i,j)=facenode(idx1(i),facetrans(j))
         end do
         if (j.eq.1.or.j.eq.2) then
            facenodet(3,j)=facenode(3,facetrans(j))
            facenodet(4,j)=facenode(4,facetrans(j))
         end if
      end do

      if ( .not.edgenodetpos2 ) then

!     --- intersection points on edges 1, 3 and 5

         call mshobstms3dq6d ( tetnodet, tetnodenew, facenodet, &
                                centroid,  edgenodet, nelemnew )

         if (nelemnew.eq.4) then

            write(*,*)'geen extra koppelingselement'

         else if (nelemnew.eq.5) then

            fluidnd(1)= tetnodet(6)
            fluidnd(2)= tetnodet(11)
            fluidnd(3)= tetnodet(2)
            fluidnd(4)= tetnodet(12)
            fluidnd(5)= tetnodet(8)
            fluidnd(6)= tetnodet(15)
            fluidnd(7)= centroid(8)

         end if

      else if (edgenodetpos2) then

!     --- intersection points on edges 1, 2 and 4

         call mshobstms3dq6e ( tetnodet, tetnodenew, facenodet, &
                               centroid,  edgenodet, nelemnew )

         if (nelemnew.eq.4) then

            write(*,*)'geen extra koppelingselement'

         else if (nelemnew.eq.5) then

            fluidnd(1)= tetnodet(2)
            fluidnd(2)= tetnodet(11)
            fluidnd(3)= tetnodet(4)
            fluidnd(4)= tetnodet(15)
            fluidnd(5)= tetnodet(7)
            fluidnd(6)= tetnodet(12)
            fluidnd(7)= centroid(8)

         end if

      end if

      call erclos ( 'mshobstms3dq6c' )

      end
