      subroutine mshoutercurvs1 ( istore, nouter, ncurvs, ioutercurvs, &
                                  lenoutercurvs, npoint, kmeshm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Organize the curves stored in array istore such that connected
!     curves are stored in parts
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     boundary
!     outer_boundary
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istore(*), nouter, ncurvs, ioutercurvs(*), lenoutercurvs, &
              npoint, kmeshm(*)

!     ioutercurvs    o    Contains information about the curves in the outer
!                         boundary (3D) in the following way
!                         Pos. 1  nparts  Number of parts the outer boundary
!                                         consists of
!                         Pos. 2-1+nparts Starting addresses of each part
!                         Per part: (first address: ioutercurvs(1+i)
!                         Pos. 1: nouter  Number of curves in part
!                         followed by the curve numbers (provided with a sign)
!                         The part itself is closed
!     istore         i    Contains the set of outer curves
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     lenoutercurvs  o    Length of array ioutercurvs
!     ncurvs         i    number of curves
!     nouter         i    Number of outer curves
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(ncurvs), icopy(nouter), i, nodes(npoint), nleft, &
              nparts, icount, icurnr, j, istar3, ipstart, numcurvs, &
              numstore(nouter), k1, k2
      logical found

!     found          If true already a possibility has been found
!     i              Counting variable
!     icopy          Copy of array istore ordened for the final sequence
!     icount         Counter
!     icurnr         curve sequence number
!     ipstart        Starting address in array
!     istar3         First position used at a part
!     iwork          integer work array of length ncurvs
!                    if iwork(i) = 0 curve i is not longer in the
!                    list of free outer curves
!     j              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     nleft          Number of free curves left
!     nodes          Array of length npoint to mark all nodes in a curve
!                    or a set of curves
!     nparts         number of parts the boundary consists of
!     numcurvs       number of curves
!     numstore       work array to store the number of curves per part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHSORTCURV    Sort closed set of curves, such that curves are stored
!                    sequentially
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork (1:ncurvs)
!     Clear array nodes ( 1:npoint)
!     Set iwork(i) # 0, if curve i in istore
!     nleft := nouter
!     nparts := 0; icount := 0
!     while ( nleft>0 ) do
!        icurnr := first curve with iwork(icurnr) # 0
!        nparts := nparts+1;  numcurvs := 1; icount := icount+1
!        icopy(icount) := iwork(icurnr); iwork(icurnr) := 0; nleft := nleft-1
!        Set nodes(i) = 1 for all nodes in curve
!        icurnr := 1
!        while ( nleft > 0 and icurnr > ncurvs ) do
!           Find curve icurnr with iwork(icurnr) > 0
!           if ( at least one node of icurnr has nodes(node)=1 ) then
!              Set nodes(i) = 1 for all nodes in curve icurnr
!              numcurvs := numcurvs+1; icount := icount+1
!              icopy(icount) := iwork(icurnr); iwork(icurnr) := 0;
!              nleft := nleft-1
!        numstore(nparts) := numcurvs
!        if ( nleft>0 ) then
!           Clear array nodes
!     ioutercurvs(1) := nparts; ipstart := nparts+2; icount:= 0
!     for i := 1 (1) nparts do
!        numcurvs := numstore(i); ioutercurvs(i+1) := ipstart;
!        ioutercurvs(ipstart) := numcurvs
!        for j := 1 (1) numcurvs do
!           ioutercurvs(ipstart+j) := icopy(icount+j)
!        icount := icount+numcurvs; ipstart := ipstart+numcurvs+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshoutercurvs1' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshoutercurvs1'
         call prinin ( istore, nouter, 'istore' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Clear array iwork (1:ncurvs)

      do i = 1, ncurvs
         iwork(i) = 0
      end do

!     --- Clear array nodes ( 1:npoint)

      do i = 1, npoint
         nodes(i) = 0
      end do

!     --- Set iwork(i) # 0, if curve i in istore

      do i = 1, nouter
         iwork(abs(istore(i))) = istore(i)   ! Use istore to get the correct
                                             ! sign
      end do
      if ( debug ) call prinin ( iwork, ncurvs, 'iwork' )

!     --- nleft := nouter
!         nparts := 0; icount := 0

      nleft = nouter
      nparts = 0  ! number of parts
      icount = 0  ! number of curves in icopy
      istar3 = kmeshm(2)-1 ! starting address of kmeshm part 3 minus 1

!     --- while ( nleft>0 ) do

      do while ( nleft.gt.0 )

!     --- icurnr := first curve with iwork(icurnr) = 1

         icurnr = 1
         do while ( iwork(icurnr).eq.0 )
            icurnr = icurnr+1
         end do
         if ( debug ) write(irefwr,*) 'icurnr, nleft ', icurnr, nleft

!        --- nparts := nparts+1;  numcurvs := 1; icount := icount+1
!            icopy(icount) := iwork(icurnr); iwork(icurnr) := 0
!            nleft := nleft-1

         nparts = nparts+1
         numcurvs = 1                   ! number of curves for this part
         icount = icount+1              ! next position in icopy
         icopy(icount) = iwork(icurnr)  ! curve number with sign
         iwork(icurnr) = 0
         nleft = nleft-1
         if ( debug ) write(irefwr,*) 'nparts, icount, nleft ', &
            nparts, icount, nleft

!        --- Set nodes(i) = 1 for all nodes in curve

         k1 = kmeshm(4+icurnr)+istar3
         k2 = kmeshm(5+icurnr)-1+istar3
         do i = k1, k2
            nodes(kmeshm(i)) = 1
         end do

         icurnr = 1

!        --- while ( nleft > 0 and icurnr > ncurvs ) do

         do while ( nleft.gt.0 .and. icurnr.le.ncurvs )

!        --- nleft > 0 and icurnr > ncurvs
!            Find curve icurnr with iwork(icurnr) > 0

            do while ( iwork(icurnr).eq.0 .and. icurnr.le.ncurvs )
               icurnr = icurnr+1
            end do
            if ( icurnr.gt.ncurvs ) go to 200
            if ( debug ) write(irefwr,*) 'icurnr, nleft ', icurnr, nleft

            found = .false.
            if ( iwork(icurnr).ne.0 ) then

!           --- Only curves that have been filled may be used

               k1 = kmeshm(4+icurnr)+istar3
               k2 = kmeshm(5+icurnr)-1+istar3
               do i = k1, k2
                  if ( nodes(kmeshm(i)).gt.0 ) then

!                 --- Node with nodes(inode) = 1 found

                     found = .true.
                     go to 100

                  end if

               end do

            end if  ! ( kmeshm(istar3+1).gt.0 )

100         if ( found ) then

!           --- at least one node of icurnr has nodes(node)=1
!               Set nodes(i) = 1 for all nodes in curve icurnr

               do i = k1, k2
                  nodes(kmeshm(i)) = 1
               end do

!              --- numcurvs := numcurvs+1; icount := icount+1
!                  icopy(icount) := iwork(icurnr); iwork(icurnr) := 0;
!                  nleft := nleft-1

               numcurvs = numcurvs+1
               icount = icount+1
               icopy(icount) = iwork(icurnr)
               iwork(icurnr) = 0
               nleft = nleft-1
               if ( debug ) &
                  write(irefwr,*) 'numcurvs, icurnr, icount, nleft ', &
                                   numcurvs, icurnr, icount, nleft
               icurnr = 0

            end if  ! ( found )
            icurnr = icurnr+1
            if ( debug ) write(irefwr,*) 'found, icurnr ', found, icurnr

200      end do  ! while ( nleft.gt.0 .and. icurnr.le.ncurvs )

!        --- numstore(nparts) := numcurvs

         numstore(nparts) = numcurvs

         if ( nleft.gt.0 ) then

!        --- nleft>0
!            Clear array nodes

            do j = 0, numcurvs-1
               icurnr = abs(icopy(icount-j))
               k1 = kmeshm(4+icurnr)+istar3
               k2 = kmeshm(5+icurnr)-1+istar3
               do i = k1, k2
                  nodes(kmeshm(i)) = 0
               end do
            end do  ! i = 0, numcurvs-1

         end if  ! ( nleft.gt.0 )

      end do  ! while ( nleft.gt.0 )

      if ( debug ) then
         write(irefwr,*) 'nparts, icount ', nparts, icount
         call prinin ( icopy, icount, 'icopy' )
         call prinin ( numstore, nparts, 'numstore' )
      end if

!     --- Sort curves in icopy

      call mshsortcurv ( icopy, kmeshm, icount )

!     --- ioutercurvs(1) := nparts; ipstart := nparts+2; icount:= 0

      ioutercurvs(1) = nparts
      ipstart = nparts+2
      icount= 0

!     --- for i := 1 (1) nparts do

      do i = 1, nparts

!     --- numcurvs := numstore(i); ioutercurvs(i+1) := ipstart;
!         ioutercurvs(ipstart) := numcurvs

         numcurvs = numstore(i)
         ioutercurvs(i+1) = ipstart
         ioutercurvs(ipstart) = numcurvs

!        --- for j := 1 (1) numcurvs do
!               ioutercurvs(ipstart+j) := icopy(icount+j)

         do j = 1, numcurvs
            ioutercurvs(ipstart+j) = icopy(icount+j)
         end do  ! j = 1, numcurvs

!        --- icount := icount+numcurvs; ipstart := ipstart+numcurvs+1

         icount = icount+numcurvs
         ipstart = ipstart+numcurvs+1

      end do  ! i = 1, nparts

      lenoutercurvs = ipstart-1

1000  call erclos ( 'mshoutercurvs1' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshoutercurvs1'

      end if

      end
