      subroutine mshpavbnquad1 ( iedge1, iedge4, inode1, inode2, &
                                 nmeshc, nmeshl, iwork, nmeshh, &
                                 nmeshd, markedges, iworkelem, &
                                 iworkedge, nquads, iworkquads, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-03-2009 Jump in case quadrilateral does not exist
!        version  1.0  date 07-07-2004
!
!   copyright (c) 2004-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find quadrilateral with all 4 nodes on the boundary
!     The first boundary edge iedge1 is given with nodes inode1 and inode2
!     The boundary connecting edge iedge4 is also given
!     From inode2 there are more than one edges connecting the opposite boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     node
!     boundary
!     quadrilateral
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iedge1, iedge4, inode1, inode2, nmeshc(*), nmeshl(*), &
              iwork(*), nmeshh(*), nmeshd(*), markedges(*), nquads, &
              iworkelem(*), iworkedge(*), iworkquads(2,*)
      double precision coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iedge1         i    edge sequence number
!     iedge4         i    edge sequence number
!     inode1         i    First node in edge
!     inode2         i    Second node in edge
!     iwork         i/o   integer work array of length nedges
!                         At start iwork must be set to zero
!                         After each call iwork is reset to 0
!     iworkedge     i/o   work array of length nedges to be used to mark
!                         edges
!                          0:  not marked yet
!                         -1:  boundary edge that has been used already
!                         -2:  diagonal of quadrilateral that may be removed
!     iworkelem     i/o   Work array of length nelem to be used to mark
!                         elements
!                         0:  element is not special
!                         1:  element is triangle with all nodes on the boundary
!                         2:  elements is triangle, that is part of a
!                             quadrilateral with all 4 points on the boundary
!     iworkquads    i/o   work array of size nelem, containing sets of
!                         triangles that may be considered as quads
!                         (1,.) first triangle
!                         (2,.) second triangle
!     markedges      i    array of length nedges indicating whether a
!                         edge is internal or on the boundary
!                         -2:  internal edge between 2 boundary nodes
!                         -1:  boundary edge
!                          0:  standard internal edge
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshd         i    nmesh part d, contains edge numbers of elements
!     nmeshh         i    nmesh part h, contains element numbers corresponding
!                         to edges
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to nodes
!     nquads        i/o   number of quads stored in iworkquads
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode4, iedge5, itrian1, itrian2, iedge3, inode3, ipstart
      logical acceptquad

!     acceptquad     Check if quadrilateral is acceptable (true) or not
!                    (false)
!     iedge3         edge sequence number
!     iedge5         edge sequence number
!     inode3         first node in second edge
!     inode4         Second node in second edge
!     ipstart        Starting address in array
!     itrian1        Sequence number of first triangle
!     itrian2        Sequence number of second triangle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniedge

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIEDGE        Returns with edge number corresponding to 2 nodes
!     ININODONEDGE   Find other end node on edge if one of the end nodes and
!                    the ege number are given
!     MSHPAVBNQUAD3  Find triangle corresponding to two edges as well as
!                    opposite triangle corresponding to "diagonal" edge
!     MSHPAVCHKQUAD  Check if quadrilateral to be subdivided is acceptable
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The situation is as follows
!
!      4   e3 3
!     -x------x- &
!      |\     |\
!      | \    |
!   e4 |  \e5 |e2
!      |    \ |/
!      x------x- &
!      1  e1  2
!
!     inode4 := end point of edge4
!     iedge5 := edge between nodes 2 and 4
!     itrian1 := triangle spanned by e1, e4 and e5
!     itrian2 := other triangle connected to edge 5
!     e3 := boundary edge in itrian2
!     Mark e3, e5 and itrian1/2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavbnquad1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         iedge3 = 0
         itrian1 = 0
         itrian2 = 0
         write(irefwr,*) 'Debug information from mshpavbnquad1'
         write(irefwr,1) 'iedge1, iedge4, inode1, inode2', &
                          iedge1, iedge4, inode1, inode2
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- inode4 := end point of edge4

      call ininodonedge ( iedge4, inode1, inode4, nmeshc )
      if ( ierror/=0 ) go to 1000

!     --- iedge5 := edge between nodes 2 and 4

      iedge5 = iniedge ( inode2, inode4, nmeshl, iwork )
      if ( iedge5==0 .or. ierror/=0 ) go to 1000

!     --- itrian1 := triangle spanned by e1, e4 and e5
!         itrian2 := other triangle connected to edge 5
!         e3 := boundary edge in itrian2

      call mshpavbnquad3 ( itrian1, itrian2, iedge1, iedge5, &
                           iedge3, nmeshh, nmeshd, markedges )
      if ( ierror/=0 .or. iedge3<=0 ) go to 1000

!     --- Check if quadrilateral to be subdivided is acceptable

      if ( iworkedge(iedge3)==0 ) then

!     --- iworkedge(iedge3) = 0, so edge3 has not been treated yet
!         Check if quadrilateral to be subdivided is acceptable

         ipstart = nmeshc(iedge3)
         inode3 = nmeshc(ipstart+1)
         if ( inode3==inode4 ) inode3 = nmeshc(ipstart+2)
         call mshpavchkquad ( inode1, inode2, inode3, inode4, &
                              acceptquad, coor )
         if ( acceptquad ) then

!        --- Quad is acceptable, mark e3, e5 and itrian1/2

            if ( debug ) &
               write(irefwr,1) 'iedge3, iedge5, itrian1, itrian2', &
                                iedge3, iedge5, itrian1, itrian2
            iworkedge(iedge3) = -1
            iworkedge(iedge5) = -2

            iworkelem(itrian1) = 2
            iworkelem(itrian2) = 2
            nquads = nquads+1
            iworkquads(1,nquads) = itrian1
            iworkquads(2,nquads) = itrian2

         end if  ! ( acceptquad ) then

      end if  ! ( iworkedge(iedge3)==0 )

1000  call erclos ( 'mshpavbnquad1' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'inode4, iedge3, itrian1, itrian2 ', &
                          inode4, iedge3, itrian1, itrian2
         write(irefwr,*) 'End mshpavbnquad1'

      end if

      end
