      subroutine mshplaxis30 ( iv, nnodes, cord, ski, eta, zta, erbase )
! ======================================================================
!
!        programmer    Paul Bonnier, Niek Praagman
!        version  1.1  date 29-08-2008 Forcheck adjustments
!        version  1.0  date 27-05-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Improve Delaunaymesh in projectionplane and next go back to the
!     original space.
!     Run through all elements and check length of sides.
!     If ratio of sides too large or ratio of triangle too small
!     consider adding nodes if allowed.
!     Check all elements and adjust for the new node.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, iv
      double precision cord(3,*), ski, eta, zta, erbase

!     cord           i    array of length 3 x npoint containing the co-ordinates
!                         of the nodal points with respect to the global
!                         numbering
!     erbase         o    sum of suared distances to propsed plane
!     eta            o    coefficient y in surface equation
!     iv             i    indicator which basisdirection
!     nnodes         i    number of nodes in usergiven rough mesh
!     ski            o    coefficient x in surface equation
!     zta            o    coefficient z in surface equation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer      i
      double precision  a11, a12, a13, a22, a23, a33, a14, a24, a34, &
                        xi, yi, zi, det, ak, dist

!     a11            matrix and rhs element (symmetric)
!     a12            matrix and rhs element (symmetric)
!     a13            matrix and rhs element (symmetric)
!     a14            matrix and rhs element (symmetric)
!     a22            matrix and rhs element (symmetric)
!     a23            matrix and rhs element (symmetric)
!     a24            matrix and rhs element (symmetric)
!     a33            matrix and rhs element (symmetric)
!     a34            matrix and rhs element (symmetric)
!     ak             normation factor
!     det            determinant of matrix
!     dist           distance of node to plane
!     i              Counting variable
!     xi             x-coordinate submitted node
!     yi             y-coordinate submitted node
!     zi             z-coordinate submitted node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSH636         Compute determinant of matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2983   Wrong call to mshplaxis30
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill system of equations for the determination of the planevariables
!     of the kkm plane.
!     Compute the sum of the distances of all nodes to the proposed
!     plane.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshplaxis30' )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshplaxis30'
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize coefficients:

      a11 = 0
      a12 = 0
      a13 = 0
      a22 = 0
      a23 = 0
      a33 = 0

      a14 = 0
      a24 = 0
      a34 = 0

!     --- Check value for direction:

      if ( iv<1 .or. iv>3 ) then
         call errsub ( 2983, 0, 0, 0 )
         go to 1000
      end if

!     --- Fill matrix using all submitted nodes:

      do i=1,nnodes

         if ( iv==1 ) then
            xi = cord(1,i)
            yi = cord(2,i)
            zi = cord(3,i)
         else if ( iv==2 ) then
            xi = cord(2,i)
            yi = cord(3,i)
            zi = cord(1,i)
         else if ( iv==3 ) then
            xi = cord(3,i)
            yi = cord(1,i)
            zi = cord(2,i)
         end if

         a11 = a11 +  1D0
         a12 = a12 + xi
         a13 = a13 + yi
         a22 = a22 + xi*xi
         a23 = a23 + xi*yi
         a33 = a33 + yi*yi

         a14 = a14 + zi
         a24 = a24 + xi*zi
         a34 = a34 + yi*zi

      end do

!     --- Compute determinant of system:

      call msh636( a11, a12, a13, a12, a22, a23, a13, a23, a33, det )

!     --- Check value and if needed jump out of routine:

      if ( abs(det)<1d-10 ) then

!     --- Zero determinant: jump out of routine
!         selected vector in base-plane?
!         set large distances and leave routine

          erbase = 1d10
          goto 1000
      end if

!     --- Compute solution (ski,eta,zta) using the rhs terms:

      call msh636( a14, a12, a13, a24, a22, a23, a34, a23, a33, ski )
      ski = ski / det

      call msh636( a11, a14, a13, a12, a24, a23, a13, a34, a33, eta )
      eta = eta / det

      call msh636( a11, a12, a14, a12, a22, a24, a13, a23, a34, zta )
      zta = zta / det

!     --- Compute sum of distances to proposed plane and store
!         value in erbase:

      erbase = 0

      ak = 1d0 / ( eta*eta + zta*zta + 1D0 )

      do i=1,nnodes

         if ( iv==1 ) then

            xi = cord(1,i)
            yi = cord(2,i)
            zi = cord(3,i)

         else if ( iv==2 ) then

            xi = cord(2,i)
            yi = cord(3,i)
            zi = cord(1,i)

         else

            xi = cord(3,i)
            yi = cord(1,i)
            zi = cord(2,i)

         end if

         dist   = ski    + eta * xi  + zta * yi - zi
         erbase = erbase + ak  * dist * dist

      end do

1000  call erclos ( 'mshplaxis30' )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End debugging mshplaxis30'
      end if  ! ( debug )

      end
