      subroutine mshplaxsurf ( bcord, kbndpt, holeinfo, nholes, &
                               iinput, rinput, ncoarlc, doublecurvs, &
                               numcurvboun, boundary, irefplax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 20-06-2013 Extra parameter irefplax
!        version  3.1  date 02-11-2010 Dynamical allocation workarrays
!        version  3.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create surface mesh using surface generator framesurf
!     Allow extra internal coarseness points
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irefplax, kbndpt(*), holeinfo(2,0:*), &
              nholes, iinput(*), ncoarlc, &
              numcurvboun, boundary(2,numcurvboun)
      double precision bcord(*), rinput(*)
      logical doublecurvs

!     bcord          i    array containing the coordinates of the boundaries
!     boundary       i    Array of size 2 x numcurvboun containing
!                         information of the boundary of the surface
!                         (1,i) contains the i-th curve number (including sign)
!                         (2,i) contains the starting address in kbndpt
!     doublecurvs    i    If true the boundary contains double curves
!     holeinfo       o    Integer array of size 2 x (nholes+2)
!                         Contains information about holes in surfaces and where
!                         a new part of the boundary starts
!                         Pos (1,i-1) contains the local sequence number of the
!                            first curve of part i provided with a sign
!                            If the sign is positive the boundary is
!                            created counter clockwise, otherwise clockwise
!                         Pos (2,i-1) contains the local sequence number of the
!                            first node of of part i
!                         Pos (1,nholes+1) contains the number of curves of the
!                         boundary
!     iinput         i    integer input array, see subroutine mesh
!     irefplax       i    File reference number for plaxis
!     kbndpt         o    array containing the nodal point numbers of the
!                         boundary
!     ncoarlc        i    Number of extra user points
!     nholes         o    Number of holes in the boundary of a surface
!     numcurvboun    i    Number of curves in the boundary
!     rinput         i    Real input array, see subroutine MESHBODY
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npoh, nelh, iposiinp, &
              iposiinpcrv, i, nextrnods
      integer, allocatable :: itria(:,:)
      double precision, allocatable :: cord(:,:)
      double precision, allocatable :: coorusp(:,:)

!     coorusp        Work array of size 4 x nuspextra to store the coordinates
!                    of the transformed extra user points and coarsenesses
!     cord           Work array to store frame coordinates of nodes for MSHOCE
!     i              Loop variable
!     iposiinp       Starting address in array iinput with respect to the
!                    extra user points (internal points)
!     iposiinpcrv    Starting address in array iinput with respect to the
!                    extra user curves (internal curves)
!     itria          int work array to store the frame triangle array for MSHOCE
!     nelh           Estimate of the number of elements
!     nextrnods      Summation of nodes on each extra (internal) curve
!     npoh           Estimate of the number of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORMSHSUR1   Compare estimated size of npointlc and nelemlc with actual size
!     MSHDOUBLECURVS Map nodes of double curves onto each other
!     MSHPLAXIS      Make a mesh of triangles in a 3D surface for an area
!                    which is given by the line elements of BCORD. A description
!                    of the surface is given by a rough triangulation as given
!                    by the points and triangles in the arrays itria and cord
!     MSHPLAXREAD1   Read set of triangles that defines the frame for the
!                    submesh generator framesurf
!     MSHSRFFRAMECP  Copy coordinates of user points from rinput into array
!                    coorusp  Special case for framesurf
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshplaxsurf' )
      debug = .false. .and. ioutp>=0

      allocate( coorusp(4,max(1,ncoarlc)), stat = error)
      if ( error/=0 ) call eralloc ( error, 4*ncoarlc, 'coorusp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshplaxsurf'
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get constants

      iposiinp   = max(1,ncoarstart)  ! address in iinput for userpoints
      iposiinpcrv= max(1,ncurvstart)  ! address in iinput for curve numbers

      npoh = npointlc
      nelh = nelemlc

      if ( ncoarlc>0 ) then

!     --- ncoarlc>0, so there are extra points, fill coordinates in coorusp

         call mshsrfframecp ( rinput(irnp1), coorusp, iinput(iposiinp), &
                              ncoarlc, rinput(irnp1+ndim*nuspnt), &
                              jcoars, nuspnt )
      end if  ! ( ncoarlc>0 )

!     --- Allocate space for helparrays cord and itria:

      allocate( cord(3,nnodes), stat = error )
      if ( error/=0 ) call eralloc ( error, 3 * nnodes, 'cord ' )
      allocate( itria(3,ntria), stat = error )
      if ( error/=0 ) call eralloc ( error, 3 * ntria , 'itria' )

      call mshplaxread1 ( nnodes, ntria, cord, itria, isurnr, irefplax )

!     --- Determine number of exta coordinates in bcord:

      nextrnods = 0

!     --- Loop over all extra curves

      do i=1, nextracrvs
         nextrnods = nextrnods + kbndpt(lnkbnd+i)
      end do

      call mshplaxis ( isurnr, coor2d, kmeshclc, inpelm, nbndpt, kbndpt, &
                       bcord, npointlc, nelemlc, boundary, numcurvboun, &
                       ntria, itria, cord, nnodes, holeinfo, nholes, &
                       ncoarlc, coorusp, iinput(iposiinp), nextracrvs, &
                       kbndpt(lnkbnd+1), iinput(iposiinpcrv), nextrnods, &
                       rinput, ndim, nuspnt )

      if ( doublecurvs ) then

!     --- doublecurvs true, hence there are double curves in the boundary
!         To avoid problems with the volume generator, we map these points
!          on the same node numbers and reduce npointlc

         call mshdoublecurvs ( kmeshclc, nelemlc, inpelm, coor2d, 3, &
                               npointlc, kbndpt, nbndpt, ncurvs, &
                               boundary, numcurvboun, isurnr )

      end if  ! ( doublecurvs ... )
      call errormshsur1 ( nelh, npoh, nelemlc, npointlc, isurnr )

1000  deallocate ( coorusp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coorusp' )
      deallocate ( cord   , stat = error )
      if ( error/=0 ) call erdealloc ( error, 'cord'    )
      deallocate ( itria  , stat = error )
      if ( error/=0 ) call erdealloc ( error, 'itria'   )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshplaxsurf'

      end if  ! ( debug )
      call erclos ( 'mshplaxsurf' )

      end
