      subroutine mshrefpar1 ( iseqkm2, newmapping, istorest, &
                              newtransinfo, mappinglc, npoint_glob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-05-2012
!
!   copyright (c) 2012-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array mappingnew with the global node numbers
!
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     refine
!     parallel
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer newmapping(*), istorest(4,*), newtransinfo(*), &
              mappinglc(*), npoint_glob, iseqkm2

!     iseqkm2        i    Sequence number with respect to km of second mesh
!     istorest       i    array of length (4,iacnodes) containing information
!                         for the various blocks
!                         istorest(1,iblock):  starting address of block iblock
!                         istorest(2,iblock):  number of edges of block iblock
!                         istorest(3,iblock):  starting address in transportinfo
!                         istorest(4,iblock):  sequence number of neighbor
!     mapping        i    contains global node numbers for old mesh
!     newmapping     o    contains global node numbers for refined mesh
!     newtransinfo   i    New array transportinfo
!     npoint_glob    o    Number of nodes in global mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iblock, icount, istartnew, nneigh, i, &
              lenbufarray, icontext, npointnew
      integer, allocatable :: ibufarray(:)
      integer, pointer :: send_buffer(:)
      integer, save :: send_buffer_size

!     i              Counting variable
!     iblock         Block sequence number
!     ibufarray      Buffer array to store information to be sent or received
!     icontext       Tag for sending and receiving, is used as check
!     icount         Counter
!     istartnew      New starting address in array
!     lenbufarray    Length of ibufarray
!     nneigh         Number of neighbors of node
!     npointnew      Number of nodes in refined mesh
!     send_buffer    send buffer
!     send_buffer_size  Length of the send buffer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIBARRIERMPI  Synchronize parallel sending and receiving (mpi)
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copy first npoint nodes from old mapping array
!     For all blocks iblock with sequence number smaller than inodenr
!        Get global node numbers of interface with inodenr
!        Put these numbers in newmapping
!        Get npoint_glob
!     Fill empty positions in newmapping with node numbers
!     Raise npoint_glob
!     For all blocks ibock with sequence number larger than inodenr
!        Send global node numbers of interface with iblock
!        Send npoint_glob
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar1'
         write(irefwr,1) 'iacnodes, inodenr, npoint_glob', &
                          iacnodes, inodenr, npoint_glob
         call prinin ( mappinglc, npoint, 'mappinglc' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npointnew = km(iseqkm2)%kmesh(8)

      allocate ( ibufarray(npointnew), stat = error )
      if ( error/=0 ) call eralloc ( error, npointnew, 'ibufarray' )
      if ( ierror/=0 ) go to 1000

!     --- Copy first npoint nodes from old mapping array

      newmapping(1:npoint) = mappinglc(1:npoint)
      newmapping(npoint+1:npointnew) = 0

!     --- For all blocks iblock with sequence number smaller than inodenr

      icontext = 24
      do i = 1, inodenr-1

!     --- Get global node numbers of interface with inodenr
!         Put these numbers in newmapping
!         Get npoint_glob

         icount = npoint
         call inireceivempi ( ibufarray, icount, SPINT, iblock, &
                              icontext )

         npoint_glob = ibufarray(1)

         if ( istorest(4,iblock)>0 ) then

!        --- istorest(4,iblock)>0, hence there is an interface of iblock and
!            inodenr

            istartnew = istorest(4,iblock)  ! starting address in newtransinfo
            nneigh = newtransinfo(istartnew)  ! number of nodes at interface
            newmapping(newtransinfo(istartnew+1:istartnew+nneigh)) = &
               ibufarray(3:2+nneigh)  ! new node numbers

         end if  ! ( istorest(4,iblock)>0 )

      end do  ! i = 1, inodenr-1

!     --- Fill empty positions in newmapping with node numbers
!         Raise npoint_glob

      do i = npoint+1, npointnew

         if ( newmapping(i)==0 ) then

!        --- node has not yet been set

            npoint_glob = npoint_glob+1
            newmapping(i) = npoint_glob

         end if  ! ( newmapping(i)==0 )

      end do  ! i = npoint+1, npointnew

!     --- Compute length of buffer

      lenbufarray = 1
      do iblock = inodenr+1, iacnodes

         lenbufarray = 1
         if ( istorest(4,iblock)>0 ) then

!        --- istorest(4,iblock)>0, hence there is an interface of iblock and
!            inodenr

            istartnew = istorest(4,iblock)  ! starting address in newtransinfo
            nneigh = newtransinfo(istartnew)  ! number of nodes at interface
            lenbufarray = lenbufarray+2+nneigh

         end if  ! ( istorest(4,iblock)>0 )

      end do  ! iblock = inodenr+1, nbocks

      lenbufarray = lenbufarray+(iacnodes-1)*npoint_glob
      send_buffer_size = 4*lenbufarray

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_size, iacnodes )

      allocate(send_buffer(send_buffer_size))

      call inibufattmpi ( send_buffer, send_buffer_size )

!     --- For all blocks ibock with sequence number larger than inodenr

      do iblock = inodenr+1, iacnodes

!     --- Send global node numbers of interface with iblock
!         Send npoint_glob

         ibufarray(1) = npoint_glob
         lenbufarray = 1
         if ( istorest(4,iblock)>0 ) then

!        --- istorest(4,iblock)>0, hence there is an interface of iblock and
!            inodenr

            istartnew = istorest(4,iblock)  ! starting address in newtransinfo
            nneigh = newtransinfo(istartnew)  ! number of nodes at interface
            ibufarray(2) = nneigh
            ibufarray(3:2+nneigh) = &
               newmapping(newtransinfo(istartnew+1:istartnew+nneigh)) ! nodes
            lenbufarray = 2+nneigh

         end if  ! ( istorest(4,iblock)>0 )

         call inibsendmpi ( ibufarray, lenbufarray, SPINT, iblock, &
                            icontext )

      end do  ! iblock = inodenr+1, nbocks

      icontext = 26
      if ( inodenr==iacnodes ) then

!     --- Last node number, send npoint_glob to all other nodes

         icontext = 26
         do iblock = 1, iacnodes-1
            call inibsendmpi ( npoint_glob, 1, SPINT, iblock, &
                               icontext )
         end do  ! iblock = 1, iacnodes-1

      else

!     --- Other node number, receive npoint_glob

         icount = 1
         call inireceivempi ( npoint_glob, icount, SPINT, iblock, &
                              icontext )

      end if  ! ( inodenr==iacnodes )

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_size )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

!     --- Synchronize communication by calling mpi_barrier
!         This avoids that something is written to other nodes, &
!         before reading by those other nodes has been done

      call inibarriermpi

      deallocate ( ibufarray, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ibufarray' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'npoint_glob', npoint_glob
         call prinin ( newmapping, npointnew, 'newmapping' )
         write(irefwr,*) 'End mshrefpar1'

      end if  ! ( debug )
      call erclos ( 'mshrefpar1' )

      end
