      subroutine mshrefpar9 ( storenodes, istartnode, kmeshmcopy, &
                              kmeshmglobnew, kmeshmglobold, numnodes, &
                              npointold, icurnr, lencurvenew, &
                              lencurveold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a new curve from the old global curve and all changes
!     that have been read from the various nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     parallel
!     curve
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer storenodes(*), istartnode(*), kmeshmcopy(iacnodes,*), &
              kmeshmglobnew(*), kmeshmglobold(*), numnodes, npointold, &
              icurnr, lencurvenew, lencurveold

!     icurnr         i    Curve sequence number
!     istartnode    i/o   Array of size iacnodes containing the next position
!                         in kmeshmcopy to be used for each parallel block
!     kmeshmcopy     i    Work array of length (iacnodes,lenkmeshm) to store
!                         all curves that have new nodal point numbers per
!                         parallel node
!                         (.,1) contains the used length for node .
!                         The next positions contain for each curve stored:
!                         Pos. i: curve number
!                              i+1: number of nodal points in curve
!                              followed by nodal point numbers
!     kmeshmglobnew  i    Array kmeshmglob corresponding to new mesh
!     kmeshmglobold  o    Array kmeshmglob corresponding to old mesh
!     lencurvenew    i    Number of nodes in new curve
!     lencurveold    i    Number of nodes in old curve
!     npointold      i    Number of nodes in old mesh
!     numnodes       i    Number of parallel blocks stored in storenodes
!     storenodes     i    Contains the parallel blocks that contain a new
!                         description of curve icurnr
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstartglold, ipstartglnew, i, iblock, ipstart, lenblock, &
              iendnode(iacnodes), inodeold, newnode, iblocksav, &
              inodeblock, icount
      logical finished

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     iblock         Block sequence number
!     iblocksav      Help parameter to store old value of iblock
!     icount         Counter
!     iendnode       Contains the last position of curve in kmeshmcopy
!     inodeblock     Node number in curvve in kmeshmcopy
!     inodeold       Node number in old kmeshmglob
!     ipstart        Starting address in array
!     ipstartglnew   Starting address in array kmeshmglobnew
!     ipstartglold   Starting address in array kmeshmglobold
!     lenblock       Number of nodes in curve as stored in kmeshmcopy
!     newnode        New node number ot be used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3102   Next node of curves to be merged in parallel case differs on
!           2 submeshes
!    3103   icount>10000
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar9' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar9'
         write(irefwr,1) 'icurnr, numnodes, npointold', &
                          icurnr, numnodes, npointold
         call prinin ( kmeshmglobold, lencurveold, 'kmeshmglobold' )
         call prinin ( storenodes, numnodes, 'storenodes' )
         call prinin ( istartnode, iacnodes, 'istartnode' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ipstartglold = 1
      ipstartglnew = 1
      iendnode = 0

!     --- Fill starting and end address of curve in startblock/iendblock

      do i = 1, numnodes
         iblock = storenodes(i)
         ipstart = istartnode(iblock)
         lenblock = kmeshmcopy(iblock,ipstart)
         istartnode(iblock) = ipstart+1
         iendnode(iblock) = ipstart+lenblock
      end do  ! i = 1, numnodes

      if ( debug ) then
         call prinin ( istartnode, iacnodes, 'istartnode' )
         call prinin ( iendnode, iacnodes, 'iendnode' )
      end if  ! ( debug )

      finished = .false.

      icount = 0
      do while ( .not. finished .and. icount<10000 )

         icount = icount+1
         inodeold = kmeshmglobold(ipstartglold)
         newnode = 0

         if ( debug ) write(irefwr,1) 'inodeold', inodeold

         do i = 1, numnodes

            iblock = storenodes(i)         ! block number
            ipstart = istartnode(iblock)   ! starting address

            if ( ipstart<=iendnode(iblock) ) then
               inodeblock = kmeshmcopy(iblock,ipstart)  ! node at curve
            else
               inodeblock = 0
            end if  ! ( ipstart<=iendnode(iblock) )

            if ( debug ) &
               write(irefwr,1) 'i, iblock, ipstart, inodeblock', &
                                i, iblock, ipstart, inodeblock

            if ( inodeblock>npointold ) then

!           --- New node found

               istartnode(iblock) = istartnode(iblock)+1
               if ( newnode==0 ) then

!              --- new node not yet found, store in newnode

                 newnode = inodeblock
                 iblocksav = iblock

               else if ( newnode/=inodeblock ) then

!              --- Error next node different on two submeshes

                  call errint ( iblocksav, 1 )
                  call errint ( iblock, 2 )
                  call errint ( newnode, 3 )
                  call errint ( inodeblock, 4 )
                  call errint ( icurnr, 5 )
                  call errsub ( 3102, 5, 0, 0 )
                  go to 1000

               end if  ! ( newnode==0 )

            end if  ! ( inodeblock>npointold )

         end do  ! i = 1, numnodes

         if ( debug ) &
            write(irefwr,1) 'newnode, inodeold', newnode, inodeold

         if ( newnode>0 ) then

!        --- New node found, store in kmeshmglobnew

            kmeshmglobnew(ipstartglnew) = newnode
            ipstartglnew = ipstartglnew+1
            newnode = 0

         else

!        --- Not new node found, store old node in kmeshmglobnew

            kmeshmglobnew(ipstartglnew) = inodeold
            ipstartglnew = ipstartglnew+1

!           --- raise ipstart in all blocks

            ipstartglold = ipstartglold+1
            if ( ipstartglold>lencurveold ) finished = .true.

            do i = 1, numnodes

               iblock = storenodes(i)         ! block number
               ipstart = istartnode(iblock)
               ipstart = min(ipstart+1,iendnode(iblock)+1)
               istartnode(iblock) = ipstart  ! starting address

            end do  ! i = 1, numnodes

         end if  ! ( newnode>0 )

         if ( debug ) &
            write(irefwr,1) 'ipstartglold, ipstartglnew, lencurveold', &
                             ipstartglold, ipstartglnew, lencurveold

      end do  ! while ( .not. finished )

      lencurvenew = ipstartglnew-1

      if ( icount>=10000 ) then

!     --- icount too large

         call errint ( icount, 1 )
         call errint ( 10000, 2 )
         call errsub ( 3103, 2, 0, 0 )

      end if  ! ( icount==10000 )

1000  call erclos ( 'mshrefpar9' )
      if ( debug ) then

!     --- Debug information

         call prinin ( kmeshmglobnew, lencurvenew, 'kmeshmglobnew' )
         write(irefwr,*) 'End mshrefpar9'

      end if  ! ( debug )

      end
