      subroutine mshshift2d ( skipnod, iworkpoint, coornew, nmap, eps, &
                              iwork, edgemoved )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 19-09-2013 New parameter list
!        version  2.2  date 22-05-2013 Correction nodes on curves
!        version  2.1  date 19-09-2012 New call to mshshift2d4
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Shift node of intersecting curve onto boundary if
!     node is close (depending on eps) to boundary
!     See also subroutine mshshiftnodes
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer skipnod(*), nmap, iworkpoint(nmap,*), &
              iwork(nelem,*), edgemoved(*)
      double precision coornew(2,*), eps

!     coornew       i/o   Array containing the new coordinates of the nodes
!     edgemoved      o    Array of size nedges indicating if nodes on an edge
!                         have been moved (1) or not (0)
!     eps            i    Local accuracy
!     iwork          i    integer work array of length nelem*2
!                         See subroutine mshobstms2d
!     iworkpoint    i/o   Work array of length npointnew
!                         This array is filled as follows:
!                         iworkpoint(i) = 0 if node i outside obstacle
!                         iworkpoint(i) = 1 if node i inside obstacle
!                         iworkpoint(i) = 2 if node i inside obstacle (near
!                                           boundary)
!                         iworkpoint(i) = 3 if node i on obstacle
!     nmap           i    length of first parameter of imap and rmap
!     skipnod       i/o   Array of npoint long indicating if a node has been
!                         moved or not
!                         skipnod(i) may take the following values
!                         0:  node has not been moved
!                         1:  node lies on a curve
!                         3:  node has been moved
!                         4:  node is end point of curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: markedge(:), markbnedges(:)

!     markbnedges    Array to mark all edges at the curves
!                    if markbnedges(e) = 0 the edge e is not on a curve
!                    if markbnedges(e)>0 the edge e is on curve
!                    with sequence number markbnedges(e)
!     markedge       Array of size nedges
!                    If edge i is not intersected markedge(i) = 0, &
!                    otherwise markedge(i)>0 contains the starting address
!                    icross in array icrossobst corresponding to edge i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHMARKCUREDGE Mark all edges on the curves with the corresponding
!                    curve number
!     MSHSHIFT2D1    Mark nodes that ly on curves
!     MSHSHIFT2D2    Mark all edges that are intersected by the obstacle
!     MSHSHIFT2D4    Shift node of intersecting curve onto boundary if
!                    node is close (depending on eps) to boundary
!     MSHSHIFT2D5    Shift node of intersecting curve onto boundary if
!                    node is close (depending on eps) to boundary
!     MSHSHIFT3D1    Mark nodes that ly on surfaces
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHL  Fill array nmesh part l
!     SEPFLIELEMOBST Fill or activate array kmesh ad  part 2: ielemobst
!     SEPFLPOINTOBST Fill array kmesh ad  part 1: ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- create temporary arrays

      allocate ( markedge(nedges), markbnedges(nedges), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*nedges, 'markedge' )
      if ( ierror/=0 ) go to 1000

      markedge = 0
      markbnedges = 0

!     --- Mark nodes that ly on curves (skipnod:=1 for these points)
!         Mark also all edges that ly on curves

      if ( ndim==2 ) then

!     --- ndim = 2: use curves

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(42)==0 ) call sepfillnmeshl
         call mshshift2d1 ( iworkpoint, skipnod )
         call mshmarkcuredge ( markbnedges )

      else

!     --- ndim = 3: use surfaces

         call mshshift3d1 ( iworkpoint, skipnod )

      end if  ! ( kmesh(6)==2 )

!     --- Mark all edges that are intersected

      call sepflpointobst
      call sepflielemobst
      call mshshift2d2 ( iwork, markedge )
      if ( debug ) call prinin ( markedge, nedges, 'markedge' )

!     --- Move nodes

      if ( ndim==2 ) then

!     --- ndim = 2, use new subroutine

      call mshshift2d5 ( skipnod, iworkpoint, coornew, &
                         nmap, eps, iwork, edgemoved )

      else

!     --- Old routine

      call mshshift2d4 ( skipnod, iworkpoint, coornew, &
                         nmap, eps, iwork, &
                         edgemoved, markedge, markbnedges )
      end if  ! ( .false. )

!     --- Remove temporary arrays

      deallocate ( markedge, markbnedges, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'markedge' )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( coornew, npoint, 2, 'coornew' )
         write(irefwr,*) 'End mshshift2d'

      end if  ! ( debug )
      call erclos ( 'mshshift2d' )

      end
