      subroutine mshshift2d4 ( skipnod, iworkpoint, &
                               coornew, nmap, eps, &
                               iwork, edgemoved, markedge, markbnedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 19-09-2013 New parameter list
!        version  3.0  date 22-05-2013 Extra parameter markbnedges
!        version  2.1  date 21-11-2012 Correction nodes near curve
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Shift node of intersecting curve onto boundary if
!     node is close (depending on eps) to boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer skipnod(*), iworkpoint(*), nmap, &
              iwork(nelem,*), &
              edgemoved(*), markedge(*), markbnedges(*)
      double precision coornew(ndim,*), eps

!     coornew       i/o   Array containing the new coordinates of the nodes
!     edgemoved     i/o   Array of size nedges indicating if nodes on an edge
!                         have been moved (1) or not (0)
!     eps            i    Defines when the intersection is close to a node
!     iwork          i    integer work array
!                         See subroutine mshobstms2d
!     iworkpoint    i/o   Work array of length npointnew
!                         This array is filled as follows:
!                         iworkpoint(i) = 0 if node i outside obstacle
!                         iworkpoint(i) = 1 if node i inside obstacle
!                         iworkpoint(i) = 2 if node i inside obstacle (near
!                                           boundary)
!                         iworkpoint(i) = 3 if node i on obstacle
!     markbnedges    i    Array to mark all edges at the curves
!                         if markbnedges(e) = 0 the edge e is not on a curve
!                         if markbnedges(e)>0 the edge e is on curve
!                         with sequence number markbnedges(e)
!     markedge       i    Array of size nedges
!                         See subroutine mshshift2d
!     nmap           i    length of first parameter of imap and rmap
!     skipnod       i/o   Array of npoint long indicating if a node has been
!                         moved or not
!                         See subroutine mshshift2d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, istart, i, ireledge, inpelm, &
              istarticross, ipstart, iedge, ielgrp, icross, iielem, &
              iobst, nodebegin, nodeend, jump
      integer, allocatable :: imark(:)
      double precision fact
      logical linear, moved

!     fact           factor defining the distance between obstacle and node
!     i              Counting variable
!     icross         Position of iedge in array icrossobst
!     iedge          edge sequence number
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     imark          Marks nodal points
!                    0: node must not be moved
!                    1: node must be moved
!     inpelm         number of points per element
!     iobst          Obstacle sequence number
!     ipstart        Starting address in array
!     ireledge       relative edge sequence number
!     istart         First position used at a part
!     istarticross   Starting address of obstacle in array icrossobst
!     jump           2 in case linear elements are used, 3 if quadratic
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     moved          If true a node must be moved or has been moved
!     nodebegin      Node at the begin of a crossed edge
!     nodeend        Node at the end of a crossed edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHSHIFT2D3    Mark edges of which one of the nodes has been moved
!     MSHSHIFT2D4A   Shift node of intersecting curve onto boundary if
!                    node is close (depending on eps) to boundary
!     MSHSHIFT2D4B   Find edges that is intersected by obstacle and contains
!                    nodeend
!                    Compute edge with minimum distance between obstacle and
!                    nodeend
!     MSHSHIFT2D4C   Find edges that is intersected by obstacle and contains
!                    nodeend
!                    Compute edge with minimum distance between obstacle and
!                    nodeend  (only on 3d surfaces)
!     MSHSHIFTTEST   Test if all nodes that have to be shifted are moved to
!                    the interface
!     SEPFILLNMESHL  Fill array nmesh part l
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2d4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2d4'
         write(irefwr,2) 'eps', eps
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )
  4      format ( i6, 2x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- activate arrays

      if ( nmesh(42)==0 ) call sepfillnmeshl

!     --- create temporary arrays

      allocate ( imark(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'imark' )
      if ( ierror/=0 ) go to 1000

!     --- initialize parameters

      ielem=0
      istart=0

      imark = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         linear = ishape==3 .or. ishape==11
         if ( linear ) then
            jump = 2
         else
            jump = 3
         end if  ! ( linear )

         if ( debug ) write(irefwr,1) 'inpelm, jump', inpelm, jump

!        --- loop over elements

         do iielem = 1, inelem

            ielem=ielem+1

            if ( debug ) &
               write(irefwr,1) 'ielem, iwork(ielem,1)', &
                                ielem, iwork(ielem,1)

            if ( iwork(ielem,1)/=0 ) then

!           --- iwork(ielem,1)#0, hence element is intersected

               istart = iwork(ielem,1)  ! starting address in icrossobst
               iobst = iwork(ielem,2)
               istarticross = icrossobst(iobst)

!              --- loop over crossed edges

               do i = 1, icrossobst(istart)

                  ireledge = icrossobst(istart+i)
                  icross  = istarticross+1+2*(ireledge-1)
                  iedge = icrossobst(icross)
                  fact  = rcrossobst(icrossobst(icross+1))
                  ipstart = nmeshc(iedge)

                  if ( debug ) then
                     write(irefwr,1) 'i, ireledge, icross, iedge', &
                                      i, ireledge, icross, iedge
                     write(irefwr,2) 'fact', fact
                     write(irefwr,1) 'nodes', &
                        nmeshc(ipstart+1:ipstart+jump)
                     write(irefwr,1) 'skipnod', &
                        skipnod(nmeshc(ipstart+1:ipstart+jump))
                  end if  ! ( debug )

                  moved = .false.

                  if ( skipnod(nmeshc(ipstart+1))==3 .or. &
                       skipnod(nmeshc(ipstart+jump))==3 ) then

!                 --- One of the nodes has been moved

                     edgemoved(iedge) = 1

                  else if ( fact<=eps ) then

!                 --- fact<=eps, hence the first node must be moved

                     nodeend = nmeshc(ipstart+1)  ! node to be moved
                     nodebegin = nmeshc(ipstart+jump)  ! other end node
                     moved = .true.

                     imark(nodeend) = 1  ! mark node as to be moved

                  else if ( fact>=1d0-eps ) then

!                 --- fact>=1-eps, hence the last node must be moved

                     nodebegin = nmeshc(ipstart+1)  ! node to be moved
                     nodeend = nmeshc(ipstart+jump)  ! other end node
                     fact = 1d0-fact
                     moved = .true.

                     imark(nodeend) = 1  ! mark node as to be moved

                  end if  ! ( skipnod(nmeshc(ipstart+1))==3 .and. .. )

                  if ( moved .and. debug ) then
                     write(irefwr,1) 'nodebegin, nodeend', &
                                      nodebegin, nodeend
                     write(irefwr,*) 'moved ', moved
                  end if  ! ( debug )

                  if ( moved ) then

!                 --- moved = true, hence node must be moved
!                     Find nodebegin on one of the crossed edges with
!                     shortest distance to nodeend

                     if ( skipnod(nodeend)==0 ) then

!                    --- Nodeend not on a curve
!                        Only then the shortest distance may be found

                        call mshshift2d4b ( nodeend, nodebegin, nmeshc, &
                                            nmeshl, coornew, fact, &
                                            markedge, icrossobst, &
                                            rcrossobst, jump, iedge, &
                                            ndim, eps, skipnod )

                     else if ( ndim==3 .and. skipnod(nodebegin)==1 .and. &
                               skipnod(nodeend)==1 ) then

!                    --- Both nodes on a surface
!                        The shortest distance on the surface must be found

                        call mshshift2d4c ( nodeend, nodebegin, nmeshc, &
                                            nmeshl, coornew, fact, &
                                            markedge, icrossobst, &
                                            rcrossobst, jump, iedge, &
                                            skipnod )

                     end if  ! ( skipnod(nodebegin)==0 .and. &

                     call mshshift2d4a ( skipnod, nodebegin, nodeend, &
                                         iworkpoint, coornew, &
                                         nmap, fact, moved, &
                                         iobst, ndim, markbnedges, iedge )

                     if ( moved .and. debug ) then
                        write(irefwr,1) 'nodebegin, nodeend', &
                                         nodebegin, nodeend
                        write(irefwr,2) 'coor_nodebegin', &
                                         coornew(:,nodebegin)
                        write(irefwr,*) 'moved ', moved
                     end if  ! ( debug )
                     if ( moved ) edgemoved(iedge) = 1

                  end if  ! ( moved )

               end do  ! i = 1, icrossobst(istart)

            end if  ! ( iwork(ielem)/=0 )

         end do  ! iielem = 1, inelem

      end do  ! ielgrp = 1, nelgrp

!     --- Mark edges that are connected to nodes that have been moved

      call mshshift2d3 ( skipnod, iwork, edgemoved )

!     --- Check if there are nodes that should be moved but have not

      call mshshifttest ( npoint, imark, skipnod )

!     --- Remove temporary arrays

      deallocate ( imark, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'imark' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Nodes that are moved'
         do i = 1, npoint
            if ( skipnod(i)==3 ) then
               write(irefwr,4) i, coornew(:,i)
            end if  ! ( skipnodes(i)==3 )
         end do  ! i = 1, npoint
         write(irefwr,*) 'End mshshift2d4'

      end if  ! ( debug )
      call erclos ( 'mshshift2d4' )

      end
