      subroutine mshshift2d4a ( skipnod, nodebegin, nodeend, iworkpoint, &
                                coornew, nmap, fact, moved, &
                                iobst, ndim, markbnedges, iedge )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 22-05-2013 Extra parameter markbnedges, iedge
!        version  2.2  date 21-11-2012 Correction nodes near curve
!        version  2.1  date 25-10-2012 Extra debug
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Shift node of intersecting curve onto boundary if
!     node is close (depending on eps) to boundary
!     See also subroutine mshshiftnod4
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer skipnod(*), nodebegin, nodeend, nmap, iworkpoint(nmap,*), &
              iobst, ndim, markbnedges(*), iedge
      double precision coornew(ndim,*), fact
      logical moved

!     coornew       i/o   Array containing the new coordinates of the nodes
!     fact           i    factor defining the distance between obstacle and node
!     iedge          i    edge sequence number
!     iobst          i    Obstacle sequence number
!     iworkpoint    i/o   Work array of length npointnew
!                         This array is filled as follows:
!                         iworkpoint(i) = 0 if node i outside obstacle
!                         iworkpoint(i) = 1 if node i inside obstacle
!                         iworkpoint(i) = 2 if node i inside obstacle (near
!                                           boundary)
!                         iworkpoint(i) = 3 if node i on obstacle
!     markbnedges    i    Array to mark all edges at the curves
!                         if markbnedges(e) = 0 the edge e is not on a curve
!                         if markbnedges(e) > 0 the edge e is on curve
!                         with sequence number markbnedges(e)
!     moved          o    If true a node has been moved
!     ndim           i    Dimension of the space.
!     nmap           i    length of first parameter of imap and rmap
!     nodebegin      i    Node at the begin of a crossed edge
!     nodeend        i    Node at the end of a crossed edge
!     skipnod       i/o   Array of npoint long indicating if a node has been
!                         moved or not
!                         See subroutine mshshift2d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical movenode

!     movenode       If true the node must be moved
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2d4a' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2d4a'
         write(irefwr,1) 'nodebegin, nodeend, iedge', &
                          nodebegin, nodeend, iedge
         write(irefwr,1) 'skipnod', skipnod(nodebegin), skipnod(nodeend)
         write(irefwr,1) 'markbnedges(iedge)', markbnedges(iedge)
         write(irefwr,2) 'fact', fact
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- if one of the two nodes of the edge has already been
!         shifted then don't shift the other node

      moved = .false.
      movenode = skipnod(nodebegin)<=1 .and. skipnod(nodeend)==0
      if ( .not. movenode ) then

!     --- both nodes on a curve or nodeend has been shifted

         if ( ndim==2 ) then

!        --- ndim = 2, check if both nodes on an edge of a curve

            movenode =  skipnod(nodebegin)==1 .and. &
                        skipnod(nodeend)==1 .and. &
                        markbnedges(iedge)>0

         else

!        --- ndim = 3, check if both nodes on a curve
!            The option to check if they are on an edge has not been
!            implemented yet

            movenode = skipnod(nodebegin)==1 .and. &
                       skipnod(nodeend)==1

         end if  ! ( ndim==2 )

      end if  ! ( .not. movenode )

      if ( movenode ) then

!     --- Both nodes have not been shifted, and both on a curve
!         or both not on a curve or nodeend has not been shifted

         if (debug) then
            write(irefwr,2) 'coors begin', coornew(:,nodebegin)
            write(irefwr,2) 'coors end', coornew(:,nodeend)
         end if

         coornew(:,nodeend) = (1d0-fact)*coornew(:,nodeend) + &
                               fact*coornew(:,nodebegin)

         if (debug) then
            write(irefwr,1) 'nodeend', nodeend
            write(irefwr,2) 'coors end (new)', coornew(:,nodeend)
         end if

         iworkpoint(nodeend,1) = 3  ! mark node as on the obstacle
         iworkpoint(nodeend,2) = iobst
         skipnod(nodeend) = 3     ! mark node as shifted

!        --- Store mapping information for shifted node

         imap(nodeend,1) = 2
         imap(nodeend,2) = nodeend
         imap(nodeend,3) = nodebegin
         rmap(nodeend,1) = 1d0-fact
         rmap(nodeend,2) = fact

         if ( debug ) then
            write(irefwr,1) 'imap ', imap(nodeend,1:3)
            write(irefwr,2) 'rmap ', rmap(nodeend,1:2)
         end if  ! ( debug )

         moved = .true.

      end if  ! ( skipnod(nodebegin)==0 .and. skipnod(nodeend)==0 ... )

1000  call erclos ( 'mshshift2d4a' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshift2d4a'

      end if  ! ( debug )

      end
