      subroutine mshshift2dcr3 ( numnodesb, nodesb, imarknodes, &
                                 setE, numedges, imarkelem, removenodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Remove those nodes from nodesB that are in edges that are
!      lying in 2 triangles with imarkelem >=2, &
!      i.e. all nodes of the triangles are either outside the obstacle
!      or are member of setB
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodesb, nodesb(*), imarkelem(*), &
              imarknodes(*), setE(*), numedges
      logical removenodes

!     imarkelem      i    See subroutine mshshift2dcor
!     imarknodes    i/o   See subroutine mshshift2dcor
!     nodesb        i/o   Set of nodes B of nodes on edges that are intersected
!                         where the node closest to the boundary is used
!                         At output some nodes may be removed
!     numedges       i    number of edges
!     numnodesb     i/o   Number of nodal points in array nodesb
!     removenodes   i/o   if true nodes  must be removed from nodesb
!     sete           i    Set of edges that have all nodes in B and are at the
!                         boundary of setT
!                         Only edges inside triangles with nodes outside
!                         the obstacle and in setB are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numelm, ipstart, j, ielem1, ielem2, jpstart, icount, &
              iedge, node

!     i              Counting variable
!     icount         Counter
!     iedge          edge sequence number
!     ielem1         First element number corresponding to edge
!     ielem2         Second element number corresponding to edge
!     ipstart        Starting address in array
!     j              Counting variable
!     jpstart        Starting address in array
!     node           Node number
!     numelm         Number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPFILLNMESHH  Fill array nmesh part h
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2237   Edge connected to more than two elements
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2dcr3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2dcr3'
         write(irefwr,1) 'numedges', numedges
         call prinin ( nodesb, numnodesb, 'nodesb' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate or create arrays

      if ( nmesh(38)==0 ) call sepfillnmeshh

!     --- Actual call

!     --- For all edges iedge in setE do

      do i = 1, numedges

         iedge = setE(i)  ! next edge

!        --- If ( both triangles ielem connected to iedge are outside obstacle )
!              remove both nodes from setB

         ipstart = nmeshh(iedge)
         numelm = nmeshh(ipstart)

         if ( debug ) &
            write(irefwr,1) 'i, iedge, numelm', i, iedge, numelm
         if ( numelm>2 ) then

!        --- More than 2 triangles connected to edge, not yet implemented

            call errint ( iedge, 1 )
            call errint ( numelm, 2 )
            do j = 1, numelm
               call errint ( nmeshh(ipstart+j), 2+j )
            end do  ! j = 1, numelm
            call errsub ( 2237, 2+numelm, 0, 0 )
            go to 1000

         end if  ! ( numelm>2 )

         if ( numelm==2 ) then

!        --- Exactly 2 elements connected to edge

            ielem1 = nmeshh(ipstart+1)
            ielem2 = nmeshh(ipstart+2)
            if ( debug ) then
               write(irefwr,1) 'ielem1, ielem2', ielem1, ielem2
               write(irefwr,1) 'imarkelem(ielem1/2)', &
                 imarkelem(ielem1), imarkelem(ielem2)
            end if  ! ( debug )
            if ( imarkelem(ielem1)>=2 .and. imarkelem(ielem2)>=2 ) then

!           --- Both triangles outside obstacle, remove nodes from B

               jpstart = nmeshc(iedge)   ! start of nodes in edge
               do j = 1, nmeshc(jpstart) ! number of nodes in edge
                  imarknodes(nmeshc(jpstart+j)) = 2  ! mark node as to be removed
               end do  ! j = 1, nmeshc(jpstart)
               removenodes = .true.

            end if  ! ( imarkelem(ielem1)>=2 .and. imarkelem(ielem2)>=2 )

         end if  ! ( numelm==2 )

      end do  ! i = 1, numedges

      if ( removenodes ) then

!     --- Nodes must be removed

         icount = 0
         do i = 1, numnodesb

            node = nodesb(i)
            if ( imarknodes(node)/=2 ) then

!           --- Node must be copied

               icount = icount+1
               nodesb(icount) = nodesb(i)

            end if  ! ( imarknodes(i)/=2 )

         end do  ! i = 1, numnodes
         numnodesb = icount

      end if  ! ( removenodes )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( nodesb, numnodesb, 'nodesb' )
         write(irefwr,*) 'End mshshift2dcr3'

      end if  ! ( debug )
      call erclos ( 'mshshift2dcr3' )

      end
