      subroutine mshshift2del ( iwork, nodesb, numnodesb, eps, &
                                nodesmv, nodesfac, skipnod )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodesmv with nodes that must be moved and corresponding
!     node at other side of the edge
!     Fill array nodesfac with factors for moving
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(nelem,*), numnodesb, nodesb(numnodesb), &
              nodesmv(2,*), skipnod(*)
      double precision eps, nodesfac(*)

!     eps            i    Local accuracy
!                         Defines when an intersection is close to a node
!                         (0<eps<=0.5)
!     iwork          i    integer work array
!                         See subroutine mshobstms2d
!     nodesb         i    Contains list of nodes to be moved
!     nodesfac       o    Contains factor for moving along edge, measured from
!                         first node
!     nodesmv        o    Contains set of nodes to be moved
!                         (1,.) node to be moved
!                         (2,.) second node on edge
!     numnodesb     i/o   Number of nodes that are to be moved
!     skipnod       i/o   Array of npoint long in which the nodes are marked
!                         which are not allowed to be moved
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, inpelm, jump, iielem, &
              ielem, istart, iobst, icount, i, ireledge, icross, iedge, &
              ipstart, istarticross, node1, node2, numnodes, iseq
      integer, allocatable :: imark(:), istore(:)
      double precision, allocatable :: lenfact(:)
      double precision fact, diff(2), eps1, dist
      logical linear, extradebug, movenode

!     diff           Difference between 2 coordinate sets
!     dist           Distance between 2 points
!     eps1           0.01*eps
!     extradebug     If true extra debugging statements are carried out
!     fact           factor along edge
!     i              Counting variable
!     icount         Counter
!     icross         Position of iedge in array icrossobst
!     iedge          edge sequence number
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     imark          Work array of size npoint to mark nodes in nodesb
!                    0: node does not belong to nodesb
!                    1: node belongs to nodesb
!     inpelm         number of points per element
!     iobst          Obstacle sequence number
!     ipstart        Starting address in array
!     ireledge       relative edge sequence number
!     iseq           Sequence number of node in nodesmv
!     istart         starting address in icrossobst
!     istarticross   Starting address of obstacle in array icrossobst
!     istore         Work array of length npoint to store the position of a node
!                    in array nodesmv
!     jump           if 2 the elements are linear, &
!                    if 3 quadratic
!     lenfact        Work array to store the distance of a node
!                    in nodesmv to its other node
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     movenode       Indicates if a node may be moved or not
!     node1          Node of face
!     node2          Node of face
!     numnodes       Number of nodal points in array nodesmv
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     SEPFILLKMAD3   Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFLIELEMOBST Fill or activate array kmesh ad  part 2: ielemobst
!     SEPFLPOINTOBST Fill array kmesh ad  part 1: ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2del' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2del'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate or create arrays

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(33)==0 ) call sepfillnmeshc
      call sepflpointobst
      call sepflielemobst
      if ( kmeshad(3)==0 .or. kmeshad(5)/=kmeshad(8) ) call sepfillkmad3

!     --- Actual call

      allocate ( imark(npoint), istore(npoint), lenfact(numnodesb), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'imark' )
      if ( ierror/=0 ) go to 1000

      imark = 0
      istore = 0
      imark(nodesb(1:numnodesb)) = 1

      ielem=0
      istart=0
      numnodes = 0
      eps1 = 1d-2*eps

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         linear = ishape==3 .or. ishape==11
         if ( linear ) then
            jump = 2
         else
            jump = 3
         end if  ! ( linear )

         if ( extradebug ) write(irefwr,1) 'inpelm, jump', inpelm, jump

!        --- loop over elements

         do iielem = 1, inelem

            ielem=ielem+1

            if ( iwork(ielem,1)/=0 ) then

!           --- iwork(ielem,1)#0, hence element is intersected

               if ( extradebug ) &
                  write(irefwr,1) 'ielem, iwork(ielem,1)', &
                                   ielem, iwork(ielem,1)
               istart = iwork(ielem,1)  ! starting address in icrossobst
               iobst = iwork(ielem,2)
               istarticross = icrossobst(iobst)

!              --- loop over crossed edges

               icount = 0
               do i = 1, icrossobst(istart)

                  ireledge = icrossobst(istart+i)
                  icross  = istarticross+1+2*(ireledge-1)
                  iedge = icrossobst(icross)
                  fact  = rcrossobst(icrossobst(icross+1))
                  ipstart = nmeshc(iedge)

                  if ( fact>=0d0 .and. fact<=eps .or. &
                       fact>=1d0-eps .and. fact<=1d0 ) then

!                 --- edge is intersected and node may be moved
!                     Mark nodes to be moved

                     if ( fact<=eps ) then
                        node1 = nmeshc(ipstart+1)
                        node2 = nmeshc(ipstart+jump)
                     else
                        node1 = nmeshc(ipstart+jump)
                        node2 = nmeshc(ipstart+1)
                        fact = 1d0-fact
                     end if  ! ( fact<=eps )

                     if ( debug ) then
                        write(irefwr,1) 'node1, node2, imark(node1)', &
                                         node1, node2, imark(node1)
                        write(irefwr,1) 'icount', icount
                        write(irefwr,2) 'fact', fact
                     end if  ! ( debug )

                     if ( imark(node1)==1 .or.fact<eps1 ) then

!                    --- Node belongs to set of allowed nodes
!                        check if node may be moved
!                        This is the case if node has not been moved yet
!                        or if node is on curve and other node is on the same
!                        curve too

                        movenode = skipnod(node1)==0 .or. &
                                   skipnod(node1)<0 .and. &
                                  (skipnod(node2)==skipnod(node1) .or. &
                                   skipnod(node2)==4)

                        if ( debug ) then
                           write(irefwr,1) 'skipnod(node1:2)', &
                              skipnod(node1), skipnod(node2)
                           write(irefwr,*) 'movenode', movenode
                        end if  ! ( debug )

                        if ( movenode ) then

!                       --- Node has not been moved

                           diff = coor(:,node1)-coor(:,node2)
                           dist = sqrt(dot_product(diff,diff))*fact

                           if ( debug ) write(irefwr,2) 'dist', dist
                           if ( istore(node1)==0 ) then

!                          --- First time the node is found

                              numnodes = numnodes+1
                              istore(node1) = numnodes   ! mark node
                              nodesfac(numnodes) = fact
                              nodesmv(1,numnodes) = node1
                              nodesmv(2,numnodes) = node2
                              lenfact(numnodes) = dist

                              if ( debug ) then
                                 write(irefwr,1) 'numnodes', numnodes
                              end if  ! ( debug )

                           else

!                          --- node has been found before

                              iseq = istore(node1)
                              if ( dist<lenfact(iseq) ) then

!                             --- New factor smaller than previous one

                                 nodesfac(iseq) = fact
                                 lenfact(iseq) = dist
                                 nodesmv(2,iseq) = node2

                              end if  ! ( fact<nodesfac(iseq) )

                           end if  ! ( istore(node1)==0 )

                        end if  ! ( movenode )

                     else

!                    --- Node is not allowed

                        icount = icount-1

                     end if  ! ( imark(node1)==0 )

                     if ( extradebug ) then
                        write(irefwr,1) 'ielem, iedge, nodes', ielem, &
                          iedge,nmeshc(ipstart+1), nmeshc(ipstart+jump)
                        write(irefwr,2) 'fact', fact
                     end if  ! ( debug )

                  end if  ! ( fact<=eps .or. fact>-1d0-fact )

               end do  ! i = 1, icrossobst(istart)

            end if  ! ( iwork(ielem)/=0 )

         end do  ! iielem = 1, inelem

      end do  ! ielgrp = 1, nelgrp

      numnodesb = numnodes

      deallocate ( imark, istore, lenfact, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'imark' )

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( nodesmv, numnodesb, 2, '  node1       node2' )
         call prinrl ( nodesfac, numnodesb, 'nodesfac' )
         write(irefwr,*) 'End mshshift2del'

      end if  ! ( debug )
      call erclos ( 'mshshift2del' )

      end
