      subroutine mshsurfaces5 ( kmeshn, surffaces, nsurfsold, &
                                nsurfs, nsurfsnew, &
                                extsurffaces, lenextsurffac, ipoint, &
                                iworksurfs, leniworksurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 24-11-2004 Fortran 90
!        version  1.1  date 04-08-2004 Adaptation comments
!        version  1.0  date 05-04-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check which surfaces are single and which are compound surfaces
!     Store information about compound surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     face
!     mesh
!     compound_surface
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshn(*), surffaces(*), nsurfsold, nsurfs, &
              nsurfsnew, extsurffaces(*), &
              lenextsurffac, ipoint(*), iworksurfs(*), leniworksurfs

!     extsurffaces   o    Array of length lenextsurffac containing the
!                         face sequence numbers of the new surfaces to be
!                         created
!     ihelp          o    integer work array of length nsurfs
!                         If surface i is single ihelp(i) = 0
!                         If surface i is possibly compound ihelp(i) = -1
!     ipoint         o    Array of estimated length 10*nsurfsold that is used to
!                         store the end addresses of each new surface in array
!                         extsurffaces
!                         The positions 1 to nsurfs get the value 0
!     iwork          i    integer work array
!                         iwork(iface) contains the number of surfaces
!                         face iface is stored in
!     iwork1         i    integer work array of length nfaces
!                         contains for each face iface with iwork(iface)>1 a
!                         starting address in iwork2
!     iwork2         i    integer work array of length leniwork2
!                         contains for each face iface with iwork(iface)>1 the
!                         sequence numbers of the surfaces iface is lying in
!     iworksurfs     o    Array in which the subsurfaces corresponding to a
!                         surface are stored
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     lenextsurffac i/o   Length of array extsurfface
!                         At input estimated length
!                         At output computed length
!     leniworksurfs i/o   Length of array iworksurfs
!                         At input estimated length
!                         At output computed length
!     nsurfs         i    number of surfaces
!     nsurfsnew     i/o   Last surface sequence number used
!     nsurfsold      i    Number of surfaces in the mesh read
!     surffaces      i    Array containing the face numbers of the elements
!                         stored in array kmeshn
!                         Exactly the sequence of kmeshn is used
!                         In this case it refers to kmeshn before extension
!                         with extra outer surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurnr, icount, jcount

!     icount         Counter in array iworksurfs
!     ihelp1         Work array of length nsurfs
!                    containing the number of subsurfaces
!                    corresponding to all surfaces with ihelp(i)>0
!     isurnr         Surface sequence number
!     jcount         Counter in array extsurffaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHSURFACES10  Treat the complicated surfaces containing several groups
!                    of subsurfaces that can not be identified in a simple way
!     MSHSURFACES4   Set array ihelp equal to 0 for all surfaces that do not
!                    have common faces with other surfaces
!     MSHSURFACES7   Mark all surfaces that have faces in common with other
!                    surfaces but can be considered as a subset of these
!                    surfaces as single by setting ihelp(i) = 0 for these
!                    surfaces
!     MSHSURFACES8   Store all subsurfaces corresponding to ihelp(isurnr) = 1
!                    into iworksurfs and put the starting addresses into ihelp
!     MSHSURFACES9   All surfaces with ihelp(i)=-1 refer to several groups of
!                    surfaces
!                    If all these subsurfaces are single we store these
!                    subsurfaces in iworksurfs and adapt ihelp
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ihelp := -1 (1 ... nsurfsold), meaning that these surfaces are
!     possibly compound
!     ihelp := 0 (nsurfsold+1 ... nsurfs ), meaning that these surfaces are
!     single
!     Mark all surfaces that have faces in common with other surfaces
!     but can be considered as a subset of these surfaces as single
!     by setting ihelp(i) = 0 for these surfaces
!     Set ihelp(i) = 1 if surface i is compound and all faces point to
!     the same subsurfaces (rare)
!     Store all subsurfaces corresponding to ihelp(isurnr) = 1
!     into iworksurfs and put the starting addresses into ihelp
!     Next all surfaces with ihelp(i)=-1 refer to several groups of surfaces
!     If all these subsurfaces are single we store these subsurfaces
!     otherwise we treat them separately
!     Finally all surfaces with ihelp(i)=-10 are treated
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfaces5' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfaces5'
         call prinin ( kmeshn, 2*nsurfs, 'kmeshn' )
         write(irefwr,*) 'nsurfs, nsurfsold ', nsurfs, nsurfsold

      end if
      if ( ierror.ne.0 ) go to 1000

      allocate ( ihelp1(nsurfs), stat = error )
      if ( error/=0 ) call eralloc ( error, nsurfs, 'ihelp1' )
      if ( ierror/=0 ) go to 1000

!     --- ihelp := -1 (1 ... nsurfsold), meaning that these surfaces are
!         possibly compound

      nsurfsnew = nsurfs
      jcount = 0
      icount = 0
      do isurnr = 1, nsurfsold
         ihelp(isurnr) = -1
      end do  ! isurnr = 1, nsurfsold

!     --- ihelp := 0 (nsurfsold+1 ... nsurfs ), meaning that these surfaces are
!         single

      do isurnr = nsurfsold+1, nsurfs
         ihelp(isurnr) = 0
      end do  ! isurnr = nsurfsold+1, nsurfs

!     --- Find all single surfaces and make ihelp 0 for these surfaces
!         For all surfaces stored in kmeshn and surffaces do

      call mshsurfaces4 ( kmeshn, nsurfs, nsurfsold, surffaces )
      if ( debug ) call prinin ( ihelp, nsurfs, 'ihelp/1' )

!     --- Mark all surfaces that have faces in common with other surfaces
!         but can be considered as a subset of these surfaces as single
!         by setting ihelp(i) = 0 for these surfaces
!         Set ihelp(i) = 1 if surface i is compound and all faces point to
!         the same subsurfaces (rare)

      call mshsurfaces7 ( kmeshn, nsurfs, nsurfsold, surffaces )
      if ( debug ) call prinin ( ihelp, nsurfs, 'ihelp/2' )

!     --- Now we store all subsurfaces corresponding to ihelp(isurnr) = 1
!         into iworksurfs and put the starting addresses into ihelp

      call mshsurfaces8 ( kmeshn, nsurfs, nsurfsold, surffaces, &
                          iworksurfs, icount )
      if ( debug ) call prinin ( ihelp, nsurfs, 'ihelp/3' )
      if ( icount.gt.leniworksurfs ) then

!     --- array length too small

         call errint ( icount, 1 )
         call errint ( leniworksurfs, 2 )
         call errchr ( 'iworksurfs', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Next all surfaces with ihelp(i)=-1 refer to several groups of surfaces
!         If all these subsurfaces are single we store these subsurfaces
!         otherwise we treat them separately

      call mshsurfaces9 ( kmeshn, nsurfs, nsurfsold, surffaces, &
                          iworksurfs, icount, nsurfsnew, &
                          extsurffaces, jcount, lenextsurffac, ipoint )
      if ( icount.gt.leniworksurfs ) then

!     --- array length too small

         call errint ( icount, 1 )
         call errint ( leniworksurfs, 2 )
         call errchr ( 'iworksurfs', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      if ( debug ) then

!     --- Debug information

         call prinin ( ihelp, nsurfs, 'ihelp/4' )
         call prinin ( iworksurfs, icount, 'iworksurfs' )
         call prinin ( ihelp1, nsurfs, 'ihelp1' )

      end if  ! ( debug )

!     --- Finally all surfaces with ihelp(i)=-10 are treated

      call mshsurfaces10 ( kmeshn, nsurfs, nsurfsold, surffaces, &
                           iworksurfs, icount, nsurfsnew, &
                           extsurffaces, jcount, lenextsurffac, &
                           ipoint, leniworksurfs )

      deallocate ( ihelp1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ihelp1' )

      lenextsurffac = jcount
      leniworksurfs = icount
      if ( debug ) then

!     --- Debug information

         call prinin ( ihelp, nsurfs, 'ihelp/5' )
         call prinin ( iworksurfs, icount, 'iworksurfs' )

      end if  ! ( debug )

1000  call erclos ( 'mshsurfaces5' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsurfaces5'

      end if

      end
