      subroutine er0088 ( iintbn, usol, isurfs, value )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 23-09-2011 Make nodes local
!        version  3.0  date 27-09-2010 Make work arrays local
!        version  2.2  date 16-06-2005 Set nnodes
!
!   copyright (c) 1997-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine er0088 computes the boundary integral over a set of
!    surfaces in the case ICHINT = 8, i.e. that a summation must be carried out
!    er0088 is a help subroutine for subroutine INTBOU
!    Real case
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iintbn(*), isurfs(*)
      double precision usol(*), value

!     iintbn     i    Integer input array in which the user indicates what type
!                     of integral must be computed.
!                     IINTBN must be filled as follows:
!                     IINTBN(1) Type of calling subroutine
!                               1:  BOUNIN
!                               2:  BOUNIV
!                               3:  INTBOU
!                               4:  INTBND
!                     IINTBN(2) (ICHINT) indicates what type of integral has to
!                               be computed. Possibible values:
!                               1          /  f u  ds
!                               2          /  f  u.n  ds
!                               3          /  f  u.t  ds
!                               4          /  f  n. sigma .n  ds
!                               5          /  f  n. sigma .t  ds
!                               6          /  f  n. sigma     ds
!                               7          /  f  u  n  ds
!                               8          sum f u
!                               The possibilities 3 to 6 have only been
!                               implemented for NDIM = 2, i.e. two-dimensional
!                               regions
!                               Default value: 1
!                     IINTBN(3) (ICHFUN) indication of the value of f.
!                               Possibible values:
!                               0         f = 1
!                               >0        f = func(ichfun,x,y,z) (real case)
!                                         or
!                                         f = cfunc(ichfun,x,y,z) (complex
!                                             case)
!                                         at this moment only the combination
!                                         f#1 and ichint=1 is permitted
!                               Default value: 0
!                     IINTBN(4) (IRULE) The numerical integration rule to be
!                               carried out depends on IRULE. Possibilities:
!                               Possibible values:
!                               For curves only (i.e. 2D):
!                               1  trapezoid rule,  cartesian co-ordinates
!                               2  trapezoid rule,  axisymmetric co-ordinates
!                               3  simpson's rule   cartesian co-ordinates
!                               4  simpson's rule   axisymmetric co-ordinates
!                               For surfaces only (i.e. 3D):
!                               1  Newton cotes rule, i.e. integration rule
!                                  based on the type of approximation used
!                                  in the element
!                               Default value: 1
!                     IINTBN(5) (IDIM) This parameter indicates what type of
!                               boundary integral must be computed
!                               Possible values:
!                               1: curves
!                               2: surfaces
!                               3: contact
!                               4: no_contact
!                     IINTBN(6) (NUMBER) This parameter indicates which unknown
!                               (physical degree of freedom) must be used from
!                               the (solution) vector u (IVECTR)
!                               If ICHINT = 2, 3 or 7, it is supposed that the
!                               degrees of freedom are equal to NUMBER, &
!                               NUMBER+1 and NUMBER+2 (3D)
!                               If ICHINT = 4, 5 or 6, it is supposed that the
!                               degrees of freedom are equal to NUMBER, &
!                               NUMBER+1 and NUMBER+2 (2D) or NUMBER to
!                               NUMBER+5 (3D)
!                               Default value: 1
!                     IINTBN(7) (NCURVES) Indicates the number of curves or
!                               surfaces in array ICURVS that must be used for
!                               the integration
!                               Default value: 1
!     isurfs     i    Integer array containing the surface numbers for the
!                     computation of the boundary integral.
!     usol       i    Solution vector
!     value      o    result of the summation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurnr, i, i2, i3, nnodes, inusol, inunkp, &
              k, numsrf, ichint, ichfun, number, nodp, ih, idim, iconct, &
              nconctct, nocontct, istart
      integer, allocatable :: nodes(:)
      double precision f

!     f          Value of the function to be multiplied by
!     i          Counting variable
!     i2         2 if ndim>= 2 else 1
!     i3         3 if ndim>= 3 else ndim
!     ichfun     indication of the value of f. Possibilities:
!                0   f = 1
!                >0  f = func ( ichfun, x, y, z )
!                    at this moment only the combination f#1 and ichint=1
!                    is permitted
!     ichint     indicates the kind of integral to be computed.
!                possibilities:
!                 1  /  f u  ds
!                 2  /  f  u.n  ds
!                 3  /  f  u.t  ds
!                 4  /  f  n. sigma .n  ds
!                 5  /  f  n. sigma .t  ds
!                 6  /  f  n. sigma     ds
!                 7  /  f  u  n  ds
!                 8  sum f(i) u(i)
!     iconct     Sequence number of contact
!     idim       This parameter indicates what type of boundary
!                integral must be computed
!                Possible values:
!                1: curves
!                2: surfaces
!                3: contact
!                4: no_contact
!     ih         Help variable to store a constant temporarily
!     inunkp     Number of degrees of freedom in first point
!     inusol     Position of unknown in array USOL
!     istart     First position used at a part
!     isurnr     Surface sequence number
!     k          Counting variable
!     nconctct   Number of points with contact
!     nnodes     Number of nodes in nodes
!     nocontct   Number of points that make no contact
!     nodes      Work array in which the node numbers of a surface are
!                stored. Length npoint
!     nodp       Nodal point sequence number
!     number     The number-th unknown in each nodal point of the array
!                is considered. If the array is complex number=2*i-1
!                corresponds to the real part of array i and number=2*i
!                to the imaginary part of array i
!     numsrf     number of parts where surfaces are given for generation
!                of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision func

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FUNC           Computes the value of the vector in a point as function of
!                    the co-ordinates
!     SEPSURNOD      Store all nodes in the surfaces ifirst to ilast in array
!                    nodes
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     146   surface number has incorrect value
!     451   ichois has incorrect value
!    1513   Too much surfaces in isurfs
! **********************************************************************
!
!                       PSEUDO CODE
!
!     value := 0
!     For all surfaces do
!        Compute all nodes in the surface
!        Compute the value of u in these nodes
!        Compute the value of f in these nodes
!        Add u f for all nodes to value
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0088' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0088'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- checks of the parameters

      ichfun = iintbn(3)
      idim   = iintbn(5)
      number = iintbn(6)
      if ( idim>2 ) then
         numsrf = 1
         iconct = iintbn(7)
      else
         numsrf = iintbn(7)
         iconct = 0
      end if
      if ( ichfun<0 .or. ichfun>1000 ) then
         call errint ( ichfun, 1 )
         call errint ( ichfun, 2 )
         call errint ( ichint, 3 )
         call errsub ( 451, 3, 0, 0 )
      end if
      if ( numsrf>100 ) then
         call errint ( numsrf, 1 )
         call errint ( 100, 2 )
         call errsub ( 1513, 2, 0, 0 )
      end if
      if ( iconct>maxcontc ) go to 1000
      if ( ierror/=0 ) go to 1000

      value = 0d0
      if ( ndim==1 ) then
         i2 = 1
         i3 = 1
      else if ( ndim==2 ) then
         i2 = 2
         i3 = 2
      else
         i2 = 2
         i3 = 3
      end if

      allocate ( nodes(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'ichfun , idim, number', ichfun , idim, number
         write(irefwr,1) 'numsrf, iconct, ndim', numsrf, iconct, ndim
         write(irefwr,1) 'i2, i3', i2, i3
      end if  ! ( debug )

      do k = 1, numsrf

         if ( idim==2 ) then

!        --- Standard case (surfaces)

            isurnr = isurfs(k)
            if ( isurnr<=0 .or. isurnr>nsurfs ) then
               call errint ( isurnr, 1 )
               call errint ( nsurfs, 2 )
               call errsub ( 146, 2, 0, 0 )
            end if

!           --- Compute all nodes in the surface

            nnodes = npoint
            call sepsurnod ( isurnr, isurnr, nodes, nnodes )

         else if ( idim==3 ) then

!        --- Contact points

            nconctct = kprobvp1(iconct,1)
            istart = kprobvp1(iconct,3)-1
            do i = 1, nconctct
               nodes(i) = kprobvp2(istart+i)
            end do
            nnodes = nconctct

            if ( debug ) call prinin ( nodes, nnodes, 'nodes' )

         else

!        --- No-contact points

            nconctct = kprobvp1(iconct,1)
            nocontct = kprobvp1(iconct,2)
            istart = kprobvp1(iconct,3)-1+nconctct
            do i = 1, nocontct
               nodes(i) = kprobvp2(istart+i)
            end do
            nnodes = nocontct

         end if

!        --- Compute the value of u in these nodes
!            Compute the value of f in these nodes

         if ( indprpi/=0 .and. ichfun==0 ) then

!        --- Use kprob part p, f=1

            do i = 1, nnodes
               nodp = nodes(i)
               ih = kprobpi(nodp,number)
               if ( ih/=0 ) value = value + usol(ih)
            end do

         else if ( indprpi/=0 ) then

!        --- Use kprob part p

            do i = 1, nnodes
               nodp = nodes(i)
               ih = kprobpi(nodp,number)
               if ( ih/=0 ) then
                  f = func ( ichfun,coor(1,nodp), coor(i2,nodp), &
                             coor(i3,nodp) )
                  value = value + usol(ih)*f
               end if
            end do

         else if ( indprfi/=0 .and. ichfun==0 ) then

!        --- Use kprob part f, f=1

            do i = 1, nnodes
               nodp = nodes(i)
               inusol = kprobfi(nodp)
               inunkp = kprobfi(nodp+1) - inusol
               if ( number<=inunkp ) &
                  value = value + usol(inusol+number)
            end do

         else if ( indprfi/=0 ) then

!        --- Use kprob part f

            do i = 1, nnodes
               nodp = nodes(i)
               inusol = kprobfi(nodp)
               inunkp = kprobfi(nodp+1) - inusol
               if ( number<=inunkp ) then
                  f = func ( ichfun,coor(1,nodp), coor(i2,nodp), &
                             coor(i3,nodp) )
                  value = value + usol(inusol+number)*f
               end if
            end do

         else if ( ichfun==0 ) then

!        --- One unknown per point, f=1

            do i = 1, nnodes
               nodp = nodes(i)
               value = value + usol(nodp)
            end do

         else

!        --- One unknown per point

            do i = 1, nnodes
               nodp = nodes(i)
               f = func ( ichfun,coor(1,nodp), coor(i2,nodp), &
                          coor(i3,nodp) )
               value = value + usol(nodp)*f
            end do

         end if

      end do

      deallocate ( nodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End er0088'

      end if  ! ( debug )
      call erclos ( 'er0088' )

      end subroutine er0088
