      subroutine intmap3d14c ( kmeshc, inelem, nunkploc, &
                               number, usol, usolcop, ielgrp, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solution in mesh with non-constant number of unknowns
!     per point to solution array with constant number of unknowns
!     The solution to be copied is not present in all nodes
!     Center nodes of elements and faces are skipped
!     Elements of shape 14
!     Complex version
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     3d
!     hexahedron
!     quadratic
!     vertex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduletypes
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kmeshc(inpelm,*), inelem, nunkploc, number, &
              ielgrp
      double complex usol(*), usolcop(nunkploc,*)

!     ielgrp         i    element group number
!     inelem         i    Number of elements in a specific element group
!     inpelm         i    Number of nodes in elements
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!                         kmeshc is only related to this element group
!     number         i    Thefirst unknown to be copied has sequence number
!                         number
!     nunkploc       i    Number of unknowns in a point that must be copied
!     usol           i    Solution vector
!     usolcop       i/o   Solution vector to which the solution must be
!                         copied. usolcop has nunkploc unknowns per node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ielem, node, node2, node3, imidside(16,3), k1, k2, &
              k3, k, ih
      logical present, center

!     center         If true the center is filled
!     i              Counting variable
!     ielem          Element number
!     ih             Help variable to store a constant
!     imidside       Array of size 12 * 3 containing:
!                    (i,1):  i^th mid side point
!                    (i,2) and (i,3) neighbor vertices of the midside point
!     j              Counting variable
!     k              Counting variable
!     k1             Relative node number of midside point
!     k2             Relative node number of first neighbor
!     k3             Relative node number of second neighbor
!     node           Node number
!     node2          Node number of first neighbor
!     node3          Node number of second neighbor
!     present        If true the unknowns are present in the midside points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     583   The required degree of freedom is not available in the vertices
!           of the element
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intmap3d14c' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intmap3d14c'
         write(irefwr,1) 'nunkploc, number', nunkploc, number
         call prinin ( ivertx14, 8, 'ivertx14' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if unknown is present in all vertices

      do i = 1, 8

         node = kmeshc(ivertx14(i),1)

         do j = 1, nunkploc

            if ( kprobpi(node,j+number-1)==0 ) then

!           --- Unknown not present in vertex

               call errint ( j, 1 )
               call errint ( nunkploc+number-1, 2 )
               call errint ( ielgrp, 3 )
               call errwar ( 583, 3, 0, 0 )
               go to 1000

            end if  ! ( kprobpi(node,j+number-1)==0 ) then

         end do  ! j = 1, nunkploc

      end do  ! i = 1, 4

!     --- Check if unknown is present in the midside points
!         First fill array imidside

      imidside(1,1) = 10    ! mid points at midside layer
      imidside(2,1) = 12    ! verices of this layer
      imidside(3,1) = 16
      imidside(4,1) = 18
      do j = 1, 4
         imidside(j,2) = imidside(j,1)-9
         imidside(j,3) = imidside(j,1)+9
      end do  ! j = 1, 4

      do j = 1, 3  ! j is layer
         do k = 1, 4  ! node per layer
            imidside(k+j*4,1) = 2*k+(j-1)*9  ! midside point
         end do  ! k = 1, 4  ! node per
         imidside(1+j*4,2) = 1+(j-1)*9  ! neighbor of 2
         imidside(1+j*4,3) = 3+(j-1)*9  ! neighbor of 2
         imidside(2+j*4,2) = 1+(j-1)*9  ! neighbor of 4
         imidside(2+j*4,3) = 7+(j-1)*9  ! neighbor of 4
         imidside(3+j*4,2) = 3+(j-1)*9  ! neighbor of 6
         imidside(3+j*4,3) = 9+(j-1)*9  ! neighbor of 6
         imidside(4+j*4,2) = 7+(j-1)*9  ! neighbor of 8
         imidside(4+j*4,3) = 9+(j-1)*9  ! neighbor of 8
      end do  ! j = 1, 3
      if ( debug ) call prinin1 ( imidside, 3, 12, 'imidside' )

      present = .true.
      do i = 1, 16

         node = kmeshc(imidside(i,1),1)

         do j = 1, nunkploc

            if ( kprobpi(node,j+number-1)==0 ) present = .false.

         end do  ! j = 1, nunkploc

      end do  ! i = 1, 16

!     --- Check if unknown is present in the center

      center = .true.
      node = kmeshc(14,1)
      do j = 1, nunkploc

         if ( kprobpi(node,j+number-1)==0 ) center = .false.

      end do  ! j = 1, nunkploc

      if ( present ) then

!     --- Copy solution in vertices and midside points

         do ielem = 1, inelem

            do i = 1, 27

               if ( i/=14 ) then

!              --- Center point is skipped

                  node = kmeshc(i,ielem)

                  do j = 1, nunkploc
                     usolcop(j,node) = usol(kprobpi(node,number-1+j))
                  end do  ! j = 1, nunkploc

               end if  ! ( .not. center .and. i/=14 ) then

            end do  ! i = 1, 4

         end do  ! ielem = 1, inelem

      else

!     --- Copy solution in vertices

         do ielem = 1, inelem

            do i = 1, 8

               node = kmeshc(ivertx14(i),ielem)

               do j = 1, nunkploc
                  usolcop(j,node) = usol(kprobpi(node,number-1+j))
               end do  ! j = 1, nunkploc

            end do  ! i = 1, 8

         end do  ! ielem = 1, inelem

!        --- Interpolate solution in midside points

         do i = 1, 16

            k1 = imidside(i,1)
            k2 = imidside(i,2)
            k3 = imidside(i,3)
            do ielem = 1, inelem

               node = kmeshc(k1,ielem)
               node2 = kmeshc(k2,ielem)
               node3 = kmeshc(k3,ielem)

               do j = 1, nunkploc
                  ih = kprobpi(node,number-1+j)
                  if ( ih>0 ) then
                     usolcop(j,node) = usol(ih)
                  else
                     usolcop(j,node) = 0.5d0* &
                        (usolcop(j,node2)+usolcop(j,node3))
                  end if  ! ( kprobpi(node,number-1+j)>0 )
               end do  ! j = 1, nunkploc

            end do  ! ielem = 1, inelem

         end do  ! i = 1, 16

      end if ! ( present )
      if ( center ) then

!     --- center point can be copied

         do ielem = 1, inelem

            node = kmeshc(14,ielem)

            do j = 1, nunkploc
               usolcop(j,node) = usol(kprobpi(node,number-1+j))
            end do  ! j = 1, nunkploc

         end do  ! ielem = 1, inelem

      else

!     --- center point must be interpolated

         do ielem = 1, inelem

            node = kmeshc(14,ielem)
            node2 = kmeshc(11,ielem)
            node3 = kmeshc(17,ielem)

            do j = 1, nunkploc
               ih = kprobpi(node,number-1+j)
               if ( ih>0 ) then
                  usolcop(j,node) = usol(ih)
               else
                  usolcop(j,node) = 0.5d0* &
                     (usolcop(j,node2)+usolcop(j,node3))
               end if  ! ( kprobpi(node,number-1+j)>0 )
            end do  ! j = 1, nunkploc

         end do  ! ielem = 1, inelem

      end if  ! ( center ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intmap3d14c'

      end if  ! ( debug )
      call erclos ( 'intmap3d14c' )

      end
