      subroutine intmshm3 ( kmesh1, kmesh2, map, ichois )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.7  date 30-11-2012 Long integers
!        version  2.6  date 02-03-2012 Jump in case of error
!        version  2.5  date 20-08-2003 Extra debug statements
!
!   copyright (c) 1998-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a formal interpolation from one mesh to another.
!     In fact the mapping between the meshes is computed, &
!     without knowing the function (3D version)
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmeshv
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh1, kmesh2, ichois
      type (mapping) :: map

!     ichois     i    Choice parameter
!                     Possible values:
!                     0,1: All points are interpolated; Points outside the mesh
!                          are extrapolated
!                     2:   Only points inside the old mesh or very close to the
!                          old mesh are interpolated, the other points are left
!                          unchanged
!     kmesh1     i    Standard SEPRAN array containing information of mesh 1
!     kmesh2     i    Standard SEPRAN array containing information of mesh 2
!     map       i/o   contains information of the mapping
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical check
      integer npoint, nsplit, minnodol, minnodnw, iseqkm1, iseqkm2, &
              maxnodol, maxnodnw, ipart, nx, ny, lenkmtri, nodnew, &
              nodold, nz, i
      double precision xmin, xmax, ymin, ymax, ds, dsnw, eps, xminnw, &
                       xmaxnw, yminnw, ymaxnw, zmin, zmax, zminnw, &
                       zmaxnw, dx, dy, dz, dxnw, dynw, dznw
      integer, allocatable :: ijcrol(:), ijcrnw(:), npntij(:), pntcij(:)
      integer, pointer :: kmstri(:)

!     check          Indicates if the file exist (true) or not
!     ds             Minimum distance between two points along a curve
!     dsnw           Minimum distance between two points along a curve
!                    (new mesh)
!     dx             Distance in the cells in x-direction
!     dxnw           Distance in the cells in x-direction (new mesh)
!     dy             Distance in the cells in y-direction
!     dynw           Distance in the cells in x-direction (new mesh)
!     dz             Distance in the cells in z-direction
!     dznw           Distance in the cells in x-direction (new mesh)
!     eps            Local accuracy
!     i              Counting variable
!     ijcrnw         ij cell number coornw
!     ijcrol         ij cell number coorol
!     ipart          Sequence number of part
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     kmstri         kmeshc subdivided
!     lenkmtri       Length of array kmeshtri
!                    If 0, no subdivision is required
!     maxnodnw       Largest node number in part of new mesh
!     maxnodol       Largest node number in part of old mesh
!     minnodnw       Smallest node number in part of new mesh
!     minnodol       Smallest node number in part of old mesh
!     nodnew         Number of nodes in part of new mesh
!     nodold         Number of nodes in part of old mesh
!     npntij         Contains number of coordinates in ij-cell accumulated
!                    npntij(1) = 0
!                    npntij((i-1)*nx+j+1) - npntij((i-1)*nx+j) is number
!                    of points in cell ij
!     npoint         Number of nodal points
!     nsplit         Number of parts into which the mesh has been splitted
!     nx             number of points in x direction
!     ny             number of points in y direction
!     nz             number of points in z direction
!     pntcij         Node numbers corresponding to cells
!     xmax           Maximum x-value
!     xmaxnw         Maximum x-value (new mesh)
!     xmin           Minimum x-value
!     xminnw         Minimum x-value (new mesh)
!     ymax           Maximum y-value
!     ymaxnw         Maximum y-value (new mesh)
!     ymin           Minimum y-value
!     yminnw         Minimum y-value (new mesh)
!     zmax           Maximum z-value
!     zmaxnw         Maximum z-value (new mesh)
!     zmin           Minimum z-value
!     zminnw         Minimum z-value (new mesh)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PR2031         Compute xmin, xmax, ymin, ymax, zmin, zmax from array coor
!     PR2032         Compute mapping for all new points that are present in this
!                    part of the mesh
!     PR2033         Fill array ijkcoor with the ijk-cell number of the
!                    co-ordinates
!     PR2034         compute number of nodes for each ijk combination and store
!                    in array npntij
!     PR2035         compute number of nodes for each ijk combination and store
!                    in array npntij
!     PR2036         Compute length of array kmeshtri, which is used to
!                    subdivide an existing 3D mesh into four-point tetrahedrons
!     PR2037         Subdivide a mesh consisting of tetrahedrons and/or
!                    hexahedrons into four-point tetrahedrons
!     PR2038         Find all points in imap that have not been treated yet and
!                    interpolate those points
!     PR2040         Compute xmin, xmax, ymin, ymax from array coor
!     PRININ         print 1d integer array
!     PRININ2        Print an integer two-dimensional array
!     PRINRL2        Print a real two-dimensional array
!     SEPCREATEIBUF  Create permanent space in array IBUFFR
!     SEPCREATERBUF  Create permanent space in array BUFFER
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     429   The number of disjoint parts in both meshes are unequal
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Create arrays imap, rmap
!   Clear arrays imap, rmap
!   Detect the number of sub parts the mesh consists of
!   Check if the number of sub parts of both meshes are identical
!   For each sub part ipart do
!      Compute xmin, xmax, ymin, ymax, zmin, zmax and ds from kmeshm_old
!      Correct xmin, xmax, ymin, ymax, zmin, zmax and ds from kmeshm_new
!      Compute nx, ny and nz
!      Define new rectangular grid by
!           x_i = xmin+(i-1)*ds   (i=1(1)nx+1)
!           y_j = ymin+(j-1)*ds   (j=1(1)ny+1)
!           z_j = zmin+(j-1)*ds   (j=1(1)nz+1)
!           cell_ij = (x_i, x_i+1) * (y_j, y_j+1) * (z_j, z_j+1)
!      For all nodes in coornw compute cell number i,j,k
!      For all cells i,j,k detect all nodes in coornw
!      For all nodes in coorol compute cell number i,j,k
!      Create subelement mesh consisting of triangles only
!      For all elements in subelement mesh
!          Compute mapping
!   For all points that have not been treated
!     Find points in old mesh that are the most close to new points
!     Interpolate
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intmshm3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intmshm3'

      end if

      call sepfillkmesh ( kmesh1 )

      iseqkm1 = iseqkm
      iseqkm2 = kmesh2
      npoint = km(iseqkm2)%kmesh(8)

      if ( debug ) write(irefwr,*) 'npoint1, npoint2 ', &
                                    npoint, km(iseqkm2)%kmesh(8)

!     --- Create arrays imap, rmap

      if ( associated(map%imap) ) deallocate ( map%imap, map%rmap )
      call sepclear ( map )
      allocate ( map%imap(npoint,6), map%rmap(npoint,4) )
      imap => map%imap
      rmap => map%rmap
      imap2 => imap
      rmap2 => rmap

!     --- Clear arrays imap, rmap

      imap = 0
      rmap = 0d0
      map%iseqkm1 = iseqkm1
      map%iseqkm2 = iseqkm2

!     --- Detect the number of sub parts the mesh consists of

      nsplit = max(1,km(iseqkm1)%kmesh(44))
      if ( max(1,km(iseqkm2)%kmesh(44))/=nsplit ) then

!     --- Mesh1 and mesh2 do not consist of the same number of sub parts

         call errint ( nsplit, 1 )
         call errint ( max(1,km(iseqkm2)%kmesh(44)), 2 )
         call errsub ( 429, 2, 0, 0 )

      end if

!     --- For each sub part ipart do

      minnodol = 1
      minnodnw = 1
      maxnodol = km(iseqkm1)%kmesh(8)
      maxnodnw = km(iseqkm2)%kmesh(8)
      do ipart = 1, nsplit
         if ( nsplit>1 ) then
            maxnodol = km(iseqkm1)%kmeshk(km(iseqkm1)%kmesh(7)+ipart)
            maxnodnw = km(iseqkm2)%kmeshk(km(iseqkm2)%kmesh(7)+ipart)
         end if

!        --- Compute xmin, xmax, ymin, ymax, zmin, zmax and ds from kmeshm_old

         call pr2031 ( xmin, xmax, ymin, ymax, zmin, &
                       zmax, ds, minnodol, maxnodol, dx, dy, dz )

!        --- Correct xmin, xmax, ymin, ymax, zmin, zmax and ds from kmeshm_new

         if ( km(iseqkm2)%kmesh(27)/=0 ) then

!        --- Standard case

            call sepfillkmesh(kmesh2)
            call pr2031 ( xminnw, xmaxnw, yminnw, &
                          ymaxnw, zminnw, zmaxnw, dsnw, minnodnw, &
                          maxnodnw, dxnw, dynw, dznw )
            call sepfillkmesh(kmesh1)

         else

!        --- intmshm3 is called by intcoor

            call pr2040 ( km(iseqkm2)%coor, xminnw, xmaxnw, yminnw, &
                          ymaxnw, zminnw, zmaxnw, dsnw, minnodnw, &
                          maxnodnw )
            dxnw = dx
            dynw = dy
            dznw = dz

         end if
         xmin = min(xmin,xminnw)
         ymin = min(ymin,yminnw)
         zmin = min(zmin,zminnw)
         xmax = max(xmax,xmaxnw)
         ymax = max(ymax,ymaxnw)
         zmax = max(zmax,zmaxnw)
         xmin = xmin - 0.1d-5*(xmax-xmin)
         ymin = ymin - 0.1d-5*(ymax-ymin)
         zmin = zmin - 0.1d-5*(zmax-zmin)
         xmax = xmax + 0.1d-5*(xmax-xmin)
         ymax = ymax + 0.1d-5*(ymax-ymin)
         zmax = zmax + 0.1d-5*(zmax-zmin)
         ds = min ( ds, dsnw )
         dx = min ( dx, dxnw )
         dy = min ( dy, dynw )
         dz = min ( dz, dznw )
         eps = 1d-5*ds
         nx = min(int((xmax-xmin)/dx)+1,40)
         ny = min(int((ymax-ymin)/dy)+1,40)
         nz = min(int((zmax-zmin)/dz)+1,40)
         dx = (xmax-xmin)/(nx+1)
         dy = (ymax-ymin)/(ny+1)
         dz = (zmax-zmin)/(nz+1)
         if ( debug ) then
            write(irefwr,*) 'xmin, xmax, ymin, ymax, zmin, zmax ', &
                             xmin, xmax, ymin, ymax, zmin, zmax
            write(irefwr,*) 'nx ,ny, nz ', nx ,ny, nz
            write(irefwr,*) 'dx ,dy, dz ', dx ,dy, dz
         end if

         call pr2036 ( lenkmtri )

!        --- Define new rectangular grid by
!              x_i = xmin+(i-1)*dx   (i=1(1)nx+1)
!              y_j = ymin+(j-1)*dy   (j=1(1)ny+1)
!              z_j = zmin+(j-1)*dz   (j=1(1)nz+1)
!              cell_ijk = (x_i, x_i+1) * (y_j, y_j+1) * (z_j, z_j+1)

         nodnew = maxnodnw-minnodnw+1
         nodold = maxnodol-minnodol+1

         allocate ( ijcrol(3*nodold), ijcrnw(3*nodnew), &
                    npntij(nx*ny*nz+1), pntcij(max(nodold,nodnew)), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, 2*nodold+2*nodnew, 'ijcrol' )
         if ( ierror/=0 ) go to 1000
         if ( debug ) write(irefwr,*) 'nodnew, nodold ', nodnew, nodold

         if ( lenkmtri==0 ) then

!        --- lenkmtri = 0, use old kmeshc

            kmstri => km(iseqkm1)%kmeshc

         else

!        --- lenkmtri > 0, new array

            allocate ( kmstri(lenkmtri), stat = error )
            if ( error/=0 ) call eralloc ( error, lenkmtri, 'kmstri' )
            if ( ierror/=0 ) go to 1000

         end if  ! ( lenkmtri==0 )

!        --- For all nodes in coornw compute cell number i,j,k

         call pr2033 ( km(iseqkm2)%coor, xmin, ymin, zmin, dx, dy, dz, &
                       ijcrnw, minnodnw, maxnodnw, nx, ny, &
                       nz )

!        --- For all cells i,j,k detect all nodes in coornw

         call pr2034 ( ijcrnw, minnodnw, maxnodnw, &
                       npntij, nx, ny, nz )
         call pr2035 ( ijcrnw, minnodnw, maxnodnw, &
                       npntij, nx, ny, nz, pntcij )

!        --- For all nodes in coorol compute cell number i,j,k

         call pr2033 ( km(iseqkm1)%coor, xmin, ymin, zmin, dx, dy, dz, &
                       ijcrol, minnodol, maxnodol, nx , ny, &
                       nz )

         if ( debug ) write(irefwr,*) 'lenkmtri ', lenkmtri
         if ( lenkmtri>0 ) then

!        --- lenkmtri>0, Create subelement mesh consisting of tetrahedrons only

            call pr2037 ( kmstri, minnodol, maxnodol, inelem )

         else

            inelem = km(iseqkm1)%kmesh(9)

         end if

!        --- Compute mapping for all elements in subelement mesh

         call pr2032 ( kmstri, km(iseqkm1)%coor, &
                       km(iseqkm2)%coor, inelem, ijcrol, npntij, &
                       pntcij, nx, ny, nz, npoint, eps )
         if ( debug ) call prinin ( imap, npoint, 'imap-1' )
         if ( ierror/=0 ) go to 1000

!        --- Check if there are any points in new mesh that have not been
!            treated yet

         check = .false.
         if ( ichois<=1 ) then

!        --- ichois = 1, all points must get a value

            do i = minnodnw, maxnodnw
               if ( imap(i,1)==0 ) then
                  check = .true.
                  go to 100
               end if
            end do

         end if

100      if ( check ) then

!        --- For all cells i,j detect all nodes in coornw

            call pr2034 ( ijcrol, minnodol, maxnodol, &
                          npntij, nx, ny, nz )
            call pr2035 ( ijcrol, minnodol, maxnodol, &
                          npntij, nx, ny, nz, pntcij )

!           --- For all points that have not been treated
!               Find points in old mesh that are the most close to new points
!               Interpolate

            call pr2038 ( npoint, &
                          ijcrnw, npntij,pntcij, minnodnw, maxnodnw, &
                          km(iseqkm1)%coor, km(iseqkm2)%coor, nx, ny, nz, &
                          km(iseqkm1)%kmesh(8) )
         end if
         minnodol = maxnodol+1
         minnodnw = maxnodnw+1

         deallocate ( ijcrol, ijcrnw, npntij, pntcij, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'ijcrol' )

         if ( lenkmtri>0 ) then
            deallocate ( kmstri, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'ijcrol' )
         end if  ! ( lenkmtri>0 )

      end do

1000  if ( debug ) then

!     --- Debug information

         call prinin2 ( imap, npoint, 6, 'imap' )
         call prinrl2 ( rmap, npoint, 4, 'rmap' )
         write(irefwr,*) 'End intmshm3'

      end if
      call erclos ( 'intmshm3' )

      end subroutine intmshm3
