      subroutine intsolmp ( meshin, meshout, maplc, isol1, &
                            isol2, minmax, ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.3  date 21-11-2012 Long integers
!        version  4.2  date 05-04-2005 Fortran 90
!        version  4.1  date 04-08-2004 Set points outside region to 0
!
!   copyright (c) 1997-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate array isol1 in array isol2 using the formal interpolation
!     as stored in array maplc
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmeshv
      use sepmodulemain
      use sepmodulekprob
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshin, meshout, isol1, isol2, minmax, ichoice
      type (mapping) :: maplc

!     ichoice    i    Choice parameter consisting of 2 parts ichoice_orig and
!                     del_outer according to
!                     ichoice = ichoice_orig+10*del_outer
!                     ichoice_orig:
!                     Choice parameter indicating if the new solution vector
!                     has the same structure as isol1 (0) or that it must be
!                     of type 110 (1)
!                     del_outer:
!                     if 1 all nodes with negative weights, i.e. outside the
!                     region are not interpolated. The mapping array is adapted
!                     if 0 nothing happens
!     isol1      i    Standard SEPRAN array containing information of the vector
!                     to be interpolated
!     isol2      o    Standard SEPRAN array containing information of the
!                     interpolated vector
!     maplc     i/o   contains information of the mapping
!     meshin         i    Sequence number of first mesh to be used
!     meshout        i    Sequence number of last mesh to be used
!     minmax     i    If 1 the interpolated solution is restricted to the
!                     range uold_iun(min)<=unew_iun<=uold_iun(max)
!                     for each physical unknown iun, if 0 no restriction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icompl,  inunkp, ivec, itypvc, &
              npointol, npointnw, nsplit, iseqkm1, iseqkm2
      logical compl, clear
      integer, pointer :: kmesh1, kmesh2, kprob1, kprob2

!     clear          if true the array to which must be interpolated is new.
!                    This array will be cleared
!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     inunkp         Number of degrees of freedom in isol1
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     kmesh1         Standard SEPRAN array containing information of mesh 1
!     kmesh2         Contains copy of old mesh
!     kprob1         Standard SEPRAN array containing information of the
!                    problem corresponding to mesh 1
!     kprob2         Contains copy of old problem
!     npointnw       Number of points in new mesh
!     npointol       Number of points in old mesh
!     nsplit         Number of parts into which the mesh has been splitted
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERMAPDELOUTER  Remove all entries in imap/rmap with negative weights in
!                    order to prevent extrapolation outside the original domain
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     INTSOLMP1      Map contents of solution array "usol1" into array "usol2"
!                    in case the number of degrees of freedom per array is
!                    constant
!     INTSOLMP2      Map contents of solution array "usol1" in array "usol2" in
!                    case the number of degrees of freedom in array usol1 is not
!                    constant, but in usol2 is constant
!     INTSOLMP3      Map contents of solution array "usol1" in array "usol2" in
!                    case the number of degrees of freedom in arrays usol1 and
!                    usol2 is not constant
!     PRININ         print 1d integer array
!     PRINMAP        Print the contents of the mapping array map
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2655   indprfi # 0 and indprfo = 0
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intsolmp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intsolmp'
         call ertrace
         write(irefwr,*) 'ichoice, minmax ', ichoice, minmax
         write(irefwr,1) 'isol1, isol2', isol1, isol2
  1      format ( a, 1x, (10i6) )

      end if
      kmesh1 => kmesh2d(meshin)
      kmesh2 => kmesh2d(meshout)
      kprob1 => kprob2d(meshin)
      kprob2 => kprob2d(meshout)

      iseqkm1 = kmesh1
      iseqkm2 = kmesh2

!      if ( debug ) call prinmap ( maplc, kmesh2 )

!     --- Activate array isol1

      kprob => kprob1

      call sepactsolbf2 ( isol1 )
      inunkp = max(nunkpi,nphysi)
      if ( ierror/=0 ) go to 1000
      if ( debug ) &
         write(irefwr,*) 'indprfi, indprpi, inunkp, nphysi ', &
                          indprfi, indprpi, inunkp, nphysi
      compl = ks(isol1)%compl==1
      icompl = ks(isol1)%compl

      if ( mod(ichoice,10)==0 ) then

!     --- standard case, copy info from isol1

         iprob = ks(isol1)%iprob
         ivec = ks(isol1)%ivec
         itypvc = ks(isol1)%typevec

      else

!     --- special case for intcoorbf

         iprob = 1
         ivec = 0
         itypvc = 110

      end if
      clear = isol2<=1 .or. isol2>lastsol
      if (debug ) write(irefwr,1) 'isol2',  isol2
      iseqkm = iseqkm2
      iseqkp = kprob2
      call sepcreasolbf1 ( isol2, itypvc, ivec, 'usol_new', icompl )
      if ( clear ) usoltemp = 0d0

      kprob => kprob1
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      if (debug ) then
         write(irefwr,*) 'clear, nunkpo ', clear, nunkpo
         write(irefwr,1) 'isol2',  isol2
      end if  ! (debug )

      npointol = km(iseqkm1)%kmesh(8)
      npointnw = km(iseqkm2)%kmesh(8)

      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'indprfi, indprfo ',indprfi, indprfo

      if ( ichoice/10==1 ) then

!     --- ichoice = 10 or 11, set imap/rmap 0 for points outside the region

         imap => maplc%imap
         rmap => maplc%rmap
         call ermapdelouter ( imap, rmap, npointnw )

      end if  ! ( ichoice/10==1 )

      if ( indprfi==0 .and. indprfo==0 ) then

!     --- number of unknowns per point constant
!         interpolate usol1 into usol2 using array maplc

         call intsolmp1 ( isol1, isol2, maplc, inunkp, &
                          npointnw, nunkpo, minmax, compl )

      else if ( indprfi/=0 ) then

!     --- Number of unknowns per point not constant

         nsplit = max(1,km(iseqkm1)%kmesh(44))

!        --- Actual mapping

         if ( indprfo==0 ) then

!        --- Number of unknowns constant in usol2

            call intsolmp2 ( npointol, npointnw, compl, nsplit, inunkp, &
                             iseqkm1, minmax, isol1, isol2, iseqkm2, &
                             maplc )

         else

!        --- Number of unknowns not constant in usol2

            call intsolmp3 ( npointol, npointnw, compl, nsplit, inunkp, &
                             iseqkm1, minmax, isol1, isol2, iseqkm2, &
                             maplc )

         end if  ! ( indprfo==0 )

      else

!     --- indprfi # 0, but indprfo = 0, not yet implemented

         call errsub ( 2655, 0, 0, 0 )

      end if  ! ( indprfi==0 ... )

1000  if ( debug ) write(irefwr,*) 'End intsolmp'
      call erclos ( 'intsolmp' )

      end subroutine intsolmp
