      subroutine intsolmp3 ( npointol, npointnw, compl, nsplit, &
                             inunkp, iseqkm1, minmax, isol1, isol2, iseqkm2, &
                             map )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 22-06-2014 New definition of imap(5)
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 14-06-2004
!
!   copyright (c) 2004-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map contents of solution array "usol1" in array "usol2" in case the
!     number of degrees of freedom in arrays usol1 and usol2 is not constant
!     The mapping is stored in array imap
!     Help subroutine of intsolmp
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      use sepmodulemap
      use sepmodulekprob
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npointol, npointnw, nsplit, inunkp, iseqkm1, minmax, &
              isol1, isol2, iseqkm2
      logical compl
      type (mapping) :: map

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
!     inunkp         i    Number of degrees of freedom in point
!     iseqkm1        i    Sequence number with respect to km of first mesh
!     iseqkm2        i    Sequence number with respect to km of second mesh
!     isol1          i    Standard SEPRAN array containing information of the
!                         vector to be interpolated
!     isol2          o    Interpolated solution
!     map           i/o   contains information of the mapping
!     minmax         i    If 1 the interpolated solution is restricted to the
!                         range uold_iun(min)<=unew_iun<=uold_iun(max)
!                         for each physical unknown iun, if 0 no restriction
!     npointnw       i    Number of points in new mesh
!     npointol       i    Number of points in old mesh
!     nsplit         i    Number of parts into which the mesh has been splitted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iunk, minnodol, minnodnw, maxnodol, maxnodnw, ipart, &
              npoints, icompl, icount
      integer, allocatable :: iwork(:)
      double precision, pointer :: solcop1(:),  solcop2(:)

!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     icount         number of nodes that are interpolated
!     ipart          Sequence number of part
!     iunk           Position of degree of freedom in node
!     iwork          integer work array of length: number of nodes in new
!                    mesh
!                    If iwork(i) = 1, node has been treated,
!                    if iwork(i) = 0 it is free
!     maxnodnw       Largest node number in part of new mesh
!     maxnodol       Largest node number in part of old mesh
!     minnodnw       Smallest node number in part of new mesh
!     minnodol       Smallest node number in part of old mesh
!     npoints        Number of points
!     solcop1        Array to be interpolated
!     solcop2        Result of interpolation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTCOP00       Copy usol into usolcop
!     INTCOP01       interpolate "intermediate" points
!     INTCOP02       Copy usolcop into usol
!     INTCOP04       Interpolate usol1 into usol2 using the information stored
!                    in the arrays imap and rmap
!                    Special case that not all nodes are filled
!     INTCOPCOMPL00  Copy usol into usolcop (complex case)
!     INTCOPCOMPL01  interpolate "intermediate" points (complex case)
!     INTCOPCOMPL02  Copy usolcop into usol (complex case)
!     INTCOPCOMPLUN  Interpolate usol1 into usol2 (complex case)
!     INTCOPUN       Interpolate usol1 into usol2
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     987   No unknowns in part of old mesh but there are in new mesh
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for iunk := 1 (1) nunkp do
!        for ipart := 1 (1) nsplit do
!           minnode := smallest node number in part
!           maxnode := largest node number in part
!           npoints := maxnode-minnode+1
!           Clear arrays iwork and solcop1
!           icount := 0
!           Copy unknown iunk from usol1 into solcop1
!           iwork(i):=1 for all copied points
!           icount := number of copied points
!           if ( icount#npoints ) then
!              for ielgrp := 1 (1) nelgrp do
!                 if ( itype#0 ) then
!                    if ( iunk not in all nodes of ielgrp ) then
!                       for ielem := 1 (1) nelem_ielgrp do
!                          interpolate points and update iwork and icount
!           if ( icount#0 ) then
!              if ( icount#npoints ) then
!                 error
!              else ( icount=npoints )
!                 interpolate array solcop1 into solcop2
!                 extract unknown from solcop2 into usol2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intsolmp3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intsolmp3'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set starting addresses of work arrays (see intsolmp)

      if ( compl ) then
         icompl = 2
      else
         icompl = 1
      end if  ! ( compl )

      allocate ( iwork(npointol), solcop1(npointol*icompl), &
                 solcop2(npointnw*icompl), stat = error )
      if ( error/=0 ) call eralloc ( error, npointol+npointnw, 'iwork' )
      if ( ierror/=0 ) go to 1000
      if ( debug ) &
         write(irefwr,1) 'nsplit, inunkp, nunkpo', &
                          nsplit, inunkp, nunkpo

!     --- Loop over unknowns

      do iunk = 1, min(inunkp,nunkpo)

!     --- Loop over parts

         minnodol = 1
         minnodnw = 1
         maxnodol = npointol
         maxnodnw = npointnw
         if ( debug ) &
            write(irefwr,1) 'npointol, npointnw', npointol, npointnw

         do ipart = 1, nsplit

            if ( nsplit>1 ) then
               maxnodol = km(iseqkm1)%kmeshk(km(iseqkm1)%kmesh(7)+ipart)
               maxnodnw = km(iseqkm2)%kmeshk(km(iseqkm2)%kmesh(7)+ipart)
               if ( debug ) &
                  write(irefwr,1) 'maxnodol, maxnodnw', maxnodol, maxnodnw
               if ( kprobfi(1+maxnodol)-kprobfi(minnodol)==0 ) then

!              --- No unknowns in this part for old mesh
!                  Check if this is also the case for the new mesh

                  if ( kprobfo(maxnodnw+1)-kprobfo(minnodnw)/=0 ) then

                     call errint ( ipart, 1 )
                     call errsub ( 987, 1, 0, 0 )

                  end if
                  go to 200

               end if

            end if  ! ( nsplit>1 )

            npoints = maxnodol-minnodol+1
            iwork = 0
            solcop1 = 0d0
            icount = 0

!           --- Copy unknown iunk from usol1 into solcop1
!               iwork(i):=1 for all copied points
!               icount := number of copied points

            usoltemp => ks(isol1)%sol

            if ( compl ) then

!           --- Complex case

               call intcopcompl00 ( iunk, usoltemp, solcop1, indprpi, &
                                    kprobfi, kprobpi, minnodol, maxnodol, &
                                    npointol, iwork, icount )
            else

!           --- Real case

               call intcop00 ( iunk, usoltemp, solcop1, indprpi, &
                               kprobfi, kprobpi, minnodol, maxnodol, npointol, &
                               iwork, icount )

            end if

            if ( debug ) &
               write(irefwr,*) 'icount, npoints ', icount, npoints

            if ( icount/=npoints ) then

!           --- Not all points are present, interpolate remaining points

               if ( compl ) then

!              --- Complex case

                  call intcopcompl01 ( solcop1, iwork, icount, iunk )
               else

!              --- Real case

                  call intcop01 ( solcop1, iwork,icount, iunk )

               end if

            end if
            if ( ierror/=0 ) go to 1000
            if ( debug ) then
               write(irefwr,*) 'icount-2, npoints ', icount, npoints
               call prinin ( iwork, npointol, 'iwork' )
            end if  ! ( debug )

            if ( icount==npoints ) then

!           --- icount = npoints
!               interpolate array solcop1 into solcop2

               imap => map%imap
               rmap => map%rmap

               if ( map%copy ) then

!              --- The first part of sol2 is identical to sol1 (copy)

                  solcop2(1:size(solcop1)) = solcop1
                  solcop1 => solcop2

               end if  ! ( map%copy )

               if ( compl ) then

!              --- Complex case

                  call intcopcomplun ( solcop1, solcop2, 1, minnodnw, &
                                       maxnodnw, npointnw, nunkpo )
               else

!              --- Real case

                  call intcopun ( solcop1, solcop2, 1, &
                                  minnodnw, maxnodnw, npointnw, &
                                  minmax, nunkpo )

               end if  ! ( compl )

!              --- extract unknown from solcop2 into usol2

               usoltemp => ks(isol2)%sol

!              --- if statement for intcoor2d

               if ( indprfo/=0 ) then

                  if ( compl ) then

!                 --- Complex case

                     call intcopcompl02 ( iunk, usoltemp, solcop2, &
                                          indprpo, kprobfo, kprobpo, &
                                          minnodnw, maxnodnw, npointnw )
                  else

!                 --- Real case

                     call intcop02 ( iunk, usoltemp, solcop2, &
                                     indprpo, kprobfo, kprobpo, &
                                     minnodnw, maxnodnw, npointnw )

                  end if  ! ( compl )

               end if

            else if ( icount/=0 ) then

!           --- icount # npoints and icount>0, special case

               imap => map%imap
               rmap => map%rmap
               usoltemp => ks(isol2)%sol

               if ( compl ) then

!              --- Complex case

               else

!              --- Real case

                  call intcop04 ( solcop1, usoltemp, iunk, &
                                  minnodnw, maxnodnw, npointnw, &
                                  minmax, iwork, indprpo, kprobfo, kprobpo )

               end if  ! ( compl )

            end if  ! ( icount==npoints )

200         minnodol = maxnodol+1
            minnodnw = maxnodnw+1

         end do  ! ipart = 1, nsplit

      end do  ! ipart = 1, nsplit

      deallocate ( iwork, solcop1, solcop2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intsolmp3'

      end if
      call erclos ( 'intsolmp3' )

      end subroutine intsolmp3
