      subroutine pr2021 ( coor, xmin, xmax, ymin, ymax, ds, minnode, &
                          maxnode, kmeshm, dx, dy, ncurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 15-10-2002 Extra parameter ncurvs
!        version  2.0  date 27-02-1998 Extra parameters dx and dy
!        version  1.0  date 06-12-1996
!
!   copyright (c) 1996-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute xmin, xmax, ymin, ymax from array coor
!    Compute ds from coor and kmeshm
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(2,*), xmin, xmax, ymin, ymax, ds, dx, dy
      integer kmeshm(*), minnode, maxnode, ncurvs

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ds         o    Minimum distance between two points along a curve
!     dx         o    Distance in the cells in x-direction
!     dy         o    Distance in the cells in y-direction
!     kmeshm     i    KMESH part m, contains nodal point numbers of the curves
!     maxnode    i    Maximum nodal point number in part
!     minnode    i    Minimum nodal point number in part
!     ncurvs     i    Number of curves in mesh
!     xmax       o    Maximum x-value
!     xmin       o    Minimum x-value
!     ymax       o    Maximum y-value
!     ymin       o    Minimum y-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, np, i1, i2, icurnr, ipos, iwork(ncurvs), ninner
      double precision x1, y1, x2, y2

!     i              Counting variable
!     i1             Point at curve
!     i2             Next point at curve
!     icurnr         Curve sequence number
!     ipos           position in kmeshm
!     istart         First position used at a part
!     iwork          Work array of length ncurvs
!                    iwork(i) = 0 means curve is external
!                    iwork(i) = 1 means curve is internal
!     ninner         Number of inner curves
!     np             number of nodes along curve
!     x1             x-coordinate of point i1
!     x2             x-coordinate of point i2
!     y1             y-coordinate of point i1
!     y2             y-coordinate of point i2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   initialize xmin, xmax, ymin, ymax, dx
!   For all curves do
!      For all nodes along the curve do
!         Compute xmin, xmax, ymin, ymax
!         Compute ds
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initialize xmin, xmax, ymin, ymax, dx

      xmin = coor(1,1)
      xmax = xmin
      ymin = coor(2,1)
      ymax = ymin
      dx = rinfin
      dy = rinfin
      ds = rinfin
      ninner = kmeshm(1)
      istart = kmeshm(3)

!     --- Clear array iwork

      do icurnr = 1, ncurvs
         iwork(icurnr) = 0
      end do

!     --- Set iwork equal to 1 for all internal curves

      do i = 1, ninner
         iwork(abs(kmeshm(istart+i-1))) = 1
      end do
      istart = kmeshm(2)

!     --- For all curves do

      do icurnr = 1, ncurvs

         if ( iwork(icurnr).eq.0 ) then

!        --- Boundary curve

            np = kmeshm(5+icurnr)-kmeshm(4+icurnr)
            ipos = istart+kmeshm(4+icurnr)-1
            i1 = kmeshm(ipos)
            if ( i1.lt.minnode .or. i1.gt.maxnode ) go to 100
            x1 = coor(1,i1)
            y1 = coor(2,i1)

!           --- For all nodes along the curve do

            do i = 1, np-1

!           --- Compute ds

               i2 = kmeshm(ipos+i)
               if ( i2.lt.minnode .or. i2.gt.maxnode ) go to 100
               x2 = coor(1,i2)
               y2 = coor(2,i2)
               ds = min(ds,sqrt((x2-x1)**2+(y2-y1)**2))
               if ( abs(x2-x1).gt.0.5d0*ds ) dx = min(dx,abs(x2-x1))
               if ( abs(y2-y1).gt.0.5d0*ds ) dy = min(dy,abs(y2-y1))
               x1 = x2
               y1 = y2

!              --- Compute xmin, xmax, ymin, ymax

               xmin = min(xmin,x1)
               xmax = max(xmax,x1)
               ymin = min(ymin,y1)
               ymax = max(ymax,y1)

            end do

         end if

100   end do
      if ( dx.gt.xmax-xmin ) dx = ds
      if ( dy.gt.ymax-ymin ) dy = ds

      end
