      subroutine pr2038 ( npointnw, ijcrnew, npntij, &
                          pntclij, minnodnw, maxnodnw, &
                          coorol, coornw, nx, ny, nz, npointold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 02-03-2012 Change in call to pr2039
!        version  1.2  date 15-06-2002 Extension of imap with element numbers
!        version  1.1  date 07-08-1998 Correction zh(i)
!
!   copyright (c) 1998-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Find all points in imap that have not ben treated yet and interpolate
!    those points
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npointnw, ijcrnew(3,*), npointold, &
              npntij(*), pntclij(*), minnodnw, &
              maxnodnw, nx, ny, nz
      double precision coorol(3,*), coornw(3,*)

!     coornw     i    Array containing the new co-ordinates
!     coorol     i    Array containing the old co-ordinates
!     ijcrnew    i    Contains the ij cell number of the co-ordinates of mesh2
!                     ijcrnew(1,k) contains the i number of coornw_k
!                     ijcrnew(2,k) contains the j number of coornw_k
!     maxnodnw   i    Maximum nodal point number in part (new mesh)
!     minnodnw   i    Minimum nodal point number in part (new mesh)
!     npntij     i    Contains number of coordinates in ij-cell accumulated
!                     npntij(1) = 0
!                     npntij((i-1)*nx+j+1) - npntij((i-1)*nx+j) is number
!                     of points in cell ij
!     npointnw   i    Number of nodes in new mesh
!     npointold  i    Number of nodes in old mesh
!     nx         i    number of points in x direction
!     ny         i    number of points in y direction
!     nz         i    number of points in z direction
!     pntclij    i    Contains the node numbers per cell
!                     The sequence is defined by npntij
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, inode, icell, jcell, istep, nnodes, ih, ij, n, &
              ilast, kcell, l, ih1
      double precision xh(4), yh(4), zh(4), dphidx(4,3), xgauss(4), &
                       x, y, z, xc, yc, zc, dx, dy, dz
      logical singul
      integer, allocatable :: iwork(:), ihelp(:)
      double precision, allocatable :: work(:)

!     dphidx     Contains the derivatives of the basis functions
!     dx         distance in x-direction
!     dy         distance in y-direction
!     dz         distance in z-direction
!     i          Counting variable
!     icell      i-index of cell
!     ih         Help variable to store a constant temporarily
!     ih1        Help variable to store a constant temporarily
!     ihelp      integer work array of length npointnw
!                It contains the sorted sequence of the nodes with smallest
!                distance to point to be interpolated
!     ij         help parameter to compute address of ij-cell in 1-d array
!     ilast      Last node used
!     inode      Node number
!     istep      Sequence number of step
!     iwork      integer work array of length npointnw
!                iwork contains the nodal point numbers of the nodes
!                with shortest distance to the point to be interpolated
!     j          Counting variable
!     jcell      j-index of cell
!     k          Counting variable
!     kcell      k-index of cell
!     l          Counting variable
!     n          number of nodes in cell
!     nnodes     Number of nodes in isolnw
!     singul     If true the Jacobian is smaller than eps
!     work       Work array of length npointnw
!                Contains the distances of the points stored in iwork
!                with respect to the point to be interpolated
!     x          x co-ordinate of point
!     xc         x-coordinate of point to be interpolated
!     xgauss     baricentric co-ordinates of the point
!     xh         x-coordinates of vertices
!     y          y-coordinate of point
!     yc         x-coordinate of point to be interpolated
!     yh         y-coordinates of vertices
!     z          z-coordinate of point
!     zc         x-coordinate of point to be interpolated
!     zh         z-coordinates of vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     PR2039         Compute gradient of the linear basis functions for a
!                    tetrahedron
!     PRSORT         Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    For inode = 1, npointnw
!       if ( imap(inode,1)=0 ) then
!          i,j := cell index of inode
!          Find smallest surrounding of i,j containing at least 4 old points
!          Find three points in this new region with smallest distance to inode
!          Interpolate and compute mapping
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

      ih = max(npointnw,npointold)
      allocate ( iwork(ih), work(ih), ihelp(ih), stat = error )
      if ( error/=0 ) call eralloc ( error, ih, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- For all nodes

      do inode = minnodnw, maxnodnw
         if ( imap(inode,1)==0 ) then

!        --- Node has not been treated yet
!            i,j := cell index of inode

            xc = coornw(1,inode)
            yc = coornw(2,inode)
            zc = coornw(3,inode)
            icell = ijcrnew(1,inode)
            jcell = ijcrnew(2,inode)
            kcell = ijcrnew(3,inode)

!           --- Find smallest surrounding of i,j containing at least 4 old
!               points

            do istep = 1, max(nx,ny,nz)
               nnodes = 0
               do k = max(1,kcell-istep), min(kcell+istep,nz)
                  ih1 = (k-1)*ny*nx
                  do j = max(1,jcell-istep), min(jcell+istep,ny)
                     ih = (j-1)*ny+ih1
                     do i = max(1,icell-istep), min(icell+istep,nx)
                        ij = ih+i
                        n = npntij(ij+1) - npntij(ij)
                        do l = 1, n
                           iwork(nnodes+l) = pntclij(npntij(ij)+l)
                        end do
                        nnodes = nnodes + n
                     end do
                  end do
               end do
               if ( nnodes>=8 ) then

!              --- Find four points in this new region with smallest distance
!                  to inode
!                  Interpolate and compute mapping

                  do i = 1, nnodes
                     x = coorol(1,iwork(i))
                     y = coorol(2,iwork(i))
                     z = coorol(3,iwork(i))
                     work(i) = sqrt ( (x-xc)**2+(y-yc)**2+(z-zc)**2 )
                  end do
                  call prsort ( work, ihelp, nnodes )
                  do i = 1, 4
                     ih = iwork(ihelp(i))
                     xh(i) = coorol(1,ih)
                     yh(i) = coorol(2,ih)
                     zh(i) = coorol(3,ih)
                  end do
                  ilast = 4
100               call pr2039 ( xh, yh, zh, dphidx, &
                                max(1d-20,0.1d-6*work(1)), singul )
                  if ( singul ) then

!                 --- The four points are lying in one plane, &
!                     Remove point last point and add next point

                      ilast = ilast+1
                      if ( nnodes<ilast ) go to 200
                      ih = iwork(ihelp(ilast))
                      xh(4) = coorol(1,ih)
                      yh(4) = coorol(2,ih)
                      zh(4) = coorol(3,ih)
                      go to 100

                  end if

!                 --- compute baricentric co-ordinates of inode

                  dx = coornw(1,inode) - xh(4)
                  dy = coornw(2,inode) - yh(4)
                  dz = coornw(3,inode) - zh(4)
                  xgauss(1) = dphidx(1,1)*dx+dphidx(1,2)*dy+ &
                              dphidx(1,3)*dz
                  xgauss(2) = dphidx(2,1)*dx+dphidx(2,2)*dy+ &
                              dphidx(2,3)*dz
                  xgauss(3) = dphidx(3,1)*dx+dphidx(3,2)*dy+ &
                              dphidx(3,3)*dz
                  xgauss(4) = 1d0 - xgauss(1) - xgauss(2) - &
                              xgauss(3)
                  imap(inode,1) = 3
                  do i = 1, 3
                     imap(inode,i+1) = iwork(ihelp(i))
                     rmap(inode,i) = xgauss(i)
                  end do
                  imap(inode,6) = 0
                  go to 200
               end if
            end do
200      end if
      end do

      deallocate ( iwork, work, ihelp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  end
