      subroutine pravs4 ( iresul, nphysloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-10-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map mesh on appropriate mesh and map solution on this mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     output
!     vector
!     map
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  iresul(5,3), nphysloc

!     iresul         o    Output array of length 5,1 if ichois=2 and otherwise
!                         of length 5,3
!                         The array is filled as follows:
!                         ,1 corresponds to the "vertex" mesh array (ichois=2,3)
!                         ,2 corresponds to the "triangle" mesh array
!                            (ichois=1,3)
!                         ,3 corresponds to the solution array (ichois=1,3)
!                         1,1  INDVER sequence number with respect to INFOR
!                                     for the "vertex" mesh array
!                         2,1  NPELM  number of points per element for this mesh
!                         3,1  NELEM  number of points per element for this mesh
!                         4,1  NEWVER indicates if the mesh is new (1) or is
!                                     in fact the standard mesh (0)
!                         5,1  -
!                         1,2  INDTRI sequence number with respect to INFOR
!                                     for the "triangle" mesh array
!                         2,2  NPELM  number of points per element for this mesh
!                         3,2  NELEM  number of points per element for this mesh
!                         4,2  NEWTRI indicates if the mesh is new (1) or is
!                                     in fact the standard mesh (0) or the mesh
!                                     created by the "vertex" mesh
!                         5,2  NPHYS  Max. number of degrees of freedom
!                         1,3  INDSOL sequence number with respect to INFOR
!                                     for the new solution array
!                         2,3  NEWSOL indicates if the solution array is new (1)
!                                     or is in fact the standard
!                                     solution array (0)
!                         3,3  INDPRF sequence number of KPROBF
!                         4,3  INDPRH sequence number of KPROBH
!                         5,3  INPPRP sequence number of KPROBP
!     nphysloc          o    Number of physical unknowns, or maximal degrees of
!                         freedom per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichois, ndimlc, itypvc, &
              ivec, kpivec, i, number(11), iskip, ifill, special, &
              ilevel, iskpar(1)
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     i              Counting variable
!     ichois         Choice parameter
!     ifill          Parameter for subroutine TOPQU0 indicating if the extra
!                    array iwork1 must be filled (>0) or not (0)
!     ilevel         dummy
!     iskip          Indicates if element groups must be skipped (>0)
!                    or not (0)
!     iskpar         array of length NELGRP that is only used if ISKIP > 0
!                    It indicates if element group ielgrp must be skipped
!                    iskpar(ielgrp) = 0 or not iskpar(ielgrp) = 1
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     kpivec         Starting address of information concerning arrays of
!                    special structure in KPROB
!     ndimlc         Local dimension of space
!     number         Array containing the sequence numbers of the unknowns to be
!                    considered.
!                    Position 1 contains the number of unknowns
!     special        Indicates which special situation is present
!                    Possible values:
!                    0     standard
!                    1     solution vector is of type 116 and a coloured
!                          2d plot is required
!                    2     solution vector is of type 110 or 115 and a
!                          coloured 2d plot is required, whereas there
!                          are no unknowns in the vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     TOPQU0         Subdivide mesh if necessary. Map unknowns into an organized
!                    array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     222   input vector is complex:  not yet allowed.
!     832   Vectors defined per element not permitted in this subroutine
!     833   nphys too large
!     834   ndim=3, whereas ndimlc=2 and nunkp>1
!     835   Only one-dimensional elements available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pravs4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pravs4'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Map mesh on appropriate mesh and map solution on this mesh

      if ( nsurfs==0 .and. nvolms==0 ) then

!     --- Only one-dimensional elements

         call errsub ( 835, 0, 0, 0 )
         go to 1000

      end if
      if ( nvolms==0 ) then
         ichois = 1
         ndimlc = 2
      else
         ichois = 7
         ndimlc = 3
      end if

!     --- Compute IPROB and offset for array KPROB

      iprob  = ks(isolut)%iprob
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      indpre = kprobloc(18)
      if ( indpre>0 ) then
         ndtvec = kprobloc(6)
         kprobe => kprobpart%kprobe
         kprobe2 => kprobpart%kprobe2
         kprobe3 => kprobpart%kprobe3
      end if  ! ( kprobloc(18)>0 )

      itypvc = ks(isolut)%typevec
      if ( itypvc==110 ) then

!     --- ISOL is of type 110:  solution vector

         nphysloc = kprobloc(3)

      else if ( itypvc==115 ) then

!     --- ISOL is of type 115:  array of special structure

         ivec   = ks(isolut)%ivec

         nphysloc  = kprobe2(ivec)
         if ( nphysloc<0 ) then

!        --- nphysloc < 0, constant number of degrees of freedom per point

            nphysloc = -nphysloc

         else

!        --- nphysloc > 0, number of degrees of freedom per point not constant

            nphysloc = maxval(kprobe(:,ivec))

         end if

      else

!     --- Wrong type of solution array

         call errsub ( 832, 0, 0, 0 )
         go to 1000

      end if
      if ( nphysloc<=10 ) then
         number(1) = nphysloc
         do i = 1, nphysloc
            number(i+1) = i
         end do
      else
         call errint ( nphysloc, 1 )
         call errint ( 10, 2 )
         call errsub ( 833, 2, 0, 0 )
         go to 1000
      end if
      compl = ks(isolut)%compl==1
      if ( compl ) then
         call errsub ( 222, 0, 0, 0 )
         go to 1000
      end if
      iskip = 0
      ifill = 0
      if ( ndim/=ndimlc .and. nphysloc>1 ) then

!     --- dimension of space = 3, but no volume elements

         call errsub ( 834, 0, 0, 0 )
         go to 1000

      end if
      special = 0
      ilevel = 1
      call topqu0 ( ichois, number, iskip, iskpar, iresul, compl, &
                    ifill, special, ilevel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pravs4'

      end if
      call erclos ( 'pravs4' )

      end subroutine pravs4
