      subroutine prinptrace
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 30-11-2012 Long integers
!        version  1.2  date 30-08-2006 New definition of cpost5
!        version  1.1  date 23-04-2003 New call to prcopychseq
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print values of time and coordinates of particle trace
!     Also compute and print corresponding values of solution arrays
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     particle_tracking
!     track
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulepost
      use sepmodulemap
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npart, ndim, nsteps, iparticle, istart, &
              iphys(maxvc1), nvectors, &
              i, ivec, nphystot, nphysmax, indsol, ipsolint
      double precision, allocatable :: solint(:), coorloc(:)
      type ( mapping ) :: maplc

!     coorloc        Array with the coordinates of the  points in which
!                    interpolation must be performed
!     i              Counting variable
!     indsol         Sequence number of vector in isol
!     iparticle      Sequence number of particle
!     iphys          Stores nphys for various arrays
!     ipsolint       Starting address in array solint
!     istart         First position used at a part
!     ivec           Sequence number of vector of special structure
!     maplc          Mapping array that is filled if ichoice = 2 or must
!                    be filled if ichoice = 3
!                    map is an integer array of length 5 with the following
!                    contents:
!                    1: mmimap  memory management reference to array imap
!                    2: 130 to indicate the type of array
!                    3: mmrmap  memory management reference to array rmap
!                    4: mmcoor1  memory management reference to array coor1
!                    5: mmcoor2  memory management reference to array coor2
!     ndim           Dimension of the space.
!     npart          Number of particles
!     nphysmax       Maximum number of physical unknowns of all vectors
!     nphystot       Total number of physical unknowns of all vectors
!     nsteps         Contains the number of positions stored in partout
!     nvectors       Number of vectors to be considered in this subroutine
!     solint         Contains the interpolated solution
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRCOPY2D       Copy two-dimensional array of size ndim1 x npoint into
!                    two-dimensional array of size ndim2 x npoint
!     PRCOPYCHSEQ    Interchange sequence of parameters in two-dimensional array
!                    by considering it as a one-dimensional array
!     PRINPTRACE01   Perform the actual printing for subroutine PRINPTRACE
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCLEAR       clear array map
!     SEPINTCOOR     Interpolate the solution as stored in array isol
!                    from nodal points in array kmesh onto points with
!                    coordinates stored in coor
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2230   solution array has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array map
!     get some values from ioutput
!     Activate vectors and find corresponding number of degrees of freedom
!     per point
!     For iparticle := 1 (1) npart do
!        Reserve space for coor and solint at the end of ibuffr
!        Copy coordinates from ioutput to coor
!        interpolate and store into solint
!        Reverse sequence of unknowns if necessary
!        Perform the actual printing
!        Reset ibuffr, i.e. remove last part
!     Delete arrays that are no longer used
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinptrace' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinptrace'

      end if

!     --- Clear array map

      call sepclear ( maplc )

!     --- get some values from ioutput

      npart = particinf(1,1)
      ndim = particinf(2,1)
      nvectors = iinputpst(85)

!     --- Activate vectors and find corresponding number of degrees of freedom
!         per point

      nphystot = 0
      nphysmax = 0
      do i = 1, nvectors
         ivec = igroup(i)
         indsol = imapsol(ivec,itimst)
         if ( indsol==0 ) then

!        --- Array has not been filled

            call errint ( ivec, 1 )
            call errint ( itimst, 2 )
            call errsub ( 2230, 2, 0, 0 )
            go to 1000

         end if
         call sepactsolbf2 ( isolut2d(indsol) )
         iphys(i) = nphysi
         nphystot = nphystot + nphysi
         nphysmax = max(nphysmax,nphysi)
      end do

!     --- Loop over particles

      do iparticle = 1, npart
         nsteps = particinf(1,iparticle+1)

!        --- Reserve space for coor and solint at the end of ibuffr

         length = nsteps*(ndim+nphystot)
         if ( debug ) &
            write(irefwr,*) 'nsteps,ndim,nphystot,nphysmax ', &
                             nsteps,ndim,nphystot,nphysmax

         allocate ( solint(nsteps*nphystot), coorloc(nsteps*ndim), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'solint' )
         if ( ierror/=0 ) go to 1000

!        --- Copy coordinates from ioutput to coor

         ipsolint = 1
         istart = particinf(2,iparticle+1)
         istart = 1+(istart-1)*(ndim+1)
         call prcopy2d ( nsteps, partics(istart+1), ndim+1, coorloc, ndim )

!        --- interpolate and store into solint

         do i = 1, nvectors
            nphysi = iphys(i)
            ivec = igroup(i)
            iinmap(1) = 2
            if ( i==1 ) then
               iinmap(2) = 1
            else
               iinmap(2) = 2
            end if
            indsol = imapsol(ivec,itimst)
            if ( debug ) &
               write(irefwr,*) 'i, nphysi, ivec ',i, nphysi, ivec
            call sepintcoor ( isolut2d(indsol), &
                           solint(ipsolint), coorloc, nphysi, nsteps, &
                           iinmap, maplc )
            if ( nphysi>1 ) then

!           --- nphysi>1, reverse sequence of unknowns (i,j) --> (j,i)
!               The reason is that solint in prinptrace01 is stored as a
!               nsteps x nphystot array whereas in intcoorbf it is
!               considered as a nphysi x nsteps array

               call prcopychseq ( solint(ipsolint), nsteps, nphysi )

            end if
            ipsolint = ipsolint+nphysi*nsteps

         end do

!        --- perform the actual printing

         istart = particinf(2,iparticle+1)
         istart = 1+(istart-1)*(ndim+1)

         call prinptrace01 ( partics(istart), solint, iphys, &
                             ndim, nsteps, iparticle )

         deallocate ( solint, coorloc, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'solint' )
         if ( ierror/=0 ) go to 1000

      end do

!     --- Delete arrays that are no longer used

      if ( associated(maplc%imap) ) deallocate ( maplc%imap, maplc%rmap )

1000  if ( debug ) write(irefwr,*) 'End prinptrace'
      call erclos ( 'prinptrace' )

      end subroutine prinptrace
