      subroutine printersedg3d ( nmeshc, icount, iplane, coor, plane, &
                                 marknode, markedge, npoint, nedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 06-11-2012 Extra debug
!        version  1.2  date 17-11-2006 Adapt accurracy
!        version  1.1  date 30-08-2006 Extra debug information
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark edges that are intersected (3d) by plane
!     Store mapping array
!     Only for plane in one of the coordinate directions
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     plane
!     intersection
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshc(*), icount, iplane, marknode(*), markedge(*), &
              npoint, nedges
      double precision coor(3,*), plane(4)

!     coor           i    array of length 3 x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     icount         o    Number of edges that are intersected
!     iplane         i    Defines direction of plane
!                         1:  plane x = constant
!                         2:  plane y = constant
!                         3:  plane z = constant
!                         4:  general plane
!     markedge       o    Array of size nedges
!                         If edge i is not intersected markedge(i) = 0, &
!                         otherwise markedge(i)>0 contains the sequence number
!                         of the new node in the local mesh
!     marknode       o    Array of size npoint
!                         If node i is not on plane marknode(i) = 0, &
!                         otherwise marknode(i)>0 contains the sequence number
!                         of the new node in the local mesh
!     nedges         i    number of edges
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     npoint         i    Number of nodal points
!     plane          i    Array of length 4 containing the parameters a, b, c
!                         and d describing the plane: ax=by+cz+d = 0
!                         plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iedge, ipstart, node1, node2, i, icountstart
      double precision val1, val2, eps, amin, amax, lenedg, value

!     amax           Maximum value
!     amin           Minimum value
!     eps            Local accuracy
!     i              Counting variable
!     icountstart    Start value of counter
!     iedge          edge sequence number
!     ipstart        Starting address in array
!     lenedg         length of edge
!     node1          Node of face
!     node2          Node of face
!     val1           Value of plane in node1, i.e. ax+by+cz-d
!     val2           Value of plane in node2, i.e. ax+by+cz-d
!     value          Value of intersection
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRWAR         Warnings
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINTERSEDGNOD Fill markedge, marknode, imap and rmap in case node
!                    of edge is on plane
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2836   No intersection of surface with plane found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     icount := 0
!     For all edges iedge do
!        if ( iedge is intersected by plane ) then
!           if ( intersection corresponds to node node ) then
!              if ( marknode(node)=0 ) then
!                 icount := icount+1
!                 marknode(node) := icount
!                 markedge(iedge) := icount
!                 Fill imap and rmap (icount)
!              else
!                 markedge(iedge) := marknode(node)
!           else
!              icount := icount+1
!              markedge(iedge) := icount
!              Fill imap and rmap (icount)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printersedg3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printersedg3d'
         write(irefwr,1) 'iplane', iplane
         call prinrl ( plane, 4, 'plane' )
  1      format ( a, 1x, (10i8) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iplane<4 ) then

!     --- Define accuracy (first range of coordinates)
!         Only for plane in coordinate direction

         value = plane(4)
         amin = rinfin
         amax = -rinfin
         do i = 1, npoint
            amin = min(amin,coor(iplane,i))
            amax = max(amax,coor(iplane,i))
         end do  ! i = 1, npoint
         if ( debug ) write(irefwr,2) 'amin, amax', amin, amax
         if ( amin>value .or. amax<value ) then

!        --- amin>value or amax<value, no intersection possible

            call errint ( iplane, 1 )
            call erreal ( value, 1 )
            call errwar ( 2836, 1, 1, 0 )
            ierror = 2836
            go to 1000

         end if  ! ( amin>value .or. amax<value )
         eps = 1d-6*(abs(amin)+abs(amax))  ! accuracy relative 1d-6
         if ( amax-amin<=eps ) then

!        --- amax-amin<eps, no intersection possible because region is plane

            call errint ( iplane, 1 )
            call erreal ( value, 1 )
            call errwar ( 2836, 1, 1, 0 )
            ierror = 2836
            go to 1000

         end if  ! ( amin>value .or. amax<value )
         if ( debug ) write(irefwr,2) 'amin, amax, eps', amin, amax, eps

      end if  ! ( iplane<4 )

!     --- icount := 0

      icount = 0

!     --- For all edges iedge do

      do iedge = 1, nedges

         ipstart = nmeshc(iedge)    ! start of edge in nmeshc
         node1 = nmeshc(ipstart+1)  ! first node of edge
         node2 = nmeshc(ipstart+2)  ! last node of edge
         icountstart = icount

         if ( iplane<4 ) then

!        --- Special case, plane in coordinate direction

            val1 = coor(iplane,node1)-value  ! sign defines below or above plane
            val2 = coor(iplane,node2)-value  ! sign defines below or above plane

         else

!        --- General case, plane not in coordinate direction

            val1 = plane(1)*coor(1,node1)+plane(2)*coor(2,node1)+ &
                   plane(3)*coor(3,node1)-plane(4)
            val2 = plane(1)*coor(1,node2)+plane(2)*coor(2,node2)+ &
                   plane(3)*coor(3,node2)-plane(4)

         end if  ! ( iplane<4 )
         lenedg = (coor(1,node1)-coor(1,node2))**2+ &
                  (coor(2,node1)-coor(2,node2))**2+ &
                  (coor(3,node1)-coor(3,node2))**2
         eps = 1d-6*sqrt(lenedg)

         if ( debug ) then
            write(irefwr,1) 'iedge, nmeshc(ipstart), node1, node2', &
                             iedge, nmeshc(ipstart), node1, node2
            write(irefwr,2) 'val1, val2, eps', val1, val2, eps
         end if  ! ( debug )

         if ( abs(val1)<eps ) then

!        --- |val1|<eps, intersection coincides with node 1

            call printersedgnod ( node1, iedge, icount, marknode, &
                                  markedge )
            if ( debug ) write(irefwr,*) 'coincide with node 1'

         end if  ! ( abs(val1)<eps )

         if ( abs(val2)<eps ) then

!        --- |val2|<eps, intersection coincides with node 2

            call printersedgnod ( node2, iedge, icount, marknode, &
                                  markedge )
            if ( debug ) write(irefwr,*) 'coincide with node 2'

         end if  ! ( abs(val2)<eps )

         if ( debug .and. icount>icountstart ) then
            write(irefwr,1) 'icount, imap', icount
            call prinin ( imap(icount,1), 2, 'imap' )
            write(irefwr,1) 'markedge(iedge)', markedge(iedge)
         end if  ! ( debug )

         if ( val1*val2<0d0 .and. markedge(iedge)==0 .and. &
              marknode(node1)==0 .and. marknode(node2)==0 ) then

!        --- val1 has other sign than val2, hence intersection
!            icount := icount+1
!            markedge(iedge) := icount
!            Fill imap and rmap (icount)

            icount = icount+1
            markedge(iedge) = icount
            imap(icount,1) = node1
            imap(icount,2) = node2
            rmap(icount,1) = abs(val2)/(abs(val1)+abs(val2))
            if ( debug ) then
               write(irefwr,1) 'icount, node1, node2', &
                                icount, node1, node2
               write(irefwr,2) 'val1, val2, interp', &
                                val1, val2, rmap(icount,1)
            end if  ! ( debug ) then

         end if  ! ( abs(val1)<eps )

      end do  ! iedge = 1, nedges

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printersedg3d'

      end if  ! ( debug )
      call erclos ( 'printersedg3d' )

      end
