      subroutine readsepfreq
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 20-06-2013 New definition file ref numbers
!        version  1.4  date 30-11-2012 Long integers
!        version  1.3  date 30-08-2006 New definition of cpost5
!
!   copyright (c) 1999-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read the contents of the file sepcomp.freq
!    The input is stored in ibuffr, buffer and the commons cpost...
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulemach
      use sepmoduleplot
      use sepmodulepost
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxquantities
      parameter ( maxquantities=100 )
      integer nquantities, itypequant(maxquantities), &
              numquant(maxquantities), i, ios, &
              icount, j, inum, jcount, k, &
              nreadquant
      double precision dummy
      logical check

!     check          Indicates if the file exist (true) or not
!     dummy          Dummy parameter
!     i              Counting variable
!     icount         number of unknowns per element
!     inum           Number of numbers read in a record
!     ios            IOSTAT variable, returns from reading or writing
!     itypequant     Array containing the types of the quantities to be written
!                    to the file sepcomp.freq
!                    itypequant(i) = 1, means that quantity consists of integers
!                    itypequant(i) = 2, means that quantity consists of reals
!     j              Counting variable
!     jcount         Counter
!     k              Counting variable
!     maxquantities  maximum number of quantitites that is allowed
!     nquantities    number of quantitites to be written to file sepcomp.freq
!                    per time step
!     nreadquant     Total number of quantities read per time step
!     numquant       Array containing the number of integers or reals per
!                    quantity to be written to the file sepcomp.freq
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
! **********************************************************************
!
!                       I/O
!
!      The file sepcomp.freq is an ASCII file containing a small amount
!      of output that is written for each time step
!      Organization of the file:
!      Lines 1 to 15, comment lines that must be skipped
!      These are followed by the following lines:
!      1: nquantities     (i.e. number of quantities written per time step)
!      2: Integer array itypequantities of length nquantities;
!         itypequantities(i) = 1, means that quantity i consists of integers
!         itypequantities(i) = 2, means that quantity i consists of reals
!      3: Integer array numquantities of length nquantities;
!         numquantities(i) defines the number of integers or reals quantity i
!         consists of
!      4..3+nquantities: Each of these lines contains the name of the quantity
!      Next records (one record per time step):
!         t quantity_1 quantity_2 quantity_3 ...
!         where each quantity may be an array itself
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   Estimated length of array too short
!    1317   error in opening of file
!    1320   Error in reading record of file
!    2075   number of quantities read in the file sepcomp.freq non-positive
!    2076   too uch quantities read in the file sepcomp.freq
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( file sepcomp.freq exists ) then
!        open file sepcomp.freq
!        Read and skip first 15 lines
!        Read nquantities
!        Read array itypequant
!        Read array numquant
!        Read the names of the quantities and store in namefuncs
!        nreadquant := number of quantities per time step
!        Read rest of file into dummy and determine number of time steps
!        close file sepcomp.freq
!        Reserve space
!        open file sepcomp.freq
!        Read and skip first 16 lines
!        Read array itypequant
!        Read array numquant
!        Read and skip nquantities lines
!        For all time steps do
!           Read quantities and store in buffer
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readsepfreq' )

!     --- Check if file sepcomp.freq exists

      inquire ( file='sepcomp.freq', exist=check )
      if ( .not. check ) go to 1000

!     --- open file sepcomp.freq

      iref75  = inisetnewref ()
      open ( unit = iref75, file = 'sepcomp.freq', iostat=ios )
      if ( ios/=0 ) then

!     --- Error in opening file

         call errint ( ios, 1 )
         call errchr ( 'sepcomp.freq', 1 )
         call errsub ( 1317, 1, 0, 1 )
         go to 1000

      end if

!     --- Read and skip first 15 lines

      do i = 1, 15
         read ( iref75, *, iostat = ios )
         if ( ios/=0 ) go to 990
      end do

!     --- Read nquantities

      read ( iref75, *, iostat = ios ) nquantities
      numfuncs = nquantities
      if ( nquantities<=0 .or. nquantities>maxquantities ) then

!     --- nquantities out of range

         call errint ( maxquantities, 1 )
         call errint ( nquantities, 2 )
         call errchr ( 'itypequant', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if
      if ( ios/=0 ) go to 990
      if ( nquantities<=0 ) then

!     --- nquantities<=0

         call errint ( nquantities, 1 )
         call errwar ( 2075, 1, 0, 0 )
         go to 1000

      end if

      allocate ( work(1500) )

!     --- Read array itypequant

      read ( iref75, *, iostat = ios ) (itypequant(i),i=1,nquantities)
      if ( ios/=0 ) go to 990

!     --- Read array numquant

      read ( iref75, *, iostat = ios ) (numquant(i),i=1,nquantities)
      if ( ios/=0 ) go to 990

!     --- Read the names of the quantities and store in namefuncs

      do i = 1, nquantities
         read ( iref75, 100, iostat = ios ) namefuncs(i)
100      format(a)
         if ( ios/=0 ) go to 990
      end do

!     --- Remove leading spaces from namefunc

      do i = 1, nquantities
         icount = 0
150      if ( icount<=31 .and. namefuncs(i)(1:1)==' ' ) then
            icount = icount + 1
            namefuncs(i)(1:32-icount) = namefuncs(i)(2:33-icount)
            go to 150
         end if
      end do

!     --- nreadquant := number of quantities per time step

      nreadquant = numquant(1)
      do i = 2, nquantities
         nreadquant = nreadquant+numquant(i)
      end do
      if ( nreadquant>maxfuncs ) then

!     --- too much quantities stored

         call errint ( nquantities, 1 )
         call errint ( maxfuncs, 2 )
         call errsub ( 2076, 2, 0, 0 )
         go to 1000

      end if

!     --- Read rest of file into dummy and determine number of time steps

      icount = 0
200   read ( iref75, *, iostat = ios ) (dummy,i=0,nreadquant)
      if ( ios>0 ) then

!     --- error in reading file

         go to 990

      else if ( ios==0 ) then

!     --- no error, hence next record found

         icount = icount+1
         go to 200

      end if

!     --- close file sepcomp.freq

      call inirmref ( iref75 )
      close ( iref75 )

!     --- Reserve space

      if ( associated(ifuncs(0)%arr) ) deallocate ( ifuncs(0)%arr )
      allocate ( ifuncs(0)%arr(icount+5) )
      do i = 1, nquantities
         if ( associated(ifuncs(i)%arr) ) deallocate ( ifuncs(i)%arr )
         allocate ( ifuncs(i)%arr(icount*numquant(i)+5) )
      end do
      if ( ierror/=0 ) go to 1000

!     --- Fill first 5 positions of arrays

      ifuncs(0)%arr(1) = icount+5
      ifuncs(0)%arr(2) = 117d0
      ifuncs(0)%arr(3) = icount+5
      ifuncs(0)%arr(4) = 1d0
      ifuncs(0)%arr(5) = icount
      do i = 1, nquantities
         ifuncs(i)%arr(1) = icount*numquant(i)+5
         ifuncs(i)%arr(2) = 117d0
         ifuncs(i)%arr(3) = icount*numquant(i)+5
         ifuncs(i)%arr(4) = numquant(i)
         ifuncs(i)%arr(5) = icount*numquant(i)
      end do

!     --- open file sepcomp.freq

      open ( unit = iref75, file = 'sepcomp.freq', iostat=ios )
      if ( ios/=0 ) then

!     --- Error in opening file

         call errint ( ios, 1 )
         call errchr ( 'sepcomp.freq', 1 )
         call errsub ( 1317, 1, 0, 1 )
         go to 1000

      end if

!     --- Read and skip first 16 lines

      do i = 1, 16
         read ( iref75, *, iostat = ios )
         if ( ios/=0 ) go to 990
      end do

!     --- Read array itypequant

      read ( iref75, *, iostat = ios ) (itypequant(i),i=1,nquantities)
      if ( ios/=0 ) go to 990

!     --- Read array numquant

      read ( iref75, *, iostat = ios ) (numquant(i),i=1,nquantities)
      if ( ios/=0 ) go to 990

!     --- Read and skip nquantities lines

      do i = 1, nquantities
         read ( iref75, *, iostat = ios )
         if ( ios/=0 ) go to 990
      end do

!     --- For all time steps do

      do i = 1, icount

!     --- Read quantities and store in buffer

         read ( iref75, *, iostat = ios ) (work(j),j=1,nreadquant+1)
         ifuncs(0)%arr(5+i) = work(1)
         jcount = 1
         do j = 1, nquantities
            inum = numquant(j)
            do k = 1, inum
               ifuncs(j)%arr(5+inum*(i-1)+k) = work(jcount+k)
            end do
            jcount = jcount+inum
         end do

      end do

      deallocate ( work )

990   if ( ios>0 ) then

!     --- Error while reading file

         call errint ( icount, 1 )
         call errint ( ios, 2 )
         call errchr ( 'sepcomp.freq', 1 )
         call errsub ( 1320, 2, 0, 1 )
         go to 1000

      end if  ! ( ios>0 )
1000  call erclos ( 'readsepfreq' )
      end
