      subroutine sepfillmap2d
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill mapping array for line intersecting a surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     interpolation
!     line
!     mapping_array
!     map
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplotfn
      use sepmoduleint2d
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, i1, i2, node1, node2
      double precision :: alpha, shift

!     alpha          Relative length
!     i              Counting variable
!     i1             sequence number of edge in element
!     i2             element number in element
!     node1          first node of edge
!     node2          second node of edge
!     shift          shift in funcx
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillmap2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillmap2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( imapfn(3,numint), mapfn(2,numint), &
                 funcx(numint), funcy(numint,numvecs),  stat = error )
      if ( error/=0 ) call eralloc ( error, numvecs*numint, 'imapfn' )
      if ( ierror/=0 ) go to 1000

      imapfn = 0
      mapfn = 0d0

!     --- Compute shift in funcx

      if ( abs(angle)<sqreps ) then

!     --- angle = 0, use x-coordinate of x1

         shift = x1(1)

      else if ( abs(angle-0.5d0*pi)<sqreps ) then

!     --- angle = pi/2, use y-coordinate of x1

         shift = x1(2)

      else if ( abs(angle-pi)<sqreps ) then

!     --- angle = pi, use x-coordinate of x2

         shift = x2(1)

      else if ( abs(angle-1.5d0*pi)<sqreps ) then

!     --- angle = 3 pi/2, use y-coordinate of x2

         shift = x2(2)

     else

         shift = 0d0

      end if  ! ( abs(angle)<sqreps )

      do i = 1, numint

         i1 = 2-mod(iseqx(i),2)  ! sequence number of first edge in element
         i2 = (iseqx(i)+1)/2     ! element number in listel

         i1 = listel(1+i1,i2)    ! edge number in trian2d
         i2 = listel(1,i2)       ! element number in trian2d
         node1 = trian2d(i1,i2)  ! first node

         if ( i1==3 ) i1 = 0
         node2 = trian2d(i1+1,i2) ! second node

         y(1) = coor2d(2,node1)
         y(2) = coor2d(2,node2)
         alpha = y(1)/(y(1)-y(2))

         if ( 1d0-alpha<sqreps ) then

!        --- Only node1 should be used

            imapfn(1,i) = 1
            imapfn(2,i) = node2
            mapfn(1,i) = 1d0
            funcx(i) = shift+coor2d(1,node2)

         else if ( alpha<sqreps ) then

!        --- Only node2 should be used

            imapfn(1,i) = 1
            imapfn(2,i) = node1
            mapfn(1,i) = 1d0
            funcx(i) = shift+coor2d(1,node1)

         else

!        --- Both nodes should be used

            imapfn(1,i) = 2
            imapfn(2,i) = node1
            imapfn(3,i) = node2
            mapfn(1,i) = 1d0-alpha
            mapfn(2,i) = alpha
            funcx(i) = shift+coor2d(1,node1)*mapfn(1,i)+coor2d(1,node2)*alpha

         end if  ! ( 1d0-alpha<sqreps )

      end do  ! i = 1, numint

1000  call erclos ( 'sepfillmap2d' )
      if ( debug ) then

!     --- Debug information

         call prinin1 ( imapfn, numint, 3, 'imapfn' )
         call prinrl1 ( mapfn, numint, 2, 'mapfn' )
         call prinrl ( funcx, numint, 'funcx' )
         write(irefwr,*) 'End sepfillmap2d'

      end if  ! ( debug )

      end subroutine sepfillmap2d
