      subroutine sepintmap3d3c ( usol, usolcop, nunkploc, number )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solution in mesh with non-constant number of unknowns
!     per point to solution array with constant number of unknowns
!     The solution to be copied is not present in all nodes
!     Center nodes of elements and faces are skipped
!     Complex version
!     Old subroutine intmap3d3c
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
!     intersection
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nunkploc, number
      double complex usol(*), usolcop(nunkploc,*)

!     number         i    indicates which is the first degree of freedom
!                         that must be interpolated in each point
!     nunkploc          i    indicates the number of unknowns that must be
!                         interpolated in each point containing this
!                         number of unknowns counted from number
!     usol           i    Solution vector to be copied and interpolated
!     usolcop        o    Result vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inpelm, istart, ielgrp

!     i              Counting variable
!     ielgrp         element group number
!     inpelm         number of points per element
!     istart         First position used at a part
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTMAP3D11C    Copy solution in mesh with non-constant number of unknowns
!                    per point to solution array with constant number of
!                    unknowns
!                    The solution to be copied is not present in all nodes
!                    Center nodes of elements and faces are skipped
!                    Elements of shape 11, 13, 17, 18 or 21
!                    Complex version
!     INTMAP3D12C    Copy solution in mesh with non-constant number of unknowns
!                    per point to solution array with constant number of
!                    unknowns
!                    The solution to be copied is not present in all nodes
!                    Center nodes of elements and faces are skipped
!                    Elements of shape 12, 15 or 16
!                    Complex version
!     INTMAP3D14C    Copy solution in mesh with non-constant number of unknowns
!                    per point to solution array with constant number of
!                    unknowns
!                    The solution to be copied is not present in all nodes
!                    Center nodes of elements and faces are skipped
!                    Elements of shape 14
!                    Complex version
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set output array to zero
!     For all elements do
!        For all unknowns do
!           for all non-center points do
!              if ( unknown available ) then
!                 copy
!              else
!                 interpolate
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintmap3d3c' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintmap3d3c'
         write(irefwr,1) 'nunkploc, number', nunkploc, number
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set output array to zero

      do i = 1, npoint
         do j = 1, nunkploc
            usolcop(j,i) = (0d0,0d0)
         end do  ! j = 1, nunkploc
      end do  ! i = 1, npoint

!     --- For all elements do

      istart = 1

      do ielgrp = 1, nelgrp
         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm

         select case ( ishape )

            case ( 19, 20, 22: )

!           --- Shape number not available

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            case(11, 18)

!           --- Linear tetrahedron

               call intmap3d11c ( kmeshc(istart), inelem, &
                                  nunkploc, number, usol, usolcop, &
                                  ielgrp, inpelm, 4 )

            case(12, 15, 16)

!           --- Quadratic tetrahedron

               call intmap3d12c ( kmeshc(istart), inelem, &
                                  nunkploc, number, usol, usolcop, &
                                  ielgrp, inpelm )

            case(13, 17)

!           --- Trilinear hexahedron

               call intmap3d11c ( kmeshc(istart), inelem, &
                                  nunkploc, number, usol, usolcop, &
                                  ielgrp, inpelm, 8 )

            case(14)

!           --- Quadratic hexahedron

               call intmap3d14c ( kmeshc(istart), inelem, &
                                  nunkploc, number, usol, usolcop, &
                                  ielgrp, inpelm )

            case(21)

!           --- Linear prism

               call intmap3d11c ( kmeshc(istart), inelem, &
                                  nunkploc, number, usol, usolcop, &
                                  ielgrp, inpelm, 6 )

         end select  ! case ( ishape )
         istart = istart+inelem*inpelm    ! raise starting address

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( usolcop, npoint, nunkploc*2, 'usolcop' )
         write(irefwr,*) 'End sepintmap3d3c'

      end if  ! ( debug )
      call erclos ( 'sepintmap3d3c' )

      end
