      subroutine sepstream1 ( ichois, startnode, knodp, psi, strm0, &
                              ielhlp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find start point (knodp) for computation of stream function
!     corresponding element (ielem) and start value (psi)
! **********************************************************************
!
!                       KEYWORDS
!
!     compute
!     incompressible
!     stream_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ichois, startnode, ielhlp(*)
      integer, intent(out) :: knodp
      double precision, intent(in) :: strm0
      double precision, intent(out) :: psi

!     ichois         i    If 1 ielhlp is used
!     ielhlp         i    This array of length nelgrp+5 is only used if
!                         allelements = 1
!                         ielhlp(ielgrp+5) = 0 means elements of element group
!                         ielgrp are skipped, ielhlp(ielgrp+5) = 1, means that
!                         they are assembled
!     knodp          o    Nodal point number of start node
!     psi            o    Value of stream function in start node
!     startnode      i    Starting node number
!     strm0          i    Initial value of stream function
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     179   point number of user point is incorrect
!     180   user point is not a nodal point
!     215   Expected two-dimensional problem on two-dimensional mesh does
!           not exist or starting point is not lying in an accessible element
!           group.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstream1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepstream1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( startnode==0 ) then

!     --- find which point is the first one in a "good" element group

         if ( ichois==0 ) then

!        --- Element groups are not skipped

            knodp=1
            ielem=1

         else if ( ichois==1 ) then

!        --- Some element groups are skipped

            ielem = 0

            do ielgrp = 1, nelgrp

               itype = kprobc(ielgrp)

               if ( itype>0 .and. ielhlp(5+ielgrp)/=0 .and. &
                    kmeshf(ielgrp)>2 ) then

!              --- Type number positive and element group must not be skipped
!                  and element is surface element

                  ielem = pointnel(ielgrp)+1  ! first element of this group
                  knodp = kmeshc(pointkmsc(ielgrp)+1)  ! first node
                  exit

               end if  ! ( itype>0 .or. ielhlp(5+ielgrp)/=0 )

            end do  ! ielgrp = 1, nelgrp

            if ( ielem==0 ) then

!           --- No element found

               call errsub ( 215, 0, 0, 0 )
               go to 1000

            end if  ! ( ielem==0 )

         end if  ! ( ichois==0 )
         psi=0.0d0

!        --- the element group of the other possibilities is selected later

      else if ( startnode<0 .and. startnode>=-nuspnt ) then

!     --- startnode negative hence a user point

         knodp = kmeshl(-startnode)

         psi = strm0

         if ( knodp==0 ) then
            call errint ( -startnode, 1 )
            call errsub ( 180, 1, 0, 0 )
            go to 1000
         end if

      else if ( startnode>0 .and. startnode<=npoint ) then

!     --- startnode positive hence a node

         knodp = startnode
         psi = strm0

      else

!     --- error 179 : point number out of range

         call errint ( startnode, 1 )
         call errint ( npoint, 2 )
         call errint ( 0, 3 )
         call errint ( 0, 4 )
         call errsub ( 179, 4, 0, 0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'knodp, ielem', knodp, ielem
         write(irefwr,*) 'End sepstream1'

      end if  ! ( debug )
      call erclos ( 'sepstream1' )

      end subroutine sepstream1
