      subroutine sepstream6 ( iwork, ichels, stream, velocity, &
                              inpvec, istrm, knodp, numelm, nunkploc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute stream function in all nodes except start node
!
! **********************************************************************
!
!                       KEYWORDS
!
!     compute
!     incompressible
!     stream_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: iwork(*), numelm
      integer, intent(in) :: ichels, inpvec, &
                             istrm, knodp, nunkploc
      double precision, intent(inout) :: stream(*)
      double precision, intent(in) :: velocity(*)

!     ichels         i    Choice parameter for the element subroutine
!     inpvec         i    Input vector from which the stream function must be
!                         computed
!     istrm          i    Output vector containing information of the stream
!                         function
!     iwork         i/o   integer work array. See sepstream
!     knodp          i    node number
!     numelm        i/o   Number of elements to be done
!     nunkploc       i    maximal number of degrees of freedom in a nodal point
!                         for output vector
!     stream        i/o   stream function
!     velocity       i    velocity vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: knpelm, lstep, melgrp, i, icouno, ieljel, &
                 ikelmc, shiftpra, ilast, inodp, jel, jnodp, jtel, &
                 next, numel, inunkp, inusol, nunkpsav
      integer, pointer :: index1(:), index2(:), index3(:), &
                              index4(:), index5(:)
      double precision, allocatable :: elemvc(:)
      double precision :: strmj

!     elemvc         Element vector
!     i              Counting variable
!     icouno         Number of unknowns in output vector
!     ieljel         Neigbor element
!     ikelmc         position in kmesh part c
!     ilast          Last element used
!     index1         Array of length inpelm containing the point numbers
!                    of the nodal points in the element
!     index2         Array of length icount containing the positions of the
!                    degrees of freedom in the element with respect to USOL
!     index3         contains cumulative number of degrees of freedom in
!                    nodal points ; length = inpelm+1, index3(1)=0
!     index4         See index2 but now for output vector
!     index5         See index5 but now for output vector
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     jel            Loop variable
!     jnodp          Nodal point sequence number
!     jtel           Counts number of elements treated
!     knpelm         Number of nodes per element
!     lstep          Step variable for vertices. Usually lstep = 1,
!                    but when the solution must be computed in the vertices
!                    and the element is quadratic then lstep = 2
!     melgrp         prior element group
!     next           next element number
!     numel          number of neighbor elements
!     nunkpsav       Saves value of nunkp
!     shiftpra       Start position of kproba/e in kprob
!     strmj          Stream function value in node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELS000         Element subroutine for stream function
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPFILLINDEX24 Fill arrays index2 and index4 for a given vector
!                    Corresponds to one element group
!     SEPPOINTKMESHC Fill array pointkmsc with starting addresses of element
!                    groups in kmeshc (minus 1)
!     SEPSTREAM4     Compute knpelm and lstep as function of ielgrp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     532   Not all element groups from array IELHLP could be reached.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstream6' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepstream6'
         call prinin ( pointkmsc, nelgrp, 'pointkmsc' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Adapt pointkmsc in order to simplify expression for ikelmc

      do i = 2, nelgrp
         knpelm = sepgetinpelm ( i )
         pointkmsc(i) = pointkmsc(i)-pointnel(i)*knpelm
      end do  ! i = 1, nelgrp

      if ( debug ) call prinin ( pointkmsc, nelgrp, 'pointkmsc' )
      melgrp = 0         ! prior element group
      ilast = ielem
      jtel = 1
      jnodp = knodp

!     --- start loop through elements
!         in which group of elements is actual element ielem ?

      do while ( jtel<=numelm .and. ielem>0 )

         if ( nelgrp>1 ) ielgrp = sepelgrpelem(ielem)

         if ( melgrp/=ielgrp ) then

!        --- New element group
!            Compute knpelm, lstep, icount and icouno

            call sepstream4 ( knpelm, lstep )
            if ( inpelm==2 ) then

!           --- meshconnect elements: special treatment

               icouno=2
               icount=2

            else

               icount = sepfillcountvc ( inpvec, ielgrp )
               icouno = sepfillcountvc ( istrm, ielgrp )

            end if

         end if  ! ( melgrp/=ielgrp )

         melgrp = ielgrp   ! set prior element group

!        --- create a chain list, i.e. find neighbors of ielem, and
!            also find in which vertex the psiphi  function is known already
!            position of nodal points of ielem  : ikelmc

         ikelmc=pointkmsc(ielgrp)+(ielem-1)*inpelm

         do i = 1, knpelm, lstep

!        --- find for each vertex in element ielem

            inodp=kmeshc(ikelmc+i)

!           --- find number of elements in which nodal point i is lying
!               skip points with numel=1

            numel=kmshd1(1+inodp)-kmshd1(inodp)

            do jel=1,numel

!           --- find an element in which inodp is lying
!               detect if this element has already been 'chained' or 'treated'

               ieljel=kmshd2(kmshd1(inodp)+jel)

!              --- detect the nodal point inodp with known value of psiphi
!                  function
!                  the last one found is actually used

               if (iwork(ieljel)==-1) jnodp=inodp

!              --- extend chain list with not earlier treated elements

               if (iwork(ieljel)==-3) then

!              --- set pointer to this element equal to 0

                  iwork(ieljel)=0

!                 --- set pointer with last element number equal to this element

                  iwork(ilast)=ieljel

!                 --- update number of last element of chain

                  ilast=ieljel

               end if  ! (iwork(ieljel)==-3)
            end do  ! jel=1,numel

         end do  ! i = 1, knpelm, lstep

!            local parameters
!            elemvc element vector
!            index1 contains pointers to coordinates of nodal points
!            index2 contains pointers to degrees of freedom of input vector
!                   in large input vector (type 110,115,116)
!            index3 contains cumulative number of degrees of freedom in
!                   nodal points ; length = inpelm+1, index3(1)=0
!            index4,index5 like index2,index3 for output vector (110,115,116)
!        --- create (extend) temporary array

         length = 2*icouno + 3*inpelm + icount
         allocate ( elemvc(icouno), index2(icount), index3(inpelm+1), &
                    index4(icouno), index5(inpelm+1), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'elemvc' )
         if ( ierror/=0 ) go to 1000

         ikelmc = pointkmsc(ielgrp)+(ielem-1)*inpelm
         if (ierror/=0) go to 1000

!        --- fill array index1

         index1 => kmeshc(ikelmc+1:ikelmc+inpelm)
         if ( indpra>0 ) kprobatmp => kproba(ielgrp)%arr
         if ( inpelm/=2 ) then

!        --- fill arrays index2, index3, index4, and index5

               shiftpra = sepshiftkpra ( inpvec, ielgrp )
               kprobflc => kprobfi
               indprflc =indprfi
               nunkplc = nunkpi
               call sepfillindex24 ( inpvec, index1, &
                                     index2, index3, ikelmc  )

               shiftpra = sepshiftkpra ( istrm, ielgrp )
               kprobflc => kprobf
               indprflc =indprf
               nunkplc = nunkploc
               call sepfillindex24 ( istrm, index1, &
                                     index4, index5, ikelmc )
         end if
         nunkpsav = nunkp
         nunkp = nunkploc

!        --- find value of psiphi  function at jnodp

         inusol = sepgetinusol ( jnodp, inunkp )+1
         strmj = stream(inusol)

!        --- compute element vector and put element vector in output vector

         itype=kprobc(ielgrp)

         if ( itype>0 ) then

!        --- Standard element


            call els000 ( ichels, jnodp, strmj, elemvc, coor, index1, &
                          index2, index3, ishape )
            stream(index4(1:icouno)) = elemvc(1:icouno)

         else if (itype==-1) then

!        --- Connected elements (periodic boundary conditions)

            do i = 1, 2
               inusol = sepgetinusol ( index1(i), inunkp )+1
               stream(inusol) = strmj
            end do  ! i = 1, 2

         end if  ! ( itype>0 )
         nunkp = nunkpsav

         deallocate ( elemvc, index2, index3, index4, index5, &
                      stat = error )
         if ( error/=0 ) call erdealloc ( error, 'elemvc' )
         if ( ierror/=0 ) go to 1000

         nullify(index1)

!        --- change contents of iwork for actual element and find from
!            chain list which element must be treated next

         jtel=jtel+1
         next=iwork(ielem)
         iwork(ielem)=-1
         ielem=next

      end do  ! while ( jtel<=numelm .and. ielem>0 )

      if (jtel<=numelm) then
         call errint ( melgrp, 1 )
         call errint ( knodp, 2 )
         call errwar ( 532, 2, 0, 0 )
      end if

!     --- Reset pointkmsc

      call seppointkmeshc

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( stream,10,'stream')
         write(irefwr,*) 'End sepstream6'

      end if  ! ( debug )
      call erclos ( 'sepstream6' )

      end subroutine sepstream6
