      function tolineint ( isol, values )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute line integral over vector of type 119
!
! **********************************************************************
!
!                       KEYWORDS
!
!     integral
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isol
      double precision, intent(in) :: values(3)
      double precision :: tolineint

!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     tolineint      o    Computed integral
!     values         i    Double precision or complex array in which the real
!                         or complex information concerning the line is stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, numnodes, numsubcurvs
      double precision, allocatable :: work(:,:)

!     i              Counting variable
!     numnodes       Number of nodal points in intersection
!     numsubcurvs    Number of sub curves
!     work           Work array to store coordinates and function values
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: tolineint1

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILL119      Fill array outvec with the information of array ivectr
!                    of type 119
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     TOLINEINT1     Compute the 1d integral over func, where the coordinates
!                    are given in x
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1438   Intersection consists of sub parts
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tolineint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tolineint'
         call prinin ( isol, 5, 'isol' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting address as well as some extra information

      numsubcurvs = ks(isol)%n   ! number of subcurves

      if ( numsubcurvs>1 ) then

!     --- Error: the number of subcurves is more than 1

         do i = 1, 3
            call erreal ( values(i), i )
         end do  ! i = 1, 3
         call errint ( numsubcurvs, 1 )
         call errsub ( 1438, 1, 3, 0 )
         go to 1000

      end if  ! ( numsubcurvs>1 )

      isoltemp => ks(isol)%isol
      numnodes = sum(isoltemp(1:numsubcurvs)) ! number of nodes

!     --- Copy vectors in temporary vector work

      allocate ( work(numnodes,2), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, numnodes*2, 'work' )
      if ( ierror/=0 ) go to 1000

      call erfill119 ( isol, 1, numsubcurvs, work, numnodes )

!     --- Compute integral

      tolineint = tolineint1 ( work(1,1), work(1,2), numnodes )

!     --- remove temporary array

      deallocate ( work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tolineint'

      end if  ! ( debug )
      call erclos ( 'tolineint' )

      end function tolineint
