      subroutine plafrd ( ierrtn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 25-04-2010 New call to plaf05
!        version  2.2  date 05-04-1996 New call to plaf05
!        version  2.1  date 28-06-1994 changes in datafile format
!
!   copyright (c) 1991-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read device independent SEPRAN plot file and plot the contents of this
!   file to the device chosen by the parameter posdev.
!   This subroutine allows for zooming
!   The plotting file must have been opened before, and is not closed by
!   this subroutine
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemach
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ierrtn

!     ierrtn  o   Error return code with the following meaning:
!                 0:  No error has been found
!                 1:  The file with reference number |irefpl| is empty
!                 2:  An incorrect record has been read in the plot file
!                 3:  The plot file does not end with the correct record
!                 4:  The plot file contains a string with too much characters
!                 5:  The plot file contains a polygon consisting of too many
!                     points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxpol, maxchr, ios, ichois, j, nchar, icolor, npolyg, &
              iref, icolsv, i, iside
      parameter ( maxpol = 100, maxchr = 100 )
      real x, y, angle, height, polyg(2*maxpol)
      double precision xp, yp, work(2*maxpol), xpboun, ypboun
      character string*(maxchr)
      logical finished

!    angle          angle read from the SEPRAN neutral plot file
!    finished       Indicates if the while loop is finished or not
!    height         height read from the SEPRAN neutral plot file
!    i              Counting variable
!    ichois         Choice parameter read from the SEPRAN neutral plot file
!    icolor         Colour number read from the SEPRAN neutral plot file
!    icolsv         Help variable to save the value of dcolor temporarily
!    ios            Parameter indicating the i/o status
!    iref           Plot file reference number
!    iside          Indicates at which side the cross section is positioned
!                   Possible values:
!                   0: There is no cross section point computed in this call
!                   1: The cross section point is at side y=ymint
!                   2: The cross section point is at side x=xmaxt
!                   3: The cross section point is at side y=ymaxt
!                   4: The cross section point is at side x=xmint
!    j              General loop variable
!    maxchr         Maximum number of characters allowed in a string
!                   Length of string
!    maxpol         Maximum number of polygon points allowed
!                   Dimension for array POLYG
!    nchar          Parameter read from the SEPRAN neutral plot file
!    npolyg         Number of points in the polygon to be filled
!    polyg          Contains the coordinates of the polygon to be filled
!                   This array is read from the SEPRAN neutral plot file
!    string         Contains the text to be plotted
!                   This string is read from the SEPRAN neutral plot file
!    work           Double precision copy of polyg
!    x              x-coordinate read from the SEPRAN neutral plot file
!    xp             x-coordinate in plotting units
!    xpboun         x-coordinate of point at the boundary of the plotting region
!    y              y-coordinate read from the SEPRAN neutral plot file
!    yp             y-coordinate in plotting units
!    ypboun         y-coordinate of point at the boundary of the plotting region
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     INICLS     Clear screen
!     PLAF01     Translate user coordinates into plotter coordinates
!     PLAF05     Plot polygon
!     PLAFCV     Compute scaling factors in case of zooming
!     PLAFP3     Machine dependent plotting of symbols
!     PLAFP6     Standard machine dependent plotting
!     PLAFP7     Plot text using absolute co-ordinates
!     PLAFP8     Machine dependent plotting of filling of polygons
! **********************************************************************
!
!                       I/O
!
!    The standard SEPRAN device independent file is read
!    The structure of the SEPRAN neutral plot file is as follows:
!    Each picture consists of a number of records.
!    The standard records have the following (single precision) contents:
!    ICHOIS, X, Y, ANGLE, HEIGHT, NCHAR
!    The parameter ICHOIS indicates the type of action to be performed
!    The meaning of the other parameters depends on ICHOIS
!    Possible values:
!    1.   Start the plot. The parameter Y indicates the length of the plot
!         in cm's. The parameter X indicates if small sized paper (1) or wide
!         paper should be used (2). Usually X has no meaning
!    2.   Plot with pen down to point X, Y.
!         Use colour NCHAR, if NCHAR=0 no special colour is used.
!    3.   Plot with pen up (i.e. move) to point X, Y
!    4.   Not in the file
!    5.   Close plot
!    6.   Plot text of length mod(NCHAR,100) characters and height HEIGHT
!         cm's under an angle of ANGLE degrees with X, Y as coordinates of
!         left under point of first character. Use colour NCHAR/100.
!         If NCHAR/100 = 0, no special colour is required.
!         The next record contains the text to be plotted
!    7.   Fill polygon with colour X. The number of points in the polygon
!         is given by NCHAR.
!         The next records contain the coordinates of the polygon in the
!         sequence x1,y1, x2,y2, . . . , xNCHAR,yNCHAR
!    8.   Plot symbol with symbol number mod(NCHAR,100) and center X,Y
!         Use colour NCHAR/100.
!         If NCHAR/100 = 0, no special colour is required.
!    Plotting information is written to the plot device
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Set reference number of plot file
!   Read and check first record
!   Set parameters in common cplaf in case of zooming
!   finished := false
!   while ( not finished ) do
!      read and check next record including parameter ichois
!      if ( ichois=2 or 3 ) then
!         move pen
!      else if ( ichois=5 ) then
!         close plot
!         finished := true
!      else if ( ichois=6 ) then
!         read text
!         plot text
!      else if ( ichois=7 ) then
!         read polygon
!         plot filled polygon
!      else if ( ichois=8 ) then
!         plot symbol
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      iref = abs(irefpl)
      ierrtn = 0
      icolsv = dcolor

!     --- Read first record

      if ( irefpl<0 ) then
         read (iref,iostat=ios) ichois,x,y,angle,height,nchar
      else
         read (iref,*,iostat=ios) ichois,x,y,angle,height,nchar
      end if

!     --- Check first record

      if ( ios<0 ) then

!     --- ios<0, end of file has been reached

         ierrtn = 1
         go to 1000

      else if ( ios>0 .or. ichois/=1 ) then

!     --- ios>0,or ichois # 1, contents of file incorrect

         ierrtn = 2
         go to 1000

      end if

!     --- Clear screen:

      if ( posdev(1:1)/='p' ) call inicls

!     --- Compute new parameters in common cplaf because of zooming

      jtest = 1
      ioutsd = 0
      if ( lzoom>0 ) call plafcv

!     --- Open plot

      xp = x
      yp = y
      call plafp6 ( ichois, xp, yp )

!     --- Read rest of records until end record or end of file is found

      finished = .false.
      do while ( .not. finished )

!     --- Read next record

         if ( irefpl<0 ) then
            read (iref,iostat=ios) ichois,x,y,angle,height,nchar
         else
            read (iref,*,iostat=ios) ichois,x,y,angle,height,nchar
         end if
         if ( ios<0 ) then

!        --- end of record found before ichois = 5

            ierrtn = 3
            call plafp6 ( 5, dble(x), dble(y) )
            go to 1000

         else if ( ios>0 ) then

!        --- wrong record found

            ierrtn = 2
            go to 1000

         end if

         if ( ichois==2 .or. ichois==3 ) then

!        --- ichois = 2 or 3, move with pen down or up

            if ( nchar>0 ) dcolor = nchar
            if ( lzoom==0 ) then
               xp = x
               yp = y
            else
               call plaf01 ( dble(x), dble(y), xp, yp, xpboun, ypboun, &
                             iside )
            end if
            if ( ioutsd<=2 ) call plafp6 ( ichois, xp, yp )

         else if ( ichois==6 ) then

!        --- ichois = 6, plot text

            if ( nchar>100 ) then
               dcolor = nchar/100
               nchar = mod ( nchar,100 )
            end if
            if ( nchar>maxchr ) then
               ierrtn = 4
               nchar = maxchr
            end if
            if ( irefpl<0 ) then
               read (iref,iostat=ios) string(1:nchar)
            else
               read (iref,110,iostat=ios) string(1:nchar)
110            format(a)
            end if

            if ( ios<0 ) then

!           --- end of record found before ichois = 5

               ierrtn = 3
               call plafp6 ( 5, dble(x), dble(y) )
               go to 1000

            else if ( ios>0 ) then

!           --- wrong record found

               ierrtn = 2
               go to 1000

            end if
            if ( lzoom==0 ) then
               xp = x
               yp = y
            else
               call plaf01 ( dble(x), dble(y), xp, yp, xpboun, ypboun, &
                             iside )
            end if
            if ( ioutsd<=1 ) call plafp7 ( xp, yp, string(1:nchar), &
               dble(angle), dble(height), nchar )

         else if ( ichois==7 ) then

!        --- ichois = 7, fill polygon with colour

            icolor = nint(x)
            if (y>0.0) dcolor = nint(y)
            if ( nchar>maxpol ) then
               ierrtn = 5
               nchar = maxpol
            end if
            npolyg = 2*nchar
            if ( irefpl<0 ) then
               read (iref,iostat=ios) (polyg(j),j=1,npolyg)
            else
               read (iref,*,iostat=ios) (polyg(j),j=1,npolyg)
            end if
            if ( ios<0 ) then

!           --- end of record found before ichois = 5

               ierrtn = 3
               call plafp6 ( 5, dble(x), dble(y) )
               go to 1000

            else if ( ios>0 ) then

!           --- wrong record found

               ierrtn = 2
               go to 1000

            end if
            if ( lzoom==0 ) then
               call plafp8 ( 1, icolor, nchar, polyg )
            else
               do 200 i = 1, npolyg
                  work(i) = polyg(i)
200            continue
               call plaf05 ( 1, icolor, nchar, work, 2 )
            end if

         else if ( ichois==8 ) then

!        --- ichois = 8, plot symbol

            if ( nchar>100 ) then
               dcolor = nchar/100
               nchar = mod ( nchar,100 )
            end if
            if ( lzoom==0 ) then
               xp = x
               yp = y
            else
               call plaf01 ( dble(x), dble(y), xp, yp, xpboun, ypboun, &
                             iside )
            end if
            if ( ioutsd<=1 ) call plafp3 ( nchar, xp, yp )

         else if ( ichois==5 ) then

!        --- ichois = 5, close plot

            call plafp6 ( ichois, dble(x), dble(y) )
            finished = .true.

         else

!        --- wrong record found (ichois incorrect)

            ierrtn = 2
            go to 1000

         end if

      end do  ! while ( .not. finished )

!     --- Stop the process until a return is given for the devices
!         using Calcomp, CGI or GKS

1000  if ( posdev(1:1)=='c' .or.  posdev(1:1)=='g' ) read(*,*)
      if ( posdev(1:1)/='p' ) call inicls
      dcolor = icolsv
      end
