      subroutine plfilllevels ( icompt, umin, umax, contln, ncntln, &
                                cont, ncont )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array with contour levels
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     plot
!     plot_contour
!     level
!     contour_level
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ncntln, icompt, ncont
      double precision, intent(in) :: contln(*)
      double precision, intent(inout) :: umin, umax
      double precision, intent(out) :: cont(*)

!     cont           o    Array with the contour levels
!     contln         i    User array of length ncntln + 5 (if used) in which the
!                         user is supposed to have stored the contour levels
!                         from position 6
!     icompt         i    indicates how the contour levels must be
!                         computed from NCNTLN in the case NCNTLN<=0
!                         0:  The minimum and maximum level are computed
!                         1:  The minimum level is given in contln(6), the
!                             maximum level must be computed
!                         2:  The maximum level is given in contln(7), the
!                             minimum level must be computed
!                         3:  The minimum level is given in contln(6), the
!                             maximum level in contln(7)
!     ncntln         i    Parameter to compute the number of contour levels
!     ncont          i    Number of contour levels
!     umax          i/o   largest value of function
!                         may be rounded
!     umin          i/o   smallest value of function
!                         may be rounded
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, m, n
      double precision :: stepx

!     i              Counting variable
!     m              Dummy parameter
!     n              Dummy parameter
!     stepx          Step value between maximum and minimum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PL0004         Round minimum and maximum values
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plfilllevels' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plfilllevels'
         write(irefwr,1) 'icompt, ncntln, ncont', icompt, ncntln, ncont
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( icompt>0 ) then

!     --- icompt>0, use umin, umax

         if ( icompt==1 .or. icompt==3 ) umin = contln(6)
         if ( icompt==2 .or. icompt==3 ) umax = contln(7)

      else if ( ncntln>0 ) then

!     --- icompt = 0  and  ncntln>0   Copy contln into ibuffr

         cont(1:ncont) = contln(6:5+ncont)

      end if  ! ( icompt>0 )

      if ( ncntln<0 ) then

!     --- Contour levels must be rounded

         call pl0004 ( umin, umax, m, n )

         if ( abs(umax-umin)<=sqreps*(abs(umax)+abs(umin)) ) &
            umin = umax

      end if  ! ( ncntln<0 )

      if ( ncntln<=0 ) then

!     --- Contour levels must be computed

         stepx = ( umax-umin ) / ( ncont-1d0 )
         do i = 1, ncont
            cont(i) = umin + (i-1d0) * stepx
         end do  ! i = 1, ncont

      end if  ! ( ncntln<=0 )

1000  call erclos ( 'plfilllevels' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'umin, umax', umin, umax
         call prinrl ( cont, ncont, 'cont' )
         write(irefwr,*) 'End plfilllevels'

      end if  ! ( debug )

      end subroutine plfilllevels
