      subroutine plfl03 ( xp, yp, eloc, ibnd, in, f, nbndpt, lbrpnt, &
                          esol, user, iuser )
! ======================================================================
!
!        programmer    Arno van Duin
!        version  1.1  date 03-07-1992 Replace addflux by addflx
!        version  1.0  date 12-07-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Find point that is flux-cut F away from (xp,yp).
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibnd, in, iuser(*), lbrpnt, &
              nbndpt(2*lbrpnt)
      double precision eloc(2), esol(*), f, user(*), xp, yp

!     eloc   i/o   electric field in point (xp,yp)
!     esol    i    electric field
!     f       i    required flux-cut
!     ibnd   i/o   point number of next point on the boundary
!     in      i    indicator to check for inward(2) or outward(3) flux
!     iuser   i    array containing information about the coefficients
!     lbrpnt  i    number of nodal points on the boundary
!     nbndpt  i    array with the consecutive nodal points of the boundary
!     user    i    array containing information of the coefficients
!     xp      i    x-coordinate of starting point on the boundary
!             o    x-coordinate of point with flux-cut F from the old (xp,yp)
!     yp      i    y-coordinate of starting point on the boundary
!             o    y-coordinate of point with flux-cut F from the old (xp,yp)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer aant1, aant2, errno, i, ielem, igroep, ikn1, ikn2, isign, &
              istart, j, nrofel
      double precision addflx, e1n, e1nold, epn, eps, flux, labda, &
                       x, x1, x2, y, y1, y2
      logical herh

!     aant1   number of elements node 1 belongs to
!     aant2   number of elements node 2 belongs to
!     addflx  local flux-cut
!     e1n     normal component of the electric field in the next boundary point
!     e1nold  normal component of the electric field in the previous point
!     epn     normal component of the electric field in the starting point
!     eps     dielectric constant
!     errno   error number
!     flux    total flux-cut from (xp,yp) onwards
!     herh    while variable
!     i       count variable
!     ielem   element number
!     igroep  group number
!     ikn1    node number of previous boundary point
!     ikn2    node number of next boundary point
!     isign   indicator to check for inward (1) or outward (-1) flux
!     istart  pointer to place in user where coefficients are stored
!     j       count variable
!     labda   distance between old point and new point
!     nrofel  number of elements
!     x       x-coordinate of next boundary point
!     x1      x-coordinate of crossing point
!     x2      x-coordinate of previous boundary point
!     y       y-coordinate of next boundary point
!     y1      y-coordinate crossing point
!     y2      y-coordinate of previous boundary point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision funccf

!      ERCLOS  error subroutine, deletes routine name from string
!      EROPEN  error subroutine, concatenates routine name to string
!      ERRSUB  error message routine
!      FUNCCF  function to determine the coefficient
!      INSTOP  stops the program
!      PLFL04  interpolate to find the new coordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!         1500    no element found that contains both nodal points
!         1501    coefficient equals zero which is not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
!         Walk from starting point to next point until the flux-cut equals F.
!         Flux-cut := 0
!         While flux-cut < F and not end of boundary do
!            Take next point
!            Calculate normal components of E-field:
!             En = ||E|| * sin ( gamma ) * ( ||dx|| / ||dx|| ) = &
!                = dx x E / ||dx|| = &
!                = ( dx1 * E2 - dx2 * E1 ) / length
!            If normal components point in opposite directions then
!               Calculate local flux
!               If flux-cut + local flux < F then
!                  Flux-cut := flux-cut + local flux
!                  Search next boundary point that has a normal component that
!                  points in the same direction
!                  If point is found then
!                     Calculate coordinates and E-field in the crossing point
!                  Endif
!               Else
!                  Calculate coordinates of crossing point
!                  Find new coordinates
!                  Calculate E-field in the new point
!                  Flux-cut := F
!               Endif
!            Else
!               Calculate local flux
!               If flux-cut + local flux < F then
!                  Flux-cut := flux-cut + local flux
!               Else
!                  Find new coordinates
!                  Calculate E-field in the new point
!                  Flux-cut := F
!               Endif
!            Endif
!         Endwhile
!         If flux-cut < F then
!            End of line
!         Endif
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plfl03' )

!     --- reset local flux-cut

      flux = 0d0

!     --- determine inward/outward

      if ( in==2 ) then
         isign = 1
      else
         isign = -1
      end if

!     --- while not end of boundary and flux-cut less than F do

10    if ( ibnd<=lbrpnt .and. flux<F ) then

!     --- get previous and next boundary point

         ikn1  = nbndpt(2*ibnd-1)
         ikn2  = nbndpt(2*ibnd)

!        --- get the number of elements these points belong to

         aant1 = kmeshd(ikn1+1) - kmeshd(ikn1)
         aant2 = kmeshd(ikn2+1) - kmeshd(ikn2)

!        --- determine the element they both belong to

         ielem = 0
         i = 1
20       if ( ielem==0 .and. i<=aant1 ) then
            j = 1
30          if ( ielem==0 .and. j<=aant2 ) then
               if ( kmeshd(npoint+1+kmeshd(ikn1)+i)==&
                    kmeshd(npoint+1+kmeshd(ikn2)+j)) then
                  ielem = kmeshd(npoint+1+kmeshd(ikn1)+i)
               end if
               j = j + 1
               go to 30
            end if
            i = i + 1
            go to 20
         end if

!        --- if no element found then return error message

         if ( ielem==0 ) then
            errno = 1500
            call errsub ( errno, 0, 0, 0 )
            call instop
         end if

!        --- get the coordinates of the next boundary point

         x = coor(1,ikn2)
         y = coor(2,ikn2)

!        --- Determine elementgroup for this element

         igroep = 1
         if ( kelmg/=0 ) then
            nrofel = 0
            herh = .true.
40          if ( herh ) then
               nrofel = nrofel + kmeshg(igroep)
               if ( ielem>nrofel ) then
                  igroep = igroep + 1
               else
                  herh = .false.
               end if
               goto 40
            end if
         end if

!        --- Determine dielectric constant for this element

         istart = iuser(5+igroep)
         if ( iuser(istart)==0 ) then
            errno = 1501
            call errsub ( errno, 0, 0, 0 )
            call instop
         else if ( iuser(istart)<-5 ) then
            eps = user(-iuser(istart))
         else
            eps = funccf ( iuser(istart), x, y, 0d0 )
         end if

!        --- get the distance between the starting point and the next boundary
!            --- point

         length = sqrt ( (x - xp) * (x - xp) + (y - yp) * (y - yp) )

!        --- if distance is too small then move on to the next boundary point

         if ( length<1d-20 ) then
            ikn2 = nbndpt(2*ibnd)
            xp = coor(1,ikn2)
            yp = coor(2,ikn2)
            Eloc(1) = Esol(2*ikn2-1)
            Eloc(2) = Esol(2*ikn2)
            ibnd = ibnd + 1
         else

!        --- else
!               determine the normal component of the electric field in both the
!               starting point and the next boundary point

            Epn = (Eloc(2) * (x - xp) - Eloc(1) * (y - yp)) / length
            E1n = ( Esol(2*ikn2)   * ( x - xp ) - &
                    Esol(2*ikn2-1) * ( y - yp ) ) / length

!           --- if the normal component in the next boundary point points in the
!               wrong direction then

            if ( isign/=int(sign(1d0,E1n)) ) then
               if ( isign/=int(sign(1d0,Epn)) ) then
                  labda = 0d0
               else

!              --- interpolate to find crossing point and local flux-cut

                  labda = length * Epn / ( Epn - E1n )
               end if
               if ( labda<0d0 .or. labda>length ) labda = 0d0
               addflx = abs(.5d0 * eps * labda * Epn)

!              --- if local flux-cut is not enough then

               if ( addflx + flux<F ) then
                  flux = flux + addflx
                  E1nold = E1n

!                 --- while pointing in the wrong direction and not end of
!                     --- boundary

50                if ( isign/=int(sign(1d0,E1n)) .and. &
                       ibnd<lbrpnt ) then

!                 --- get next boundary point

                     xp = x
                     yp = y
                     ibnd = ibnd + 1
                     ikn2 = nbndpt(2*ibnd)
                     x = coor(1,ikn2)
                     y = coor(2,ikn2)

!                    --- determine the normal component of the electric field in
!                        that point

                     length = sqrt ( (x - xp) * (x - xp) + &
                                     (y - yp) * (y - yp) )
                     E1nold = E1n
                     E1n = ( Esol(2*ikn2)   * ( x - xp ) - &
                             Esol(2*ikn2-1) * ( y - yp ) ) / length

!                    --- endwhile

                     go to 50
                  end if

!                 --- if pointing in the right direction then

                  if ( isign==int(sign(1d0,E1n)) ) then

!                 --- interpolate to find crossing point

                     ikn1   = nbndpt(2*ibnd-1)
                     ikn2   = nbndpt(2*ibnd)
                     x      = coor(1,ikn2)
                     y      = coor(2,ikn2)
                     x2     = coor(1,ikn1)
                     y2     = coor(2,ikn1)
                     length = sqrt((x - x2)*(x - x2) + (y - y2)*(y-y2))
                     labda  = E1nold * length / ( E1nold - E1n )
                     if ( labda<0 .or. labda>length) labda = 0

!                    --- store local electric field values and coordinates

                     Eloc(1) = labda * (Esol(2*ikn2-1) - Esol(2*ikn1-1)) &
                               / length + Esol(2*ikn1-1)
                     Eloc(2) = labda * (Esol(2*ikn2) - Esol(2*ikn1)) &
                               / length + Esol(2*ikn1)
                     xp = (x - x2) * labda / length + x2
                     yp = (y - y2) * labda / length + y2
                  else
                     ibnd = ibnd + 1
                  end if
               else

!              --- else
!                     interpolate to find new point

                  flux = F - flux
                  x1 = ( x - xp ) * labda / length + xp
                  y1 = ( y - yp ) * labda / length + yp
                  length = labda
                  call plfl04 ( xp, yp, Epn, labda, x1, y1, 0d0, eps, &
                                flux )
                  ikn2 = nbndpt(2*ibnd)

!                 --- store local electric field

                  Eloc(1) = (Esol(2*ikn2-1) - Eloc(1)) * labda/length + &
                             Eloc(1)
                  Eloc(2) = (Esol(2*ikn2)   - Eloc(2)) * labda/length + &
                             Eloc(2)
                  flux = F
               end if
            else

!           --- else
!                  calculate local flux-cut

               if ( isign/=int(sign(1d0,Epn)) ) then
                  labda   = length * Epn / ( Epn - E1n )
                  addflx = abs(.5d0 * eps * ( length - labda ) * Epn)
                  Eloc(1) = labda * (Esol(2*ikn2-1) - Eloc(1)) / length &
                            + Eloc(1)
                  Eloc(2) = labda * (Esol(2*ikn2) - Eloc(2)) / length &
                            + Eloc(2)
                  xp = (x - xp) * labda / length + xp
                  yp = (y - yp) * labda / length + yp
               else
                  addflx = abs(.5d0 * eps * length * (Epn + E1n))
               end if

!              --- if local flux-cut is not enough

               if ( flux + addflx - F<-1d-6 * F ) then

!              --- get to next boundary point

                  flux = flux + addflx
                  ikn2 = nbndpt(2*ibnd)
                  xp = coor(1,ikn2)
                  yp = coor(2,ikn2)
                  Eloc(1) = Esol(2*ikn2-1)
                  Eloc(2) = Esol(2*ikn2)
                  ibnd = ibnd + 1
               else

!              --- else
!                     interpolate to find new point

                  ikn2 = nbndpt(2*ibnd)
                  x = coor(1,ikn2)
                  y = coor(2,ikn2)
                  flux = F - flux
                  call plfl04 ( xp, yp, Epn, labda, x, y, E1n, eps, &
                                flux )

!                 --- store local electric field

                  Eloc(1) = (Esol(2*ikn2-1) - Eloc(1)) * labda/length + &
                             Eloc(1)
                  Eloc(2) = (Esol(2*ikn2)   - Eloc(2)) * labda/length + &
                             Eloc(2)
                  flux = F
               end if
            end if
         end if

!        --- endwhile

         go to 10
      end if

!     --- if end of boundary then return 0

      if ( flux<F ) ibnd = 0
      call erclos ( 'plfl03' )
      end
