      subroutine plfr21 ( iplots, rplots )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 1.4    date   05-06-1998 Include commons
!        version 1.3    date   21-04-1995 Coordinate basis added
!        version 1.2    date   07-04-1995 Spectral elements added
!
!   copyright (c) 1992-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot all the curves of a SEPRAN mesh in 3-D and eventually one of
!     the surfaces as a solid or as local elements
! **********************************************************************
!
!                       KEYWORDS
!
!     projection
!     3d
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iplots(*)
      double precision :: rplots(*)

!     iplots  i   Integer array in which the following quantities must be
!                 stored:
!                 1:  Highest position that has been filled by the user
!                 2:  choice parameter for type of projection
!                     Possible values:
!                      0 : parallel projection plot
!                      1 : perspective projection plot
!                     10 : parallel projection plot (View point automatic)
!                     11 : perspective projection plot (View point automatic)
!                     Default value: 1
!                 3:  First colour number
!                 4:  Second colour number
!                 5:  Curve number or surface number that has to be plotted
!                     with the second colour
!                 6:  Indicator whether only curves (=0) have to be considered
!                     (default) or also a surface (=1) as wireframe or as a
!                     solid (=2)
!                 7:  Indicator whether kmesh part n has to be used (=0, &
!                     default) or kmesh parts c and t (=1)
!                 8:  Indicator for basic coordinate system
!                       1   :   x - y - z
!                       2   :   z - x - y
!                       3   :   y - z - x
!     rplots  i   Double precision input array that must be filled as follows:
!                 1:    plotfm
!                 2:    yfact
!                 3-5:  eye-point
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i     , ic    , ik    , ikader, ikn   , indkmn, ipl   , &
              ipl6  , is    , ist   , istart, iuppl , npelmlc , nelemlc , &
              igrelm, istelm, ilaelm, nelemh, isitua, &
              in(3)
      double precision axis(2,4), dx, dy    , eps   , eta(3), etaa  , &
              plotfm, yfact , s     , sprev , smean , ski(3), skia  , &
              vact(3), xmean(3), x  , xa    , xao   , xdef  , xl    , &
              xmaxp , xminp , xprev , y     , ya    , yao   , ydef  , &
              yl    , ymaxp , yminp , yprev , z     , za    , zao   , &
              zdef  , zl
      logical klaar

!    axis       array to store the projection of the unit cross
!    dx         x-distance
!    dy         y-distance
!    eps        accuracy
!    eta        unity coordinate in projection plane
!    etaa       coordinate in projection plane
!    i          loop variable
!    ic         loop variable
!    igrelm     help variable to count number of elements
!    ik         loop variable
!    ikader     helpvarible to store plotting variable jkader
!    ikn        node number
!    ilaelm     help variable for last number of element of subgroup
!    in         helparray to store order of the basis system
!    indkmn     indicator whether kmesh part n has to be used
!    ipl        indicator, related to iplots(5)
!    ipl6       indicator, related to iplots(6)
!    is         loop variable
!    isitua     indicator for basis system related to iplots(8)
!    ist        local position number
!    istart     local position number
!    istelm     helpvariable to store first element number of subgroup
!    iuppl      helpvariable for penmovements
!    klaar      logical to registrate whether ready or not
!    nelemh     helpnumber of elements
!    nelemlc    number of elements
!    npelmlc    number of points in element
!    plotfm     length of plot
!    s          arc length
!    ski        unit vector in projection plane
!    skia       coordinate in projection plane
!    smean      half arclength of curve
!    sprev      local arclength
!    vact       actual viewpoint
!    x          coordinate
!    xa         coordinate
!    xao        helpvariable ( not really used here)
!    xdef       x-coordinate viewpoint
!    xl         coordinate
!    xmaxp      max x-coordinate
!    xmean      coordinates of centre of projection plane
!    xminp      min x-coordinate
!    xprev      helpvariable to store x-coordinates
!    y          coordinate
!    ya         coordinate
!    yao        coordinate (not really used here)
!    ydef       y-coordinate viewpoint
!    yfact      scaling factor
!    yl         coordinate
!    ymaxp      max y-variable
!    yminp      min y-variable
!    yprev      help variable to store y-coordinates
!    z          coordinate
!    za         coordinate
!    zao        coordinate (not really used here)
!    zdef       z-coordinate viewpoint
!    zl         coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   Resets old name of previous subroutine of higher level
!     EROPEN   Produces concatenated name of local subroutine
!     ERREAL   Place real value in error array
!     ERRWAR   Warning messages
!     PL0000   Start picture
!     PL0015   Plot axis-cross in picture
!     PLAFEP   General SEPRAN plotting routine
!     PLFR22   Plot surface
!     PLT351   Compute projection of point
!     PLT358   Compute the basis vectors in the projection plane
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      1708        Warning : viewpoint not OK
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all curves and plot (check whether curve should be
!       plotted with special colour)
!     Run (if needed) through all surfaces and plot all the elements of
!       the indicated surface in a special colour
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plfr21' )

!     --- Determine basis system

      if ( iplots(1).lt.8 ) then

!     --- Use default value

         isitua = 1

      else

!     --- Use user supplied value

         isitua = iplots(8)

      end if

      if ( isitua.eq.2 ) then

!     --- Basis is z - x - y

         in(1) = 3
         in(2) = 1
         in(3) = 2

      else if ( isitua.eq.3 ) then

!     --- Basis is y - z - x

         in(1) = 2
         in(2) = 3
         in(3) = 1

      else

!     --- Basis is x - y - z (default)

         in(1) = 1
         in(2) = 2
         in(3) = 3

      end if

      ikader = jkader
      if ( jkader.ge.0 ) jkader = 4

!     --- Run through all the curves and compute the extreme values

      xmin  =  rinfin
      ymin  =  xmin
      zmin  =  xmin
      xmax  = -xmin
      ymax  =  xmax
      zmax  =  xmax

      if ( iplots(1).ge.7 ) then
           indkmn = iplots(7)
      else
           indkmn = 0
      end if

      do ic = 1 , ncurvs

          ikn = kmeshm( 5+ic ) - kmeshm( 4+ic )

          ist = kmeshm(2) + kmeshm(4+ic) - 2

          do ik = 1 , ikn

              i =  kmeshm( ist+ik )

              x  =  coor( in(1), i )
              y  =  coor( in(2), i )
              z  =  coor( in(3), i )

              xmin = min ( xmin, x )
              xmax = max ( xmax, x )
              ymin = min ( ymin, y )
              ymax = max ( ymax, y )
              zmin = min ( zmin, z )
              zmax = max ( zmax, z )

          end do

      end do

!     --- Compute centre of projection plane and default eye - position

      xmean( in(1) ) = 0.5d0 * ( xmax - xmin )
      xmean( in(2) ) = 0.5d0 * ( ymax - ymin )
      xmean( in(3) ) = 0.5d0 * ( zmax - zmin )

      xdef = xmean(1) + ( xmax - xmin )
      ydef = xmean(2) - ( ymax - ymin )
      zdef = xmean(3) + ( zmax - zmin )

!     --- Check whether given values are allowed

      vact(1) = rplots( 2 + in(1) ) + xmean(1)
      vact(2) = rplots( 2 + in(2) ) + xmean(2)
      vact(3) = rplots( 2 + in(3) ) + xmean(3)

      if ( vact(1).lt.xmax .and. vact(1).gt.xmin .and. &
           vact(2).lt.ymax .and. vact(2).gt.ymin .and. &
           vact(3).lt.zmax .and. vact(3).gt.zmin ) then

!     --- Submitted values are not good , use default values

           call erreal ( vact(1), 1 )
           call erreal ( vact(2), 2 )
           call erreal ( vact(3), 3 )

           vact(1) = xdef
           vact(2) = ydef
           vact(3) = zdef

           call erreal ( vact(1), 4 )
           call erreal ( vact(2), 5 )
           call erreal ( vact(3), 6 )

           call errwar ( 1708, 0, 6, 0 )

      end if

!     --- Compute projection of the extreme points

      eps = 10 * epsmac

!     --- (Start with vectors of unity ski and eta )

      vact(1) = vact(1) - xmean(1) + 203.7 * eps
      vact(2) = vact(2) - xmean(2) - 201.0 * eps
      vact(3) = vact(3) - xmean(3) + 100.5 * eps

      call plt358(vact,ski,eta)

      xminp =  rinfin
      yminp =  rinfin
      xmaxp = -rinfin
      ymaxp = -rinfin

!     --- Determine extreme values

      xa = xmin - xmean(1)
      ya = ymin - xmean(2)
      za = zmin - xmean(3)

      call plt351(vact,xa,ya,za,ski,eta,xao,yao,zao,skia,etaa)

      xminp = min ( skia, xminp )
      xmaxp = max ( skia, xmaxp )

      yminp = min ( etaa, yminp )
      ymaxp = max ( etaa, ymaxp )

      xa = xmax - xmean(1)

      call plt351(vact,xa,ya,za,ski,eta,xao,yao,zao,skia,etaa)

      xminp = min ( skia, xminp )
      xmaxp = max ( skia, xmaxp )

      yminp = min ( etaa, yminp )
      ymaxp = max ( etaa, ymaxp )

      ya = ymax - xmean(2)

      call plt351(vact,xa,ya,za,ski,eta,xao,yao,zao,skia,etaa)

      xminp = min ( skia, xminp )
      xmaxp = max ( skia, xmaxp )

      yminp = min ( etaa, yminp )
      ymaxp = max ( etaa, ymaxp )

      xa = xmin - xmean(1)

      call plt351(vact,xa,ya,za,ski,eta,xao,yao,zao,skia,etaa)

      xminp = min ( skia, xminp )
      xmaxp = max ( skia, xmaxp )

      yminp = min ( etaa, yminp )
      ymaxp = max ( etaa, ymaxp )

      ya = ymin - xmean(2)
      za = zmax - xmean(3)

      call plt351(vact,xa,ya,za,ski,eta,xao,yao,zao,skia,etaa)

      xminp = min ( skia, xminp )
      xmaxp = max ( skia, xmaxp )

      yminp = min ( etaa, yminp )
      ymaxp = max ( etaa, ymaxp )

      xa = xmax - xmean(1)

      call plt351(vact,xa,ya,za,ski,eta,xao,yao,zao,skia,etaa)

      xminp = min ( skia, xminp )
      xmaxp = max ( skia, xmaxp )

      yminp = min ( etaa, yminp )
      ymaxp = max ( etaa, ymaxp )

      ya = ymax - xmean(2)

      call plt351(vact,xa,ya,za,ski,eta,xao,yao,zao,skia,etaa)

      xminp = min ( skia, xminp )
      xmaxp = max ( skia, xmaxp )

      yminp = min ( etaa, yminp )
      ymaxp = max ( etaa, ymaxp )

      xa = xmin - xmean(1)

      call plt351(vact,xa,ya,za,ski,eta,xao,yao,zao,skia,etaa)

      xminp = min ( skia, xminp )
      xmaxp = max ( skia, xmaxp )

      yminp = min ( etaa, yminp )
      ymaxp = max ( etaa, ymaxp )

!     --- Place values in x-y system

      xmin  = xminp - ( xmaxp-xminp ) / 10d0
      ymin  = yminp - ( ymaxp-yminp ) / 10d0
      xmax  = xmaxp + ( xmaxp-xminp ) / 10d0
      ymax  = ymaxp + ( ymaxp-yminp ) / 10d0

!     --- Compute projection of unit-cross
!         Origin

      xa = 0d0 - xmean(1)
      ya = 0d0 - xmean(2)
      za = 0d0 - xmean(3)

      call plt351( vact, xa, ya, za, ski, eta, xao, yao, zao, &
                   axis(1,1), axis(2,1) )

!     --- x-axis

      xa = xa + 1d0

      call plt351( vact, xa, ya, za, ski, eta, xao, yao, zao, &
                   axis(1,2), axis(2,2) )

!     --- y-axis

      xa = xa - 1d0
      ya = ya + 1d0

      call plt351( vact, xa, ya, za, ski, eta, xao, yao, zao, &
                   axis(1,3), axis(2,3) )

!     --- z-axis

      ya = ya - 1d0
      za = za + 1d0

      call plt351( vact, xa, ya, za, ski, eta, xao, yao, zao, &
                   axis(1,4), axis(2,4) )

      plotfm = rplots(1)
      yfact  = rplots(2)

      call pl0000 ( plotfm, yfact )

!     --- start picture

      call plafep ( 1, aleng, width, 0d0, 0d0, 0 )

      ipl = 0

      if ( iplots(1).gt.4 ) ipl = iplots(5)

!     --- Plot axis-cross

      call pl0015( axis, plotfm, isitua )

!     --- plot curves

      do ic = 1 , ncurvs

         ikn = kmeshm( 5+ic ) - kmeshm( 4+ic )
         ist = kmeshm(2) + kmeshm(4+ic) - 2
         iuppl = 4

         xprev = 0d0
         yprev = 0d0

         if ( ic.eq.ipl .and. nsurfs.eq.0 ) then

!        --- Set special colour

            dcolor = 2
            if ( iplots(1).gt.3 ) dcolor = iplots(4)

         else

!        --- Use standard colour

            dcolor = 5
            if ( iplots(1).gt.2 ) dcolor = iplots(3)
            dcolor = 5

         end if

         do ik = 1 , ikn

!        --- compute projection

            i  = kmeshm( ist + ik )

            xl = coor(in(1),i) - xmean(1)
            yl = coor(in(2),i) - xmean(2)
            zl = coor(in(3),i) - xmean(3)

            call plt351( vact, xl, yl, zl, ski, eta, &
                         xao, yao, zao, skia, etaa )

            if ( ik.eq.1 ) then

                 s     = 0d0

                 xprev = skia
                 yprev = etaa

            else

                 dx = skia - xprev
                 dy = etaa - yprev

                 s = s + sqrt( dx*dx + dy*dy )

                 xprev = skia
                 yprev = etaa

            end if

            call plafep ( iuppl, skia, etaa, 0d0, 0d0, 0 )
            iuppl = 5

         end do

!        --- For special curve

         if ( ic.eq.ipl .and. nsurfs.eq.0 ) then

            smean = s * 0.5d0

!           --- Find element in the center of the boundary

            do ik = 1 , ikn

!           --- compute projection

                i  = kmeshm( ist + ik )

                xl = coor(in(1),i) - xmean(1)
                yl = coor(in(2),i) - xmean(2)
                zl = coor(in(3),i) - xmean(3)

                call plt351( vact, xl, yl, zl, ski, eta, &
                             xao, yao, zao, skia, etaa )

                if ( ik.eq.1 ) then

                   xprev = skia
                   yprev = etaa
                   s     = 0d0
                   sprev = 0d0

                else

                   dx = skia - xprev
                   dy = etaa - yprev
                   sprev = s
                   s = s + sqrt( dx*dx + dy*dy )

                   xprev = skia
                   yprev = etaa

                end if

                if ( s.gt.smean ) then

!               --- s > smean, center element found

                   s = (smean-sprev) / (s-sprev)

                   skia  = skia - dx + s * dx
                   etaa  = etaa - dy + s * dy

!                  --- Set number of curve

                   call plafep ( 9, skia, etaa, 0.2d0, 0.2d0, ic )

                   go to 400

                end if

            end do

         end if

400   end do

!     --- If surfaces have to be considered, check which surface

      ipl6 = 0

      if ( iplots(1).ge.6 ) ipl6 = iplots(6)

      if ( nsurfs.gt.0 .and. ipl6.gt.0 ) then

!     --- Set color

         dcolor = 2

         istart = 2 * nsurfs + 1

!        --- Determine situation

         if ( indkmn.eq.0 ) then

!        --- KMESH part N, run through all surfaces and plot the correct
!            one

            do is = 1 , nsurfs

                npelmlc = kmeshn( is          )
                nelemlc = kmeshn( nsurfs + is )

!               --- Check whether this surface has to be plotted

                if ( is.eq.ipl ) &
                   call plfr22( kmeshn(istart), nelemlc, npelmlc, &
                                xmean, vact, ski, eta, ipl6, isitua )

!               --- Augment istart

                istart = istart + nelemlc * npelmlc

            end do

         else

!        --- KMESH part C and part T

            if ( ipl.gt.0 .and. ipl.le.nsurfs ) then

!           --- Determine starting element number

               istelm = kmesht(1,ipl)
               ilaelm = kmesht(2,ipl)

               nelemh = ilaelm - istelm + 1

!              --- Run through KMESH part C and determine the nodes of the
!                  elements of surfacegroup ipl

               istart = 0
               igrelm = 1

               i = 0

               klaar = .false.

700            if ( .not. klaar ) then

                  i = i + 1

                  if ( i.gt.nelgrp ) then

                     klaar = .true.

                  else

                     if ( kelma.ne.0 ) &
                        npelmlc = kmesha(i+1) - kmesha(i)

                     if ( nelgrp.gt.1 ) nelemlc = kmeshg(i)

                     if ( igrelm.le.istelm .and. &
                          istelm.lt.igrelm + nelemlc ) then

!                    --- Group of elements belongs to this standard
!                        element

                        istart = istart + ( istelm - 1 ) * npelmlc + 1

                        call plfr22( kmeshc(istart), nelemh, &
                                     npelmlc, xmean, vact, ski, eta, &
                                     ipl6, isitua )

                        klaar = .true.

                     else

                        igrelm = igrelm + nelemlc

                        istart = istart + npelmlc * nelemlc

                     end if

                  end if

!                 --- Go for next step

                  goto 700

               end if

            end if

         end if

      end if

!     --- Close plot file

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )

      jkader = ikader

      call erclos ( 'plfr21' )

      end
