      subroutine plobstaclegr3d ( rmeshx, nx, ny, nz, iobgrid, iobst, &
                                  kmeshc, coor, inpelm, nelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 09-03-2013 Correction length text
!        version  1.2  date 25-04-2010 New call to plaf05
!        version  1.1  date 21-11-2001 Only parts of elements inside are plotted
!        version  1.0  date 22-06-2001
!
!   copyright (c) 2001-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Plot all cells of the grid overlapping an obstacle using a colour
!    to distinguish between inner and outside cells and boundary cells
!    3d version, slices of constant z values are plotted
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     obstacle
!     grid
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz, iobgrid(0:nx-1,0:ny-1,0:nz-1,*), iobst, &
              inpelm, nelem, kmeshc(inpelm,nelem)
      double precision rmeshx(9,*), coor(3,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         ostacle
!     inpelm         i    number of points per element
!     iobgrid        i    In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     iobst          i    Obstacle sequence number
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements of the obstacle
!     nelem          i    Number of elements
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     rmeshx         i    Real part of kmesh part x (i = obstacle number)
!                         rmeshx(1,i) = xmin
!                         rmeshx(2,i) = xmax
!                         rmeshx(3,i) = ymin
!                         rmeshx(4,i) = ymax
!                         rmeshx(5,i) = dx
!                         rmeshx(6,i) = dy
!                         In R^3:
!                         rmeshx(5,i) = zmin
!                         rmeshx(6,i) = zmax
!                         rmeshx(7,i) = dx
!                         rmeshx(8,i) = dy
!                         rmeshx(9,i) = dz
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision polyg(2,4), dx, dy, dz, xv(3), yv(3), zv(3), &
                       x, y, zlow, ztop, xi(3), yi(3), z0, z1
      integer iscolr, i, j, k, itest, ielem, inodp, nintersect
      character *80 text

!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     i              Counting variable
!     ielem          Element number
!     inodp          Nodal point number
!     iscolr         Help variable to store the value of dcolor temporarily
!     itest          Help parameter to indicate if plot coordinates must be
!                    tested
!     j              Counting variable
!     k              Counting variable
!     nintersect     Number of intersections found
!     polyg          Array containing the co-ordinates of the point in the
!                    polygon in the following way:
!                    polyg(1,i) contains the x-coordinate of the first
!                    point
!                    polyg(2,i) contains the y-coordinate of the first
!                    point
!     text           String variable containing some text
!     x              x co-ordinate of point
!     xi             x-coordindates of interpolation
!     xv             x-coordindates of vertices
!     y              y co-ordinate of point
!     yi             y-coordindates of interpolation
!     yv             y-coordindates of vertices
!     z0             Minimum z-value in element
!     z1             Maximum z-value in element
!     zlow           Lowest value of z in interval
!     ztop           Highest value of z in interval
!     zv             z-coordindates of vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PL0000         Compute length and width of the plot
!     PLAF05         Plot polygon
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     PLOTINTSZC     Compute intersection of triangle with vertices stored in
!                    x, y and z with plane z = zc and store the intersections in
!                    xi and yi
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2254   inpelm # 3
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute coordinates of hexahedron
!     Compute dx, dy, dz
!     For all slices z do
!        Plot all elements with iobgrid = 0 (yellow)
!        Plot all elements with iobgrid = -1 (black)
!        Plot all elements with iobgrid>0 (green)
!        Compute intersection of obstacle with minimal z and plot
!        Compute intersection of obstacle with maximal z and plot
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plobstaclegr3d' )
      if ( inpelm/=3 ) then

!     --- Only triangles have been implemented

         call errsub ( 2254, 0, 0, 0 )
         go to 1000

      end if

!     --- Compute coordinates of hexahedron
!         Compute dx, dy, dz

      xmin = rmeshx(1,iobst)
      xmax = rmeshx(2,iobst)
      ymin = rmeshx(3,iobst)
      ymax = rmeshx(4,iobst)
      zmin = rmeshx(5,iobst)
      dx   = rmeshx(7,iobst)
      dy   = rmeshx(8,iobst)
      dz   = rmeshx(9,iobst)

!     --- Open plot

      iscolr = dcolor
      jkader = -1
      call pl0000( 15d0, 1d0 )
      itest=jmax

!     --- For all slices z do

      do k = 0, nz-1

!     --- Open plot

         call plafep( 1, aleng, width, 1d0, 15d0, itest )
         zlow = zmin+k*dz
         ztop = zlow+dz

!        --- Plot all elements with iobgrid = 0 (yellow)

         dcolor = 20
         do i = 0, nx-1
            do j = 0, ny-1
               if ( iobgrid(i,j,k,1)==0 ) then

                  x = xmin+i*dx
                  y = ymin+j*dy
                  polyg(1,1) = x
                  polyg(2,1) = y
                  polyg(1,2) = x+dx
                  polyg(2,2) = y
                  polyg(1,3) = x+dx
                  polyg(2,3) = y+dy
                  polyg(1,4) = x
                  polyg(2,4) = y+dy
                  call plaf05 ( 1, dcolor, 4, polyg, 2 )

               end if
            end do
         end do

!        --- Plot all elements with iobgrid = -1 (red)

         dcolor = 11
         do i = 0, nx-1
            do j = 0, ny-1
               if ( iobgrid(i,j,k,1)==-1 ) then

                  x = xmin+i*dx
                  y = ymin+j*dy
                  polyg(1,1) = x
                  polyg(2,1) = y
                  polyg(1,2) = x+dx
                  polyg(2,2) = y
                  polyg(1,3) = x+dx
                  polyg(2,3) = y+dy
                  polyg(1,4) = x
                  polyg(2,4) = y+dy
                  call plaf05 ( 1, dcolor, 4, polyg, 2 )

               end if
            end do
         end do

!        --- Plot all elements with iobgrid>0 (green)

         dcolor = 65
         do i = 0, nx-1
            do j = 0, ny-1
               if ( iobgrid(i,j,k,1)>0 ) then

                  x = xmin+i*dx
                  y = ymin+j*dy
                  polyg(1,1) = x
                  polyg(2,1) = y
                  polyg(1,2) = x+dx
                  polyg(2,2) = y
                  polyg(1,3) = x+dx
                  polyg(2,3) = y+dy
                  polyg(1,4) = x
                  polyg(2,4) = y+dy
                  call plaf05 ( 2, dcolor, 4, polyg, 2 )

               end if
            end do
         end do

!        --- Compute intersection of obstacle with minimal z and plot

         dcolor = 1

!        --- Loop over elements

         do ielem = 1, nelem
            do j = 1, inpelm
               inodp = kmeshc(j,ielem)
               xv(j) = coor(1,inodp)
               yv(j) = coor(2,inodp)
               zv(j) = coor(3,inodp)
            end do
            z0 = zv(1)
            z1 = z0
            do j = 2, inpelm
               z0 = min(z0,zv(j))
               z1 = max(z1,zv(j))
            end do
            if ( z0>ztop .or. z1<zlow ) then

!           --- Element must not be plotted, not in this z-interval

            else if ( z0>=zlow .and. z1<=ztop ) then

!           --- Element must be plotted once, because completely in this
!               interval

               do j = 1, inpelm
                  polyg(1,j) = xv(j)
                  polyg(2,j) = yv(j)
               end do
               polyg(1,inpelm+1) = xv(1)
               polyg(2,inpelm+1) = yv(1)
               call plaf05 ( 2, dcolor, inpelm+1, polyg, 2 )

            else if ( z0>=zlow ) then

!           --- Element must be plotted once, because lower side not intersected
!               and the intersection with the upper side must be plotted once
!               Compute intersection

               call plotintszc ( ztop, xv, yv, zv, xi, yi, nintersect )
               do j = 1, nintersect
                  polyg(1,j) = xi(j)
                  polyg(2,j) = yi(j)
               end do

              if ( nintersect==2 ) then

!             --- Two points found, a line must be drawn

                  call plaf05 ( 3, dcolor, nintersect, polyg, 2 )

               end if

            else if ( z1<=ztop ) then

!           --- Element must be plotted once, because upper side not intersected
!               and the intersection with the lower side must be plotted once
!               Compute intersection

               call plotintszc ( zlow, xv, yv, zv, xi, yi, nintersect )
               do j = 1, nintersect
                  polyg(1,j) = xi(j)
                  polyg(2,j) = yi(j)
               end do

               if ( nintersect==2 ) then

!              --- Two points found, a line must be drawn

                  call plaf05 ( 3, dcolor, nintersect, polyg, 2 )

               end if

            else

!           --- Both intersections must be plotted and computed

               call plotintszc ( zlow, xv, yv, zv, xi, yi, nintersect )
               do j = 1, nintersect
                  polyg(1,j) = xi(j)
                  polyg(2,j) = yi(j)
               end do

               if ( nintersect==2 ) then

!              --- Two points found, a line must be drawn

                  call plaf05 ( 3, dcolor, nintersect, polyg, 2 )

               end if

               call plotintszc ( ztop, xv, yv, zv, xi, yi, nintersect )
               do j = 1, nintersect
                  polyg(1,j) = xi(j)
                  polyg(2,j) = yi(j)
               end do

               if ( nintersect==2 ) then

!              --- Two points found, a line must be drawn

                  call plaf05 ( 3, dcolor, nintersect, polyg, 2 )

               end if

            end if

         end do

!        --- Compute intersection of obstacle with maximal z and plot

         dcolor = iscolr
         write(text(1:37),100) zlow, ztop
100      format('z between ',d11.3, ' and ', d11.3 )
         call plafp7 ( 15d0, 19.5d0, 'grid covering the obstacle', 0d0, &
                       0.25d0, 26 )
         call plafp7 ( 15d0, 19d0, text, 0d0, 0.25d0, 37 )
         dcolor = 11
         call plafp7 ( 15d0, 18.5d0, '-- internal', 0d0, 0.25d0, 11 )
         dcolor = 20
         call plafp7 ( 15d0, 18d0, '-- external', 0d0, 0.25d0, 11 )
         dcolor = 65
         call plafp7 ( 15d0, 17.5d0, '-- boundary', 0d0, 0.25d0, 11 )
         dcolor = 1
         call plafp7 ( 15d0, 17d0, '-- intersection with element', 0d0, &
                       0.25d0, 28 )
         dcolor = iscolr

!        --- close plot dataset

         call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )

      end do

1000  call erclos ( 'plobstaclegr3d' )

      end
