      subroutine plotf3 ( iplots, rplots, ielskp, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 29-05-2014 Extension of timebefore
!        version  2.3  date 25-11-2012 Long integers
!        version  2.2  date 11-03-2011 Plot bottom text
!
!   copyright (c) 1995-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine to make a 3-d plot with hidden lines of the solution
!   vector isol ; the number'th component is considered .
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     hidden_line
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iplots(*), ielskp(*)
      double precision rplots(*)
      character (len=*) text

!     ielskp  i   Integer array that is only used if iplots(6) (iskip) is 1
!                 In that case it must have length nelgrp and
!                 ielskp(i) = 0 means element group i must be skipped
!                 ielskp(i) = 1 means element group i must not be skipped
!     iplots  i   Integer control array in which the user defines which choices
!                 he wants for this subroutine
!                 IPLOTS must be filled  by the user in the following way:
!                 Position:
!                 1:  Defines the highest entry number of the array that has
!                     been filled by the user. This means that the user must
!                     fill all positions 1 to IPLOTS(1) in array IPLOTS.
!                     For all other positions default values are used.
!                     Hence if IPLOTS(1) = 0 or 1 only defaults are used.
!                     Remark: if IPLOTS(i)=0, also for position i the
!                             default is taken.
!                 2:  (linopt) defines the types of lines to be drawn
!                     Possible values:
!                     1: All lines are drawn parallel to the x-axis
!                     2: 2 sets of perpendicular lines are drawn
!                        The first set has the direction defined by rplots(3)
!                     Default value: 1
!                 3:  (nlines) defines the number of lines per cell minus 1
!                     A cell is defined as a square with sides dx, where dx
!                     is defined by the formula:
!                     npoint = (a/dx+1) (b/dx+1), where a and b are the length
!                     and the width of the smallest rectangle enclosing
!                     the region
!                     Default value: 1
!                 4:  (iblock) defines the mode in which the lines are plotted
!                     Possible values:
!                     1: sheet mode
!                     2: block mode
!                     Default value: 1
!                 5:  (number) the number'th physical unknown in each node is
!                     considered When this unknown does not appear in a nodal
!                     point, the point is skipped
!                     When array ISOL corresponds to a complex array, &
!                     NUMBER=2*i-1 corresponds to the real part of unknown i
!                     and NUMBER=2*i to the imaginary part of unknown i
!                     Default value: 1
!                 6:  (iskip) indicates if element groups must be skipped (1)
!                     or not (0)
!                     If iskip=1 array ielskp must be filled
!                     Default value: 0
!                 7:  (itransparant) indicates if the hidden line plot must be
!                     made (0) or a transparant plot (1)
!                     Default value: 0
!                 8:  ntext: length of bottom text
!                     If zero no text is plotted
!                     Default value: 0
!     rplots  i   Double precision input array that must be filled as follows:
!                 Values 0 are replaced by the default value
!                 1:    orientation of projection in degrees
!                       Default: 0
!                 2:    angle between projected y-axis and x-axis in degrees
!                       angle must be between 10 and 170
!                       Default: 60
!                 3:    orientation of primary intersection lines in degrees
!                       Default: 0
!                 4:    width of the plot in cm
!                       Default: 15 cm
!                 5:    height of the plot in cm
!                       Default: 15 cm
!                 6:    xmin
!                       Default: automatically computed
!                 7:    xmax
!                       Default: automatically computed
!                 8:    ymin
!                       Default: automatically computed
!                 9:    ymax
!                       Default: automatically computed
!                10:    ground value to be used in block mode
!                       all function values  at the boundary are
!                       connected to this value. ground value should
!                       be<=the minimal function value.
!                       this amount. If ground value>=0 the subroutine
!                       calculates a value:
!                       if all function values are>=0: GV = 0.
!                       if some function values are <0: GV = fmin
!                11:    Scalefactor for of y-axis.
!                       Default 0.5.
!                12:    Scalefactor for of z-axis.
!                       If one of the scales are<=0 the default
!                       value is set.
!                       Default 1.
!                13:    Multiplication factor for y-coordinates
!                       Default 1.
!     text    i  Text array containing the texts to be plotted
!                1:  Text at bottom of the picture
!                2:  Text at top of the picture
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl
      integer nomber(2), iskip, ichoic, iresul(5,3), iincop(7), &
              nwork, niwork, special, ilevel, ntext, nelgrpsav, nelemsav
      double precision rincop(14)
      integer, pointer :: kmeshcloc(:)
      double precision :: timebefore(2)

!     compl          Indicates if the vector is complex (true) or not
!     ichoic         Choice parameter for TOPQU0. With this parameter it is
!                    defined which actions will be performed. Possible values:
!                    1:  Create a submesh consisting of 3-point triangles only
!                        Create also a copy of the solution array containing one
!                        unknown only in all points of the new grid
!                        In this case array KPROB and ISOL is also used
!                        If the solution vector is complex only the real or
!                        imaginary part is used
!     iincop         Copy of array iplots extended with defaults
!     ilevel         dummy
!     iresul         Output array of subroutine TOPQU0
!                    The array is filled as follows:
!                    Pos .,1 corresponds to the "vertex" mesh array (ichois=2,3)
!                    Pos .,2 corresponds to the "triangle" mesh array
!                            (ichois=1,3)
!                    Pos .,3 corresponds to the solution array (ichois=1,3)
!                    Pos. 1,1  INDVER sequence number with respect to INFOR
!                                     for the "vertex" mesh array
!                    Pos. 2,1  NPELM  number of points per element for this mesh
!                    Pos. 3,1  NELEM  number of elements for this mesh
!                    Pos. 4,1  NEWVER indicates if the mesh is new (1) or is
!                                     in fact the standard mesh (0)
!                    Pos. 5,1  INDLIN sequence number with respect to INFOR
!                                     for the "line-elements" array
!                    Pos. 1,2  INDTRI sequence number with respect to INFOR
!                                     for the "triangle" mesh array
!                    Pos. 2,2  NPELM  number of points per element for this mesh
!                    Pos. 3,2  NELEM  number of elements for this mesh
!                    Pos. 4,2  NEWTRI indicates if the mesh is new (1) or is
!                                     in fact the standard mesh (0) or the mesh
!                                     created by the "vertex" mesh
!                    Pos. 5,2  NPHYS  Max. number of degrees of freedom
!                    Pos. 1,3  INDSOL sequence number with respect to INFOR
!                                     for the new solution array
!                    Pos. 2,3  NEWSOL indicates if the solution array is new
!                                     (1) or is in fact the standard
!                                     solution array (0)
!                    Pos. 3,3  INDPRF sequence number of KPROBF
!                    Pos. 4,3  INDPRH sequence number of KPROBH
!                    Pos. 5,3  INPPRP sequence number of KPROBP
!     iskip          Indicates if element groups must be skipped (>0) or not (0)
!     kmeshcloc      Local array kmeshc containing the node numbers of one
!                    element
!     nelemsav       Help parameter to save the old value of nelem
!     nelgrpsav      Help parameter to save the old value of nelgrp
!     niwork         Length of array iwork
!     nomber         Array where is stored which unknown has to be considered
!     ntext          Number of characters in a text
!     nwork          Maximal number of coefficients
!     rincop         Copy of array rplots extended with defaults
!     special        Indicates which special situation is present
!                    Possible values:
!                    0     standard
!                    1     solution vector is of type 116 and a coloured
!                          2d plot is required
!                    2     solution vector is of type 110 or 115 and a coloured
!                          2d plot is required, whereas there are no unknowns
!                          in the vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     PL3D01         Copy iplots and rplots into iincop and rincop extended with
!                    default values
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     PLTJ3D         Make a 3d plot of a function in R^2
!     PLTJAU         Provides an automatic estimate of the work arrays for
!                    PLOTF3
!     PLTJTR         Transparent 3D plot at 2D region
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     TOPQU0         Subdivide mesh if necessary. Map unknowns into an organized
!                    array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      62   dimension of space (ndim) has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotf3' )
      if ( itime>0 ) call ersettime ( timebefore )

      if ( ndim/=2 ) then
         call errint ( ndim, 1 )
         call errsub ( 62, 1, 0, 0 )
      end if

      nelgrpsav = nelgrp
      nelemsav = nelem

!     --- Copy arrays iplots and rplots to iincop and rincop
!         and fill default values

      call pl3d01 ( iplots, rplots, iincop, rincop )
      nomber(1) = 1
      nomber(2) = iincop(5)
      compl = .false.
      ichoic = 1
      iskip = iincop(6)
      special = 0
      ilevel = 1
      call topqu0 ( ichoic, nomber, iskip, ielskp, iresul, compl, &
                    0, special, ilevel )
      if ( ierror/=0 ) go to 1000

!     --- Make actual plot

      inelem  = iresul(3,2)
      kmeshcloc => kmeshc

!     --- nwork  = 30*inelem+20*npoint
!         niwork = 30*inelem+20*npoint
!         Estimate dimensions of work arrays for pltj3d

      if ( iresul(1,2)>0 ) kmeshcloc => kmeshctmp
      call pltjau ( coor, usolt, npoint, &
                    kmeshcloc, inelem, rincop, iincop(2), &
                    nwork, niwork )

      if ( iincop(7)==0 ) then

!     --- Standard hidden line plot

         call pltj3d ( coor, usolt, npoint, &
                       kmeshcloc, inelem, rincop, iincop(2), &
                       nwork, niwork )

      else

!     --- Transparant plot

         call pltjtr ( coor, usolt, npoint, &
                       kmeshcloc, inelem, rincop)

      end if

      if ( iplots(1)>7 ) then

!     --- iplots(1)>7, hence Pos 8 has been filled

         ntext = iplots(8)
         call plafp7 ( 1d0, 0.25d0, text, 0d0, .25d0, &
                       ntext )

      end if  ! ( iplots(1)>7 )

      call plafep (2, 0d0, 0d0, 0d0, 0d0, 0)  ! close plot

      kmeshc => km(iseqkm)%kmeshc
      nelgrp = nelgrpsav
      nelem = nelemsav
      inelem = nelem
      if ( iresul(1,2)>0 .and. iresul(1,2)/=kelmc ) deallocate(kmeshctmp)
      if ( iresul(2,3)==1 ) call seprmsolinfo ( iresul(1,3) )

1000  call erclmn ( 'plotf3', timebefore )
      end subroutine plotf3
