      subroutine plotvcb1 ( iplots, inpvc1, inpvc2, rplots, text )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  2.4  date 29-05-2014 Extension of timebefore
!        version  2.3  date 11-03-2013 Initialize pointer
!        version  2.2  date 25-11-2012 Long integers
!
!   copyright (c) 2004-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    a vector is computed from degrees of freedom idgfd1,idgfd2 of the
!    standard sepran vectors inpvc1,inpvc2
!    inpvci must be of any type 110,115,116  i=1,2
!    automatic scaling is performed if abs(factor)<epsmac
!    for real vectors
!         idgfd1,idgfd2>0 the degrees of freedom are from inpvc1
!         idgfd1,idgfd2<0  idgfd1 is from inpvc1, idgfd2 from inpvc2
!    for a complex vector inpvc1
!         idgfd1 : the real and complex parts of degree of freedom idgfd1
!                form a vector
!    ndim of the mesh must be 2
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     vector
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision rplots(*)
      integer inpvc1, inpvc2, iplots(*)
      character (len=*) text(*)

!     inpvc1         i    Arrays containing the information of the vectors to be
!     inpvc2         i    computed. Only when IDGFD1 and IDGFD2 are negative,
!                         and IVCTR1 corresponds to a real array, both arrays
!                         are used, otherwise only IVCTR1 is utilised.
!                         See IDGFD1 and IDGFD2.
!                         IVCTR1 and IVCTR2 may be arrays of different types,
!                         see the PG
!     iplots         i    Integer control array in which the user defines which
!                         choice he wants for this subroutine
!                         IPLOTS must be filled  by the user in the following
!                         way:
!                         Position:
!                         1:  Defines the highest entry number of the array that
!                             has been filled by the user. This means that the
!                             user must fill all positions
!                             1 to IPLOTS(1) in array IPLOTS.
!                             For all other positions default values are used.
!                             Hence if IPLOTS(1)=0 or 1 only defaults are used.
!                             Remark: if IPLOTS(i)=0, also for position i the
!                                     default is taken.
!                         2:  idgfd1
!                             Default value: 1
!                         3:  idgfd2
!                             When both IDGFD1 and IDGFD2 are positive, the
!                             vector to be plotted is composed of the
!                             IDGFD1-th and IDGFD2-th
!                             degree of freedom of IVCTR1.
!                             When both IDGFD1 and IDGFD2 are negative, the
!                             vector to be plotted is composed of the
!                             -IDGFD1-th degree
!                             of freedom of the input vector IVCTR1 and the
!                             -IDGFD2-th degree of freedom of the input vector
!                             IVCTR2. If the input vector IVCTR1 corresponds to
!                             a complex array, the vector to be plotted is
!                             composed of the real and imaginary part of
!                             the IDGFD1-th degree of freedom of IVCTR1.
!                             If in a nodal point not both degrees of freedom
!                             are present, the nodal point is skipped.
!                             Default value: 2
!                         4:  inner indication whether inner boundaries must be
!                             plotted (1) or not (0)
!                             if zero level set must be plotted raised by 10
!                             (2D only)
!                             Default value: 0
!                         5:  Indicates if the computed factor must be stored in
!                             rplots(3) (1) or not (0)
!                             Default value: 0
!                         6:  (ITEXT) indicates if text must be plotted in the
!                             picture.
!                             Possible values:
!                             0:  No text at all
!                             1:  Extra text at bottom
!                             2:  Extra text at bottom and top
!     rplots         i    Double precision input array that must be filled as
!                         follows:
!                         1:    plotfm   Length of plot in centimeters
!                         2:    yfact    Scale factor; all coordinates are
!                                        multiplied by this factor before
!                                        plotting
!                         3:    factor   Scale factor to be applied to the
!                                        vector.
!                                        If FACTOR=0, the scale is computed,
!                                        otherwise the length of each vector is
!                                        multiplied by FACTOR before plotting
!     text           i    Text for output
!                         1:  Bottom text
!                         2:  Text at top
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision almax, avmax, amin, dx, dy, sch, plotfm, yfact, &
                       factor
      integer idgfd1, idgfd2, icoun1, icoun2, ktimes, ikader, ichw, &
              ikelmc, inner, itypv1, itypv2, jskip, itext, &
              num1, num2, istorefact, plotzerolev
      logical compli, bouncurv
      double precision, allocatable :: coorc(:)
      integer, allocatable :: iwork(:), index1(:), index2(:), index3(:)
      integer, allocatable :: index4(:), index5(:)
      double precision :: timebefore(2)

!     almax
!     amin           Minimum value in solution array
!     avmax
!     bouncurv       if true the 2D boundaries are plotted using icurvs/curves
!                    otherwise using kmeshm and coor
!     compli         denotes whether vector is complex (true) or not (false)
!     coorc          array with coordinates of centroid of elemen
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     factor         Scale factor to be applied to the vector. If FACTOR=0, the
!                    scale is computed, otherwise the length of each vector is
!                    multiplied by by FACTOR before plotting
!     ichw
!     icoun1
!     icoun2
!     idgfd1         See iplots
!     idgfd2         See iplots
!     ikader         Help variable to store jkader temporarily
!     ikelmc         position in kmesh part c
!     index1         Array of length inpelm containing the point numbers
!                    of the nodal points in the element
!     index2         Array of length icount containing the positions of the
!                    degrees of freedom in the element with respect to USOL
!     index3         array with number of degrees of freedom,
!                    cumulatively, in nodal points of actual element,
!                    or of degrees of freedom per element
!     index4         array with positions in vectr2 of degrees
!                    of freedom in nodal points of actual element,
!                    or of degrees of freedom per element
!     index5         arrays with number of degrees of freedom,
!                    cumulatively, in nodal points of actual element,
!                    or of degrees of freedom per element
!     inner          Information of how the curves must be plotted
!                    0,1      only the outer boundaries are plotted
!                      2      only the inner boundaries are plotted
!                      3      all boundaries are plotted
!     istorefact     If 1 the factor computed must be stored in rplots(3)
!                    if 0 not
!     itext          Text sequence number
!     itypv1
!     itypv2
!     iwork          integer work array
!                    if iwork(iknpt)=0 nodal point iknpt should be considered
!                                   =1 nodal point has already been 'seen'
!                                   =-1 nodal point is outside range
!                                       or does not contain degree of freedom
!     jskip          Skip element groups (>0) or not (0) or volumes (<0)
!     ktimes         mod(itimes,10)
!     num1
!     num2
!     plotfm         Length of plot in centimeters
!     plotzerolev    if 1 then plot boundary of zero level set
!     sch            scale factor to be applied to the vector
!     yfact          Scale factor; all coordinates are multiplied by this factor
!                    before plotting
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     PL0003         Open plot, plots frame work and computes plotting
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLLGVC         plot block of text containing the scale factors
!     PLOTBN         Plot boundary of region
!     PLTOPBOTTEXT   Plot text at top or bottom of plot if required
!     PRPLOTZEROLEV  Plot zero level set curve
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     TO0017         determine number of degrees of freedom per element group
!     TO0018         fill array index1 with pointers to nodes of element
!     TO0019         fill array index2,index2 with pointers to degrees of
!                    freedom
!                    and cumulative numbers of degrees of freedom in nodal
!                    points
!     TO0027         revise scale factors for real vectors
!     TO0028         revise scale factors for complex vectors
!     TO0029         plot real vector
!     TO0030         plot complex vector
! **********************************************************************
!
!                       I/O
!
!     Files used:
!     - standard SEPRAN files
!     - file for plotter (screen, plotter, of disk file)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      62   dimension of space (ndim) has wrong value
!     200   First input vector is of illegal type
!     202   One of the input vectors is real the other one complex
!     212   parameters idgfd1 and idgfd2 have different sign
!     213   parameter idgfd1 and idgfd2 should both be positive
!     214   parameter idgfd1 has a non-positive value
!     215   Expected two-dimensional problem on two-dimensional mesh does
!           not exist or starting point is not lying in an accessible element
!           group.
!     759   Second input vector is of illegal type
!    1444   The solution vector is nearly equal to zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!     See PG 9.6.2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initializations and tests

      call eropen ( 'plotvcb1' )
      if ( itime>0 ) call ersettime ( timebefore )
      if ( ierror/=0 ) go to 1000
      compli=.false.
      if (ks(inpvc1)%compl==1) compli=.true.
      npointcac=npoint
      nelemcac=nelem
      idgfd1 = 1
      idgfd2 = 2
      inner = 1
      istorefact = 0
      itext = 0
      plotzerolev = 0
      if ( iplots(1)>1 ) idgfd1 = iplots(2)
      if ( iplots(1)>2 ) idgfd2 = iplots(3)
      if ( iplots(1)>3 ) then
         inner  = mod(iplots(4),10)
         plotzerolev = iplots(4)/10
      end if  ! ( iplots(1)>3 ) then
      if ( iplots(1)>4 ) istorefact = iplots(5)
      if ( iplots(1)>5 ) itext = iplots(6)
      plotfm = rplots(1)
      yfact  = rplots(2)
      factor = rplots(3)

!     --- determine pointer to input vector and relevant parts of krpob
!         for pinpointing degrees of freedom (kfree=ibfree : temporary !
!         determine number of degees of freedom in nodal points nunkpo
!         and total number of degrees of freedom in mesh nusol1

      itypv1=ks(inpvc1)%typevec
      if ( itypv1/=110 .and. itypv1/=115 .and. &
           itypv1/=116 ) then
         call errint ( itypv1, 1 )
         call errsub ( 200, 1, 0, 0)
         ierror=0
         goto 1000
      end if

!     --- determine problem space

!     --- ndim = 1 or 3 are not allowed

      if (ndim/=2) then
         call errint ( ndim, 1 )
         call errsub ( 62, 1, 0, 0)
         ierror=0
         goto 1000
      end if
      iprob  = ks(inpvc1)%iprob

!     --- Compute starting address of kprob

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      kprobc => kprobpart%kprobc

!     --- determine pointers to nodal point numbers, coordinates, and curves

      call sepactsolbf2 ( inpvc1 )
      kprobfo => kprobfi
      indprfo = indprfi
      nunkpo = nunkpi
      bouncurv = .false.
      if ( kelmh>0 ) then

!     --- kmesh part h has been filled
!         Check if icurvs and curves have been filled

         if ( kmeshh(3,1)>0 .and. kmeshh(5,1)>0 ) bouncurv = .true.

      end if

!     --- test idgfd1,idgfd2

      num1=idgfd1
      num2=idgfd2
      if (.not.compli) then

!     --- real input vector(s)

         if (idgfd1*idgfd2==0) then
            goto 999
         else if (idgfd1*idgfd2<0) then

!        --- idgfd1,idgfd2 have different sign

            call errint ( idgfd1, 1 )
            call errint ( idgfd2, 2 )
            call errsub ( 212, 2, 0, 0)
            ierror=0
            goto 1000
         end if

!        --- degrees of freedom from different input vectors ?

         if (idgfd1<=0) then

!        --- two vectors : determine pointers

            num1=-idgfd1
            num2=-idgfd2
            itypv2=ks(inpvc2)%typevec
            if ( itypv2/=110 .and. itypv2/=115 .and. &
                 itypv2/=116 ) then
               call errchr ( ' ', 1 )
               call errint ( 1, 1 )
               call errint ( itypv2, 2 )
               call errsub ( 759, 2, 0, 1 )
               ierror=0
               goto 1000
            end if
            if (ks(inpvc1)%compl/=ks(inpvc2)%compl) then
               call errchr ( ' ', 1 )
               call errchr ( ' ', 3 )
               call errint ( 1, 1 )
               call errint ( 1, 2 )
               if ( ks(inpvc1)%compl==0 ) then
                  call errchr ( 'real', 2 )
                  call errchr ( 'complex', 4 )
               else
                  call errchr ( 'complex', 2 )
                  call errchr ( 'real', 4 )
               end if
               call errsub ( 202, 2, 0, 4 )
               ierror=0
               goto 1000
            end if
            call sepactsolbf2 ( inpvc2 )

         end if

      else if (idgfd1<=0) then

!     --- complex input vector

         call errint ( idgfd1, 1 )
         call errsub ( 214, 1, 0, 0)
         ierror=0
         goto 1000

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( coorc(nelemcac*ndim), iwork(npointcac), &
                 index1(npelm), index2(npelm*nunkpo), &
                 index3(npelm+1), index4(npelm*nunkpi), &
                 index5(npelm+1), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nelemcac*ndim+npointcac+npelm*(nunkpo+nunkpi), &
                        'coorc' )
      if ( ierror/=0 ) go to 1000
      iwork = 0
      coorc = 0d0

      vectin => ks(inpvc1)%sol
      if ( ierror/=0 ) go to 1000

      if (idgfd1>0) then

!     --- one vector : set pointers to vector2 equal to that of vector1

         vectin2 => vectin
         kprobfo => kprobfi

      else

         usoltemp => ks(inpvc2)%sol

      end if
      if (ierror/=0) then
         ierror=0
         goto 1000
      end if

!     --- initialize plotdataset, plot frame if required

      ktimes = mod(jtimes,10)
      textx = 'x'
      texty = 'y'
      if ( ktimes==0 .or. ktimes==1 ) then
         if ( bouncurv ) then

!        --- Use arrays curves and icurvs

            call pl0003 ( ikader, 3, plotfm, yfact, 11 )

         else

!        --- Use arrays coor and kmeshm

            call pl0003 ( ikader, 3, plotfm, yfact, 1 )

         end if

      end if
      if ( ierror/=0 ) go to 1000

!     --- meaning of some local parameters
!         nunkpo,nunkpi number of degrees of freedom in a nodal point
!                   is constant only if indprfo/indprfi =0
!         nusol1,nusol2 total number of degrees of freedom of input and
!                   output vector
!         ipvci1,ipvci2   pointer to vectors
!         clear allocated space for large vector iwork and coorc
!         determine if scale factor has to be determined or not
!         special care for type of vector = 116 (determine centroid)
!         ichw  =0    selection by clipping (of outgoing arrows) in plafep
!               /=0 selection by excluding points outside
!                     (xmin,xmax)*(ymin,ymax); gives slightly better scaling

      sch=factor
      ichw=1
      if ( abs(sch)>epsmac .and. itypv1/=116 ) goto 600
      jskip=0
      amin=rinfin
      almax=epsmac
      avmax=epsmac

!     --- initialize number of elements and number of nodal points per element

      ielem=0
      ikelmc=1

!     --- outer loop over all groups of elements

      do 500 ielgrp=1,nelgrp

!     --- nelgrp>1 determine number of elements of this group
!                    and number of nodal points of an element

         inelem=Topgrp(ielgrp)%inelem
         inpelm=Topgrp(ielgrp)%inpelm
         ifirst=0

!        --- determine itype and test if group must be skipped

         itype=kprobc(ielgrp)
         if ( itype==0 .or. itype==-1 .and. &
             ( itypv1==115 .or. itypv2==115 )) then

!        --- update ikelmc and ielem as a whole group of elements will be
!            skipped

            ikelmc=ikelmc+inpelm*inelem
            jskip=jskip+1
            ielem=ielem+inelem
            goto 500
         else
            continue
         end if

!        --- determine pointers and allocate (temporary) space for help
!            variables  index1,index2,index3 (,index4,index5)
!            local parameters
!            index1 contains pointers to coordinates of nodal points
!            index2 contains pointers to degrees of freedom of input vector1
!                  in large input vector (type 110,115,116)
!            index3 contains cumulative number of degrees of freedom in
!                  nodal points ; length = inpelm+1, index3(1)=0
!            index4,index5 like index2,index3 for input vector2 (110,115,116)

         call to0017(indprfo,nunkpo,inpvc1, icoun1)
         if ( ierror/=0 ) go to 1000
         if (idgfd1<0) &
            call to0017(indprfi,nunkpi,inpvc2, icoun2)

!        --- goto 1000 if an error has been detected (no plot)

         if (ierror/=0) then
            ierror=0
            goto 1000
         end if

!        --- loop over all elements of this group

         do 450 irelem=1,inelem

!        --- fill arrays index1, index2, index3, index4, and index5

            ielem=ielem+1
            call to0018(kmeshc(ikelmc), index1)
            if ( ierror/=0 ) go to 1000
            ikelmc=ikelmc+inpelm
            call to0019 ( nunkpo,inpvc1, index1, kprobfo, indprfo, &
                         index2, index3 )
            if ( ierror/=0 ) go to 1000

!           --- select for different cases

            if (.not.compli) then
               if (idgfd1<0) then

!              --- determine index4,index5 for idgfd1<0

                  call to0019 ( nunkpi,inpvc2, index1, kprobfi, indprfi, &
                                index4, index5 )

               else

!              --- Copy index2 and 3 into 4 and 5

                  index4 = index2
                  index5 = index3

               end if

!              --- revise scale factors for real vector(s)

               call to0027(itypv1,num1,num2,vectin, &
                  vectin2,coor, coorc, iwork, &
                   index1, index2, index3, &
                   index4, index5,amin,almax,avmax)

            else

!           --- revise scale factors for complex input vector

             call to0028(itypv1,num1,vectin, &
                coor, coorc, iwork, &
                 index1, index2, index3, &
                amin,almax,avmax)
          end if
          if ( ierror/=0 ) go to 1000
450       continue
500   continue

!     --- determine average scale factor

      if ( avmax<=epsmac*1.01d0 ) then
         call errwar ( 1444, 0, 0, 0 )
         go to 1000
      end if
      sch=sqrt(amin+almax/avmax)

!     --- no two-dimensional problem on two-dimensional mesh ?

      if (jskip==nelgrp) call errsub ( 215, 0, 0, 0)
600   if (ierror/=0) then
         ierror=0
         goto 1000
      end if

!     --- reset factor for special case with type of vector = 116

      if ((ktimes>1 .or. abs(factor)>epsmac) .and. &
          (itypv1==116)) sch=factor
      if ( ktimes==0 .or. ktimes==1 ) then

         if ( bouncurv ) then

!        --- Use arrays curves and icurvs

            call pl0003 ( ikader, 3, sch, yfact, 12 )

         else

!        --- Use arrays coor and kmeshm

            call pl0003 ( ikader, 3, sch, yfact, 2 )

         end if
         if ( ierror/=0 ) go to 1000

!        --- the length and the width of the plot are corrected in order
!            to plot a part of the vectors outside the rectangle

          aleng=(xmax-xmin)/xfct
          width=(ymax-ymin)/yfct
          dx=(xmax-xmin)*((aleng+2.5d0)/aleng-1d0)*0.5d0
          dy=(ymax-ymin)*((width+2.5d0)/width-1d0)*0.5d0
          xmint=xmin-dx
          xmaxt=xmax+dx
          ymint=ymin-dy
          ymaxt=ymax+dy

      else

!     --- ktimes #0 or 1, hence this part of another plot
!         do not change boundaries

          xmint=xmin
          xmaxt=xmax
          ymint=ymin
          ymaxt=ymax

      end if

      if ( ierror/=0 ) go to 1000

!     --- plot vector

      if (.not.compli) then
      call to0029(itypv1,ichw,num1,num2,vectin,vectin2, &
             coor, coorc, iwork,npointcac, &
             nelemcac,nunkpo,nunkpi, &
             indprfo,indprfi, kprobfo, kprobfi, sch)

!     --- complex input vector

      else
         call to0030(itypv1,ichw,num1,vectin, &
             coor, coorc, iwork,npointcac, &
             nelemcac,nunkpo,indprfo, kprobfo, sch)

      end if
      if ( ierror/=0 ) go to 1000

      if ( ktimes<=1 ) then

!     --- ktimes = 0 or 1, i.e. first plot
!         Plot boundary of region

         call plotbn ( 1, inner, bouncurv )

         if ( ierror/=0 ) go to 1000

!        --- Plot block of text containing the scale factors in
!            x- and y-direction as well as the scale factor for the vectors
!            This possibility is only implemented for SEPPOST (jtimes>=10)

         if ( jtimes/10==1 ) call pllgvc ( sch )

!        --- Plot text at top and bottom if required

         call pltopbottext ( itext, text, rplots(1) )

         if ( plotzerolev==1 ) then

!        --- plotzerolev=1, plot zero level set, with color red

            dcolor = 11  ! set color red
            call prplotzerolev
            dcolor = 0   ! reset color

         end if  ! ( plotzerolev==1 ) then

      end if
      if ( istorefact==1 ) rplots(3) = sch

      deallocate ( coorc, iwork, index1, index2, index3, index4, index5, &
                   stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coorc' )
      if ( ierror/=0 ) go to 1000

!     --- close plot dataset

      if ( ktimes==0 .or. ktimes==3 ) call plafep ( 2, 0d0, 0d0, &
           0d0, 0d0, 0 )
      jkader=ikader
      goto 1000

!     --- idgfd1 or idgfd2 is zero

999   call errint ( idgfd1, 1 )
      call errint ( idgfd2, 2 )
      call errsub ( 213, 2, 0, 0)
      ierror=0
1000  call erclmn ( 'plotvcb1', timebefore )
      end subroutine plotvcb1
