      subroutine pltj3d ( coor, fval, npoint, kmeshc, nelem, &
                          rlinp, intinp, nwork, niwork )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.9  date 12-05-2014 call plafep
!        version  1.8  date 11-03-2011 Remove close plot
!        version  1.7  date 05-01-1999 array coor saved and restored
!
!   copyright (c) 1995-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       pltj3d puts all of the linpack, listpack and peelalgorithm together to
!       deliver a 3d plot with a fairly (O(nlog n) operations for n points)
!       effective algorithm.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepltj
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intinp(5), npoint, kmeshc(*), nelem, nwork, niwork
      double precision coor(2,*), fval(*), rlinp(14)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     fval    i    function values in nodes
!     intinp  i    integer array of .. elements
!                  (1) : 1 lines are drawn parallel to x-axis
!                        2 2 sets of lines are drawn:
!                          one set determined by rlinp (3) and the
!                          other orthogonal
!                        All other values default to 1.
!                  (2) : Number of lines per cel -1.
!                        A cell has width dx defined by
!                        npoint = (a/dx + 1) (b/dx + 1)
!                        in which a and b are the smallest rectangle sides
!                        parallel to the x and y axes that wholly contain V.
!                        This is perfect for a rectangle with equidistant
!                        steps.
!                  (3) : 1: sheet mode
!                        2: block mode
!                        IMPORTANT: Setting every entry of intinp 0 gets
!                        default values for all variables
!                  (4) : If pltjau has been called intinp(4) contains
!                        ndimw, otherwise it should contain 0.
!                  (5) : If pltjau has been called intinp(5) contains
!                        ncint, otherwise it should contain 0.
!     kmeshc  i    kmesh part c, contains the nodal points corresponding to the
!                  elements
!     nelem   i    Number of elements
!     niwork  i    declared dimension of iwork.
!                  iwork must accomodate kgrad (nelem), iwork of prjlinw
!                  (2*ndimwork), (ac)nseg (0:nlines + 1), (ac)kseg (0:nsegs +
!                  1), nextra (2, nsegs), kextra (2, 2, nsegs) and ncomp (2,
!                  nsegs). nsegs is the total number of segments and this
!                  depends of course on the number of lines and the number of
!                  breaks in each intersection. The precise length of niwork
!                  is known after the call to prlnsw. If niwork is too small
!                  an error is signalled, together with a printout of the
!                  value required. (Total estimate: nelem + 2*ndimwork +
!                  nlines + 9 * nsegs + 4. For an estimate of ndimwork see
!                  nwork)
!     npoint  i    Number of nodal points
!     nwork   i    declared dimension of work. work must accomodate xcint,
!                  fcint, elkey and diael (prlnsw output arrays) together
!                  with the work array needed there. This makes the total
!                  dimension 2 * (Nelem + Ncint + ndimwork).
!                  Both Ndimwork and Ncint must be estimated. For ndimwork we
!                  take the oversafe estimate SHAPE * 6 * SQRT (2* Nelem).
!                  (Shape being the ratio of the largest and smallest
!                  diameter.)
!                  The rest of the work array can be handed out to the
!                  coordinate arrays, the size of which can be estimated by:
!                  Ncint = 2 * (ic2 + 1) * npoint. The precise value of ncint
!                  is known after the call to prlnsw. If nwork is too small an
!                  error is signalled together with a printout of the value
!                  required.
!     rlinp   i    dp array of 13 elements.
!                  (1) orientation of projection in degrees. Default 0
!                  (2) angle of projected y-axes wrt x-axes (degrees)
!                      Default 60 deg (10<phi<170). Anything outside
!                      this interval (notably phi = 0) defaults to 60.
!                  (3) orientation of primary intersection lines.
!                      Default 0 degrees.
!                  (4) width of plot in cm (Default 20). (anything<1)
!                  (5) height of plot in cm (Default 20).(anything<1)
!                  (6) xminwindow  minimum dataregion with respect to
!                      the center of the figure to be plotted.
!                  (7) xmaxwindow maximum dataregion with respect to
!                      the center of the figure being plotted.
!                  NB if xmaxwindow - xminwindow<=0 a reasonable
!                     value is calculated by the plotting routine.
!                  (8) yminwindow minimum data region with respect to
!                      the figure being plotted.
!                  (9) ymaxwindow maximum data region with respect to
!                      the figure being plotted.
!                  NB if ymaxwindow - yminwindow<=0 a reasonable
!                     value is calculated by the plotting routine.
!                  (10) ground value: only important in BLOCK mode.
!                       all function values  at the boundary are
!                       connected to this value. ground value should
!                       be<=the minimal function value.
!                       this amount. If ground value>=0 the subroutine
!                       calculates a value:
!                       if all function values are>=0: GV = 0.
!                       if some function values are <0: GV = fmin
!                  (11) Scalefactor for of y-axis. Default 0.5.
!                  (12) Scalefactor for of z-axis. Default 1.
!                       If one of the scales are<=0 the default
!                       value is set.
!                  (13) y-fact
!                       IMPORTANT: Setting every entry of rlinp 0 gets
!                       default values for all variables
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision pi, dy, shape, ymin0, ymax0, work(2*npoint+nwork)
      integer nlines, ofkgrad, ofiworkp, ofacnseg, ofackseg, &
              ofnextra, ofkextra, ofncomp, ofxcint, offcint, ofworkp, &
              cntseg, ilines, ncint, ndimw, ofdiael, ofelkey, totseg, &
              totpts, ipoint, inumbr, iwork(niwork)
      logical revrse

!     cntseg
!     dy      length of vector in y-direction
!     ilines  Counting variable
!     inumbr
!     ipoint  Counting variable
!     iwork   integer work array
!     ncint   Number of interpolation points
!     ndimw   ?
!     nlines  number of parts where line elements are created
!     ofackseg
!     ofacnseg
!     ofdiael
!     ofelkey
!     offcint
!     ofiworkp
!     ofkextra
!     ofkgrad
!     ofncomp
!     ofnextra
!     ofworkp
!     ofxcint
!     pi
!     revrse
!     shape
!     totpts
!     totseg
!     work      dp workarray
!     ymax0
!     ymin0
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRCHR  Put character in error message
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     PLAFEP  General SEPRAN low level plotting subroutine
!     PLMORE  User subroutine to plot extra information in the plot
!     PLTJAC  Accumulates entries in nseg and kseg
!     PLTJAN  Analyzes xcint and inserts boundary structure.
!     PLTJBP  Basic plotting algorithm
!     PLTJCM  Calculates maximum minimum x- and y- xoordinate.
!     PLTJDE  Calculates  dy, xmin, xmax, ymin and ymax, fmin, fmax
!     PRJLIN  Make intersections
!     PRROTA  rotate coordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1257   nwork too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!       First prjlin makes intersections. These are analysed by pltjan to
!       insert the proper boundary information. The output  (nseg and kseg) of
!       prjlin is accumulated for ease of work. This is put into the basic
!       plotting algorithm pltjbp. If two sets of lines are required  the
!       procedure is repeated.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('pltj3d')

!     --- save coordinate array before the rats get it

      do ipoint = 1, npoint
        work (nwork + 2*ipoint - 1) = coor (1, ipoint)
        work (nwork + 2*ipoint    ) = coor (2, ipoint)
      end do
      call pltjde ( dy, coor, fval, npoint, rlinp, intinp, inumbr )
      call plafep (1, rlinp (4), rlinp (5), 0d0, 0d0, inumbr)

      shape = (ymaxh - yminh) / (xmaxh - xminh)
      if (shape<1d0) then
         shape = 1d0 / shape
      end if
50    continue

!     --- The following part is executed twice if intinp(1) = 2
!         First we determine the number of intersection lines for the given
!         angle of intersection, and the dy calculated by DECODE.

      if (abs (rlinp (3)) >1d-3) then
         call prrota ( coor, rlinp(3), npoint )
      end if
      call pltjcm ( coor, npoint, ymin0, ymax0, 2 )
      nlines = nint((ymax0 - ymin0)/dy) + 1
      if (nlines>51) then
         nlines = 51
         dy = .02*(ymax0-ymin0)
      end if
      if ( abs (rlinp(3))>1d-3 )  then
         call prrota ( coor, -rlinp(3), npoint )
      end if

!     --- Set offsets for workarrays

      if (intinp(4)==0) then
         ndimw = int (shape * 6 * sqrt (real (2*nelem)))
      else
         ndimw = intinp(4)
      end if
      ofkgrad = 1
      ofiworkp = ofkgrad + nelem
      ofacnseg = ofiworkp  + ndimw
      ofackseg = ofacnseg + nlines + 2

!     --- CHECK 1 on array iwork

      if (ofackseg + 2 * (nlines + 1)>niwork) then

!     --- ERROR: array IWORK too small

         call errint (ofackseg + 2 * (nlines + 1) - niwork, 1)
         call errint (1, 2)
         call errchr ('IWORK', 1)
         call errsub (1257, 2, 0, 1)

!        --- exit

         goto 99999
      end if
      ofelkey = 1
      ofdiael = ofelkey + nelem
      ofworkp = ofdiael + nelem
      ofxcint = ofworkp + 2 * ndimw
      ncint = (nwork - 2 * (nelem + ndimw))/2

!     --- CHECK 1 on array work

      if (ncint<2*nlines) then

!     --- Error: array work too small

         call errchr ('WORK', 1)
         call errint (2 * nlines - ncint, 1)
         call errint (1, 2)
         call errsub (1257, 2, 0, 1)

!        --- exit

         goto 99999
      else
         offcint = ofxcint + ncint
      end if

!     --- Make primary intersections

      call prjlin ( fval, coor, kmeshc, nelem, npoint, rlinp(3), &
                    xc, yc, nlines, iwork(ofacnseg+1), &
                    iwork(ofackseg+1), work(ofxcint), &
                    work(offcint), ncint, ymin0, ymax0, &
                    work(ofelkey), work(ofdiael), iwork(ofkgrad), &
                    work(ofworkp), iwork(ofiworkp), ndimw )
      dy = (ymax0 - ymin0) /(nlines - 1)

!     --- Only at this point the necessary workarrays can be estimated
!         we calculate the total number of segments (totseg) and the
!         total number of intersection points (totpts)
!         After that we may set the offsets of the rest of the work
!         arrays

      totseg = 0
      totpts = 0
      do 100 ilines = 1, nlines
         totseg = totseg + iwork (ofacnseg + ilines)
100   continue
      do 110 cntseg = 1, totseg
         totpts = totpts + iwork (ofackseg + cntseg)
110   continue

!     --- Checkpoint 2 on array work

      if (ncint<totpts) then

!     --- Error: work too small

         call errint (2 * (totpts - ncint), 1)
         call errchr ('work', 1)
         call errint (2, 2)
         call errsub (1257, 2, 0, 1)

!        --- exit

         goto 99999
      end if

!     --- Checkpoint 2 on array iwork

      if (niwork<ofackseg + 9 * totseg + 2) then

!     --- Error: iwork too small

         call errint (ofackseg + 9 * totseg + 2, 1)
         call errint (2, 2)
         call errchr ('iwork', 1)
         call errsub (1257, 2, 0, 1)

!        --- exit

         goto 99999
      end if

!     --- Since it is now absolutely certain that all auxiliary arrays
!         fit into iwork , we may hand out the remaining offsets

      ofnextra = ofackseg + totseg + 2
      ofkextra = ofnextra + 2 * totseg
      ofncomp  = ofkextra + 4 * totseg

!     --- set variables that govern the plot direction

      if (cth - scfi * sth<0) then
         revrse = .true.
      else
         revrse = .false.
      end if

!     --- Now the real work.
!         1. analysis of the boundary & component structure of the domain on
!            which the function is defined (anlsnc)
!         2. accmulation of pointer arrays (accseg)
!         3. Basic Plotting aLGorithm for Non Convex regions

      call pltjan ( work(ofxcint), iwork(ofacnseg + 1), &
                    iwork(ofackseg + 1), nlines, iwork(ofnextra), &
                    iwork(ofkextra), iwork(ofncomp),dy )

      call pltjac ( iwork(ofacnseg), iwork(ofackseg), nlines, revrse )
      call pltjbp ( iwork(ofacnseg), iwork(ofackseg), nlines, &
                    work(ofxcint), work(offcint), ncint, &
                    iwork(ofnextra), iwork(ofkextra), iwork(ofncomp), &
                    ymin0, dy, intinp(3) )

      if ( ierror/=0 ) goto 99999

      if ( intinp(1)==2 ) then
         rlinp (3) = rlinp (3) + 90d0
         pi = 4 * atan (1d0)
         cth = cos ((rlinp (1) - rlinp (3) + 1d0) * pi / 180d0)
         sth = sin ((rlinp (1) - rlinp (3) + 1d0) * pi / 180d0)
         intinp(1) = 1
         goto 50
      end if
      call plmore

!     --- exit & error exit
!         restore coor

99999 do ipoint = 1, npoint
        coor (1, ipoint) = work (nwork + 2*ipoint - 1)
        coor (2, ipoint) = work (nwork + 2*ipoint    )
      end do

      call erclos ('pltj3d')

      end
