      subroutine pltjau ( coor, fval, npoint, kmeshc, nelem, &
                          rlinp, intinp, nwork, niwork )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.2  date 12-05-2014 new call to pltdje
!        version  1.1  date 23-12-1998 correct for yfact
!        version  1.0  date 21-12-1998
!
!   copyright (c) 1998-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   pltjau provides an automatic estimate of the work arrays work and
!   iwork used in pltj3d, the 3-d visualisation package.
!   If used with the exact same input variables (with the exception of
!   the workarrays) it gives nwork and niwork that will be necessary
!   to operate pltj3d.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepltj
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intinp(5), npoint, kmeshc(*), nelem, nwork, niwork
      double precision coor(*), fval(*), rlinp(14)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     fval    i    function values in nodes
!     intinp  i    integer array of .. elements
!                  (1) : 1 lines are drawn parallel to x-axis
!                        2 2 sets of lines are drawn:
!                          one set determined by rlinp (3) and the
!                          other orthogonal
!                        All other values default to 1.
!                  (2) : Number of lines per cel -1.
!                        A cell has width dx defined by
!                        npoint = (a/dx + 1) (b/dx + 1)
!                        in which a and b are the smallest rectangle sides
!                        parallel to the x and y axes that wholly contain V.
!                        This is perfect for a rectangle with equidistant
!                        steps.
!                  (3) : 1: sheet mode
!                        2: block mode
!                        IMPORTANT: Setting every entry of intinp 0 gets
!                        default values for all variables
!                  (4) ndimw. Is estimated using prjest.
!                  (5) ncint. Is estimated using prjest
!     kmeshc  i    kmesh part c, contains the nodal points corresponding to the
!                  elements
!     nelem   i    Number of elements
!     niwork  o    declared dimension of iwork in pltj3d.
!                  Is estimated using prjest.
!     npoint  i    Number of nodal points
!     nwork   i    declared dimension of work in pltj3d
!                  Is estimated using prjest.
!     rlinp   i    dp array of 13 elements.
!                  (1) orientation of projection in degrees. Default 0
!                  (2) angle of projected y-axes wrt x-axes (degrees)
!                      Default 60 deg (10<phi<170). Anything outside
!                      this interval (notably phi = 0) defaults to 60.
!                  (3) orientation of primary intersection lines.
!                      Default 0 degrees.
!                  (4) width of plot in cm (Default 20). (anything<1)
!                  (5) height of plot in cm (Default 20).(anything<1)
!                  (6) xminwindow  minimum dataregion with respect to
!                      the center of the figure to be plotted.
!                  (7) xmaxwindow maximum dataregion with respect to
!                      the center of the figure being plotted.
!                  NB if xmaxwindow - xminwindow<=0 a reasonable
!                     value is calculated by the plotting routine.
!                  (8) yminwindow minimum data region with respect to
!                      the figure being plotted.
!                  (9) ymaxwindow maximum data region with respect to
!                      the figure being plotted.
!                  NB if ymaxwindow - yminwindow<=0 a reasonable
!                     value is calculated by the plotting routine
!                  (10) ground value: only important in BLOCK mode.
!                       all function values  at the boundary are
!                       connected to this value. ground value should
!                       be<=the minimal function value.
!                       this amount. If ground value>=0 the subroutine
!                       calculates a value:
!                       if all function values are>=0: GV = 0.
!                       if some function values are <0: GV = fmin
!                  (11) Scalefactor for of y-axis. Default 0.5.
!                  (12) Scalefactor for of z-axis. Default 1.
!                       If one of the scales are<=0 the default
!                       value is set.
!                  (13) y-fact
!                       IMPORTANT: Setting every entry of rlinp 0 gets
!                       default values for all variables
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dy, ymin0, ymax0, work(2*(nelem+npoint))
      integer ncint, ndimw, nibuf, nrbuf, nlines, ipoint, inumbr, &
              iwork(nelem)

!     dy
!     inumbr
!     ipoint
!     iwork      integer work array of size nelem
!     ncint      Number of interpolation points
!     ndimw
!     nibuf
!     nlines
!     nrbuf
!     work       dp workarray of size 2*nelem
!     ymax0
!     ymin0
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     PLTJCM
!     PLTJDE    Calculates  dy, xmin, xmax, ymin and ymax, fmin, fmax
!     PRJEST    core routine to estimate number of intersection points
!     PRROTA    rotate coordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pltjau' )
      do ipoint = 1, 2*npoint
         work (2*nelem+ipoint) = coor(ipoint)
      end do
      call pltjde ( dy, coor, fval, npoint, rlinp, intinp, inumbr )
      do ipoint = 1, npoint
      coor (2*ipoint - 1) = coor (2*ipoint - 1) - xc
      coor (2*ipoint) = coor (2*ipoint) - yc
      end do
      if (abs (rlinp (3)) >1d-3) then
         call prrota ( coor, rlinp(3), npoint )
      end if
      call pltjcm ( coor, npoint, ymin0, ymax0, 2 )
      nlines = nint((ymax0 - ymin0)/dy) + 1
      if (nlines>51) then
         nlines = 51
         dy = .02*(ymax0-ymin0)
      end if

      call prjest ( coor, kmeshc, nelem, npoint, nlines, &
                    work , work(nelem+1), iwork, &
                    ndimw, ncint, nibuf, nrbuf )
      intinp (4) = ndimw
      intinp (5) = ncint + ndimw
      niwork = nibuf
      nwork = nrbuf + ndimw
      if ( abs (rlinp(3))>1d-3 )  then
         call prrota ( coor, -rlinp(3), npoint )
      end if
      if (intinp(1)==2) then
         call prrota (coor, rlinp(3) + 90d0, npoint)
         call pltjcm (coor, npoint, ymin0, ymax0, 2)
         nlines = nint((ymax0 - ymin0)/dy) + 1
         if (nlines>51) then
            nlines = 51
            dy = .02*(ymax0-ymin0)
         end if
         call prjest ( coor, kmeshc, nelem, npoint, nlines, &
                       work , work(nelem+1), iwork, &
                       ndimw, ncint, nibuf, nrbuf )
         intinp (4) = max0 (ndimw, intinp(4))
         intinp (5) = max0 (ncint + ndimw, intinp(5))
         niwork = max0 (nibuf, niwork)
         nwork = max0 (nrbuf + ndimw, nwork)

!        --- call prrota (coor, -rlinp(3) - 90d0, npoint)

      end if

!     --- correct for y-scale

!         if (yscale>1d0) then
!         do ipoint = 1, npoint
!           coor (2, ipoint) = coor (2, ipoint)/yscale
!         end do
!         else
!         do ipoint = 1, npoint
!           coor (1, ipoint) = coor (1, ipoint)*yscale
!         end do
!         end if

!         --- restore coor

      do ipoint = 1, 2*npoint
        coor (ipoint) = work (2*nelem + ipoint)
      end do
      call erclos ( 'pltjau' )
      end
