      subroutine pltjde ( dy, coor, fval, npoint, rlinp, intinp, &
                          inumbr )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  2.0  date 12-05-2014 extra output parameter inumbr
!        version  1.6  date 05-06-1998 Small correction
!        version  1.5  date 19-02-1996 better calculation of ground
!                                      value in BLOCK mode
!
!   copyright (c) 1995-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       pltjde calculates  dy, xmin, xmax, ymin and ymax, fmin, fmax
!       in the common block /cplot/. It sets the defaults where necessary.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepltj
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, intinp(*)
      double precision coor(2, npoint), rlinp(*), fval(npoint), &
                       dy

!     coor    i    coordinate array
!     dy      o    distance of plotting lines
!     fval    i    function value array
!     intinp  i    integer array of .. elements
!                  (1) : 1 lines are drawn parallel to x-axis
!                        2 2 sets of lines are drawn:
!                          one set determined by rlinp (3) and the
!                          other orthogonal
!                        All other values default to 1.
!                  (2) : Number of lines per cel - 1.
!                        A cell has width dx defined by
!                        npoint = (a/dx + 1) (b/dx + 1)
!                        in which a and b are the smallest rectangle sides
!                        parallel to the x and y axes that wholly contain V.
!                        This is perfect for a rectangle with equidistant
!                        steps.
!                  (3) : 0: block mode
!                        1: sheet mode
!                        any other value defaults to block mode
!                           rlinp(13) (yscale)
!                        IMPORTANT: Setting every entry of intinp 0 gets default
!                        values for all variables
!     inumbr  o
!     npoint  i    number of function values
!     rlinp   i    dp array of 12 elements.
!                  (1) orientation of projection in degrees. Default 0
!                  (2) angle of projected y-axes wrt x-axes (degrees)
!                      Default 60 deg (10<phi<170). Anything outside
!                      this interval (notably phi = 0) defaults to 60.
!                  (3) orientation of primary intersection lines.
!                      Default 0 degrees.
!                  (4) width of plot in cm (Default 20). (anything<1)
!                  (5) height of plot in cm (Default 20).(anything<1)
!                  (6) xminwindow  minimum dataregion with respect to
!                      the center of the figure to be plotted.
!                  (7) xmaxwindow maximum dataregion with respect to
!                      the center of the figure being plotted.
!                  NB if xmaxwindow - xminwindow<=0 a reasonable
!                     value is calculated by the plotting routine.
!                  (8) yminwindow minimum data region with respect to
!                      the figure being plotted.
!                  (9) ymaxwindow maximum data region with respect to
!                      the figure being plotted.
!                  NB if ymaxwindow - yminwindow<=0 a reasonable
!                     value is calculated by the plotting routine.
!                  (10) ground value: only important in BLOCK mode.
!                       all function values  at the boundary are
!                       connected to this value. ground value should
!                       be<=the minimal function value.
!                       if it is not, the minimal value is taken.
!                  (11) Scalefactor for of y-axis. Default 0.5.
!                  (12) Scalefactor for of z-axis. Default 1.
!                       If one of the scales are<=0 the default
!                       value is set.
!                  (13) yfact
!                       IMPORTANT: Setting every entry of rlint 0 gets default
!                       values for all variables
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inumbr, ipoint
      double precision pi, a, b, dxproj, dxorth, zrangl, dzrangl

!     a
!     b
!     dxorth     see dxproj, but for orthogonal lines
!     dxproj     auxiliary quantity measuring the ration between dx
!                and the projected dx. If this is 0 or close to 0
!                some action must be taken.
!     dzrangl    minimal increment, that makes projection non singular
!     ipoint
!     pi
!     zrangl     angle for which projection is singular
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLTJCM  ?
!     PLTJFM  ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- First set default values for intinp

      if (intinp (1)/=1) then
         intinp (1) = 2

!        --- Default: two sets of intersecting lines

      end if

      if (intinp (3)/=1) then
         intinp (3) = 0

!        --- Default: block mode

      end if

      pi = 4d0 * atan (1d0)
      inumbr = 0

!     --- Calculate maxima and minima of x, y and z coordinates

      call pltjcm (coor, npoint, xminh, xmaxh, 1)
      call pltjcm (coor, npoint, yminh, ymaxh, 2)
      call pltjfm (fval, npoint, fminh, fmaxh)

!     --- rlinp (10) contains the (BLOCK mode) ground value. It is used
!         only if less than fminh.

      if (rlinp (10)<fminh) then
         fminh = rlinp (10)
      end if

!     --- Patch 05 june 1998
!         Autoscale x and y  coordinates

      if (abs(rlinp(13) - 1d0)<1d-10) then
        yscale = (xmaxh - xminh)/(ymaxh - yminh)
      else if (rlinp (13) >0) then
        yscale = rlinp (13)
      else
        yscale = 1d0
      end if

      if (yscale>1d0) then
        do ipoint = 1, npoint
          coor (2, ipoint) = coor (2, ipoint)*yscale
        end do
        ymaxh = ymaxh*yscale
        yminh = yminh*yscale
      else
        do ipoint = 1, npoint
          coor (1, ipoint) = coor (1, ipoint)/yscale
        end do
        xmaxh = xmaxh/yscale
        xminh = xminh/yscale
      end if

!     --- end patch

!         --- Calculate centre and sides of virtual rectangle that region is
!          contained in

      xc = .5d0 * (xminh + xmaxh)
      yc = .5d0 * (yminh + ymaxh)
      a = xmaxh - xminh
      b = ymaxh - yminh

!     --- Calculate cell width
!         This formula follows from the equation:
!         npoint = (a / dy + 1) * (b / dy + 1)

      dy = (a + b + sqrt ((a + b) ** 2 + 4 * a * b * (npoint - 1))) / &
         (2 * (npoint - 1))
      dy = dy / (intinp(2) + 1)
      if ((rlinp(2)>170d0) .or. (rlinp (2)<10d0)) then
         rlinp (2) = 60d0
      end if
      if ( rlinp (11)<=0d0 ) then

!     --- default for y scale factor

         rlinp (11) = .5d0

      end if
      if ( (rlinp (12)<=0d0) ) then

!     --- default for z scale factor

         zscale = 1d0
      else
         zscale = rlinp (12)
      end if
      ssfi = rlinp (11) * sin (rlinp (2) * pi / 180d0)
      scfi = rlinp (11) * cos (rlinp (2) * pi / 180d0)
      short = rlinp (11)
      sth = sin ((rlinp (1) - rlinp (3) + 1d0) * pi / 180d0)
      cth = cos ((rlinp (1) - rlinp (3) + 1d0) * pi / 180d0)

!     --- in case you are wondering: we add 1 degree to avoid a nasty type
!         of singularity caused by taking 'always' nice angles.
!         (1-12-95)

!         --- if intersection lines are vertical or almost vertical in the
!         projection plane the plot becomes ugly. We change the angle
!         a bit so that everything comes up roses. (26-01-95)

      dxproj = cth - scfi * sth
      dxorth = -sth - scfi * cth
      if (abs (dxproj)<1d-1) then

!     --- calculate for which angle this is exactly zero

         zrangl = atan (1/scfi)
         if (sth<0) then
            zrangl = zrangl + pi
         end if
         if (dxproj>=0) then
            dxproj = 1.2d-1
         else
            dxproj = -1.2d-1
         end if

!        --- calculate increment so that the angle is no longer singular

         dzrangl = - dxproj/(sin (zrangl) + scfi * cos (zrangl))
         zrangl = zrangl + dzrangl

!        --- calculate new sth, cth

         sth = sin (zrangl)
         cth = cos (zrangl)
         rlinp (1) = rlinp(3) + zrangl * 180 / pi

      else if (abs (dxorth)<1d-1 .and. intinp (1)==2) then

!     --- another possibility is, that the orthogonal lines will be plotted
!         under a singular angle.
!         calculate for which angle this is exactly zero

         zrangl = -atan (scfi)
         if (sth>0) then
            zrangl = zrangl + pi
         end if
         if (dxorth>=0) then
            dxorth = 1.2d-1
         else
            dxorth = -1.2d-1
         end if

!        --- calculate increment so that the angle is no longer singular

         dzrangl = - dxorth/(cos (zrangl) - scfi * sin (zrangl))
         zrangl = zrangl + dzrangl

!        --- calculate new sth, cth

         sth = sin (zrangl)
         cth = cos (zrangl)
         rlinp (1) = rlinp(3) + zrangl * 180 / pi

      end if

!     --- end of singular angle patch (26-01-95)

      if ((rlinp (9) - rlinp (8))>0) then

!     --- rlinp(8), (9) contain ymin & ymax

         ymax = rlinp (9)
         ymin = rlinp (8)
         inumbr = 1

      else

!     --- ymin & ymax must be calculated
!         This is done in such a way, that the cube bounded by (xminh, &
!         yminh, fminh) and (xmaxh, ymaxh, fmaxh) will always fit into the
!         picture, however it is rotated in the x, y plane.

         ymax = fmaxh + .5d0 * abs (ssfi) * sqrt ((xmaxh - xminh) ** 2 + &
          (ymaxh - yminh) ** 2)
         ymin = fminh - .5d0 * abs (ssfi) * sqrt ((xmaxh - xminh) ** 2 + &
          (ymaxh - yminh) ** 2)
      end if
      if (rlinp (7) - rlinp (6)>0) then

!     --- rlinp (6), (7) contain xmin, xmax
!         this is always with respect to the center of the object.

         xmax = rlinp (7)
         xmin = rlinp (6)
         inumbr = 1
      else

!     --- xmin & xmax must be calculated
!         in plotting coordinates they are centered around 0
!         (that is xc in real life)

         xmax = .5d0 * sqrt ((1d0 + scfi ** 2) * ((xmaxh - xminh) ** 2 &
            + (ymaxh - yminh) ** 2))
         xmin = - xmax
      end if
      if (rlinp (4)<1d0) then
         rlinp (4) = 20d0
      end if
      if (rlinp (5)<1d0) then
         rlinp (5) = 20d0
      end if

!     --- call plafep (1, rlinp (4), rlinp (5), 0d0, 0d0, inumbr)

      end
