      subroutine pltr02 ( up, vp, ski, eta, t, tend, x, y, xl, yl, &
                          dref, dout, istag, nsteps, partout, maxsteps, &
                          itypeinteg, difeps, icallfn, repeat )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.3  date 24-08-2001 Logical istored is added (Ivo Wenneker)
!        version  2.2  date 09-06-2001 Correction for small steps
!        version  2.1  date 28-12-2000 Reduce partout
!        version  2.0  date 31-03-1996 Extra parameters nsteps and partout
!        version  1.3  date 22-01-1993 adjustment timestep determination
!        version  1.2  date 18-12-1991 correction for time writing
!        version  1.1  date 14-07-1991 DOUT added
!        version  1.0  date 08-03-1991
!
!   copyright (c) 1991-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine for particle-tracking in the unit-triangle
!     velocity has been given in the three corner points
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     particle_tracking
!     unit_triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision up(3), vp(3), ski, eta, t, partout(3,*), &
                       tend, x, y, xl(3), yl(3), dref, dout, difeps
      integer istag, nsteps, maxsteps, icallfn, itypeinteg
      logical repeat

!     difeps     i    reference accuracy for errorterm
!     dout       o    length of track in this element
!     dref       i    reference length
!     eta       i/o   first local coordinate location particle
!     icallfn    i    Indicates if for each position in the particle trace
!                     a user subroutine FNPARTIC must be called (1) or not (0)
!     istag      o    indicator whether particle is in stagnation point
!                     istag=1 or not istag=0
!     itypeinteg i    Defines the type of integration aplied
!                     Possible values:
!                     1:  Heun (explicit)
!                     2:  Runge-Kutta 4 (explicit)
!                     3:  Trapezoid rule (implicit)
!     maxsteps   i    Maximum number of positions that may be stored in partout
!     nsteps    i/o   Contains the number of positions stored in partout
!     partout   i/o   Array containing the actual information concerning the
!                     particle trajectories
!                     partout(1,i) contains t_i
!                     partout(2-3,i) contains x_i and y_i
!     repeat    i/o   If true the process has not yet been finished
!     ski       i/o   at input coordinate of startlocation
!                     at output of end location
!     t         i/o   actual time
!     tend       i    time at which the integration should
!                     terminate
!     up         i    array with u-velocities in corners of triangle
!     vp         i    array with v-velocities in corners of triangle
!     x         i/o   global x-coordinate
!     xl         i    global x-coordinates of triangle corners
!     y         i/o   global y-coordinate
!     yl         i    global y-coordinates of corners of triangle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision afst, diff, dx, dy, dt, eps, &
                       etadub, etaold, releps, skidub, &
                       skiold, uex, ustart, vex, &
                       vstart, xold, yold, t0, t1, t2, t3, &
                       ski2, ski3, ski4, eta2, eta3, eta4, &
                       u1, u2, u3, u4, v1, v2, v3, v4, matA(2,2), &
                       bleft(2), dski, deta, theta
      integer istap, istep, ja
      logical istored

!     afst        local distance
!     bleft       Right-hand side (needed for Crank-Nicolson method)
!     deta        Eta(new) - eta(old)
!     diff        difference (l2-norm) between two points
!     dski        Ski(new) - ski(old)
!     dt          time step
!     dx          local x-distance
!     dy          local y-distance
!     eps         accuracy parameter
!     eta2        Predictor for eta in RK4-procedure
!     eta3        Predictor for eta in RK4-procedure
!     eta4        Predictor for eta in RK4-procedure
!     etadub      second estimate for new eta
!     etaold      old value for eta
!     istap       number of steps already taken in this element
!     istep       counter for step halving
!     istored     Logical indicating whether a point of the triangle
!                 currently under consideration has been added to partout
!                 (istored = .true.) or has not been added (istored = .false.)
!     ja          local indicator value
!     matA        Matrix (needed for Crank-Nicolson method)
!     releps      accuracy related to machine accuracy
!     ski2        Predictor for ski in time integration procedure
!     ski3        Predictor for ski in time integration procedure
!     ski4        Predictor for ski in time integration procedure
!     skidub      second estimate for new ski
!     skiold      old value for ski
!     t0          help timevariable for crossing point
!     t1          help timevariable for face ski = 0
!     t2          help timevariable for face eta = 0
!     t3          help timevariable for face ski+eta = 1
!     theta       Parameter theta in Crank-Nicolson time integration method
!     u1          Predictor for u in time integration procedure
!     u2          Predictor for u in time integration procedure
!     u3          Predictor for u in time integration procedure
!     u4          Predictor for u in time integration procedure
!     uex         first extra velocity component u-direction
!     ustart      startvalue velocity component u-direction
!     v1          Predictor for v in time integration procedure
!     v2          Predictor for v in time integration procedure
!     v3          Predictor for v in time integration procedure
!     v4          Predictor for v in time integration procedure
!     vex         first extra velocity component v-direction
!     vstart      startvalue velocity component v-direction
!     xold        x-value old position
!     yold        y-value old position
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERREAL    Put real in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     ERRWAR    Print warnings
!     FNPARTIC  User function subroutine that is called in eah step of the
!               particle tracer
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     936   particle trapped in element: 100 steps, terminate track
!    1638   Particle array too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine stepsize
!     Compute new position using the modified Euler method
!     Check whether particle is outside element and if so
!       compute point where elementboundary has been crossed
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pltr02' )
      eps     = 1d-4
      releps  = 10 * epsmac
      xold    = x
      yold    = y
      istap   = 0
      istag   = 0
      dout    = 0d0
      istored = .false.

!     --- Determine stepsize

100   istap = istap + 1
      ustart = (1d0-ski-eta)*up(1) + ski*up(2) + eta*up(3)
      vstart = (1d0-ski-eta)*vp(1) + ski*vp(2) + eta*vp(3)
      if ( istap.eq.1 ) then

!     --- First step, compute stepsize (conservative approach)

         dt   = 0.08 / sqrt( ustart*ustart + vstart*vstart )
         diff = 1d0
      else

!     --- Next steps, compute stepsize in accordance with last result

         if ( diff.lt.1d-4 ) diff = 1d-4
         if ( diff.gt.1d-1 ) diff = 1d-1
         dt = 0.8 * dt * sqrt( difeps / diff )
      end if
      if ( t+dt.gt.tend ) dt = ( 1d0 + releps ) * ( tend - t )
      istep = 0

!     --- Set old values

      skiold = ski
      etaold = eta

!     --- Start of actual time integration

120   ski = skiold + dt * ustart
      eta = etaold + dt * vstart

      if ( itypeinteg.eq.1 ) then

!     --- Step 1 of Heun time integration

         u1   = ustart
         v1   = vstart

!        --- Step 2

         ski2 = ski
         eta2 = eta
         u2   = (1d0-ski2-eta2)*up(1) + ski2*up(2) + eta2*up(3)
         v2   = (1d0-ski2-eta2)*vp(1) + ski2*vp(2) + eta2*vp(3)

!        --- Update

         skidub = skiold + dt * ( u1 + u2 ) / 2d0
         etadub = etaold + dt * ( v1 + v2 ) / 2d0

      else if ( itypeinteg.eq.2 ) then

!     --- RK4 (Runge-Kutta 4) time integration
!         Step 1

         u1     = ustart
         v1     = vstart

!        --- Step 2

         ski2   = skiold + 0.5d0 * dt * u1
         eta2   = etaold + 0.5d0 * dt * v1
         u2     = (1d0-ski2-eta2)*up(1) + ski2*up(2) + eta2*up(3)
         v2     = (1d0-ski2-eta2)*vp(1) + ski2*vp(2) + eta2*vp(3)

!        --- Step 3

         ski3   = skiold + 0.5d0 * dt * u2
         eta3   = etaold + 0.5d0 * dt * v2
         u3     = (1d0-ski3-eta3)*up(1) + ski3*up(2) + eta3*up(3)
         v3     = (1d0-ski3-eta3)*vp(1) + ski3*vp(2) + eta3*vp(3)

!        --- Step 4

         ski4   = skiold + dt * u3
         eta4   = etaold + dt * v3
         u4     = (1d0-ski4-eta4)*up(1) + ski4*up(2) + eta4*up(3)
         v4     = (1d0-ski4-eta4)*vp(1) + ski4*vp(2) + eta4*vp(3)

!        --- Update

         skidub = skiold + dt*( u1 + 2d0*(u2 + u3) + u4 )/6d0
         etadub = etaold + dt*( v1 + 2d0*(v2 + v3) + v4 )/6d0

      else if ( itypeinteg.eq.3 ) then

!     --- Crank-Nicolson method time integration

         theta = 0.5d0

!        --- Build linear system: matA*dx = bleft

         matA(1,1) = theta*dt*( up(1) - up(2) ) + 1d0
         matA(1,2) = theta*dt*( up(1) - up(3) )
         matA(2,1) = theta*dt*( vp(1) - vp(2) )
         matA(2,2) = theta*dt*( vp(1) - vp(3) ) + 1d0

         bleft(1) = dt*ustart
         bleft(2) = dt*vstart

!        --- Solve linear system

         detA = matA(1,1)*matA(2,2) - matA(1,2)*matA(2,1)
         dski = ( matA(2,2)*bleft(1) - matA(1,2)*bleft(2) )/detA
         deta = (-matA(2,1)*bleft(1) + matA(1,1)*bleft(2) )/detA

!        --- Update ski and eta

         skidub = skiold + dski
         etadub = etaold + deta

      end if

!     --- Check whether timestep dt is too large

      uex = ski - skidub
      vex = eta - etadub
      diff = sqrt( uex * uex + vex * vex )
      if ( diff.gt.difeps .and. istep.lt.5 ) then

!     --- Difference too large take smaller step and do new calculation

         dt = dt * 0.8 * sqrt( difeps / diff )
         istep = istep + 1
         goto 120
      end if

!     --- New position determined

      ski  =  skidub
      eta  =  etadub

!     --- Determine global position

      x = xl(1) + (xl(2)-xl(1))*ski + (xl(3)-xl(1))*eta
      y = yl(1) + (yl(2)-yl(1))*ski + (yl(3)-yl(1))*eta
      dx = x - xold
      dy = y - yold
      afst = sqrt( dx * dx + dy * dy )
      dout = dout + afst
      if ( afst.gt.dref ) then
         ja   = 1
         xold = x
         yold = y
      else
         ja   = 0
      end if

      if ( ( -eps.lt.ski .and. 1d0+eps.gt.ski )  .and. &
           ( -eps.lt.eta .and. 1d0+eps.gt.eta )  .and. &
           ( -eps.lt.1d0-ski-eta ) )  then

!     --- (ski,eta) inside triangle

         t    = t + dt
         if ( ja.eq.1 ) then

!        --- New point

            istored = .true.
            nsteps  = nsteps + 1
            if ( nsteps.gt.maxsteps ) then
               call errint ( nsteps, 1 )
               call errsub ( 1638, 1, 0, 0 )
               repeat = .false.
               go to 1000
            end if
            partout(1,nsteps) = t
            partout(2,nsteps) = x
            partout(3,nsteps) = y
            if ( icallfn.gt.0 ) &
                 call fnpartic ( t, x, y, 0d0, .false., repeat )
            if ( .not.repeat ) go to 1000
         end if
         if ( istap.gt.100 .or. istep.gt.4 ) then

!        --- Particle trapped in element : Stagnation point

            call erreal ( xl(1), 1 )
            call erreal ( yl(1), 2 )
            call erreal ( xl(2), 3 )
            call erreal ( yl(2), 4 )
            call erreal ( xl(3), 5 )
            call erreal ( yl(3), 6 )
            call errwar ( 936, 0, 6, 0 )
            istag = 1
            go to 1000
         end if
         if ( t.ge.tend ) goto 1000
         goto 100
      end if

!     --- Outside triangle, interpolate to obtain crossing point

      if ( ski.gt.0 ) then
         t1 = 2d0
      else
         if ( abs( skiold - ski ).lt.eps ) then
            t1 = 2d0
         else
            t1 = skiold / ( skiold - ski )
         end if
      end if
      if ( eta.gt.0 ) then
         t2 = 2d0
      else
         if ( abs( etaold - eta ).lt.eps ) then
            t2 = 2d0
         else
            t2 = etaold / ( etaold - eta )
         end if
      end if
      if ( ski+eta.lt.1d0 ) then
         t3 = 2d0
      else
         if ( abs( ski+eta-skiold-etaold ).lt.eps ) then
            t3 = 2d0
         else
            t3 = ( 1d0-skiold-etaold ) / ( ski+eta-skiold-etaold )
         end if
      end if
      t0 = min( t1, t2, t3 )
      t0 = t0 + 1d-4
      ski = skiold + t0 * ( ski - skiold )
      eta = etaold + t0 * ( eta - etaold )

!     --- Adjust time in accordance with the interpolation

      t = t + t0 * dt

!     --- Determine global position

      x = xl(1) + (xl(2)-xl(1))*ski + (xl(3)-xl(1))*eta
      y = yl(1) + (yl(2)-yl(1))*ski + (yl(3)-yl(1))*eta

      if ( partout(1,nsteps).lt.t-1d-3*tend .or. (.not.istored) ) then

!     --- Elapsed time after last stored particle is more than 0.1 percent
!         of end time, or no particle has yet been stored in the considered
!         triangle

         nsteps = nsteps + 1
         if ( nsteps.gt.maxsteps ) then
            call errint ( nsteps, 1 )
            call errsub ( 1638, 1, 0, 0 )
            repeat = .false.
            go to 1000
         end if
         partout(1,nsteps) = t
         partout(2,nsteps) = x
         partout(3,nsteps) = y
      end if
      if ( icallfn.gt.0 ) &
           call fnpartic ( t, x, y, 0d0, .false., repeat )
      dx = x - xold
      dy = y - yold
      dout = dout - afst
      afst = sqrt( dx * dx + dy * dy )
      dout = dout + afst
      xold = x
      yold = y
1000  call erclos ( 'pltr02' )
      end
