      subroutine pltr13 ( up, vp, wp, ski, eta, zta, t, tend, x, y, z, &
                          xl, yl, zl, dref, nsteps, &
                          partout, maxsteps, icallfn, repeat )
! ======================================================================
!
!         programmer   Niek praagman
!         version 3.0  date  06-04-1996 Extra parameters
!         version 2.0  date  11-02-1993 redesign
!         version 1.1  date  18-12-1991 correction for time writing added
!         version 1.0  date  31-05-1991
!
!
!
!   copyright (c) 1991-1996  "ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine for particle-tracking in the unit-tetrahedron while the
!     velocity is constant over the tetrahedron
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     particle_tracking
!     tetrahedron
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision up, vp, wp, ski, eta, zta, t, tend, x, y, z, &
                       dref, xl(4), yl(4), zl(4), partout(4,*)
      integer nsteps, maxsteps, icallfn
      logical repeat

!     dref       i    reference distance for plotting
!     eta        i    local coordinate
!     icallfn    i    Indicates if for each position in the particle trace
!                     a user subroutine FNPARTIC must be called (1) or not (0)
!     maxsteps   i    Maximum number of positions that may be stored in partout
!     nsteps    i/o   Contains the number of positions stored in partout
!     partout   i/o   Array containing the actual information concerning the
!                     particle trajectories
!                     partout(1,i) contains t_i
!                     partout(2-3,i) contains x_i and y_i
!     repeat    i/o   If true the process has not yet been finished
!     ski       i/o   first local coordinate
!     t         i/o   actual traveling time
!     tend       i    end time at which the integration should terminate
!     up         i    u velocity in tetrahedron
!     vp         i    v velocity in tetrahedron
!     wp         i    w velocity in tetrahedron
!     x          i    global x-coordinate of particle
!     xl         i    global x-coordinates of nodes
!     y          i    global y-coordinate of particle
!     yl         i    global y-coordinates of nodes
!     z          i    global z-coordinate of particle
!     zl         i    global z-coordinates of nodes
!     zta       i/o   third local coordinate
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision afst, dt, dx, dy, dz, eps, etaold, skiold, &
                       ztaold, term1, term2, xold, yold, zold, &
                       t0, t1, t2, t3, t4
      integer ja

!     afst      distance
!     dt        timestep
!     dx        x-distance
!     dy        y-distance
!     dz        z-distance
!     eps       local accuracy
!     etaold    old eta position
!     ja        indicator whether reference distance has been travelled
!     skiold    old ski position
!     t0        help t variable for crossing
!     t1        help t variable for crossing
!     t2        help t variable for crossing
!     t3        help t variable for crossing
!     t4        help t variable for crossing
!     term1     sum of old coordinates
!     term2     sum of new coordinates
!     xold      old x position
!     yold      old y position
!     zold      old z position
!     ztaold    old zta position
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    deconcatenate name from sequence of names
!     EROPEN    concatenate name to sequence of names
!     ERRINT    Put integer in error message
!     ERRSUB    give error message
!     FNPARTIC  User function subroutine that is called in eah step of the
!               particle tracer
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1638   Particle array too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      eps = 1d-4

      xold = x
      yold = y
      zold = z

100   continue

!     --- Determine stepsize

      dt = 2d0 / sqrt( up * up + vp * vp + wp * wp )

      if ( t+dt.gt.tend ) dt = (tend - t) * (1d0 + eps)

!     --- set old values

      skiold = ski
      etaold = eta
      ztaold = zta

      ski    = ski + dt * up
      eta    = eta + dt * vp
      zta    = zta + dt * wp

!     --- Determine global position

      x = xl(1) + ( xl(2) - xl(1) ) * ski + &
                  ( xl(3) - xl(1) ) * eta + ( xl(4) - xl(1) ) * zta
      y = yl(1) + ( yl(2) - yl(1) ) * ski + &
                  ( yl(3) - yl(1) ) * eta + ( yl(4) - yl(1) ) * zta
      z = zl(1) + ( zl(2) - zl(1) ) * ski + &
                  ( zl(3) - zl(1) ) * eta + ( zl(4) - zl(1) ) * zta

      dx = x - xold
      dy = y - yold
      dz = z - zold

      afst = sqrt( dx * dx + dy * dy + dz * dz )

      ja = 0

      if ( afst.gt.dref ) then

         xold = x
         yold = y
         zold = z

         ja = 1

      end if

      if ( ( -eps.lt.ski .and. 1d0+eps.gt.ski )  .and. &
           ( -eps.lt.eta .and. 1d0+eps.gt.eta )  .and. &
           ( -eps.lt.zta .and. 1d0+eps.gt.zta )  .and. &
           ( -eps.lt.1d0-ski-eta-zta ) )  then

!     --- Adjust time

         t = t + dt

         if ( ja.eq.1 ) then

!        --- New point

            nsteps = nsteps + 1
            if ( nsteps.gt.maxsteps ) then
               call eropen( 'pltr13' )
               call errint ( nsteps, 1 )
               call errsub ( 1638, 1, 0, 0 )
               call erclos( 'pltr13' )
               repeat = .false.
               go to 1000
            end if
            partout(1,nsteps) = t
            partout(2,nsteps) = x
            partout(3,nsteps) = y
            partout(4,nsteps) = z
            if ( icallfn.gt.0 ) &
                 call fnpartic ( t, x, y, z, .false., repeat )
            if ( .not.repeat ) go to 1000

         end if
         if ( t.ge.tend ) goto 1000

         goto 100

      end if

!     --- Outside tetrahedron, interpolate to obtain crossing point

      if ( ski.gt.0 ) then

         t1 = 2d0

      else

         if ( abs( skiold - ski ).lt.eps ) then
            t1 = 2d0
         else
            t1 = skiold / ( skiold - ski )
         end if

      end if

      if ( eta.gt.0 ) then

         t2 = 2d0

      else

         if ( abs( etaold - eta ).lt.eps ) then
            t2 = 2d0
         else
            t2 = etaold / ( etaold - eta )
         end if

      end if

      if ( zta.gt.0 ) then

         t2 = 2d0

      else

         if ( abs( ztaold - zta ).lt.eps ) then
            t3 = 2d0
         else
            t3 = ztaold / ( ztaold - zta )
         end if

      end if

      if ( ski+eta+zta.lt.1d0 ) then

         t4 = 2d0

      else

         term1 = ski    + eta    + zta
         term2 = skiold + etaold + ztaold

         if ( abs( term1 - term2 ).lt.eps ) then
            t4 = 2d0
         else
            t4 = ( 1d0 - term2 ) / ( term1 - term2 )
         end if

      end if

      t0 = min( t1, t2, t3, t4 )
      t0 = t0 + 1d-5

      ski = skiold + t0 * ( ski - skiold )
      eta = etaold + t0 * ( eta - etaold )
      zta = ztaold + t0 * ( zta - ztaold )

!     --- Adjust time in accordance with the interpolation

      t = t + t0 * dt

!     --- Determine global position

      x = xl(1) + (xl(2)-xl(1))*ski + &
                  (xl(3)-xl(1))*eta + (xl(4)-xl(1))*zta
      y = yl(1) + (yl(2)-yl(1))*ski + &
                  (yl(3)-yl(1))*eta + (yl(4)-yl(1))*zta
      z = zl(1) + (zl(2)-zl(1))*ski + &
                  (zl(3)-zl(1))*eta + (zl(4)-zl(1))*zta

      nsteps = nsteps + 1
      if ( nsteps.gt.maxsteps ) then
         call eropen( 'pltr13' )
         call errint ( nsteps, 1 )
         call errsub ( 1638, 1, 0, 0 )
         call erclos( 'pltr13' )
         repeat = .false.
         go to 1000
      end if
      partout(1,nsteps) = t
      partout(2,nsteps) = x
      partout(3,nsteps) = y
      partout(4,nsteps) = z
      if ( icallfn.gt.0 ) &
           call fnpartic ( t, x, y, z, .false., repeat )

1000  continue

      end
