      subroutine pltracout ( partout, ndim, iprint, iplot, istoutp, &
                             partreduced, istepprint, tstepout, &
                             tstepmark, tstepprint, nsteps, iparticle, &
                             imark, nview )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-04-2010 New call to plaf05
!        version  1.0  date 29-12-2000
!
!   copyright (c) 2000-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Print and/or plot the particle trajectories
!    If necessary the particle trajectories are interpolated and stored in
!    the output array partout or partreduced
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     print
!     output
!     particle_tracking
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, iprint, iplot, istoutp, istepprint, nsteps, &
              iparticle, imark, nview
      double precision partout(ndim+1,*), partreduced(ndim+1,*), &
                       tstepout, tstepmark, tstepprint

!     imark          i    Indicates if markers must be set in the plot (>0)
!                         or not (0)
!     iparticle      i    Sequence number of particle
!     iplot          i    Indicates if a plot must be made (>0) or not (0)
!     iprint         i    Indicates if a print must be made (>0) or not (0)
!     istepprint     i    Indicates if tstep_print is used for the printing
!                         (1) or not (0)
!     istoutp        i    If 1 the output must be stored in ioutput
!     ndim           i    Dimension of the space.
!     nsteps         i    Contains the number of positions stored in partout
!     nview          i    Defines type of viewing
!                         Possible values
!                         1 : Projection on plane y=constant seen from y=  oo
!                         2 : Projection on plane x=constant seen from x=  oo
!                         3 : Projection on plane y=constant seen from y= -oo
!                         4 : Projection on plane x=constant seen from x= -oo
!     partout        i    Array containing the actual information concerning the
!                         particle trajectories
!                         partout(1,i) contains t_i
!                         partout(2-3,i) contains x_i and y_i
!     partreduced    o    Array containing the actual information concerning the
!                         particle trajectories
!                         partout(1,i) contains t_i
!                         partout(2-3,i) contains x_i and y_i
!                         The difference with partout is that this array may
!                         contain the values in by the user chosen time
!                         intervals, while partout contains the time steps
!                         detected by the computation
!     tstepmark      i    time step wth respect to markers
!     tstepout       i    time step wth respect to output
!     tstepprint     i    time step wth respect to print
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical finished
      double precision tmark, x(3), t, dx, alpha1
      integer i, j, ipos, icount

!     alpha1         multiplication factor
!     dx             length of vector in x-direction
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     i              Counting variable
!     icount         Counter
!     ipos           Position in array
!     j              Counting variable
!     t              actual time
!     tmark          time at which a marker must be set
!     x              array with the co-ordinates of a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAF05         Plot polygon
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLTR15         Plot part of 3-D track
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( iprint>0 ) then
!        print particle trace, depending on istepprint
!     if ( iplot>0 ) then
!        plot particle trace
!        if ( imark>0 ) then
!           plot markers
!        if ( istoutp>0 ) then
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pltracout' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pltracout'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug .or. iprint>0 ) then

!     --- iprint>0, print particle trace

         if ( ndim==2 ) then
            write(iprint,100) iparticle, '        x               y'
100         format ( /' Trace of particle ',i3/'          t      ', a )
         else
            write(iprint,100) iparticle, &
             '         x               y               z'
         end if
         if ( istepprint==0 ) then

!        --- Print as computed

            if ( ndim==2 ) then
               write(iprint,110) ((partout(j,i),j=1,3),i=1,nsteps)
110            format((ss, 3(1pe16.6e2)))
            else
               write(iprint,120) ((partout(j,i),j=1,4),i=1,nsteps)
120            format((ss, 4(1pe16.6e2)))
            end if

         else

!        --- Print as indicated by tstepprint

            tmark = partout(1,1)
            t = tmark
            finished = .false.
            ipos = 1
            do while ( .not. finished )
               tmark = tmark + tstepprint
               do while ( t<tmark+sqreps .and. ipos<=nsteps )
                  ipos = ipos+1
                  t = partout(1,ipos)
               end do
               if ( ipos>nsteps ) then
                  finished = .true.
               else
                  alpha1 = (t-tmark)/(t-partout(1,ipos-1))
                  do i = 1, ndim
                     dx = partout(i+1,ipos)-partout(i+1,ipos-1)
                     x(i) = partout(i+1,ipos)-alpha1*dx
                  end do
                  write(iprint,120) tmark, (x(i),i=1,ndim)
               end if

            end do

         end if

      end if

      if ( iplot>0 ) then

!     --- iplot = 1, plot particle trace

         dcolor = 3
         if ( ndim==2 ) then

!        --- ndim = 2, first plot trace

            call plaf05 ( 3, dcolor, nsteps, partout(2,1), 3 )

         else

!        --- ndim = 3, first plot trace

            call pltr15 ( nsteps, partout(2,1), nview, 4 )

         end if
         icount = 0
         if ( imark>0 ) then

!        --- Markers must be set
!            First put markers in partreduced

            tmark = partout(1,1)
            t = tmark
            finished = .false.
            ipos = 1
            do while ( .not. finished )
               tmark = tmark + tstepmark
               do while ( t<tmark+sqreps .and. ipos<nsteps )
                  ipos = ipos+1
                  t = partout(1,ipos)
               end do
               if ( ipos>=nsteps ) then

                  finished = .true.

               else

                  icount = icount+1
                  alpha1 = (t-tmark)/(t-partout(1,ipos-1))
                  do i = 1, ndim
                     dx = partout(i+1,ipos)-partout(i+1,ipos-1)
                     partreduced(i,icount) = partout(i+1,ipos)-alpha1*dx
                  end do

               end if

            end do

         end if

         if ( ndim==3 ) then

!        --- ndim = 3, project

            if ( nview==1 ) then

               do i = 1, icount
                  do j = 1, 3
                     x(j) = partreduced(j,i)
                  end do
                  partreduced(1,i) = x(1) + 0.77d0*x(2)
                  partreduced(2,i) = x(3) + 0.45d0*x(2)
               end do

            else if ( nview==2 ) then

               do i = 1, icount
                  do j = 1, 3
                     x(j) = partreduced(j,i)
                  end do
                  partreduced(1,i) = x(2) - 0.77d0*x(1)
                  partreduced(2,i) = x(3) - 0.45d0*x(1)
               end do

            else if ( nview==3 ) then

               do i = 1, icount
                  do j = 1, 3
                     x(j) = partreduced(j,i)
                  end do
                  partreduced(1,i) =-x(1) - 0.77d0*x(2)
                  partreduced(2,i) = x(3) - 0.45d0*x(2)
               end do

            else if ( nview==4 ) then

               do i = 1, icount
                  do j = 1, 3
                     x(j) = partreduced(j,i)
                  end do
                  partreduced(1,i) =-x(2) + 0.77d0*x(1)
                  partreduced(2,i) = x(3) + 0.45d0*x(1)
               end do

            end if

         end if

         do i = 1, icount
            call plafep ( 6, partreduced(1,i), partreduced(2,i), &
                          0d0, 0d0, iparticle*5 )
         end do

      end if

      if ( istoutp>1 ) then

!     --- Store array in partreduced

         tmark = partout(1,1)
         t = tmark
         finished = .false.
         ipos = 1
         icount = 1
         do i = 1, ndim+1
            partreduced(i,icount) = partout(i,1)
         end do
         do while ( .not. finished )
            tmark = tmark + tstepout
            do while ( t<tmark+sqreps .and. ipos<nsteps )
               ipos = ipos+1
               t = partout(1,ipos)
            end do
            if ( ipos>=nsteps ) then

               finished = .true.

            else

               icount = icount+1
               alpha1 = (t-tmark)/(t-partout(1,ipos-1))
               do i = 1, ndim
                  dx = partout(i+1,ipos)-partout(i+1,ipos-1)
                  partreduced(i+1,icount) = partout(i+1,ipos)-alpha1*dx
               end do
               partreduced(1,icount) = tmark

            end if

         end do
         nsteps = icount

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pltracout'

      end if  ! ( debug )
      call erclos ( 'pltracout' )

      end subroutine pltracout
