      subroutine prcontourcomp1 ( markelems, area, nelem, kmeshc, &
                                  coor, kmesho, plotpos, numelms, &
                                  activeelems, icolor, func, &
                                  marknodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-04-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all neighboring elements of element corresponding to the
!     same region satisfying function value>specified value
!     Compute area and plot elements if necessary
!     The mesh must consist of linear trangles only
!     Special case of 1 node completely in region
! **********************************************************************
!
!                       KEYWORDS
!
!     contour_level
!     area
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, markelems(nelem), kmeshc(3), kmesho(3), &
              numelms, activeelems(nelem), marknodes(*)
      double precision area, coor(2,*), func(*)
      logical plotpos

!     activeelems   i/o   Contains the list of still active elements
!                         These elements are neighbors of the elements in the
!                         bubble
!     area          i/o   Area of bubble
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     func           i    Contains the scaled and shifted function values
!     icolor         i    Sequence number of colour
!     kmeshc         i    kmesh part c, contains the nodal point numers of the
!                         element
!     kmesho         i    kmesh part o, contains the neighbor elements of the
!                         element
!     markelems     i/o   See subroutine prcontourcomp
!     marknodes     i/o   See subroutine prcontourcomp
!     nelem          i    Number of elements
!     numelms       i/o   Number of elements in activeelems
!     plotpos        i    If true the bubble must be plotted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ineigh, icolor, icount, i1, node, node1, &
              node2, nodepos
      double precision polyg(2,3), arealoc, x1(2), x2(2)

!     arealoc        Area of small triangle
!     i              Counting variable
!     i1             Local edge that is not intersected
!     icount         Counter
!     ineigh         neighbor
!     node           Node number
!     node1          Node number
!     node2          Node number
!     nodepos        Node number of node with positive funtion value
!     polyg          Array containing the co-ordinates of the point in the
!                    polygon in the following way:
!                    polyg(1,i) contains the x-coordinate of the first
!                    point
!                    polyg(2,i) contains the y-coordinate of the first
!                    point
!     x1             contains coordinates of first intersection point
!     x2             contains coordinates of second intersection point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHAREA        Compute the area of the triangle spanned by x1-x2-x3
!                    The sign of the are defines the direction
!     PLAF05         Plot polygon
!     PRININ         print 1d integer array
!     PRROOT         Find coordinates where function value is equal to 0
!                    function value on end points is given
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Mark element and all its non-marked neighbors with num_bubbles
!     Compute area
!     Plot element if necessary
!     In this case two edges are intersected
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcontourcomp1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcontourcomp1'
         write(irefwr,2) 'area', area
         call prinin ( kmeshc, 3, 'kmeshc' )
         call prinin ( kmesho, 3, 'kmesho' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find node with positive value

      icount = 0
      do i = 1, 3

         node = kmeshc(i)
         if ( marknodes(node)<0 ) then
            if ( icount==0 ) then
               node1 = node
               icount = 1
            else
               node2 = node
            end if  ! ( icount==0 )
         else if ( marknodes(node)>0 ) then
            nodepos = node
            i1 = i+1
            if ( i1>3 ) i1 = 1
         end if  ! ( marknodes(node)<0 )

      end do  ! i = 1, 3

      if ( debug ) &
         write(irefwr,1) 'i1, nodepos, node1, node2', &
                          i1, nodepos, node1, node2

!     --- Find non-marked neighbors

      do i = 1, 3

         if ( i1/=i ) then

!        --- Side contains positive point

            ineigh = mod(kmesho(i),nelem+1)
            if ( ineigh>0 ) then

!           --- neighbor found

               if ( markelems(ineigh)==0 ) then

!              --- new neighbor, add to list

                  numelms = numelms+1
                  activeelems(numelms) = ineigh
                  markelems(ineigh) = -2

               end if  ! ( markelems(ineigh)==0 )

            end if  ! ( ineigh>0 )

         end if  ! ( ineg/=i )

      end do  ! i = 1, 3

!     --- Compute coordinates of intersection point, store in x

      if ( debug ) then
         write(irefwr,2) 'coor(nodepos)', coor(1:2,nodepos)
         write(irefwr,2) 'coor(node1)', coor(1:2,node1)
         write(irefwr,2) 'coor(node2)', coor(1:2,node2)
         write(irefwr,2) 'func', func(nodepos), func(node1), &
                          func(node2)
      end if  ! ( debug )

      call prroot ( x1, coor(1,nodepos), coor(1,node1), &
                    func(nodepos), func(node1), 2 )
      call prroot ( x2, coor(1,nodepos), coor(1,node2), &
                    func(nodepos), func(node2), 2 )

      if ( debug ) then
         write(irefwr,2) 'x1', x1
         write(irefwr,2) 'x2', x2
      end if  ! ( debug )

      call msharea ( coor(1,nodepos), x1, x2, arealoc )

      area = area+abs(arealoc)
      if ( debug ) write(irefwr,2) 'area, arealoc', area, arealoc

      if ( plotpos ) then

!     --- Plot small triangle

         polyg(1:2,1) = coor(1:2,nodepos)
         polyg(1:2,2) = x1
         polyg(1:2,3) = x2

         call plaf05 ( 1, icolor, 3, polyg, 2 )

      end if  ! ( plotpos )

1000  call erclos ( 'prcontourcomp1' )
      if ( debug ) then

!     --- Debug information

         call prinin ( activeelems, numelms, 'activeelems' )
         write(irefwr,2) 'area', area
         write(irefwr,*) 'End prcontourcomp1'

      end if  ! ( debug )

      end
