      subroutine seppfield ( user, iuser, esol, ichois, rchois, work, lenwrk )
! ======================================================================
!
!        programmer    Arno van Duin
!        version  1.0  date 04-04-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to compute, and / or plot the electric field lines
!     in a region as given by the mesh of KMESH, with the potential field
!     as given by Vsol, and with the electric field as given by esol.
!     In array ICHOIS the following possibilities are distinguished
!     ICHOIS(1)  =  indicator to plot field lines
!                    1 : Make general plot of field lines
!                    2 : Make plot of field lines starting on some part of the
!                        boundary
!                    3 : Make plot of field lines ending on some part of the
!                        boundary
!                    4 : Make plot of field line through point X
!                    5 : Make plot of field line starting in point X
!                    6 : Make plot of field line ending in point X
!     ICHOIS(2)  =  first user point of boundary part (ICHOIS(1) = 2 or 3)
!     ICHOIS(3)  =  last user point of boundary part (ICHOIS(1) = 2 or 3)
!     Old subroutine pfield
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer esol(*), ichois(*), iuser(*), lenwrk
      double precision rchois(*), user(*), work(*)

!     esol    i    Standard SEPRAN array containing information of the electric
!                  field array
!     ichois  i    Choice parameter array (see description above)
!     iuser   i    Integer array in which information concerning the
!                  coefficients is stored
!     lenwrk  i    length of array work
!             o    number of positions filled in array work
!     rchois  i    choise parameter array:
!                     RCHOIS(1)  =  PLOTfM
!                     RCHOIS(2)  =  YfACT
!                     RCHOIS(3)  =  f
!                     RCHOIS(4)  =  X1
!                     RCHOIS(5)  =  Y1
!     user    i    Real array in which information concerning the coefficients
!                  is stored.
!     work    o    array with lengths of field lines
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ibnd, idoubl, iinner, ikader, ikeus, in, lbrpnt, lnnr, maxsteps
      double precision eloc(2), f, lenfln, oldlen, plotfm, x1, y1, &
                       yfact
      double precision, allocatable :: trace(:)
      integer, allocatable :: index1(:), index2(:), index3(:), index4(:), &
                              itel(:)
      double precision :: timebefore(2)

!     eloc           E-field in point (x1,y1)
!     f              flux-cut
!     ibnd           while-loop variable
!     idoubl         parameter for plotbn, default value
!     iinner         parameter for plotbn, plot all (also inner-) boundaries
!     ikader         parameter for pl0003, plot kader
!     ikeus          parameter for pl0003, take standard colour
!     in             checking parameter on inward(2) or outward(3) flux
!     index1
!     index2
!     index3
!     index4
!     itel
!     lbrpnt         number of boundary nodal points
!     lenfln         length of field line
!     lnnr           number of field lines
!     maxsteps       maximum number of positions in a particle trace
!     oldlen         length of field line
!     plotfm         Plotting factor. When PLOTfM<0, then the length of
!                    the plot is computed by - (xmax-xmin) * PLOTfM
!                    and the width by - (ymax-ymin) * YfACT * PLOTfM
!                    else the length of the plot is equal to PLOTfM cm
!                    When by some mistake the user has given | PLOTfM | a value
!                    smaller than 1 cm, then the length is chosen equal to
!                    the default length stored in common block consta
!     trace          Array containing the actual information concerning the
!                    particle trajectories
!                    trace(1,i) contains t_i
!                    trace(2-3,i) contains x_i and y_i
!     x1             x-coordinate of starting point
!     y1             y-coordinate of starting point
!     yfact          Multiplication for the y-co-ordinates. With this
!                    parameter the user can influence the scale in y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     PL0003         Open plot, plots frame work and computes plotting
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLFL00         get (part of) boundary
!     PLFL01         get starting point and electric field in starting point
!     PLFL02         determine field line through starting point
!     PLFL03         walk d(F) along the boundary to obtain new starting point
!     PLOTBN         Plot boundary of region
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1505   array size too small to store the lengths of all field lines
!    1976   flux-cut has illegal value, must be>0
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialise some parameters and reserve some space
!     Open plot
!     Plot boundaries
!     Case ICHOIS(1) of
!       1,2,3 : If value for flux-cut is illegal then
!                  adjust value and give warning
!               Endif
!               Get boundary
!               Get starting point and electric field in that point
!               While not end of boundary
!                  Determine field line through starting point
!                  Obtain new starting point
!               Endwhile
!       4     : Determine field line starting in starting point
!               Determine field line ending in starting point
!       5     : Determine field line starting in starting point
!       6     : Determine field line ending in starting point
!     Endcase
!     Close plot
!     free temporary arrays
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppfield' )
      if ( itime>0 ) call ersettime ( timebefore )

      if ( ierror/=0 ) go to 1000
      allocate ( index1(3*nelem), index2(npoint+1), index3(2*npoint), &
                 index4(4*npoint+100), itel(nelem) )
      maxsteps = 20000

      allocate ( trace(5*maxsteps), stat = error )
      if ( error/=0 ) call eralloc ( error, 5*maxsteps, 'trace' )
      if ( ierror/=0 ) go to 1000
      usoltemp => ks(esol(1))%sol
      in     = ichois(1)
      plotfm = rchois(1)
      yfact  = rchois(2)
      f      = rchois(3)
      x1     = rchois(4)
      y1     = rchois(5)
      dcolor = 0
      jmark  = 5
      ikader = 0
      ikeus  = 0
      lnnr   = 0
      idoubl = 0
      iinner = 3

!     --- open plot

      textx = 'x'
      texty = 'y'
      call pl0003 ( ikader, 1, plotfm, yfact, ikeus )

!     --- plot boundaries

      call plotbn ( idoubl, iinner, .false. )

      if ( in==1 .or. in==2 .or. in==3 ) then

!     --- test on illegal flux

         if ( .not. ( f>0d0 ) ) then
            call erreal ( f, 1 )
            if ( abs(f)>=0d0 ) then
               f = 1d0
            else
               f = abs(f)
            end if
            call erreal ( f, 2 )
            call errsub ( 1976, 0, 2, 0 )
         end if

!        --- get (part of) boundary

         call plfl00 ( npoint, index2, nelem, index1, kmeshc, &
                       lbrpnt, index3, index4, coor, &
                       ichois(2), ichois(3), kmeshl )

!        --- get starting point and electric field in starting point

         call plfl01 ( coor, index3, &
                       Eloc, usoltemp, in, x1, y1 )

!        --- while not end of boundary

         ibnd = 1
         lenfln = 0d0
160      if ( ibnd<=lbrpnt .and. ibnd>0 ) then
            dcolor = dcolor + 1
            if ( dcolor==10 ) dcolor = 2

!           --- determine field line through starting point

            call plfl02 ( x1, y1, in, usoltemp, nelem, &
                          kmeshc, coor, itel, lenfln, trace, &
                          maxsteps )
            lnnr = lnnr + 1

!           --- report length of field line

            write( irefwr, * ) '   Length of field line :', lnnr, lenfln

            if ( lnnr<=lenwrk ) then
               work(lnnr) = lenfln
            else
               if ( lenwrk/=0 .and. lnnr==lenwrk + 1 ) then
                  call errwar ( 1505, 0, 0, 0 )
               end if
            end if

!           --- walk d(f) along the boundary to obtain new starting point

            call plfl03 ( x1, y1, Eloc, ibnd, in, f, &
                          index3, lbrpnt, &
                          usoltemp, user, iuser )
            go to 160
         end if
         if ( lnnr<=1 ) then
            if ( in==1 ) then
               write ( irefwr, 100 )
               write ( irefwr, 200 )
100            format('   No more field lines, probable cause:')
200            format('   - chosen flux-cut is too large, try a', &
                      ' smaller value.')
300            format('   - flux is directed outwards, try "towards"', &
                      ' (ICHOIS(1)=3)')
400            format('   - flux is directed inwards, try "from"', &
                      ' (ICHOIS(1)=2)')
            else if ( in==2 ) then
               write ( irefwr, 100 )
               write ( irefwr, 300 )
               write ( irefwr, 200 )
            else if ( in==3 ) then
               write ( irefwr, 100 )
               write ( irefwr, 400 )
               write ( irefwr, 200 )
            end if
         end if

      else if ( in==4 ) then

!     --- determine field line starting in starting point

         dcolor = dcolor + 1
         if ( dcolor==10 ) dcolor = 2
         call plfl02 ( x1, y1, 2, usoltemp, &
                       nelem, kmeshc, coor, &
                       itel, oldlen, trace, maxsteps )

!        --- determine field line ending in starting point

         dcolor = dcolor + 1
         if ( dcolor==10 ) dcolor = 2
         call plfl02 ( x1, y1, 3, usoltemp, &
                       nelem, kmeshc, coor, &
                       itel, lenfln, trace, maxsteps )
         lnnr = lnnr + 1

!        --- report length of field line

         write ( irefwr, * ) '   Length of field line :', lnnr, &
                             oldlen + lenfln

         if ( lnnr<=lenwrk ) then
            work(lnnr) = oldlen + lenfln
         else
            if ( lenwrk/=0 .and. lnnr==lenwrk + 1 ) then
               call errwar ( 1505, 0, 0, 0 )
            end if
         end if

      else if ( in==5 ) then

!     --- determine field line starting in starting point

         dcolor = dcolor + 1
         if ( dcolor==10 ) dcolor = 2
         call plfl02 ( x1, y1, 2, usoltemp, &
                       nelem, kmeshc, coor, &
                       itel, lenfln, trace, maxsteps )
         lnnr = lnnr + 1

!        --- report length of field line

         write ( irefwr, * ) '   Length of field line :', lnnr, lenfln

         if ( lnnr<=lenwrk ) then
            work(lnnr) = lenfln
         else
            if ( lenwrk/=0 .and. lnnr==lenwrk + 1 ) then
               call errwar ( 1505, 0, 0, 0 )
            end if
         end if

      else if ( in==6 ) then

!     --- determine field line ending in starting point

         dcolor = dcolor + 1
         if ( dcolor==10 ) dcolor = 2
         call plfl02 ( x1, y1, 3, usoltemp, &
                       nelem, kmeshc, coor, &
                       itel, lenfln, trace, maxsteps )
         lnnr = lnnr + 1

!        --- report length of field line

         write ( irefwr, * ) '   Length of field line :', lnnr, lenfln

         if ( lnnr<=lenwrk ) then
            work(lnnr) = lenfln
         else
            if ( lenwrk/=0 .and. lnnr==lenwrk + 1 ) then
               call errwar ( 1505, 0, 0, 0 )
            end if
         end if

      end if

      deallocate ( trace, itel, index1, index2, index3, index4, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'trace' )

      lenwrk = lnnr
      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )

1000  call erclmn ( 'seppfield', timebefore )
      end subroutine seppfield
