      subroutine sepplcompview ( istartsurnr, viewp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the viewpoint for the plotting of the obstacles by looking at
!     the line through centroid of starting surface and centroid of volume
!     Old subroutine plcompview
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     plot_parameter
!     surface
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: istartsurnr
      double precision :: viewp(*)

!     istartsurnr    i    Surface number of starting surface
!     viewp          o    Array containing the view point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nodp, nnodes
      integer, allocatable :: nodes(:)
      double precision x, y, z, xmin, ymin, zmin, xmax, ymax, zmax, &
                       x0, y0, z0, xc, yc, zc, l, l_x, l_y, l_z, alpha

!     alpha          constant to compute values
!     i              Counting variable
!     l              General loop variable
!     l_x            xmax-xmin
!     l_y            ymax-ymin
!     l_z            zmax-zmin
!     nnodes         Number of nodes
!     nodes          Contains the node numbers that must be filled
!     nodp           Nodal point sequence number
!     x              x co-ordinate of point
!     x0             x co-ordinate of centroid
!     xc             array with the coordinates of the centre point of the
!                    element
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     y              y co-ordinate of point
!     y0             y co-ordinate of centroid
!     yc             y-coordinate of the midth-point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     z              z co-ordinate of point
!     z0             z co-ordinate of centroid
!     zc             z-coordinate of the midth-point
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPSURNOD      Store all nodes in the surfaces ifirst to ilast in array
!                    nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find all nodes in starting surface and store in nodes
!     Compute centroid (x0,y0,z0) of this surface
!     Compute centroid (xc,yc,zc) of volume as well as xmin, xmax, ...
!     l_x = xmax-xmin, l_y = ymax-ymin, l_z = zmax-zmin, &
!     L = max(l_x,l_y,l_z)
!     alpha = 1.5L / ||xc - x0||
!     x_view = x0+alpha(xc-x0)+l_x
!     y_view = y0+alpha(yc-y0)-l_y
!     z_view = z0+alpha(zc-z0)+l_z
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepplcompview' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepplcompview'

      end if
      nnodes = npoint

      allocate ( nodes(nnodes), stat = error )
      if ( error/=0 ) call eralloc ( error, nnodes, 'nodes' )
      if ( ierror/=0 ) go to 1000

!     --- Find all nodes in starting surface and store in nodes

      call sepsurnod ( istartsurnr, istartsurnr, nodes, nnodes )

!     --- Compute centroid (x0,y0,z0) of this surface

      x0 = 0d0
      y0 = 0d0
      z0 = 0d0
      do i = 1, nnodes
         nodp = nodes(i)
         x0 = x0+coor(1,nodp)
         y0 = y0+coor(2,nodp)
         z0 = z0+coor(3,nodp)
      end do
      x0 = x0/nnodes
      y0 = y0/nnodes
      z0 = z0/nnodes

!     --- Compute centroid (xc,yc,zc) of volume as well as xmin, xmax, ...

      xc = 0d0
      yc = 0d0
      zc = 0d0
      xmin = rinfin
      ymin = xmin
      zmin = xmin
      xmax = - xmin
      ymax = xmax
      zmax = xmax
      do i = 1, npoint
         x = coor(1,i)
         y = coor(2,i)
         z = coor(3,i)
         xc = xc+x
         yc = yc+y
         zc = zc+z
         xmin = min(x,xmin)
         ymin = min(y,ymin)
         zmin = min(z,zmin)
         xmax = max(x,xmax)
         ymax = max(y,ymax)
         zmax = max(z,zmax)
      end do
      xc = xc/npoint
      yc = yc/npoint
      zc = zc/npoint

!     --- l_x = xmax-xmin, l_y = ymax-ymin, l_z = zmax-zmin, &
!         L = max(l_x,l_y,l_z)
!         alpha = 1.5L / ||xc - x0||
!         x_view = x0+alpha(xc-x0)+l_x
!         y_view = y0+alpha(yc-y0)-l_y
!         z_view = z0+alpha(zc-z0)+l_z

      l_x = xmax-xmin
      l_y = ymax-ymin
      l_z = zmax-zmin
      l = max(l_x,l_y,l_z)
      alpha = 1.5d0*l/sqrt((xc-x0)**2+(yc-y0)**2+(zc-z0)**2)
      viewp(1) = x0+alpha*(xc-x0)+l_x
      viewp(2) = y0+alpha*(yc-y0)-l_y
      viewp(3) = z0+alpha*(zc-z0)+l_z

      deallocate ( nodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepplcompview'

      end if
      call erclos ( 'sepplcompview' )

      end
