      subroutine sepplot3d ( number, isol, &
                             base, height, angle, linopt )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.0  date 01-06-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine to make a 3-d plot with hidden lines of the solution
!     vector isol ; the number'th component is considered .
!     old subroutine plot3d
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     hidden_line
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecactl
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, isol, linopt
      double precision base, angle, height(*)

!     angle   i   direction from which the user wants to
!                 see the picture in degrees
!     base    i   parameter to indicate the length of the
!                 base of the picture .
!     height  i   height of the picture and if base<0
!                 also the max reference value in height(2)
!                 and the min reference value in height(3)
!     isol    i   Standard SEPRAN array containing information of the solution
!     linopt  i   linold + 10 * (nstep-1)
!                 with
!                 linold
!                   direction in which the surface-lines
!                   are drawn
!                     1  :  only parallel to the y-axis
!                     2  :  only parallel to the x-axis
!                     3  :  in both directions
!                     4  :  not yet available
!                     5  :  pictures with all possibilities
!                           are made sequentially
!                 nstep
!                   indicator for points in regular grid
!                     (nstep+1)*(nstep+1)*npoint positions are
!                     used in the regular grid.
!     number  i   reserved
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl
      integer linold, k, nstep, kwadrt, ispec, larray

      double precision factor, helpf, refval(2)
      integer, allocatable :: nstart(:)
      double precision, allocatable :: amag(:), bmag(:)
      double precision :: timebefore(2)

!     amag           array to be filled with magnitudes
!                    of function to be plotted .
!     bmag       i   help array of nstep*nstep*npoint positions
!     compl          Indicates if the vector is complex (true) or not
!     factor         reserved
!     helpf           reserved
!     ispec          reserved
!     k              General loop variable
!     kwadrt
!     larray
!     linold
!     nstart         array of nelem+1 positions containing
!                    the start positions of all elements sequentially
!     nstep
!     refval
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     PLT301         ?
!     PLT302         ?
!     PLT303         ?
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      62   dimension of space (ndim) has wrong value
!      90   array isol has not been filled, or the boundary conditions have
!           not been stored in the solution vector
!     102   NUMBER has wrong value
!     477
!     478
!     528
!     529
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepplot3d' )
      if ( itime>0 ) call ersettime ( timebefore )
      if ( ks(isol)%typevec/=110 .and. ks(isol)%typevec/=115 ) then
         call errint ( ks(isol)%typevec, 1 )
         call errsub ( 90, 1, 0, 0 )
      end if
      if ( ndim/=2 ) then
         call errint ( ndim, 1 )
         call errsub ( 62, 1, 0, 0 )
      end if
      linold=mod(linopt, 10)
      if ( linold<0 .or. linold>5 .or. linold==4 ) then
         call errint ( linopt, 1 )
         call errsub ( 477, 1, 0, 0 )
      end if
      if ( abs(base)<1d0 ) then
         call erreal ( base, 1 )
         call errsub ( 478, 0, 1, 0 )
      end if
      if ( height(1)<0d0 ) then
         call erreal ( height(1), 1 )
         call errsub ( 528, 0, 1, 0 )
      end if
      if ( angle<0d0 .or. angle>360d0 ) then
         call erreal ( angle, 1 )
         call errsub ( 529, 0, 1, 0 )
      end if

!     --- check whether number is o.k.:

      iprob  = ks(isol)%iprob
      call sepactsolbf2 ( isol )
      compl = ks(isol)%compl==1
      if ( compl ) then

!     --- complex case

         if ( number<=0 .or. number>2*nphysi )  then
            call errint ( number, 1 )
            call errint ( 2*nphysi, 2 )
            call errsub ( 102, 2, 0, 0 )
         end if
      else

!     --- real case

         if ( number<=0 .or. number>nphysi )  then
            call errint ( number, 1 )
            call errint ( nphysi, 2 )
            call errsub ( 102, 2, 0, 0 )
         end if
      end if

!     --- create help array with starting addresses kelem

      nstep=linopt/10+1
      kwadrt=nstep*nstep

      allocate ( nstart(nelem+1), amag(npoint), bmag(kwadrt*npoint),&
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nelem+(1+kwadrt)*npoint, 'nstart' )
      if ( ierror/=0 ) go to 1000

      call plt301 ( nstart )

!     --- determine function to be plotted
!         determine how accurate function is interpolated

      usoltemp => ks(isol)%sol
      call plt302 ( npoint, nphysi, indprfi, kprobfi, amag, &
                    usoltemp, compl, number, ispec )

!     --- compute factor

      factor = height(1) / abs(base)
      if ( base<0 ) then
         refval(1) =   height(2)
         refval(2) =   height(3)
         base      = - base
      else
         refval(1) =   0
         refval(2) =   0
      end if
      if ( factor<1d0 ) factor = 1d0

!     --- Extra arrays for PLTISO

      helpf = sqrt(50d0+npoint)
      larray = nint(nstep*helpf*2d0)

!     --- make 3d-plot

      call plt303 ( number, compl, kprobfi, nstart, &
                    amag, bmag, larray, angle, linopt, base, &
                    factor, ispec, refval )

      deallocate ( nstart, amag, bmag, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nstart' )

1000  call erclmn ( 'sepplot3d', timebefore )
      end subroutine sepplot3d
