      subroutine sepplpoinobst
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     plot nodal points in the obstacle with marks (x)
!     The colour is changed in order to see the difference between
!     internal nodes and nodes near the boundary of the obstacle
!     Old subroutine plpoinobst
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inodp, iobst, istart, numpntobst, &
              numinnobst, numbndobst, iscolr, numinonobst
      integer, allocatable :: iwork(:)
      double precision x, y

!     inodp          Nodal point number
!     iobst          Obstacle sequence number
!     iscolr         Help variable to store the value of dcolor temporarily
!     istart         First position used at a part
!     iwork          Work array of length npoint
!                    iwork(i)=1 node i is inside the obstacle
!                    iwork(i)=2 node i is near the boundary of the obstacle
!                    iwork(i)=3 node i is on the boundary of the obstacle
!                    iwork(i)=4 node i is on the boundary of the obstacle
!                                      and inside obstacle
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numinonobst    Number of nodal points on the boundary of the obstacle
!                    which are only in elements completely inside the obstacle
!     numpntobst     Number of nodal points inside the obstacle
!     x              x co-ordinate of point
!     y              y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork (1:npoint)
!     For all obstacles iobst do
!        Fill array iwork(i) with
!          1 if node i is inside the obstacle
!          2 if node i is near the boundary of the obstacle
!          3 if node i is on the boundary of the obstacle
!          4 if node i is on the boundary of the obstacle and inside obstacle
!        Plot point with colour depending on iwork
!        Clear iwork
!     Plot text
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      iscolr = dcolor

!     --- allocate work array

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork (1:npoint)

      iwork = 0

!     --- For all obstacles iobst do

      do iobst = 1, numobst

!     --- Fill array iwork(i) with
!         1 if node i is inside the obstacle
!         2 if node i is near the boundary of the obstacle
!         3 if node i is on the boundary of the obstacle
!         4 if node i is on the boundary of the obstacle and inside obstacle

         istart = ipointobst(iobst)
         numpntobst = ipointobst(istart)
         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         numinonobst = ipointobst(istart+3)
         istart = istart+3
         iwork(ipointobst(istart+1:istart+numpntobst)) = 1
         istart = istart+numpntobst
         iwork(ipointobst(istart+1:istart+numinnobst)) = 2
         istart = istart+numinnobst
         iwork(ipointobst(istart+1:istart+numbndobst)) = 3
         istart = istart+numbndobst
         iwork(ipointobst(istart+1:istart+numinonobst)) = 4

!        --- Plot point with colour depending on iwork

         do inodp = 1, npoint
            if ( iwork(inodp)/=0 ) then
               x = coor(1,inodp)
               y = coor(2,inodp)
               if ( iwork(inodp)==1 ) then
                  dcolor = 84+iobst
               else if ( iwork(inodp)==2 ) then
                  dcolor = 73+iobst
               else if ( iwork(inodp)==4 ) then
                  dcolor = 104-iobst
               else
                  dcolor = 0
               end if
               call plafep ( 6, x, y, 0d0, 0d0, dcolor )
            end if
         end do

         if ( iobst<numobst ) then

!        --- Clear iwork

            iwork(ipointobst(istart+1:istart+numinonobst)) = 0
            istart = istart-numbndobst
            iwork(ipointobst(istart+1:istart+numbndobst)) = 0
            istart = istart-numinnobst
            iwork(ipointobst(istart+1:istart+numinnobst)) = 0
            istart = istart-numpntobst
            iwork(ipointobst(istart+1:istart+numpntobst)) = 0

         end if  ! ( iobst<numobst )

      end do

!     --- Plot text

      dcolor = iscolr
      call plafp7 ( 5d0, 19.5d0, 'nodes in obstacle', 0d0, 0.25d0, 17 )
      dcolor = 85
      call plafp7 ( 5d0, 19d0, 'x  inside', 0d0, 0.25d0, 9 )
      dcolor = 74
      call plafp7 ( 5d0, 18.5d0, 'x  near boundary', 0d0, 0.25d0, 16 )
      dcolor = 0
      call plafp7 ( 5d0, 18d0, 'x  on boundary', 0d0, 0.25d0, 14 )
      dcolor = 103
      call plafp7 ( 5d0, 17.5d0, 'x in/on boundary', 0d0, 0.25d0, 16 )
      dcolor = iscolr

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  end
