      subroutine topsurfmesh ( vertex, iskip, iskpar, ilevel, nelemlc, &
                               subver, iresul, subtri, idegfd, itrian, &
                               centroid, special, neleml, isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     construct a mesh consisting of surface elements only
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iskip, iskpar(*), ilevel, idegfd, isol(*)
      integer, intent(inout) :: nelemlc, iresul(5,*), neleml, &
                                itrian(*), centroid, special
      logical, intent(in) :: vertex, subver, subtri

!     centroid      i/o   If 1 a 9 point element has been found of which the
!                         only 8 points have been filled
!     idegfd         i    Degree of freedom sequence number
!     ilevel         i    Defines the level of elements to be plotted
!     iresul        i/o   See subroutine topqu0
!     iskip          i    Indicates if element groups must be skipped (>0)
!                         or not (0)
!     iskpar         i    array of length NELGRP that is only used if ISKIP>0
!                         It indicates if element group ielgrp must be skipped
!                         iskpar(ielgrp) = 0 or not iskpar(ielgrp) = 1
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     itrian        i/o   work array of length nelgrp, see topqu2
!     neleml        i/o   Number of "line"-elements
!     nelemlc       i/o   Number of elements
!     special       i/o   Standard situation: special=0 at input
!                         If special=2 and the unknown is present in the
!                         vertices, special is set to 0, else if the unknown is
!                         only present in the midside points, special
!                         remains unchanged
!     subtri         o    Indicates if the mesh must be subdivided into
!                         triangles and also the solution copied (true) or not
!     subver        i/o   Indicates if a "vertex" mesh must be created (true)
!                         or not
!     vertex         i    indicates if only the vertices of the surface elements
!                         have to be filled (true) or all points (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iskipel, ntria, nplsub, lensub, nelemt, i, &
                 inpelm, leng
      integer, pointer :: kelem(:)

!     i              Counting variable
!     inpelm         Number of nodes in element
!     iskipel        if 0 no elements must be skipped, if 1 array elemskip
!                    is used to detect which elements must be skipped
!     kelem          Local array kmeshc
!     leng           Help parameter to indicate the length of arrays
!     lensub         If zero the original mesh is used, otherwise
!                    lensub>0 gives the length of the KMESH part c
!     nelemt         Number of elements in the new mesh
!     nplsub         Number of points in the submesh consisting of elements
!                    with vertex nodes only
!     ntria          number of triangles or quadrilaterals after removal of
!                    double ones
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHG  Fill array nmeshg
!     TOPQ02         Create a submesh consisting of triangles or quadrilaterals
!                    where only the vertices are used.
!     TOPQU2         Check if mesh must be subdivided and compute length of
!                    "KMESHC"
!     TOPQU3         Subdivide mesh into triangles
!     TOPQU8         Split volumes in triangles and quadrilaterals
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     First create all faces of the elements
!     Next find all volume elements corresponding to the faces
!     Finally store all faces that correspond to only one volume element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topsurfmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topsurfmesh'
         call ertrace
         write(irefwr,1) 'nelem, npoint, leng', nelem, npoint, length
         write(irefwr,*) 'subver ', subver
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      leng = length

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(31)==0 ) call sepfillnmesha
      if ( nmesh(37)==0 ) call sepfillnmeshg

      allocate ( kelem(leng), stat = error )
      if ( error/=0 ) call eralloc ( error, leng, 'kelem' )
      if ( ierror/=0 ) go to 1000

      if ( .not. associated(elemskip) ) iskipel = 0

      call topqu8 ( kelem, ntria, inpelm, vertex, &
                    iskip, iskpar, iskipel, ilevel )
      if ( ierror/=0 ) go to 1000
      if ( subtri ) then
      end if
      nelemlc = ntria

      if ( subver ) then

!     --- Create a vertex mesh

         if ( debug ) write(irefwr,1) 'inpelm', inpelm
         if ( inpelm>4 ) then

!        --- surface mesh must be adapted

            if ( inpelm==9 ) then
               nplsub = 4
            else
               nplsub = 3
            end if
            allocate ( kmeshcsub(nelemlc*nplsub) )
            iresul(1,1) = nelemlc*nplsub
            if ( ierror/=0 ) go to 1000
            call topq02 ( nelemlc, kelem, kmeshcsub, inpelm, nplsub )
            iresul(2,1) = nplsub

         else

!        --- Use surface mesh

            allocate ( kmeshcsub(nelemlc*inpelm) )
            iresul(1,1) = nelemlc*inpelm
            if ( ierror/=0 ) go to 1000
            kmeshcsub = kelem(1:nelemlc*inpelm)
            iresul(2,1) = inpelm

         end if

         iresul(3,1) = nelemlc
         iresul(4,1) = 1

      end if  ! ( subver )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'subtri ', subtri
      if ( subtri ) then

!     --- Make a submesh consisting of triangles only

         if ( inpelm==4 .or.inpelm==5 ) then
            ishape = 5   ! linear quadrilateral
         else if ( inpelm==9 ) then
            ishape = 6   ! quadratic quadrilateral
         else if ( inpelm>4 ) then
            ishape = 4   ! quadratic triangle
         else
            ishape = 3   ! linear triangle
         end if
         kmeshc => kelem

!        --- Check if mesh must be subdivided into subtriangles or not

         inelem = nelemlc
         nelgrp = 1
         call topqu2 ( idegfd, lensub, itrian, nelemt, &
                       neleml, special, centroid, inpelm )
         if ( ierror/=0 ) go to 1000

         if ( debug ) write(irefwr,1) 'lensub', lensub
         if ( lensub>0 ) then

!        --- lensub>0, so mesh must be subdivided into triangles

            allocate ( kmeshctmp(lensub) )
            iresul(1,2) = lensub
            if ( ierror/=0 ) go to 1000
            call topqu3 ( itrian, neleml, inpelm )

         else if ( subver ) then

!        --- The same grid as for the vertices is used

            nelemt = nelemlc
            kmeshctmp => kmeshcsub

         else

!        --- lensub = 0, no subdivision

            nelemt = nelemlc
            allocate ( kmeshctmp(nelemlc*3) )
            kmeshctmp = kelem(1:nelemlc*3)
            iresul(1,2) = nelemlc*3

         end if
         iresul(3,2) = nelemt

      end if  ! ( subtri )

      deallocate ( kelem, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kelem' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topsurfmesh'

      end if  ! ( debug )
      call erclos ( 'topsurfmesh' )

      end subroutine topsurfmesh
