      subroutine creatv01 ( isol, inpcre, rincre, islold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 10-04-2013 Adaptation format
!        version  2.0  date 07-01-2012 Complete new body
!        version  1.13 date 14-07-2011 Extra debug
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     create a SEPRAN real or complex array (vector)
!     A part of the array is filled by creatv01
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: isol(*), inpcre(*), islold(*)
      double precision rincre(*)

!     inpcre  i    Array containing the integer input
!                  In this array it is indicated how the arrays must be created
!                  and filled
!                  If INPCRE has been given by the user must be filled as
!                  follows:
!                  Pos 1: number of entries filled by the user
!                  Pos 2: number of different vectors in isol
!                  Per vector:
!                  Pos 1:  ICHVC
!                      2:  IPROB
!                      3:  IVEC
!                      4:  ICOMPL
!                      5:  IFILL
!                  If IFILL>0, followed by extra information as described in
!                  the programmer's guide
!                  If INPCRE is filled by subroutine CREATE or PRESDF Pos. 1
!                  has a different meaning:
!                  Pos. 1 = -isubr-100*(ivecsq-1) with
!                  isubr sequence number of caling subroutine:
!                      1:  CREATE
!                      2:  PRESDF
!                      3:  CREAVC
!                      4:  PRESTM
!                      5:  ERCRET
!                      6:  ERPRES
!                      7:  ERPRES called from structure
!                      8:  ERCRET called from structure
!                  ivecsq sequence number of first vector
!                  At output inpcre(1) gets the sequence number of the last
!                  vector filled
!                  From position 2 as if filled by the user
!     islold  i    User input array in which the user puts information
!                  of all preceding solutions
!                  Integer array of length 5 x numold
!     isol   i/o   Array(s) to be filled
!     rincre  i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ifirst, ipstin, isubr, ivecsq, kvec, nv

!     ifirst         Value of first position of array INPCRE
!     ipstin         Starting address in array INPCRE
!     isubr          Indicates by what subroutine creatv01 has been called.
!                    Possible values:
!                    1:  By the user or CREATE
!                    2:  By PRESDF or PRESTM
!     ivecsq         Sequence number of vector in isol
!     kvec           Loop variable over the vectors
!     nv             Number of vectors to be created
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCREATSUB     Basis subroutine for the creation of one vector
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1113   First position of INPCRE has wrong value
!    1126   Number of vectors to be created out of range ( <1 or>10000 )
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check input
!     Compute starting addresses of INPCRE, RINCRE
!     Compute nv
!     For kvec := 1 (1) nv
!         Compute ICHVC, IPROB, IVEC, ICOMPL, IFILL
!         Create vector or compute starting addresses
!         If ( IFILL=0 ) then
!            Set vector equal to 0
!         else if ( IFILL>0 ) then
!            For k := 1 (1) IFILL  do
!               Select type of creation
!               Get necessary vectors in memory
!               Fill vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'creatv01' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from creatv01'
         call prinin ( inpcre, 12, 'inpcre' )
         call prinrl ( rincre, 1, 'rincre' )
  1      format ( a, 1x, (10i8) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Check input

      ifirst = inpcre(1)
      ipstin = 2
      if ( ifirst<-10000 .or. ifirst>=10000 ) then
         call errint ( ifirst, 1 )
         call errsub ( 1113, 1, 0, 0 )
      end if
      if ( ifirst<0 ) then

!     --- ifirst<0

         isubr = - ifirst
         ivecsq = isubr/100
         isubr = mod(isubr,100)
         ifirst = 1000000

      else

!     --- ifirst>0

         isubr = 1
         ivecsq = 0

      end if
      if ( debug ) &
         write(irefwr,1) 'isubr, ivecsq, ifirst', isubr, ivecsq, ifirst
      call sepfillkmesh ( kmesh )

!     --- Compute nv (number of calls to create vector)

      if ( debug ) write(irefwr,1) 'ifirst, ipstin', ifirst, ipstin
      if ( ifirst>=ipstin ) then
         nv = inpcre(ipstin)
      else
         nv = 1
      end if
      if ( debug ) write(irefwr,1) 'nv', nv
      if ( nv<=0 .or. nv>10000 ) then
         call errint ( nv, 1 )
         call errsub ( 1126, 1, 0, 0 )
      end if

!     --- Loop over nv vectors

      do kvec = 1+ivecsq, nv+ivecsq

!     --- basis routine for creation

         call ercreatsub ( isol, inpcre, rincre, islold, ifirst, ipstin, &
                           ivecsq, nv, kvec, isubr )
         if ( ierror/=0 ) go to 1000

      end do

1000  if ( inpcre(1)<-4 ) inpcre(1) = ivecsq+nv

      if ( debug ) then

!     --- Debug information

         call prinin1 ( isol, 4, 5, 'isol' )
         write(irefwr,*) 'End creatv01'

      end if
      call erclos ( 'creatv01' )

      end subroutine creatv01
