      subroutine eigenvbf ( iineig, iuser, user, eigvals )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 29-05-2014 Extension of timebefore
!        version  1.3  date 05-12-2012 Long integers
!        version  1.2  date 09-10-1999 Allow variables
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     computes eigenvalues and eigenvectors of
!     Symmetric positive matrices (including the generalized eigenvalue
!     problem)
! **********************************************************************
!
!                       KEYWORDS
!
!     eigenvalue
!     eigenvector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmoduleinput
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iineig(*), iuser(*)
      double precision user(*), eigvals(*)

!     eigvals    o    Array of length neigv where all eigenvalues are stored
!     iineig     i    Input array defined by the user
!                     With this array the user may indicate which actions
!                     he wants to be performed by this subroutine
!                     iineig must be filled as follows:
!                     Pos. 1: Last entry number that has been filled by the
!                             user. If 0 or 1 only defaults are used
!                             All positions 1 to iineig(1) of aray iineig must
!                             have been filled by the user
!                          2: IREAD Defines how the input must be read
!                                   Possible values:
!                                  -1:  No input is required
!                                   0:  All SEPRAN input has been read by
!                                       subroutine SEPSTN until
!                                       END_OF_SEPRAN_INPUT or
!                                       end of file has been found
!                                       This input is used
!                                   1:  The input is read as described for
!                                       EIGENVAL
!                             Default value: -1
!                          3: ISEQNR, This parameter is only used if IREAD=0
!                             It indicates the sequence number of the input
!                             for the eigenvalue computation to be used
!                             Default value: 1
!                          4: IVECTR, sequence number of first eigenvector
!                             vector with respect to array ISOL
!                             Default value: 0
!                          5: numveclc, number of vectors that
!                             have been stored in isol before
!                             Default value: 1
!                          6: neigv, i.e. number of eigenvalues computed
!                             output parameter that is only filled if
!                             iineig(1)>5
!                             Default value: 1
!     iuser      i    Integer user array to pass user information from
!                     main program to subroutine. See STANDARD PROBLEMS
!     user       i    Real user array to pass user information from
!                     main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iread, iseqnr, isubr, ivectr, i, irnarr, iinarr, &
              numveclc, leniin, lenrin, niinp, nrinp, neigv, maxcal, ipos
      integer, allocatable :: iinput(:)
      double precision, allocatable :: rinput(:)
      double precision :: timebefore(2)

!     i              Counting variable
!     iinarr         Infor entry with respect to array IINEIG in subroutine
!                    TO0097
!     iinput         integer input array
!     ipos           Position of input arrays in sepmoduleinput
!     iread          Defines how the input must be read
!                    Possible values:
!                    -1:  No input is required
!                     0:  All SEPRAN input has been read by
!                         subroutine SEPSTN until
!                         END_OF_SEPRAN_INPUT or
!                         end of file has been found
!                         This input is used
!                     1:  The input is read as described for
!                         EIGENVAL
!     irnarr         Infor entry with respect to array RINEIG in subroutine
!                    TO0097
!     iseqnr         Sequence number with respect to the storage in array ISEPIN
!     isubr          Indicates the type of calling subroutine
!     ivectr         Sequence number of first eigenvector in ISOL
!                    If 0, no eigenvectors are computed
!     leniin         Computed length of array IINEIG
!     lenrin         Computed length of array RINEIG
!     maxcal         Maximum number of coefficient inputs read
!     neigv          Number of eigenvalues
!     niinp          Length of integer input array
!     nrinp          Length of real input array
!     numveclc         Number of vectors stored in isol
!     rinput         Real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     INICHINPSOL    Adapt input array with respect to variables
!     READEIGV       Read the input for the eigenvalue subroutine EIGENVAL
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     TO0097         Compute eigenvalues and eigenvectors of a matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2051   No input for computation of eigenvalues found
!    2052   Too many eigenvectors
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array iineig by copying from iineig and extending
!     with default values
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eigenvbf' )
      if ( itime>0 ) call ersettime ( timebefore )

!     --- Fill array inpeig by copying from iineig and extending
!         with default values

      iread  = 0
      iseqnr = 1
      numveclc = 0
      ivectr = 0
      neigv  = 1
      niinp  =12
      nrinp  = 1
      if ( iineig(1)>=2 ) iread  = iineig(2)
      if ( iineig(1)>=3 ) iseqnr = iineig(3)
      if ( iineig(1)>=4 ) ivectr = iineig(4)
      if ( iineig(1)>=5 ) numveclc = iineig(5)
      if ( iineig(1)>=6 ) neigv  = iineig(6)
      isubr = 1
      if ( ioutp>=0 .and. itimloop<=1 ) then

!     --- Pint information

         if ( ivectr==0 ) then
            write(irefwr,100) neigv
100         format ( ' Compute ',i3,' eigenvalues ' )
         else
            write(irefwr,110) neigv, ivectr, ivectr+neigv-1
110         format ( ' Compute ',i3,' eigenvalues and eigenvectors'/ &
                     ' Store eigenvectors as vectors ', i3, ' to ', i3 )
         end if

      end if

      if ( iread==0 ) then

!     --- iread = 0, Input has been read by readal

         call sepgetinpfinf ( 23, iseqnr, ipos, maxcal )
         if ( in(ipos)%used ) then

!        --- Input for eigenvalues equations found

            leniin = size(in(ipos)%iinput)
            lenrin = size(re(ipos)%rinput)

            allocate ( iinput(leniin), rinput(lenrin), stat = error )
            if ( error/=0 ) call eralloc ( error, leniin+lenrin, 'iinput' )
            if ( ierror/=0 ) go to 1000
            iinput = in(ipos)%iinput
            rinput = re(ipos)%rinput

         else

!        --- No input for eigenvalues equations found

            call errsub ( 2051, 0, 0, 0 )
            go to 1000

         end if

      else if ( iread==1 ) then

!     --- iread = 1, Input must be read

         allocate ( iinput(niinp), rinput(nrinp), stat = error )
         if ( error/=0 ) call eralloc ( error, niinp+nrinp, 'iinput' )
         if ( ierror/=0 ) go to 1000
         iseqnr = 1

         call readeigv ( iinput, rinput, isubr, iseqnr, niinp, nrinp )
         leniin = niinp

      else

!     --- iread = 0, Fill default values

         allocate ( iinput(niinp), rinput(nrinp), stat = error )
         if ( error/=0 ) call eralloc ( error, niinp+nrinp, 'iinput' )
         if ( ierror/=0 ) go to 1000

         iinput = 0
         iinput(3) = 1
         rinput(1) = 1d-5
         leniin = 1

      end if

!     --- Adapt variables

      call inichinpsol ( iinput, leniin )

!     --- Compute eigenvalues and eigenvectors

      if ( ivectr>0 ) then
         iinput(1) = 1
      else
         iinput(1) = 0
      end if
      iinput( 9) = neigv
      iinput(11) = numveclc
      iinput(12) = ivectr

      if ( ivectr>0 ) then

!     --- inpeig(1) = 1, compute eigenvectors

         iinput(1) = 1
         if ( ivectr+numveclc-1>maxvc1 ) then

!        --- Too many eigenvectors to store in isolut2d
!            Give warning and exclude computation of eigenvectors

            call errint ( numveclc, 1 )
            call errint ( ivectr, 2 )
            call errint ( maxvc1, 3 )
            call errwar ( 2052, 3, 0, 0 )
            iinput(1) = 0

         end if

      end if

      call to0097 ( iinput, rinput, iuser, user, eigvals )

      if ( iinput(1)>0 ) then

!     --- inpeig(1) = 1, raise numveclc

         numveclc = max(numveclc, ivectr+iinput(11))
         if ( iineig(1)>=5 ) iineig(5) = numveclc

      end if

      deallocate ( iinput, rinput, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iinput' )

1000  call erclmn ( 'eigenvbf', timebefore )
      end subroutine eigenvbf
