      subroutine ercreatstand3 ( nnodes, inpcre, kfun, ipstin, islold, &
                                 kdeg, kvec, isolut, maxphys, mvec, istartvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 22-04-2013 Correction vector loop
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 09-01-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill one vector for one option only
!     kfun>3000, function of old vectors
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, inpcre(*), kfun, ipstin, istartvc, &
              kdeg, kvec, isolut(*), maxphys, mvec, islold(*)

!     inpcre         i    Array containing the integer input
!                         See subroutine creatv01
!     ipstin         i    Starting address in array INPCRE
!     islold         i    User input array in which the user puts information
!                         of all preceding solutionss
!                         Integer array of leng 5 x numold
!     isolut        i/o   Standard SEPRAN array containing information of
!                         the solution vectors (same as isol)
!     istartvc       i    Starting position of sequence numbers of vectors in inpcre
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         0:  Not filled
!                         <0:  Constant value stored in RINCRE(-KFUN)
!                         >0:  Function subroutine
!     kvec           i    Loop variable over the vectors to be created
!     maxphys        i    Maximum number of unknowns per point
!     mvec           i    Number of old vectors to be used in FUNCVECT
!     nnodes         i    Number of nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ivectmp, numarray
      double precision, allocatable :: coorcp(:), uoldcp(:,:), ucopy(:)

!     coorcp         Contains the copy of the coordinates for a set of
!                    nodes
!     i              Counting variable
!     ivectmp        Reduced vector number
!     numarray       Number of indexed vectors in vector array
!     ucopy          array of length nunkp x inpelm containing the values
!                    of the solution vector in the nodal points
!     uoldcp         Contains the values of all vectors needed in all
!                    nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPCOR       Copy coordinates for a set of nodes
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILCRV       Fill array vector in the nodal points stored in array NODES
!     ERFILLVC       Copy the contents of array vector into array uoldcp for all
!                    points in array nodes and all degrees of freedom
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     FUNCVECT       User subroutine to fill a vector as function of old vectors
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatstand3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatstand3'
         call ertrace
         write(irefwr,1) 'maxphys, mvec', maxphys, mvec
         write(irefwr,1) 'nnodes', nnodes
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( coorcp(nnodes*ndim), uoldcp(nnodes*maxphys,mvec), &
                 ucopy(nnodes*nphyso), stat = error )
      if ( error/=0 ) &
          call eralloc ( error, nnodes*(mvec*maxphys+nphyso+ndim), &
                         'nodes' )
      if ( ierror/=0 ) go to 1000

!     --- Clear both arrays

      uoldcp = 0d0
      ucopy = 0d0

!     --- Copy coordinates to coorcp

      if ( debug ) write(irefwr,*) 'before ercopcor'
      call ercopcor ( coor, coorcp, ndim, nnodes, nodes )

!     --- Put old solution vectors in uoldcp

      if ( debug ) write(irefwr,*) 'before loop'
      do i = 1, mvec
         call inireducevec ( inpcre(istartvc+i-1), ivectmp, numarray )
         call sepactsolbf2 ( islold(ivectmp) )
         usoltemp => ks(islold(ivectmp))%sol
         call erfillvc ( usoltemp, nodes, nnodes, uoldcp(1,i) )
      end do  ! i = 1, mvec
      if ( debug ) write(irefwr,*) 'after loop'

!     --- Put the new vector in array ucopy by calling the user
!         subroutine FUNCVECT

      call funcvect ( kfun-3000, ndim, coorcp, nnodes, uoldcp, mvec, &
                      ucopy, maxphys )
      if ( debug ) write(irefwr,*) 'after funcvect'

!     --- Finally copy contents to array vector

      usoltemp => ks(isolut(kvec))%sol
      if ( debug ) write(irefwr,*) 'before erfilcrv'
      call erfilcrv ( usoltemp, nodes, nnodes, &
                      kdeg, inpcre(ipstin+2), ucopy )

      if ( debug ) write(irefwr,*) 'before deallocate'

      deallocate ( coorcp, uoldcp, ucopy, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coorcp' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercreatstand3'

      end if  ! ( debug )
      call erclos ( 'ercreatstand3' )

      end subroutine ercreatstand3
