      subroutine ercreatsub2 ( ichvc, itypvc, new, isolut, &
                               icompl, kvec, isubr, ivec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-13-2013 Extension with change_vector
!        version  1.0  date 27-12-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine type of vector to be created
!     iprob, icompl, ivec may be adapted in case of an old vector
!     new is determined
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulecons
      use sepmoduleplot
      use sepmodulekprob
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isolut(*), kvec, ichvc, ivec, icompl, &
              itypvc, isubr
      logical new

!     ichvc          i    Indication of the type of vector to be created.
!                         Possible values
!                         0: Vector either exists or is a solution vector
!                         1: Vector is a solution vector
!                         2: Vector is an array of special structure
!                         3: Vector is an array of special structure defined per
!                            element
!                         4: Vector is an array of special structure defined per node
!                            per element
!                         5: Capacity vector
!                         6: Existing vector that must not be cleared
!     icompl        i/o   Indication if the vector to be created is real (0) or
!                         complex
!                         May be changed if old vector is used
!     isolut         i    Standard SEPRAN array containing information of
!                         the solution vectors (same as isol)
!     isubr          i    Indicates by what subroutine creatv01 has been called.
!                         Possible values:
!                         1:  By the user or CREATE
!                         2:  By PRESDF or PRESTM
!     itypvc         i    Indication of the type of solution vector:
!                         110: solution vector
!                         115, 116: vectors of special structure
!     ivec          i/o   Sequence number of vector of special structure
!                         May be changed if old vector is used
!     kvec           i    Sequence number over the vectors
!                         Loop parameter in call ing subroutine
!     new            o    If true the vector to be created is new, &
!                         otherwise an old vector is adapted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jcompl, jdegfd, jprob, jvec

!     jcompl         If 0 then the vector is real otherwise (1) it is
!                    complex
!     jdegfd         Number of degrees of freedom per point
!     jprob          Value of iprob as stored in isolut at input
!     jvec           Value of ivec as stored in isolut at input
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatsub2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatsub2'
         write(irefwr,1) 'itypvc, kvec, ichvc', itypvc, kvec, ichvc
         write(irefwr,1) 'isolut(kvec)', isolut(kvec)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ichvc==0 ) then

!     --- ichvc = 0, Either old vector or solution vector

         if ( itypvc/=110 .and. itypvc/=115 .and. &
              itypvc/=116 .and. itypvc/=126 ) then

!        --- New vector

            itypvc = 110
            new    = .true.

         else if ( isolut(kvec)<=0 .or. &
                   isolut(kvec)>lastsol ) then

!        --- New vector

            itypvc = 110
            new    = .true.

         else

!        --- Old vector

            new = .false.

         end if

      else if ( ichvc==1 ) then

!     --- ichvc = 1, solution vector

         if ( itypvc/=110 .or. isolut(kvec)<=0 .or. &
              isolut(kvec)>lastsol )  then

!        --- New vector

            itypvc = 110
            new    = .true.

         else

!        --- Possibly old vector

            jprob  = ks(isolut(kvec))%iprob
            jcompl = ks(isolut(kvec))%compl
            if ( isubr==7 ) then

!           --- call from structure, copy info from old vector

               new = .false.

            else if ( iprob/=jprob .or. icompl/=jcompl ) then
               new = .true.
            else if ( nusol/=ks(isolut(kvec))%nusol ) then
               new = .true.
            else
               new = .false.
            end if

         end if

      else if ( ichvc==2 ) then

!     --- ichvc = 2, vector of special structure

         if ( itypvc/=115 .or. isolut(kvec)<=0 .or. &
              isolut(kvec)>lastsol )  then

!        --- New vector

            itypvc = 115
            new    = .true.
            if ( ivec==0 ) itypvc = 110  ! use solution vector if ivec = 0

         else

!        --- Possibly old vector

            jprob  = ks(isolut(kvec))%iprob
            jcompl = ks(isolut(kvec))%compl
            jvec   = ks(isolut(kvec))%ivec
            if ( isubr==8 ) then

!           --- call from structure, copy info from old vector

               new = .false.

            else if ( iprob/=jprob .or. icompl/=jcompl .or. &
                      jvec/=ivec ) then
               new = .true.
            else
               new = .false.
            end if

         end if

      else if ( ichvc==3 ) then

!     --- ichvc = 3, vector of special structure per element

         if ( itypvc/=116 .or. isolut(kvec)<0 .or. &
              isolut(kvec)>lastsol )  then

!        --- New vector

            itypvc = 116
            new = .true.

         else

!        --- Possibly old vector

            jprob  = ks(isolut(kvec))%iprob
            jcompl = ks(isolut(kvec))%compl
            jvec   = ks(isolut(kvec))%ivec
            if ( iprob/=jprob .or. icompl/=jcompl .or. &
                 jvec/=ivec ) then
               new = .true.
            else
               new = .false.
            end if

         end if

      else if ( ichvc==4 ) then

!     --- ichvc = 4, vector of special structure per element per node

         if ( itypvc/=126 .or. isolut(kvec)<0 .or. &
              isolut(kvec)>lastsol )  then

!        --- New vector

            itypvc = 126
            new = .true.

         else

!        --- Possibly old vector

            jprob  = ks(isolut(kvec))%iprob
            jcompl = ks(isolut(kvec))%compl
            jvec   = ks(isolut(kvec))%ivec
            if ( iprob/=jprob .or. icompl/=jcompl .or. &
                 jvec/=ivec ) then
               new = .true.
            else
               new = .false.
            end if

         end if

      else if ( ichvc==5 ) then

!     --- ichvc = 5, vector with capacities

         if ( itypvc/=801 .or. isolut(kvec)<0 .or. &
              isolut(kvec)>lastsol )  then

!        --- New vector

            itypvc = 801
            new = .true.

         else

!        --- Possibly old vector

            jprob  = ks(isolut(kvec))%iprob
            jcompl = ks(isolut(kvec))%compl
            jvec   = ks(isolut(kvec))%ivec
            nphys = ks(isolut(kvec))%nphys
            jdegfd = ks(isolut(kvec))%nusol/jvec
            if ( iprob/=jprob .or. icompl/=jcompl .or. &
                 jvec/=ivec .or. jdegfd/=nphys ) then
               new = .true.
            else
               new = .false.
            end if

         end if

      else if ( ichvc==6 ) then

!     --- ichvc = 6, old vector

         new = .false.

      end if

      if ( debug ) write(irefwr,*) 'new ', new

      if ( .not. new ) then

!     --- old vector, compute iprob, ivec and icompl

         iprob  = ks(isolut(kvec))%iprob
         icompl = ks(isolut(kvec))%compl
         ivec   = ks(isolut(kvec))%ivec
         jcart  = ks(isolut(kvec))%jcart

      end if

      nphyso = nphys

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercreatsub2'

      end if  ! ( debug )
      call erclos ( 'ercreatsub2' )

      end
