      subroutine erfillquadsur ( nnodes, nodes, rincre, kfun, &
                                 kdeg, vector, numpoints, &
                                 curvenodes, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 06-04-2005 Extra debug information
!        version  2.0  date 15-10-2004 Extension with part of circle
!        version  1.0  date 17-03-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill boundary condition of the shape quadratic function with
!     given mean value and band width along a surface
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
!     quadratic
!     polynomial
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(*), kfun, kdeg, numpoints, &
              curvenodes(*), isurnr
      double precision rincre(*), vector(*)

!     curvenodes     i    Contains the nodes of the boundary curve
!     isurnr         i    Surface sequence number
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         9001:  Fill quadratic boundary condition along curve
!                         9002:  Fill half-quadratic boundary condition along
!                                curve
!                         9003:  Fill quadratic boundary condition with
!                                boundary layer along circular surface
!                         9004:  Fill quadratic boundary condition with
!                                boundary layer and temperature difference
!                                along circular surface
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Contains the node numbers that must be filled
!     numpoints      i    Number of points in icurnr
!     rincre         i    Double precision array containing the values of the
!                         constants corresponding to the boundary condition
!                         Pos. 1:  mean value
!                         Pos. 2:  boundary layer thickness
!                         Pos. 3:  difference between maximum value and value at
!                                  the boundary
!     vector        i/o   Array to be filled with the boundary condition
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision origin(3), radius, delta, mean_val, dif_val

!     delta          boundary layer thickness
!     dif_val        difference between maximum value and value at the boundary
!     mean_val       mean value of the boundary condition
!     origin         Contains the coordinates of the origin of the circle
!     radius         Radius of sphere or circle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLQUADSUR1 Fill boundary condition of the shape quadratic function
!                    with given mean value and boundary layer width along a
!                    surface
!     ERFILLQUADSUR2 Fill boundary condition of the shape quadratic function
!                    with given mean value, difference between minimum and
!                    maximum value and boundary layer width along a surface
!     ERFILLRAD      Compute centroid and radius of circle in 3d
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     (o_x,o_y,o_z) :=  centroid of circle
!     Radius := radius of circle
!     Check if region is a circle
!     delta := boundary_layer_width
!     if ( delta = 0 ) then
!        delta := radius
!     if ( kfun = 9003 ) then
!        Fill quadratic profile with zero boundary conditions
!     else ( kfun = 9004 )
!        Fill quadratic profile using difference
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillquadsur' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillquadsur'
         call prinin ( nodes, nnodes, 'nodes' )
         call prinrl ( rincre, 3, 'rincre' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- (o_x,o_y,o_z) :=  centroid of circle
!         Radius := radius of circle
!         Check if region is a circle

      call erfillrad ( nnodes, nodes, coor, origin, radius, numpoints, &
                       curvenodes, isurnr )
      if ( debug ) then
         write(irefwr,*) 'radius ', radius
         call prinrl ( origin, 3, 'origin' )
      end if  ! ( debug )

!     --- delta := boundary_layer_width
!         if ( delta = 0 ) then
!            delta := radius

      mean_val = rincre(1)
      delta = rincre(2)
      if ( delta<0d0 .or. delta>radius ) delta = radius

      if ( kfun==9003 ) then

!     --- kfun = 9003: Fill quadratic profile with zero boundary conditions

         call erfillquadsur1 ( nnodes, nodes, origin, radius, &
                               mean_val, delta, kdeg, vector )

      else

!     --- kfun = 9004: Fill quadratic profile using difference

         dif_val = rincre(3)
         call erfillquadsur2 ( nnodes, nodes, origin, radius, &
                               mean_val, delta, kdeg, vector, dif_val )

      end if  ! ( kfun==9003 )

1000  call erclos ( 'erfillquadsur' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillquadsur'

      end if

      end
