      subroutine ergetifill ( inpcre, ifill, ipstin, ifirst, &
                              obstaclefound )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 14-07-2011 Extension with kfill = 22
!        version  1.2  date 16-08-2004 Boundary conditions for levelset
!        version  1.1  date 10-05-2004 Correction quadratic boundary conditions
!
!   copyright (c) 2000-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if values must be given in obstacles
!     If so set obstaclefound true
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     essential_boundary_condition
!     create
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ifill, ipstin, ifirst, inpcre(*)
      logical obstaclefound

!     ifill          i    Indicates how the vector must be filled.
!                         Possible values:
!                         -1  The corresponding vector is not filled, &
!                             only created
!                          0  The corresponding vector is created and made
!                             equal to zero
!                         m>0 The filling of the vector is defined by m commands
!     ifirst         i    Value of first position of array INPCRE
!     inpcre         i    Integer input array for subroutine creatvbf
!     ipstin         i    Starting address in array INPCRE
!     obstaclefound  o    If true values must be given in points in an obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kfill, ipstinloc, jpstin, k, kdeg, kfun, mvec, n

!     ipstinloc      Local value of ipstin
!     jpstin         Help variable to compute new value of ipstin partly
!     k              Counting variable
!     kdeg           Indication which degrees of freedom must be filled
!                    Possibilities:
!                    0:    All degrees of freedom
!                    >0:   Only the KDEGth degree of freedom
!                    <0:   The -KDEG degrees of freedom stored in array
!                          IARDEG must be filled
!     kfill          Indication how the nodal points must be filled into
!                    array NODES
!                    Possible values:
!                     0:  The nodes are filled in all points
!                     1:  The nodes are filled in n user points
!                     2:  The nodes are filled in the curves C1 to C2
!                     3:  The nodes are filled in the surfaces S1 to S2
!                     5:  The nodes are filled in the elements i to j
!                     6:  The nodes are filled in the element group ielgrp
!                     7:  The nodes are filled in the volumes V1 to V2
!                     8:  The nodes are filled in the user points P1 to P2
!                     9:  The nodes are filled in the active points of
!                         obstacle i
!                    10:  The nodes are filled in the active points of
!                         contact i
!                    11:  The nodes are filled in the non-active points
!                         of contact i
!                    12:  The nodes are filled using a file with node
!                         numbers
!                    13:  The nodes are filled using a file with elements
!                    14:  The nodes are filled using a file with capacities
!                    15:  The global unknowns are filled for global group
!                         iglgrp
!                    16:  All nodes in the obstacle are filled
!                    17:  Only the inner nodes in the obstacle are filled
!                    18:  Only the boundary nodes in the obstacle are filled
!                    19:  Only the boundary nodes on the obstacle are filled
!                    20:  Only the boundary nodes on the boundary with
!                         zero level set are filled
!     kfun           Indication of how the degrees of freedom must be
!                    filled.
!                    Possibilities:
!                    0:  Not filled
!                    <0:  Constant value stored in RINCRE(-KFUN)
!                    >0:  Function subroutine
!     mvec           Number of old vectors to be used in FUNCVECT
!     n              Number of commands
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ergetifill' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ergetifill'
         call ertrace
         write(irefwr,1) 'ipstin, ifirst, ifill', ipstin, ifirst, ifill
         call prinin ( inpcre, 20, 'inpcre' )
  1      format ( a, 1x, (10i8) )

      end if
      if ( ierror/=0 ) go to 1000

      ipstinloc = ipstin
      obstaclefound = .false.
      do k = 1, ifill

!     --- i=ipstinloc

         jpstin = ipstinloc + 1

!        --- i+1

         kdeg   = 0
         kfun   = 0
         kfill  = 0
         if ( ifirst>=jpstin ) kdeg   = inpcre(jpstin)
         if ( kdeg<0 ) jpstin = jpstin - kdeg
         if ( ifirst>=jpstin+1 ) kfun   = inpcre(jpstin+1)
         if ( debug ) write(irefwr,1) 'k, jpstin, kdeg, kfun', &
            k, jpstin, kdeg, kfun
         if ( kfun>=9001 .and. kfun<=9004 ) then

!        --- kfun = 9001 ... 9004, quadratic function along curves

            jpstin = jpstin+1

         else if ( kfun>3000 ) then

!        --- kfun>3000, function of old vectors

            mvec     = inpcre(jpstin+2)
            jpstin = jpstin+mvec+1

         end if
         if ( ifirst>=jpstin+2 .and. kfun>=9001 .and. kfun<=9004 ) &
            kfill  = inpcre(jpstin+2)
         if ( debug ) write(irefwr,1) 'jpstin, kfill', jpstin, kfill
         jpstin = jpstin + 3

!        --- i+4

         if ( kfill==1 ) then
            n = 1
            if ( ifirst>=jpstin ) n = inpcre(jpstin)
            jpstin = jpstin + 1

!           --- i+5

         end if

!        --- raise ipstinloc

         ipstinloc = jpstin - 1

!        --- i+4

         if ( kfill==1 ) then
            ipstinloc = ipstinloc + 1 + n

!           --- i+6

            ipstinloc = ipstinloc - 1

!           --- i+5, 5 items per function when n=1, are other cases OK ?

         else if ( kfill==2 .or.kfill==4 .or.kfill==8 ) then
            ipstinloc = ipstinloc + 3
         else if ( kfill==3 .or.kfill==5 .or.kfill==7 ) then
            ipstinloc = ipstinloc + 3 + inpcre(ipstinloc+3)
         else if ( kfill==6 .or. kfill==22 ) then
            ipstinloc = ipstinloc + 2 + inpcre(ipstinloc+2)
         else if ( kfill>=16 .and. kfill<=20 ) then
            obstaclefound = .true.
            ipstinloc = ipstinloc + 1
         else if ( kfill>=9 ) then
            ipstinloc = ipstinloc + 1
         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'obstaclefound', obstaclefound
         write(irefwr,*) 'End ergetifill'

      end if
      call erclos ( 'ergetifill' )

      end
