      subroutine ervl20 ( n, icheig, eval, cmat, &
                          smas, m, restrt, alpha, beta, maxnev, nev, &
                          ev, maxnx, nx, mlow, &
                          low, nlow, minit, eveps, niter, ieraut )
! ======================================================================
!
!        programmer    Zdenek Vlasek
!        version  1.10 date 13-05-2010 Fortran 90
!        version  1.9  date 21-09-1992 Bug when nonexistent x(im)
!        version  1.8  date 17-03-1991 New call of ervl00
!
!   copyright (c) 1986-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!        The Lanczos process is performed for the matrix A, where
!           A = inv(smas) * cmat * inv(smas)
!        matrix  cmat  must be symmetric,
!        matrix  smas  must be symmetric positive definite diagonal
!        In the Lanczos process , applied on the matrix A,
!        a tridiagonal matrix T is calculated
!        ( here, T is represented by alpha(*)  and  beta(*) )
!        and its eigenvalues bear some relation to those of A.
!        This routine implements the Parlett+Reid's heuristic method
!        of control of the Lanczos process. The localization of the
!        eigenvalues is updated after each Lanczos step .
!        This routine detects those eigenvalues of the matrix T
!        that are very close to eigenvalues of A , with given relative
!        ( with respect to norm2 of A ) precision  eveps
!   remarks !!
!   ervl20  should determine eigenvalue intervals, which represent
!           the eigenvalues of  A
!   Caution !! Form the nature of the generalized Lanczos process
!           and the P+R control process is clear, that some other
!           (especially those very close to the determined ones)
!           eigenvalues may remain undiscovered.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, m, ieraut, icheig(4), maxnev, nev, maxnx, &
              nx, niter, mlow, nlow, minit(maxnev)
      double precision alpha(m), beta(m), &
                       smas(*), eval(mlow), ev(maxnev,2), &
                       eveps
      logical restrt, low
      type (rmatrix) :: cmat

!     alpha            Scratch-array.
!                      alpha (*)  is the diagonal of the matrix  T
!     beta             double beta(m-1).  Scratch-array.
!                      beta (*)  contains superdiagonal elements of
!                      the matrix  T
!                        alpha(1)  beta (1)  0         0
!                        beta (1)  alpha(2)  beta (2)  0
!                        0         beta (2)  alpha(3)  beta (3)
!     cmat       i    The elements of the matrix C,
!                     stored by the Sepran's jmetod=5
!     ev         o    ev(maxnev,2).  The pair  ev(k, 1..2)  represents
!                     the k-th of  nev  intervals, each of them contains
!                     an eigenvalue of the matrix  A
!     eval       o    Smallest min(mlow,nlow) eigenvalues
!     eveps      i    Maximum relative error, an exact eigenvalue
!                     of A should be in the  eveps * norm2(A)
!                     neighbourhood of the middle point of the interval
!                     [ ev(i,1), ev(i,2) ], where  i = 1, nev
!     icheig     i    integer array
!                     1):  not used
!                     2):  indication whether an information about the
!                          iteration process must be printed (1) or not (0),
!                          or demand for detailed debugging printing (2)
!                     3):  not used
!                     4):  indication how to fill the initial vector in the
!                          Lanczos process, for different initial vectors
!                          the e.v. intervals appear in a different order.
!                           =0 :       vj(i) = i
!                          otherwise: random vj(*) in the range [-1,1],
!                                     abs(icheig(4)) becomes a seed
!     ieraut     o    Error indication.
!                     ieraut = 0  Problem was solved:  at least
!                                 mlow  lovest e.v. intervals found.
!                                 The restart is possible.
!                     ieraut = 1  Less then  mlow  lovest e.v. found.
!                                 It is possible to restart this
!                                 subroutine with bigger  m  or
!                                 with new values  eveps,  mlow.
!                     ieraut = 2  Overflow of  maxnev.  List of the
!                                 eigenvalue intervals in not finished.
!                                 Restart is possible.
!                     ieraut = 3  Too small  eveps  with respect to
!                                 round-off error estimation.
!                                 Restart is not possible.
!                     ieraut = 4  Overflow of  maxnx.
!                                 Restart is possible.
!                     ieraut = 5  Attempt to restart, but the restart
!                                 is not possible.
!                                 Probably the first call of ervl20.
!                                 No results.
!                     ieraut = 6  ervl00  has returned  beta(j) = 0.0.
!                                 No results.
!                     ieraut = 7  mlow>n or mlow<1 . No results.
!     low        o    The smallest eigenvalue was located
!     m          i    Upper bound for number of iterations in the
!                     Lanczos process
!     maxnev     i    Number of rows in the actual declaration
!                     of the array  ev(maxnev,2)
!     maxnx      i    Declared number of elements in the array  x .
!                     The value recomended by P+R page 147 (iv) is
!                     maxnx = 3 * maxnev,  4 * maxnev is more safe
!     minit      o    The value  minit(j)
!                     tells that j-th eigenvalue interval was located
!                     after minit(j)-th iteration, j=1..nev
!     mlow       i
!     n          i    The number of rows and the number of columns
!                     of the original matrix C , on which
!                     the Lanczos process should be applied
!     nev        o    The number of detected eigenvalue intervals
!     niter      o    ast iteration of the generalized Lanczos
!                     process, which was actualy used
!     nlow      i/o   input: Number of the smallest eigenvalues that should
!                     be located
!                     output: Number of smallest eigenvalues which where
!                     actually located
!     nx         i    The currently occupied part of the array  x(*)
!                     is  x(1..nx).  This value can be used for a rough
!                     estimation of a minimum value of  maxnx
!     restrt     i    if .false. then the process starts from the
!                     beginning ( as on the first call) .
!                     if .true. then the Lanczos process should use
!                     existing part of the tridiagonal matrix T and
!                     all intermediate results.
!                     Also the P+R control process should restart, but
!                     only if the precision  eveps  was not changed.
!                     For this restart, the subroutine ervl20 keeps some
!                     values in the saved local variables
!     smas       i    The diagonal, stored as the r-h-side vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nnzer, iersub, &
              ishift, ii, im, im1, i1, i2, ifirst, inew, ier, &
              k, k1, k2, nij, nn(4), nsame, nfix, &
              npol, nzer, nzerg, igap, ngap01, mvl00
      integer :: ni(maxnx), jit(maxnev)
      double precision aplus, aminus, sq, ro, th1, th2, zer, pol, amu, &
                       delj, dd(4), xex, temp, dis, anorm, &
                       erbnd
      logical lprint, deb, solved, existm, gap, gaps01, cancel, epilog
      integer          jdonpa, jdone, noerbn, nxit
      double precision :: x(maxnx), vj(n), vm(n), u(n), del(maxnx), &
                          uw(n), xit(maxnev)
      double precision :: oldeps, epsrel, tol, tolc

!     aminus
!     amu
!     anorm
!     aplus
!     cancel     whether during this update the "epilog" stage was canceled
!                ( see P+R top of page 147 )
!     dd         (1..4)      corresponding values of     del(j,x)
!     deb
!     del             del(maxnx).  Scratch-array for P+R process
!     delj
!     dis
!     epilog     whether the control process is in the "epilog" stage,
!                when each gap should pass the empty phase before
!                the process can ( arbitrary ) deside that new e.v.
!                intervals will not appear,
!                see P+R, top of the page 147
!     epsrel     tolerance
!     erbnd
!     existm     whether point x(im) in step  j-1  exists
!     gap        whether (x(i),x(i+1))  is in gap or not
!     gaps01     whether the sequence of the lowest gaps
!                with 0 or 1 Ritz(j) values still lasts
!     i          current position in list  x(*)  for the step  j-1
!     i1         neighbouring subscripts in step  j-1
!     i2         neighbouring subscripts in step  j-1
!     ier
!     iersub
!     ifirst
!     igap       subscript of the initial point of the
!                current last gap in step  j
!     ii
!     im         neighbouring subscripts in step  j-1
!     im1
!     inew
!     ishift
!     j          current number of the Lanczos step
!     jdone      how many P+R steps was done and hence
!                need not be repeated after the restart
!     jdonpa     how many P+R steps was dohow many ervl00 steps was done and
!                hence need not be repeated after the restart
!     jit        jit(maxnev).  Scratch array for calculation
!                of  minit(*)
!     k
!     k1
!     k2
!     lprint
!     mvl00      Parameter m for ervl00
!     nfix
!     ngap01     number of lowest gaps with 0 or 1 Ritz(j) value
!     ni         ni(maxnx).  Scratch-array for P+R process
!                ni(*)  stores values of the P+R's function "ni", but
!                  +1  is added to each value, otherwise the proposed
!                flagging ( bottom of page 137 ) will fail for  ni=0,
!                so here  ni = +- ( P+R's ni + 1 )
!     nij
!     nn         (1..4)      corresponding values of  +- (ni(j,x)+1)
!     nnzer      The number of nonzeros in the matrix A.
!                It is used for error-estimation.
!     noerbn     number of P+R steps during which the error bound
!                estimation procedure ervl24 shall not be called,
!                see P+R at the finish of the section 5.1
!     npol
!     nsame      number of last points in step  j  which have equal  abs(ni),
!                see P+R bottom of section 3
!     nxit
!     nzer
!     nzerg
!     oldeps     old value of  eveps,  if  eveps<>oldeps  during
!                the restart, then P+R process cannot restart,
!                whereas ervl00 may be restarted
!     pol
!     ro
!     solved
!     sq
!     temp
!     th1
!     th2
!     tol        tolerance
!     tolc       tolerance
!     u          Scratch-array
!     uw         Scratch-array
!     vj         work array
!     vm         Scratch-array
!     x          double x(maxnx).  Scratch-array for P+R process, it
!                it used for separation and localization of
!                eigenvalues of the matrix  T.
!     xex
!     xit        xit(maxnev).  Scratch array for calculation
!                of  minit(*).  The arrays jit(*), xit(*) store the
!                iteration numbers and midpoints of e.v. intervals,
!                in the chronological order.
!     zer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     ERVL00    carry out lanczos process for the e.v. problem
!     ERVL22    evaluates the functions  delta(j,x), ni(j,x),
!               for a new value  x,  see P+R (3.1), (3.2);
!               to enable flagging by minus sign ( as proposed
!               in P+R bottom of p.139 ) the value  ni  here
!               is  1 + P+R's ni
!     ERVL23    updating from  delta (j-1, x) ,  ni (j-1, x)
!                        to    delta (j, x) ,    ni (j, x),
!               see P+R (3.1), (3.2);
!               the value  ni  here  is  +- ( 1 + P+R's ni),
!               the updating preserves the sign
!     ERVL24    error bound for an approximate eigenvalue
!               of the matrix  A  , P+R part 4
!     ERVL25    3-point interpolation by (2,1) rational
!               function with a given range for one of zeros
!               and optionally also a given range for the pole
!               (see P+R (3.7) )
!     ERVL26    2-point interpolation by (2,1) rational
!               function with a given pole, with given range
!               for one of the zeros, see P+R (3.7) and below
!     INSTOP    Stop the program
!     PRANDOM        generates a random vector in parallel
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1403   An internal error in SEPRAN, in the eigenvalue package ervl20
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Lanczos process:
!        the subroutine ervl00, adapted lsvlan from ACCULIB is used
!
!   control process:
!        B.N.Parlett and J.K.Reid:
!            Tracking the Progress of the Lanczos Algorithm
!                         for Large Symmetric Eigenproblems,
!            IMA Journal of Numerical Analysis (1981) 1, 135-155
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data jdone /0/, jdonpa/0/, oldeps/1d0/
! ======================================================================
!
      call eropen ( 'ervl20' )
      debug = .false.! .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ervl20'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lprint = icheig(2)>=1

!     --- debug listings ?

      deb = icheig(2)==2 .or. debug

      if ( mlow<1 .or. mlow>n ) then
        ieraut = 7
        goto 4000
      end if

!     --- number of nonzero elements in the subdiagonal part of C
!         (jmetod=5)

      nnzer = n + 2*irowsii(n+1)

      if ( debug ) write( irefwr,1) 'iprint, nnzer', iprint, nnzer

!     --- initial values of j, nlow, nev, jdone, jdonpa
!         prepare  j, nlow, nev  for "goto return"

      j = 0
      nlow = 0
      nev = 0
      if ( .not. restrt ) then

!     --- both P+R and (gen.) Lanczos start from the beginning

        jdone = 1
        jdonpa = 1

!       --- fill initial vector vj(*)

        if ( icheig(4)==0 ) then

          do i = 1, n
            vj(i) = i
          end do

        else

!       --- fill vj(*) by random numbers from [-1,1]
!           re-initialize the random number generator

           call prandom ( n, vj )
           vj = 1d0 - 2d0*vj

        end if

      else

!     --- use the old values whenever possible
!         ( it should be  1<=jdonpa<=jdone )

        if ( 1>jdonpa .or. jdonpa>jdone ) then

!       --- probably the first call of ervl20

          ieraut = 5
          goto 4000
        end if

!       --- start P+R process from the beginning ?

        if ( abs(eveps-oldeps)>0d0 .or. m<jdonpa ) jdonpa = 1

      end if

!     --- store  eveps  for a future restart

      oldeps = eveps

!     --- check the minimum value of  maxnx  for the case j=2

      if ( maxnx<4 ) goto 1300

      if ( jdone==1 ) then

!     --- ervl00 for  j=1  and  j=2

        mvl00 = 2
        call ervl00 ( n, 1, mvl00, .false., vj, u, vm, uw, &
                      alpha, beta, cmat, smas )
        if ( debug ) then
           call prinrl ( alpha, mvl00, 'alpha' )
           call prinrl ( beta, mvl00, 'beta' )
        end if  ! ( debug )

!       --- the main results:  alpha(1..2),  beta(1..2)

        if ( abs(beta(1))<=0d0 .or. abs(beta(2))<=0d0 ) then
          ieraut = 6
          jdone = 1
          goto 3000
        end if

        jdone = 2

      end if

      if ( jdonpa==1 ) then

!     --- initialize P+R process
!         see P+R top of page 140
!         calculate Ritz values for j=2 ( quadratic equation ),
!         use the matrix  | alpha(1)  beta (1) |
!                         | beta (1)  alpha(2) |

        aplus  =    (alpha(1)+alpha(2)) * 0.5d0
        aminus = abs(alpha(1)-alpha(2)) * 0.5d0
        sq = aminus**2 + beta(1)**2
        ier = 1
        if ( sq<=0d0 ) goto 3900

!       --- there is  sq>0

        sq = sqrt(sq)
        ro = (sq-aminus) * 0.5d0

!       --- Ritz values for step j=2 are  th1, th2

        th1 = aplus - sq
        th2 = aplus + sq

!       --- intervals [x(1),x(2)],  [x(3),x(4)] around th1, th2
!           when x(1), x(2) are symmetric around th1, then eventual
!              bisection will make troubles due to  del (2, th1) = 0, &
!              that is why a small nonsymmetry is introduced here

        x(1) = th1 - ro
        x(2) = th1 + 0.99d0 * ro
        x(3) = th2 - ro
        x(4) = th2 + 0.99d0 * ro

!       --- number of points in  x(*), see P+R (3.3)

        nx = 4
        j = 2

!       --- initialize also  del(*), ni(*), &
!              as defined in (3.1), (3.2)

        do i = 1, nx
          call ervl22 ( alpha, beta, j, x(i), del(i), ni(i) )
        end do

!       --- first values of  anorm, tol, tolc,  see P+R (3.11)

        anorm = max ( abs ( x(1) ), abs ( x(nx) ) )

!       --- maximal relative error ( it does not depend on  j )
!              taking into account also the machine accuracy and  n, &
!              see P+R the first term in (3.11)

        epsrel =  max ( eveps, 2 * n * epsmac )

!       --- maximal absolute error

        tol = epsrel * anorm
        tolc = tol

        if ( deb ) write (irefwr, 990 ) j, anorm, epsrel, tol, tolc
  990   format ( / '    j   anorm     epsrel      tol       tolc', &
          /    i5,  4e10.3 / )

!       --- initialize  noerbn  ( skips of call ervl24 ), P+R part 5.1

        noerbn = 0

!       --- other initializations

        nxit = 0
        jdonpa = 2
        epilog = .false.

      end if

!     --- now it is  2<=jdonpa<=jdone
!         loop: ervl00 and P+R for the steps j>=3

      do j = jdonpa + 1, m

!     --- the last turn (j=m) is also a reason for the epilog stage, &

        if ( j==m ) epilog = .true.

        if ( deb .and. epilog ) write (irefwr, *) ' e p i l o g'

        if ( j>jdone ) then

!       --- do one Lanczos step, from  j-1  to  j

          mvl00 = j
          call ervl00 ( n, j, mvl00, .true., vj, u, vm, uw, &
                        alpha, beta, cmat, smas )

!         --- the main results are  alpha(j),  beta(j)

          if ( abs(beta(j))<=0d0 ) then
            ieraut = 6
            jdone = j - 1
            goto 3000
          end if

        end if

!       --- P+R control process, the progress from step  j-1  to step  j
!           reduce the  " no call ervl24 "  counter:  P+R finish of 5.1

        noerbn = noerbn - 1

!       --- extend the set x(*) on the right, if needed
!           ( something like x(nx) = +infinity  is bad in interpolations )
!           we have  x(1..nx),  del(1..nx),  ni(1..nx)  from step  j-1
!             ( ni(*)  is  the P+R's value  ni(j-1,x)+1  with eventual
!               minus sign,  the additive +1 allowes us to flag also  0 )
!           ni(nx)  in the step  j  should be equal to j+1, &
!             only then no Ritz(j) value is on the right of  x(nx)

!           --- store step  j-1  values

 40       delj = del(nx)
          nij = ni(nx)

!         --- update  delj, nij  from step  j-1  to step  j

          call ervl23 ( alpha, beta, j, x(nx), delj, nij )

          if ( nij==j+1 ) goto 50

!         --- there is at least one Ritz(j) value on the right of x(nx)
!             10% extension of the spectrum

          if ( nx>=maxnx ) goto 1300
          x(nx+1) = x(nx) + 0.1d0 * ( x(nx)-x(1) )
          nx = nx + 1

!         --- calculate  del, ni  for this new x(nx), for step  j-1

          call ervl22 ( alpha, beta, j-1, x(nx), del(nx), ni(nx) )
        goto 40

!       --- now all  Ritz(j)  values are<=x(nx)
!           shift the set  x(1..nx)  to the right
!           to the end of the array x(*)

 50     ishift = maxnx - nx
        do i = nx, 1, -1
          inew = i + ishift
          x(inew) = x(i)
          del(inew) = del(i)
          ni(inew) = ni(i)
        end do
        ifirst = inew

!       --- now the subscript range for step  j-1  is  ifirst..maxnx
!           extend the set x(ifirst..maxnx) on the left, if needed
!           abs (ni(ifirst))  in step  j  should be equal to 1, &
!              only then no Ritz(j) value is on the left of  x(ifirst)
!           store step  j-1  values

 70       delj = del(ifirst)
          nij = ni(ifirst)

!         --- update  delj, nij  from step  j-1  to step  j

          call ervl23 ( alpha, beta, j, x(ifirst), delj, nij )

          if ( abs (nij)==1 ) goto 80

!         --- there is at least one Ritz(j) value on the left, &
!             10% extension of the spectrum

          if ( ifirst<=1 ) goto 1300
          x(ifirst-1) = x(ifirst) - 0.1d0 * ( x(maxnx)-x(ifirst) )
          ifirst = ifirst - 1

!         --- calculate  del, ni  for this new x(ifirst), for step  j-1

          call ervl22 ( alpha, beta, j-1, &
                       x(ifirst), del(ifirst), ni(ifirst) )
        goto 70

!       --- now no Ritz(j) value is on the left of  x(ifirst)
!           value  nx  from step  j-1  is no more valid
!           initialize the updating of  x(*),  del(*),  ni(*)
!                                from step  j-1  to step  j
!           step  j-1  now occupies positions  ifirst .. maxnx
!           step  j    will occupy positions        1 .. new nx
!           update norm2 of the matrix  A  and update  tol, &
!              see P+R (3.11)

 80     anorm = max ( abs(x(ifirst)), abs(x(maxnx)) )
        tol = epsrel * anorm

!       --- tolc  is as it was for  j=2  or as was reduced last time
!           use the first two points from step  j-1  to initialize
!              the arrays  dd(3..4), nn(3..4);  they will be shifted
!              to the  dd(2..3),  nn(2..3)  in loop 110

        i = ifirst
        do  ii = 3, 4
          dd(ii) = del(i)
          nn(ii) = ni(i)
          call ervl23 ( alpha, beta, j, x(i), dd(ii), nn(ii) )

!         --- now nn(*) belong to the step j

          i = i + 1
        end do

!       --- initialize  nx,  x(1),  del(1),  ni(1)  for step  j

        if ( ifirst<2 ) goto 1300
        nx = 1
        x(1) = x(ifirst)
        del(1) = dd(3)
        ni(1) = nn(3)

!       --- initialize inspection of gaps between e.v. intervals
!              ( the first interval must not be an e.v. interval )
!              see P+R section 5.3 for the inspection

        igap = 1
        gap = .true.
        gaps01 = .true.
        ngap01 = 0
        cancel = .false.

!       --- initialize the counter of absolute values equal to  ni(nx)

        nsame = 1

!       --- loop: update intervals  ( x(i), x(i+1) ],  i=ifirst, &
!           outer loop  goto 100  increases the value  i
!           inner loop  goto 120  decreases the value  i  (recursion)
!           jump        goto 200, goto 500  leaves the inner loop
!           jump        goto 600            leaves the outer loop

!           initialize the control variable of both loops

        i = ifirst

!       --- existence of  x(i-1) alias x(im)

        existm = .false.

!       --- the outer loop: do something with ( x(i), x(i+1) )
!           for possible interpolation in the range [x(i-1), x(i+2)]
!              values of both  step  j-1  and step  j  may be needed
!           del(i-1..i+1)  and  ni(i-1..i+1)  for step  j  are
!              already in  dd(2..4), nn(2..4)

100       im = i - 1
          i1 = i + 1
          i2 = i + 2

!         --- shift  dd(2..4), nn(2..4) to the left, update  x(i+2)

          do ii = 1, 3
            dd(ii) = dd(ii+1)
            nn(ii) = nn(ii+1)
          end do
          if ( i2<=maxnx ) then

!         --- x(i+2)  exists, update  dd(4),  nn(4)

            dd(4) = del(i2)
            nn(4) = ni(i2)
            call ervl23 ( alpha, beta, j, x(i2), dd(4), nn(4) )
          end if

!         --- now in  dd(1..4)  are updated  del(i-1..i+2)
!                 in  nn(1..4)  are updated   ni(i-1..i+2)
!                   ( i-1  or  i+2  need not exist )
!             the inner loop
!             x(i) was updated, it is  x(nx) = x(i)
!             it is  1<=igap<=nx, &
!                igap  marks beginning of a new gap witch should end at
!                x(nx)  or  later,  depending on value of  gap
!             gaps01  =  all gaps left of x(igap) have <=1 zeros
!             ngap01  =  number of lowest gaps on left of x(igap), &
!                           each with <=1 zeros

120          ier = 2
             if ( abs(x(nx)-x(i))>0d0 ) goto 3900
             ier = 3
             if ( abs(del(nx)-dd(2))>0d0 ) goto 3900
             ier = 4
             if ( abs(ni(nx)-nn(2))>0d0 ) goto 3900

!            --- update the interval  ( x(i), x(i+1) ], &
!                gap  tells whether the interval belongs to gap or not
!                   ( if nothing happens )
!                number of poles and zeros:  npol,  nzer  in step  j

             npol = abs ( ni(i1) ) - abs ( ni(i) )
             nzer = abs ( nn(3) ) - abs ( nn(2) )

             ier = 5
             if ( nzer<0 ) goto 3900

             if ( .not. gap ) then

!            --- an old e.v. interval

               ier = 6
               if ( ni(nx)>=0 ) goto 3900

               if ( nzer==0 ) then

!              --- old e.v. interval without zeros in step  j
!                  put it in the gap

               if ( lprint ) write (irefwr, '(2a/a,2i4,2f23.15)' ) &
                   'warning: eigva2.ervl20:  old e.v. interval ', &
                   'has no zeros in new step :', &
                   '     j, i, x(i), x(i+1)=', j, i, x(i), x(i1)

!              --- not e.v. interval any more, transforme it into gap, &
!                     change both in step  j-1  and  j

                 ni(i) = abs ( ni(i) )
                 ni(nx) = abs ( ni(nx) )
                 nn(2) = abs ( nn(2) )
                 ni(i+1) = abs ( ni(i+1) )
                 nn(3) = abs ( nn(3) )
                 gap = .true.

!                --- correct the chronological list of e.v.

                 k1 = 0
                 do k = 1, nxit

!                --- to drop xit(k) from the list ?

                   if ( x(i)<xit(k) .and. xit(k)<x(i+1) ) then
                     k1 = k1 + 1
                     if ( lprint ) write (irefwr,*) &
                       '   dropped from the list of e.v.:  j=', jit(k)
                     do ii = k, nxit-k1
                       jit(ii) = jit(ii+1)
                       xit(ii) = xit(ii+1)
                     end do
                   end if
                 end do

                 if ( k1/=1 ) then

!                --- not found, why ?

                   if ( lprint ) write (irefwr,*) &
                     'warning:  not found or found more then one'
                 end if

                 nxit = nxit - k1

               else

!              --- both old and new e.v. interval, x(nx) is end of gap
!                  inspect the gap

                 ier = 7
                 if ( igap>=nx ) goto 3900

!                --- number of zeros in the gap:  nzerg

                 k = abs(ni(igap))
                 nzerg = abs(ni(nx)) - k
                 if ( nzerg==0 ) then

!                --- an empty gap, delete all points between  x(igap)
!                       and  x(nx)

                   if ( nx>igap + 1 ) then
                     nx = igap + 1
                     x(nx) = x(i)
                     del(nx) = dd(2)
                     ni(nx) = nn(2)
                   end if
                 end if

!                --- update  gaps01  ( all gaps contain 0 or 1 zeros )

                 if ( gaps01 ) then
                   if ( nzerg<=1 ) then
                     ngap01 = ngap01 + 1
                   else
                     gaps01 = .false.
                   end if
                 end if

!                --- update  epilog,  ni(igap)

                 if ( epilog ) then

                   if ( nzerg>=2 ) then

!                  --- to cancel this epilog ?

                     if ( ngap01<=mlow ) then
                       epilog = .false.
                       cancel = .true.
                     end if
                   else if ( nzerg==0 ) then

!                  --- fix the gap (mark it by minus sign), &
!                         (it is in the empty phase)

                     ni(igap) = - k
                   end if

                 else

!                --- remove the possible negative sign

                   ni(igap) = k
                 end if

!                --- gap processing ends

               end if

!              --- update  x(i+1)

               goto 500

             end if

!            --- ( x(i), x(i+1) ] is in a gap
!                a gap interval, interpolate or nothing ?

             if ( npol==1 .and. nzer==1 ) then

!            --- try the rational interpolation in [(x(k),x(k+2)], &
!                see P+R (3.7)
!                select  k = i  or  k = i-1  ?

               k = i

!              --- correct this  k , if needed

               if ( existm ) then

!              --- x(i-1) exists

                 if ( i2<=maxnx ) then

!                --- both  x(i-1)  and  x(i+2)  exist

                   if ( x(i)-x(im)<x(i2)-x(i1) ) k = im
                 else

!                --- no x(i+2)

                   k = im
                 end if
               end if

!              --- interpolate in [x(k),x(k+2)],  use  dd(k-i+2..k-i+4)
!                  in P+R it is not clear whether to use step  j-1
!                     or step  j  here
!                  let us do it for step  j-1,  it saves time
!                     ( then  dd(1 and 4),  nn(1 and 4)  are never used )
!                  excluded   rational interpolation for step  j:  i1-k  is 1
!                  or 2, &
!                  excluded      zer  and  pol  must be in  ( x(k), x(k+1) )
!                  excluded  call ervl25 ( x(k), dd(k-i+2), i1-k, solved, &
!                  zer,pol,amu)
!                  excluded  write (irefwr,*) '    ervl25  j   solved = ', j, &
!                  solved
!                  excluded   no call ervl25 for j
!                  excluded  if ( .not. solved ) then
!                    try interpolate step  j-1 ,  without specification
!                       where the  pol  must be  ( negative  iwhere )

                 call ervl25 ( x(k), del(k), k-i1, solved, zer,pol,amu )

                 if ( .not. solved ) then

!                --- use bisection

                   goto 140
                 end if

!                --- zer  from step  j-1  is  pol  for step  j

                 pol = zer

!                --- interpolate for step  j  in [x(i), x(i+1)] only, &
!                       pol  is fixed, use  dd(2..3)

                 call ervl26 ( x(i), dd(2), pol, solved, zer, amu )

                 if ( .not. solved ) then

!                --- the problem remains, use bisection

                   goto 140
                 end if

!                --- excluded  end if
!                                rational interpolation was successfull
!                                convergence test P+R (3.8)

               if ( min ( abs(zer-x(i)), abs(x(i1)-zer) ) >= &
                    50d0 * max ( abs(zer-pol), tolc )  ) then

!              --- P+R (3.8) does not hold, &
!                     calculate an extra point P+R (3.9)

                 xex = zer + ( zer - pol )
                 dis = 0.01d0 * ( x(i1) - x(i) )
                 if ( xex<x(i) + dis ) then
                   xex = x(i) + dis
                 else if ( xex>x(i1) - dis ) then
                   xex = x(i1) - dis
                 end if

!                --- put the extra point in the step  j-1, &

                 goto 160

               else

!              --- P+R (3.8) holds,  zer, pol  are relatively close

                 if ( abs ( zer - pol )<=tolc ) then

!                --- think about the error estimation

                   if ( noerbn<0 ) then

!                  --- both P+R (3.8) and (3.10) hold, &
!                         and  call ervl24  is allowed now
!                      compute the error bound P+R (4.10)

                     goto 200

                   else

!                  --- call ervl24  is forbidden now

                   end if

                 else

!                --- P+R (3.8) holds, (3.10) does not hold
!                    no action in P+R for this case
!                       ( no extra point, no error bound )

                 end if

               end if

!              --- end of rational interpolation, update  x(i+1)

               goto 500

             end if

!            --- nzer = npol = 1  does not hold here
!                test P+R (3.4) for bisection, at least two zeros ?

             if ( nzer<=1 ) then

!            --- (nzer, npol) = (0, any) or (1, 0) or (1, 2 or more)
!                let it be, update  x(i+1)

               goto 500

             end if

!            --- bisection:  nzer>=2  or
!                nzer=npol=1, but rational interpolation has failed
!                bisect the interval ( x(i), x(i+1) )

140          xex = 0.5d0 * ( x(i) + x(i1) )

!            --- compare the interval length with  tolc

             if ( xex - x(i)<tolc ) then

!            --- compute error bound for  xex

               zer = xex
               goto 200

             end if

!            --- insert the point  xex  in the step  j-1
!                next part is usable also for other sources of  xex
!                insert  xex  between  x(i)  and  x(i+1)  in step j-1

160          im1 = im - 1
             if ( im1<=nx ) goto 1300

             if ( existm ) then
               x(im1) = x(im)
               del(im1) = del(im)
               ni(im1) = ni(im)
             else

!            --- new x(im1) doesn't exist

             end if

             x(im) = x(i)
             del(im) = del(i)
             ni(im) = ni(i)

!            --- values  del, ni  for the new point

             x(i) = xex
             call ervl22 ( alpha, beta, j-1, x(i), del(i), ni(i) )
             dd(4) = dd(3)
             nn(4) = nn(3)

             dd(3) = del(i)
             nn(3) = ni(i)
             call ervl23 ( alpha, beta, j, x(i), dd(3), nn(3) )

             i2 = i1
             i1 = i

!            --- here  i  decreases

             i = im
             im = im1

!            --- update existm: new existm = existence of the new x(im1)
!                                             = existence of the old x(im)
!                                             = old existm
!                use the recursion, update interval ( (x(i), x(i1) ], &
!                   values  dd(1..4), nn(1..4)  are already updated
!                end of the inner loop

             ier = 8
             if ( .not. gap ) goto 3900

           goto 120

!          --- continuation of the outer loop
!              estimate the errorbound of  zer

200        call ervl24 ( nnzer, tol, alpha, beta, j, &
                         zer, anorm, erbnd, deb, iersub )

           if ( iersub==1 ) then

!          --- too small  tol  with respect to round-off errors
!                 (infinite loops possible)
!              restart of P+R is impossible

             ieraut = 3
             goto 3000

           end if

           if ( erbnd<tol ) then

!          --- error bound is acceptable, see P+R part 5.1
!              setup an e.v. interval around  zer, (3.11)
!              mark the interval by negative  ni  on the left
!              remove inner points of the e.v. interval
!              solve possible collisions with the e.v.
!                 intervals both on the left and on the right

                 th1 = zer - tol
                 th2 = zer + tol

!                --- store this interval and this  j  in the list

                 if ( nxit<maxnev ) then
                   nxit = nxit + 1
                   jit(nxit) = j
                   xit(nxit) = ( th1 + th2 ) / 2d0
                 else
                   write (irefwr,*) 'warning: eigva2.ervl20', &
                     '   maxnev overflow', maxnev
                 end if

                 if ( lprint ) write (irefwr, 993 ) j, th1, th2
  993            format ('    j=', i4, 3x, 'th1=', f25.15, &
                         5x, 'th2 =', f25.15 )

!                --- reduce points in step  j, if needed
!                    store the old  nx

                 k = nx
  220            if ( x(nx)>=th1 ) then

!                --- exclude  x(nx)

                   nx = nx -1

!                  --- th1 can become new x(1), in this case
!                         include one new point  x(1)

                   if ( nx==0 ) then
                     nx = 1
                     x(1) = th1 - ( th2-th1 )
                     call ervl22 ( alpha, beta, j, x(1), del(1), ni(1) )
                   end if

                   goto 220

                 end if

                 if ( k/=nx ) then

!                --- restore the values  gap,  ngap01, gaps01, igap

                   igap = 1
                   gap = .true.
                   gaps01 = .true.
                   ngap01 = 0
                   do k = 2, nx

!                  --- gap  is status of  k-1, k

                     if ( ni(k)<0 ) then

!                    --- x(k)  is beginning of a fixed gap
!                                       or of e.v. interval

                       if ( gap ) then

!                      --- x(k) is begin of e.v interval = end of gap

                         if ( gaps01 ) then

!                        --- update  gaps01,  ngap01

                           if ( abs(ni(k))-abs(ni(igap))<=1 ) then
                             ngap01 = ngap01 + 1
                           else
                             gaps01 = .false.
                           end if
                         end if
                       else

!                      --- x(k) is the end of an e.v. interval

                         igap = k
                       end if
                       gap = .not. gap
                     else

!                    --- ni(k)>0

                       if ( .not. gap ) then

!                      --- x(k)  is the end of an e.v. interval

                         igap = k
                         gap = .true.
                       else

!                      --- x(k)  is an intermediate in a gap

                       end if
                     end if
                   end do

                 end if

                 if ( .not. gap ) then

!                --- there is a collision with an e.v. interval, &
!                       the left interval will became shorter

                   x(nx+1) = th1 - 0.01d0 * ( th1 - x(nx) )
                   nx = nx + 1
                   call ervl22 &
                     ( alpha, beta, j, x(nx), del(nx), ni(nx) )
                   gap = .true.
                 end if

!                --- include th1, th2 in step  j

                 nx = nx + 2
                 if ( nx>=im ) goto 1300
                 x(nx-1) = th1
                 x(nx) = th2
                 do ii = nx-1, nx
                   call ervl22 &
                     ( alpha, beta, j, x(ii), del(ii), ni(ii))
                 end do

                 nzer = abs(ni(nx)) - abs(ni(nx-1))
                 if ( nzer/=1 ) then

!                --- zer  has jumped out of this e.v. interval, &
!                       or another  zer  has accured in it

                   if ( lprint ) write (irefwr,*) &
                     'warning :  in new e.v. interval,  nzer =', nzer
                 end if

!                --- mark this e.v. interval by minus

                 ni(nx-1) = - abs(ni(nx-1))
                 nsame = 1

!                --- calculate new  igap, &
!                       it is the next to the last negative

                 igap = nx - 2
  245            if ( igap>1 ) then
                   if ( ni(igap-1)>0 ) then
                     igap = igap - 1
                     goto 245
                   end if
                 end if

!                --- reduce the gap left of  x(nx-1)
!                    .. number of zeros in the gap

                 nzerg = abs( ni(nx-1) ) - abs ( ni(igap) )

                 if ( nzerg==0 ) then

!                --- shift  nx-1, nx  to the  igap+1, igap+2

                   if ( igap+2<nx ) then
                     do k = 0, 1
                       k1 = igap + k + 1
                       k2 = nx - 1 + k
                       x(k1) = x(k2)
                       del(k1) = del(k2)
                       ni(k1) = ni(k2)
                     end do
                     nx = igap + 2
                   end if
                 end if

!                --- update  gaps01  ( all gaps contain 0 or 1 zeros )

                 if ( gaps01 ) then
                   if ( nzerg<=1 ) then
                     ngap01 = ngap01 + 1
                   else
                     gaps01 = .false.
                   end if
                 end if

!                --- igap  for next turn

                 igap = nx

!                --- reduce points in step  j-1

  250            if ( x(i1)<=th2 ) then

!                --- shift  i  to the right

                   if ( ni(i1)<0 ) then
                     gap = .not. gap
                   else
                     if ( .not. gap ) gap = .true.
                   end if

!                  --- if x(i+2) does not exist then finish step  j

                   if ( i1==maxnx ) goto 600
                   im = i
                   i = i1
                   i1 = i2
                   i2 = i2 + 1

!                  --- update  dd(1..4), nn(1..4)

                   do ii = 1, 3
                     dd(ii) = dd(ii+1)
                     nn(ii) = nn(ii+1)
                   end do
                   if ( i2<=maxnx ) then

!                  --- new x(i+2) exists

                     dd(4) = del(i2)
                     nn(4) = ni(i2)
                     call ervl23 ( alpha, beta, j, x(i2), dd(4), nn(4))
                   end if
                   goto 250
                 end if

                 if ( .not. gap ) then

!                --- collision with an e.v. interval in step  j-1
!                    that interval becomes a gap interval

                   gap = .true.
                 end if

!                --- it is not clear in P+R part 5.1, whether to
!                       change  tolc  also here
!                    no change  tolc

           else

!          --- error bound is not acceptable
!              reduce  tolc  as in (5.1)
!                 if the second term in  max () is bigger, &
!                 then signalize "no error bound call" for next
!                 j/100 steps
!              (without "min"  tolc  can increase drastically)

                 tolc = min ( tolc, abs(zer-pol) * (tol/erbnd)**2 )
                 temp = 5 * epsmac * anorm
                 if ( tolc<=temp ) then
                   tolc = temp
                   noerbn = j / 100
                 end if
           end if

!          --- finish: update the point  x(i+1)  itself
!              if more  ni(*) have the same absolute value
!                 then reduce the  x(*)  in step  j
!              update  nsame  with respect to both reduction and  x(i1)

500        if ( abs(ni(nx))==abs(nn(3)) ) then
             if ( nsame==4 ) then

!            --- discard the point  nx-1

               ii = nx - 1
               x(ii) = x(nx)
               del(ii) = del(nx)
               ni(ii) = ni(nx)

!              --- decrease  nx

               nx = ii

!              --- new  nsame  will be again  = 4

             else
               nsame = nsame + 1
             end if
           else
             nsame = 1
           end if

!          --- transfer the point  x(i1)  to the step  j

           nx = nx + 1

           if ( nx>=i ) goto 1300

           ier = 9
           if ( x(i1)<=x(nx-1) ) goto 3900

           x(nx) = x(i1)
           del(nx) = dd(3)
           ni(nx) = nn(3)

!          --- update  igap,  gap

           if ( .not. gap ) igap = nx
           if ( ni(nx)<0 ) then
             gap = .not. gap
           else
             if ( .not. gap ) gap = .true.
           end if

!          --- next i

           i = i + 1
           existm = .true.

         if ( i<maxnx ) goto 100

!        --- end of updating from  j-1  to  j
!            the last gap, if any

         if ( igap<nx ) then
           k = abs(ni(igap))
           nzerg = ni(nx) - k
           if ( gaps01 ) then
             if ( nzerg<=1 ) then
               ngap01 = ngap01 + 1
             else
               gaps01 = .false.
             end if
           end if
           if ( epilog ) then
             if ( nzerg>=2 ) then
               if ( ngap01<=mlow ) then
                 epilog = .false.
                 cancel = .true.
               end if
             else if ( nzerg==0 ) then
               ni(igap) = - k
             end if
           else
             ni(igap) = k
           end if
         end if

!        --- termination conditions
!            calculate  nfix

600      gap = .true.
         nfix = 0
         do i = 2, nx-1
           if ( ni(i)<0 ) then
             if ( gap ) nfix = nfix + 1
             gap = .not. gap
           else
             if ( .not. gap ) gap = .true.
           end if
         end do

         if ( deb ) write (irefwr,*) 'j =', j, '   nx =', nx, &
                  '   nfix =', nfix, &
                  '   ngap01 =', ngap01

         if ( epilog .or. j==m ) then

!        --- calculate  nlow

           do i = 1, nx
             if ( ni(i)>0 ) goto 630
           end do
           i = nx
  630      if ( i==nx ) i = i + 1
           nlow = max ( 0, i/2 - 1 )
           if ( deb ) write (irefwr,*) ' epilog,  n l o w  =', nlow

         end if

         if ( nfix>=n ) then

!        --- the most hard criterion holds

           nlow = n
           ieraut = 0
           goto 2000
         end if

!        --- more soft conditions

         if ( epilog ) then
           if ( nlow>=mlow.or. nlow==nx/2 ) then

!          --- it looks that the problem is solved

             ieraut = 0
             goto 2000
           end if
         end if

!        --- an epilog during the next turn ?

         if ( ngap01>mlow .or. ngap01==nx/2 .or. j==m-1 ) then

!        --- next turn with the epilog regime

           epilog = .true.
         else

!        --- next turn without an epilog regime

           if ( epilog ) cancel = .true.
           epilog = .false.
         end if

!        --- remove the minus signs from fixed gaps

         if ( cancel ) then
           if ( deb ) write (irefwr,*) 'epilog was  c a n c e l e d'

           igap = 1
           gap = .true.
           do i = 2, nx
             if ( ni(i)<0 ) then
               if ( gap ) then

!              --- e.v. interval  i, i+1

                 ni(igap) = abs(ni(igap))
               else

!              --- e.v. interval  i-1, i

                 igap = i
               end if
               gap = .not. gap
             else if ( .not. gap ) then

!            --- e.v. interval was  i-1, i

               gap = .true.
               igap = i
             end if
           end do
         end if

      end do

!     --- m  Lanczos steps was not enough

      j = m
      ieraut = 1
      goto 2000

!     --- finish
!         work array  x(1..maxnx)  overflow,  no restart of P+R

 1300 ieraut = 4
      goto 3000

!     --- calculate  nev,  ev(*)

2000  gap = .true.
      nev = 0
      do i = 2, nx-1
        if ( ni(i)<0 ) then
          if ( gap ) then

!         --- e.v. interval  i, i+1

            if ( nev>=maxnev ) then

!           --- ev(*,*) is full, no problem for P+R, &
!               restart is possible

              ieraut = 2
              goto 3000
            end if
            nev = nev + 1
            ev(nev,1) = x(i)
            ev(nev,2) = x(i+1)
          end if
          gap = .not. gap
        else
          if ( .not. gap ) gap = .true.
        end if
      end do

!------ prepare  jdone,  jdonpa  for possible restart

3000  jdone = max ( j, jdone )
      if ( ieraut<=2 ) then

!     --- restart of P+R is possible

        jdonpa = j
      else

!     --- P+R finished in an undefined stage, no restart of P+R

        jdonpa = 1
        nev = 0
        nlow = 0
      end if
      niter = j

      low = nlow>0

      do k = 1, nev

        if ( k<=min ( nlow, mlow ) ) &
          eval(k) = ( ev(k,1) + ev(k,2) ) / 2

!       --- fill minit(k) = minimum number of iterations to get
!             the interval [ ev(k,1), ev(k,2) ]

        do i = 1, nxit
          if ( xit(i)>=ev(k,1) .and. xit(i)<=ev(k,2) ) then
            minit(k) = jit(i)
            goto 3300
          end if
        end do

        write (irefwr,*) 'warning, eigva2.ervl20:  minit(k) not found', &
          '  k=', k

        minit(k) = niter

3300  end do
      goto 4000

!     --- an internal error was found, no results

 3900 call errint ( ier, 1 )
      call errsub ( 1403, 1, 0, 0 )
      call instop

4000  if ( deb ) write ( irefwr, '( /1x,a/(1x,i5,2f15.6) )' ) &
         '    i       alpha(i)        beta(i)', &
         ( i, alpha(i), beta(i), i=1,niter )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ervl20'

      end if  ! ( debug )
      call erclos ( 'ervl20' )

      end subroutine ervl20
