      subroutine proventhalpy ( amat, row, col, diag, rhsd, &
                                enthalpy, n, eps, ndim, iprint, &
                                temperature, massmat, rinenthalpy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     perform the actual overrelaxation process for the enthalpy method
!
! **********************************************************************
!
!                       KEYWORDS
!
!     constraint
!     linear_solver
!     overrelaxation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmoduletime
      use sepmodulecomio
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, ndim, iprint
      integer row(*), col(*), diag(*)
      double precision amat(*), rhsd(*), enthalpy(*), eps, &
                       temperature(*), massmat(*), rinenthalpy(*)

!     amat           i    part 2 of large matrix
!     col            i    part 2 of array intmat containing the column numbers
!                         of the non-zero elements of the matrix
!     diag           i    position of diagonal elements of matrix
!     enthalpy      i/o   enthalpy vector
!     eps            i    |eps| is the required accuracy
!                         if eps>0, it concerns the absolute accuracy, eps<0
!                         means reative accuracy
!     iprint         i    Defines the level of output
!                         0:  no special print output
!                         1:  extra information of iteration process is printed
!     massmat        i    mass matrix
!     n              i    number of unknowns
!     ndim           i    Dimension of the space.
!     rhsd           i    Right-hand-side vector
!     rinenthalpy    i    Real input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     row            i    first part of array intmat containing the number of
!                         off-diagonal elements in each row accumulated
!     temperature    o    temperature vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical check, estom0, estom1, eachst, finished, &
              relative
      integer i, levsub, k, iter, j
      double precision eps1, eps2, rn, olddif, differ, sum, omopt, rmu, &
                       omega1, anorm
      double precision rho, L, kappa_l, kappa_s, c_l, c_s, v, w, &
                       tempold(n)

!     L              latent heat
!     anorm          max-norm of vector
!     c_l            capacity (liquid)
!     c_s            capacity (solid)
!     check          indicates if the norm of the difference must be computed
!                    (true) or not (false)
!     differ         max-norm of unew-enthalpy
!     eachst         indicates whether lambda must be computed in each step
!                    (true) or not (false)
!     eps1           Value of epsilon corrected for the "1"-norm
!     eps2           Value of eps1 corrected for the computed eigenvalue
!     estom0         indicates whether omega_0 must be computed (true)
!                    or not (false)
!     estom1         indicates whether omega_1 must be computed (true)
!                    or not (false)
!     finished       indicates whether the iteration is finished (true)
!                    or not (false)
!     i              General loop variable
!     iter           iteration level
!     j              General loop variable
!     k              n**(1/ndim)
!     kappa_l        diffusion (liquid)
!     kappa_s        diffusion (solid)
!     levsub         Sub iteration level. For this level a new condition must be
!                    checked or carried out
!     olddif         "1"-norm of u at preceding iteration - u at two
!                    iterations back
!     omega1         Help variable to store the computed value of omega
!                    temporarily
!     omopt          Estimation of optimal omega
!     relative       If true a relative accuracy is required
!     rho            density rho
!     rmu            Parameter mu
!     rn             Help variable to make n real
!     sum            Help variable to compute a sum
!     tempold        Copy of temperature
!     v
!     w
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     479   Lambda >= 1
!     480   Diagonal of matrix contains zero's
!     481   Accuracy not reached in nmax steps
! **********************************************************************
!
!                       PSEUDO CODE
!
!   The method performed is described int the Programmer's Guide
!   unew := enthalpy
!   check diagonal of matrix
!   eps1 := eps (1+sqrt(n))
!   number_of_iterations := 0
!   check := false
!   estimate_omega_0 := true
!   estimate_omega_1 := true
!   each_step := false
!   difference := eps_machine
!   old_difference := 0
!   if ( iprint >= 1 ) then
!      check := true
!   if ( niter1=-2 ) then
!      estimate_omega_0 := false
!      estimate_omega_1 := false
!      eps2 := (1-lambda)*eps1
!   else ( niter1#-2 )
!      lambda := 1000
!      if ( niter1=-1 ) then
!         niter1 := n^(1/ndim)
!      if ( niter2=-1 ) then
!         niter2 := n^(1/ndim)
!      if ( niter1=0 and niter2=0 ) then
!         estimate_omega_0 := false
!         estimate_omega_1 := false
!         each_step := true
!         old_difference := eps_machine
!      else if ( niter1=0 ) then
!         estimate_omega_0 := false
!         sub_iteration_level := niter2-1
!      else ( niter1#0 )
!         sub_iteration_level := niter1-1
!   finished := false
!   For iter := 1(1) nmax while ( not finished ) do
!      unew := one_step_overrelaxation ( enthalpy )
!      if ( check ) then
!         difference := || unew - enthalpy ||
!      unew := enthalpy
!      number_of_iterations := number_of_iterations + 1
!      if ( difference = 0 ) then
!         finished = true
!      else if ( difference < eps1 ) then
!         finished := true
!      if ( not finished and number_of_iterations >= sub_iteration_level ) then
!         if ( estimate_omega_0 ) then
!            if ( not check and old_difference=0 ) then
!               check := true
!            else if ( old_difference=0 ) then
!               old_difference := difference
!            else
!               lambda := difference / old_difference
!               if ( lambda >= 1 ) then
!                  error
!                  omega_opt := 1
!                  lambda := 1
!               else
!                  mu := 1/lambda ( (lambda-1+omega)/omega )^2
!                  omega_opt := 2/(1+sqrt(1-mu))
!               omega := omega_opt
!               omega1 := 1+alpha (omega_opt - 1)
!               if ( difference < (1-lambda)*eps1 ) then
!                  finished := true
!               else
!                  omega := omega1
!                  estimate_omega_0 := false
!                  sub_iteration_level := sub_iteration_level + niter2 - 1
!                  if ( estimate_omega_1 ) then
!                     old_difference := 0
!                  if ( iprint >= 1 ) then
!                     sub_iteration_level := sub_iteration_level + 1
!                  else
!                     check := false
!         else if ( estimate_omega_1 ) then
!            if ( not check and old_difference=0 ) then
!               check := true
!            else if ( old_difference=0 ) then
!               old_difference := difference
!            else
!               lambda := difference / old_difference
!               if ( lambda >= 1 ) then
!                  error
!                  omega_opt := 1
!                  lambda := 1
!               else
!                  mu := 1/lambda ( (lambda-1+omega)/omega )^2
!                  omega_opt := 2/(1+sqrt(1-mu))
!               omega := omega_opt
!               omega1 := 2+beta (omega_opt - 1)
!               if ( difference < (1-lambda)*eps1 ) then
!                  finished := true
!               else
!                  omega := omega1
!                  estimate_omega_1 := false
!               eps2 := (1-lambda) eps1
!         else if ( each_step ) then
!            lambda := difference / old_difference
!            old_difference := difference
!            if ( difference < (1-lambda)eps1 ) then
!               finished := true
!         else ( lambda has been computed )
!            if ( difference < eps2 ) then
!               finished := true
!   if ( not finished ) then
!      error
!      finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'proventhalpy' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from proventhalpy'
         write(irefwr,1) 'niter1, niter2', niter1, niter2
         write(irefwr,2) 'eps', eps
         call prinrl ( rhsd, 10, 'rhsd' )
         write(irefwr,2) 'tstep', tstep
         iprint = 2
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      anorm = 1d0

!     --- Extract parameters from rinenthalpy

      kappa_s = rinenthalpy(1)
      kappa_l = rinenthalpy(2)
      L       = rinenthalpy(3)
      c_s     = rinenthalpy(4)
      c_l     = rinenthalpy(5)
      rho     = rinenthalpy(6)

      if ( debug ) then
         write(irefwr,2) 'kappa_s, kappa_l, L', kappa_s, kappa_l, L
         write(irefwr,2) 'c_s, c_l, rho', c_s, c_l, rho
         call prinrl ( temperature, 10, 'temperature' )
      end if  ! ( debug )

!     --- copy temperature in tempold

      call dcopy ( n, temperature, 1, tempold, 1 )

!     --- check diagonal of matrix

      do i = 1, n
         if ( abs(amat(diag(i))).le.0d0 ) then
            call errint ( i, 1 )
            call errsub ( 480, 1, 0, 0)
            go to 1000
         end if
      end do

!     --- Initialisations

      rn = n
      if ( eps.lt.0d0 ) then
         relative = .true.
      else
         relative = .false.
      end if
      eps1 = abs(eps)
      niter = 0
      levsub = 1
      check = .true.
      estom0 = .true.
      estom1 = .true.
      eachst = .false.
      olddif = 0d0
      differ = epsmac
      if ( iprint.ge.1 ) check = .true.
      if ( niter1.eq.-2 ) then

!     --- niter1 = -2, lambda and omega have been computed before

         estom0 = .false.
         estom1 = .false.
         eps2 = (1d0-alam) * eps1
         check = .true.

      else

!     --- niter1 # -2

         alam = 1000d0
         k =  nint(n**(1d0/ndim))
         if ( niter1.eq.-1 ) niter1 = k
         if ( niter2.eq.-1 ) niter2 = k
         if ( niter1.eq.0 .and. niter2.eq.0 ) then

!        --- niter1=niter2=0, the first steps are skipped
!            lambda is computed each step

            estom0 = .false.
            estom1 = .false.
            eachst = .true.
            olddif = epsmac
         else if ( niter1.eq.0 ) then

!        --- niter1 = 0, skip first part

            estom0 = .false.
            levsub = niter2-1

         else

!        --- niter1 # 0

            levsub = niter1-1
            if ( check ) levsub = levsub + 1

         end if

      end if
      if ( iprint.ge.1 ) write ( irefwr,150) niter1, niter2, levsub
150   format(' niter1, niter2, subiteration level', 3i6)
      finished = .false.
      do iter = 1, nmax
         if ( finished ) go to 1000

!        --- actual overrelaxation process

         do i = 1, n
            sum = rhsd(i)
            do j = row(i)+1, diag(i)-1
               sum = sum - tstep*amat(j) * temperature(col(j))
            end do
            do j = diag(i)+1, row(i+1)
               sum = sum - tstep*amat(j) * temperature(col(j))
            end do

!           --- enthalpy correction

            if (sum.gt.massmat(i)*rho*L) then
               v = (sum - massmat(i)*rho*L)/(tstep*amat(diag(i)) + &
                                        rho*c_l/kappa_l*massmat(i))
            else if (sum.lt.0) then
               v = sum/(tstep*amat(diag(i)) + rho*c_s/kappa_s*massmat(i))
            else
               v = 0
            end if

            w = tempold(i)+omega*(v-tempold(i))

            if (w*tempold(i).gt.0) then
               temperature(i) = w
            else
               temperature(i) = v
            end if

            enthalpy(i) = (sum - tstep*amat(diag(i))*temperature(i))/ &
                           massmat(i)

         end do

         if ( check ) then

!        --- compute max-norm of difference vector

            differ = 0d0
            do i = 1, n
               differ = max(differ,abs(temperature(i)-tempold(i)))
            end do
            if ( differ.gt.0d0 .and. relative ) then
               anorm = 0d0
               do i = 1, n
                  anorm = max(anorm,abs(temperature(i)))
               end do
            end if
         end if
         if ( iprint.ge.1 ) write ( irefwr, 220 ) niter, differ/anorm, &
                            alam, omega
220      format(' iteration', i5, '  differ:', d10.3, '  alam:', d12.5, &
                '  omega:', d10.3 )

!        --- copy temperature in tempold

         call dcopy ( n, temperature, 1, tempold, 1 )
         niter = niter+1
         if ( differ.le.0d0 ) finished = .true.
         if ( .not. finished .and. niter.ge.levsub ) then

!        --- Now some actions have to be carried out

            if ( estom0 ) then

!           --- omega 0 must be estimated

               if ( .not. check .and. olddif.le.0d0 ) then

!              --- first time, set check

                  check = .true.

               else if ( olddif.le.0d0 ) then

!              --- second time, set olddif

                  olddif = differ

               else

!              --- third time, estimate lambda and omega

                  alam = differ / olddif
                  if ( alam.ge.1d0 ) then
                     if ( differ.lt.eps1 ) then
                         finished = .true.
                     else

!                    --- Error 479: lambda too large

                        call errint ( niter, 1 )
                        call erreal ( alam, 1 )
                        call errsub ( 479, 1, 1, 0 )
                        omopt = 1d0
                        alam = 1d0
                     end if
                  else
                     rmu = ( 1d0/alam)*((alam-1d0+omega)/omega)**2
                     omopt = 2d0/(1d0+sqrt(1d0-rmu))
                     omega = omopt
                     omega1 = 1d0 + alpha *  ( omopt - 1d0 )
                  end if
                  if ( differ.lt.(1d0-alam)*eps1*anorm ) then
                     finished = .true.
                  else
                     omega = omega1
                     estom0 = .false.
                     levsub = niter + niter2 - 1
                  end if
                  if ( estom1 ) olddif = 0
                  if ( iprint.ge.1 ) then
                     levsub = levsub + 1
                  else
                     check = .false.
                  end if
               end if

            else if ( estom1 ) then

!           --- omega 1 must be estimated

               if ( .not. check .and. olddif.le.0d0 ) then

!              --- first time, set check

                  check = .true.

               else if ( olddif.le.0d0 ) then

!              --- second time, set olddif

                  olddif = differ

               else

!              --- third time, estimate lambda and omega

                  alam = differ / olddif
                  if ( alam.ge.1d0 ) then
                     if ( differ.lt.eps1 ) then
                         finished = .true.
                     else

!                    --- Error 479: lambda too large

                        call errint ( niter, 1 )
                        call erreal ( alam, 1 )
                        call errsub ( 479, 1, 1, 0 )
                        omopt = 1d0
                        alam = 1d0
                     end if

                  else

                     rmu = (1d0/alam)*( (alam-1d0+omega)/omega)**2
                     omopt = 2d0/(1d0+sqrt(1d0-rmu))

                  end if
                  omega = omopt
                  omega1 = 2d0 + beta * ( omopt - 2d0 )
                  if ( differ.lt.(1d0-alam)*eps1*anorm ) then
                     finished = .true.
                  else
                     omega = omega1
                     estom1 = .false.
                     eps2 = (1d0-alam) * eps1
                  end if

               end if

            else if ( eachst ) then

!           --- The eigenvalue must be computed in each step

               alam = differ / olddif
               olddif = differ
               if ( differ.lt.(1d0-alam)*eps1*anorm ) finished = .true.

            else

!           --- alam has already been computed

               if ( differ.lt.eps2*anorm ) finished = .true.

            end if

         end if

      end do
      if ( .not. finished ) then

!     --- Error 481: the accuracy has not been reached yet

         call errint ( niter, 1 )
         call erreal ( alam, 1 )
         call erreal ( differ/((sqrt(rn)+1d0)*anorm), 2 )
         call errsub ( 481, 1, 2, 0 )
      end if
1000  call erclos ( 'proventhalpy' )
      end
