      subroutine prpcgrad1 ( iu0, iun, iinsol, rinsol1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 13-12-2017 New storage of intmat
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 11-02-2004
!
!   copyright (c) 2004-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     finite-element preconditioned conjugate-gradient solver
!
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulematr
      use sepmodulecactl
      use sepmodulekprob
      use sepmoduleuser
      use sepmodulemain
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iu0, iun, iinsol(*)
      double precision rinsol1(*)

!     iinsol         i    Contains information concerning the solver
!                         Contents
!                         1: maxiter i.e. maximum number of iterations
!                         2: kprint, defines the amount of output
!                            Possible values:
!                            0: no output
!                            1: the number of iterations is printed
!                            2: the convergence of the iteration process is
!                               showed
!     iu0            i    Contain information of the initial approximation
!     iun            o    Contain information of the solution
!     rinsol1        i    Contains information concerning the solver
!                         Contents
!                         1: eps, i.e. accuracy with respect to residual
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idum(1), izn, nc, kprint, maxiter, j
      double precision alpha2, beta2, alpha1, beta1, &
                       eps, rdum(1), norm1
      double precision, allocatable :: rn(:), zn(:), vn(:), pn(:), resn(:)

!     alpha1         help variable to compute alpha2
!     alpha2         parameter of the CG iteration loop
!     beta1          help variable to compute beta2
!     beta2          parameter of the CG iteration loop
!     eps            Local accuracy
!     idum           dummy variable
!     izn            Work array to store a sepran vector of structure usol
!                    with essential boundary conditions zero
!     j              Counting variable
!     kprint         defines level of output, see iinsol
!     maxiter        maximum number of iterations
!     nc             counting variable
!     norm1          norm of vector
!     pn             Work array to store the vector p^n
!     rdum           dummy variable
!     resn           Work array to store the residual
!     rn             Work array to store the vector r^n
!     vn             Work array to store the vector v^n
!     zn             Work array to store the vector z^n
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer idamax
      double precision ddot

!     DAXPY          Overwrite double precision dy with double precision
!                    da*dx + dy
!     DCOPY          Copy double precision dx to double precision dy
!     DDOT           (CDOT)     Inner product of two vectors
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     IDAMAX         Blas subroutine defining the index of the maximum
!                    element in an array
!     PRMAPBACK      Map vector from global numbering to local numbering without
!                    boundary conditions
!     PRPARMAP       Map vector from global numbering to local numbering without
!                    boundary conditions
!                    In case of parallel computing global values are filled by
!                    sending and receiving from neighbor processors
!     PRPCGRADBLD    Compute residual for the spectral element equations in an
!                    element by element way
!     PRPCGRADSOL    Solve system of equations using the matrix stored on the
!                    finite element mesh and the right-hand side corresponding
!                    to the spectral problem
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPCREAVC      Create vector
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2481   Maximum number of iteration has been reached
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if the system LV = F is to be solved then:
!      o         o
!     R = F - L V
!       o    o
!     HZ  = R
!      o    o
!     P  = Z
!     n     n   n     n    n
!    a  = (R , Z )/ (P , LP )
!     n+1  n    n  n
!    V  = V  + a  P
!     n+1  n    n   n
!    R  = R  - a  LP
!      n+1   n+1
!    HZ   = R
!     n     n+1 n+1   n   n
!    b  = (R  ,Z  )/(R  ,Z  )
!     n+1   n+1    n   n
!    P  = Z     + b  P
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data izn/0/
! ======================================================================
!
      call eropen ( 'prpcgrad1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prpcgrad1'
         call ertrace

      end if
      if ( ierror/=0 ) go to 1000

!     --- Create temporary arrays at the end of ibuffr
!         It concerns the arrays R^n, Z^n, V^n and P^n
!         They contain the degrees of freedom without boundary conditions

      iprob = ks(iu0)%iprob
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      nusol  = kprobloc(5)
      nrusol = kprobloc(66)

      allocate ( rn(nusol), zn(nusol), vn(nusol), pn(nusol), &
                 resn(nusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nusol, 'rn' )
      if ( ierror/=0 ) go to 1000

!     --- Copy parameters from iinsol and rinsol1

      eps = rinsol1(1)
      maxiter = iinsol(1)
      kprint = iinsol(2)
      if ( kprint>0 ) write ( irefwr, 100 )
100   format(//'Output for the spectral CG method with finite', &
                ' element preconditioner'//)
      if ( debug .or. kprint>1 ) write ( irefwr, 110 )
110   format(' Step       residual')

!     --- initialize vector zn (izn) with 0 (necessary for building)

      idum(1) = 0
      rdum(1) = 0d0
      iprob = ks(iu0)%iprob
      call sepcreavc ( 0, 1, iprob, 0, izn, idum, rdum )
      if ( ierror/=0 ) go to 1000

!     --- initialize the counter

      nc = 0

!     --- build vector L V ^ o (rn); subtract it from the right-hand side

      call prpcgradbld ( iu0, vn, rn, itype, .true. )
      if ( ierror/=0 ) go to 1000

!     --- Copy u0 into un (iun)

      call sepcopyvc ( iu0, iun )
      if ( ierror/=0 ) go to 1000

!     --- Map f into z^n (temporarily)
!         Map iun into vn

      usoltemp => ks(irhsd(1))%sol
      call prparmap ( usoltemp, zn )
      usoltemp => ks(iun)%sol
      call prparmap ( usoltemp, vn )

!     --- rn := f - rn (residual)

      do j = 1, nrusol
         rn(j) = zn(j)-rn(j)
      end do

!     --- exit if the accuracy is reached
!         Compute maximum value of residual

      norm1 = abs(rn(idamax(nrusol, rn, 1)))

      if ( debug .or. kprint>1 ) write(irefwr,120) nc, norm1
120   format ( i5, d15.3 )
      if ( norm1<eps ) go to 900

!     --- calculate Z ^ o

      call prpcgradsol ( izn, rn, zn )
      keep = 2

!     --- p^0 := Z^0

      call dcopy ( nrusol, zn, 1, pn, 1 )

      if ( ierror/=0 ) go to 1000

!     --- alpha1 := (R^0,Z^0)

      alpha1 = ddot ( nrusol, rn, 1, pn, 1 )
      beta1 = alpha1

      if ( ierror/=0 ) go to 1000

!     --- begin iterations

200   nc = nc + 1
      if ( nc>maxiter ) then

!     --- maximal number of iterations exceeded

         call errint ( nc-1, 1 )
         call errsub ( 2481, 1, 0, 0 )
         go to 1000

      end if

!     --- calculate a ^ n
!         first build L P ^ n, store in resn

      call prpcgradbld ( izn, pn, resn, itype, .false. )

!     --- alpha2 := (r^n,z^n) / (p^n,Lp^n)

      alpha2 = alpha1/ddot ( nrusol, resn, 1, pn, 1 )

!     --- calculate the new approximation:
!         v^n+1 = v^n + alpha2 p^n

      call daxpy ( nrusol, alpha2, pn, 1, vn, 1 )

!     --- calculate the new residual
!         r^n+1 = r^n - alpha2 Lp^n

      call daxpy ( nrusol, -alpha2, resn, 1, rn, 1 )

!     --- exit if the accuracy is reached

      norm1 = abs(rn(idamax(nrusol, rn, 1)))

      if ( debug .or. kprint>1 ) write(irefwr,120) nc, norm1
      if ( abs(norm1)<eps ) go to 900

!     --- calculate Z ^ n+1  (H z^n+1 = r^n+1)

      call prpcgradsol ( iun, rn, zn )

!     --- b^n = ( R^n+1,z^n+1) / (r^n,z^n)

      alpha1 = ddot ( nrusol, rn, 1, zn, 1 )
      beta2 = alpha1/ beta1
      beta1 = alpha1

!     --- calculate P ^ n+1 = z^n+1 + b^n p^n

      do j = 1, nrusol
         pn(j) = zn(j) + beta2 * pn(j)
      end do

      go to 200

900   if ( kprint>0 ) write(irefwr,910) nc, norm1
910   format ( 'Number of iterations is: ', i5, &
               '   Residual is ', d15.3 )

!     --- Clean up

1000  if ( nc>0 ) call seprmsolinfo ( izn )

!     --- Map solution back into iun

      usoltemp => ks(iun)%sol
      call prmapback ( usoltemp, vn )
      keep = 0

      deallocate ( rn, zn, vn, pn, resn, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'rn' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prpcgrad1'

      end if
      call erclos ( 'prpcgrad1' )

      end subroutine prpcgrad1
