      subroutine prpcgradsol ( iun, rn, zn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 28-05-2014 New call to sepsolvel
!        version  1.2  date 05-12-2012 Long integers
!        version  1.1  date 21-10-2007 New call to sepsolvel
!
!   copyright (c) 2004-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of equations using the matrix stored on the finite
!     element mesh and the right-hand side corresponding to the spectral problem
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulevecs
      use sepmodulemain
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iun(*)
      double precision :: rn(*), zn(*)

!     iun            i    Must contain the solution vector in SEPRAN form
!                         is used to construct a temporary array with the
!                         same structure
!     rn             i    Contains the irght-hand side for the system of
!                         equations
!     zn             o    Solution of linear system
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ires

!     ires           Work array to store the result vector in standard sepran
!                    format
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRMAPBACK      Map vector from global numbering to local numbering without
!                    boundary conditions
!     PRPARMAP       Map vector from global numbering to local numbering without
!                    boundary conditions
!                    In case of parallel computing global values are filled by
!                    sending and receiving from neighbor processors
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPLINSOL      Body of linear solver
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Map rn into ires
!     Solve system of linear equations
!     Map result into Z^n
!     Remove array ires
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prpcgradsol' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prpcgradsol'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Map rn into ires

      ires = 0
      call sepcopyvc ( iun, ires )
      usoltemp => ks(ires)%sol
      call prmapback ( usoltemp, rn )

!     --- Solve system of linear equations

      iseqsol = 1
      iseqreac = 0
      keepmat = 1
      iseqsolut = 1
      keep = 1

      iseqmt = stiffmat(1)

      call seplinsol ( stiffmat, iun, ires )

!     --- Map izn into Z^n

      usoltemp => ks(iun(1))%sol
      call prparmap ( usoltemp, zn )

!     --- Remove array ires

      call seprmsolinfo ( ires )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prpcgradsol'

      end if
      call erclos ( 'prpcgradsol' )

      end subroutine prpcgradsol
