      subroutine sepal ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-05-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of linear equations of special form using the
!     Augmented Lagrange appoach of Benzi et al.
!     Old subroutine pral
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     matrix
!     real
!     krylov
!     conjugate_gradients
!     augmented_lagrange
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesimple
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq), rhsd(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: normrhsd
      double precision, allocatable :: schurmat(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     normrhsd       Norm of right-hand side vector
!     schurmat       Contains the approximate inverse Schur complement
!                    matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRWAR         Print warnings
!     SEPALGCR       Solve a system of linear equations of special form using
!                    the Augmented Lagrange appoach
!     SEPSIMPLEDEL   Delete all temporary created arrays in the simple process
!     SEPSIMPLEMATQ  Create the preconditioning matrix for Q depending on iprec
!     SEPSIMPLERES   Compute the residuals r_u and r_p to be used at the start
!                    of the simple algorithm
!     SEPUPDATERHSAL Update the right-hand side vector for the Augmented
!                    Lagrange Method
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2446   Right-hand side identical 0
!    2447   Norm of right-hand side is small
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The al algorithm can be formulated as follows:
!
!     Let the system of equations be:
!
!     | Q    G | |u|  = |f_u|
!     | D    0 | |p|    |f_p|
!
!     input:  Q, G, D, f_u, f_p and S^-1 the approximate Schur complex inverse
!             Matrix
!     output: u, p
!
!     The systems of equations are computed approximately
!     We use a GCR method with preconditioner:
!
!     | Q    G    |
!     | 0    S^-1 |
!
!     This means that in each step we solve:
!
!     z_2 = S^-1 r_2
!     z_1 = Q^-1 ( r_1 - B^T z_2 )
!     Code
!
!     Fill constants
!     Compute or activate start vector
!     Renumber right-hand side and solution arrays
!     Compute the residual r_u = f_u - Q u - G p
!     Compute the residual r_p = f_p - D u
!     Create the matrices as well as the preconditioning matrices with respect
!     to the AL algorithm
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepal' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepal'

      end if
      IM => intmt(A%iseqim)

!     --- Fill constants

      algamma = rinsol(2,1)

      allocate(worky1(np), worky2(nu), stat = error)
      if ( error/=0 ) call eralloc ( error, nrusol+nu+np, 'worksim' )
      if ( ierror/=0 ) go to 1000

      Q%scaling = 0     ! no scaling

!     --- Update right-hand side with contribution of essential boundary
!         conditions due to augmented lagrangian
!         The pressure mass matrix is inverted

      call sepupdaterhsal ( rhsd )

!     --- Compute the residual r_u = f_u - Q u - G p
!         Compute the residual r_p = f_p - D u
!         The arrays dp and du are created and cleared

      call sepsimpleres ( normrhsd, A, usol, rhsd )
      if ( ierror/=0 ) go to 1000

      if ( normrhsd<=0d0 ) then

!     --- Right-hand side is identical 0, return

         call errwar ( 2446, 0, 0, 0 )
         go to 500

      else if ( normrhsd<=1d-10 ) then

!     --- Right-hand side is small: give warning

         call erreal ( normrhsd, 1 )
         call errwar ( 2447, 0, 1, 0 )

      end if

!     --- Create the preconditioning matrix for the velocity part

      call sepsimplematq
      if ( ierror/=0 ) go to 1000

!     --- Perform the GCRsimple algorithm

      allocate(schurmat(np), stat = error)
      if ( error/=0 ) call eralloc ( error, np, 'schurmat' )
      if ( ierror/=0 ) go to 1000

!     --- Create array schurmat containing (mu M^-1 + gamma W^-1)

      schurmat = 1d0/S_PM(IM%diagPM(1:np)) + algamma*S_PMD
      if ( ierror/=0 ) go to 1000

      Q%matmultsp = 1     ! special matrix vector multiplication in cg
      call sepalgcr ( normrhsd, schurmat, A, usol )
      Q%matmultsp = 0     ! reset

      if ( ierror/=0 ) go to 1000

!     --- invert the pressure mass matrix back

      S_PMD = 1d0/S_PMD

!     --- Fill the output values into inpsol(2)

      if ( inpsol(22,1)==1 .and. ierror/=0 ) inpsol(2,1) = 1

!     --- Delete temporary created arrays in reversed sequence

500   call sepsimpledel ( A )

      deallocate ( schurmat, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'schurmat' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepal'

      end if  ! ( debug )
      call erclos ( 'sepal' )

      end
