      subroutine sepalgebr ( ichois, idegfd, ivec1, ivec2, ivec3, &
                             a, b, p, q, ipoint )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  1.7  date 29-05-2014 Extension of timebefore
!        version  1.6  date 17-05-2008 New call to to0035
!        version  1.5  date 02-12-2006 Extension of iconst
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     algebraic manipulations with vectors
!     (for vectors with ivec1(2)=110 and npboun>0 periodical degrees
!     (of freedom are taken into account only once for ichois=1,2,3,7)
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision a,b,p,q
      integer ichois, idegfd, ivec1, ivec2, ivec3, ipoint

!    a        i/o  In- and output real constants for the
!                  manipulation depending on ichois
!    b        i/o  see a
!    ichois    i   Choice parameter indicating what type of function
!                  is chosen by the user.  Possible values:
!                   1:    Substract:  VEC3 = VEC1 - A
!                   2:    A is average value of VEC1
!                         At output IPOINT is number of points
!                   3:    Linear combination
!                         VEC3 = A * VEC1 + B * VEC2
!                   4:    VEC1(IDEGFD) in IPOINT is stored in A
!                   5:    VEC1(IDEGFD) in IPOINT = A
!                   6:    The minimum value of VEC1 is stored in A
!                         The maximum value of VEC1 is stored in B
!                         Real vectors only
!                   7:    The inner product VEC1 . VEC2 is stored in A
!                         The number of points used is stored in IPOINT
!                   8:    VEC3 = A * |VEC1|
!                         VEC1 is complex, VEC3 is real
!                   9:    VEC3 = A * arg( VEC1 )
!                         VEC1 is complex, VEC3 is real
!                  10:    The inner product VEC1 . VEC2 is stored in A
!                         The number of points used is stored in IPOINT
!                         Boundary conditions are excluded
!                  13:    VEC3 is a function of VEC1 and VEC2. The user
!                         subroutine FUNALG or FUNALC is called.
!                  18:    VEC3 = conjg(VEC1)
!    idegfd    i   In this integer given by the user, he must store the degrees
!                  of freedom that are requested for the special purpose.
!                  In most applications only one degree of freedom per point
!                  is required, and the array may be replaced by an integer
!                  variable but not by a constant.
!                  If the variable has value 0, then all degrees of freedom
!                  are considered.
!                  If the variable has value -1, then all degrees of freedom
!                  are considered, except those corresponding to essential
!                  boundary conditions
!    ipoint   i/o  Nodal point number depending on ichois
!    ivec1     i   First input vector to be manipulated
!                  ivec1 must be a SEPRAN array of length 5, which may be
!                  of type 110, 115, 116 or 119
!    ivec2     i   Second input vector to be manipulated (if necessary)
!                  If only one array is needed, ivec2 may be replaced by a
!                  dummy
!    ivec3     i   Third input vector to be manipulated (if necessary)
!                  If only one or two arrays are needed, ivec3 may be replaced
!                  by a dummy
!    p        i/o  See a
!    q        i/o  See a
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision const(4)
      integer itypvc, iconst(2), jdegfd(1), indelgrp(1)
      logical compli
      double precision :: timebefore(2)

!    compli       Indication if IVEC1 corresponds to a complex array (true) or
!                 to a real one (false)
!    const        Real array to be used by TO0035, containing some constants
!    iconst       Integer array that must be used by subroutine TO0035 to
!                 indicate some choices
!    indelgrp     dummy
!    itypvc       Type number of array
!    jdegfd       Help array to store idegfd
!    timebefore   Time at entrance of subroutine
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRININ         print 1d integer array
!     TO0022         test if vectors are of equal type
!     TO0035         algebraic manipulations for vectors of type 110, 115
!                    and 116
!     TO0106         algebraic manipulations for vectors of type 119
!     TO0110         algebraic manipulations (special possibilities)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS has wrong value
!     217   IDEGFD has wrong value
!     224   IVEC1 corresponds to real vector not permitted in
!           combination with this value of ICHOIS
!     803   IVEC1 corresponds to complex vector not permitted in
!           combination with this value of ICHOIS
!     855   IDEGFD = 0 in combination with ICHOIS = 4 or 5
!    1153   Array IVEC1 has not been filled
!    1155   IDEGFD has wrong value in combination with type 119
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepalgebr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepalgebr'
         write(irefwr,1) 'ivec1, ivec2, ivec3', ivec1, ivec2, ivec3
         write(irefwr,1) 'ichois, idegfd', ichois, idegfd
         write(irefwr,2) 'a, b, p, q', a, b, p, q
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( itime>0 ) call ersettime ( timebefore )
      if ( idegfd<-1 ) then
         call errint ( 1, idegfd )
         call errsub ( 217, 1, 0, 0 )
      end if
      if ( ichois<=0 .or. ichois>10 .and. ichois/=13 .and. &
           ichois/=18 ) then
         call errint ( 1, ichois )
         call errsub ( 1, 1, 0, 0 )
      end if
      if ( idegfd==0 .and. (ichois==5 .or. ichois==4 ) ) &
           call errsub ( 855, 0, 0, 0 )
      itypvc = ks(ivec1)%typevec
      if ( itypvc/=110 .and. itypvc/=115 .and. itypvc/=116 .and. &
           itypvc/=119 .and. itypvc/=127 ) then
         call errint ( itypvc, 1 )
         call errsub ( 1153, 1, 0, 0 )
      end if

!     --- Check if VEC1 is of the correct type

      if ( ks(ivec1)%compl==0 ) then
         compli = .false.
      else
         compli = .true.
      end if
      if ( compli .and. ichois==6 ) then
         call errint ( 1, ichois )
         call errsub ( 803, 1, 0, 0 )
      end if
      if ( .not. compli .and. ( ichois==8 .or. ichois==9 &
           .or. ichois==18 ) ) then
         call errint ( 1, ichois )
         call errsub ( 224, 1, 0, 0 )
      end if
      if ( itypvc==119 ) then

!     --- itypvc = 119

         if ( idegfd/=0 .and. idegfd/=1 ) then
            call errint ( 1, idegfd )
            call errsub ( 1155, 1, 0, 0 )
         end if
      end if
      if ( ichois==3 .or. ichois==7 .or. ichois==10 .or. &
           ichois==13 ) then

!     --- ichois = 3, 7, 10 or 13; hence array ivec2 is used
!         Check if arrays ivec1 and ivec2 are identical

         call to0022 ( ivec1, ivec2, 1, 2 )

      end if

!     --- Actual call

      if ( ierror/=0 ) go to 1000
      iconst(1) = 0
      iconst(2) = 0
      if ( ichois==4 .or. ichois==5 ) then

!     --- ichois = 4,5  store ipoint in iconst

         iconst(1) = ipoint
      end if
      if ( ichois==1 .or. ichois==5 ) then
         if ( compli ) then
            const(1) = a
            const(2) = b
         else
            const(1) = a
         end if
      else if ( ichois==8 .or. ichois==9 ) then
         const(1) = a
      else if ( ichois==3 ) then
         if ( compli ) then
            const(1) = a
            const(2) = b
            const(3) = p
            const(4) = q
         else
            const(1) = a
            const(2) = b
         end if
      end if
      jdegfd(1) = idegfd
      if ( itypvc==119 ) then
         call to0106 ( ichois, jdegfd, ivec1, ivec2, &
                       ivec3, const, iconst )
      else if ( ichois==4 .or. ichois==5 ) then
         call to0110 ( ichois, jdegfd, ivec1, ivec2, const, iconst )
      else
         call to0035 ( ichois, jdegfd, ivec1, ivec2, &
                       ivec3, const, iconst, &
                       0, indelgrp, .true. )
      end if
      if ( ichois==2 .or. ichois==4 .or. ichois==7 &
           .or. ichois==10 ) then
         if ( compli ) then
            a = const(1)
            b = const(2)
         else
            a = const(1)
         end if
      else if ( ichois==6 ) then
         a = const(1)
         b = const(2)
      end if
      if ( ichois==7 .or. ichois==10 ) then
         ipoint = iconst(1)
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepalgebr'

      end if  ! ( debug )
      call erclmn ( 'sepalgebr', timebefore )

      end subroutine sepalgebr
