      subroutine sepaliter ( dup, schurmat, niteru, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-05-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform the preconditioning step in the GCR part of the Augmented
!     Lagrange Method
!     Old subroutine praliter
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
!     augmented_lagrange
!     krylov
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: niteru
      double precision, intent(in) :: schurmat(np)
      double precision, intent(inout) :: work(nu)
      double precision, target :: dup(nu+np)

!     dup            o    Array containing the correction for (u,p)
!     niteru         o    Number of iterations to solve the u-equations
!     schurmat       i    Contains the approximate inverse Schur complement
!                         matrix
!     work                work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The algorithm can be formulated as follows:
!
!     Let the system of equations be:
!
!     | Q    G | |u|  = |f_u|
!     | D    0 | |p|    |f_p|
!
!     input:  Q, G, D, f_u, f_p and S^-1 the approximate Schur complex inverse
!             Matrix
!     output: u, p
!
!     The systems of equations are computed approximately
!     We use a GCR method with preconditioner:
!
!     | Q    G    |
!     | 0    S^-1 |
!
!     This means that in each step we solve:
!
!     z_2 = S^-1 r_2
!     z_1 = Q^-1 ( r_1 - B^T z_2 )
!
!     Method:
!
!     Multiply r_2 by S^-1 (result z_2)
!     Compute B^T z_2
!     Compute r_1 - B^T z_2
!     Solve Q z_1 = r_1 - B^T z_2 by cg type methods
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepaliter' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepaliter'
         call prinrl ( ru, nu, 'res_u' )
         call prinrl ( rp, np, 'res_p' )

      end if
      if ( ierror/=0 ) go to 1000

      du => dup(1:nu)
      dp => dup(nu+1:nu+np)

!     --- Multiply r_2 by -S^-1 (result z_2)
!         r_2 = rhs_p, z_2 = dp
!         We use the minus sign since the Schur complement is equal to
!         - ( gamma+mu ) M_p, whereas the matrix contains M_p

      dp = -schurmat*rp
      if ( debug ) call prinrl ( dp, np, 'dp/0' )

!     --- Compute B^T z_2, store in work

      work = G*dp
      if ( debug ) call prinrl ( work, nu, 'work' )

!     --- Compute r_1 - B^T z_2, store in rhsu

      rhsu = ru-work
      if ( debug ) call prinrl ( rhsu, nu, 'rhsu' )

!     --- Solve Q z_1 = r_1 - B^T z_2 by cg type methods

      du = Q .div. rhsu

      if ( ierror/=0 ) then

!     --- The process halted because of an error

         write(irefwr,*) 'The error occurred in the Q-matrix'
         go to 1000

      end if

      niteru = Q%numiter ! number of iterations carried out

1000  if ( debug ) then
         call prinrl ( du, nu, 'du' )
         call prinrl ( dp, np, 'dp' )
         write(irefwr,*) 'End sepaliter'
      end if
      call erclos ( 'sepaliter' )

      end
