      subroutine sepblocktria ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve a system of linear equations of special form using the
!     Block Triangular preconditioner.
!     The method is accelerated by the GCR process
!     Old subroutine prblocktria
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     matrix
!     real
!     krylov
!     conjugate_gradients
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: normrhsd

!     normrhsd       Norm of right-hand side vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRWAR         Print warnings
!     SEPBLOCKGCR    Solve a system of linear equations of special form using
!                    the Block Triangular preconditioner
!     SEPCOPYMATPM   Copy real PM matrix in type matrix (simple)
!     SEPSIMPLEDEL   Delete all temporary created arrays in the simple process
!     SEPSIMPLEMATPM Create the preconditioning matrix for PM depending on iprec
!     SEPSIMPLEMATQ  Create the preconditioning matrix for Q depending on iprec
!     SEPSIMPLERES   Compute the residuals r_u and r_p to be used at the start
!                    of the simple algorithm
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2446   Right-hand side identical 0
!    2447   Norm of right-hand side is small
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepblocktria' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepblocktria'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute the residual r_u = f_u - Q u - G p
!         Compute the residual r_p = f_p - D u
!         The arrays dp and du are created and cleared

      call sepsimpleres ( normrhsd, A, usol, rhsd )
      if ( ierror/=0 ) go to 1000

      if ( normrhsd<=0d0 ) then

!     --- Right-hand side is identical 0, return

         call errwar ( 2446, 0, 0, 0 )
         go to 500

      else if ( normrhsd<=1d-10 ) then

!     --- Right-hand side is small: give warning

         call erreal ( normrhsd, 1 )
         call errwar ( 2447, 0, 1, 0 )

      end if

!     --- Make relation between R and pressure mass matrix

      call sepcopymatpm ( 3 )

!     --- Create the the preconditioning matrices

      call sepsimplematq
      call sepsimplematpm ( .false., 3 )
      if ( ierror/=0 ) go to 1000

!     --- Perform the GCRsimple algorithm

      call sepblockgcr ( normrhsd, A, usol )
      if ( ierror/=0 ) go to 1000

!     --- Fill the output values into inpsol(2)

      if ( inpsol(22,1)==1 .and. ierror/=0 ) inpsol(2,1) = 1

      if ( ierror/=0 ) go to 1000

!     --- Delete temporary created arrays in reversed sequence

500   call sepsimpledel ( A )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepblocktria'

      end if  ! ( debug )
      call erclos ( 'sepblocktria' )

      end
