      subroutine sepcgcr ( A, x, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve linear system of equation by the preconditioned CGR method
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(inout) :: A
      double complex, intent(inout) :: x(A%neq), b(A%neq)

!     A              i    Matrix A
!     b              i    Right-hand side
!     x              o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double complex, pointer :: res(:), v(:), s(:), work(:)
      double complex, allocatable, target :: svec(:,:), vvec(:,:)
      integer :: nmax, iter, ngcr, imin, i
      double complex :: alpha
      double precision :: alphamin, bnorm, vnorm, rnorm, rnormprev
      logical :: finished

!     alpha          variable used in the iteartion
!     alphamin       Minimum value of alpha
!     bnorm          Norm of the right-hand side (b,b)
!                    they are not
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     imin           Position in array for which minimum is reached
!     iter           Iteration sequence number
!     ngcr           Maximum number of GCR iterations
!     nmax           maximum number of iterations that may be performed by
!                    the CG-process. In any way no more than the number of
!                    degrees of freedom iterations are carried out.
!     res            residual
!     rnorm          Norm of residual
!     rnormprev      Norm of prior residual
!     s              active vector s during iteration
!     svec           Work array of size ngcr x n_u to store the vectors
!                    s during the iteration
!     v              active vector v during iteration
!     vnorm          L2 cnorm of v
!     vvec           Work array of size ngcr x n_u to store the vectors
!                    v during the iteration
!     work           Work array of length n to store the help vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCCGSTART    Fill starting value in array x and compute initial residual
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The algorithm for one step reads
!
!        s^(k+1) := dx, r = res  (! input )
!        v^(k+1) := A s^(k+1)
!        For i := 1 (1) k do
!           alpha := ( v^(k+1), v^i)
!           v^(k+1) := v^(k+1)- alpha v^i
!           s^(k+1) := s^(k+1)- alpha s^i
!        v^(k+1) := v^(k+1)/||v^(k+1)||
!        s^(k+1) := s^(k+1)/||v^(k+1)||
!        alpha := ( r, v^(k+1))
!        x := x + alpha s^(k+1)
!        r := r - alpha v^(k+1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcgcr' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcgcr'
         write(irefwr,1) 'n', A%neq
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      bnorm = cnorm(b,A%neq)
      if ( bnorm<=0d0 ) then

!     --- Right-hand side is equal to zero

         x =0d0
         go to 1000  ! rhs is equal to 0

      end if  ! ( bnorm<=0d0 )

      nmax = A%nmax                  ! maximum number of iterations
      if ( nmax==0 ) nmax=10*A%neq
      ngcr = A%ntrunk                ! (maximum) number of gcr iterations

      if ( debug ) &
         write(irefwr,1) 'nmax ,iprec, iprint, ngcr', &
                          nmax, A%type_prec, A%iprint, ngcr

!     --- Allocate svec and vvec

      allocate( svec(A%neq,0:ngcr+1), vvec(A%neq,0:ngcr+1), res(A%neq), &
                work(A%neq), stat = error)
      if ( error/=0 ) call eralloc ( error, 2*A%neq*(ngcr+2), 'svec' )
      if ( ierror/=0 ) go to 1000

      v => vvec(:,0)
      s => svec(:,0)

!     --- Set start vector and initial residu

      call sepccgstart ( A, x, b, res, work )

      finished = .false.

      rnorm = cnorm(res,A%neq)

      if ( A%irelerr==1 ) A%eps = A%eps*rnorm

      iter = 0

      if ( A%iprint>1 ) write(irefwr,100) iter, rnorm, 1d0
100   format( 'iteration    cnorm      speed'/ i6, ":", 2d12.3 )

      do while ( .not. finished .and. iter<nmax )

!        --- Preconditioned matrix vector multiplication

         s = res/A

         if ( A%eistat ) then
            v = res + A%S * s
            v = A/v
            v = v+s
         else
            v = A*s
         end if  ! ( A%eistat )

         rnormprev = rnorm

         imin = min(iter+1,ngcr)
         alphamin = rinfin

         do i = 1, min(iter,ngcr)

            alpha = dot_product(v, vvec(:,i)) ! ( v^(k+1), v^i)

            if ( iter>ngcr .and. abs(alpha)<alphamin ) then

!           --- alpha<alphamin, set alpha = alphamin and imin = i
!               Only if the number of iterations exceeds ngcr we compute
!               the minimum value of alpha
!               We apply the truncated GCR process by putting the new vector in
!               position imin

               alphamin = abs(alpha)
               imin = i

            end if

            v = v-alpha * vvec(:,i) !v^(k+1)=v^(k+1)-alpha v^i
            s = s-alpha * svec(:,i) !s^(k+1)=s^(k+1)-alpha s^i

         end do  ! i = 1, min(iter,ngcr)

         vnorm = cnorm(v,A%neq)

         if ( debug) write(irefwr,2) 'vnorm', vnorm

         v = v/vnorm
         s = s/vnorm

!        --- Copy result to new vector

         vvec(:,imin) = v
         svec(:,imin) = s

         alpha = dot_product(v,res)  ! v*res
         x = x+alpha*s
         res = res-alpha*v

         if ( debug) write(irefwr,2) 'alpha, alphamin', alpha, alphamin

         iter = iter+1  ! raise iter

         rnorm = cnorm(res,A%neq)

         if ( A%iprint>1 ) &
            write(irefwr,110) iter, rnorm, rnorm/rnormprev
110      format( i6, ":", 2d12.3 )

         finished = rnorm<A%eps

      end do  ! while ( .not. finished .and. iter<nmax )

      if ( A%iprint==1 ) write(irefwr,120) iter
120   format ( 'The number of iterations carried out in the GCR', &
               ' process is ', i5 )

      deallocate ( svec, vvec, res, work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'svec' )

1000  call erclos ( 'sepcgcr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcgcr'

      end if  ! ( debug )

      end subroutine sepcgcr
