      subroutine sepcgprint2 ( res, parvers, A, lambda, nmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform printing CG process when iteration has not converged
!
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      integer, intent(in) :: nmax
      double precision, intent(in) :: res(A%neq)
      logical, intent(in) :: parvers
      double precision, intent(in) :: lambda

!     A              i    Matrix A
!     lambda         i    Approximation of smallest eigenvalue
!     nmax           i    maximum number of iterations that may be performed by
!                         the CG-process. In any way no more than the number of
!                         degrees of freedom iterations are carried out.
!     parvers        i    If true it concerns the parallel version
!     res            i    residual
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nwarnsave
      double precision :: reprint

!     nwarnsave      Variable to store the value of nwarn
!     reprint        Auxiliary variable to print a real number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
!    2138   The estimated value of lambda is larger than the required lambda
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcgprint2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcgprint2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%iprint==1 ) then

!     --- iprint = 1  Print information

         reprint = norm(res, A)

         write ( irefwr, 350 ) nmax, reprint
350      format(' The number of executed iterations in the', &
                ' CG-process is', i5/ &
                ' The 2-norm of the residual is:', ss,1pe12.2e2 )

      end if

      if ( A%iaccuracy==0 ) then
         if ( A%alammin<lambda ) then
            if ( A%iprint>=0 ) then
               call erreal ( lambda, 1 )
               call erreal ( A%alammin, 2 )
               if ( A%iprint>=2 ) then
                  nwarnsave = nwarn
                  nwarn = -100
                  call errwar ( 2138, 0, 2, 0 )
                  nwarn = nwarnsave + 1
               else
                  call errwar ( 2138, 0, 2, 0 )
               end if
            end if
         end if
      end if

      call errint ( nmax, 1 )
      if ( A%aterror==1) then
         A%numiter = nmax
         call errwar ( 676, 1, 0, 0 )
      else
         call errsub ( 676, 1, 0, 0 )
      end if

1000  call erclos ( 'sepcgprint2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcgprint2'

      end if  ! ( debug )

      end subroutine sepcgprint2
