      subroutine sepcgstart ( A, x, b, res, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill starting value in array x and compute initial residual
!
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     start
!     residual
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      double precision, intent(in) :: b(A%neq)
      double precision, intent(out) :: x(A%neq), res(A%neq)
      double precision, intent(out), optional :: work(A%neq)

!     A              i    Matrix A
!     b              i    Right-hand side
!     res            o    Residual vector
!     work           o    Work array to store pbar in case of cgs
!     x              o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: matmultsp, scaling

!     matmultsp      Help parameter to save the value of A%matmultsp
!     scaling        Help parameter to save the value of A%scaling
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRANDOM        generates a random vector in parallel
!     PRINRL         Print 1d real vector
!     PRPARRANDM     Computes random number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcgstart' )
      debug = .false. .and. ioutp>=0
!debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcgstart'
         write(irefwr,1) 'n', A%neq
         write(irefwr,1) 'startvec, typesolv', A%startvec, A%typesolv
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      matmultsp = 0
      if ( A%matmultsp==2 ) then ! temporary
         matmultsp = A%matmultsp  ! save
         scaling = A%scaling
         A%matmultsp = 0          ! temporary
         A%scaling = 0    ! matrix is not scaled
      end if  ! ( A%matmultsp==2 )

      if ( A%startvec==0 ) then

!     --- istart = 0, set start vector equal to zero

         x = 0d0

      else

!     --- istart # 0, start vector is given by the user
!                     Compute residual vector  r = y - A x

         if ( A%startvec==2 ) then

!        --- Start vector is random

            call prandom ( A%neq, x )
            if ( parallel .and. typecoupl==0 ) then
               call prparrandm ( A%neq, x, blockunkinfo, A%sendinfo )
            else if ( parallel ) then
               call sepsolcopsol ( x, xglob )
            end if  ! ( parallel .and. typecoupl==0 )

         end if

      end if

      if ( debug ) call prinrl ( x, A%Neq, 'x' )

      if ( A%typesolv==2 ) then

!     --- Cgs method, store Ax in work

         work = A*x                       ! y = A x
         if ( A%eistat ) work = work+2d0*x  ! correction Eisenstat
         res = b-work                     ! residual
         work = res
         if ( A%eistat ) work = A/work      ! Multiply residual by U^(-1)
         if ( A%eistat ) res = A/res        ! Pre multiply residual by E^(-1)

      else

!     --- Other method, compute residual

         res = residual ( A, x, b )

      end if  ! ( A%typesolv==2 )

      if ( matmultsp==2 ) then ! reset
         A%matmultsp = matmultsp
         A%scaling = scaling
      end if  ! ( A%matmultsp==2 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( b, A%Neq, 'b' )
         call prinrl ( res, A%Neq, 'res' )
         write(irefwr,*) 'End sepcgstart'

      end if  ! ( debug )
      call erclos ( 'sepcgstart' )

      end subroutine sepcgstart
