      subroutine sepcidriter ( A, x, sparam, res, rnorm, pshadow )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform main loop for idr iteration
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     parallel
!     idr
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(inout) :: A
      double complex, intent(inout) :: x(A%neq), res(A%neq)
      double precision, intent(inout) :: rnorm
      integer, intent(in) :: sparam
      double complex, intent(in) :: pshadow(A%neq,sparam)

!     A              i    Matrix A
!     pshadow        i    Array of size n x s
!                         Contains all shadow vectors of size n each
!                         So pshadow(i,j) contains i^th component of p_j
!     res           i/o   residual
!     rnorm         i/o   Norm of residual
!     sparam         i    Defines the parameters s in IDR(s)
!     x             i/o   Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k, iter, priter(10), n, maxiter
      double complex :: mvec(sparam), cvec(sparam), omega, help(5), &
                        mmat(sparam,sparam), &
                        alpha(sparam), psi(sparam), beta, som
      double precision :: residu(10), angle, rho
      double complex, allocatable :: work(:), tvec(:,:), dr(:,:), &
                                     dx(:,:)
      logical :: extradebug

!     alpha          parameter in method
!     angle          angle to compute omega
!     beta           parameter in method
!     cvec           Vector c in method
!                    they are not
!     dr             Matrix dr defined in method by Sonneveld/van Gijzen
!     dx             Matrix dx defined in method by Sonneveld/van Gijzen
!     extradebug     If true extra debugging statements are carried out
!     help           work array to store the inner products
!                    (tvec(:,1),tvec(:,2)), (tvec(:,2),tvec(:,2)), &
!                    (tvec(:,1),tvec(:,1)), (r,r) and (r,tvec(:,2))
!                    in that sequence
!     i              Counting variable
!     iter           Iteration sequence number
!     j              Counting variable
!     k              Counting variable
!     maxiter        maximum number of iterations
!     mmat           Matrix M defined in method
!     mvec           Number of old vectors to be used in FUNCVECT
!     n              size of matrix and vectors
!     omega          Parameter in method
!     priter         Output array to print residuals
!     psi            Work array to store the inner products of pshadow and dR
!     residu         Output array to print residuals
!     rho            help parameter to compute omega
!     som            Help parameter to compute a sum
!     tvec           Vectors v and t in method
!                    tvec(:,1) = v;  tvec(:,2) = t
!     work           Work array of size n to store an intermediate array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRINCOMPL      Print 1d complex vector
!     PRINCOMPL1     Print 2d complex vector
!     PRINCOMPL2     Print a complex two-dimensional array
!     PRINTITER      Fill arrays priter and residu
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The matlab code for this part is given by (Collignon/van Gijzen)
!     This is the minisync variant
!
!     iter = 0
!     M = I      %  size s
!     om = 1
!     f = Q'*r   %  New right-hand size for small system
!     while ( rnorm>tolb & iter<maxit )
!
!        for k = 1:s
!           c = M(k:s,k:s)\f(k:s)  % Solve small system
!           v = r - dR(:,k:s)*c    % make v orthogonal to Q
!           if ( prec )
!              v = P\v
!           end
!           dX(:,k) = dX(:,k:s)*c + om*v
!           dR(:,k) = A*dX(:,k)     % Compute dR(:,k) = A dX(:,k)
!           psi = Q'*dR     %  s inner products (combined)
!           % Bi-Orthogonalise the new basis vectors:
!           Solve M alpha(1:k-1) = psi(1:k-1)
!           dR(:,k) = dR(:,k) - alpha(1:k-1)*dR(:,1:k-1)
!           dX(:,k) = dX(:,k) - alpha(1:k-1)*dX(:,1:k-1)
!           % New column of M = Q'*dR  (first k-1 entries are zero)
!           M(i,k) = psi(i) - alpha(1:k-1)*M(i,1:k-1)
!           %  Make r orthogonal to p_i, i = 1..k
!           beta = f(k)/M(k,k)
!           r = r - beta*dR(:,k)
!           x = x + beta*dX(:,k)
!           iter = iter + 1
!           normr = cnorm(r)
!           if ( normr<tolb | iter==maxit )
!              break
!           end
!           % New f = Q'*r (first k  components are zero)
!           if ( k <s )
!              f(1:k) = 0
!              f(k+1:s)   = f(k+1:s) - beta*M(k+1:s,k)
!           end
!        end     %  for k = 1:s
!     % Now we have sufficient vectors in dR_j to compute residual in dR_j+1
!     % Note: r is already perpendicular to Q so v = r
!        v = r
!        if ( prec )
!           v = P\v
!        end
!        t = A*v
!        om = t'*r/(t'*t);  f = -Q't  % s+2 inner products combined
!        r = r - om*t
!        x = x + om*v
!        normr = cnorm(r)
!        f = om*f
!        iter = iter + 1
!
!     end % while ( rnorm>tolb & iter<maxit )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcidriter' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.

!     --- Get parameters

      n = A%neq
      maxiter = A%nmax       ! maximum number of iterations
      if ( maxiter==0 ) maxiter = 10*n

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcidriter'
         write(irefwr,1) 'maxiter, sparam,', maxiter, sparam
         write(irefwr,2) 'rnorm, eps', rnorm, A%eps
         if ( extradebug ) then
            call princompl ( res, n, 'res' )
            call princompl2 ( pshadow, n, sparam, 'pshadow' )
         end if  ! ( extradebug )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate( work(n), tvec(n,1:2), dr(n,sparam), dx(n,sparam), &
                stat = error)
      if ( error/=0 ) call eralloc ( error, 3*n, 'work' )
      if ( ierror/=0 ) go to 1000

      angle = 0.7d0
      if ( debug ) write(irefwr,1) 'iprint', A%iprint

!     --- iter = 0
!         M = I      %  size s
!         om = 1

      iter = 0
      mmat = 0d0
      dr = 0d0
      dx = 0d0
      do i = 1, sparam
         mmat(i,i) = 1d0
      end do  ! i = 1, sparam
      omega = 1d0

!     --- f = (r'*Q)'     %  New right-hand side for small system

      do k = 1, sparam
         mvec(k) = dot_product ( pshadow(:,k), res )
      end do  ! k = 1, sparam

!     --- while ( rnorm>tolb & iter<maxit )

      do while ( rnorm>A%eps .and. iter<maxiter )

         if ( debug ) then
            call princompl ( mvec, sparam, 'f' )
            call princompl1 ( mmat, sparam, sparam, 'M' )
         end if  ! ( debug )

!        --- for k = 1:s

         do k = 1, sparam

!        --- c = M(k:s,k:s)\f(k:s)  % Solve small system
!            Since M is a lower triangular matrix the solution is
!            straight-forward
!            M and f are global quantities, they are the same on all
!            computers, so this can be done locally

            cvec(k) = mvec(k)/mmat(k,k)
            do j= k+1, sparam
               som = mvec(j)
               do i = k, j-1
                  som = som-mmat(j,i)*cvec(i)
               end do  ! i = k, k+j-1
               cvec(j) = som/mmat(j,j)
            end do  ! j= k+1, sparam

!           --- v = r - dR(:,k:s)*c    % make v orthogonal to Q

            do j = 1, n
               tvec(j,1) = (0d0,0d0)
               do i = k, sparam
                  tvec(j,1) = tvec(j,1)-dr(j,i)*cvec(i)
               end do  ! i = 1, sparam
               tvec(j,1) = res(j)+tvec(j,1)
            end do   ! j = 1, n

            if ( extradebug ) then
               write(irefwr,1) 'k', k
               call princompl ( cvec(k), sparam-k+1, 'c' )
               call princompl ( tvec, n, 'v' )
            end if  ! ( debug )

!           --- dX(:,k) = dX(:,k:s)*c + om*v

            som = (0d0,0d0)
            do i = 1, n
               som = (0d0,0d0)
               do j = k, sparam
                  som = som+dx(i,j)*cvec(j)
               end do  ! j = 1, sparam
               dx(i,k) = som+omega*tvec(i,1)
            end do  ! i = 1, n
            if ( extradebug ) then
               call princompl2 ( dx, n, sparam, 'U' )
            end if  ! ( debug )

!           --- dR(:,k) = A*dX(:,k)     % Compute dR(:,k) = A dX(:,k)

            work = dx(:,k)/A         ! ILU
            dr(:,k) = A*work

            if ( extradebug ) then
               call princompl2 ( dr, n, sparam, 'G' )
            end if  ! ( debug )

!           --- Bi-Orthogonalise the new basis vectors:
!               Solve M alpha(1:k-1) = psi(1:k-1)

            do i = 1, k-1
               som = (0d0,0d0)
               do j = 1, n
                  som = som+pshadow(j,i)*dr(j,k)
               end do  ! j = 1, n
               alpha(1) = som / mmat(i,i)
               if ( debug ) write(irefwr,2) 'alpha', alpha
               dr(1:n,k) = dr(1:n,k) - alpha(1)*dr(1:n,i)
               dx(1:n,k) = dx(1:n,k) - alpha(1)*dx(1:n,i)
            end do  ! i = 1, k-1

            if ( extradebug ) then
               call princompl2 ( dx, n, sparam, 'U' )
               call princompl2 ( dr, n, sparam, 'G' )
            end if  ! ( debug )

!           --- New column of M = Q'*dR  (first k-1 entries are zero)
!               M(k:s,k) = (dR(:,k)'*Q(:,k:s))'

            do i = k, sparam
               som = (0d0,0d0)
               do j = 1, n
                  som = som+pshadow(j,i)*dr(j,k)
               end do  ! j = 1, n
               mmat(i,k) = som
            end do  ! i = k, sparam

            if ( debug ) &
               call princompl2 ( mmat, sparam, sparam, 'M' )

!           --- Make r orthogonal to p_i, i = 1..k
!               beta = psi(k)/M(k,k)
!               r = r - beta*dR(:,k)
!               x = x + beta*dX(:,k)

            beta = mvec(k)/mmat(k,k)
            res = res - beta*dr(1:n,k)
            x = x + beta*dx(1:n,k)

            if ( debug ) then
               write(irefwr,2) 'beta', beta
               call princompl ( res, n, 'r' )
               call princompl ( x, n, 'x' )
            end if  ! ( debug )

!           --- iter = iter + 1
!               normr = cnorm(r)

            iter = iter+1
            rnorm = cnorm ( res, n )
            if ( debug ) then
               write(irefwr,1) 'iter', iter
               write(irefwr,2) 'rnorm', rnorm
            end if  ! ( debug )

            if ( A%iprint>1 ) &
               call printiter ( A%iprint, iter, rnorm, priter, &
                                residu, 1 )

            if ( rnorm<A%eps ) go to 200

!           --- New f = Q'*r (first k  components are zero)

            if ( k<sparam ) then

!           --- k<sparam
!               f(k+1:s) = f(k+1:s) - beta*M(k+1:s,k)

               mvec(1:k-1) = 0d0
               mvec(k+1:sparam) = mvec(k+1:sparam) - &
                                  beta*mmat(k+1:sparam,k)

            end if  ! ( k<sparam )
            if ( debug ) call princompl ( mvec, sparam, 'f' )

         end do  ! k = 1, sparam

!        --- Now we have sufficient vectors in dR_j to compute residual in
!            dR_j+1
!            Note: r is already perpendicular to Q so v = r
!            v = r
!            if ( prec )
!               v = P\v
!            end
!            t = A*v
!            om = (t'*r)/(t'*t);  f = -Q't  % s+2 inner products combined
!            r = r - om*t
!            x = x + om*v
!            normr = cnorm(r)
!            f = om*f

         tvec(:,1) = res

         work = tvec(:,1)/A
         tvec(:,2) = A*work

         help(1) = dot_product ( tvec(:,1), tvec(:,2) )
         help(2) = dot_product ( tvec(:,2), tvec(:,2) )
         help(3) = dot_product ( tvec(:,1), tvec(:,1) )

         do i = 1, sparam
            mvec(i) = dot_product ( pshadow(:,i), tvec(:,2) )
         end do  ! i = 1, sparam

         omega = help(1)/help(2)
         rho = abs(help(1)/sqrt(help(3)*help(2)))
         if ( abs(rho )<angle ) omega = omega*angle/abs(rho)

         res = res - omega * tvec(:,2)
         x = x + omega * tvec(:,1)

!        --- normr = cnorm(r)
!            iter = iter + 1

         iter = iter+1

         rnorm = cnorm ( res, n )

         mvec = -mvec*omega

         call printiter ( A%iprint, iter, rnorm, priter, residu, 1 )

         if ( debug ) then
            write(irefwr,2) 'omega', omega
            call princompl ( tvec(1,1), n, 'v' )
            call princompl ( tvec(1,2), n, 't' )
            call princompl ( res, n, 'r' )
            call princompl ( x, n, 'x' )
            write(irefwr,1) 'iter', iter
            write(irefwr,2) 'rnorm', rnorm
         end if  ! ( debug )

      end do  ! while ( rnorm>A%eps .and. iter<maxiter ) do

      if ( rnorm>A%eps ) then

!     --- accuracy has not been reached, give error message

         call errint ( maxiter, 1 )
         if ( A%aterror==1) then
            A%numiter=maxiter
            call errwar ( 676, 1, 0, 0 )
         else
            call errsub ( 676, 1, 0, 0 )
         end if

      end if  ! ( rnorm>A%eps )

200   if ( A%iprint==2 .and. iter>10 ) then

!     --- iprint = 2 Print final residuals

         do j = max(1,22-iter), 10
            write ( irefwr, 220 ) priter(j), residu(j)
220         format( i10, ss,1pe15.2e2 )
         end do

      end if

      if ( A%iprint>=1 ) then

!     --- iprint>=1  Print information

         write ( irefwr,250 ) iter, rnorm
250      format(' The number of executed iterations in the', &
                ' IDR process is', i5/  &
                ' The 2-norm of the residual is:', ss,1pe12.2e2 )

      end if
      A%numiter = iter
      A%itererr = 0

      deallocate ( work, tvec, dr, dx, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )

1000  call erclos ( 'sepcidriter' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcidriter'

      end if  ! ( debug )

      end subroutine sepcidriter
