      subroutine sepcmatdiv ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = A^(-1) x, where A is a preconditioning matrix
!     a symmetric compact storage is used, hence only D and L
!     The preconditioning matix has been created before
!     In other words the ILU inverse of A is stored
!     Solve A x = y, where A is an ILU preconditioning matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulecmat   ! definition of matrices
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: x(A%neq)
      double complex :: y(A%neq)

!     A              i    Matrix A
!     x              i    vector to be multiplied
!     y              o    result Ax
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iprec
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     iprec          type of preconditioning
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCSCMATDIV   Compute y = A^(-1) x, where A is a preconditioning matrix
!                    a symmetric compact storage is used, hence only D and L
!     SEPCSCMATDIVU  Compute y = A^(-1) x, where A is a preconditioning matrix
!                    a symmetric compact storage is used, hence only D and L
!                    Eisenstat preconditioning
!     SEPCSCMATDIVEU Compute y = A^(-1) x, where A is a preconditioning matrix
!                    a symmetric compact storage is used, hence only D and L
!                    Eisenstat preconditioning
!     SEPZRCMATDIV   Compute y = A^(-1) y, where A is a preconditioning matrix
!                    a row-compact storage is used, hence D, L and U
!     SEPZRCMATDIVNS Compute y = A^(-1) y, where A is a preconditioning matrix
!                    a row-compact storage is used, hence D, L and U
!                    Matrix is not scaled
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcmatdiv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcmatdiv'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iprec = A%type_prec
      IM => intmt(intmat)

      if ( debug ) &
         write(irefwr,1) 'iprec, scaling ', iprec, A%scaling

      if ( iprec<=1 ) then

!     --- No action, just copy

         y = x

      else if ( iprec==2 .or. iprec==4 .or. iprec==7 ) then

!     --- Eisenstat preconditioning, only D U y = x

         if ( IM%symmetric ) then

!        --- (non-)symmetric compact matrix

            y = x
            call sepcscmatdivU ( A, y )

         else

!        --- (non-)symmetric row-compact matrix

            y = x
            call sepcscmatdiveU ( A, y )

         end if  ! ( IM%symmetric )

      else

!     --- Standard preconditioning, only A y = x

         if ( IM%symmetric ) then

!        --- symmetric compact matrix

            y = x
            call sepcscmatdiv ( A, y )

         else

!        --- non-symmetric row-compact matrix

            y = x
            if ( A%scaling==0 ) then
               call sepzrcmatdivns ( A, y )
            else
               call sepzrcmatdiv ( A, y )
            end if  ! ( A%scaling==0 )

         end if  ! ( IM%symmetric )

      end if  ! ( iprec<=1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcmatdiv'

      end if  ! ( debug )
      call erclos ( 'sepcmatdiv' )

      end subroutine sepcmatdiv
