      subroutine sepcopyvec ( iold, inew, iincopy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the contents of array array iold in inew
!     Also a part of the array may be copied
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iold, iincopy(*)
      integer, intent(inout) :: inew

!     iincopy      i   defines which part of the array is copied
!                      iincopy must be filled as follows:
!                      1:  Number of entries filled by the user
!                          If 0 or 1 only default values are used
!                      2:  copy_part defines whether the complete array is
!                          copied or not. Possible values:
!                          0  The complete array is copied
!                          1  Only the part not corresponding to essential
!                             boundary conditions is copied
!                          2  Only the part corresponding to essential boundary
!                             conditions is copied
!                          3  Copy the coordinates of the mesh as stored in IOLD
!                             (=kmesh) into INEW
!                      Default value: 0
!                      3:  IPROB, problem number
!                          Only used if copy_part = 3
!                      Default value: 1
!     inew         o   new array (gets the same structure as iold)
!     iold         i   old array (must be a SEPRAN array)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: itypvc, fstblk, i, copypart, j, ivec, leng
      character (len=16) :: namarr1
      logical :: sepranvector

!     copypart       Indicates which part of the array is copied, see
!                    iincopy
!     fstblk         First blank found in name
!     i              Counting variable
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     j              Counting variable
!     leng           size of array
!     namarr1         Name of array
!     sepranvector   if true the vector is a known seran vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     INISTR         find structure of vector
!     PRININ         print 1d integer array
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPCRSOLINFO   Create a new vector in ks
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong type
!    1072   Copying of a set of unknowns coupled to essential boundary
!           conditions is only possible for vectors of the type solution vector
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extract parameters from iincopy
!     if ( copypart<=2 ) then
!        standard copy
!     else
!        copy coordinates
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopyvec' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopyvec'
         call ertrace
         call prinin ( iincopy, 1, 'iincopy' )
  1      format ( a, 1x, (10i10) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Extract parameters from iincopy

      copypart = 0
      if ( iincopy(1)>1 ) copypart = max ( 0, min(3,iincopy(2)) )
      itypvc = ks(iold)%typevec

      if ( debug ) write(irefwr,1) 'copypart, itypvc', copypart, itypvc
      if ( copypart<=2 ) then

!     --- copypart<=2, standard case

         itypvc = ks(iold)%typevec
         sepranvector = itypvc==110 .or. itypvc==115 .or. &
                        itypvc==116 .or. itypvc==126 .or. &
                        itypvc==129
         if ( .not.sepranvector ) then

!        --- Not a known sepran vector

            call errint ( itypvc, 1 )
            call errsub ( 60, 1, 0, 0 )
            go to 1000

         end if  ! ( .not.sepranvector )
         if ( ierror/=0 ) go to 1000
         if ( copypart/=0 .and. itypvc/=110 ) then

!        --- Restricted copying only possible for solution vectors

            call errint ( itypvc, 1 )
            call errsub ( 1072, 1, 0, 0 )

         end if
         if ( ierror/=0 ) go to 1000

         iprob = ks(iold)%iprob
         iseqkp = kprob
         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobloc => kprobpart%kprobloc
         kprobf => kprobpart%kprobf
         kprobh => kprobpart%kprobh
         kprobhinv => kprobpart%kprobhinv
         indprf = kprobloc(19)
         indprh = kprobloc(50)
         call sepfillkprob

!        --- end tests

         namarr1 = namesol(iold)
         fstblk = index(namarr1,' ')
         if ( fstblk>1 ) then

!        --- Still space left at end of name

            namarr1(fstblk:9) = '_cp'

         else if ( fstblk==1 ) then

!        --- Array has not yet a name

            namarr1 = 'inew'

         end if
         if ( itypvc==129 ) then
            ivectin => ks(iold)%isol
            leng = size(ivectin)
            call sepcrsolinfo ( inew, 0, leng )
            ks(inew)%isol(1:leng) = ivectin(1:leng)
            vectin => ks(iold)%sol
            usoltemp => ks(iold)%sol
            leng = size(usoltemp)
            allocate ( ks(inew)%sol(leng) )
            ks(inew)%sol(1:leng) = vectin(1:leng)
            ks(inew)%typevec = itypvc
            ks(inew)%compl = ks(iold)%compl
            ks(inew)%nusol = ks(iold)%nusol
            ks(inew)%iprob = ks(iold)%iprob
            ks(inew)%iseqkp = iseqkp
            ks(inew)%ivec = ks(iold)%ivec
            ks(inew)%n = ks(iold)%ivec
            ks(inew)%jcart = ks(iold)%jcart
            ks(inew)%nphys = ks(iold)%nphys
            go to 1000

         end if  ! ( itypvc==129 )
         if ( copypart/=0 ) then
            indprhlc = indprh
         else
            indprhlc = 0
         end if
         usolt => ks(iold)%sol
         call sepcreasolbf1 ( inew, itypvc, ks(iold)%ivec, namarr1, &
                              ks(iold)%compl )
         if ( copypart==0 ) then

!        --- Copy complete array

            usoltemp = usolt

         else if ( copypart==1 ) then

!        --- Copy only internal part of array

            leng = size(usolt) - kprobloc(10) - kprobloc(34)
            if ( indprhlc==0 ) then
               usoltemp(1:leng) = usolt(1:leng)
            else
               do i = 1, leng
                  j = kprobhinv(i)
                  usoltemp(j) = usolt(j)
               end do
            end if

         else

!        --- Copy only boundary conditions

            leng = size(usoltemp)
            if ( indprhlc==0 ) then
               do i = leng-kprobloc(10)+1, leng
                  usoltemp(i) = usolt(i)
               end do
            else
               do i = leng-kprobloc(10)+1, leng
                  j = kprobhinv(i)
                  usoltemp(j) = usolt(j)
               end do
            end if

         end if

!        --- copy extra information for this type of vectors

         ks(inew)%typevec = itypvc
         ks(inew)%n = ks(iold)%n
         ks(inew)%jcart = ks(iold)%jcart
         ks(inew)%nphys = ks(iold)%nphys
         ks(inew)%iprob = ks(iold)%iprob

      else

!     --- copypart = 3, copy coordinates

         itypvc = 0
         iprob = 1
         if ( iincopy(1)>2 ) iprob = iincopy(3)

!        --- Find type of array corresponding to ndim degrees of freedom

         call inistr ( ndim,itypvc, ivec )
         if ( ierror/=0 ) go to 1000

!        --- Create array

         namarr1 = 'coor-cp'
         call sepcreasolbf1 ( inew, itypvc, ivec, namarr1, ks(iold)%compl )
         if ( ierror/=0 ) go to 1000

!        --- Copy

         usoltemp => ks(inew)%sol
         leng = size(usoltemp)
         call dcopy ( leng, coor, 1, usoltemp, 1 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopyvec'

      end if
      call erclos ( 'sepcopyvec' )

      end
