      subroutine sepcpolgmr ( A, x, b, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     innerloop for GMRESR.
!     It uses the GMRES method with preconditioning.
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     parallel
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(in) :: A
      double complex, intent(inout) :: work(A%neq,*), x(A%neq)
      double complex, intent(in) :: b(A%neq)

!     A              i    Matrix A
!     b              i    Right-hand side
!     work                Work array, see sepgmresr
!     x             i/o   Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: imax, itmax, n, j, k, inner
      double precision rnorm
      double complex t, hh(101,100), c(100), s(100), rs(101)

!     c              Array determining the plane rotations which transform the
!                    upper Hessenberg-matrix into an upper triangular matrix.
!                    they are not
!     hh             Array containing an upper Hessenberg-matrix of dimension
!                    (i+1).
!     imax           Parameter controlling the size of the subspaces, thus
!                    determining the restarts of the outer iteration
!                    process.
!                    The iterations restart after imax iterations, &
!                    with the latest iterate as new starting vector. The
!                    parameter should be larger than zero.
!     inner          Loop variable for the inner iteration.
!     itmax          The maximum number of iterations to be performed.
!     j              Counting variable
!     k              Counting variable
!     n              number of basis functions for each component
!     rnorm          Norm of residual
!     rs             Array containing the norms of the residuals from the inner
!                    iterations.
!     s              Array determining the plane rotations which transform the
!                    upper Hessenberg-matrix into an upper triangular matrix.
!     t              Auxiliary variable.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcpolgmr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcpolgmr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      imax = max(1,min(A%ninner,100))   ! number of inner iterations
      itmax  = min(imax,100)

      n = A%neq

!     --- Set residual r = b

      work(:,1) = b

!     --- Norm of the initial residual r

      rnorm = cnorm ( work, n )
      rs(1) = rnorm

!     --- Check for convergence.

      if ( rnorm<A%eps ) goto 1000
      work(:,1) = work(:,1)/rnorm

!     --- While loop check on accuracy ( nmax+1 is used because
!           the residual of the previous iteration is always checked )

      do inner = 1, itmax

!     --- Inner iteration step, apply modified Gram-Schmidt to form a
!         new column of hh.

         work(:,inner+2) = work(:,inner)/A

         if ( A%eistat ) then
            work(:,inner+1) = work(:,inner) + A%S * work(:,inner+2)
            work(:,inner+1) = A/work(:,inner+1)
            work(:,inner+1) = work(:,inner+1)+work(:,inner+2)
         else
            work(:,inner+1) = A*work(:,inner+2)
         end if  ! ( A%eistat )

         do j = 1,inner
            t = dot_product ( work(:,j), work(:,inner+1) )
            hh(j,inner) = t
            work(:,inner+1) = work(:,inner+1) -t*work(:,j)
         end do  ! j = 1,inner
         hh(inner+1,inner) = cnorm ( work(:,inner+1), n )

!        --- Perform plane rotations on new column.

         do k = 2, inner
            t = hh(k-1,inner)
            hh(k-1,inner) =  c(k-1)*t + s(k-1)*hh(k,inner)
            hh(k,inner)   = -s(k-1)*t + c(k-1)*hh(k,inner)
         end do

!        --- Calculate new plane rotation.

         t = sqrt(hh(inner,inner)**2 + hh(inner+1,inner)**2)
         if (abs(t)<=0.0d0) t = epsmac
         c(inner) = hh(inner,inner)/t
         s(inner) = hh(inner+1,inner)/t
         rs(inner+1) = -s(inner)*rs(inner)
         rs(inner) = c(inner)*rs(inner)
         hh(inner,inner) = c(inner)*hh(inner,inner) + &
                           s(inner)*hh(inner+1,inner)

!        --- Determine residual norm.

         rnorm = abs(rs(inner+1))

!        --- Check for termination of inner iterations.

         if ( rnorm<A%eps .or. inner==itmax ) then

!        --- Compute new solution. First solve upper triangular system.

            rs(inner) = rs(inner)/hh(inner,inner)
            do k = inner-1, 1, -1
               t = rs(k)
               do j = k+1, inner
                  t = t-hh(k,j)*rs(j)
               end do
               rs(k) = t/hh(k,k)
            end do

!           --- Form linear combinations of residuals to get solution.

            do j = 1, n
               x(j) = rs(1)*work(j,1)
            end do
            do j = 2, inner
               x = x + rs(j)*work(:,j)
            end do
            goto 1000
         else
            do j = 1, n
               work(j,inner+1) = work(j,inner+1)/hh(inner+1,inner)
            end do
         end if
         if ( A%iprint>=2 ) then

!        --- iprint>=2,  Maximal output

            write ( irefwr, 350 ) inner, rnorm
350         format ( ' Inner iteration number',i10, &
                     '  residual',1pe15.2e2 )
         end if
      end do

1000  call erclos ( 'sepcpolgmr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcpolgmr'

      end if  ! ( debug )

      end subroutine sepcpolgmr
