      subroutine sepcreapreczsm ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written pA%MDermission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     The routine sepcreapreczsm computes the complex preconditioning matrix
!     for the CG-process
!     Old subroutine pzcg01
! **********************************************************************
!
!                       KEYWORDS
!
!   complex
!   conjugate_gradients
!   preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulematstr
      use sepmodulecmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, j, l, mj, ml, id, kd
      double complex :: sum1

!     i       Counting variable
!     id             Position of diagonal element i
!     j       Help variable to store pointers
!     k       Counting variable
!     kd             Position of diagonal elementk
!     l       Help variable to store pointers
!     mj      Help variabe to store positions in A%cols
!     ml      Help variabe to store positions in A%cols
!     sum1    Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCPRECEISSYM compute the preconditioning matrix for the CG-process
!                    complex matrix
!     SEPCPRECSYMGS  compute the preconditioning matrix for the CG-process
!                    complex matrix
!     SEPPRECZIGGT   Compute GG^T decomposition for system of complex
!                    linear equations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1405   This combination of the storage scheme and the preconditioner
!           is not implemented.
! **********************************************************************
!
!                       PSEUDO CODE
!
!
!     In case of ipreco = 5 we have:
!
!     The preconditioning matrix must be transformed from the form
!
!               -1
!            L D  U to L U, where L and U have the square root of the
!            diagonal on their main diagonal
!
!            So from:
!
!                                          -1
!                     | d    0    0 |   | d     0     0 |  | d    u    u   |
!                     |  11         |   |  11           |  |  11   12   13 |
!                     |             |   |       -1      |  |               |
!          A  =       | l    d    0 |   | 0    d      0 |  | 0    d    u   |
!                     |  21   22    |   |       22      |  |       22   23 |
!                     |             |   |             -1|  |               |
!                     | l    l   d  |   | 0     0    d  |  | 0    0    d   |
!                        31   32  33                  33                33
!
!            to
!                                                               _    _
!                     | sqrt(d  )  0       0    |   | sqrt(d )  u    u      |
!                     |       11                |   |       11   12   13    |
!                     |   _                     |   |                 _     |
!          A  =       |   l    sqrt(d  )   0    |   | 0    sqrt(d )   u     |
!                     |    21        22         |   |            22   23    |
!                     |   _      _              |   |                       |
!                     |   l      l    sqrt(d  ) |   | 0       0    sqrt(d ) |
!                          31     32        33                           33
!
!                                  _
!       It is easily verified that l   = l  / sqrt(d  )
!                                   ik    ik        kk
!                                  _
!                                  u   = u  / sqrt(d  )
!                                   kj    kj        kk
!
!       Afterwards the matrix is scaled by pre- and post multiplying by
!
!             -1
!       sqrt(D  ) as for the other preconditioning matrices
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcreapreczsm' )

      select case ( A%type_prec )
      case(1)

!     --- ipreco = 1   Diagonal scaling

         A%MD = sqrt(A%S(A%diag))

      case(2)

!        --- ipreco = 2,  Eisenstat preconditioning

        call sepcpreceissym ( A )

      case(3)

!        --- ipreco = 3,  Incomplete LU-decomposition

         call sepprecziggt ( A )

      case(4)

!     --- ipreco = 4,  Symmetric Gauss-Seidel

         call sepcprecsymgs ( A )

      case default

!     --- Preconditioner not available

         call errint ( A%type_storage, 1 )
         call errint ( A%type_prec, 2 )
         call errsub ( 1405, 2, 0, 0 )

      end select  ! case ( ipreco )

      call erclos ( 'sepcreapreczsm' )
      end subroutine sepcreapreczsm
