      subroutine sepkrylov ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 23-02-2016 New parameters list
!        version  1.0  date 02-04-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of linear equations
!     Real, serial case
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmoduleproj
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iprec
      double precision, allocatable :: scalmat(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     iprec          Type of preconditioner, see inpsol
!     scalmat        Diagonal of scaling matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
!     PRINRL         Print 1d real vector
!     SEPMAKEPREC    Create preconditioning matrix if necessary
!     SEPMAKESCAL    Compute a scaling matrix sqrt(D) such that all diagonal
!                    elements of the matrix sqrt(D)^-1 A sqrt(D)^-1
!                    are approximately 1
!     SEPPRINMAT6    Print matrix that is stored as row-compact real matrix
!     SEPSCALMAT     Scale the original large sparse matrix
!     SEPSCALMATBCK  Rescale the original large sparse matrix
!     SEPSOLVEMETH   Solve system of linear equations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2247   Scaling not implemented for symmetrical matrices
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepkrylov' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepkrylov'
         call sepprinmat6 ( A )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      IM => intmt(A%iseqim)
      nglob = nrusol

!     --- Allocate arrays

      allocate ( scalmat(nrusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nrusol, 'scalmat' )
      if ( ierror/=0 ) go to 1000

!     --- Scale matrix by the diagonal if necessary

      iprec = ipreco
      A%type_prec = iprec
      if ( iscaling>0 .and. iprec/=5 .and. .not. A%symmetric .and. &
           imetproj==0 ) then

!     --- Only scaling if no projection and not iprec=5

         call sepmakescal ( A, scalmat, iscaling )
         A%type_prec = 1  ! local value of iprec
         A%scaling = iscaling
         call sepscalmat ( A, scalmat, usol, rhsd )
         iprec = mod(iprec,100)

      else if ( iscaling>0 .and. iprec==5 ) then

!     --- Scaling is not used in symmetrical case

         call errwar ( 2247, 0, 0, 0 )

      end if  ! ( iscaling>0 .and. iprec/=5 .and. .not. A%symmetric )
      A%type_prec = iprec

!     --- Create preconditioning matrix if necessary

      if ( .not. IM%direct ) call sepmakeprec ( A )

!     --- Store preconditioning matrix in type matrix

      A%type_prec = mod(A%type_prec,100)

!     --- Define type of scaling

      if ( A%type_prec>0 ) then
         if ( parallel .and. typecoupl==1 ) then
            A%scaling = 0
         else if ( .not. A%symmetric .and. A%type_prec==3 ) then
            A%scaling = 3
         else
            A%scaling = 2
         end if  ! ( .not. A%symmetric ... )

      end if  ! ( iprec>0 )

!     --- Store information about block preconditioning in in sepmodulemat

      allocate ( A%work1(A%neq), stat = error )
      if ( error/=0 ) call eralloc ( error, A%neq, 'work1' )

      if ( debug ) call prinrl ( A%MD, nrusol, 'diag_pre' )

!     --- Solve system of equations

      call sepsolvemeth ( A, usol, rhsd )

!     --- Re-scale the original matrix, rhsd and the solution

      if ( iscaling>0 .and. iprec/=5 .and. .not. A%symmetric ) then

!     --- Only scaling if no projection and not iprec=5

         iprec = A%type_prec
         A%type_prec = 1      ! local value of iprec
         A%scaling = iscaling
         call sepscalmatbck ( A, scalmat, usol, rhsd )
         A%type_prec = iprec  ! reset iprec

      end if  ! ( iscaling>0 .and. iprec/=5 .and. .not. A%symmetric )

      if ( associated(A%work1) ) then
         deallocate ( A%work1, stat = error )
          if ( error/=0 ) call erdealloc ( error, 'work1' )
      end if  ! ( allocated(A%work1) )

      if ( nproj>0 ) then
         deallocate ( A%workstorage, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'workstorage' )
      end if  ! ( nproj>0 )
      if ( itotit==0 ) then
         itotit = itotit + A%numiter
      else
         itotit = itotit + A%numiter
         if ( A%iprint>=1 ) write(irefwr,210) itotit
210      format ( ' Accumulated number of iterations ', i8 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepkrylov'

      end if  ! ( debug )
      call erclos ( 'sepkrylov' )

      end subroutine sepkrylov
