      subroutine seplsciter ( dup, normrhsd, niterp, &
                              work, rhsup, ps, rud )
! ======================================================================
!
!        programmer    Guus Segal/Mehfooz ur Rehman
!        version  7.0  date 29-09-2014 New parameter list
!        version  6.0  date 03-04-2013 Extra parameter kprob
!        version  5.1  date 06-02-2013 New definition inpsol(4)
!
!   copyright (c) 2007-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform a lsc iteration as part of some outer loop
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
!     simple
!     krylov
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: niterp
      double precision, intent(in) :: normrhsd
      double precision, intent(inout) :: ps(np), rud(nu+np)
      double precision, target :: work(nu+np), dup(nu+np), rhsup(nu+np)

!     dup            o    Array containing the correction for (u,p)
!     niterp         o    Number of iterations to solve the p-equations
!     normrhsd       i    Norm of right-hand side vector
!     ps                  Work array to store p* &
!     rhsup                Right-hand side vector
!     rud                 Work array of length nu+np to store right-hand side
!                         of intermediate equation
!     work                Work array to store intermediate vectors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: worku(:), workp(:)

!     workp          Work array of length np to store intermediate vectors
!     worku          Work array of length nu to store intermediate vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPSIMPLEDPSOL Solve R dp = b_2 - D du by cg type methods
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The lsc algorithm can be formulated as follows:
!
!     Let the system of equations be:
!
!     | Q    G | |u|  = |f_u|
!     | D    0 | |p|    |f_p|
!
!     input:  Q, G, D, f_u, f_p
!     output: du, dp
!
!     Q' = diag(Q) or diag(M), with M the velocity mass matrix
!     In case of LSC we use M and in case of QLSC we use Q
!
!     solution:
!     The systems of equations are computed approximately
!
!         D Q'^-1 G p = r_p;   R = D Q'^-1 G
!                 r_p = D Q'^-1 Q  Q'^-1 G p
!         D Q'^-1 G p = f_p - r_p
!                 r_u = r_u - Gp
!                 Q u = r_u - Gp
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplsciter' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplsciter'
         call prinrl ( ru, nu, 'res_u' )
         call prinrl ( rp, np, 'res_p' )

      end if
      if ( ierror/=0 ) go to 1000

      du => dup(1:nu)
      dp => dup(nu+1:nu+np)
      rhsu => rhsup(1:nu)
      rhsp => rhsup(nu+1:nu+np)
      worku => work(1:nu)
      workp => work(nu+1:nu+np)

!     --- Define the parameter simpler is also used in case of msimpler)
!         In case of hsimple, we start with 1 step simple
!         followed by simpler steps

      niterp = 0   ! starting value of niterp

!     --- Solve R p = r_p    by cg type methods, store in du(nu+1 ... nu+np)
!         First rhsp := b_2

      rhsp = rp
      if ( debug ) call prinrl ( rhsp, np, 'rhsp')

      call sepsimpledpsol ( normrhsd, niterp, 1, workp )

!     --- Copy in array ps

      ps = dp
      if ( debug ) call prinrl ( ps, np, 'ps' )

!     --- Compute r_p = D Q'^-1 Q  Q'^-1 G p
!         First multiply G ps, store in work

      worku = G * ps
      if ( debug ) call prinrl ( work, nu, 'work/1' )

!     --- Compute Q'^-1  G ps, store in work

      worku = worku/qmatprime
      if ( debug ) call prinrl ( work, nu, 'work/2' )

!     --- Compute Q Q'^-1  G ps, store in rud

      rud = Q * worku
      if ( debug ) call prinrl ( rud, nu, 'rud' )

!     --- Compute Q'^-1 Q Q'^-1  G ps, store in rud

      du = -rud(1:nu)/qmatprime

!     --- Solve  R p = f_p - r_p = f_p - D Q'^-1 Q Q'^-1  G ps

      rhsp = 0d0
      call sepsimpledpsol ( normrhsd, niterp, 0, workp )

!     --- Copy into ps

      ps = dp

!     --- Now Solve Q du = r_u(1:n_u) - G ps
!         First G ps in work

      worku = G*ps

!     --- rhsu := ru - G ps

      rhsu = ru - worku

!     --- solve system of equations Q u = rhsu

      du = Q .div. rhsu
      if ( ierror/=0 ) then

!     --- The process halted because of an error

         write(irefwr,*) 'The error occurred in the Q-matrix'
         go to 1000

      end if

      if ( debug ) call prinrl ( du, nu, 'du/2' )

1000  call erclos ( 'seplsciter' )
      if ( debug ) then
         call prinrl ( dup, nu+np, 'du' )
         write(irefwr,*) 'End seplsciter'
      end if

      end
