      subroutine sepmakescal ( A, diag, iscaling )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute a scaling matrix sqrt(D) such that all diagonal elements of the
!     matrix sqrt(D)^-1 A sqrt(D)^-1 are approximately 1
!     D is defined as the row sum of the absolute values of the matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     iterative
!     matrix
!     scaling
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      integer, intent(in) :: iscaling
      double precision, intent(out) :: diag(A%neq)

!     A              i    Matrix A
!     diag           o    Diagonal of the scaling matrix
!     iscaling       i    Choice of diagonal scaling. Possible values:
!                         0: No scaling
!                         1: Only row scaling
!                         2: Row and column scaling
!                         3: Only column scaling
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      integer, pointer :: idia(:)

!     i              Counting variable
!     idia           Positions of diagonal elements
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2245   Matrix is singular
! **********************************************************************
!
!                       PSEUDO CODE
!
!     scal := |diag|
!     For each row i in the matrix do
!        scal(i) := scal(i) + |a_ij|
!     scal := sqrt(scal)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmakescal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmakescal'
         write(irefwr,1) 'iscaling', iscaling
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      idia => A%diag

      do i = 1, A%neq
         diag(i) = abs(A%S(idia(i)))
      end do  ! i = 1, A%neq

!     --- For each row i in the matrix do
!            scal(i) := scal(i) + |a_ij|

      do i = 1, A%neq
         do j =  A%rows(i)+1, idia(i)-1
            diag(A%cols(j)) = diag(A%cols(j))+abs(A%S(j))
         end do  ! j =  A%rows(i)+1, idia(i)-1
         do j = idia(i)+1, A%rows(i+1)
            diag(A%cols(j)) = diag(A%cols(j))+abs(A%S(j))
         end do  ! j = idia(i)+1, A%rows(i+1)
      end do  ! i = 1, A%neq

      do i = 1, A%neq
         if ( diag(i)<=0d0 ) then
            call errint ( i, 1 )
            call errsub (2245, 1, 0, 0 )
         end if
      end do

      if ( iscaling==2 ) diag = sqrt(diag)  ! symmetric scaling

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( diag, A%neq, 'scal' )
         write(irefwr,*) 'End sepmakescal'

      end if  ! ( debug )
      call erclos ( 'sepmakescal' )

      end subroutine sepmakescal
