      subroutine sepparadappdiag ( diag, prdiagsav, A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-05-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy diagonal of preconditioning matrix in prdiagsav
!     Initialize parallelization
!     Old subroutine prcg04par
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     preconditioning
!     scaling
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      double precision, intent(out) :: prdiagsav(A%neq)
      double precision, intent(inout) :: diag(A%neq)

!     A              i    Matrix A
!     diag          i/o   diagonal of the preconditioning matrix
!     prdiagsav      o    A copy of the diagonal of the preconditioning matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: seqblocks(iacnodes), j, sendstart, k, numneigh, &
                 iblock, icontext, nneigh
      double precision :: usend(A%neq)
      integer, pointer :: send_buffer(:)
      integer, save :: send_buffer_size

!                    they are not
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     j              Counting variable
!     k              Counting variable
!     nneigh         Number of overlapping unknowns
!     numneigh       number of neighbouring blocks
!     send_buffer    send buffer
!     send_buffer_size  Length of the send buffer
!     sendstart      starting address in array sendinfo
!     seqblocks      Work array of length nblocks (=nsurfs) which
!                    is used to find the sequence of block numbers
!                    in array blockunkinfo
!                    seqblocks(iblock) = 0: block iblock not present
!                    seqblocks(iblock) = i: block iblock is the i-th block
!                    in blockunkinfo
!     usend          Help array to be used to send reals to other processor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBARRIERMPI  Synchronize parallel sending and receiving (mpi)
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     INIRECEIVEMPI  Receive information from other node (mpi)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepparadappdiag' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepparadappdiag'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- a copy is made of the local array prdiag into prdiagsav

      prdiagsav = diag

!     --- make prdiag equal to the global prdiag

      if ( A%symmetric .or. A%scaling==2 ) then

!     --- prdiag is squared before the summation

         diag = diag**2

      end if

      numneigh = blockunkinfo(1)

!     --- Store sequence of block numbers in array seqblocks

      seqblocks = 0

      do j = 1, numneigh
         iblock =  blockunkinfo(2*j)
         seqblocks(iblock) = j
      end do

!     --- For all neighbour blocks do

      send_buffer_size = 0
      do j = 1, numneigh
         sendstart = blockunkinfo(2*j+1)
         nneigh = A%sendinfo(sendstart)
         send_buffer_size = send_buffer_size + nneigh*8
      end do

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_size, numneigh )

      allocate(send_buffer(send_buffer_size))

      call inibufattmpi ( send_buffer, send_buffer_size )

      do j = 1, numneigh
         iblock = blockunkinfo(2*j)
         sendstart = blockunkinfo(2*j+1)
         nneigh = A%sendinfo(sendstart)
         do k = 1, nneigh
            usend(k) = diag(A%sendinfo(sendstart+k))
         end do

!        --- Send array usend

         icontext = 4
         call inibsendmpi ( usend, nneigh, SPDOUBLE, iblock, icontext )

      end do

      do j = 1, numneigh

!     --- Receive next array and store in usend
!         Get corresponding block number

         icontext = 4
         nneigh = A%neq
         call inireceivempi ( usend, nneigh, SPDOUBLE, iblock, &
                              icontext )

         sendstart = blockunkinfo(2*seqblocks(iblock)+1)

         do k = 1, nneigh
            diag(A%sendinfo(sendstart+k)) = &
                diag(A%sendinfo(sendstart+k)) + usend(k)
         end do

      end do

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_size )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

      if ( A%symmetric .or. A%scaling==2 ) then

!     --- the sqrt of prdiag is taken after the summation

         diag = sqrt ( diag )

      end if

!     --- Synchronize communication by calling mpi_barrier
!         This avoids that something is written to other nodes, &
!         before reading by those other nodes has been done

      call inibarriermpi

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepparadappdiag'

      end if  ! ( debug )
      call erclos ( 'sepparadappdiag' )

      end
