      subroutine sepprofcs ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the L D L^T  decomposition of a complex symmetric profile matrix
!     Old subroutine profcs
! **********************************************************************
!
!                       KEYWORDS
!
!     complex
!     linear_solver
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulesolve
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A             i/o   Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: pj, p, ii, k1, i, j, jj
      double precision :: rowsum
      double complex, allocatable :: r(:)

!     i              General loop variable
!     ii             Position of i th diagonal element in array A%MD
!     j              General loop variable
!     jj             Position of j th diagonal element in array A%MD
!     k1             Help variable to store a quantity
!     p              Max (pi, pj)
!     pj             Column number of at most left non-zero element in row i
!     r              Work array of length nrusol to store intermediate variables
!     rowsum         Row norm of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double complex :: zdotu
      double precision :: dzasum

!     DZASUM         (CSASUM)   Sum of magnitudes of vector components
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRINMTDIR      print large matrix in case of a direct storage
!     ZDOTU          Complex inner product (no complex conjugate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     104   matrix has small pivots
!    1535   matrix is singular
!    1820   The matrix found in the solver is completely zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!               T
!      The L D L   decomposition of the matrix A is based on the following
!      formulas:
!
!              n               T
!      a   =  sum   l    d    l
!       ij    k=1    ik   kk   kj
!
!      this implies:
!
!                        j-1
!      l   = ( a   -     sum      l    d    l   ) / d      ( j=P  (1) i-1 )
!       ij      ij   k=max(P ,P )  ik   kk   jk      jj         i
!                           i  j
!
!
!                    i-1
!      d   =   a   - sum  l    d    l
!       ii      ii   k=P   ik   kk   ik
!                       i
!
!      Here P   denotes the column number of the first non-zero element in row i
!            i
!
!
!      For the practical implementation we use the work array r to store
!      products l   d
!                ik  kk
!
!
!      Hence the final algorithm becomes:
!
!
!      For i := minnum  (1)  maxnum
!
!         For j := P  (1) i-1
!                   i
!
!                              j-1
!            l   = ( a   -     sum      r   l   ) / d
!             ij      ij   k=max(P ,P )  k   jk      jj
!                                 i  j
!
!            r   = l   d
!             j     ij  jj
!
!         end for j
!
!
!                       i-1
!         d   =   a   - sum  r  l
!          ii      ii   k=P   k  ik
!                          i
!
!      end for i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprofcs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprofcs'
         write(irefwr,*) 'original matrix'
         call prinmtdir ( A%MD )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( r(nrusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nrusol, 'r' )
      if ( ierror/=0 ) go to 1000

!     --- compute row norm of part of the matrix in order to be able to check
!         the stopping criterion for the pivot process

      rowsum = 0d0
      do  i = 1, nrusol
         ii = A%Mrows(i)
         k1 = ii-(i-A%Mcols(i))
         rowsum = max ( rowsum, 2d0*dzasum (ii-k1+1, A%MD(k1), 1) &
                        -abs(A%MD(ii)) )
      end do
      if ( rowsum<=0d0 ) then
         call errsub ( 1820, 0, 0, 0 )
         go to 1000
      end if
      rowsum = rowsum*epsmac

!     --- decomposition algorithm
!         Row loop

      do i = 1, nrusol

         ii = A%Mrows(i)
         pj = A%Mcols(i)

!        --- Loop over columns

         do j = pj, i-1

            p = max ( pj, A%Mcols(j) )
            jj = A%Mrows(j)
            if ( j<=p ) then
               r(j) = A%MD(ii-i+j)
            else
               r(j) = A%MD(ii-i+j)-zdotu(j-p, r(p), 1, A%MD(jj-j+p), 1)
            end if
            A%MD(ii-i+j) = r(j) / A%MD(jj)

         end do

!        --- Diagonal element

         if ( i>pj ) A%MD(ii) = A%MD(ii) - &
                        zdotu(i-pj, r(pj), 1, A%MD(ii-i+pj), 1 )

!        --- Check pjvot element:

         if ( abs(A%MD(ii))<=0d0 ) then
            call errint ( i, 1 )
            call errsub ( 1535, 1, 0, 0 )
            go to 1000
         else if ( abs(A%MD(ii))<rowsum ) then
            call errint ( i, 1 )
            call erreal ( abs(A%MD(ii)), 1 )
            call erreal ( rowsum, 2 )
            call errwar ( 104, 1, 2, 0 )
         end if

      end do

      deallocate ( r, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'r' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'decomposition'
         call prinmtdir ( A%MD )
         write(irefwr,*) 'End sepprofcs'

      end if  ! ( debug )
      call erclos ( 'sepprofcs' )

      end subroutine sepprofcs
