      subroutine sepprofcu ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the L U  decomposition of a complex profile matrix
!     Old subroutine profcu
! **********************************************************************
!
!                       KEYWORDS
!
!     complex
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulesolve
      use sepmodulecmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A             i/o   Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: pj, p, ii, k1, k2, i, j, jj, ij, ji
      double precision :: rowsum

!     i              General loop variable
!     ii             Position of i th diagonal element in array A%MD
!     ij             Position of element ij in array A%MD
!     j              General loop variable
!     ji             Position of element ji in array A%MD
!     jj             Position of j th diagonal element in array A%MD
!     k1             Help variable to store a quantity
!     k2             Help variable to store a quantity
!     p              Max (pi, pj)
!     pj             Column number of at most left non-zero element in row i
!     rowsum         Row norm of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double complex :: zdotu
      double precision :: dzasum

!     DZASUM         (CSASUM)   Sum of magnitudes of vector components
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ZDOTU          Complex inner product (no complex conjugate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     104   matrix has small pivots
!    1535   matrix is singular
!    1820   The matrix found in the solver is completely zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!      The L U   decomposition of the matrix A is based on the following
!      formulas:
!              n
!      a   =  sum   l    u                  l   = 1
!       ij    k=1    ik   kj                 kk
!      this implies:
!                     j-1
!      u   = a   -    sum      l   u       ( j=P  (1) i-1 )
!       ji    ji   k=max(P ,P ) jk  ki          i
!                         i  j
!                        j-1
!      l   = ( a   -     sum      l    u   ) / d      ( j=P  (1) i-1 )
!       ij      ij   k=max(P ,P )  ik   kj      jj         i
!                           i  j
!                         i-1
!      u  = d   =   a   - sum  l    u
!       ii   ii      ii   k=P   ik   ki
!                            i
!      Here P   denotes the column number of the first non-zero element in row i
!            i
!     Pseudo code
!     Compute row norm of matrix and multiply by machine accuracy
!     For i = 1 (1) n
!         For j = P  (1) i-1
!                  i
!             P = max ( P , P )
!                        i   j
!             If ( P<j ) then
!                 adapt u   with inner product
!                        ji
!                 adapt l   with inner product
!                        ij
!             l   = l   / d
!              ij    ij    ii
!         adapt d   with inner product
!                ii
!         if (d  <||A|| epsmac ) then
!              ii
!            Give error message and stop the program
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprofcu' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprofcu'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- compute row norm of part of the matrix in order to be able to check
!         the stopping criterion for the pivot process

      rowsum = 0d0
      do i = 1, nrusol
         ii = A%Mrows(i)
         k1 = i-A%Mcols(i)
         rowsum = max ( rowsum, dzasum (2*k1+1, A%MD(ii-k1), 1) )
      end do
      if ( rowsum<=0d0 ) then
         call errsub ( 1820, 0, 0, 0 )
         go to 1000
      end if
      rowsum = rowsum*epsmac

!     --- decomposition algorithm
!         Row loop

      do i = 1, nrusol

         ii = A%Mrows(i)
         pj = A%Mcols(i)
         if ( pj<i ) then

!        --- pj<i

            k1 = ii+i
            k2 = ii-i

!           --- Loop over columns

            do j = pj, i-1

               p = max ( pj, A%Mcols(j) )
               jj = A%Mrows(j)
               ij = k2+j
               if ( p<j ) then

!              --- p<j

                  ji=k1-j
                  A%MD(ji) = A%MD(ji) - zdotu ( j-p, A%MD(jj-j+p), 1, &
                                        A%MD(ji+1), -1 )
                  A%MD(ij) = A%MD(ij) - zdotu ( j-p, A%MD(k2+p), 1, &
                                        A%MD(jj+1), -1 )
               end if
               A%MD(ij) = A%MD(ij)/A%MD(jj)

            end do

!           --- Diagonal element

            A%MD(ii) = A%MD(ii) - zdotu ( i-pj, A%MD(k2+pj), &
                                  1, A%MD(ii+1), -1 )
         end if

!        --- Check pjvot element:

         if ( abs(A%MD(ii))<=0d0 ) then
            call errint ( i, 1 )
            call errsub ( 1535, 1, 0, 0 )
            go to 1000
         else if ( abs(A%MD(ii))<rowsum ) then
            call errint ( i, 1 )
            call erreal ( abs(A%MD(ii)), 1 )
            call erreal ( rowsum, 2 )
            call errwar ( 104, 1, 2, 0 )
         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprofcu'

      end if  ! ( debug )
      call erclos ( 'sepprofcu' )

      end subroutine sepprofcu
