      subroutine sepprofsolsm ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-12-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ********************************************************************
!
!                       DESCRIPTION
!
!     solve system of linear equations by gaussian elimination
!     where the matrix has been decomposed by subroutine sepprofsm
!     This implies that the matrix is supposed to be a real symmetric
!     profile matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A              i    Matrix A
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer pi, i

!     i              Counting variable
!     pi             Column number of at most left non-zero element in row i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: ddot

!     DAXPY          Overwrite double precision dy with double precision
!                    da*dx + dy
!     DDOT           (CDOT)     Inner product of two vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                      Method
!
!               T
!      The L D L   decomposition of the matrix A has been computed by
!      subroutine PROFSM
!      Now the system of equations:   A x = f must be solved, hence:
!
!                                                    T
!         L D U  x = f   or:     U  x  =  y     U = L
!                                D  y  =  v
!                                L  v  =  f
!
!
!      First:  L v = f
!
!             i                                   i-1
!      f  =  sum   l    v      or:    v   = f   - sum   l   v
!       i    k=1    ik   k             i     i    k=P    ik  k
!                                                    i
!
!      Next:  D y = v          or:    y   =  v  / d
!                                      i      i    ii
!
!      Finally  U x = y
!
!             n                                    n
!      y  =  sum   u    x      or:    x   = y   - sum   u   x
!       i    k=i    ik   k             i     i    k=i+1  ik  k
!
!
!      Because of the profile storage it is not easy to get u   so the
!                                                            ik
!      order must be reversed:   x:=y
!
!      i = n (-1) 1
!         j = P  (1) i-1
!              i
!
!             x  = x  - u   x                u   = l
!              j    j    ji  i                ji    ij
!
!
!
!      Hence the final algorithm becomes:
!
!
!      For i := 1 (1) n
!
!                   i-1
!         x  = f  - sum  l   x
!          i    i   k=P   ij  j
!                      i
!
!      end for i
!
!      For i := 1 (1) n :    x  = x  / d
!                             i    i    ii
!
!      For i := n (-1) 1
!
!         For j := P  (1) i-1
!                   i
!
!            x  = x  - l   x
!             j    j    ij  i
!
!         end for j
!
!      end for i
!
!
!      In the case of super elements, it is not possible to store the
!      complete matrix into memory, and therefore each time a part of
!      the matrix is read, the corresponding part of the algorithm carried out
!      and the next part read
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!      *  Loop over rows

      usol = rhsd
      do i = 1, A%neq

         pi = A%Mcols(i)
         if ( pi<i ) &
            usol(i) = usol(i) - ddot ( i-pi, A%MD(A%Mrows(i)-i+pi), &
                                       1, usol(pi), 1 )

      end do

!   *  Next the last diagonal part
!      Hence
!                      Dy = v

      usol = usol / A%MD( A%Mrows )

!   *  Finally the back substitution
!      Hence
!                      Ux = y

      do i = A%neq, 1, -1
         pi = A%Mcols(i)
         if ( i>pi ) call daxpy ( i-pi, -usol(i), &
                          A%MD(A%Mrows(i)-i+pi), 1, usol(pi), 1 )
      end do

      end
