      subroutine sepprofsubcs ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     solve system of linear equations by gaussian elimination
!     where the matrix has been decomposed by subroutine sepprofcs
!     Old subroutine prspcs
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     complex
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulesolve
      use sepmodulematr
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A              i    Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: pj, i

!     i              Counting variable
!     pj             Column number of at most left non-zero element in row i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double complex :: zdotu

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINCOMPL      Print 1d complex vector
!     PRININ         print 1d integer array
!     SEPPRINMAT4    Print inverse matrix stored as profile complex matrix
!     ZAXPY          Overwrite complex dy with complex da*dx + dy
!     ZDOTU          Complex inner product (no complex conjugate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!               T
!      The L D L   decomposition of the matrix A has been computed by
!      subroutine SEPPROFCS
!      Now the system of equations:   A x = f must be solved, hence:
!
!                                                    T
!         L D U  x = f   or:     U  x  =  y     U = L
!                                D  y  =  v
!                                L  v  =  f
!
!
!      First:  L v = f
!
!             i                                   i-1
!      f  =  sum   l    v      or:    v   = f   - sum   l   v
!       i    k=1    ik   k             i     i    k=P    ik  k
!                                                    i
!
!      Next:  D y = v          or:    y   =  v  / d
!                                      i      i    ii
!
!      Finally  U x = y
!
!             n                                    n
!      y  =  sum   u    x      or:    x   = y   - sum   u   x
!       i    k=i    ik   k             i     i    k=i+1  ik  k
!
!
!      Because of the profile storage it is not easy to get u   so the
!                                                            ik
!      order must be reversed:   x:=y
!
!      i = n (-1) 1
!         j = P  (1) i-1
!              i
!
!             x  = x  - u   x                u   = l
!              j    j    ji  i                ji    ij
!
!
!
!      Hence the final algorithm becomes:
!
!
!      For i := 1 (1) n
!
!                   i-1
!         x  = f  - sum  l   x
!          i    i   k=P   ij  j
!                      i
!
!      end for i
!
!      For i := 1 (1) n :    x  = x  / d
!                             i    i    ii
!
!      For i := n (-1) 1
!
!         For j := P  (1) i-1
!                   i
!
!            x  = x  - l   x
!             j    j    ij  i
!
!         end for j
!
!      end for i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprofsubcs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprofsubcs'
         call princompl ( zrhsd, nrusol, 'rhsd' )
         call prinin ( A%Mrows, nrusol, 'rows' )
         call prinin ( A%Mcols, nrusol, 'cols' )
         call sepprinmat4 ( A )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
!
!     --- Carry out the part L v = f  and then D y = v
!         Loop over rows

      do i = 1, nrusol

         pj = A%Mcols(i)
         if ( pj>=i ) then

!        --- pj >= i, no summation

            zusol(i) = zrhsd(i)

         else

!        --- pj < i, summation

            zusol(i) = zrhsd(i) - zdotu ( i-pj, A%MD(A%Mrows(i)-i+pj), &
                                        1, zusol(pj), 1 )

         end if

      end do

!     --- Next the last diagonal part
!         Hence Dy = v

      do i = 1, nrusol
         zusol(i) = zusol(i) / A%MD( A%Mrows(i) )
      end do

!     --- Finally the back substitution
!         Hence Ux = y

      do i = nrusol, 1, -1
         pj = A%Mcols(i)
         if ( i.gt.pj ) call zaxpy ( i-pj, -zusol(i), &
                          A%MD(A%Mrows(i)-i+pj), 1, zusol(pj), 1 )
      end do

1000  if ( debug ) then

!     --- Debug information

         call princompl ( zusol, nrusol, 'usol' )
         write(irefwr,*) 'End sepprofsubcs'

      end if  ! ( debug )
      call erclos ( 'sepprofsubcs' )

      end subroutine sepprofsubcs
