      subroutine sepprofsubcu ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     solve system of linear equations by gaussian elimination
!     where the matrix has been decomposed by subroutine sepprofcu
!     Old subroutine prspcu
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulesolve
      use sepmodulematr
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A              i    Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: pj, i, ii

!     i              Counting variable
!     ii             Position of diagonal element ii
!     pj             Column number of at most left non-zero element in row i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double complex :: zdotu

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ZAXPY          Overwrite complex dy with complex da*dx + dy
!     ZDOTU          Complex inner product (no complex conjugate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      The L U decomposition of the matrix A has been computed by
!      subroutine SEPPROFCU
!      Now the system of equations:   A x = f must be solved, hence:
!
!                                                    T
!         L  U  x = f   or:     U  x  =  y
!                               L  y  =  f
!
!
!      First:  L y = f
!
!             i                                   i-1
!      f  =  sum   l    y      or:    y   = f   - sum   l   y
!       i    k=1    ik   k             i     i    k=P    ik  k
!                                                    i
!
!
!      Finally  U x = y
!
!             n                                      n
!      y  =  sum   u    x      or:    x   = ( y   - sum   u   x ) / u
!       i    k=i    ik   k             i       i    k=i+1  ik  k     ii
!
!
!      Because of the profile storage it is not easy to get u   so the
!                                                            ik
!      order must be reversed:   x:=y
!
!      i = n (-1) 1
!
!         x  = x  / u
!          i    i    ii
!
!         j = P  (1) i-1
!              i
!
!             x  = x  - u   x
!              j    j    ji  i
!
!
!
!      Hence the final algorithm becomes:
!
!
!      For i := 1 (1) n
!
!                   i-1
!         x  = f  - sum  l   x
!          i    i   k=P   ij  j
!                      i
!
!      end for i
!
!
!      For i := n (-1) 1
!
!         x  = x  / u
!          i    i    ii
!
!         For j := P  (1) i-1
!                   i
!
!            x  = x  - u   x
!             j    j    ji  i
!
!         end for j
!
!      end for i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprofsubcu' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprofsubcu'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Carry out the part L y = f
!         Loop over rows

      do i = 1, nrusol

         pj = A%Mcols(i)
         if ( pj>=i ) then

!        --- pj >= i, no summation

            zusol(i) = zrhsd(i)

         else

!        --- pj < i, summation

            zusol(i) = zrhsd(i) - zdotu ( i-pj, A%MD(A%Mrows(i)-i+pj), &
                                          1, zusol(pj), 1 )

         end if

      end do

!     --- Finally the back substitution
!         Hence Ux = y

      do i = nrusol, 1, -1
         ii = A%Mrows(i)
         zusol(i) = zusol(i) / A%MD(ii)
         pj = A%Mcols(i)
         if ( i>pj ) call zaxpy ( i-pj, -zusol(i), &
                          A%MD(ii+1), -1, zusol(pj), 1 )
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprofsubcu'

      end if  ! ( debug )
      call erclos ( 'sepprofsubcu' )

      end subroutine sepprofsubcu
