      subroutine seprcmatdiv ( A, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = A^(-1) y, where A is a preconditioning matrix
!     a row-compact storage is used, hence D, L and U
!     The preconditioning matrix has been created before
!     In other words the ILU inverse of A is stored
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat   ! definition of matrices
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision, intent(inout) :: y(A%neq)

!     A              i    Matrix A
!     y             i/o   Right-hand side and rolution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double precision :: sum1
      integer, pointer :: diag(:), row(:), col(:)
      double precision, pointer :: mat(:)

!     col            Column numbers of matrix
!     diag           Positions of diagonal elements of the large matrix
!     i              General loop variable
!     j              General loop variable
!     mat            elements of matrix
!     row            row numbers of matrix
!     sum1           help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINMTROWCOMP  Print matrix in case of amg row compact storage
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprcmatdiv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprcmatdiv'
         call prinrl ( y, A%neq, 'y' )
         call prinmtrowcomp ( A%MD )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      diag => A%Mdiag
      row => A%Mrows
      col => A%Mcols
      mat => A%MD

!     --- Forward substitution L y = x

      do i = 1, A%neq

         sum1 = y(i)
         do j = row(i)+1, diag(i)-1
            sum1 = sum1-mat(j)*y(col(j))
         end do  ! j = row(i)+1,diag(i)-1
         y(i) = sum1

      end do  ! i = 1, A%neq

      if ( debug ) call prinrl ( y, A%neq, 'y/1')

!     --- Finally backward substitution U y = y

      do i = A%neq-1, 1, -1

         sum1= y(i)
         do j = diag(i)+1, row(i+1)

            sum1 = sum1-mat(j)*y(col(j))

         end do  ! j = diag(i)+1, row(i+1)
         y(i) = sum1

      end do  ! i = A%neq, 2, -1

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, A%neq, 'y/2')
         write(irefwr,*) 'End seprcmatdiv'

      end if  ! ( debug )
      call erclos ( 'seprcmatdiv' )

      end subroutine seprcmatdiv
