      subroutine seprcmatdiveL ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = A^(-1) x, where A is a preconditioning matrix
!     a row-compact storage is used, hence only D and L
!     The preconditioning matrix has been created before
!     In other words a part of the ILU is executed (The U part)
!     This routine is meant for eistenstat preconditioning
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat   ! definition of matrices
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%neq)
      double precision :: y(A%neq)

!     A              i    Matrix A
!     x              i    Solution vector
!     y              o    Result  y = A^(-1) x
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double precision :: sum1

!     i              General loop variable
!     j              General loop variable
!     sum1           help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector stored
!     SEPPRINMAT9    Print matrix that is stored as row-compact real matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprcmatdiveL' )

      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprcmatdiveL'
         call prinrl ( x, A%neq, 'x' )
         call sepprinmat9 ( A )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First multiplication by diagonal:  Y = D x

      do i = 1, A%neq

         y(i) = A%S(A%diag(i))*x(i)

      end do  ! i = 1, A%neq

!     --- Next forward substitution L y = x

      do i = 1, A%neq

         sum1 = x(i)
         do j = A%rows(i)+1, A%diag(i)-1
            sum1 = sum1-A%S(j)*y(A%cols(j))
         end do
         y(i) = sum1

      end do  ! i = 1, A%neq

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, A%neq, 'y' )
         write(irefwr,*) 'End seprcmatdiveL'

      end if  ! ( debug )
      call erclos ( 'seprcmatdiveL' )

      end subroutine seprcmatdiveL
