      subroutine sepschuriter ( iterations, A )
! ======================================================================
!
!        programmer    Mehfooz ur Rehman
!        version  1.0  date 11-05-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform a schur complement iteration as part of some outer loop
!     It concerns a block preconditioning with pressure mass matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     schur_method
!     linear
!     linear_solver
!     solve
!     simple
!     krylov
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: iterations(*)
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     iterations     i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: niters(2)
      double precision, allocatable :: rhspress(:), work(:), ux(:)

!     niters
!     rhspress       Right-hand side for p-equation
!     ux             Work array of length nu to store intermediate vector
!     work           Work array of length nu to store intermediate vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPSOLVEBFIBT  Solve a system of linear equations of special form using
!                    the schur complement method
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     In this subroutine we perform the main steps of the Schur complement
!     method as described in prschurmethod
!     It concerns the following steps
!
!           Q u_f  = ru           inaccurately
!       -DQ^-1G p* = rp - D uf    inaccurately by GCR
!            Q ul  = G p*         inaccurately
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepschuriter' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepschuriter'
         write(irefwr,1) 'nu, np', nu, np
         call prinrl ( ru, nu, 'res_u' )
         call prinrl ( rp, np, 'res_p' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( rhspress(np), ux(nu), work(nu), stat = error)
      if ( error/=0 ) call eralloc ( error, nu, 'work' )
      if ( ierror/=0 ) go to 1000

!     --- Solve Q uf = ru, the result is stored in du(1:nu)

      du = Q .div. ru
      if ( ierror/=0 ) then

!     --- The process halted because of an error

         write(irefwr,*) 'The error occurred in the Q-matrix'
         go to 1000

      end if

      iterations(1) = Q%numiter
      if ( debug ) call prinrl ( du, nu, 'du/1' )
      niters(1) = 0

!     --- Store rp into rhspress

      rhspress = rp

!     --- Compute D uf store result in work(1:np)

      work(1:np) = D*du
      if ( debug ) call prinrl ( work, np, 'D uf' )

!     --- rhspress = rp - D uf

      rhspress = rp - work(1:np)

!     --- Solve -DQ^-1G p* = rp = D uf    inaccurately by GCR
!         Result in du(nu+1:nu+np)

      call sepsolvebfibt ( rhspress, niters, A )
      if ( ierror/=0 ) then

!     --- The process halted because of an error

         write(irefwr,*) 'The error occurred in the -DQ^-1G-matrix'
         go to 1000

      end if
      if ( debug ) call prinrl ( dp, np, 'p/1' )

!     --- Compute G p, store result in work(1:nu)

      work = G*dp

!     --- solve Q ul = Gp, store result in ux

      ux = Q.div.work

      iterations(2) = niters(1)
      iterations(3) = niters(2)
      iterations(4) = Q%numiter

!     --- Update u

      du = du-ux

      if ( ierror/=0 ) then

!     --- The process halted because of an error

         write(irefwr,*) 'The error occurred in the Q-matrix'
         go to 1000

      end if

1000  deallocate ( ux, rhspress, work, stat = error)
      if ( error/=0 ) call erdealloc ( error, 'work' )

      call erclos ( 'sepschuriter' )
      if ( debug ) then
         call prinrl ( du, nu+np, 'du' )
         write(irefwr,*) 'End sepschuriter'
      end if

      end
