      subroutine sepsimpleiter ( dup, normrhsd, niterp, &
                                 iter, work, rhsup, ps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform a simple iteration as part of some outer loop
!     It might be either a simple or simpler iteration
!     Old subroutine prsimpleiter
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
!     simple
!     krylov
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulesimple
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niterp, iter
      double precision normrhsd, ps(np)
      double precision, target :: dup(nu+np), rhsup(nu+np), work(nu+np)

!     dup            o    Array containing the correction for (u,p)
!     iter           i    Iteration number of GCR process
!     niterp         o    Number of iterations to solve the p-equations
!     normrhsd       i    Norm of right-hand side vector
!     ps                  Work array to store p*
!     rhsup               Right-hand side vector
!     work                Work array of length nu to store intermediate vectors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: worku(:), workp(:)
      logical simpler

!     simpler        If true simpler instead of simple is applied
!     workp          Work array of length np to store intermediate vectors
!     worku          Work array of length nu to store intermediate vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPSIMPLEDPSOL Solve R dp = b_2 - D du by cg type methods
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The simple algorithm can be formulated as follows:
!
!     Let the system of equations be:
!
!     | Q    G | |u|  = |f_u|
!     | D    C | |p|    |f_p|
!
!     input:  Q, G, D, C, f_u, f_p
!     output: du, dp
!
!     Q' = diag(Q)
!
!     solution:
!           Q u = f_u
!     (C - D Q'^-1 G) dp = f_2 - D u
!           u = u - Q'^-1 G dp
!
!     In case of simpler the system is solved in the following way:
!     The systems of equations are computed approximately
!
!      (C - D Q'^-1 G) p* = f_2 - D Q' f_u
!           Q u = f_u - Gp* &
!      (C - D Q'^-1 G) dp = f_2 - D u - C p* &
!           u = u - Q'^-1 G dp
!           p* = p* + dp
!
!     Method:
!
!     Solve Q du = f_1 by cg type methods
!     Create the matrix R = C - D Q'^-1 G
!     Solve R dp = b_2 - D du by cg type methods
!     du = du - Q'^-1 G dp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsimpleiter' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsimpleiter'
         write(irefwr,*) 'simpler ', simpler
         call prinrl ( ru, nu, 'res_u' )
         call prinrl ( rp, np, 'res_p' )
         call prinrl ( qmatprime, nu, 'qmatprime' )

      end if
      if ( ierror/=0 ) go to 1000

      du => dup(1:nu)
      dp => dup(nu+1:nu+np)
      rhsu => rhsup(1:nu)
      rhsp => rhsup(nu+1:nu+np)
      worku => work(1:nu)
      workp => work(nu+1:nu+np)

!     --- Define the parameter simpler (is also used in case of msimpler)
!         In case of hsimple, we start with 1 step simple
!         followed by simpler steps

      simpler = inpsol(3,1)==8 .or. iter>1 .and. inpsol(3,1)==12 .or. &
                inpsol(3,1)==13
      niterp = 0   ! starting value of niterp

      rhsu = ru

      if ( simpler ) then

!     --- Solve R ps = b_2 - DQ'^-1 r_u(1:n_u)   by cg type methods
!         First du = Q'^-1 r_u(1:n_u)
!         then R ps = b_2 - Ddu

         du = ru/qmatprime

!        --- First rhsp := b_2

         rhsp = rp
         if ( debug ) call prinrl ( rhsp, np, 'rhsp')

         call sepsimpledpsol ( normrhsd, niterp, 0, workp )

         ps = dp
         if ( debug ) call prinrl ( ps, np, 'dp/0' )

!        --- Now Solve Q du   = r_u(1:n_u) - G ps

         worku = G*ps

         rhsu = rhsu - worku

      end if  ! ( simpler )

!     --- solve system of equations

      du = Q .div. rhsu

      if ( ierror/=0 ) then

!     --- The process halted because of an error

         write(irefwr,*) 'The error occurred in the Q-matrix'
         go to 1000

      end if

      if ( debug ) call prinrl ( du, nu, 'du/1' )

!     --- Solve -R dp = b_2 - D us by cg type methods
!         First rhsp := b_2

      rhsp = rp(1:np)
      if ( debug ) call prinrl ( rhsp, np, 'rhsp')

      if ( simpler ) then

!     --- Compute C p* and store in work

         workp = C*ps

!        --- Subtract from rhsp

         rhsp = rhsp-workp

      end if  ! ( simpler )

      call sepsimpledpsol ( normrhsd, niterp, 0, workp )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinrl ( dup, nu+np, 'du/3' )

!     --- du = du - Q'^-1 G dp
!         work := G dp

      worku = G*dp

!     --- Multiply by Q'^-1

      worku = worku/qmatprime
      if ( debug ) call prinrl ( work, nu, 'Q^-1 G^T dp' )

!     --- subtract from du

      du = du - worku

      omega = rinsol(5,1)   ! relaxation parameter

      if ( omega>0d0 .and. abs(omega-1d0)>1d-5 ) then

!     --- omega/=0 and omega /=1, apply relaxation

         dp = omega*dp

      end if  ! ( omega>0d0 .and. abs(omega-1d0)>1d-5 )

      if ( simpler ) dp = dp + ps

1000  if ( debug ) then
         call prinrl ( dup, nu+np, 'du' )
         write(irefwr,*) 'End sepsimpleiter'
      end if
      call erclos ( 'sepsimpleiter' )

      end
