      subroutine sepsimplematr
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create the matrix R = D Q'^-1 G for the SIMPLE algorithm
!
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     matrix
!     krylov
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iwork(nrusolvel), j, k, l, l1, posU(np)
      double precision :: som
      logical :: symmat
      type (intmatrix), pointer :: IM
      integer, pointer :: idiag(:)

!     IM             Refers to intmt(intmat)
!     i              Row index
!     idiag          Position of diagonal elements in matrix
!     iwork          integer work array of length nrusolvel to store column
!                    numbers
!     j              Column index
!     k              Counting variable
!     l              General loop variable
!     l1             general loop variable
!     posU           Contains next positions of column numbers in cols to
!                    be used
!     som            Help parameter to compute a sum
!     symmat         Indicates if the matrix is symmetric (true) or
!                    not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPPRINMAT6    Print matrix that is stored as row-compact real matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The matrix R is defined as D Q'^-1 G, &
!     where D is the divergence matrix (size nu x np), &
!     Q' is the diagonal of Q (size nu) and
!     G is the gradient matrix (size np x nu) stored in transposed form
!     D and G^T may be identical in which case the matrix is symmetrical
!
!     Let q_i be the i-th diagonal element of Q'^-1 then
!
!     r_ij = d_ik q_k g_kj
!
!     Only the non-zero elements of R are stored
!
!     Code:
!
!     Clear matrix R (outside this subroutine!)
!     Clear array iwork(1:nu)
!     for i := 1 (1) np do
!        for all non zero entries k in row i of D do
!           r_ii := r_ii + d_ik q_k g_ki
!        set iwork(k) = 1, for all non-zero entries k in row i of D
!        for all columns j in the lower triangle of R that are unequal to 0 do
!           for all non zero entries k in row j of D with iwork(k)>0 do
!              r_ij := r_ij + d_ik q_k g_kj
!           if ( matrix is non-symmetrical ) then
!              for all non zero entries k in row j of D with iwork(k)>0 do
!                 r_ji := r_ji + d_jk q_k g_ki
!        clear iwork
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsimplematr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsimplematr'
         write(irefwr,1) 'np', np
         call prinin ( irowsG, np+1, 'irowsG' )
         call prinin ( icolsG, irowsG(np+1), 'icolsG' )
         call prinrl ( S_Qprime, nu, 'S_Qprime' )
         call prinrl ( S_DIV, irowsG(np+1), 'S_DIV' )
         call prinrl ( S_G, irowsG(np+1), 'S_G' )
         call prinin ( irowsDG, np+1, 'irowsDG' )
         call prinin ( icolsG, irowsDG(np+1), 'icolsDG' )
  1      format ( a, 1x, (10i6) )

      end if

      IM => intmt(iseqim)
      symmat = IM%Gsymmetric

!     --- Clear array iwork(1:nu)

      iwork = 0
      if ( IM%Gsymmetric ) then

!     --- for i := 1 (1) np do

         idiag => R%diag
         do i = 1, np

!        --- for all non zero entries k in row i of D do
!               r_ii := r_ii + d_ik q_k g_ki
!               set iwork(k) = 1

            do j = irowsG(i)+1, irowsG(i+1)

               k = icolsG(j)
               S_R(idiag(i)) = S_R(idiag(i)) + S_DIV(j)*S_G(j)/S_Qprime(k)
               iwork(k) = j

            end do  ! j = irowsG(i)+1, irowsG(i+1)

!           --- for all columns j in the lower triangle of R that are
!               unequal to 0 do

            do l = irowsDG(i)+2, irowsDG(i+1)

               j = icolsDG(l)

!              --- for all non zero entries k in row j of D with iwork(k)>0 do
!                  r_ij := r_ij + d_ik q_k g_kj

               do l1 = irowsG(j)+1, irowsG(j+1)

                  k = icolsG(l1)
                  if ( iwork(k)>0 ) S_R(l) = S_R(l)+ &
                     S_DIV(iwork(k))*S_G(l1)/S_Qprime(k)

               end do  ! l1 = irowsG(j)+1, irowsG(j+1)

            end do  ! l = irowsG(i)+1, irowsG(i+1)

!           --- clear iwork

            do j = irowsG(i)+1, irowsG(i+1)
               iwork(icolsG(j)) = 0
            end do

         end  do  ! i = 1, nrusolpress

      else


!     --- matrix is non-symmetrical
!         for i := 1 (1) np do

         idiag => IM%diagDG
         posU = R%diag+1
         do i = 1, np

!        --- for all non zero entries k in row i of D do
!               r_ii := r_ii + d_ik q_k g_ki
!               set iwork(k) = 1

           som = S_R(idiag(i))
           do j = irowsG(i)+1, irowsG(i+1)
               k = icolsG(j)
               som = som + S_DIV(j)*S_G(j)/S_Qprime(k)
               iwork(k) = j
            end do  ! j = irowsG(i)+1, irowsG(i+1)
            S_R(idiag(i)) = som

!           --- for all columns j in the lower triangle of R that are
!               unequal to 0 do

            do l = irowsDG(i)+1, idiag(i)-1

               j = icolsDG(l)

!              --- matrix is non-symmetrical
!                  for all non zero entries k in row j of D with iwork(k)>0 do
!                     r_ji := r_ji + d_jk q_k g_ki

               do l1 = irowsG(j)+1, irowsG(j+1)

                  k = icolsG(l1)
                  if ( iwork(k)>0 ) then
                     S_R(l) = S_R(l) + &
                        S_DIV(iwork(k))*S_G(l1)/S_Qprime(k)
                     S_R(posU(j)) = S_R(posU(j)) + &
                        S_DIV(l1)*S_G(iwork(k))/S_Qprime(k)
                  end if

               end do  ! l1 = irowsG(j)+1, irowsG(j+1)
               posU(j) = posU(j)+1


            end do  ! l = irowsG(i)+1, irowsG(i+1)

!           --- clear iwork

            do j = irowsG(i)+1, irowsG(i+1)
               iwork(icolsG(j)) = 0
            end do

         end  do  ! i = 1, nrusolpress

      end if  ! ( IM%Gsymmetric )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Matrix R '
         call sepprinmat6 ( R )
         write(irefwr,*) 'End sepsimplematr'
      end if
      call erclos ( 'sepsimplematr' )

      end
