      subroutine sepsolvelitref ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-10-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     iterative refinement and/or
!     Compute and print the residual
!     Old subroutine solvelitref
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulesolve
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A              i    Structure of type matrix containing the necessary
!                         information of the matrix
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision residu, normres, normrhsd
      double precision, pointer :: resid(:), ucor(:)
      logical :: finished

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     normres        Norm of residual
!     normrhsd       Norm of right-hand side vector
!     resid          residual vector
!     residu         Norm of residual
!     ucor           correction vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPMATVEC      Matrix vector multiplication for serial matrices
!     SEPPROFILESOL  Solve system of real linear equations by Gaussian
!                    elimination
!                    The matrix is supposed to be decomposed before
!     SEPRMMATRIX    Remove array with sequence number iseq from realmt
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsolvelitref' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsolvelitref'
         write(irefwr,1) 'iseqmt2, iseqsol', iseqmt2, iseqsol
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( resid(A%neq), ucor(A%neq) )

!     --- while loop iterative refinement

      niter = 0
      finished = .false.

300   if ( .not. finished ) then

!     --- Compute and print the residual
!         First res := Ax, then res = f - A x

         niter = niter + 1
         A%S => realmt(iseqmt2)%S_ii     ! Set A to original matrix
         call sepmatvec ( A, usol, resid )  ! A usol = resid
         resid =  rhsd(1:A%neq) - resid
         A%S => realmt(iseqmt)%S_ii      ! Reset A

         normres = norm2(resid)

         if ( debug ) write(irefwr,2) 'normres', normres

!        --- Print norm of residual

         if ( inpsol(12,1)==1 ) then

!        --- absolute norm

            write ( irefwr, 310 ) normres
310         format ( ' Norm of residual is ', ss, 1pe10.2e2 )

         else if ( inpsol(12,1)==2 ) then

!        --- Compute norm of right-hand side

            normrhsd = norm2(rhsd)

            if ( normrhsd<epsmac**3 ) then

               write ( irefwr, 320 ) normrhsd, epsmac**3
320            format ( ' The norm of the right-hand side is ', &
                        ss, 1pe10.2e2, ', which is less than ', &
                        1pe10.2e2/ &
                        'The relative norm is not computed ')
               write ( irefwr, 310 ) normres

            else

               write ( irefwr, 330 ) normres / normrhsd
330            format( ' Norm of residual divided by norm of right-', &
                       'hand side is ', ss, 1pe10.2e2 )
            end if

         end if

         if ( inpsol(3,1)==0 .and. inpsol(5,1)>0 ) then

!        --- Iterative refinement

            if ( niter>=inpsol(5,1) ) finished = .true.

!           --- Solve correction = A^(-1) res

            call sepprofilesol ( A, ucor, resid )

!           --- u := u + correction

            usol = usol+ucor

            residu = normres
            normres = 1d0

            if ( inpsol(6,1)>=2 ) then
               write(irefwr,340) niter, residu
340            format(' Iteration ', i4,'  Residual is ', ss, &
                      1pe11.2e2 )
            end if
            if ( residu<rinsol(1,1) ) then
               if ( inpsol(6,1)>=1 ) write(irefwr,350) niter
350            format(' Iterative improvement finished after ',i4, &
                      ' iterations' )
               finished = .true.
            end if

         else

!        --- No iterative refinement

            finished = .true.

         end if

         go to 300

      end if
      inpsol(1,1) = niter

!     --- Delete copied matrix

      if ( iseqmt2>0 ) call seprmmatrix ( iseqmt2 )
      deallocate ( resid, ucor )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsolvelitref'

      end if  ! ( debug )
      call erclos ( 'sepsolvelitref' )

      end
