      subroutine sepsymmlq ( A, x, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Apply the symmlq method of Paige en Saunders to solve the
!     linear system of equations AMAT x = RHSD      ( A x = b )
!     where the matrix A and the preconditioning matrix M should be symmetric
!     The matrix may not be singular, but it is allowed that the matrix is
!     indefinite
!     In case of a singular matrix, minres should be used, whereas for
!     a positive definite matrix CG should be applied
! **********************************************************************
!
!                       KEYWORDS
!
!     symmlq
!     iterative
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      double precision, intent(inout) :: x(A%neq), b(A%neq)

!     A              i    Matrix A
!     b              i    Right-hand side
!     x              o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: alfa, beta, beta1, bstep, cs, bsnorm, xnorm, &
                          cgnorm, dbar, delta, denom, diag, &
                          epsa, epsln, epsr, epsx, &
                          gamma, gbar, gmax, gmin, &
                          lqnorm, oldb, qrnorm, rhs1, rhs2, &
                          s, sn, snprod, t, tnorm, &
                          x1cg, ynorm2, zbar, z, bnorm, &
                          Anorm, Acond, rnorm, ynorm
      double precision, allocatable :: ybar(:), r2(:), v(:), work(:), &
                                       res(:)
      integer :: istop, iter, nmax, n
      logical :: finished

!     acond          Estimate of the condition of the (preconditioned) matrix
!     alfa           parameter in the process
!     anorm          Estimate of the norm of the (preconditioned) matrix
!     beta           parameter in the process
!     beta1          parameter in the process
!     bnorm          Norm of the right-hand side (b,b)
!     bsnorm         Preconditioned norm of the right-hand side
!                    (M^(-1)b,M^(-1)b)
!     bstep          parameter in the process
!     cgnorm         norm to check convergence
!     cs             parameter in the process
!     dbar           parameter in the process
!     delta          parameter in the process
!     denom          Denominator
!     diag           parameter in the process
!     epsa           eps * anorm
!     epsln          eps * ynorm * anorm
!     epsr           eps1 * ynorm * anorm
!     epsx           parameter in the process
!     finished       if true the iteration has finished
!     gamma          parameter in the process
!     gbar           parameter in the process
!     gmax           parameter in the process
!     gmin           parameter in the process
!     istop          If not zero, the process must be terminated
!     iter           Iteration sequence number
!     lqnorm         norm of right-hand side
!     n              number of equations
!     nmax           The maximum number of iterations that may be performed
!     oldb           Old value of beta
!     qrnorm         norm
!     r2             Work array of length n
!     res            Work array of length n
!     rhs1           parameter in the process
!     rhs2           parameter in the process
!     rnorm          An estimate of the norm of the final
!                    transformed residual vector
!     s              parameter in the process
!     sn             parameter in the process
!     snprod         parameter in the process
!     t              parameter in the process
!     tnorm          parameter in the process
!     v              Work array of length n
!     work           Work array of length n
!     x1cg           parameter in the process
!     xnorm          Norm of the estimated solution
!     ybar           Work array of length n
!     ynorm          An estimate of the norm of xbar.
!                    This is sqrt( x'Mx ).  If precon is false, &
!                    ynorm is an estimate of norm(x).
!     ynorm2         parameter in the process
!     z              parameter in the process
!     zbar           parameter in the process
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPCGNORMS     Compute some norms necessary for the termination criterion
!                    of the cg proces
!     SEPCGSTART     Fill starting value in array x and compute initial residual
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
!     677   The maximal accuracy with respect to the condition of the
!           matrix is reached
!    2814   Preconditioner non-positive
!    2815   the matrix is ill-conditioned
!    2816   The solution of the system of equations has converged to an
!           eigenvector of the matrix
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsymmlq' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsymmlq'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      n = A%neq

      bnorm = norm(b, A)
      if ( bnorm<=0d0 ) then

!     --- Right-hand side is equal to zero

         x = 0d0
         go to 1000  ! rhs is equal to 0

      end if  ! ( bnorm<=0d0 )

      allocate(ybar(n), r2(n), v(n), work(n), res(n), stat = error)
      if ( error/=0 ) call eralloc ( error, 5*n, 'work' )
      if ( ierror/=0 ) go to 1000

      nmax   = A%nmax       ! maximum number of iterations
      if ( nmax==0 ) nmax=10*n

      if ( debug ) write(irefwr,1) 'n, nmax', n, nmax

!     --- Compute norm of right-hand side (squared)
!         Some other norms with respect to the termination criterion are
!         computed as well

      call sepcgnorms ( x, b, A, bnorm, bsnorm, xnorm, work )
      if ( bnorm<=0d0 ) go to 1000  ! right-hand side zero, hence solution 0

      A%eps = 2d0*epsmac   ! adapted machine accuracy

!     --- Print heading if iprint>0

      if ( A%iprint>0 ) write ( irefwr, 200 )

200   format(//' Output for the SYMMLQ method', &
               ' (symmetric indefinite matrix)'//)

!     --- initialize

      istop  = 0
      Anorm  = 0d0
      Acond  = 0d0
      rnorm  = 0d0
      ynorm  = 0d0

!     --- Set start vector and initial residu

      call sepcgstart ( A, x, b, res, work )

      ybar = res
      if ( A%precon ) ybar = res/A

      beta1  = dotprod ( ybar, res, A )

      if ( debug ) write(irefwr,2) 'beta1', beta1

!     --- Test for an indefinite preconditioner.

      if ( beta1<0d0 ) then

!     --- Preconditioner looks non-positive, stop process

         call errsub ( 2814, 0, 0, 0 )
         go to 1000

      end if  ! ( beta1<0d0 )
      if ( A%iprint>=2 ) write(irefwr,210)
210   format(// 5x, 'iter', 7x, 'x1(cg)', 10x, &
               'norm(r)', 5x, 'bstep', 7x, 'norm(A)', 3X, 'cond(A)')

!     --- Here and later, v is really P * (the Lanczos v).

      beta1  = sqrt( beta1 )
      s      = 1d0 / beta1
      v      = s * res

      res = A*v

!     --- Set up res for the second Lanczos vector.
!         Again, res is beta * P * v2  where  P = C**(-1).
!         res and beta will be zero or very small if b is an eigenvector.

      alfa   = dotprod ( v, res, A )
      res = res - alfa / beta1 * ybar

!     --- Make sure  r2  will be orthogonal to the first  v.

      z      = dotprod ( v, res, A )
      s      = dotprod ( v, v, A )
      res = res - z / s * v

      r2 = res

      if ( A%precon ) r2 = res/A

      oldb   = beta1
      beta   = dotprod ( r2, res, A )

      if ( beta<0d0 ) then

!     --- Preconditioner looks non-positive, stop process

         call errsub ( 2814, 0, 0, 0 )
         go to 1000

      end if  ! ( beta<0d0 )

!     --- Cause termination (later) if beta is essentially zero.

      beta   = sqrt( beta )
      if ( beta<=A%eps ) then
         istop = -1
      end if

!     --- See if the local reorthogonalization achieved anything.

      denom  = sqrt( s ) * norm( r2, A )  +  A%eps
      s      = z / denom
      t      = dotprod ( v, r2, A )/denom

!     --- Initialize other quantities.

      cgnorm = beta1
      gbar   = alfa
      dbar   = beta
      rhs1   = beta1
      rhs2   = 0d0
      bstep  = 0d0
      snprod = 1d0
      tnorm  = alfa**2 + beta**2
      ynorm2 = 0d0
      gmax   = abs( alfa ) + A%eps
      gmin   = gmax

      work = v

!     --- Main iteration loop.

      do iter = 1, nmax

         Anorm  = sqrt( tnorm  )
         ynorm  = sqrt( ynorm2 )
         epsa   = Anorm * A%eps
         epsx   = Anorm * ynorm * A%eps
         epsr   = Anorm * ynorm * A%eps1
         diag   = gbar
         if ( abs(diag)<=0d0 ) diag = epsa

         lqnorm = sqrt( rhs1**2 + rhs2**2 )
         qrnorm = snprod * beta1
         cgnorm = qrnorm * beta / abs( diag )
         A%numiter = iter

!        --- Estimate  cond(A).
!            In this version we look at the diagonals of  L  in the
!            factorization of the tridiagonal matrix,  T = L*Q.
!            Sometimes, T(k) can be misleadingly ill-conditioned when
!            T(k+1) is not, so we must be careful not to overestimate Acond.

         if ( lqnorm<=cgnorm ) then
            Acond  = gmax / gmin
         else
            denom  = min( gmin, abs( diag ) )
            Acond  = gmax / denom
         end if
         if ( Acond >=0.1d0/A%eps ) then

!        --- Matrix is ill-conditioned, stop computing

            call erreal ( acond, 1 )
            call erreal ( 0.1d0/A%eps, 2 )
            call errsub ( 2815, 0, 2, 0 )
            go to 1000

         end if  ! ( Acond >=0.1d0/A%eps )

!        --- See if any of the stopping criteria are satisfied.
!            In rare cases, istop is already -1 from above (Abar = const * I).

         if ( istop==0 ) then

!        --- istop = 0, do the checking

            if ( epsx>=beta1 ) then

!           --- epsx>=beta1: solution has converged to an eigenvector

               call errwar ( 2816, 0, 0, 0 )
               istop = 3
               finished = .true.

            end if  ! ( epsx  >=beta1 )

            if ( cgnorm<=epsx ) then

!           --- cgnorm<=epsx: maximum accuracy reached

               call erreal ( epsx, 1 )
               call errwar ( 677, 0, 1, 0 )
               istop = 2
               finished = .true.

            end if  ! ( cgnorm<=epsx )

            if ( cgnorm<=epsr ) then

!           --- cgnorm<=epsr: accuracy reached

               istop = 1
               finished = .true.

            end if  ! ( cgnorm<=epsr )

         end if  ! ( istop==0 )

         if ( A%iprint>=2 ) then

!        --- iprint>=2, maximal output

            zbar   = rhs1 / diag
            z      = (snprod * zbar  +  bstep) / beta1
            x1cg   = x(1)  +  work(1) * zbar

            write(irefwr, 220) iter, x1cg, cgnorm, bstep/beta1, &
                               Anorm, Acond
220         format(1p, i8, e19.10, e11.2, e14.5, 2e10.2)

         end if  ! ( A%iprint>=2 )
         if ( finished ) exit

!        --- Obtain the current Lanczos vector  v = (1 / beta)*res
!            and set up  res  for the next iteration.

         s = 1d0 / beta

         v = s * res

         res = A*v

         res = res - beta / oldb * ybar
         alfa   = dotprod ( v, res, A )
         res = res - alfa / beta * ybar
         ybar = r2
         r2 = res

         if ( A%precon ) r2 = res/A
         oldb   = beta
         beta   = dotprod ( r2, res, A )
         if ( beta<0d0 ) then
            istop = 6
            finished = .true.
            exit
         end if
         beta   = sqrt( beta )
         tnorm  = tnorm  +  alfa**2  +  oldb**2  +  beta**2

!        --- Compute the next plane rotation for  Q.

         gamma  = sqrt( gbar**2 + oldb**2 )
         cs     = gbar / gamma
         sn     = oldb / gamma
         delta  = cs * dbar  +  sn * alfa
         gbar   = sn * dbar  -  cs * alfa
         epsln  = sn * beta
         dbar   =            -  cs * beta

!        --- Update  x.

         z      = rhs1 / gamma
         s      = z * cs
         t      = z * sn

         x  = (work * s   +   v * t)  +  x
         work  =  work * sn  -   v * cs

!        --- Accumulate the step along the direction  rhsd, &
!            and go round again.

         bstep  = snprod * cs * z  +  bstep
         snprod = snprod * sn
         gmax   = max( gmax, gamma )
         gmin   = min( gmin, gamma )
         ynorm2 = z**2  +  ynorm2
         rhs1   = rhs2  -  delta * z
         rhs2   =       -  epsln * z

      end do  ! iter = 1, nmax

      if ( .not. finished ) then

!     --- If the subroutine reaches this stage, too much iterations have
!         been carried out

         if ( A%iprint==1 ) then

!        --- iprint = 1  Print information

            write(irefwr,250) nmax, cgnorm
250         format(' The number of executed iterations in the', &
                   ' SYMMLQ-process is', i5/ &
                   ' The 2-norm of the residual is:', ss,1pe12.2e2 )

         end if

         call errint ( nmax, 1 )
         if ( A%aterror==1) then
            A%numiter=nmax
            call errwar ( 676, 1, 0, 0 )
         else
            call errsub ( 676, 1, 0, 0 )
         end if
         go to 1000

      end if  ! ( .not. finished )

!     --- Move to the CG point if it seems better.
!         In this version of SYMMLQ, the convergence tests involve
!         only cgnorm, so we're unlikely to stop at an LQ point, &
!         EXCEPT if the iteration limit interferes.

      if (cgnorm<=lqnorm) then
         zbar   = rhs1 / diag
         bstep  = snprod * zbar  +  bstep
         ynorm  = sqrt( ynorm2  +  zbar**2 )
         rnorm  = cgnorm
         x = x + zbar*work
      else
         rnorm  = lqnorm
      end if

!     --- Display final status.

      if ( A%iprint>=1 ) then
         write(irefwr,810) A%numiter, Anorm, Acond, rnorm, ynorm
810      format ( 'The iteration process has converged after ', i5, &
                  ' steps'/ &
                  'Anorm =', e12.4, 6x, 'Acond =', e12.4 / &
                  'rnorm =', e12.4, 6x, 'ynorm =', e12.4)
      end if

      deallocate ( ybar, r2, v, work, res, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )

1000  call erclos ( 'sepsymmlq' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsymmlq'

      end if  ! ( debug )

      end subroutine sepsymmlq
