      subroutine septriiter ( dup, A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-05-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform a block triangular iteration as part of some outer loop
!     It concerns a block preconditioning with pressure mass matrix
!     Old subroutine prtriiter
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
!     simple
!     krylov
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, target :: dup(nu+np)
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     dup             o    Array containing the correction for (u,p)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, allocatable :: work(:)

!     work           Work array of length nu to store intermediate vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The tridiagonal preconditioner is used to solve the following system
!     of equations:
!
!     | Q    G | |u|  = |f_u|
!     | D    0 | |p|    |f_p|
!
!     input:  Q, G, D, f_u, f_p
!     output: du, dp
!
!     We use the following precontioner:
!
!     | Q    G   |
!     | 0   -M_p |
!
!
!     Since the preconditioner can not be inverted easily, &
!     the preconditioning system is solved iteratively with a
!     limited accuracy.
!     As a consequence the original system must be solved by GCR, since
!     the inaccurate preconditioner can be considered as one that changes
!     in each step
!
!     In this subroutine we perform the solution of the system with the
!     preconditioner, hence we solve:
!
!     - M_p  p = f_p
!     Q u = f_u - Gp
!
!     Mark that in reality we work with du, dp, ru and rp
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septriiter' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septriiter'
         write(irefwr,1) 'nu, np', nu, np
         call prinrl ( ru, nu, 'res_u' )
         call prinrl ( rp, np, 'res_p' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( work(nu), stat = error)
      if ( error/=0 ) call eralloc ( error, nu, 'work' )
      if ( ierror/=0 ) go to 1000

      du => dup(1:nu)
      dp => dup(nu+1:nu+np)

!     --- Solve M_p dp = r_2

      dp = R.div.rp
      dp = -dp

!     --- Now Solve Q dup   = r_u(1:n_u) - G  dp
!         First we compute G dp and store in work (length nu)

      work = G*dp

!     --- Next put r_u(1:n_u) - G  dp in work

      work = ru - work

!     --- solve system of equations (velocity)

      du = Q.div.work

      if ( ierror/=0 ) then

!     --- The process halted because of an error

         write(irefwr,*) 'The error occurred in the Q-matrix'
         go to 1000

      end if

1000  deallocate ( work, stat = error)
      if ( error/=0 ) call erdealloc ( error, 'work' )

      if ( debug ) then
         call prinrl ( dup, nu+np, 'dup' )
         write(irefwr,*) 'End septriiter'
      end if
      call erclos ( 'septriiter' )

      end
