      subroutine sepvecman ( ichois, idegfd, ivec1, &
                             ivec2, ivec3, const, iconst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    sepvecman is a vector manipulation subroutine
!    It replaces the old SEPRAN subroutines ALGEBR, ANORM, DIFFVC and MANIVC
!    furthermore some extra possibilities have been introduced
!    sepvecman is able to manipulate both vectors of common type (110, 115, 116)
!    as vectors of the special type 119
!    sepvecman does not exist anymore in the new SEPRAN manuals
!    It has now been replaced by subroutine MANVEC,
!    The actual tasks are performed by subroutine TO0035
!     Old subroutine vecman
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision const(*)
      integer ichois, idegfd, ivec1, ivec2, ivec3, &
              iconst(*)

!    const    i/o  In- and output real array containing constants for the
!                  manipulation depending on ichois
!    ichois    i   Choice parameter indicating what type of function
!                  is chosen by the user.  Possible values:
!                   1:    Subtract:  VEC3 = VEC1 - CONST(1)
!                   2:    CONST(1) is average value of VEC1
!                         At output IPOINT is number of points
!                   3:    Linear combination
!                         VEC3 = CONST(1) * VEC1 + CONST(2) * VEC2
!                   4:    VEC1(IDEGFD) in IPOINT is stored in CONST(1)
!                   5:    VEC1(IDEGFD) in IPOINT = CONST(1)
!                   6:    The minimum value of VEC1 is stored in CONST(1)
!                         The maximum value of VEC1 is stored in CONST(2)
!                         Real vectors only
!                   7:    The inner product VEC1 . VEC2 is stored in CONST(1)
!                         The number of points used is stored in IPOINT
!                   8:    VEC3 = CONST(1) * |VEC1|
!                         VEC1 is complex, VEC3 is real
!                   9:    VEC3 = CONST(1) * arg( VEC1 )
!                         VEC1 is complex, VEC3 is real
!                  10:    The inner product VEC1 . VEC2 is stored in CONST(1)
!                         The number of points used is stored in IPOINT
!                         Boundary conditions are excluded
!                  11:    The minimum and maximum values of the array
!                         represented by IVEC1 are computed.
!                         Besides that the co-ordinates for which these
!                         values are reached are stored in const
!                         Restriction: Only one unknown may be considered
!                                      per point
!                         At output CONST(1) contains the minimum, CONST(2)
!                         the maximum value, CONST(3) - (5), contain the
!                         co-ordinates for which the minimum is reached and
!                         CONST(6) - (8), contain the co-ordinates for which
!                         the maximum is reached. Real vectors only.
!                   12:   See 11, however, the minimum and maximum values of the
!                         absolute value of the array are used
!                         (Real and complex)
!                   13:   VEC3 is a function of VEC1 and VEC2. The user
!                         subroutine FUNALG or FUNALC is called.
!                   14:   VEC3(i) = sqrt ( VEC1(idegfd **2 + &
!                                                        i
!                                          VEC1(idegfd(2) **2  )
!                                                        i
!                         Not yet available
!                   15:   CONST(1) = || VEC1 - VEC 2 ||
!                         The type of norm must be given in ICONST(1).
!                         Possible values:
!                         1       l1-norm
!                         2       l2-norm
!                         3       max-norm
!                         4       l1-norm / number of degrees of freedom
!                         5       l2-norm / number of degrees of freedom
!                        10       max-norm  the nodal point number
!                                 corresponding to the maximum is stored in
!                                 iconst(2)
!                   16:   CONST(1) = || VEC1 ||
!                         For the type of norms available: see ICHOIS = 15
!                   17,18: reserved in to0035
!                   19:   VEC3 is a function of VEC1 and VEC2. The SEPRAN
!                         subroutine FUNADA is called.
!    iconst   i/o  In- and output integer array containing constants for the
!                  manipulation depending on ichois
!    idegfd    i   In this integer given by the user, he must store the degrees
!                  of freedom that are requested for the special purpose.
!                  In most applications only one degree of freedom per point
!                  is required, and the array may be replaced by an integer
!                  variable but not by a constant.
!                  If the variable has value 0, then all degrees of freedom
!                  are considered.
!                  If the variable has value -1, then all degrees of freedom
!                  are considered, except those corresponding to essential
!                  boundary conditions
!    ivec1     i   First input vector to be manipulated
!                  ivec1 must be a SEPRAN array of length 5, which may be
!                  of type 110, 115, 116 or 119
!    ivec2     i   Second input vector to be manipulated (if necessary)
!                  If only one array is needed, ivec2 may be replaced by a
!                  dummy
!    ivec3     i   Third input vector to be manipulated (if necessary)
!                  If only one or two arrays are needed, ivec3 may be replaced
!                  by a dummy
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypvc, idum(1), jdegfd(1), indelgrp(1)
      logical compli

!     compli         Indication if IVEC1 corresponds to a complex array (true)
!                    or to a real one (false)
!     idum           General dummy variable
!     indelgrp       dummy
!     itypvc         Type number of array
!     jdegfd         Help array to store idegfd
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     TO0022         test if vectors are of equal type
!     TO0035         Algebraic manipulations for vectors of type 110, 115
!                    and 116
!     TO0106         Algebraic manipulations for vectors of type 119
!     TO0107         Algebraic manipulations (min/max of vector)
!     TO0108         Algebraic manipulations (length of vector or extract one
!                    unknown)
!     TO0110         Algebraic manipulations (special possibilities)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!     217   IDEGFD has wrong value
!     224   input vector (INVEC1 or IVEC1) is not complex
!     314   ICONST(1)/ICHNRM has wrong value, whereas ICHOIS = 15,
!           16 or ANORM/DIFFVC has been called
!     803   Input vector (INVEC1 or IVEC1) is complex
!     855   IDEGFD = 0, whereas IINVEC(1) = 2 or 28 (ICHOIS)
!    1153   vector INVEC1 corresponds to a wrong type of vector,
!    1154   Array IVEC1 has type number 119, whereas ICHOIS = 15,
!           16 or ANORM/ DIFFVC has been called
!    1155   IDEGFD has wrong value in combination with type 119
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- tests

      call eropen ( 'sepvecman' )
      if ( idegfd<-1 ) then
         call errint ( 1, idegfd )
         call errsub ( 217, 1, 0, 0 )
      end if
      if ( ichois<=0 .or. ichois==14 .or. ichois==17 &
           .or. ichois==18 .or. ichois>19 ) then
         call errint ( 1, ichois )
         call errsub ( 1, 1, 0, 0 )
      end if
      if ( idegfd==0 .and. (ichois==5 .or. ichois==4 ) ) then
         call errint ( 1, ichois )
         call errsub ( 855, 0, 0, 0 )
      end if
      itypvc = ks(ivec1)%typevec
      if ( itypvc/=110 .and. itypvc/=115 .and. itypvc/=116 &
           .and. itypvc/=119 ) then
         call errint ( 1, itypvc )
         call errsub ( 1153, 1, 0, 0 )
      end if

      if ( ichois==15 .or.ichois==16 ) then

!     --- ichois = 15, 16; Special tests

         if ( iconst(1)<0 .or. iconst(1)>5 .and. iconst(1)/=10 ) then
            call errint ( 1, iconst(1) )
            call errsub ( 314, 1, 0, 0 )
         end if
         if ( itypvc==119 ) call errsub ( 1154, 0, 0, 0 )

      end if

!     --- Check if VEC1 is of the correct type

      if ( ks(ivec1)%compl==0 ) then

         compli = .false.

      else

         compli = .true.

      end if
      if ( compli .and. ( ichois==6 .or. ichois==11 .or. &
           ichois==14 ) ) then
         call errint ( 1, ichois )
         call errsub ( 803, 1, 0, 0 )
      end if
      if ( .not. compli .and. ( ichois==8 .or. ichois==9 ) ) then
         call errint ( ichois, 1 )
         call errsub ( 224, 1, 0, 0 )
      end if
      if ( itypvc==119 ) then

!     --- itypvc = 119

         if ( idegfd/=0 .and. idegfd/=1 ) then
            call errint ( 1, idegfd )
            call errsub ( 1155, 1, 0, 0 )
         end if

      end if

      if ( ichois==3 .or. ichois==7 .or. ichois==10 .or. &
           ichois==13 .or. ichois==15 .or. ichois==19 ) then

!     --- ichois = 3, 7, 10, 13, 15 or 19; hence array ivec2 is used
!         Check if arrays ivec1 and ivec2 are identical

         call to0022 ( ivec1, ivec2, 1, 2 )

      end if

!     --- Actual call

      idum(1) = 0
      jdegfd(1) = idegfd
      if ( ierror==0 ) then
         if ( itypvc==119 ) then
            if ( ichois/=9 ) then
               call to0106 ( ichois, jdegfd, ivec1, &
                             ivec2, ivec3, const, iconst )
            else
               call to0106 ( ichois, jdegfd, ivec1, &
                             ivec2, ivec3, const, idum )
            end if
         else
            if ( ichois==4 .or. ichois==5 .or. &
                 ichois>=15 .and. ichois<=16 .and. iconst(1)==10 ) then

!           --- Value in one point or maximum and corresponding node

               call to0110 ( ichois, jdegfd, ivec1, &
                             ivec2, const, iconst )

            else if ( ichois==11 .or. ichois==12 ) then

!           --- Compute minimum or maximum of vector

               call to0107 ( ichois, jdegfd, ivec1, const )

            else if ( ichois==14 .or. ichois==26 ) then

!           --- Compute length of vector

               call to0108 ( ichois, jdegfd, ivec1, &
                             ivec2, ivec3, iconst )

            else if ( ichois/=9 ) then
               call to0035 ( ichois, jdegfd, ivec1, &
                             ivec2, ivec3, const, iconst, &
                             0, indelgrp, .true. )
            else
               call to0035 ( ichois, jdegfd, ivec1, &
                             ivec2, ivec3, const, idum, &
                             0, indelgrp, .true. )
            end if
         end if

      end if

      call erclos ( 'sepvecman' )

      end subroutine sepvecman
