      subroutine toglob00 ( iinbld, ielhlp, elemmtloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.0  date 24-07-2015 Use sepmodulebuild
!        version  6.2  date 13-04-2015 Get nrusol from kprob(66)
!        version  6.1  date 10-02-2012 use getnpelmelgrp
!
!   copyright (c) 1998-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill part of matrix and right-hand side corresponding
!     to global unknowns
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     right_hand_side
!     global_unknown
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinbld(*), ielhlp(*)
      double precision, intent(inout) :: elemmtloc(*)

!     elemmtloc           work array to store local element matrix
!     ielhlp         i    This array of length nelgrp+5 is only used if
!                         allelements=1
!                         ielhlp(ielgrp+5) = 0 means elements of element group
!                         ielgrp  are skipped, ielhlp(ielgrp+5) = 1, means that
!                         they are assembled
!     iinbld         i    User input array for subroutine BUILD
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, icrv1, icrv2, icurnr, inunkpgr, &
                 inusolgr, isrf1, isrf2, istart, isurnr, itypegr, &
                 ipkprobx, nnodes, j, nrusolgr, iglgrp, iend, &
                 isubsf, jsurnr, nsubsf
      integer, allocatable :: colinv(:,:)

!     colinv         array of length inunkpgr * nusol containing
!                    the positions of the columns for the rows
!                    corresponding to
!                    inusolgr ... inusolgr+inunkpgr-1
!     i              Counting variable
!     icrv1          First curve number
!     icrv2          Last curve number
!     icurnr         Curve sequence number
!     iend           Last position used at a part
!     iglgrp         global element group number
!     inunkpgr       Number of global unknowns in a group
!     inusolgr       Position of first global unknown in a group with respect
!                    to solution array
!     ipkprobx       Starting address of array kprob part x
!     isrf1          First surface number
!     isrf2          Last surface number
!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
!     isurnr         Surface sequence number
!     itypegr        Type of region for a group of global unknowns
!                    1: series of curves
!                    2: series of surfaces
!                    3: series of volumes
!                    4: complete region
!     j              Counting variable
!     jsurnr         Surface sequence number
!     nnodes         Number of nodes in isolnw
!     nrusolgr       Number of free unknowns minus number of global unknowns
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELMSUBGL       Call specific element subroutine for global unknowns
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     TOGLOB01       Fill array index2 with the positions of the unknowns in the
!                    nodal points of an element
!     TOGLOB02       Add element matrix and element vector to large matrix and
!                    vector in case of global unknowns
!     TOGLOB03       Fill array icolinv with the positions of the columns for
!                    the rows corresponding to global unknowns
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2002   The possibility global groups corresponding to the complete region
!           or to volumes has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initializations
!    For all global groups do
!       Find type of elements
!       If ( type of elements = curves ) then
!          For all curves required do
!             Build element matrix and vector
!             Store matrix and vector into large matrix
!       else if ( type of elements = surfaces ) then
!          For all surfaces required do
!             Build element matrix and vector
!             Store matrix and vector into large matrix
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toglob00' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toglob00'
         call prinin ( irowsii, nrusol+1, 'irowsii' )
         call prinin ( icolsii, irowsii(nrusol+1), 'icolsii' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initializations

      matrix = addmat
      vector = addvec

      nrusolgr = nrusol-nglunk-nglcoupl
      ipkprobx = 3
      inusolgr = nrusolgr+1
      numns = nelgrp+numnat

!     --- Loop over global groups

      do iglgrp = 1, nglgrp
         ielgrp   = iglgrp+numns
         itype    = kprobx(ipkprobx+1)
         inunkpgr = kprobx(ipkprobx+2)
         itypegr  = kprobx(ipkprobx+3)
         ishape   = kprobx(ipkprobx+4)
         irelem = 0
         notvec = 0
         notmat = 0
         ifirst = 0

         if ( itype==0 ) go to 500

!        --- Compact storage, fill array colinv with positions of columns

         allocate ( colinv(inunkpgr, nusol) )
         call toglob03 ( colinv, inunkpgr, inusolgr )

         if ( itypegr==1 ) then

!        --- itypegr = 1, group refers to curves

            icrv1 = kprobx(ipkprobx+5)
            icrv2 = kprobx(ipkprobx+6)

!           --- Loop over curves

            do icurnr = icrv1, icrv2
               istart = kmeshm(icurnr+4)
               nnodes = kmeshm(icurnr+5)-istart
               istart = istart+kmeshm(2)-2
               inpelm = ishape+1

!              --- Loop over elements in curves

               do i = 1, nnodes-1, ishape

!              --- Fill array index1 with node numbers of elements

                  irelem = irelem+1
                  ielem = ielem+1
                  do j = 1, inpelm
                     index1(j) = kmeshm(istart+j)
                  end do
                  istart = istart+inpelm-1

!                 --- Fill array index2 with the sequence numbers of the
!                     unknowns and compute icount
!                     index5 contains the original numbering

                  call toglob01

!                 --- Fill element matrix and element vector

                  call elmsubgl ( coor, elemmtloc, elemvc, &
                                  index1, ishape, index4 )
                  if ( notvec==1 ) vector = .false.
                  if ( notmat==1 ) matrix = .false.
                  if ( .not. vector .and. .not. matrix ) go to 500

!                 --- Add element matrix and element vector to large matrix
!                     and vector

                  call toglob02 ( inunkpgr, inusolgr, elemmtloc, colinv )

               end do

            end do

         else if ( itypegr==2 ) then

!        --- itypegr = 2, group refers to surfaces

            isrf1 = kprobx(ipkprobx+5)
            isrf2 = kprobx(ipkprobx+6)
            if ( kelmn>0 ) then

!           --- Kmesh part n exists

               istart = 2*nsurfs
               do isurnr = 1, isrf1-1
                  istart = istart + kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
               end do

               do isurnr = isrf1, isrf2

                  inpelm = kmeshn(isurnr)
                  inelem = kmeshn(isurnr+nsurfs)
                  ishape = kmeshs(nsurfs+1+8*(isurnr-1)+2)
                  nnodes = inpelm*inelem
                  do i = 1, inelem

!                 --- Fill array index1 with node numbers of elements

                     do j = 1, inpelm
                        index1(j) = kmeshn(istart+j)
                     end do
                     istart = istart+inpelm
                     irelem = irelem+1

!                    --- Fill array index2 with the sequence numbers of the
!                        unknowns and compute icount

                     call toglob01

!                    --- Fill element matrix and element vector

                     call elmsubgl ( coor, elemmtloc, elemvc, &
                                     index1, ishape, index4 )

!                    --- Add element matrix and element vector to large matrix
!                        and vector

                     call toglob02 ( inunkpgr, inusolgr, elemmtloc, &
                                     colinv )

                  end do

               end do

            else

!           --- Kmesh part n does not exist

               do jsurnr = isrf1, isrf2

                  if ( kelmr>0 ) then

!                 --- kmesh part r exists, there are subsurfaces

                     nsubsf = kmeshr(jsurnr+1) - kmeshr(jsurnr)

                  else

!                 --- kmesh part r does not exist, there are no subsurfaces

                     nsubsf = 0

                  end if
                  do isubsf = 1, max(1,nsubsf)

                     if ( nsubsf>0 ) then
                        isurnr = abs(kmeshr(kmeshr(jsurnr)+isubsf))
                     else
                        isurnr = jsurnr
                     end if

!                    --- Compute starting and end address in kmesh part c

                     istart = sepgetkmeshc ( isurnr, iend )
                     inpelm = sepgetnpelmsurf ( isurnr, 0 )
                     istart = istart-1

                     ishape = kmeshs(nsurfs+1+8*(isurnr-1)+2)
                     nnodes = inpelm*inelem

                     do i = 1, inelem

!                    --- Fill array index1 with node numbers of elements

                        do j = 1, inpelm
                           index1(j) = kmeshc(istart+j)
                        end do
                        istart = istart+inpelm
                        irelem = irelem+1

!                       --- Fill array index2 with the sequence numbers of the
!                           unknowns and compute icount

                        call toglob01

!                       --- Fill element matrix and element vector

                        call elmsubgl ( coor, elemmtloc, elemvc, &
                                        index1, ishape, index4 )

!                       --- Add element matrix and element vector to large
!                           matrix and vector

                        call toglob02 ( inunkpgr, inusolgr, elemmtloc, &
                                        colinv )

                     end do

                  end do

               end do

            end if

         else

!        --- This possibility has not yet been implemented

            call errsub ( 2002, 0, 0, 0 )

         end if
500      inusolgr = inusolgr+inunkpgr
         ipkprobx = ipkprobx+6
         ifirst = 1

         deallocate ( colinv )

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toglob00'

      end if  ! ( debug )
      call erclos ( 'toglob00' )

      end subroutine toglob00
