      subroutine tolincombinat ( idegfd, invec1, invec2, rinvec, iresvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-11-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute linear combination of 2 vectors and store in third vector
!     The vectors may have different length
!     resvec = a vec1 + b vec2
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer idegfd, invec1(5), invec2(5), iresvc(5)
      double precision rinvec(*)

!     idegfd         i    Degree of freedom sequence number
!                         If 0 all degrees of freedom
!     invec1         i    Standard SEPRAN array containing information of the
!                         first input vector
!     invec2         i    Standard SEPRAN array containing information of the
!                         second input vector
!     iresvc        i/o   Standard SEPRAN array containing information of the
!                         output solution vector
!     rinvec         i    Real user input array for subroutine MANVEC
!                         rinvec(1:2) contain the factors a and b
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indprf1
      integer, pointer :: kprobf1(:)

!     indprf1        indication if kprob part f has been filled (>0)
!                    or not (0) for invec1
!     kprobf1        kprob part f corresponding to invec1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     TOLINCOMBINAT1 Compute linear combination of 2 vectors and
!                    store in third vector
!                    The vectors may have different length
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tolincombinat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tolincombinat'
         call prinin ( invec1, 5, 'invec1' )
         call prinin ( invec2, 5, 'invec2' )
         call prinin ( iresvc, 5, 'iresvc' )
         write(irefwr,1) 'idegfd', idegfd
         call prinrl ( rinvec, 2, 'rinvec' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- activate arrays

      call sepactsolbf2 ( invec1 )
      indprf1 = indprfi
      kprobf1 => kprobfi
      if ( indprf1==0 ) indprf1 = -nunkpi

      if ( invec2(1)/=invec1(1) ) then

!     --- Array invec2 differs from invec1

         call sepactsolbf2 ( invec2 )
         if ( indprfi==0 ) indprfi = -nunkpi

      end if  ! ( invec2(1)==invec1(1) )
      indprfo = indprfi
      kprobfo => kprobfi

      if ( iresvc(1)==invec1(1) ) then

!     --- Array iresvc is equal to invec1

         indprfi = indprf1

      else if ( iresvc(1)==invec2(1) ) then

!     --- Array iresvc is equal to invec2

         indprfi = indprfo

      else

!     --- Array iresvc differs from invec1 and invec2

         call sepactsolbf2 ( iresvc )
         if ( indprfo==0 ) indprfi = -nunkpi

      end if  ! ( invec2(1)==invec1(1) )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting addresses

      vectin => ks(invec1(1))%sol
      vectin2 => ks(invec2(1))%sol
      resvec => ks(iresvc(1))%sol
      if ( ierror/=0 ) go to 1000

!     --- Actual manipulation

      call tolincombinat1 ( vectin, vectin2, resvec, indprf1, indprfo, &
                            indprfi, kprobf1, kprobfo, kprobfi, npoint, &
                            idegfd, rinvec(1), rinvec(2) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tolincombinat'

      end if  ! ( debug )
      call erclos ( 'tolincombinat' )

      end subroutine tolincombinat
