      subroutine praitken ( nnodes, vn, vstore, itermethod )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform Aitken acceleration to an iteration process
!     The last three iterations must have been stored
! **********************************************************************
!
!                       KEYWORDS
!
!     acceleration
!     iteration
!     iterative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, itermethod
      double precision vn(nnodes), vstore(nnodes,3)

!     itermethod     i    Indicates if the Aitken method must be
!                         applied point-wise (1) or in vector form (2)
!     nnodes         i    Number of nodes in the arrays
!     vn             o    Result of the operation
!     vstore         i    Contains the last three iterations
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision mu, mu1
      double precision, allocatable :: vh(:)

!     error          Return value of allocate or deallocate
!     i              Counting variable
!     mu             parameter to compute the new vn with Aitken
!     mu1            help parameter to compute the new vn with Aitken
!     vh             Help vector to store an immediate vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3146   Division by zero in the Aitken process
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'praitken' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from praitken'
         write(irefwr,1) 'nnodes, itermethod', nnodes, itermethod
         call prinrl ( vstore(1,1), nnodes, 'vstore_1' )
         call prinrl ( vstore(1,2), nnodes, 'vstore_2' )
         call prinrl ( vstore(1,3), nnodes, 'vstore_3' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( itermethod==1 ) then

!     --- Pointwise Aitken

         do i = 1, nnodes
            mu = vstore(i,3)-vstore(i,2)
            mu1 = vstore(i,3)-2d0*vstore(i,2)+vstore(i,1)

            if ( abs(mu1)<1d-50 ) then

!           --- Error mu1 too small

               call errwar ( 3146, 0, 0, 0 )

            else

!           --- Standard case, no problem at all

               mu1 = mu/mu1
               vn(i) = vstore(i,3) - mu1*mu

            end if  ! ( abs(mu1)<1d-50 )

         end do  ! i = 1, nnodes

      else

!     --- vector Aitken


         allocate ( vh(nnodes), stat = error )
         if ( error/=0 ) call eralloc ( error, nnodes, 'vh' )
         if ( ierror/=0 ) go to 1000

         vh = vstore(:,3)-2d0*vstore(:,2)+vstore(:,1)
         mu = dot_product(vstore(:,3)-vstore(:,2),vh)
         mu1 = dot_product(vh,vh)
         if ( abs(mu1)<1d-50 ) then

!        --- Error mu1 too small

            call errwar ( 3146, 0, 0, 0 )

         else

!        --- Standard case, no problem at all

            mu = mu/mu1
            vn = vstore(:,3) - mu*(vstore(:,3)-vstore(:,2))

         end if  ! ( abs(mu1)<1d-50 )

         deallocate ( vh, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'vh' )

      end if  ! ( itermethod==1 )

1000  call erclos ( 'praitken' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'mu', mu
         call prinrl ( mu, nnodes, 'mu' )
         write(irefwr,*) 'End praitken'

      end if  ! ( debug )

      end
