      subroutine prbearmasscon ( inpbear, rinbear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 01-09-2016 New parameter list
!        version  2.0  date 17-12-2015 New parameter list
!        version  1.4  date 24-12-2014 New definition clinsl
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform the complete iteration for one time step
!     of the mass conservation scheme of Kumar
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_conservation
!     bearing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulekmesh
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpbear(*)
      double precision :: rinbear(*)

!     inpbear        i    Integer input array for bearing subroutine
!                         This array must be filled as follows:
!                          1: Sequence number of coefficients input
!                          2: Problem number
!                          3: Sequence number of pressure in solution array
!                          4: Sequence number of linear solver input
!                          5: Sequence number of reaction_force
!                             in solution array
!                          6: Maximum number of iterations
!                          7: Print_level
!                          8: ichoice, type of solution method (not yet used)
!                             1:   Kumar's mass conservation method
!                          9-10: Not yet used
!     rinbear        i    Real input array for bearing subroutine
!                         1:   Cavity pressure
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer max_iter, iseqpress, inpsolver(14), &
              nset1a, nset1b, nchanged, iter, inpsep(2), &
              iseqreac, iprint, ioutpsave, nch, numvecsav, &
              istart, iseqprob
      double precision p_cav

!     inpsep         Integer input array for subroutine prnewprob
!     inpsolver      Integer input array for subroutine prlins
!     ioutpsave      Help parameter to store the value of ioutp
!     iprint         Defines print level
!     iseqpress      Sequence number of pressure in set of solution arrays
!     iseqprob       Sequence number of problem input
!     iseqreac       Sequence number of reaction vector in array isolut
!                    If <0 also the right-hand side contribution is added
!     istart         dummy
!     iter           Iteration sequence number
!     max_iter       maximum number of iterations
!     nch            Number of characters in the name of vector "ivectr"
!     nchanged       Number of points that have moved from one set to another
!     nodes          integer work array of length npoint that is filled
!                    as follows:
!                    0: degree of freedom is unknown (belongs to set 1a)
!                    1: degree of freedom is known,belongs to set 1b
!                    2: degree of freedom is known,belongs to set 2
!                    3: degree of freedom is known,belongs to set 3 (bc)
!     nset1a         Number of unknowns in set 1a
!     nset1b         Number of unknowns in set 1b
!     numvecsav      Help parameter to store the value of numvecmn
!     p_cav          Pressure in cavitation region
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INCOMGEN       Compute matrix structure for free boundary problem
!     PRBEARMASSCON2 Detect set 2 of all points with rho<rho_liquid
!                    Make set 1a equal to the rest of the nodes and 1b equal
!                    to the empty set
!     PRBEARMASSCON3 Detect set 2 of all points with pressure<p_cav
!                    Make set 1a equal to the rest of the nodes and 1b equal
!                    to the empty set
!     PRBEARMASSCON4 Fill kprob ac with points that have prescribed pressure
!                    due to cavity
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRLINS         Build system of equations and solves linear system of
!                    equations
!     PROBDFEXT      Reads and stores problem definition
!     SEPPRINTV      Print solution vector or vector of special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2849   Process not yet converged, still nodes to be swapped
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Start:  set set 1b to 0
!             nchanged := npoint;  iter := 0
!     While ( nchanged>0 .and. iter<max_iter ) do
!        iter := iter+1
!        solve pressure and reaction force
!        Move nodes with negative pressure to set 1b
!        Move nodes with non-negative d rho /dt to set 1a
!        nchanged := number of nodes that have been moved
!        redefine problem and matrix structure
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prbearmasscon' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prbearmasscon'
         call prinin ( inpbear, 8, 'inpbear' )
         call prinrl ( rinbear, 1, 'rinbear' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iprob     = inpbear(1)           ! problem number
      max_iter  = inpbear(6)           ! maximum number of iterations
      iseqpress = inpbear(3)           ! sequence number of pressure
      iseqreac  = inpbear(5)           ! sequence number of reaction force
      iprint    = inpbear(7)           ! print level
      p_cav     = rinbear(1)           ! cavity pressure
      if ( debug ) iprint = 2

!     --- Print information if required

      nch = index(names(iseqpress),' ')-1
      if ( nch<=0 ) nch = len(names(iseqpress))

      if ( debug .or. ioutp>=0 .and. nitercom<=1 .and. itimloop<=1 ) then

!     --- Output required

         write ( irefwr, 100 ) names(iseqpress)(1:nch), iseqpress
100      format ( ' Solve vector ', a, &
                  ' with sequence number ', i3, &
                  ' by Kumars mass conservation method' )

      end if  ! ( ioutp>=0 .and. nitercom<=1 .and. itimloop<=1 )

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc

!     --- Fill array inpsolver for linear solver

      inpsolver(1) = inpbear(1)     ! sequence number of coefficients
      inpsolver(2) = iprob          ! problem number
      inpsolver(3) = iseqpress      ! sequence number of pressure
      inpsolver(4) = inpbear(4)     ! sequence  number of solver
      inpsolver(5) = 0              ! no defect correction
      inpsolver(6) = 0              ! no linear subelements
      inpsolver(7) = iseqreac       ! sequence number of reaction force
      inpsolver(8:14) = 0           ! not used

!     --- Start:  set set 1b to 0
!                 nchanged := npoint;  iter := 0

      nusol = kprobloc(5)       ! number of unknowns
      nbound = kprobloc(10)     ! number of prescribed unknowns
      nrusol = nusol-nbound
      nset1b = 0                    ! empty set
      nset1a = nusol-nbound         ! number of free unknowns
      nchanged = nusol              ! start value
      iter = 0                      ! iteration number


      pressure => ks(isolut2d(iseqpress))%sol

      allocate ( nodes(npoint) )

      if ( debug ) &
         write(irefwr,1) 'nusol, nbound, nset1a', &
                          nusol, nbound, nset1a

!     --- While ( nchanged>0 .and. iter<max_iter ) do

      do while ( nchanged>0 .and. iter<max_iter )

!     --- iter := iter+1
!         solve pressure and reaction force

         iter = iter+1
         ioutpsave = ioutp            ! save value of ioutp
         if ( iter>1 ) ioutp = -1    ! suppress output
         numvecsav = numvecmn
         if ( numvecmn==iseqreac ) numvecmn = numvecmn-1
         istart = istartmn
         istartmn = 0
         call prlins ( inpsolver )
         if ( ierror/=0 ) go to 1000
         reac => ks(isolut2d(iseqreac))%sol
         ioutp = ioutpsave            ! reset ioutp
         numvecmn = numvecsav
         istartmn = istart

         if ( iprint>1 ) then
            call sepprintv ( isolut2d(iseqpress), 'pressure' )
         end if  ! ( iprint>1 )

         if ( iprint>2 ) then
            call sepprintv ( isolut2d(iseqreac), 'reaction_force' )
         end if  ! ( iprint>1 )

!        --- Move nodes with negative pressure to set 1b
!            Move nodes with non-negative d rho /dt to set 1a
!            nchanged := number of nodes that have been moved

         if ( iter==1 ) then

!        --- First iteration, initialize arrays

            nrusol = nusol-nbound
            call prbearmasscon2 ( nset1a, nset1b, p_cav )
            nchanged = nset1b

            if ( iprint>0 ) write(irefwr,110)
110         format ( //'Iteration   number of nodes swapped' )

         else

!        --- Next iteration, adapt

            call prbearmasscon3 ( p_cav, nset1a, nset1b, nchanged )

         end if  ! ( iter==1 )
         if ( iprint>0 ) write(irefwr,120) iter, nchanged
120      format ( i6, 5x, i8 )

         if ( nchanged>0 ) then

!        --- First fill kprob part ac

            call prbearmasscon4

!           --- redefine problem and matrix structure

            ioutpsave = ioutp    ! save value of ioutp
            ioutp = -1          ! suppress output

            call probdfext ( 3, iseqprob )
            if ( ierror/=0 ) go to 1000

!           --- adapt intmat

            call incomgen ( -1, intmat1d )
            ioutp = ioutpsave    ! reset ioutp

         end if  ! ( nchanged>0 )

      end do  ! while ( nchanged>0 .and. iter<max_iter )

      if ( iprint>0 ) write(irefwr,130)
130   format ( // )

      if ( nchanged>0 ) then

!     --- warning still nodes that are swapped

         call errint ( max_iter, 1 )
         call errint ( nchanged, 2 )
         call errwar ( 2849, 2, 0, 0 )

      end if  ! ( nchanged>0 )

!     --- Remove superfluous arrays

      deallocate ( nodes )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prbearmasscon'

      end if  ! ( debug )
      call erclos ( 'prbearmasscon' )

      end
