      subroutine prcompeig ( amat, nrowmat, eigenvals, eigenvecs, &
                             nrot, amatcopy, bhelp, zhelp )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  1.0  date 15-06-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Computes the eigenvalues and vectors of the real symmetric matrix amat.
!    This program is n^3 complex so should only be used for very small amat.
!    The eigenvalues are sorted in descending order.
!    The algorithm is based on Numerical Recipies in Fortran.
! **********************************************************************
!
!                       KEYWORDS
!
!     eigenvalue
!     eigenvector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nrowmat, nrot
      double precision amat(nrowmat,nrowmat), eigenvals(nrowmat), &
                       eigenvecs(nrowmat,nrowmat), bhelp(nrowmat), &
                       zhelp(nrowmat), amatcopy(nrowmat,nrowmat)

!     amat           i    matrix from which the eigenvalues must be computed
!     amatcopy            Work array of size nrowmat x nrowmat
!     bhelp               Work array of size nrowmat
!     eigenvals      o    Array containing the eigenvalues
!     eigenvecs      o    Array containing the eigenvectors per column
!     nrot           i    Number of rotations applied
!     nrowmat        i    dimension of the matrix
!     zhelp               Work array of size nrowmat
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  ip, iq, i, j, k
      double precision c, s, t, tau, g, h, sm, tresh, theta, eps, p

!     c         the cosine of the rotation angle phi
!     eps            Local accuracy
!     g         help variable
!     h         help variable
!     i         loop variable
!     ip        loop variable
!     iq        loop variable
!     j         loop variable
!     k         loop variable
!     p         help variable
!     s         the sine of the rotation angle phi
!     sm        sum of of-diagonal elements
!     t         the tangent of the rotation angle phi
!     tau       the tangent of 2 phi
!     theta     cotangent of 2 phi
!     tresh     threshold value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2178   Too many iterations
! **********************************************************************
!
!                       PSEUDO CODE
!
!       See numerical recipies in fortran for Jacobi
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initialize

      eps = 10d0 * epsmac

      do ip=1,nrowmat
        do iq=1,nrowmat
          amatcopy(ip,iq)=amat(ip,iq)
          eigenvecs(ip,iq)=0.0d0
        end do
        eigenvecs(ip,ip)=1d0
      end do

      do ip=1,nrowmat
        bhelp(ip)=amatcopy(ip,ip)
        eigenvals(ip)=bhelp(ip)
        zhelp(ip)=0.0d0
      end do

!     --- start collecting eigenvalues and vectors

      nrot=0
      i=0
 10   i=i+1
      if ( i.gt.50 ) then

!     --- 50 iterations should never happen

         call eropen ( 'prcompeig' )
         call erclos ( 'prcompeig' )
         call errint ( i, 1 )
         call errint ( 50, 2 )
         call errsub ( 2178, 2, 0, 0 )
         go to 1000

      else

        sm=0.0d0
        do ip=1,nrowmat-1
          do iq=ip+1, nrowmat
            sm=sm+abs(amatcopy(ip,iq))
          end do
        end do

!       --- set the threshold for accepting a rotation

        if(i.lt.4)then
          tresh=0.2*sm/nrowmat**2
        else
          tresh=0.
        end if

        do ip=1,nrowmat-1

          do iq=ip+1,nrowmat

            g=100d0*abs(amatcopy(ip,iq))

            if ( i.gt.4 .and. &
                 abs(eigenvals(ip))+g.le.abs(eigenvals(ip)) .and. &
                 abs(eigenvals(iq))+g.le.abs(eigenvals(iq)) ) then

              amatcopy(ip,iq)=0.0d0

            else if ( abs(amatcopy(ip,iq)).gt.tresh )then

              h=eigenvals(iq)-eigenvals(ip)

              if(abs(h)+g.le.abs(h))then
                t=amatcopy(ip,iq)/h
              else
                theta=0.5*h/amatcopy(ip,iq)
                t=1d0/(abs(theta)+sqrt(1d0+theta**2))
                if(theta.lt.0.)t=-t
              end if

              c=1d0/sqrt(1d0+t**2)
              s=t*c
              tau=s/(1d0+c)
              h=t*amatcopy(ip,iq)
              zhelp(ip)=zhelp(ip)-h
              zhelp(iq)=zhelp(iq)+h
              eigenvals(ip)=eigenvals(ip)-h
              eigenvals(iq)=eigenvals(iq)+h
              amatcopy(ip,iq)=0

              do j=1,ip-1
                g=amatcopy(j,ip)
                h=amatcopy(j,iq)
                amatcopy(j,ip)=g-s*(h+g*tau)
                amatcopy(j,iq)=h+s*(g-h*tau)
              end do

              do j=ip+1,iq-1
                g=amatcopy(ip,j)
                h=amatcopy(j,iq)
                amatcopy(ip,j)=g-s*(h+g*tau)
                amatcopy(j,iq)=h+s*(g-h*tau)
              end do

              do j=iq+1,nrowmat
                g=amatcopy(ip,j)
                h=amatcopy(iq,j)
                amatcopy(ip,j)=g-s*(h+g*tau)
                amatcopy(iq,j)=h+s*(g-h*tau)
              end do

              do j=1,nrowmat
                g=eigenvecs(j,ip)
                h=eigenvecs(j,iq)
                eigenvecs(j,ip)=g-s*(h+g*tau)
                eigenvecs(j,iq)=h+s*(g-h*tau)
              end do

              nrot=nrot+1
            end if
          end do
        end do

        do ip=1,nrowmat
          bhelp(ip)=bhelp(ip)+zhelp(ip)
          eigenvals(ip)=bhelp(ip)
          zhelp(ip)=0.
        end do

      end if

!     --- continue to search until converged upto machine underflow, &
!         then sort

      if (sm.gt.eps) then
         goto 10
      else

        do i=1,nrowmat-1
          k=i
          p=eigenvals(i)

          do j=i+1,nrowmat
            if(eigenvals(j).ge.p)then
              k=j
              p=eigenvals(J)
            end if
          end do

          if(k.ne.i)then
            eigenvals(k)=eigenvals(i)
            eigenvals(i)=p

            do j=1,nrowmat
              p=eigenvecs(j,i)
              eigenvecs(j,i)=eigenvecs(j,k)
              eigenvecs(j,k)=p
            end do

          end if

        end do

      end if

!     --- multiply eigenvectors such that their sign in the last
!         dimension (i.e. eigenvecs(ndim,i)) is the same as the sign
!         of the eigenvalue

      do i=1,nrowmat
         if ( sign(1d0,eigenvals(i)).ne.sign(1d0,eigenvecs(nrowmat,i)) ) &
            then
            do j=1,nrowmat
               eigenvecs(j,i) = -1d0 * eigenvecs(j,i)
            end do
         end if
      end do

1000  end
