      subroutine prcompinv01 ( iinputinv, rinputinv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 25-10-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Solve the system of equations arising in the inverse problem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     inverse_problem
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputinv(*)
      double precision rinputinv(*)

!     iinputinv      i    Integer input array for subroutine prcompinv
!                         Contents
!                          1: Largest entry that has been filled by the user
!                          2: iseqsl:  sequence number of input for linear
!                                      solver
!                          3: iseqcf:  sequence number of input for coefficients
!                          4: iveccapacity: sequence number of the capacity
!                                           vector in array isolut
!                          5: ibegincv: curve number of first electrode
!                          6: iendcv : curve number of last electrode
!                          7: ivecsol: sequence number of the solution
!                                      vector in array isolut
!                          8: ivecreact: sequence number of the reaction
!                                        vector in array isolut
!                          9: ielgrploc: element group sequence number
!                                        corresponding to inverse problem
!                         10: iveceps: sequence number in array solut of
!                                      array in which the values of epsilon
!                                      are stored
!                         11: method: type of method to solve the
!                                     inverse problem
!     rinputinv      i    Double precision input array for subroutine
!                         prcompinv
!                         Contents
!                          1: value of delta_eps
!                          2: value of regularization parameter mu
!                          3: value of eps_ref
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ibegincv, iendcv, nelectrodes, ielgrploc, &
              iveccapacity,  iveceps, lengcvec, method
      double precision, pointer :: capacitycp(:)

!     capacitycp     Copy of capacity
!     ibegincv       First curve number
!     ielgrploc      local element group number
!     iendcv         Last curve number
!     iveccapacity   Sequence number of capacity vector in isol
!     iveceps        Sequence number of array epsilon in isol
!     lengcvec       Length of the capacity vectors
!     method         see USER INPUT in subroutine ELM600
!     nelectrodes    Number of electrodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRSOLVEINV01   Solve system of equations for an inverse problem using the
!                    formula (S^T S + mu I) eps = S^T C*
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Solve inverse problem depending on the method used
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompinv01' )

!     --- Solve inverse problem depending on the method used

      method = 1
      ibegincv = 1
      iendcv   = 2
      ielgrploc = 1
      iveccapacity = 3
      iveceps = 4
      if ( iinputinv(1)>3 ) iveccapacity = iinputinv(4)
      if ( iinputinv(1)>4 ) ibegincv = iinputinv(5)
      if ( iinputinv(1)>5 ) iendcv = iinputinv(6)
      if ( iinputinv(1)>8 ) ielgrploc = iinputinv(9)
      if ( iinputinv(1)>9 ) iveceps = iinputinv(10)
      if ( iinputinv(1)>10 ) method = iinputinv(11)
      nelectrodes = iendcv-ibegincv+1

!     --- Compute length of vector C: N (N-1)/2

      lengcvec = nelectrodes*(nelectrodes-1)/2
      inelem = Topgrp(ielgrploc)%inelem

!     --- Copy c_measured to ic

      allocate ( capacitycp(size(capacity)) )
      capacitycp = capacity

      if ( method==1 ) then

!     --- Method = 1: eps = (S^T S + mu I)^-1 S^T C* &

         call prsolveinv01 ( veceps, matrixs, &
                             rinputinv, capacitycp, inelem, &
                             lengcvec, vecc0, vecc1 )

      end if

!     --- delete array matrixs

      deallocate ( matrixs, capacitycp )

      call erclos ( 'prcompinv01' )
      end subroutine prcompinv01
