      subroutine prcomppsi ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 07-08-2014 use seprmbufarr
!        version  1.3  date 12-04-2012 Extra debug
!        version  1.2  date 06-02-2012 Allow indexed vectors
!
!   copyright (c) 2009-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute pseudo stream function psi
!     If this is the first call a new problem is defined
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     stream_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscpsi
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqpsi, iseqvel, numpnts, numcurvs, numsurfs, istartpnts, &
              istartcurvs, istartsurfs, iprob
      logical :: solve_system

!                    they are not
!     iprob          Actual problem number
!     iseqpsi        sequence number of array psi in isol
!     iseqvel        sequence number of velocity in isol
!     istartcurvs    starting address of curves in inpsep
!     istartpnts     starting address of points in inpsep
!     istartsurfs    starting address of surfaces in inpsep
!     numcurvs       Number of curves
!     numpnts        Number of points
!     numsurfs       number of surfaces
!     solve_system   if true psi is computed by solving a convection equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INIVECTR       Return with sequence number of vector
!     PLOTSURF3D     Plot all elements in a three-dimensional surface
!     PRCOMPPSIMAT   Build matrix and zero right-hand side for
!                    pseudo stream function
!     PRCOMPPSISOLV  Solve linear system corresponding to
!                    pseudo stream function
!     PRCOMPPSISTART Create problem definition for psi and create array psi
!     PRININ         print 1d integer array
!     SEPRMMATRIX    Remove array with sequence number iseq from realmt
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     349   Subroutine not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create structure for arry psi and initialize psi
!     Mark all nodes on start curve
!     While ( array psi has not been filled completely ) do
!        Find all neighbor elements in free surface that are not marked
!        Compute psi in these elements
!        Mark nodes in these elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcomppsi' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcomppsi'
         call prinin ( inpsep, 5, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      solve_system = .true.     ! Compute psi by solving a system of equations

!     --- Get constants from inpsep

      iseqpsi = inivectr(inpsep(1))  ! sequence number of array psi in isol
      iseqvel = inivectr(inpsep(2))  ! sequence number of velocity in isol
      numpnts = inpsep(3)  ! number of points
      numcurvs = inpsep(4) ! number of curves
      numsurfs = inpsep(5) ! number of surfaces
      if ( debug ) then
         write(irefwr,1) 'iseqpsi, iseqvel', iseqpsi, iseqvel
         write(irefwr,1) 'numpnts, numcurvs, numsurfs', &
                          numpnts, numcurvs, numsurfs
         write(irefwr,1) 'isolut', isolut
      end if  ! ( debug )
      isolut => isolut2d(iseqpsi)

!     --- Check if problem must be created

      istartpnts = 6
      istartcurvs = istartpnts+numpnts    ! starting address of curves
      istartsurfs = istartcurvs+numcurvs  ! starting address of surfaces

      if ( isolut==0 ) then

!     --- Vector psi does not yet exist, create problem and psi vector
!         Store in array kprobloc

         if ( debug .and. ndim==3 ) call plotsurf3d ( inpsep(istartsurfs) )

         call prcomppsistart ( inpsep(istartsurfs), inpsep(istartcurvs), &
                               numcurvs, numsurfs )
         numvecmn = max(numvecmn,iseqpsi)
         if ( ierror/=0 ) go to 1000

      end if  ! ( isolut==0 )
      if ( solve_system ) then

!     --- Fill array psi by solving system of equations
!         First build matrix and right-hand side

         iprob = ks(isolut)%iprob  ! problem number
         intmat => intmat1d(iprob)
         stiffmat => stiffm(iprob:maxprb)
         call prcomppsimat ( iseqvel, iseqpsi, numvecmn )
         if ( ierror/=0 ) go to 1000

!        --- Next solve system of equations

         call prcomppsisolv
         if ( ierror/=0 ) go to 1000

!        --- Delete temporary arrays

         call seprmsolinfo ( irhsd(1) )  ! right-hand side
         call seprmmatrix ( stiffmat )

      else

!     --- Compute array psi by generalizing the psi method to 3d
!         See sep1009
!         Not available

         call errsub ( 349, 0, 0, 0 )

!        --- call prcomppsi1 ( isolut2d, kmesh, kprob, &
!            +                     iseqpsi, iseqvel, numsurfs, &
!            +                     inpsep(istartsurfs), numcurvs, &
!            +                     inpsep(istartcurvs) )

      end if

!     --- Raise istartmn

      istartmn = istartmn+6+numpnts+numcurvs+numsurfs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcomppsi'

      end if  ! ( debug )
      call erclos ( 'prcomppsi' )

      end
