      subroutine prcreatephase1 ( phaseold, temperature, tm )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 07-02-2006
!
!   copyright (c) 2005-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize phase vectors that keep track of the phase of an
!     element. Actual body of prcreatephase.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemushy
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: phaseold(*)
      double precision temperature(*), tm

!     phaseold      i/o   Array containing the phase of each element
!                         for the old situation
!                         0: element is solid
!                         1: element is liquid
!                         2: element is mushy
!     temperature    i    temperature vector of length npoint
!     tm             i    Melting temperature
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ikelmc, ielem, ielgrp, irelem, neg, node, k, pos
      double precision eps

!     eps            local accuracy
!     ielem          Element number
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     k              Counting variable
!     neg            Number of nodes with 'negative' sign
!     node           Node number
!     pos            Number of nodes with 'positive' sign
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get constants
!     For all element groups ielgrp do
!       if ( element group belongs to lower dimension ) then
!          Fill array update with zeros
!       else ( element groups has dimension ndim )
!          For all elements ielem in group do
!             if ( element is intersected ) then
!               update(ielem) := irefvl+1
!             else ( element is not intersected)
!               update(ielem) := 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcreatephase1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcreatephase1'
         write(irefwr,*) 'tm = ', tm

      end if
      if ( ierror/=0 ) go to 1000
      eps = 1d-6

!     --- Get constants

      ikelmc = 0           ! start address in kemshc
      ielem = 0            ! last element number used

      if (debug) call prinrl(temperature, npoint, 'temperature')

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

!     --- Compute some element group dependent quantities

         inelem = sepgetinelem ( ielgrp )
         inpelm = sepgetinpelm ( ielgrp )
         ishape = sepgetshape ( ielgrp )
         if ( ndim==1 .and. ishape<1 .or. &
              ndim==2 .and. ishape<3 .or. &
              ndim==3 .and. ishape<10 ) then

!        --- Element of lower dimension
!            raise ikelmc and ielem
!            Fill array update with zeros

            do irelem = 1, inelem
               phaseold(ielem+irelem) = 0
            end do  ! irelem = 1, inelem
            ikelmc = ikelmc+inelem*inpelm
            ielem = ielem+inelem

         else

!        --- Not an element of lower dimension
!            Loop over elements

         if (debug) print *, 'Start loop over elements'
            if ( abs(Tm)<eps) then
               tsol = -tdelta
               tliq = tdelta
            else
               tsol = (1d0 - tdelta)*Tm
               tliq = (1d0 + tdelta)*Tm
            end if

            do irelem = 1, inelem

               if (mushy) then
                  neg = 0
                  pos = 0

                  do k = 1, inpelm
                     ikelmc = ikelmc+1
                     node = kmeshc(ikelmc)
                     if ( temperature(node)<tsol ) then
                        neg = neg + 1
                     else if ( temperature(node)>tliq ) then
                        pos = pos + 1
                     end if !
                     if (debug) print *, 'tsol, tliq', tsol, tliq
                     if (debug) print *, 'temp,pos,neg', &
                        temperature(node), pos, neg
                  end do  ! k = 1, inpelm

                  ielem = ielem+1

!                 --- mark element phase

                  if ( neg==inpelm ) then

!                 --- element is solid

                     phaseold(ielem) = 0

                  else if ( pos==inpelm ) then

!                 --- element is liquid

                     phaseold(ielem) = 1

                  else

!                 --- In this element we have an interface

                     if (debug) print *, 'Intersected element located'

                     phaseold(ielem) = 2

                  end if  ! ( neg==m .or. neg==0 )

               else

!              --- count the number of nodes that have temperature<Tm

                  neg = 0
                  do k = 1, inpelm
                     ikelmc = ikelmc+1
                     node = kmeshc(ikelmc)
                     if ( temperature(node)<Tm-eps ) then
                        neg = neg + 1
                     end if
                  end do  ! k = 1, inpelm

                  if (debug) print *, 'neg = ', neg

                  ielem = ielem+1

!                 --- mark element phase

                  if ( neg==inpelm ) then

!                 --- element is solid

                     phaseold(ielem) = 0

                  else if ( neg==0 ) then

!                 --- element is liquid

                     phaseold(ielem) = 1

                  else

!                 --- In this element we have an interface

                     if (debug) print *, 'Intersected element located'

                     phaseold(ielem) = 2

                  end if  ! ( neg==m .or. neg==0 )
               end if ! mushy
            end do  ! iielem = 1, inelem

         end if  ! ( ndim==1 .and. ishape<1 )

      end do  ! ielgrp = 1, nelgrp

      if (debug) call prinin(phaseold, nelem, 'phaseold')

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcreatephase1'

      end if
      call erclos ( 'prcreatephase1' )

      end
