      subroutine prfillinpsol ( ireadsl, inpsol, rinsol, &
                                leninpsol, lenrinsol, eps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.3  date 08-09-2014 Extra debug
!        version  1.2  date 15-11-2012 Long integers
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays inpsol and rinsol for linear solver depending on
!     input in input file or defaults
!     This subroutine is meant for time-dependent cases
! **********************************************************************
!
!                       KEYWORDS
!
!     default
!     input
!     fill
!     linear_solver
!     time_dependent
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ireadsl, inpsol(*), leninpsol, lenrinsol
      double precision rinsol(*), eps

!     eps            i    accuracy with respect to linear solver
!     inpsol         o    Integer input array for subroutine solvel
!                         See solvelbf for a description
!     ireadsl        i    Sequence number of input for linear solver
!     leninpsol      i    Declared length of array inpsol
!     lenrinsol      i    Declared length of array rinsol
!     rinsol         o    Contains information concerning the linear solver
!                         See the Programmers Guide for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipos, lenisl, lenrsl, maxcal

!                    they are not
!     ipos           Position of input arrays in sepmoduleinput
!     lenisl         Length of array inpsol
!     lenrsl         Length of array rinsol
!     maxcal         Maximum number of coefficient inputs read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHINPSOL    Adapt input array with respect to variables
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPGETINPFINF  Extract infor entries of integer and real input arrays from
!                    ISEPIN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillinpsol' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillinpsol'
         write(irefwr,1) 'ireadsl, leninpsol, lenrinsol', &
                          ireadsl, leninpsol, lenrinsol
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ireadsl>0 ) then

!     --- ireadsl>0, check if input for solve is available

         call sepgetinpfinf ( 8, ireadsl, ipos, maxcal )
         if ( ierror/=0 ) go to 1000

         if ( maxcal==0 ) then
            ireadsl = -1
         else if ( .not. in(ipos)%used ) then
            ireadsl = -1
         end if  ! ( maxcal==0 )

         if ( debug ) write(irefwr,1) 'ipos', ipos

      end if
      if ( ireadsl==-1 ) then

!     --- ireadsl = -1, use default values

         inpsol(1) = 10
         inpsol(2) = 0
         inpsol(3) = 0
         inpsol(4) = 6
         inpsol(5) = 0
         inpsol(6) = 0
         inpsol(7) = 0
         inpsol(8) = 20
         inpsol(9) = 1
         inpsol(10)= 0   ! Do not keep preconditioning matrix
         rinsol(1) = eps
         call inichkarlen ( 10, leninpsol, 'inpsol' )
         call inichkarlen ( 1, lenrinsol, 'rinsol' )

      else if ( ireadsl>0 ) then

!     --- ireadsl>0, use values stored in input

         lenisl = min(leninpsol,size( in(ipos)%iinput ))
         lenrsl = min(lenrinsol,size( re(ipos)%rinput ))

         if ( debug ) write(irefwr,1) 'lenisl, lenrsl', lenisl, lenrsl
         inpsol(1:lenisl) = in(ipos)%iinput(1:lenisl)
         rinsol(1:lenrsl) = re(ipos)%rinput(1:lenrsl)

         inpsol(1) = lenisl
         call inichinpsol ( inpsol(2), lenisl-1 )  !Adapt variables

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsol, 20, 'inpsol' )
         call prinrl ( rinsol, 20, 'rinsol' )
         write(irefwr,*) 'End prfillinpsol'

      end if
      call erclos ( 'prfillinpsol' )

      end subroutine prfillinpsol
