      subroutine prfillintfnmc2 ( numelms, nmeshctmp, istart, numbndobst, &
                                  iwork, numintersect, lennmeshc, jstart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill nmeshctmp for interface (Node numbers of the edges)
!     The local numbering of the interface is used
!     2d only and edges on the interface are skipped
! **********************************************************************
!
!                       KEYWORDS
!
!     interface
!     level_set
!     node
!     edge
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numelms, lennmeshc, nmeshctmp(lennmeshc), istart, &
              numbndobst, iwork(*),numintersect, jstart

!     istart        i/o   Starting address in icrossobst
!     iwork          i    integer work array to mark all global nodes on
!                         the interface
!                         iwork(i) = 0: node not on interface
!                         iwork(i) > 0: the node is on the interface.
!                         The value indicates the sequence number in array
!                         ipointobst
!     jstart        i/o   Next free starting address in nmeshc
!     lennmeshc     i/o   Declared length of array nmeshc
!                         At output the computed length
!     nmeshctmp        i/o   nmesh part c, contains local node numbers
!                         corresponding to local edges of interface
!     numbndobst     i    Number of nodal points on the boundary of the obstacle
!     numelms        i    Number of elements that are intersected by the
!                         interface
!     numintersect   i    Number of intersections with edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, ikelmc, inpelm, j, ih, nedgeslc
      logical debugloc

!     debugloc       If true local debug statements are carried out otherwise
!     i              Counting variable
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     j              Counting variable
!     nedgeslc         number of "local" edges
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3138   number of edges in element > 2
!    3139   number of edges in element = 1 and no interface nodes found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillintfnmc2' )
      debug = .false. .and. ioutp>=0
      debugloc = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillintfnmc2'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- For all elements do (only applicable in 2d)

      do i = 1, numelms

         nmeshctmp(i) = jstart
         nedgeslc = icrossobst(istart+2)  ! number of "local" edges
         nmeshctmp(jstart) = 2
         istart = istart+2

         if ( debugloc ) write(irefwr,1) 'i, nedgeslc', i, nedgeslc

         select case ( nedgeslc )

         case(1)

!        --- special case only 1 edge in element
!            one of the other nodes must be on the interface
!            Find this node

            if ( numbndobst==0 ) then

!           --- Only one edge found, but there are no interface nodes

               call errint ( nmeshctmp(jstart), 1 )
               call errsub ( 3139, 1, 0, 0 )
               go to 1000

            end if  ! ( numbndobst==0 )

            nmeshctmp(jstart+1) = icrossobst(istart+1)
            ielem = icrossobst(istart-1)
            ikelmc = sepstkmeshc ( ielem, inpelm )

            if ( debugloc ) then
               write(irefwr,1) 'ielem, inpelm, node1', &
                                ielem, inpelm, nmeshctmp(jstart+1)
               call prinin ( kmeshc(ikelmc), inpelm, 'element' )
               write(irefwr,1) 'iwork', &
                  iwork(kmeshc(ikelmc:ikelmc+inpelm-1))
            end if  ! ( debug )

            do j = 0, inpelm-1

               ih = iwork(kmeshc(ikelmc+j))
               if ( ih>0 ) then

!              --- Node in set of boundary points found

                  nmeshctmp(jstart+2) = ih+numintersect
                  exit

               end if  ! ( iwork(kmeshc(ikelmc+j)==1 )

            end do  ! j = 0, inpelm-1

         case(2)

!        --- simple case 2 edges per element are intersected

            nmeshctmp(jstart+1:jstart+nedgeslc) = &
               icrossobst(istart+1:istart+nedgeslc)

         case default

!        --- more than 2 edges per element: not implemented

            call errint ( nmeshctmp(jstart), 1 )
            call errint ( nedgeslc, 2 )
            call errsub ( 3138, 2, 0, 0 )
            go to 1000

         end select  ! case ( iedges )

         istart = istart+nedgeslc
         jstart = jstart+3       ! 2 nodes + number of nodes

      end do  ! i = 1, numelms

      call inichkarlen ( jstart-1, lennmeshc, 'nmeshc' ) ! check array length

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillintfnmc2'

      end if  ! ( debug )
      call erclos ( 'prfillintfnmc2' )

      end
