      subroutine prfillrefvec1 ( tm, reflvl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 21-04-2006 included mushy
!        version  1.0  date 21-04-2005
!
!   copyright (c) 2005-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array update with information about elements that have
!     a phase change interface
! **********************************************************************
!
!                       KEYWORDS
!
!     interface
!     phase_change
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulemushy
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer reflvl
      double precision tm

!     reflvl         i    Level of refinement
!     tm             i    Melting temperature
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ikelmc, ielem, ielgrp, irelem, neg, pos, node, k
      double precision eps

!     eps            local accuracy
!     ielem          Element number
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     k              Counting variable
!     neg            Number of nodes with 'negative' sign
!     node           Node number
!     pos            Number of nodes with 'positive' sign
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get constants
!     For all element groups ielgrp do
!       if ( element group belongs to lower dimension ) then
!          Fill array update with zeros
!       else ( element groups has dimension ndim )
!          For all elements ielem in group do
!             if ( element is intersected ) then
!               update(ielem) := irefvl+1
!             else ( element is not intersected)
!               update(ielem) := 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillrefvec1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillrefvec1'

      end if
      if ( ierror/=0 ) go to 1000
      eps = 1d-6

      if (mushy) then
         if ( abs(Tm)<eps ) then
            tsol = -tdelta
            tliq = tdelta
         else
            tsol = (1d0 - tdelta)*Tm
            tliq = (1d0 + tdelta)*Tm
         end if
      end if

      ikelmc = 0           ! start address in kemshc
      ielem = 0            ! last element number used

      if (debug) then
         call prinrl(temperature, npoint, 'temperature')
      end if

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

!     --- Compute some element group dependent quantities

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         ishape = kmeshf(ielgrp)  ! shape number
         if ( ndim==1 .and. ishape<1 .or. &
              ndim==2 .and. ishape<3 .or. &
              ndim==3 .and. ishape<10 ) then

!        --- Element of lower dimension
!            raise ikelmc and ielem
!            Fill array update with zeros

            do irelem = 1, inelem
               update(ielem+irelem) = 0
            end do  ! irelem = 1, inelem
            ikelmc = ikelmc+inelem*inpelm
            ielem = ielem+inelem

         else

!        --- Not an element of lower dimension
!            Loop over elements

         if (debug) print *, 'Start loop over elements'

            do irelem = 1, inelem

!           --- count the number of nodes that have temperature<Tm

               neg = 0
               pos = 0
               do k = 1, inpelm
                  ikelmc = ikelmc+1
                  node = kmeshc(ikelmc)
                  if (mushy) then
                     if ( temperature(node)>tliq ) pos = pos + 1
                     if ( temperature(node)<tsol ) neg = neg + 1
                  else
                     if ( temperature(node)<Tm ) neg = neg + 1
                  end if ! (mushy)
               end do  ! k = 1, inpelm

               if (debug) print *, 'neg, pos = ', neg, pos

               ielem = ielem+1

!              --- If at least one of the nodes has 'opposite' sign, mark
!                  it to be refined. By marking the element with a value of
!                  reflvl + 1, &
!                  we can distinguish it from it's direct neighbors.

               if (mushy) then

                  if (neg==inpelm .or. pos==inpelm) then

                     update(ielem) = 0

                  else

                     update(ielem) = reflvl + 1

                  end if ! (neg==inpelm .or. pos==inpelm)

               else if ( neg==inpelm .or. neg==0 ) then

!              --- All nodes have same sign, no interface in element

                  update(ielem) = 0

               else

!              --- In this element we have an interface

                  if (debug) print *, 'Intersected element located'

                  update(ielem) = reflvl + 1

               end if  ! ( mushy )

            end do  ! iielem = 1, inelem

         end if  ! ( ndim==1 .and. ishape<1 )

      end do  ! ielgrp = 1, nelgrp

      if (debug) call prinin(update, nelem, 'update')

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillrefvec1'

      end if
      call erclos ( 'prfillrefvec1' )

      end
