      subroutine prgridsize ( kmeshquant, ndim, origin, h, dx, nx, &
                              npoint, ntot, xmaxar )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute nx, dx and ntot from information in kmeshquant and h
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     grid
!     rectangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, nx(ndim), npoint, ntot
      double precision kmeshquant(2,ndim), origin(ndim), dx(ndim), h, &
                       xmaxar(ndim)

!     dx             o    contains the difference between xmax, xmin etc
!     h              i    Step size to be used to compute nx
!                         If zero h is estimated
!     kmeshquant     i    Contains xmin, xmax etc
!     ndim           i    Dimension of the space.
!     npoint         i    Number of nodal points
!     ntot           o    Product of all elements of nx
!     nx             o    contains number of points in each direction
!     origin         o    Contains the coordinates of the origin
!     xmaxar         o    Contains the coordinates xmax, ymax ..
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision area

!     area           Area of the region defined by xmin, xmax, etc
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prgridsize' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prgridsize'
         call ertrace
         write(irefwr,2) 'h', h
         write(irefwr,1) 'npoint', npoint
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store xmin, ymin, zmin in origin

      origin = kmeshquant(1,:)
      xmaxar = kmeshquant(2,:)
      dx = kmeshquant(2,:)-origin

      if ( debug ) then
         call prinrl ( origin, ndim, 'origin' )
         call prinrl ( xmaxar, ndim, 'xmaxar' )
         call prinrl ( dx, ndim, 'dx' )
      end if  ! ( debug )

      if ( h<=0d0 ) then

!     --- Step size has not been given, compute from mesh

         area = dx(1)
         if ( ndim>=2 ) then
            area = area*dx(2)
            if ( ndim==3 ) area = area*dx(3)
         end if  ! ( ndim>=2 )
         h = area/(npoint**(1d0/ndim)-1d0)

      end if  ! ( h<=0d0 )

      nx = int(dx/h)
      if ( debug ) call prinin ( nx, ndim, 'nx/1' )
      do i = 1, ndim
         if ( abs(h*nx(i)-dx(i))>h*1d-4 ) nx(i) = nx(i)+1
      end do  ! i = 1, ndim
      if ( debug ) call prinin ( nx, ndim, 'nx/2' )
      nx = min(nx,100)
      dx = dx/nx
      ntot = nx(1)
      if ( ndim>=2 ) then
         ntot = ntot*nx(2)
         if ( ndim==3 ) ntot = ntot*nx(3)
      end if  ! ( ndim>=2 )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( nx, ndim, 'nx' )
         call prinrl ( dx, ndim, 'dx' )
         write(irefwr,*) 'End prgridsize'

      end if  ! ( debug )
      call erclos ( 'prgridsize' )

      end
