      subroutine prkirchoff ( enthalpy, temp, n, ichoice, &
                              rinenthalpy )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 01-07-2004
!
!   copyright (c) 2004-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform temperature in normalized temperature and compute
!     the corresponding enthalpy or transform the normalized temperature back
!     to original temeprature
! **********************************************************************
!
!                       KEYWORDS
!
!     enthalpy
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          n, ichoice
      double precision enthalpy(*), temp(*), rinenthalpy(*)

!     enthalpy       o    enthalpy vector
!     ichoice        i    defines the type of transformation
!                         0: Transform physical temperature into normalized
!                            Kirchoff temperature and compute enthalpy
!                         1: Transform normalized Kirchoff temperature back to
!                            physical temperature
!                         2: Compute enthalpy from physical temperature
!                         3: Transform physical temperature into normalized
!                            Kirchoff temperature
!     n              i    number of degrees of freedom
!     rinenthalpy    i    Real input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     temp          i/o   temperature vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision melt_temp, kappa_s, kappa_l, capacity_s, &
                       capacity_l, rho, L, help

!     capacity_l     heat capacity (liquid)
!     capacity_s     heat capacity (solid)
!     help           Help variable to store a constant temporarily
!     i              Counting variable
!     kappa_l        coefficient in diffusion term (liquid)
!     kappa_s        coefficient in diffusion term (solid)
!     l              General loop variable
!     melt_temp      melting temperature
!     rho            density rho
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prkirchoff' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prkirchoff'
         write(irefwr,*) 'ichoice', ichoice

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set constants

      kappa_s    = rinenthalpy(1)
      kappa_l    = rinenthalpy(2)
      L          = rinenthalpy(3)
      capacity_s = rinenthalpy(4)
      capacity_l = rinenthalpy(5)
      rho        = rinenthalpy(6)
      melt_temp  = rinenthalpy(7)

      if ( ichoice==0 ) then

!     --- ichoice = 0, compute normalized temperature and enthalpy

         do i = 1, n
            if (temp(i)>=melt_temp) then
               temp(i) = kappa_l*(temp(i) - melt_temp)
               enthalpy(i) = rho*capacity_l*temp(i)/kappa_l + rho*L
            else if (temp(i)<melt_temp) then
               temp(i) = kappa_s*(temp(i) - melt_temp)
               enthalpy(i) = rho*capacity_s*temp(i)/kappa_s
            end if
         end do

      else if ( ichoice==1 ) then

!     --- ichoice = 1, backtransformation of temperature

         do i = 1, n
            if (temp(i)>0) then
               temp(i) = 1/kappa_l*temp(i) + melt_temp
            else if (temp(i)<0) then
               temp(i) = 1/kappa_s*temp(i) + melt_temp
            else
               temp(i) = melt_temp
            end if
         end do

      else if ( ichoice==2 ) then

!     --- ichoice = 2, compute enthalpy

         do i = 1, n
            if (temp(i)>=melt_temp) then
               help = kappa_l*(temp(i) - melt_temp)
               enthalpy(i) = rho*capacity_l*help/kappa_l + rho*L
            else if (temp(i)<melt_temp) then
               help = kappa_s*(temp(i) - melt_temp)
               enthalpy(i) = rho*capacity_s*help/kappa_s
            end if
         end do

      else

!     --- ichoice = 3, compute normalized temperature

         do i = 1, n
            if (temp(i)>=melt_temp) then
               temp(i) = kappa_l*(temp(i) - melt_temp)
            else if (temp(i)<melt_temp) then
               temp(i) = kappa_s*(temp(i) - melt_temp)
            end if
         end do

      end if

1000  call erclos ( 'prkirchoff' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prkirchoff'

      end if

      end
