      subroutine prmakedistpseu ( inpsep, rinsep, igrad, &
                                  eps, err, err0, nnodes, phiold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 25-04-2013 Use getparm
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 21-08-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make the level set function phi a distance function
!     Use pseudo time integration
! **********************************************************************
!
!                       KEYWORDS
!
!     time_integration
!     level_set
!     distance
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmoduleuser
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), nnodes, igrad(5)
      double precision rinsep(*), eps, err, err0, phiold(*)

!     eps            i    Local accuracy with respect to the distance function
!     err           i/o   Parameter to be kept for prmakedisterr
!     err0          i/o   Parameter to be kept for prmakedisterr
!     igrad         i/o   Sepran array containing information of the gradient
!     inpsep         i    Contains the integer input
!                         See subroutine readscmakedist for a description
!     nnodes         i    Number of nodes in node
!     phiold        i/o   Copy of phi from the previous iteration (time step)
!     rinsep         i    Contains the real input for this subroutine
!                         See subroutine readscmakedist for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iupwind, iseqphi, maxiter, istop, &
              typenorm, i, iprob, nusol, iincopy(8), iprint
      double precision :: rincopy(10), epsdiff
      logical first, finished

!     epsdiff        Diffusion parameter
!     finished       Indicates if accuracy has been reached (true) or not
!                    (false)
!     first          If true it concerns the first time step
!     i              Counting variable
!     iincopy        Is used to store the integer contents of ctimen
!     iprint         Defines the amount of printing (0 none, 2 maximal)
!     iprob          Actual problem number
!     iseqphi        Sequence number of level set function phi in isolut2d
!     istop          Indicates if program must stop in case of error (0,1)
!                    or must continue (2,3)
!     iupwind        Integer indicating type of upwind discretization.
!     maxiter        maximum number of iterations
!     nusol          Total number of unknowns
!     rincopy        Is used to store the real contents of ctimen
!     typenorm       Defines the type of norm that is used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr
      double precision getparm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETPARM        Return with real defined by inpsep/rinsep
!     INIVECTR       Return with sequence number of vector
!     PRCOMPGRAD     Compute gradient of a vector with one component per point
!     PRCOPYCTIMEN   Copy contents of ctimen into iincop and rincop
!     PRCOPYCTIMENBC Copy contents of iincop and rincop into ctimen
!     PRDELMATS      Delete matrices and right-hand-side vector for time
!                    dependent problem
!     PRFILLIINTIM   Fill array iintim with information stored in array inpsep
!                    Fill common ctimen with information stored in rinsep
!     PRININ         print 1d integer array
!     PRMAKEDISTERR  Compute difference between phi and phiold and check
!                    accuracy
!     SEPTIMROUT     General time integration of a discretized partial
!                    differential equation of first or second order in time
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakedistpseu' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakedistpseu'
         call prinin ( inpsep,14, 'inpsep' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      iupwind = inpsep(1)
      maxiter = inpsep(2)
      iprint = inpsep(3)
      istop = inpsep(10)
      typenorm = max(1,inpsep(11))
      iseqphi = inivectr(inpsep(6))
      complrun = .false.

      iprob = ks(isolut2d(iseqphi))%iprob  ! problem number for phi
      nusol = ks(isolut2d(iseqphi))%nusol

      if ( debug ) then
         write(irefwr,1) 'iseqphi, maxiter, iupwind, istop, typenorm', &
                          iseqphi, maxiter, iupwind, istop, typenorm
         write(irefwr,2) 'eps', eps
      end if  ! ( debug )

!     --- save old values of t, tstep etc by copying ctimen into arrays
!         iincopy and rincopy

      call prcopyctimen ( iincopy, rincopy )

!     --- Fill sepmoduletime with information of equation

      call prfilliintim ( inpsep, rinsep )

!     --- Set some parameters and clear arrays

      first = .true.

!     --- Fill array iuser

      allocate ( iuser(13), user(6) )
      iuserlc => iuser
      userlc => user

      iuser = 0
      iuser(1) = 13 ! length of array
      iuser(3) = 13 ! length of array
      iuser(6) = 7  !starting address of information (1 el group)
      if ( iupwind==0 ) iuser(8) = 132

      epsdiff = getparm ( inpsep, rinsep, 12, 'epsdiff' )

      iuser(13) = 6  ! refers to diffusion coefficient
      user = 0d0
      user(1) = 6d0 ! length of array
      user(3) = 6d0 ! length of array
      user(6) = epsdiff ! diffusion coefficient

      if ( debug ) then
         write(irefwr,1) 'number of smoothing steps', inpsep(13)
         write(irefwr,2) 'diffusion parameter epsilon', epsdiff
      end if  ! ( debug )

      intmatar => intmat1d(iprob:maxprb)
      isolut2dred => isolut2d(iseqphi:size(isolut2d))
      if ( inpsep(13)>0 ) then

!     --- inpsep(13)>0, perform inpsep(13) smoothing steps

         iuser(5) = inpsep(13)  ! number of smoothing steps
         do i = 1, inpsep(13)

!        --- Perform one time step

            call septimrout ( first, finished )

         end do  ! i = 1, inpsep(13)

      end if  ! ( inpsep(13)>0 )

!     --- Loop over pseudo time steps until accuracy is reached

      iuser(5) = 0  ! no smoothing steps left
      do i = 1, maxiter

!     --- Perform one time step

         call septimrout ( first, finished )

!        --- Check accuracy

         if ( igrad(1)>0 ) then
            call prcompgrad ( igrad, iseqphi )
         end if  ! ( igrad(1)>0 ) then
         call prmakedisterr ( phi1, phiold, nodes, nnodes, &
                              nusol, eps, iprint, i, finished, maxiter, &
                              istop, err, err0, grad, typenorm, ndim )

         if ( finished ) exit

      end do  ! i = 1, maxiter

!     --- Delete matrices and right-hand side

      call prdelmats ( massmt, stiffm, mattot, irhsd )

!     --- reset values of t, tstep etc by copying arrays
!         iincopy and rincopy back to ctimen

      call prcopyctimenbc ( iincopy, rincopy )

      nullify ( iuserlc, userlc )
      deallocate ( iuser, user )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmakedistpseu'

      end if  ! ( debug )
      call erclos ( 'prmakedistpseu' )

      end subroutine prmakedistpseu
