      subroutine prmindist3d ( markintf, x, coor, numfound, nodes, &
                               nmeshk, mindist, nmeshg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find minimum distance of node to set of faces in 3d interface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     interface
!     node
!     distance
!     minimum
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer markintf(*), numfound, nodes(numfound), nmeshk(*), &
              nmeshg(*)
      double precision x(3), coor(3,*), mindist

!     coor           i    array of length 3 x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     markintf      i/o   Array of size nfaces or nfaces to mark the faces or
!                         faces of the interface (1) else 0
!     mindist       i/o   Minimum distance of node wrt interface
!     nmeshg         i    nmesh part c, contains node numbers corresponding to
!                         faces
!     nmeshk         i    nmesh part l, contains face numbers corresponding
!                         to nodes
!     nodes          i    Contains node numbers in interface that are part
!                         of the set of cells under consideration
!     numfound       i    Number of nodes in nodes
!     x              i    co-ordinates of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode, ipstart, j, numfaces, iface, node1, node2, &
              jpstart, node3
      double precision dist
      logical inside

!     dist           Distance between two points
!     i              Counting variable
!     iface          face sequence number
!     inode          Node sequence number
!     inside         indicates if the perpendicular intersects
!                    between x1 and x2 (true) or not (false)
!     ipstart        Starting address in array nmeshk
!     j              Counting variable
!     jpstart        Starting address in array nmeshg
!     node1          Node of face
!     node2          Node of face
!     node3          Node of face
!     numfaces       number of faces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision erdistplanepnt

!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDISTPLANEPNT Compute the distance between triangle x1, x2, x3 and
!                    point xp
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmindist3d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmindist3d'
         call prinrl ( x, 3, 'x' )
         write(irefwr,2) 'mindist', mindist
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d14.6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over all nodes in set

      do i = 1, numfound

         inode = nodes(i)  ! node in set

!        --- Find all faces in interface connected to inode that have not
!            been considered yet

         ipstart = nmeshk(inode)    ! starting address in nmeshk
         numfaces = nmeshk(ipstart) ! number of faces connected to inode

         if ( debug ) &
            write(irefwr,1) 'i, inode, numfaces', i, inode, numfaces

         do j = 1, numfaces

            iface = nmeshk(ipstart+j)  ! face sequence number

            if ( debug ) &
               write(irefwr,1) 'j, iface, markintf(iface)', &
                                j, iface, markintf(iface)

            if ( markintf(iface)==1 ) then

!           --- interface face connected to node that has not been treated yet

               markintf(iface) = 2  ! mark as treated

               jpstart = nmeshg(iface)     ! starting address in nmeshg
               node1 = nmeshg(jpstart+1)   ! first node
               node2 = nmeshg(jpstart+2)   ! second node
               node3 = nmeshg(jpstart+3)   ! third node

!              --- Compute distance of node with coordinates in x to iface

               dist = erdistplanepnt ( x, coor(1,node1), coor(1,node2), &
                                       coor(1,node3), inside, .true. )
               mindist = min ( mindist,  dist )

               if ( debug ) then
                  write(irefwr,1) 'node1, node2, node3', &
                                   node1, node2, node3
                  write(irefwr,2) 'dist, mindist', dist, mindist
               end if  ! ( debug )

            end if  ! ( markintf(iface)==1 )

         end do  ! j = 1, numfaces

      end do  ! i = 1, numfound

1000  call erclos ( 'prmindist3d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmindist3d'

      end if  ! ( debug )

      end
