      subroutine prparrandm ( n, usol, blockunkinfo, sendinfo )
! ======================================================================
!
!        programmer    Kees Vuik
!        version  1.1  date 05-10-2008 Introduce allocate
!        version  1.0  date 15-10-2001 Parallel random vector
!
!   copyright (c) 2001-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This function generates a random vector in parallel
!
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, blockunkinfo(*), sendinfo(*)
      double precision usol(n)

!     blockunkinfo   i    Array with global block information for surface
!                         number isurnr. In this case it is related to
!                         non-prescribed degrees of freedom and not to nodes
!                         Contents:
!                         Pos. 1  m ( number of neihgbour blocks)
!                         Pos. 2*i   block number of i-th neighbour
!                         Pos. 2*i+1 starting address of i-th neighbour in
!                                    array sendinfo
!     n              i    Size of the matrix
!     sendinfo       i    Array containing information of the transport of
!                         unknowns between neighbouring blocks
!                         The starting address of each neighbour block is
!                         stored in array blockuninfo
!                         For each neighbour block the following information
!                         is stored
!                         Pos. 1 number of common unknowns n on common side of
!                                both blocks
!                         Pos. 2 ... n+1  unknown numbers of the unknowns to be
!                                sent to or received from block i
!                         If a common point in one block contains more
!                         unknowns than in the other block only the minimum
!                         of both is transported since the other ones do
!                         not belong to both blocks
!     usol          i/o   Random vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, k, numneigh, nneigh, sendstart, iblock, icontext, &
              seqblocks(iacnodes), error
      double precision, allocatable :: usend(:), multiply(:)
      integer, dimension(:), pointer :: send_buffer
      integer, save :: send_buffer_size

!     error        Return value of allocate or deallocate
!     iblock       Block sequence number
!     icontext     Tag for sending and receiving, is used as check
!     j            loop counter
!     k            loop counter
!     multiply     Work array containing multiplication factors
!     nneigh       Number of overlapping unknowns
!     numneigh     number of neighbouring blocks
!     send_buffer  send buffer
!     send_buffer_size  Length of the send buffer
!     sendstart    starting address in array sendinfo
!     seqblocks    Work array of length nblocks (=nsurfs) which
!                  is used to find the sequence of block numbers
!                  in array blockunkinfo
!                     seqblocks(iblock) = 0: block iblock not present
!                     seqblocks(iblock) = i: block iblock is the i-th block
!                  in blockinfo
!     usend          Help array to be used to send reals to other processor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     INIRECEIVEMPI  Receive information from other node (mpi)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prparrandm' )
      allocate(usend(n), multiply(n), stat = error)
      if ( error/= 0 ) then

!     --- error#0, so not enough space

         call errint ( n, 1 )
         call errint ( error, 2 )
         call errchr ( 'usend', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

      multiply = 1d0

      numneigh = blockunkinfo(1)

      send_buffer_size = 0
      do j = 1, numneigh
         sendstart = blockunkinfo(2*j+1)
         nneigh = sendinfo(sendstart)
         send_buffer_size = send_buffer_size + nneigh*8
      end do

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_size, numneigh )

      allocate(send_buffer(send_buffer_size))

      call inibufattmpi ( send_buffer, send_buffer_size )

!     --- Store sequence of block numbers in array seqblocks

      seqblocks(1:iacnodes) = 0

      do j = 1, numneigh
         iblock =  blockunkinfo(2*j)
         seqblocks(iblock) = j
      end do

      do j = 1, numneigh
         iblock = blockunkinfo(2*j)
         sendstart = blockunkinfo(2*j+1)
         nneigh = sendinfo(sendstart)
         do k = 1, nneigh
            usend(k) = usol(sendinfo(sendstart+k))
         end do

!        --- Send array usend

         icontext = 1
         call inibsendmpi ( usend, nneigh, SPDOUBLE, iblock, &
                            icontext )

      end do

      do j = 1, numneigh

!     --- Receive next array and store in usend
!         Get corresponding block number

         icontext = 1
         nneigh = n
         call inireceivempi ( usend, nneigh, SPDOUBLE, iblock, &
                              icontext )

         sendstart = blockunkinfo(2*seqblocks(iblock)+1)

         do k = 1, nneigh
            usol(sendinfo(sendstart+k)) = &
                usol(sendinfo(sendstart+k)) + usend(k)
            multiply(sendinfo(sendstart+k)) = &
                multiply(sendinfo(sendstart+k)) + dble(1)
         end do

      end do

      usol = usol/multiply

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_size )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

1000  deallocate ( usend, multiply, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'usend', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )
      call erclos ( 'prparrandm' )
      end
