      subroutine prpartpnlnconp ( inpsep, numcomps, xp, bigM, maxiter, &
                                  eps, rinsep, iseqconc, iseqvn, cpart, &
                                  k, substitutional, dimsub, iseqg, &
                                  kinf, itermethod, iseqiter, iprint, &
                                  iseqcs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-04-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     One time step for the integration of a particle is carried out
!     It concerns a system of concentration equations with a nonlinear
!     set of mixed boundary conditions
!     The interface concentration csol is computed in a loop until
!     convergence is reached
! **********************************************************************
!
!                       KEYWORDS
!
!     particle
!     level_set
!     time_dependent
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), numcomps, maxiter, iseqconc, iseqvn, iseqg, &
              dimsub, substitutional(numcomps), itermethod, iseqiter, &
              iprint, iseqcs
      double precision xp(numcomps), bigM, eps, cpart(numcomps), &
                       rinsep(*), k(numcomps), kinf

!     bigm           o    Parameter M in the method
!     cpart          o    Value of the concentrations in the particle
!     dimsub         o    Dimension of the set of substitutional elements
!     eps            o    Accuracy with respect to the iteration process
!                         Computation is stopped once |factor-1|<eps
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscpnlnconc for a description
!     iprint         o    Defines the amount of output to be printed
!     iseqconc       o    Sequence number of concentration vectors
!     iseqcs         o    Sequence number of vector cs
!     iseqg          o    Sequence number of right-hand side g
!     iseqiter       o    Sequence number of numiter in which the number of
!                         iterations is stored
!     iseqvn         o    Sequence number of boundary velocity
!     itermethod     o    Type of iteration method
!     k              o    parameter k in the method
!     kinf           o    parameter kinf in the method
!     maxiter        o    maximum number of iterations
!     numcomps       o    Number of components
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscpnlnconc for a description
!     substitutional o    Set of substitutional elements
!     xp             o    x-coordinate in plotting units
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqcpart, iseqcinf, iseqxinf, iseqsub, iseqk
      double precision cinf(numcomps), xinf(numcomps)

!     cinf           Parameter cinf in the method
!     iseqcinf       Sequence number of cinf in cuscons
!     iseqcpart      Sequence number of cpart in cuscons
!     iseqk          Sequence number of kinetic energy k with respect to
!                    array isol
!     iseqsub        sequence number of substitutional in cuscons
!     iseqxinf       sequence number of xinf in cuscons
!     xinf           Parameter xinf in the method
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getdimens, getintparm, getscalind
      double precision getparm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETCONVARARRAY Copy the values of a constant or scalar array from cuscons
!                    into a local array
!     GETDIMENS      Return with dimension of constant or scalar corresponding
!                    to iseq
!     GETINTPARM     Return with real/scalar defined by inpsep/rinsep
!     GETINTVARARRAY Copy the values of an integer constant or scalar array
!                    from cuscons into a local array
!     GETPARM        Return with real defined by inpsep/rinsep
!     GETSCALIND     Return with actual scalar number, where the original number
!                    is the modified one stored in the input array
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!   Depending on the values in IREAD input may be read from the standard
!   input file by subroutine READFRST
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     In each time step we have to solve a set of convection-diffusion
!     equations for each particle component
!     At the interface the following boundary conditions must be satisfied:
!     D_e dc_e/dn + vn c_e = g_e, &
!     where D_e is the diffusion coefficient in the convection-diffusion e
!           c_e is the concentration of component e
!           vn is the unknown interface velocity and g_e the right-hand side
!     g_e is equal to c_e^p * vn, with c_e^p the particle concentration
!     To compute vn the following relations must be satisfied:
!
!     x_e^p = c_e^p/sum_e(c_e^p) is the mole fraction within the precipitate
!     c_e^s = c_e^p/K_e vn + c_e the mole fraction along the interface
!         with c_e the concentration at the interface and K_e the given
!         interface reaction speed
!     M is the maximum total concentration of substitutional elements
!     M = cinf(1)/xinf(1) = sum(cinf(S))-sum(cinf)
!        with cinf and xinf given and S the set of substitutional elements
!     I is the set of interstitial elements, the complement of S
!     The equations to be satisfied are
!     prod((x_e^s)^x_e^p) = kinf exp ( zeta kappa ),      (1)
!     with
!     x_e^s = c_e^s / (sum(c_I^s) + M)
!     zeta = gamma V_m / (R_g T ) is given
!     kappa is the curvature at the interface
!     convergence is reached if (1) is satisfied accurately enough
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prpartpnlnconp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prpartpnlnconp'
         call prininlarge ( inpsep, 19, 'inpsep' )
  1      format ( a, 1x, (10i10) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      iseqconc = inpsep(2)-5000    ! sequence number of concentration vector
      iseqvn = inpsep(3)           ! sequence number of normal velocity
      iseqg = getscalind(inpsep(4))  ! sequence number of g
      iseqcinf = inpsep(7)         ! position of cinf in cuscons
      iseqxinf = inpsep(8)         ! position of xinf in cuscons
      iseqcpart = inpsep(9)        ! position of cpart in cuscons
      iseqk = inpsep(10)           ! position of k in cuscons
      iseqsub = inpsep(11)         ! position of substitutional in cuscons
      maxiter = getintparm(inpsep(12))     ! maximum number of iterations
      itermethod = getintparm(inpsep(13))  ! iteration method
      iprint = getintparm(inpsep(14))      ! print level
      eps = getparm ( inpsep, rinsep, 15, 'accuracy' )

!     --- Copy parameters

      call getconvararray ( iseqcinf, cinf, 'cinf', numcomps )
      call getconvararray ( iseqxinf, xinf, 'xinf', numcomps )
      call getconvararray ( iseqcpart, cpart, 'cpart', numcomps )
      call getconvararray ( iseqk, k, 'k', numcomps )
      kinf = getparm ( inpsep, rinsep, 18, 'kinf' )
      iseqiter = getscalind ( inpsep(19) )
      iseqcs = mod(inpsep(20),5000)       ! sequence number of cs vector
      dimsub = getdimens(iseqsub)
      call getintvararray ( iseqsub, substitutional, 'substitutional', &
                            dimsub )
      if ( ierror/=0 ) go to 1000

!     --- Compute some parameters
!         x_e^p = c_e^p/sum_e(c_e^p)

      xp(1:numcomps) = cpart(1:numcomps)/sum(cpart(1:numcomps))
      bigM = cinf(1)/xinf(1)+ &
             sum(cinf(substitutional(1:dimsub)))-sum(cinf(1:numcomps))

1000  call erclos ( 'prpartpnlnconp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iseqconc, iseqvn, itermethod, iseqg', &
                          iseqconc, iseqvn, itermethod, iseqg
         write(irefwr,1) 'iprint, maxiter', iprint, maxiter
         write(irefwr,2) 'M, kinf', bigM, kinf
         call prinrl ( cinf, numcomps, 'cinf' )
         call prinrl ( xinf, numcomps, 'xinf' )
         call prinrl ( cpart, numcomps, 'cpart' )
         call prinrl ( xp, numcomps, 'xp' )
         call prinrl ( k, numcomps, 'k' )
         call prinin ( substitutional, dimsub, 'substitutional' )
         write(irefwr,*) 'End prpartpnlnconp'

      end if  ! ( debug )

      end
