      subroutine prsetrefine ( meshnr, isol, tm, reflvl, &
                               nelembasis, refine, &
                               basismesh, rewind, first, &
                               irewind, iprint )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.4  date 11-12-2012 Long integers
!        version  1.3  date 21-04-2006 Added keeprefmesh/irewind
!        version  1.2  date 30-09-2005 Always refine when rewinding
!
!   copyright (c) 2005-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set the value of logical refine. If refine is true, the mesh for the
!     new time step is to be refined. If refine is false, the old mesh is
!     reused
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshnr, isol(*), reflvl, iprint, irewind, &
              nelembasis, basismesh
      double precision tm
      logical refine, rewind, first

!     basismesh      i    Mesh sequence number of the basis mesh
!     first          i    If true it concerns the first step
!     iprint         o    Print level, 0 is no printing
!     irewind        i    Number of time steps since last rewind
!     isol           i    Standard SEPRAN array containing information of
!                         the solution vector (same as isol)
!     meshnr         i    Mesh sequence number
!     nelembasis     i    Number of elements in basis mesh
!     refine         i    When true, the mesh is locally refined
!     reflvl         i    Level of refinement
!     rewind         i    When true, the current time level is rewinded
!                         to a previous time level
!     tm             i    Melting temperature
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical keeprefmesh
      integer nelemold, ireflvl, iseqkm1, i

!     ireflvl        Current level of refinement
!     iseqkm1        Sequence number with respect to km of first mesh
!     keeprefmesh    If true, the local refinement is kept for
!                    three consecutive time steps after an 'interface
!                    outside refinement' warning is given
!     nelemold       Number of elements of the previous mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHUPDATE      Determine whether a mesh update is required or not
!     PRFILLREFVEC1  Fill array update with information about elements that have
!                    a phase change interface
!     PRFILLREFVEC2  Fill array refinenew of length nelemold with required
!                    levels of refinement
!     PRMARKNEIGH1   Fill array refine2 for all neighbors of elements that are
!                    intersected by the melting front
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKMESHD  Fill kmesh part d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsetrefine' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsetrefine'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute constants

      nelemold = km(iseqkm)%kmesh(9)

      keeprefmesh = .true.

!     --- Create some arrays
!         Refinenew contains per element of the
!         basis mesh the level of refinement required for this element
!         based on the solution vector as computed above.
!         In case the solution was computed on the basis mesh, the filling
!         of vector refinenew is trivial: just a copy vector update.
!         In case the solution has been computed on a refined mesh, each
!         component of vector refinenew will be filled with the maximum
!         level of refinement required as found in the subelements of each
!         basis mesh element.

      allocate ( refinenew(nelembasis), update(nelemold)  )

!     --- Fill array update

      temperature => ks(isol(1))%sol
      call prfillrefvec1 ( tm, reflvl )

!     --- Fill the refinement vector for the new mesh refinenew

      if ( debug ) write(irefwr,1) 'meshnr, basismesh', meshnr, basismesh
      if ( meshnr==basismesh ) then

!     --- meshnr = basis mesh
!         Copy update to refinenew

         if ( associated(refinenew) ) deallocate ( refinenew )
         allocate ( refinenew(size(update)) )
         refinenew = update

      else

!     --- meshnr>basismesh
!         Use mapping to fill refinenew from update

         call prfillrefvec2 ( refinenew, update, &
                              refmapold, nelembasis, nelemold )

      end if  ! ( meshnr==basismesh )

!     --- Mark neighboring elements in case we marked an element as
!         intersected

      call sepfillkmesh(kmesh2d(basismesh))
      if ( kelmd>0 ) call sepfillkmeshd
      do ireflvl = reflvl, 1, -1
         if ( iprint>1 ) write(irefwr,*) &
            'current level of refinement', ireflvl
         call prmarkneigh1 ( ireflvl )
      end do
      call sepfillkmesh(kmesh2d(meshnr))

!     --- In case the computed refinenew==refineold, the mesh does not
!         need to be updated, and refine will keep the value false.
!         When no enthalpy equation is solved, refinement is not needed
!         as well.
!         Otherwise, it is set to true

      refine = .false.
      rewind = .false.

!     --- Determine whether we need to refine and/or rewind based on the
!         interface velocity

      call mshupdate ( refine, nelembasis, reflvl, &
                       rewind, iprint )

!     --- In case we rewind, combine refineold and refinenew to create a new
!         refinenew. The old refinement is needed to ensure that the old
!         solution is not partially mapped onto a coarser grid. The new
!         refinement is to ensure that a new interface is captured on a fine
!         grid.

      if ( (rewind .and. .not. first) .or. &
                             (keeprefmesh .and. irewind<3) ) then
         if ( iprint>1 ) write(irefwr,*) &
            'rewinding time and using combined refinenew'

         do i = 1, nelembasis
            refinenew(i) = max(refinenew(i), refineold(i))
         end do

      end if

!     --- Remove semi-permanent arrays

      deallocate ( update )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsetrefine'

      end if
      call erclos ( 'prsetrefine' )

      end subroutine prsetrefine
