      subroutine seppjparainit
! ======================================================================
!
!        programmer    Fred Vermolen
!        version  1.2  date 02-11-2013 Debug statements
!        version  1.1  date 31-10-2002 Dirichlet conditions everywhere,
!                                      checker board geometry
!        version  1.0  date 01-03-2002
!
!   copyright (c) 2001-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   In this subroutine the sparse projection vector of the
!   neighboring blocks, the vector only lives at the interface,
!   is initialized. Also the corresponding entries in the
!   integer-array from intprj are filled with the appropriate
!   degrees of freedom.
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleproj
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idxdgf, numneigh, nneigh, i, j, k, nnproj2, &
              istartinfo, nbnusol, inodp, inusol, nbnode, sendstart
      integer, allocatable :: checkdgf(:)

!     checkdgf       array that is used to indicate whether a neighbor point
!                    has been checked
!                    they are not
!     i              Loop variable
!     idxdgf         index for aprojvc
!     inodp          Nodal point number
!     inusol         degree of freedom / index of unknown
!     istartinfo     starting address for node numbers in array transportinfo
!     j              Loop variable
!     k              Loop variable
!     nbnode         node-number of neighbor points of interface points
!     nbnusol        index of unknown of neighbor points of interface points
!     nneigh         number of unknowns at the interface that is shared by
!                    a particular neighbor
!     nnproj2        Total number of entries of vector PROJVC2
!     numneigh       Number of neighboring blocks
!     sendstart      starting address in array sendinfo
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Goal: Creating sparse projection vectors from the constant parts
!           and the average overlaps at the interface nodes
!
!     Semi PSD: seppjparainit
!
!     The matrix vector multiplication A z_i is carried out over several
!     blocks.
!     So the degrees of freedom of neighboring blocks that give a non-zero
!     contribution must be stored
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjparainit' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjparainit'
         write(irefwr,1) 'npoint, nrusol, nnproj2', &
                          npoint, nrusol, nnproj2
         write(irefwr,1) 'indprh', indprh
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( checkdgf(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'checkdgf' )
      if ( ierror/=0 ) go to 1000

      idxdgf = 0
      numneigh = blockunkinfo( 1 )

      if ( debug ) write(irefwr,1) 'numneigh', numneigh

      projrowsa( 1 ) = 0

      do i = 1, numneigh

         sendstart = blockunkinfo(2*i+1)  ! starting address of i^th neighbor
         nneigh = sendinfo(sendstart)     ! number of interface neighbors
         projrowsa(i+1) = projrowsa(i) + nneigh

         if ( debug ) write(irefwr,1) 'i, nneigh', i, nneigh

         checkdgf = 0

!        --- Here the same sequence as in the sequential intprj
!            should be used
!            The first part of projrowsa is a copy intprj but extra positions
!            are necessary

         do j = 1,nneigh

            idxdgf = idxdgf + 1

!           --- Initialize the neighbor nodes with zeros

            inusol = sendinfo(sendstart+j)
            projcolsatmp(idxdgf) = inusol  ! interface degfd
            checkdgf(inusol) = 1

         end do
         call inichkarlen ( idxdgf, nproj*npoint, 'projcolsatmp' )

         if ( debug ) then
            call prinin ( projrowsa, nproj+1, 'projrowsa/1' )
            call prinin ( projcolsatmp, idxdgf, 'projcolsatmp/1' )
         end if  ! ( debug )

         istartinfo = blockinfo(2*i+1)

!        --- in the next part neighbor nodes of interface nodes in neighbor
!            blocks are stored in projrowsa

         if (indprh==0) then

            do j = 1,nneigh
!
!           --- Find node-number of boundary node for use in
!               kmeshe to incorporate the neighbors of interface
!               nodes as well

               inodp = transinfo(istartinfo+j)    ! node in interface
               do k = kmshe1(inodp)+1,kmshe1(inodp+1)
                  nbnode = kmshe2(k)         ! neighbor node
!
!                 --- Deze optie indien Dirichlet voorwaarde
!                      nbnusol = kprobh(nbnode)
!                      Deze optie indien enkel Neumann voorwaarde

                  nbnusol = nbnode
                  if (checkdgf(nbnusol)==0) then
                     checkdgf(nbnusol) = 1
                     projrowsa(i+1) = projrowsa(i+1)+1
                     idxdgf = idxdgf + 1
                     projcolsatmp(idxdgf) = nbnusol  ! neighbor degfd
                  end if
               end do
            end do

         else

            do j = 1,nneigh

!           --- Find node-number of boundary node for use in
!               kmeshe to incorporate the neighbors of interface
!               nodes as well

               inusol = sendinfo(sendstart+j)
               inodp = kprobhinv(inusol)
               do k = kmshe1(inodp)+1,kmshe1(inodp+1)
                  nbnode = kmshe2(k)  ! neighbor node

!                 --- Deze optie indien Dirichlet voorwaarde
!                     nbnusol = kprobh(nbnode)
!                     Deze optie indien enkel Neumann voorwaarde

                  nbnusol = kprobh(nbnode)  ! neighbor degfd
                  if ((checkdgf(nbnusol)==0) .and. &
                   (nbnusol<=nrusol)) then
                     checkdgf(nbnusol) = 1
                     projrowsa(i+1) = projrowsa(i+1)+1
                     idxdgf = idxdgf + 1
                     projcolsatmp(idxdgf) = nbnusol
                  end if
               end do
            end do

         end if

      end do

      deallocate ( checkdgf, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'checkdgf' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( projrowsa, nproj+1, 'projrowsa' )
         call prinin ( projcolsatmp, idxdgf, 'projcolsatmp' )
         write(irefwr,*) 'End seppjparainit'

      end if  ! ( debug )
      call erclos ( 'seppjparainit' )

      end
