      subroutine seppjparmain2 ( projvc1, projvc3 )
! ======================================================================
!
!        programmer    Fred Vermolen
!        version  1.0  date 29-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Creates projection vectors for the parallel deflated iccg algorithm.
!     The space spanned by the projection vectors approximates the space
!     spanned by the small eigenvectors. Subsequently the vectors Az
!     are computed and the matrix E = Z^T A Z is created.
!     Old subroutine prpjparmain2
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemat   ! definition of matrices
      use sepmodulekmesh
      use sepmoduleproj
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision projvc1(nproj*npoint), &
                       projvc3(iacnodes,iacnodes)

!     projvc1             See sepmoduleproj: projU
!     projvc3             See sepmoduleproj: projE
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, k, numneigh, &
              naneigh, nneigh, icontext, inusol, twonumneigh, i, &
              indmate, iblock, sendstart, istartprojAU, l, &
              seqblocks(iacnodes), &
              globbloc(iacnodes), mi, istart, &
              blocknumber(iacnodes), numneighsend, index, iistart, &
              nij, knusol, numsend, jvec, nreceive
      integer numneighE
      double precision, allocatable :: projvctmp(:), usend(:), &
                                       mate(:), projAUtmp(:)
      type (rmatrix) :: A
      type (intmatrix), pointer :: IM

!     A              Matrix A
!     IM             Refers to intmt(intmat)
!     blocknumber    Array of global block numbers that are neighbor of the
!                    present block and need additional communication
!                    they are not
!     globbloc       local block number of a global block number
!     i              Loop variable
!     iblock         index of block
!     icontext       sequence number of communication with a neighboring block
!     iistart        starting address in parinfo
!     index          index for local neighbor number
!     indmate        temporary index of neighboring block
!     inusol         Position of unknown in array USOL
!     istart         starting address for parinfo for the additional
!                    communication
!     istartprojAU   Starting address of entries in projvc that correspond
!                    to interface nodes that are shared by a particular
!                    neighbor
!     j              Counting variable
!     jvec           index for vector number for extra communication
!     k              Counting variable
!     knusol         Degree of freedom
!     l              Counting variable
!     mate           Double precision array that contains the complete
!                    matrix E
!     mi             number of vectors per neighbor for extra communication
!     naneigh        nneigh + number of unknowns in the present block that are
!                    a neighbor of the interfacial nodes that are shared
!                    with a particular neighbor
!     nij            number of points for extra communication
!     nneigh         number of unknowns at the interface that is shared by
!                    a particular neighbor
!     nreceive       number of elements for receiving after extra communication
!     numneigh       Number of neighboring blocks
!     numneighE      number of neighboring blocks of a particular node
!                    this variable is used when the present block has index 1
!     numneighsend   number of neighbors that need extra communication
!     numsend        number of elements for the sending before extra
!                    communication
!     projAUtmp      Temporary array to store projAU from the sparse
!                    matrix-vector multiplication ERMULTAZ
!     projvctmp      Temporary array to store projvc1 to use in sparse
!                    matrix-vector multiplication ERMULTAZ
!     sendstart      starting address in array sendinfo
!     seqblocks      Work array of length nblocks (=nsurfs) which
!                    is used to find the sequence of block numbers
!                    in array blockinfo
!                    seqblocks(iblock) = 0: block iblock not present
!                    seqblocks(iblock) = i: block iblock is the i-th block
!                    in blockinfo
!     twonumneigh    = 2*numneigh
!     usend          Help array to be used to send reals to other processor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     INISENDMPI     Send information to other node (mpi)
!     PRDECOSM       Compute LDL^T decomposition of symmetric full matrix
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPMUL1        performs matrix vector multiplication
!     SEPPJPARAINIT  Initialize vector A z
!     SEPPJPARSPAR   Construct and send projection vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjparmain2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjparmain2'
         write(irefwr,1) 'iacnodes, nproj, nrusol', &
                          iacnodes, nproj, nrusol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      IM => intmt(realmt(iseqmt)%intmatref)

!     --- Initializing data

      numneigh = blockunkinfo(1)

      length = max(nrusol,2*nproj+2)

      if ( debug ) write(irefwr,1) 'length', length

      allocate ( projvctmp(nrusol), usend(length), &
                 mate(numneigh+1), projAUtmp(nrusol), stat = error )
      length = length+2*nrusol+numneigh
      if ( error/=0 ) call eralloc ( error, length, 'projvctmp' )
      if ( ierror/=0 ) go to 1000

!     --- Construction of projection vector in present block
!         Here also the number of interface nodes that are part
!         of neighboring processor are stored in intprj(1:numneigh+1)

      call seppjparspar ( projvc1, seqblocks, globbloc )

      blocknumber(1:iacnodes) = 0

      numneighsend = 0

!     --- Loop to create a list of neighbor blocks that share an interface with
!         the present block and that share a non-empty set of other neighbor
!         blocks.

      if ( numneigh>1 ) then

         do i = 1, numneigh
            istart = blockunkinfo(1+2*numneigh+i)
            mi = blockunkinfo(istart)
            if ( mi/=0 ) then
               iblock = blockunkinfo(2*i)
               numneighsend = numneighsend + 1
               blocknumber(numneighsend) = iblock
            end if  ! ( mi/=0 )
         end do  ! i = 1, numneigh

      end if  ! ( numneigh>1 )

      if ( debug ) then
         write(irefwr,1) 'numneigh, numneighsend', numneigh, numneighsend
         call prinin ( blocknumber, numneighsend, 'blocknumber' )
      end if  ! ( debug )

!     --- Initialize projAU and fill the list of unknowns (degrees
!         of freedom) of the interfaces and those that neighbor the
!         interface nodes into intprja.

      call seppjparainit

      allocate ( projAU(nrusol+projrowsa(numneigh+1)), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nrusol+projrowsa(numneigh+1), 'projAU' )
      if ( ierror/=0 ) go to 1000

      projAU = 0d0

!     --- Matrix-vector multiplication for full projection vector

      if ( debug ) call prinrl ( projvc1, nrusol, 'projvc1' )
      A%S => S_ii
      A%diag => IM%diag
      A%rows => irowsii
      A%cols => icolsii
      A%neq = nrusol
      call sepnscmatvec ( A, projvc1, projAU )
      if ( debug ) call prinrl ( projAU, nrusol, 'projAU' )

      numneigh = blockunkinfo(1)
      do i = 1, numneigh

!     --- Give projvc as input and projAU as output
!         neighstart and neighend are starting and final address
!         for projvc, starting and final addresses for projAU
!         have been constituted in the previous lines
!         numnneigh is the total number of entries for projAU

         naneigh = projrowsa(i+1) - projrowsa(i)

         istartprojAU = nrusol + 1 + projrowsa(i)
         iblock = blockunkinfo(2*i)

         projvctmp = 0
         projAUtmp = 0

         nneigh = projrows(i+1) - projrows(i)

!        --- Store interface values from projvc1 into projvctmp

         do k = 1,nneigh
             inusol = projcolstmp(projrows(i)+k)
             projvctmp(inusol) = projvc1(nrusol+projrows(i)+k)
         end do

         if ( debug ) then
            write(irefwr,1) 'i, nneigh', i, nneigh
            call prinrl ( projvctmp, nrusol, 'projvctmp' )
         end if  ! ( debug )

!        --- Now sparse matrix-vector multiplication for projection vectors
!            from the neighboring processors
!            Compute A z_i for interface, store in projAUtmp

         A%S => S_ii
         A%diag => IM%diag
         A%rows => irowsii
         A%cols => icolsii
         A%neq = nrusol
         call sepnscmatvec ( A, projvctmp, projAUtmp )
         if ( debug ) call prinrl ( projAUtmp, nrusol, 'projAUtmp' )
         do k = 1,naneigh
            inusol = projcolsatmp(projrowsa(i)+k)
            projAU(istartprojAU+k-1) = projAUtmp(inusol)
         end do
         if ( debug ) &
            call prinrl ( projAU(istartprojAU), naneigh, 'projAU/extra' )

      end do

!     --- Now sparse multiplication of interface nodes of neighboring
!         blocks has taken place, the results have to be sent to the
!         processors that deal with the neighboring blocks

      numneigh    = blockunkinfo(1)

      do j = 1, numneigh
          iblock = blockunkinfo(2*j)
          sendstart = blockunkinfo(2*j+1)
          nneigh = sendinfo(sendstart)
          istartprojAU = nrusol+projrowsa(j)
          do k = 1,nneigh
              usend(k) = projAU(istartprojAU+k)
          end do
          icontext = 8
          call inisendmpi( usend,nneigh,SPDOUBLE,iblock,icontext )
      end do

!     --- Now sparse results projAU have to be received from neighboring
!         processors

      do i = 1, numneigh
          icontext = 8
          nneigh = nrusol
          call inireceivempi( usend,nneigh,SPDOUBLE,iblock, &
                                    icontext)

!         --- sendstart = blockunkinfo(2*seqblocks(iblock)+1)

          sendstart = blockunkinfo(1+2*globbloc(iblock))
          do k = 1, nneigh
              inusol = sendinfo(sendstart+k)

!             --- Update of projAU by addition of result from neighbor

              projAU(inusol) = projAU(inusol) + usend(k)
          end do
      end do

      if ( debug ) call prinrl ( projAU, nrusol, 'projAU/3' )

!     --- Send interface values to neighboring blocks

      do j = 1, numneigh
         iblock = blockunkinfo(2*j)
         sendstart = blockunkinfo(1+2*j)
         nneigh = sendinfo(sendstart)
         do k = 1, nneigh
            inusol = sendinfo(sendstart+k)
            usend(k) = projAU(inusol)
         end do
         icontext = 9
         call inisendmpi(usend,nneigh,SPDOUBLE,iblock,icontext)
      end do

!     --- Receive interface values from neighboring blocks
!         Store in second part of projAU

      do i = 1, numneigh
         icontext = 9
         nneigh = nrusol
         call inireceivempi ( usend, nneigh, SPDOUBLE, iblock, &
                              icontext )
         sendstart = blockunkinfo(1+2*globbloc(iblock))
         istartprojAU = nrusol + projrowsa(seqblocks(iblock))
         do k = 1,nneigh
            projAU(istartprojAU+k) = usend(k)
         end do
         if ( debug ) then
            write(irefwr,1) 'interface', i
            write(irefwr,1) 'istartprojAU, iblock', istartprojAU, iblock
            call prinrl ( projAU(istartprojAU+1), nneigh, 'projAU/4' )
         end if  ! ( debug )

      end do

!     --- Now extra information to neighboring blocks that share an interface
!         with the present block. Consider a point which is on an interface
!         with a neighboring block (say iblock). Further let this point be a
!         neighbor point
!         of some other blocks (say jvec). Hence, this point is not located on
!         the interfaces
!         with these blocks (jvec). Then, the values of the vectors Az that
!         belong to these
!         blocks (jvec) have to be sent to and from the neighboring block
!         iblock.
!         This is the extra information that has to be sent. It occurs for
!         instance when
!         the domain of computation is divided into a blockwise structure, &
!         whereas when
!         the domain is divided into layers, then this extra sending is not
!         necessary.
!         First we prepare for sending:

      if ( numneigh>1 ) then

         do i = 1, numneighsend
            numsend = 0
            iblock = blocknumber(i)

!           --- Find neighboring block to and from which we need extra data
!               sending and receiving

            index = seqblocks(iblock)
            istart = blockunkinfo(1+2*numneigh+index)
            mi = blockunkinfo(istart)

!           --- Give the amount of vectors that are sent and received from this
!               neighbor

            do j = 1,mi
               jvec = blockunkinfo(istart+2*j-1)

!              --- Find vector to be sent and received

               iistart = blockunkinfo(istart+2*j)
               nij = sendinfo(iistart)

!              --- Find the amount of unknown numbers to be sent and received

               istartprojAU = nrusol+projrowsa(seqblocks(jvec))
               do k = 1,nij
                  knusol = sendinfo(iistart+k)

!                 --- Find the unknown number (degree of freedom) to be
!                     sent/received

                  naneigh=projrowsa(seqblocks(jvec)+1)- &
                          projrowsa(seqblocks(jvec))

!                 --- A loop over all unknown numbers of the Az of jvec to find
!                     the corresponding
!                     the corresponding value of Az, which is to be sent.
!                     Therefore, this
!                     value, with jvec, is stored in the array usend.

                  do l = 1,naneigh
                     inusol = projcolsatmp(l+projrowsa(seqblocks(jvec)))
                     if (inusol==knusol) then
                        numsend = numsend + 1
                        usend(2*numsend-1) = jvec
                        usend(2*numsend)   = projAU(istartprojAU+l)
                     end if
                  end do
               end do
            end do
            icontext = 10
            nreceive = 2*numsend

!           --- perform the actual send operation

            call inisendmpi(usend,nreceive,SPDOUBLE,iblock,icontext)

         end do

!        --- Now, we receive Az values from other blocks / processors

         do i = 1,numneighsend

            icontext = 10
            nreceive = 2*nrusol

!           --- receive from other processors

            call inireceivempi(usend,nreceive,SPDOUBLE,iblock,icontext)

!           --- Determine the amount of vectors to be received

            numsend = int(nreceive /2)

!           --- Back to local numbering

            index = globbloc(iblock)
            istart = blockunkinfo(1+2*numneigh+index)
            mi = blockunkinfo(istart)
            do j = 1,numsend

!           --- specify the vector number of Az to be received

               jvec = int(usend(2*j-1))
               iistart = blockunkinfo(istart+2*j)
               nij = sendinfo(iistart)

!              --- Determine the amount of unknown numbers to be received

               istartprojAU = nrusol+projrowsa(globbloc(jvec))
               naneigh=projrowsa(globbloc(jvec)+1)-projrowsa(globbloc(jvec))
               do k = 1, nij

!              --- Find the unknown number (degree of freedom) of whom the
!                  value of Az is received

                  knusol = sendinfo(iistart+k)

!                 --- Loop to update the value of Az

                  do l = 1,naneigh
                     inusol = projcolsatmp(l+projrowsa(globbloc(jvec)))
                     if (inusol==knusol) then
                        projAU(istartprojAU+l) = &
                        projAU(istartprojAU+l) + usend(2*j)
                     end if
                  end do
               end do
            end do
         end do

      end if

!     --- Now matrix E is constituted by computation of inner products
!         First on present processor

      mate(1) = ddot ( nrusol, projvc1, 1, projAU, 1)

!     --- Subsequently on the neighboring processors

      do i = 1, numneigh
         mate(i+1) = 0
         naneigh = projrowsa(i+1)-projrowsa(i)
         istartprojAU = nrusol+projrowsa(i)
         do j = 1, naneigh
            inusol = projcolsatmp(j+projrowsa(i))
            mate(i+1) = mate(i+1) + &
                     projvc1(inusol)*projAU(istartprojAU+j)
         end do
      end do

!     --- If the present processor is not on the zeroth block
!         then send the computed entries of E to the zeroth block, &
!         where the matrix E is 'inverted'. If the present processor
!         is the zeroth block then the information of the matrix E
!         should be received.
!         These entries are to be sent to processor 1
!         if the present processor is not 1

      if (inodenr/=1) then

         usend(1) = inodenr
         usend(numneigh+2) = mate(1)
         do i = 1, numneigh

!        --- Beware blockunkinfo is integer

            iblock = blockunkinfo(2*i)
            usend(i+1) = iblock

!           --- Beware E(i) is double

            usend(numneigh+2+i) = mate(i+1)
         end do

         iblock = 1
         twonumneigh = 2*(numneigh+1)
         icontext = 11
         call inisendmpi(usend,twonumneigh,SPDOUBLE,iblock,icontext)

      else

         if ( debug ) then
            write(irefwr,1) 'inodenr, numneigh', inodenr, numneigh
            call prinin ( blockunkinfo(1), 2*numneigh, 'blockunkinfo' )
         end if  ! ( debug )

         do i = 1, nproj
            do j = 1, nproj
               projvc3(i,j) = 0
            end do
         end do
         projvc3(1,1) = mate(1)
         do i = 1, numneigh
            indmate = blockunkinfo(2*i)
            projvc3(1,indmate) = mate(i+1)
         end do
         do i = 2, nproj
            icontext = 11
            twonumneigh = 2*(nproj+1)
            if ( debug ) write(irefwr,1) 'i, twonumneigh',i, twonumneigh
            call inireceivempi(usend,twonumneigh,SPDOUBLE,iblock, &
                               icontext)
            numneighE = int(twonumneigh / 2)
            if ( debug ) &
               write(irefwr,1) 'numneighE, iblock', numneighE, iblock
            do j = 1, numneighE
               indmate = int(usend(j))
               if ( debug ) write(irefwr,1) 'j, indmate', j, indmate
               projvc3(iblock,indmate) = usend(numneighE+j)
            end do
         end do

!        --- Compute the Cholesky decomposition of E
!            and store in the same positions

         call prdecosm ( nproj, projvc3 )

!        --- The output is projvc, so projvc1, projAU, projvc3
!            should be stored into projvc. This is similar to
!            the procedures of Gjalt Wijma.

      end if

      if ( ioutp>=2 ) then

!     --- ioutp>=2, extra output

         write(irefwr,150)
150      format('Matrix E has been made')

      end if

      deallocate ( projvctmp, usend, mate, projAUtmp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'projvctmp' )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( projvc1, nrusol+1+projrows(numneigh+1), 'projvc1' )
         call prinrl ( projAU, nrusol+1+projrowsa(numneigh+1), 'projAU' )
         call prinrl ( projvc3, nproj*nproj, 'projvc3' )
         write(irefwr,*) 'End seppjparmain2'

      end if  ! ( debug )
      call erclos ( 'seppjparmain2' )

      end
