      subroutine seppjsubp
! ======================================================================
!
!        programmer    Gjalt Wijma
!        version  1.0  date 14-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     In this subroutine sub-problems are assembled and solved. The
!     sub-solutions are used for the non-constant parts of the projection
!     vectors.
!     Old subroutine prpjsubp
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleproj
      use sepmodulekprob
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (rmatrix) :: A
      integer :: iproj, isand, ishale, frstproj, &
                 ninbound, idxprojv, nlocdgfd, i1, i2, first2
      logical :: calcssol
      double precision, allocatable :: diag(:)

!     A              Structure of type matrix containing the necessary
!                    information of the matrix
!     calcssol   Logical variable, if true then the right hand side is
!                filled and a sub-solutions needs to be computed for
!                the combination of the current shale set and
!                the current sand set.
!     diag           Diagonal of the large matrix
!     first2     Start of loop
!     frstproj   First projection vector for shale set iSHALE
!     i1         Loop variable, runs from FIRST1 to LAST1
!     i2         Loop variable, runs from FIRST2 to LAST2
!     idxprojv   Index into array RTSPROJV
!     iproj      Dummy variable, number of a projection vector.
!     isand      Dummy variable, set number of a sand set.
!     ishale     Dummy variable, set number of a shale set
!     ninbound   Total number of degrees of freedom on internal boundaries
!                of the current shale set.
!     nlocdgfd   Number of degrees of freedom in the subsystem.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCG          Solve linear system by conjugate gradients
!     SEPMAKEPREC    Create preconditioning matrix if necessary
!     SEPPJFILLA     Fill information of submatrix into structure A of
!                    type rmatrix
!     SEPPJSUBB      Assemble the right hand side for the sub-system
!     SEPPJSUBF      Copy the sub-solution in a temporary array in BUFFER
!     SEPPJSUBI      Initialize variables concerning the solution process
!     SEPPJSUBM      Assemble the sub-matrix
!     SEPPJSUBR      Renumber the degrees of freedom in the shale layer and
!                    determine the structure of the sub-matrix
!     SEPSCALMAT     Scale the original large sparse matrix
!     SEPSCALMATBCK  Rescale the original large sparse matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Goal: Creating the non-constant parts of the projection vectors
!
!     Note that in this subroutine we loop over the shale layers for
!     which a subsolution has to be calculated. This approach is chosen
!     because then a shale layer only has to be renumbered once and
!     maximally one sub-matrix has to be assembled for a shale layer.
!
!     Semi PSD: seppjsubp
!
!     First initialize all variables that control the sub-solution
!     process in prpjSUBI
!
!     Next we loop over all interesting shale sets:
!
!     LOOP over all shale sets for which one or more sub-solutions
!     have to be calculated; see array SOLSHALE.
!        Renumber the degrees of freedom in the shale layer and
!        determine the structure of the submatrix in prpjSUBR.
!        Assemble the sub-matrix in prpjSUBM
!        LOOP over all sand neighbours
!           Check if for the current shale and current sand
!           set a subsolution has to be constructed. If thats
!           the case a right hand side is assembled or a
!           previous solution is reused in prpjSUBB
!           IF a subsolution has to be calculated THEN
!              Solve the subsystem with ICCG
!              Copy the solution to the temporay array
!              RTSPROJV in subroutin prpjSUBF
!           END IF
!        END LOOP
!     END LOOP
!
!     --- WARNING: in the declaration of MCPOS, MCCOL and MMCOEFF is assumed
!         that there are no more then 100 entries in a row of the large
!         matrix.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjsubp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjsubp'
         write(irefwr,1) 'nrusol', nrusol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize variables concerning the solution process of the
!         subsystem.

      call seppjsubi ( A )

      length = irowsii(nrusol+1)
      allocate ( nodcheck(npoint), pntlist(npoint), dmap(2*nusol), &
                 locintmt(length), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*(npoint+nusol), 'nodcheck' )
      if ( ierror/=0 ) go to 1000

!     --- Loop over all shale layers for which a solution needs to be
!         calculated

      idxprojv = 0

      do i1 = 1, nshale

!     --- Activate the arrays in iBUFRR

         ishale = solshale(i1)

!        --- Create a temporary array in IBUFFR for the first part of
!            the structure of the submatrix

         nlocdgfd = nsetnode(ishale+1)+nsetnode(ishale)

!        --- Renumber the nodes in the current shale layer in order to get
!            a local numbering and determine the structure (read size) of
!            the sub-matrix. Determine also the nodes that lie on the
!            interfaces between sand and shale layers.

         call seppjsubr ( ishale, nlocdgfd, ninbound )

         if ( debug ) write(irefwr,1) 'ishale, nlocdgfd', ishale, nlocdgfd

         length = irowsii(nrusol+1)
         allocate ( locrhsd(nlocdgfd), locsol(nlocdgfd), locmat(length), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, nlocdgfd, 'locrhsd' )
         if ( ierror/=0 ) go to 1000

!        --- Fill rows, columns and matrix

         call seppjsubm ( ishale, nlocdgfd )

!        --- Transform this information into structure A

         call seppjfillA ( A, nlocdgfd )

         call sepmakeprec ( A )  ! Make preconditioner
         allocate ( diag(A%neq) )
         diag = A%MD(A%diag)

!        --- Loop over all connected sand layers, assemble the right hand
!            side for the subsystem.

         first2   = spconsetsh(ishale)+1
         isand    = spconsetsa(first2)
         frstproj = sandproj(isand)

         do i2 = first2, spconsetsh(1+ishale)

            isand = spconsetsa(i2 )
            iproj = sandproj(isand)

            if ( debug ) write(irefwr,1) 'isand, iproj', isand, iproj

!           --- Check if a projection vector for this sand set must
!               be made and whether a subsystem needs to be solved.
!               In the latter case a right hand side is made

            call seppjsubb( ishale, isand, iproj, &
                            nlocdgfd, ninbound, &
                            frstproj, idxprojv, calcssol )
            if ( debug ) write(irefwr,*) 'calcssol', calcssol

            if ( calcssol ) then

!           --- Solve a subsystem

               call sepscalmat ( A, diag, locsol, locrhsd )
               call sepcg ( A, locsol, locrhsd )
               call sepscalmatbck ( A, diag, locsol, locrhsd )

!              --- Subsitute the solution in the array RTSPROJV

               call seppjsubf ( iproj, ishale, idxprojv )

            end if

         end do

         if ( associated(Pre_ii) ) deallocate(Pre_ii)
         if ( associated(locrhsd) ) deallocate(locrhsd)
         if ( associated(locsol) ) deallocate(locsol)
         if ( associated(locmat) ) deallocate(locmat)

         deallocate ( A%diag,  A%rows, A%cols, A%S, diag )

      end do

      deallocate ( nodcheck, pntlist, dmap, locintmt, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodcheck' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppjsubp'

      end if  ! ( debug )
      call erclos ( 'seppjsubp' )

      end subroutine seppjsubp
