      subroutine seppjsubr ( ishale, nlocdgfd, ninbound )
! ======================================================================
!
!        programmer     Gjalt Wijma
!        version  1.0  date 14-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine renumbers the degrees of freedom in the shale set.
!     The nodes that lie on the interfaces with the surrounding sand
!     sets are saved. The structure of the submatrix is also determined.
!     Old subroutine prpjsubr
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleproj
      use sepmodulekprob
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer ishale, nlocdgfd, ninbound

!     ishale      i  Set number of a shale set
!     ninbound    o  Total number of degrees of freedom on internal
!                    boundaries of the current shale set.
!     nlocdgfd    o  Number of degrees of freedom in the subsystem.
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inodp, idegfd, npntlist, ipntlist, ilocdgfd, iset, indx1, &
              jnodp, jdegfd, jset

!     idegfd         Dummy variable, number of a degree of freedom
!     ilocdgfd       Dummy variable, number of a degree of freedom, local
!                    numbering.
!     indx1          Loop variable, runs from FIRST to LAST
!     inodp          Dummy variable, nodal point number
!     ipntlist       Index into PNTLIST
!     iset           Dummy variable, set number.
!     jdegfd         Dummy variable, degree of freedom.
!     jnodp          Dummy variable, nodal point number.
!     jset           Dummy variable, set number.
!     npntlist       Number of nodal points in PNTLIST
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1667   There is an element group that consists of separate areas
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Goal: renumbering of degrees of freedom in shale set iSHALE.
!
!     Note that we are trying to create a sparse, real, symmetric
!     matrix. So only the lower half of the matrix is stored in
!     sparse form, see a SEPRAN manual for details.
!
!     Note that in this subroutine only the structure of the sub-
!     matrix is determined, in this case that is the number of
!     elements in each row of the lower triangular part of the
!     submatrix
!
!     Semi PSD: seppjsubr
!
!     We start with the first node in set iSHALE and add it to the
!     array PNTLIST.
!
!     DO until all nodes in PNTLIST are processed
!        Take the next node from array PNTLIST and determine
!        which set it lies in.
!        IF the node lies in the shale set THEN
!           add a new degree of freedom, store it in DMAP and
!           remember that we work with a new row in the
!           submatrix.
!        ELSE
!           The node lies on an interface. The accompanying degree
!           of freedom is saved for later in array DMAP. (These
!           degrees of freedom are used for building the right
!           hand side in routine seppjsubr).
!        END IF
!        Next all neighbors of the current node are checked:
!        LOOP over all neighbors
!           IF the neighbor node lies in the shale set THEN
!              there are two possibilities:
!              IF the neighbor node has been checked before AND
!              the current node lies in the shale set THEN
!                 add a new column in the submatrix for the
!                 current row (=> the neighbor has been
!                 renumbered before the current node, so its
!                 number has to be lower.)
!              ELSE
!                 The node has not been checked. Add it to the
!                 back of array PNTLIST and mark it as checked in
!                 array NODCHECK.
!              END IF
!           ELSEIF the neighbor node lies on an interface AND the
!           neighbor node has not been checked
!           THEN
!              The neighbor is added to the back of pntlist.
!           END IF
!        END LOOP
!     END DO
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjsubr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjsubr'
         write(irefwr,1) 'ishale', ishale
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays NODCHECK and DMAP

      nodcheck = 0
      dmap = 0

!     --- There are no nodes on an internal boundary yet, so there
!         are no nodes connected to them either.

      ninbound = 0
      nlocdgfd = nsetnode(ishale+1) - nsetnode(ishale)

!     --- renumbering in order to get local numbers of degree of freedom.

      inodp = setnode(nsetnode(ishale)+1)
      pntlist(1) = inodp
      npntlist = 1
      ipntlist = 0
      ilocdgfd = 0
      locintmt(1) = 0
      nodcheck(inodp) = 1

100   if ( ipntlist<npntlist ) then
         ipntlist = ipntlist+1
         inodp = pntlist(ipntlist)
         idegfd = kprobh(inodp)
         iset = nodeset(inodp)

         if ( iset==ishale ) then

!        --- Node lies in the shale set, a new local degree
!            of freedom is added.

            ilocdgfd = ilocdgfd+1
            dmap(idegfd) = ilocdgfd
            dmap(nusol+ilocdgfd) = idegfd
            locintmt(ilocdgfd+1) = locintmt(ilocdgfd)

         else

!        --- Node lies on an internal boundary

            ninbound = ninbound+1
            dmap(nusol + nlocdgfd + ninbound) = idegfd

         end if

         do indx1 = kmshe1(inodp)+1, kmshe1(inodp+1)

            jnodp = kmshe2(indx1)
            jdegfd = kprobh(jnodp)
            jset = nodeset(jnodp)

            if ( jset==ishale ) then

!           --- The connected node lies in the shale set

               if ( nodcheck(jnodp)==1 ) then

!              --- The connected node has been checked

                  if ( dmap(jdegfd)>0.and. iset==ishale ) then

!                 --- The connected node has already a new number,
!                     since that number must be less than iLOCDGFD
!                     we get a new matrix element for the subsystem
!                     in row nLOCDGFD.

                     locintmt(1+ilocdgfd) = locintmt(1+ilocdgfd)+1

                  end if

               else

!              --- The connected node has not been checked yet

                  npntlist = npntlist+1
                  pntlist(npntlist) = jnodp
                  nodcheck(jnodp) = 1

               end if

            else if ( jset<0 ) then

!           --- The connected node lies on an internal boundary, &

               if ( nodcheck(jnodp)==0 ) then

!              --- The connected node lies on an internal boundary, &
!                  and has not been checked out yet, so it is added
!                  to the list.

                  npntlist = npntlist+1
                  pntlist(npntlist) = jnodp
                  nodcheck(jnodp) = 1

               end if

            end if

         end do
         go to 100

      end if

!     --- ASSUMED IS THAT ALL NODES IN AN ELEMENT GROUP ARE CONNECTED!

      if ( ilocdgfd/=nlocdgfd ) call errsub ( 1667, 0, 0, 0 )

1000  call erclos ( 'seppjsubr' )
      if ( debug ) then

!     --- Debug information

         call prinin ( locintmt, nlocdgfd+1, 'locintmt/1' )
         write(irefwr,*) 'End seppjsubr'

      end if  ! ( debug )

      end subroutine seppjsubr

