      subroutine septimeulexpl ( finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!
!     subroutine for the time integration of the discretized partial
!     differential equation
!               .
!           M * u + S * u = F
!               -       -   - &
!     It has been assumed that the matrices M and S are not dependent
!     of the solution vector u . Hence the system is linear . M and
!     S may of course depend on place and time .
!     Solution method used is the Explicit Euler method . The user
!     has to give the stepsize tstep.
!     Old subroutine pr1107
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduleuser
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if the time loop has been finished (true)
!                         or not  (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isolprec(maxprb), itemp, i, ieqn, ieqnmat
      integer, target :: isol(maxprb+1)
      logical finishtm, first

!     finishtm       Indicates if the time loop has been finished (true)
!                    or not (false
!     first          If true it concerns the first step
!     i              Counting variable
!     ieqn           Equation sequence number
!     ieqnmat        Equation sequence number for matrix
!     isol           Standard SEPRAN array containing information of the
!                    solution array
!     isolprec       Solution at preceding level
!     itemp          Index for temporary array ISTART
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINTHIS       Print time history
!     SEPCOPYVEC     copy Sepran array
!     SEPLOCTRN      Compute (back-)transformation in case of local transform
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMBOUNCOND Fill boundary conditions in array isol and check if the
!                    array corresponds to the correct problem
!     SEPTIMEULEXST  Perform one step in an Euler explicit scheme for one
!                    equation
!     SEPTIMIINBLD   Fill some parameters in iinbld and initialize
!                    some other parameters for time integration
!     SEPTIMRESET    reset all arrays with respect to time integration that
!                    are no longer used
!     SEPTIMUSER     Fill arrays iuser and user for a specific
!                    problem in a sequence of problems
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimeulexpl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimeulexpl'
         write(irefwr,*) 'Debug information from pr1107'
         write(irefwr,2) 't, t0, tstep, tout', t, t0, tstep, tout
         write(irefwr,2) 'tend, toutend', tend, toutend
         write(irefwr,*) 'complrun', complrun
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays

      isol = 0
      isolprec = 0
      itemp = 0

!     --- Start of process
!         Set parameters

      finishtm = .false.
      first = .false.
      if ( t<=t0+0.1d0*tstep ) first = .true.

      if ( debug ) write(irefwr,1) 'ncopeqtim', ncopeqtim

!     --- Initialize array isol

      do i = 1, ncopeqtim

!     --- First the vector is copied from isolut2dred

         call sepcopyvec ( isolut2dred(Teq(i)%iseqfr), isol(i), 0 )

!        --- Next it is transformed from Cartesian coordinates to new
!            coordinates in case of local transformations

         call seploctrn ( 1, isol(i) )

      end do

      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinin ( isolut2dred(Teq(1)%iseqfr), 5, 'isolut2dred' )

!     --- While loop over time-steps

      do while ( .not. finishtm )

!     --- t<tout
!         Copy solution at old time level into isolprec

         if ( debug ) write(irefwr,2) 't, tout', t, tout
         do i = 1, ncopeqtim

            call sepcopyvec ( isolut2dred(Teq(i)%iseqfr), isolprec(i), 0 )

!           --- Next it is transformed from Cartesian coordinates to new
!               coordinates in case of local transformations

            call seploctrn ( 1, isolprec(i) )

         end do
         if ( ierror/=0 ) go to 1000

!        --- set some parameters, which are used in the time loop

         dt = tstep

!        --- Put boundary conditions into isol

         if ( itimbc<2 .and. (first .or. itimbc==0) ) then

!        --- Either first call, or boundary conditions are time-dependent
!            Mark that for the ncopeqtim problems only one
!            call to presbc is used
!            The boundary conditions are always given in the transformed form
!            They must be filled on the new time level

            t = t+dt  ! raise t
            if ( debug ) write(irefwr,2) 't before bc', t
            isol2d => isol
            call septimbouncond ( first )
            t = t-dt  ! reset t

         end if  ! ( itimbc<2 .and. (first .or. itimbc==0) )
         if ( ierror/=0 ) go to 1000

         iseqdt = 0

         do ieqn = 1, ncopeqtim

!        --- First set the correct parameters in iinbld

            ieqnmat = ieqn+icalltim-1
            call septimiinbld ( first, ieqn )
            if ( ierror/=0 ) go to 1000

!           --- Compute starting addresses for the arrays iuser and user

            intmat => intmatar(ieqnmat)
            iprobtim = intmt(intmat)%iprob
            massmat => massmt(ieqnmat)
            stiffmat => stiffm(ieqnmat:maxprb)
            totmat => mattot(ieqnmat)

            call  septimuser ( first, ieqn )
            if ( ierror/=0 ) go to 1000

!           --- Perform one step Euler explicit

            if ( debug ) write(irefwr,1) 'itimcf', itimcf
            call septimeulexst ( first, isol, irhsd(ieqnmat), &
                                 isolprec(ieqn), ieqn )

            if ( debug ) then
               write(irefwr,1) 'ieqn', ieqn
               call sepprintv ( isol(ieqn), 'solution' )
            end if  ! ( debug )
            if ( ierror/=0 ) go to 1000

!           --- copy isol into isolut2dred
!               First save isolut2dred in itemp and then copy itemp into isol
!               In this way isol contains the solution at the old level
!               and isolut2dred at the new level

            itemp = 0
            call sepcopyvec ( isolut2dred(Teq(ieqn)%iseqfr), itemp, 0 )
            call sepcopyvec ( isol(ieqn), &
                             isolut2dred(Teq(ieqn)%iseqfr), 0 )
            call sepcopyvec ( itemp, isol(ieqn), 1 )
            call seprmsolinfo ( itemp )

         end do  !  ieqn = 1, ncopeqtim

         if ( complrun .or. keeptim/=1 )  t = t+dt

         if ( nhist>0 ) then

!        --- print time history

            call printhis ( nhist, nodhist, t, ncopeqtim, &
                            isolut2dred(Teq(1)%iseqfr), first )

         end if
         if ( t>=tout-0.0001d0*tstep .or. &
              .not. complrun ) finishtm = .true.
         first = .false.
         if ( debug ) write(irefwr,2) 't, tout', t, tout

      end do  ! while ( .not. finishtm )

!     --- While loop has been finished, reset quantities

      iflag = 3

      if ( t>=tend-0.00001*tstep ) then

!     --- t = tend   reset all arrays

         call septimreset ( irhsd(ieqnmat), &
                            isol, isolprec(ieqn), finished )

      end if  ! ( t>=tend-0.00001*tstep )

!     --- reset arrays isol and isolprec

      do i = 1, maxprb+1
         if ( isol(i)/=0 ) &
            call seprmsolinfo ( isol(i) )
         if ( i/=maxprb+1 ) then
            if ( isolprec(i)/=0 ) &
               call seprmsolinfo ( isolprec(i) )
         end if  ! ( i/=maxprb+1 )
      end do  ! i = 1, maxprb

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimeulexpl'

      end if  ! ( debug )
      call erclos ( 'septimeulexpl' )

      end subroutine septimeulexpl
