      subroutine septimnewmark ( finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine for the time integration of the discretized partial
!     differential equations:
!             2    2
!         M  d u/dt    +  S  u  = f
!     The equations are discretized by a Newmark scheme or the generalized
!     alpha scheme
!     Old subroutine pr1114
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
!     newmark
!     generalized_alpha
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduleuser
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if the time loop has been finished (true)
!                         or not  (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: maxprbloc=10
      integer, target :: isol(maxprb+1)
      integer i, ieqn, irhsdold(2), &
              niter, limit, iseqmat, iaction, &
              istart, istextra, istieq, inneritr
      double precision tsteps
      logical first, stationar, finishtm, linear, converged
      character (len=25) text
      save tsteps, first, niter, irhsdold

!     converged      If true the non-linear iteration has been converged
!     finishtm       Indicates if the time loop has been finished (true)
!                    or not (false)
!     first          If true it concerns the first time step
!     i              Counting variable
!     iaction        Defines the type of action to be performed
!     ieqn           Equation sequence number
!     inneritr       Sequence number of inner iteration in case of non-linear
!                    iteration per time-step
!     irhsdold       Sepran array to store the vector
!                    alpha_f F^n - alpha_m M a^n  - alpha_f S u^n
!     iseqmat        Sequence number of matrix in set of matrices
!     isol           Standard SEPRAN array containing information of the
!                    solution array
!     istart         First position used at a part
!     istextra       Start position of extra information in iintim
!     istieq         Start position of the equations in iintim
!     limit          Defines type of limiter applied to a solution
!                    Possible values:
!                    0: no limiter
!                    1: solution is limited between minimum and maximum value
!                       at start
!     linear         If true the linear time-iteration is used, if false an
!                    iteration per time step is performed
!     maxprbloc      Local value of maxprb (Must be at least equal to maxprb)
!     niter          Iteration sequence number
!     stationar      Indicates if a stationary solution must be found (true)
!                    or not (false)
!     text           text with respect to writing or plotting
!     tsteps         help variable to save the value of tstep
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINTHIS       Print time history
!     SEPCNVCHK      Check convergence of iteration
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPCOPYVEC     copy Sepran array
!     SEPNEWMARKST   Compute starting value for Newmark and generalized
!                    alpha scheme
!     SEPNEWMARKSTEP Perform one step in the solution of the wave equation
!                    Either Newmark or generalized alpha is used
!     SEPNEWMARKUPDH Compute F* = alpha_s F^n - M alpha_m a^n - S alpha_f u^n
!                    Also compute h^n = u^n + dt v^n + dt^2/2 ( 1 - 2 beta) a^n
!     SEPNEWMARKUPDR Compute a^n+1 and v^n+1
!                    as part of the generalized alpha method
!                    Update the mesh if necessary
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMBOUNCOND Fill boundary conditions in array isol and check if the
!                    array corresponds to the correct problem
!     SEPTIMEXTRA    Perform extra actions in time step
!     SEPTIMIINBLD   Fill some parameters in iinbld and initialize
!                    some other parameters for time integration
!     SEPTIMRESET    reset all arrays with respect to time integration that
!                    are no longer used
!     SEPTIMTHETA1   Start the time integration for suboutine septimtheta
!     SEPTIMTHETA2   Set some parameters during a time loop
!     SEPTIMUSER     Fill arrays iuser and user for a specific
!                    problem in a sequence of problems
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      The Newmark methods is given by:
!
!      u^n+1 = u^n + dt v^n + dt^2/2 [ ( 1 - 2 beta) a^n + 2 beta a^n+1 ] (1)
!      v^n+1 = v^n + dt [ (1-gamma) a^n + gamma a^n+1 ]                   (2)
!      M a^n+1 + S u^n+1 = F^n+1                                          (3)
!
!      with u the displacement, v the velocity and a the acceleration
!      These vectors are stored as three consecutive vectors
!      At t=0: a^n+1 is computed from (3)
!      In the next steps (1) is substituted in (3) to compute a^n+1
!      After that u^n+1 and v^n+1 are computed from (1) and (2)
!      The Newmark method can also be written as:
!      (M + beta S dt^2) u^n+1 = -[M + (1/2-gamma+beta) S dt^2] u^n-1 + &
!      [2M - (1/2+gamma-2 beta) S dt^2] u^n + dt^2 F* &
!      with F* = (1/2-gamma+beta)F^n-1 + (1/2+gamma-2 beta)F^n + beta F^n+1
!
!      beta = 0, gamma = 1/2 gives the central difference scheme
!      beta = 1/4, gamma = 1/2 is second order accurate
!
!      The generalized alpha scheme replaces the third step by:
!
!      M (1-alpha_m) a^n+1 + S (1-alpha_f) u^n+1 = (1-alpha_f) F^n+1 + F*, &
!      with F* = alpha_f F^n - M alpha_m a^n - S alpha_f u^n
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data irhsdold /2*0/
! ======================================================================
!
      call eropen ( 'septimnewmark' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimnewmark'
         write(irefwr,2) 't, t0, tstep, tout', t, t0, tstep, tout
         write(irefwr,2) 'tend', tend
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if

      call inichkarlen ( maxprb, maxprbloc, 'diffsl' )
      if ( ierror/=0 ) go to 1000
      ieqn = 1      ! temporary only one equation

      first = .false.
      if ( t<=t0+0.1d0*tstep ) first = .true.
      tsteps = tstep  ! save value of tstep

!     --- Clear arrays

      isol(1:maxprb) = 0

!     --- Start of process
!         Set parameters

      call septimtheta1 ( finishtm, first, linear )
      if ( debug ) write(irefwr,*) 'first', first
      if ( ierror/=0 ) go to 1000

!     --- Initialize array isol

      call sepcopyvec ( isolut2dred(iseqfrtim), isol, 0 )
      if ( debug ) call sepprintv ( isolut2dred(iseqfrtim), 'sol_in' )
      iseqmat = 1
      massmat => massmt(iseqmat)
      stiffmat => stiffm(iseqmat:maxprb)
      totmat => mattot(iseqmat)

!     --- Temporarily:  Set (4) and (5) in case of updated Lagrangian

      if ( lineartim==3 .or. lineartim==4 ) then

!     --- Updated Lagrangian
!         At this moment it must be programmed such that the first vector
!         is u, the second one un and the third one du
!         u is the displacement vector per pseudo time step
!         un is the total displacement vector
!         du is the iterative solution
!         This is due to the programming of elm200
!         elm200 should be adapted as soon as possible
!         We start with u a copy of un, this is for septimnewmarkstart
!         At the end of that subroutine u is cleared

         Teq(4)%iseqfr = iseqfrtim+1  ! un
         call sepcopyvc ( isolut2dred(Teq(4)%iseqfr), isolut2dred(iseqfrtim) )

      end if ! ( iimtim(3)==3 ) then

      if ( t<=t0*(1d0+epsmac) ) then

!     --- First time-step, i.e. du/dt is given in isolut2dred(*,Teq(2)%iseqfr)
!         For central differences: Set u(-dt) into isolut2dred(*,2)
!         In case of Newmark store a^0 into isolut2dred(,3)
!         In order to fill a^0 in the boundary it is necessary
!         to compute u^1 at the boundary, store in isol

         t = t+tstep  ! Set t equal to t^1
         isol2d => isol
         call septimbouncond ( first )
         t = t-tstep  ! Reset t

         call sepnewmarkst ( isol, irhsd(iseqmat), ieqn, irhsdold )

      end if  ! ( t<=t0*(1d0-epsmac) )
      if ( ierror/=0 ) go to 1000

!     --- While loop over time-steps

      do while ( .not. finishtm )

!     --- t<tout
!         set some parameters, which are used in the time loop

         call septimtheta2 ( tsteps, 1 )

!        --- Compute vector h^n, which depends on solution at t = n
!            and also vector
!            Store both vectors in irhsdold

         call sepnewmarkupdh ( irhsdold, irhsd(iseqmat) )

         if ( lineartim==3 .or. lineartim==4 ) then

!        --- lineartim=3, 4, Updated Lagrange method(Newton)
!            Clear array u

            usol => ks(isolut2dred(iseqfrtim))%sol
            usol = 0d0

         end if  ! ( lineartim==3 .or. lineartim==4 )

!        --- Put boundary conditions into isol

         if ( itimbc<2 .and. (first .or. itimbc==0) ) then

!        --- Either first call, or boundary conditions are time-dependent
!            Mark that for the ncopeq problems only one
!            call to presbc is used
!            The boundary conditions are always given in the transformed form
!            They must be filled on the new time level

            isol2d => isol
            call septimbouncond ( first )
            if ( debug ) call sepprintv ( isol, 'sol_bc' )

         end if  ! ( itimbc<2 .and. (first .or. itimbc==0) )

         iseqdt = 0
         istextra = 0
         inneritr = 0
         converged = .false.
         ieqn = 1

         do while ( .not. converged .and. inneritr<maxitertim )

!        --- While loop for non-linear iteration
!            Loop over all problems

            inneritr = inneritr+1
            if ( debug ) &
               write(irefwr,*) 'inneritr, maxiter, converged ', &
                                inneritr, maxitertim, converged

!           --- First set the correct parameters in iinbld

            call septimiinbld ( first, ieqn )

!           --- Compute starting addresses for the arrays iuser and user

            call septimuser ( first, ieqn )
            if ( ierror/=0 ) go to 1000

!           --- Perform one time step Newmark scheme

            call sepnewmarkstep ( isol, irhsd(iseqmat), &
                                  first, ieqn, irhsdold )
            first = .false.  ! From now on first is false

            if ( Teq(ieqn)%lenextra>0 ) then

!            --- Extra actions must be performed

               call septimextra ( ieqn, first )

            end if  ! ( Teq(ieqn)%lenextra>0 )

            if ( debug ) write(irefwr,1) 'lineartim', lineartim
            if ( lineartim==3 .or. lineartim==4 ) then

!           --- Non-linear iteration, check on convergence

               call sepcnvchk ( isol, inneritr, converged, text )

            else

!           --- Linear process, no repeating

               converged = .true.

            end if  ! ( lineartim==3, 4 )

!           --- end while loop

         end do  ! while ( .not. converged .and. inneritr<maxitertim )

         if ( nhist>0 ) then

!        --- print time history

            call printhis ( nhist, nodhist, t, ncopeqtim, &
                            isolut2dred(iseqfrtim), first )

         end if

         if ( t>=tout-0.0001d0*tstep .or. .not. complrun ) &
            finishtm = .true.
         if ( debug ) write(irefwr,2) 't, tout', t, tout

!        --- update F* and h^n
!            Compute v^n+1 and a^n+1

         call sepnewmarkupdr ( irhsdold )

         if ( ierror/=0 ) go to 1000

      end do  ! while ( .not. finishtm )

!     --- While loop has been finished, reset quantities

      tstep = tsteps
      iflag = 3

      if ( t>=tend-0.00001*tstep ) then

!     --- t = tend   reset all arrays

         call septimreset ( irhsd(iseqmat), isol, isol, finished )
         if ( irhsdold(1)/=0 ) call seprmsolinfo ( irhsdold(1) )
         if ( irhsdold(2)/=0 ) call seprmsolinfo ( irhsdold(2) )

      end if  ! ( t>=tend-0.00001*tstep )

!     --- reset arrays isol and isolprec

      do i = 1, maxprb
         if ( isol(i)/=0 ) call seprmsolinfo ( isol(i) )
      end do  ! i = 1, maxprb
      if ( isol(2)/=0 )  call seprmsolinfo ( isol(2) )
      if ( isol(3)/=0 )  call seprmsolinfo ( isol(3) )

1000  if ( debug ) then
         write(irefwr,2) 't', t
         call sepprintv ( isolut2dred(iseqfrtim), 'u^n+1' )
         call sepprintv ( isolut2dred(Teq(2)%iseqfr), 'v^n+1' )
         call sepprintv ( isolut2dred(Teq(3)%iseqfr), 'a^n+1' )
         write(irefwr,*) 'End septimnewmark'
      end if  ! ( debug )
      call erclos ( 'septimnewmark' )

      end
