      subroutine septimsecond ( finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine for the time integration of the discretized partial
!     differential equations:
!             2    2
!         M  d u/dt    +  S  u  = f
!     The equations are discretized by a second order finite difference scheme
!     with fixed time-step
!     Old subroutine pr1110
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduleuser
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if the time loop has been finished (true)
!                         or not  (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: maxprbloc=10
      integer, target :: isol(maxprb+1)
      integer i, ieqn, iseqfr, &
              isolprec(maxprb), niter, iaction, &
              istart, istextra, istieq, inneritr
      double precision tsteps
      logical first, finishtm, linear, converged
      character (len=25) text
      save tsteps, first, niter

!     converged      If true the non-linear iteration has been converged
!     finishtm       Indicates if the time loop has been finished (true)
!                    or not (false)
!     first          If true it concerns the first time step
!     i              Counting variable
!     iaction        Defines the type of action to be performed
!     ieqn           Equation sequence number
!     inneritr       Sequence number of inner iteration in case of non-linear
!                    iteration per time-step
!     iseqfr         See array iintim
!     isol           Standard SEPRAN array containing information of the
!                    solution array
!     isolprec       Solution at preceding level
!     istart         First position used at a part
!     istextra       Start position of extra information in iintim
!     istieq         Start position of the equations in iintim
!     linear         If true the linear time-iteration is used, if false an
!                    iteration per time step is performed
!     maxprbloc      Local value of maxprb (Must be at least equal to maxprb)
!     niter          Iteration sequence number
!     text           text with respect to writing or plotting
!     tsteps         help variable to save the value of tstep
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINTHIS       Print time history
!     SEPCOPYVEC     copy Sepran array
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMBOUNCOND Fill boundary conditions in array isol and check if the
!                    array corresponds to the correct problem
!     SEPTIMEXTRA    Perform extra actions in time step
!     SEPTIMIINBLD   Fill some parameters in iinbld and initialize
!                    some other parameters for time integration
!     SEPTIMRESET    reset all arrays with respect to time integration that
!                    are no longer used
!     SEPTIMSECONDST Perform one step in the solution of the wave equation
!     SEPTIMTHETA1   Start the time integration for suboutine septimtheta
!     SEPTIMTHETA2   Set some parameters during a time loop
!     SEPTIMUSER     Fill arrays iuser and user for a specific
!                    problem in a sequence of problems
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      The standard central difference scheme is applied:
!          n+1    n    n-1
!         u   - 2u  + u           n      n
!      M ---------------- +  S u    = f
!            dt**2
!              n+1   -1  2          n        n    n-1
!      Hence  u   = M  dt  ( f - S u  ) + 2 u  - u
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimsecond' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimsecond'
         write(irefwr,2) 't, t0, tstep, tout', t, t0, tstep, tout
  2      format ( a, 1x, (5d12.4) )

      end if
      call inichkarlen ( maxprb, maxprbloc, 'diffsl' )
      if ( ierror/=0 ) go to 1000

      first = .false.
      if ( t<=t0+0.1d0*tstep ) first = .true.
      tsteps = tstep  ! save value of tstep

!     --- Clear arrays

      isol = 0
      isolprec = 0
      iseqfr = iseqfrtim

!     --- Start of process
!         Set parameters

      call septimtheta1 ( finishtm, first, linear )
      do i = 1, ncopeqtim

         call sepcopyvec ( isolut2dred(Teq(i)%iseqfr), isol(i), 0 )
      end do  ! i = 1, ncopeqtim

      if ( debug ) &
         call sepprintv ( isolut2dred(iseqfr), 'sol_in' )

      if ( t<=t0*(1d0-epsmac) ) then

!     --- First time-step, i.e. du/dt is given in isolut2dred(*,iseqfr+1)
!         Set u(-dt) into isolut2dred(*,2)

         u1 => ks(isolut2dred(iseqfr))%sol
         u2 => ks(isolut2dred(iseqfr+1))%sol
         u2 = u1 -tstep*u2

      end if  ! ( t<=t0*(1d0-epsmac) )

!     --- While loop over time-steps

      do while ( .not. finishtm )

!     --- t<tout
!         set some parameters, which are used in the time loop

         call  septimtheta2 ( tsteps, 1 )

!        --- Put boundary conditions into isol

         if ( itimbc<2 .and. (first .or. itimbc==0) ) then

!        --- Either first call, or boundary conditions are time-dependent
!            Mark that for the ncopeqtim problems only one
!            call to presbc is used
!            The boundary conditions are always given in the transformed form
!            They must be filled on the new time level

            isol2d => isol
            call septimbouncond ( first )
            if ( debug ) call sepprintv ( isol, 'sol_bc' )

         end if  ! ( itimbc<2 .and. (first .or. itimbc==0) )

         iseqdt = 0
         istextra = 0
         inneritr = 0
         converged = .false.
         do while ( .not. converged .and. inneritr<maxitertim )

!        --- While loop for non-linear iteration
!            Loop over all problems

            inneritr = inneritr+1
            if ( debug ) write(irefwr,*) 'inneritr, converged, ncopeqtim ', &
               inneritr, converged, ncopeqtim

            do ieqn = 1, ncopeqtim

!           --- First set the correct parameters in iinbld

               call septimiinbld ( first, ieqn )

!              --- Compute starting addresses for the arrays iuser and user

               intmat => intmatar(ieqn)
               iprobtim = intmt(intmat)%iprob
               call  septimuser ( first, ieqn )

!              --- Perform one time step central difference scheme

               call septimsecondst ( isol, first, ieqn )

!              --- copy isol into isolut2dred
!                  First save isolut2dred in isolut2dred(.,2)
!                  In this way isolold(.,2) contains the solution at the old
!                  level
!                  and isolut2dred(.,1) at the new level

               call sepcopyvec ( isolut2dred(Teq(ieqn)%iseqfr), &
                                 isolut2dred(Teq(ieqn)%iseqfr+1), 0 )
               call sepcopyvec ( isol(ieqn), &
                                 isolut2dred(Teq(ieqn)%iseqfr), 0 )

               if ( Teq(ieqn)%lenextra>0 ) then

!               --- Extra actions must be performed

                  call septimextra ( ieqn, first )

               end if  ! ( Teq(ieqn)%lenextra>0 )

!              --- end of loop over the equations

            end do

!           --- Linear process, no repeating

            converged = .true.

!           --- end while loop

         end do

         if ( nhist>0 ) then

!        --- print time history

            call printhis ( nhist, nodhist, t, ncopeqtim, &
                            isolut2dred(iseqfr), first )

         end if
         if ( t>=tout-0.0001d0*tstep .or. .not. complrun ) &
            finishtm = .true.
         first = .false.
         if ( debug ) write(irefwr,2) 't, tout', t, tout
         if ( ierror/=0 ) go to 1000

      end do

!     --- While loop has been finished, reset quantities

      tstep = tsteps
      iflag = 3

      if ( t>=tend-0.00001*tstep ) then

!     --- t = tend   reset all arrays

         call septimreset ( irhsd, isol, isolprec, finished )

      end if  ! ( t>=tend-0.00001*tstep )

!     --- reset arrays isol and isolprec

      do i = 1, maxprb
         if ( isol(i)/=0 ) call seprmsolinfo ( isol(i) )
         if ( isolprec(i)/=0 ) call seprmsolinfo ( isolprec(i) )
      end do  ! i = 1, maxprb
      if ( isol(2)/=0 ) call seprmsolinfo ( isol(2) )

1000  if ( debug ) then
         write(irefwr,2) 't', t
         call sepprintv ( isolut2dred(iseqfr),'u^n+1' )
         call sepprintv ( isolut2dred(iseqfr+1), 'u^n' )
         write(irefwr,*) 'End septimsecond'
      end if  ! ( debug )
      call erclos ( 'septimsecond' )

      end
