      subroutine septmenthalpy2 ( first, isolprec, iinenthalpy, &
                                  rinenthalpy, ideltatemp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-12-2017 New storage of intmat
!        version  1.0  date 12-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Perform one step in an Euler implicit scheme for the enthalpy equation.
!    Coefficients may depend on time and previous solutions
!    It is supposed that the time has been raised outside this subroutine
!    Also that boundary conditions have been filled as well as arrays iuser and
!    user
!    This is the basic subroutine with respect to time integration.
!    All decisions are made outside this subroutine.
!    Old subroutine prtmenthalpy2
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulematr
      use sepmodulemnsub
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleintmat
      use sepmodulesolve
      use sepmodulecons
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isolprec(2), iinenthalpy(*), ideltatemp
      double precision rinenthalpy(*)
      logical first

!     first          i    If true it concerns the first step
!     ideltatemp    i/o   Sepran array of length 5 containing information
!                         concerning the temperature difference during the
!                         iteration
!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     isolprec       i    Solution at previous time-level
!     rinenthalpy    i    Real input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqlc, ihelp, ihelp1, iter, &
              iseqtemp, irhsdspecial, iseqenth, totiter, &
              ntimesteps
      double precision mu, eps, resnorm, meaniter, resnorm1, xhist, &
                       yhist, zhist
      logical converged
      save meaniter, totiter, ntimesteps, resnorm1
      double precision, pointer :: MASS(:)
      double precision, pointer :: STIFF(:), TOTM(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     MASS           Refers to mass matrix
!     STIFF          Refers to stifness matrix
!     TOTM           Refers to combination of mass and stiffness matrix
!     converged      If true the iteration process has been converged
!     eps            Local accuracy
!     ihelp          sepran help array to store a temporary vector
!     ihelp1         sepran help array to store a temporary vector
!     irhsdspecial   sepran help array to store a temporary vector
!     iseqenth       Sequence number of enthalpy with respect to array isol
!     iseqlc         Sequence number
!     iseqtemp       Sequence number of temperature in solution vector
!     iter           Iteration sequence number
!     meaniter       Average number of iterations per time step
!     mu             relaxation parameter in iteration process
!     ntimesteps     Total number of time steps
!     resnorm        Norm of residual
!     resnorm1       Norm of initial residual
!     totiter        Total number of iterations
!     xhist          x-coordinate of point to keep a history of
!     yhist          y-coordinate of point to keep a history of
!     zhist          z-coordinate of point to keep a history of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FINDXY         Find nodal point closest to a specific point
!     PRCOMPENTHALP2 Compute the enthalpy, given the temperature
!                    Newton approach
!     PRCOMPENTHALP3 Compute the special part of the right-hand side of the
!                    newton iteration, given the temperature and enthalpy
!                    in the previous iteration
!                    Newton approach, i.e. enthalpy is defined per
!                    integration point
!     PRCOMPENTHALP4 Compute the enthalpy, given the temperature
!                    update the enthalpy during the iterations
!                    Newton approach, i.e. enthalpy is defined per integration
!                    point
!     PRININ         print 1d integer array
!     SEPCOPINPSOL   Copy arrays inpsl1 and rinsl1 into inpsol and rinsol
!                    into sepmodulesolve
!     SEPCOPYMT      Copy matrix
!     SEPLINSOL      Body of linear solver
!     SEPMAVER       perform matrix vector multiplication
!     SEPPRINTMAT    Print matrix
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMBUILDMAT Build mass matrix, stiffness matrix and right-hand side
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1907   No convergence before maximum number of iterations has been reached
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Differential equations to be solved:
!
!        .
!     M  u   +  S  u   =  f
!
!     Application of Euler
!
!     -Euler part (predictor):
!
!       n+1     n
!      u    -  u            n+1      n
!    M ----------- +  S  u     =  f          (n is actual time-level)
!          dt
!
!
!    or. &
!
!                 n+1           n    n+1
!   ( M/dt + S ) u     =  M/dt u  + f
!
!
!   Effect of boundary conditions:
!
!   We split the matrix into an internal part and a part due to
!   boundary conditions. The Euler part can then be written as:
!
!                        .
!   |  M       M   |  |  u  |      |  S       S   |  |  u  |      |  f  |
!   |   ii      ir |  |   i |      |   ii      ir |  |   i |      |   i |
!   |              |  |  .  |   +  |              |  |     |  =   |     |
!   |  M       M   |  |  u  |      |  S       S   |  |  u  |      |  f  |
!   |   ri      rr |  |   r |      |   ri      rr |  |   r |      |   r |
!
!   The last row is deleted, hence the discretized equations become:
!
!                   n+1       n+1        n+1            n            n+1   n
!   ( M/dt  + S  ) u    =  ( f    - S   u   )  + M/dt  u   - M/dt ( u   - u )
!      ii      ii   i         i      ir  r        ii    i     ir     r     r
!
!
!
!   For the enthalpy equation we have a more difficult approach:
!
!   dH/dt + ST = Q --> M (H^(n+1)-H^n)/dt + ST^(n+1) = Q^(n+1)
!   Define Q^ = Q^(n+1) + M H^n/dt
!
!   Iteration defined by Nedjar
!
!   T^1 = T^n with boundary conditions at n+1
!   H^1 = H^n with boundary conditions at n+1
!   T=tau(H),  H^(i+1) = H^i + delta H^i
!   T^(i+1) = T^i + delta T^i = tau(H^i) +  tau'(H^i) + delta H^i
!   So:  delta H^i = [delta T^i+(T^i-tau(H^i))]/tau'(H^i)
!   Since it is difficult to compute 1/tau'(H^i), we approximate it by
!   a constant mu
!
!   Then:  (M stands for M/dt)
!   M (H^i + delta H^i) + S (T^i + delta T^i) = Q^
!   or:
!   mu M (delta T^i+(T^i-tau(H^i))) + S delta T^i = Q - M H^i - S T^i
!   or:
!   (mu M + S ) delta T^i = Q^ - S T^i + G
!
!   with G = - M H^i - mu M (T^i-tau(H^i))
!
!   Since we suppose that H is computed per integration point, the vector
!   G is computed by deriv separately
!
!   Store boundary conditions into usol
!   Build matrix M and S
!   Build right-hand side f without the effect of boundary conditions
!      i.e. the part fi
!   help := M un, i.e. Mii ui + Mir ur
!   help := help + dt f
!   M_help = M + dt S
!   help1 := M_helpir ur (n+1)
!   help  := help - help1
!   Solve M_help isol = help
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septmenthalpy2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septmenthalpy2'
         write(irefwr,*) 'dt ', dt
         call prinin ( isolprec, 2, 'isolprec' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ihelp = 0
      ihelp1 = 0
      irhsdspecial = 0

      if ( first ) then

!     --- First call of septmenthalpy2

         totiter = 0
         meaniter = 0d0

      end if
      iseqlc = 1
      iinbldtim(13) = iseqlc
      iinbldtim(4) = 2  ! diagonal mass matrix
      iseqtemp = iinenthalpy(3)
      iseqenth = iinenthalpy(2)
      if ( debug ) then
         write(irefwr,*) 'iseqlc, iseqtemp, iseqenth ', &
                          iseqlc, iseqtemp, iseqenth
         write(irefwr,*) 'first, iprint ', first, iprinttim
      end if  ! ( debug )

!     --- build matrices M , S and vector F on new time level
!         in case of enthalpy we compute mu M with rho = 1

      call septimbuildmat ( isolut2d(iseqtemp), irhsd, isolut2d, first, 1 )
      if ( ierror/=0 ) go to 1000
      STIFF => realmt(stiffmat(1))%S_ii

      IM => intmt(realmt(stiffmat(1))%intmatref)
      MASS => realmt(massmat)%S_ii

      MASS = MASS/dt
      if ( .false. .and. debug ) then

!     --- Debug information

         write(irefwr,*) 'mass matrix divided by dt'
         call sepprintmat ( massmat )

      end if  ! ( debug )

!     --- determine new solution at time level t (which is in fact t+dt)
!         help :=  M * H(old)
!         This can not be done by maver since H is defined per element
!         per integration point

      call prcompenthalp4 ( isolprec, ihelp, iinenthalpy, dt )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call sepprintv ( ihelp, 'M * H(old)' )

!     --- ihelp := ihelp + f

      rhsd => ks(irhsd(1))%sol
      help => ks(ihelp)%sol
      if ( Teq(1)%iconsrhs/=2 ) then

!     --- f # 0

         help = help + rhsd

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute effect of boundary conditions and add to right-hand side
!         help1 := M_helpir ur (n+1)

      call sepmaver ( stiffmat, isolut2d(iseqtemp), ihelp1, 6 )
      if ( ierror/=0 ) go to 1000

!     --- help := help - help1

      help1 => ks(ihelp1)%sol
      help = help - help1
      if ( ierror/=0 ) go to 1000
      if ( debug ) call sepprintv ( ihelp, 'M * H(old) + rhsd' )

!     --- solve system to obtain isol

      if ( first ) then

!     --- First call, set some parameters in inpsoltim

         IM => intmt(intmat)
         if ( IM%direct .and. .not. IM%nested .and. .not. IM%mumps ) then

!        --- direct method

            if ( inpsoltim(3)>0 ) inpsoltim(3) = 0

         else

!        --- iterative method
!            use old solution and keep preconditioning if matrix is constant

            if ( inpsoltim(3)==0 ) inpsoltim(3) = 1
            inpsoltim(9)  = 1
            if ( Teq(1)%imasstot/=0 ) inpsoltim(10) = 1

         end if ! ( IM%direct .and. .not. IM%nested .and. .not. IM%mumps )

      else

!     --- next call, reuse preconditioning is possible

         if ( Teq(1)%imasstot/=0 ) inpsoltim(10) = 2

      end if

!     --- Solve (non) linear system

      mu = rinenthalpy(8)  ! value of mu (ad-hoc) stored in rinenthalpy(8)
      eps = max(absaccurtim,relaccurtim)
      if ( debug ) write(irefwr,*) 'mu, eps, maxiter ', mu, eps, maxitertim

      if ( first .or. Teq(1)%imasstot==0 ) then

!     --- Make matrix mu M + S and store in totmat
!         Since mu is already stored in M it is programmed as M + S

         call sepcopymt ( stiffmat, totmat )  !totmat := S
         if ( ierror/=0 ) go to 1000

!        --- Multiply mass matrix by 1/dt

         TOTM => realmt(totmat)%S_ii
         if ( realmt(massmat)%diagonal ) then
            TOTM(IM%diag) = TOTM(IM%diag)+MASS
         else
            TOTM = TOTM+MASS
         end if  ! ( realmt(massmat(1))%diagonal )

         if ( ierror/=0 ) go to 1000
         if ( .false. .and. debug ) then

!        --- Debug information

            write(irefwr,*) 'S + mu M'
            call sepprintmat ( totmat )

         end if  ! ( debug )

      else
         TOTM => realmt(totmat)%S_ii
      end if  ! ( first .or. Teq(1)%imasstot==0 )

      converged = .false.
      iter = 0
      do while ( .not. converged )

!     --- Non-linear iteration on Delta T
!         (mu M + S ) delta T^i = Q^ - S T^i + G
!         with G = - M H^i - mu M (T^i-tau(H^i))
!         Compute S T^i, store in ihelp1

         iter = iter+1
         call sepmaver ( stiffmat, isolut2d(iseqtemp), &
                        ihelp1, 4 )

         if (debug) call sepprintv ( ihelp1, 'S T^i' )
         if ( ierror/=0 ) go to 1000

!        --- Compute vector G and store in irhsdspecial

         call prcompenthalp3 ( numvecmn, iinenthalpy, irhsdspecial, dt )

         if (debug) call sepprintv ( irhsdspecial, 'G' )
         if ( ierror/=0 ) go to 1000

!        --- Create right-hand side Q^ - S T^i + G, store in irhsdspecial


         rhsdsp => ks(irhsdspecial)%sol
         rhsdsp = rhsdsp - help1 + help
         if (debug) call sepprintv ( ihelp,'Q^' )

         if (debug) call sepprintv ( irhsdspecial, 'Q^ - S T^i + G' )

         if ( ierror/=0 ) go to 1000

!        --- Solve system of equations, store in ideltatemp

         ireadinpsol = -1
         iseqsol = 1
         iseqreac = 0
         iseqintmat = 1
         keepmat = 1
         iseqsolut = 1
         iseqmt = totmat
         call sepcopinpsol ( inpsoltim, rinsoltim )
         call seplinsol ( totmat, ideltatemp, irhsdspecial )
         if ( ierror/=0 ) go to 1000
         deltatemp => ks(ideltatemp)%sol

         if (iter==1) then
            resnorm1 = norm2 ( deltatemp )/size(deltatemp)

!           --- resnorm1 = inidnrm2 ( irhsdspecial )

         end if

!        --- Compute norm of right-hand side

!!!!!     --- Tijdelijk vervangen door  ||ideltatemp||

         resnorm = norm2 ( deltatemp )/size(deltatemp)

!        --- resnorm = inidnrm2 ( irhsdspecial )nuso

         if ( debug ) call sepprintv ( ideltatemp, 'ideltatemperature' )

!        --- Compute T^(i+1) and H^(i+1)

         temperature => ks(isolut2d(iseqtemp))%sol
         temperature = deltatemp + temperature

         call prcompenthalp2 ( numvecmn, iinenthalpy, 43 )
         if ( ierror/=0 ) go to 1000

         if ( debug ) then
             call sepprintv ( isolut2d(iseqtemp), 'Temperature' )
             call sepprintv ( isolut2d(iseqenth), 'Enthalpy' )
         end if  ! ( debug )
         if ( iprinttim>=2 ) write(irefwr,100) iter, resnorm
100      format ( 'iter, residual ', i4, d12.3 )

         if ( resnorm<eps*resnorm1 ) then

!        --- Residual<eps, convergence has been reached

            converged = .true.

         else if ( iter>=maxitertim ) then

!        --- No convergence after maxiter iterations

            converged = .true.
            call errint ( iter, 1 )
            call errsub ( 1907, 1, 0, 0 )
            go to 1000

         end if  ! ( resnorm<eps )

      end do  ! while ( .not. converged )

      if (debug) then

         ntimesteps = ntimesteps + 1

         totiter = totiter + iter
         meaniter = totiter/dble(ntimesteps)

         xhist = 0d0
         yhist = 0d0
         zhist = 0d0
         call findxy ( coor, xhist, yhist, zhist, &
                       npoint, nodhist, ndim, iprinttim )

         temperature => ks(isolut2d(iseqtemp))%sol

         if ( iprinttim>1 ) then
            print *, 'nodhist', nodhist
            print *, 't, temphist', t, temperature(nodhist)

            if (t>=tend-0.00001*tstep) then
               print *, 'totiter, meaniter', totiter, meaniter
            end if
         end if ! ( iprinttim>1 )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Reset mass matrix by multiplying by dt

      MASS = MASS*dt

!     --- Delete the arrays ihelp, ihelp1, irhsdspecial, ideltatemp

      call seprmsolinfo ( ihelp1 )
      call seprmsolinfo ( ihelp )
      call seprmsolinfo ( irhsdspecial )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septmenthalpy2'

      end if
      call erclos ( 'septmenthalpy2' )

      end subroutine septmenthalpy2
