      subroutine incm01fict ( istore, icheck )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmodulematstr
!        version  3.0  date 17-12-2015 New parameter list
!        version  2.4  date 18-12-2002 Check on ishape
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill part of irowsii corresponding to fictitious unknowns
!   Help subroutine for incm01
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     fictitious_domain
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istore(*), icheck(*)

!     icheck              work array of length nusol to indicate if a degree of
!                         is present in array istore (1) or not (0)
!     istore              work array of length nusol to store the non-prescribed
!                         neighbour degrees of freedom of a point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inusol, isum, knusol, node, k, l, i1, i2, inunkp, &
              icurnr, inunkpgr, nusolclassic, istoreloc(54), &
              ielemstruct, ikelmcfluid, ikelmcstruct, inpelmfluid, &
              inpelmstruct, ishapelam, icurve1, icurve2, igrp, j, &
              ielemlowstruct, ielemfluid, ielemlowfluid, inpelm, &
              ifluidgrp1, ikelmc, iposkproby, &
              iregionlam, istructgrp, nfictpnt, inunkp_elm, &
              isurnr, ishape_curve

!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     icurnr         Curve sequence number
!     icurve1        First curve number of a set
!     icurve2        Last curve number of a set
!     ielemfluid     Element sequence number of fluid element
!     ielemlowfluid  Lowest element sequence number of fluid elements
!     ielemlowstruct Lowest element sequence number of structural elements
!     ielemstruct    Element sequence number of structural element
!     ifluidgrp1     First fluid element group number
!     igrp           Boundary element group number read
!     ikelmc         position in kmesh part c
!     ikelmcfluid    Position in kmeshc part c with respect to fluid elements
!     ikelmcstruct   Position in kmeshc part c with respect to structural
!                    element
!     inpelm         number of points per element
!     inpelmfluid    number of points per fluid element
!     inpelmstruct   number of points per structural element
!     inunkp         Number of degrees of freedom in point
!     inunkp_elm     Number of unknowns per point
!     inunkpgr       Number of global unknowns in a group
!     inusol         Position of unknown in array USOL
!     iposkproby     Position in kprob y
!     iregionlam     type of region where the lambda's are defined:
!                     1:  curves
!                     2:  surfaces
!     ishape_curve   Shape of lambda basis functions
!     ishapelam      Shape of lambda basis functions
!     istoreloc      Local array to store quantities
!                    In this case the nodes of the structural and fluid element
!                    are stored
!     istructgrp     Structural element group
!     isum           Integer variable to compute a sum
!     isurnr         Surface sequence number
!     j              Counting variable
!     k              Counting variable
!     knusol         Position of unknown in array USOL
!     l              General loop variable
!     nfictpnt       Total number of points with fictitious unknowns
!     node           Node number
!     nusolclassic   Number of free classical unknowns, i.e fictitious unknowns
!                    and global unknowns excluded
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Initialise parameters
!      For igrp := 1 (1) nfictgrp
!         Get some relevant parameters
!         Compute starting address of structural elements in kmeshc
!         Compute starting address of fluid elements in kmeshc
!         if ( region is of type curves ) then
!            For all curves in region do
!               Find structural and fluid element containing the lambdas
!               For all elements along curve
!                  For all lambda points in element
!                     For all nodes in the structural element
!                        Store the nodes in istoreloc
!                     For all nodes in the fluid element
!                        Store the nodes in istoreloc
!                     For all nodes in istoreloc
!                        For all unknowns in node
!                           if (unknown not prescribed ) then
!                              raise irowsii
!         else ( region is of type surfaces )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm01fict' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm01fict'
         call prinin ( irowsii, nusol+1, 'irowsii' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- kprob(51)>0, there are ficitious elements
!         For each element group there are extra unknowns
!         These extra unknowns only adapt the corresponding rows and columns
!         Initialise parameters

      nfictpnt = 0
      nusolclassic = nusol-nglunk-nfictunk

      if ( debug ) &
         write(irefwr,1) 'nfictgrp, nusolclassic, nrusol', &
                          nfictgrp, nusolclassic, nrusol

!     --- Loop over fictitious groups

      do igrp = 1, nfictgrp

!     --- Redefine inpelm for multiple fictitious groups

         if ( igrp>1 ) inpelm = npelm

!        --- Clear array istore

         do i = 1, nusol
            icheck(i) = 0
         end do

!        --- Get some relevant parameters

         iposkproby = kproby(10+igrp)-1
         inunkpgr   = kproby(iposkproby+2)
         iregionlam = kproby(iposkproby+3)
         ishapelam  = kproby(iposkproby+4)
         icurve1    = kproby(iposkproby+5)
         icurve2    = kproby(iposkproby+6)
         istructgrp = kproby(iposkproby+7)
         ifluidgrp1 = kproby(iposkproby+8)
         inunkp_elm = kproby(iposkproby+10)/inunkpgr

!        --- Compute starting address of structural elements in kmeshc

         ikelmcstruct = 0
         ielemlowstruct = 1
         do i = 1, istructgrp-1
            inpelm = sepgetinpelm(i)
            inelem = sepgetinelem(i)
            ikelmcstruct = ikelmcstruct+inelem*inpelm
            ielemlowstruct = ielemlowstruct+inelem
         end do
         inpelmstruct = sepgetinpelm(istructgrp)

!        --- Compute starting address of fluid elements in kmeshc

         ikelmcfluid = 0
         ielemlowfluid = 1
         do i = 1, ifluidgrp1-1
            inpelm = sepgetinpelm(i)
            inelem = sepgetinelem(i)
            ikelmcfluid = ikelmcfluid+inelem*inpelm
            ielemlowfluid = ielemlowfluid+inelem
         end do
         inpelmfluid = sepgetinpelm(ifluidgrp1)

         if ( iregionlam==1 ) then

!        --- region is of type curve
!            ishape := shape number of structural element

            ishape = sepgetshape(istructgrp)
            if ( ishape==3 .or. ishape==5 ) then

!           --- ishape = 3 or 5

               ishape_curve = 1

            else if ( ishape==4 .or. ishape==6 ) then

!           --- ishape = 4 or 6

               ishape_curve = 2

            else

!           --- Shape number not implemented

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               ishape_curve = 1

            end if

!           --- loop over curves

            do icurnr = icurve1, icurve2

               i1 = kmeshm(4+icurnr)
               i2 = kmeshm(5+icurnr)-1

!              --- Find structural and fluid element containing the lambdas
!                  Loop over elements along curve

               do i = i1, i2-1, ishape_curve

!              --- Loop over lambda points in element

                  do j = 1, ishapelam
                     isum = 0
                     nfictpnt = nfictpnt+1
                     ielemstruct = kprobfict(1,nfictpnt)

!                    --- Loop over the nodes in the structural element
!                        Store the nodes in istoreloc

                     ikelmc = ikelmcstruct+ &
                              (ielemstruct-ielemlowstruct)*inpelmstruct
                     do k = 1, inpelmstruct
                        istoreloc(k) = kmeshc(ikelmc+k)
                     end do

!                    --- Loop over the nodes in the fluid element
!                        Store the nodes in istoreloc

                     ielemfluid = kprobfict(2,nfictpnt)
                     ikelmc = ikelmcfluid+ &
                              (ielemfluid-ielemlowfluid)*inpelmfluid
                     do k = 1, inpelmfluid
                        istoreloc(k+inpelmstruct) = kmeshc(ikelmc+k)
                     end do

!                    --- For all nodes in istoreloc

                     do k = 1, inpelmstruct+inpelmfluid
                        node = istoreloc(k)
                        inusol = sepgetinusol ( node, inunkp )
                        if ( indprr==0 ) then

!                       --- indprr = 0

                           do l = 1, inunkp

                               if ( icheck(inusol+l)==0 ) then

!                              --- New unknown found

                                  icheck(inusol+l) = 1
                                  isum = isum+1
                                  istore(isum) = inusol+l

                               end if

                            end do

                        else

!                       --- indprr>0

                           do l = 1, inunkp
                              knusol = kprobr(inusol+l)
                              if ( icheck(knusol)==0 .and. &
                                   knusol<=nrusol ) then

!                             --- New unknown found

                                 icheck(knusol) = 1
                                 isum = isum+1
                                 istore(isum) = knusol

                              end if

                           end do

                        end if

                     end do

!                    --- Fill array irowsii for all fictitious unkowns
!                        corresponding to ifictgrp

                     do l = 1, inunkpgr
                        knusol = l+nusolclassic
                        if ( indprr>0 ) knusol = kprobr(knusol)
                        irowsii(knusol+1) = isum+l-1
                     end do
                     nusolclassic = nusolclassic+inunkpgr
                     do l = 1, isum
                        icheck(istore(l)) = 0
                     end do

                  end do

               end do

            end do

         else

!        --- region is of type surface
!            loop over surfaces

            do isurnr = icurve1, icurve2

               inelem = kmeshn(nsurfs+isurnr)
               inpelm = kmeshn(isurnr)

               if ( debug ) &
                  write(irefwr,1) 'isurnr, inelem, inpelm', &
                                   isurnr, inelem, inpelm

!              --- Find structural and fluid element containing the lambdas
!                  Loop over elements along surface

               do i = 1, inelem

!              --- Loop over lambda points in element

                  do j = 1, inunkp_elm
                     isum = 0
                     nfictpnt = nfictpnt+1
                     ielemstruct = kprobfict(1,nfictpnt)

!                    --- Loop over the nodes in the structural element
!                        Store the nodes in istoreloc

                     ikelmc = ikelmcstruct+ &
                              (ielemstruct-ielemlowstruct)*inpelmstruct
                     do k = 1, inpelmstruct
                        istoreloc(k) = kmeshc(ikelmc+k)
                     end do

!                    --- Loop over the nodes in the fluid element
!                        Store the nodes in istoreloc

                     ielemfluid = kprobfict(2,nfictpnt)
                     ikelmc = ikelmcfluid+ &
                              (ielemfluid-ielemlowfluid)*inpelmfluid
                     do k = 1, inpelmfluid
                        istoreloc(k+inpelmstruct) = kmeshc(ikelmc+k)
                     end do

                     if ( debug ) then
                        write(irefwr,1) 'i, nfictpnt, ielemstruct', &
                                         i, nfictpnt, ielemstruct
                        write(irefwr,1) 'ielemfluid', ielemfluid
                        call prinin ( istoreloc, inpelmstruct+inpelmfluid, &
                                      'istoreloc' )
                     end if  ! ( debug )

!                    --- For all nodes in istoreloc

                     do k = 1, inpelmstruct+inpelmfluid
                        node = istoreloc(k)
                        inusol = sepgetinusol ( node, inunkp )

                        if ( debug ) &
                           write(irefwr,1) 'k, node, inunkp, inusol, indprr', &
                                            k, node, inunkp, inusol, indprr

                        if ( indprr==0 ) then

!                       --- indprr = 0

                           do l = 1, inunkp

                              if ( icheck(inusol+l)==0 ) then

!                             --- New unknown found

                                 icheck(inusol+l) = 1
                                 isum = isum+1
                                 istore(isum) = inusol+l

                              end if

                           end do

                        else

!                       --- indprr>0

                           do l = 1, inunkp
                              knusol = kprobr(inusol+l)

                              if ( debug ) &
                                 write(irefwr,1) 'l, knusol, icheck(knusol)', &
                                                  l, knusol, icheck(knusol)

                              if ( icheck(knusol)==0 .and. &
                                   knusol<=nrusol ) then

!                             --- New unknown found

                                 icheck(knusol) = 1
                                 isum = isum+1
                                 istore(isum) = knusol

                              end if

                           end do

                        end if

                     end do

                     if ( debug ) call prinin ( istore, isum, 'istore' )

!                    --- Fill array irowsii for all fictitious unkowns
!                        corresponding to ifictgrp

                     if ( debug ) write(irefwr,1) 'inunkpgr', inunkpgr

                     do l = 1, inunkpgr
                        knusol = l+nusolclassic
                        if ( indprr>0 ) knusol = kprobr(knusol)
                        irowsii(knusol+1) = isum+l-1
                     end do
                     nusolclassic = nusolclassic+inunkpgr
                     do l = 1, isum
                        icheck(istore(l)) = 0
                     end do

                  end do

               end do

            end do

         end if

!        --- Raise parameters

         iposkproby = iposkproby+12

      end do
1000  call erclos ( 'incm01fict' )
      if ( debug ) then

!     --- Debug information

         call prinin ( irowsii, nusol+1, 'irowsii' )
         write(irefwr,*) 'End incm01fict'

      end if  ! ( debug )

      end
