      subroutine incm02 ( iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version 14.0  date 06-01-2016 Use sepmodulematstr
!        version 13.0  date 17-12-2015 New parameter list
!        version 12.4  date 30-03-2014 Extension with jmethod = 52/53
!
!   copyright (c) 1989-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute second part of contact matrix, i.e. fill array imat part 2
!     alias icolsii.
!     For each internal row the neighbors are stored.
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(3,*)

!     iwork          i    work array of length 3*npoint to indicate if a point
!                         is periodical or not
!                         iwork has been filled by incm01
!                         If iwork(1,i) = 0 then point is not periodical
!                         For a chain of periodical points, iwork is
!                         filled as follows:
!                         iwork(1,first_node_in_chain) = &
!                              sequence number in iwork(2,.)
!                         iwork(1,next_nodes in chain) = -first_node_in_chain
!                         iwork(2,.) contains for each
!                         chain of periodical points the number of points that
!                         are connected to the first point.
!                         These numbers are accumulated
!                         iwork(2,1) = 1
!                         iwork(2,i+1)-iwork(2,i) = number of points connected
!                         to nodal point i
!                         iwork(3,.) contains the nodal points
!                         connected to the first point in a chain in the
!                         sequence given by iwork(2,.)
!                         Hence iwork(3,.) positions iwork(2,i) to
!                         iwork(2,i+1)-1
!                         contain all nodes from the chain starting with node i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inext, nodp1, iseqlc, inunkp, &
              inusol, nfdeg, node, nrep, numnei, &
              ihelpunk(20), icoupled, ncoupled, inunkpmin, &
              ilowhelp, imark(nunkp), &
              knunkp, nfdegmid, numneimid, kfree(2,nunkp)
      integer, allocatable :: icheck(:), istore(:,:), ihelp(:), jwork(:)
      logical diffunk, extracentr, extraneigh
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     diffunk        If true there are coupled nodes with boundary conditions
!                    that are not periodical
!     extracentr     If true extra fillin for the centres of discontinuous
!                    Navier-Stokes elements is required
!     extraneigh     If true extra fillin for one row of extra neighbors is
!                    required
!     icheck         work array of length nusol to indicate if a degree of
!                    freedom is present in array istore (1) or not (0)
!     icoupled       Number of coupled nodes that have been treated for this
!                    specific node
!     ihelp          work array of length npoint
!                    It is used to indicate if a point belongs to
!                    the set of periodical points corresponding to a
!                    specific periodic point in the following way:
!                    ihelp(i) = 1, point i corresponds to the periodic
!                    point
!                    ihelp(i) = 0, it does not
!     ihelpunk       Work array indicating if unknowns in coupled nodes are
!                    periodical (0) or not (1)
!     ilowhelp       Lower boundary of array ihelp
!     imark          Work array of length nunkp to indicate which degrees
!                    of freedom are coupled or decoupled per node
!                    imark(i) = 0, degree of freedom is coupled to all others
!                    imark(i) = 1, degree of freedom is coupled to all
!                                  degrees of freedom with imark(i) = 0
!     inext          Next point in curve
!     inunkp         Number of degrees of freedom in point
!     inunkpmin      Lowest degree of freedom that must be taken into account
!     inusol         Position of unknown in array USOL
!     iseqlc
!     istore         work array of length 3*nusol
!                    The first nusol positions are used to store the
!                    non-prescribed neighbor degrees of freedom of a point
!                    In the second nusol positions the position of the last
!                    column number stored for each row is put
!                    Hence the next free position for a column in row i
!                    is defined by istore(nusol+i)
!     jwork          work array of length nrusol to be used for sorting
!     kfree          work array of length nunkp to store the non-prescribed
!                    degrees of freedom in a node
!     knunkp         number of degrees of freedom
!     ncoupled       Number of coupled nodes that are connected to this
!                    specific node including the node
!     nfdeg          Number of free degrees of freedom in a point
!     nfdegmid       Number of free degrees of freedom corresponding to the
!                    first ndim+1 unknowns
!     node           Node number
!     nodp1          Node sequence number
!     nrep           ?
!     numnei         Number of neighbors
!     numneimid      Number of standard neighbors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INCM01CLEAR    Clear array icheck
!     INCM01HELP     if point is periodical mark all points in the chain in
!                    array ihelp
!                    Fill array ihelpunk with information about periodical nodes
!                    Compute diffunk and ncoupled
!     INCM01NEIGH    consider the neighbors of node
!                    put real degrees of freedom in istore
!                    Mark those one used in icheck
!                    Repeat for periodical node
!     INCM01NODE     put real degrees of freedom of nodal point node into kfree
!                    Mark unknowns in icheck
!     INCM01PERIOD   Adapt icheck, kfree and istore for periodical nodes
!     INCM021        Fill part of imat2 corresponding to global unknowns
!     INCM02FICT     Fill part of imat2 corresponding to fictitious unknowns
!     INCM02ICOL     Fill array icolsii with column numbers of non-zero entries
!                    per row of the matrix
!     INCM02ICOL2    Fill array icolsii with column numbers of non-zero entries
!                    per row of the matrix
!                    Special case of decoupled unknowns
!     INFIL2         Create extra fill-in
!     PR0004         Print INTMAT corresponding to internal matrix
!     SEPSORTCOLS    Sort column numbers per row of internal matri
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array ICHECK
!     Fill essential boundary conditions in ICHECK
!     For node = 1 (1) NUMBER OF NODES do
!        If ( iwork(1,node)>=0 ) then
!           Store free unknows in array kfree
!           If ( there are free unknowns in node ) then
!              If ( node is not periodical ) then
!                 Store all free unknowns coupled to point in ISTORE
!              else (node is periodical) then
!                 Store all free unknowns coupled to point and to corresponding
!                 periodical nodal points in ISTORE
!              Store contents of kfree and ISTORE into icolsii
!     Sort array icolsii
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm02' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm02'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

!     --- Set constants

      inunkpmin = 1
      ilowhelp = 1
      if ( debug ) then
         write(irefwr,1) 'nusol, nrusol, npoint', &
                          nusol, nrusol, npoint
      end if  ! ( debug )

      allocate ( icheck(nusol), istore(nusol,3), ihelp(npoint), &
                 jwork(nrusol), stat=error )
      if ( error/=0 ) call eralloc ( error, 5*nusol+npoint, 'icheck' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array imark
!         degrees of freedom that are decoupled get value 1, all others 0

      imark = 0
      imark(decoupled(1:numdec)) = 1

      if ( extrafillin<=3 ) then

!     --- Standard situation with respect to fill-in

         extracentr = mod(extrafillin,2)==1
         extraneigh = mod(extrafillin/2,2)==1

      else

         extracentr = .false.
         extraneigh = .true.
         if ( extrafillin==7 ) inunkpmin = ndim+2

      end if

!     --- clear icheck

      icheck(1:nrusol) = 0
      istore(1:nrusol,3) = irowsii(1:nrusol)
      icheck(nrusol+1:nusol) = 1

!     --- Clear array ihelp

      ihelp(1:npoint) = 0

!     --- Loop over all nodes

      diffunk = .false.
      do node = 1, npoint

         nfdeg = 0
         numnei = 0
         if ( iwork(1,node)>=0 ) then

!        --- Point is not a periodical node, or is the basis point of a chain
!            if point is periodical mark all points in the chain in array ihelp
!            Fill array ihelpunk with information about periodical nodes
!            Compute diffunk, iseqlc, inusol, icoupled and ncoupled

            call incm01help ( iwork, node, ihelp, icoupled, ncoupled, &
                              knunkp, ihelpunk, diffunk, iseqlc, &
                              inusol, ilowhelp, inunkp )

!           --- put real degrees of freedom of nodal point node into kfree
!               Mark unknowns in icheck

            call incm01node ( kfree, icheck, nfdeg, nfdegmid, &
                              inusol, istore, numnei, ihelpunk, inunkp )

         end if
         if ( ierror/=0 ) go to 1000

         if ( debug ) write(irefwr,1) 'node, nfdeg', node, nfdeg

!        --- Find neighbor unknowns of free degrees of freedom

         if ( nfdeg==0 ) go to 500

!        --- nfdeg>0, so there are nfdeg free unknowns in node

         if ( iwork(1,node)==0 ) then

!        --- node is not periodical

            nrep  = 1

         else

            nrep = iwork(2,iseqlc+1) - iwork(2,iseqlc) + 1
            inext = iwork(2,iseqlc)

         end if

         nodp1  = node

!        --- consider the neighbors of node
!            put real degrees of freedom in istore
!            Mark those ones used in icheck
!            Repeat for periodical node

250      call incm01neigh ( kfree, ihelp, nodp1, &
                            icheck, numnei, istore, nrep, diffunk, &
                            inext, iwork, extraneigh, &
                            knunkp, numneimid, inunkpmin, &
                            ihelpunk, ilowhelp )
         if ( ierror/=0 ) go to 1000

!        --- Fill array icolsii

         if ( numdec==0 ) then

!        --- Standard case: all degrees of freedom per node are coupled

            call incm02icol ( nfdegmid, kfree, irowsii, nfdeg, &
                              numnei, istore, numneimid, jwork )

         else

!        --- Special case: not all degrees of freedom per node are coupled

            call incm02icol2 ( nfdegmid, kfree, nfdeg, numnei, &
                               istore, numneimid, imark )

         end if  ! ( numdec==0 )
         if ( ierror/=0 ) go to 1000

!        --- Clear icheck

         call incm01clear ( numnei, icheck, istore, kfree, nfdeg, &
                            diffunk, node, nodp1, iwork, iseqlc )
         if ( ierror/=0 ) go to 1000

!        --- Reset ihelp

500      if ( iwork(1,node)>0 ) then

!        --- Basis point of chain

            ihelp(iwork(3,iwork(2,iseqlc):iwork(2,iseqlc+1)-1)) = 0
            ihelp(node) = 0

         end if

550      if (  diffunk .and. icoupled<ncoupled ) then

!        --- The node is periodical with degrees of freedom that are not
!            coupled
!            Loop over all coupled nodes for the non-coupled degrees of freedom
!            First mark all unknowns corresponding to the other points
!            in the chain

            call incm01period ( node, nodp1, icheck, iwork, icoupled, &
                                nfdeg, numnei, kfree, iseqlc, &
                                istore, nrep, ihelpunk )

            if ( nfdeg==0 ) then

!           --- No free degrees of freedom, skip

               go to 550

            else

!           --- Free degrees of freedom, repeat

               go to 250

            end if

         end if  ! (  diffunk .and. icoupled<ncoupled )

!        --- Clear icheck

         icheck( istore(1:numnei,1) ) = 0
         icheck( kfree(1,1:nfdeg) ) = 0

      end do  ! node = 1, npoint

      if ( indpry>0 ) then

!     --- if indpry>0, there are fictitious elements
!         treat these separately in incm01fict

          call incm02fict ( istore )

      end if

!     --- indprx>0, there are global elements
!         For each element group there are extra unknowns
!         These extra unknowns only adapt the corresponding rows

      if ( indprx>0 ) call incm021 ( istore )

      if ( extracentr ) then

!     --- extracentr = true, fill extra fill in for neighbors of pressure

         call infil2 ( ihelp, ihelp(nelem+1) )

      end if

      call sepsortcols  ! sort column numbers in increasing sequence

      deallocate ( icheck, istore, ihelp, jwork, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'icheck' )

1000  if ( debug ) then

!     --- Debug information

         call pr0004 ( jmethod, nrusol, irowsii, icolsii )
         write(irefwr,*) 'End incm02'

      end if  ! ( debug )
      call erclos ( 'incm02' )

      end
