      subroutine prob04cvnew ( iinput, iinp, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 17-03-2015 Adaptation npoint for parallel
!        version  1.0  date 12-04-2012
!
!   copyright (c) 2012-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for a set of curves
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nkbnd

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iexclu, jstep, inum,  numcrv, npointscurve, &
              nparts, npointlc
      integer, allocatable :: nodes(:), ipoint(:)
      logical found

!     found          If true a warning has been found in a curve
!     i              Counting variable
!     iexclu         Indicates which points are excluded
!     inum           Number of curves stored
!     ipoint         Array of length nparts+1 containing the starting
!                    addresses in array inodes of the various parts
!                    the boundary consists of.
!                    ipoint(1) = 1, &
!                    number of nodes at part i = ipoint(i+1)-ipoint(i)
!     jstep          Step in loop
!     nodes          integer work array of length npoint with node numbers
!     nparts         number of parts the boundary consists of
!     npointlc       Number of nodal points
!     npointscurve   number of points along a curve
!     numcrv         number of parts where curves are given for generation
!                    of boundary conditions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PROB04CVOUTER  perform the outer boundary part of prob04cvold/new
!     PROB04CVSTEP   perform the jstep part of prob04cvold/new
!     SEPCURVENODES  Store all nodes in one curves in array inodes and gaps in
!                    these curves in array ipoint
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     558   The parameter l for the definition of essential boundary conditions
!           or connected elements has an incorrect value
!    1030   Not all nodes in curve correspond to nodal points
!    1626   Curve boundary conditions given whereas there are no surfaces
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04cvnew' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04cvnew'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numcrv = iinput(7)

      if ( numcrv==0 ) go to 1000  ! jump if no curves are used

      npointlc = max(npoint,kmeshm(3)-kmeshm(2))

      if ( debug ) write(irefwr,1) 'npointlc, ncurvs', npointlc, ncurvs

      allocate ( nodes(npointlc), ipoint(ncurvs+1), stat = error )
      if ( error/=0 ) call eralloc ( error, npointlc, 'nodes' )
      if ( ierror/=0 ) go to 1000

      if ( numcrv>0 .and. kelmm==0 ) then

!     --- numcrv>0, whereas kmeshm has not been filled

         call errsub ( 1626, 0, 0, 0 )
         numcrv = 0

      end if
      if ( debug ) write(irefwr,1) 'ncurvs, numcrv', ncurvs, numcrv

      do i = 1, numcrv

         found = .false.
         jstep = iinput(iinp+2)
         inum = iinput(iinp+3)
         if ( debug ) write(irefwr,1) 'inum, jstep', inum, jstep
         if ( inum>0 ) then

!        --- Standard case, set of inum curves

            if ( jstep>0 ) then

!           --- jstep>0,  compute iexclu

               iexclu = jstep/100
               if ( iexclu>3 ) then
                  call errint ( iexclu, 1 )
                  call errint ( jstep, 2 )
                  call errsub ( 558, 2, 0, 0 )
               end if
               jstep = jstep-100*iexclu
            else
               iexclu = 0
            end if

            if ( debug ) then
               call prinin ( iinput(iinp+3), inum+1, 'icurvs' )
               write(irefwr,1) 'jstep, iexclu', jstep, iexclu
            end if  ! ( debug )

            nparts = 0
            call sepcurvenodes ( iinput(iinp+3), ipoint, nodes, nparts, kmeshm )

            if ( debug ) then
               write(irefwr,1) 'nparts', nparts
               call prinin ( ipoint, nparts+1, 'ipoint' )
            end if  ! ( debug )

            npointscurve = ipoint(nparts+1)-1
            if ( ierror/=0 ) go to 1000

            call prob04cvstep ( jstep, iexclu, npointscurve, found, &
                                nodes, nkbnd )
            if ( debug ) write(irefwr,*) 'after prob04cvstep'

            iinp = iinp+inum+3

         else  ! ( inum = -1 )

!        --- Special case, all outer boundaries

            call prob04cvouter ( nkbnd, nodes, found )
            iinp = iinp+3

         end if  !  ( inum>0 )

         if ( .not. parallel .and. found ) then

!        --- Warning some nodes have not been filled

            call errint ( i, 1 )
            call errwar ( 1030, 1, 0, 0 )

         end if

      end do  ! i = 1, numcrv

      if ( debug ) write(irefwr,*) 'before deallocate'
      deallocate ( nodes, ipoint, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04cvnew'

      end if  ! ( debug )
      call erclos ( 'prob04cvnew' )

      end
