      subroutine prob04cvold ( iinput, iinp, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for curves icrv1 to icrv2
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nkbnd

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iexclu, ifirst, ilast, jstep, &
              kstep, numcrv, npointscurve
      integer, allocatable :: nodes(:)
      logical found

!     found          If true a warning has been found in a curve
!     i              Counting variable
!     iexclu         Indicates which points are excluded
!     ifirst         First curve (surface) number read
!     ilast          Last curve (surface) number read
!     jstep          Step in loop
!     kstep          Step in loop
!     nodes          integer work array of length npoint with node numbers
!     npointscurve   number of points along a curve
!     numcrv         number of parts where curves are given for generation
!                    of boundary conditions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PROB04CVOUTER  perform the outer boundary part of prob04cvold/new
!     PROB04CVSTEP   perform the jstep part of prob04cvold/new
!     SEPER0026      fill nodal points of curves icrv1 to icrv2 in array iwork
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     558   The parameter l for the definition of essential boundary conditions
!           or connected elements has an incorrect value
!    1030   Not all nodes in curve correspond to nodal points
!    1626   Curve boundary conditions given whereas there are no surfaces
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04cvold' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04cvold'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numcrv = iinput(7)

      allocate ( nodes(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

      if ( numcrv>0 .and. kelmm==0 ) then

!     --- numcrv>0, whereas kmeshm has not been filled

         call errsub ( 1626, 0, 0, 0 )
         numcrv = 0

      end if
      if ( debug ) write(irefwr,1) 'ncurvs, numcrv', ncurvs, numcrv
      do i = 1, numcrv

         found = .false.
         jstep = iinput(iinp+2)
         ifirst = iinput(iinp+3)
         ilast = iinput(iinp+4)
         if ( ilast==0 ) ilast = ifirst
         iinp = iinp+4
         if ( debug ) write(irefwr,1) 'ifirst, ilast', ifirst, ilast
         if ( ifirst>0 ) then

!        --- Standard case, curves from ifirst to ilast

            if ( jstep>0 ) then

!           --- jstep>0,  compute iexclu

               iexclu = jstep/100
               if ( iexclu>3 ) then
                  call errint ( iexclu, 1 )
                  call errint ( jstep, 2 )
                  call errsub ( 558, 2, 0, 0 )
               end if
               jstep = jstep-100*iexclu
            else
               iexclu = 0
            end if
            kstep = abs(jstep)+1
            call seper0026 ( ifirst, ilast, kstep, npointscurve, &
                             nodes, .false., kmeshm )
            if ( ierror/=0 ) go to 1000

            call prob04cvstep ( jstep, iexclu, npointscurve, &
                                found, nodes, nkbnd )

         else  ! ( ifirst = -1 )

!        --- Special case, all outer boundaries

            call prob04cvouter ( nkbnd, nodes, found )

         end if  ! ( ifirst>0 )

         if ( .not. parallel .and. found ) then

!        --- Warning some nodes have not been filled

            call errint ( i, 1 )
            call errwar ( 1030, 1, 0, 0 )

         end if

      end do  ! i = 1, numcrv

      deallocate ( nodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04cvold'

      end if  ! ( debug )
      call erclos ( 'prob04cvold' )

      end
