      subroutine prob05sfnew ( iinp, inunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential
!     boundary conditions for a set of surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, inunkp

!     iinp          i/o   last position used in array iinputkp
!     inunkp        i/o   Number of degrees of freedom in point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, iskipbn, i, istart, jinp, &
              ncurvsact, nsurfaces, istsrf, isurnr, ibit
      integer, allocatable :: iwork(:), icurvind(:)

!     i              Counting variable
!     ibit           Actual bit pattern
!     icurvind       Integer work array of length ncurvs that is filled as
!                    follows
!                    icurvind(i) = -1, at the boundary of the surfaces
!                    icurvind(i) = 1, or not in the surfaces icurvind(i)=0
!     inpelm         number of points per element
!     iskipbn        Defines if the outer boundary must be skipped
!                    Possible values:
!                    0:  the outer boundary must not be skipped
!                    1:  the complete outer boundary must be skipped
!                    2:  the inner boundary must be skipped as well as some
!                        explicitly given outer curves
!                    3:  the outer boundary must be skipped except some
!                        explicitly given outer curves
!     istart         First position used at a part
!     istsrf         Starting address of surfaces in iinputkp
!     isurnr         Surface sequence number
!     iwork          integer work array of length npoint
!     jinp           last position used in array iinputkp
!     ncurvsact      Actual number of curves to be used
!     nsurfaces      Number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PROB05SFELS    Fill KPROB part g with information concerning the essential
!                    boundary conditions for set of elements in surfaces
!     PROB09S1       Mark all points on external curves with a -1 in iwork
!     SEPEXTKMESHT   Compute starting and ending address of elements in kmeshc
!     SEPIINFSURF    Find starting address of surface isurnr in array kmeshn
!                    Also find number of element and number of points per
!                    element in isurnr
!     SEPOUTCR       Find all curves surrounding a set of surfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05sfnew' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05sfnew'
         write(irefwr,1) 'iinp', iinp
         call prinin ( iinputkp(iinp+1), 5+iinputkp(iinp+2)+iinputkp(iinp+1), &
                       'iinputkp' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), icurvind(ncurvs), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, npoint+ncurvs, 'isurfaces' )
      if ( ierror/=0 ) go to 1000

      ibit = iinputkp(iinp+1)
      inpelm = iinputkp(iinp+2)
      nsurfaces = iinputkp(iinp+3)
      istsrf = iinp+3
      iinp = iinp+3+nsurfaces

      if ( debug ) &
         write(irefwr,1) 'inpelm, nsurfaces, istsrf, iinp', &
                          inpelm, nsurfaces, istsrf, iinp

!     --- clear array iwork

      iwork = 0

!     --- compute iskipbn

      jinp = iinp+inpelm+2
      iskipbn = iinputkp(jinp-1)
      ncurvsact = iinputkp(jinp)

      if ( iskipbn>=1 ) then

!     --- Boundary must be skipped, &
!         first find curves which correspond to boundary of surfaces

         call sepoutcr ( icurvind, iinputkp(istsrf), nsurfaces )

!        --- Next mark all points on the outer curves by -1

         call prob09s1 ( icurvind, iwork, iskipbn, &
                         ncurvsact, iinputkp(jinp+1) )

      end if

!     --- Loop over surfaces

      do i = 1, nsurfaces

!     --- compute the number of elements  inelem

         isurnr = iinputkp(istsrf+i)
         if ( debug ) write(irefwr,1) 'i, isurnr', i, isurnr

         if ( kelmn>0 ) then

!        --- kelmn>0, use kmesh part n

            call sepiinfsurf ( istart, inpelm, isurnr )

         else

!        --- kelmn = 0, use kmesh part t and kmeshc
!            compute inpelm and inelem

            call sepextkmesht ( isurnr, isurnr, istart, inpelm )
            istart = istart-1

         end if  ! ( kelmn>0 )

!        --- Fill kprobg for all nodes in surfaces

         call prob05sfels ( inpelm, isurnr, isurnr, &
                            istart, iinp, iwork, inunkp, ibit )

      end do  ! i = 1, nsurfaces

      iinp = jinp+ncurvsact

      deallocate ( iwork, icurvind, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05sfnew'

      end if  ! ( debug )
      call erclos ( 'prob05sfnew' )

      end
