      subroutine probnatblevel3 ( natlevset, iinp, &
                                  iposgrp, ikprbi, iobstfaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kprob part b and iposgrp with information of elements for
!     natural boundary conditions defined on zero levelset points
!     3 dimensions only
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     natural
!     problem_definition
!     level_set
!     natural_boundary_condition
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer natlevset, iinp, &
              iposgrp(*), ikprbi, iobstfaces(*)

!     iinp          i/o   last position used in array iinputkp
!     ikprbi        i/o   last position used in array kprob part i
!     iobstfaces     i    contains face sequence numbers of boundary
!                         of levelset
!     iposgrp       i/o   work array of length numnat in which the starting
!                         addresses of the element groups in KPROB part i are
!                         stored
!     natlevset      i    number of parts where zero level set elements
!                         are defined
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer igrp, ibngrp, inum, i, nnode_face, numbounelms, &
              inpelm, numfaces, iface, jstart, ilev, kstart

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     iface          Edge sequence number
!     igrp           Boundary element group number read
!     ilev           Level set sequence number
!     inpelm         number of points per element
!     inum           Number of numbers read in a record
!     jstart         First position used at a part
!     kstart         First position used at a part
!     nnode_face     number of nodes corresponding to face
!     numbounelms    Number of elements that have all points on the
!                    boundary of the obstacle or a set of surfaces
!     numfaces       Number of faces in boundary
!     numnatlc         Number of different boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     270   number of nodal points in elements of the same group for the
!           creation of boundary elements, not the same
!    1093   NPELM incorrect for boundary group
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probnatblevel3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probnatblevel3'
         write(irefwr,1) 'iinp', iinp
         call prinin ( iinputkp(iinp+1), 6, 'iinputkp' )
         call prinin ( iobstfaces, 10, 'iobstfaces' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      inpelm = 0

!     --- Loop over set of natural boundary conditions corresponding to
!         zero level set

      do igrp = 1, natlevset

         ibngrp = iinputkp(iinp+1)         ! boundary group
         ilev = iinputkp(iinp+2)           ! levelset sequence number
         inum = max(1,iinputkp(iinp+5) )   ! number of surfaces

         if ( debug ) then
            call prinin ( iinputkp(iinp+1), 5+inum, 'iinputkp' )
            write(irefwr,1) 'igrp, ibngrp, inum, iinp', &
                             igrp, ibngrp, inum, iinp
         end if  ! ( debug )

         iinp = iinp+5+inum

!        --- Find all faces with all points on boundary

         kstart = iobstfaces(ilev)
         numfaces = iobstfaces(kstart)
         numbounelms = 0

         if ( debug ) &
            write(irefwr,1) 'ilev, kstart, numfaces', &
                             ilev, kstart, numfaces

         do i = 1, numfaces

            iface = iobstfaces(kstart+i)       ! face number
            jstart = nmeshg(iface)
            nnode_face = nmeshg(jstart)

            numbounelms = numbounelms+1

            if  ( debug ) &
               write(irefwr,1) 'i, jstart, iface, nnode_face, inpelm', &
                                i, jstart, iface, nnode_face, inpelm

            if ( inpelm==0 ) then

!           --- first element found

               inpelm = nnode_face

            else if ( inpelm/=nnode_face ) then

!           --- error: non constant number of nodes in elements

               call errint ( ibngrp, 1 )
               call errint ( inpelm, 2 )
               call errint ( nnode_face, 3 )
               call errsub ( 270, 3, 0, 0 )
               go to 1000

            end if  ! ( inpelm==0 )

            if ( debug ) then
               write(irefwr,1) 'iface', i
               call prinin ( nmeshg(jstart+1), nnode_face, 'nodes' )
            end if  ! ( debug )

         end do  ! i = 1, numfaces

!        --- Raise kprob part b and fill kprob part j

         kprobb(ibngrp) = kprobb(ibngrp)+numbounelms

         if ( kprobj(ibngrp,1)==0 ) then

!        --- kprob part j has not yet been filled

            kprobj(ibngrp,1) = inpelm           ! number of nodes
            kprobj(ibngrp,2) = inpelm-1  ! shape number

         else if ( kprobj(ibngrp,1)/=inpelm ) then

!        --- error kprob part j has been filled with other value

            call errint ( kprobj(ibngrp,1), 1 )
            call errint ( ibngrp, 2 )
            call errint ( inpelm, 3 )
            call errsub ( 1093, 3, 0, 0 )

         end if  ! ( kprobj(ibngrp,1)==0 )

!        --- Raise ikprbi and iposgrp

         ikprbi = ikprbi+numbounelms*inpelm
         iposgrp(ibngrp) = iposgrp(ibngrp)+numbounelms*inpelm
         if ( debug ) write(irefwr,1) 'numbounelms', numbounelms

      end do  ! igrp = 1, natlevset

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinp, iposgrp(ibngrp)', iinp, iposgrp(ibngrp)
         call prinin ( kprobb, kprobloc(12), 'kprob part b' )
         write(irefwr,*) 'End probnatblevel3'

      end if  ! ( debug )
      call erclos ( 'probnatblevel3' )

      end
