      subroutine probparglob4 ( nunkp, npointloc, map, indprfglob, &
                                indprfloc, kprobfloc, kprobfglob, &
                                nrusolloc, kprobhloc, kprobhglob, &
                                nusolloc, nrusolglob, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 17-09-2014 Extra parameters
!        version  1.0  date 27-03-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill positions in kprobhglob with a -1 if they correspond to
!     prescribed boundary conditions
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: indprfglob, indprfloc, nunkp, npointloc, &
                             map(npointloc), kprobfloc(*), &
                             kprobfglob(*), nrusolloc, nusolloc, &
                             kprobhloc(2*nusolloc)
      integer, intent(inout) :: kprobhglob(*), nrusolglob
      integer :: iwork(nusolloc)

!     indprfglob     i    Indicates if array kprobfglob has been filled (>0)
!     indprfloc      i    Indicates if array kprobfloc has been filled (>0)
!     iwork               Work array to find the correct sequence in
!                         kprobhglob
!     kprobfglob     i    Array kprobf corresponding to global mesh
!     kprobfloc      i    Array kprobf corresponding to local mesh
!     kprobhglob    i/o   Array kprobh corresponding to global mesh
!     kprobhloc      i    Array kprobh corresponding to local mesh
!     map            i    Mapping array from loclal to global nodes
!     npointloc      i    Number of points in local mesh
!     nrusolglob    i/o   Global number of unknowns
!                         At input number of unknowns in blocks with smaller
!                         sequence number
!                         At input number including this block
!     nrusolloc      i    Number of free degrees of freedom in local mesh
!     nunkp          i    Maximum number of unknowns in a point
!     nusolloc       i    Local number of degrees of freedom (in block)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, inunkp, node, startloc, startglob, j

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     j              Counting variable
!     node           Node number
!     startglob      starting address -1 for node in kprobhglob
!     startloc       starting address -1 for node in kprobhloc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probparglob4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probparglob4'
         write(irefwr,1) 'nunkp, npointloc, nrusolloc, nusolloc', &
                          nunkp, npointloc, nrusolloc, nusolloc
         write(irefwr,1) 'nrusolglob', nrusolglob
         call prinin ( map, npointloc, 'map' )
         call prinin ( kprobhloc, nusolloc, 'kprobhloc' )
         call prinin ( kprobhloc(1+nusolloc), nusolloc, 'kprobhlocinv' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iwork = 0

      inunkp = nunkp
      do i = 1, npointloc

         node = map(i)
         if ( indprfloc>0 ) then

!        --- indprfloc>0, hence variable number of unknowns

            startloc = kprobfloc(i)
            inunkp = kprobfloc(i+1)-startloc

         else

!        --- indprfloc = 0, hence constant number of unknowns

            startloc = nunkp*(i-1)

         end if  ! ( indprfloc>0 )
         if ( indprfglob>0 ) then
            startglob = kprobfglob(node)
         else
            startglob = nunkp*(node-1)
         end if  ! ( indprglob>0 )

!        --- mark all degrees of freedom that are prescribed by -1

         do j = 1, inunkp

            if ( kprobhloc(startloc+j)>nrusolloc ) then

!           --- Degree of freedom is prescribed

               kprobhglob(startglob+j) = -1
               iwork(startloc+j) = -1  ! mark as prescribed

            else if ( kprobhglob(startglob+j)==0 ) then

!           --- Global degree of freedom has not been given a number
!               Mark position in kprobhglob in iwork

               iwork(startloc+j) = startglob+j  ! position in kprobhglob

            end if  ! ( kprobhloc(startloc+j)>nrusolloc )

         end do  ! j = 1, inunkp

      end do  ! i = 1, npointloc

      if ( debug ) call prinin ( iwork, nusolloc, 'iwork' )

      do i = 1, nusolloc

         j = kprobhloc(nusolloc+i)  ! position in kprobh inverse

         if ( iwork(j)>0 ) then

!        --- iwork(j)>0, hence unknown has no global number yet
!            Raise nrusolglob and store in appropriate position of kprobhglob

            nrusolglob = nrusolglob+1
            kprobhglob(iwork(j)) = nrusolglob

         end if  ! ( kprobhglob(startglob+j)==0 )

      end do  ! i = 1, nusolloc

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kprobhglob, nrusolloc, 'kprobhglob' )
         write(irefwr,*) 'End probparglob4'

      end if  ! ( debug )
      call erclos ( 'probparglob4' )

      end
