      subroutine probparglob5 ( nunkp, npointloc, kprobfglob, maploc, &
                                indprfloc, indprfglob, kprobhglob, &
                                nrusolloc, kprobhloc, kprobfloc, &
                                kprobh, nusol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 17-09-2014 Extra parameter kprobh
!        version  1.0  date 29-03-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute mapping from global kprobh to local kprobh for all nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nunkp, npointloc, kprobfglob(*), maploc(*), &
                             indprfloc, indprfglob, kprobfloc(*), &
                             kprobh(*), nusol
      integer, intent(out) ::  kprobhglob(*), nrusolloc
      integer, intent(inout) ::  kprobhloc(*)

!     indprfglob     i    indicates if kprobfglob is filled (>0) or not (0)
!     indprfloc      i    indicates if kprobfloc is filled (>0) or not (0)
!     kprobfglob     i    Global array kprobf
!     kprobfloc      i    Local array kprobf
!     kprobh         i    KPROB part h: contains the new numbering of the
!                         unknowns
!     kprobhglob     o    Global array kprobh
!     kprobhloc     i/o   Local array kprobh
!     maploc         i    Local mapping array, gives global node numbers of
!                         local nodes
!     npointloc      i    Number of points in local mesh
!     nrusolloc      i    Local number of unknowns
!     nunkp          i    Maximum number of unknowns in a point
!     nusol          i    Total number of unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: inunkp, i, node, startglob, j, inusol, icount
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     icount         Counter
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     iwork          integer work array to store copy of kprobhloc
!     j              Counting variable
!     node           Node number
!     startglob      Starting position in kprobhglob
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nrusolloc = 0
!     for i := 1 (1) npointlocal do
!        node := maplocal(i)
!        inunkp := number of unknowns in local node i
!        istart := starting address in global array kprobh
!        for j := 1 (1) inunkp do
!           if ( kprobhglob(istart+j)>0 ) then
!              nrusolloc = nrusolloc+1
!              kprobhloc(nrusolloc) = kprobhglob(istart+j)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probparglob5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probparglob5'
         write(irefwr,1) 'nunkp, npointloc, indprfloc, indprfglob', &
                          nunkp, npointloc, indprfloc, indprfglob
         call prinin ( maploc, npointloc, 'maploc' )
         call prinin ( kprobhglob, 10, 'kprobhglob' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- nrusolloc = 0

      nrusolloc = 0
      inunkp = nunkp

!     --- for i := 1 (1) npointloc do

      do i = 1, npointloc

!     --- node := maploc(i)
!         inunkp := number of unknowns in local node i
!         istart := starting address in global array kprobh

         node = maploc(i)
         if ( indprfloc>0 ) inunkp = kprobfloc(i+1)-kprobfloc(i)
         if ( indprfglob>0 ) then
            startglob = kprobfglob(node)
         else
            startglob = nunkp*(node-1)
         end if  ! ( indprglob>0 )

         if ( debug ) &
            write(irefwr,1) 'i, node, inunkp, startglob', &
                             i, node, inunkp, startglob

!        --- for j := 1 (1) inunkp do

         do j = 1, inunkp

!        --- if ( kprobhglob(istart+j)>0 ) then
!              nrusolloc = nrusolloc+1
!              kprobhloc(nrusolloc) = kprobhglob(istart+j)

            inusol = kprobhglob(startglob+j)

            if ( debug ) write(irefwr,1) 'j, inusol', j, inusol

            if ( inusol>0 ) then

!           --- free degree of freedom found

               nrusolloc = nrusolloc+1
               kprobhloc(nrusolloc) = inusol

            end if  ! ( inusol>0 )

         end do  ! j = 1, inunkp

      end do  ! i = 1, npointlocal

!     --- Refill kprobhloc such that the sequence of the local kprobh
!         is used
!         First copy kprobhloc into work array iwork

      allocate ( iwork(nrusolloc), stat = error )
      if ( error/=0 ) call eralloc ( error, nrusolloc, 'iwork' )
      if ( ierror/=0 ) go to 1000

      iwork(1:nrusolloc) = kprobhloc(1:nrusolloc)

      if ( kprobh(1)>0 ) then

!     --- kprobh(1)>0, hence kprobh has been filled

         icount= 0    ! counter of non-described degrees of freedom

         if ( debug ) then
            write(irefwr,1) 'nusol, nrusolloc', nusol, nrusolloc
            call prinin ( kprobh, nusol, 'kprobh' )
         end if  ! ( debug )

         do i = 1, nusol     ! loop over all degrees of freedom in block

            inusol = kprobh(i)  ! sequence number of degfd in block

            if ( inusol<=nrusolloc ) then

!           --- Free degree of freedom, copy

               icount = icount+1
               kprobhloc(inusol) = iwork(icount)

            end if  ! ( inusol<=nrusol )

         end do  ! i = 1, nusol

      end if  ! ( kprobh(1)>0 )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nrusolloc', nrusolloc
         call prinin ( kprobhloc, nrusolloc, 'kprobhloc' )
         write(irefwr,*) 'End probparglob5'

      end if  ! ( debug )
      call erclos ( 'probparglob5' )

      end
