      subroutine prord0per ( iinplc, iinputlc, lengrllc, rinputlc, ielgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-10-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information for periodical boundary conditions
!     Type number -1
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: iinplc, iinputlc(*), lengrllc
      integer, intent(in) :: ielgrp
      double precision, intent(inout) :: rinputlc(*)

!     ielgrp         i    element group number
!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   See subroutine prord0
!     lengrllc      i/o   See subroutine prord0
!     rinputlc      i/o   See subroutine prord0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icount, iinplcsav, inum, i, ihelp(20), inunkp, ihelp1(20)
      logical :: finished

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     icount         Counter
!     ihelp          Work array of length 20 to check if a degree of freedom is
!                    already in use in a point
!                    ihelp(i) = 0, degree of freedom i is not in use
!                    ihelp(i) = 1, degree of freedom i is in use
!     ihelp1         Work array of length 20 to store the degrees of freedom
!     iinplcsav      Help parameter to save value of iinplc
!     inum           Number of numbers read in a record
!     inunkp         Number of degrees of freedom in point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibit

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIBIT         Translate degrees of freedom into bit pattern
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     839   Degree of freedom has incorrect value
!    1796   Multiplication factor too small
!    2028   Degree of freedom found more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord0per' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord0per'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncode = 3
      code(1) = 'CONSTANT'
      code(2) = 'FACTOR'
      code(3) = 'DEGFD'
      ncode(1:nncode) = 3
      ncodmx(1) = 8
      ncodmx(2) = 6
      ncodmx(3) = 5

      itype(1:nncode) = 1
      itype(3) = 14
      tabmin(1:nncode) = -1d9
      tabmax(1:nncode) = 1d9
      codemain(nncodemain+1) = codesub(1)
      ncodemain(nncodemain+1) = ncodesub(1)
      nmain = -nncodemain-1

!     --- Check if the next record is of the type
!         constant = c, factor = alpha, degfd = ...

      finished = .false.
      icount = 0
      iinplcsav = iinplc+1   ! start information per. bc.
      iinplc = iinplc+1
      do while ( .not. finished )

!     --- Since more of these records are allowed, this process is
!         repeated until no further cards are found

         table(1:2) = -1d20
         table(3) = 0
         tabmin(3) = 20d0
         tabmax(3) = 20d0
         iretur = 0
         if ( debug ) write(irefwr,*) 'before inxt02/2'
         call sepreadsetkeyw

         if ( ierror/=0 ) go to 1000
         icode = iretur
         if ( icode>0 ) then

!        --- new type of keyword found

            finished = .true.
            iinputlc(iinplcsav) = max(1,icount)  ! number of parts
            if ( icount==0 ) then

!           --- No record found

               rinputlc(lengrllc+1) = 0d0  ! default constant
               rinputlc(lengrllc+2) = 1d0  ! default factor
               iinputlc(iinplc+1) = 0      ! all degrees of freedom
               iinputlc(iinplc+2) = lengrllc+1  ! start in rinput
               lengrllc = lengrllc + 2
               iinplc = iinplc+2
               icode = 8

            end if

         else

!        --- information found

            icount = icount+1
            rinputlc(lengrllc+1) = 0d0
            rinputlc(lengrllc+2) = 1d0
            if ( table(1)>-1d20 ) &
               rinputlc(lengrllc+1) = table(1)  ! constant
            if ( table(2)>-1d20 ) &
               rinputlc(lengrllc+2) = table(2)  ! factor
            if ( abs(rinputlc(lengrllc+2))<=epsmac ) then

!           --- Error 1796:  multiplication factor too small

               call erreal ( rinputlc(lengrllc+2), 1 )
               call errint ( ielgrp, 1 )
               call errsub ( 1796, 1, 1, 0 )

            end if

            iposut = nint(table(3))
            if ( iposut==0 ) then

!           --- No degree of freedom found, i.e. other type of record

               iinputlc(iinplc+1) = 0   ! all degrees of freedom

            else

!           --- degrees of freedom found

               inum = nint(tabmax(3))
               ihelp = 0
               do i = 1, inum
                  inunkp = nint(tabout(iposut+i-1))
                  if ( inunkp<1 .or. inunkp>20 ) then

!                 --- i-th degree of freedom out of range

                     call errint ( inunkp, 1 )
                     call errint ( 20, 2 )
                     call errsub ( 839, 2, 0, 0 )
                     call errcrd
                     inunkp = 20

                  end if
                  if ( ihelp(inunkp)==0 ) then

!                 --- Degree of freedom is new

                     ihelp1(i) = inunkp

                  else

!                 --- Degree of freedom has been found before

                     call errint ( inunkp, 1 )
                     call errwar ( 2028, 1, 0, 0 )
                     call errcrd

                  end if
               end do
               inunkp = inibit ( ihelp1, inum )
               iinputlc(iinplc+1) = inunkp

            end if

            iinputlc(iinplc+2) = lengrllc+1

            lengrllc = lengrllc + 2
            iinplc = iinplc+2
            icode = 8

         end if

      end do  ! while ( .not. finished )

1000  if ( debug ) then

!     --- Debug information

         icount = iinputlc(iinplcsav)
         write(irefwr,*) 'local periodical boundary conds'
         write(irefwr,1) 'iinplcsav', iinplcsav
         write(irefwr,1) 'lengrllc, iinplc, icount', &
                            lengrllc, iinplc, icount
         call prinin ( iinputlc(iinplcsav), 1+2*icount, &
                       'iinputlc' )
         write(irefwr,*) 'Contents are: icount, followed by', &
                         ' pairs of idegfd, start in rinput'
         call prinrl ( rinputlc(lengrllc-1), 2, 'rinputlc' )
         write(irefwr,*) 'Contents are: pairs of constant, factor'
         write(irefwr,*) 'End prord0per'

      end if  ! ( debug )
      call erclos ( 'prord0per' )

      end subroutine prord0per
