      subroutine sepcommatnest
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-05-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute matrix matrix structure for nested dissection
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: irowsnew(:), icolsnew(:), rows(:), cols(:)
      integer :: lencols, lengnew, mfact
      real :: fact
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     cols           Work array to store the column numbers of the matrix
!                    including fill in upper triangular part
!     fact           Help parameter to compute mfact
!     icolsnew       Work array to store the column numbers of the matrix
!                    upper triangular part
!     irowsnew       Work array to store starting addresses (-1) in icolsnew
!     lencols        length of the part columns of the matrix structure
!     lengnew        New length of array cols
!     mfact          Multiplication factor
!     rows           Work array to store starting addresses (-1) in cols
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPCOMMATLOWUP store integer information of lower triangular part
!                    including diagonal into irowsii and icolsii
!     SEPCOMMATNEST1 Compute length of array cols and fill array rows
!     SEPCOMMATNEST2 Fill array cols
!     SEPCOMMATREV   Fill arrays irowsnew and icolsnew with upper triangular
!                    part if icolsii and irowsii contain the lower triangular
!                    part and vice versa
!     SEPINTMAT3     Transform symmetric row-compact storage scheme to
!                    asymmetric row-compact storage
!     SEPPRINCOMPSTR print part of array intmat corresponding to internal
!                    unknowns. Compact storage
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcommatnest' )
      IM => intmt(intmat)
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcommatnest'
         call sepprincompstr ( nrusol, IM%rowsii, IM%colsii )
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( IM%symmetric ) then

!     --- Store upper triangular part instead of under triangular part
!         in  irowsnew and icolsnew

         allocate ( irowsnew(nrusol+1), icolsnew(size(IM%colsii)), &
                    rows(nrusol+1), IM%rowsiiprec(nrusol+1), &
                    IM%colsiiprec(size(IM%colsii)) )
         irowsnew = IM%rowsii
         icolsnew = IM%colsii

         irowsii => IM%rowsiiprec
         icolsii => IM%colsiiprec
         call sepcommatrev ( irowsii, icolsii, IM%rowsii, IM%colsii, nrusol )
         if ( debug ) call sepprincompstr ( nrusol, irowsii, icolsii )

      else

!     --- asymmetric case, store lower triangular part including diagonal
!         into irowsii and icolsii
!         Store upper part in irowsnew and icolsnew

         length = (size(IM%colsii)-nrusol)/2+nrusol
         allocate ( irowsnew(nrusol+1), icolsnew(length), &
                    rows(nrusol+1), IM%rowsiiprec(nrusol+1), &
                    IM%colsiiprec(length) )
         irowsii => IM%rowsiiprec
         icolsii => IM%colsiiprec
         call sepcommatlowup ( irowsnew, icolsnew )
         if ( debug ) then
            write(irefwr,*) 'lower part'
            call sepprincompstr ( nrusol, irowsii, icolsii )
            write(irefwr,*) 'upper part'
            call sepprincompstr ( nrusol, irowsnew, icolsnew )
         end if  ! ( debug

      end if  ! ( IM%symmetric )

!     --- Compute length of array cols and fill array rows

      call sepcommatnest1 ( irowsnew, icolsnew, rows )
      if ( debug ) call prinin ( rows, nrusol, 'rows' )

!     --- Fill array cols

      length = 0
      lengnew = 1
      fact = 0.5/ndim
      mfact = max(3,2*nint(real(nrusol)**fact))
      do while ( lengnew>length )

!     --- Add columns corresponding to fill-in to cols
!         This is done by adding the columns of rows with lower number that are
!         connected to actual row number
!         This is done repeatedly since fill-in adds columns to rows
!         The process is stopped when no extra entries for the fill-in are found

         lencols = rows(nrusol+1)
         allocate ( cols(lencols*mfact) )
         call sepcommatnest2 ( irowsnew, icolsnew, rows, cols, &
                               lencols*mfact )
         length = lengnew
         lengnew = rows(nrusol+1)

         if ( debug ) write(irefwr,1) 'length, lengnew', length, lengnew

         if ( lengnew>length ) then

!         --- Loop not yet finished,
!             copy rows and cols into irowsnew and icolsnew
!             Refill icolsii and irowsii

            deallocate ( icolsnew, IM%colsiiprec )
            allocate ( icolsnew(lengnew), IM%colsiiprec(lengnew) )
            icolsnew = cols(1:lengnew)
            irowsnew = rows
            icolsii => IM%colsiiprec
            call sepcommatrev ( irowsii, icolsii, irowsnew, icolsnew, nrusol )
            deallocate(cols)

         end if  ! ( rows(nrusol+1)>length )

      end do  ! while ( lengnew>length )

!     --- Store lower triangular part instead of upper triangular part
!         in irowsnew  and icolsnew

      deallocate ( IM%colsiiprec )

      if ( IM%symmetric ) then

!     --- Store upper triangular part instead of lower triangular part
!         in IM%rowsiiprec and IM%colsiiprec

         allocate ( IM%colsiiprec(lengnew), IM%diagprec(nrusol) )

         IM%rowsiiprec = rows
         IM%colsiiprec = cols(1:lengnew)
         IM%diagprec(1:nrusol) = IM%rowsiiprec(1:nrusol)+1

      else

!     --- asymmetric case, store complete part
!         in IM%rowsiiprec and IM%colsiiprec

         length = 2*lengnew-nrusol
         allocate ( IM%colsiiprec(length), IM%diagprec(nrusol) )

         call sepintmat3 ( nrusol, IM%rowsiiprec, IM%colsiiprec, &
                           length, rows, cols, lengnew, IM%diagprec )

      end if  ! ( IM%symmetric )

      deallocate ( irowsnew, icolsnew, rows, cols )

1000  if ( debug ) then

!     --- Debug information

         call sepprincompstr ( nrusol, IM%rowsii, IM%colsii )
         call sepprincompstr ( nrusol, IM%rowsiiprec, IM%colsiiprec )
         call prinin ( IM%diag, nrusol, 'IM%diag' )
         write(irefwr,*) 'End sepcommatnest'

      end if  ! ( debug )
      call erclos ( 'sepcommatnest' )

      end subroutine sepcommatnest
