      subroutine sepcommatpar
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create the structure of the off-diagonal matrix blocks
!     Parallel case
!     Old subroutine incommatpar
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulecpack
      use sepmodulematstr
      use sepmodulekprob
      use sepmoduleintmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numunks, leniwork, i
      integer :: lenstore, lensend, lenrec
      integer, allocatable :: icopy(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     icopy          work array to store a temporary copy of sendunknowns
!     leniwork       Computed length of array iwork
!     lenrec         Length of array recunknowns
!     lensend        Length of array sendunknowns
!     lenstore       Computed length of array istore
!     numunks        Number of unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INCOMMATPAR1   send information of rowsfi to all neighbor blocks with
!                    lower number and receive information rowsfi from all
!                    neighbor blocks with higher number
!     INCOMMATPAR10  Check if there are interface unknowns from higher blocks
!                    that have not been stored in sendunknowns
!                    If so adapt sendunknowns
!     INCOMMATPAR2   Compute length of new array istore
!     INCOMMATPAR3   Copy information of istore into array storeinfo
!     INCOMMATPAR4   send information of icolsfi to all neighbor blocks with
!                    lower number and receive from neighbors with higher number
!     INCOMMATPAR5   Fill row information for the part of the rows in a block
!                    that refers to unknowns outside the block
!                    Hence the global matrix Sif (Siout)
!     INCOMMATPAR6   Fill column information for the part of the rows in a block
!                    that refers to unknowns outside the block
!                    Hence the global matrix Sif (Siout)
!     INCOMMATPAR8   Construct array sendunknowns with the global sequence
!                    numbers of the unknowns to be transported to
!                    neighbor blocks
!     INCOMMATPAR9   Send the contents of array sendunknowns in global numbers
!                    to other blocks
!                    Receive these arrays and create array recunknowns
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     send information to all neighbor blocks with lower number
!     receive information from all neighbor blocks with higher number
!     Store all information received
!     Create structure of off diagonal matrix blocks and store
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcommatpar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcommatpar'
         write(irefwr,1) 'nrusol, ninterface', nrusol, ninterface
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

!     --- Get starting addresses

      length = blockinfor(1)+blockinfor(2)+3
      numunks = blockinfor(length)

      if ( debug ) write(irefwr,1) 'numunks', numunks

      lenstore = iacnodes+3*(3*numunks+1)+1
      allocate ( istore(lenstore), stat = error )
      if ( error/=0 ) call eralloc ( error, lenstore, 'istore' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'lenstore/1, nrusol', lenstore, nrusol

!     --- Fill array kprobhglobinv with inverse of array kprobhglob

      istore = 0
      kprobhglobinv = 0
      do i = 1, nrusol
         kprobhglobinv(kprobhglob(i)) = i
      end do  ! i = 1, nrusol

!     --- Send and receive row information of interface unknowns

      call incommatpar1 ( leniwork )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,1) 'lenstore/2', lenstore

!     --- Compute length of new array istore

      call incommatpar2 ( lenstore )
      if ( ierror/=0 ) go to 1000

!     --- Create structure of information to be received

      allocate ( IM%storeinfo(lenstore), stat = error )
      if ( error/=0 ) call eralloc ( error, lenstore, 'storeinfo' )
      if ( ierror/=0 ) go to 1000
      storeinfo => IM%storeinfo

!     --- Copy information of istore in storeinfo

      if ( lenstore>iacnodes ) then

!     --- lenstore>iacnodes

         call incommatpar3 ( leniwork )

      else

!     --- lenstore=iacnodes: clear array

         storeinfo = 0

      end if  ! ( lenstore>iacnodes )
      if ( ierror/=0 ) go to 1000

!     --- send and receive column information of interface unknowns

      call incommatpar4 ( leniwork )
      if ( ierror/=0 ) go to 1000

!     --- Construct the information of the off diagonal blocks
!         i.e. the parts of the rows that refer to unknowns in other blocks
!         This part is built from S_fi and S_if
!         First we compute the size
!         Create irowsout

      allocate ( IM%rowsout(1:nrusol+1), stat = error )
      if ( error/=0 ) call eralloc ( error, nrusol+1, 'irowsout' )
      if ( ierror/=0 ) go to 1000
      irowsout(0:nrusol) => IM%rowsout

      call incommatpar5
      if ( ierror/=0 ) go to 1000

!     --- Fill columns in temporary array and recompute length
!         Also irowsout is refilled

      length = max(1000,10*irowsout(nrusol))
      allocate ( icolsout(length), iwork(nusolglob), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*length, 'icolsout' )
      if ( ierror/=0 ) go to 1000

      call incommatpar6
      if ( ierror/=0 ) go to 1000
      allocate ( IM%colsout(length), stat = error )
      IM%colsout = icolsout(1:length)
      deallocate ( icolsout )
      icolsout => IM%colsout

!     --- Fill array sendunknowns with numbers of unknowns to be sent
!         to neighboring blocks

      lensend = iacnodes+nusol+1+irowsout(nrusol)
      lenrec = lensend

      allocate ( sendunknowns(0:lensend), recunknowns(lenrec), &
                 icopy(lensend+1), stat = error )
      if ( error/=0 ) call eralloc ( error, lensend, 'sendunknowns' )
      if ( ierror/=0 ) go to 1000

      call incommatpar8 ( lensend )

!     --- Check if there are interface unknowns from higher blocks
!         that have not been stored in sendunknowns
!         If so adapt sendunknowns

      call incommatpar10 ( lensend, icopy )
      allocate ( IM%sendunknowns(lensend) )
      IM%sendunknowns = sendunknowns(0:lensend-1)
      deallocate ( sendunknowns )
      sendunknowns(0:lensend-1) => IM%sendunknowns

!     --- Create array recunknowns indicating the unknowns to be received
!         from other blocks

      call incommatpar9 ( lenrec )
      allocate ( IM%recunknowns(lenrec) )
      IM%recunknowns = recunknowns(1:lenrec)
      deallocate ( recunknowns )
      recunknowns =>  IM%recunknowns

      deallocate ( istore, icopy, iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'istore' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcommatpar'

      end if  ! ( debug )
      call erclos ( 'sepcommatpar' )

      end subroutine sepcommatpar
