      subroutine seper0084 ( icrv1, icrv2, jstap, n, iwork )
! ======================================================================
!
!        programmer     Guus Segal
!        version  1.0  date 16-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     fill nodal points of curves icrv1 to icrv2 in array iwork
!     In this case only one-dimensional elements are assumed
!     Hence kmeshm can not be used
!     Old subroutine er0084
! ********************************************************************
!
!                      KEYWORDS
!
!     curve
!     fill
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! ********************************************************************
!
!                 INPUT / OUTPUT    PARAMETERS
!
      integer icrv1, icrv2, jstap, n, iwork(*)

!     icrv1   i    First curve number to be stored in iwork
!     icrv2   i    Last curve number to be stored in iwork
!                  icrv1 to icrv2 must be subsequent curves
!     iwork   o    array containing the nodal point numbers of the curves
!                  subsequently
!     jstap   i    number of points per element minus one (curve elements)
!     n       o    number of points on the curves icrv1 to icrv2
! ********************************************************************
!
!                    COMMON BLOCKS
!
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer iw, k, ifirst, ilast, inpelm, istart, &
              neltot, ielgrp, i, jnpelm, jnelem, ielem

!     i       Counting variable
!     ielem   Element number
!     ielgrp  element group number
!     ifirst  First curve (surface) number read
!     ilast   Last curve (surface) number read
!     inpelm  number of points per element
!     istart  First position used at a part
!     iw      Address in array IWORK
!     jnelem  number of elements in an element group
!     jnpelm  Number of nodes in an element
!     k       Counting variable
!     neltot  number of elements in an element goup
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!      45   first point in part of boundary is not equal to last point
!     100   JSTAP has wrong value
!     185   first curve number (icrv1) is out of range
!     186   last curve number (icrv2) is out of range
!    1528   Number of nodes not constant in elements along icrv1 to icrv2
!    1529   The curves icrv1 to icrv2 are not contiguous
! ********************************************************************
!
!                    PSEUDO CODE
!
!   Find begin and element numbers of the curves
!   Check if these curves are contiguous
!   Check if all curves belong to the same element group
!   Check jstap
!   Find starting address in kmeshc
!   Fill first node in iwork
!   For all elements do
!     if ( first node not equal to last node in iwork ) then
!        error message
!     fill all nodes in iwork except first one
! ======================================================================
!
      call eropen ( 'seper0084' )
      if ( icrv1<=0 .or. icrv1>ncurvs ) then
         call errint ( icrv1, 1 )
         call errint ( ncurvs, 2 )
         call errsub ( 185, 2, 0, 0 )
      end if
      if ( icrv2<icrv1 .or. icrv2>ncurvs ) then
         call errint ( icrv2, 1 )
         call errint ( icrv1, 2 )
         call errint ( ncurvs, 3 )
         call errsub ( 186, 3, 0, 0 )
      end if

!     --- Find begin and element numbers of the curves
!         Check if these curves are contiguous

      ifirst = kmesht(1,icrv1)
      ilast  = kmesht(2,icrv1)
      do 100 k = icrv1+1, icrv2
         if ( kmesht(1,k)/=ilast ) then
            call errint ( kmesht(1,k), 1 )
            call errint ( ilast, 2 )
            call errint ( k, 3 )
            call errint ( 0, 4 )
            call errsub ( 45, 4, 0, 0 )
         end if
         ilast  = kmesht(2,k)
100   continue

!     --- Check if all curves belong to the same element group

      if ( kelma==0 ) then

!     --- only one element group or all elements are of the same type

         inpelm = npelm
         istart = (ifirst-1)*inpelm

      else

!     --- more element groups
!         Find first element group
!         Find starting address in kmeshc

        neltot = 0
        istart = 0
        do 120 ielgrp = 1, nelgrp
           inpelm = Topgrp(ielgrp)%inpelm
           inelem = Topgrp(ielgrp)%inelem
           if ( ifirst<neltot+inelem ) then

!          --- First element belongs to element group ielgrp

              istart = istart + (ifirst-1-neltot)*inpelm

!             --- Check to which group the last one belongs

              do 110 i = ielgrp, nelgrp
                 jnpelm = Topgrp(i)%inpelm
                 jnelem = Topgrp(i)%inelem
                 if ( jnpelm/=inpelm ) then

!                --- Error number of nodes per element not constant

                    call errint ( icrv1, 1 )
                    call errint ( inpelm, 2 )
                    call errint ( icrv1, 3 )
                    call errint ( icrv2, 4 )
                    call errint ( jnpelm, 5 )
                    call errsub ( 1528, 5, 0, 0 )

                 end if
                 neltot = neltot+jnelem
                 if ( neltot>=ilast ) go to 150

110           continue
              go to 150

           end if
           istart = istart + inpelm*inelem
           neltot = neltot + inelem
120     continue

      end if

150   inelem = ilast-ifirst+1
      n = inelem*(inpelm-1)

!     --- Check jstap

      inelem = n/jstap
      if ( inelem*jstap/=n ) then
         call errint ( inelem*jstap, 1 )
         call errint ( n, 2 )
         call errsub ( 100, 2, 0, 0 )
      end if
      n = n+1

!     --- Fill first node in iwork

      istart = istart+1
      iwork(1) = kmeshc(istart)
      iw = 1
      do 200 ielem = ifirst, ilast
         if ( kmeshc(istart)/=iwork(iw) ) then
            call errint ( icrv1, 1 )
            call errint ( icrv2, 2 )
            call errsub ( 1529, 2, 0, 0 )
         end if

!        --- fill all nodes in iwork except first one

         do 190 i = 1, inpelm-1
            iwork(iw+i) = kmeshc(istart+i)
190      continue
         istart = istart + inpelm
         iw = iw + inpelm-1

200   continue

      call erclos ( 'seper0084' )

      end
