      subroutine sepflpointobs2
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array ipointobst (2d)
!     Old subroutine mshfillpoinobs
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduleplot
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenipointobst, iobst, nx, ny, leniobgrid1, &
              lencurvs, ikader, iscolr, ipipoin
      integer, allocatable :: iwork(:), iobgrid(:), iobgrid1(:)
      integer, allocatable :: ipointobstlc(:)

!     ikader         Help variable to store jkader temporarily
!     iobgrid        position i,j contains:
!                    -1:  cell inside obstacle
!                     0:  cell outside obstacle
!                    >0:  cell contains part of the obstacle
!                    The value contains a starting address in iobgrid1
!     iobgrid1       Must be used in connection with iobgrid
!                    iobgrid contains the starting addresses for each cell
!                    i,j
!                    From this starting address iobgrid1 contains
!                    the number of curve parts of the obstacle
!                    intersecting the cell followed by the sequence
!                    numbers of these curve parts
!     iobst          Obstacle sequence number
!     ipipoin        Starting address in array ipointobst
!     ipointobstlc   Local array ipointobst
!     iscolr         Help variable to store the value of dcolor temporarily
!     iwork          integer work array
!     lencurvs       Overestimate of the number of points along the new
!                    curves
!     leniobgrid1    estimated and actual length of array iobgrid1
!     lenipointobst  length of array ipointobst
!     nx             number of cells in x direction
!     ny             number of cells in y direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERTRACE        Prints trace back of subroutines
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSHOBSFLPERM   Store array iobgrid as permanent array in ibuffr
!     PL0003         Open plot, plots frame work and computes plotting
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLOBSTACLEGRID Plot rectangular grid covering obstacle
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPINITKMESHAD Initialize kmesh ad
!     SEPOBSFL       Fill array iobgrid with grid information of one obstacle
!     SEPOBSTMARKBT1 Fill ipointobst part 4, i.e. the set of boundary nodes of
!                    an obstacle that are only inside obstacle elements
!     SEPOBSTMARKPT  Mark all nodal points in the mesh with information if they
!                    are within an obstacle or outside an obstacle
!     SEPPLOTBN      Plot boundary of region
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2206   KMESH part x has not been filled
!    2207   There are no obstacles
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepflpointobs2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepflpointobs2'
         call ertrace
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initializations

      if (debug)  write(irefwr,1) 'numobst, npoint', numobst, npoint
      if ( numobst==0 ) then

!     --- No obstacles

         call errwar ( 2207, 0, 0, 0 )
         go to 1000

      end if

      if ( kelmd==0 ) call sepfillkmeshd

!     --- Initialize kmesh part ad

      call sepinitkmeshad

!     --- Reserve temporary space for array ipointobstlc

      lenipointobst = 5*numobst+npoint

      allocate ( ipointobstlc(lenipointobst), iwork(npoint+nelem), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, lenipointobst,'ipointobst' )
      if ( ierror/=0 ) go to 1000

!     --- Set starting address for first obstacle

      ipipoin = numobst+1

!     --- for iobst := 1 (1) numobst do

      do iobst = 1, numobst

!     --- Mark all nodes of mesh in obstacle

         ipointobstlc(iobst) = ipipoin
         if ( kelmx==0 ) then

!        --- error, kmesh part x has not been filled

            call errsub ( 2206, 0, 0, 0 )
            go to 1000

         end if
         nx = kmeshx(3,iobst)
         ny = kmeshx(4,iobst)

         if ( debug ) write(irefwr,1) 'nx, ny', nx, ny
         if ( ierror/=0 ) go to 1000

         if ( kmeshx(5,iobst)==0 ) then

!        --- Array has not yet been filled

            leniobgrid1 = 2*nx*ny
            allocate ( iobgrid(nx*ny), iobgrid1(leniobgrid1), &
                       stat = error )
            if ( error/=0 ) &
               call eralloc ( error, nx*ny+leniobgrid1, 'iobgrid' )
            if ( ierror/=0 ) go to 1000

            lencurvs = size ( curvesmsh )
            call sepobsfl ( iobgrid, nx, ny, iobst, lencurvs, iobgrid1, &
                            leniobgrid1 )

!           --- Make array iobgrid permanent

            call mshobsflperm ( iobgrid, iobgrid1, &
                                nx, ny, leniobgrid1, iobst )

            deallocate ( iobgrid, iobgrid1, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'iobgridcop' )

            if ( debug .or. plotobst ) then

!           --- open plot

               jkader = -1
               textx = 'x'
               texty = 'y'
               call pl0003 ( ikader, 1, 15d0, 1d0, 10 )

!              --- plot elements

               call plobstaclegrid ( rmeshx, nx, ny, &
                                     iobgrid2d, iobst )

!              --- plot boundary

               iscolr = dcolor
               dcolor = 1
               call sepplotbn ( 0, 3, .true. )
               dcolor = iscolr

!              --- close plot dataset

               call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )

            end if
         end if

         lencurvs = size ( curvesmsh )
         call sepobstmarkpt ( iobst, nx, ny, iobgrid2d, &
                              ipointobstlc(ipipoin), iwork, lencurvs, &
                              iobgrid2d1 )

!        --- Fill part 4 of ipointsobst of nodes on the boundary of the
!            obstacle which are only in elements completely inside
!            the obstacle

         call sepobstmarkbt1 ( ipointobstlc(ipipoin) )

!        --- raise ipipoin

         ipipoin = ipipoin+4+ &
            ipointobstlc(ipipoin)+ipointobstlc(ipipoin+1)+ &
            ipointobstlc(ipipoin+2)+ipointobstlc(ipipoin+3)

         call inichkarlen ( ipipoin, lenipointobst, 'ipointobst' )
         if ( ierror/=0 ) go to 1000

      end do

!     --- Make array ipointobst permanent
!         Copy contents of temporary array into permanent array

      lenipointobst = ipipoin-1
      if ( associated(km(iseqkm)%ipointobst) ) &
         deallocate(km(iseqkm)%ipointobst)
      allocate(km(iseqkm)%ipointobst(lenipointobst))
      kmeshad(1) = lenipointobst
      kmeshad(6) = iseqobst

!     --- Copy contents of temporary array into permanent array

      ipointobst => km(iseqkm)%ipointobst
      ipointobst = ipointobstlc(1:lenipointobst)

      deallocate ( ipointobstlc, iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ipointobstlc' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepflpointobs2'

      end if  ! ( debug )
      call erclos ( 'sepflpointobs2' )

      end subroutine sepflpointobs2
