      subroutine sepintmatprof
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-10-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform integer information for row-compact storage (non-symmetric)
!     to profile storage
!     Store result in irowsii/icolsiiprec
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM
      integer :: i, ilow, istart
      integer, pointer :: rows(:), cols(:)

!     IM             Refers to intmt(intmat)
!     cols           Refers to column numbers
!     i              Counting variable
!     ilow           smallest column number in row
!     istart         First position in a row
!     rows           Refers to start of column numbers per row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0004         Print INTMAT corresponding to internal matrix
!     SEPCOMMATREV   Fill arrays irowsnew and icolsnew with upper triangular
!                    part if icolsii and irowsii contain the lower triangular
!                    part and vice versa
!     SEPPRINCOMPSTR print part of array intmat corresponding to internal
!                    unknowns. Compact storage
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintmatprof' )
      debug = .false. .and. ioutp>=0
      IM => intmt(iseqim)
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintmatprof'
         call sepprincompstr ( nrusol, IM%rowsii, IM%colsii )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( IM%symmetric ) then
         allocate ( rows(nrusol+1), cols(size(IM%colsii)) )
         call sepcommatrev ( rows, cols, IM%rowsii, IM%colsii, nrusol )
         if ( debug ) call sepprincompstr ( nrusol, rows, cols )
      else
         rows => IM%rowsii
         cols => IM%colsii
      end if  ! ( IM%symmetric )

      allocate ( IM%rowsiiprec(nrusol), IM%colsiiprec(nrusol) )

      IM%rowsiiprec(1) = 1
      IM%colsiiprec(1) = 1
      istart = 1

      do i = 2, nrusol
         ilow = cols(rows(i)+1)    ! element with smallest column number in row
         IM%colsiiprec(i) = ilow
         IM%rowsiiprec(i) = istart+i-ilow+1  ! address of diagonal element
         if ( IM%symmetric ) then
            istart = IM%rowsiiprec(i)  ! Start address of new row
         else
            istart = IM%rowsiiprec(i)+i-ilow  ! Start address of new row
         end if  ! ( IM%symmetric )
      end do  ! i = 2, nrusol
      if ( IM%symmetric ) deallocate ( rows, cols )

1000  if ( debug ) then

!     --- Debug information

         call pr0004 ( IM%jmethod, nrusol, IM%rowsiiprec, IM%colsiiprec )
         write(irefwr,*) 'End sepintmatprof'

      end if  ! ( debug )
      call erclos ( 'sepintmatprof' )

      end subroutine sepintmatprof
