      subroutine seplevels ( viewp, iskpar, elements, elmlevels, nlevels, &
                             iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reorder a restricted set of elements in an ordered level set, starting
!     from a set of elements with the largest distance to a given view point
!     Old subroutine toplevels
! **********************************************************************
!
!                       KEYWORDS
!
!     level
!     reordering
!     distance
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iskpar(*), elements(*), elmlevels(*), nlevels, iinput(*)
      double precision :: viewp(*)

!     elements       o    Array of length elmlevels(nlevels) (<=nelem)
!                         containing the reordered element sequence numbers
!                         So elements(1) contains the first element number to
!                         be used, elements(2) the second one and so on
!     elmlevels      o    Array of length nlevels (<nelem) containing the
!                         levels in array elements
!                         The first level consists of the elements 1 to
!                         elmlevels(1) of array elements
!                         The second level contains the elements  elmlevels(1)+1
!                         to elmlevels(2) and so on
!     iinput         i    Integer array with information of the user
!                         iinput must be filled as follows
!                         Pos. 1: iskip Defines if elements must be skipped
!                                 <0, skip -iskip volumes as indicated in
!                                     iksipar
!                                  0, do not skip elements
!                                 >0, skip iskip element groups as indicated in
!                                     iksipar
!                         Pos. 2: idefstart, defines how the start level must
!                                 be computed
!                                 1: Start with set of nodes with largest
!                                    distance to view point
!                                 2: Start with set of surfaces indicated in
!                                    positions 3 and 4
!                         Pos. 3: ifirst, first surface number
!                         Pos. 4: ilast, last surface number
!     iskpar         i    array of length NELGRP that is only used if ISKIP>0
!                         It indicates if element group ielgrp must be skipped
!                         iskpar(ielgrp) = 0 or not iskpar(ielgrp) = 1
!     nlevels        o    Number of entries in array elmlevels
!     viewp          i    Array containing the first view point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iskip, i, icount, jcount, ielem, ielemlow, &
              ielemtop, inode, j, numelm, nnodes, &
              nskipelms, idefstart, ifirst, ilast, jnode
      integer, allocatable :: iwork(:), nodes(:), ipoints(:), ielems(:)
      logical neighfound

!     i              Counting variable
!     icount         Counter
!     idefstart      defines how the start level must be computed
!     ielem          Element number
!     ielemlow       Smallest element number in group
!     ielems         Work array of length nelem indicating if an element
!                    is still free (0), must be skipped (-1) or is already
!                    in use (1)
!     ielemtop       Largest element number in group
!     ifirst         First curve (surface) number read
!     ilast          Last curve (surface) number read
!     inode          Node sequence number
!     ipoints        Work array of length npoint containing a set of nodes
!     iskip          Indicates if element groups must be skipped (>0)
!                    or not (0)
!     iwork          integer work array of length npoint to store a copy
!                    of nodes
!     j              Counting variable
!     jcount         Counter
!     jnode          Node sequence number
!     neighfound     If true a neighbour has been found
!     nnodes         Number of nodes
!     nodes          Work array of length npoint containing a set of nodes
!     nskipelms      Number of elements that must be skipped
!     numelm         Number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPDISTNOD     Find all nodes with the largest distance to a given view
!                    point
!     SEPSKIPELMS    Fill array ielems with a 0 for elements that must not be
!                    skipped and with a 1 for those who must
!     SEPSURNOD      Store all nodes in the surfaces ifirst to ilast in array
!                    nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2205   Not all active elements are connected
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array ielems
!    Define range of elements ielemlow to ielemtop from iskip and iskpar
!    Set ielems(i) = -1 for all elements not in ielemlow to ielemtop
!    Find starting address in array KMESHC
!    Clear array ipoints
!    Mark all points in the range of elements:
!       For ielem := ielemlow (1) ielemtop do
!          For all nodes inode in element ielem do
!             ipoints(inode) := 1
!    Find smallest distance min_dis between nodes in elements
!    eps := 0.5*min_dis
!    Find node i1 (x_1) with largest distance to viewpoint x_0
!    Find all nodes below the plane perpendicular to the line from viewpoint
!    to -viewpoint that goes through x_1 corrected with a distance eps in the
!    direction of x_0
!    This set defines the set of starting nodes
!    nlevels := 0;  neighfound := true
!    While elmlevels(nlevels)<ielemtop-ielemlow+1 and neighfound do
!       nlevels := nlevels+1
!       Find all elements corresponding to set of starting nodes with
!          ielems(i) # 0
!          Set ielems(i) = nlevels for all these elements
!          Store elements in array elements and the number of elements in
!             elmlevels(nlevels) accumulated
!          neighfound := elmlevels(nlevels)-elmlevels(nlevels-1)>0
!       Set ipoints equal to 1 for all nodes in array nodes
!       Find all neighbour nodes of nodes in array nodes with ipoints(node)=0
!       and store in array nodes
!    If elmlevels(nlevels)<ielemtop-ielemlow+1 then
!       give error message (not yet implemented)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplevels' )
      iskip = iinput(1)
      idefstart = iinput(2)

      allocate ( iwork(npoint), nodes(npoint), ipoints(npoint), &
                 ielems(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint+nelem, 'nodes' )
      if ( ierror/=0 ) go to 1000

!     --- Define range of elements ielemlow to ielemtop from iskip and iskpar
!         Set ielems(i) = -1 for all elements not in ielemlow to ielemtop

      call sepskipelms ( iskip, iskpar, ielems, &
                         ielemlow, ielemtop, nskipelms )
      numelm = nelem-nskipelms

!     --- Store set of starting nodes in nodes

      if ( idefstart==1 ) then

!     --- idefstart = 1, find nodes with largest distance

         call sepdistnod ( viewp, ielems, ipoints, nodes, nnodes )

      else

!     --- idefstart = 2, find nodes in surfaces ifirst to ilast

         ifirst = iinput(3)
         ilast = iinput(4)
         nnodes = npoint
         call sepsurnod ( ifirst, ilast, nodes, nnodes )

      end if

!     --- Clear array ipoints

      ipoints = 0

!     --- Set ipoints equal to 1 for all nodes in array nodes

      ipoints(nodes(1:nnodes)) = 1

!     --- nlevels := 0;  neighfound := true

      nlevels = 1
      neighfound = .true.
      icount = 0
      elmlevels(nlevels) = 0

!     --- While elmlevels(nlevels)<nelem-nskipelms and neighfound do

100   if ( neighfound .and. icount<numelm ) then

!     --- new set of elements

         neighfound = .false.

!        --- Find all elements corresponding to set of starting nodes with
!               ielems(i) # 0
!               Set ielems(i) = nlevels for all these elements
!               Store elements in array elements and the number of elements in
!                  elmlevels(nlevels) accumulated
!               neighfound := elmlevels(nlevels)-elmlevels(nlevels-1)>0

         do i = 1, nnodes

            inode = nodes(i)

            do j = kmshd1(inode)+1, kmshd1(inode+1)

               ielem = kmshd2(j)

               if ( ielems(ielem)==0 ) then

!              --- new element found

                  ielems(ielem) = 1
                  icount = icount + 1
                  elements(icount) = ielem
                  neighfound = .true.

               end if

            end do

         end do
         elmlevels(nlevels) = icount

!        --- Copy array nodes into iwork

         iwork(1:nnodes) = nodes(1:nnodes)

!        --- Find all neighbour nodes of nodes in array nodes with
!            ipoints(node)=0 and store in array nodes

         jcount = 0
         do i = 1, nnodes
            inode = iwork(i)
            do j = kmshe1(inode)+1, kmshe1(inode+1)

               jnode = kmshe2(j)

               if ( ipoints(jnode)==0 ) then

!              --- New point found

                  jcount = jcount+1
                  nodes(jcount) = jnode
                  ipoints(jnode) = 1

               end if

            end do

         end do
         nnodes = jcount
         nlevels = nlevels+1
         go to 100

      end if
      nlevels = nlevels-1

      if ( elmlevels(nlevels)<numelm ) then

!     --- error (not yet implemented)

         call errint ( elmlevels(nlevels), 1 )
         call errint ( numelm, 2 )
         call errsub ( 2205, 2, 0, 0 )

      end if

      deallocate ( iwork, nodes, ipoints, ielems, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'seplevels' )

      end
