      subroutine seplevflpoint ( iseqlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmesh part ad in case of levelset functions
!     Part ipointobst
!     Old subroutine mshlevflpoint
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     element
!     node
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iseqlc

!     iseqlc         o    Sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenipointobst

!     lenipointobst  length of array ipointobst
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSHLEVELMARKPT Fill array ipointobst for the first time for the level set
!                    function. All nodes with phi >= 0 are marked
!     PRININ         print 1d integer array
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPFILLKMESHAD Fill array kmesh part d
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPOBSTMARKBT  Fill ipointobst part 4, i.e. the set of boundary nodes of
!                    an obstacle that are only inside obstacle elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2645   The number of degrees of freedom corresponding to the level set
!           function is not equal to 1 in all points.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplevflpoint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplevflpoint'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize kmesh part ad

      call sepfillkmeshad ( iseqlc )

!     --- Activate arrays

      if ( km(iseqkm)%kmesh(18)==0 ) call sepfillkmeshd
      call sepactsolbf2 ( isolut )

      if ( indprfi>0 .or. nunkpi/=1 .or. nphysi/=1 ) then

!     --- Either we have indprfi>0, i.e. number of unknowns per node
!         not constant or nunkpi # 1, nphysi#1, which means not one unknown per
!         node. This can not be a level set function

         call errint ( indprfi, 1 )
         call errint ( nunkpi, 2 )
         call errint ( nphysi, 3 )
         call errsub ( 2645, 3, 0, 0 )

      end if

!     --- Allocate temporary space for array ipointobst

      length = max(2*npoint+5,2*nelem+5,npoint+5)
      if ( debug ) write(irefwr,1) 'length', length

      if ( associated(ipointobst) ) nullify(ipointobst)
      allocate ( ipointobst(length), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, length, 'ipointobst' )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting position of level set function

      ipointobst(1) = 2
      usoltemp => ks(isolut)%sol

!     --- Fill first part of array ipointobst based on level set function

      call mshlevelmarkpt ( ipointobst(2), npoint, usoltemp )
      lenipointobst = 5+ipointobst(2)+ipointobst(4)

      if ( debug ) then
         write(irefwr,1) 'lenipointobst', lenipointobst
         call prinin ( ipointobst, lenipointobst, 'ipointobst' )
      end if  ! ( debug )

!     --- Fill part 4 of ipointobst of nodes on the boundary of the
!         obstacle which are only in elements completely inside
!         the obstacle

      call sepobstmarkbt ( lenipointobst )

      if ( debug ) write(irefwr,1) 'lenipointobst/2', lenipointobst
      call inichkarlen ( lenipointobst, size(ipointobst), 'ipointobst' )
      if ( ierror/=0 ) go to 1000

!     --- Make array ipointobst permanent in ibuffr

      length = lenipointobst
      if ( associated(km(iseqkm)%ipointobst) ) &
         deallocate(km(iseqkm)%ipointobst)
      allocate ( km(iseqkm)%ipointobst(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'ipointobst' )
      if ( ierror/=0 ) go to 1000

      kmeshad(1) = length
      km(iseqkm)%ipointobst = ipointobst(1:length)

      kmeshad(6) = iseqlc

!     --- remove temporary array

      deallocate ( ipointobst, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ipointobst' )

      ipointobst => km(iseqkm)%ipointobst

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seplevflpoint'

      end if  ! ( debug )
      call erclos ( 'seplevflpoint' )

      end
