      subroutine sepobstmarkbt1 ( ipointobstloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill ipointobstloc part 4, i.e. the set of boundary nodes of an obstacle
!     that are only inside obstacle elements
!     Old subroutine mshobstmarkbt1.f90
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipointobstloc(*)

!     ipointobstloc i/o   Array containing information about the nodes in an
!                         obstacle
!                         In this subroutine it contains information for one
!                         obstacle only
!                         See PG Section 24.2 for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, numpntobst, &
              numbndobst, inpelm, ikelmc, &
              ielgrp, iielem, ielem, inodp, numinondobst, istart
      integer, allocatable :: iwork(:), iworkelem(:)
      logical inside

!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     ikelmc         position in kmesh part c
!     inodp          Nodal point number
!     inpelm         number of points per element
!     inside         indicates if a node or element is inside the obstacle
!                    (true) or not (false)
!     istart         First position used at a part
!     iwork          integer work array of length npoint indicating if a
!                    point is inside the obstacle or not
!                    iwork(i) = 0, node i is outside the obstacle
!                    iwork(i) = 1, node i is inside the obstacle
!                    iwork(i) = 2, node i is on boundary of the obstacle
!     iworkelem      integer work array of length nelem indicating if an
!                    element is inside the obstacle or not
!                    iworkelem(i) = 0, element i is not inside the obstacle
!                    iworkelem(i) = 1, element i is completely inside
!                    the obstacle
!     j              Counting variable
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numinondobst   Number of nodal points on the boundary of the obstacle
!                    which are only inside elements in the obstacle
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork positions 1 to npoint
!     Set iwork(i) = 1 for all points i inside the obstacle
!     Set iwork(i) = 2 for all points i on the boundary of the obstacle
!     Clear array iworkelem positions 1 to nelem
!     Set iworkelem(i) = 1 if all points of element in the obstacle
!       (including boundary)
!     numinondobst := 0
!     For all nodes i on the boundary of the obstacle do
!        if ( all elements j containing node i are have iworkelem(j) = 1 ) then
!           numinondobst := numinondobst+1; add node i to part 4
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepobstmarkbt1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepobstmarkbt1'
         call prinin ( ipointobstloc, 4, 'ipointobst_0' )
         istart = 5
         if ( ipointobstloc(1)>0 ) call prinin &
            ( ipointobstloc(istart), ipointobstloc(1), 'ipointobst_1' )
         istart = istart+ipointobstloc(1)
         if ( ipointobstloc(2)>0 ) call prinin &
            ( ipointobstloc(istart), ipointobstloc(2), 'ipointobst_2' )
         istart = istart+ipointobstloc(2)
         if ( ipointobstloc(3)>0 ) call prinin &
            ( ipointobstloc(istart), ipointobstloc(3), 'ipointobst_3' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- allocate work arrays

      allocate ( iwork(npoint), iworkelem(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint+nelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork positions 1 to npoint

      iwork = 0

!     --- Set iwork(i) = 1 for all points i inside the obstacle

      numpntobst = ipointobstloc(1)+ipointobstloc(2)
      iwork(ipointobstloc(5:4+numpntobst)) = 1

!     --- Set iwork(i) = 2 for all points i on the boundary of the obstacle

      numbndobst = ipointobstloc(3)
      iwork(ipointobstloc(5+numpntobst:4+numpntobst+numbndobst)) = 2

!     --- Clear array iworkelem positions 1 to nelem

      iworkelem = 0

!     --- Set iworkelem(i) = 1 if all nodes of element i in the obstacle
!         (including boundary)
!         For ielgrp := 1 (1) nelgrp do

      ikelmc = 0
      ielem = 0

      do ielgrp = 1, nelgrp

!     --- inelem := number of elements in element group ielgrp

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm

!        --- For iielem := 1 (1) inelem

         do iielem = 1, inelem

            ielem = ielem+1
            inside = .true.

            do i = 1, inpelm

               inodp = kmeshc(ikelmc+i)
               if ( iwork(inodp)==0 ) inside = .false.  !node not in obstacle

            end do  ! i = 1, inpelm
            ikelmc = ikelmc + inpelm

            if ( inside ) iworkelem(ielem) = 1

         end do  ! iielem = 1, inelem

      end do  ! ielgrp = 1, nelgrp

!     --- numinondobst := 0

      numinondobst = 0
      istart = numpntobst+numbndobst+4

!     --- For all nodes i on the boundary of the obstacle do
!         if ( all elements j containing node i are have iworkelem(j) = 1 ) then
!            numinondobst := numinondobst+1; add node i to part 4

      do i = 1, numbndobst

         inodp = ipointobstloc(4+i+numpntobst)   ! node on boundary of obstacle
         inside = .true.

         do j = kmshd1(inodp)+1, kmshd1(inodp+1)

            ielem = kmshd2(j)
            if ( iworkelem(ielem)==0 ) inside = .false.  ! node not inside

         end do  ! j = kmshd1(inodp)+1, kmshd1(inodp+1)

         if ( inside ) then

!        --- Node is inside the obstacle

            numinondobst = numinondobst+1
            ipointobstloc(istart+numinondobst) = inodp

         end if  ! ( inside )

      end do  ! i = 1, numpntobst

      ipointobstloc(4) = numinondobst

      deallocate ( iwork, iworkelem, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( ipointobstloc, istart+numinondobst, 'ipointobst' )
         write(irefwr,*) 'End sepobstmarkbt1'

      end if
      call erclos ( 'sepobstmarkbt1' )

      end
