      subroutine sepqu0obst ( iskip, iskpar, iresul, ilevel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check if a mesh must be subdivided into sub elements
!    If necessary perform the subdivision and construct an output sol. array
!    The following tasks are performed:
!    If there are volume elements check whether a surface grid or a volume
!    grid is needed
!    Construct either a volume grid or construct a grid consisting of the
!    outer surface only
!    If required make a new grid consisting of the vertices of this
!    grid only
!    If required subdivide the original grid or the surface grid into
!    triangular or tetrahedral elements based on the
!    presence of the required degree of freedom.
!    Furthermore store the required degree of freedom in an output vector
!    Finally (if required, i.e. ifill=1) fill array ihelp. In ihelp it is
!    stored which new elements (elements ihelp(i)+1 up to ihelp(i+1)
!    originated from the old i'th element group. If properties for group i
!    have been given, for example the porosity, these properties can be used
!    for the new elements utilizing array ihelp
!    Remark: both output meshes consist of one type of elements only!
!    Besides that some arrays are checked (KMESH, KPROB, ISOL)
!    and some arrays are activated (COOR, new grids, new solution array, &
!    old solution array, KPROB parts f/h/p), depending on ichois
!    Warning:  subroutine TOPQU0 uses memory management subroutines
!              This means that all pointers must be recomputed after
!              this subroutine
!    If more types of elements must be used, subroutine PL0013 must be
!    incorporated
!     Old subroutine topqu0obst
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iresul(5), iskip, iskpar(*), ilevel

!     ilevel         i   Defines the level of elements to be plotted
!     iresul         o   Output array of length 5
!                        Contents
!                        1: -
!                        2: npelmw
!                        3: nelemlc
!                        4: if > 0, array kmeshcsub has been allocated
!                        5: if > 0, array kmeshctri has been allocated
!     iskip          i   Indicates if element groups must be skipped (>0) or
!                        not (0)
!     iskpar         i   array of length NELGRP that is only used if ISKIP>0
!                        It indicates if element group ielgrp must be skipped
!                        iskpar(ielgrp) = 0 or not iskpar(ielgrp) = 1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical volume, subtri, subver, vertex, elemskipallc
      integer i, nelext, neleml, i1, i2, npelmw
      integer, allocatable :: tria(:), tricol(:)

!     elemskipallc   if true elemskip is allocated locally
!     i              Counting variable
!     i1             Help parameter to compute start address
!     i2             Help parameter to compute start address
!     neleml         Number of "line"-elements
!     nelext         Number of boundary elements
!     npelmw         Local number of nodes per element
!     subtri         Indicates if the mesh must be subdivided into triangles and
!                    also the solution copied (true) or not
!     subver         Indicates if a "vertex" mesh must be created (true) or not
!     tria           New array KMESHC, where all multiple elements have
!                    been canceled
!     tricol         See subroutine topqu9obst (itriaout)
!     vertex         indicates if only the vertices of the surface elements
!                    have to be filled (true) or all points (false)
!     volume         Indication if the mesh contains volume elements (true) or
!                    not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     SEPFILLELOBS   Fill array elemobst for each element with information of
!                    elements in relation to the obstacle
!     SEPQU7         Compute length of array "KMESHC", check if mesh contains
!                    volumes
!     SEPQU8OBST     Split volumes with obstacles in triangles
!     TOPQU9OBST     Remove double elements in volumes with obstacles
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1382   Value of NPELM is not yet implemented for this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Perform some standard checks
!   Clear array IRESUL
!   Activate standard arrays
!   Check if the mesh contains volume elements
!   If ( volume elements are available and ichois<6 ) then
!      Create mesh of surface elements only
!      If ( vertex mesh must be created ) then
!         Create vertex mesh
!      If ( triangle mesh must be created ) then
!         Create triangle mesh
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepqu0obst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepqu0obst'
         call ertrace
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check arrays KMESH, KPROB, ISOL and compute IPKPRB

      subtri = .false.
      subver = .true.
      elemskipallc = .false.
      vertex = subver .and. .not. subtri
      nelemlc = nelem

!     --- Clear array IRESUL

      iresul = 0

!     --- In the case of 3D it is checked whether the grid contains
!         volume elements

      call sepqu7 ( volume, vertex, neleml )
      if ( ierror/=0 ) go to 1000

!     --- Create array elemobst with information of the status of
!         all elements with respect to the obstacle

      allocate ( elemobst(nelemlc) )
      call sepfillelobs

!     --- Volume elements found, construct a mesh consisting of surface
!         elements only
!         In this case double triangles or quadrilaterals are removed
!         reserve space for array "KMESHC"
!         this array is made semi-permanent

      length = size(kmeshc)
      if ( debug ) &
         write(irefwr,1) 'neleml, nelemlc', neleml, nelemlc
      allocate ( kmeshctmp(length), kmeshctri(length/3), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'kmeshc' )
      if ( ierror/=0 ) go to 1000

      if ( ierror/=0 ) go to 1000

!     --- make reservation for "line-elements" array

      if ( neleml/=0 ) allocate ( kmeshcline(2*neleml) )

      if ( .not. associated(elemskip) ) then

         allocate ( elemskip(nelem) )
         elemskip = 0
         elemskipallc = .true.

      end if

      npelmw = npelm
      call sepqu8obst ( nelemlc, npelmw, nelext, vertex, kmeshcline, &
                        neleml, iskip, iskpar, ilevel, kmeshctri )

      if ( elemskipallc )  deallocate(elemskip)

!     --- remove double elements

      if ( nelemlc+nelext==0 ) go to 900
      allocate ( tria((nelemlc+nelext)*npelmw), tricol(nelemlc+nelext), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, (nelemlc+nelext)*npelmw, 'tria' )
      if ( ierror/=0 ) go to 1000
      call topqu9obst ( kmeshctmp, nelemlc, npelmw, tria, &
                        ntria, npoint, kmeshctri, tricol )
      if ( ierror/=0 ) go to 1000

!     --- Copy array "itriangle" into "KMESHC"

      length = ntria*npelmw
      kmeshctmp(1:length) = tria(1:length)

!     --- If necessary copy boundary elements into "KMESHC"

      if ( nelext>0 ) then
         length = nelext*npelmw
         i1 = 1+npelmw*(nelemlc-nelext)
         i2 = 1+npelmw*ntria
         kmeshctmp(i2:i2+length-1) = kmeshctmp(i1:i1+length-1)
      end if

!     --- Copy array "itriaout" into "kmeshctri"

      kmeshctri(1:ntria) = tricol(1:ntria)

!     --- If necessary copy boundary elements into "kmeshctri"

      if ( nelext>0 ) then
         i1 = 1+nelemlc-nelext
         i2 = 1+ntria
         kmeshctri(i2:i2+nelext-1) = kmeshctri(i1:i1+nelext-1)
         ntria = ntria + nelext
      end if

      nelemlc = ntria

!     --- Create a vertex mesh

      if ( npelmw>4 ) then

!     --- surface mesh must be adapted, not yet implemented

         call errint ( npelmw, 1 )
         call errsub ( 1382, 1, 0, 0 )

      else

!     --- Use surface mesh

         allocate ( kmeshcsub(nelemlc*npelmw) )
         length = nelemlc*npelmw
         kmeshcsub = kmeshctmp(1:length)
         iresul(2) = npelmw
         iresul(5) = nelemlc

      end if

      deallocate ( tria, tricol, kmeshctmp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'tria' )

      if ( iresul(5)==0 ) deallocate ( kmeshctri )

 900  iresul(3) = nelemlc
      iresul(4) = 1

1000  deallocate ( elemobst )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepqu0obst'

      end if  ! ( debug )
      call erclos ( 'sepqu0obst' )

      end subroutine sepqu0obst
