      subroutine sepqu7 ( volume, vertex, neleml )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute length of array consisting of all triangles or quadrilaterals
!   in a 3D mesh. Compute also the length of eventually created "line"
!   elements.
!     Old subroutine topqu7
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     length
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduletypes
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: neleml
      logical volume, vertex

!     neleml  o    Number of "line" elements in mesh
!     vertex  i    indicates if only the vertices of the surface elements
!                  have to be filled (true) or all points (false)
!     volume  o    Indication if the mesh contains volume elements (true) or not
!                  (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ilow, ihigh, ielgrp, npelnw, nsurfe
      logical triang, bound, sevenp, quadrt

!     bound   Indicates if boundary elements are present (true) or not (false)
!             they are not
!     ielgrp  element group number
!     ihigh   Indicates the highest dimensionality in the grid.
!             If ihigh=0 only 2D elements are present, otherwise (1) there also
!             3D elements
!     ilow    Indicates the lowest dimensionality in the grid.
!             If ilow=0 only 3D elements are present, otherwise (1) there also
!             2D elements
!     npelnw  Number of points per element in KELEM
!     nsurfe  Number of surfaces of a three-dimensional element as stored in
!             IBOUN3
!     quadrt  Indicates if quadratic elements have been found (true) or not
!     sevenp  Indicates if elements with 7 points have been found (true) or not
!     triang  Indicates if triangular elements have been found (true) or not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
!    1847   Mesh consists of both triangles and quadrilaterals
!    1848   Mesh consists of both linear and quadratic elements.
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepqu7' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepqu7'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      ilow  = 0
      ihigh = 0

!     --- Determine whether triangles or quadrilaterals are used

      triang = .false.
      quadrt = .false.
      sevenp = .false.
      if ( debug ) &
         write(irefwr,1) 'nelgrp, inelem, kelmf, kelmg', &
                          nelgrp, inelem, kelmf, kelmg

      do ielgrp = 1, nelgrp

         ishape = kmeshf ( ielgrp )
         if ( ishape>50 ) go to 100  ! skip interface elements
         if ( ishape==19 .or. ishape==20 .or. &
              ishape>=23  ) then

!        --- Shape number not available

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )

         end if

         if ( ishape>=3 .and. ishape<10 ) ilow  = 1
         if ( ishape>=10 ) ihigh = 1

         if ( ishape== 3 .or. ishape== 4 .or. ishape== 7 .or. &
              ishape== 8 .or. ishape==11 .or. ishape==12 .or. &
              ishape>=15 .or. ishape==18 .or. &
              ishape>=21 .and. ishape<=22 ) triang = .true.
         if ( ishape== 4 .or. ishape== 6 .or. ishape== 7 .or. &
              ishape== 8 .or. ishape==12 .or. ishape>=14 .and. &
              ishape<=17 .or. &
              ishape>=22 .and. ishape<=2 ) quadrt = .true.
         if ( ishape== 7 .or. ishape==15 .or. ishape==16 .or. &
              ishape==23 ) sevenp = .true.

100   end do
      if ( debug ) then
         write(irefwr,1) 'ilow, ihigh, ishape', ilow, ihigh, ishape
         write(irefwr,*) 'triang, vertex ', triang, vertex
      end if  ! ( debug )

      if ( triang ) then

!     --- Triangles have been found, check if there are no quadrilaterals

         do ielgrp = 1, nelgrp

            ishape = kmeshf ( ielgrp )
            if ( ishape== 5 .or. ishape== 6 .or. ishape==13 .or. &
                 ishape==14 ) then

!           --- Error, mesh consists of triangles and quadrilaterals
!               This possibility has not yet been implemented

               call errint ( ielgrp, 1 )
               call errsub ( 1847, 1, 0, 0 )
               go to 1000

            end if

         end do

      end if

      if ( .not. vertex ) then

!     --- All points should be stored, not only the vertices
!         Check if all elements are of the same type

         if ( quadrt ) then

            do ielgrp = 1, nelgrp

               ishape = kmeshf ( ielgrp )
               if ( ishape== 3 .or. ishape==5 .or. ishape==11 .or. &
                    ishape==13 ) then

!              --- Error, mesh consists of linear and quadratic elementx
!                  This possibility has not yet been implemented

                  call errint ( ielgrp, 1 )
                  call errsub ( 1848, 1, 0, 0 )

               else if ( ishape==8 ) then

!              --- Element shape 8 not yet implemented

                  call errint ( ishape, 1 )
                  call errsub ( 800, 1, 0, 0 )

               end if

            end do

         end if

      end if
      if ( ierror/=0 ) go to 1000

      if ( triang ) then
         if ( .not. vertex ) then
            if ( sevenp ) then
               npelnw = 7
            else if ( quadrt ) then
               npelnw = 6
            else
               npelnw = 3
            end if
         else
            npelnw = 3
         end if
      else if ( quadrt .and. .not. vertex ) then
         npelnw = 9
      else
         npelnw = 4
      end if
      bound = ilow+ihigh==2
      volume = ihigh==1
      length = 0
      neleml = 0
      if ( debug ) write(irefwr,1) 'npelnw', npelnw

!     --- Loop over all element groups

      do ielgrp = 1, nelgrp

         ishape = kmeshf ( ielgrp )
         inelem = Topgrp(ielgrp)%inelem
         if ( debug ) &
            write(irefwr,1) 'ielgrp, ishape, inelem', &
                             ielgrp, ishape, inelem

         if ( ishape>=1 .and. ishape<=2 ) then

!        --- 1D line elements, raise number of line elements

            neleml = neleml + inelem

         else if ( ishape>=3 .and. ishape<=10 ) then

!        --- 2D elements, extend length of array kmeshcnew

            length = length + inelem*npelnw
            if ( bound ) length = length + inelem*npelnw

         else if ( ishape>=11 .and. ishape<=18 ) then

!        --- 3D elements, extend length of array kmeshcnew

            nsurfe = itypbs(2,ishape-10)
            length = length + inelem*npelnw*nsurfe
            if ( debug ) then
               write(irefwr,1) 'ishape, nsurfe, inelem, npelnw', &
                                ishape, nsurfe, inelem, npelnw
               write(irefwr,1) 'nsurfe, length', &
                                nsurfe, length
            end if  ! ( debug )

         else if ( ishape==21 .or. ishape==22 ) then

!        --- prisms, extend length of array kmeshcnew
!            element contains 8 faces

            nsurfe = 8
            length = length + inelem*npelnw*nsurfe

         else if ( ishape>50 ) then

!        --- interface elements, skip

         else if ( ishape>0 ) then

!        --- Shape number not available

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'length', length
         write(irefwr,*) 'End sepqu7'

      end if
      call erclos ( 'sepqu7' )

      end
