      subroutine sepstrucsimple ( lengin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create matrix structure for simple type methods
!     Schur storage
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesimple
      use sepmodulekprob
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: lengin

!     lengin         i    Length of array imatcop (see incommatsimp)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, nq, nd, iend, ibegin, j, npm
      logical :: qsymmetric, dgsymmetric
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     dgsymmetric    If true the pressure matrix is symmetric
!     i              Counting variable
!     ibegin         Starting value for loop counter
!     iend           End value for loop counter
!     j              Counting variable
!     nd             Number of entries in matrix D
!     npm            Size of second part of structure of pressure mass
!                    matrix
!     nq             Number of entries in lower triangular part of Q
!     qsymmetric     If true the velocity matrix is symmetric
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPCOMMATDG    Construct the matrix structure of the matrix DG given the
!                    structure of the matrix D
!     SEPCOMMATPM    Create matrix structure for pressure mass matrix
!     SEPINTMATQ     Transform symmetric compact storage scheme to row-compact
!                    storage. Special application q-matrix for simple
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2621   Simple will be applied but there is no p-last renumbering
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstrucsimple' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepstrucsimple'
         call prinin ( icolsii, 10, 'icolsii' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      kprobac => kprobpart%kprobac

!     --- Check if plast ordering is available

      if ( kprobloc(59)==0 ) then

!     --- Error kprob(59) has not been filled, hence no plast ordering

         call errint ( IM%jmethod, 1 )
         call errsub ( 2621, 1, 0, 0 )
         go to 1000

      end if

      if ( kprobac(1)/=2 ) then

!     --- Error ngroups # 2, hence no plast ordering

         call errint ( IM%jmethod, 1 )
         call errsub ( 2621, 1, 0, 0 )
         go to 1000

      end if

!     --- Get parameters nu and np
!         Store nu and np in intmatnew(7-8)

      nu = kprobac(2)
      np = kprobac(3)-nu
      nq = irowsii(nu+1)

      IM%nu = nu
      IM%np = np

      nrusolpress = np
      nrusolvel = nu
      sizeqmat = nq

      if ( debug ) write(irefwr,1) 'nu, np ', nu, np
      qsymmetric = IM%Qsymmetric
      dgsymmetric = IM%Gsymmetric
      if ( debug ) &
         write(irefwr,*) 'qsymmetric, dgsymmetric ', &
                          qsymmetric, dgsymmetric
      allocate ( IM%rowsQ(nu+1), IM%rowsG(np+1), &
                 IM%colsQ(nq), icolsG(lengin), &
                 icolsPM(lengin), icolsDG(lengin), stat = error )
      if ( error/=0 ) call eralloc ( error, nu+nq+3*np+3*lengin+4, 'irowsQ' )
      if ( ierror/=0 ) go to 1000

!     --- Fill rowsQ
!         Compute nQ and store in intmatnew(4)

      IM%rowsQ = irowsii(1:1+nu)

      if ( debug ) then
         call prinin ( IM%rowsQ, nu+1, 'rowsQ' )
         write(irefwr,*) 'nq ', nq
      end if  ! ( debug )

      IM%colsQ = icolsii(1:nq)
      call sepintmatq

      if ( debug ) call prinin ( IM%colsQ, nq, 'intmat1b' )

!     --- Fill irowsG
!         Compute nD

      nd = 0
      IM%rowsG(1) = 0
      ibegin = irowsii(nu+1)+1

      if ( debug ) &
         write(irefwr,1) 'nu, np, ibegin', nu, np, ibegin

      do i = 1, np
         iend = irowsii(nu+1+i)

         do j = ibegin, iend
            if ( icolsii(j)<=nu ) then

!           --- column number smaller than nu+1

               icolsG(1+nd) = icolsii(j)
               nd = nd+1

            end if

         end do  ! j = ibegin, iend
         ibegin = iend+1
         IM%rowsG(1+i) = nd

      end do  ! i = 1, np

      allocate ( IM%colsG(nd) )
      IM%colsG = icolsG(1:nd)
      deallocate ( icolsG )


      if ( debug ) then
         call prinin ( IM%rowsG, np+1, 'rowsG,' )
         call prinin ( IM%colsG, nd, 'colsG' )
         write(irefwr,1) 'nu, np, nd, nq', nu, np, nd, nq
      end if

!     --- Fill rowsPM and colsPM

      call sepcommatpm ( np, nu, npm )

!     --- Fill rowsDG and colsDG

      call sepcommatdg

!     --- Length of matrix is defined by
!         Standard matrix consisting of Q, D and C
!         i.e. nq + nd + npm (provided D and G are each others transpose)
!         + diagonals of Q and dg (=nu+np)
!         The pressure matrix is supposed to be symmetric
!         Extra matrices: PM, diag(velocity mass matrix),
!         diag(pressure mass matrix)
!         i.e. npm + nu + 2*np

      if ( associated(irowsii) ) deallocate(irowsii)
      if ( associated(icolsii) ) deallocate(icolsii)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepstrucsimple'

      end if  ! ( debug )
      call erclos ( 'sepstrucsimple' )

      end subroutine sepstrucsimple
