      subroutine topbv1period ( iwork, jumps, &
                                factors, istoreleft, lenistore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Nodal points of which the unknowns are connected through periodical
!     boundary conditions (possibly with jump) are marked
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), istoreleft(*), lenistore
      double precision :: jumps(*), factors(*)

!     factors        o    Array containing the factors for periodical boundary
!                         conditions
!     istoreleft     o    Contains all periodical right neighbor unknowns
!                         for all left periodical neighbor unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreleft(1) contains nusol+1
!                         The number of neighbors of unknown i is stored in
!                         istoreleft(i+1)-istoreleft(i)
!                         The second part contains the neighbors
!     iwork         i/o   Semi-permanent work array of length nusol in which
!                         information about essential boundary conditions and
!                         periodical boundary conditions is stored
!                         iwork is filled as follows:
!                         iwork(i) = 0: means unknown i is free
!                         iwork(i)>0: means unknown i is prescribed
!                                       (essential bc)
!                         iwork(i)<0: means unknown i refers to unknown
!                                       iwork(i),
!                         which means that unknown i has the same value,
!                         possibly extended with a jump
!                         Subroutine TOPBV2 has already initialised iwork with
!                         non-negative values
!     jumps          o    Array of size nusol containing the jumps
!                         for the periodical boundary conditions
!     lenistore      i    Estimated length of arrays istoreleft and istoreright
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: nodepairs(:,:)
      logical, save :: first=.true.

!     first          If true it concerns the first call
!     nodepairs      Work array of size 2*npoint in which the pairs of node
!                    numbers corresponding to the periodical boundary conditions
!                    are stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!                    management index mmindex and copy the contents of
!                    iarray to ibuffr
!     TOPBV5         Mark nodal points of which the unknowns are connected
!                    through periodical boundary conditions
!     TOPBV7         Nodal points of which the unknowns are connected through
!                    periodical boundary conditions (possibly with jump)
!                    are marked
!                    New version without the use of connection elements
!     TOPPERADAP     Make new arrays kmesh part e and kmesh part j in case of
!                    periodical boundary conditions
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv1period' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv1period'
         write(irefwr,1) 'iinputkp(52), lenistore', iinputkp(52), lenistore
         write(irefwr,1) 'iinputkp(15)', iinputkp(15)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iinputkp(15)>1 .and. iinputkp(52)>0 ) then

!     --- iinputkp(52)>0, hence periodical boundary conditions without
!         connection elements


         allocate ( nodepairs(2,npoint), stat = error )
         if ( error/=0 ) call eralloc ( error, npoint, 'nodepairs' )
         if ( ierror/=0 ) go to 1000

         if ( debug ) write(irefwr,*) 'before topbv7'
         call topbv7 ( iwork, jumps, factors, istoreleft, &
                       lenistore, nodepairs )

!        --- Store pairs of nodes in kprob part kprobperiodnodes
!            The number of pairs in kprob(62) the mm seq number in kprob(63)

         kprobloc(62) = numpairs   ! number of pairs of coupled nodes
         if ( associated(kp(iseqkp)%kprobsub(iprob)%kprobper) ) &
            deallocate(kp(iseqkp)%kprobsub(iprob)%kprobper)
         allocate ( kp(iseqkp)%kprobsub(iprob)%kprobper(2,numpairs) )
         kprobper => kp(iseqkp)%kprobsub(iprob)%kprobper
         kprobper = nodepairs(:,1:numpairs)
         kprobloc(63) = numpairs

         deallocate ( nodepairs, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nodepairs' )

         if ( .not. first ) then

!        --- First time periodical boundary conditions are found
!            Recompute kmesh parts e and j

            first = .false.  ! prevent doing this more than once
            call topperadap

         end if  ! ( .not. periodical )

      else

!     --- iinputkp(52)=0, hence periodical boundary conditions with
!         connection elements


         call topbv5 ( iwork, jumps, factors, istoreleft, lenistore )

      end if  ! ( iinputkp(52)>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topbv1period'

      end if  ! ( debug )
      call erclos ( 'topbv1period' )

      end
