      subroutine topbv501 ( istoreleft, istoreright )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 18-06-2012 Use subroutines topbv0501x
!        version  3.1  date 09-12-2005 Debug statements
!        version  3.0  date 26-10-2005 Extra parameter iprinp
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill number of right neighbour unknowns in istoreleft and
!     number of left neighbour unknowns in istoreright
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: istoreleft(*), istoreright(*)

!     istoreleft     o    Contains all periodical right neighbour unknowns
!                         for all left periodical neighbour unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreleft(1) contains nusol+1
!                         The number of neighbours of unknown i is stored in
!                         istoreleft(i+1)-istoreleft(i)
!                         The second part contains the neighbours
!                         In this subroutine the first part is stored
!     istoreright    o    Contains all periodical left neighbour unknowns
!                         for all right periodical neighbour unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreright(1) contains nusol+1
!                         The number of neighbours of unknown i is stored in
!                         istoreright(i+1)-istoreright(i)
!                         The second part contains the neighbours
!                         In this subroutine the first part is stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inunkp, ielgrp, itype, iielem, iprinp, &
              ikelmc, iinp, indtvc, istart, &
              idegper(nunkp), i, icount, &
              idegfd, iinp1, inpelm, k, nodp1, nodp2

      double precision rjump, rmult, rmultloc(nunkp), &
                       rjumploc(nunkp)

!     i              Counting variable
!     icount         Counter
!     idegfd         Degree of freedom sequence number
!     idegper        Array of size nunkp containing a 1 if the
!                    corresponding unknown is periodical and a 0 if not
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     iinp           last position used in array iinputkp
!     iinp1          first pos. iinputkp definition curves
!     ikelmc         position in kmesh part c
!     indtvc         Number of vectors of special structure
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in isol1
!     iprinp         Starting address in array rinputkp
!     istart         Starting address
!     itype          Type number of element
!     k              Counting variable
!     nodp1          Node sequence number
!     nodp2          Node sequence number
!     rjump          Jump
!     rjumploc       Array of size nunkp containing the jumps for each
!                    degree of freedom
!     rmult          Multiplication factor for the solution
!     rmultloc       Array of size nunkp containing the factors for
!                    each degree of freedom.
!                    Not used in this subroutine, however, must be declared
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     TOPBV5011      Fill number of right neighbour unknowns in istoreleft and
!                    number of left neighbour unknowns in istoreright
!                    One couple of nodes only
!     TOPBV5012      Accumulate arrays istoreleft and istoreright
!     TOPBV5LOC      Update the arrays idegper, rjumploc and rmultloc with
!                    information about jump and factors as defined by
!                    the bit pattern in idegfd
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1796   Multiplication factor too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear arrays istoreleft and istoreright
!     For all element groups do
!        if ( element is periodical ) then
!           store which unknowns are periodical in idegper
!           For all elements of this group do
!              find left and right node
!              Add positions in istoreleft and istoreright
!     Acumulate arrays istoreleft and istoreright
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv501' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv501'
         call prinin ( iinputkp, 100, 'iinputkp' )
         call prinrl ( rinputkp, 20, 'rinputkp' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      inunkp = nunkp
      iinp1 = intarprb(1)
      iinp = iinp1
      ntype  = iinputkp(1)
      rjump = 0d0
      rmult = 1d0
      iprinp = 0
      ikelmc = 0

!     --- Clear arrays istoreleft and istoreright

      istoreleft(1:nusol) = 0
      istoreright(1:nusol) = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

!     --- Find standard quantities

         itype = kprobc(ielgrp)
         inpelm = Topgrp(ielgrp)%inpelm
         if ( itype>0 ) then
            do k  =  2, ntype
               itype = max (itype,iinputkp(iinp+k-1))
            end do
         end if
         iinp = iinp+ntype
         if ( itype>0 .and. itype<100) then
            inunkp = iinputkp(iinp)
            indtvc = iinputkp(iinp+1)
            nphys  = iinputkp(iinp+2)
            iinp = iinp+3
            if ( inunkp==0 ) iinp = iinp+inpelm
            if ( indtvc>0 ) iinp = iinp+indtvc*inpelm
            if ( nphys>0 ) iinp = iinp+nphys*inpelm
         end if
         inelem = Topgrp(ielgrp)%inelem
         if ( itype/=-1 ) then

!        --- Standard element

            ikelmc = ikelmc+inelem*inpelm

         else

!        --- Element with periodical boundary conditions
!            Store which unknowns are coupled in idegper

            rjump  = rinputkp(iprinp+1)
            rmult  = rinputkp(iprinp+2)
            if ( debug ) then
               write(irefwr,1) 'iprinp', iprinp
               write(irefwr,2) 'rjump, rmult', rjump, rmult
            end if  ! ( debug )
            if ( abs(rmult)<=epsmac ) then

!           --- Error 1796:  multiplication factor too small

               call erreal ( rjump, 1 )
               call errint ( ielgrp, 1 )
               call errsub ( 1796, 1, 1, 0 )

            end if

!           --- Fill arrays idegper with default values

            idegper(1:nunkp) = 0

            if ( iinputkp(15)<=0 ) then

!           --- iinputkp(15)<0 corresponds to the special VIp adaptation
!                              should be removed in the future

               idegfd = 0
               idegper(1:nunkp) = 1

            else if ( iinputkp(15)<=7 ) then

!           --- iinputkp(15)<7 old input file, only one bit pattern

               idegfd = iinputkp(iinp)
               iinp = iinp+1
               if ( idegfd>0 ) then

!              --- Not all degrees of freedom are prescribed

                  call topbv5loc ( idegfd, rjump, rmult, idegper, &
                                   rjumploc, rmultloc, nunkp, ielgrp )

               else

!              --- All degrees of freedom are prescribed

                  idegper(1:nunkp) = 1

               end if

            else

!           --- iinputkp(15)>7 new input file, more bit patterns possible

               icount = iinputkp(iinp)
               iinp = iinp+1
               do i = 1, icount
                  idegfd = iinputkp(iinp)
                  istart = iinputkp(iinp+1)
                  iinp = iinp+2
                  rjump = rinputkp(istart)
                  rmult = rinputkp(istart+1)
                  if ( abs(rmult)<=epsmac ) then

!                 --- Error 1796:  multiplication factor too small

                     call erreal ( rjump, 1 )
                     call errint ( ielgrp, 1 )
                     call errsub ( 1796, 1, 1, 0 )

                  end if
                  if ( idegfd>0 ) then

!                 --- Not all degrees of freedom are prescribed

                     call topbv5loc ( idegfd, rjump, rmult, idegper, &
                                      rjumploc, rmultloc, nunkp, &
                                      ielgrp )

                  else

!                 --- All degrees of freedom are prescribed

                     idegper(1:nunkp) = 1

                  end if

               end do

            end if
            iprinp = iprinp + 2

!           --- Loop over periodical elements

            do iielem = 1, inelem

               nodp1 = kmeshc(ikelmc+1)
               nodp2 = kmeshc(ikelmc+2)
               ikelmc = ikelmc+2

               call topbv5011 ( nodp1, nodp2, indprf, kprobf, idegper, &
                                nunkp, istoreleft, istoreright )

            end do

         end if

      end do

!     --- Acumulate arrays istoreleft and istoreright

      call  topbv5012 ( nusol, istoreleft, istoreright )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iprinp', iprinp
         write(irefwr,*) 'End topbv501'

      end if
      call erclos ( 'topbv501' )

      end
