      subroutine topbv502 ( istoreleft, istoreright, rjumpleft, &
                            rjumpright, rmultleft, rmultright, iwork, itemp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 24-03-2003 debug information
!        version  2.0  date 04-09-2002 Extra parameter nunkp
!        version  1.0  date 14-06-2002
!
!   copyright (c) 2002-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill rest of arrays istoreleft and istoreright
!     Fill jumps in array jumpleft and jumpright and
!     multipication factors in rmultleft and rmultfact
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: istoreleft(*), istoreright(*), iwork(*), itemp(*)
      double precision :: rjumpleft(*), rjumpright(*), &
                          rmultleft(*), rmultright(*)

!     istoreleft    i/o   Contains all periodical right neighbour unknowns
!                         for all left periodical neighbour unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreleft(1) contains nusol+1
!                         The number of neighbours of unknown i is stored in
!                         istoreleft(i+1)-istoreleft(i)
!                         The second part contains the neighbours
!                         At input the first part must have been filled
!     istoreright   i/o   Contains all periodical left neighbour unknowns
!                         for all right periodical neighbour unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreright(1) contains nusol+1
!                         The number of neighbours of unknown i is stored in
!                         istoreright(i+1)-istoreright(i)
!                         The second part contains the neighbours
!                         At input the first part must have been filled
!     itemp         i/o   Temporaray work array of length nusol to indicate
!                         if an unknown corresponds to a "left" periodical
!                         boundary condition (1) or not (0)
!                         At input itemp must have been cleared
!     iwork          i    Semi-permanent work array of length nusol in which
!                         information about essential boundary conditions and
!                         periodical boundary conditions is stored
!                         iwork is filled as follows:
!                         iwork(i) = 0: means unknown i is free
!                         iwork(i) > 0: means unknown i is prescribed
!                                       (essential bc)
!                         iwork(i) < 0: means unknown i refers to unknown
!                                       iwork(i), which means that unknown i
!                                       has the same value, possibly
!                                       extended with a jump
!     rjumpleft      o    Array wih the same length as the second part of
!                         istoreleft containing the jumps w.r.t the right part
!     rjumpright     o    Array wih the same length as the second part of
!                         istoreright containing the jumps w.r.t the left part
!     rmultleft      o    Array wih the same length as the second part of
!                         istoreleft containing the factors w.r.t the right part
!     rmultright     o    Array wih the same length as the second part of
!                         istoreright containing the factors w.r.t the left part
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inunkp, ielgrp, itype, j, iielem, &
              ikelmc, iinp, indtvc, &
              idegper(nunkp), i, icount, &
              idegfd, iinp1, inpelm, k, iprinp, nodp1, nodp2, istart, &
              istartleft(nusol), istartright(nusol)
      double precision rjump, rmult, rmultloc(nunkp), &
                       rjumploc(nunkp)

!     i              Counting variable
!     icount         Counter
!     idegfd         Degree of freedom sequence number
!     idegper        Array of size nunkp containing a 1 if the
!                    corresponding unknown is periodical and a 0 if not
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     iinp           last position used in array iinputkp
!     iinp1          first pos. iinputkp definition curves
!     ikelmc         position in kmesh part c
!     indtvc         Number of vectors of special structure
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in isol1
!     iprinp         Starting address of array rinputkp in IBUFFR
!     istart         First position used at a part
!     istartleft     Array of size nusol that is used to keep track of the last
!                    position used in istoreleft w.r.t. unknown i
!     istartright    Array of size nusol that is used to keep track of the last
!                    position used in istoreright w.r.t. unknown i
!     itype          Type number of element
!     j              Counting variable
!     k              Counting variable
!     nodp1          Node sequence number
!     nodp2          Node sequence number
!     rjump          Jump
!     rjumploc       Array of size nunkp containing the jumps for each
!                    degree of freedom
!     rmult          Multiplication factor for the solution
!     rmultloc       Array of size nunkp containing the factors for
!                    each degree of freedom.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     TOPBV5LOC      Update the arrays idegper, rjumploc and factorloc with
!                    information about jump and factors as defined by
!                    the bit pattern in idegfd
!     TOPBV7021      Fill rest of arrays istoreleft and istoreright
!                    Fill jumps in array jumpleft and jumpright and
!                    multipication factors in rmultleft and rmultfact
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear arrays istoreleft and istoreright
!     For all element groups do
!        if ( element is periodical ) then
!           store which unknowns are periodical in idegper
!           For all elements of this group do
!              find left and right node
!              Add positions in istoreleft and istoreright
!     Acumulate arrays istoreleft and istoreright
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv502' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv502'

      end if

      inunkp = nunkp
      iinp1 = intarprb(1)
      iinp = iinp1
      ntype  = iinputkp(1)
      rjump = 0d0
      rmult = 1d0
      iprinp = 0
      ikelmc = 0

!     --- Copy first part of arrays istoreleft and istoreright into
!         istartleft and istartright

      do i = 1, nusol
         istartleft(i) = istoreleft(i)
         istartright(i) = istoreright(i)
      end do

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

!     --- Find standard quantities

         itype = kprobc(ielgrp)
         inpelm = Topgrp(ielgrp)%inpelm
         if ( itype>0 ) then
            do k  =  2, ntype
               itype = max (itype,iinputkp(iinp+k-1))
            end do
         end if
         iinp = iinp+ntype
         if ( itype>0 .and. itype<100) then
            inunkp = iinputkp(iinp)
            indtvc = iinputkp(iinp+1)
            nphys  = iinputkp(iinp+2)
            iinp = iinp+3
            if ( inunkp==0 ) iinp = iinp+inpelm
            if ( indtvc>0 ) iinp = iinp+indtvc*inpelm
            if ( nphys>0 ) iinp = iinp+nphys*inpelm
         end if
         inelem = Topgrp(ielgrp)%inelem
         if ( itype/=-1 ) then

!        --- Standard element

            ikelmc = ikelmc+inelem*inpelm

         else

!        --- Element with periodical boundary conditions
!            Store which unknowns are coupled in idegper

            rjump  = rinputkp(iprinp+1)
            rmult  = rinputkp(iprinp+2)

!           --- Fill arrays idegper, rjumploc and rmultloc with default values

            do i = 1, nunkp
               idegper(i) = 0
               rjumploc(i) = 0d0
               rmultloc(i) = 1d0
            end do

            if ( iinputkp(15)<=0 ) then

!           --- iinputkp(15) < 0 corresponds to the special VIp adaptation
!                              should be removed in the future

               idegfd = 0
               do i = 1, nunkp
                  idegper(i) = 1
                  rjumploc(i) = rjump
                  rmultloc(i) = rmult
               end do

            else if ( iinputkp(15)<=7 ) then

!           --- iinputkp(15) < 7 old input file, only one bit pattern

               idegfd = iinputkp(iinp)
               iinp = iinp+1
               if ( idegfd>0 ) then

!              --- Not all degrees of freedom are prescribed

                  call topbv5loc ( idegfd, rjump, rmult, idegper, &
                                   rjumploc, rmultloc, nunkp, ielgrp )

               else

!              --- All degrees of freedom are prescribed

                  do j = 1, nunkp
                     idegper(j) = 1
                  end do

               end if

            else

!           --- iinputkp(15) > 7 new input file, more bit patterns possible

               icount = iinputkp(iinp)
               iinp = iinp+1
               do i = 1, icount
                  idegfd = iinputkp(iinp)
                  istart = iinputkp(iinp+1)
                  iinp = iinp+2
                  rjump = rinputkp(istart)
                  rmult = rinputkp(istart+1)
                  if ( idegfd>0 ) then

!                 --- Not all degrees of freedom are prescribed

                     call topbv5loc ( idegfd, rjump, rmult, idegper, &
                                      rjumploc, rmultloc, nunkp, &
                                      ielgrp )

                  else

!                 --- All degrees of freedom are prescribed

                     do j = 1, nunkp
                        idegper(j) = 1
                        rjumploc(j) = rjump
                        rmultloc(j) = rmult
                     end do

                  end if

               end do

            end if
            iprinp = iprinp + 2

!           --- Loop over periodical elements

            do iielem = 1, inelem

               nodp1 = kmeshc(ikelmc+1)
               nodp2 = kmeshc(ikelmc+2)
               ikelmc = ikelmc+2

!              --- Fill arrays for nodp1 and nodp2

               call topbv7021 ( nodp1, nodp2, indprf, kprobf, idegper, &
                                nunkp, istoreleft, istoreright, &
                                rjumpleft, rmultleft, istartleft, &
                                istartright, rjumploc, rmultloc, iwork, &
                                rjumpright, rmultright, itemp, nusol )

            end do

         end if

      end do

      if ( debug ) write(irefwr,*) 'End topbv502'
      call erclos ( 'topbv502' )

      end
