      subroutine topfillkmeshm3 ( npoint, nedges, ncurves, kmeshm3, &
                                  kmeshm2, nodelist, lennodelist, &
                                  edgenodes, nmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 09-09-2005 Extra parameters nodes, edges
!        version  1.0  date 22-07-2003
!
!   copyright (c) 2003-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Fill arrays kmeshm2 and kmeshm3 for inner curves
!      i.e. parts 2 and 3 of kmesh part m
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     intersection
!     curve
!     topology
!     element
!     element
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nedges, ncurves, kmeshm3(*), kmeshm2(*), &
              edgenodes(2,*), nodelist(*), lennodelist, nmeshc(*)

!     edgenodes      i    Array of size 2 x lennodelist containing the edge
!                         numbers connected to the nodes stored in nodelist
!                         in the sequence of nodelist
!                         Only the edges on the boundary are taken into
!                         account, so each node has at most 2 edge numbers
!                         if node with sequence number j has only one edge
!                         connected to it edgenodes(2,j) = 0
!     kmeshm2        o    Part 2 of array kmesh part m
!     kmeshm3        o    Part 3 of array kmesh part m
!     lennodelist    i    Number of nodes in nodelist
!     ncurves        i    Number of curves that must be adapted
!     nedges         i    number of edges
!     nmeshc         i    nmesh part c
!     nodelist       i    Array of size lennodelist containing node numbers
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, icount, inode, j, &
              ifirst, iedge, numnodes, kstart, &
              nodes(npoint), edges(nedges)
      logical repeat, finished

!     edges          Work array of size nedges
!                    if edges(i) = 0, edge i has not yet been used, &
!                    otherwise it has
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     icount         Counter
!     iedge          edge sequence number
!     ifirst         First node on a curve
!     inode          Node sequence number
!     istart         First position used at a part
!     j              Counting variable
!     kstart         First position used at a part
!     nodes          Work array of size npoint
!                    if nodes(i) = 0, node i has not yet been used, &
!                    otherwise it has
!     numnodes       Number of nodal points in array nodes
!     repeat         If true the process has not yet been finished
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         warnings
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2455   Internal error: nodelist exhausted
!    2456   Internal error: both edges already in use
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array nodes(1:npoint)
!     Clear array edges(1:nedges)
!     ncurves := 0; istart := 0; icount := 0
!
!     repeat := true
!     while ( repeat ) do
!        find next free node from nodelist (inode)
!        istart := istart+1; kmeshm3(istart) := inode
!        ifirst := inode; nodes(ifirst) := 1
!        finished := false
!        while ( not finished ) do
!           iedge := first edge corresponding to inode
!           if ( edges(iedge) # 0 ) then
!              iedge := second edge corresponding to inode
!              if ( edges(iedge) # 0 ) then
!                 error: both edges already used
!           edges(iedge) := 1
!           inode := first node on edge iedge
!           if ( nodes(inode) # 0 ) then
!              inode := second node on edge iedge
!              if ( nodes(inode) # 0 ) then
!                 if ( inode = ifirst ) then
!                 else
!                    error: not a closed boundary?
!                 finished := true
!           istart := istart+1; kmeshm3(istart) := inode
!           icount := icount+1
!        icount := icount-1; ncurves := ncurves+1
!        if ( icount>=number of nodes in nodelist ) then
!           repeat := false
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topfillkmeshm3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topfillkmeshm3'
         call prinin ( nodelist, lennodelist, 'nodelist' )
         call prinin1 ( edgenodes, lennodelist, 2, 'edgenodes' )

      end if

!     --- First reorganize nodelist and edgenodes such that all nodes
!         corresponding to one edge only are put in front of these arrays

      icount = 1
      do i = 1, lennodelist

         if ( edgenodes(2,i)==0 ) then

!        --- Node nodelist(i) corresponds to one edge
!            Switch if necessary

            if ( i==icount ) then

!           --- i = icount, no switch necessary
!               raise icount

               icount = icount+1

            else

!           --- i>icount, so switch

               kstart = nodelist(icount)
               nodelist(icount) = nodelist(i)
               nodelist(i) = kstart
               kstart = edgenodes(1,icount)
               edgenodes(1,icount) = edgenodes(1,i)
               edgenodes(1,i) = kstart
               kstart = edgenodes(2,icount)
               edgenodes(2,icount) = edgenodes(2,i)
               edgenodes(2,i) = kstart
               icount = icount+1

            end if  ! ( i==icount )

         end if  ! ( edgenodes(2,i)==0 )

      end do  ! i = 1, lennodelist

!     --- Clear array nodes(1:npoint)
!         Clear array edges(1:nedges)

      do i = 1, lennodelist
         nodes(nodelist(i)) = i
      end do
      do i = 1, nedges
         edges(i) = 0
      end do

!     --- ncurves := 0; istart := 0; icount := 0

      ncurves = 0
      kstart = 0
      icount = 0
      kmeshm2(1) = 1

!     --- repeat := true

      repeat = .true.

!     --- while ( repeat ) do

      do while ( repeat )

!     --- find next free node from nodelist (inode)

         inode = 0
         j = 0
         do while ( inode==0 )

            j = j+1
            if ( j>lennodelist ) then

!           --- Error: nodelist exhausted
!               this can not happen unless there is programming error

               call errsub ( 2455, 0, 0, 0 )
               go to 1000

            end if
            inode = nodelist(j)
            if ( nodes(inode)<0 ) inode = 0

         end do

!        --- istart := istart+1; kmeshm3(istart) := inode
!            ifirst := inode; nodes(ifirst) := 1
!            finished := false

         kstart = kstart+1
         kmeshm3(kstart) = inode
         ifirst = inode
         nodes(ifirst) = -nodes(ifirst)
         finished = .false.

!        --- while ( not finished ) do

         do while ( .not. finished )

!        --- iedge := first edge corresponding to inode

            j = abs(nodes(inode))
            iedge = edgenodes(1,j)
            if ( edges(iedge)/=0 ) then

!           --- edges(iedge) # 0, hence first edge already in use
!               iedge := second edge corresponding to inode

               iedge = edgenodes(2,j)
               if ( iedge==0 ) then

!              --- End of curve found

                  finished = .true.

               else if ( edges(iedge)/=0 ) then

!              --- edges(iedge) # 0, hence second edge also in use
!                  error: both edges already used

                  call errint ( edgenodes(1,j), 1 )
                  call errint ( edgenodes(2,j), 2 )
                  call errint ( inode, 3 )
                  call errwar ( 2456, 3, 0, 0 )
                  finished = .true.
                  go to 100

               end if  ! ( iedge==0 )

100         end if
            if ( .not. finished ) then

!           --- edges(iedge) := 1

               edges(iedge) = 1

!              --- inode := first node on edge iedge

               istart = nmeshc(iedge)
               numnodes = nmeshc(istart)
               inode = nmeshc(istart+1)
               if ( nodes(inode)<0 ) then

!              --- nodes(inode)<0; first node already in use
!                  inode := second node on edge iedge

                  inode = nmeshc(istart+numnodes)
                  if ( nodes(inode)<0 ) then

!                 --- nodes(inode)<0; second node also in use
!                     finished := true

                     finished = .true.

                  end if ! ( nodes(inode)==0 )

               end if  ! ( nodes(inode)/=0 )

            end if  ! ( .not. finished )

!           --- istart := istart+1; kmeshm3(istart) := inode
!               icount := icount+1

            if ( inode/=kmeshm3(kstart) ) then

!           --- Last node not already used

               kstart = kstart+1
               kmeshm3(kstart) = inode
               nodes(inode) = -nodes(inode)

            end if
            icount = icount+1

         end do  ! while ( .not. finished )

!        --- icount := icount-1; ncurves := ncurves+1

         ncurves = ncurves+1
         kmeshm2(ncurves+1) = kstart+1

         if ( icount>=lennodelist ) then

!        --- icount>=number of nodes in nodelist; finished
!            repeat := false

            repeat = .false.

         end if

      end do  ! while ( repeat )

1000  call erclos ( 'topfillkmeshm3' )
      if ( debug ) then

!     --- Debug information

         call prinin ( kmeshm2, ncurves+1, 'kmeshm2' )
         call prinin ( kmeshm3, kstart, 'kmeshm3' )
         write(irefwr,*) 'End topfillkmeshm3'

      end if

      end
