      subroutine toplm7 ( kmeshmp3, npoint, lenkmeshm3, nedges, nmeshh, &
                          nmeshc, ncurvs, kmeshmp2, kmeshm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 06-03-2015 Extra Debug
!        version  1.1  date 07-11-2002 Correction for more curves
!        version  1.0  date 05-08-2002
!
!   copyright (c) 2002-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill boundary of a 2D mesh in kmeshm, where the information has been
!     stored in nmeshc and nmeshh
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     2d
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshmp3(*), npoint, lenkmeshm3, nedges, nmeshh(*), &
              nmeshc(*), ncurvs, kmeshmp2(*), kmeshm(*)

!     kmeshm         o    KMESH part m, contains nodal point numbers of the
!                         curve
!                         In fact array kmeshm is stored in 3 parts which must
!                         be contiguous
!                         Pos. 1 to 4: actual kmeshm, i.e. this array
!                         Pos. 5 ... : actual kmeshm part 2 (kmeshmp2)
!                             The length is computed here
!                         Rest: actual kmeshm part 3 (kmeshmp3)
!                             It starts too far but is copied to the correct
!                             positions in kmeshm
!     kmeshmp2       o    See kmeshm
!     kmeshmp3       o    See kmeshm
!     lenkmeshm3     o    Length of kmeshm part 3
!     ncurvs         i    Number of curves in mesh
!     nedges         i    number of edges
!     nmeshc         i    nmesh part c
!     nmeshh         i    nmesh part h
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(2,npoint), i, iedge, ipstart, nedge_elem, &
              nnode_edge, node1, node2, istart, istart_edge, icount, &
              inext
      logical found

!     found          If true already a possibility has been found
!     i              Counting variable
!     icount         Counter
!     iedge          edge sequence number
!     inext          Next point in curve
!     ipstart        Starting address in array
!     istart         First position used at a part
!     istart_edge    Start edge
!     iwork          integer work array of size 2 x npoint
!                    iwork is used to store edges corresponding to nodes
!                    on the boundary
!                    iwork(.,i) = 0 means node not on the boundary or already
!                                   been treated
!                    iwork(.,i) # 0, contains the edge numbers on the boundary
!                                   corresponding to node i
!     nedge_elem     number of edges corresponding to element
!     nnode_edge     number of nodes corresponding to edge
!     node1          Node of edge
!     node2          Node of edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork (1:2,1:npoint)
!     Fill iwork for all edges that are connected to one element only
!        iwork(1,i) := first edge corresponding to node i
!        iwork(2,i) := Second edge corresponding to node i
!     icount := 0
!     lenkmeshm3 := number of edges in iwork +1
!     While ( icount<lenkmeshm3 ) do
!        istart := smallest non-zero entry in iwork(1,.)
!        istart_edge := iwork(1,istart); iwork(1,istart) := 0
!        kmeshmp3(icount+1) := istart
!        inext := last node on edge istart_edge
!        kmeshmp3(icount+2) := inext
!        iedge := 0
!        icount := icount+2
!        While ( iedge # istart_edge ) do
!           iedge := iwork(1,inext)
!           if ( iedge = 0 ) then
!              iedge := iwork(2,inext)
!           iwork(1,iedge) := 0
!           inext := other node on edge iedge
!           icount := icount+1
!           kmeshmp3(icount) := inext
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toplm7' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'nmeshc: node numbers of edges'
         call prinnmeshx ( nedges, nmeshc )
         write(irefwr,*) 'nmeshh: element numbers corresponding to ', &
                         'edges'
         call prinnmeshx ( nedges, nmeshh )
  1      format ( a, 1x, (10i6) )

      end if

!     --- Clear array iwork (1:2,1:npoint)

      do i = 1, npoint
         iwork(1,i) = 0
         iwork(2,i) = 0
      end do
      lenkmeshm3 = 1
      ncurvs = 0
      kmeshmp2(1) = 1

!     --- Fill iwork for all edges that are connected to one element only
!            iwork(1,i) := first edge corresponding to node i
!            iwork(2,i) := Second edge corresponding to node i

      do iedge = 1, nedges
         ipstart = nmeshh(iedge)
         nedge_elem = nmeshh(ipstart)
         if ( nedge_elem==1 ) then

!        --- face is coupled to one node only

            ipstart = nmeshc(iedge)
            nnode_edge = nmeshc(ipstart)
            node1 = nmeshc(ipstart+1)
            node2 = nmeshc(ipstart+nnode_edge)
            lenkmeshm3 = lenkmeshm3+1
            if ( iwork(1,node1)==0 ) then

!           --- iwork(1,node1) = 0

               iwork(1,node1) = iedge

            else

!           --- iwork(1,node1) # 0

               iwork(2,node1) = iedge

            end if
            if ( iwork(1,node2)==0 ) then

!           --- iwork(1,node2) = 0

               iwork(1,node2) = iedge

            else

!           --- iwork(1,node2) # 0

               iwork(2,node2) = iedge

            end if

         end if

      end do
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lenkmeshm3',lenkmeshm3
         call prinin1(iwork,npoint,2,'iwork')

      end if
      icount = 0
      do while ( icount<lenkmeshm3 )

!     --- istart := smallest non-zero entry in iwork(1,.)

         found = .false.
         istart = 0
         do while ( .not. found .and. istart<npoint )
            istart = istart+1
            if ( iwork(1,istart)/=0 ) found = .true.
         end do
         ncurvs = ncurvs+1
         if ( ncurvs>1 ) lenkmeshm3 = lenkmeshm3+1
         if ( debug ) then

!        --- Debug information

            write(irefwr,1) 'ncurvs', ncurvs

         end if

!        --- istart_edge := iwork(1,istart); iwork(1,istart) := 0
!            kmeshmp3(1) := istart

         istart_edge = iwork(1,istart)
         iwork(1,istart) = 0
         kmeshmp3(icount+1) = istart

!        --- inext := last node on edge istart_edge
!            kmeshmp3(2) := inext

         ipstart = nmeshc(istart_edge)
         nnode_edge = nmeshc(ipstart)
         node1 = nmeshc(ipstart+1)
         node2 = nmeshc(ipstart+nnode_edge)
         if ( iwork(1,node1)==0 ) then

!        --- iwork(1,node1) = 0, hence inext = node2

            inext = node2

         else

!        --- iwork(1,node1) # 0, hence inext = node1

            inext = node1

         end if
         kmeshmp3(icount+2) = inext
         iwork(1,inext) = 0
         if ( debug ) then

!        --- Debug information

            write(irefwr,1) 'node1, node2, istart,istart_edge, inext', &
                             node1, node2, istart,istart_edge, inext

         end if

!        --- iedge := 0
!            icount = icount+2

         iedge = 0
         icount = icount+2

!        --- While ( iedge # istart_edge ) do

         do while ( istart/=inext )

!        --- iedge := iwork(1,inext)
!            iwork(1,inext) := 0
!            inext := other node on edge iedge

            iedge = iwork(1,inext)
            if ( debug ) write(irefwr,1) 'inext, iedge', inext, iedge
            if ( iedge==0 ) then
               iedge = iwork(2,inext)
               iwork(2,inext) = 0
            else
               iwork(1,inext) = 0
            end if

            ipstart = nmeshc(iedge)
            nnode_edge = nmeshc(ipstart)
            node1 = nmeshc(ipstart+1)
            node2 = nmeshc(ipstart+nnode_edge)
            if ( node1==inext ) then

!           --- node1 = inext

               inext = node2

            else

!           --- node2 = inext

               inext = node1

            end if
            if ( iwork(1,inext)==iedge ) then

!           --- edge already in use, clear iwork

               iwork(1,inext) = 0

            else if ( iwork(2,inext)==iedge ) then

!           --- edge already in use, clear iwork

               iwork(2,inext) = 0

            end if
            if ( debug ) then

!           --- Debug information

               write(irefwr,1) 'node1, node2, iedge, inext, icount', &
                                node1, node2, iedge, inext, icount

            end if

!           --- icount := icount+1
!               kmeshmp3(icount) := inext

            icount = icount+1
            kmeshmp3(icount) = inext

         end do  ! while ( istart/=inext )
         kmeshmp2(ncurvs+1) = icount+1

      end do  ! while ( icount<lenkmeshm3 )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icount', icount
         call prinin(kmeshmp3,icount,'kmeshmp3')

      end if
      if ( ncurvs>npoint ) then

!     --- Length of array too small

         call errint ( ncurvs, 1 )
         call errint ( npoint, 2 )
         call errchr ( 'kmeshmp2', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if

      if ( lenkmeshm3>2*npoint ) then

!     --- Length of array too small

         call errint ( lenkmeshm3, 1 )
         call errint ( 2*npoint, 2 )
         call errchr ( 'kmeshmp3', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if

!     --- Move all information to final positions in kmeshm
!         First positions 1 to 4

      kmeshm(1) = 0
      kmeshm(2) = 6+ncurvs
      kmeshm(3) = 6+ncurvs+lenkmeshm3
      kmeshm(4) = 0

!     --- The positions 5 to ncurvs+5 have already been filled in kmeshmp2
!         Now kmeshmp3 must be filled from the first position in kmesmp2

      kmeshmp2(ncurvs+2:ncurvs+1+icount) = kmeshmp3(1:icount)

      call erclos ( 'toplm7' )

      end subroutine toplm7
