      subroutine fncomprs ( ichoice, s, se, eps, detf, matpar, makese )
! ======================================================================
!
!        programmer     Sander Gielen
!        version 1.1    date   01-12-1997 added new functions
!                                         Sander Gielen & Marcel Rutten
!        version 1.0    date   26-08-1996
!
!
!
!   copyright (c) 1996-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     FNCOMPRS :  routine for ELM250. (In)compressibility function.
!     Compute 2nd-Piola-Kirchoff stress from given Green-Lagrange
!     strains for compressibility term. This relationship MUST BE
!     an isotropic function!
!     The determinant of the deformation gradient, can be
!     used in this relationship
! **********************************************************************
!
!                       KEYWORDS
!
!      elasticity_equation
!      material_behaviour
!      solid
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical          makese
      double precision s(6), eps(6), matpar(10), se(6,6), detf
      integer ichoice

!     detf       i    determinant of deformation gradient
!     eps        i    Green-Lagrange strains
!                     eps(i)  : i: components (symmetric!);
!     ichoice    i    ?
!     makese     i    se must be computed (true) or not (false)
!     matpar     i    material parameters
!     s          o    2nd-PK-stresses.
!                     s(j) :    j: component (1=11, 2=22, 3=33, &
!                                             4=12, 5=23, 6=31)
!     se         o    tangential matrix.
!                     se(i,j) : i,j components (symmetric!)
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer          i,j, icomprs
      double precision c(6), dc(6), g, dgdJ, scale1, scale2, &
                       scale3, penal, b
      double precision ddc(6,6)

!     b          exponent of compression function (type 3 and 4)
!     c          right Cauch-Green strain
!     dc         first derivative of c
!     ddc        second derivative of c
!     dgdj       ?
!     g
!     i          counter
!     icomprs    compression function number
!     j          counter
!     penal      penalty factor (1/bulk-modulus)
!     scale1     scale factor
!     scale2     scale factor
!     scale3     scale factor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     INSTOP    Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1027   The required compressibility model has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! Routine to compute the penalty function contribution to the tangential
! stiffness matrix plus the stress vector in a non-linear solid mechanics
! problem:
! General form ( strain energy density ):
!         / J
!   W = - |    p dJ
!         / 1
! with p the hydrostatic pressure and J the determinant of the deformation
! gradient
! The pressure is expressed in terms of a penalty function:
!          1
!   p = - --- g(J)
!         eps
!  with g(J) the penalty function and eps the penalty parameter
!  Possible options for g(J):
!  ichoice = 10: g(J) = J-1
!  ichoice = 20: g(J) = ln(J)
!                        1     2b
!  ichoice = 30: g(J) = --- ( J   - 1 )
!                        2b
!                        1    1    1
!  ichoice = 40: g(J) = --- ( -  - -- )
!                       n-1   J     n
!                                 J
!                             2
!  ichoice = 50: g(J) = 1/2 (J  -1) J
! The corresponding stress vector s and tangential stiffness matix se are:
!        dW     1  1      dIII
!  s  =  -- =  --- - g(J) ---- &
!        dE    eps J       dC
!         2                                                  2
!        d W    1      1  dg   1         dIII dIII   2      d III
!  se =  --- = --- ( [ -- -- - -- g(J) ] ---- ---- + - g(J) ----- )
!          2   eps      2 dJ    3         dC   dC    J         2
!        dE            J       J                             dC
!  with E the Green-Lagrange strain tensor and C the right Cauchy-Green
!  strain tensor. III is the third invariant of C.
! **********************************************************************
!
!
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ddc /36*0d0/
! ======================================================================
!

      icomprs = ichoice/10
      icomprs = icomprs - 10*(icomprs/10)

!     --- material parameters

      penal = 1d0/matpar(6)
      b     = matpar(7)
      goto ( 110, 120, 130, 140, 150 ) icomprs

      call eropen ('fncomprs')
      call errint ( icomprs, 1 )
      call errsub ( 1027, 1, 0, 0 )
      call erclos ('fncomprs')
      call instop

 110  g = detf-1d0
      dgdJ = 1d0
      goto 200

 120  g = log(detf)
      dgdJ = 1d0/detf
      goto 200

 130  if ( b.eq.0d0 ) b = 1d0/6d0
      g = (1d0/(2d0*b)) * ( detf**(2d0*b)-1d0)
      dgdJ = detf**(2d0*b-1d0)
      goto 200

 140  if ( b.eq.0d0 ) b = 10d0
      g = (1d0/(b-1d0)) * ( (1d0/detf)-(1d0/detf**b))
      dgdJ = (1d0/(b-1d0))* (b/(detf**(b+1d0)) - 1d0/(detf*detf))
      goto 200

 150  g = 5d-1*(detf*detf -1d0)*detf
      dgdJ = 5d-1*(3d0*detf*detf -1d0)
      goto 200

!     --- set coefficients according to penalty function type

 200  scale1 = g/(penal*detf)
      scale2 = (2d0*g)/(penal*detf)
      scale3 = (1d0/(penal*detf*detf)) * (dgdJ -g/detf)

!     --- from Green-Lagrange strains (eps) to right Cauchy-Green strains (C)

      if ( jcart.eq.2 ) then
        eps(5) = 0d0
        eps(6) = 0d0
      end if
      do i=1,3
         c(i)   = 2d0*eps(i) + 1d0
         c(i+3) = 2d0*eps(i+3)
      end do

!     --- derivative of IIIc to c:

      dc(1) = c(2)*c(3) - c(5)*c(5)
      dc(2) = c(1)*c(3) - c(6)*c(6)
      dc(3) = c(1)*c(2) - c(4)*c(4)
      dc(4) = c(5)*c(6) - c(3)*c(4)
      dc(5) = c(4)*c(6) - c(1)*c(5)
      dc(6) = c(4)*c(5) - c(2)*c(6)

!     --- and the second derivative:

      if ( makese ) then
         ddc(1,2) =  c(3)
         ddc(1,3) =  c(2)
         ddc(1,5) = -c(5)

         ddc(2,1) =  c(3)
         ddc(2,3) =  c(1)
         ddc(2,6) = -c(6)

         ddc(3,1) =  c(2)
         ddc(3,2) =  c(1)
         ddc(3,4) = -c(4)

         ddc(4,3) = -c(4)
         ddc(4,4) = -c(3)
         ddc(4,5) =  c(6)
         ddc(4,6) =  c(5)

         ddc(5,1) = -c(5)
         ddc(5,4) =  c(6)
         ddc(5,5) = -c(1)
         ddc(5,6) =  c(4)

         ddc(6,2) = -c(6)
         ddc(6,4) =  c(5)
         ddc(6,5) =  c(4)
         ddc(6,6) = -c(2)
      end if

!     --- add stress components

      do i=1,6
         s(i) = s(i) + dc(i)*scale1
      end do

      if ( makese ) then

!     --- add tangential stiffness components

         do i=1,6
            do j=1,6
               se(i,j) = se(i,j) + ddc(i,j)*scale2 + dc(i)*dc(j)*scale3
            end do
         end do
      end if

      end
