      subroutine el0001
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 06-01-2016 Use sepmodulebuild
!        version  5.0  date 12-08-2013 Extra parameter coeffarr
!        version  4.4  date 10-06-2013 Extra debug
!
!   copyright (c) 1985-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   compute element matrix corresponding to a special element
!   itype = -2:   boundary conditions of the type u = constant
!   sigma times the laplace matrix is filled
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ipphi, ipphixi, ippsi, ippsixi, ipdpsidx
      logical linear
      double precision x(27), w(9), xgauss(27), &
                       dphidx(3*9*9), qmat(3*3*9), tang(27)
      save ipphi, ipphixi, ippsi, ippsixi, ipdpsidx

!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     i              Counting variable
!     ipdpsidx       Starting address of array dpsidx
!     ipphi          Starting address of array phi in array
!     ipphixi        Starting address of array phixi in array
!     ippsi          Starting address of psi
!     ippsixi        Starting address of array psixi
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     w              array of length m containing the weights for
!                    integration
!     x              x co-ordinate of point
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3004         Compute integrals of the shape
!                    /_e alpha grad phi_i . grad phi_j dx
!     EL3008         Compute contribution to element stiffness matrix
!                    corresponding to the second derivative part of a
!                    one-dimensional equation
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0001' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0001'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 ) then

!     --- First call in group, initialize some quantities

         call elcomm
         n = inpelm
         m = n
         irule = 1
         linear = ishape==1 .or. ishape==3
         if ( ishape>6 ) then

!        --- Error: shape number not available

            call errint ( ishape, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errsub ( 1771, 4, 0, 0 )

         end if
         if ( linear ) then
            mn = n
            jsecnd = 1
         else
            mn = m*n
            jsecnd = 2
         end if
         allocate ( E%coeffs(m,m) )
         E%coeffs = 0d0
         E%coeffs(:,1) = sigma
         if ( m>2 ) E%coeffs(:,3) = sigma
         if ( m>5 ) E%coeffs(:,6) = sigma
         notvec = 1
         ipphi = 1
         ipphixi = ipphi+m*n
         ippsi = ipphixi+m*n*2
         ippsixi = ippsi+m*n
         ipdpsidx = ippsixi+m*n*2

      end if
      if ( ndim==3 ) iweigh = 31

!     --- Fill basis functions

      call elm800basefn ( coor, x, w, dphidx, xgauss, coeffarr(ipphi), &
                          index1, qmat, tang, ishape, &
                          coeffarr(ipphixi), coeffarr(ippsi), &
                          coeffarr(ipdpsidx), coeffarr(ippsixi) )

      if ( matrix ) then

!     --- Fill Laplacian matrix

         if ( ishape<=2 ) then

!        --- R1

            call el3008 ( E%coeffs, elemmt, w, dphidx, n, n )

         else

!        --- R2

            call el3004 ( E%coeffs, elemmt, w, dphidx, m, n, n )

         end if

      end if

      if ( E%last==1 ) deallocate ( E%coeffs )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( elemmt, n, n, 'elemmt' )
         write(irefwr,*) 'End el0001'

      end if  ! ( debug )
      call erclos ( 'el0001' )

      end
