      subroutine el0801flcoeffs ( ncoef, ishift, iuser, user, &
                                  array, jboun, jwcons, isub, isup )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.2  date 06-02-2014 Extra debug
!        version  1.1  date 28-10-2009 Allow jboun = 6
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize coefficients for elements of type 801 and 907
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     pressure_correction
!     initialization
!     parabolic_equation
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncoef, ishift, iuser(*), jboun, jwcons, isub, isup
      double precision array(*), user(*)

!     array          o    Real work array in which the values of the
!                         coefficients are stored
!                         See el0801
!     ishift        i/o   Shift in coefficient sequence number
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jboun         i/o   Defines the type of boundary condition.
!                         Possible values:
!                         2:  Standard mixed (Robbins) boundary condition
!                         3:  Boundary condition of type 3
!                         4:  Boundary condition of type 4 (not yet available)
!     jwcons        i/o   Indication if the weight function is constant
!                         (1) or not (2)
!     ncoef          o    Number of coefficients in the equation
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, isubcf
      logical funchk

!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              Counting variable
!     isubcf         Under bound for coefficients check
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2863   option coefficient depending on old vector has not yet been
!           implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0801flcoeffs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0801flcoeffs'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ipos = ipos + 1
      ishift = ishift+1
      if ( jboun<=3 ) then
         ncoef = 2
      else if ( jboun==5 ) then
         ncoef = ndim
         icoor = 11
      else if ( jboun==6 ) then
         ncoef = 10
      else
         ncoef = 8
      end if

      if ( debug ) &
         write(irefwr,1) 'jboun, ncoef, icoor', jboun, ncoef, icoor

      do i = 1, ncoef

         call elusr3 ( i, array(1+(i-1)*m), iuser, user, i+ishift )

         if ( itype==907 .and. ind(i)>2002 ) then

!        --- Option not yet implemented

            call errint ( itype, 1 )
            call errint ( ielgrp, 2 )
            call errsub ( 2863, 2, 0, 0 )

         end if  ! ( itype==907 .and. ind(i)>2002 )

      end do  ! i = 1, ncoef

!     --- Test array lengths, number of nodes and number of degrees of freedom

      call prinlg ( 'iuser', ipos-1, user, iuser )
      call prinlg ( 'user', -nuser, user, iuser )

!     --- Compute the parameters jzero and jconsf

      if ( jwcons==1 .and. ind(1)<0 ) then
         jzero = 1
      else
         jzero = 2
      end if

      if ( jwcons==1 .and. ind(2)<0 ) then
         jconsf = 1
      else
         jconsf = 2
      end if

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      isubcf = 1

      do i = isubcf, ncoef
         isub = i

!        --- If ind(i)>0 then paramater i corresponds to a function

         if ( ind(i)>0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i)>0 ) goto 190

      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do
      ind(lengthmst) = 0
      funchk = .false.
      do i = isub, isup
         if ( ind(i)>0 .and. ind(i)<2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j)==ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if ( ind(i)>1000 .and. ind(i)<=2000 ) ind(lengthmst) = 1
         if ( ind(i)>10000 ) funchk = .true.
      end do

      if ( ind(1)==0 ) notmat = 1
      if ( ind(2)==0 .or. jboun==5 ) notvec = 1
      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2

1000  call erclos ( 'el0801flcoeffs' )
      if ( debug ) then

!     --- Debug information

         call prininlarge ( ich, isup, 'ich' )
         call prininlarge ( ind, isup, 'ind' )
         call prinin ( ist, isup, 'ist' )
         write(irefwr,*) 'End el0801flcoeffs'

      end if  ! ( debug )

      end
