      subroutine el0810 ( iuser, user, jelp, itypeeq, jheat, isub, &
                          isup, array, iseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 16-06-2005
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM810 or
!     ELD810 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, itypeeq, jheat, isub, isup, &
              iseq

!     array          o    Real work array in which the values of the
!                         coefficients in the integration points are stored
!                         in the sequence:
!                              1D                  2D            3D
!                         1:   kappa_S
!                         2:   kappa_L
!                         3:   Capacity_S
!                         4:   Capacity_L
!                         5:   Latent_Heat
!                         6:   Melt_Temp
!                         7:   u 1              u 1            u 1
!                         8:                    u 2            u 2
!                         9:                                   u 3
!                        10:   beta
!                        11:   f
!                        12:   rho
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     iseq           i    Sequence number of vector in VECOLD/ISLOLD
!                         In case of a call by ELI810 it is the parameter JDEGFD
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     itypeeq        o    if 0 the standard enthalpy equation is solved
!                         if 1 the equation is restricted to a solid temperature
!                         equation
!     iuser          i    User array in which integer information of the
!                         coefficients is stored (See Standard Problems)
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELM810. Possible vaues:
!                         1:  Subroutine ELP610  (Linear line element)
!                         2:  Subroutine ELP611  (Quadratic line element)
!                         3:  Subroutine ELP620  (Linear triangle)
!                         4:  Subroutine ELP622  (Quadratic triangle)
!                         5:  Subroutine ELP621  (Bilinear quadrilateral)
!                         6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                        11:  Subroutine ELP630  (Linear tetrahedron)
!                        12:  Subroutine ELP631  (Quadratic tetrahedron)
!                        13:  Subroutine ELP632  (Trilinear brick)
!                        14:  Subroutine ELP633  (Triquadratic  brick)
!     jheat          o    Integer parameter indicating if the mass matrix for
!                         the heat equation must be computed
!                         Possible values:
!                         0:  no mass matrix
!                         1:  diagonal mass matrix with constant coefficient
!                         2:  diagonal mass matrix with variable coefficient
!                        11:  full mass matrix with constant coefficient
!                        12:  full mass matrix with variable coefficient
!     user           i    User array in which real information of the
!                         coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons, ncoef, i, j, isubcf, icoef, kchois, ishift, mrule, &
              ichtrn, metup, interpol, iqmat, iqmat1
      logical linear, funchk
      double precision rho, cs, cl, mu

!     cl             Specific heat of the liquid (used in enthalpy method)
!     cs             Specific heat of the solid (used in enthalpy method)
!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              Counting variable
!     ichtrn         Defines the direction of the computation of an integral
!     icoef          Coefficient sequence number
!     interpol       If 0 the standard interpolation is applied, &
!                    if 1 the interpolation is restricted to linear subelements
!     iqmat          Indication whether a transformation has to be performed if
!                    the elements are of a lower dimension than the dimension of
!                    the coordinates.  If 0 no tranformation is performed
!                    (standard case) if 1 then the velocities are mapped to
!                    the dimension of the elment
!     iqmat1         Indication whether the right-hand side has to be
!                    transformed
!     ishift         Shift in coefficient sequence number
!     isubcf         Under bound for coefficients check
!     j              Counting variable
!     jwcons         Indication if the weight function is constant (1)
!                    or not (2)
!     kchois         Indication what type of parameters must be computed
!                    Possible values:
!                    0:  Standard method, the calling subroutine is ELM810
!                        It is supposed that an element matrix must be computed
!                    1:  Subroutine ELD810 calls this subroutine, only
!                        derivatives must be computed
!                    2:  Subroutine ELD810 or ELI810 calls this subroutine, &
!                        a user function defined as first parameter in
!                        IUSER/USER must be filled in elemvc (icheld=9)
!                    3:  Subroutine ELD810 calls this subroutine, &
!                        alpha times the gradient must be stored in ELEMVC
!                        The first 3 parameters of IUSER/USER are used
!     linear         Indicates whether the element is a linear simplex (true)
!                    or not (false)
!     metup          Upwind parameter
!     mrule          Type of integration rule as given by the user
!     mu             relaxation parameter for the Newton enthalpy method
!     ncoef          Number of coefficients in the equation
!     rho            density rho (used in enthalpy method)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     EL0800DER      Compute some parameters related to derivatives or
!                    integrals
!     EL0800IN       Compute the number of integration points for elements of
!                    type 800
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIREPSCAL     Corrects integer value in case of scalars
!     PRINLG         Print and checks length of user arrays
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     In the case of a call by BUILD:
!     1:  Not used
!     2:  Upwind: not used
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         See type 800
!     5:  Type of equation
!         0: standard enthalpy with phase change
!         1: tenperature equation, no phase change
!     6:  kappa_S
!     7:  kappa_L
!     8:  Capacity_S
!     9:  Capacity_L
!    10:  Latent_Heat
!    11:  Melt_Temp
!    12:  u 1       (1D/2D/3D)
!    13:  u 2       (2D/3D)
!    14:  u 3       (3D)
!    15:  beta      (1D/2D/3D)
!    16:  f         (1D/2D/3D)
!    17:  rho cp    (1D/2D/3D)
!    18:  Not yet used (must be 0)
!    19:  Not yet used (must be 0)
!    20:  Not yet used (must be 0)
!    21:  iseqtemp
!    22:  iseqenth
!         It indicates the vector sequence number of the vector to
!     In the case of a call by INTEGR:
!     1:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     2:  Type of co-ordinate system
!         This parameter consists of 2 parts: jcart and jtransform
!         according to jcart + 10*jtransform
!         Possible values:
!         jcart:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!         jtransform: transformation from 3d to 2d
!         0:  standard, no special transformation
!         1:  transformation from 3d to 2d, with z constant
!             first coordinate becomes x, second y
!         2:  transformation from 3d to 2d, with x constant
!             first coordinate becomes y, second z
!         3:  transformation from 3d to 2d, with y constant
!             first coordinate becomes x, second z
!         4:  transformation from 3d to 2d, using cylinder co-ordinates
!             xsi = atan2(x,y), eta = z
!     3:  Not yet used (must be 0)
!     4:  f
!     In the case of a call by DERIV:
!     1:  ISEQ:  Sequence number of vector in VECOLD/ISLOLD from which UOLD and
!         derivatives must be computed.
!         If 0 or 1 the first vector is used
!         If more vectors are needed, these vectors are supposed to be stored
!         sequentially, where the first one is indicated by ISEQ
!     2-5:  Not yet used
!         In the case ICHELD = 21 position 6 contains information about the
!         function f to be filled:
!     6:  Function f
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     264   jdegfd has wrong value
!     341   Icheli has wrong value
!     406   Value of icheld not allowed
!     771   Rho is non-positive
!    1305   Coefficient has wrong value
!    1863   Value of icheli has not yet been implemented for this element shape
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0810' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0810'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( icheld<0 ) then
         ichtrn = -icheld/10
      else
         ichtrn = 0
      end if

!     --- Set a number of defaults

      iweigh = 1
      ideriv = 1
      irule  = 1
      igausp = 0
      iphi   = 0
      jcart  = 1
      icoor  = 1
      ncomp  = 1
      jdiag  = 1
      idim   = 1
      jderiv = 0
      jtrans = 0
      jind   = 1
      kstep  = 1
      interpol = 0
      iqmat = 0
      iqmat1 = 0
      ishift = 0

!     --- Set parameters depending on the shape numbers

      call el0800in ( iuser, kchois, metup, mrule, ncoef, linear, &
                      jelp, ishift, interpol, iqmat, iqmat1, &
                      iseq, jheat, jwcons )

!     --- if derivatives are computed set iseq equal to iseqi1

      if ( icheld>=0 ) iseq = iseqi1

!     --- set some parameters with respect to derivatives

      call el0800der (icheld, iweigh, jderiv, ndimlc, ideriv, &
                      icheli, ix )

      if ( icheld==4 .or. icheld>=7 .and. icheld<=10 .or. &
           icheld==14 .or. icheld>=17 .and. icheld<=20 .or. &
           icheld>=22 .and. icheld<=30 .or. icheld==34 .or. &
           icheld>=37 .and. icheld<=40 .or. icheld>45 ) then

         call errint ( icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0 )

      else if ( icheli>8 .or. ichtrn>ndim ) then

         call errint ( -icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 341, 3, 0, 0 )

      end if
      if ( icheli>=3 .and. icheli<=5 ) then

         ix = icheli-2
         if ( ix>ndim ) then
            call errint ( -icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 341, 3, 0, 0 )
         end if

      end if
      if ( icheli==6 .and. (iseq<=0 .or. iseq>ndim) ) then
         call errint ( iseq, 1 )
         call errint ( ndim, 2 )
         call errsub ( 264, 2, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

      if ( icheli==6 ) ncomp = ndim

      mn = m * n

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficients

      if ( ncoef>0 ) then

         nuser = 0

         isub = 1
         itypeeq = inirepscal ( iuser(ipos) )  ! type of equation
         if ( debug ) write(irefwr,1) 'itypeeq', itypeeq

         if ( itypeeq<0 .or. itypeeq>1 ) then

!        --- error itypeeq has wrong value

            call errint ( 5, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call errint ( itypeeq, 5 )
            call errchr ( 'itypeeq', 1 )
            call errsub ( 1305, 5, 0, 1 )

         end if  ! ( itypeeq<0 .or. itypeeq>1 )

         ipos = ipos + 1
         ishift = ishift+2-isub

         do i = isub, ncoef
            call elusr3 ( i, array(1+(i-1)*m), iuser, user, i+ishift )
            if ( debug ) then
               write(irefwr,1) 'coefficient ', i
               call prinrl ( array(1+(i-1)*m), m, 'coeff' )
            end if  ! ( debug )
         end do

!        --- Set coefficient of mass matrix to mu for itype = 810

         if ( icheld==0 ) then

!        --- itype = 810, enthalpy equation by newton method
!            The density rho (coeff 12/17) is overwritten by the constant mu

            ipos = ipos+3   ! coefficients 18-20 are skipped
            iseqtemp = iuser(ipos)   ! sequence number of temperature
            iseqenth = iuser(ipos+1) ! sequence number of enthalpy
            if ( debug ) &
               write(irefwr,1) 'iseqtemp, iseqenth', &
                                iseqtemp, iseqenth
            ind(12) = -1
            rho = array(1+11*m)
            cs  = array(1+2*m)
            if ( debug ) &
               write(irefwr,2) 'rho, cs', rho, cs
            if ( itypeeq==1 ) then

!           --- Heat equation, constant coefficients

               cl = cs
               mu = rho*cl

            else

!           --- Enthalpy equation

               cl  = array(1+3*m)
               mu = rho * min(cs, cl)

            end if  ! ( itypeeq==1 )

            do i = 1, m
               array(i+11*m) = mu
            end do

         end if

         if ( ierror/=0 ) go to 1000
         if ( icheld>=41 ) then

!        --- itype = 810, enthalpy equation by newton method

            ipos = ipos+3   ! coefficients 18-20 are skipped
            iseqtemp = iuser(ipos)   ! sequence number of temperature
            iseqenth = iuser(ipos+1) ! sequence number of enthalpy
            ipos = ipos+2

         end if  ! ( itype==810 )

!        --- Test array lengths, number of nodes and number of degrees
!            of freedom

         call prinlg ( 'iuser', ipos-1, user, iuser )
         call prinlg ( 'user', -nuser, user, iuser )
         if ( icheld>=41 ) then

!        --- icheld>=41, We do not use coefficients 1 to 11

            do i = 1, 11
               ind(i) = 0
            end do  ! i = 1, 11

         end if  ! ( icheld==41 )

      end if  ! ( ncoef>0 )
      if ( ierror/=0 ) go to 1000

      if ( icheld==44 ) then

!     --- Special case, itype 810, icheld = 44, we use isolprec

         iseqtemp = 1
         iseqenth = 2

      end if  ! ( icheld==44 )

      if ( kchois==0 .and. vector .or. kchois==2 ) then

!     --- Vector = true, the element vector must be computed
!         or kchois = 2, the first parameter must be computed
!         Compute jconsf

         if ( kchois==0 ) then

            icoef = 11

         else

            icoef = 1

         end if
         if ( ind(icoef)==0 ) then

!        --- F = 0, no right-hand side

            notvec = 0

         else if ( ind(icoef)<0 .or. ind(icoef)==2002 ) then

!        --- F is constant (per element)

            jconsf = jwcons

         else

!        --- General

            jconsf = 2

         end if

      end if

      if ( jheat>0 ) then

!     --- jheat>0, i.e. compute mass matrix

         if ( ind(12)<0 .and. array(12*m)<=0 ) then

!        --- Error rho non-positive

            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 771, 3, 0, 0 )

         else if ( ind(12)==0 ) then

!        --- Warning, rho = 0.
!            rho is made equal to 1

            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errwar ( 771, 3, 0, 0 )
            ind(12) = -1
            do i = 1, n
               array(12*m-1+i) = 1d0
            end do
            jheat = jwcons + 10*(jheat-1)
         else if ( ind(12)<0 .or. ind(12)==2002 ) then
            jheat = jwcons + 10*(jheat-1)
         else
            jheat = 2 + 10*(jheat-1)
         end if

      end if
      if ( ierror/=0 ) go to 1000

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      if ( matrix .and. kchois==0 .or. kchois>=2 ) then
         isubcf = isub
      else
         isubcf = 11
      end if

      do i = isubcf , ncoef
         isub = i

!        --- If ind(i)>0 then paramater i corresponds to a function

         if ( ind(i)>0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i)>0 ) goto 190

      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do
      ind(lengthmst) = 0
      funchk = .false.
      do i = isub, isup
         if ( ind(i)>0 .and. ind(i)<2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j)==ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if ( ind(i)>1000 .and. ind(i)<=2000 ) ind(lengthmst) = 1
         if ( ind(i)>10000 ) funchk = .true.
      end do

      if ( icheli>1 .and. icheli/=7 .or. icheld>0 .and. &
           icheld/=21 ) ind(lengthmst) = 1
      if ( icheld==0 ) then

!     --- Matrix, get temperature

         ind(lengthmst) = 1

      else if ( icheld==41 ) then

!     --- Special case of enthalpy computation from temperature (type 810)

         ind(lengthmst) = 1

      else if ( icheld>=42 .and. icheld<=45 ) then

!     --- Special case of right-hand side for enthalpy computation
!         using newton (type 810)

         ind(lengthmst) = 2
         iphi = 1
         iweigh = 1

      end if  ! ( icheld==41 )

      if ( icheld==0 .and. &
           (mod(metup,20)>=7 .and. mod(metup,20)<=11) ) then
         jsol = 1
         jderiv = 1
      end if
      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2

      if ( jelp==4 .and. jcart==1 .and. metup==0 .and. &
           icheld<=0 .and. irule==1 ) kstep = 2

      if ( ichtrn>0 ) then

!     --- ichtrn>0, special integral

         if ( jelp/=1 ) then
            call errint ( -icheld, 1 )
            call errint ( jelp, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 1863, 5, 0, 0 )
         else
            jtrans = jtrans + 100*ichtrn
         end if

      end if
      if (iqmat1==1) iweigh = iweigh + 20
      if ( igausp==1 .and. mod(iweigh,10)==0 ) iweigh = iweigh+1

1000  irule = irule + 100*interpol

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0810'

      end if
      call erclos ( 'el0810' )

      end
