      subroutine el0813m ( iuser, user, jelp, kfirst, jheat, isub, &
                           isup, array )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 20-01-2012
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM813
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, kfirst, jheat, isub, isup

!     array          o    Real work array in which the values of the
!                         coefficients in the integration points are stored
!                         in the sequence:
!                              1D                  2D            3D
!                         1:   alpha 11         alpha 11       alpha 11
!                         2:                    alpha 12       alpha 12
!                         3:                    alpha 22       alpha 13
!                         4:                                   alpha 22
!                         5:                                   alpha 23
!                         6:                                   alpha 33
!                         7:   u 1              u 1            u 1
!                         8:                    u 2            u 2
!                         9:                                   u 3
!                        10:   beta             beta           beta
!                        11:   f                f              f
!                        12:   rho cp           rho cp         rho cp
!                        13:   gamma_1          gamma_1        gamma_1
!                        14:                    gamma_2        gamma_2
!                        15:                                   gamma_3
!                        16:   g_extra          g_extra        g_extra
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    User array in which integer information of the
!                         coefficients is stored (See Standard Problems)
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELM800. Possible vaues:
!                         1:  Subroutine ELP610  (Linear line element)
!                         2:  Subroutine ELP611  (Quadratic line element)
!                         3:  Subroutine ELP620  (Linear triangle)
!                         4:  Subroutine ELP622  (Quadratic triangle)
!                         5:  Subroutine ELP621  (Bilinear quadrilateral)
!                         6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                        11:  Subroutine ELP630  (Linear tetrahedron)
!                        12:  Subroutine ELP631  (Quadratic tetrahedron)
!                        13:  Subroutine ELP632  (Trilinear brick)
!                        14:  Subroutine ELP633  (Triquadratic  brick)
!     jheat          o    Integer parameter indicating if the mass matrix for
!                         the heat equation must be computed
!                         Possible values:
!                         0:  no mass matrix
!                         1:  diagonal mass matrix with constant coefficient
!                         2:  diagonal mass matrix with variable coefficient
!                        11:  full mass matrix with constant coefficient
!                        12:  full mass matrix with variable coefficient
!     kfirst         o    This parameter indicates the smallest velocity
!                         component that is not equal to zero in the convective
!                         terms
!                         For example kfirst = 2, means that the y- &
!                         component of the velocity is unequal to zero
!     user           i    User array in which real information of the
!                         coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons, ncoef, ishift, mrule, &
              metup, interpol, iqmat, iqmat1
      logical linear

!     interpol       If 0 the standard interpolation is applied, &
!                    if 1 the interpolation is restricted to linear subelements
!     iqmat          Indication whether a transformation has to be performed if
!                    the elements are of a lower dimension than the dimension of
!                    the coordinates.  If 0 no tranformation is performed
!                    (standard case) if 1 then the velocities are mapped to
!                    the dimension of the elment
!     iqmat1         Indication whether the right-hand side has to be
!                    transformed
!     ishift         Shift in coefficient sequence number
!     jwcons         Indication if the weight function is constant (1)
!                    or not (2)
!     linear         Indicates whether the element is a linear simplex (true)
!                    or not (false)
!     metup          Upwind parameter
!     mrule          Type of integration rule as given by the user
!     ncoef          Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0800BETA     Compute jzero corresponding to zeroth order part
!     EL0800FILCOF   Fill all real coefficients in array
!     EL0800INMAT    Compute the number of integration points for this element
!                    group using array iuser
!     EL0800JHEAT    Compute the parameter jheat
!                    ishift and ipos are adapted
!     EL0800SUBP     Compute parameter metup defining the type of upwinding
!     EL0800VEC      Compute parameter jconsf corresponding to rhs vector
!     EL0813LIN      Fill information concerning coefficient 5
!                    raise ipos, ishift
!     EL1002         Compute JSECND for diffusion terms
!     EL1013         Compute KFIRST and IDIM for convection terms
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     See subroutine elm813
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0813m' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0813m'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set parameters depending on the shape numbers

      call el0800inmat ( iuser, metup, mrule, linear, &
                         jelp, ishift, interpol, iqmat, iqmat1, &
                         jheat, jwcons )
      ncoef = 15
      if ( ierror/=0 ) go to 1000

      mn = m * n

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficients

      nuser = 0
      isub = 1

!     --- Fill information concerning coefficient 5 (raise ipos, ishift)

      call el0813lin ( iuser, isub, iseqtemp, ishift )

!     --- Fill all real coefficients

      call el0800filcof ( iuser, user, isub, array, ishift, ncoef )
      if ( ierror/=0 ) go to 1000

!     --- Next consider all terms carefully in order to get the cheapest
!         computation

      if ( matrix .or. metup>0 ) then

!     --- matrix = true  The element matrix must be computed.
!         or metupw>0:  Upwinding
!                         Compute jsecnd, idim and jzero
!         First consider the diffusion tensor:

         call el1002 ( 0, 5, jwcons, jelp, iqmat, array, ndim )

!        --- Next consider the Convective terms:
!            compute idim and first component idim

         call el1013 ( 6, 5, kfirst, jelp, iqmat, ndim )

!        --- Constant part of the equation:
!            compute jzero

         call el0800beta ( kfirst, jwcons )

      end if  ! ( matrix .or. metup>0 )
      if ( ierror/=0 ) go to 1000

!     --- Compute jconsf

      if ( vector ) call el0800vec ( jwcons, 0 )

      if ( jheat>0 ) call el0800jheat ( array, jheat, jelp, jwcons )
      if ( ierror/=0 ) go to 1000

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup
!         adapt kstep and iweigh if necessary

      ind(lengthmst) = 0
      if ( mod(metup,20)>=7 .and. mod(metup,20)<=11 ) ind(lengthmst) = 1

!     --- Compute parameters isub and isup defining the lowest and largest
!         coefficient depending on space
!         Adapt some paramaters

      if ( .not. matrix ) isub = 11
      call el0800subp ( isub, isup, ncoef, jelp, iqmat1, metup )

1000  irule = irule + 100*interpol

      call erclos ( 'el0813m' )
      if ( debug ) then

!     --- Debug information

         call prinin ( ind, ncoef, 'ind' )
         call prinin ( ist, ncoef, 'ist' )
         write(irefwr,1) 'isub, isup, igausp, m, n, iweigh, iphi', &
                          isub, isup, igausp, m, n, iweigh, iphi
         write(irefwr,1) 'irule, metup', irule, metup
         write(irefwr,1) 'jsecnd', jsecnd
         write(irefwr,*) 'End el0813m'

      end if  ! ( debug )

      end
