      subroutine el0816 ( iuser, user, jelp, isub, isup, array )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM816
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elliptic_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: user(*)
      double precision, intent(out) :: array(*)
      integer, intent(in) :: iuser(*)
      integer, intent(out) :: jelp, isub, isup

!     array          o    Real work array in which the values of the
!                         coefficients in the integration points are stored
!                         in the sequence:
!                         1:   u
!                         2:   w
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    User array in which integer information of the
!                         coefficients is stored (See Standard Problems)
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELM816. Possible vaues:
!                         1:  Subroutine ELP610  (Linear line element)
!                         2:  Subroutine ELP611  (Quadratic line element)
!                         3:  Subroutine ELP620  (Linear triangle)
!                         4:  Subroutine ELP622  (Quadratic triangle)
!                         5:  Subroutine ELP621  (Bilinear quadrilateral)
!                         6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                        11:  Subroutine ELP630  (Linear tetrahedron)
!                        12:  Subroutine ELP631  (Quadratic tetrahedron)
!                        13:  Subroutine ELP632  (Trilinear brick)
!                        14:  Subroutine ELP633  (Triquadratic  brick)
!     user           i    User array in which real information of the
!                         coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: jwcons, ncoef, ishift, mrule, interpol
      logical :: linear

!     interpol       dummy
!     ishift         Shift in coefficient sequence number
!     jwcons         Indication if the weight function is constant (1)
!                    or not (2)
!     linear         Indicates whether the element is a linear simplex (true)
!                    or not (false)
!     mrule          Type of integration rule as given by the user
!     ncoef          Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0800ELP      Set parameters depending on shape of element
!     EL0800FILCOF   Fill all real coefficients in array
!     EL0800INM      Compute the parameter m for an element
!     EL0800RULE     Fill the parameters irule and jcart from array iuser
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERROR42        Print error message 42
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     See subroutine elm816
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0816' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0816'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set parameters depending on the shape numbers

!         --- Set defaults

      call elcomm

!     --- Set parameters depending on the shape numbers

      call el0800elp ( linear, jelp )
      if ( debug ) write(irefwr,1) 'n, m, jelp', n, m, jelp

!     --- Get information of first 4 coefficients
!         Fill the arrays IND, MST and ARRAY with information of the first
!         coefficient

      ipos = iuser ( ielgrp + 5 )
      ishift = 0
      if ( ipos<7 ) call error42 ( ielgrp, ipos, ielem, itype )

      ipos = ipos+2       ! The first 2 parameters are not used
      ishift = ishift+2

!     --- Compute the parameters jcart and irule

      call el0800rule ( 0, interpol, ishift, iuser, mrule )
      if ( debug ) write(irefwr,1) 'm, mrule', m, mrule

      if ( irule/=1 ) then

!     --- irule>1, Gauss rule

         call el0800inm ( m, jelp, jwcons, irule, igausp, jdiag, &
                          iphi )

      end if
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,1) 'm, irule', m, irule

      ncoef = 2
      mn = m * n

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficients

      nuser = 0
      isub = 1

      ipos = ipos+1       ! The fifth parameter is not used
      ishift = ishift+1

!     --- Fill all real coefficients

      call el0800filcof ( iuser, user, isub, array, ishift, ncoef )
      if ( ierror/=0 ) go to 1000

      if ( ierror/=0 ) go to 1000

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup
!         adapt kstep and iweigh if necessary

      isub = 1
      isup = 2
      if ( jelp==4 .and. jcart==1 .and. irule==1 ) kstep = 2
      if ( igausp==1 .and. mod(iweigh,10)==0 ) iweigh = iweigh+1

1000  call erclos ( 'el0816' )
      if ( debug ) then

!     --- Debug information

         call prininlarge ( ind, ncoef, 'ind' )
         call prinin ( ist, ncoef, 'ist' )
         call prinin ( ivecnums, ncoef, 'ivecnums' )
         call prinrl1 ( array, ncoef, m, 'array' )
         write(irefwr,1) 'isub, isup, igausp, m, n, iweigh, iphi', &
                          isub, isup, igausp, m, n, iweigh, iphi
         write(irefwr,1) 'irule', irule
         write(irefwr,*) 'End el0816'

      end if  ! ( debug )

      end
