      subroutine el2011g ( acof, elemvc, phi, rho, w, u, dudx, xgauss, &
                           elvcsub, goertler, kappa )
! ======================================================================
!
!        programmer    Guus Segal/ Ralf Biertuempfel
!        version  2.0  date 27-02-2010 Change of parameter list
!        version  1.0  date 28-09-1998
!
!   copyright (c) 1998-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the convective part of the element vector for navier-stokes
!     and add in consisting element vector for a 3-d element
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision acof, rho(m), w(m), elemvc(ndimlc*n), elvcsub(n), &
                       u(m,*), dudx(m,ndimlc,*), phi(n,m), &
                       xgauss(m,ndimlc), goertler, kappa

!     acof      i    Multiplication factor Adams-Bashfort
!     dudx      i    Real m x ndimlc  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                      j     k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     elemvc    i    Element vector
!     elvcsub        Work array to store a sub element vector
!     goertler  i    Goertler number
!     kappa     i    dimensionless wall curvature
!     phi       i    array of lenght n * m, values of shape
!                    functions in integration points
!     rho       i    density rho
!     u         i    array of length m x ncomp in which the values of
!                    u in the integration points are stored the sequence:
!                    u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                     i   j
!     w         i    array of length m containing the weights for integration
!     xgauss    i    array of length m x ndim containing the co-ordinates of
!                    the gauss points.  array xgauss is only filled when gauss
!                    integration is applied.
!                    xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                      i                    i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision rhow(m), uconv(m), work(m)
      integer i, j, k

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     rhow           Contains rho w
!     uconv          Work array to store coefficient (convection)
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3003  Compute integrals of the shape / f(x) phi (x) dx
!                                            e         i
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the convective vector results in a 3x1 block vector, each
!     consisting of nx1 vectors of the following shape:
!            |   F    |
!            |    u   |
!            |        |
!            |   F    |
!     F =    |    v   |
!            |        |
!            |   f    |
!            |    w   |
!    The sub vectors are defined by:
!    Fu(i) = rho ( u du/dx + v du/dy + w du/dz ) phi (i)
!    Fv(i) = rho ( u dv/dx + v dv/dy + w dv/dz  + u u) phi (i)
!    Fw(i) = rho ( u dw/dx + v dw/dy + w dw/dz ) phi (i)
!    For the expression in cylinder co-ordinates see subroutine el2010
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- multiply the weight by rho and store in work(k,1)

      rhow = w * rho

      jconsf = 2
      jadd = 0
      if ( jcart<=2 ) then

!     --- jcart = 1, Cartesian co-ordinates
!         or jcart = 2, Axi-symmetrical co-ordinates without swirl

         do i = 1, ndimlc

!        --- Put u dui/dx + v dui/dy in uconv

            uconv(1:m) = u(1:m,1) * dudx(1:m,1,i) &
                       + u(1:m,2) * dudx(1:m,2,i)

            if ( ndimlc==3 ) then

!           --- Put third part in work(k,2)

               do k = 1, m
                  uconv(k) = uconv(k) + u(k,3) * dudx(k,3,i)

                  if (i==2) then

!                 --- put goertler part in work(k,2)

                     uconv(k) = uconv(k) - &
                                 goertler * kappa * (u(k,1)**2)
                  end if
               end do
            end if

!           --- Compute contribution to one element vector and store in elvcsub

            call el3003 ( uconv, elvcsub, rhow, phi, m, n )

!           --- Add contribution to element vector

            do j = 1, n
               elemvc((j-1)*ndimlc+i) = elemvc((j-1)*ndimlc+i) + &
                                        acof*elvcsub(j)
            end do

         end do

      else

!     --- jcart = 3, Polar co-ordinates
!         Put u du/dr + v/r du/dy - v**2/r in work(k,2)

         do k = 1, m
            uconv(k) = u(k,1)*dudx(k,1,1) + &
                        (u(k,2)*dudx(k,2,1)-u(k,2)**2)/xgauss(k,1)
         end do

!        --- Compute contribution to one element vector and store in elvcsub

         call el3003 ( uconv, elvcsub, work, phi, m, n )

!        --- Add contribution to element vector

         do j = 1, n
            elemvc(2*j-1) = elemvc(2*j-1) + acof*elvcsub(j)
         end do

!        --- Put u dv/dr + v/r dv/dy + u*v/r in work(k,2)

         do k = 1, m
            uconv(k) = u(k,1)*dudx(k,1,2) + &
                        (u(k,2)*dudx(k,2,2)+u(k,1)*u(k,2))/xgauss(k,1)
         end do

!        --- Compute contribution to one element vector and store in elvcsub

         call el3003 ( uconv, elvcsub, work, phi, m, n )

!        --- Add contribution to element vector

         do j = 1, n
            elemvc(2*j) = elemvc(2*j) + acof*elvcsub(j)
         end do

      end if

      end
