      subroutine el3000 ( u, dudx, phi, dphidx, trans, ugauss, mdiv, &
                          rhsdivtr, lstep, lsub, lsup )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 23-08-2006 Extra parameters
!        version  2.0  date 18-10-1998 Extra parameters
!        version  1.0  date 13-02-1994 Subroutine replaces part of el3007
!
!   copyright (c) 1994-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill part of solution due to transformation. It has been assumed that
!    the solution in the other points has been filled by el2005.
!    Also the derivatives are filled
!     EL3000 is called by several types of element subroutines
!            It must be preceded by ELP subroutines of the 6-series
! **********************************************************************
!
!                       KEYWORDS
!
!     extract
!     extract_unknown
!     fill
!     fill_solution
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision  u(n,ncomp), phi(n,m), &
                        dudx(m,ndimlc,ncomp), dphidx(n,m,ndimlc), &
                        ugauss(m,ncomp), rhsdivtr(*), &
                        trans(ncomp*(n-jtrans),ncomp*ncomp)
      integer mdiv, lstep, lsub, lsup

!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dudx           o    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     lstep          o    Step variable for vertices. Usually lstep = 1, &
!                         but when the solution must be computed in the vertices
!                         and the element is quadratic then lstep = 2
!     lsub           o    Under index for the direction of the derivative to be
!                         computed
!     lsup           o    Upper index for the direction of the derivative to be
!                         computed
!     mdiv           i    Defines if divergence right-hand side is used (1)
!                         or not (0)
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     rhsdivtr       i    Contains values of transformed divergence right-hand
!                         side (if mdiv=1)
!     trans          i    Array of length n-jtrans x ncomp**2 to store the
!                         transformation matrix with which the n - jtrans to n
!                         components of the vector u must be computed. Is only
!                         used if jtrans>0
!     u             i/o   Real n x ncomp array in which the values of the
!                         solution vector in the nodal points of the element
!                         are stored in the sequence:
!                         u  (x ) = u(j,i)  j = 1(1)n, i = 1 (1) ncomp
!                          i   j
!                         Array u is always filled
!     ugauss         o    array of length m x ncomp in which the values of
!                         u in the integration points are stored the sequence:
!                         u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                          i   j
!                         ugauss is only filled if jdiag>1 in common celint
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, kderiv

!     i              Counting variable
!     j              Counting variable
!     kderiv
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ELFILLDERIV    Fill the derivatives of the old solution in nodal points
!     ELFILLUGAUSS   Fill the old solution in Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      lsub=1

!     --- first set some constants

      kderiv = abs(jderiv)
      if ( kderiv==1 .or. kderiv==5 ) then

!     --- kderiv = 1 or 5:  The gradient must be computed
!                           Components 1 to ndim

         lsub=1
         lsup=ndimlc

      else if ( kderiv>1 ) then

!     --- kderiv 2,3,4, 6, 7 or 8:  Component kderiv-1 must be computed

         lsub=kderiv-1
         if ( kderiv>5 ) lsub = lsub-4
         lsup=lsub

      end if

      if ( kderiv>1 .and. kderiv<6 .and. &
           ndimlc==2 .and. n>=6 ) then

!     --- kderiv>1, hence computation in vertices
!                     Furthermore quadratic 2D element
!                     Skip points between

         lstep=2

      else
         lstep=1

      end if

      if ( kderiv>1 .and. ndimlc==3 .and. (n/=4 .and. n/=8) ) &
         lstep=2

!     --- Compute extra unknowns by transformation
!                  n-1
!         u (n) =  sum  trans (l)  u (l)
!          k       l=1       k      k

      do j = 1, ncomp
         u(n,j) = 0d0
         do i = 1, ncomp
            u(n,j) = u(n,j) + ddot ( n-jtrans, trans(i,j), ncomp, &
                     u(1,i), 1 )
         end do
      end do

      if ( mdiv==1 ) then

!     --- mdiv = 1, correct by right-hand side

         do j = 1, ncomp
            u(n,j) = u(n,j) - rhsdivtr(j)
         end do

      end if

      if ( jsol/=0 .and. jtrans==0 ) then

!     --- Compute solution in Gauss points
!         Only in case there is no transformation

         call elfillugauss ( u, phi, ugauss, n, ncomp, m, jsol, &
                             jdiag )

      end if  ! ( jsol/=0 .and. jtrans==0 )

      if ( jderiv>0 .and. jtrans==0 ) then

!     --- jderiv>0,  compute derivatives
!         Only in case there is no transformation

         call elfillderiv ( u, dphidx, dudx, n, ncomp, m, jdercn, &
                            lsub, lsup, lstep, ndimlc )

      end if  ! ( jderiv>0 .and. jtrans==0 )

      end
