      subroutine el3008 ( alpha, s, w, dphidx, ndecl, nsrow )
! ======================================================================
!
!
!        programmer     Guus Segal
!        version 1.1    date   12-04-1993 Correction variable coefficients
!        version 1.0    date   14-03-1992
!
!
!
!   copyright (c) 1992,1993  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     Compute contribution to element stiffness matrix corresponding to the
!     second derivative part of a one-dimensional equation, i.e.
!
!
!        s  = s   + / (alpha grad phi ) . grad phi dx
!         ij   ij   e                j            i
!
!                                   k             k               k
!           = s   + sum  w  (alpha(x ) grad phi (x ) . grad phi (x )
!              ij    k    k                    j               i
!
!
!     EL3008 is called by several types of element subroutines
!            It must be preceded by ELP subroutines of the 6-series
!
! ********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    elliptic_equation
!
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer ndecl, nsrow

      double precision alpha(m), s(nsrow,nsrow), w(m), &
                       dphidx(ndecl,m)

!     alpha    i     array of length m in which the values of alpha
!                    are stored
!     dphidx   i     Array of length n x m containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k);
!                         i        k
!                    derivatives are constant and only k = 1 is filled.
!     ndecl    i     Number of basis functions (n)
!     nsrow    i     Size of the element matrix S
!     s        o     Element stiffness matrix to be filled
!                    Since SEPRAN assumes an one-dimensional storage, S
!                    must be stored transposed.
!                    S has size nsrow x nsrow, but only the part (1:n,1:n)
!                    is filled.
!     w        i     array of length m containing the weights for the
!                    numerical integration
!     work           Work array of length m, to store products of
!                    alpha x w temporarily
! ********************************************************************
!
!                      COMMON BLOCKS
!
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      double precision a11con, sum, work(m)
      integer i, j, k

!    a11con   Constant value of alpha
!    i        Counting variable
!    j        Counting variable
!    k        Counting variable
!    sum      Help variable to store a sum
! ********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
! ======================================================================
!
      if ( jsecnd.eq.1 ) then

!     * jsecnd = 1   a11 and w constant

         a11con = w(1) * alpha(1) * m
         if ( n.eq.2 ) then

!         *  n = 2;  Linear element ( derivatives constant )

            s ( 1,1 ) = a11con * dphidx(1,1) * dphidx(1,1)
            s ( 2,1 ) = - s(1,1)
            s ( 2,2 ) =   s(1,1)

         else

!         *  n # 2  other cases,  summation over integration points

            do 130 i = 1,n
               do 120 j = 1,i
                  sum = 0d0
                  do 110 k = kstep, m, kstep
                     sum = sum + dphidx(i,k) * dphidx(j,k)
110               continue
                  s ( i,j ) = a11con * sum
120            continue
130         continue

         end if

      else

!        * jsecnd > 1   general

         if ( n.eq.2 ) then

!         *  n = 2;  Linear element ( derivatives constant )

            a11con = 0d0
            do 410 k = kstep, m, kstep
               a11con = a11con + w(k) * alpha(k)
410         continue
            do 430 i=1,n
               do 420 j=1,i
                  s ( i,j ) = a11con * dphidx(i,1) * dphidx(j,1)
420            continue
430         continue

         else

!         *  n # 2  other cases,  summation over integration points
!         *  Store w(k)  x alpha  (k) in work(k)

            work = w*alpha
            do 480 i = 1,n
               do 470 j = 1,i
                  sum = 0d0
                  do 460 k = kstep, m, kstep
                     sum = sum + work(k)*dphidx(i,k)*dphidx(j,k)
460               continue
                  s ( i,j ) = sum
470            continue
480         continue

         end if

      end if

!    *  the upper triangle of s is computed by transposition

      if ( n.eq.2 ) then

         s(1,2) = s(2,1)

      else

         do 510 i = 1, n
            do 500 j = 1, i-1
               s ( j,i ) = s ( i,j )
500         continue
510      continue

      end if

      end
