      subroutine el6062 ( ichois, tout, txxn, txyn, txzn, tyyn, tyzn, &
                          tzzn, u, v, w, ux, uy, uz, vx, vy, vz, wx, wy, &
                          wz, w1, d1, det, p1, p2, p3, p4, p5, p6, p7, &
                          p8, p9, work, f, theta, mlab, mone1, m1 )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  2.0  date 24-05-2001 Remove superfluous parameters
!        version  1.3  date 10-12-1997 Tensor L set to 0 in gas domains
!        version  1.2  date 09-10-1997 GH: theta defined per node
!        version  1.1  date 19-08-1997 Compressible Leonov model
!        version  1.0  date 12-04-1997
!
!   copyright (c) 1997-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the right hand side for the elastic Finger tensor evolution
!   equation using the compressible Leonov model. (3D version.)
!   Developed for Gerard Haagh
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
!     visco_elastic
!     3d
!     leonov
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mone1, m1, ichois
      double precision tout(m1), txxn(m1),  txyn(m1),  txzn(m1), &
                       tyyn(m1),  tyzn(m1),  tzzn(m1), &
                       u(m1), v(m1), w(m1), ux(m1), uy(m1), uz(m1), &
                       vx(m1), vy(m1), vz(m1), wx(m1), wy(m1), wz(m1)
      double precision w1(mone1), d1(mone1,mone1), &
                       det(mone1,mone1), &
                       p1(mone1,mone1), p2(mone1,mone1), &
                       p3(mone1,mone1), p4(mone1,mone1), &
                       p5(mone1,mone1), p6(mone1,mone1), &
                       p7(mone1,mone1), p8(mone1,mone1), &
                       p9(mone1,mone1), &
                       work(m1), theta(m1), mlab(m1), f(m1)

!    d1        i   Indicates the values of derivatives of the basisfunctions
!                  in the interpolation points in the x-direction correspon- &
!                  ding to the reference element, length MONE1 x MONE1
!    det       i   Transformation determinant, length M1
!    f             Work array
!    ichois    i   Number of the Finger tensor component to be computed
!    m1        i   Number of integration points
!    mlab      i   Material labels
!    mone1     i   Number of integration points in one dimension
!    p1        i   Indicate partial derivatives of the coordinates due to
!                  the isoparametric transformation, length M1
!    p2        i   See p1
!    p3        i   See p1
!    p4        i   See p1
!    p5        i   See p1
!    p6        i   See p1
!    p7        i   See p1
!    p8        i   See p1
!    p9        i   See p1
!    theta     i   Relaxation time
!    tout      o   Right hand side of Finger tensor evolution equation
!    txxn      i   xx-component of previous Finger tensor solution
!    txyn      i   xy-component of previous Finger tensor solution
!    txzn      i   xz-component of previous Finger tensor solution
!    tyyn      i   yy-component of previous Finger tensor solution
!    tyzn      i   yz-component of previous Finger tensor solution
!    tzzn      i   zz-component of previous Finger tensor solution
!    u         i   Velocity in x-direction
!    ux        i   Velocity gradient du/dx
!    uy        i   Velocity gradient du/dy
!    uz        i   Velocity gradient du/dz
!    v         i   Velocity in y-direction
!    vx        i   Velocity gradient dv/dx
!    vy        i   Velocity gradient dv/dy
!    vz        i   Velocity gradient dv/dz
!    w         i   Velocity in z-direction
!    w1        i   Indicates the Legendre Gauss-Lobatto weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element, length MONE1
!    work      i   Work array
!    wx        i   Velocity gradient dw/dx
!    wy        i   Velocity gradient dw/dy
!    wz        i   Velocity gradient dw/dz
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jddprv, jzprv
      double precision uxorg(27), uyorg(27), uzorg(27), vxorg(27), &
                       vyorg(27), vzorg(27), wxorg(27), wyorg(27), &
                       wzorg(27)
      double precision rdum

!     i        Loop parameter
!     jddprv   Keep JADD
!     jzprv    Keep JZERO
!     rdum     Dummy parameter
!     uxorg    Original velocity gradient du/dx
!     uyorg    Original velocity gradient du/dy
!     uzorg    Original velocity gradient du/dz
!     vxorg    Original velocity gradient dv/dx
!     vyorg    Original velocity gradient dv/dy
!     vzorg    Original velocity gradient dv/dz
!     wxorg    Original velocity gradient dw/dx
!     wyorg    Original velocity gradient dw/dy
!     wzorg    Original velocity gradient dw/dz
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL6039         compute contributions to spectral element residue vector
!                    corresponding to the Helmholtz part of the equation in the
!                    case of finite element preconditionning
!     EL6083         compute contributions to the spectral element vector
!                    corresponding to the righthand side of the equation in 3-D
!                    case
!     EL6095         compute the convection residual in the case of
!                    3-D case visco elastic solver
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'el6062' )

!     --- Keep JADD and JZERO, just not to disturb diff. eqn

      jddprv = jadd
      jzprv  = jzero
      if (ichois.ne.1 .and. ichois.ne.2 .and. ichois.ne.3  .and. &
          ichois.ne.4 .and. ichois.ne.5 .and. ichois.ne.6  )then

!     --- ichois has wrong value

         call errint ( ichois, 1 )
         call errsub ( 1, 1, 0, 0 )
         go to 1000

      end if

!     --- For all integration points in air/gas domain:
!         set velocity gradient tensor to null tensor;
!         maintain orginal values in **org arrays.

      if ( m1.gt.27 ) then

!     --- m1 too small
!         Arrays uxorg, uyorg, etc. too short

         call errchr ( 'uxorg', 1 )
         call errint ( 27, 1 )
         call errint ( m1, 2 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      do i = 1, m1
         if( mlab(i).lt.0.5d0 )then
            uxorg(i) = ux(i)
            uyorg(i) = uy(i)
            uzorg(i) = uz(i)
            vxorg(i) = vx(i)
            vyorg(i) = vy(i)
            vzorg(i) = vz(i)
            wxorg(i) = wx(i)
            wyorg(i) = wy(i)
            wzorg(i) = wz(i)
            ux(i) = 0d0
            uy(i) = 0d0
            uz(i) = 0d0
            vx(i) = 0d0
            vy(i) = 0d0
            vz(i) = 0d0
            wx(i) = 0d0
            wy(i) = 0d0
            wz(i) = 0d0
         end if
      end do

!     --- Build F = tr(B) + tr(inv(B))
!         first compute det(B); this should be equal to 1d0

      do i = 1, m1
         f(i) = txxn(i) * tyyn(i) * tzzn(i) - txxn(i)* &
                tyzn(i)**2d0 - tyyn(i) * txzn(i)**2d0 - &
                tzzn(i) * txyn(i)**2d0 + 2d0 * txyn(i) * &
                tyzn(i) * txzn(i)
      end do
      do i = i, m1
         if( abs( f(i) - 1.0d0 ).gt.1.d-1) then
            write (*,*) 'Error in F : det (B) = ', f(i)
            call instop
         end if
      end do

!     --- compute II(B)

      do i = 1, m1
         f(i) = (txxn(i) * tyyn(i)  + tyyn(i)*tzzn(i) + &
                 tzzn(i) * txxn(i)  - txyn(i)**2d0 - &
                 tyzn(i)**2d0 - txzn(i)**2d0 )

!        --- substract II(B) from tr(B)

         f(i) = txxn(i) + tyyn(i) + tzzn(i) - f(i)
      end do
      goto ( 100, 200, 300, 400, 500, 600 )ichois

!     --- Compute XX-component of Finger tensor
!         Build S11 txxn : [ -(u ddx + v ddy + w ddz) + &
!                            2/3 ( 2 ux - vy - wz) - 1/(2 theta)
!                            (txxn - 1/3 F) ] txxn
!         Build Helmholtz part

100   do i = 1, m1
         work(i) =  2d0/3d0 * (2d0 * ux(i) - vy(i) - wz(i) ) - &
            (1d0/(2d0 * theta(i)) ) * ( txxn(i) - 1d0/3d0 * f(i) )
      end do

!     --- Initialize TOUT

      jadd = 0

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6039( work, tout, txxn, w1, det, mone1, m1 )

!     --- Add (subtract) convection term to TOUT

      jadd = 1
      rdum = -1.0d0
      call el6095( u, v, w, rdum,  tout, txxn, w1, &
                   d1, p1, p2, p3, p4, p5, p6, p7, p8, p9, mone1 )

!     --- Build and add S12 txyn to tout : ( 2 uy - 1/(2 theta) * txyn )txyn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = 2d0 * uy(i) - (1d0/(2d0*theta(i))) * txyn(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, txyn, w1, det, mone1, m1 )

!     --- Build and add S13 txzn to tout : ( 2 uz - 1/(2 theta) * txzn )txzn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = 2d0 * uz(i) - (1d0/(2d0*theta(i))) * txzn(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, txzn, w1, det, mone1, m1 )

!     --- Other contributions

      do i = 1, m1
         work(i) = 1d0/(2d0 * theta(i))
      end do
      jadd = 1
      call el6083 ( work, tout, w1, det, mone1, m1 )
      go to 900

!     --- Compute xy-component of Finger tensor
!         Build S21 txxn :  vx * txxn
!         Build Helmholtz part

200   do i = 1, m1
         work(i) = vx(i)
      end do

!     --- Initialize TOUT

      jadd = 0

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6039( work, tout, txxn, w1, det, mone1, m1 )

!     --- Build and add S22 txyn to tout  :  [ -(u ddx + v ddy + w ddz) + &
!                            1/3 ( ux + vy - 2 wz) - 1/(2 theta)
!                            (txxn - 1/3 F) ] txyn
!         Build Helmholtz part

      do i = 1, m1
         work(i) =  1.0d0/3.0d0 * (ux(i) + vy(i) - 2d0 *  wz(i) ) - &
            (1d0/(2d0 * theta(i)) ) * (txxn(i) - 1d0/3d0 * f(i) )
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039(work, tout, txyn, w1, det, mone1, m1)

!     --- Add convection term to TOUT

      jadd = 1
      rdum = - 1.0d0
      call el6095(u, v, w, rdum,  tout, txyn, w1, &
                  d1, p1, p2, p3, p4, p5, p6, p7, p8, p9, mone1)

!     --- Build S23 txzn :  vz txzn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = vz(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, txzn, w1, det, mone1, m1 )

!     --- Build S24 tyyn :  (uy -1/(2 theta) txyn) tyyn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = uy(i) - (1d0/(2d0*theta(i))) * txyn(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, tyyn, w1, det, mone1, m1 )

!     --- Build S25 tyzn :  ( uz - 1/(2 theta) txzn ) tyzn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = uz(i) - (1d0/(2d0*theta(i))) * txzn(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, tyzn, w1, det, mone1, m1 )

!     --- no contribution from tzzn

      go to 900

!     --- Compute xz-component of Finger tensor
!         Build S31 txxn :  wx txxn
!         Build Helmholtz part

300   do i = 1, m1
         work(i) = wx(i)
      end do

!     --- Initialize TOUT

      jadd = 0

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6039( work, tout, txxn, w1, det, mone1, m1 )

!     --- Build S32 txyn :  wy txyn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = wy(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, txyn, w1, det, mone1, m1 )

!     --- Build S33 txzn :  [ -(u ddx + v ddy + w ddz) + &
!                            1/3 ( ux - 2 vy + wz) - 1/(2 theta)
!                            (txxn - 1/3 F) ] txxn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = 1d0/3d0 * (ux(i) -2d0 * vy(i) + wz(i) ) - &
            (1d0/(2d0 * theta(i)) ) * ( txxn(i) - 1d0/3d0 * f(i) )
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, txzn, w1, det, mone1, m1 )

!     --- Add Convection term to TOUT

      jadd = 1
      rdum = -1.0d0
      call el6095( u, v, w, rdum,  tout, txzn, w1, &
                   d1, p1, p2, p3, p4, p5, p6, p7, p8, p9, mone1 )

!     --- S34 zero contribution
!         Build S35 tyzn :  ( uy - (1/( 2 theta)) txyn ) tyzn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = uy(i) - (1d0/(2d0*theta(i))) * txyn(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, tyzn, w1, det, mone1, m1 )

!     --- Build S36 tzzn :  ( uz - (1/( 2 theta)) txzn ) tzzn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = uz(i) - (1d0/(2d0*theta(i))) * txzn(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, tzzn, w1, det, mone1, m1 )
      go to 900

!     --- Compute yy-component of Finger tensor
!         Build S42 txyn :  (2 vx- (1/( 2 theta)) txyn ) txyn
!         Build Helmholtz part

400   do i = 1, m1
         work(i) = 2d0 * vx(i) - (1d0/(2d0*theta(i))) * txyn(i)
      end do

!     --- Initialize TOUT

      jadd = 0

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6039( work, tout, txyn, w1, det, mone1, m1 )

!     --- Build S44 tyyn : [ -(u ddx + v ddy + w ddz) + &
!                            2/3 ( - ux +2 vy - wz) - 1/(2theta)
!                            (tyyn - 1/3 F) ] tyyn
!         Build Helmholtz part

      do i = 1, m1
         work(i) =  2d0/3d0 * (- ux(i) + 2d0*vy(i) - wz(i) ) - &
            (1d0/(2d0 * theta(i)) ) * ( tyyn(i) - 1d0/3d0 * f(i) )
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, tyyn, w1, det, mone1, m1 )

!     --- Add Convection term to TOUT

      jadd = 1
      rdum = -1.0d0
      call el6095( u, v, w, rdum,  tout, tyyn, w1, &
                   d1, p1, p2, p3, p4, p5, p6, p7, p8, p9, mone1 )

!     --- Build S45 tyzn :  (2 vz - (1/( 2 theta)) tyzn ) tyzn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = 2d0 * vz(i) - (1d0/(2d0*theta(i))) * tyzn(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, tyzn, w1, det, mone1, m1 )

!     --- Other contributions

      do i = 1, m1
         work(i) = 1d0/(2d0 * theta(i))
      end do
      jadd = 1
      call el6083( work, tout, w1, det, mone1, m1 )
      go to 900

!     --- Compute yz-component of Finger tensor
!         Build S52 txyn :  wx txyn
!         Build Helmholtz part

500   do i = 1, m1
         work(i) = wx(i)
      end do

!     --- Initialize TOUT

      jadd = 0

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6039( work, tout, txyn, w1, det, mone1, m1 )

!     --- Build S53 txzn :  (vx - (1/( 2 theta)) txyn ) txzn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = vx(i)- (1d0/(2d0*theta(i))) * txyn(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, txzn, w1, det, mone1, m1 )

!     --- Build S54 tyyn :  wy  tyyn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = wy(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, tyyn, w1, det, mone1, m1 )

!     --- Build S55 tyzn : [ - (u ddx + v ddy + w ddz) - &
!                            1/3 ( -  2 ux + vy + wz) - 1/(2theta)
!                            (tyyn - 1/3 F) ] tyzn
!         Build Helmholtz part

      do i = 1, m1
         work(i) =  1d0/3d0 * (-2d0* ux(i) + vy(i) + wz(i) ) - &
            (1d0/(2d0 * theta(i)) ) * ( tyyn(i) - 1d0/3d0 * f(i) )
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, tyzn, w1, det, mone1, m1 )

!     --- Add convection term to TOUT

      jadd = 1
      rdum = -1.0d0
      call el6095( u, v, w, rdum,  tout, tyzn, w1, &
                   d1, p1, p2, p3, p4, p5, p6, p7, p8, p9, mone1)

!     --- Build S56 tzzn :  (vz - (1/(2 * theta) ) * tyzn ) tzzn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = vz(i) - (1d0/(2d0 * theta(i)) ) * tyzn(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, tzzn, w1, det, mone1, m1 )
      go to 900

!     --- Compute zz-component of Finger tensor
!         Build S63 txzn : (  2 wx - 1/(2 * theta) * txzn ) * txzn
!         Build Helmholtz part

600   do i = 1, m1
         work(i) =  2d0 * wx(i) - 1d0/(2d0 * theta(i)) * txzn(i)
      end do

!     --- Initialize TOUT

      jadd = 0

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6039( work, tout, txzn, w1, det, mone1, m1 )

!     --- Build S65 tyzn :  ( 2 wy - 1/(2*theta) tyzn ) tyzn
!         --- Build Helmholtz part

      do i = 1, m1
         work(i) =  2d0 * wy(i) - 1d0/(2d0*theta(i))*tyzn(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, tyzn, w1, det, mone1, m1 )

!     --- Build S66 tzzn : [ -(u ddx + v ddy + w ddz) + &
!                            2/3 ( - ux - vy + 2 wz) - 1/(2theta)
!                            (tzzn - 1/3 F) ] tzzn
!         Build Helmholtz part

      do i = 1, m1
         work(i) =  2d0/3d0 * (- ux(i) - vy(i) + 2d0 * wz(i) ) - &
            (1d0/(2d0 * theta(i)) ) * ( tzzn(i) - 1d0/3d0 * f(i) )
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6039( work, tout, tzzn, w1, det, mone1, m1 )

!     --- Add convection term to TOUT

      jadd = 1
      rdum = -1.0d0
      call el6095( u, v, w, rdum,  tout, tzzn, w1, &
                   d1, p1, p2, p3, p4, p5, p6, p7, p8, p9, mone1 )

!     --- Other contributions

      do i = 1, m1
         work(i) = 1d0/(2d0 * theta(i))
      end do
      jadd = 1
      call el6083 ( work, tout, w1, det, mone1, m1 )

!     --- Restore jadd, jzero and velocity gradients to previous values

900   jadd  = jddprv
      jzero = jzprv
      do i = 1, m1
         if( mlab(i).lt.0.5d0 )then
            ux(i) = uxorg(i)
            uy(i) = uyorg(i)
            uz(i) = uzorg(i)
            vx(i) = vxorg(i)
            vy(i) = vyorg(i)
            vz(i) = vzorg(i)
            wx(i) = wxorg(i)
            wy(i) = wyorg(i)
            wz(i) = wzorg(i)
         end if
      end do
1000  call erclos( 'el6062' )
      end
