      subroutine el6080 ( f, old, wxp, pdxv, p1p, &
                          p2p, p3p, p4p, work, mpone, mp )
! ======================================================================
!
!        programmer    Peter Minev/ Luc Timmermans
!        version  1.2  date 15-11-2012 Debug statements
!        version  1.1  date 28-01-1996 Layout
!        version  1.0  date 31-03-1995
!
!   copyright (c) 1995-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the weak divergence of a vector - 2-D spectral elements
!     EL6080 is called by the element subroutine ELD607
!            it must be preceded by the subroutines ELP640, ELP646
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mpone, mp
      double precision f(mp), old(2*mp), wxp(mpone), &
                       pdxv(mpone,mpone), &
                       p1p(mpone,mpone), p2p(mpone,mpone), &
                       p3p(mpone,mpone), p4p(mpone,mpone), work(mp)

!     f       i/o  array of length mp containing the righthandside
!                  or the residual
!     mp       i   number of pressure points in the element
!     mpone    i   number of pressure points in the one dimension
!     old      i   array of length 2*mp containing the old velocity
!     p1p      i   arrays of length mpone x mpone containing the partial
!                  derivatives of the coordinates due to transformation
!                  for the pressure
!     p2p      i   See p1p
!     p3p      i   See p1p
!     p4p      i   See p1p
!     pdxv     i   array of length mpone x mpone contaning the values of the
!                  derivatives of the basisfunctions for the velocity in the
!                  pressure points in the x-direction
!     work     -   work array, length mp
!     wxp      i   array of length mpone containing the pressure weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, m1, l, ij, lj, im
      double precision sum

!     i              counting variable
!     ij             position in the vector f
!     im             position in the vector old
!     j              counting variable
!     l              counting variable
!     lj             position in the vector old
!     m1             counting variable
!     sum            help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el6080' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el6080'
         write(irefwr,1) 'jadd, mp, mpone', jadd, mp, mpone
  1      format ( a, 1x, (10i6) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize f to zero if necessary

      if (jadd==0) then
         do i = 1, mp
            f(i) = 0d0
         end do
      end if

!     --- add to/subtract from f the divergence part (L1)
!         store first help factor in work

      do i = 1, mpone
         ij = i
         do j = 1, mpone
            work(ij) = wxp(i)*wxp(j)*p4p(i,j)
            ij = ij + mpone
         end do
      end do

!     --- add first factor to f

      do i = 1, mpone
         ij = i
         do j = 1, mpone
            sum = 0d0
            do l = 1, mpone
               lj = l + (j - 1)*mpone
               sum = sum + pdxv(i,l)*old(lj)
            end do
            f(ij) = f(ij) + work(ij)*sum
            ij = ij + mpone
         end do
      end do

!     --- store second help factor in work

      do i = 1, mpone
         ij = i
         do j = 1, mpone
            work(ij) = -wxp(i)*wxp(j)*p2p(i,j)
            ij = ij + mpone
         end do
      end do

!     --- add second factor to f

      do i = 1, mpone
         ij = i
         do j = 1, mpone
            sum = 0d0
            do m1 = 1, mpone
               im = i + (m1 - 1)*mpone
               sum = sum + pdxv(j,m1)*old(im)
            end do
            f(ij) = f(ij) + work(ij)*sum
            ij = ij + mpone
         end do
      end do

!     --- add to/subtract from f the divergence part (L2)
!         store first help factor in work

      do i = 1, mpone
         ij = i
         do j = 1, mpone
            work(ij) = -wxp(i)*wxp(j)*p3p(i,j)
            ij = ij + mpone
         end do
      end do

!     --- add first factor to f

      do i = 1, mpone
         ij = i
         do j = 1, mpone
            sum = 0d0
            do l = 1, mpone
               lj = l + (j - 1)*mpone
               sum = sum + pdxv(i,l)*old(lj+mp)
            end do
            f(ij) = f(ij) + work(ij)*sum
            ij = ij + mpone
         end do
      end do

!     --- store second help factor in work

      do i = 1, mpone
         ij = i
         do j = 1, mpone
            work(ij) = wxp(i)*wxp(j)*p1p(i,j)
            ij = ij + mpone
         end do
      end do

!     --- add second factor to f

      do i = 1, mpone
         ij = i
         do j = 1, mpone
            sum = 0d0
            do m1 = 1, mpone
               im = i + (m1-1)*mpone
               sum = sum + pdxv(j,m1)*old(im+mp)
            end do
            f(ij) = f(ij) + work(ij)*sum
            ij = ij + mpone
         end do
      end do
1000  call erclos ( 'el6080' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el6080'
      end if  ! ( debug )
      end
