      subroutine el807rhs ( elemvc, massmt, stiffness, lvector, &
                            fvec, tempold, numrhs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 16-06-2010 New parameter list
!        version  1.0  date 01-02-2005
!
!   copyright (c) 2005-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute right-hand side vectors for enthalpy equation solved by
!     Newton method according to Fachinotti et al
! **********************************************************************
!
!                       KEYWORDS
!
!     enthalpy
!     right_hand_side
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numrhs
      double precision elemvc(icount,numrhs), massmt(icount,icount), &
                       stiffness(icount,icount), tempold(*), &
                       lvector(*), fvec(*)

!     elemvc         o    Element vector(s)
!     fvec           i    Contains the load vector per node
!     lvector        i    Contains the latent heat vector per node
!     massmt         i    Parameter referring to mass matrix (not used here)
!     numrhs         i    number of right-hand sides to be computed in one call
!     stiffness      i    contains the diffusion part of the element matrix
!     tempold        i    Temperature at previous iteration in nodes of element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision rhsd1(icount),  rhsd2(icount)

!     i              Counting variable
!     rhsd1          Help array to store the result of M T
!     rhsd2          Help array to store the result of S T
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERMATVEC       Matrix vector multiplication for full matrix
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     rhs_1 = M T /tstep - S T + L /tstep
!     rhs_2 = - M T /tstep - L /tstep - F
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el807rhs' )
      debug = .false.

!     --- debug =  t>0.7d-8 .and. ielem>1508

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el807rhs'
         call prinrl ( fvec, icount, 'fvec' )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- rhs_1 = M T /tstep - S T + L /tstep

      if ( debug ) then
         call prinrl1 ( massmt, icount, icount, 'massmt in el807rhs' )
         call prinrl1 ( stiffness, icount, icount, &
                        'stiffness in el807rhs' )
         call prinrl ( tempold, icount, 'tempold in el807rhs' )
         call prinrl ( lvector, icount, 'lvector in el807rhs' )
         write(irefwr,2) 'tstep', tstep
      end if  ! ( debug )

      call ermatvec ( massmt, tempold, rhsd1, icount )  !  M T
      call ermatvec ( stiffness, tempold, rhsd2, icount )  !  S T
      do i = 1, icount
         elemvc(i,1) = (rhsd1(i)+lvector(i))/tstep !  M T /tstep +  L /tstep
      end do  ! i = 1, icount

      if ( debug ) then
         call prinrl(rhsd1, icount, 'rhsd1 = MT')
         call prinrl(rhsd2, icount, 'rhsd2 = ST')
         call prinrl(elemvc, icount, 'elemvc')
      end if  ! ( debug )

      if ( numrhs==2 ) then

!     --- numrhs = 2, two right-hand side vectors
!         rhs_2 = - M T /tstep - L /tstep - F

         do i = 1, icount
            elemvc(i,2) = elemvc(i,1) + fvec(i) !   M T /tstep +  L /tstep + F
         end do  ! i = 1, icount

      end if  ! ( numrhs==2 )

      do i = 1, icount
         elemvc(i,1) = -elemvc(i,1) - rhsd2(i) !  - M T /tstep -  L /tstep - ST
      end do  ! i = 1, icount

1000  if ( debug ) then

!     --- Debug information

         call prinrl(elemvc(1,1), icount, 'elemvc (R1)')
         call prinrl(elemvc(1,2), icount, 'elemvc (R2)')
         write(irefwr,*) 'End el807rhs'

      end if
      call erclos ( 'el807rhs' )

      end
