      subroutine eld600 ( coorloc, iuser, user, islold, swork )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.7  date 09-03-2013 Replace elflr7 by elflr71
!        version  2.6  date 12-12-2012 Long integers
!
!   copyright (c) 1995-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element derivative vector and weights for second order elliptic
!     equations (General case) or for local error estimates
!     One, two and three dimensional elements
!     ELD600 is a help subroutine for subroutine DERIV (TO0047)
!     it is called through the intermediate subroutine eld000
!     So:
!     DERIV
!     TO0047
!       -  Loop over elements
!          -  ELD000
!             - ELD600
!    The subroutine is able to handle the following element shapes and cases:
!    spectral line element (1D)
!    spectral quadrilateral (2D)
!    spectral brick (3D)
!    cartesian co-ordinates
!    Type numbers permitted:
!    Type 600: standard linear case
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduletime
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coorloc(*), user(*), swork(*)
      integer iuser(*), islold(*)

!     coorloc     i     array of length ndimlc x npoint containing the co-ordinates
!                    of the nodal points with respect to the global numbering
!                    x  = coorloc (1,i);  y  = coorloc (2,i);  z  = coorloc (3,i);
!                     i                 i                 i
!     islold   i     User input array in which the user puts information
!                    of all preceding solutions
!                    Integer array of length 5 x numold
!     iuser    i     Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     swork          the work array contains several quantities of the high
!                    order spectral element discretization, the first position
!                    of each quantity is pointed at by an integer pointer
!     user     i     Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, kfirst, jheat, icounw, ipd1, &
              iseq, itypln,lowork, iparry, ipx1re, ipx, ipw1, &
              ipdet, ip1, ip2, ip3, ip4, ip5, ip6, ip7, ip8, ip9, &
              iworklc, iworklc2, iworklc3, ipold, ipgrd
      save isub, isup, jelp, kfirst, jheat, icounw, iseq, iparry, &
           ipdet, ip1, ip2, ip3, ip4, ip5, ip6, ip7, ip8, ip9, &
           iworklc,  iworklc2, iworklc3, ipold, ipx1re,ipx,ipw1,ipd1, &
           ipgrd, lowork
      double precision delta, csmago, rdum(1)

!     csmago         Smagorinsky constant
!     delta          LES filter size
!     i              Counting variable
!     icounw         length of elemwg(*)
!     ip1            indicate partial derivatives of the coordinates due to
!                    the isoparametric transformation, length m
!     ip2            See ip1
!     ip3            See ip1
!     ip4            See ip1
!     ip5            See ip1
!     ip6            See ip1
!     ip7            See ip1
!     ip8            See ip1
!     ip9            See ip1
!     iparry         indicates array in which the coefficients are stored
!                    length 2 x m
!     ipd1           indicates the values of derivatives of the basisfunctions
!                    in the interpolation points in the x-direction correspon-
!                    ding to the reference element, length mone x mone
!     ipdet          transformation determinant, length m
!     ipgrd
!     ipold          indicates the old solution in the nodal points, length m
!     ipw1           indicates the Legendre Gauss-Lobatto weights for
!                    numerical integration in the x-direction corresponding
!                    to the reference element, length mone
!     ipx            indicates the coordinates of the corresponding points
!                    in the element, length m x ndimlc
!     ipx1re         indicates the Legendre Gauss-Lobatto points in the
!                    x-direction corresponding to the reference element [-1,1],
!                    length mone
!     iseq           Sequence number of vector in VECOLD/ISLOLD
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     itypln         Dummy parameter
!     iworklc          integer work array
!     iworklc2         integer work array
!     iworklc3         integer work array
!     jelp           Choice parameter to distinguish between the types of
!                    element subroutines to be called. Possible values:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP622  (Quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic  brick)
!     jheat          Integer parameter indicating if the mass matrix for the
!                    heat equation must be computed (>0) or not ( = 0)
!                    If jheat = 1, then the coefficient for the mass matrix is
!                    constant, otherwise (jheat = 2) it is variable
!     kfirst         This parameter indicates the smallest velocity component
!                    that is not equal to zero in the convective terms
!                    For example kfirst = 2, means that the y-
!                    component of the velocity is unequal to zero
!     lowork
!     rdum           dummy variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0600         calls the following subroutines:
!     EL3007         Fills old solution and derivatives
!     EL6005         compute the spectral element jacobian matrix (Newton)
!     EL6006         compute the spectral element implicit non-linear term
!                    (Picard)
!     EL6006A        ?
!     EL6011         compute the spectral element convection residual
!     EL6012         compute the spectral element Helmholtz residual
!     EL6015         compute contributions to spectral element residue vector
!                    corresponding to the mass matrix
!     EL6018         compute contributions to spectral element residue vector
!                    corresponding to the diffusive part of the equation in the
!                    case of finite element preconditioning
!     EL6032         compute the spectral element Helmholtz residual
!     EL6034         compute contributions to spectral element residue vector
!                    corresponding to the diffusive part of the equation in the
!                    case of finite element preconditioning
!     EL6036         compute contributions to spectral element residue vector
!                    corresponding to the diffusive part of the equation in the
!                    case of finite element preconditioning
!     EL6037         compute the convection residual in the case of spectral
!                    element
!     EL6038         compute the spectral element mass matrix residual - 2-D
!                    case
!     EL6039         compute contributions to spectral element residue vector
!                    corresponding to the Helmholtz part of the equation in the
!                    case of finite element preconditionning
!     EL6051         compute the strong gradient of a scalar in the case of
!                    spectral elements
!     EL6052         compute the strong gradient of a scalar in the case of
!                    spectral elements
!     EL6055         Compute strong gradients in case of spectral elements 3d
!     EL6056         Compute strong gradient
!     EL6060         Compute the stresses are for the
!                    two-dimensional upper convected maxwell model
!     EL6061         Computes eddy viscosity for LES according to the
!                    Smagorinsky model
!     EL6062         Compute the stresses are for the
!                    three-dimensional upper convected maxwell model
!     EL6063         Compute the right hand side for the elastic Finger tensor
!                    evolution equation using the compressible Leonov model
!     EL6066         Computes eddy viscosity for LES according to the
!                    Smagorinsky model
!     EL6074         Compute the weak gradient of a scalar function (pressure)
!                    for righthandside/residual of the time-dependent
!                    Navier-Stokes equations; 2-D case.
!     EL6080         compute the spectral element divergence residual; 2-D
!     EL6081         computes vorticity; 2-D
!     EL6082         compute the residual of the first derivative
!     EL6085         compute contributions to spectral element stiffness
!                    residual in the case of matrix-free iterative solver
!                    - 3-D case
!     EL6086         compute the spectral element mass residual; 3-D case
!     EL6088         compute contributions to spectral element residue vector
!                    corresponding to the diffusive part of the equation in the
!                    case of non-constant diffusion
!     EL6091         compute the spectral element convection residual; 3-D
!     EL6092         compute the residual of the first derivative
!     EL6093         compute the spectral element divergence residual; 3-D
!     EL6094         computes gradient of divergence of velocity (weak); 3-D
!     EL6099         compute the residual of the first derivative
!     EL6212         interpolates from a given vector in a given point; 2-D
!     EL6213         Determine element number for set of arbitrary points
!     EL6220         3D Interpolation of solution in an arbitrary point
!     EL6221         Determine element number for set of arbitrary points
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELP640         compute basisfunctions, derivatives and weights for
!                    numerical integration corresponding to the reference
!                    element (element independent)
!     ELP642         compute the integration points, determinant and
!                    partial derivatives, two-dimensional case
!                    (element dependent)
!     ELP646         compute transformation determinant and
!                    partial derivatives  in 3-D
!     ELP647         store integration points
!                    transform weights and derivatives from the reference
!                    element
!     ELS004         swap parts of element vector for systems in the the case of
!                    a linear spectral element due to different nodal numbering
!     ELVSWAP        swap parts of element vector in the the case of a linear
!                    spectral element due to different nodal numbering
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  ISEQ:  Sequence number of vector in VECOLD/ISLOLD from which UOLD and
!         derivatives must be computed.
!         If 0 or 1 the first vector is used
!         If more vectors are needed, these vectors are supposed to be stored
!         sequentially, where the first one is indicated by ISEQ
!     2-5:  Not yet used
!         In the cases ICHELD = 6, 16, 26-29 the positions 6-11 contain
!         information about the matrix A:
!     6:  alpha 11  (1D/2D/3D)
!     7:  alpha 12  (2D/3D)
!     8:  alpha 13  (3D)
!     9:  alpha 22  (2D/3D)
!    10:  alpha 23  (3D)
!    11:  alpha 33  (3D)
!         In the case ICHELD = 21 position 6 contains information about the
!         function f to be filled:
!     6:  Function f
!     Structure of output arrays:
!     Solution array:  1 unknown per point (IVEC=0)
!     Arrays of special structure:
!     IVEC = 5:   ndimlc unknowns per point
!     ICHELD                 IVEC
!       1                      0
!       2                      5
!       3                      5
!       4                      -
!       5                      5
!       6                      5
!       7                      -
!       8                      -
!       9                      -
!      10                      -
!      11                      -
!      12                      -
!      13                      -
!      14                      -
!      15                      -
!      16                      -
!      17                      -
!      18                      -
!      19                      -
!      20                      -
!      21                      -
!      22                      -
!      23                      -
!      24                      -
!      25                      0
!      26                      -
!      27                      -
!      28                      -
!      29                      -
!      30                      0
!      31                      0
!      32                      0
!      33                      0
!      34                      0
!      35                      0
!      36                      0
!      37                      0
!      38                      0
!      39                      0
!      40                      0
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      62   dimension of space (ndimlc) has wrong value
!     406   Value of icheld not allowed
!    1682   Option not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( ifirst=0 ) then
!        Define element independent quantities
!        Compute length of array elemwg
!        Compute pointers in array SWORK
!     fi
!     Compute basis functions and related quantities
!     Fill old solution into array UOLD and if necessary the derivatives
!     Compute variable coefficients if necessary
!     Compute derived quantities
!                              METHOD
!       Differential equation:
!         - div alpha grad phi + u . grad phi + beta phi = f
!                                -
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eld600' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eld600'
         write(irefwr,1) 'ifirst', ifirst
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         iparry = 1
         itypln = 0
         call  el0600 ( iuser, user, jelp, kfirst, jheat, isub, isup, &
                        swork(iparry), iseq, itypln )
         ipx1re = iparry  + 20*m
         ipx    = ipx1re  + mone
         ipw1   = ipx     + m*ndimlc
         ipd1   = ipw1    + mone
         ipdet  = ipd1    + mone*mone
         ip1    = ipdet   + m
         ip2    = ip1     + m
         ip3    = ip2     + m
         ip4    = ip3     + m
         ip5    = ip4     + m
         ip6    = ip5     + m
         ip7    = ip6     + m
         ip8    = ip7     + m
         ip9    = ip8     + m
         iworklc  = ip9     + m
         iworklc2 = iworklc   + m
         iworklc3 = iworklc2  + m
         ipgrd  = iworklc3  + m
         ipold  = ipgrd   + ndimlc**2 * m

!        --- total length (20 + 13*m + ndimlc*m + 2* ndimlc**2 * m + mone*mone + &
!                          mone)
!            compute points and weights for numerical integration
!            compute derivatives of the basisfunctions
!            corresponding to the reference element

         call elp640 ( 1, swork(ipx1re), swork(ipw1), swork(ipd1), &
                       swork(ipdet), mone )

         if ( ierror/=0 ) go to 1000

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( islold, numold, isub, isup, iuser )

!        --- Length of elemwg(*) to be filled

         if ( icheld==1 ) then
            icounw = m
         else if ( icheld==2 ) then
            icounw = ndimlc*m
         else
            icounw = 0
         end if

      end if

!     --- Initialize weight vector, if any

      if ( icounw>0) elemwg(1:icounw) = 1d0

      if ( debug ) &
         write(irefwr,1) 'icheld, icounw, ndimlc, m, n, ix', &
                          icheld, icounw, ndimlc, m, n, ix

!     --- compute basis functions and weights for numerical integration
!         if icheld==40 (interpolation) then skip basis functions

      if ( icheld==39 .or. icheld==40 ) goto 101

      if ( ndimlc==3 ) then

         call elp646 ( coorloc, index1, swork(ipx), swork(ipx+m), &
                       swork(ipx+2*m), &
                       swork(ipd1), swork(ipdet), swork(ip1), &
                       swork(ip2),  swork(ip3),   swork(ip4), &
                       swork(ip5),  swork(ip6),   swork(ip7), &
                       swork(ip8),  swork(ip9),   mone )

      else if ( ndimlc==2 ) then

         call elp642(coorloc, index1, swork(ipx), swork(ipd1), &
                     swork(ipdet), swork(ip1), swork(ip2), &
                     swork(ip3),   swork(ip4), mone, m)

      else if ( ndimlc==1 ) then

         call elp647( coorloc, index1, swork(ipx), swork(ipdet), mone)

      else

         call errint ( ndimlc, 1 )
         call errsub ( 62, 1, 0, 0 )
         go to 1000

      end if

101   do i = 1, numold
         lowork = ipold + (i-1)*m
         call el3007 ( swork(lowork), rdum, rdum, &
                       rdum, m, m, ndimlc, ncomp, &
                       index1, rdum, swork(lowork),jtrans, &
                       numold, i )
      end do

!     --- Fill variable coefficients if necessary

      do i = isub, isup
         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71(i, user, swork(iparry), &
                        swork(ipx), swork(ipx), m, m, rdum, &
                        1, index1, iuser, swork(ipold), swork)
         end if
      end do

      if ( itimloop/=0 .and. itimloop/=1) itimloop = 1

      if ( icheld==1 ) then

!     --- Gradients strong

         if ( ndimlc==3 ) then

            call el6055 ( elemvc1, elemwg,  swork(ipold), &
                          swork(ipd1), swork(ipdet), &
                          swork(ip1),  swork(ip2), &
                          swork(ip3),  swork(ip4), &
                          swork(ip5),  swork(ip6), &
                          swork(ip7),  swork(ip8), &
                          swork(ip9),  mone, ix )

         end if

         if ( ndimlc==2 ) then

             call el6051( elemvc1, elemwg, swork(ipold), &
                          swork(ipd1), swork(ip1), swork(ip2), &
                          swork(ip3),  swork(ip4), swork(ipdet), &
                          swork(iworklc), mone, ix)
             if (mone==2) call els004(elemvc1, m)

         end if

         if ( ndimlc==1 ) then
             call errchr ( '1d grad to do', 1 )
             call errsub ( 1682, 0, 0, 1 )
             go to 1000
         end if

      end if

      if ( icheld==2 ) then

         if ( ndimlc==3 ) then

            call el6056 ( elemvc1, elemwg,  swork(ipold), &
                          swork(ipd1), swork(ipdet), &
                          swork(ip1),  swork(ip2), &
                          swork(ip3),  swork(ip4), &
                          swork(ip5),  swork(ip6), &
                          swork(ip7),  swork(ip8), &
                          swork(ip9),  mone )

         end if

         if ( ndimlc==2 ) then

            call el6052(elemvc1, elemwg, swork(ipold), &
                        swork(ipd1), swork(ip1), swork(ip2), &
                        swork(ip3),  swork(ip4), swork(ipdet), &
                        swork(iworklc), mone)

         end if

      end if

      if ( icheld==3 ) then

!     --- icheld = 3, not available

         call errint ( icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0 )
         go to 1000

      end if

!     --- compute the divergence of the velocity

      if ( icheld==4 ) then

!     --- icheld = 4, not available

         call errint ( icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0 )
         go to 1000

      end if

      if ( icheld==5 ) then

          if ( ndimlc==2 ) then

              call el6081(elemvc1, swork(ipold), swork(ipw1), &
                          swork(ipd1), swork(ip1), &
                          swork(ip2), swork(ip3), swork(ip4), &
                          swork(ip9), mone, m)

           else

!          --- icheld = 5, not available

              call errint ( icheld, 1 )
              call errint ( itype, 2 )
              call errint ( ielgrp, 3 )
              call errsub ( 406, 3, 0, 0 )
              go to 1000

           end if

      end if

      if ( icheld==6 ) then

!     --- icheld = 6, not available

         call errint ( icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0 )
         go to 1000

      end if

!     --- Calculate Residual

      if ( icheld==25 ) then

!     --- fe-preconditioning, build residual vector
!         then calculate the residual

         jadd = 0

         if ( jsecnd>0 ) then

!        --- First second derivative part (Diffusive terms)

           if ( ndimlc==3 ) then

              call el6085 ( swork(iparry), elemvc1, swork(ipw1), &
                            swork(ipd1), swork(ipdet), &
                            swork(ip1), swork(ip2), swork(ip3), &
                            swork(ip4), swork(ip5), swork(ip6), &
                            swork(ip7), swork(ip8), swork(ip9), &
                            swork(iworklc), swork(ipold), mone)
              jadd = 1

           end if

           if ( ndimlc==2 ) then

              call el6034 ( swork(iparry), elemvc1,     swork(ipold), &
                            swork(ipw1),   swork(ipd1), &
                            swork(ipdet),  swork(ip1), &
                            swork(ip2),    swork(ip3), swork(ip4), &
                            swork(iworklc),  mone)
              jadd = 1

           end if

           if ( ndimlc==1 ) then

             call el6018 ( swork(iparry), elemvc1, swork(ipold), &
                           swork(ipdet),  swork(ipw1), swork(ipd1), &
                           m, swork(iworklc) )
             jadd = 1

           end if

         end if

!        --- then first derivative part (convective terms)

         if ( idim>0 ) then

            if ( ndimlc==3 ) then

               call errchr ( '3D Convection Residual not done yet', 1 )
               call errsub ( 1682, 0, 0, 1 )
               go to 1000

            end if

            if ( ndimlc==2 ) then

               call el6037 ( swork(iparry+6*m), swork(ipw1), &
                             swork(ipd1), swork(ip1), swork(ip2), &
                             swork(ip3),  swork(ip4), swork(iworklc), &
                             mone, m, elemvc1, swork(ipold) )
                jadd = 1

            end if

            if ( ndimlc==1 ) then

                call el6011(swork(iparry+6*m), elemvc1, swork(ipold), &
                            swork(ipw1), swork(ipd1), m, swork(iworklc))
                jadd = 1

            end if

         end if

!        --- next the constant part (Helmholtz terms)

         if ( jzero>0 ) then

            if ( ndimlc==3 ) then

               call el6039 ( swork(iparry+9*m), elemvc1, &
                             swork(ipold),      swork(ipw1), &
                             swork(ipdet),      mone, m)

            end if

            if ( ndimlc==2 ) then

               call el6032 ( swork(iparry+9*m), elemvc1, &
                             swork(ipold), swork(ipw1), swork(ipw1), &
                             swork(ipdet), mone, m)

            end if

            if ( ndimlc==1 ) then

               call el6012 ( swork(iparry+9*m), elemvc1, swork(ipold), &
                             swork(ipw1), swork(ipdet), m)

            end if

         end if

         if ( iflag==2 ) then

!        --- add the mass residual

           if ( ndimlc==3 ) then
                call el6086 ( elemvc1, swork(ipw1), swork(ipdet), &
                              swork(ipold), mone )
           else if ( ndimlc==2 ) then
                call el6038 ( elemvc1, swork(ipw1), swork(ipdet), &
                              swork(ipold), mone )
           else
                call el6015 ( elemvc1, swork(ipold), swork(ipw1), &
                              swork(ipdet), mone )
           end if

         end if

      end if

      if ( icheld==26 ) then

!     --- fe-preconditioning, build residual vector
!         then calculate the residual

         jadd = 0

         if ( jsecnd>0 ) then

!        --- First second derivative part (Diffusive terms)

           if ( ndimlc==3 ) then
               jsecnd = 2

               call el6088(swork(ipold), swork(ipold + m), &
                           swork(ipold + 2*m), swork(ipold + 3*m), &
                           swork(ipw1),   swork(ipd1), &
                           swork(ip1),    swork(ip2), swork(ip3), &
                           swork(ip4),    swork(ip5), swork(ip6), &
                           swork(ip7),    swork(ip8), swork(ip9), &
                           swork(ipdet),  swork(iworklc), &
                           swork(iworklc2), swork(iworklc3), mone, &
                           elemvc1, ix)

                jsecnd = 1
                jadd = 1

           end if

           if ( ndimlc==2 ) then
              jsecnd = 2
              call el6036(swork(ipold), swork(ipold + m), &
                            swork(ipold + 2*m), swork(ipw1), &
                            swork(ipd1),  swork(ip1), swork(ip2), &
                            swork(ip3),   swork(ip4), swork(ipdet), &
                            swork(iworklc), swork(iworklc2), mone, &
                            elemvc1, ix)
              jadd = 1
           end if

           if (ndimlc==1 ) then
             call el6018(swork(iparry), elemvc1, swork(ipold), &
                         swork(ipdet),  swork(ipw1), swork(ipd1), &
                         m, swork(iworklc))
             jadd = 1
           end if

         end if

!        --- then first derivative part (convective terms)

         if ( idim>0) then
            if ( ndimlc==3 ) then
               call errchr ( '3D Convection Residual not done yet', 1 )
               call errsub ( 1682, 0, 0, 1 )
               go to 1000
           end if

            if ( ndimlc==2 ) then
               call el6037(swork(iparry+6*m), swork(ipw1), &
                           swork(ipd1), swork(ip1), swork(ip2), &
                           swork(ip3),  swork(ip4), swork(iworklc), &
                           mone, m, elemvc1, swork(ipold))
                jadd = 1
            end if

            if ( ndimlc==1 ) then

                call el6011(swork(iparry+6*m), elemvc1, swork(ipold), &
                            swork(ipw1), swork(ipd1), m, swork(iworklc))
                jadd = 1
            end if
         end if

!        --- next the constant part (Helmholtz terms)

         if ( jzero>0 ) then

            if ( ndimlc==3 ) then
               call el6039(swork(iparry+9*m), elemvc1, &
                           swork(ipold+(ix-1)*m),      swork(ipw1), &
                           swork(ipdet),      mone, m)
            end if

            if ( ndimlc==2 ) then
               call el6032(swork(iparry+9*m), elemvc1, &
                    swork(ipold+(ix-1)*m), swork(ipw1), swork(ipw1), &
                           swork(ipdet), mone, m)
            end if

            if ( ndimlc==1 ) then
               call el6012(swork(iparry+9*m), elemvc1, swork(ipold), &
                           swork(ipw1), swork(ipdet), m)
            end if

         end if

         if ( iflag==2) then

!        --- add the mass residual

           if ( ndimlc==3 ) then
                call el6086( elemvc1, swork(ipw1), swork(ipdet), &
                             swork(ipold+(ix-1)*m), mone)
           else if ( ndimlc==2 ) then
                call el6038( elemvc1, swork(ipw1), swork(ipdet), &
                             swork(ipold+(ix-1)*m), mone)
           else
                call el6015( elemvc1, swork(ipold), swork(ipw1), &
                             swork(ipdet), mone)
           end if

         end if

      end if

      if ( icheld==30 ) then

!     --- Compute the first component of the gradient

        if ( ix==1) then

          if ( ndimlc==3 ) then
           if (itimloop==1) then
            call el6099(elemvc1, swork(ipold), swork(ipw1), &
                        swork(ipd1), &
                        swork(ip1), swork(ip2), swork(ip3), &
                        swork(ip4), swork(ip5), swork(ip6), &
                        swork(ip7), swork(ip8), swork(ip9), &
                        mone, 1)
           else

!          --- swork(ipdet) is used as a work space

             call el6092( elemvc1, swork(ipold),    swork(ipw1), &
                          swork(ipd1), swork(ipdet), &
                          swork(ip1),  swork(ip2), swork(ip3), &
                          swork(ip4),  swork(ip5), swork(ip6), &
                          swork(ip7),  swork(ip8), swork(ip9), &
                          mone, 1)
           end if
          end if

          if ( ndimlc==2 ) then
           jadd = 0
           if (itimloop==1) then
              call el6074( elemvc1,      swork(ipold), swork(ipw1), &
                           swork(ipd1), swork(ip1),   swork(ip2), &
                           swork(ip3),  swork(ip4), &
                           swork(ip9),  mone, m, 1)
          else
              call el6082( elemvc1,  swork(ipold),   swork(ipw1), &
                           swork(ipd1), swork(ip1), swork(ip2), &
                           swork(ip3),  swork(ip4), swork(ip9), &
                           mone, m, 1)
          end if
         end if
         if ( ndimlc==1 ) then

!        --- icheld = 30, not available

            call errint ( icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 406, 3, 0, 0 )
            go to 1000

         end if

       end if

!      --- Compute the second component of the gradient (only-3D)

       if ( ix==2) then
         if ( ndimlc==3 ) then
           if (itimloop==1) then
            call el6099(elemvc1, swork(ipold), swork(ipw1), &
                        swork(ipd1), &
                        swork(ip1), swork(ip2), swork(ip3), &
                        swork(ip4), swork(ip5), swork(ip6), &
                        swork(ip7), swork(ip8), swork(ip9), &
                        mone, 2)
           else

!          --- swork(ipdet) is used as a work space

             call el6092( elemvc1, swork(ipold), swork(ipw1), &
                          swork(ipd1), swork(ipdet), &
                          swork(ip1), swork(ip2), swork(ip3), &
                          swork(ip4), swork(ip5), swork(ip6), &
                          swork(ip7), swork(ip8), swork(ip9), &
                          mone, 2)
           end if
         end if
         if ( ndimlc==2 ) then

!        --- icheld = 30, not available

            call errint ( icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 406, 3, 0, 0 )
            go to 1000

         end if
         if ( ndimlc==1 ) then

!        --- icheld = 30, not available

            call errint ( icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 406, 3, 0, 0 )
            go to 1000

         end if
        end if

!       --- Compute the second component of the gradient (only 2D/3D)

        if ( ix==3) then
         if ( ndimlc==3 ) then
           if (itimloop==1) then
               call el6099(elemvc1, swork(ipold),    swork(ipw1), &
                           swork(ipd1), swork(ip1), swork(ip2), &
                           swork(ip3),  swork(ip4), swork(ip5), &
                           swork(ip6),  swork(ip7), swork(ip8), &
                           swork(ip9),  mone,       3)
           else
               call el6092( elemvc1, swork(ipold), swork(ipw1), &
                            swork(ipd1), swork(ipdet), &
                            swork(ip1),  swork(ip2), swork(ip3), &
                            swork(ip4),  swork(ip5), swork(ip6), &
                            swork(ip7),  swork(ip8), swork(ip9), &
                            mone, 3)
           end if
         end if
         if ( ndimlc==2 ) then
            if ( itimloop==1 ) then
               call el6074( elemvc1, swork(ipold), swork(ipw1), &
                            swork(ipd1), swork(ip1), swork(ip2), &
                            swork(ip3),  swork(ip4), swork(ip9), &
                            mone, m, 2)
            else
                call el6082( elemvc1, swork(ipold), swork(ipw1), &
                             swork(ipd1), swork(ip1), swork(ip2), &
                             swork(ip3),  swork(ip4), swork(ip9), &
                             mone, m, 2)
            end if
         end if
         if ( ndimlc==1 ) then

!        --- icheld = 30, not available

            call errint ( icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 406, 3, 0, 0 )
            go to 1000

         end if
       end if
      end if

!     --- compute the divergence of the velocity

      if ( icheld==31 ) then
         if ( ndimlc==3 ) then
            call el6093(swork(ipold), swork(ipold+m), swork(ipold+2*m), &
                        swork(ipw1), swork(ipd1), &
                        swork(ip1),  swork(ip2), swork(ip3), &
                        swork(ip4),  swork(ip5), swork(ip6), &
                        swork(ip7),  swork(ip8), swork(ip9), &
                        mone, elemvc1)
         end if
         if ( ndimlc==2 ) then
            jadd = 0
            call el6080(elemvc1, swork(ipold), swork(ipw1), &
                        swork(ipd1), swork(ip1), &
                        swork(ip2),  swork(ip3), swork(ip4), &
                        swork(ip9),  mone, m)
         end if
         if ( ndimlc==1 ) then

!        --- icheld = 31, not available

            call errint ( icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 406, 3, 0, 0 )
            go to 1000

         end if
      end if

!     --- compute the gradient of divergence of the velocity

      if ( icheld==32 ) then
          if ( ix==1) then

!         --- first component

             if ( ndimlc==3 ) then
                call el6094(swork(ipold),     swork(ipold+m), &
                            swork(ipold+2*m), swork(ipold+3*m), &
                            swork(ipw1),      swork(ipd1), &
                            swork(ip1),       swork(ip2), swork(ip3), &
                            swork(ip4),       swork(ip5), swork(ip6), &
                            swork(ip7),       swork(ip8), swork(ip9), &
                            swork(ipdet), mone, elemvc1, 1)
             else

!            --- icheld = 31, not available

                 call errint ( icheld, 1 )
                 call errint ( itype, 2 )
                 call errint ( ielgrp, 3 )
                 call errsub ( 406, 3, 0, 0 )
                 go to 1000

             end if
         end if
         if ( ix==2) then

!        --- second component

             if ( ndimlc==3 ) then
                call el6094(swork(ipold),     swork(ipold+m), &
                            swork(ipold+2*m), swork(ipold+3*m), &
                            swork(ipw1),      swork(ipd1), &
                            swork(ip1), swork(ip2), swork(ip3), &
                            swork(ip4), swork(ip5), swork(ip6), &
                            swork(ip7), swork(ip8), swork(ip9), &
                            swork(ipdet), mone, elemvc1, 2)
             else

!            --- icheld = 31, not available

                call errint ( icheld, 1 )
                call errint ( itype, 2 )
                call errint ( ielgrp, 3 )
                call errsub ( 406, 3, 0, 0 )
                go to 1000

             end if
          end if
          if ( ix==3) then

!         --- third component

             if ( ndimlc==3 ) then
                call el6094(swork(ipold), swork(ipold+m), &
                            swork(ipold+2*m), swork(ipold+3*m), &
                            swork(ipw1), swork(ipd1), &
                            swork(ip1),  swork(ip2), swork(ip3), &
                            swork(ip4),  swork(ip5), swork(ip6), &
                            swork(ip7),  swork(ip8), swork(ip9), &
                            swork(ipdet), mone, elemvc1, 3)
             else

!            --- icheld = 31, not available

                call errint ( icheld, 1 )
                call errint ( itype, 2 )
                call errint ( ielgrp, 3 )
                call errsub ( 406, 3, 0, 0 )
                go to 1000

             end if
        end if
      end if

!     --- compute the convection residual

      if ( icheld==33 ) then
          if ( ndimlc==3 ) then
              call el6091(swork(ipold+m), swork(ipold+2*m), &
                     swork(ipold+3*m), &
                     swork(ipw1), swork(ipd1), &
                     swork(ip1),  swork(ip2), swork(ip3), &
                     swork(ip4),  swork(ip5), swork(ip6), &
                     swork(ip7),  swork(ip8), swork(ip9), &
                     mone, elemvc1, swork(ipold))
           end if
           if ( ndimlc==2 ) then
               jadd = 0
               call el6037(swork(ipold+m), swork(ipw1), &
                           swork(ipd1), swork(ip1), swork(ip2), &
                           swork(ip3),  swork(ip4), swork(ip9), mone, &
                           m, elemvc1, swork(ipold))
           end if
           if ( ndimlc==1 ) then
               jadd = 0
               call el6011( swork(ipold+m), elemvc1, swork(ipold), &
                            swork(ipw1), &
                            swork(ipd1), mone, swork(ip9))
           end if
       end if

!      --- compute NON-constant viscosity

      if ( icheld==34 ) then
          if ( ix==1 ) then
            if ( ndimlc==3 ) then
               jsecnd = 2
               call el6088(swork(ipold), swork(ipold + m), &
                           swork(ipold + 2*m), swork(ipold + 3*m), &
                           swork(ipw1),   swork(ipd1), &
                           swork(ip1),    swork(ip2), swork(ip3), &
                           swork(ip4),    swork(ip5), swork(ip6), &
                           swork(ip7),    swork(ip8), swork(ip9), &
                           swork(ipdet),  swork(iworklc), &
                           swork(iworklc2), swork(iworklc3), mone, &
                           elemvc1, 1)
                jsecnd = 1
             end if
             if ( ndimlc==2 ) then
                jsecnd = 2
                call el6036(swork(ipold), swork(ipold + m), &
                            swork(ipold + 2*m), swork(ipw1), &
                            swork(ipd1),  swork(ip1), swork(ip2), &
                            swork(ip3),   swork(ip4), swork(ipdet), &
                            swork(iworklc), swork(iworklc2), mone, &
                            elemvc1, 1)
                jsecnd = 1
             end if
             if ( ndimlc==1 ) then

!            --- icheld = 34, not available

                call errint ( icheld, 1 )
                call errint ( itype, 2 )
                call errint ( ielgrp, 3 )
                call errsub ( 406, 3, 0, 0 )
                go to 1000

             end if
          end if
          if ( ix==2 ) then
            if ( ndimlc==3 ) then
                jsecnd = 2
                call el6088(swork(ipold), swork(ipold + m), &
                            swork(ipold + 2*m), swork(ipold + 3*m), &
                            swork(ipw1),   swork(ipd1), &
                            swork(ip1),    swork(ip2), swork(ip3), &
                            swork(ip4),    swork(ip5), swork(ip6), &
                            swork(ip7),    swork(ip8), swork(ip9), &
                            swork(ipdet),  swork(iworklc), &
                            swork(iworklc2), swork(iworklc3), mone, &
                            elemvc1, 2)
                jsecnd = 1
             end if
             if ( ndimlc==2 ) then
                jsecnd = 2
                call el6036(swork(ipold), swork(ipold + m), &
                            swork(ipold + 2*m), swork(ipw1), &
                            swork(ipd1),  swork(ip1), swork(ip2), &
                            swork(ip3),   swork(ip4), swork(ipdet), &
                            swork(iworklc), swork(iworklc2), mone, &
                            elemvc1, 2)
                jsecnd = 1
             end if
             if ( ndimlc==1 ) then

!            --- icheld = 34, not available

                call errint ( icheld, 1 )
                call errint ( itype, 2 )
                call errint ( ielgrp, 3 )
                call errsub ( 406, 3, 0, 0 )
                go to 1000

             end if
          end if
          if ( ix==3 ) then
            if ( ndimlc==3 ) then
                jsecnd = 2
                call el6088(swork(ipold), swork(ipold + m), &
                            swork(ipold + 2*m), swork(ipold + 3*m), &
                            swork(ipw1),   swork(ipd1), &
                            swork(ip1),    swork(ip2), swork(ip3), &
                            swork(ip4),    swork(ip5), swork(ip6), &
                            swork(ip7),    swork(ip8), swork(ip9), &
                            swork(ipdet),  swork(iworklc), &
                            swork(iworklc2), swork(iworklc3), mone, &
                            elemvc1, 1)
                jsecnd = 1
             end if
             if ( ndimlc==2 ) then

!            --- icheld = 34, not available

                call errint ( icheld, 1 )
                call errint ( itype, 2 )
                call errint ( ielgrp, 3 )
                call errsub ( 406, 3, 0, 0 )
                go to 1000

             end if
             if ( ndimlc==1 ) then

!            --- icheld = 34, not available

                call errint ( icheld, 1 )
                call errint ( itype, 2 )
                call errint ( ielgrp, 3 )
                call errsub ( 406, 3, 0, 0 )
                go to 1000

             end if
          end if
      end if
      if ( icheld==35 ) then

!     --- compute eddy viscosty according to the Smagorinsky model
!         - swork(ipold) = u_1
!         - swork(ipold+m) = u_2
!         - swork(ipold+2*m) = u_3 (3-D only)
!         - elemvc1 = eddy viscosity according to Smagorinsky

         delta  = user(60)
         csmago = user(61)
         if ( ndimlc==1 ) then

!        --- icheld = 35, not available

                call errint ( icheld, 1 )
                call errint ( itype, 2 )
                call errint ( ielgrp, 3 )
                call errsub ( 406, 3, 0, 0 )
                go to 1000

         end if
         if ( ndimlc==2 ) then
            call el6061(swork(ipold), swork(ipold + m), &
                        swork(ipd1), swork(ip1),swork(ip2),swork(ip3), &
                        swork(ip4), swork(ipdet), mone, elemvc1, elemwg, &
                        delta, csmago)
         end if
         if ( ndimlc==3 ) then
            call el6066(swork(ipold), swork(ipold + m), &
                        swork(ipold + 2*m), swork(ipd1), swork(ip1), &
                        swork(ip2), swork(ip3), swork(ip4), swork(ip5), &
                        swork(ip6), swork(ip7), swork(ip8),swork(ip9), &
                        swork(ipdet), mone, elemvc1, elemwg, &
                        delta, csmago)
         end if
      end if
      if ( icheld==37 ) then
        if ( ndimlc==3 ) then

!       --- Note that now ipgrd is used to store weak gradients
!           perform call for u_x

            call el6099 ( swork(ipgrd), swork(ipold), swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 1 )

!           --- perform call for u_y

            call el6099 ( swork(ipgrd+1*m),swork(ipold),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 2 )

!           --- perform call for u_z

            call el6099 ( swork(ipgrd+2*m),swork(ipold),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 3 )

!           --- perform call for v_x

            call el6099 ( swork(ipgrd+3*m),swork(ipold+1*m),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 1 )

!           --- perform call for v_y

            call el6099 ( swork(ipgrd+4*m),swork(ipold+1*m),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 2 )

!           --- perform call for v_z

            call el6099 ( swork(ipgrd+5*m),swork(ipold+1*m),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 3 )

!           --- perform call for w_x

            call el6099 ( swork(ipgrd+6*m),swork(ipold+2*m),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 1 )

!           --- perform call for w_y

            call el6099 ( swork(ipgrd+7*m),swork(ipold+2*m),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 2 )

!           --- perform call for w_z

            call el6099 ( swork(ipgrd+8*m),swork(ipold+2*m),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 3 )

!           --- compose full rhs vector, from nine gradients
!               and the viscosity

            call el6006( elemvc1, swork(ipgrd), swork(ipgrd+m), &
                         swork(ipgrd+2*m), swork(ipgrd+3*m), &
                         swork(ipgrd+4*m), swork(ipgrd+5*m), &
                         swork(ipgrd+6*m), swork(ipgrd+7*m), &
                         swork(ipgrd+8*m), swork(iparry), m, ix)
        end if
      end if
      if ( icheld==38 ) then
         if ( ndimlc==3 ) then

!        --- Calculate u_x, u_y, u_z
!                      v_x, v_y, v_z
!                      w_x, w_y, w_z
!            perform call for u_x

            call el6055 ( swork(ipgrd), elemwg,  swork(ipold), &
                          swork(ipd1),    swork(ipdet), &
                          swork(ip1),     swork(ip2), &
                          swork(ip3),     swork(ip4), &
                          swork(ip5),     swork(ip6), &
                          swork(ip7),     swork(ip8), &
                          swork(ip9),     mone, 1 )

!           --- perform call for u_y

            call el6055 ( swork(ipgrd+1*m), elemwg,  swork(ipold), &
                          swork(ipd1),    swork(ipdet), &
                          swork(ip1),     swork(ip2), &
                          swork(ip3),     swork(ip4), &
                          swork(ip5),     swork(ip6), &
                          swork(ip7),     swork(ip8), &
                          swork(ip9),     mone, 2 )

!           --- perform call for u_z

            call el6055 ( swork(ipgrd+2*m), elemwg,  swork(ipold), &
                          swork(ipd1),    swork(ipdet), &
                          swork(ip1),     swork(ip2), &
                          swork(ip3),     swork(ip4), &
                          swork(ip5),     swork(ip6), &
                          swork(ip7),     swork(ip8), &
                          swork(ip9),     mone, 3 )

!           --- perform call for v_x

            call el6055 ( swork(ipgrd+3*m), elemwg,  swork(ipold+m), &
                          swork(ipd1),    swork(ipdet), &
                          swork(ip1),     swork(ip2), &
                          swork(ip3),     swork(ip4), &
                          swork(ip5),     swork(ip6), &
                          swork(ip7),     swork(ip8), &
                          swork(ip9),     mone, 1 )

!           --- perform call for v_y

            call el6055 ( swork(ipgrd+4*m), elemwg,  swork(ipold+m), &
                          swork(ipd1),    swork(ipdet), &
                          swork(ip1),     swork(ip2), &
                          swork(ip3),     swork(ip4), &
                          swork(ip5),     swork(ip6), &
                          swork(ip7),     swork(ip8), &
                          swork(ip9),     mone, 2 )

!           --- perform call for v_z

            call el6055 ( swork(ipgrd+5*m), elemwg,  swork(ipold+m), &
                          swork(ipd1),    swork(ipdet), &
                          swork(ip1),     swork(ip2), &
                          swork(ip3),     swork(ip4), &
                          swork(ip5),     swork(ip6), &
                          swork(ip7),     swork(ip8), &
                          swork(ip9),     mone, 3 )

!           --- perform call for w_x

            call el6055 ( swork(ipgrd+6*m), elemwg, swork(ipold+2*m), &
                          swork(ipd1),    swork(ipdet), &
                          swork(ip1),     swork(ip2), &
                          swork(ip3),     swork(ip4), &
                          swork(ip5),     swork(ip6), &
                          swork(ip7),     swork(ip8), &
                          swork(ip9),     mone, 1 )

!           --- perform call for w_y

            call el6055 ( swork(ipgrd+7*m), elemwg, swork(ipold+2*m), &
                          swork(ipd1),    swork(ipdet), &
                          swork(ip1),     swork(ip2), &
                          swork(ip3),     swork(ip4), &
                          swork(ip5),     swork(ip6), &
                          swork(ip7),     swork(ip8), &
                          swork(ip9),     mone, 2 )

!           --- perform call for w_z

            call el6055 ( swork(ipgrd+8*m), elemwg, swork(ipold+2*m), &
                          swork(ipd1),    swork(ipdet), &
                          swork(ip1),     swork(ip2), &
                          swork(ip3),     swork(ip4), &
                          swork(ip5),     swork(ip6), &
                          swork(ip7),     swork(ip8), &
                          swork(ip9),     mone, 3 )

!           --- material labels (swork(ipold+6*m)) added

            call el6062 ( ix, elemvc1,    swork(ipold+3*m), &
                       swork(ipold+4*m), swork(ipold+5*m), &
                       swork(ipold+6*m), swork(ipold+7*m), &
                       swork(ipold+8*m), swork(ipold), &
                       swork(ipold+m),   swork(ipold+2*m), &
                       swork(ipgrd),     swork(ipgrd+m), &
                       swork(ipgrd+2*m), swork(ipgrd+3*m), &
                       swork(ipgrd+4*m), swork(ipgrd+5*m), &
                       swork(ipgrd+6*m), swork(ipgrd+7*m), &
                       swork(ipgrd+8*m), &
                       swork(ipw1),swork(ipd1), swork(ipdet), &
                       swork(ip1), swork(ip2),  swork(ip3), swork(ip4), &
                       swork(ip5), swork(ip6),  swork(ip7), swork(ip8), &
                       swork(ip9), swork(iworklc),swork(iworklc2), &
                       swork(ipold+9*m), swork(ipold+10*m), &
                       mone, m )

!           --- Note that now ipgrd is used to store weak gradients
!               perform call for u_x
!               Mark that due to the go to it is impossible to reach the
!               next statement

            call el6099 ( swork(ipgrd), swork(ipold), swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 1 )

!           --- perform call for u_y

            call el6099 ( swork(ipgrd+1*m),swork(ipold),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 2 )

!           --- perform call for u_z

            call el6099 ( swork(ipgrd+2*m),swork(ipold),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 3 )

!           --- perform call for v_x

            call el6099 ( swork(ipgrd+3*m),swork(ipold+1*m),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 1 )

!           --- perform call for v_y

            call el6099 ( swork(ipgrd+4*m),swork(ipold+1*m),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 2 )

!           --- perform call for v_z

            call el6099 ( swork(ipgrd+5*m),swork(ipold+1*m),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 3 )

!           --- perform call for w_x

            call el6099 ( swork(ipgrd+6*m),swork(ipold+2*m),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 1 )

!           --- perform call for w_y

            call el6099 ( swork(ipgrd+7*m),swork(ipold+2*m),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 2 )

!           --- perform call for w_z

            call el6099 ( swork(ipgrd+8*m),swork(ipold+2*m),swork(ipw1), &
                          swork(ipd1),  swork(ip1),   swork(ip2), &
                          swork(ip3),   swork(ip4),   swork(ip5), &
                          swork(ip6),   swork(ip7),   swork(ip8), &
                          swork(ip9),   mone, 3 )

!           --- compose full rhs vector, from nine gradients
!               and the viscosity

            call el6006a( elemvc1, swork(ipgrd), swork(ipgrd+m), &
                         swork(ipgrd+2*m), swork(ipgrd+3*m), &
                         swork(ipgrd+4*m), swork(ipgrd+5*m), &
                         swork(ipgrd+6*m), swork(ipgrd+7*m), &
                         swork(ipgrd+8*m), swork(iparry), m, ix)
987      end if
         if ( ndimlc==2 ) then

!        --- Calculate u_x, u_y, v_x, v_y
!            perform call for u_x

            call el6051(swork(ipgrd), swork(iworklc2), swork(ipold), &
                        swork(ipd1),  swork(ip1),    swork(ip2), &
                        swork(ip3),   swork(ip4),    swork(ipdet), &
                        swork(iworklc), mone, 1)

!           --- perform call for u_y

            call el6051(swork(ipgrd+m), swork(iworklc2), swork(ipold), &
                        swork(ipd1),    swork(ip1),    swork(ip2), &
                        swork(ip3),     swork(ip4),    swork(ipdet), &
                        swork(iworklc),   mone, 2)

!           --- perform call for v_x

            call el6051(swork(ipgrd+2*m), swork(iworklc2), swork(ipold+m), &
                        swork(ipd1),      swork(ip1),    swork(ip2), &
                        swork(ip3),       swork(ip4),    swork(ipdet), &
                        swork(iworklc),     mone, 1)

!           --- perform call for v_y

            call el6051(swork(ipgrd+3*m), swork(iworklc2), swork(ipold+m), &
                        swork(ipd1),      swork(ip1),    swork(ip2), &
                        swork(ip3),       swork(ip4),    swork(ipdet), &
                        swork(iworklc),     mone, 2)

!           --- material labels (swork(ipold+6*m)) added

            call el6063( ix, elemvc1, swork(ipold+2*m), &
                         swork(ipold+3*m), swork(ipold+4*m), &
                         swork(ipgrd),     swork(ipgrd+m), &
                         swork(ipgrd+2*m), swork(ipgrd+3*m), &
                         swork(ipw1), swork(ipdet), &
                         swork(iworklc),swork(iworklc2), &
                         swork(ipold+5*m), swork(ipold+6*m), &
                         mone, m )

!           --- Mark that due to the go to it is impossible to reach the next
!               statement

            call el6060 ( ix, elemvc1,    swork(ipold+2*m), &
                       swork(ipold+3*m), swork(ipold+4*m), &
                       swork(ipold),     swork(ipold+m), &
                       swork(ipgrd),     swork(ipgrd+m), &
                       swork(ipgrd+2*m), swork(ipgrd+3*m), &
                       swork(ipw1),      swork(ipd1),  swork(ipdet), &
                       swork(ip1),       swork(ip2),   swork(ip3), &
                       swork(ip4),       swork(iworklc), swork(iworklc2), &
                       user, iuser,  mone, m, 1 )

!           --- first component  : 2 eta dudx (weak)
!               second component : eta ( dudy  + dvdx) (weak)
!               third component  : 2 eta dudx (weak)
!               Note : swork(ipgrd**) is here used to store
!                   the gradients weak !!
!               compute dudx (weak)

            jadd = 0
            call el6074( swork(ipgrd),  swork(ipold),   swork(ipw1), &
                         swork(ipd1),   swork(ip1),     swork(ip2), &
                         swork(ip3),    swork(ip4), &
                         swork(iworklc2), mone, m,      1)

!           --- compute dudy (weak)

            jadd = 0
            call el6074( swork(ipgrd+m), swork(ipold),   swork(ipw1), &
                         swork(ipd1),    swork(ip1),     swork(ip2), &
                         swork(ip3),     swork(ip4), &
                         swork(iworklc2), mone, m,      2)

!           --- compute dvdx (weak)

            jadd = 0
            call el6074( swork(ipgrd+2*m), swork(ipold+m), swork(ipw1), &
                         swork(ipd1),    swork(ip1),     swork(ip2), &
                         swork(ip3),     swork(ip4), &
                         swork(iworklc2), mone, m,      1)

!           --- compute dvdy (weak)

            jadd = 0
            call el6074( swork(ipgrd+3*m), swork(ipold+m), swork(ipw1), &
                         swork(ipd1),    swork(ip1),     swork(ip2), &
                         swork(ip3),     swork(ip4), &
                         swork(iworklc2), mone, m,      2)

!           --- compose full rhs vector, from four gradients
!               and the viscosity

            call el6005( elemvc1, swork(ipgrd), swork(ipgrd+m), &
                         swork(ipgrd+2*m), swork(ipgrd+3*m), &
                         swork(iparry), m, ix)
988      end if

      end if

      if ( icheld==39 ) then

         if ( ndimlc==3 ) then
            if ( iuser(1)>iuser(2) ) then
               call el6221(coorloc, index1, swork(ipx), &
                           user,iuser,mone, m)
            end if
         end if

         if ( ndimlc==2 ) then

            if ( iuser(1)>iuser(2) ) then
               call el6213(coorloc, index1, swork(ipx), &
                            user, iuser, mone, m)
            end if
         end if

         if ( ndimlc==1 ) then
            call errchr ('Element interpolation not possible in 1D', 1 )
            call errsub (1682, 0, 0, 1 )
            go to 1000
         end if

      end if

      if ( icheld==40 ) then

!     --- iuser(1) is # of interpolation points
!         iuser(2) is # of points done

         if ( ndimlc==3 ) then
            if ( iuser(1)>iuser(2) ) then
               call el6220(coorloc, index1, swork(ipx), swork(ipold), &
                           swork(ip1), swork(ip2), swork(ip3), &
                           user,iuser,numold, mone, m)
            end if
         end if
         if ( ndimlc==2 ) then

!        --- iuser(1) is # of interpolation points
!            iuser(2) is # of points done

            if ( iuser(1)>iuser(2) ) then
               call el6212(coorloc, index1, swork(ipx), swork(ipold), &
                           swork(ip1), swork(ip2), &
                           user, iuser, numold, mone, m)
            end if
         end if
         if ( ndimlc==1 ) then
            call errchr ( 'Interpolation not possible in 1D', 1 )
            call errsub ( 1682, 0, 0, 1 )
            go to 1000
         end if

      end if

!     --- swap in the case of linear elements

      if ( mone==2.and.ndimlc>1 ) call elvswap( elemvc1, m, ndimlc )

1000  call erclos ( 'eld600' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( elemvc1, m, 'elemvc1' )

         write(irefwr,*) 'End eld600'

      end if  ! ( debug )

      end
