      subroutine eld607 ( coorloc, iuser, user, swork )
! ======================================================================
!
!        programmer    Peter Minev
!        version  2.9  date 09-03-2013 Replace elflr7 by elflr71
!        version  2.8  date 12-12-2012 Long integers
!        version  2.7  date 18-09-2002 New call to el3007
!
!   copyright (c) 1994-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     computes the convection residual in the case of an
!     explicitly performed convection; initial purpose is
!     to be used for explicit-implicit treatment of scalar
!     convection-diffusion equation;
!     1-D, 2-D and 3-D case
!     ELD607 is a help subroutine for subroutine DERIV
!     it is called through the intermediate subroutine ELD000
!     so:
!     DERIV
!       -  loop over spectral elements
!          -  ELD000
!             -  ELD607
!     itype = 607  :   three-dimensional spectral element
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecactl
      use sepmoduletime
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*)
      double precision coorloc(*), user(*), swork(*)

!     coorloc     i     array of length npoint containing the coorlocdinates of
!                    the nodal points with respect to the global numbering
!                    x  = coorloc (1,i); y  = coorloc(2,i)
!                     i                i
!     iuser    i     integer user array to pass user information from
!                    main program to subroutine, see STANDARD PROBLEMS
!     swork          spectral element work array
!                    total length:  (15+ndim) * m + (2+mone) * mone
!     user     i     real user array to pass user information from
!                    main program to subroutine, see STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipx1re,ipx,ipw1,ipd1, ipd2,ipdet, jelp, i, lowork, &
              ip1, ip2, ip3, ip4, ip5, ip6, ip7, ip8, ip9, ipold
      double precision rdum
      save ipx1re,ipx,ipw1,ipd1, ipdet, jelp, &
           ip1, ip2, ip3, ip4, ip5, ip6, ip7, ip8, ip9, ipold

!     i          Counting variable
!     ip1        indicate partial derivatives of the coorlocdinates due to
!                the isoparametric transformation, length m
!     ip2        See ip1
!     ip3        See ip1
!     ip4        See ip1
!     ip5        See ip1
!     ip6        See ip1
!     ip7        See ip1
!     ip8        See ip1
!     ip9        See ip1
!     ipd1     indicates the values of derivatives of the basisfunctions
!               in the interpolation points in the x-direction correspon- &
!               ding to the reference element, length mone x mone
!     ipd2
!     ipdet     transformation determinant, length m
!     ipold     indicates the old solutions in the nodal points, length m x 4
!     ipw1      indicates the Legendre Gauss-Lobatto weights for
!               numerical integration in the x-direction corresponding
!               to the reference element, length mone
!     ipx       indicates the coorlocdinates of the corresponding points
!               in the element, length m x 3
!     ipx1re    indicates the Legendre Gauss-Lobatto points in the
!               x-direction corresponding to the reference element [-1,1], &
!               length mone
!     jelp      choice parameter to distinguish between the types of element
!               subroutines called, possible values:
!               1: subroutine elp640 (one-dimensional)
!               (more values not yet done)
!     lowork    local work variable
!     rdum      dummy variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3007    Fills old solution and derivatives
!     EL6000    Performs initialization for subroutines ELx600
!     EL6011    compute the spectral element convection residual
!     EL6037    compute the convection residual in the case of spectral elements
!     EL6053    compute contributions to the spectral element vector
!               and the spectral element residual
!     EL6074    Compute the weak gradient of a scalar function (pressure)
!               for righthandside/residual of the time-dependent Navier-Stokes
!               equations; 2-D case.
!     EL6080    compute the spectral element divergence residual; 2-D
!     EL6081    computes vorticity; 2-D
!     EL6082    compute the residual of the first derivative
!     EL6083    compute contributions to the spectral element vector
!               corresponding to the righthand side of the equation in 3-D case
!     EL6091    compute the spectral element convection residual; 3-D
!     EL6092    compute the residual of the first derivative
!     EL6093    compute the spectral element divergence residual; 3-D
!     EL6094    computes gradient of divergence of velocity (weak); 3-D
!     EL6099    compute the residual of the first derivative
!     EL6206    calculates L2 norm of a vector; 2-D
!     EL6207    calculates L2 norm of a vector; 3-D
!     EL6208    calculates L2 norm of a vector; 1-D
!     EL6212    interpolates from a given vector in a given point; 2-D
!     EL6220    3D Interpolation of solution in an arbitrary point
!     ELD007    initialisations for subroutine ELD607
!     ELFLR71   Fill variable coefficients in array ARRAY
!     ELP640    compute basisfunctions, derivatives and weights for
!               numerical integration corresponding to the reference
!               element (element independent)
!     ELP642    compute the integration points, determinant and
!               partial derivatives, two-dimensional case
!               (element dependent)
!     ELP646    compute transformation determinant and
!               partial derivatives  in 3-D
!     ELP647    store integration points
!               transform weights and derivatives from the reference element
!     ELUSR3    Extracts information concerning the coefficients from IUSER
!               and USER.
!     ELVSWAP   swap parts of element vector in the the case of a linear
!               spectral element due to different nodal numbering
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     INSTOP    Stop the program
! **********************************************************************
!
!                       I/O
!
!     the user provides the coefficients of convection as old solutions
!     for the scalar
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      62   dimension of space (ndim) has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!                  METHOD
!     differential equation:
!          u . grad c + beta c = 0
!                              - &
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('eld607')

      if (ifirst==0) then

!     --- ifirst = 0, compute element independent quantities
!         perform initialisations, set iparry

         ipx1re = 1

         call eld007(jelp)

!        --- set pointers in the workarray

         ipx    = ipx1re + mone
         ipw1   = ipx + ndimlc*m
         ipd1  = ipw1 + mone
         ipd2   = ipd1 + mone*mone
         ipdet  = ipd2 + mone*mone
         ip1    = ipdet + m
         ip2    = ip1 + m
         ip3    = ip2 + m
         ip4    = ip3 + m
         ip5    = ip4 + m
         ip6    = ip5 + m
         ip7    = ip6 + m
         ip8    = ip7 + m
         ip9    = ip8 + m
         ipold  = ip9 + m

!        --- compute points and weights for numerical integration
!            compute derivatives of the basisfunctions
!            corresponding to the reference element

         call elp640(1, swork(ipx1re), swork(ipw1), swork(ipd1), &
                     swork(ipdet), mone)

!        --- ipdet is used as a dummy space

         if ( ierror/=0 ) go to 1000

      end if

!     --- compute integration points, determinant and partial derivatives

      if ( ndimlc==3 ) then

         call elp646(coorloc,index1,swork(ipx),swork(ipx+m),swork(ipx+2*m), &
                     swork(ipd1), swork(ipdet), swork(ip1), &
                     swork(ip2), swork(ip3), swork(ip4), &
                     swork(ip5), swork(ip6), swork(ip7), &
                     swork(ip8), swork(ip9), mone)

      else if ( ndimlc==2 ) then

         call elp642 ( coorloc, index1, swork(ipx), swork(ipd1), &
                       swork(ipdet), swork(ip1), swork(ip2), &
                       swork(ip3), swork(ip4), mone, m)

      else if ( ndimlc==1 ) then

         call elp647 ( coorloc, index1, swork(ipx), swork(ipdet), mone )

      else

         call errint ( ndimlc, 1 )
         call errsub ( 62, 1, 0, 0 )
         go to 1000

      end if

!     --- fill old solution  and the coefficients of convection, &
!         or velocity or pressure  as old solutions

      do i = 1, numold

         lowork = ipold + (i-1)*m
         call el3007 ( swork(lowork), rdum, rdum, rdum, m, m, &
                       ndimlc, ncomp, index1, rdum, &
                       swork(lowork), jtrans, numold, i )
      end do

      go to ( 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 ) icheld

!     --- compute the convection residual

1     if( ndimlc==3 ) then

         call el6091(swork(ipold+m), swork(ipold+2*m), swork(ipold+3*m), &
                     swork(ipw1), swork(ipd1), &
                     swork(ip1), swork(ip2), swork(ip3), &
                     swork(ip4), swork(ip5), swork(ip6), &
                     swork(ip7), swork(ip8), swork(ip9), &
                     mone, elemvc1, swork(ipold))
         go to 1000
      end if

      if( ndimlc==2 ) then

          jadd = 0
          call el6037(swork(ipold+m), swork(ipw1), &
                      swork(ipd1), swork(ip1), swork(ip2), &
                      swork(ip3), swork(ip4), swork(ip9), mone, m, &
                      elemvc1, swork(ipold))

!         --- ip9 is used as a work space

          go to 1000
      end if

      if( ndimlc==1 ) then

         jadd = 0
         call el6011( swork(ipold+m), elemvc1, swork(ipold), swork(ipw1), &
                      swork(ipd1), mone, swork(ip9))

!        --- ip9 is used as a work space

         go to 1000
      end if

2     if( ndimlc==3 ) then

         if(itimloop==1) then
            call el6099(elemvc1, swork(ipold), swork(ipw1), &
                        swork(ipd1), &
                        swork(ip1), swork(ip2), swork(ip3), &
                        swork(ip4), swork(ip5), swork(ip6), &
                        swork(ip7), swork(ip8), swork(ip9), &
                        mone, 1)
         else

!        --- swork(ipdet) is used as a work space

             call el6092( elemvc1, swork(ipold), swork(ipw1), &
                          swork(ipd1), swork(ipdet), &
                          swork(ip1), swork(ip2), swork(ip3), &
                          swork(ip4), swork(ip5), swork(ip6), &
                          swork(ip7), swork(ip8), swork(ip9), &
                          mone, 1)
         end if
         go to 1000
      end if

      if( ndimlc==2 ) then

         jadd = 0
         if( itimloop==1 ) then
             call el6074( elemvc1, swork(ipold), swork(ipw1),swork(ipd1), &
                   swork(ip1), swork(ip2), swork(ip3), swork(ip4), &
                   swork(ip9), mone, m, 1)

!            --- ip9 is used as a work space

         else
            call el6082( elemvc1, swork(ipold), swork(ipw1), swork(ipd1), &
                   swork(ip1), swork(ip2), swork(ip3), swork(ip4), &
                   swork(ip9), mone, m, 1)
         end if
         go to 1000
      end if

      if( ndimlc==1 ) then
          write(*,*) ' This possibility is not available in 1-D case'
          call instop
      end if

!     --- compute the second component of the gradient (only 3-D)

3     if( ndimlc==3 ) then

         call el6099(elemvc1, swork(ipold), swork(ipw1), &
                     swork(ipd1), &
                     swork(ip1), swork(ip2), swork(ip3), &
                     swork(ip4), swork(ip5), swork(ip6), &
                     swork(ip7), swork(ip8), swork(ip9), &
                     mone, 2)
         go to 1000
      end if

      if( ndimlc==2 ) then
         write(*,*) ' This possibility is not available in 2-D case'
         call instop
      end if

      if( ndimlc==1 ) then
          write(*,*) ' This possibility is not available in 1-D case'
          call instop
      end if

!     --- compute the third component of the gradient (only 2-D and 3-D)

4     if( ndimlc==3 ) then
         if(itimloop==1) then
            call el6099(elemvc1, swork(ipold), swork(ipw1), &
                    swork(ipd1), &
                    swork(ip1), swork(ip2), swork(ip3), &
                    swork(ip4), swork(ip5), swork(ip6), &
                    swork(ip7), swork(ip8), swork(ip9), &
                    mone, 3)

         else

!        --- swork(ipdet) is used as a work space

            call el6092( elemvc1, swork(ipold), swork(ipw1), &
                      swork(ipd1), swork(ipdet), &
                      swork(ip1), swork(ip2), swork(ip3), &
                      swork(ip4), swork(ip5), swork(ip6), &
                      swork(ip7), swork(ip8), swork(ip9), &
                      mone, 3)
         end if

         go to 1000

      end if

      if( ndimlc==2 ) then
         jadd = 0
         if( itimloop==1 ) then
            call el6074( elemvc1, swork(ipold), swork(ipw1), swork(ipd1), &
                   swork(ip1), swork(ip2), swork(ip3), swork(ip4), &
                   swork(ip9), mone, m, 2)

!           --- ip9 is used as a work space

         else
            call el6082( elemvc1, swork(ipold), swork(ipw1), swork(ipd1), &
                   swork(ip1), swork(ip2), swork(ip3), swork(ip4), &
                   swork(ip9), mone, m, 2)
         end if
         go to 1000
      end if

      if( ndimlc==1 ) then
         write(*,*) ' This possibility is not available in 1-D case'
         call instop
      end if

!     --- compute the divergence of the velocity

5     if( ndimlc==3 ) then
         call el6093(swork(ipold), swork(ipold+m), swork(ipold+2*m), &
                     swork(ipw1), swork(ipd1), &
                   swork(ip1), swork(ip2), swork(ip3), &
                   swork(ip4), swork(ip5), swork(ip6), &
                   swork(ip7), swork(ip8), swork(ip9), &
                   mone, elemvc1)
         go to 1000
      end if

      if( ndimlc==2 ) then
         jadd = 0
         call el6080(elemvc1, swork(ipold), swork(ipw1), &
                     swork(ipd1), swork(ip1), &
                     swork(ip2), swork(ip3), swork(ip4), &
                     swork(ip9), mone, m)

!        --- ip9 is used as a work space

         go to 1000
      end if

      if( ndimlc==1 ) then
          write(*,*) ' This possibility is not available in 1-D case'
          call instop
      end if

!     --- compute the gradient of divergence of the velocity
!         first component
!         swork(ipold+3*m) is used temporarly as a work array

6     if( ndimlc==3 ) then
         call el6094(swork(ipold), swork(ipold+m), swork(ipold+2*m), &
                   swork(ipold+3*m), swork(ipw1), swork(ipd1), &
                   swork(ip1), swork(ip2), swork(ip3), &
                   swork(ip4), swork(ip5), swork(ip6), &
                   swork(ip7), swork(ip8), swork(ip9), &
                   swork(ipdet), mone, elemvc1, 1)
         go to 1000

      else
          write(*,*) 'This possibility is not available in 1-D,2-d case'
          call instop
      end if

!     --- compute the gradient of divergence of the velocity
!         second component
!         swork(ipold+3*m) is used temporarly as a work array

7     if( ndimlc==3 ) then
         call el6094(swork(ipold), swork(ipold+m), swork(ipold+2*m), &
                   swork(ipold+3*m), swork(ipw1), swork(ipd1), &
                   swork(ip1), swork(ip2), swork(ip3), &
                   swork(ip4), swork(ip5), swork(ip6), &
                   swork(ip7), swork(ip8), swork(ip9), &
                   swork(ipdet), mone, elemvc1, 2)
        go to 1000

      else
          write(*,*) 'This possibility is not available in 1-D,2-D case'
          call instop
      end if

!     --- compute the gradient of divergence of the velocity
!         third component
!         swork(ipold+3*m) is used temporarly as a work array

8     if( ndimlc==3 ) then
         call el6094(swork(ipold), swork(ipold+m), swork(ipold+2*m), &
                   swork(ipold+3*m), swork(ipw1), swork(ipd1), &
                   swork(ip1), swork(ip2), swork(ip3), &
                   swork(ip4), swork(ip5), swork(ip6), &
                   swork(ip7), swork(ip8), swork(ip9), &
                   swork(ipdet), mone, elemvc1, 3)
        go to 1000

      else
          write(*,*) 'This possibility is not available in 1-D,2-D case'
          call  instop
      end if

!     --- compute the vorticity (only 2-D)

9     if( ndimlc==2 ) then
         jadd = 0
         call el6081(elemvc1, swork(ipold), swork(ipw1), &
                     swork(ipd1), swork(ip1), &
                     swork(ip2), swork(ip3), swork(ip4), &
                     swork(ip9), mone, m)

!        --- ip9 is used as a work space

         go to 1000
      end if

      if( ndimlc==1.or.ndimlc==3 ) then
          write(*,*) 'This possibility is not available in 1-D,3-D case'
          call  instop
      end if

!     --- compute a source term
!         first load the cofficient in swork(ipold); swork(ip7,8,9)
!         is used as a work space

10    if( jconsf==0 ) then

!     --- set the vector to zero

         do i = 1, m
            elemvc1(i) = 0d0
         end do

         go to 1000

      else

!     --- compute the source
!         load non-constant coefficient

         if( jconsf==2 ) &
            call elflr71(2, user, swork(ipold-m), &
                   swork(ipx), swork(ipx), m, m, rdum, &
                   1, index1,  iuser, swork(ip8), swork(ip7))

!        --- load constant coefficient

         if( jconsf==1 ) &
             call elusr3 ( 2, swork(ipold), iuser, user, 1)

         jadd = 0

         if( ndimlc==3 ) then
            call el6083( swork(ipold), elemvc1, swork(ipw1), &
                         swork(ipdet), mone, m)
            go to 1000
         end if

         if( ndimlc==2 ) then
            call el6053(swork(ipold), elemvc1, swork(ipw1), &
                        swork(ipdet), mone, m)
            go to 1000
         end if

         if( ndimlc==1 ) then
            call el6000(swork(ipold), elemvc1, swork(ipw1), &
                        swork(ipdet),m)
            go to 1000
         end if

      end if

11    if (ndimlc==2) then

!     --- iuser(1) is # of interpolation points
!         iuser(2) is # of points done

         if (iuser(1)>iuser(2) ) then
            call el6212(coorloc, index1, swork(ipx), swork(ipold), &
                        swork(ip1), swork(ip2), &
                        user, iuser, numold, mone, m)
         end if
         go to 1000
      end if
      if (ndimlc==3) then
         if (iuser(1)>iuser(2) ) then
            call el6220(coorloc, index1, swork(ipx), swork(ipold), &
                        swork(ip1),swork(ip2),swork(ip3), &
                        user,iuser,numold, mone, m)
         end if
         go to 1000
      end if
      if (ndimlc==1) print *, 'Interpolation not possible in 1D'
      goto 1000

12    if (ndimlc==3) &
         call el6207(coorloc, index1, swork(ipx), swork(ipx+m), &
                     swork(ipx+2*m), swork(ipd1), swork(ipdet), &
                     swork(ip1), swork(ip2), swork(ip3), swork(ip4), &
                     swork(ip5), swork(ip6), swork(ip7), &
                     swork(ip8), swork(ip9), swork(ipold), &
                     swork(ipx1re), swork(ipw1), swork(ipd1), &
                     swork(ipd2), user, mone, m )
      if (ndimlc==2) &
         call el6206(coorloc, index1, swork(ipx), swork(ipd1), &
                     swork(ipdet), swork(ip1), swork(ip2), &
                     swork(ip3), swork(ip4), swork(ipx1re), &
                     swork(ipw1),swork(ipd1),swork(ip5), &
                     swork(ipold),user,mone, m)
      if (ndimlc==1) &
         call el6208(coorloc, index1, swork(ipx), swork(ipold), &
                     swork(ipx1re), swork(ipw1), swork(ipd1), &
                     swork(ipdet),user,mone,m)
      goto 1000

!     --- swap in the case of linear elements

1000  if( mone==2.and.ndimlc>1 ) call elvswap( elemvc1, m, ndimlc)

      call erclos('eld607')

      end
