      subroutine eldiscint1mush ( massmt, stiffmt, dldtmt, latentvc, &
                                  dphidx, temperature, coefarr, m, n, &
                                  intrule, phi, h, ind, mloc )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 30-05-2006
!
!   copyright (c) 2005-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Construct element mass matrix, stiffness matrix, derivative latent heat
!   matrix and latent heat vector using discontinous integration.
!   This is the mushy case.
!   The code differs from the isothermal case, since the dLdT matrix and
!   L-vector are computed differently.
!   This is the 1D version.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemushy
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          m, n, intrule, ind(2), mloc
      double precision temperature(*), coefarr(m,*), &
                       dphidx(n,m), massmt(n,n), h, phi(n,mloc), &
                       stiffmt(n,n), dldtmt(n,n), latentvc(n)

!     coefarr        i    Array containing the coefficients of the PDE
!                         Pos. (.,1)   ksolid
!                         Pos. (.,2)   kfluid
!                         Pos. (.,3)   rho
!                         Pos. (.,4)   csolid
!                         Pos. (.,5)   cfluid
!                         Pos. (.,6)   Q
!                         Pos. (.,7)   L
!                         Pos. (.,8)   Tmelt
!     dldtmt         o    Element dldt matrix
!     dphidx         i    derivatives of basis functions
!     ind            i    index array containing info on which node has
!                         minimum and which has maximum temperature
!     intrule       i     Numerical integration rule to be used for
!                         discontinuous integration
!                         1: Newton-Cotes, 1st order, 'substraction approach'
!                         2: Newton-Cotes, 2nd order, 'substraction approach'
!                         3: Newton-Cotes, 1st order, &
!                            'subdivision into 3 subtriangles'
!                         4: Newton-Cotes, 1st order, &
!                            'averaging of heat capacity'
!     latentvc       o    Element latent heat vector
!     m              i    number of integration points
!     massmt         o    Element mass matrix
!     n              i    number of basis functions
!     stiffmt        o    Element stiffness matrix
!     temperature    i    temperature vector in integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision rho, cs, cl, dh(mloc+1), latentheat, &
                       kl, ks, cm, km, &
                       phixs(mloc,mloc), &
                       phicut(n,mloc), &
                       tempcut(mloc+1)

!     cl             Specific heat of the liquid
!     cm             Specific heat of the mushy
!     cs             Specific heat of the solid
!     dh             partial length of edge
!     h              width of the element
!     kl             Diffusivity of the liquid
!     km             Diffusivity of the mushy
!     ks             Diffusivity of the solid
!     latentheat     latent heat
!     mloc           local copy of m modified for Newton-Cotes
!     phi            basis functions
!     phicut         phi matrix for intersected element
!     phixs          phi values in intersection point xs
!     rho            density rho
!     tempcut        Temperature in integration points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELDISCINT1M    1D discontinuous integration, integrals over mushy
!                    subelement
!     ELDISCINT1SL   1D discontinuous integration, integrals over solid
!                    or liquid subelement
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL2        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldiscint1mush' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldiscint1mush'
         write(irefwr,*) 'tsol, tliq', tsol, tliq
         write(irefwr,*) 'temp(1), temp(2)', temperature(1), &
                                             temperature(2)
         call prinrl2(dphidx, n, m, 'dphidx')
      end if

!     --- get parameters

      ks  = coefarr(1,1)
      kl  = coefarr(1,2)
      rho = coefarr(1,3)
      cs  = coefarr(1,4)
      cl  = coefarr(1,5)
      latentheat = coefarr(1,7)

      cm = 0.5d0*(cs+cl)
      km = 0.5d0*(ks+kl)

      if ( temperature(ind(1))>tsol .and. &
                                        temperature(ind(2))<tliq) then

!     --- element is fully mushy

         if (debug) print *, 'element is fully mushy'

         call eldiscint1m(m, n, intrule, rho, cm, km, latentheat, phi, &
                          dphidx, h, temperature, massmt, stiffmt, &
                          dldtmt, latentvc)

      else if ( temperature(ind(1))<tsol .and. &
                                    temperature(ind(2))>tliq ) then

         if (debug) then
            if (temperature(1)<tsol) then
               print *, '---S---|---M---|---L---'
            else
               print *, '---L---|---M---|---S---'
            end if
         end if

         dh(1) = h*(tsol - temperature(ind(1)))/ &
                            (temperature(ind(2))-temperature(ind(1)))
         dh(3) = h*(temperature(ind(2))-tliq)/ &
                            (temperature(ind(2))-temperature(ind(1)))
         dh(2) = h - dh(1) - dh(3)
         if (debug) print *, 'dh(1), dh(2), dh(3)', dh(1), dh(2), dh(3)

         phixs(ind(1),1) = 1d0 - dh(1)/h
         phixs(ind(2),1) = dh(1)/h

         phixs(ind(1),2) = dh(3)/h
         phixs(ind(2),2) = 1d0 - dh(3)/h

         phicut(ind(1),1) = phi(ind(1),ind(1))
         phicut(ind(2),1) = phi(ind(2),ind(1))
         phicut(ind(1),2) = phixs(ind(1),1)
         phicut(ind(2),2) = phixs(ind(2),1)

         tempcut(1) = temperature(ind(1))
         tempcut(2) = tsol
         call eldiscint1sl(m, n, intrule, rho, cs, ks, latentheat, &
                           phicut, dphidx, dh(1), tempcut, massmt, &
                           stiffmt, latentvc)
         if (debug)  call prinrl(latentvc, n, 'latenvc')

         phicut(ind(1),1) = phixs(ind(1),1)
         phicut(ind(2),1) = phixs(ind(2),1)
         phicut(ind(1),2) = phixs(ind(1),2)
         phicut(ind(2),2) = phixs(ind(2),2)

         tempcut(1) = tsol
         tempcut(2) = tliq

         call eldiscint1m(m, n, intrule, rho, cm, km, latentheat, &
                          phicut, dphidx, dh(2), tempcut, massmt, &
                          stiffmt, dldtmt, latentvc)

         if (debug)  call prinrl(latentvc, n, 'latenvc')
         phicut(ind(1),1) = phixs(ind(1),2)
         phicut(ind(2),1) = phixs(ind(2),2)
         phicut(ind(1),2) = phi(ind(1),ind(2))
         phicut(ind(2),2) = phi(ind(2),ind(2))

         tempcut(1) = tliq
         tempcut(2) = temperature(ind(2))

         call eldiscint1sl(m, n, intrule, rho, cl, kl, latentheat, &
                           phicut, dphidx, dh(3), tempcut, massmt, &
                           stiffmt, latentvc)

         if (debug)  call prinrl(latentvc, n, 'latenvc')
      else ! single intersection point

         if ( temperature(ind(1))<tsol ) then
            if (debug) print *, '---S---|---M---'

            dh(1) = h*(tsol - temperature(ind(1)))/ &
                            (temperature(ind(2))-temperature(ind(1)))

         else
            if (debug) print *, '---M---|---L---'

            dh(1) = h*(tliq - temperature(ind(1)))/ &
                               (temperature(ind(2))-temperature(ind(1)))

         end if
         dh(2) = h - dh(1)

         phixs(ind(1),1) = 1d0 - dh(1)/h
         phixs(ind(2),1) = dh(1)/h

         phicut(ind(1),1) = phi(ind(1),ind(1))
         phicut(ind(2),1) = phi(ind(2),ind(1))
         phicut(ind(1),2) = phixs(ind(1),1)
         phicut(ind(2),2) = phixs(ind(2),1)

         if ( temperature(ind(1))<tsol ) then

            tempcut(1) = temperature(ind(1))
            tempcut(2) = tsol

            call eldiscint1sl(m, n, intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, dh(1), tempcut, massmt, &
                              stiffmt, latentvc)

         else

            tempcut(1) = temperature(ind(1))
            tempcut(2) = tliq

            call eldiscint1m(m, n, intrule, rho, cm, km, latentheat, &
                             phicut, dphidx, dh(1), tempcut, massmt, &
                             stiffmt, dldtmt, latentvc)

         end if

         if (debug)  call prinrl(latentvc, n, 'latenvc')

         phicut(ind(1),1) = phixs(ind(1),1)
         phicut(ind(2),1) = phixs(ind(2),1)
         phicut(ind(1),2) = phi(ind(1),ind(2))
         phicut(ind(2),2) = phi(ind(2),ind(2))

         if ( temperature(ind(1))<tsol ) then

            tempcut(1) = tsol
            tempcut(2) = temperature(ind(2))
            call eldiscint1m(m, n, intrule, rho, cm, km, latentheat, &
                             phicut, dphidx, dh(2), tempcut, massmt, &
                             stiffmt, dldtmt, latentvc)

         else

            tempcut(1) = tliq
            tempcut(2) = temperature(ind(2))

            call eldiscint1sl(m, n, intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, dh(2), tempcut, massmt, &
                              stiffmt, latentvc)

         end if

         if (debug)  call prinrl(latentvc, n, 'latenvc')
      end if ! element not fully mushy

      call erclos ( 'eldiscint1mush' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eldiscint1mush'

      end if

      end
