      subroutine eldiscint2 ( massmt, stiffmt, dldtmt, latentvc, &
                              dphidx, temperature, coefarr, x, &
                              intrule )
! ======================================================================
!
!        programmer    John Brusche
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 16-06-2010 New parameter list
!        version  1.2  date 09-11-2005 Fixed error in computation of phi_cut2
!                                      method for quadratic terms (mass/dldt)
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Construct element mass matrix, stiffness matrix, derivative latent heat
!   matrix and latent heat vector using discontinous integration.
!   This is the 2D version.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmodulemushy
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          intrule
      double precision temperature(*), coefarr(m,*), massmt(n,n), &
                       dphidx(n,m,ndim), x(n,ndim), &
                       latentvc(n), stiffmt(n,n), dldtmt(n,n)

!     coefarr        i    Array containing the coefficients of the PDE
!                         Pos. (.,1)   k_solid
!                         Pos. (.,2)   k_fluid
!                         Pos. (.,3)   rho
!                         Pos. (.,4)   c_solid
!                         Pos. (.,5)   c_fluid
!                         Pos. (.,6)   Q
!                         Pos. (.,7)   L
!                         Pos. (.,8)   T_melt
!     dldtmt         o    Element dldt matrix
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     intrule       i    Numerical integration rule to be used for
!                         discontinuous integration
!                         1: Newton-Cotes, 1st order, 'substraction approach'
!                         2: Newton-Cotes, 2nd order, 'substraction approach'
!                         3: Newton-Cotes, 1st order, &
!                            'subdivision into 3 subtriangles'
!                         4: Newton-Cotes, 1st order, &
!                            'averaging of heat capacity'
!     latentvc       o    Element latent heat vector
!     massmt         o    Element mass matrix
!     stiffmt        o    Element stiffness matrix
!     temperature    i    temperature vector in integration points
!     x              i    x-coordinates of the integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mloc
      parameter ( mloc = 3 )
      integer i, j
      double precision phi(n, mloc), dphidxcut(n, ndim)

!     dphidxcut      Locally adapted dphidx
!     i              Counting variable
!     j              Counting variable
!     mloc           local copy of m modified for Newton-Cotes
!     phi            array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                    phi_i(xg^k) = phi (i,k)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELDISCINT2ISO  2D discontinuous integration, isothermal case
!     ELDISCINT2MUSH 2D discontinuous integration, artificial mushy case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldiscint2' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldiscint2'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Initialize

      do i = 1, n
         do j = 1, n
            stiffmt(i,j) = 0d0
            massmt(i,j) = 0d0
            dldtmt(i,j) = 0d0
         end do
            latentvc(i) = 0d0
      end do

!     --- fill local phi

      do i = 1, n
         do j = 1, mloc
            if (i==j) then
               phi(i,j) = 1d0
            else
               phi(i,j) = 0d0
            end if
         end do
      end do

      do i = 1, n
         do j = 1, ndim
            dphidxcut(i,j) = dphidx(i,1,j)
         end do
      end do

      if (mushy) then

         call eldiscint2mush ( massmt, stiffmt, dldtmt, latentvc, &
                               dphidxcut, temperature, coefarr, &
                               intrule, phi, mloc, x )

      else ! isothermal

         call eldiscint2iso ( massmt, stiffmt, dldtmt, latentvc, &
                              dphidx, temperature, coefarr, &
                              intrule, phi, mloc, x )

      end if ! (mushy)

1000  call erclos ( 'eldiscint2' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eldiscint2'

      end if

      end
