      subroutine eldiscint2mush ( massmt, stiffmt, dldtmt, latentvc, &
                                  dphidx, temperature, coefarr, &
                                  intrule, phi, mloc, x )
! ======================================================================
!
!        programmer    John Brusche
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 30-05-2006
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Construct element mass matrix, stiffness matrix, derivative latent heat
!   matrix and latent heat vector using discontinous integration.
!   This is the mushy case.
!   The code differs from the isothermal case, since the dLdT matrix and
!   L-vector are computed differently.
!   This is the 1D version.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmodulemushy
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          intrule, mloc
      double precision temperature(*), coefarr(m,*), x(n, ndim), &
                       dphidx(n,ndim), massmt(n,n), phi(n,mloc), &
                       stiffmt(n,n), dldtmt(n,n), latentvc(n)

!     coefarr        i    Array containing the coefficients of the PDE
!                         Pos. (.,1)   ksolid
!                         Pos. (.,2)   kfluid
!                         Pos. (.,3)   rho
!                         Pos. (.,4)   csolid
!                         Pos. (.,5)   cfluid
!                         Pos. (.,6)   Q
!                         Pos. (.,7)   L
!                         Pos. (.,8)   Tmelt
!     dldtmt         o    Element dldt matrix
!     dphidx         i    derivatives of basis functions
!     intrule       i    Numerical integration rule to be used for
!                         discontinuous integration
!                         1: Newton-Cotes, 1st order, 'substraction approach'
!                         2: Newton-Cotes, 2nd order, 'substraction approach'
!                         3: Newton-Cotes, 1st order, &
!                            'subdivision into 3 subtriangles'
!                         4: Newton-Cotes, 1st order, &
!                            'averaging of heat capacity'
!     latentvc       o    Element latent heat vector
!     massmt         o    Element mass matrix
!     stiffmt        o    Element stiffness matrix
!     temperature    i    temperature vector in integration points
!     x              i    x-coordinates of the integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nsol, nliq, nmush, xsol(2), xliq(2), xmush(3), &
              nx(3), inttype
      double precision rho, cs, cl, latentheat, det, nsign, &
                       kl, ks, cm, kmushy, v(3, mloc), h(2), dh(2), &
                       phixs(n,3), xs(2,2), phicut(n,3), &
                       tempcut(3)

!     cl             Specific heat of the liquid
!     cm             Specific heat of the mushy
!     cs             Specific heat of the solid
!     det            determinant
!     dh             partial length of edge
!     h              width of the element
!     i              Counting variable
!     inttype        type of intersection
!     j              Counting variable
!     kl             Diffusivity of the liquid
!     kmushy             Diffusivity of the mushy
!     ks             Diffusivity of the solid
!     latentheat     latent heat
!     mloc           local copy of m modified for Newton-Cotes
!     nliq           number of liquid nodes
!     nmush          number of mushy nodes
!     nsign          determines whether contribution is added or subtracted
!     nsol           number of solid nodes
!     nx             new/local numbering of the nodes
!     phi            basis functions
!     phicut         phi matrix for intersected element
!     phixs          phi values in intersection point xs
!     rho            density rho
!     tempcut        local variant of temperature
!     v              help matrix to determine determinant
!     xliq           node numbers of liquid nodes
!     xmush          node numbers of mushy nodes
!     xs             coordinates of intersection points
!     xsol           node numbers of solid nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ercdet

!     ELDISCINT2M    2D discontinuous integration, integrals over mushy
!                    subelement
!     ELDISCINT2SL   2D discontinuous integration, integrals over solid
!                    or liquid subelement
!     ERCDET         Compute a 3x3 determinant
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INSTOP         Stop the program
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldiscint2mush' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldiscint2mush'
         write(irefwr,*) 'tsol, tliq', tsol, tliq
         call prinrl(temperature, 3, 'temperature')
      end if

!     --- get parameters

      ks  = coefarr(1,1)
      kl  = coefarr(1,2)
      rho = coefarr(1,3)
      cs  = coefarr(1,4)
      cl  = coefarr(1,5)
      latentheat = coefarr(1,7)

      cm = 0.5d0*(cs+cl)
      kmushy = 0.5d0*(ks+kl)

      nsol = 0
      nliq = 0
      nmush = 0

!     --- determine type of intersection and fix node numbering

      do i = 1, mloc
         if (temperature(i)<tsol) then
            nsol = nsol + 1
            xsol(nsol) = i
         else if (temperature(i)>tliq) then
            nliq = nliq + 1
            xliq(nliq) = i
         else
            nmush = nmush + 1
            xmush(nmush) = i
         end if
      end do

      if (nmush==3) then
         inttype = 1 ! fully mushy
         do i = 1, mloc
            nx(i) = i
         end do
      else if (nmush==0) then
         inttype = 2 ! 2 liquid/solid, 1 solid/liquid node
         if (nsol==1) then
            nx(1) = xsol(1)
         else
            nx(1) = xliq(1)
         end if
         nx(2) = mod(nx(1), mloc) + 1
         nx(3) = mod(nx(1)+1, mloc) + 1
      else if (nsol==nliq) then
         inttype = 3 ! 1 liquid node, 1 mushy node, 1 solid node
         nx(1) = xsol(1)
         nx(2) = xmush(1)
         nx(3) = xliq(1)
      else
         inttype = 4 ! liquid-mushy or solid mushy
         if (nsol==1) then
            nx(1) = xsol(1)
         else if (nliq==1) then
            nx(1) = xliq(1)
         else
            nx(1) = xmush(1)
         end if
         nx(2) = mod(nx(1), mloc) + 1
         nx(3) = mod(nx(1)+1, mloc) + 1
      end if

      if (debug) then
         print *, 'nx (new numbering)', nx(1), nx(2), nx(3)
         print *, 'intersection type', inttype
         print *, 'temperature', temperature(nx(1)), temperature(nx(2)), &
                          temperature(nx(3))
         print *, 'nsol, nliq, nmush', nsol, nliq, nmush

      end if

      do i = 1, mloc
         do j = 1, mloc
            phicut(nx(i), j) = phi(nx(i), nx(j))
         end do
      end do

      tempcut(1) = temperature(nx(1))
      tempcut(2) = temperature(nx(2))
      tempcut(3) = temperature(nx(3))

!     --- do i = 1, n
!            do j = 1, mloc
!               phicut(i,j) = phi(i,j)
!            end do
!         end do

!         --- compute matrices and vector on special element

      if (inttype==1) then ! fully mushy

!     --- compute det such that |det| = 2*surface whole element

         do i = 1, mloc
            v(1,i) = 1d0
            v(2,i) = x(i,1)
            v(3,i) = x(i,2)
         end do

         det = ercdet(v(1,1), v(1,2), v(1,3))
         det = abs(det)

      call eldiscint2m ( intrule, rho, cm, kmushy, latentheat, phicut, &
                          dphidx, det, tempcut, massmt, stiffmt, &
                          dldtmt, latentvc, 1d0, mloc)

      else if (inttype==2) then

!     --- compute det such that |det| = 2*surface whole element

         do i = 1, mloc
            v(1,i) = 1d0
            v(2,i) = x(i,1)
            v(3,i) = x(i,2)
         end do

         det = ercdet(v(1,1), v(1,2), v(1,3))
         det = abs(det)

         if (nsol==1) then

            call eldiscint2sl( intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, det, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .true.)

         else

            call eldiscint2sl( intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, det, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .false.)

         end if

!        --- compute lengths of intersected edges

         do i = 1, 2
            h(i) = sqrt((x(nx(1),1) - x(nx(i+1),1))**2 + &
                     (x(nx(1),2) - x(nx(i+1),2))**2  )
            if (debug) write(irefwr,*) 'h ', h(i)
         end do

!        --- compute intersection positions

         do i = 1, 2
            if (nsol==1) then
               dh(i) = h(i)*(tsol - temperature(nx(1)))/ &
                       (temperature(nx(i+1)) - temperature(nx(1)))
            else
               dh(i) = h(i)*(tliq - temperature(nx(1)))/ &
                       (temperature(nx(i+1)) - temperature(nx(1)))
            end if

            if (debug) write(irefwr,*) 'dh ', dh(i)
         end do

!        --- compute coordinates of intersection points

         do i = 1, 2
            do j = 1, 2
               if ( abs(x(nx(1),j)-x(nx(i+1),j))< sqreps ) then
                  xs(i,j) = x(nx(1),j)
               else if (x(nx(1),j)<x(nx(i+1),j)) then
                  xs(i,j) = x(nx(1),j) + &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               else
                  xs(i,j) = x(nx(1),j) - &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               end if
            end do
         end do

!        --- compute det such that |det| = 2*surface of subtriangle

         do i = 1, mloc
            v(1,i) = 1d0
         end do
         do i = 1, 2
            v(i+1,1) = x(nx(1),i)
            do j = 1, 2
               v(i+1,j+1) = xs(j,i)
            end do
         end do

         det = ercdet(v(1,1), v(1,2), v(1,3))
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

!        --- construct phicut

         phixs(nx(1), 1) = 1d0 - dh(1)/h(1)
         phixs(nx(1), 2) = 1d0 - dh(2)/h(2)
         phixs(nx(2), 1) = dh(1)/h(1)
         phixs(nx(2), 2) = 0d0
         phixs(nx(3), 1) = 0d0
         phixs(nx(3), 2) = dh(2)/h(2)

         phicut(nx(1), 1) = phi(nx(1),nx(1))
         phicut(nx(1), 2) = phixs(nx(1),1)
         phicut(nx(1), 3) = phixs(nx(1),2)
         phicut(nx(2), 1) = phi(nx(2),nx(1))
         phicut(nx(2), 2) = phixs(nx(2),1)
         phicut(nx(2), 3) = phixs(nx(2),2)
         phicut(nx(3), 1) = phi(nx(3),nx(1))
         phicut(nx(3), 2) = phixs(nx(3),1)
         phicut(nx(3), 3) = phixs(nx(3),2)

         if (nsol==1) then
            tempcut(1) = temperature(nx(1))
            tempcut(2) = tsol
            tempcut(3) = tsol

            call eldiscint2sl( intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, det, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .false.)
         else
            tempcut(1) = temperature(nx(1))
            tempcut(2) = tliq
            tempcut(3) = tliq

            call eldiscint2sl( intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, det, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .true.)
         end if
         call eldiscint2m( intrule, rho, cm, kmushy, latentheat, &
                          phicut, dphidx, det, tempcut, massmt, &
                          stiffmt, dldtmt, latentvc, -1d0, mloc)

!        --- compute intersection positions

         do i = 1, 2
            if (nsol==1) then
               dh(i) = h(i)*(tliq - temperature(nx(1)))/ &
                       (temperature(nx(i+1)) - temperature(nx(1)))
            else
               dh(i) = h(i)*(tsol - temperature(nx(1)))/ &
                       (temperature(nx(i+1)) - temperature(nx(1)))

            end if

            if (debug) write(irefwr,*) 'dh ', dh(i)
         end do

!        --- compute coordinates of intersection points

         do i = 1, 2
            do j = 1, 2
               if ( abs(x(nx(1),j)-x(nx(i+1),j))< sqreps ) then
                  xs(i,j) = x(nx(1),j)
               else if (x(nx(1),j)<x(nx(i+1),j)) then
                  xs(i,j) = x(nx(1),j) + &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               else
                  xs(i,j) = x(nx(1),j) - &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               end if
            end do
         end do

!        --- compute det such that |det| = 2*surface of subtriangle

         do i = 1, mloc
            v(1,i) = 1d0
         end do
         do i = 1, 2
            v(i+1,1) = x(nx(1),i)
            do j = 1, 2
               v(i+1,j+1) = xs(j,i)
            end do
         end do

         det = ercdet(v(1,1), v(1,2), v(1,3))
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

         phixs(nx(1), 1) = 1d0 - dh(1)/h(1)
         phixs(nx(1), 2) = 1d0 - dh(2)/h(2)
         phixs(nx(2), 1) = dh(1)/h(1)
         phixs(nx(2), 2) = 0d0
         phixs(nx(3), 1) = 0d0
         phixs(nx(3), 2) = dh(2)/h(2)

         phicut(nx(1), 1) = phi(nx(1),nx(1))
         phicut(nx(1), 2) = phixs(nx(1),1)
         phicut(nx(1), 3) = phixs(nx(1),2)
         phicut(nx(2), 1) = phi(nx(2),nx(1))
         phicut(nx(2), 2) = phixs(nx(2),1)
         phicut(nx(2), 3) = phixs(nx(2),2)
         phicut(nx(3), 1) = phi(nx(3),nx(1))
         phicut(nx(3), 2) = phixs(nx(3),1)
         phicut(nx(3), 3) = phixs(nx(3),2)

         if (nsol==1) then

            tempcut(1) = temperature(nx(1))
            tempcut(2) = tliq
            tempcut(3) = tliq
            call eldiscint2sl( intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, det, massmt, &
                              stiffmt, latentvc, -1d0, mloc, .true.)
         else

            tempcut(1) = temperature(nx(1))
            tempcut(2) = tsol
            tempcut(3) = tsol
            call eldiscint2sl( intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, det, massmt, &
                              stiffmt, latentvc, -1d0, mloc, .false.)

         end if
         call eldiscint2m( intrule, rho, cm, kmushy, latentheat, &
                          phicut, dphidx, det, tempcut, massmt, &
                          stiffmt, dldtmt, latentvc, 1d0, mloc)

      else if (inttype==3) then

!     --- compute det such that |det| = 2*surface whole element

         do i = 1, mloc
            v(1,i) = 1d0
            v(2,i) = x(i,1)
            v(3,i) = x(i,2)
         end do

         det = ercdet(v(1,1), v(1,2), v(1,3))
         det = abs(det)

!        --- compute integrals over whole element

         call eldiscint2m( intrule, rho, cm, kmushy, latentheat, &
                          phicut, dphidx, det, tempcut, massmt, &
                          stiffmt, dldtmt, latentvc, 1d0, mloc)

!        --- compute lengths of intersected edges

         do i = 1, 2
            h(i) = sqrt((x(nx(1),1) - x(nx(i+1),1))**2 + &
                     (x(nx(1),2) - x(nx(i+1),2))**2  )
            if (debug) write(irefwr,*) 'h ', h(i)
         end do

!        --- compute intersection positions

         do i = 1, 2
            dh(i) = h(i)*(tsol - temperature(nx(1)))/ &
                       (temperature(nx(i+1)) - temperature(nx(1)))
            if (debug) write(irefwr,*) 'dh ', dh(i)
         end do

!        --- compute coordinates of intersection points

         do i = 1, 2
            do j = 1, 2
               if ( abs(x(nx(1),j)-x(nx(i+1),j))< sqreps ) then
                  xs(i,j) = x(nx(1),j)
               else if (x(nx(1),j)<x(nx(i+1),j)) then
                  xs(i,j) = x(nx(1),j) + &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               else
                  xs(i,j) = x(nx(1),j) - &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               end if
            end do
         end do

!        --- compute det such that |det| = 2*surface of subtriangle

         do i = 1, mloc
            v(1,i) = 1d0
         end do
         do i = 1, 2
            v(i+1,1) = x(nx(1),i)
            do j = 1, 2
               v(i+1,j+1) = xs(j,i)
            end do
         end do

         det = ercdet(v(1,1), v(1,2), v(1,3))
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

         phixs(nx(1), 1) = 1d0 - dh(1)/h(1)
         phixs(nx(1), 2) = 1d0 - dh(2)/h(2)
         phixs(nx(2), 1) = dh(1)/h(1)
         phixs(nx(2), 2) = 0d0
         phixs(nx(3), 1) = 0d0
         phixs(nx(3), 2) = dh(2)/h(2)

         phicut(nx(1), 1) = phi(nx(1),nx(1))
         phicut(nx(1), 2) = phixs(nx(1),1)
         phicut(nx(1), 3) = phixs(nx(1),2)
         phicut(nx(2), 1) = phi(nx(2),nx(1))
         phicut(nx(2), 2) = phixs(nx(2),1)
         phicut(nx(2), 3) = phixs(nx(2),2)
         phicut(nx(3), 1) = phi(nx(3),nx(1))
         phicut(nx(3), 2) = phixs(nx(3),1)
         phicut(nx(3), 3) = phixs(nx(3),2)

!        --- subtract mushy integral over solid subtriangle and add
!            integral over solid

         tempcut(1) = temperature(nx(1))
         tempcut(2) = tsol
         tempcut(3) = tsol

         call eldiscint2m( intrule, rho, cm, kmushy, latentheat, &
                          phicut, dphidx, det, tempcut, massmt, &
                          stiffmt, dldtmt, latentvc, -1d0, mloc)
         call eldiscint2sl( intrule, rho, cs, ks, latentheat, &
                           phicut, dphidx, det, massmt, &
                           stiffmt, latentvc, 1d0, mloc, .false.)

!        --- call prinrl(latentvc, n, 'latentvc 2')
!            --- compute lengths of intersected edges

         do i = 1, 2
            h(i) = sqrt((x(nx(3),1) - x(nx(i),1))**2 + &
                     (x(nx(3),2) - x(nx(i),2))**2  )
            if (debug) write(irefwr,*) 'h ', h(i)
         end do

!        --- compute intersection positions

         do i = 1, 2
            dh(i) = h(i)*(tliq - temperature(nx(3)))/ &
                       (temperature(nx(i)) - temperature(nx(3)))
            if (debug) write(irefwr,*) 'dh ', dh(i)
         end do

!        --- compute coordinates of intersection points

         do i = 1, 2
            do j = 1, 2
               if ( abs(x(nx(3),j)-x(nx(i),j))< sqreps ) then
                  xs(i,j) = x(nx(3),j)
               else if (x(nx(3),j)<x(nx(i),j)) then
                  xs(i,j) = x(nx(3),j) + &
                        abs(x(nx(3),j)-x(nx(i),j))*dh(i)/h(i)
               else
                  xs(i,j) = x(nx(3),j) - &
                        abs(x(nx(3),j)-x(nx(i),j))*dh(i)/h(i)
               end if
            end do
         end do

!        --- compute det such that |det| = 2*surface of subtriangle

         do i = 1, mloc
            v(1,i) = 1d0
         end do
         do i = 1, 2
            v(i+1,1) = x(nx(3),i)
            do j = 1, 2
               v(i+1,j+1) = xs(j,i)
            end do
         end do

         det = ercdet(v(1,1), v(1,2), v(1,3))
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

         phixs(nx(1), 1) = dh(1)/h(1)
         phixs(nx(1), 2) = 0d0
         phixs(nx(2), 1) = 0d0
         phixs(nx(2), 2) = dh(2)/h(2)
         phixs(nx(3), 1) = 1d0 - dh(1)/h(1)
         phixs(nx(3), 2) = 1d0 - dh(2)/h(2)

         phicut(nx(1), 1) = phi(nx(1),nx(3))
         phicut(nx(1), 2) = phixs(nx(1),1)
         phicut(nx(1), 3) = phixs(nx(1),2)
         phicut(nx(2), 1) = phi(nx(2),nx(3))
         phicut(nx(2), 2) = phixs(nx(2),1)
         phicut(nx(2), 3) = phixs(nx(2),2)
         phicut(nx(3), 1) = phi(nx(3),nx(3))
         phicut(nx(3), 2) = phixs(nx(3),1)
         phicut(nx(3), 3) = phixs(nx(3),2)

!        --- subtract mushy integral over solid subtriangle and add
!            integral over solid

         tempcut(1) = temperature(nx(3))
         tempcut(2) = tliq
         tempcut(3) = tliq

         call eldiscint2m( intrule, rho, cm, kmushy, latentheat, &
                          phicut, dphidx, det, tempcut, massmt, &
                          stiffmt, dldtmt, latentvc, -1d0, mloc)
         call eldiscint2sl( intrule, rho, cl, kl, latentheat, &
                           phicut, dphidx, det, massmt, &
                           stiffmt, latentvc, 1d0, mloc, .true.)

      else if (inttype==4) then

!     --- compute det such that |det| = 2*surface whole element

         do i = 1, mloc
            v(1,i) = 1d0
            v(2,i) = x(i,1)
            v(3,i) = x(i,2)
         end do

         det = ercdet(v(1,1), v(1,2), v(1,3))
         det = abs(det)

!        --- compute integrals over whole element

         nsign = 1d0
         if (nsol==2) then

            call eldiscint2sl( intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, det, massmt, &
                              stiffmt, latentvc, nsign, mloc, .false.)

         else if (nliq==2) then

            call eldiscint2sl( intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, det, massmt, &
                              stiffmt, latentvc, nsign, mloc, .true.)

         else

            call eldiscint2m( intrule, rho, cm, kmushy, latentheat, &
                             phicut, dphidx, det, tempcut, massmt, &
                             stiffmt, dldtmt, latentvc, nsign, mloc)

         end if ! (nsol==2)

!        --- compute lengths of intersected edges

         do i = 1, 2
            h(i) = sqrt((x(nx(1),1) - x(nx(i+1),1))**2 + &
                     (x(nx(1),2) - x(nx(i+1),2))**2  )
            if (debug) write(irefwr,*) 'h ', h(i)
         end do

!        --- compute intersection positions

         do i = 1, 2
            if (nsol==1 .or. nsol==2) then
               dh(i) = abs(h(i)*(tsol - temperature(nx(1)))/ &
                            (temperature(nx(i+1)) - temperature(nx(1))))
            else if (nliq==1 .or. nliq==2) then
               dh(i) = abs(h(i)*(tliq - temperature(nx(1)))/ &
                            (temperature(nx(i+1)) - temperature(nx(1))))
            end if
            if (debug) write(irefwr,*) 'dh ', dh(i)
         end do

!        --- compute coordinates of intersection points

         do i = 1, 2
            do j = 1, 2
               if ( abs(x(nx(1),j)-x(nx(i+1),j))< sqreps ) then
                  xs(i,j) = x(nx(1),j)
               else if (x(nx(1),j)<x(nx(i+1),j)) then
                  xs(i,j) = x(nx(1),j) + &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               else
                  xs(i,j) = x(nx(1),j) - &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               end if
            end do
         end do

!        --- compute det such that |det| = 2*surface of subtriangle

         do i = 1, mloc
            v(1,i) = 1d0
         end do
         do i = 1, 2
            v(i+1,1) = x(nx(1),i)
            do j = 1, 2
               v(i+1,j+1) = xs(j,i)
            end do
         end do

         det = ercdet(v(1,1), v(1,2), v(1,3))
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

!        --- construct phicut

         phixs(nx(1), 1) = 1d0 - dh(1)/h(1)
         phixs(nx(1), 2) = 1d0 - dh(2)/h(2)
         phixs(nx(2), 1) = dh(1)/h(1)
         phixs(nx(2), 2) = 0d0
         phixs(nx(3), 1) = 0d0
         phixs(nx(3), 2) = dh(2)/h(2)

         phicut(nx(1), 1) = phi(nx(1),nx(1))
         phicut(nx(1), 2) = phixs(nx(1),1)
         phicut(nx(1), 3) = phixs(nx(1),2)
         phicut(nx(2), 1) = phi(nx(2),nx(1))
         phicut(nx(2), 2) = phixs(nx(2),1)
         phicut(nx(2), 3) = phixs(nx(2),2)
         phicut(nx(3), 1) = phi(nx(3),nx(1))
         phicut(nx(3), 2) = phixs(nx(3),1)
         phicut(nx(3), 3) = phixs(nx(3),2)

         if (nsol==2) then

            tempcut(1) = temperature(nx(1))
            tempcut(2) = tsol
            tempcut(3) = tsol

            call eldiscint2sl( intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, det, massmt, &
                              stiffmt, latentvc, -1d0, mloc, .false.)
            call eldiscint2m( intrule, rho, cm, kmushy, latentheat, &
                             phicut, dphidx, det, tempcut, massmt, &
                             stiffmt, dldtmt, latentvc, 1d0, mloc)

         else if (nliq==2) then

            tempcut(1) = temperature(nx(1))
            tempcut(2) = tliq
            tempcut(3) = tliq

            call eldiscint2sl( intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, det, massmt, &
                              stiffmt, latentvc, -1d0, mloc, .true.)
            call eldiscint2m( intrule, rho, cm, kmushy, latentheat, &
                             phicut, dphidx, det, tempcut, massmt, &
                             stiffmt, dldtmt, latentvc, 1d0, mloc)

         else if (nsol==1) then

            tempcut(1) = temperature(nx(1))
            tempcut(2) = tsol
            tempcut(3) = tsol

            call eldiscint2m( intrule, rho, cm, kmushy, latentheat, &
                             phicut, dphidx, det, tempcut, massmt, &
                             stiffmt, dldtmt, latentvc, -1d0, mloc)
            call eldiscint2sl( intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, det, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .false.)
         else

            tempcut(1) = temperature(nx(1))
            tempcut(2) = tliq
            tempcut(3) = tliq

            call eldiscint2m( intrule, rho, cm, kmushy, latentheat, &
                             phicut, dphidx, det, tempcut, massmt, &
                             stiffmt, dldtmt, latentvc, -1d0, mloc)
            call eldiscint2sl( intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, det, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .true.)

         end if ! (nsol==2)
      end if ! (inttype==1)

      if (debug) then
         if (dh(1)<0d0 .or. dh(2)<0d0) then
            print *, 'inttype', inttype
            print *, 'nsol, nliq, nmush', nsol, nliq, nmush
            print *, 'dh(1), dh(2)', dh(1), dh(2)
            call instop
         end if
      end if

      call erclos ( 'eldiscint2mush' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eldiscint2mush'

      end if

      end
