      subroutine eldiscint3prism( intrule, coefarr, phasetype, nx, &
                                 indextype, phi, dphidx, x, temperature, &
                                 massmt, stiffmt, dldtmt, latentvc, &
                                 nsign, mloc)
! ======================================================================
!
!        programmer    John Brusche
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 12-06-2006
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Construct element mass matrix, stiffness matrix, derivative latent heat
!   matrix and latent heat vector using discontinous integration.
!   This is the mushy case. Prism.
!   The code differs from the isothermal case, since the dLdT matrix and
!   L-vector are computed differently.
!   This is the 3D version.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          intrule, mloc, phasetype, nx(4), indextype
      double precision temperature(*), &
                       dphidx(n,ndim), massmt(n,n), phi(n,6), &
                       stiffmt(n,n), dldtmt(n,n), latentvc(n), nsign, &
                       coefarr(m,*), x(6,ndim)

!     coefarr        i    Array containing the coefficients of the PDE
!                         Pos. (.,1)   ksolid
!                         Pos. (.,2)   kfluid
!                         Pos. (.,3)   rho
!                         Pos. (.,4)   csolid
!                         Pos. (.,5)   cfluid
!                         Pos. (.,6)   Q
!                         Pos. (.,7)   L
!                         Pos. (.,8)   Tmelt
!     dldtmt        i/o   Element dldt matrix
!     dphidx         i    derivatives of basis functions
!     indextype      i     index array to be used
!     intrule        i    Numerical integration rule to be used for
!                         discontinuous integration
!                         1: Newton-Cotes, 1st order, 'substraction approach'
!                         2: Newton-Cotes, 2nd order, 'substraction approach'
!                         3: Newton-Cotes, 1st order, &
!                            'subdivision into 3 subtriangles'
!                         4: Newton-Cotes, 1st order, &
!                            'averaging of heat capacity'
!     latentvc      i/o   Element latent heat vector
!     massmt        i/o   Element mass matrix
!     mloc           i    local copy of m modified for Newton-Cotes
!     nsign          i    determines whether contribution is added or
!                         subtracted
!     phasetype      i    Type of phase of the prism
!                         0: mushy
!                         1: solid
!                         2: liquid
!     phi            i    basis functions
!     rho            i    density
!     stiffmt       i/o   Element stiffness matrix
!     temperature    i    temperature vector in integration points
!     x              i    x-coordinates of the integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, index1(4,3)
      double precision xcut(n,ndim), phicut(n,mloc), deters(3), &
                       tempcut(mloc), cl, cs, cm, kl, ks, kmushy, rho, &
                       latentheat, e(4,3)

!     cl             Specific heat of the liquid
!     cm             Specific heat of the mushy
!     cs             Specific heat of the solid
!     deters         determinants of subtetrahedra
!     e              Work array of length 4 x 3, to store the factors
!                     ij
!                    e.
!     i              Counting variable
!     index1         index array
!     j              Counting variable
!     k              Counting variable
!     kl             Diffusivity of the liquid
!     kmushy             Diffusivity of the mushy
!     ks             Diffusivity of the solid
!     latentheat    latent heat
!     nx             new/local numbering of the nodes
!     phicut        phi matrix for intersected element
!     tempcut        Temperature in integration points
!     xcut           local variant of coordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELCOMPDET      compute the determinant for a triangle in 3D space
!     ELDISCINT3M    3D discontinuous integration, integrals over mushy
!     ELDISCINT3SL   3D discontinuous integration, integrals over solid
!                    or liquid subelement
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldiscint3prism' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldiscint3prism'
         call prinin(nx, 4, 'nx prism')
      end if

!     --- get parameters

      ks  = coefarr(1,1)
      kl  = coefarr(1,2)
      rho = coefarr(1,3)
      cs  = coefarr(1,4)
      cl  = coefarr(1,5)
      latentheat = coefarr(1,7)

      cm = 0.5d0*(cs+cl)
      kmushy = 0.5d0*(ks+kl)

!     --- Fill index matrix for individual tetrahedra

      index1(1,1) = 1
      index1(2,1) = 2
      index1(3,1) = 3
      index1(4,1) = 5

      index1(1,2) = 2
      index1(2,2) = 3
      index1(3,2) = 4
      index1(4,2) = 5

      if (indextype==1) then
         index1(1,3) = 3
      else if (indextype==2) then
         index1(1,3) = 2
      end if
      index1(2,3) = 4
      index1(3,3) = 5
      index1(4,3) = 6

!     --- fill x for all three tetrahedra and compute determinants

      do i = 1, 3
         do j = 1, ndim
            do k = 1, n
               xcut(k,j) = x(index1(k,i),j)
            end do
         end do
         call elcompdet( deters(i), e, xcut, 0 )
         deters(i) = abs(deters(i))
         if (debug) call prinrl2(xcut, n, ndim,'xcut')
      end do

      if (debug) then
         call prinrl(deters, 3, 'deters')
         call prinrl2(x, 6, ndim, 'x')
         call prinrl2(phi, n, 6, 'phi')

!        --- call instop

      end if

!     --- compute determinants for all three tetrahedra

      do i = 1, 3
      end do

!     --- fill phicut and tempcut for all three tetrahedra

      do k = 1, 3
         do i = 1, n
            do j = 1, mloc
               phicut(i,j) = phi(i,index1(j,k))
               tempcut(j) = temperature(index1(j,k))
            end do
         end do

!        --- compute massmt, stiffmt, dldtmt and latentvc

         if ( phasetype==0 ) then

            call eldiscint3m( intrule, rho, cm, kmushy, latentheat, &
                             phicut, dphidx, deters(k), &
                             tempcut, massmt, stiffmt, dldtmt, &
                             latentvc, nsign, mloc)

         else if (phasetype==1 ) then

            call eldiscint3sl( intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, deters(k), &
                              tempcut, massmt, stiffmt, latentvc, &
                              nsign, mloc, .false.)

         else

            call eldiscint3sl( intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, deters(k), &
                              tempcut, massmt, stiffmt, latentvc, &
                              nsign, mloc, .true.)

         end if ! ( phasetype==0 )

      end do ! k = 1, 3

      call erclos ( 'eldiscint3prism' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eldiscint3prism'

      end if

      end
