      subroutine elgrad ( rho, gradphi, gradrho, xgauss )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the gradient of a function in an integration point
!
! **********************************************************************
!
!                       KEYWORDS
!
!     gradient
!     numerical_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision rho(n), gradphi(n,m,ndimlc), gradrho(m,ndimlc), &
                       xgauss(m,*)

!     gradphi        i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!     gradrho        o    gradient of function in integration points
!                         gradrho(k,idim) = d rho (x^k) / dx^idim
!     rho            i    function of which the gradient must be computed
!                         in integration points
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jdim, k, j
      double precision sum

!     j              Counting variable
!     jdim           Loop variable
!     k              Counting variable
!     sum            help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3044   r = 0 in integration point
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The gradient of a function rho in an integration point is
!     given by
!     grad rho(x^k) = sum_l rho(l) grad(phi_l(x^k))
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elgrad' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elgrad'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jcart<=2 ) then

!     --- jcart = 1, 2, Cartesian and Cylinder coordinates

         do jdim = 1, ndimlc

            do k = 1, m
               sum = 0d0
               do j = 1, n
                  sum = sum+rho(j)*gradphi(j,k,jdim)
               end do  ! j = 1, n
               gradrho(k,jdim) = sum
            end do  ! k = 1, m

         end do  ! jdim = 1, ndimlc

      else

!     --- jcart = 3, Polar coordinates
!         grad  rho = ( d rho/dr, d rho / (r dphi) )
!         First r-derivative

         do k = 1, m
            sum = 0d0
            do j = 1, n
               sum = sum+rho(j)*gradphi(j,k,1)
            end do  ! j = 1, n
            gradrho(k,1) = sum
         end do  ! k = 1, m

!        --- Next phi-derivative

         do k = 1, m
            if ( abs(xgauss(k,1))<=0d0 ) then

!           --- error: r=0, hence division by zero

               call errint(ielgrp,1)
               call errint(itype,2)
               call errint(ielem,3)
               call errint(irelem,4)
               call errsub ( 3044, 4, 0, 0 )
               go to 1000

            end if  ! ( xgauss(k,1)==0d0 )
         end do  ! k = 1, m

         do k = 1, m
            sum = 0d0
            do j = 1, n
               sum = sum+rho(j)*gradphi(j,k,2)/xgauss(k,1)
            end do  ! j = 1, n
            gradrho(k,2) = sum
         end do  ! k = 1, m

      end if  ! ( jcart==1 )

1000  call erclos ( 'elgrad' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elgrad'

      end if  ! ( debug )

      end
