      function eli100 ( coor2d, iuser, user, index1, islold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.5  date 30-07-2013 Redefinition of phiksi
!        version  4.4  date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 1985-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for second order ellipitic equations
!     Two dimensional elements
!     ELI100 is a help subroutine for subroutine INTEGR, &
!     it is called through the intermediate subroutine eli000
!     So:
!     INTEGR
!       -  Loop over elements
!          -  ELI000
!             - ELI100
!     itype=100 :    cartesian coordinates   linear triangle
!     itype=101 :    axisymmetric coordinates  linear triangle
!     itype=104 :    cartesian coordinates   quadratic triangle
!     itype=105 :    axisymmetric coordinates  quadratic triangle
!     itype=108 :    cartesian co-ordinates  bilinear quadrilateral
!     itype=109 :    axi-symmetric co-ordinates  bilinear quadrilateral
!     itype=110 :    cartesian co-ordinates  biquadratic quadrilateral
!     itype=111 :    axi-symmetric co-ordinates  biquadratic quadrilateral
! **********************************************************************
!
!                       KEYWORDS
!
!     element_integral
!     integral
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
!                     /celwrk/
!          Work common block to store local arrays
!          CELWRK contains exactly 1500 real positions, the last 500
!          are reserved for the ELP subroutines
      double precision x, w, dphidx, xgauss, phi, array, work, dudx, &
                       dum, uold, ugauss, phiksi
      common /celwrk/ x(18), w(9), dphidx(162), xgauss(9), phi(81), &
                      array(72), uold(9), ugauss(9), dudx(18), &
                      work(613), phiksi(162), dum(338)
      save /celwrk/

!          The following parameters are used in CELWRK
!     array          1 x m array to store the coefficients for each
!                    integration point
!                    Contents:
!                    Pos 1-m:       f
!     dphidx         Array of length inpelm x m x 2  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx = dphidx (i,k,1);  d phi / dy = dphidx (i,k,2);
!                         i                             i
!                      in node k
!     dum            500 positions reserved for the ELP subroutines
!     dudx           Contains the derivatives of the solution in the integration
!     node
!     phi      o     Array of length inpelm x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                           k
!                    phi (xg ) = phi (i,k)
!                       i
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     ugauss         Array to store the old solution in the integration points
!     uold           Array to store the old solution in the nodal points
!     x              array of length inpelm x 2 containing the x and
!                    y-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i,1);  y  = x (i,2);
!                     i              i
!     xgauss   o     Array of length m x 2 containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                      i                    i
!     w              array of length m containing the weights for the
!                    numerical integration
!     work           Work array
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - &
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision eli100, coor2d(2,*), user(*)
      integer iuser(*), index1(*), islold(numold)

!     coor2d     i     array of length 2 x npoint containing the co-ordinates of
!                    the nodal points with respect to the global numbering
!                    x  = coor2d (1,i);  y  = coor2d (2,i);
!                     i                 i
!     eli100   o     Computed integral
!     index1   i     Array of length inpelm containing the point numbers
!                    of the nodal points in the element
!     islold   i     User input array in which the user puts information
!                    of all preceding solutions
!                    Integer array of length 5 x numold
!     iuser    i     Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     user     i     Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, kfirst, jheat
      double precision qmat(3,27), tang(9*3*2)
      save isub, isup, jelp, kfirst, jheat

!     i              Counting variable
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     jelp           Indication of the type of basis function subroutine must
!                    be called by ELM100. Possible vaues:
!                    1:  Subroutine ELP620  (Linear triangle)
!                    2:  Subroutine ELP621  (Bilinear quadrilateral)
!                    3:  Subroutine ELP622  (Quadratic triangle)
!                    4:  Subroutine ELP623  (Biquadratic quadrilateral)
!     jheat          Integer parameter indicating if the mass matrix for the
!                    heat equation must be computed (>0) or not ( = 0)
!                    If jheat = 1, then the coefficient for the mass matrix is
!                    constant, otherwise (jheat = 2) it is variable
!     kfirst         This parameter indicates the smallest velocity component
!                    that is not equal to zero in the convective terms
!                    For example kfirst = 2, means that the y- &
!                    component of the velocity is unequal to zero
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision elic00

!     EL0100         Initialize parameters for subroutine ELD100
!     EL3007         Fills old solution and derivatives
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIC00         Compute integrals of the shape / f(x) dx
!                                                   e
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the parameter icheli and
!     the arrays IUSER and USER in the usual way
!     The sequence is:
!     1:  f
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1781   The possibility ifunc>10000 not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Depending on icheli the following integrals are computed
!          1:   / f(x) dx
!          2:   / f(x) u(x) dx
!          3-4: / f(x) du(x)/dx  dx   i=icheld-2
!                              i
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
!        In the axi-symmetric case they include the factor 2 pi r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eli100' )
      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         call  el0100 ( iuser, user, jelp, kfirst, jheat, isub, isup, &
                        array, -icheli, 0 )

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( islold, numold, isub, isup, iuser )

      end if

!     --- compute basis functions and weights for numerical integration

      call elm800basefn ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          qmat, tang, jelp, phiksi, work, work, work )
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( ind(lengthmst)==1 ) then

         call el3007 ( uold, dudx, phi, dphidx, m, n, ndimlc, &
                       ncomp, index1, dum, ugauss, jtrans, &
                       numold, 1 )

      end if

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            if ( ind(i)>10000 ) then
               call errint ( i, 1 )
               call errsub ( 1781, 1, 0, 0 )
            end if
            call elflr71 ( i, user, array, x, xgauss, &
                          m, n, phi, 1, index1, iuser, ugauss, work )

         end if

      end do

!     --- Compute integral depending on icheli

      eli100 = elic00 ( icheli, array, w, ugauss, dudx, work )

1000  if ( ierror/=0 ) call instop

      call erclos ( 'eli100' )

      end
