      function eli118 ( coor2d, iuser, user, index1, islold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.4  date 09-03-2013 Replace elflr7 by elflr71
!        version  2.3  date 12-12-2012 Long integers
!
!   copyright (c) 1992-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill integral over an element
!     One-dimensional elements
!     ELI118 is a help subroutine for subroutine INTEGR, &
!     it is called through the intermediate subroutine eli001
!     So:
!     INTEGR
!       -  Loop over elements
!          -  ELI001
!             - ELI118
!     itype=118 :    cartesian coordinates   linear line element
!     itype=119 :    cartesian coordinates   quadratic line element
!     itype=122 :    polar coordinates   linear line element
! **********************************************************************
!
!                       KEYWORDS
!
!     element_integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduleconsts
      implicit none
!
!                     /celwrk/
!          Work common block to store local arrays
!          CELWRK contains exactly 1500 real positions, the last 500
!          are reserved for the ELP subroutines
      double precision x, w, dphidx, xgauss, phi, array, work, dudx, &
                       dum, uold, ugauss
      common /celwrk/ x(18), w(9), dphidx(162), xgauss(9), phi(81), &
                      array(72), uold(9), ugauss(9), dudx(18), &
                      work(613), dum(500)
      save /celwrk/

!          The following parameters are used in CELWRK
!     array          1 x m array to store the coefficients for each
!                    integration point
!                    Contents:
!                    Pos 1-m:       f
!     dphidx         Array of length inpelm x m x 2  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx = dphidx (i,k,1);  d phi / dy = dphidx (i,k,2);
!                         i                             i
!                      in node k
!     dum            500 positions reserved for the ELP subroutines
!     dudx           Contains the derivatives of the solution in the integration
!     node
!     phi      o     Array of length inpelm x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                           k
!                    phi (xg ) = phi (i,k)
!                       i
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     ugauss         Array to store the old solution in the integration points
!     uold           Array to store the old solution in the nodal points
!     x              array of length inpelm x 2 containing the x and
!                    y-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i,1);  y  = x (i,2);
!                     i              i
!     xgauss   o     Array of length m x 2 containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                      i                    i
!     w              array of length m containing the weights for the
!                    numerical integration
!     work           Work array
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - &
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision eli118, coor2d(*), user(*)
      integer iuser(*), index1(*), islold(numold)

!     coor2d     i     array of length npointcac containing the co-ordinates of
!                    the nodal points with respect to the global numbering
!                    x  = coor2d (i);
!                     i
!     eli118   o     Value of the integral
!     index1   i     Array of length inpelm containing the point numbers
!                    of the nodal points in the element
!     islold   i     User input array in which the user puts information
!                    of all preceding solutions
!                    Integer array of length 5 x numold
!     iuser    i     Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     user     i     Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical linear
      double precision h , h1
      save linear

!     h           Length of element
!     h1          Help variable to compute the element matrix
!     i           Counting variables
!     linear      True:   itype = 118, linear element
!                 False:  itype = 119, quadratic element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision elic00

!     EL3007  Fills old solution and derivatives
!     ELCOMM  Initializes the commons celp and celint with standard values
!     ELFLR71 Fill variable coefficients in array ARRAY
!     ELIC00  Compute integrals of the shape / f(x) dx
!                                            e
!     ELIND0  Fill array ICH in COMMON CELIAR for preceding solutions
!     ELUSR3  Extracts information concerning the coefficients from IUSER
!             and USER.
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
!     PRINLG  Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!         The user provides the coefficients through the parameter icheli and
!         the arrays IUSER and USER in the usual way
!         The sequence is:
!         1:  f
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       3   Element has incorrect number of nodal points for this type
!      42   Array IUSER(.) has value which is too small
!      74   Incorrect number of degrees of freedom for this type
!    1781   The possibility ifunc>10000 not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Depending on icheli the following integrals are computed
!          1:   / f(x) dx
!          2:   / f(x) u(x) dx
!          3-4: / f(x) du(x)/dx  dx   i=icheld-2
!                              i
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
!        In the axi-symmetric case they include the factor 2 pi r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eli118' )
      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities
!         Fill some standard values in the common blocks celp and celint

         call elcomm

         if ( itype==118 .or. itype==122 ) then

            linear = .true.
            n = 2
            m = 2
            mn = 2

         else if ( itype==119 ) then

            linear = .false.
            n = 3
            m = 3
            mn = 9

         end if

         jdercn = 0

!        --- Fill the arrays IND, MST and ARRAY with information of the
!            coefficient

         ipos = iuser ( ielgrp + 5 )
         if ( ipos<7 ) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 42, 5, 0, 0 )
            go to 1000
         end if
         nuser = 0
         call elusr3 ( 1, array, iuser, user, 1 )

!        --- Test array lengths, number of nodes and number of
!            degrees of freedom

         call prinlg ( 'iuser', ipos-1, user, iuser )
         call prinlg ( 'user', -nuser, user, iuser )
         if ( inpelm/=n ) then
            call errint ( inpelm, 1 )
            call errint ( n, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 3, 5, 0, 0 )
            go to 1000
         end if

         if ( icount/=n ) then
            call errint ( icount, 1 )
            call errint ( n, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 74, 5, 0, 0 )
            go to 1000
         end if

         ist(1) = 0
         if ( ierror/=0 ) go to 1000

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( islold, numold, 1, 1, iuser )

      end if

!     --- compute basis functions and weights for numerical integration

      do i = 1, inpelm
         x(i) = coor2d(index1(i))
      end do

      h = x(inpelm) - x(1)

      if ( linear ) then

         dphidx(1) = -1d0 / h
         dphidx(2) = dphidx(1)
         dphidx(3) = -dphidx(1)
         dphidx(4) = dphidx(3)
         w(1) = abs(h) * 0.5d0
         w(2) = w(1)
         if ( itype==122 ) then

!        --- Polar co-ordiantes

            w(1) = x(1) * w(1) * 2d0*pi
            w(2) = x(2) * w(2) * 2d0*pi

         end if

      else

         h1 = 1d0 / h
         dphidx(1) = -3d0 * h1
         dphidx(2) =      - h1
         dphidx(3) =        h1

         dphidx(4) =  4d0 * h1
         dphidx(5) =  0d0
         dphidx(6) = -4d0 * h1

         dphidx(7) =      - h1
         dphidx(8) =        h1
         dphidx(9) =  3d0 * h1

         w(1) = abs(h) / 6d0
         w(2) = 4 * w(1)
         w(3) = w(1)

      end if

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      call el3007 ( uold, work, phi, dphidx, m, n, ndim, &
                    ncomp, index1, dum, ugauss, jtrans, &
                    numold, 1 )

      if ( ind(1)>0 .or. ist(1)>0 ) then

!     --- Coefficient is variable

         if ( ind(i)>10000 ) then
            call errint ( i, 1 )
            call errsub ( 1781, 1, 0, 0 )
         end if
         call elflr71 ( 1, user, array, x, xgauss, &
                       m, n, phi, 1, index1, &
                       iuser, ugauss, work )

      end if

!     --- Compute integral depending on icheli

      eli118 = elic00 ( icheli, array, w, ugauss, dudx, work )

1000  call erclos ( 'eli118' )
      if ( ierror/=0 ) call instop
      end
