      subroutine elm600 ( swork )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.5  date 09-03-2013 Replace elflr7 by elflr71
!        version  2.4  date 12-12-2012 Long integers
!
!   copyright (c) 1995-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for second order elliptic equations
!     and related heat equation
!     One, two and three dimensional elements
!     ELM600 is a help subroutine for subroutine BUILD (TO0050)
!     it is called through the intermediate subroutine el0000
!     So:
!     BUILD
!     TO0050
!       -  Loop over elements
!          -  EL0000
!             - ELM600
!    The subroutine is able to handle the following element shapes and cases:
!    spectral line element (1D)
!    spectral quadrilateral (2D)
!    spectral brick (3D)
!    cartesian co-ordinates
!    diagonal mass matrix
!    Type 600: standard linear case
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision swork(*)

!     swork          Work array in which the following information is stored:
!                    the work array contains several quantities of the high
!                    order spectral element discretization, the first position
!                    of each quantity is pointed at by an integer pointer
!                    pointers used (See local parameters)
!                    total length:  (14 + ndim) * m + (2 + mone)* mone
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, kfirst, jheat, &
              iseq, itypln, iparry, ipx1re, ipx, ipw1, ipd1, &
              ipdet, ip1, ip2, ip3, ip4, ip5, ip6, ip7, ip8, ip9, &
              ipwork, ipold
      double precision rdum(1)
      save isub, isup, jelp, kfirst, jheat, itypln, iparry, ipx1re, ipx, &
           ipw1, ipd1, ipdet, ip1, ip2, ip3, ip4, ip5, ip6, ip7, ip8, &
           ip9, ipwork, ipold

!     i          Counting variable
!     ip1        indicate partial derivatives of the coordinates due to
!                the isoparametric transformation, length m
!     ip2        See ip1
!     ip3        See ip1
!     ip4        See ip1
!     ip5        See ip1
!     ip6        See ip1
!     ip7        See ip1
!     ip8        See ip1
!     ip9        See ip1
!     iparry     indicates array in which the coefficients are stored
!                length 2 x m
!     ipd1       indicates the values of derivatives of the basisfunctions
!                in the interpolation points in the x-direction correspon- &
!                ding to the reference element, length mone x mone
!     ipdet      transformation determinant, length m
!     ipold      indicates the old solution in the nodal points, length m
!     ipw1       indicates the Legendre Gauss-Lobatto weights for
!                numerical integration in the x-direction corresponding
!                to the reference element, length mone
!     ipwork     work array, length m
!     ipx        indicates the coordinates of the corresponding points
!                in the element, length m x ndim
!     ipx1re     indicates the Legendre Gauss-Lobatto points in the
!                x-direction corresponding to the reference element [-1,1], &
!                length mone
!     iseq       Dummy not used
!     isub       Integer help parameter for do-loops to indicate the
!                smallest coefficient number that is dependent on space
!     isup       Integer help parameter for do-loops to indicate the
!                largest coefficient number that is dependent on space
!     itypln     Indicates the type of lineariation to be performed
!                Possible values:
!                1: Picard
!                2: Newton
!     jelp       Choice parameter to distinguish between the types of
!                element subroutines to be called. Possible values:
!                 1:  Subroutine ELP610  (Linear line element)
!                 2:  Subroutine ELP611  (Quadratic line element)
!                 3:  Subroutine ELP620  (Linear triangle)
!                 4:  Subroutine ELP622  (Quadratic triangle)
!                 5:  Subroutine ELP621  (Bilinear quadrilateral)
!                 6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                11:  Subroutine ELP630  (Linear tetrahedron)
!                12:  Subroutine ELP631  (Quadratic tetrahedron)
!                13:  Subroutine ELP632  (Trilinear brick)
!                14:  Subroutine ELP633  (Triquadratic  brick)
!     jheat      Integer parameter indicating if the mass matrix for the
!                heat equation must be computed
!                Possible values:
!                 0:  no mass matrix
!                 1:  diagonal mass matrix with constant coefficient
!                 2:  diagonal mass matrix with variable coefficient
!                11:  full mass matrix with constant coefficient
!                12:  full mass matrix with variable coefficient
!     kfirst     This parameter indicates the smallest velocity component
!                that is not equal to zero in the convective terms
!                For example kfirst = 2, means that the y- &
!                component of the velocity is unequal to zero
!     rdum       dummy variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0600    calls the following subroutines:
!     EL3007    Fills old solution and derivatives
!     EL6000    Performs initialization for subroutines ELx600
!     EL6001    compute the spectral element convection matrix
!     EL6002    compute the spectral element Helmholtz matrix
!     EL6016    compute contributions to spectral element stiffness matrix
!               corresponding to the diffusive part of the equation
!     EL6017    compute the spectral element diagonal mass matrix
!     EL6022    compute the spectral element Helmholtz matrix
!     EL6027    compute the spectral element mass matrix (diagonal)
!     EL6041    compute contributions to spectral element stiffness matrix
!               corresponding to the convection part of the equation
!     EL6044    compute contributions to spectral element stiffness matrix
!               corresponding to the diffusive part of the equation
!     EL6050    compute contributions to the spectral element vector
!               corresponding to the helmholtz part of the equation in 3-D case
!     EL6053    compute contributions to the spectral element vector
!               and the spectral element residual
!     EL6083    compute contributions to the spectral element vector
!               corresponding to the righthand side of the equation in 3-D case
!     EL6084    compute contributions to spectral element stiffness matrix
!               corresponding to the diffusive part of the equation - 3-D case
!     EL6087    compute the spectral element diagonal mass matrix; 3-D case
!     EL6090    compute contributions to spectral element stiffness matrix
!               corresponding to the 3D convection part of the equation
!     ELFLR71   Fill variable coefficients in array ARRAY
!     ELIND0    Fill array ICH in COMMON CELIAR for preceding solutions
!     ELP640    compute basisfunctions, derivatives and weights for
!               numerical integration corresponding to the reference
!               element (element independent)
!     ELP642    compute the integration points, determinant and
!               partial derivatives, two-dimensional case
!               (element dependent)
!     ELP646    store global integration points
!               compute transformation determinant and partial derivatives in 3D
!     ELP647    store integration points
!               transform weights and derivatives from the reference element
!     ELSWAP    switch parts of matrix in the case of linear elements
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays E%iuser and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  This parameter is no longer in use.
!         The parameter imas in BUILD defines the type of mass matrix
!         Now the parameter is reserved for future purposes
!     2:  Extra information about the convection terms:
!         This information consists of the following parts:
!         metupw + 1000*icompress, &
!         with metuwp information about the upwinding applied
!          and icompress information about the form of the convection term
!         Possible values for metupw:
!         0: no upwinding
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates (not yet possible)
!         2: Polar co-ordinates (not yet possible)
!     5:  Not yet used
!     6:  alpha 11  (1D/2D/3D)
!     7:  alpha 12  (2D/3D)
!     8:  alpha 13  (3D)
!     9:  alpha 22  (2D/3D)
!    10:  alpha 23  (3D)
!    11:  alpha 33  (3D)
!    12:  u 1       (1D/2D/3D)
!    13:  u 2       (2D/3D)
!    14:  u 3       (3D)
!    15:  beta      (1D/2D/3D)
!    16:  f         (1D/2D/3D)
!    17:  rho cp    (1D/2D/3D)
!    18:  Not yet used (must be 0)
!    19:  Not yet used (must be 0)
!    20:  Not yet used (must be 0)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1617   Option coefficients depending on old solution not yet implemented
!    1781   The possibility ifunc>10000 not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!       Differential equation:
!    rho cp d phi/d t - div alpha grad phi +rho cp u . grad phi + beta phi = f
!                                                  - &
!        The following formulae are used:
!           S   =  S      +  S   +  S
!                   alpha     u      beta
!           S     (i,j) = / alpha grad phi  . grad phi   de
!            alpha        e               j           i
!           S     (i,j) = / u . grad phi   phi  de
!            u            e -           j     i
!           S     (i,j) = / beta phi   phi  de
!            beta        e          j     i
!           M (i,j) = / rho cp phi  phi  de
!                     e           i    j
!           F(i)    =  /  f phi  de
!                      e       i
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm600' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm600'
         write(irefwr,1) 'ielem, ielgrp', ielem, ielgrp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000


      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         iparry = 1
         icheld = 0

         call  el0600 ( E%iuser, E%user, jelp, kfirst, jheat, isub, isup, &
                        swork(iparry), iseq, itypln )

!        --- set pointers in the workarray

         ipx1re = iparry + 20*m
         ipx    = ipx1re + mone
         ipw1   = ipx + m*ndim
         ipd1   = ipw1 + mone
         ipdet  = ipd1 + mone*mone
         ip1    = ipdet + m
         ip2    = ip1 + m
         ip3    = ip2 + m
         ip4    = ip3 + m
         ip5    = ip4 + m
         ip6    = ip5 + m
         ip7    = ip6 + m
         ip8    = ip7 + m
         ip9    = ip8 + m
         ipwork = ip9 + m
         ipold  = ipwork + m

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )

         if ( ind(12)<0 .and. ind(7)<0 ) then

!        --- rho cp has been given, and u is constant
!            multiply u by rho

            do i = kstep, m, kstep
               swork(i+6*m) = swork(i+6*m)*swork(11*m+i)
            end do

         end if
         if ( ind(12)<0 .and. ind(8)<0 ) then

!        --- rho cp has been given, and v is constant
!            multiply v by rho

            do i = kstep, m, kstep
               swork(i+7*m) = swork(i+7*m)*swork(11*m+i)
            end do

         end if
         if ( ind(12)<0 .and. ind(9)<0 ) then

!        --- rho cp has been given, and w is constant
!            multiply w by rho

            do i = kstep, m, kstep
               swork(i+8*m) = swork(i+8*m)*swork(11*m+i)
            end do

         end if

      end if

!     --- compute basis functions and weights for numerical integration

      call elp640(1, swork(ipx1re), swork(ipw1), swork(ipd1), &
                     swork(ipwork), mone)

!     --- compute integration points, determinant and partial derivatives

      if( ndim==3 ) then
         call elp646(coor,index1,swork(ipx),swork(ipx+m), &
                     swork(ipx+2*m), &
                     swork(ipd1), swork(ipdet), swork(ip1), &
                     swork(ip2), swork(ip3), swork(ip4), &
                     swork(ip5), swork(ip6), swork(ip7), &
                     swork(ip8), swork(ip9), mone)
      end if

      if( ndim==2 ) then
         call elp642(coor, index1, swork(ipx), swork(ipd1), &
                     swork(ipdet), swork(ip1), swork(ip2), &
                     swork(ip3), swork(ip4), mone, m)
      end if

      if( ndim==1 ) then
        call elp647( coor, index1, swork(ipx), swork(ipdet), mone)
      end if
      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) then

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

         call el3007 ( swork(ipold), rdum, rdum, rdum, m, m, &
                       ndim, ncomp, index1, rdum, swork(ipold), &
                       jtrans, numold, 1 )

      end if

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) then
         call errsub ( 1617, 0, 0, 0 )
         go to 1000
      end if

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            if ( ind(i)>10000 ) then
               call errint ( i, 1 )
               call errsub ( 1781, 1, 0, 0 )
            end if
            call elflr71(i, E%user, swork(iparry), &
                        swork(ipx), swork(ipx), m, m, rdum, &
                         1, index1, E%iuser, swork(ipold), swork )
         end if

      end do

      if ( ind(12)/=0 ) then

!     --- rho cp has been given, multiply velocity by rho cp

         if ( ind(7)>0 ) then
            do i = kstep, m, kstep
              swork(6*m+i) = swork(6*m+i)*swork(11*m+i)
            end do
         end if

         if ( ind(8)>0 ) then
            do i = kstep, m, kstep
               swork(7*m+i) = swork(7*m+i)*swork(11*m+i)
            end do
         end if

         if ( ind(9)>0 ) then
            do i = kstep, m, kstep
               swork(8*m+i) = swork(8*m+i)*swork(11*m+i)
            end do
         end if

      end if

!     --- Compute modified basis functions in case of upwinding
!         NOT DONE YET
!         --- Fill parts of matrix if necessary

      if ( matrix ) then

!     --- matrix = true    compute element matrix

         jadd = 0

         if ( jsecnd>0 ) then

!        --- First second derivative part (Diffusive terms)

           if( ndim==3 ) then

             call el6084(swork(iparry), E%mat, swork(ipw1), &
                         swork(ipd1), swork(ipdet), &
                         swork(ip1), swork(ip2), swork(ip3), &
                         swork(ip4), swork(ip5), swork(ip6), &
                         swork(ip7), swork(ip8), swork(ip9), &
                         swork(ipwork), mone, m)

             jadd = 1
             if ( mone==2 ) call elswap(E%mat, m, ndim)
             go to 3000

           end if

           if( ndim==2 ) then
             call el6044(swork(iparry), E%mat, swork(ipw1), &
                         swork(ipd1), &
                         swork(ipdet), swork(ip1), swork(ip2), &
                         swork(ip3), swork(ip4), swork(ipwork), &
                         mone, m)
             jadd = 1
             if (mone==2) call elswap(E%mat, m, ndim)
             go to 3000

           end if

           if( ndim==1 ) then
              call el6016(swork(iparry), E%mat, swork(ipw1), &
                          swork(ipd1), m, swork(ipwork), swork(ipdet))
              jadd = 1
              go to 3000

           end if

         end if

!        --- Next first derivative part  (convective terms)

3000     if (idim>0) then

            if (ndim==3) then

              call el6090(swork(iparry+6*m), swork(iparry+7*m), &
                     swork(iparry+8*m), E%mat, &
                     swork(ipw1), swork(ipd1), &
                     swork(ip1),  swork(ip2), swork(ip3), &
                     swork(ip4),  swork(ip5), swork(ip6), &
                     swork(ip7),  swork(ip8), swork(ip9), &
                     swork(ipwork), mone, m)
               jadd = 1

             end if

            if (ndim==2) then
               call el6041(swork(iparry+6*m), E%mat, swork(ipw1), &
                           swork(ipd1), swork(ip1), swork(ip2), &
                           swork(ip3), swork(ip4), swork(ipwork), &
                           mone, m)
               jadd = 1
               go to 4000
            end if

            if (ndim==1) then
               call el6001(swork(iparry+6*m), E%mat, swork(ipw1), &
                           swork(ipd1), m, swork(ipwork))
               jadd = 1
            end if

         end if

!        --- Finally constant part

4000     if ( jzero>0 ) then

            if (ndim==3) then
               call el6050(swork(iparry+9*m), E%mat, &
                           swork(ipw1), swork(ipdet), &
                           mone, m)
               jadd = 1
            end if

            if (ndim==2) then
               call el6022(swork(iparry+9*m), E%mat, &
                           swork(ipw1), swork(ipw1), swork(ipdet), &
                           mone, m)
               jadd = 1
            end if

            if (ndim==1) then
                call el6002(swork(iparry+9*m), E%mat, swork(ipw1), m, &
                            swork(ipdet))
               jadd = 1

            end if

         end if

      end if

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

         jadd = 0

         if( ndim==3 ) then
           call el6083( swork(iparry+10*m), E%vec, &
                        swork(ipw1), swork(ipdet), &
                        mone, m)
         end if

         if( ndim==2 ) then
            call el6053(swork(iparry+10*m), E%vec, swork(ipw1), &
                       swork(ipdet), mone, m)
         end if

         if( ndim==1 ) then

           call el6000(swork(iparry+10*m),E%vec, &
                       swork(ipw1),swork(ipdet),m)
          end if

      end if

!     --- Fill mass matrix if necessary

      if ( notmas==0 ) then

!     --- notmas = 0, compute diagonal mass matrix

         if( ndim==3 ) then
             call el6087(E%mmat, swork(ipw1), swork(ipdet), &
                      mone, m)
         end if

         if( ndim==2 ) then
            call el6027(E%mmat, swork(ipw1), swork(ipdet), &
                        mone, m)
         end if

         if ( ndim==1 ) then
            call el6017(E%mmat, swork(ipw1), swork(ipdet), m)
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( E%vec, icount, 'E%vec' )
         write(irefwr,*) 'End elm600'

      end if  ! ( debug )
      call erclos ( 'elm600' )

      end subroutine elm600
