      subroutine elm609 ( swork )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  1.2  date 09-03-2013 Replace elflr7 by elflr71
!        version  1.1  date 12-12-2012 Long integers
!        version  1.0  date 05-01-1997
!
!   copyright (c) 1997-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill spectral element matrix and vector for second order elliptic
!     equations
!     two unknowns per nodal point
!     two-dimensional quadrilateral elements
!     ELM609 is a help subroutine for subroutine BUILD
!     it is called through the intermediate subroutine EL0000
!     so:
!     BUILD
!       -  loop over spectral elements
!          -  EL0000
!             -  ELM609
!     itype = 609  :   two-dimensional spectral element
!                      two unknowns per nodal point
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision swork(*)

!     swork          spectral element work array
!                    the work array contains several quantities of the high
!                    order spectral element discretization, the first position
!                    of each quantity is pointed at by an integer pointer
!                    total length:  m*m + 24*m + 4*mone
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isub, isup, modelv, i, &
              iparry, ipxre, ipx, ipw, ipd1, &
              ipd2, ipdet, ip1, ip2, ip3, ip4, ipwork, ipold, &
              ipmatr, ipmatt, ipmatp, ipmatq, ipvec1, ipvec2
      double precision rdum(1)
      save iparry, ipxre, ipx, ipw, ipd1, &
           ipd2, ipdet, ip1, ip2, ip3, ip4, ipwork, ipold, &
           ipmatr, ipmatt, ipmatp, ipmatq,ipvec1, ipvec2, &
           isub, isup, modelv

!     i         Counting variable
!     ip1       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     ip2       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     ip3       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     ip4       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     iparry    indicates array in which the coefficients are stored
!               length 7 x m
!     ipd1      indicates the values of derivatives of the basisfunctions
!               in the interpolation points in the x-direction correspon- &
!               ding to the reference element, length mone x mone
!     ipd2      indicates the values of 2nd derivatives of the basisfunctions
!               in the interpolation points in the x-direction correspon- &
!               ding to the reference element, length mone x mone
!     ipdet     transformation determinant, length m
!     ipmatp
!     ipmatq
!     ipmatr    temporary work array to store part of the matrix, &
!               length m x m
!     ipmatt    temporary work array to store part of the matrix, &
!               length m x m
!     ipold     indicates the old solution in the nodal points, length m
!     ipvec1    temporary work array to store part of the vector, &
!               length m
!     ipvec2    temporary work array to store part of the vector, &
!               length m
!     ipw       indicates the Legendre Gauss-Lobatto weights for
!               numerical integration in the x-direction corresponding
!               to the reference element, length mone
!     ipwork    work array, length m
!     ipx       indicates the coordinates of the corresponding points
!               in the element, length m x 2
!     ipxre     indicates the Legendre Gauss-Lobatto points in the
!               x-direction corresponding to the reference element [-1,1], &
!               length mone
!     isub     integer help parameter for do-loops to indicate the smallest
!              coefficient number that is dependent on space
!     isup     integer help parameter for do-loops to indicate the largest
!              coefficient number that is dependent on space
!     modelv   choice parameter, see USER INPUT
!     rdum     dummy variable
!              (more values not yet done)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0609    performs the initialisations for ELM609/ELD609
!     EL6003    compute the spectral element righthand side term
!     EL6023    compute the spectral element righthand side term
!     EL6026    compose full spectral element vector
!     EL6027    compute the spectral element mass matrix (diagonal)
!     EL6045    compose s out of smatr, tmatr, pmatr and qmatr
!     ELFLR71   Fill variable coefficients in array ARRAY
!     ELIND0    Fill array ICH in COMMON CELIAR for preceding solutions
!     ELP640    compute basisfunctions, derivatives and weights for
!               numerical integration corresponding to the reference
!               element (element independent)
!     ELP642    compute the integration points, determinant and
!               partial derivatives, two-dimensional case
!               (element dependent)
!     ELSWAP    switch parts of matrix in the case of linear elements
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     USER INPUT
!     the user provides the coefficients through the arrays E%iuser and
!     USER in the usual way, the sequence is:
!     1:  eta
!     2:  f1
!     3:  f2
!     4:  modelv
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('elm609')
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm609'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if (ifirst==0) then

!     --- ifirst = 0, compute element independent quantities
!         perform initialisations, set iparry

         iparry = 1
         call el0609(E%iuser, E%user, isub, isup, swork(iparry), &
                     0, modelv)

!        --- set pointers in the workarray

         ipxre  = iparry + 4*m
         ipx    = ipxre  + mone
         ipw    = ipx    + 2*m
         ipd1   = ipw    + mone
         ipd2   = ipd1   + m
         ipdet  = ipd2   + m
         ip1    = ipdet  + m
         ip2    = ip1    + m
         ip3    = ip2    + m
         ip4    = ip3    + m
         ipwork = ip4    + m
         ipold  = ipwork + m
         ipmatr = ipold  + 2*m
         ipmatt = ipmatr + m*m
         ipmatp = ipmatt + m*m
         ipmatq = ipmatp + m*m
         ipvec1 = ipmatq + m*m
         ipvec2 = ipvec1 + m

!        --- fill array ich (common celiar) for preceeding solutions

         call elind0( E%islold, numold, isub, isup, E%iuser)

!        --- kstep = 1
!            compute points and weights for numerical integration
!            compute derivatives of the basisfunctions
!            corresponding to the reference element

         call elp640(1, swork(ipxre), swork(ipw), swork(ipd1), &
                     swork(ipd2), mone)
      end if
      if (ierror/=0) go to 1000

!     --- compute integration points, determinant and derivatives

      call elp642(coor, index1, swork(ipx), swork(ipd1), &
                  swork(ipdet), swork(ip1), swork(ip2), swork(ip3), &
                  swork(ip4),   mone,       m)

!     --- fill variable coefficients if necessary
!         first fill old solution if necessary

      do 100 i = isub, isup

!     --- coefficient is variable

         call elflr71(i, E%user, swork(iparry), &
                     swork(ipx), swork(ipx), m, m, rdum, &
                     1, index1, &
                     E%iuser, swork(ipold), swork )
100   continue
      if (matrix) then

!     --- matrix = true, compute spectral element matrix

         jadd = 0

!        --- first second derivative part (diffusive terms)

         call el6003(swork(iparry), swork(ipmatr), swork(ipw), &
                     swork(ipw),    swork(ipd1),   swork(ipd1), &
                     swork(ipdet),  swork(ip1),    swork(ip2), &
                     swork(ip3),    swork(ip4),    swork(ipwork), &
                     mone,          m,             1)
         jadd = 0
         call el6003(swork(iparry), swork(ipmatt), swork(ipw), &
                     swork(ipw),    swork(ipd1),   swork(ipd1), &
                     swork(ipdet),  swork(ip1),    swork(ip2), &
                     swork(ip3),    swork(ip4),    swork(ipwork), &
                     mone,          m,             2)
         jadd = 0
         call el6003(swork(iparry), swork(ipmatp), swork(ipw), &
                     swork(ipw),    swork(ipd1),   swork(ipd1), &
                     swork(ipdet),  swork(ip1),    swork(ip2), &
                     swork(ip3),    swork(ip4),    swork(ipwork), &
                     mone,          m,             3)
         jadd = 0
         call el6003(swork(iparry), swork(ipmatq), swork(ipw), &
                     swork(ipw),    swork(ipd1),   swork(ipd1), &
                     swork(ipdet),  swork(ip1),    swork(ip2), &
                     swork(ip3),    swork(ip4),    swork(ipwork), &
                     mone,          m,             4)
         jadd = 1
         if (mone==2) call elswap(swork(ipmatr), m, ndimlc)
         if (mone==2) call elswap(swork(ipmatt), m, ndimlc)
         if (mone==2) call elswap(swork(ipmatp), m, ndimlc)
         if (mone==2) call elswap(swork(ipmatq), m, ndimlc)

!        --- compose full spectral element matrix

         call el6045(E%mat, swork(ipmatr), swork(ipmatt), &
                     swork(ipmatp), swork(ipmatq),m)
      end if

!     --- fill spectral element mass matrix if necessary

      if (notmas==0) then

!     --- notmas = 0, compute diagonal mass matrix

         call el6027(swork(ipvec1), swork(ipw), swork(ipdet), &
                        mone, m)
         call el6026(E%mmat, swork(ipvec1), swork(ipvec1), m)
      end if

!     --- fill spectral element vector if necessary

      if (vector) then

!     --- vector = true, compute element vector

         jadd = 0
         call el6023(swork(iparry+1*m), swork(ipvec1), &
                     swork(ipw), swork(ipw), swork(ipdet), &
                     mone, m)
         jadd = 1
         call el6023(swork(iparry+2*m), swork(ipvec2), &
                     swork(ipw), swork(ipw), swork(ipdet), &
                     mone, m)
         jadd = 1

!        --- compose full spectral element vector

         call el6026(E%vec, swork(ipvec1), swork(ipvec2), m)
      end if
1000  if ( debug ) then

!     --- Debug information

         call prinrl ( E%vec, icount, 'E%vec' )
         call prinrl1 ( E%mat, icount, icount, 'E%mat' )
         write(irefwr,*) 'End elm609'

      end if  ! ( debug )
      call erclos('elm609')

      end subroutine elm609
