      subroutine elm615 ( swork )
! ======================================================================
!
!        programmer    Patrick Anderson / Maykel Verschueren
!        version  2.4  date 09-03-2013 Replace elflr7 by elflr71
!        version  2.3  date 12-12-2012 Long integers
!        version  2.2  date 04-09-2001  Correction swork
!
!   copyright (c) 1997-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill spectral element matrix and vector for second order elliptic
!     equations
!     two unknowns per nodal point
!     two-dimensional quadrilateral elements
!     ELM615 is a help subroutine for subroutine BUILD
!     it is called through the intermediate subroutine EL0000
!     so:
!     BUILD
!       -  loop over spectral elements
!          -  EL0000
!             -  ELM615
!     itype = 615  :   1D/2D/3D-dimensional spectral element
!                      two unknowns per nodal point
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision swork(*)

!     swork          spectral element work array
!                    the work array contains several quantities of the high
!                    order spectral element discretization, the first position
!                    of each quantity is pointed at by an integer pointer
!                    total length:  m*m + 24*m + 4*mone
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isub, isup, modelv, i, &
              iparry, ipxre, ipx, ipw, ipd1, &
              ipd2, ipdet, ip1, ip2, ip3, ip4, &
              ip5, ip6, ip7, ip8, ip9, ipwork, ipold, &
              ipmatr, ipvec1, ipvec2
      double precision rdum(1)
      save iparry, ipxre, ipx, ipw, ipd1, &
           ipd2, ipdet, ip1, ip2, ip3, ip4, ip5, ip6, ip7, ip8, ip9, &
           ipwork, ipold, ipmatr, ipvec1, ipvec2, &
           isub, isup, modelv

!     i         Counting variable
!     ip1       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     ip2       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     ip3       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     ip4       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     ip5       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     ip6       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     ip7       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     ip8       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     ip9       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     iparry    indicates array in which the coefficients are stored
!               length 7 x m
!     ipd1      indicates the values of derivatives of the basisfunctions
!               in the interpolation points in the x-direction correspon- &
!               ding to the reference element, length mone x mone
!     ipd2      indicates the values of 2nd derivatives of the basisfunctions
!               in the interpolation points in the x-direction correspon- &
!               ding to the reference element, length mone x mone
!     ipdet     transformation determinant, length m
!     ipmatr    temporary work array to store part of the matrix, &
!               length m x m
!     ipold     indicates the old solution in the nodal points, length m
!     ipvec1    temporary work array to store part of the vector, &
!               length m
!     ipvec2    temporary work array to store part of the vector, &
!               length m
!     ipw       indicates the Legendre Gauss-Lobatto weights for
!               numerical integration in the x-direction corresponding
!               to the reference element, length mone
!     ipwork    work array, length m
!     ipx       indicates the coordinates of the corresponding points
!               in the element, length m x 2
!     ipxre     indicates the Legendre Gauss-Lobatto points in the
!               x-direction corresponding to the reference element [-1,1], &
!               length mone
!     isub     integer help parameter for do-loops to indicate the smallest
!              coefficient number that is dependent on space
!     isup     integer help parameter for do-loops to indicate the largest
!              coefficient number that is dependent on space
!     modelv   choice parameter, see USER INPUT
!     rdum     dummy variable
!              (more values not yet done)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0615         performs the initialisations for ELM615/ELD615
!     EL6000         Performs initialization for subroutines ELx600
!     EL6004         compute the spectral element diffusion matrix
!     EL6010         compute contributions to spectral element stiffness matrix
!     EL6013         compute the spectral element righthand side residual
!     EL6017         compute the spectral element diagonal mass matrix
!     EL6026         compose full spectral element vector
!     EL6027         compute the spectral element mass matrix (diagonal)
!     EL6045         compose s out of smatr, tmatr, pmatr and qmatr
!     EL6053         compute contributions to the spectral element vector
!                    and the spectral element residual
!     EL6083         compute contributions to the spectral element vector
!                    corresponding to the righthand side of the equation in 3-D
!                    case
!     EL6087         compute the spectral element diagonal mass matrix; 3-D case
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELP640         compute basisfunctions, derivatives and weights for
!                    numerical integration corresponding to the reference
!                    element (element independent)
!     ELP642         compute the integration points, determinant and
!                    partial derivatives, two-dimensional case
!                    (element dependent)
!     ELP646         compute transformation determinant and
!                    partial derivatives  in 3-D
!     ELP647         store integration points
!                    transform weights and derivatives from the reference
!                    element
!     ELSWAP         switch parts of matrix in the case of linear elements
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
!     USER INPUT
!     the user provides the coefficients through the arrays E%iuser and
!     USER in the usual way, the sequence is:
!     1:  cn
!     2:  vx
!     3:  vy
!     4:  vz
!     5:  Mxx
!     6:  Mxy
!     7:  Mxz
!     8:  Myx
!     9:  Myy
!    10:  Myz
!    11:  Mzx
!    12:  Mzy
!    13:  Mzz
!    14:  s
!    15:  r
!    16:  exx
!    17:  exy
!    18:  exz
!    19:  eyx
!    20:  eyy
!    21:  eyz
!    22:  ezx
!    23:  ezy
!    24:  ezz
!    25:  f1
!    26:  f2
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('elm615')

      if (ifirst==0) then

!     --- ifirst = 0, compute element independent quantities
!         perform initialisations, set iparry

         iparry = 1
         call el0615(E%iuser, E%user, isub, isup, swork(iparry), &
                     0, modelv)

!        --- set pointers in the workarray

         ipxre  = iparry + 26*m
         ipx    = ipxre  + mone
         ipw    = ipx    + ndimlc*m
         ipd1   = ipw    + mone
         ipd2   = ipd1   + mone*mone
         ipdet  = ipd2   + mone*mone
         ip1    = ipdet  + m
         ip2    = ip1    + m
         ip3    = ip2    + m
         ip4    = ip3    + m
         ip5    = ip4    + m
         ip6    = ip5    + m
         ip7    = ip6    + m
         ip8    = ip7    + m
         ip9    = ip8    + m
         ipwork = ip9    + m
         ipold  = ipwork + m
         ipmatr = ipold  + 2*m
         ipvec1 = ipmatr + 4*m*m
         ipvec2 = ipvec1 + m

!        --- fill array ich (common celiar) for preceeding solutions

         call elind0( E%islold, numold, isub, isup, E%iuser)

!        --- kstep = 1
!            compute points and weights for numerical integration
!            compute derivatives of the basisfunctions
!            corresponding to the reference element

         call elp640(1, swork(ipxre), swork(ipw), swork(ipd1), &
                     swork(ipd2), mone)
      end if

      if (ierror/=0) go to 1000

!     --- compute integration points, determinant and derivatives

      if( ndimlc==3 ) then
         call elp646(coor,index1,swork(ipx),swork(ipx+m),swork(ipx+2*m), &
                     swork(ipd1), swork(ipdet), swork(ip1), &
                     swork(ip2), swork(ip3), swork(ip4), &
                     swork(ip5), swork(ip6), swork(ip7), &
                     swork(ip8), swork(ip9), mone)
      end if

      if( ndimlc==2 ) then
         call elp642(coor, index1, swork(ipx), swork(ipd1), &
                     swork(ipdet), swork(ip1), swork(ip2), swork(ip3), &
                     swork(ip4),   mone,       m)
      end if

      if( ndimlc==1 ) then
        call elp647( coor, index1, swork(ipx), swork(ipdet), mone)
      end if

!     --- fill variable coefficients if necessary
!         first fill old solution if necessary

      do i = isub, isup

!     --- coefficient is variable

         call elflr71(i, E%user, swork(iparry), &
                     swork(ipx), swork(ipx), m, m, rdum, &
                      1, index1, E%iuser, swork(ipold), swork )

      end do

      if (matrix) then

!     --- matrix = true, compute spectral element matrix

!         --- first second derivative part (diffusive terms)

         if (ndimlc==1) then
            do i = 1, 4
               call el6013(swork(iparry), swork(ipmatr + (i-1) * m*m), &
                           swork(ipw),   swork(ipd1), &
                           swork(ipdet), swork(ipwork), m, i)
             end do
         end if

         if (ndimlc==2) then
            do i = 1, 4
               call el6004(swork(iparry), swork(ipmatr + (i-1) * m*m), &
                           swork(ipw),    swork(ipw),    swork(ipd1), &
                           swork(ipd1),   swork(ipdet),  swork(ip1), &
                           swork(ip2),    swork(ip3),    swork(ip4), &
                           swork(ipwork), mone, m, i)

               if (mone==2) &
                  call elswap(swork(ipmatr+ (i-1) * m*m), m, ndimlc)
            end do
         end if

         if (ndimlc==3) then
            do i = 1, 4
               call el6010(swork(iparry), swork(ipmatr + (i-1) * m*m), &
                           swork(ipw),    swork(ipd1), &
                           swork(ipdet),  swork(ip1), &
                           swork(ip2),    swork(ip3),    swork(ip4), &
                           swork(ip5),    swork(ip6),    swork(ip7), &
                           swork(ip8),    swork(ip9),    swork(ipwork), &
                           mone, m, i)

               if (mone==2) &
                  call elswap(swork(ipmatr+ (i-1) * m*m), m, ndimlc)
            end do
         end if

!        --- compose full spectral element matrix

         call el6045(E%mat, swork(ipmatr), swork(ipmatr + m*m), &
                     swork(ipmatr + 2*m*m), swork(ipmatr + 3*m*m), m)

      end if

!     --- fill spectral element mass matrix if necessary

      if (notmas==0) then

!     --- notmas = 0, compute diagonal mass matrix

         if( ndimlc==3 ) then
             call el6087(swork(ipvec1), swork(ipw), swork(ipdet), &
                      mone, m)
         end if

         if( ndimlc==2 ) then
            call el6027(swork(ipvec1), swork(ipw), swork(ipdet), &
                        mone, m)
         end if

         if( ndimlc==1 ) then
            call el6017(swork(ipvec1), swork(ipw), swork(ipdet), m)
         end if

         call el6026(E%mmat, swork(ipvec1), swork(ipvec1), m)

      end if

!     --- fill spectral element vector if necessary

      if (vector) then

!     --- vector = true, compute element vector

         if( ndimlc==3 ) then
            jadd = 0
            call el6083(swork(iparry+24*m), swork(ipvec1), &
                        swork(ipw), swork(ipdet), &
                        mone, m)
            jadd = 0
            call el6083(swork(iparry+25*m), swork(ipvec2), &
                        swork(ipw), swork(ipdet), &
                        mone, m)
         end if

         if( ndimlc==2 ) then
            jadd = 0
            call el6053(swork(iparry+24*m), swork(ipvec1), &
                        swork(ipw), swork(ipdet), &
                        mone, m)
            jadd = 0
            call el6053(swork(iparry+25*m), swork(ipvec2), &
                        swork(ipw), swork(ipdet), &
                        mone, m)
         end if

         if( ndimlc==1 ) then
            jadd = 0
            call el6000(swork(iparry+24*m), swork(ipvec1), &
                        swork(ipw), swork(ipdet), m)
            jadd = 0
            call el6000(swork(iparry+25*m), swork(ipvec2), &
                        swork(ipw), swork(ipdet), m)
         end if

!        --- compose full spectral element vector

         call el6026(E%vec, swork(ipvec1), swork(ipvec2), m)
      end if

1000  if (ierror/=0) call instop

      call erclos('elm615')

      end
