      subroutine elm800map ( work, imapf, jelp, qmat, ah )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 02-10-2002
!
!   copyright (c) 2002-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map velocities, right-hand side and diffusion tensor onto surface
!     Special help subroutine for elm800
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     diffusion
!     convection
!     map
!     right_hand_side
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imapf, jelp
      double precision work(*), ah(3,3), qmat(*)

!     ah             o    Help matrix that stores diffusion tensor
!     imapf          i    If 1 the right-hand side is mapped from higher
!                         dimension to lower dimension
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx. Possible values:
!                          1:  Subroutine ELP610  (Linear line element)
!                          2:  Subroutine ELP611  (Quadratic line element)
!                          3:  Subroutine ELP620  (Linear triangle)
!                          4:  Subroutine ELP624  (Extended quadratic triangle)
!                          5:  Subroutine ELP621  (Bilinear quadrilateral)
!                          6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                         11:  Subroutine ELP630  (Linear tetrahedron)
!                         12:  Subroutine ELP631  (Quadratic tetrahedron)
!                         13:  Subroutine ELP632  (Trilinear brick)
!                         14:  Subroutine ELP633  (Triquadratic brick)
!     qmat           i    transformation matrix global -> local coordinates
!                         in case of a 3D element
!     work          i/o   Work array, see elm800
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision v3dh(3), ak(3,3), v1d(3), vs, vx, al(3,3)

!     ak             Intermediate help matrix that stores diffusion tensor
!     al             Help matrix that stores diffusion tensor in local coor2d
!                    system
!     i              Counting variable
!     v1d            Velocities in the local coordinate system
!     v3dh           Help array to store the 3 velocity components temporarily
!     vs             Help quantity to define the mapping
!     vx             Help quantity to define the mapping
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     YMULAB         Matrix-matrix multiplication
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm800map' )
      if ( ndim.eq.2 ) then
         ah(3,1) = 0d0
         ah(1,3) = 0d0
         ah(3,2) = 0d0
         ah(2,3) = 0d0
         ah(3,3) = 0d0
      end if

!     --- perform mapping of velocities

      do i = 1, m

         v3dh(1) = work(6*m+i)
         v3dh(2) = work(7*m+i)
         v3dh(3) = work(8*m+i)
         call ymulab(qmat, v3dh, v1d, 3, 3, 3, 3, 3, 1, 1)

         if ( ndim.eq.2 ) then
            ah(1,1) = work(0*m+i)
            ah(2,1) = work(1*m+i)
            ah(1,2) = work(1*m+i)
            ah(2,2) = work(2*m+i)
         else if ( ndim.eq.3 ) then
            ah(1,1) = work(0*m+i)
            ah(2,1) = work(1*m+i)
            ah(1,2) = work(1*m+i)
            ah(3,1) = work(2*m+i)
            ah(1,3) = work(2*m+i)
            ah(2,2) = work(3*m+i)
            ah(3,2) = work(4*m+i)
            ah(2,3) = work(4*m+i)
            ah(3,3) = work(5*m+i)
         end if

!        --- ak = trans(qmat)*ah

         call ymulab(qmat, ah, ak, 3, 3, 3, 3, 3, 3, 2)

!        --- al = ak * qmat

         call ymulab(ak, qmat, al, 3, 3, 3, 3, 3, 3, 1)

         if ( ndim.eq.2 ) then
            work(0*m+i) = al(1,1)
            work(1*m+i) = al(2,1)
            work(2*m+i) = al(2,2)
         else if ( ndim.eq.3 ) then
            work(0*m+i) = al(1,1)
            work(1*m+i) = al(2,1)
            work(2*m+i) = al(3,1)
            work(3*m+i) = al(2,2)
            work(4*m+i) = al(3,2)
            work(5*m+i) = al(3,3)
         end if

         if ( ndim.eq.2 .and. jelp.lt.3 ) then

!        --- line elements

            work(6*m+i)=v1d(3)
            work(7*m+i)=0d0
            work(8*m+i)=0d0

         else if ( ndim.eq.3 .and. jelp.lt.3 ) then

!        --- line elements

            work(6*m+i)=v1d(3)
            work(7*m+i)=0d0
            work(8*m+i)=0d0

         else if ( ndim.eq.3 .and. jelp.lt.7 ) then

!        --- surface triangles

            work(6*m+i)=v1d(1)
            work(7*m+i)=v1d(2)
            work(8*m+i)=0d0

         end if

         if ( imapf.gt.0 ) then

!        --- compute dx/ds, put into f

            vx = v3dh(1)**2 + v3dh(2)**2 + v3dh(3)**2
            vs = work(6*m+i)**2 + work(7*m+i)**2 + work(8*m+i)**2
            work(10*m+i) = work(10*m+i) * (vs/vx)

         end if

      end do
      call erclos ( 'elm800map' )
      end
