      subroutine elm800massmat ( jheat, metup, metupc, rho, &
                                 phi, psiupw, elemms, wrk1, w )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 12-08-2011 extra debug
!        version  1.2  date 20-10-2010 new call to el3002
!        version  1.1  date 07-08-2006 Debug statements
!
!   copyright (c) 2006-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element mass matrix for elements of type 800, i.e.
!     general second order elliptic equation with one degree of freedom
!     per point
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elliptic_equation
!     mass_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jheat, metup, metupc
      double precision rho(*), psiupw(*), elemms(*), wrk1(*), w(*), &
                       phi(*)

!     elemms         o    Element mass matrix
!     jheat          i    Integer parameter indicating if the mass matrix for
!                         the heat equation must be computed (>0) or not ( = 0)
!                         If jheat = 1, then the coefficient for the mass matrix
!                         is constant, otherwise (jheat = 2) it is variable
!     metup          i    Upwind parameter
!     metupc         i    If 0 the complete upwinding must be taken into account
!                         otherwise (1) only the convective terms
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     psiupw         i    Array of size n*m containing the adapted basis
!                         functions for upwind
!     rho            i    array with density rho in integration points
!     w              i    array of length m containing the weights for
!                         integration
!     wrk1                Work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ih

!     ih             Help variable to store a constant temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3002         Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                                   e            i       j
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     EL3006         Compute integrals of the shape / beta(x) phi (x) psi (x) dx
!                                                   e            i       j
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm800massmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm800massmat'
         write(irefwr,1) 'jheat, jzero, metup, metupw, metupc', &
                          jheat, jzero, metup, metupw, metupc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jheat>10 ) then

!     --- jheat = 11, 12:  Non-diagonal mass matrix

         jadd = 0
         ih = jzero
         jzero = jheat-10

         if ( metup==0 .or. metupw>=100 .or. metupc==1 ) then
            call el3002 ( rho, elemms, w, phi, m, n, n )
         else
            call el3006 ( rho, elemms, w, phi, psiupw )
         end if
         jzero = ih

      else

!     --- jheat = 1, 2, i.e. diagonal mass matrix

         jadd = 0
         ih = jconsf
         jconsf = jheat
         call el3003 ( rho, elemms, w, phi, m ,n )
         jconsf = ih

      end if

1000  call erclos ( 'elm800massmat' )
      if ( debug ) then

!     --- Debug information

         if ( jheat>10 ) then
            call prinrl1 ( elemms, icount, icount, 'mass matrix' )
         else
            call prinrl ( elemms, icount, 'diagonal mass matrix' )
         end if  ! ( jheat>10 )
         write(irefwr,*) 'End elm800massmat'

      end if  ! ( debug )

      end
