      subroutine elm805massmat ( jheat, rho1, rho2, phi, elemms, wrk1, &
                                 w, icounthalf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-05-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element mass matrix for elements of type 805, i.e.
!     general second order elliptic equation with two degrees of freedom
!     per point
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elliptic_equation
!     mass_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jheat, icounthalf
      double precision rho1(*), elemms(*), wrk1(*), w(*), &
                       phi(*), rho2(*)

!     elemms         o    Element mass matrix
!     icounthalf     i    icount/2, i.e. size of submatrix
!     jheat          i    Integer parameter indicating if the mass matrix for
!                         the heat equation must be computed (>0) or not ( = 0)
!                         If jheat = 1, then the coefficient for the mass matrix
!                         is constant, otherwise (jheat = 2) it is variable
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     rho1           i    array with density rho in integration points
!                         (first eq)
!     rho2           i    array with density rho in integration points
!                         (second eq)
!     w              i    array of length m containing the weights for
!                         integration
!     wrk1                Work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer metup, metupc, i
      double precision elemms1(icounthalf**2), elemms2(icounthalf**2), &
                       psiupw(1)

!     elemms1        Element mass matrix for first equation
!     elemms2        Element mass matrix for second equation
!     i              Counting variable
!     metup          dummy
!     metupc         dummy
!     psiupw         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3010         Fill small matrix elemmt into larger matrix s
!     ELM800MASSMAT  Fill element mass matrix for elements of type 800, i.e.
!                    general second order elliptic equation with one degree of
!                    freedom per point
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm805massmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm805massmat'
         write(irefwr,1) 'jheat', jheat
         call prinrl ( rho1, m, 'rho1' )
         call prinrl ( rho2, m, 'rho2' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      metup = 0
      metupc = 0

!     --- Fill element mass matrix for first component

      call elm800massmat ( jheat, metup, metupc, rho1, &
                           phi, psiupw, elemms1, wrk1, w )

!     --- Fill element mass matrix for first component

      call elm800massmat ( jheat, metup, metupc, rho2, &
                           phi, psiupw, elemms2, wrk1, w )

      if ( jheat==1 ) then

!     --- Put both diagonal matrices into diagonal element matrix

         do i = 1, icounthalf
            elemms(2*i-1) = elemms1(i)
            elemms(2*i) = elemms2(i)
         end do  ! i = 1, icounthalf

      else

!     --- Put both matrices into element matrix

         call el3010 ( elemms, elemms1, icount, icounthalf, 1, 1, &
                       2, .false., 1d0 )

         call el3010 ( elemms, elemms2, icount, icounthalf, 2, 2, &
                       2, .false., 1d0 )

      end if  ! ( jheat==1 )

1000  call erclos ( 'elm805massmat' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm805massmat'
         if ( jheat==1 ) then
            call prinrl ( elemms1, icounthalf, 'elemms1' )
            call prinrl ( elemms2, icounthalf, 'elemms2' )
            call prinrl ( elemms, icount, 'elemms' )
         else
            call prinrl1 ( elemms1, icounthalf, icounthalf, 'elemms1' )
            call prinrl1 ( elemms2, icounthalf, icounthalf, 'elemms2' )
            call prinrl1 ( elemms, icount, icount, 'elemms' )
         end if  ! ( jheat==1 )

      end if  ! ( debug )

      end
