      subroutine elm808matrix ( jelp, alpha1, elemmt, w, dphidx, &
                                phi, coor, index1, iuser, user, vecloc, &
                                phiksi, x, xgauss, symm, uvel1, kfirst, &
                                beta1, dudx, idefcr, ndim, &
                                alpha2, uvel2, beta2, icounthalf, &
                                alpha12, uvel12, beta12, &
                                alpha21, uvel21, beta21 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 12-12-2012 Long integers
!        version  2.0  date 20-10-2010 extra parameter jmethod
!        version  1.0  date 10-05-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for elements of type 808, i.e.
!     general second order elliptic equation with two degreed of freedom
!     per point
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jelp, index1(*), &
              iuser(*), kfirst, icounthalf, idefcr, ndim
      double precision elemmt(*), w(*), uvel1(m,*), beta1(*), &
                       dphidx(n,m,*), coor(*), user(*), &
                       x(*), xgauss(*), vecloc(*), &
                       dudx(*), alpha1(m,*), phi(*), phiksi(*), &
                       uvel2(m,*), beta2(*), alpha2(m,*), &
                       alpha12(m,*), uvel12(m,*), beta12(*), &
                       alpha21(m,*), uvel21(m,*), beta21(*)
      logical symm

!     alpha1         i    array of length m x 3 in which the values of
!                         alpha  , alpha  , alpha
!                              11       12       22
!                         are stored in the sequence:
!                                                 k
!                         alpha ( k,i ) = alpha (x )   i = 1(1)3, k=1(1)m
!                                              i
!                         Refers to the first equation
!     alpha12        i    See alpha1, but now for second part of first equation
!     alpha2         i    See alpha1, but now for second equation
!     alpha21        i    See alpha1, but now for first part of second equation
!     beta1          i    array of length m containing the values of the
!                         third coefficient in the integration points
!                         Refers to the first equation
!     beta12         i    See beta1, but now for second part of first equation
!     beta2          i    See beta1, but now for second equation
!     beta21         i    See beta1, but now for first part of second equation
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     elemmt         o    Element matrix
!     icounthalf     i    icount/2, i.e. size of submatrix
!     idefcr         i    If 1 defect correction is applied, if 0 not
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     kfirst         i    First user point of a curve
!     ndim           i    Dimension of the space.
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phiksi         i    array of length n x m to store the values of dphi/dxsi
!                         according to:
!                                     k
!                         dphi /dxsi(x ) = phiksi(i,k)
!                             i
!     symm           i    Indication whether the resulting matrix is symmetrical
!                         (true) or not (false)
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     uvel1          i    Contains the velocity vector for first equation
!     uvel12         i    Contains the velocity vector for second part of
!                         first equation
!     uvel2          i    Contains the velocity vector for second equation
!     uvel21         i    Contains the velocity vector for first part of
!                         second equation
!     vecloc         i    Work array in which all old solution vectors for the
!                         integration points are stored
!     w              i    array of length m containing the weights for
!                         integration
!     x              i    co-ordinates of points in element
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypln, metup, metupc
      double precision elemmt1(icounthalf,icounthalf), &
                       elemmt2(icounthalf,icounthalf), psiupw(1)
      logical second

!     elemmt1        element matrix for first unknown
!     elemmt2        element matrix for second unknown
!     itypln         dummy
!     metup          dummy
!     metupc         dummy
!     psiupw         dummy
!     second         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3010         Fill small matrix elemmt into larger matrix s
!     ELM800MATRIX   Fill element matrix for elements of type 800, i.e.
!                    general second order elliptic equation with one degree of
!                    freedom per point
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm808matrix' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm808matrix'
         call prinrl1 ( alpha1, 3, m, 'alpha11' )
         call prinrl1 ( uvel1, 3, m, 'uvel11' )
         call prinrl ( beta1, m, 'beta11' )
         call prinrl1 ( alpha2, 3, m, 'alpha22' )
         call prinrl1 ( uvel2, 3, m, 'uvel22' )
         call prinrl ( beta2, m, 'beta22' )
         call prinrl1 ( alpha12, 3, m, 'alpha12' )
         call prinrl1 ( uvel12, 3, m, 'uvel12' )
         call prinrl ( beta12, m, 'beta12' )
         call prinrl1 ( alpha21, 3, m, 'alpha21' )
         call prinrl1 ( uvel21, 3, m, 'uvel21' )
         call prinrl ( beta21, m, 'beta21' )
         write(irefwr,1) 'kfirst, jzero', kfirst, jzero
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      itypln = 1
      metup = 0
      metupc = 0
      second = .false.

!     --- Fill first element matrix

      call elm800matrix ( jelp, alpha1, elemmt1, w, dphidx, &
                          psiupw, phi, coor, index1, &
                          iuser, user, vecloc, &
                          second, phiksi, x, xgauss, symm, &
                          metup, uvel1, kfirst, metupc, &
                          beta1, itypln, dudx, idefcr, ndim )

!     --- Fill second element matrix (11)

      call elm800matrix ( jelp, alpha2, elemmt2, w, dphidx, &
                          psiupw, phi, coor, index1, &
                          iuser, user, vecloc, &
                          second, phiksi, x, xgauss, symm, &
                          metup, uvel2, kfirst, metupc, &
                          beta2, itypln, dudx, idefcr, ndim )

      if ( debug ) then
         call prinrl1 ( elemmt1, icounthalf, icounthalf, 'elemt_11' )
         call prinrl1 ( elemmt2, icounthalf, icounthalf, 'elemt_22' )
      end if  ! ( debug )

!     --- Put both matrices into element matrix (22)

      call el3010 ( elemmt, elemmt1, icount, icounthalf, 1, 1, &
                    2, .false., 1d0 )

      call el3010 ( elemmt, elemmt2, icount, icounthalf, 2, 2, &
                    2, .false., 1d0 )

!     --- Fill third element matrix (12)

      call elm800matrix ( jelp, alpha12, elemmt1, w, dphidx, &
                          psiupw, phi, coor, index1, &
                          iuser, user, vecloc, &
                          second, phiksi, x, xgauss, symm, &
                          metup, uvel12, kfirst, metupc, &
                          beta12, itypln, dudx, idefcr, ndim )
      if ( debug ) then
         call prinrl1 ( elemmt1, icounthalf, icounthalf, 'elemt_12' )
      end if  ! ( debug )

!     --- Put matrix into element matrix (12)

      call el3010 ( elemmt, elemmt1, icount, icounthalf, 2, 1, &
                    2, .false., 1d0 )

!     --- Fill fourth element matrix (21)

      call elm800matrix ( jelp, alpha21, elemmt1, w, dphidx, &
                          psiupw, phi, coor, index1, &
                          iuser, user, vecloc, &
                          second, phiksi, x, xgauss, symm, &
                          metup, uvel21, kfirst, metupc, &
                          beta21, itypln, dudx, idefcr, ndim )
      if ( debug ) then
         call prinrl1 ( elemmt1, icounthalf, icounthalf, 'elemt_21' )
      end if  ! ( debug )

!     --- Put matrix into element matrix (21)

      call el3010 ( elemmt, elemmt1, icount, icounthalf, 1, 2, &
                    2, .true., 1d0 )

1000  call erclos ( 'elm808matrix' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( elemmt, icount**2, 'elemmt' )
         write(irefwr,*) 'End elm808matrix'

      end if  ! ( debug )

      end
