      subroutine elm811
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 12-08-2013 Extra parameters
!        version  1.3  date 12-12-2012 Long integers
!
!   copyright (c) 2009-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, B%without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for computation of pseudo stream
!     function psi defined by / u.n d Gamma
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
!     phi           i/o   array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phixi         i/o   Array of length n x m x ndim containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element.
!                         Array phi is only filled when ifirst = 0 and must be
!                         kept by the program
!     psi           i/o   Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     psixi         i/o   Array of length nvert x m x ndim containing the values
!                         of the derivatives of the basis functions in the Gauss
!                         points in the reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jelp, iseqvel, iflipflop, metup
      double precision dudx(3*3*inpelm), dpsidx(3*inpelm*8), &
                       work(inpelm*inpelm+inpelm*7), &
                       ugauss(3*inpelm), uvel(3*inpelm), &
                       velocity(3*inpelm), alpha(1)
      logical linear
      save iseqvel

!     alpha          dummy
!     B%phider         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = B%phider (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = B%phider (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = B%phider (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dpsidx         dummy
!     dudx           dummy
!     iflipflop      dummy
!     iseqvel        Sequence number of velocity in set of vectors
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELxxxx.
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     metup          Upwind parameter
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     ugauss         Preceding solution in Gauss points
!     uvel           Contains the velocity vector wrt convection (1)
!     velocity       Contains the velocity vector in the nodes
!     w              array of length m containing the weights for
!                    integration
!     work           Work array to be used by some under subroutines
!     x              array of length n x ndim containing the coordinates of
!                    the nodes
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0800ELP      Set parameters depending on shape of element
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     EL3001         Compute integrals of the shape / u(x).grad phi phi (x) dx
!                                                   e -            j   i
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELM800BASEFN   Compute the basis functions B%phi, its derivatives and other
!                    related quantities for various element shapes
!     ELM800UPWBASE  Compute the modified basis functions psi due to upwinding
!     ELM811VEC      Fill element right-hand side vector for computation of
!                    pseudo stream function
!     ELMMAPVELSURF  Map velocity vector onto surface using array qmat
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERROR42        Print error message 42
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  Sequence number of velocity vector
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates
!         2: Polar co-ordinates
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1787   iseq has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!       Differential equation:
!
!      grad psi = u.n
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm811' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm811'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0

!        --- initialize parameters in commons

         call elcomm

!        --- Set parameters depending on the shape numbers

         call el0800elp ( linear, jelp )
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         call sepelbasefnch
allocate ( B%psi(B%n,B%m))
B%fillpsi = .true.
         ipos = E%iuser ( ielgrp + 5 )
         if ( ipos<7 ) call error42 ( ielgrp, ipos, ielem, itype )

         iseqvel = E%iuser(ipos)   ! sequence number of velocity vector

         if ( debug ) write(irefwr,1) 'iseqvel', iseqvel

         if ( iseqvel==0 ) iseqvel = 1

         if ( iseqvel<=0 .or. iseqvel>numold ) then

!        --- iseqvel out of range

            call errint ( iseqvel, 1 )
            call errint ( numold, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 1787, 5, 0, 0 )

         end if  ! ( iseqvel<=0 .or. iseqvel>numold )

      end if  ! ( ifirst==0 )
      if ( ierror/=0 ) go to 1000

!     --- compute basis functions and weights for numerical integration

      B%compqmat = .true.
      B%rule = 3
      B%m = n

      call sepelbasefunc
      iweigh = 1
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'jelp', B%type_basisfunc
         call prinrl1 ( B%phider, m, n, 'dphidx' )
         call prinrl1 ( B%phider(1,1,2), m, n, 'dphidy' )
         call prinrl1 ( B%qmat, m, 9, 'qmat' )
      end if  ! ( debug )

!     --- Fill array velocity with the velocity

      jind = 1       ! constant number of unknowns required
      ncomp = ndim   ! number of components
      call el2005 ( velocity, dudx, B%phi, B%phider, E%nodes, &
                    ugauss, iseqvel )
      if ( ierror/=0 ) go to 1000
      idim = ndim-1
      if ( debug ) then
         call prinrl1 ( velocity, n, ndim, 'velocity' )
         call prinrl1 ( ugauss, m, ndim, 'ugauss' )
      end if  ! ( debug )

!     --- Fill element matrix (convective term, using velocity)
!         The velocity is mapped onto the surface by using the tangential
!         vectors

      uvel = ugauss    ! velocity in integration points

!     --- Map uvel onto surface

      call elmmapvelsurf ( uvel, B%qmat )

      metup = 1      ! Type of upwind method
      jsecnd = 0     ! No contribution of diffusion
      call elm800upwbase ( metup, B%type_basisfunc, alpha, uvel, B%phi, &
                           B%psi, B%xyz, .false., B%phider, dudx, iflipflop, &
                           dpsidx, B%w )

!      call sepelbaseupw
      if ( debug ) then
         call prinrl1 ( B%psi, m, n, 'psi' )
         call prinrl1 ( ugauss, m, 3, 'ugauss' )
      end if  ! ( debug )

!     --- Compute element matrix

      call el3001 ( uvel, E%mat, B%w, B%psi, B%phider, ndim-1, n, work )
      if ( ierror/=0 ) go to 1000

!     --- Fill element vector

      if ( debug ) then
         call prinrl1 ( velocity, 2, n, 'velocity' )
      end if  ! ( debug )

      call elm811vec ( ugauss )

      if ( E%last==1 ) then
         call sepeldealloc
      end if  ! ( E%last==1 )

1000  if ( debug ) then

         write(irefwr,*) 'element ', ielem, '  itype ',itype
         call prinrl(E%vec,icount,'elemvc')
         call prinrl1(E%mat,icount,icount,'elemmt')
         write(irefwr,*) 'End elm811'

      end if
      call erclos ( 'elm811' )

      end
