      subroutine elm812
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 12-08-2013 Extra parameters
!        version  1.1  date 10-06-2013 Correction length dphidx
!
!   copyright (c) 2009-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for smoothing operator
!     on surface
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
!     phi           i/o   array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phixi         i/o   Array of length n x m x ndim containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element.
!                         Array phi is only filled when ifirst = 0 and must be
!                         kept by the program
!     psi           i/o   Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     psixi         i/o   Array of length nvert x m x ndim containing the values
!                         of the derivatives of the basis functions in the Gauss
!                         points in the reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jelp
      double precision alpha(inpelm), work(3*inpelm)
      logical linear

!     alpha          Contains the diffussion constant (1)
!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dpsidx         dummy
!     jelp           Choice parameter to distinguish between the types of
!                    element subroutines to be called. Possible values:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP622  (Quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     w              array of length m containing the weights for
!                    integration
!     work           Work array to be used by some under subroutines
!     x              array of length n x ndim containing the coordinates of
!                    the nodes
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0800ELP      Set parameters depending on shape of element
!     EL3004         Compute integrals of the shape
!                    /_e alpha grad phi_i . grad phi_j dx
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELM800BASEFN   Compute the basis functions B%phi, its derivatives and other
!                    related quantities for various element shapes
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!       Differential equation:
!       - div  grad phi  = 0
!                                                  - &
!        The following formulae are used:
!           S (i,j) = / grad phi  . grad phi   de
!                    e          j           i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm812' )
      debug = .false.

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0

!        --- initialize parameters in commons

         call elcomm

!        --- Set parameters depending on the shape numbers

         call el0800elp ( linear, jelp )
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         call sepelbasefnch

      end if
      if ( ierror/=0 ) go to 1000

!     --- compute basis functions and weights for numerical integration

      call sepelbasefunc
      iweigh = 1
      if ( ierror/=0 ) go to 1000

!     --- Fill element matrix (diffusion term, using alpha = 1)

      alpha(1:m) = 1d0
      jsecnd = 2   ! Laplace, i.e. a_12 = 0; a_11 = a_22
      call el3004 ( alpha, E%mat, B%w, B%phider, m, n, n )
      if ( ierror/=0 ) go to 1000

      notvec = 1   ! element vector = 0

      if ( E%last==1 ) then
         call sepeldealloc
      end if  ! ( E%last==1 )

1000  if ( debug ) then

!     --- debug = true

         write(irefwr,*) 'element ', ielem, '  itype ',itype
         call prinrl1(E%mat,icount,icount,'elemmt')

      end if
      call erclos ( 'elm812' )

      end
