      subroutine elmatinv ( s, sinv, singular )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 13-08-2012 Extra parameter singular
!        version  1.0  date 29-12-1996
!
!   copyright (c) 1996-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the inverse of a 3 x 3 matrix s and store in sinv
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_inversion
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision s(3,3), sinv(3,3)
      logical singular

!     s              i    3x3 matrix to be inverted
!     singular      i/o   At input: if false and the matrix is singular
!                                   an error message is given
!                                   if true and the matrix is singular
!                                   no error message is given but singular
!                                   gets the value true
!     sinv           o    inversion of s
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision det

!     det            Determinant of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2186   The matrix to be inverted is singular
!    3111   Determinant of matrix is small compared to the elements of diagonal
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The inverse of the matrix s is given by:
!    det = s_11 ( s_22 s_33 - s_23 s_32 ) + &
!          s_12 ( s_23 s_31 - s_21 s_33 ) + &
!          s_13 ( s_21 s_32 - s_22 s_31 )
!    det  s_11^-1 = s_22 s_33 - s_23 s_32
!    det  s_12^-1 = s_32 s_13 - s_33 s_12
!    det  s_13^-1 = s_12 s_23 - s_13 s_22
!    det  s_21^-1 = s_23 s_31 - s_21 s_33
!    det  s_22^-1 = s_33 s_11 - s_31 s_13
!    det  s_23^-1 = s_13 s_21 - s_11 s_23
!    det  s_31^-1 = s_21 s_32 - s_22 s_31
!    det  s_32^-1 = s_31 s_12 - s_32 s_11
!    det  s_33^-1 = s_11 s_22 - s_12 s_21
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmatinv' )
      det = s(1,1) * ( s(2,2)*s(3,3) - s(2,3)*s(3,2) ) + &
            s(1,2) * ( s(2,3)*s(3,1) - s(2,1)*s(3,3) ) + &
            s(1,3) * ( s(2,1)*s(3,2) - s(2,2)*s(3,1) )
      if ( abs(det)<=0d0 ) then

!     --- matrix singular

         if ( singular ) go to 1000  ! jump
         call errsub ( 2186, 0, 0, 0 )
         go to 1000
      else if ( abs(det)<=1d-6*(abs(s(1,1))+abs(s(2,2))+abs(s(3,3))) ) &
         then

!     --- Warning determinant small compared to elements of matrix

         if ( singular ) go to 1000  ! jump
         call erreal ( det, 1 )
         call erreal ( s(1,1), 2 )
         call errwar ( 3111, 0, 2, 0 )
      end if  ! ( abs(det)<=0d0 )
      singular = .false.
      det = 1d0/det
      sinv(1,1) = det * ( s(2,2)*s(3,3) - s(2,3)*s(3,2) )
      sinv(1,2) = det * ( s(3,2)*s(1,3) - s(3,3)*s(1,2) )
      sinv(1,3) = det * ( s(1,2)*s(2,3) - s(1,3)*s(2,2) )
      sinv(2,1) = det * ( s(2,3)*s(3,1) - s(2,1)*s(3,3) )
      sinv(2,2) = det * ( s(3,3)*s(1,1) - s(3,1)*s(1,3) )
      sinv(2,3) = det * ( s(1,3)*s(2,1) - s(1,1)*s(2,3) )
      sinv(3,1) = det * ( s(2,1)*s(3,2) - s(2,2)*s(3,1) )
      sinv(3,2) = det * ( s(3,1)*s(1,2) - s(3,2)*s(1,1) )
      sinv(3,3) = det * ( s(1,1)*s(2,2) - s(1,2)*s(2,1) )
1000  call erclos ( 'elmatinv' )
      end
