      subroutine elmfictlam ( coor2d, index1, ishape_curve, ndimloc, &
                              xgauss, philam, w, ishapelam, &
                              computephi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.5  date 29-07-2013 New call to elp621
!        version  3.4  date 07-04-2004 Extension for shape 6 (RvL)
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute gauss points, weights and basis functions for lambda in fictitious
!    element
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     fictitious_domain
!     basis_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*), ishapelam, ndimloc, ishape_curve
      double precision coor2d(ndimloc,*), xgauss(*), philam(*), w(*)
      logical computephi

!     computephi     i    if true the basis functions in the point must be
!                         computed
!     coor2d           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     index1         i    Array of length jnpelm containing the point numbers
!                         of the nodal points in the element
!     ishapelam      i    Shape of lambda basis functions
!     ishape_curve   i    Shape of the curve on which the lambdas are defined
!     ndimloc        i    Dimension of the space.
!     philam         o    Contains the values of the lambda basis functions
!                         in the integration points
!                         philam(i,j) coor2desponds to the i-th point, &
!                         j-th integration point
!     w              o    array of length maxlambda containing the weights for
!                         integration
!     xgauss         o    array of length maxlambda x ndim containing the
!                         co-ordinates of the gauss points.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x(27), y(3), phi(9,9), phider(81,2), xksi(3), &
                       yksi(3), qmat(1), psi(7*3), dpsidx(9*4*2), &
                       phiksi(84)
      integer i, j, indexvert(4), jnpelm

!     dpsidx         Array of length n x m x l  containing the derivatives
!                    of the linear basis functions in the sequence:
!                    d psi / dx (xg ) = psix (i,k,1);
!                         i        k
!                    d psi / dy (xg ) = psix (i,k,2);
!                         i        k
!                    d psi / dz (xg ) = psix (i,k,3);
!                         i        k
!     i              Counting variable
!     indexvert      Contains the node numbers of the vertices
!     j              Counting variable
!     jnpelm         Numebr of points per element
!     phi            array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                    phi_i(xg^k) = phi (i,k)
!                    array phi is only filled when gauss integration is
!                    applied
!     phider         Contains the derivatives of the basis functions in the
!                    reference element
!                    phider(i,j) = d phi_i/ ds (x^j)
!     phiksi         contains derivatives in reference element for gauss
!                    rule.
!     psi            Array of length n x m containing the values of the
!                    modified basis functions in the integration points
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     x              array with the x co-ordinates of the nodes
!     xksi           Contains dx / d ksi in the integration points
!     y              array with the y co-ordinates of the nodes
!     yksi           Contains dy / d ksi in the integration points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP620         Compute basis functions and weights for linear triangle
!     ELP621         Compute basis functions and weights for bilinear
!                    quadrilateral
!     ELP622         Compute basis functions and weights for quadratic triangle
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( ndim = 2 ) then
!        jnpelm := ishapelam+1
!        Fill coordinates in arrays x and y
!        if ( ishapelam = 1 ) then
!            Fill everything for constant element
!        else if ( ishapelam = 2 ) then
!            Fill phi with values of quadratic basis functions in Gauss points
!            Fill Gauss points
!            if ( computephi = true ) then
!               Fill philam with linear basis functions in Gauss points
!               Fill phider with derivatives of basis functions in reference
!               element
!               Compute dx/dxsi and dx/deta
!               Compute weights
!        else if ( ishapelam>2 ) then
!           error message
!     else ( ndim =3 ) then
!        Call element subroutines
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmfictlam' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmfictlam'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndim = ndimloc
      if ( ndim==2 ) then

!     --- ndim = 2

         jnpelm = ishape_curve+1

!        --- Fill coordinates in arrays x and y

         do i = 1, jnpelm
            x(i) = coor2d(1,index1(i))
            y(i) = coor2d(2,index1(i))
         end do

         if ( ishape_curve==1 ) then

!        --- Linear element, 1 point Gauss rule

            m = 1
            xgauss(1) = (x(1)+x(2))*0.5d0
            xgauss(2) = (y(1)+y(2))*0.5d0
            if ( computephi ) then

!           --- computephi = true, compute phi and weight

               philam(1) = 1d0
               w(1) = sqrt((x(2)-x(1))**2+(y(2)-y(1))**2)

            end if

         else if ( ishape_curve==2 ) then

!        --- Quadratic element

            if ( ishapelam==1) then

!           --- 1 point Gauss rule

               m = 1
               xgauss(1) = x(2)
               xgauss(2) = y(2)
               if ( computephi ) then

!              --- computephi = true, compute phi and weight

                  w(1) = sqrt((x(3)-x(1))**2+(y(3)-y(1))**2)
                  philam(1) = 1d0

               end if

            else if ( ishapelam==2 ) then

!           --- 2 point Gauss rule
!               Fill phi with values of quadratic basis functions in Gauss
!               node

               do j = 1, 2
                  phi(1,j) = (1d0-alam4(j)) * ( 2d0*(1d0-alam4(j))-1d0 )
                  phi(2,j) = 4d0 * alam4(j) * (1d0-alam4(j))
                  phi(3,j) = alam4(j) * ( 2d0*alam4(j)-1d0 )
               end do

!              --- Fill Gauss points

               m = 2
               do j = 1, 2
                  xgauss(j)   = x(1)*phi(1,j)+x(2)*phi(2,j)+ &
                                x(3)*phi(3,j)
                  xgauss(j+m) = y(1)*phi(1,j)+y(2)*phi(2,j)+ &
                                y(3)*phi(3,j)
               end do
               if ( computephi ) then

!              --- computephi = true, compute philam and weight
!                  phi(1) = 1-s, phi(2) = s

                  philam(1) = 1d0-alam4(1)
                  philam(2) = 1d0-alam4(2)
                  philam(3) = alam4(1)
                  philam(4) = alam4(2)

!                 --- Fill phider with derivatives of basis functions in
!                     --- reference
!                     element

                  do j = 1, 2
                     phider(1,j) = -(4d0*(1d0-alam4(j))-1d0)
                     phider(2,j) = 4d0*(1d0-2d0*alam4(j))
                     phider(3,j) = 4d0*alam4(j)-1d0
                  end do

!                 --- Compute dx/dxsi and dx/deta

                  do j = 1, 2
                     xksi(j) = 0d0
                     do i = 1, 3
                        xksi(j) = xksi(j)+x(i)*phider(i,j)
                     end do
                     yksi(j) = 0d0
                     do i = 1, 3
                        yksi(j) = yksi(j)+y(i)*phider(i,j)
                     end do

                  end do

!                 --- Compute w

                  do i = 1, 2
                     w(i) = sqrt(xksi(i)**2+yksi(i)**2)*0.5d0
                  end do

               end if

            else

!           --- ishapelam>2, has not yet been implemented

               call errint ( ishapelam, 1 )
               call errsub ( 800, 1, 0, 0 )

            end if

         end if

      else

!     --- ndim = 3

         iweigh = 0
         iphi = 0
         jcart  = 1
         interp = 0
         isecdv = 0
         icoor = 1
         ideriv = 0
         igausp = 1
         jdiag = 2
         irule = 2
         if ( ishape_curve==3 ) then
            n = 3
         else if ( ishape_curve==4 ) then
            n = 6
         else if ( ishape_curve==5 ) then
            n = 4
         else
            n = 9
         end if
         do i = 1, n
            x(i) = coor2d(1,index1(i))
            x(i+n) = coor2d(2,index1(i))
            x(i+2*n) = coor2d(3,index1(i))
         end do
         if ( computephi ) then

!        --- computephi = true, compute phi and weight

            iweigh = 1
            iphi = 1

         end if

         if ( ishape_curve==3 ) then

!        --- Linear triangular element, 1 point Gauss rule

            if ( ishapelam==5 ) then

               m = 1
               ifirst = 0
               call elp620 ( coor2d, x, w, phider, xgauss, philam, &
                             index1, qmat )
               philam(1) = 1d0

            else if ( ishapelam==6 ) then

               m = 3
               ifirst = 0

!              --- use only vertices

               indexvert(1) = index1(1)
               indexvert(2) = index1(3)
               indexvert(3) = index1(5)

               irule = 1

               call elp620 ( coor2d, x, w, phider, xgauss, philam, &
                             indexvert, qmat )
               philam(1) = 1d0

            end if

         else if ( ishape_curve==4 ) then

!        --- Quadratic triangle, 4 point Gauss rule

            m = 4
            call elp622 ( coor2d, x, w, phider, xgauss, philam, index1, &
                          qmat, psi, dpsidx, phiksi )

         else if ( ishape_curve==5 ) then

!        --- Bilinear quadrilateral, 1 point Gauss rule

            m = 1
            ifirst = 0
            call elp621 ( coor2d, x, w, phider, xgauss, philam, index1, &
                          qmat, phiksi )

            philam(1) = 1d0

         else if ( ishape_curve==6 ) then

!        --- Bi-quadratic quadrilateral

            if ( ishapelam==5 ) then

!           --- 1 point Gauss rule

               m = 1

!              --- use only vertices

               indexvert(1) = index1(1)
               indexvert(2) = index1(3)
               indexvert(3) = index1(5)
               indexvert(4) = index1(7)

               irule = 2
               ifirst = 0
               call elp621 ( coor2d, x, w, phider, xgauss, philam, &
                             indexvert, qmat, phiksi )

!              --- philam = 1, because there is only one lamba instead of 4

               philam(1) = 1d0

            else if ( ishapelam==6 ) then

!           --- 4 point Gauss rule

               m = 4

!              --- use only vertices

               indexvert(1) = index1(1)
               indexvert(2) = index1(3)
               indexvert(3) = index1(5)
               indexvert(4) = index1(7)

               irule = 3
               ifirst = 0
               call elp621 ( coor2d, x, w, phider, xgauss, philam, &
                             indexvert, qmat, phiksi )

            else

!           --- ishapelam>2, has not yet been implemented

               call errint ( ishapelam, 1 )
               call errsub ( 800, 1, 0, 0 )

            end if

         end if

      end if

1000  call erclos ( 'elmfictlam' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( philam, m, n, 'philam' )
         call prinrl ( w, m, 'w' )
         call prinrl1 ( xgauss, m, ndim, 'xgauss' )
         write(irefwr,*) 'End elmfictlam'

      end if  ! ( debug )

      end
