      subroutine elp610 ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          qmat, tang )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.0  date 10-06-2013 Use subroutines
!        version  3.10 date 12-02-2010 Fill dphidx completely
!
!   copyright (c) 1992-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for linear line element
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP610 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(2)
      double precision coor2d(ndim,*), x(2,ndim), w(m), dphidx(2,m), &
                       xgauss(m,ndim), phi(2,m), qmat(3,3), tang(m,2)

!     coor2d     i     array of length number of nodal points containing
!                    the co-ordinates of the nodal points with respect to
!                    the global numbering
!                    x  = coor2d (i)
!                     i
!     dphidx   o     Array of length 2 x m  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx = dphidx (i,1);  d phi / dy = dphidx (i,1,2);
!                         i                             i
!     index1   i     Array of length 2 containing the nodal point numbers
!                    of the element
!     phi      o     Array of length 2 x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                           k
!                    phi (xg ) = phi (i,k)
!                       i
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     qmat     o     transformation matrix global -> local coordinates
!                    in case of a 3D element
!     tang     o     In this array the tangential vector is stored in the case
!                    that icoor>10
!                    tang(i,j) contains the j th component of the tangential
!                    vector in the i th integration point
!     w        o     array of length m containing the weights for the
!                    numerical integration
!     x       i/o    array of length 2 containing the
!                    x-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i)
!                     i
!     xgauss   o     Array of length m containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i)
!                      i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision h, hloc, d, amin
      integer i, k, jinteg, iweigo, iqmat, icooro, itang, jcartsav
      double precision v1work(3), v2work(3), xloc(2), wh(3), &
                       v3work(3), lambda(3)
      save jinteg, iweigo, iqmat, icooro, itang, jcartsav, lambda, wh

!     amin           Help variable to compute a minimum
!     d              Help variable to compute the weights
!     h              Help variable to store part of a weight
!     hloc           Local weight
!     i              Counting variable
!     icooro         if 1 the coordinates must be filled
!     iqmat          if 1 the qmat matrix must be filled
!     itang          if 1 the tangential vector must be filled
!     iweigo         if 1 the weights must be filled
!     jcartsav       help variable to save the value of jcart
!     jinteg         jtrans/100, indicates the direction of integration
!                    0: usual direction
!                    >0: x_jinteg direction
!     k              Counting variable
!     lambda         Array containing the linear basis functions in the
!                    integration points
!     v1work         Contains the first vector in qmat
!     v2work         Contains the second vector in qmat
!     v3work         Contains the third vector in qmat
!     wh             Contains the integration weights for a unit element
!     xloc           Contains the values of the local co-ordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     ELP610FIRST    Fill element independent quantities for a linear line
!                    element
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR0    Print error message errornr for element
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!        h   =  x2 - x1
!        d phi_1/dx = -1/h        d phi_2/dx = 1/h
!
!        Newton Cotes rule:  /_e f(x) dx =|h|/2 [ f(x^1)+f(x^2) ]
!
!        The matrix qmat is filled as follows:
!        qmat(.,3) = (x2-x1)/||x2-x1||   t vector
!        ej is unity vector in direction of mallest component of |qmat(.,3)|
!        qmat(.,2) = (qmat(.,3) x ej) / ||qmat(.,3) x ej||
!        qmat(.,1) = qmat(.,2) x qmat(.,3) ! normal vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp610' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp610'
         write(irefwr,1) 'igausp', igausp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 ) then

!     --- ifirst = 0  (First call)

         call elp610first ( wh, lambda, phi, jinteg, iweigo, iqmat, &
                            itang, jcartsav, icooro )

      end if  ! ( ifirst==0 )
      if ( ierror/=0 ) go to 1000

!     --- Store coordinates in x

      do k = 1, ndim
         x(1:2,k) = coor2d(k,index1(1:2))
      end do

!     --- Compute length of element

      if ( ndim==1 ) then
         v1work(1) = x(2,1)-x(1,1)
         h = v1work(1)
      else
         v1work(1:ndim) = x(2,1:ndim)-x(1,1:ndim)
         h = dot_product(v1work(1:ndim),v1work(1:ndim))
         h = sqrt(dot_product(v1work(1:ndim),v1work(1:ndim)))
      end if
      hloc = abs(h)
      if ( jinteg>=1 ) hloc = x(2,jinteg) - x(1,jinteg)

      if ( abs(h)<=0d0 ) then

!     --- Error 105: h is zero

         call errorelvar0 ( 105 )
         call prinin(index1,2,'node numbers')
         call prinrl2(x,2,ndim,'co-ordinates')
         go to 1000

      end if  ! ( hloc<=0d0 )

      if ( igausp==1 .or. irule>1 .and. jcartsav==3 ) then

!     --- gauss points must be filled

         do i = 1, m
            xgauss(i,1:ndim) = x(1,1:ndim)+lambda(i)*v1work(1:ndim)
         end do

      end if  ! ( igausp==1 .or. irule>1 .and. jcartsav==3 ) then

      if ( iweigo==1 ) then

!     --- iweigo = 1  Fill weights

         if ( irule==1 .and. jcartsav==3 ) then

!        --- irule=1 and jcart==3, polar coordinates

            d = h * pi / 3d0
            w(1) = d * ( 2d0*x(1,1) + x(2,1) )
            w(2) = d * ( x(1,1) + 2d0*x(2,1) )

         else if ( irule>1 .and. jcartsav==3 ) then

!        --- irule>1 and jcart==3, polar coordinates

            w(1:m) = w(1:m) * xgauss(1:m,1) * pi2

         else

!        --- Standard case

            w(1:m) = hloc*wh(1:m)

         end if  ! ( irule==1 .and. jcartsav==3 )

      end if  ! ( iweigo==1 )

      if ( ideriv==1 ) then

!     --- ideriv = 1   Fill derivatives

         dphidx(1,1) = -1d0 / h
         dphidx(2,1) = -dphidx(1,1)
         dphidx(1,2:m) = dphidx(1,1)
         dphidx(2,2:m) = dphidx(2,1)

      end if

      if ( icooro==1 ) then

!     --- icoor = 1   Fill co-ordinates of nodes in array x

         do i = 1, 2
            x(i,1:ndim) = coor2d(1:ndim,index1(i))
         end do

         if ( ndim>1 .and. iqmat==1 ) then

!        --- ndim>1, transform to 1D
!            v3work = x2 - x1

            xloc(1) = 0d0
            v3work(1) = x(2,1)-x(1,1)
            v3work(2) = x(2,2)-x(1,2)
            if ( ndim==2 ) then
               v3work(3) = 0d0
               xloc(2) = sqrt( v3work(1)**2 + v3work(2)**2 )
            else
               v3work(3) = x(2,3)-x(1,3)
               xloc(2) = sqrt( v3work(1)**2 + v3work(2)**2 + &
                               v3work(3)**2 )
            end if
            if ( iqmat>=1 ) then

!           --- iqmat>=1, Fill array qmat

               h = 1d0/xloc(2)

!              --- Compute index of smallest component of |v3work|

               k = 1
               amin = abs(v3work(3))
               if ( abs(v3work(1))>amin ) then
                  k = 2
                  amin = abs(v3work(1))
               end if
               if ( abs(v3work(2))>amin ) k = 3

               do i = 1, 3
                  v3work(i) = v3work(i)*h
                  v2work(i) = 0d0
               end do
               v2work(k) = 1d0

!              --- v3work is normalized
!                  Set v2work equal to ej

               call eloutr ( v3work, v2work, v1work )

!              --- v1work = (qmat(.,3) x ej) / ||qmat(.,3) x ej||

               h = 1d0/sqrt ( v1work(1)**2 + v1work(2)**2 + &
                              v1work(3)**2 )
               do i = 1, 3
                  v2work(i) = v1work(i)*h
               end do

!              --- normalize v2work
!                  v1work = qmat(.,2) x qmat(.,3)

               call eloutr ( v2work, v3work, v1work )
               do i = 1, 3
                  qmat(1,i) = v1work(i)  ! n
                  qmat(2,i) = v2work(i)  ! t1
                  qmat(3,i) = v3work(i)  ! t2
               end do

            end if

         end if

      end if

      if ( itang>=1 ) then

!     --- icoor>=10  Fill tang

         do i = 1, ndim
            tang(1:m,i) = (x(2,i) - x(1,i))/abs(h)
         end do

      end if
      if ( iqmat>5 ) then

!     --- iqmat = 6 or 7, fill e_zeta in qmat

         do i = 1, ndim
            qmat(1,i) = (x(2,i) - x(1,i))/abs(h)
         end do

      end if

1000  call erclos ( 'elp610' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( xgauss, m, 'xgauss' )
         write(irefwr,*) 'End elp610'

      end if  ! ( debug )

      end
