      subroutine elp621 ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          qmat, dphidxi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.0  date 10-06-2013 Use subroutines
!        version  3.15 date 09-03-2013 Correction declaration wh
!
!   copyright (c) 1989-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for bilinear quadrilateral
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP621 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadrilateral
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(4)
      double precision coor2d(ndim,*), x(4,ndim), w(*), &
                       dphidx(4,m,ndim), &
                       xgauss(m,ndim), phi(4,m), qmat(3,3,*), &
                       dphidxi(4,m,2)

!     coor2d     i     array of length 2 x number of nodal points containing
!                    the co-ordinates of the nodal points with respect to
!                    the global numbering
!                    x  = coor2d (1,i);  y  = coor2d (2,i);
!                     i                 i
!     dphidx   o     Array of length 4 x m x 2  containing the derivatives
!                    of the basis functions in the sequence:
!                                 k
!                    d phi / dx (x )= dphidx (i,k,1);
!                         i
!                                 k
!                    d phi / dy (x )= dphidx (i,k,2);
!                         i
!     dphidxi  o     contains derivatives in reference element for gauss
!                    rule.
!                    dphidxi(i,k,l) = d phi_i/ d xi_l (x^k)
!     index1   i     Array of length 4 containing the nodal point numbers
!                    of the element
!     phi      o     Array of length 4 x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                           k
!                    phi (xg ) = phi (i,k)
!                       i
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     qmat     o     transformation matrix global -> local coordinates
!                    in case of a 3D element
!     w        o     array of length m containing the weights for the
!                    numerical integration
!     x       i/o    array of length 4 x 2 containing the x- and
!                    y-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i,1);  y  = x (i,2);
!                     i              i
!     xgauss   o     Array of length m x 2 containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                      i                    i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision h, dxdxi(m,3,3), &
                       wh(8), xg(4,2), vec(3), &
                       dxidx(4,2,3), delta(4), weight(4)
      integer i, k, iqmat, iweigo, &
              intrule, jtransfo, jcartorg, inorm, itang
      save xg, weight

!     delta          work array of length 4 to store the values of the jacobian
!                    in the nodes
!     dxdxi          work array of length mx3x3 to store the values of
!                    dx/dxi, dx/deta, dy/dxi and dy/deta according to:
!                    dx/dxi(x_k) = dxdxi(k,1,1)
!                    dx/deta(x_k) = dxdxi(k,1,2)
!                    dy/dxi(x_k) = dxdxi(k,2,1)
!                    dy/deta(x_k) = dxdxi(k,2,2) etc.
!     dxidx          work array of length 7x2x3 to store the values of dxi/dx, &
!                    dxi/dy, dxi/dz and deta/dx ... according to:
!                             k                             k
!                    dxi/dx(x ) = dxdxi(k,1,1)   dxi/dy(x ) = dxdxi(k,1,2)
!     h              Help variable to store a quantity temporarily
!     i              Counting variable
!     inorm          Indicates which vector in qmat is equal to the normal
!                    vector (iqmat = 2: 3;iqmat = 3: 1)
!     intrule        Actual intergration rule
!     iqmat          if 1 the qmat matrix must be filled
!     itang          Indicates which vector in qmat is equal to the first
!                    tangential vector (iqmat = 2: 1;iqmat = 3: 3)
!     iweigo         if 1 the weights must be filled
!     jcartorg       Parameter jcart_orig as defined in jcart
!     jtransfo       Parameter jtransform as defined in jcart
!     k              Counting variable
!     vec            work array to store a vector
!     weight         Contains the weights for the integration rule in the
!                    reference element
!     wh             Array of length 4 containing the values of the Jacobian in
!                    the vertices or for the Gauss integration rules work array
!                    to store some help variables
!     xg             Array of length 4x2 containing the co-ordinates of the
!                    Gauss points in the reference element for a four point
!                    Gauss rule
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELFILLQMAT     Fill the transformation matrix qmat for the mapping of a
!                    3d surface onto a 2d surface
!     ELP3DTO2DDELTA Compute the jacobian delta and dx/dxi for a 2d element
!                    in 3d
!     ELP3DTO2DDERIV Compute the derivatives dphi/dx for a 2d element in 3d
!     ELP620TRANSFO  Compute x as function of jtransfo
!     ELP621DERIV    Compute derivatives of basis functions in case of
!                    bi-linear quadrilateral
!     ELP621FIRST    Fill element independent quantities for a bilinear
!                    quadrilateral
!     ELP621JACOB    Compute the Jacobian of a quadrilateral
!     ELPCOOR        fill co-ordinates of nodes in array x
!     ELPGAUSS       Fill Gauss coordinates
!     ELTRANS3DQUAD  Map coordinates from a bilinear quadrilateral surface in
!                    R^3 onto a two-dimensional representation
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!         ij       j     i+1    i
!        e   = (-1)  (  x    - x    )   cyclic: i=1,2,3,4 ;   j=1,2
!                        j+1    j+1
!              1     2    1          4    1          3    2    1    4
!        x  = x  + (x  - x ) xi + (x  - x ) eta + (x  - x  + x  - x ) xi eta
!         i    i     i    i          i    i          i    i    i    i
!                                   i = 1,2
!                  1    12        42
!        So: x  = x  + e   xi - e   eta + A xi eta
!             1    1
!                  1    11        41
!            x  = x  - e   xi + e   eta + B xi eta
!             2    2
!            |   2    1                  4    1          |
!            |  x  - x  + A eta         x  - x  + A xi  |
!            |   1    1                  1    1          |
!        J  =|                                           |
!            |   2    1                  4    1          |
!            |  x  - x  + B eta         x  - x  + B xi  |
!            |   2    2                  2    2          |
!             1    2    3    4        12    32
!        A = x  - x  + x  - x    = - e   - e
!             1    1    1    1
!             3    4    2    1       11    31
!        B = x  - x  - x  + x    =  e   + e
!             2    2    2    2
!       |J| = J  + J  xi + J  eta
!              0    1        2
!              41   12    11   42
!        J  = e  * e   - e  * e
!         0
!                   12         11
!        J  = B  * e   - A  * e
!         1
!                   41         42
!        J  = A  * e   + B  * e
!         2
!        dphi / dx  = dphi / dxi dxi/dx  + dphi / deta deta/dx
!                 j                      j                      j
!                       41
!        dxi/dx  = [  e   + B xi ] / |J|
!               1
!                       42
!        dxi/dx  = [  e   - A xi ] / |J|
!               2
!                       11
!        deta/dx  = [  e   - B eta ] / |J|
!               1
!                       12
!        deta/dx  = [  e   + A eta ] / |J|
!               2
!                                          1   4       k      k
!        Newton Cotes rule:  / f(x) dx =  --- sum [ f(x ) |J(x )| ]
!                            e             4  k=1
!     In case of 3D elements we use the following formulae:
!     x = sum phi (xi,eta) x    dx/dxi = sum dphi /dxi x   dx/deta=...
!     -    i     i          -i    -         i      i      -i
!             a                      a                         a
!     dphi /dx = dphi / dxi  dxi/dx   +  dphi / deta  deta/dx
!         i          i                         i
!                                                                        -1
!     | dxi/dx   deta/dx   dzeta/dx  |   | dx/dxi   dy/dxi   dz/dxi |
!     | dxi/dy   deta/dy   dzeta/dy  | = | dx/deta   dy/deta   dz/deta |
!     | dxi/dz   deta/dz   dzeta/dz  |   | dx/dzeta  dy/dzeta  dz/dzeta|
!     zeta is choosen perpendicular to the xi-eta direction, which implies
!     that we may choose:
!     dx / dzeta = dx / dxi times  dx / deta / || dx/dxi x dx/deta ||
!      -            -                -              -         - &
!     where times or x is the outer product
!     The Jacobian of the dx/dxi matrix is given by:
!     J = dx/dzeta ( dy/dxi dz/deta - dy/deta dz/dxi )
!       + dy/dzeta ( dz/dxi dx/deta - dz/deta dx/dxi )
!       + dz/dzeta ( dx/dxi dy/deta - dx/deta dy/dxi )
!     and since dx/dzeta before normalization is given by
!         dx/dzeta = dy/dxi dz/deta - dy/deta dz/dxi
!         dy/dzeta = dz/dxi dx/deta - dz/deta dx/dxi
!         dz/dzeta = dx/dxi dy/deta - dx/deta dy/dxi
!     we see that the normalization factor is equal to
!        sqrt(     ( dy/dxi dz/deta - dy/deta dz/dxi )^2
!                + ( dz/dxi dx/deta - dz/deta dx/dxi )^2
!                + ( dx/dxi dy/deta - dx/deta dy/dxi )^2 ) = J
!      The derivatives dxi/dx and so on follow from the inverse, &
!      which in this 3x3 case leads to:
!      dxi/dx = ( dy/deta  dz/dzeta - dy/dzeta dz/deta  ) / J
!      dxi/dy = ( dz/deta  dx/dzeta - dz/dzeta dx/deta  ) / J
!      dxi/dz = ( dx/deta  dy/dzeta - dx/dzeta dy/deta  ) / J
!      deta/dx = ( dy/dzeta dz/dxi  - dy/dxi  dz/dzeta ) / J
!      deta/dy = ( dz/dzeta dx/dxi  - dz/dxi  dx/dzeta ) / J
!      deta/dz = ( dz/dzeta dy/dxi  - dx/dxi  dy/dzeta ) / J
!     To compute qmat, we use the lamina coordinate system as introduced by
!     Hughes:
!      Define e_xi = x,xi / ||x,xi||
!      Define e_eta = x,eta / ||x,eta||
!      e_3(l) = e_xi x e_eta / ||e_xi x e_eta||
!      e_alpha = (e_xi + e_eta)/2 / || (e_xi + e_eta)/2 ||
!      e_beta = e_3(l) x e_alpha / || e_3(l) x e_alpha ||
!      e_1(l) = srqt(2)/2 ( e_alpha - e_beta )
!      e_2(l) = srqt(2)/2 ( e_alpha + e_beta )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp621' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp621'
         write(irefwr,1) 'ifirst, n, m, ideriv', ifirst, n, m, ideriv
         write(irefwr,1) 'inpelm', inpelm
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iweigo = mod(iweigh,10)
      intrule = mod(irule,100)
      iqmat  = iweigh/10
      jcartorg = mod(jcart,10)
      jtransfo = jcart/10
      itang = 0
      inorm = 0
      kstep = 1

      if ( ifirst==0 ) then

!     --- ifirst = 0  (First call)

         call elp621first ( xg, phi, dphidxi, intrule, jcartorg, weight )

      end if  ! ( ifirst==0 )

!     --- Store coordinates in x

      call elpcoor ( icoor, coor2d, index1, x, n, ndim, n )

!     --- Fill gauss points

      call elpgauss ( ndim, intrule, x, xgauss, phi )

      if ( ndim>2 ) then

!     --- Quadrilateral in three-dimensional space
!         transform co-ordinates

         if ( iqmat>5 ) then

!        --- iqmat>5, transform 3d coordinate system to new 2d system

            call eltrans3dquad ( iqmat, x, intrule, qmat, dphidxi )

         end if

      end if

      if ( ndim==2 .or. iqmat==1 .or. iqmat>5 .or. jtransfo>0 ) then

!     --- standard situation

         if ( jtransfo>1 .and. jtransfo<5 ) then

!        --- Perform transformation

            call elp620transfo ( x, jtransfo )

         end if  ! ( jtransfo>1 .and. jtransfo<5 )

         if ( iweigo==1 .or. ideriv==1 ) then

!        --- ideriv = 1  Compute derivatives or iweigo = 1: weights

            call elp621jacob ( x, wh, isubr, index1, &
                               xg, intrule, dxidx, ideriv )

         end if  ! ( iweigo==1 .or. ideriv==1 )

         if ( ideriv==1 ) then

!        --- ideriv = 1   Fill derivatives

            call elp621deriv ( intrule, dphidx, dxidx, dphidxi )

         end if

         if ( iweigo==1 ) then

!        --- iweigo = 1  Fill weights

            if ( jcartorg==1 ) then

!           --- jcartorg = 1  Cartesian co-ordinates

               w(1:m) = weight(1:m)*abs(wh(1:m))

            else if ( jcartorg>=2 ) then

!           --- jcartorg = 2  Axi-symmetric co-ordinates

               if ( intrule==1 ) then

!              --- intrule = 1  Newton-Cotes rule

                  wh(1:4) = abs(wh(1:4))*x(1:4,1)
                  h = 1d0/36d0
                  w(1) = h * (4d0*wh(1)+2d0*(wh(2)+wh(4))+wh(3))
                  w(2) = h * (4d0*wh(2)+2d0*(wh(3)+wh(1))+wh(4))
                  w(3) = h * (4d0*wh(3)+2d0*(wh(4)+wh(2))+wh(1))
                  w(4) = h * (4d0*wh(4)+2d0*(wh(1)+wh(3))+wh(2))

               else

!              --- intrule>1  point Gauss rule

                  w(1:m) = abs(wh(1:m))*xgauss(1:m,1)*weight(1:m)

               end if  ! ( intrule==1 )
               if ( jcartorg==2 ) w(1:m) = w(1:m)*pi2

            end if  ! ( jcartorg==1 )

         end if  ! ( iweigo==1 )

!        --- MR reset local coordinates:

         if ( iqmat>=2 ) then
            do k = 1, ndim
               x(1:4,k) = coor2d(k,index1(1:4))
            end do
         end if

      else

!     --- ndim = 3

         if ( iweigo==1 .or. ideriv==1 ) then

!        --- ideriv = 1  compute derivatives or iweigo = 1: weights
!            compute dx/dxi, dy/dxi, dx/deta, dy/deta;  store in dxdxi
!            Store jacobian in delta

            call elp3dto2ddelta ( x, dphidxi, dxdxi, delta, index1 )

         end if

         if ( iweigo==1 ) w(1:m) = weight(1:m)*delta(1:m)

         if ( ideriv==1 ) then

!        --- ideriv = 1   fill derivatives

            call elp3dto2dderiv ( dxdxi, delta, dphidx, dphidxi )

         end if

         if ( iqmat>1 ) then

!        --- Fill array qmat
!            Loop over integration points

            if ( iqmat==2 ) then

!           --- iqmat = 2, standard transformation t1, t2, n

               inorm = 1
               itang = 3

            else

!           --- iqmat = 3, transformation in elm250 n, t1, t2

               inorm = 1
               itang = 3

            end if

            call elfillqmat ( iqmat, dxdxi, m, itang, inorm, qmat )

         end if

      end if

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w(1..m ) contains the weights
!         w(m+1 .. m+n) the diagonal of lumped the mass matrix

         if ( intrule==1 ) then

!        --- intrule = 1  Newton Cotes rule, just a copy

            do i = 1, 4
               w(m+i)= w(i)
            end do

         else if ( jcartorg==1 ) then

!        --- jcartorg = 1  Cartesian co-ordinates

            do i = 1, 4
               w(m+i)=0.25d0*abs(wh(i))
            end do

         else if ( jcartorg==2 ) then

!        --- jcartorg = 2  Axi-symmetric co-ordinates

            do i = 1, 4
               wh(m+i) = abs(wh(i))*x(i,1)
            end do
            h = 1d0/36d0*pi2
            w(m+1) = h * (4d0*wh(1)+2d0*(wh(2)+wh(4))+wh(3))
            w(m+2) = h * (4d0*wh(2)+2d0*(wh(3)+wh(1))+wh(4))
            w(m+3) = h * (4d0*wh(3)+2d0*(wh(4)+wh(2))+wh(1))
            w(m+4) = h * (4d0*wh(4)+2d0*(wh(1)+wh(3))+wh(2))

         end if  ! ( intrule==1 )

      end if  ! ( itype>=900 .and. itype<=903 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iqmat, m, itang, inorm', &
                          iqmat, m, itang, inorm
         write(irefwr,1) 'igausp, ndim, intrule', igausp, ndim, intrule
         if ( iqmat>0 ) then
            vec(1:3) = qmat(1,1:3,1)
            call prinrl ( vec, 3, 'normal' )
            vec(1:3) = qmat(3,1:3,1)
            call prinrl ( vec, 3, 't1' )
            vec(1:3) = qmat(2,1:3,1)
            call prinrl ( vec, 3, 't2' )
            call prinrl ( w, m, 'w' )
         end if  ! ( iqmat>0 )
         call prinrl ( w, m, 'w' )
         call prinrl2 ( dphidx(1,1,1), n, m, 'dphidx' )
         call prinrl2 ( dphidx(1,1,2), n, m, 'dphidy' )
         call prinrl2 ( x, n, ndim, 'x' )
         call prinrl2 ( xgauss, m, ndim, 'xgauss' )
         call prinrl2 ( phi, n, m, 'phi' )
         write(irefwr,*) 'End elp621'

      end if  ! ( debug )
      call erclos ( 'elp621' )

      end
