      subroutine elp621det ( isubr, a, wh, index1, x, xg, intrule )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-08-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the Jacobian determinant of the bilinear quadrilateral
!     Store in wh
!     It is assumed that the factors a in the bilinear approximation are
!     computed before
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadrilateral
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduledebug
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, index1(n), intrule
      double precision a(3,2), x(n,2), wh(*), xg(m,2)

!     a              i    Contains the coefficients of the bi-linear polynomial
!                         except the constant part a0
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     intrule        i    Actual integration rule
!     isubr          i    Indicates the type of calling subroutine
!     wh             o    Array of length m containing the values of the
!                         Jacobian in the integration points
!                         Minimal length: 4
!     x              i    contains coordinates of nodes of the element
!     xg             i    Array of length mx2 containing the co-ordinates of the
!                         Gauss points in the reference element for a four point
!                         Gauss rule
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision j0, j1, j2
      logical allneg, allpos

!     allneg         Indication the values of the Jacobian in the vertices
!                    are all negative (true) or not (false)
!     allpos         Indication the values of the Jacobian in the vertices
!                    are all positive (true) or not (false)
!     j0             Constant part of jacobian
!     j1             Linear part of jacobian
!     j2             Linear part of jacobian
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     230   Jacobian of element is zero or has changing sign
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp621det' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp621det'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute the determinant J in the vertices and store in wh
!         a1 b2 - a2 b1 + (a1 b3 - a3 b1) xsi + (a3 b2 - a2 b3) eta

      J0 = a(1,1) * a(2,2) - a(2,1) * a(1,2)
      J1 = a(1,1) * a(3,2) - a(3,1) * a(1,2)
      J2 = a(3,1) * a(2,2) - a(2,1) * a(3,2)

      if ( debug ) write(irefwr,2) 'J', J0, J1, J2

      wh(1) = J0
      wh(2) = J0 + J1
      wh(3) = J0 + J1 + J2
      wh(4) = J0 + J2
      if ( debug ) call prinrl ( wh, 4, 'wh' )

      allneg = wh(1)<0 .and. wh(2)<0 .and. wh(3)<0 .and. wh(4)<0
      allpos = wh(1)>0 .and. wh(2)>0 .and. wh(3)>0 .and. wh(4)>0
      if ( checksign .and. allneg ) then
         write(irefwr,*) 'area of element ', ielem, ' is negative'
      end if

      if( .not. allneg .and. .not. allpos ) then

         if ( isubr==0 ) then
            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 230, 3, 0, 0 )
         else
            call errint ( ielgrp, 1 )
            call errint ( ielem, 2 )
            call errchr ( 'surface', 1 )
            call errsub ( 1540, 2, 0, 1 )
         end if
         call prinin(index1,n,'node numbers')
         call prinrl2(x,n,2,'co-ordinates')

      end if

      if ( intrule>1 ) then

!     --- Gauss rule, recompute wh in integration points

         wh(1:m) = J0+J1*xg(1:m,1)+J2*xg(1:m,2)

      end if  ! ( intrule>1 )

1000  call erclos ( 'elp621det' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp621det'

      end if  ! ( debug )

      end
