      subroutine elp624intpol ( coor, index1, phi, xcoord, ycoord, &
                                xsi0, eta0 )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 19-11-2002
!
!   copyright (c) 2000-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the values of the basis function in a specific point
!    Extended quadratic triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     interpolation
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(2,*), phi(*), xcoord, ycoord, eta0, xsi0
      integer index1(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     eta0           i    Value of eta in previous iteration
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     phi            o    array of length 4 containing the values of the basis
!                         functions in the point xcoord, ycoord
!     xcoord         i    x-coordinate of point
!     xsi0           i    Value of xsi in previous iteration
!     ycoord         i    y-coordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x(7), y(7), determinant, xsi1, dphidxsi(7), &
                       gradfmin1(2,2), distance, fnewton(2), gradf(2,2), &
                       eta1, lambda(3), dphideta(7), &
                       sumx, sumy, phicen, dlamde, dlamdk
      integer i, j, iter

!     determinant    jacobian of element
!     distance       distance between two quantities
!     dlamde         eta derivative of lamc
!     dlamdk         ksi derivative of lamc
!     dphideta       Array containing dphi_i/deta in a point
!     dphidxsi       Array containing dphi_i/dxsi in a point
!     eta1           New value of eta
!     fnewton        Contains the values of the funcion on which the Newton
!                    process is carried out
!     gradf          Gradient of function in Newton process
!     gradfmin1      Inverse of gradf
!     i              Counting variable
!     iter           Iteration sequence number
!     j              Counting variable
!     lambda         Array containing the linear basis functions in a point
!     phicen         Value of alam1*alam2*alam3 in integration point
!     sumx           x-sum
!     sumy           y-sum
!     x              array containing the x-coordinates of the nodes
!     xsi1           New value of xsi
!     y              array containing the y-coordinates of the nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-diemsional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The seven nodes define seven quadratic basis functions phi_i(x,y)
!    In order to compute these basis functions a transformation to
!    a standard triangle is performed (see elp624)
!
!    The quadratic interpolation is defined by:
!
!    x = sum x_i phi_i(xsi,eta)
!    y = sum y_i phi_i(xsi,eta)
!
!    Hence in point (x,y) we have
!         f(1) =  x - sum x_i phi_i(xsi,eta) = 0
!         f(2) =  y - sum y_i phi_i(xsi,eta) = 0
!    This is a non-linear system of equations that can be solved by Newton:
!
!    F' ^(-1) (xsi^k+1 - xsi^k) = -f(1)
!             (eta^k+1 - eta^k) = -f(2)
!
!    With F' = ( df(1)/dxsi, df(2)/dxsi )  =  (a1 + a3 eta, b1 + b3 eta )
!              ( df(1)/deta, df(2)/deta )  =  (a2 + a3 xsi, b2 + b3 xsi )
!
!         F' ^(-1) = 1/det ( df(2)/deta,-df(2)/dxsi )
!                          (-df(1)/deta, df(1)/dxsi )
!
!    The starting value is found by computing the xsi eta for the linear
!    basis functions corresponding to the triangle:
!
!    x = x_1 (1-xsi-eta) + x_2 xsi + x_3 eta
!    y = y_1 (1-xsi-eta) + y_2 xsi + y_3 eta
!
!    Hence:
!
!    x = x_1 + (x_2-x_1) xsi + (x_3-x_1) eta
!    y = y_1 + (y_2-y_1) xsi + (y_3-y_1) eta
!
!    xsi  =  | x_2-x_1 x_3-x_1 |^(-1) |x-x_1|
!    eta     | y_2-y_1 y_3-y_1 |      |y-y_1|
!
!    The basis functions and its derivatives are defined in subroutine elp622
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do j = 1, 7
         x(j) = coor(1,index1(j))
         y(j) = coor(2,index1(j))
      end do

!     --- First estimate for xsi and eta, using the linear interpolation

      gradf(1,1) = x(3)-x(1)
      gradf(1,2) = x(5)-x(1)
      gradf(2,1) = y(3)-y(1)
      gradf(2,2) = y(5)-y(1)
      determinant = (gradf(1,1)*gradf(2,2)- &
                     gradf(1,2)*gradf(2,1))
      determinant = 1d0/determinant
      if ( abs(determinant).le.0d0 ) then
         call eropen ( 'elp624intpol' )
         call errint ( ielem, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 105, 3, 0, 0 )
         call erclos ( 'elp624intpol' )
         call prinin(index1,7,'node numbers')
         call prinrl2(x,7,2,'co-ordinates')
         go to 1000
      end if
      gradfmin1(1,1) = determinant*gradf(2,2)
      gradfmin1(1,2) =-determinant*gradf(1,2)
      gradfmin1(2,1) =-determinant*gradf(2,1)
      gradfmin1(2,2) = determinant*gradf(1,1)
      xsi0 = gradfmin1(1,1)*(xcoord-x(1))+gradfmin1(1,2)*(ycoord-y(1))
      eta0 = gradfmin1(2,1)*(xcoord-x(1))+gradfmin1(2,2)*(ycoord-y(1))

!     --- Perform Newton iteration

      distance = 1d0
      iter = 0
      do while ( distance.gt.1d-6 .and. iter.le.5 )

!     --- Stop if the distance is less than 1d-6 or number of
!         iterations is 5

         iter = iter+1
         lambda(1) = 1d0 - xsi0-eta0
         lambda(2) = xsi0
         lambda(3) = eta0
         phicen = lambda(1)*lambda(2)*lambda(3)
         phi(7) = 27d0 * phicen
         do i = 1, 3
            phi(2*i-1) = lambda(i) * (2d0*lambda(i)-1d0)+ &
                         3d0*phicen
         end do
         phi(2) = 4d0 * lambda(1) * lambda(2) - 12d0*phicen
         phi(4) = 4d0 * lambda(2) * lambda(3) - 12d0*phicen
         phi(6) = 4d0 * lambda(3) * lambda(1) - 12d0*phicen
         dlamdk = lambda(3) * (lambda(1)-lambda(2))
         dphidxsi(1) = -4d0*lambda(1)+1d0+3d0*dlamdk
         dphidxsi(3) =  4d0*lambda(2)-1d0+3d0*dlamdk
         dphidxsi(5) =  3d0*dlamdk
         dphidxsi(2) =  4d0*(lambda(1)-lambda(2))-12d0*dlamdk
         dphidxsi(4) =  4d0* lambda(3)-12d0*dlamdk
         dphidxsi(6) =  -4d0* lambda(3)-12d0*dlamdk
         dphidxsi(7) =  27d0*dlamdk
         dlamde = lambda(2) * (lambda(1)-lambda(3))
         dphideta(1) =  -4d0*lambda(1)+1d0+3d0*dlamde
         dphideta(3) =  3d0*dlamde
         dphideta(5) =  4d0*lambda(3)-1d0+3d0*dlamde
         dphideta(4) =  4d0* lambda(2)-12d0*dlamde
         dphideta(2) =  -4d0* lambda(2)-12d0*dlamde
         dphideta(6) =  4d0*(lambda(1)-lambda(3))-12d0*dlamde
         dphideta(7) =  27d0*dlamde
         sumx = ddot ( 7, phi, 1, x, 1 )
         sumy = ddot ( 7, phi, 1, y, 1 )
         fnewton(1) = xcoord-sumx
         fnewton(2) = ycoord-sumy
         gradf(1,1) = -ddot ( 7, dphidxsi, 1, x, 1 )
         gradf(1,2) = -ddot ( 7, dphideta, 1, x, 1 )
         gradf(2,1) = -ddot ( 7, dphidxsi, 1, y, 1 )
         gradf(2,2) = -ddot ( 7, dphideta, 1, y, 1 )
         determinant = (gradf(1,1)*gradf(2,2)- &
                        gradf(1,2)*gradf(2,1))
         determinant = 1d0/determinant
         if ( abs(determinant).le.0d0 ) then
            call eropen ( 'elp624intpol' )
            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 105, 3, 0, 0 )
            call prinin(index1,7,'node numbers')
            call prinrl2(x,7,2,'co-ordinates')
            call erclos ( 'elp624intpol' )
            go to 1000
         end if
         gradfmin1(1,1) = determinant*gradf(2,2)
         gradfmin1(1,2) =-determinant*gradf(1,2)
         gradfmin1(2,1) =-determinant*gradf(2,1)
         gradfmin1(2,2) = determinant*gradf(1,1)
         xsi1 = xsi0 - gradfmin1(1,1)*fnewton(1) - &
                       gradfmin1(1,2)*fnewton(2)
         eta1 = eta0 - gradfmin1(2,1)*fnewton(1) - &
                       gradfmin1(2,2)*fnewton(2)
         distance = (xsi1-xsi0)**2 + (eta1-eta0)**2
         xsi0 = xsi1
         eta0 = eta1

      end do

1000  end
