      subroutine elp631straight ( x, intern, imidp, index1, iweigo, w, &
                                  wh, comppsi, dpsidx, dphidx, alambd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-06-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute derivatives of basis functions for quadratic tetrahedron
!     Straight element
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     3d
!     quadratic
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intern(10), imidp(6,2), index1(n), iweigo
      double precision x(n,3), dpsidx(4,3), dphidx(n,m,3), alambd(4,m), &
                       wh(m)
      logical comppsi

!     alambd         i    Contains values of the linar basis functions
!                         in the integration points
!     comppsi        i    if true the linear basis functions must be evaluated
!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                              d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dpsidx         o    Array of length n x m x l  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                              i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                              i        k
!                         d psi / dz (xg ) = psix (i,k,3);
!                              i        k
!     imidp          i    Contains the i,j numbering of the mid side points in
!                         the sequence of intern
!                         So basis function
!                         phi(intern(4+i)) = 4 lambda(imidp(i,1))
!                                              lambda(imidp(i,2))
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     intern         i    Internal numbering of the nodes, vertices first
!     iweigo         i    if 1 the weights must be filled
!     w              o    array of length m containing the weights for
!                         integration
!     wh             i    Array of length 4 containing the values of the
!                         Jacobian in the vertices or for the Gauss integration
!                         rules work array to store some help variables
!     x              i    x co-ordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, i1, i2, index1cp(4)
      double precision e(4,3), xvert(4,3), djacob, w(m)

!     djacob         jacobian of element
!     e              Work array of length 4 x 3, to store the factors
!                     ij
!                    e.        See method
!     i              Counting variable
!     i1             i-index of mid side point ij
!     i2             j-index of mid side point ij
!     index1cp       Copy of index1 with respect to the vertices only
!     j              Counting variable
!     xvert          Contains the coordinates of the vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP630JACOB    Compute the Jacobian of a tetrahedron
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp631straight' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp631straight'
         call prinrl2 ( x, n, 3, 'x' )
         call prinin ( intern, 10, 'intern' )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy coordinates of vertices into xvert

      xvert(1:4,:) = x(intern(1:4),:)
      index1cp(1:4) = index1(intern(1:4))
      call elp630jacob ( e, xvert, djacob, 0, index1cp, &
                         ielem, ielgrp, itype, ideriv, nvert )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,2) 'djacob', djacob

      if ( iweigo==1 ) then

!     --- iweigo = 1  Fill weights

         w(1:m) = abs(djacob)*wh(1:m)

      end if

      if ( comppsi ) then

!     --- Compute linear basis functions

         dpsidx(1:4,1:3) = e(1:4,1:3)

      end if  ! ( comppsi )

      if ( ideriv==1 ) then

!     --- Compute derivatives

         do j = 1, 3
            do i = 1, 4
               dphidx(intern(i),1:m,j) = e(i,j)*(4d0*alambd(i,1:m)-1d0)
            end do
         end do

         do j = 1, 3
            do i = 1, 6
               i1 = imidp(i,1)
               i2 = imidp(i,2)
               dphidx(intern(i+4),1:m,j) = 4d0*(e(i1,j)*alambd(i2,1:m)+ &
                                                e(i2,j)*alambd(i1,1:m))
            end do
         end do

      end if  ! ( ideriv==1 )

1000  call erclos ( 'elp631straight' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( w, m, 'w' )
         write(irefwr,*) 'End elp631straight'

      end if  ! ( debug )

      end
