      subroutine elpupwind3d ( alpha, u, phi, psi, dphidx, second, &
                               dcdx, jelp, x, dlamdx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 20-06-2013 New call to elp631
!        version  2.0  date 04-07-2003 Extra parameter dlamdx
!        version  1.0  date 15-01-2003
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute modified basis functions for upwind elements
!    Three dimensional elements
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwinding
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision alpha(m,6), u(m,3), phi(n,m), psi(n,m), &
                       dphidx(n,m,3), dcdx(m,ndimlc), x(*), dlamdx(*)
      integer jelp
      logical second

!     alpha          i    Array of length m x 3 containing the factors alpha_ij
!                         in the integration points
!     dcdx           i    Contains the values of dc/dx in the integration points
!                         This is used in case of non-linear upwinding
!     dlamdx         o    Contains derivatives of linear basis functions
!     dphidx         i    Array of length n x m x 3 containing the values of
!                         the derivatives of the basis functions in the
!                         integration points
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELelpupwind3dx. Possible values:
!     phi            i    Array of length n x m containing the values of the
!                         basis functions in the integration points
!     psi            o    Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     second         o    Indicates if the second order term is significant or
!                         may be neglected in the upwinding part
!     u              i    Array of length m x 3 containing the velocity in the
!                         integration points
!     x              i    co-ordinates of nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, metupl
      double precision xi, beta, cgnorm, unorm(m), eps(m), &
                       h(m), tau(m), u1(m,3), tau1(m)

!     beta           Parameter beta in method
!     cgnorm         Norm of dphidx in integration point for specific value of i
!     eps            Contains the diffusion
!     h              Distances in direction of velocity in integration points
!     i              Counting variable
!     k              Counting variable
!     metupl         mod(metupw,100): defines type of upwinding
!     tau            Contains parameter tau in integration points
!     tau1           Contains parameter tau parallel in integration points
!     u1             Parallel velocity
!     unorm          length of velocity vector in integration points
!     xi             Factor xi, see method
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELPUPWDIST     Compute distance h for modified basis functions for upwind
!                    elements. Three dimensional elements
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!                                                       new
!       The basis functions are modified such that:  phi    = phi + p
!                                                       i        i   i
!       or:
!          new  k          k        k
!       phi   (x ) = phi (x ) + p (x )
!          i            i        i
!       The basis funtions phi are assumed to be filled before
!       p_i = tau u . grad phi_i
!
!                   h xi
!       with tau = ------ beta = ||u|| h / 2 eps
!                   2||u||
!                      - &
!       xi depends on metupw according to:
!       metupw = 1:  Classical upwind scheme  xi = 1
!                2:  Il'in    xi = coth(beta) - 1/beta
!                                             beta/3     -3<beta<3
!                3:  Doubly assymptotic: xi = &
!                                             sign(beta)  |beta|>3
!                                         -1-1/beta     beta<1
!                4:  critical approx: xi = 0            -1<beta<1
!                                          1-1/beta     beta>1
!       eps is either alpha (if a11 = a22) or alpha in the direction of
!       the velocity
!              T
!       eps = u  alpha  u        (tensor alpha)
!             -         - &
!       h = max distance in element in the direction of u
!                                                       - &
!       For a tetrahedron:  h  = ||u || / max (|u . grad lambda|) in the vert.
!                            i      i      j     i             j
!       First tau and h are computed in the vertices and afterwards tau is
!       interpolated to the integration points
!    In case of the discontinuity capturing methods we define u   as
!                                                              ||
!                                            2
!    u   = u . grad(c) grad(c) / || grad(c)||
!     ||
!    In that case the upwind basis functions become
!       p  = ( tau u  + tau   u  ) . grad phi
!        i        1-       2  -||            i
!    with tau  the tau as described above and tau  is tau   (DC1) or
!            1                                   2       ||
!    max(0,tau  -tau) (DC2)      tau   is the tau based upon u
!             ||                    ||                        ||
!    In this case we limit ourselves to the doubly assymptotic approximation
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elpupwind3d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elpupwind3d'
         write(irefwr,1) 'jsecnd', jsecnd
         call prinrl1 ( dphidx(1,1,1), m, n, 'dphidx' )
         call prinrl1 ( dphidx(1,1,2), m, n, 'dphidy' )
         call prinrl1 ( dphidx(1,1,3), m, n, 'dphidz' )
         call prinrl1 ( u, m, 3, 'u' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill norm of velocity in vertices into array unorm

      metupl = mod ( metupw, 100 )
      do i = 1, m
         unorm(i) = sqrt ( u(i,1)**2 + u(i,2)**2 + u(i,3)**2 )
      end do

!     --- Store component of alpha tensor in u-direction into eps

      if ( jsecnd==0 ) then

!     --- jsecnd = 0,   eps = 0

         do i = 1, m
            eps(i) = 0d0
         end do

      else if ( jsecnd==1 ) then

!     --- jsecnd = 1,   eps = alpha = a22 = a33  (constant)  a12=a13=a23 = 0

         do i = 1, m
            eps(i) = alpha(1,1)
         end do

      else if ( jsecnd==2 ) then

!     --- jsecnd = 2,   eps = alpha = a22 =a33   a12=a13=a23 = 0

         do i = 1, m
            eps(i) = alpha(i,1)
         end do

      else

!     --- jsecnd>2,   General case
!                      eps is defined by the formula  eps = u^T alpha u

         do i = 1, m
            if ( unorm(i)>0d0 ) then
               eps(i) = (     u(i,1)**2*alpha(i,1)+ &
                          2d0*u(i,1)*alpha(i,2)*u(i,2)+ &
                          2d0*u(i,1)*alpha(i,3)*u(i,3)+ &
                              u(i,2)**2*alpha(i,4)+ &
                          2d0*u(i,2)*alpha(i,5)*u(i,3)+ &
                              u(i,3)**2*alpha(i,6) ) / unorm(i)
            else
               eps(i) = alpha(i,1)
            end if
         end do

      end if

!     --- Compute the distance h in the vertices

      call elpupwdist ( u, dphidx, h, unorm, jelp, x, nvert, dlamdx )

      if ( debug ) then
         call prinrl ( h, m, 'h' )
         call prinrl ( eps, m, 'eps' )
      end if  ! ( debug )

!     --- Compute tau in the vertices

      do i = 1, m
         if ( eps(i)>1d-10 .and. metupl<=5 ) then

!        --- eps(i)>1d-10,  hence second order term plays a role

            beta = unorm(i)*h(i)/(2d0*eps(i))
            if ( beta<1d-10 ) then
               xi = 0d0
            else if ( metupl==1 ) then

!           --- metupl = 1   classical upwind scheme

               xi = 1d0
            else if ( metupl==2 ) then

!           --- metupl = 2   il'in scheme

               xi = 1d0/tanh(beta)-1d0/beta
            else if ( metupl==3 .or. metupl==7 .or. metupl==8 ) then

!           --- metupl = 3, 7 or 8    doubly assymptotic approximation

               if ( abs(beta)>3d0 ) then
                  xi=sign(1d0,beta)
               else
                  xi = beta / 3d0
               end if
            else if ( metupl==4 ) then

!           --- metupl = 4    critical approximation

               if ( beta<-1d0 ) then
                  xi = -1d0 - 1d0 / beta
               else if ( beta>1d0 ) then
                  xi = 1d0 - 1d0 / beta
               else
                  xi = 0d0
               end if
            else if ( metupl==5 ) then

!           --- metupl = 5

               if ( unorm(i)<=1d-5*eps(i) ) then
                  xi = 0d0
               else
                  beta = beta*beta
                  xi = sqrt(beta/(9d0+beta))
               end if
            end if

         else if ( metupl==6 ) then

!        --- metupl = 6

            if ( h(i)>epsmac ) then
               beta = (2d0*unorm(i)/h(i))**2 + &
                      (4d0*eps(i)/h(i)**2)**2
            else
               beta = 0d0
            end if
            if ( tstep>0d0 ) beta = beta + 4d0/(tstep*tstep)
            tau(i) = 1d0/sqrt(beta)

         else
            xi = 1d0

         end if

         if ( metupl<=8 .and. metupl/=6 ) then
            if ( unorm(i)>epsmac ) then
               tau(i) = h(i)*xi/(2d0*unorm(i))
            else
               tau(i) = 0d0
            end if
         end if

      end do

!     --- compute new basis functions, it has been assumed that the old
!         basis functions are already stored in array phi

      if ( jdercn==0 ) then

!     --- jdercn = 0,  constant derivatives

         do k = 1, m
            do i = 1, n
               psi(i,k) = tau(k) * ( u(k,1)*dphidx(i,1,1)+ &
                          u(k,2)*dphidx(i,1,2)+u(k,3)*dphidx(i,1,3)) + &
                          phi(i,k)
            end do
         end do

      else

!     --- jdercn = 1,  variable derivatives

         do k = 1,m
            do i = 1, n
               psi(i,k) = tau(k) * ( u(k,1)*dphidx(i,k,1)+ &
                          u(k,2)*dphidx(i,k,2)+u(k,3)*dphidx(i,k,3) ) + &
                          phi(i,k)
            end do
         end do
      end if

      do k = 1, m
         if ( abs(eps(k))*1d2>unorm(k) ) second = .true.
      end do

      if ( metupl==7 .or. metupl==8 ) then

!     --- Discontinuity capturing
!         First compute u
!                        ||

         do k = 1, m
            cgnorm = dcdx(k,1)**2 + dcdx(k,2)**2 + dcdx(k,3)**2
            if ( cgnorm<epsmac ) then
               u1(k,1) = 0d0
               u1(k,2) = 0d0
               u1(k,3) = 0d0
            else
               xi = ( u(k,1)*dcdx(k,1) + u(k,2)*dcdx(k,2) + &
                      u(k,3)*dcdx(k,3) )/cgnorm
               u1(k,1) = xi*dcdx(k,1)
               u1(k,2) = xi*dcdx(k,2)
               u1(k,3) = xi*dcdx(k,3)
            end if
            unorm(k) = sqrt ( u1(k,1)**2 + u1(k,2)**2 + u1(k,3)**2 )

         end do

!        --- Store component of alpha tensor in u1-direction into eps

         if ( jsecnd>2 ) then

!        --- jsecnd>2,  General case
!                         eps is defined by the formula  eps = u^T alpha u

            do i = 1, m
               if ( unorm(i)>0d0 ) then
                  eps(i) = (     u1(i,1)**2*alpha(i,1)+ &
                             2d0*u1(i,1)*alpha(i,2)*u1(i,2)+ &
                             2d0*u1(i,1)*alpha(i,3)*u1(i,3)+ &
                                 u1(i,2)**2*alpha(i,4)+ &
                             2d0*u1(i,2)*alpha(i,5)*u1(i,3)+ &
                                 u1(i,3)**2*alpha(i,6) ) / unorm(i)
               else
                  eps(i) = alpha(i,1)
               end if
            end do

         end if

!        --- Compute the distance h in the vertices

         call elpupwdist ( u1, dphidx, h, unorm, jelp, x, nvert, &
                           dlamdx )

!        --- Compute tau in the vertices

         do i = 1, m
            if ( eps(i)>1d-10 .and. metupl<=5 ) then

!           --- eps(i)>1d-10,  hence second order term plays a role

               beta = unorm(i)*h(i)/(2d0*eps(i))
               if ( beta<1d-10 ) then
                  xi = 0d0
               else

!              --- metupl = 3, 7 or 8    doubly assymptotic approximation

                  if ( abs(beta)>3d0 ) then
                     xi=sign(1d0,beta)
                  else
                     xi = beta / 3d0
                  end if
               end if
            else
               xi = 1d0
            end if
            if ( unorm(i)>epsmac ) then
               tau1(i) = h(i)*xi/(2d0*unorm(i))
            else
               tau1(i) = 0d0
            end if
         end do
         if ( metupl==8 ) then
            do k = 1, m
               tau1(k) = max(0d0,tau1(k)-tau(k))
            end do
         end if
         do i = 1, n
            do k = 1, m
               psi(i,k) = tau1(k) * ( u1(k,1)*dphidx(i,k,1)+ &
                                      u1(k,2)*dphidx(i,k,2)+ &
                                      u1(k,3)*dphidx(i,k,3) ) + psi(i,k)
            end do
         end do
      end if

1000  call erclos ( 'elpupwind3d' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( psi, m, n, 'psi' )
         write(irefwr,*) 'End elpupwind3d'

      end if  ! ( debug )

      end
