      subroutine elup04 ( alpha, dphidx, eps, user, index1, &
                          wrk1, coor2d, jelp, vecloc, &
                          iuser, a1, a2, a3, phiksi, phi, x, xgauss, &
                          w, ifirstdif )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.4  date 27-05-2014 Extension for irule = 22
!        version  2.3  date 20-06-2013 New call to elp631
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute help arrays for second derivative part in case of upwinding
!     Trilinear and tri-quadratic three-dimensional elements
! **********************************************************************
!
!                       KEYWORDS
!
!     elliptic_equation
!     upwinding
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/celwrkf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*), iuser(*), jelp, ifirstdif
      double precision alpha(m,6), dphidx(n,n,3), phi(*), &
                       eps(n,6), user(*), wrk1(*), &
                       coor2d(*), vecloc(*), phiksi(*), x(n,3), &
                       xgauss(m,3), w(m)
      double precision a1(27,27), a2(27,27), a3(27,27)

!     a1             o    Work array to store parts of the computation of the
!                         second derivative term
!                                    l             l     l                  l
!                         a1(l)= a (x )d phi /dx (x ) + a  (x ) d phi /dy (x )
!                                 11        j      12                j
!                                       l              l
!                               +  a  (x ) d phi /dz (x )
!                                   13          j
!                         The two-dimensional array is stored as follows:
!                         a1(l,j) = a1(l)   j = 1 (1) n
!     a2             o    See a1, however now for the second term
!     a3             o    See a1, however now for the third term
!     alpha          i    Array of length m x 6 containing the factors alpha
!                                                                           ij
!                         in the integration points
!     coor2d           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dphidx         i    Array of length n x m x 2 containing the values of the
!                         derivatives of the basis functions in the integration
!                         node
!     eps                 Work array in which the values of alpha ij in the
!                         nodal points are stored
!     ifirstdif      i    Local parameter ifirst with same meaning as ifirst
!                         with respect to elm800dif
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           i    Choice parameter to distinguish between the types of
!                         element subroutines to be called. Possible values:
!                         4:  Subroutine ELP622  (Quadratic triangle)
!                         5:  Subroutine ELP621  (Bilinear quadrilateral)
!                         6:  Subroutine ELP623  (Biquadratic quadrilateral)
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phiksi        i/o   work array for elp633
!                         This work array must be saved and is reused by elp633
!                         To avoid conflicts this array must be different from
!                         the standard phiksi array
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     vecloc              Work array in which all old solution vectors for the
!                         integration points are stored
!     wrk1                Real work array of length n
!     x              i    array of length n x ndim containing the coordinates of
!                         the nodes
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, ihcoor2d, ihweig, ihrule, ihiphi, ihgaus, ih, msave, &
              ifirstsave
      double precision wh(27), dpsidx(4,3), ugauss(27)

!     dpsidx         Array of length n x m x l  containing the derivatives
!                    of the linear basis functions in the sequence:
!                    d psi / dx (xg ) = psix (i,k,1);
!                         i        k
!                    d psi / dy (xg ) = psix (i,k,2);
!                         i        k
!                    d psi / dz (xg ) = psix (i,k,3);
!                         i        k
!     i              Counting variable
!     ifirstsave     Help variable to store the value of ifirst temporarily
!     ih             Help variable to store a constant temporarily
!     ihcoor2d         Help variable to store the value of icoor temporarily
!     ihgaus         Help variable to store the value of igauss temporarily
!     ihiphi         Help variable to store the value of iphi temporarily
!     ihrule         Help variable to store the value of irule temporarily
!     ihweig         Help variable to store the value of iweigh temporarily
!     k              Counting variable
!     msave          Help variable to store the value of m temporarily
!     ugauss         Preceding solution in Gauss points
!     w              array of length m containing the weights for
!                    integration
!                    rules work array to store some help variables
!     wh             Help array to store the weights temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELP631         Compute basis functions and weights for quadratic
!                    tetrahedron
!     ELP632         Compute basis functions and weights for trilinear brick
!     ELP633         Compute basis functions and weights for triquadratic brick
!     ELP638         Compute basis functions and weights for trilinear prism
!     ELP639         Compute basis functions and weights for triquadratic prism
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Compute the coefficients of the second derivative term (eps) in the
!    nodal points
!    Compute dphidx in the nodal points
!    Compute a1, a2 and a3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elup04' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elup04'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute eps in the nodal points

      ih = jdiag
      jdiag = 1
      ifirstsave = ifirst
      do i = 1, 6

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, user, eps, x, xgauss, &
                          m, n, phi, 1, index1, &
                          iuser, ugauss, vecloc )

         else

!        --- Coefficient is constant

            eps(1:n,i) = alpha(1,i)

         end if

      end do

!     --- Compute dphidx in the basis points

      ihcoor2d = icoor
      ihiphi = iphi
      ihrule = irule
      ihweig = iweigh
      ihgaus = igausp
      icoor  = 0
      iphi   = 0
      irule  = 1
      iweigh = 0
      igausp = 0
      msave  = m
      m      = n
      if ( jelp==12 ) then

!     --- jelp = 12, quadratic tetrahedron

         call elp631 ( coor2d, x, w, dphidx, xgauss, phi, index1, work, &
                       dpsidx, phiksi )

      else if ( jelp==13 ) then

!     --- jelp = 13, trilinear hexahedron

         call elp632 ( coor2d, x, w, dphidx, xgauss, phi, index1 )

      else if ( jelp==21 ) then

!     --- jelp = 21, trilinear prism

         ifirst = ifirstdif
         call elp638 ( coor2d, x, wh, dphidx, xgauss, phi, index1, &
                       phiksi )

      else if ( jelp==14 ) then

!     --- jelp = 14, triquadratic hexahedron
!         To avoid conflicts with the first call of elp633, ifirst is changed

         ifirst = ifirstdif
         call elp633 ( coor2d, x, wh, dphidx, xgauss, phi, index1, &
                       phiksi, work, work, work )

      else if ( jelp==22 ) then

!     --- jelp = 22, triquadratic prism

         ifirst = ifirstdif
         call elp639 ( coor2d, x, wh, dphidx, xgauss, phi, index1, &
                       phiksi )

      else

!     --- Other shape numbers have not yet been implemented

         call errint ( jelp, 1 )
         call errsub ( 800, 1, 0, 0 )
         go to 1000

      end if
      icoor  = ihcoor2d
      iphi   = ihiphi
      irule  = ihrule
      iweigh = ihweig
      igausp = ihgaus
      m      = msave
      jdiag  = ih

!     --- Fill arrays a1 and a2

      do i = 1, n
         do k = 1, n
            a1(k,i) = eps(k,1)*dphidx(i,k,1)+eps(k,2)*dphidx(i,k,2)+ &
                      eps(k,3)*dphidx(i,k,3)
            a2(k,i) = eps(k,2)*dphidx(i,k,1)+eps(k,4)*dphidx(i,k,2)+ &
                      eps(k,5)*dphidx(i,k,3)
            a3(k,i) = eps(k,3)*dphidx(i,k,1)+eps(k,5)*dphidx(i,k,2)+ &
                      eps(k,6)*dphidx(i,k,3)
         end do
      end do
      ifirst = ifirstsave
      ifirstdif = 1

1000  call erclos ( 'elup04' )

      if ( debug ) then

!     --- Debug information

         call prinrl1 ( a1, n, n, 'a1' )
         write(irefwr,*) 'End elup04'

      end if

      end
