      subroutine sepelbihar ( coeff )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the element matrix for the biharmonic equation
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     beam
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: coeff(B%m)

!     coeff          i    coefficient in biharmonic equation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      double precision :: work(B%m), h
      double precision, pointer :: phixx(:,:), phix(:,:), xg(:)

!     h              Help variable to store a constant temporarily
!     i              General loop variable
!     j              General loop variable
!     k              General loop variable (quadrature points)
!     phix           First derivative of basis function phi
!     phixx          second derivative of basis function phi
!     work           work array to store help arrays
!     xg             x-coordinates of Gauss points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The part of the element matrix is given by
!
!     / coeff d^2 phi_i dx^2  d^2 phi_j dx^2  d Omega
!
!     The numerical values are given by
!
!     sum_k w(k) {coeff d^2 phi_i dx^2  d^2 phi_j dx^2 }(x_k)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelbihar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelbihar'
         write(irefwr,1) 'cart', B%cart
         call prinrl ( B%w, B%m, 'w' )
         call prinrl ( coeff, B%m, 'coeff' )
  1      format ( a, 1x, (10i6) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phixx => B%phider2(:,:,1)
      phix => B%phider(:,:,1)
      xg => B%xyzg(:,1)

!     --- Store w coeff in work

      work = coeff*B%w

      if ( debug ) call prinrl ( work, B%m, 'work' )

      select case ( B%nd )

      case (1)  ! ndim = 1

         if ( B%cart==1 ) then

!        --- Cartesian coordinates

            do i = 1, B%n
               do j = 1, i
                  h = sum ( work(:)*phixx(i,:)*phixx(j,:) )
                  E%mat(i,j) = E%mat(i,j)+h
                  if ( j/=i ) E%mat(j,i) = E%mat(j,i)+h
               end do  ! j = 1, i
            end do  ! i = 1, B%n

         else

!        --- Axi-symmetric coordinates

            do i = 1, B%n

               do j = 1, i

                  h = 0d0

                  do k = 1, B%m

                    if ( abs(work(k))>0d0 ) then

!                   --- Not r = 0

                       h = h + work(k)* &
                       ( phix(i,k)+xg(k)*phixx(i,k) )/xg(k)* &
                       ( phix(j,k)+xg(k)*phixx(j,k) )/xg(k)

                    end if  ! ( abs(work(i))>0d0 )
                  end do  ! k = 1, B%m

                  E%mat(i,j) = E%mat(i,j)+h
                  if ( j/=i ) E%mat(j,i) = E%mat(j,i)+h

               end do  ! j = 1, i

            end do  ! i = 1, B%n

         end if  ! ( B%cart==1 )

      case default

!     --- Value of ndim not implemented

         call errint ( B%nd, 1 )
         call errint ( 1, 2 )
         call errint ( 1, 3 )
         call errsub ( 419, 3, 0, 0 )
         go to 1000

      end select  ! case ( B%nd )

1000  call erclos ( 'sepelbihar' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%mat, B%n, B%n, 'element matrix' )
         write(irefwr,*) 'End sepelbihar'

      end if  ! ( debug )

      end subroutine sepelbihar
