      subroutine sepelconvectfu
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for convection equation
!     The convection term is given by a non-linear function of grad c
!     No upwind
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: jdiag, ind(3)
      double precision, target :: veloc(B%m,3)
      double precision :: f(B%m)

!     ind            indicates if the "new" velocity components are
!                    constant of variable
!     jdiag          if 1 the basis functions are 0 or 1 in the quadrature pnts
!                    if  0 they are general
!     f              right-hand side term due to convection
!     veloc          Contains the new velocity components for this
!                    term. They are -1 or 1 depending on the sign of the
!                    derivatives
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     SEPELCONVECT0  Fill element matrix for convection equation
!     SEPELCONVECT7  Fill arrays veloc and ind in case of non-linear convection
!     SEPELLOAD      Fill the element vector with a load
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The convection term is given by g(grad c)
!
!     using newton linearization we get
!
!     g^n+1 = g^n + grad ( c^n+1 - c^n ) dg/d grad c
!
!     Hence the right-hand side gets an extra term -g^n + c^n dg/d grad c
!
!     The velocity vector is dg/d grad c
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelconvectfu' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelconvectfu'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill arrays veloc and ind

      call sepelconvect7 ( veloc, ind, f )

      if ( E%methodupw>0 ) then

!     --- upwind compute psi

         E%veloc => veloc
         call sepelbaseupw

      end if  ! ( E%methodupw>0 )

      if ( debug ) then
         call prinrl1 ( veloc, B%nd, B%m, 'uvel' )
         call prinrl1 ( B%psi, B%m, B%n, 'psi' )
      end if  ! ( debug )

!     --- Update matrix with convection part defined by veloc

      call sepelconvect0 ( veloc, ind )

!     --- Update right-hand side with f

      call sepelload ( f )

1000  call erclos ( 'sepelconvectfu' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelconvectfu'

      end if  ! ( debug )

      end subroutine sepelconvectfu
