      subroutine sepeld22
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector for flow computation of Reynolds equation
!     Both compressible and incompressible bearing
!     One and two dimensional elements
!     Old subroutine el1012
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     compressible
!     incompressible
!     element_vector
!     lubrication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, l
      integer, pointer :: ind(:)
      double precision, pointer :: h(:), mu(:), diffusion(:), gradp(:,:)
      double precision, pointer :: rho(:), f(:,:), coeff(:), dphidx(:,:,:)
      double precision, pointer :: ht(:), k(:), p0(:), RT(:), p(:), &
                                   u1(:), u2(:), w(:)

!     RT             Contains the constant RT from the ideal gas law
!     coeff          work array to store a combination of coefficients
!                    they are not
!     diffusion      Contains h^3/(12 mu)
!     dphidx         derivatives of the basis functions
!     f              source vector
!     gradp          gradient of the prior pressure
!     h              layer thickness
!     ht             squeeze ht
!     i              Counting variable
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3,4
!     k              Constant k
!     l              General loop variable
!     mu             viscosity
!     p              prior pressure
!     p0             p0 in equation
!     rho            density rho
!     u1             velocity of lower surface
!     u2             velocity of upper surface
!     w              the weights for the quadrature
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELLOAD      Fill the element vector with a load
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The flow is given by:
!    Incompressible bearing:
!    / (h^3/(12 mu) (grad p - rho f) + h/2 u ) grad phi  dx  + &
!                                 -        -           i
!    / (ht + k(p-p0) ) phi  dx
!                         i
!    Compressible bearing (adiabatic)
!    / (p h^3/(12 mu) (grad p - p/RT f) + ph/2 u) grad phi  dx + &
!                                    -         -          i
!                2   2
!    / (pht + k(p -p0 ) ) phi  dx
!                            i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld22' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld22'
         write(irefwr,1) 'typebear', E%typebear
         write(irefwr,1) 'n, m, nd, ndim', B%n, B%m, B%nd, E%ndim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      h => E%coeffs(:,1)
      mu => E%coeffs(:,2)
      ht => E%coeffs(:,3)
      k => E%coeffs(:,4)
      p0 => E%coeffs(:,5)
      rho => E%coeffs(:,9)
      RT => E%coeffs(:,10)
      f => E%coeffs(:,11:12)
      w => B%w
      ind => E%ind
      dphidx => B%phider(:,:,:)

      p => uprev(:,1)
      gradp => gradoldvec(:,:,1)
      diffusion => E%coeffs(:,18)
      coeff => E%coeffs(:,17)

      diffusion = h**3/(12d0*mu)

      E%vec = 0d0

      if ( E%typebear==0 ) then

!     --- incompressible bearing
!         part / (h^3/(12 mu) (grad p - rho f) - h/2 u ) grad phi_i dx

         do l = 1, min(2,B%nd)
            u1 => E%coeffs(:,5+l)
            u2 => E%coeffs(:,14+l)
            coeff = diffusion*gradp(:,l) - 0.5d0*h*(u1+u2)
            if ( ind(9)/=0 .and. ind(10+l)/=0 ) &
               coeff = coeff - diffusion*rho*f(:,l)
            coeff = coeff*w
            do i = 1, B%n
               E%vec(i) = E%vec(i)+sum(coeff*dphidx(i,:,l))
            end do  ! i = 1, B%n
         end do  ! l = 1, min(2,B%nd)

         if ( ind(3)/=0 .or. ind(4)/=0 ) then

!        --- part / (ht + k(p-p0) ) phi_i dx

            coeff = 0d0
            if ( ind(3)/=0 ) coeff = ht
            if ( ind(4)/=0 ) coeff = coeff + k*(p-p0)
            call sepelload ( coeff )

         end if

      else

!     --- compressible bearing
!         part / (p h^3/(12 mu) (grad p - p/RT f) - ph/2 u) grad phi_i dx

         diffusion = p*diffusion
         do l = 1, min(2,B%nd)
            u1 => E%coeffs(:,5+l)
            u2 => E%coeffs(:,14+l)
            coeff = diffusion*gradp(:,l) - 0.5d0*h*p*(u1+u2)
            if ( ind(9)/=0 .and. ind(10+l)/=0 ) &
               coeff = coeff - diffusion*p/RT*f(:,l)
            coeff = coeff*w
            do i = 1, B%n
               E%vec(i) = E%vec(i)+sum(coeff*dphidx(i,:,l))
            end do  ! i = 1, B%n
         end do  ! l = 1, min(2,B%nd)

         if ( ind(3)/=0 .or. ind(4)/=0 ) then

!        --- part / (pht + k(p^2-p0^2) ) phi_i dx

            coeff = 0d0
            if ( ind(3)/=0 ) coeff = ht
            if ( ind(4)/=0 ) coeff = coeff + k*(p**2-p0**2)

            call sepelload ( coeff )

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( E%vec, B%n, 'elemvc' )
         write(irefwr,*) 'End sepeld22'

      end if  ! ( debug )
      call erclos ( 'sepeld22' )

      end subroutine sepeld22
