      subroutine sepeldc062 ( elemvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives and weights in elemvc and elemwg
!     with respect to subroutine sepeld705
!     icheld = 6, 16, 26  Compute - A gradient
!     ndim = 2
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex :: elemvc(ndimlc,B%n)

!     elemvc         o    Element vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double complex, pointer :: dudx(:,:), a11(:), a22(:), a12(:)
      double complex :: som

!     a11            Diffusion coefficient a_11
!     a12            Diffusion coefficient a_12
!     a22            Diffusion coefficient a_22
!                    they are not
!     dudx           Reference to array gradoldvec
!                    (derivatives of solution in nodes)
!     i              Counting variable
!     som            Help parameter to store a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The formula to be computed is:
!     g1 = - a11 ux - a12 uy
!     g2 = - a21 ux - a22 uy
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldc062' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldc062'
         write(irefwr,1) 'typediff', E%typediff
         write(irefwr,1) 'm, rule', B%m, B%rule
         write(irefwr,*) 'derivconst ', B%derivconst
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dudx => gradoldcvec(:,:,1)
      a11 => E%ccoeffs(:,1)
      a12 => E%ccoeffs(:,2)
      a22 => E%ccoeffs(:,4)

      if ( B%derivconst ) then

!     --- Derivatives are constant

         if ( E%typediff==1 ) then

!        ---  E%typediff = 1 i.e. a12 = 0, a11 = a22 = a

            if ( icheld==6 .or. icheld==36 ) then
               do i = 1, n
                  elemvc(1,i) = -dudx(1,1) * a11(i)
                  elemvc(2,i) = -dudx(1,2) * a11(i)
               end do
            else
               som = sum(a11(:))
               elemvc(1,1) = -dudx(1,1)*som/n
               elemvc(2,1) = -dudx(1,2)*som/n
            end if

         else if ( E%typediff==2 ) then

!        ---  E%typediff = 2 i.e. a12 = 0

            if ( icheld==6 .or. icheld==36 ) then
               do i = 1, n
                  elemvc(1,i) = -dudx(1,1) * a11(i)
                  elemvc(2,i) = -dudx(1,2) * a22(i)
               end do
            else
               elemvc(1,1) = -dudx(1,1)*sum(a11(:))/n
               elemvc(2,1) = -dudx(1,2)*sum(a22(:))/n
            end if

         else

!        --- E%typediff = 3 i.e. general

            if ( icheld==6 .or. icheld==36 ) then
               do i = 1, n
                  elemvc(1,i) = - dudx(1,1) * a11(i) &
                                - dudx(1,2) * a12(i)
                  elemvc(2,i) = - dudx(1,1) * a12(i) &
                                - dudx(1,2) * a22(i)
               end do
            else
               som = sum(a12(:))
               elemvc(1,1) = -(dudx(1,1)*sum(a11(:))+dudx(1,2)*som)/n
               elemvc(2,1) = -(dudx(1,1)*som+dudx(1,2)*sum(a22(:)))/n
            end if

         end if

      else

!     --- General element

         if ( E%typediff==1 ) then

!        ---  E%typediff = 1 i.e. a12 = 0, a11 = a22 = a

            if ( icheld==6 .or. icheld==36 ) then
               do i = 1, n
                  elemvc(1,i) = - dudx(i,1) * a11(i)
                  elemvc(2,i) = - dudx(i,2) * a11(i)
               end do
            else
               elemvc(1,1) = -sum(dudx(:,1)*a11(:))/n
               elemvc(2,1) = -sum(dudx(:,2)*a11(:))/n
            end if

         else if ( E%typediff==2 ) then

!        ---  E%typediff = 2 i.e. a12 = 0

            if ( icheld==6 .or. icheld==36 ) then
               do i = 1, n
                  elemvc(1,i) = - dudx(i,1) * a11(i)
                  elemvc(2,i) = - dudx(i,2) * a22(i)
               end do
            else
               elemvc(1,1) = -sum(dudx(:,1)*a11(:))/n
               elemvc(2,1) = -sum(dudx(:,2)*a22(:))/n
            end if

         else

!        --- E%typediff = 3 i.e. general

            if ( icheld==6 .or. icheld==36 ) then
               do i = 1, n
                  elemvc(1,i) = - dudx(i,1) * a11(i) &
                                - dudx(i,2) * a12(i)
                  elemvc(2,i) = - dudx(i,1) * a12(i) &
                                - dudx(i,2) * a22(i)
               end do
            else
               elemvc(1,1) = -sum(dudx(:,1)*a11(:)+dudx(:,2)*a12(:))/n
               elemvc(2,1) = -sum(dudx(:,1)*a12(:)+dudx(:,2)*a22(:))/n
            end if

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldc062'

      end if  ! ( debug )
      call erclos ( 'sepeldc062' )

      end subroutine sepeldc062
