      subroutine sepeldiffup06
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add contribution of second derivative part to matrix in the case of
!     upwind
!     Old subroutine elup06
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elliptic_equation
!     diffusion
!     upwind
!     convection_diffusion
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k, l, n, m
      double precision, pointer :: dphidx(:,:,:)
      double precision, pointer :: alpha(:,:)
      double precision a1(B%n), a2(B%n), a3(B%n), som, sum1

!     a1             Work array to store parts of the computation of the
!                    second derivative term
!                                  l              l          l              l
!                    a1(l) =  a  (x ) d phi /dx (x )  + a  (x ) d phi /dy (x )
!                              11          j             12          j
!
!                                  l              l
!                          +  a  (x ) d phi /dz (x )
!                              13          j
!                    The two-dimensional array is stored as follows:
!                    a1(l,j) = a1(l)   j = 1 (1) n
!     a2             See a1, however now for the second term
!     a3             See a1, however now for the third term
!     alpha          Array of length m x 3 containing the factors alpha
!                                                                      ij
!                    in the integration points
!     dphidx         Array of length n x m x 2 containing the values of the
!                    derivatives of the basis functions in the integration
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     l              Counting variable
!     m              Number of quadrature points
!     n              Number of basis functions
!     som            Parameter to compute a sum
!     sum1           Parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!
!
!        The contribution of the second derivative upwind terms is:
!
!        sum  d/dx  alpha   dphi / dx   (psi -phi )
!         kl      k      kl     j    l      i    i
!
!
!        In order to avoid the computation of second derivatives of basis
!        functions we apply the following approximation:
!
!          klj                             n    klj
!         T    = alpha   dphi / dx     ~  sum  T     phi
!                     kl     j    l    ~  m=1   m       m
!
!
!                   klj     n    klj
!        So d/dx   T    ~  sum  T     d phi / dx
!               k       ~  m=1   m         m    k
!
!
!     For i := 1 (1) n do
!        For j := 1 (1) n do
!            For l := 1 (1) n do
!               a1(l) := alpha11 dphi j / dx1 + alpha12 dphi j / dx2   (xl)
!               a2(l) := alpha12 dphi j / dx1 + alpha22 dphi j / dx2   (xl)
!            For k := 1 (1) m do
!               sec_der jk := sum a1(l) dphi l / dx1 + a2(l) dphi l /dx2  (xk)
!                              l
!               sij := sij - wk sec_der jk ( psi i (xk) - phi i (xk) )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldiffup06' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldiffup06'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dphidx => B%phider
      alpha => E%difftensor
      n = B%n
      m = B%m

!     --- First store some terms to compute the second derivative
!
      do i = 1, n
         do j = 1, n
            do l = 1, n
               a1(l) = alpha(l,1)*dphidx(j,l,1)+alpha(l,2)*dphidx(j,l,2) &
                     + alpha(l,3)*dphidx(j,l,3)
               a2(l) = alpha(l,2)*dphidx(j,l,1)+alpha(l,4)*dphidx(j,l,2) &
                     + alpha(l,5)*dphidx(j,l,3)
               a3(l) = alpha(l,3)*dphidx(j,l,1)+alpha(l,5)*dphidx(j,l,2) &
                     + alpha(l,6)*dphidx(j,l,3)
            end do

!        --- second derivative part

            som = 0d0
            do k = 1, m
               sum1 = 0d0
               do l = 1, n
                  sum1 = sum1 + a1(l)*dphidx(l,k,1)+a2(l)*dphidx(l,k,2) &
                              + a3(l)*dphidx(l,k,3)
               end do
               som = som - (B%psi(i,k)-B%phi(i,k)) * sum1 * B%w(k)
            end do
            E%mat(j,i) = E%mat(j,i) + som
         end do
      end do

1000  call erclos ( 'sepeldiffup06' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%mat, n, n, 'E')
         write(irefwr,*) 'End sepeldiffup06'

      end if  ! ( debug )

      end subroutine sepeldiffup06
