      subroutine sepeldiffus1 ( coeffs, ind, typediff )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill subelement matrix for diffusion equation
!     It is assumed that diffusion coefficients are stored in coefficients
!     1 to 6
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in), target :: coeffs(B%m,6)
      integer, intent(in) :: ind(6), typediff

!     coeffs         i    contains the diffusion coefficients in the quadrature
!                         points:
!                         (.,1): D_xx
!                         (.,2): D_xy
!                         (.,3): D_xz
!                         (.,4): D_yy
!                         (.,5): D_yz
!                         (.,6): D_zz
!     ind            i    Indicates if coefficients are 0, constant or variable
!     typediff       i    Defines type of diffusion tensor
!                         1:  diffusion is a scalar
!                         2:  diffusion is a diagonal tensor
!                         3:  diffusion is a symmetric tensor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELDIFFDIAG  Fill element matrix for diffusion equation
!                    The diffusion is a diagonal tensor
!     SEPELDIFFTENS  Fill element matrix for diffusion equation
!                    The diffusion is a tensor
!     SEPELLAPLACE   Compute the element matrix for the Laplace equation
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldiffus1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldiffus1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case(typediff)

      case(1)

!     --- Diffusion is scalar

         call sepellaplace ( coeffs(:,1) )

      case(2)

!     --- Diffusion is diagonal tensor

         call sepeldiffdiag ( coeffs, ind )

      case(3)

!     --- Diffusion is tensor

         call sepeldifftens ( coeffs, ind )

      end select  ! case(typediff)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldiffus1'

      end if  ! ( debug )
      call erclos ( 'sepeldiffus1' )

      end subroutine sepeldiffus1
