      subroutine sepelfillccof ( isub )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill all complex coefficients in E%ccoeffs
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     initialization
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: isub

!     isub           i    Integer help parameter for do-loops to indicate the
!                         smallest coefficient i that is dependent on space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, nuserh, maxpos, jelgrp, ivectr, icoeff
      double precision :: help

!     help           help variable to compute a product
!     i              Counting variable
!     icoeff         sequence number of coefficient
!     ivectr         reduced vector sequence number
!     jelgrp         Sequence number of element group
!     maxpos         Last position in iuser that may be used
!     nuserh         Help variable to compute nuser
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inivec
      double precision :: getscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETSCAL        Return with value of scalar
!     INIVEC         Return with vector number corresponding to ivecin
!     PRINCOMPL1     Print 2d complex vector
!     PRININ         print 1d integer E%ccoeffs
!     PRINLG         Print and checks length of E%user E%ccoeffss
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!     795   Number of old vector out of range
!     796   ICHOIS for FUNCCF incorrect
!     797   Constant for multiplication incorrect of reference to not allowed
!    2263   element group number exceeds iuser(2)
!    2264   ipos in iuser too large for element group
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelfillccof' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelfillccof'
         write(irefwr,1) 'isub, ncoef, ipos, ishift', &
                          isub, E%ncoef, E%ipos, E%ishift
  1      format ( a, 1x, (10i12) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%ielgrp>E%iuser(2) ) then

!     --- ielgrp too large

         call errint ( E%ielgrp, 1 )
         call errint ( E%iuser(2), 2 )
         call errint ( E%itype, 3 )
         call errsub ( 2263, 3, 0, 0 )
         go to 1000

      end if

      if ( E%ielgrp==E%iuser(2) ) then

!     --- ielgrp is largest element group

         maxpos = E%iuser(1)

      else

!     --- ielgrp is not the largest element group

         maxpos = E%iuser(E%ielgrp+6)-1

         if ( maxpos<=0 ) then

!        --- maxpos<0, because next element not stored
!            first check if there are larger elements that have coefficients

            jelgrp = E%ielgrp+1
100         if ( jelgrp<=E%nelgrp ) then

!           --- jelgrp<=nelgrp

               maxpos = E%iuser(jelgrp+6)-1
               if ( maxpos<=0 ) then

!              --- still no value found

                  jelgrp = jelgrp+1
                  go to 100

               end if

            else

!           --- ielgrp is largest element group

               maxpos = E%iuser(1)

            end if

         end if

      end if
      if ( E%ipos>maxpos ) then

!     --- ipos too large

         call errint ( E%ielgrp, 1 )
         call errint ( E%itype, 2 )
         call errint ( 1, 3 )
         call errint ( E%ipos, 4 )
         call errint ( maxpos, 5 )
         call errsub ( 2264, 5, 0, 0 )
         go to 1000

      end if

      E%ccoeffs = 0d0

      do i = isub, E%ncoef

         icoeff = i+E%ishift
         if ( E%ipos>maxpos ) then

!        --- ipos too large

            call errint ( E%ielgrp, 1 )
            call errint ( E%itype, 2 )
            call errint ( icoeff, 3 )
            call errint ( E%ipos, 4 )
            call errint ( maxpos, 5 )
            call errsub ( 2264, 5, 0, 0 )
            go to 1000

         end if
         E%ind(i) = E%iuser(E%ipos)
         E%mst(i) = 0
         nuserh = 0
         if ( debug ) write(irefwr,1) 'i, ipos, E%ind(i)', i, E%ipos, E%ind(i)
         E%ipos = E%ipos+1

         if ( E%ind(i)==0 ) then

!        --- E%ind(i) = 0, clearing has already been performed
!            coefficient is zero

         else if ( E%ind(i)<0 ) then

!        --- E%ind(i)<0, coefficient is constant

            if ( E%ind(i)>=-5 ) then
               call errint ( E%ipos-1, 1 )
               call errint ( E%ind(i), 2 )
               call errint ( E%ielem, 3 )
               call errint ( E%itype, 4 )
               call errint ( E%ielgrp, 5 )
               call errsub ( 42, 5, 0, 0 )
            end if
            nuserh = -E%ind(i)
            E%ccoeffs(:,i) = cmplx(E%user(nuserh),E%user(nuserh+1), kind=8)

         else if ( E%ind(i)>100000000 ) then

!        --- special case, scalar found, fill constant value

            help = getscal ( E%ind(i)-100000000 )
            E%ind(i) = -E%ind(i)
            E%ccoeffs(:,i) = help

            if ( abs(help)<=0d0 ) E%ind(i) = 0

            if ( debug ) write(irefwr,2) 'help', help

         else

!        --- E%ind(i)>0

            if ( E%ind(i)==2003 ) then

!           --- E%ind(i) = 2003, coefficient is equal to an old vector

               E%ivecnums(i) = E%ipos
               E%mst(i) = E%iuser(E%ipos+1)
               ivectr = inivec ( E%iuser(E%ipos) )
               if ( ivectr<=0 .or. ivectr>=1000 ) then
                  call errint ( ivectr, 1 )
                  call errint ( E%ipos, 2 )
                  call errint ( E%ielem, 3 )
                  call errint ( E%itype, 4 )
                  call errint ( E%ielgrp, 5 )
                  call errsub ( 795, 5, 0, 0 )
               end if
               E%ipos = E%ipos + 2

               indvector(ivectr) = 1  ! vector is used

            else if ( E%ind(i)==2004 ) then

!           --- E%ind(i) = 2004, coefficient is equal to an old vector
!                              multiplied by a constant or function

               E%mst(i) = E%iuser(E%ipos+1)
               E%ivecnums(i) = E%ipos
               ivectr = inivec ( E%iuser(E%ipos) )
               if ( ivectr<=0 .or. ivectr>=1000 ) then
                  call errint ( ivectr, 1 )
                  call errint ( E%ipos, 2 )
                  call errint ( E%ielem, 3 )
                  call errint ( E%itype, 4 )
                  call errint ( E%ielgrp, 5 )
                  call errsub ( 795, 5, 0, 0 )
               end if
               if ( E%iuser(E%ipos+2)>=1000 ) then
                  call errint ( E%iuser(E%ipos+2), 1 )
                  call errint ( E%ipos+2, 2 )
                  call errint ( E%ielem, 3 )
                  call errint ( E%itype, 4 )
                  call errint ( E%ielgrp, 5 )
                  call errsub ( 796, 5, 0, 0 )
               end if
               if ( E%iuser(E%ipos+2)<=0 ) then

!              --- E%iuser(ipos+2)<0,  multiply by constant

                  if ( E%iuser(E%ipos+2)>=-5) then
                     call errint ( E%iuser(E%ipos+2), 1 )
                     call errint ( E%ipos+2, 2 )
                     call errint ( E%ielem, 3 )
                     call errint ( E%itype, 4 )
                     call errint ( E%ielgrp, 5 )
                     call errsub ( 797, 5, 0, 0 )
                  end if
               end if
               E%ipos = E%ipos + 3

               indvector(ivectr) = 1  ! vector is used

            else if ( E%ind(i)==2001 .or. E%ind(i)==2002 ) then

!           --- E%ind(i) = 2001 or 2002

               E%mst(i) = E%iuser(E%ipos)-1
               if ( E%ind(i)==2001 ) nuserh = E%mst(i)+E%npoint
               if ( E%ind(i)==2002 ) nuserh = E%mst(i)+E%nelem
               E%ipos = E%ipos+1

            end if

         end if
         E%nuser = max( E%nuser, nuserh )

      end do  ! i = isub, E%ncoef

!     --- Test coeffs lengths, i of nodes and i of degrees
!         of freedom

      call prinlg ( 'iuser', E%ipos-1, E%user, E%iuser )
      call prinlg ( 'user', -E%nuser, E%user, E%iuser )

      if ( E%filloldvec ) indvector(iseqprevsol) = 1

1000  if ( debug ) then

!     --- Debug information

         call prinin ( E%ind, E%ncoef, 'ind' )
         call prinin ( E%ist, E%ncoef, 'ist' )
         call prinin ( E%ivecnums, E%ncoef, 'ivecnums' )
         call princompl1 ( E%ccoeffs, E%ncoef, B%m, 'E%ccoeffs' )
         write(irefwr,*) 'End sepelfillccof'

      end if  ! ( debug )
      call erclos ( 'sepelfillccof' )

      end
