      subroutine sepelfillqmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the transformation matrix qmat for the mapping of a 3d surface
!     onto a 2d surface depending on the parameter iqmat
!     Old subroutine elfillqmat
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     element
!     transformation
!     2d
!     3d
!     basis_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, j, idir, m
      double precision :: norm, exi(3), eeta(3), eh(3), eidir(3), ezeta(3)
      double precision, pointer :: dxdxi(:,:,:), qmat(:,:,:)

!                    they are not
!     dxdxi          Contains dx/dxi, dx/deta, dy/dxi and dy/deta
!     eeta           base vector in eta direction
!     eh             help vector
!     eidir          vector in e_idir direction
!     exi            base vector in xi direction
!     ezeta          base vector in zeta direction
!     idir           Direction of base vector i
!     j              Counting variable
!     k              Counting variable
!     m              number of quadrature points
!     norm           norm of vector
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     To compute qmat, we use the lamina coordinate system as introduced by
!     Hughes: (iqmat<6)
!      Define e_xi = x,xi / ||x,xi||
!      Define e_eta = x,eta / ||x,eta||
!      e_3(l) = e_xi x e_eta / ||e_xi x e_eta||
!      The rest is the lamina coordinate system:
!      (This part is skipped since it does not add anything usefull)
!      e_alpha = (e_xi + e_eta)/2 / || (e_xi + e_eta)/2 ||
!      e_beta = e_3(l) x e_alpha / || e_3(l) x e_alpha ||
!      e_1(l) = srqt(2)/2 ( e_alpha - e_beta )
!      e_2(l) = srqt(2)/2 ( e_alpha + e_beta )
!
!     or in case iqmat>5:
!
!      idir = iqmat-5
!      Define e_xi = x,xi / ||x,xi||
!      Define e_eta = x,eta / ||x,eta||
!      n = e_xi x e_eta / ||e_xi x e_eta||
!      e_zeta is projection of e_idir onto surface:
!         e_zeta = e_idir - n(idir) n
!         e_xi = e_zeta x n / ||e_zeta x n||
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelfillqmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelfillqmat'
         write(irefwr,1) 'E%iqmat, m, itang, inorm', &
                          E%iqmat, B%m, B%itang, B%inorm
         call prinrl1 ( B%dxdxig(1,1,1), 3, B%m, 'dxdxi' )
         call prinrl1 ( B%dxdxig(1,1,2), 3, B%m, 'dxdeta' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if

      dxdxi => B%dxdxig
      qmat => B%qmat
      m = B%m

      if ( E%iqmat<6 ) then

!     --- Standard case

         do k = 1, m

!        --- Define e_xi = x,xi / ||x,xi||

            norm = 1d0/sqrt(dot_product(dxdxi(k,1:3,1),dxdxi(k,1:3,1)))
            exi(:) = dxdxi(k,:,1)*norm

!           --- Define e_eta = x,eta / ||x,eta||

            norm = 1d0/sqrt(dot_product(dxdxi(k,1:3,2),dxdxi(k,1:3,2)))
            eeta(:) = dxdxi(k,:,2)*norm

!           --- e_3(l) = e_xi x e_eta / ||e_xi x e_eta||

            call eloutr ( exi, eeta, eh )
            norm = 1d0/sqrt(dot_product(eh,eh))
            eh = eh*norm

            if ( debug ) then
               write(irefwr,1) 'k', k
               call prinrl ( exi, 3, 'exi' )
               call prinrl ( eeta, 3, 'eeta' )
               call prinrl ( eh, 3, 'eh' )
            end if  ! ( debug )

            qmat(B%itang,:,k) = exi   ! t1
            qmat(2,:,k) = eeta        ! t2
            qmat(B%inorm,:,k) = eh    ! n

            if ( debug ) then
               write(irefwr,2) 't1', qmat(B%itang,1:3,k)
               write(irefwr,2) 't2', qmat(2,1:3,k)
               write(irefwr,2) 'n', qmat(3,1:3,k)
            end if  ! ( debug )

         end do  ! k= 1, m

      else

!     --- iqmat>5, special case, map onto Cartesian direction

         idir = E%iqmat-5
         eidir = 0d0
         eidir(idir) = 1d0

!        --- Loop over integration points

         do k = 1, m

!        --- Define e_xi = x,xi / ||x,xi||

            norm = 1d0/sqrt(dot_product(dxdxi(k,1:3,1),dxdxi(k,1:3,1)))
            exi(:) = dxdxi(k,:,1)*norm

!           --- Define e_eta = x,eta / ||x,eta||

            norm = 1d0/sqrt(dot_product(dxdxi(k,1:3,2),dxdxi(k,1:3,2)))
            eeta(:) = dxdxi(k,:,2)*norm

!           --- e_3(l) = e_xi x e_eta / ||e_xi x e_eta||

            call eloutr ( exi, eeta, eh )
            norm = 1d0/sqrt(dot_product(eh,eh))
            eh = eh*norm

!           --- e_zeta = e_idir - n(idir) n

            ezeta = eidir - eh(idir) * eh
            norm = 1d0/sqrt(dot_product(ezeta,ezeta))
            ezeta = ezeta*norm

!           --- e_xi = e_zeta x n / ||e_zeta x n||

            call eloutr ( ezeta, eh, exi )
            norm = 1d0/sqrt(dot_product(exi,exi))
            exi = exi*norm

!           --- Store in qmat

            qmat(2,:,k) = ezeta  ! t1
            qmat(3,:,k) = exi    ! t2
            qmat(1,:,k) = eh     ! n

         end do  ! k= 1, m

      end if  ! E%iqmat<6

      call erclos ( 'sepelfillqmat' )
      if ( debug ) then

         write(irefwr,*) 'End sepelfillqmat'

      end if  ! ( debug )

      end
