      subroutine sepelm302
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill element matrix and element vector for line element for
!     elasticity-flow interaction  (bearing)
!     Old subroutine elm302
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     elasticity_equation
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: delta
      double precision, pointer :: x(:), pold(:), dphidx(:)

!     delta          jacobian of element
!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                         d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!     pold           Contains the pressure in the prior iteration
!     x              x co-ordinates of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPEL0302      Fill element independent quantities for element type 302
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELFILLVAR   Fill space dependent coefficients
!     SEPELMAT302    Fill element matrix for type 30
!     SEPELOLDVECS   Fill old vectors for element
!     SEPELVEC302    Fill element vector for type 302
!     SEPPRINELMAT   Print element matrix and vector
! **********************************************************************
!
!                       I/O
!
!      The user must given 6 coefficients and parameters in the following way:
!     1    Diameter of rod
!     2    eta
!     3    k
!     4    p0
!     5    u
!     6    method = linear + 10 * itangstress
!          with
!          linear:
!            1       stationary Picard
!            2       stationary Newton
!            3       instationary Picard
!            4       instationary Newton
!          itangstress:
!            0       shear stress is taken into account
!            1       shear stress is not taken into account
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm302' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm302'
         write(irefwr,1) 'ielem', E%ielem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%mat = 0d0
      E%vec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0302

!     --- Fill basis functions

      call sepelbasefunc

!     --- Fill old vectors in quadrature points if necessary

      call sepeloldvecs

      if ( debug ) then
         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
      end if  ! ( debug )

!     --- Fill variable coefficients if necessary

      if ( E%isupcfvar>=E%isubcfvar ) call sepelfillvar

!     --- compute hinit   h^n(= hinit + v) and dp /dz

      x => B%xyz(:,1)
      dphidx => B%phider(:,1,1)
      pold => E%ugauss(:,3)
      E%h0 = x-E%coeffs(:,1)*0.5d0
      E%hn = E%h0+E%ugauss(:,1)
      E%dpndz = pold(1)*dphidx(1)+pold(2)*dphidx(2)

      delta = (B%w(2)+B%w(1))*pi/3d0
      E%waxi(1) = delta*(2d0*x(1)+x(2))
      E%waxi(2) = delta*(x(1)+2d0*x(2))

!     --- Fill element matrix

      if ( matrix .and. E%notmat==0 ) call sepelmat302
      if ( vector .and. E%notvec==0 ) call sepelvec302

!     --- Remove basis functions and coefficients in case of last call

      if ( debug ) write(irefwr,1) 'last', E%last
      if ( E%last==1 ) call sepeldealloc

1000  if ( debug ) then

!     --- Debug information

         call sepprinelmat
         write(irefwr,*) 'End sepelm302'

      end if  ! ( debug )
      call erclos ( 'sepelm302' )

      end subroutine sepelm302
