      subroutine sepelm805
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for second order elliptic equations
!     and related heat equation with two unknowns per point
!     In case of defect correction, the first matrix contains the upwind
!     element matrix and the second one the standard Galerkin matrix
!     The element vectors are identical for both
!     Old subroutine elm805
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: veloc(:,:)
      integer :: i

!     i              Counting variable
!     veloc          velocity or rho * velocity
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPEL0805      initialize parameters for subroutine SEPELM805
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELCONVECT10 Fill (sub) element matrix for convection equation
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELDIFFUS1   Fill subelement matrix for diffusion equation
!     SEPELFILLVAR   Fill space dependent coefficients
!     SEPELLOAD1     Fill the element vector with a load
!     SEPELMASSMAT1  Fill the element mass matrix; one unknown per node
!                    Consistent mass matrix
!     SEPELOLDVECS   Fill old vectors for element
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  This parameter is no longer in use.
!         The parameter imas in BUILD defines the type of mass matrix
!         Now the parameter is reserved for future purposes
!     2:  At this moment only central discretization is available
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates
!         2: Polar co-ordinates
!     5:  This parameter depends on the type number to be used
!         In case of type number 803 it indicates the type of linearization:
!         Possible values:
!         0,1: Picard
!         2:   Newton
!     Parameters 6-20 corresponding to first unknown:
!     6:  alpha 11  (1D/2D/3D)
!     7:  alpha 12  (2D/3D)
!     8:  alpha 13  (3D)
!     9:  alpha 22  (2D/3D)
!    10:  alpha 23  (3D)
!    11:  alpha 33  (3D)
!    12:  u 1       (1D/2D/3D)
!    13:  u 2       (2D/3D)
!    14:  u 3       (3D)
!    15:  beta      (1D/2D/3D)
!    16:  f         (1D/2D/3D)
!    17:  rho cp    (1D/2D/3D)
!    18:  Not yet used (must be 0)
!    19:  Not yet used (must be 0)
!    20:  Not yet used (must be 0)
!    Parameters: 21-35 corresponding to second unknown have exactly
!    the same meaning
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Differential equation:
!    rho cp d phi/d t - div alpha grad phi +rho cp u . grad phi + beta phi = f
!                                                  - &
!        The following formulae are used:
!           S   =  S      +  S   +  S
!                   alpha     u      beta
!           S     (i,j) = / alpha grad phi  . grad phi   de
!            alpha        e               j           i
!           S     (i,j) = / u . grad phi   phi  de
!            u            e -           j     i
!           S     (i,j) = / beta phi   phi  de
!            beta        e          j     i
!           M (i,j) = / rho cp phi  phi  de
!                     e           i    j
!           F(i)    =  /  f phi  de
!                      e       i
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
!        In the axi-symmetric case they include the factor 2 pi r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm805' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm805'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%mat = 0d0
      E%vec = 0d0

      if ( E%first==0 ) then

!     --- Define element independent quantities
!         only for first element

         call sepel0805

         if ( .not. E%rhoone ) allocate ( veloc(B%m,E%ndim) )

      end if  ! ( E%first==0 )

!     --- Fill basis functions

      call sepelbasefunc

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

!     --- Fill all coefficients

      if ( E%isupcfvar>=E%isubcfvar ) call sepelfillvar

      if ( debug ) then
         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,*) 'B%fillqmat ', B%fillqmat
      end if  ! ( debug )

      if ( matrix ) then

!     --- Fill element matrix

         E%submat = 0d0
         if ( E%diffusion1 ) &
            call sepeldiffus1 ( E%coeffs, E%ind, E%typediff1 )

         if ( E%convection1 ) then
            if ( E%rhoone ) then
               veloc => E%coeffs(:,7:9)
            else
               do i = 1, E%ndim
                  veloc(:,i) = E%coeffs(:,6+i)*E%coeffs(:,12)
               end do  ! i = 1, E%ndim
            end if  ! ( E%rhoone )
            call sepelconvect10 ( veloc, E%ind(7) )
         end if  ! ( E%convection1 )

         if ( E%zeroterm1 ) call sepelmassmat1 ( E%coeffs(:,10) )

         E%mat(1:E%icount:2,1:E%icount:2) = E%submat

         E%submat = 0d0

         if ( E%diffusion2 ) &
            call sepeldiffus1 ( E%coeffs(:,16), E%ind(16), &
                                E%typediff2 )

         if ( E%convection2 ) then
            if ( E%rhoone ) then
               veloc => E%coeffs(:,22:24)
            else
               do i = 1, E%ndim
                  veloc(:,i) = E%coeffs(:,21+i)*E%coeffs(:,27)
               end do  ! i = 1, E%ndim
            end if  ! ( E%rhoone )
            call sepelconvect10 ( veloc, E%ind(22) )
         end if  ! ( E%convection2 )

         if ( E%zeroterm2 ) call sepelmassmat1 ( E%coeffs(:,25) )

         E%mat(2:E%icount:2,2:E%icount:2) = E%submat

      end if  ! ( matrix )

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

         if ( E%sourceterm1 ) then
            E%subvec = 0d0
            call sepelload1 ( E%coeffs(:,11), B%jdiag, B%phi )
            E%vec(1:E%icount:2) = E%subvec
         end if  ! ( E%sourceterm )
         if ( E%sourceterm2 ) then
            E%subvec = 0d0
            call sepelload1 ( E%coeffs(:,26), B%jdiag, B%phi )
            E%vec(2:E%icount:2) = E%subvec
         end if  ! ( E%sourceterm )

      end if  ! ( vector .and. notvec/=1 )

!     --- Fill mass matrix if necessary

      if ( notmas==0 ) then

!     --- notmas = 0,  compute mass matrix

         if ( E%imas==1 ) then

!        --- diagonal mass matrix

            E%mmat(1:E%icount:2,1) = E%coeffs(:,12)*B%w
            E%mmat(2:E%icount:2,1) = E%coeffs(:,27)*B%w

         else

!        --- full mass matrix

            call sepelmassmat1 ( E%coeffs(:,12) )
            E%mmat(1:E%icount:2,1:E%icount:2) = E%submat

            call sepelmassmat1 ( E%coeffs(:,27) )
            E%mmat(2:E%icount:2,2:E%icount:2) = E%submat

         end if  ! ( E%imas==1 )

      end if  ! ( notmas==0 )

      if ( E%last==1 ) then
         call sepeldealloc
         if ( .not. E%rhoone ) deallocate ( veloc )
         deallocate ( E%subvec, E%submat )
      end if  ! ( E%last==1 )

1000  if ( debug ) then

!     --- debug = true

         write(irefwr,1) 'ielem, itype', ielem, itype
         write(irefwr,1) 'imas', imas
         call prinrl(E%vec,icount,'E%vec')
         call prinrl1(E%mat,icount,icount,'E%mat')
         if ( notmas==0 ) &
            call prinrl1(E%mmat,icount,icount,'E%mmat')

      end if
      call erclos ( 'sepelm805' )

      end subroutine sepelm805
