      subroutine sepelmlinelas3d ( Dw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for linear elasticity problem
!     3d case
!     Old subroutine el8108
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elasticity_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: Dw(E%dimD,E%dimD,m)

!     Dw             i    Dmat * w
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, target :: Stemp(n,n)
      double precision, pointer :: S(:,:)
      double precision, pointer :: phix(:,:), phiy(:,:), phiz(:,:)
      integer :: i, j, nd

!     S              (sub part of) element matrix
!     Stemp          sub part of element matrix
!     i              Counting variable
!     j              Counting variable
!     nd             Dimension of Matrix D
!     phix           Array of length n x m x ndim  containing the
!                    derivatives of the basis functions in the sequence:
!                    d phi / dx (xg ) = phix (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = phix (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = phix (i,k,3);
!                         i        k
!                    If the element is a linear tetrahedron (n=4), then
!                    the derivatives are constant and only k = 1 is filled.
!     phiy           dphi/dy
!     phiz           dphi/dz
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPELPHIXPHIX  Fill part of element matrix
!                    / C dphi_i/dx_k dphi_j/dx_l dOmega
!                    Non-symmetric matrix
!     SEPELPHIXPHIXA Fill part of element matrix
!                    / C dphi_i/dx_k dphi_j/dx_l dOmega
!                    Add to S-matrux
!                    Symmetric matrix
!     SEPELPHIXPHIXB Fill part of element matrix
!                    / C dphi_i/dx_k dphi_j/dx_l dOmega
!                    Add to S-matrux
!                    Non-symmetric matrix
!     SEPELPHIXPHIXS Fill part of element matrix
!                    / C dphi_i/dx_k dphi_j/dx_l dOmega
!                    Symmetric matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                                               T
!     The element stiffness matrix is given by B  D  B,
!     with B the matrix:
!     |   phi       0          0       |
!     |      x                         |
!     |                                |
!     |    0       phi         0       |
!     |               y                |
!     |                                |
!     |    0        0         phi      |
!     |                          z     |
!     |                                |
!     |   phi      phi         0       |
!     |      y        x                |
!     |                                |
!     |    0       phi        phi      |
!     |               z          y     |
!     |                                |
!     |   phi       0         phi      |
!     |      z                   x     |
!     |                                |
!     Each phi stands for a series of basis functions
!     The matrix D is a 6x6 matrix with elements d
!                                                 ij
!     Computation of the stiffness matrix results in a 3x3 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S      S     |
!            |    uu      uv     uw   |
!            |                        |
!            |   S       S      S     |
!     S =    |    vu      vv     vw   |
!            |                        |
!            |   S       S      S     |
!            |    wu      wv     ww   |
!     The submatrices are defined by:
!     S    =  S
!      xy      yx
!     S uu =  phix ( phix d11 + phiy d14 + phiz d16 ) +
!             phiy ( phix d41 + phiy d44 + phiz d46 ) +
!             phiz ( phix d61 + phiy d64 + phiz d66 )
!     S uv =  phix ( phiy d12 + phix d14 + phiz d15 ) +
!             phiy ( phiy d42 + phix d44 + phiz d45 ) +
!             phiz ( phiy d62 + phix d64 + phiz d65 )
!     S uw =  phix ( phiz d13 + phiy d15 + phix d16 ) +
!             phiy ( phiz d43 + phiy d45 + phix d46 ) +
!             phiz ( phiz d63 + phiy d65 + phix d66 )
!     S vv =  phiy ( phiy d22 + phix d24 + phiz d25 ) +
!             phix ( phiy d42 + phix d44 + phiz d45 ) +
!             phiz ( phiy d52 + phix d54 + phiz d55 )
!     S vw =  phiy ( phiz d23 + phiy d25 + phix d26 ) +
!             phix ( phiz d43 + phiy d45 + phix d46 ) +
!             phiz ( phiz d53 + phiy d55 + phix d56 )
!     S ww =  phiz ( phiz d33 + phiy d35 + phix d36 ) +
!             phiy ( phiz d53 + phiy d55 + phix d56 ) +
!             phix ( phiz d63 + phiy d65 + phix d66 )
!     In case of igprob = 1, these matrices reduce considerably to:
!     S uu =  phix phix d11 + phiy phiy d44 + phiz phiz d66
!     S uv =  phix phiy d12 + phiy phix d44
!     S uw =  phix phiz d13 + phiz phix d66
!     S vv =  phiy phiy d22 + phix phix d44 + phiz phiz d55
!     S vw =  phiy phiz d23 + phiz phiy d55
!     S ww =  phiz phiz d33 + phiy phiy d55 + phix phix d66
!     The submatrices are first stored in swork and put into s
!     These formulae must also be integrated by the integration rule
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmlinelas3d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmlinelas3d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      phiz => B%phider(:,:,3)

      nd = 3

!     --- Matrix is symmetrical, so first lower part is stored
!         S uu =  phix phix d11 + phiy phiy d44 + phiz phiz d66

      S => E%mat(1:3*n:3,1:3*n:3)
      call sepelphixphixs ( phix, phix, Dw(1,1,:), S )
      call sepelphixphixa ( phiy, phiy, Dw(4,4,:), S )
      call sepelphixphixa ( phiz, phiz, Dw(6,6,:), S )

      if ( debug ) call prinrl1 ( S, n, n, 'suu' )

!     --- S uv =  phix phiy d12 + phiy phix d44

      S => Stemp
      call sepelphixphix ( phix, phiy, Dw(1,2,:), S )
      call sepelphixphixb ( phiy, phix, Dw(4,4,:), S )

!     --- Put s into E%mat

      do i = 1, n
         do j = 1, i-1
            E%mat(2+(i-1)*nd,1+(j-1)*nd) = S(i,j)
            E%mat(1+(i-1)*nd,2+(j-1)*nd) = S(j,i)
         end do  ! j = 1, i-1
         E%mat(2+(i-1)*nd,1+(i-1)*nd) = S(i,i)
      end do  ! i = 1, n

      if ( debug ) call prinrl1 ( S, n, n, 'suv' )

!     --- S uw =  phix phiz d13 + phiz phix d66

      S => Stemp
      call sepelphixphix ( phix, phiz, Dw(1,3,:), S )
      call sepelphixphixb ( phiz, phix, Dw(6,6,:), S )

!     --- Put s into E%mat

      do i = 1, n
         do j = 1, i-1
            E%mat(3+(i-1)*nd,1+(j-1)*nd) = S(i,j)
            E%mat(1+(i-1)*nd,3+(j-1)*nd) = S(j,i)
         end do  ! j = 1, i-1
         E%mat(3+(i-1)*nd,1+(i-1)*nd) = S(i,i)
      end do  ! i = 1, n

      if ( debug ) call prinrl1 ( S, n, n, 'suw' )

!     --- S vv =  phiy phiy d22 + phix phix d44 + phiz phiz d55

      S => E%mat(2:3*n:3,2:3*n:3)
      call sepelphixphixs ( phiy, phiy, Dw(2,2,:), S )
      call sepelphixphixa ( phix, phix, Dw(4,4,:), S )
      call sepelphixphixa ( phiz, phiz, Dw(5,5,:), S )

      if ( debug ) call prinrl1 ( S, n, n, 'svv' )

!     --- S vw =  phiy phiz d23 + phiz phiy d55

      S => Stemp
      call sepelphixphix ( phiy, phiz, Dw(2,3,:), S )
      call sepelphixphixb ( phiz, phiy, Dw(5,5,:), S )

!     --- Put s into E%mat

      do i = 1, n
         do j = 1, i-1
            E%mat(3+(i-1)*nd,2+(j-1)*nd) = S(i,j)
            E%mat(2+(i-1)*nd,3+(j-1)*nd) = S(j,i)
         end do  ! j = 1, i-1
         E%mat(3+(i-1)*nd,2+(i-1)*nd) = S(i,i)
      end do  ! i = 1, n

      if ( debug ) call prinrl1 ( S, n, n, 'svw' )

!     --- S ww =  phiz phiz d33 + phiy phiy d55 + phix phix d66

      S => E%mat(3:3*n:3,3:3*n:3)
      call sepelphixphixs ( phiz, phiz, Dw(3,3,:), S )
      call sepelphixphixa ( phiy, phiy, Dw(5,5,:), S )
      call sepelphixphixa ( phix, phix, Dw(6,6,:), S )

      if ( debug ) call prinrl1 ( S, n, n, 'sww' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelmlinelas3d'

      end if  ! ( debug )
      call erclos ( 'sepelmlinelas3d' )

      end subroutine sepelmlinelas3d
