      subroutine sepelp01
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     First order 1d element (2 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: h, xdiff(1:E%ndim), hloc, d
      double precision, pointer :: x(:), xg(:), phix(:,:), lambda(:)
      integer :: k

!     d              Help variable to compute the weights
!     h              Step size (x2-x1)
!     hloc           Local weight
!     k              Loop variable
!     lambda         Contains values of lambda_1 in quadrature points
!     phix           First derivative of basis function phi
!     x              x-coordinates of nodal points
!     xdiff          Difference vector between first and last node
!     xg             x-coordinates of Gauss points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRORELVAR0    Print error message errornr for element
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
!     SEPELP01QMAT   Fill array qmat with transformation to local coordinates
!                    Linear 1-dimensional elements
!     SEPELPGAUSS    Fill coordinates of Gauss points
!     SEPELPLAMBDA1D Fill the one-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The basis functions are defined as follows:
!     third degree polynomials
!     phi_1(x1) = 1, phi_1(x2) = 0
!     phi_2(x1) = 0, phi_2(x2) = 1
!
!     Let lambda be the linear basis functions and h = x2-x1
!
!     From these demands one easily derives:
!
!     phi_1 = lambda_1
!     phi_2 = lambda_2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp01' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp01'
         write(irefwr,1) 'ndim, ifirst, jcart', E%ndim, E%first, B%cart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x  => B%xyz(:,1)
      xg => B%xyzg(:,1)
      phix => B%phider(:,:,1)
      B%lamder => B%phider
      B%lamderc(1,:,1) = -1d0
      B%lamderc(2,:,1) = 1d0

!     --- First call: compute some element independent quantities

      if ( E%first==0 ) call sepelp01first
      lambda => B%lam(2,:)

!     --- Compute length of element

      if ( E%ndim==1 ) then

!     --- Standard: 1d

         h = x(2)-x(1)

      else

!     --- ndim>1: line element

         xdiff = B%xyz(2,:)-B%xyz(1,:)
         h = sqrt(dot_product(xdiff,xdiff))

      end if  ! ( E%ndim==1 )
      hloc = abs(h)

      if ( hloc<=0d0 ) then

!     --- Error 105: h is zero

         call errorelvar0 ( 105 )
         call prinrl2(B%xyz,2,E%ndim,'co-ordinates')
         go to 1000

      end if  ! ( abs(h)<=0d0 )

!     --- Fill Gauss points

      if ( E%ndim==1 ) then

!     --- Standard: 1d

         call sepelpgauss

      else

!     --- ndim>1: line element

         do k = 1, B%m
            B%xyzg(k,:) = B%xyz(1,:)+lambda(k)*xdiff(:)
         end do  ! k = 1, B%m

      end if  ! ( E%ndim==1 )

!     --- Fill weights

      if ( B%rule==1 .and. B%cart>=2 ) then

!     --- irule=1 and jcart==3, polar coordinates

         d = hloc * pi / 3d0
         B%w(1) = d * ( 2d0*x(1) + x(2) )
         B%w(2) = d * ( x(1) + 2d0*x(2) )

      else if ( B%rule>1 .and. B%cart>=2 ) then

!     --- irule>1 and jcart==3, polar coordinates

         B%w = hloc * B%wc * xg * pi2 * 2d0

      else

!     --- Standard case

         B%w = hloc*B%wc

      end if  ! ( B%rule==1 .and. B%cart>=2 )

      if ( B%fillderiv ) then

!     --- fillderiv = true, fill first derivatives

         d = 1d0/h
         phix(1,:) = -d
         phix(2,:) = d

      end if  ! ( B%fillderiv )

      if ( E%ndim>1 .and. B%fillqmat ) call sepelp01qmat ( xdiff, h )

      if ( E%ktang>0 ) then
         do k = 1, E%ndim
            B%tang(:,k) = (B%xyz(2,k) - B%xyz(1,k))/abs(h)
         end do  ! i = 1, E%ndim
      end if  ! ( E%ktang>0 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( B%w, m, 'w' )
         write(irefwr,*) 'fillqmat ', B%fillqmat
         write(irefwr,*) 'End sepelp01'

      end if  ! ( debug )
      call erclos ( 'sepelp01' )

      end subroutine sepelp01
