      subroutine sepelp03loc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill local coordinates for triangle in 3d
!     Depending on E%iqmat, also qmat is filled
! **********************************************************************
!
!                       KEYWORDS
!
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      integer :: n, m, ndim
      double precision :: et1(E%ndim), et2(E%ndim), &
                          normal(E%ndim), ezeta(E%ndim), exi(E%ndim)
      double precision, pointer :: x(:,:), qmat(:,:,:), xloc(:,:)

!     et1            First tangential vector
!     et2            Second tangential vector
!     exi            base vector in xi direction
!     ezeta          base vector in zeta direction
!     i              Counting variable
!     m              number of quadrature points
!     n              number of basis functions for each component
!     ndim           Dimension of the space.
!     normal         Vector containg the normal on the element
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     x              x co-ordinate of point
!     xloc           co-ordinates of vertices in local (xi-zeta) coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELLOCCOOR      Compute local coordinates of a surface element in 3d given
!                    the transformation by base vectors
!     ELOUTR         Compute cross product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp03loc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp03loc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x => B%xyz
      qmat => B%qmat
      xloc => B%xloc
      n = B%n
      m = B%m
      ndim = E%ndim

!     --- Compute e_t1 and e_t2

      et1(:) = x(2,:)-x(1,:)
      et2(:) = x(3,:)-x(1,:)
      et1 = et1/sqrt(dot_product(et1,et1))
      et2 = et2/sqrt(dot_product(et2,et2))

!     --- Compute normal n

      call eloutr ( et1, et2, normal )
      normal = normal/sqrt(dot_product(normal,normal))

!     --- Compute e_zeta

      if ( E%iqmat<=5 ) then

!     --- E%iqmat = 5, e_zeta = e_t1

         ezeta = et1

      else

!     --- E%iqmat>5, e_zeta = projection of e_i onto surface

         i = E%iqmat-5
         et1 = 0d0
         et1(i) = 1d0
         ezeta = et1-normal(i)*normal
         ezeta = ezeta/sqrt(dot_product(ezeta,ezeta))

      end if

!     --- Compute  e_xi = (e_zeta x n) / ||e_zeta x n||

      call eloutr ( normal, ezeta, exi )
      exi = exi/sqrt(dot_product(exi,exi))

!     --- Compute local coordinates

      call elloccoor ( x, xloc, ezeta, exi, 3 )
      if ( E%iqmat/=0 ) then

!     --- E%iqmat # 0, fill array qmat with e_zeta, e_xi and n

         do i = 1, m
            qmat(1,1:ndim,i) = normal(1:ndim) ! n
            qmat(2,1:ndim,i) = ezeta(1:ndim)  ! t1
            qmat(3,1:ndim,i) = exi(1:ndim)    ! t2
         end do  ! i = 1, m

      end if

1000  call erclos ( 'sepelp03loc' )
      if ( debug ) then

!     --- Debug information

         call prinrl2 ( x, 3, 3, 'coordinates' )
         call prinrl2 ( B%xloc, 3, 2, 'local-coordinates' )
         call prinrl ( et1, 3, 'et1' )
         call prinrl ( et2, 3, 'et2' )
         call prinrl ( ezeta, 3, 'ezeta' )
         call prinrl ( exi, 3, 'exi' )
         call prinrl ( normal, 3, 'normal' )
         if ( E%iqmat/=0 ) call prinrl2( B%qmat, 3, 3, 'qmat' )
         write(irefwr,*) 'End sepelp03loc'

      end if  ! ( debug )

      end subroutine sepelp03loc
