      subroutine sepelp04
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     Quadratic triangle (6 nodes)
!     Old routine: elp622
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision :: rc, h, h1, h2
      double precision, pointer :: x(:), xg(:), jac(:)
      logical :: curved

!     h              help variable to store a quantity temporarily
!     i              counting variable
!     jac            Jacobian in the quadrature points
!     rc             Help variable to store part of a weight for
!                    axi-symmetric co-ordinates
!     x              Contains x co-ordinates of nodes
!     xg             x-coordinates of Gauss points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELP04FIRST  compute some element independent quantities
!                    Quadratic triangle (6 nodes)
!     SEPELP04JACOB  Compute Jacobian and derivatives of basis functions
!     SEPELPGAUSS    Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp04' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp04'
         write(irefwr,1) 'ifirst, n, m', E%first, B%n, B%m
         write(irefwr,1) 'inpelm, irule', E%npelm, B%rule
         write(irefwr,1) 'kstep', B%kstep
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x => B%xyz(:,1)
      xg => B%xyzg(:,1)
      jac => B%jac

      if ( E%first==0 ) call sepelp04first

!     --- Fill Gauss points

      call sepelpgauss

!     --- Compute Jacobian and derivatives of basis functions

      call sepelp04jacob ( curved )

!     --- Fill weights

      if ( B%cart==1 ) then

!     --- B%cart = 1  Cartesian co-ordinates

         B%w = B%wc*abs(jac(1:B%m))

      else if ( B%cart>=2 ) then

!     --- B%cart = 2  Axi-symmetric co-ordinates

         if ( B%rule==1 ) then

!        --- B%rule = 1  Newton-Cotes rule

            h = 1d0/120d0
            rc = x(1) + x(3) + x(5)
            do i = 1, 5, 2
               B%w(i) = h* ( 3d0*x(i)-rc ) * abs(jac(i))
            end do
            rc = 2d0*rc
            h  = 4d0*h
            B%w(2) = h * (rc-x(5)) * abs(jac(2))
            B%w(4) = h * (rc-x(1)) * abs(jac(4))
            B%w(6) = h * (rc-x(3)) * abs(jac(6))

         else

!        --- B%rule>1  Gauss rule

            B%w = abs(jac)*xg*B%wc

         end if  ! ( B%rule==1 )
         if ( B%cart==2 ) B%w = B%w*pi2

         if ( debug ) call prinrl ( B%w, B%m, 'w' )

      end if  ! ( B%cart==1 )

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w(1..m ) contains the weights
!         B%w2(1 .. m+n) the diagonal of the mass matrix
!         It may not be lumped, since the lumped mass matrix is singular

         if ( B%rule>2 ) then

!        --- B%rule>2 Gauss rule, use w and phi

            do i = 1, 6
               B%w2(i) = sum(B%w*B%phi(i,:)**2)
            end do  ! i = 1, 6

         else

!        --- B%rule<3, we have to use an approximation
!            Since this vector is only used for scaling,
!            it does not have to be very accurate

            if ( curved ) then
               h = sum(abs(jac))/m
            else
               h = abs(jac(1))
            end if  ! ( curved )
            h1 = h/60d0        ! vertices
            h2 = 4d0*h/45d0    ! mid points
            if ( B%cart==1 ) then

!           --- B%cart = 1, Cartesian coordinates

               B%w2(1:5:2) = h1
               B%w2(2:6:2) = h2

            else

!           --- B%cart = 2, Axi-symmetric coordinates

               B%w2(1:5:2) = h1*x(1:5:2)
               B%w2(2:6:2) = h2*x(2:6:2)

            end if  ! ( B%cart==1 )

         end if  ! ( B%rule>2 )
         if ( debug ) call prinrl ( B%w2(1), 6, 'diag mass matrix' )

      end if  ! ( itype>=900 .and. itype<=903 )

1000  call erclos ( 'sepelp04' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( B%phider(1,1,1), B%m, B%n, 'dphidx' )
         call prinrl1 ( B%phider(1,1,2), B%m, B%n, 'dphidy' )
         write(irefwr,*) 'End sepelp04'

      end if  ! ( debug )

      end subroutine sepelp04
