      subroutine sepelp04jac2d ( jac, dxdxi, m, kstep, dphidxi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the Jacobian of an element and store in jac
!     triangle (curved element)
!     2d
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
!     jacobian
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: m, kstep
      double precision, intent(in) :: dphidxi(B%n,m,2)
      double precision, intent(out) :: jac(m), dxdxi(m,3,3)

!     dphidxi        i    Contains values of dphi/dxi  in quadrature points
!     dxdxi          o    Contains the values of dx/dxsi
!     jac            o    Jacobian in the quadrature points
!     kstep          i    Step for quadrature points
!     m              i    number of quadrature points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ndim, ksign, jsign, i, j, k, n
      double precision, pointer :: x(:,:)

!     i              Counting variable
!     j              Counting variable
!     jsign          Help parameter to compute sign of the Jacobian
!     k              Counting variable
!     ksign          Help parameter to compute sign of the Jacobian
!     n              Number of basis functions
!     ndim           Dimension of space
!     x              Contains coordinates of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR0    Print error message errornr for element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp04jac2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp04jac2d'
         call prinrl2 ( B%xyz, B%n, E%ndim, 'x' )
         call prinrl2 ( dphidxi(1,1,1), B%n, B%m, 'dphi/dxi' )
         call prinrl2 ( dphidxi(1,1,2), B%n, B%m, 'dphi/deta' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndim = E%ndim
      n = B%n
      x => B%xyz

      do j = 1, ndim
         do i = kstep, m, kstep
            dxdxi(i,j,1) = dot_product(x(1:n,j),dphidxi(1:n,i,1))
            dxdxi(i,j,2) = dot_product(x(1:n,j),dphidxi(1:n,i,2))
         end do
      end do
      do k = kstep, m, kstep
         jac(k) = dxdxi(k,1,1)*dxdxi(k,2,2) - dxdxi(k,1,2)*dxdxi(k,2,1)
      end do

!     --- check sign of jacobian

      ksign = nint(sign ( 1d0,jac(kstep) ))
      do k = 2*kstep, m, kstep
         jsign = nint(sign ( 1d0, jac(k) ))
         if ( ksign/=jsign ) then
            call errorelvar0 ( 230 )
            go to 1000
         end if
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( jac, m, 'jac' )
         write(irefwr,*) 'End sepelp04jac2d'

      end if  ! ( debug )
      call erclos ( 'sepelp04jac2d' )

      end subroutine sepelp04jac2d
