      subroutine sepelp04jaccur
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for quadratic
!     triangle (curved element)
!     See old subroutine elp622curved
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
!     jacobian
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: m, k, i, kstep, n
      double precision :: det(B%n), e1(3,2)
      double precision, target :: dxdxi1(B%m,3,3), dxdxi2(B%n,3,3)
      double precision, pointer :: dphidxi(:,:,:), dphidx(:,:,:)
      double precision, pointer :: dxdxi(:,:,:), dpsidx(:,:,:), jac(:)

!     det            Contains the Jacobian for the nodes
!     dphidx         Contains the values of dphi/dx
!     dphidxi        Contains values of dphi/dxi in quadrature points
!     dpsidx         Contains values of dpsi/dx in quadrature points,
!                    with psi the linear basis functions
!     dxdxi          Contains the values of dx/dxsi
!     dxdxi1         Contains the values of dx/dxsi in quadrature points
!     dxdxi2         Contains the values of dx/dxsi in nodal points
!     e1             Work array of length 3 x 2, to store the factors
!                    e^ij See method
!     i              Counting variable
!     jac            Jacobian in the quadrature points
!     k              Counting variable
!     kstep          Step for quadrature points
!     m              Number of quadrature points
!     n              Number of basis functions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELP03JACOB  Compute Jacobian and derivatives of basis functions
!                    Linear triangle (3 nodes)
!     SEPELP04JAC2D  Compute the Jacobian of an element and store in jac
!                    triangle (curved element)
!     SEPELP3D2DDER  Compute the derivatives dphi/dx for a 2d element in 3d
!     SEPELP3D2DJAC  Compute the jacobian delta and dx/dxi for a 2d element
!                    in 3d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp04jaccur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp04jaccur'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dphidx => B%phider
      dphidxi => B%phiderc
      dxdxi => dxdxi1
      m = B%m
      n = B%n
      kstep = B%kstep
      dpsidx => B%lamder
      jac => B%jac

      if ( debug ) then
         write(irefwr,1) 'm, n, kstep, ndim', m, n, kstep, E%ndim
      end if  ! ( debug )

!        --- fill jacobian in jac

      if ( E%ndim>2 ) then

!     --- Standard 3D case

         call sepelp3d2djac

      else

!     --- Standard 2D case

         B%xloc => B%xyz
         call sepelp03jacob ( e1, jac )   ! just to fill lamder
         call sepelp04jac2d ( jac, dxdxi, m, kstep, dphidxi )

      end if  ! ( E%ndim>2 )

      if ( debug ) call prinrl ( jac, m, 'jac' )

      if ( B%fillderiv ) then

!     --- fillderiv = true, fill first derivatives

         if ( E%ndim>2 .and. (E%iqmat==0 .or. E%iqmat==3) ) then

!        --- Standard 3D case

            call sepelp3d2dder

         else

!        --- Standard 2D case

            if ( debug ) then

!           --- debug information

               call prinrl1 ( dphidxi(1,1,1), m, n, 'dphidxi' )
               call prinrl1 ( dphidxi(1,1,2), m, n, 'dpdeta' )

            end if  ! ( debug )

            do k = kstep, m, kstep
               do i = 1, n
                  dphidx(i,k,1) = ( dphidxi(i,k,1)*dxdxi(k,2,2) - &
                            dphidxi(i,k,2)*dxdxi(k,2,1) ) / jac(k)
                  dphidx(i,k,2) = (-dphidxi(i,k,1)*dxdxi(k,1,2) + &
                            dphidxi(i,k,2)*dxdxi(k,1,1) ) / jac(k)
               end do
            end do

         end if  ! ( E%ndim>2 )

      end if  ! ( B%fillderiv )

      if ( B%comppsi ) then

!     --- Compute linear basis functions

         do k = 1, 2
            dpsidx(1,:,k) = e1(1,k)
            dpsidx(3,:,k) = e1(3,k)
            dpsidx(2,:,k) = -e1(1,k) - e1(3,k)
         end do

      end if  ! ( B%comppsi )

      if ( B%fillderivn .and. B%rule>1 ) then

!     --- fillderivn = true, fill first derivatives in nodes

         dphidx => B%phidern
         dphidxi => B%phidercn
         dxdxi => dxdxi2
         call sepelp03jacob ( e1, det )   ! just to fill lamder
         call sepelp04jac2d ( det, dxdxi, n, 1, dphidxi )
         do k = 1, n
            do i = 1, n
               dphidx(i,k,1) = ( dphidxi(i,k,1)*dxdxi(k,2,2) - &
                         dphidxi(i,k,2)*dxdxi(k,2,1) ) / det(k)
               dphidx(i,k,2) = (-dphidxi(i,k,1)*dxdxi(k,1,2) + &
                         dphidxi(i,k,2)*dxdxi(k,1,1) ) / det(k)
            end do
         end do

      else if ( B%fillderivn ) then

!     --- fillderivn = true, fill first derivatives in nodes

         B%phidern = B%phider

      end if  ! ( B%fillderiv )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( dphidx(1,1,1), m, n, 'dphidx' )
         call prinrl1 ( dphidx(1,1,2), m, n, 'dphidy' )
         if ( E%ndim==3 ) call prinrl1 ( dphidx(1,1,3), m, n, 'dphidz' )
         write(irefwr,*) 'End sepelp04jaccur'

      end if  ! ( debug )
      call erclos ( 'sepelp04jaccur' )

      end subroutine sepelp04jaccur
