      subroutine sepelp05first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     Bi-linear quadrilateral (4 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: lambda(4), weight(3), h
      integer :: ms
      character (len=22), parameter :: text = 'Bilinear quadrilateral'

!     h              Help variable to store part of a weight
!     lambda         Contains values of lambda in quadrature points (1d)
!     ms             sqrt(m)
!     text           String variable containing some text
!     weight         Help array to store one-dimensional weights
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     ERRSUB         Error messages
!     SEPELPLAMBDA1D Fill the one-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1497   Type of integration rule not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp05first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp05first'
         write(irefwr,1) 'irule, m, n', B%rule, B%m, B%n
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( B%cart>2 ) call errorelvar ( B%cart-1, 1769, text, 0 )

!     --- Fill array xsi and eta for reference element (B%xyzgc)

      B%xyzgc = 0d0

      select case (B%rule)

      case (1)

!     --- Newton Cotes rule

         if ( B%m/=4 ) call errorelvar2 ( B%m, B%rule, 1424, text, 0 )
         if ( ierror/=0 ) go to 1000
         B%xyzgc(2:3,1) = 1d0
         B%xyzgc(3:4,2) = 1d0
         B%wc = 0.25d0

         B%jdiag = 1   ! basis functions satisfy phi_i(x_j) = delta_ij

      case(2)

!     --- intrule = 2  One point Gauss rule

         B%xyzgc(1,1:2) = 0.5d0
         B%wc(1) = 1d0

      case(3)

!        --- intrule = 3  Four point Gauss rule

         call sepelplambda1d ( lambda, text, B%rule, 2, weight )
         B%xyzgc(1:2,1) = lambda(1:2)
         B%xyzgc(3,1) = lambda(2)
         B%xyzgc(4,1) = lambda(1)
         B%xyzgc(1:2,2) = lambda(1)
         B%xyzgc(3:4,2) = lambda(2)
         B%wc(1:B%m) = 0.25d0

      case default

!     --- Quadrature not available

         call errint ( B%rule, 1 )
         call errsub ( 1497, 1, 0, 0)
         go to 1000

      end select  ! case (B%rule)

!     --- Fill basis functions in quadrature points

      B%phic(1,:) = (1d0-B%xyzgc(:,1))*(1d0-B%xyzgc(:,2))
      B%phic(2,:) = B%xyzgc(:,1)*(1d0-B%xyzgc(:,2))
      B%phic(3,:) = B%xyzgc(:,1)*B%xyzgc(:,2)
      B%phic(4,:) = (1d0-B%xyzgc(:,1))*B%xyzgc(:,2)
      B%phi = B%phic

!     --- Compute dphi/dxi, store in phiderc

      B%phiderc(1,:,1) = -(1d0-B%xyzgc(:,2))
      B%phiderc(2,:,1) =   1d0-B%xyzgc(:,2)
      B%phiderc(3,:,1) =       B%xyzgc(:,2)
      B%phiderc(4,:,1) =      -B%xyzgc(:,2)
      B%phiderc(1,:,2) = -(1d0-B%xyzgc(:,1))
      B%phiderc(2,:,2) =      -B%xyzgc(:,1)
      B%phiderc(3,:,2) =       B%xyzgc(:,1)
      B%phiderc(4,:,2) =   1d0-B%xyzgc(:,1)

      B%lamderc => B%phiderc
      B%lamder => B%phider

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp05first'

      end if  ! ( debug )
      call erclos ( 'sepelp05first' )

      end subroutine sepelp05first
