      subroutine sepelp06first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     Bi-quadratic quadrilateral (9 nodes)
!     Old routine: elp623first
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: indpks(9), indpet(9), i, k, indxi(9), indeta(9), m
      double precision :: theta(3,3), dtetds(3,3), s, h, lambda(3), wh(3)
      double precision, pointer :: weight(:), xg(:,:), phi(:,:), dphidxi(:,:,:)
      double precision, pointer :: psi(:,:), psixi(:,:,:)
      character (len=26), parameter :: text = 'Bi-quadratic quadrilateral'

!     dphidxi        Contains values of dphi/dxi  in quadrature points
!     dtetds         d theta_i / ds in integration points
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     indeta         eta index for node
!     indpet         etha index for integration point
!     indpks         xi index for integration point
!     indxi          xi index for node
!     k              Counting variable
!     lambda         Array containing the linear 1d basis functions in the
!                    integration points
!     m              number of quadrature points
!     phi            contains basis functions
!     psi            contains linear basis functions
!     psixi          Contains the values of the derivatives of the
!                    linear basis functions in the Gauss points in
!                    the reference element.
!     s              Mean value of derivative in element
!     text           String variable containing some text
!     theta          Array of length 3x3 containing the one-dimensional
!                    basis functions in the one-dimensional integration points
!     weight         Contains weights for reference element
!     wh             Contains weights for one-dimensional reference element
!     xg             coordinates of Gauss points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPELPLAMBDA1D Fill the one-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp06first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp06first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      weight => B%wc
      phi => B%phi
      psi => B%lam
      psixi => B%lamderc

      m = B%m

      if ( B%cart>2 ) call errorelvar ( B%cart-1, 1769, text, 0 )

      indxi = 1
      indxi(2) = 2
      indxi(3:5) = 3
      indxi(6) = 2
      indxi(9) = 2
      indeta = 1
      indeta(4) = 2
      indeta(5:7) = 3
      indeta(8:9) = 2

!     --- Fill array xsi and eta for reference element (B%xyzgc)

      B%xyzgc = 0d0

      if ( B%rule==1 .or. B%fillderivn ) then

!     --- Newton Cotes rule

         theta = 0d0
         do i = 1, 3
            theta(i,i) = 1d0
            s = (i-1) * 0.5d0
            dtetds(1,i) = 4d0 * s - 3d0
            dtetds(2,i) =-8d0 * s + 4d0
            dtetds(3,i) = 4d0 * s - 1d0
         end do
         indpet = indeta
         indpks = indxi

         if ( B%fillderivn ) then

!        --- Fill the arrays dphi/dxi and dphi/deta in the nodes

            dphidxi => B%phidercn
            xg => B%xyzgcn

            xg(:,1) = (indpks-1)*0.5d0
            xg(:,2) = (indpet-1)*0.5d0

            do k = 1, 9
               do i = 1, 9
                  dphidxi(i,k,2) = dtetds(indeta(i),indpet(k))* &
                                   theta(indxi(i),indpks(k))
                  dphidxi(i,k,1) = dtetds(indxi(i),indpks(k))* &
                                   theta(indeta(i),indpet(k))
               end do
            end do

         end if  ! ( B%fillderivn )

      end if  ! ( B%rule==1 .or. B%fillderivn )

      dphidxi => B%phiderc
      xg => B%xyzgc

      select case (B%rule)

      case (1)

!     --- Newton Cotes rule

         if ( m/=9 ) call errorelvar2 ( m, B%rule, 1424, text, 0 )
         if ( ierror/=0 ) go to 1000

         xg(:,1) = (indpks-1)*0.5d0
         xg(:,2) = (indpet-1)*0.5d0

         h = 1d0 / 36d0
         do k = 1, 7, 2
            weight(k) = h
         end do
         h = 4d0*h
         do k = 2, 8, 2
            weight(k) = h
         end do
         weight(9) = 4d0*h
         phi = 0d0
         do i = 1, 9
            phi(i,i) = 1d0
         end do
         if ( B%comppsi ) then

!        --- Compute linear basis functions

            psi = 0d0
            do i = 1, 4
               psi(i,(2*i-1)) = 1d0
               psi(i,2*i) = 0.5d0
               psi(i,9) = 0.25d0
            end do
            psi(1,8) = 0.5d0
            psi(2,2) = 0.5d0
            psi(3,4) = 0.5d0
            psi(4,6) = 0.5d0
            do k = 1, m
               psixi(1,k,1) = -(1d0-xg(k,2))
               psixi(2,k,1) = 1d0-xg(k,2)
               psixi(3,k,1) = xg(k,2)
               psixi(4,k,1) = -xg(k,2)
               psixi(1,k,2) = -(1d0-xg(k,1))
               psixi(2,k,2) = -xg(k,1)
               psixi(3,k,2) = xg(k,1)
               psixi(4,k,2) = 1d0-xg(k,1)
            end do

         end if  ! ( comppsi )

      case(2, 3)

!     --- intrule = 2  Four point Gauss rule

         call sepelplambda1d ( lambda, text, B%rule+1, B%rule, wh )
         do k = 1, B%rule
            theta(1,k) = 2d0 * (lambda(k)-0.5d0) * (lambda(k)-1d0)
            theta(2,k) =-4d0 * lambda(k) * (lambda(k)-1d0)
            theta(3,k) = 2d0 * lambda(k) * (lambda(k)-0.5d0)
            dtetds(1,k) = 4d0 * lambda(k) - 3d0
            dtetds(2,k) =-8d0 * lambda(k) + 4d0
            dtetds(3,k) = 4d0 * lambda(k) - 1d0
         end do

         if ( B%rule==2 ) then

!        --- intrule = 2  Four point Gauss rule

            do k = 1, 2
               indpks(k) = k
               indpks(k+2) = k
               indpet(k) = 1
               indpet(k+2) = 2
               xg(k,1) = lambda(k)
               xg(k+2,1) = lambda(k)
               xg(k,2) = lambda(1)
               xg(k+2,2) = lambda(2)
            end do
            weight(1:B%m) = 0.25d0

         else

!        --- intrule = 3  Nine point Gauss rule

            do k = 1, 9
               indpks(k) = indxi(k)
               indpet(k) = indeta(k)
               xg(k,1) = lambda(indpks(k))
               xg(k,2) = lambda(indpet(k))
            end do
            weight = wh(indxi) * wh(indeta)

         end if  ! ( B%rule==2 )
         if ( B%comppsi ) then

!        --- Compute linear basis functions

            do k = 1, m
               psi(1,k) = ( 1d0-xg(k,1) ) * ( 1d0-xg(k,2) )
               psi(2,k) = xg(k,1) * ( 1d0-xg(k,2) )
               psi(3,k) = xg(k,1) * xg(k,2)
               psi(4,k) = ( 1d0-xg(k,1) ) * xg(k,2)
               psixi(1,k,1) = -(1d0-xg(k,2))
               psixi(2,k,1) = 1d0-xg(k,2)
               psixi(3,k,1) = xg(k,2)
               psixi(4,k,1) = -xg(k,2)
               psixi(1,k,2) = -(1d0-xg(k,1))
               psixi(2,k,2) = -xg(k,1)
               psixi(3,k,2) = xg(k,1)
               psixi(4,k,2) = 1d0-xg(k,1)
            end do

         end if  ! ( comppsi )

      end select  ! case (intrule)

      if ( B%rule>1 ) then

         do i = 1, 9
            phi(i,1:B%m) = theta(indxi(i),indpks(1:B%m))* &
                       theta(indeta(i),indpet(1:B%m))
         end do  ! i = 1, 9

      end if  ! ( B%rule>2 )

!     --- Fill the arrays dphi/dxi and dphi/deta in the integration points

      do k = 1, m
         do i = 1, 9
            dphidxi(i,k,2) = dtetds(indeta(i),indpet(k))* &
                             theta(indxi(i),indpks(k))
            dphidxi(i,k,1) = dtetds(indxi(i),indpks(k))* &
                             theta(indeta(i),indpet(k))
         end do
      end do

1000  call erclos ( 'sepelp06first' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( B%wc, B%m, 'wc' )
         write(irefwr,*) 'End sepelp06first'

      end if  ! ( debug )

      end subroutine sepelp06first
