      subroutine sepelp10
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     Extended linear triangle (4 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: e1(3,2), delta, h, rc, gradc
      double precision, pointer :: x(:), xg(:), lambda(:,:), dphidx(:,:,:)
      double precision, pointer :: phi(:,:)
      integer :: k, i, j

!     delta          Jacobian of the element
!     dphidx         Contains values of dphi/dx in quadrature points
!     e1             Work array of length 3 x 2, to store the factors
!                    e^ij See method
!     gradc          Gradient of lambda_1 * lambda_2 * lambda_3
!     h              Help variable to store part of a weight
!     i              Counting variable
!     j              Counting variable
!     k              Loop variable
!     lambda         Contains values of lambda in quadrature points
!     phi            contains basis functions
!     rc             Help variable to store part of a weight for axisymmetric
!                    co-ordinates
!     x              x-coordinates of nodal points
!     xg             x-coordinates of Gauss points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELP03JACOB  Compute Jacobian and derivatives of basis functions
!                    Linear triangle (3 nodes)
!     SEPELP10FIRST  compute some element independent quantities
!                    Extended linear triangle (4 nodes)
!     SEPELPGAUSS    Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!         ij        i+1    i+2    i+1
!        e   =  (-1)    ( x    - x    )   cyclic: i=1,2,3 ;   j=1,2
!                          j      j
!                    32  11    12 31
!        delta   =  e   e   - e  e
!                          ij
!        d phi  / dx   =  e   /  delta
!             i     j
!                                        |delta|       1     2     3
!        Newton Cotes rule:  / f(x) dx = -------- [ f(x )+f(x )+f(x ) ]
!                            e              6
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp10' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp10'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x  => B%xyz(:,1)
      xg => B%xyzg(:,1)
      dphidx => B%phider
      phi => B%phi
      lambda => B%lam

      m = B%m

      if ( E%first==0 ) call sepelp10first

      if ( E%npelm==3 ) &
         B%xyz(4,:) = ( B%xyz(1,:)+B%xyz(2,:)+B%xyz(3,:) ) / 3d0

!     --- Fill Gauss points

      call sepelpgauss

!     --- Compute Jacobian and derivatives of basis functions

      B%xloc => B%xyz
      call sepelp03jacob ( e1, delta )

!     --- Fill weights

      if ( B%rule==1 .and. B%cart==2 ) then

!     --- jcart = 2  Axi-symmetric co-ordinates

         h = abs ( delta ) / 24d0
         rc = sum(x)
         B%w = h*(x+rc) * pi2

      else

!     --- Standard case

         B%w = abs(delta)*B%wc
         if ( B%cart==2 ) B%w = B%w * xg * pi2  ! axisymmetric

      end if  ! ( intrule==1 .and. jcartorg==2 )

      if ( debug ) call prinrl ( B%w, B%m, 'weights' )

      if ( B%fillderiv ) then

!     --- fillderiv = true, fill first derivatives

         if ( B%rule==1 ) then

!        --- Newton Cotes rule

            do i = 1, 4
               do k = 1, 2
                  dphidx(1:3,i,k) = e1(1:3,k)
               end do
               dphidx(4,i,1:2) = 0d0
            end do  !  i = 1, 4

         else

!        --- Gauss rule

            do j = 1, 2
               do k = 1, m
                  gradc = lambda(2,k)*lambda(3,k)*e1(1,j) + &
                          lambda(3,k)*lambda(1,k)*e1(2,j) + &
                          lambda(1,k)*lambda(2,k)*e1(3,j)
                  do i = 1, 3
                     dphidx(i,k,j) = e1(i,j)-9d0*gradc
                  end do
                  dphidx(4,k,j) = 27d0*gradc
               end do  ! k = 1, m
            end do  ! j = 1, 2

         end if  ! ( B%rule==1 )

      end if  ! ( B%fillderiv )

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w contains the weights
!         w2 the diagonal of the mass matrix

         do i = 1, B%n
            B%w2(i) = sum(B%w*B%phi(i,:)**2)
         end do  ! i = 1, B%n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( dphidx(1,1,1), m, 4, 'dphi/dx' )
         call prinrl1 ( dphidx(1,1,2), m, 4, 'dphi/dy' )
         write(irefwr,*) 'End sepelp10'

      end if  ! ( debug )
      call erclos ( 'sepelp10' )

      end subroutine sepelp10
