      subroutine sepelp12
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     Quadratic tetrahedrons
!     Old subroutine elp631
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     tetrahedron
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer intern(10), imidp(6,2)
      double precision :: jac(B%m)
      integer :: i

!     imidp          Contains the i,j numbering of the mid side points in the
!                    sequence of intern
!                    So basis function
!                    phi(intern(4+i)) = 4 lambda(imidp(i,1)) lambda(imidp(i,2))
!     intern         Internal numbering of the nodes, vertices first
!     jac            Array of length m containing the values of the
!                    Jacobian in the vertices or for the Gauss integration
!                    rules work array to store some help variables
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELP12FIRST  compute some element independent quantities
!                    Quadratic tetrahedrons
!     SEPELP12JACOB  Compute Jacobian and derivatives of basis functions
!                    Quadratic tetrahedrons
!     SEPELPGAUSS   Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!             ij     i+1     i
!        delta   =  x    -  x       cyclic;  i = 1,2,3,4;  j =  1,2,3
!                    j       j
!         ij        i        i+1,j+1     i+2,j+2        i+1,j+2     i+2,j+1
!        e   =  (-1)  [ delta       delta        - delta       delta        ]
!                        11  11        12  12       13  13
!        djacob = - delta   e   - delta   e  - delta   e
!                             ij
!        d lambda  / dx   =  e   /  djacob
!                i     j
!        lambda ( x,y,z ) is linear basis function corresponding to
!                         4-point tetrahedron
!        phi    = lambda  ( 2 lambda  - 1 )   i = 1,2,3,4  (vertices)
!           i            i           i
!        phi   = 4 lambda  lambda     1<=i<j<=4   (midside points)
!           ij           i       j
!        grad phi  = grad lambda  ( 4 lambda  - 1 )   i = 1,2,3,4  (vertices)
!                i               i           i
!        grad phi   = 4 [ grad(lambda )lambda  + grad(lambda )lambda ]
!                ij                  i       j              j       i
!                  1<=i<j<=4   (midside points)
!                                        |djacob|      1     2     3     4
!        Newton Cotes rule:  / f(x) dx = -------- [-f(x )-f(x )-f(x )-f(x ) ]
!                            e             120
!            |djacob|      5     6     7     8     9     10
!        + 4 -------- [ f(x )+f(x )+f(x )+f(x )+f(x )+f(x  ) ]
!              120
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data intern / 1, 3, 5, 10, 2, 6, 7, 4, 8, 9 /
      data imidp  / 1, 1, 1, 2, 2, 3, &
                    2, 3, 4, 3, 4, 4 /
! ======================================================================
!
      call eropen ( 'sepelp12' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp12'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%first==0 ) call sepelp12first ( intern, imidp )

!     --- Fill Gauss points

      call sepelpgauss

!     --- Compute Jacobian and derivatives of basis functions

      call sepelp12jacob ( jac, intern, imidp )

!     --- Fill weights

      B%w = B%wc*abs(jac(1:B%m))

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w contains the weights
!         w2 the diagonal of the mass matrix

         do i = 1, B%n
            B%w2(i) = sum(B%w*B%phi(i,:)**2)
         end do  ! i = 1, B%n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  call erclos ( 'sepelp12' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp12'

      end if  ! ( debug )

      end subroutine sepelp12
