      subroutine sepelp13jacob1 ( m, n, w, phixi, dphidx, detinv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute dphidx and w, defining the Jacobian of the element
!     Quadratic tetrahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     tetrahedron
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: m, n
      double precision, intent(in) :: phixi(n,m,3)
      double precision, intent(out) :: w(m), dphidx(n,m,3), detinv(m,3,3)

!     detinv         o    Work array of length m x 3 x 3, to store the inverse
!                         of the jacobian matrix as follows:
!                          -1    k
!                         J   ( x ) =  detinv ( k, i, j )
!                          ij
!     dphidx         o    Contains values of dphi/dx in quadrature points
!     m              i    number of quadrature points
!     n              i    number of basis functions
!     phixi          i    Array of length n x m x 3 containing the values of the
!                         derivatives of the basis functions in the Gauss points in
!                         the reference element.
!     w              o    Array of length m containing the values of the
!                         Jacobian in the vertices or for the Gauss integration
!                         rules work array to store some help variables
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k, ksign, jsign
      double precision :: dxdxi(m,3,3)
      double precision, pointer :: x(:,:)
      logical :: check

!     check          Indicates if at least one component of w is negative
!     dxdxi          d x_i / d xsi integration points
!     i              Counting variable
!     j              Counting variable
!     jsign          Help parameter to compute sign of the Jacobian
!     k              Counting variable
!     ksign          Help parameter to compute sign of the Jacobian
!     x              coordinates of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
!     230   Jacobian of element is zero or has changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp13jacob1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp13jacob1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x => B%xyz
      do j = 1, 3
         do i = 1, m
            do k = 1, 3
               dxdxi(i,j,k) = dot_product ( x(:,k), phixi(:,i,j) )
            end do  ! k = 1, 3
         end do  ! i = 1, m
      end do  ! j = 1, 3

      if ( debug ) then
         call prinrl ( dxdxi(1,1,1), m, 'dx/dx(1,1)' )
         call prinrl ( dxdxi(1,2,1), m, 'dx/dx(2,1)' )
         call prinrl ( dxdxi(1,3,1), m, 'dx/dx(3,1)' )
         call prinrl ( dxdxi(1,1,2), m, 'dx/dx(1,2)' )
         call prinrl ( dxdxi(1,2,2), m, 'dx/dx(2,2)' )
         call prinrl ( dxdxi(1,3,2), m, 'dx/dx(3,2)' )
         call prinrl ( dxdxi(1,1,3), m, 'dx/dx(1,3)' )
         call prinrl ( dxdxi(1,2,3), m, 'dx/dx(2,3)' )
         call prinrl ( dxdxi(1,3,3), m, 'dx/dx(3,3)' )
      end if  ! ( debug )

      detinv(:,1,1) = dxdxi(:,2,2)*dxdxi(:,3,3)-dxdxi(:,3,2)*dxdxi(:,2,3)
      detinv(:,2,1) = dxdxi(:,3,2)*dxdxi(:,1,3)-dxdxi(:,1,2)*dxdxi(:,3,3)
      detinv(:,3,1) = dxdxi(:,1,2)*dxdxi(:,2,3)-dxdxi(:,2,2)*dxdxi(:,1,3)
      w = dxdxi(:,1,1)*detinv(:,1,1)+dxdxi(:,2,1)* &
          detinv(:,2,1)+dxdxi(:,3,1)*detinv(:,3,1)

      if ( B%fillderiv ) then

!     --- fillderiv true,  compute all factors of inverse

         detinv(:,1,2) = dxdxi(:,2,3)*dxdxi(:,3,1)-dxdxi(:,3,3)*dxdxi(:,2,1)
         detinv(:,2,2) = dxdxi(:,3,3)*dxdxi(:,1,1)-dxdxi(:,1,3)*dxdxi(:,3,1)
         detinv(:,3,2) = dxdxi(:,1,3)*dxdxi(:,2,1)-dxdxi(:,2,3)*dxdxi(:,1,1)
         detinv(:,1,3) = dxdxi(:,2,1)*dxdxi(:,3,2)-dxdxi(:,3,1)*dxdxi(:,2,2)
         detinv(:,2,3) = dxdxi(:,3,1)*dxdxi(:,1,2)-dxdxi(:,1,1)*dxdxi(:,3,2)
         detinv(:,3,3) = dxdxi(:,1,1)*dxdxi(:,2,2)-dxdxi(:,2,1)*dxdxi(:,1,2)

      end if

      if ( debug ) then
         call prinrl ( w, m, 'w' )
         call prinrl ( detinv(1,1,1), m, 'detinv_11' )
         call prinrl ( detinv(1,2,1), m, 'detinv_21' )
         call prinrl ( detinv(1,3,1), m, 'detinv_31' )
         call prinrl ( detinv(1,1,2), m, 'detinv_12' )
         call prinrl ( detinv(1,2,2), m, 'detinv_22' )
         call prinrl ( detinv(1,3,3), m, 'detinv_32' )
         call prinrl ( detinv(1,1,3), m, 'detinv_13' )
         call prinrl ( detinv(1,2,3), m, 'detinv_23' )
         call prinrl ( detinv(1,3,3), m, 'detinv_33' )
      end if  ! ( debug )

!     --- check sign of jacobian

      ksign = nint( sign (1d0, w(1)) )
      if ( checksign .and. ksign<0 ) then
         write(irefwr,*) 'area of element ', E%ielem, ' is negative'
      end if
      check = .false.
      do i = 1, m
         if ( abs(w(i))<=0d0 ) check = .true.
         jsign = nint(sign(1d0, w(i)))
         if( jsign/=ksign ) then
            call errint ( E%ielem, 1 )
            call errint ( E%itype, 2 )
            call errint ( E%ielgrp, 3 )
            call errsub ( 230, 3, 0, 0 )
            call prinin(E%nodes,n,'node numbers')
            call prinrl2(x,n,3,'co-ordinates')
            go to 1000
         end if
      end do
      if ( check ) then
         call errint ( E%ielem, 1 )
         call errint ( E%itype, 2 )
         call errint ( E%ielgrp, 3 )
         call errsub ( 105, 3, 0, 0 )
         call prinin(E%nodes,n,'node numbers')
         call prinrl2(x,n,3,'co-ordinates')
         go to 1000
      end if

      if ( B%fillderiv ) then

!     --- fillderiv true,  compute all factors of inverse

         do i = 1, n
            do k = 1, m
               dphidx(i,k,1:3) = (phixi(i,k,1)*detinv(k,1,1:3)+ &
                                  phixi(i,k,2)*detinv(k,2,1:3)+ &
                                  phixi(i,k,3)*detinv(k,3,1:3))/w(k)
            end do
         end do

      end if  ! ( B%fillderiv )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( dphidx(1,1,1), m, n, 'dphi/dx' )
         call prinrl1 ( dphidx(1,1,2), m, n, 'dphi/dy' )
         call prinrl1 ( dphidx(1,1,3), m, n, 'dphi/dz' )
         write(irefwr,*) 'End sepelp13jacob1'

      end if  ! ( debug )
      call erclos ( 'sepelp13jacob1' )

      end subroutine sepelp13jacob1
