      subroutine sepelp16
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     Extended quadratic tetrahedron (15 nodes)
!     Old subroutine elp637
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     tetrahedron
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer intern(15), imidp(6,2), i
      double precision :: jac(B%m)

!     imidp          Contains the i,j numbering of the mid side points in the
!                    sequence of intern
!     intern         Internal numbering of the nodes, vertices first
!     jac            Jacobian in the quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELP13JACOB  Compute Jacobian and derivatives of basis functions
!                    3d Tri-linear hexahedron (8 nodes)
!     SEPELP16FIRST  compute some element independent quantities
!                    Extended tri-Quadratic tetrahedron (15 nodes)
!     SEPELPGAUSS    Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!             ij     i+1     i
!        delta   =  x    -  x       cyclic;  i = 1,2,3,4;  j =  1,2,3
!                    j       j
!         ij        i        i+1,j+1     i+2,j+2        i+1,j+2     i+2,j+1
!        e   =  (-1)  [ delta       delta        - delta       delta        ]
!                        11  11        12  12       13  13
!        djacob = - delta   e   - delta   e  - delta   e
!                             ij
!        d lambda  / dx   =  e   /  djacob
!                i     j
!        lambda ( x,y,z ) is linear basis function corresponding to
!                         4-point tetrahedron
!
!        lamc = lambda lambda lambda lambda
!                     1      2      3      4
!
!        --- for vertices i = 1,2,3,4 : by permutation
!
!        phi    = lambda  ( 2 lambda  - 1 ) +3 (lambda lambda lambda +
!           i           i           i                 i      j      k
!       +         lambda lambda lambda + lambda lambda lambda ) -
!                       k      l      i        l      i      j
!       +         4 lamc
!
!        --- instead of lambda  ( 2 lambda  - 1 ) also the following
!                              i           i
!            expression can be used:
!            (--- indices by cyclic permutation ---)
!
!            lambda - 2 lambda (lambda +lambda +lambda )
!                  i          i       j       k       l
!
!            because of the relationship:
!
!            lambda + lambda + lambda + lambda = 1
!                  1        2        3        4
!
!        --- for midside points : 1<=i<j<=4
!
!        phi   = 4 lambda  lambda  - 12 ( lambda lambda lambda +
!           ij           i       j              i      j      k
!       +        lambda lambda lamdba ) + 32 lamc
!                      i      j      l
!
!        --- for midface points : 1<=i<j<k<l<=4
!
!        phi   = 27 lambda lambda lambda  - 108 lamc
!           ijk           i      j      k
!
!        --- centroid tetrahedron
!
!        phi    = 256 lamc
!           c
!
!        --- Integration-rule: Gauss quadrature
!
!        * Gauss rule: / f(x) dx = |djacob| * SUM [ w  f(xg )]
!                      e                             i     i
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data intern / 1, 3, 5, 10, 2, 6, 7, 4, 8, &
                    9, 11, 12, 13, 14, 15/, &
           imidp  / 1, 1, 1, 2, 2, 3, &
                    2, 3, 4, 3, 4, 4 /
! ======================================================================
!
      call eropen ( 'sepelp16' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp16'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%first==0 ) call sepelp16first ( intern, imidp )

!     --- Fill Gauss points

      call sepelpgauss

!     --- Compute Jacobian and derivatives of basis functions

      call sepelp13jacob ( jac )

!     --- Fill weights

      B%w = B%wc*abs(jac(1:B%m))

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w contains the weights
!         w2 the diagonal of the mass matrix

         do i = 1, B%n
            B%w2(i) = sum(B%w*B%phi(i,:)**2)
         end do  ! i = 1, B%n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp16'

      end if  ! ( debug )
      call erclos ( 'sepelp16' )

      end subroutine sepelp16
