      subroutine sepelp16first ( intern, imidp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     Extended quadratic tetrahedron (15 nodes)
!     Old subroutine elp637
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     tetrahedron
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: intern(15), imidp(6,2)

!     imidp          i    Contains the i,j numbering of the mid side points in
!                         the sequence of intern
!                         So basis function
!                         phi(intern(4+i)) = 4 lambda(imidp(i,1))
!                                              lambda(imidp(i,2))
!     intern         i    Internal numbering of the nodes, vertices first
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: m, i, k, irule, j
      double precision :: h, dlamdxi(4,3), l123, l124, l134, l234, lamc
      double precision, pointer :: phi(:,:), lambda(:,:), wh(:)
      double precision, pointer :: phixi(:,:,:)
      character (len=30), parameter :: text = 'Extended quadratic tetrahedron'

!     dlamdxi        Contains the derivatives of the linear basis functions
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     irule          type of quadrature rule
!     j              Counting variable
!     k              Counting variable
!     l123           Help variable corresponding to baricentric coordinate 123
!     l124           Help variable corresponding to baricentric coordinate 124
!     l134           Help variable corresponding to baricentric coordinate 134
!     l234           Help variable corresponding to baricentric coordinate 234
!     lambda         Array containing the values of lambda in the integration
!                    points  lambda(i,k) = lambda_i(xk)
!     lamc           Help variable. See pseudo code.
!     m              number of quadrature points
!     phi            contains basis functions
!     phixi          Array of length 10 x m x 3 containing the values of the
!                    derivatives of the basis functions in the Gauss points in
!                    the reference element
!     text           String variable containing some text
!     wh             Contains the weights in the reference element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     SEPELP16PHIXI  Compute dphi/dxi in quadrature points
!                    Extended tri-Quadratic tetrahedron (15 nodes)
!     SEPELPLAMBDA3D Fill the three-dimensional linear basis functions in case
!                    of a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp16first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp16first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phi => B%phi
      phixi => B%phiderc
      wh => B%wc

      m = B%m

      if ( B%cart>1 ) call errorelvar ( B%cart-1, 1769, text, 0 )

      if ( B%rule==1 .or. B%fillderivn ) then

!     --- Fill lambdanc for Newton Cote rule

         lambda => B%lamn
         lambda = 0d0
         h = 1d0/3d0
         lambda(:,11:14) = h
         lambda(:,15) = 0.25d0
         do i = 1, 4
            lambda(i,intern(i)) = 1d0
            do k = 1, 6
               if ( imidp(k,1)==i .or. imidp(k,2)==i ) then
                  lambda(i,intern(k+4)) = 0.5d0
               end if
            end do  ! k = 1, 6
         end do  ! i = 1, 4
         lambda(1,13) = 0d0
         lambda(2,14) = 0d0
         lambda(3,12) = 0d0
         lambda(4,11) = 0d0

      end if  ! ( B%rule==1 .or. B%fillderivn )

      lambda => B%lam

!     --- Fill array lambda, phi

      select case ( B%rule )

      case(1)

!     --- intrule = 1  Newton Cotes rule

         lambda = B%lamn
         phi = 0d0
         do i = 1, 15
            phi(i,i) = 1d0
         end do

         h = 1d0/5040d0
         wh(intern(1:4)) = 17d0*h
         wh(intern(5:10)) = 32d0*h
         wh(11:14) = 81d0*h
         wh(15) = 256d0*h

      case(2, 3)

!     --- 11, 15 point Gauss rule; just copy the lambdas

         irule = B%rule  ! save rule
         B%rule = irule+3  !  11 or 16-point Gauss rule
         call sepelplambda3d ( lambda, text )
         B%rule = irule  ! reset rule

      end select  ! case ( intrule )

      if ( B%rule>1 ) then

!     --- Fill array phi_i

         do i = 1, m

            lamc = lambda(1,i)*lambda(2,i)*lambda(3,i)*lambda(4,i)
            L123 = lambda(1,i)*lambda(2,i)*lambda(3,i)
            L124 = lambda(1,i)*lambda(2,i)*lambda(4,i)
            L134 = lambda(1,i)*lambda(3,i)*lambda(4,i)
            L234 = lambda(2,i)*lambda(3,i)*lambda(4,i)

            phi(1,i) = lambda(1,i)-2d0*lambda(1,i)*(lambda(2,i) &
                       +lambda(3,i)+lambda(4,i)) &
                       +3d0*(L123 + L124 + L134) - 4d0*lamc
            phi(2,i) = 4d0*lambda(1,i)*lambda(2,i) &
                       -12d0*(L123 + L124) + 32d0*lamc
            phi(3,i) = lambda(2,i)-2d0*lambda(2,i)*(lambda(1,i) &
                       +lambda(3,i)+lambda(4,i)) &
                       +3d0*(L123 + L124 + L234) - 4d0*lamc
            phi(4,i) = 4d0*lambda(2,i)*lambda(3,i) &
                       -12d0*(L123 + L234) + 32d0*lamc
            phi(5,i) = lambda(3,i)-2d0*lambda(3,i)*(lambda(1,i) &
                       +lambda(2,i)+lambda(4,i)) &
                       +3d0*(L123 + L134 + L234) - 4d0*lamc
            phi(6,i) = 4d0*lambda(1,i)*lambda(3,i) &
                       -12d0*(L123 + L134) + 32d0*lamc
            phi(7,i) = 4d0*lambda(1,i)*lambda(4,i) &
                       -12d0*(L124 + L134) + 32d0*lamc
            phi(8,i) = 4d0*lambda(2,i)*lambda(4,i) &
                       -12d0*(L124 + L234) + 32d0*lamc
            phi(9,i) = 4d0*lambda(3,i)*lambda(4,i) &
                       -12d0*(L134 + L234) + 32d0*lamc
            phi(10,i)= lambda(4,i)-2d0*lambda(4,i)*(lambda(1,i) &
                       +lambda(2,i)+lambda(3,i)) &
                       +3d0*(L234 + L124 + L134) - 4d0*lamc
            phi(11,i)= 27d0*L123 - 108d0*lamc
            phi(12,i)= 27d0*L124 - 108d0*lamc
            phi(13,i)= 27d0*L234 - 108d0*lamc
            phi(14,i)= 27d0*L134 - 108d0*lamc
            phi(15,i)= 256d0*lamc
         end do

      end if  ! ( intrule>1 )

!     --- fill d phi / d xsi,  d phi / d eta and dphi d zeta
!         first d lambda/ dxi

      dlamdxi(1,1:3) = -1d0
      dlamdxi(2:4,1:3) = 0d0
      do i = 1, 3
         dlamdxi(i+1,i) = 1d0
      end do  ! i = 1, 3

!     --- Compute dphi/dxi

      call sepelp16phixi ( B%phiderc, B%lam )

      do i = 1, m
         B%lamderc(1:4,i,1:3) = dlamdxi(1:4,1:3)
      end do  ! i = 1, m

      if ( B%fillderivn ) then

!     --- Fill phidercn

         if ( B%rule==1 ) then

!        --- Newton Cotes rule

            B%phidercn = B%phiderc

         else

!        --- Gauss rule

            call sepelp16phixi ( B%phidercn, B%lamn )

         end if  ! ( B%rule==1 )

      end if  ! ( B%fillderivn )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp16first'

      end if  ! ( debug )
      call erclos ( 'sepelp16first' )

      end subroutine sepelp16first
