      subroutine sepelplambda2d ( lambda, text, irule, m, weight )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the two-dimensional linear basis functions in case of
!     a non-Newton Cotes rule
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: irule, m
      double precision lambda(3,m), weight(m)
      character (len=*) text

!     irule          i    Type of quadrature rule
!     lambda         o    Array containing the linear basis functions in the
!                         integration points
!     m              i    Number of quadrature points
!     text           i    String variable containing some text
!     weight         o    Weights for quadrature
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, ind(4), i
      double precision :: d

!     d              help parameter to compute a constant
!     i              Counting variable
!     ind            index array to define cyclic permutation for three numbers
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelplambda2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelplambda2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( irule )

         case(2)

!        --- irule = 2  one-point Gauss rule

            if ( m/=1 ) call errorelvar2 ( m, irule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000
            d = 1d0/3d0
            lambda(:,1) = d
            weight(1) = 0.5d0

         case(3)

!        --- irule = 3  four-point Gauss rule

            if ( m/=4 ) call errorelvar2 ( m, irule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000
            d = 1d0/3d0
            lambda(1:3,2:4) = .2d0
            do i = 1, 3
               lambda(i,1) = d
               lambda(i,1+i) = .6d0
            end do
            weight(1) = -.28125d0
            weight(2:4) = .2604166666666667d0

         case(4)

!        --- irule = 4  seven-point Gauss rule

            if ( m/=7 ) call errorelvar2 ( m, irule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000
            d = 1d0/3d0
            do i = 1, 2
               ind(i) = i+1
               ind(i+2) = i
            end do  ! i = 1, 3
            do i = 1, 3
               lambda(i,1) = d
               lambda(i,1+i) = 0.059715871789770d0
               lambda(i,1+ind(i)) = 0.470142064105115d0
               lambda(i,1+ind(i+1)) = 0.470142064105115d0
               lambda(i,4+i) = 0.797426985353087d0
               lambda(i,4+ind(i)) = 0.101286507323456d0
               lambda(i,4+ind(i+1)) = 0.101286507323456d0
            end do
            weight(1) = 0.1125d0
            weight(2:4) = 0.066197076394253d0
            weight(5:7) = 0.0629695902724135d0

         case(5)

!        --- irule = 5 second order Newton Cotes rule

            if ( m/=3 ) call errorelvar2 ( m, irule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000
            lambda(:,1:3) = 0.5d0
            lambda(1,2) = 0d0
            lambda(2,3) = 0d0
            lambda(3,1) = 0d0
            d = 1d0/6d0
            weight = d

         case default

!        --- Error value of irule not implemented

            call errorelvar ( irule, 1770, text, 1 )

      end select  ! case ( irule )

1000  call erclos ( 'sepelplambda2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelplambda2d'

      end if  ! ( debug )

      end subroutine sepelplambda2d
