      subroutine sepelplambda3d ( lambda, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the two-dimensional linear basis functions in case of
!     a non-Newton Cotes rule
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision lambda(4,B%m)
      character (len=*) text

!     lambda         o    Array containing the linear basis functions in the
!                         integration points
!     text           i    String variable containing some text
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ind1(6), ind2(6)
      double precision :: h, h1, h2, h3, alpha, beta, gamma, delta

!     alpha          constant
!     beta           constant
!     delta          constant
!     gamma          constant
!     h              help parameter to compute a constant
!     h              help parameter to compute a constant
!     h1             help variable to store a quantity
!     h2             help variable to store a quantity
!     h3             help variable to store a quantity
!     i              Counting variable
!     ind1           index array to define cyclic permutation for four numbers
!     ind2           index array to define cyclic permutation for four numbers
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1424   m does not correspond to irule
!    1770   Type of integration rule incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelplambda3d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelplambda3d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( B%rule )

         case(2)

!        --- B%rule = 2  one-point Gauss rule

            if ( B%m/=1 ) call errorelvar2 ( B%m, B%rule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000
            lambda(:,1) = 0.25d0
            B%wc(1) = 1d0/6d0

         case(3)

!        --- B%rule = 3  four-point Gauss rule

            if ( B%m/=4 ) call errorelvar2 ( B%m, B%rule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000
            h = 1d0/24d0
            B%wc = h
            h1 = (5d0-sqrt(5d0))*0.05d0
            lambda = h1
            h2 = (5d0+3d0*sqrt(5d0))*0.05d0
            do i = 1, 4
               lambda(i,i)         = h2
            end do

         case(4)

!        --- B%rule = 4: five-point Gauss rule

            if ( B%m/=5 ) call errorelvar2 ( B%m, B%rule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000
            h = 3d0/40d0
            B%wc(1) = -2d0/15d0
            B%wc(2:5) = h
            h = 1d0/6d0
            lambda = h
            lambda(1:4,1) = 0.25d0
            do i = 1, 4
               lambda(i,1+i) = 0.5d0
            end do

         case(5)

!        --- B%rule = 5: 11 point Gauss rule

            if ( B%m/=11 ) call errorelvar2 ( B%m, B%rule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000
            h = -13155555555556d-15
            h1 =  7622222222222d-15
            h2 = 24888888888889d-15

            B%wc(1) = h
            B%wc(2:5) = h1
            B%wc(6:11) = h2

!           --- Fill array with barycentric coordinates of the gauss-points
!               according to lambda(i,k)=lambda_i (x_k )

            alpha = 785714285714286d-15
            beta  = 71428571428571d-15
            gamma = 399403576166799d-15
            delta = 100596423833201d-15

            ind2(1) = 3
            ind2(5) = 1
            ind2(6) = 2
            do i = 1, 3
               ind1(i) = i+1
               ind1(i+3) = i
               ind2(i+1) = i
            end do  ! i = 1, 3

            do i = 1, 4
               lambda(i,1)           = 0.25d0
               lambda(i,1+i)         = alpha
               lambda(i,1+ind1(i))   = beta
               lambda(i,1+ind1(i+1)) = beta
               lambda(i,1+ind1(i+2)) = beta
               if ( i==1 ) then
                  lambda(1,6:8)   = gamma
                  lambda(i,9:11) = delta
               else
                  lambda(i,4+i)         = gamma
                  lambda(i,8+ind2(1+i)) = gamma
                  lambda(i,8+ind2(2+i)) = gamma
                  lambda(i,7+i)         = delta
                  lambda(i,5+ind2(1+i)) = delta
                  lambda(i,5+ind2(2+i)) = delta
               end if
            end do

         case(6)

!        --- B%rule = 6: 15 point Gauss rule

            if ( B%m/=15 ) call errorelvar2 ( B%m, B%rule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000

            h  = 30283678097089d-15
            h1 = 6026785714286d-15
            h2 = 11645249086029d-15
            h3 = 10949141561386d-15

            B%wc(1) = h
            B%wc(2:5) = h1
            B%wc(6:9) = h2
            B%wc(10:15) = h3

!           --- Fill array with barycentric coordinates of the gauss-points
!               according to lambda(i,k)=lambda_i(x_k)

            alpha = 66550153573664d-15
            beta  = 433449846426336d-15
            do i = 1, 3
               ind1(i) = i+1
               ind1(i+3) = i
            end do  ! i = 1, 3

            do i = 1, 4
               lambda(i,1)           = 0.25d0
               lambda(i,1+i)         = 0d0
               lambda(i,1+ind1(i))   = 1d0/3d0
               lambda(i,1+ind1(i+1)) = 1d0/3d0
               lambda(i,1+ind1(i+2)) = 1d0/3d0
               lambda(i,5+i)         = 8d0/11d0
               lambda(i,5+ind1(i))   = 1d0/11d0
               lambda(i,5+ind1(i+1)) = 1d0/11d0
               lambda(i,5+ind1(i+2)) = 1d0/11d0

               lambda(i,9+i) = alpha
               lambda(i,9+ind1(i)) = alpha
               lambda(i,9+ind1(i+1)) = alpha
               lambda(i,11+ind1(i)) = beta
               lambda(i,11+ind1(i+1)) = beta
               lambda(i,11+ind1(i+2)) = beta
            end do
            lambda(2,10)=lambda(1,10)
            lambda(2,14)=lambda(1,10)
            lambda(2,15)=lambda(1,10)
            lambda(2,11)=lambda(1,15)
            lambda(2,12)=lambda(1,15)
            lambda(2,13)=lambda(1,15)
            lambda(3,10)=lambda(1,15)
            lambda(3,12)=lambda(1,15)
            lambda(3,14)=lambda(1,15)
            lambda(3,11)=lambda(1,10)
            lambda(3,13)=lambda(1,10)
            lambda(3,15)=lambda(1,10)
            lambda(4,10)=lambda(1,15)
            lambda(4,11)=lambda(1,15)
            lambda(4,15)=lambda(1,15)
            lambda(4,12)=lambda(1,10)
            lambda(4,13)=lambda(1,10)
            lambda(4,14)=lambda(1,10)

         case default

!        --- Error value of B%rule not implemented

            call errorelvar ( B%rule, 1770, text, 1 )
            B%rule = 1

      end select  ! case ( B%rule )

1000  call erclos ( 'sepelplambda3d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelplambda3d'

      end if  ! ( debug )

      end subroutine sepelplambda3d
