      subroutine sepelpupw03nl
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute modified basis functions for upwind elements
!     Two dimensional elements (triangles)
!     The method of Mizukami and Hughes described in
!     A Petrov-Galerkin finite element method for convection-dominated flows:
!     an accurate upwinding technique for satisfying the maximum principle
!     Comp. Methods in Applied Mechanics and Engineering 50 (1985), pp 181-193
!     is applied
!     To prevent flip-flop during the non-linear iteration an extra flipflop
!     array has been introduced
!     See old subroutine elpuplinmax
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwind
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, kpos, jpos, ind(5), npos, m, n
      double precision :: phimax, beta, a(2), bb(2), c(3), normc, &
                          unorm, eps, h, factor, fact1, ugradphi(3), &
                          uparallel(2), ap, bp, normgradc, ugradc, tau
      integer :: iflipflop, metupwind
      double precision, pointer :: phi(:), psi(:), phix(:), phiy(:)
      double precision, pointer :: uvel(:), vvel(:), dcdx(:), x(:,:)

!     a              Vector pointing from point 1 to 2 (cyclic)
!     ap             u . grad c grad c . a
!     bb             Vector pointing from point 1 to 3 (cyclic)
!     beta           Parameter beta in method
!     bp             u . grad c grad c . b
!     c              see Mizukami (p)
!     dcdx           Contains the values of dc/dx and dc/dy
!     eps            diffusion in the direction of the velocity
!     fact1          Multiplication factor
!     factor         Multiplication factor
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     iflipflop      parameter to prevent flip-flop behavior in non-linear
!                    upwind
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3
!     jpos           Last negative inner product u . grad phi found
!     kpos           Last positive inner product u . grad phi found
!     m              number of quadrature points
!     metupwind      Type of upwind method
!     n              number of basis functions
!     normc          Norm of avarage c
!     normgradc      length of vector grad c
!     npos           Number of positive inner products u . grad phi
!     phi            Contains basis functions
!     phimax         Max j ( |u_i. grad phi_j| )
!     phix           d phi_i /d x
!     phiy           d phi_i /d y
!     psi            Contains upwind basis functions
!     tau            Work array of length n to store the relative distances
!                    between points
!     ugradc         u . grad c
!     ugradphi       u . grad phi
!     unorm          Length of velocity vector
!     uparallel      u . grad c grad c
!     uvel           u component of velocity
!     vvel           v component of velocity
!     x              Contains coordinates of vertices of element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     SEPELPUPW1     Compute modified basis functions for upwind elements
!                    One-dimensional elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1378   Parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!                                                       new
!       The basis functions are modified such that:  phi    = phi + p
!                                                       i        i   i
!       or:
!          new  k          k        k
!       phi   (x ) = phi (x ) + p (x )
!          i            i        i
!       The basis funtions phi are assumed to be filled before
!
!       To compute p_i we follow the method of Mizukami and Hughes
!
!       p_i is in their notation c_i
!
!       First we detect in which vertex or edge zone the velocity points
!       If in vertex zone then only one of the inner products u . grad phi
!       is positive and p_i is given by ( 2/3, -1/3, -1/3 ) cyclic
!       If in edge zone then two of the inner products u . grad phi
!       are positive
!       This is the difficult case described in the article
!       If u . grad phi = 0 we use the standard upwind
!       else the method depends on the vectors a, b and their inner products
!       with u and also of grad c
!       In this case grad c may flip flop and therefore it is detected if
!       one of the psi's is equal to 1, if so this situation is kept
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind /1, 2, 3, 1, 2 /
! ======================================================================
!
      call eropen ( 'sepelpupw03nl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpupw03nl'
         write(irefwr,1) 'iflipflop, methodupw', E%iflipflop, E%methodupw
  1      format ( a, 1x, (10i8) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill norm of velocity into array unorm

      uvel => E%veloc(:,1)
      vvel => E%veloc(:,2)
      phix => B%phider(:,1,1)
      phiy => B%phider(:,1,2)
      phi => B%phi(:,1)
      psi => B%psi(:,1)
      dcdx => gradoldvec(1,:,1)
      x => B%xyz

      iflipflop = E%iflipflop
      metupwind = E%methodupw
      m = B%m
      n = B%n

      if ( m/=1 ) then

!     --- Number of quadrature points is not 1

         call errchr ( 'm', 1 )
         call errint ( m, 1 )
         call errint ( 1, 2 )
         call errint ( 1, 3 )
         call errsub ( 1378, 3, 0, 1 )
         go to 1000

      end if  ! ( B%m/=1 )

!     --- Store component of alpha tensor in u-direction into eps

      call sepelpupw1 ( eps, unorm, uvel, vvel )

      if ( debug ) write(irefwr,2) 'eps, unorm', eps, unorm

!     --- Compute the distance h in the centroid

      phimax = max ( abs(uvel(1)*phix(1)+vvel(1)*phiy(1)), &
                     abs(uvel(1)*phix(2)+vvel(1)*phiy(2)), &
                     abs(uvel(1)*phix(3)+vvel(1)*phiy(3)) )
      if ( phimax.gt.epsmac ) then
         h = unorm/phimax
      else
         h = 0d0
      end if

!     --- Compute beta and factor

      if ( eps>1d-15 ) then

!     --- eps > 1d-10,  hence second order term plays a role

         beta = unorm*h/eps
         if ( beta<=2d0 ) then

            psi(1:n) = phi(1:n)
            go to 1000

         end if
         factor = 1d0-2d0/beta

      else

         factor = 1d0

      end if

!     --- Determine the inner products of u and grad phi
!         Compute number of positive elements

      npos = 0
      do i = 1, n
         ugradphi(i) = uvel(1)*phix(i)+vvel(1)*phiy(i)
         if ( ugradphi(i)>0d0 ) then

!        --- inner product of u and grad phi positive

            npos = npos+1
            kpos = i

         else if ( ugradphi(i)<0d0 ) then

!        --- inner product of u and grad phi negative

            jpos = i

         end if

      end do

!     --- Compute updates c

      fact1 = 1d0/3d0
      c = -fact1
      if ( npos==1 ) then

!     --- npos = 1,   velocity directs into vertex part i

         c(kpos) = 2d0*fact1

      else

!     --- npos = 2,   velocity directs into edge part i

         ugradc = uvel(1)*dcdx(1)+vvel(1)*dcdx(2)
         normgradc = dcdx(1)**2+dcdx(2)**2
         normc = sqrt(dot_product(c,c)/3d0)
         if ( abs(ugradc)<=1d-5*normc*unorm ) then

!        --- u . grad c = 0

            tau = fact1/abs(ugradphi(jpos))
            do i = 1, n
               c(i) = tau*ugradphi(i)
            end do

         else

!        --- u . grad c # 0

            ugradc = ugradc/normgradc
            uparallel(1) = ugradc*dcdx(1)
            uparallel(2) = ugradc*dcdx(2)
            a(1) = x(ind(jpos+1),1)-x(jpos,1)
            a(2) = x(ind(jpos+1),2)-x(jpos,2)
            bb(1) = x(ind(jpos+2),1)-x(jpos,1)
            bb(2) = x(ind(jpos+2),2)-x(jpos,2)
            ap = a(1)*uparallel(1)+a(2)*uparallel(2)
            bp = bb(1)*uparallel(1)+bb(2)*uparallel(2)
            if ( ap>0d0 .and. bp>0d0 ) then

!           --- Both inner products positive

               tau = fact1/abs(ugradphi(jpos))
               c = tau*ugradphi

            else if ( ap>0d0 ) then

!           --- Inner product with a positive

               c(ind(jpos+1)) = 2d0*fact1

            else

!           --- Inner product with b positive

               c(ind(jpos+2)) = 2d0*fact1

            end if

         end if

      end if

      if ( metupwind==10 ) then

!     --- metupwind = 10, fill flipflop array

         jpos = 4
         do i = 1, 3
            if(abs(c(i))>0.66666d0) jpos=i
         end do
         iflipflop = jpos

      else if ( metupwind==11 ) then

!     --- metupwind = 11, use flipflop array

         if ( iflipflop==4 ) then

            jpos = 4
            do i = 1, 3
               if ( abs(c(i))>0.66666d0 ) jpos = i
            end do
            iflipflop=jpos

         else

            c = -fact1
            c(iflipflop)=2d0/3d0

         end if

      end if

      psi = phi+c*factor
      E%iflipflop = iflipflop

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iflipflop', E%iflipflop
         call prinrl ( psi, 3, 'psi' )
         write(irefwr,*) 'End sepelpupw03nl'

      end if  ! ( debug )
      call erclos ( 'sepelpupw03nl' )

      end subroutine sepelpupw03nl
