      subroutine sepelpupw05
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute modified basis functions psi in case of upwind
!     Two dimensional elements (quadrilaterals)
! **********************************************************************
!
!                       KEYWORDS
!     basis_function
!     element
!     upwind
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, metupw, ind(4), j, n, m
      double precision :: xi, beta, xhalf(4,2), uksi, ueta, hksi, heta, &
                          cgnorm, tau1(B%m), u1(B%m), u2(B%m), &
                          tau(B%m), h(B%m), unorm(B%m), eps(B%m)
      double precision, pointer :: uvel(:), vvel(:), phix(:,:), phiy(:,:)
      double precision, pointer :: alpha11(:), alpha12(:), alpha22(:)
      double precision, pointer :: psi(:,:), phi(:,:), x(:,:)
      double precision, pointer :: dcdx(:,:)

!     alpha11        Diffusion coefficient D_11
!     alpha12        Diffusion coefficient D_12
!     alpha22        Diffusion coefficient D_22
!     beta           Parameter beta in method
!     cgnorm         Norm of dphidx in integration point for specific value of i
!     dcdx           Gradient of old solution
!     eps            Array of length 9 to store the value of the tensor alpha
!                    in the direction of the velocity in the vertices
!     h              Array of length 9 to store the values of the diameter h
!                    in the vertices
!     heta           Distance in eta direction
!     hksi           Distance in ksi direction
!     i              Counting variable
!     ind            Array containing the cyclic permutation from 1 to 4
!     j              Counting variable
!     k              Counting variable
!     m              number of quadrature points
!     metupw         type of upwind
!     n              number of basis functions for each component
!     phi            values of basis functions in quadrature points
!     phix           Contains x-derivatives of basis functions in quadrature
!                    points
!     phiy           Contains y-derivatives of basis functions in quadrature
!                    points
!     psi            values of upwind basis functions in quadrature points
!     tau            Array of length 9 to store the values of tau in the
!                    integration points
!     tau1           Array of length 9 to store the values of tau parallel in
!                    the integration points
!     u1             Parallel velocity
!     u2             Parallel velocity
!     ueta           Velocity component in eta direction
!     uksi           Velocity component in ksi direction
!     unorm          array of length 9 to store the norm of the velocity in the
!                    vertex
!     uvel           u-component velocity
!     vvel           v-component velocity
!     x              coordinates in nodes
!     xhalf          Contains the co-ordinates of the mid-side points
!     xi             Factor xi, see method
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELPUPW1     Compute modified basis functions for upwind elements
!                    One-dimensional elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                                                       new
!       The basis functions are modified such that:  phi    = phi + p
!                                                       i        i   i
!       or:
!          new  k          k        k
!       phi   (x ) = phi (x ) + p (x )
!          i            i        i
!       The basis funtions phi are assumed to be filled before
!       p  = tau u . grad phi
!        i       -           i
!                   h xi                    ||u|| h
!       with tau = ------ beta =  --------
!                   2||u||                   2 eps
!                      -
!       xi depends on metupw according to:
!       metupw = 1:  Classical upwind scheme  xi = 1
!                2:  Il'in    xi = coth(beta) - 1/beta
!                                             beta/3     -3<beta<3
!                3:  Doubly assymptotic: xi =
!                                             sign(beta)  |beta|>3
!                                         -1-1/beta     beta<1
!                4:  critical approx: xi = 0            -1<beta<1
!                                          1-1/beta     beta>1
!       eps is either alpha (if a11 = a22) or alpha in the direction of
!       the velocity
!              T                 2
!       eps = u  alpha  u / ||u||       (tensor alpha)
!             -         -     -
!       h = max distance in element in the direction of u
!                                                       -
!       h is computed as follows:
!       h = ( |u   | h    + |u   | h     ) / ||u||
!               ksi   ksi     eta   eta        -
!       h eta = || x34 - x12 ||      h ksi = || x23 - x41 ||
!       x34 = (x3 + x4)/2     etc.
!              -    -
!     4 +------------+3
!       |            |
!       |            |
!       |            |
!       +------------+
!       1            2
!       u ksi= ux ( x23-x14) + uy ( y23-y14)
!       u eta= ux ( x34-x12) + uy ( y34-y12)
!       First tau and h are computed in the vertices and afterwards tau is
!       interpolated to the integration points
!    In case of the discontinuity capturing methods we define u   as
!                                                              ||
!                                            2
!    u   = u . grad(c) grad(c) / || grad(c)||
!     ||
!    In that case the upwind basis functions become
!       p  = ( tau u  + tau   u  ) . grad phi
!        i        1-       2  -||            i
!    with tau  the tau as described above and tau  is tau   (DC1) or
!            1                                   2       ||
!    max(0,tau  -tau) (DC2)      tau   is the tau based upon u
!             ||                    ||                        ||
!    In this case we limit ourselves to the doubly assymptotic approximation
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind / 2, 3, 4, 1 /
! ======================================================================
!
      call eropen ( 'sepelpupw05' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpupw05'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      uvel => E%veloc(:,1)
      vvel => E%veloc(:,2)
      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      psi => B%psi
      phi => B%phi
      if ( E%filloldderiv ) dcdx => gradoldvec(1:B%m,1:E%ndim,1)
      x => B%xyz

      m = B%m
      n = B%n
      metupw = E%methodupw

      if ( debug ) then
         call prinrl1 ( phi, m, n, 'phi' )
         call prinrl1 ( phix, m, n, 'dphi/dx' )
         call prinrl1 ( phiy, m, n, 'dphi/dy' )
         call prinrl ( uvel, m, 'u' )
         call prinrl ( vvel, m, 'v' )
      end if  ! ( debug )

!     --- Fill norm of velocity in vertices into array unorm

      call sepelpupw1 ( eps, unorm, uvel, vvel )

      if ( debug ) then
         call prinrl ( eps, m, 'eps' )
         call prinrl ( unorm, m, 'unorm' )
      end if  ! ( debug )

!     --- Compute the distance h in the vertices

      if ( n==4 .or. n==5 ) then

!     --- four-point quadrilateral

         do j = 1, 2
            do i = 1, 4
               xhalf(i,j) = 0.5d0 * ( x(ind(i),j) + x(i,j) )
            end do
         end do

      else

!     --- nine-point quadrilateral

         do j = 1, 2
            do i = 1, 4
               xhalf(i,j) = x(2*i-1,j)
            end do
         end do

      end if
      do i = 1, m
         if ( unorm(i)>epsmac ) then
            uksi = uvel(i) * ( xhalf(2,1) - xhalf(4,1) ) + &
                   vvel(i) * ( xhalf(2,2) - xhalf(4,2) )
            ueta = uvel(i) * ( xhalf(3,1) - xhalf(1,1) ) + &
                   vvel(i) * ( xhalf(3,2) - xhalf(1,2) )
            h(i) = ( abs(uksi) + abs(ueta) ) / unorm(i)
         else
            hksi = sqrt ( (xhalf(2,1)-xhalf(4,1))**2 + &
                          (xhalf(2,2)-xhalf(4,2))**2 )
            heta = sqrt ( (xhalf(3,1)-xhalf(1,1))**2 + &
                          (xhalf(3,2)-xhalf(1,2))**2 )
            h(i) = 0.5d0 * ( hksi + heta )
         end if
      end do
      if ( debug ) call prinrl ( h, m, 'h' )

!     --- Compute tau in the vertices

      do i = 1, m
         if ( eps(i)>1d-10 .and. metupw<=8 .and. metupw/=6 ) then

!        --- eps(i)>1d-10,  hence second order term plays a role

            beta = unorm(i)*h(i)/(2d0*eps(i))
            if ( beta<1d-10 ) then
               xi = 0d0
            else if ( metupw==1 ) then

!           --- metupw = 1   classical upwind scheme

               xi = 1d0

            else if ( metupw==2 ) then

!           --- metupw = 2   il'in scheme

               xi = 1d0/tanh(beta)-1d0/beta

            else if ( metupw==3 .or. metupw==7 .or. metupw==8 ) then

!           --- metupw = 3, 7 or 8    doubly assymptotic approximation

               if ( abs(beta)>3d0 ) then
                  xi=sign(1d0,beta)
               else
                  xi = beta / 3d0
               end if

            else if ( metupw==4 ) then

!           --- metupw = 4    critical approximation

               if ( beta<-1d0 ) then
                  xi = -1d0 - 1d0 / beta
               else if ( beta>1d0 ) then
                  xi = 1d0 - 1d0 / beta
               else
                  xi = 0d0
               end if

            else if ( metupw==5 ) then

!           --- metupw = 5

               if ( unorm(i)<=1d-5*eps(i) ) then
                  xi = 0d0
               else
                  beta = beta*beta
                  xi = sqrt(beta/(9d0+beta))
               end if
            end if

         else if (metupw==6 ) then

!        --- metupw = 6

            if ( h(i)>epsmac ) then
               beta = (2d0*unorm(i)/h(i))**2 + &
                      (4d0*eps(i)/h(i)**2)**2
            else
               beta = 0d0
            end if
            if ( tstep>0d0 ) beta = beta + 4d0/(tstep*tstep)
            tau(i) = 1d0/sqrt(beta)

         else

            xi = 1d0

         end if
         if ( metupw<=8 .and. metupw/=6 ) then
            if ( unorm(i)>epsmac ) then
               tau(i) = h(i)*xi/(2d0*unorm(i))
            else
               tau(i) = 0d0
            end if
         end if
      end do
      if ( debug ) call prinrl ( tau, m, 'tau' )

!     --- compute new basis functions, it has been assumed that the old
!         basis functions are already stored in array phi

      do k = 1, m
         do i = 1, n
            psi(i,k) = tau(k) * ( uvel(k)*phix(i,k)+ &
                                  vvel(k)*phiy(i,k) ) + phi(i,k)
         end do
      end do

      if ( B%cart==1 ) then

         do k = 1, m
            if ( abs(unorm(k))>0d0 .and. &
                 abs(eps(k))*1d2>unorm(k) ) B%secondupw = .true.
         end do

      end if

      if ( metupw==7 .or. metupw==8 ) then

!     --- Discontinuity capturing
!         First compute u
!                        ||

         do k = 1, m

            cgnorm = dcdx(k,1)**2 + dcdx(k,2)**2
            if ( cgnorm<epsmac ) then
               u1(k) = 0d0
               u2(k) = 0d0
            else
               xi = (uvel(k)*dcdx(k,1) + vvel(k)*dcdx(k,2))/cgnorm
               u1(k) = xi*dcdx(k,1)
               u2(k) = xi*dcdx(k,2)
            end if

         end do

         call sepelpupw1 ( eps, unorm, u1, u2 )

!        --- Compute the distance h in the vertices

         do i = 1, m
            if ( unorm(i)>epsmac ) then
               uksi = u1(i) * ( xhalf(2,1) - xhalf(4,1) ) + &
                      u2(i) * ( xhalf(2,2) - xhalf(4,2) )
               ueta = u1(i) * ( xhalf(3,1) - xhalf(1,1) ) + &
                      u2(i) * ( xhalf(3,2) - xhalf(1,2) )
               h(i) = ( abs(uksi) + abs(ueta) ) / unorm(i)
            else
               hksi = sqrt ( (xhalf(2,1)-xhalf(4,1))**2 + &
                             (xhalf(2,2)-xhalf(4,2))**2 )
               heta = sqrt ( (xhalf(3,1)-xhalf(1,1))**2 + &
                             (xhalf(3,2)-xhalf(1,2))**2 )
               h(i) = 0.5d0 * ( hksi + heta )
            end if
         end do

!        --- Compute tau in the vertices

         do i = 1, m
            if ( eps(i)>1d-10 ) then

!           --- eps(i)>1d-10,  hence second order term plays a role

               beta = unorm(i)*h(i)/(2d0*eps(i))
               if ( beta<1d-10 ) then
                  xi = 0d0
               else

!              --- metupw = 3, 7 or 8    doubly assymptotic approximation

                  if ( abs(beta)>3d0 ) then
                     xi=sign(1d0,beta)
                  else
                     xi = beta / 3d0
                  end if
               end if
            else
               xi = 1d0
            end if
            if ( unorm(i)>epsmac ) then
               tau1(i) = h(i)*xi/(2d0*unorm(i))
            else
               tau1(i) = 0d0
            end if
         end do
         if ( metupw==8 ) then
            do k = 1, m
               tau1(k) = max(0d0,tau1(k)-tau(k))
            end do
         end if
         do i = 1, n
            do k = 1, m
               psi(i,k) = tau1(k) * ( u1(k)*phix(i,k)+ &
                                      u2(k)*phiy(i,k) ) + psi(i,k)
            end do
         end do
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelpupw05'

      end if  ! ( debug )
      call erclos ( 'sepelpupw05' )

      end subroutine sepelpupw05
