      subroutine sepelpupw1 ( eps, unorm, u1, u2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array eps with values of the tensor alpha
!     in the direction of the velocity in the vertices
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwind
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: u1(B%m), u2(B%m)
      double precision, intent(out) :: unorm(B%m),  eps(B%m)

!     eps            o    Array of length m to store the value of the tensor
!                         alpha in the direction of the velocity in the vertices
!     u1             i    Parallel velocity first component
!     u2             i    Parallel velocity second component
!     unorm          o    array of length m to store the norm of the velocity in the
!                         vertex
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: alpha11(:), alpha12(:), alpha22(:)
      integer :: m, i

!     alpha11        Diffusion coefficient D_11
!     alpha12        Diffusion coefficient D_12
!     alpha22        Diffusion coefficient D_22
!     i              Counting variable
!     m              Number of quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelpupw1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpupw1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      alpha11 => E%difftensor(:,1)
      alpha12 => E%difftensor(:,2)
      alpha22 => E%difftensor(:,4)
      m = B%m

      unorm = sqrt ( u1**2 + u2**2 )

!     --- Store component of alpha tensor in u-direction into eps

      select case (E%typediff)

      case(0)

!     --- no diffusion,   eps = 0

         eps = 0d0

      case(1)

!     --- typediff = 1,   diffusion is scalar

         eps = alpha11

      case(2)

!     --- diffusion is diagonal tensor


         do i = 1, m
            if ( unorm(i)>0d0 ) then
               eps(i) = ( u1(i)**2*alpha11(i)+u2(i)**2*&
                          alpha22(i) ) / unorm(i)**2
            else
               eps(i) = alpha11(i)
            end if
         end do

      case(3)

!     --- General case
!         eps is defined by the formula  eps = u^T . alpha . u

         do i = 1, m
            if ( unorm(i)>0d0 ) then
                eps(i) = ( u1(i)**2*alpha11(i)+2d0*u1(i)* &
                           alpha12(i)*u2(i) + u2(i)**2* &
                           alpha22(i) ) / unorm(i)**2
            else
               eps(i) = alpha11(i)
            end if
         end do

      end select  ! case (E%typediff)

1000  call erclos ( 'sepelpupw1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelpupw1'

      end if  ! ( debug )

      end subroutine sepelpupw1
