      subroutine el0211 ( iuser, user, jelp, isub, isup, array, iload, &
                          isequ, iseqgam, index4, ifsi, igeom )
! ======================================================================
!
!        programmer    Martijn Cox & Niels Driessen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 20-06-2008 extra option igeom
!        version  1.0  date 04-07-2004 derived from el0210
!
!   copyright (c) 2004-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM211
!     in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     element_matrix
!     element_vector
!     initialization
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, isub, isup, iload, isequ(ndim,inpelm), &
              iseqgam(inpelm), index4(numold,*), ifsi, igeom

!     array          o    Real work array in which the values of the
!                         coefficients in the integration points are stored
!     ifsi           o    Sequence number of old solution vector for
!                         fully coupled fluid-structure interaction
!     igeom          o    Defines type of geometry for the rigid body
!     iload          o    Indicates the type of boundary condition.
!                         Possible values:
!                         0: Prescribed load in co-ordinate directions
!                         1: Prescribed load given in normal and tangential
!                            direction
!                         2: Prescribed load given in normal and tangential
!                            direction + linearization
!     index4         i    Two-dimensional integer array of length NUMOLD x
!                         INPELM containing the number of unknowns per point
!                         accumulated in array VECOLD with respect to the
!                         present element.
!                         For example INDEX4(i,1) contains the number of
!                         unknowns in the first point with respect to the
!                         i th vector stored in VECOLD.
!                         The number of unknowns in the j th point with respect
!                         to i th vector in VECOLD is equal to
!                         INDEX4(i,j) - INDEX4(i,j-1)  (j>1)
!     iseqgam        o    Sequence for contact pressure dof's
!     isequ          o    Sequence for displacement dof's
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELxxxx. Possible vaues:
!                          1:  Subroutine ELP610  (Linear line element)
!                          2:  Subroutine ELP611  (Quadratic line element)
!                          3:  Subroutine ELP620  (Linear triangle)
!                          4:  Subroutine ELP624  (Extended quadratic triangle)
!                          5:  Subroutine ELP621  (Bilinear quadrilateral)
!                          6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                         11:  Subroutine ELP630  (Linear tetrahedron)
!                         12:  Subroutine ELP631  (Quadratic tetrahedron)
!                         13:  Subroutine ELP632  (Trilinear brick)
!                         14:  Subroutine ELP633  (Triquadratic brick)
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncoef, i, j, isubcf, mrule, isum, inunkp, ishift
      logical funchk

!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     ishift         Shift in coefficient sequence number
!     isubcf         Under bound for coefficients check
!     isum           Integer variable to compute a sum
!     j              Counting variable
!     mrule          Type of integration rule as given by the user
!     ncoef          Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
!     INSTOP         Stop the program
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!
!    +-------+-------------------------------------------------------------+ &
!    | coef  | description                                                 |
!    +-------+-------------------------------------------------------------+ &
!    |   1   | coordinate system for boundary condition                    |
!    |       +---------+----+----------------------------------------------+ &
!    |                 |  0 | global system                                |
!    |                 |  1 | local system                                 |
!    |                 |  2 | local system and the boundary-integral is    |
!    |                 |    | linearised                                   |
!    |                 |    | (non-symmetric matrix)                       |
!    +-------+---------+----+----------------------------------------------+ &
!    |   2   | ifsi_solution (integer refers to sequence number of solution|
!    |       |                vector for fully coupled fluid-solid inter.) |
!    +-------+-------------------------------------------------------------+ &
!    |   3   | type of numerical integration                               |
!    |       +---------+----+----------------------------------------------+ &
!    |                 |  0 | default value, defined by element            |
!    |                 |  1 | Newton-Cotes                                 |
!    |                 |  3 | Gauss (not implemented)                      |
!    +-------+---------+----+----------------------------------------------+ &
!    |   4   | type of geometry for rigid body                             |
!    +-------+-------------------------------------------------------------+ &
!    |   5   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!    |   6   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!    |   7   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!    |   8   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!    |   9   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!    |  10   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!    |  11   | not used                                                    |
!    +---------------------------------------------------------------------+ &
!    |    VECTORS NEEDED                                                   |
!    +---------------------------------------------------------------------+ &
!    | The following vectors are needed (in this order)                    |
!    +----+-----------+----------------------------------------------------+ &
!    |  1 |  u        |  incremental solution                              |
!    |  2 |  un       |  old solution at increment n                       |
!    |  3 |  usrvec   |  user defined vector  (if iusrvec>=1)            |
!    |  4 |  icont    |  help array for contact                            |
!    +----+-----------+----------------------------------------------------+ &
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!      74   Incorrect number of degrees of freedom for this type
!    1014   Sequence number of mesh velocity out of range
!    1016   Structure of mesh velocity vector and velocity vector different
!    1304   For coefficient only positive constants are allowed
!    1766   Type of integration rule incorrect
!    1771   Shape number not yet implemented
!    2065   incorrect number of unknowns in element
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0211' )
      call elcomm
      ncomp  = ndim
      jtrans = 0
      jderiv = 0
      ideriv = 1
      jind = 0
      icoor = 2
      ishift = 0

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficients

      ipos = iuser (ielgrp + 5)
      if (ipos<7) then
         call errint (ielgrp+5, 1)
         call errint (ipos, 2)
         call errint (ielem, 3)
         call errint (itype, 4)
         call errint (ielgrp, 5)
         call errsub (42, 5, 0, 0)
      end if

      nuser = 0

!     --- Get the parameter iload

      iload = inirepscal(iuser(ipos))
      if (iload<0 .or. iload>2) then
         call errint (iload, 1)
         call errint (2, 2)
         call errint (ielem, 3)
         call errint (itype, 4)
         call errint (ielgrp, 5)
         call errsub (1766, 5, 0, 0)
         iload = 0
      end if

      if (iload==1 .or. iload==2) iweigh = 31
      ipos = ipos + 1
      ishift = ishift+1

!     --- Fill flag ifsi

      ifsi = inirepscal(iuser(ipos))
      if (ifsi<0 .or. ifsi>numold) then

!     --- ifsi out of range

          call errint ( ifsi, 1 )
          call errint ( numold, 2 )
          call errsub ( 1014, 2, 0, 0 )
      end if
      if ( ifsi>0 ) then

!     --- Check if ifsi and iseqin have the same number of unknowns

          do i = 1, inpelm
             do j = 1,2
                if ( index4(iseqin+j-1,i)/=index4(ifsi,i) ) then
                   call errint ( ifsi, 1 )
                   call errint ( index4(ifsi,i), 2)
                   call errint ( i, 3 )
                   call errint ( ndim, 4 )
                   call errchr ( namesol(ifsi), 1 )
                   call errsub ( 1016, 4, 0, 1 )
                end if
             end do
          end do
      end if
      ipos = ipos + 1
      ishift = ishift+1

!     --- Get integration rule

      mrule = inirepscal(iuser(ipos))
      ipos = ipos + 1
      ishift = ishift+1
      if (mrule<0 .or. mrule>40) then
         call errint (mrule, 1)
         call errint (3, 2)
         call errint (ielem, 3)
         call errint (itype, 4)
         call errint (ielgrp, 5)
         call errsub (1766, 5, 0, 0)
      end if

!     --- get type of rigid body

      igeom = inirepscal(iuser(ipos))
      ipos = ipos + 1
      ishift = ishift+1
      if (igeom<0 .or. igeom>7) then
         call errint (igeom, 1)
         call errint (3, 2)
         call errint (ielem, 3)
         call errint (itype, 4)
         call errint (ielgrp, 5)
         call errsub (1766, 5, 0, 0)
      end if

      ipos = ipos + 1
      ishift = ishift+2

!     --- Set integration rule

      if (mrule>0) then

!     --- integration rule was given by the user

         irule = mrule

      else

!     --- default is Newton-Cotes

         irule = 1

      end if

!     --- Set jdiag = 2 when Gauss integration

      if (irule/=1) jdiag = jdiag + 2

!     --- Set parameters depending on the shape numbers

      goto (198, 102, 198, 198, 198, 106, 107, 198, 198, 198, &
            198, 198, 198, 198, 198, 198, 198, 198), ishape

!     --- Quadratic line

 102     jelp = ishape
         n = 3
         m = n
         jdercn = 1
         if (irule==2) m = 2

!        --- fill array isequ

         isum = 0
         do i = 1, inpelm
            inunkp = index4(1,i)-isum
            if (inunkp<ncomp) then
               call errint (inunkp, 1)
               call errint (i, 2)
               call errint (ncomp, 3)
               call errint (ielem, 4)
               call errint (itype, 5)
               call errint (ielgrp, 6)
               call errsub (2065, 6, 0, 0)
            end if
            do j = 1, ncomp
               isequ(j,i) = isum+j
            end do
            isum = isum+inunkp
            iseqgam(i) = isum
         end do

         goto 199

!        --- Biquadratic quadrilateral

 106     jelp = ishape
         n = 9
         m = n
         jdercn = 1
         if (irule==2) m = 4

!        --- fill array isequ and iseqgam

         isum = 0
         do i = 1, inpelm
            inunkp = index4(1,i)-isum
            if (inunkp<ncomp) then
               call errint (inunkp, 1)
               call errint (i, 2)
               call errint (ncomp, 3)
               call errint (ielem, 4)
               call errint (itype, 5)
               call errint (ielgrp, 6)
               call errsub (2065, 6, 0, 0)
            end if
            do j = 1, ncomp
               isequ(j,i) = isum+j
            end do
            isum = isum+inunkp
            iseqgam(i) = isum
         end do

         goto 199

!        --- Extended biquadratic triangle

 107     jelp = ishape
         n = 7
         m = n
         jdercn = 1
         if (irule==2) m = 4

!        --- fill array isequ and iseqgam

         isum = 0
         do i = 1, inpelm
            inunkp = index4(1,i)-isum
            if (inunkp<ncomp) then
               call errint (inunkp, 1)
               call errint (i, 2)
               call errint (ncomp, 3)
               call errint (ielem, 4)
               call errint (itype, 5)
               call errint (ielgrp, 6)
               call errsub (2065, 6, 0, 0)
            end if
            do j = 1, ncomp
               isequ(j,i) = isum+j
            end do
            isum = isum+inunkp
            iseqgam(i) = isum
         end do

         goto 199

!        --- Error 1771: Shape not available

 198     call errint (ishape, 1)
         call errint (ielem, 2)
         call errint (itype, 3)
         call errint (ielgrp, 4)
         call errsub (1771, 4, 0, 0)

 199  if (ierror/=0) goto 1000

!     --- Check number of unknowns

      if (icount<n*ndim ) then

!     --- Number of unknowns incorrect

         call errint (icount, 1)
         call errint (n*ndim, 2)
         call errint (ielem, 3)
         call errint (itype, 4)
         call errint (ielgrp, 5)
         call errsub (74, 5, 0, 0)
      end if

!     --- Fill some element constants

      mn = m * n
      iphi = 1
      igausp = 1

      ncoef = 6

!     --- Clear array ARRAY

      do i = 1, ncoef*m
         array(i) = 0d0
      end do

!     --- And fill ARRAY with coefficients

      do i = 1 , ncoef
         call elusr3 ( i, array(1+(i-1)*m), iuser, user, i+ishift )
      end do

!     --- Test coefficients

      do i = 4, 6
         if (ind(i)<0 .and. array(1+(i-1)*m)<0d0) then
            call errint (i+5, 1)
            call errint (itype, 2)
            call errint (ielem, 3)
            call errint (ielgrp, 4)
            call erreal (array(1+(i-1)*m), 1)
            call errsub (1304, 4, 1, 1)
            call errchr ('c', 1)
            ind(i) = 0
         end if
      end do

!     --- Test array lengths, number of nodes and number of degrees of freedom

      call prinlg ('iuser', ipos-1, user, iuser)
      call prinlg ('user', -nuser, user, iuser)

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      isubcf = 1
      do i = isubcf , ncoef
         isub = i

!        --- If ind(i)>0 then paramater i corresponds to a function

         if (ind(i)>0) goto 170
      end do
      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i)>0 ) goto 190
      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do

      funchk = .false.
      do i = isub, isup
         if (ind(i)>0 .and. ind(i)<2000) then

!        --- Only the function calls are checked

            do j = i+1,isup
               if (ind(j)==ind(i)) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if (ind(i)>1000 .and. ind(i)<=2000) ind(lengthmst) = 1
         if (ind(i)>10000) funchk = .true.
      end do

!     --- Old solutions need to be computed

      ind(lengthmst) = 3

      if (funchk) ind(lengthmst) = ind(lengthmst) + 2

 1000 if ( ierror/=0 ) call instop

      notmas = 1
      call erclos ('el0211')

      end
