      subroutine el0410 ( iuser, user, jelp, jtime, eps, cn, clamb, &
                          mconv, modelv, isub, isup, coeffintpnts, &
                          thetdt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 27-02-2010 Extra parameters
!        version  2.3  date 08-02-2006 Fortran 90
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM900 or
!     ELD900 in the case that IFIRST = 0 and type numbers 400, 402, 406 and 410
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     array coeffintpnts is initialized with the values of the coefficients
!     The reason to use this subroutine is that type numbers 900 and 410
!     have a different input, but the same task
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), coeffintpnts(*), eps, cn, clamb, thetdt
      integer iuser(*), jelp, jtime, isub, isup, mconv, modelv

!     clamb          i    Parameter lambda with respect to viscosity model
!     cn             i    Power in power law model
!     coeffintpnts  i/o   Real array in which the values of the coefficients
!                         in the integration points are stored in the sequence:
!                         In the case of eld900 or elm900:
!                              2D                  3D
!                         1:   rho              rho
!                         2:   omega            omega
!                         3:   f1               f1
!                         4:   f2               f2
!                         5:                    f3
!                         6:   etha             etha
!                           In the case of eli900:
!                           1:   f
!     eps            i    Local accuracy
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    Integer user array to pass user information
!                         from main program to subroutine. See STANDARD PROBLEMS
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELD900.
!     jtime          o    Integer parameter indicating if the mass matrix for
!                         the time-dependent equation must be computed (>0)
!                         or not ( = 0)
!                         Possible values:
!                           0:  no mass matrix
!                           1:  diagonal mass matrix with constant coefficient
!                           2:  diagonal mass matrix with variable coefficient
!                          11:  full mass matrix with constant coefficient
!                          12:  full mass matrix with variable coefficient
!                         101:  refers to the special possibility of the theta
!                               method directly applied
!                         102:  refers to the special possibility of the theta
!                               method directly applied with variable rho
!     mconv          i    see USER INPUT
!     modelv         i    Defines the type of viscosity model
!                         Possible values:
!                         1:  newtonian liquid
!                         2:  power law liquid
!                         3:  carreau liquid
!                         4:  plastico viscous liquid
!                       100:  user provided model depending on grad u
!                       101:  user provided model depending on the second
!                             invariant
!                       102:  user provided model depending on the second
!                             invariant, &
!                             the co-ordinates and the velocity
!     thetdt         o    1/(theta dt) for the case of the theta method
!                         immediately applied
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncoef, i, j, isubcf, kchois, ichsav, ishift
      double precision etha
      logical funchk

!     etha    Viscsity parameter
!     funchk  Indicates whether there are coefficients that must be
!             evaluated by funcc3 (true) or not (false)
!     i       Counting variable
!     ichsav  Help variable to save the value of icheld
!     ishift  Shift in coefficient sequence number
!     isubcf  Under bound for coefficients check
!     j       Counting variable
!     kchois  Indication what type of parameters must be computed
!             Possible values:
!             0:  Standard method, the calling subroutine is ELM900
!                 It is supposed that an element matrix must be computed
!             1:  Subroutine ELD900 calls this subroutine, only
!                 derivatives must be computed
!             2:  Subroutine ELI900 calls this subroutine, only
!                 integrals must be computed
!     ncoef   Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Replace ivalue by scalar if necessary
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  Parameter eps for the penalty function method
!     2:  density rho
!     3:  Information about the convective terms (mconv)
!         Possible values:
!         0: convective terms are skipped (Stokes flow)
!         1: convective terms are linearized by Picard iteration
!         2: convective terms are linearized by Newton iteration
!         3: convective terms are linearized by the incorrect Picard
!            iteration
!         4: convective terms are linearized by successive substitution
!         10-13: See mconv-10, however, also the theta method for time
!            integration is applied
!     4:  angular velocity
!     5:  F1  (Body force in 1-direction)
!     6:  F2  (Body force in 2-direction)
!     7:  F3  (Body force in 3-direction)
!         In 2D all next numbers are reduced by one
!     8:  type of viscosity model used (modelv)
!         Possible values:
!         0,1:  newtonian liquid
!         2:    power law liquid
!         3:    carreau liquid
!         4:    plastico viscous liquid
!       100:    user provided model depending on grad u
!       101:    user provided model depending on the second invariant
!       102:    user provided model depending on the second invariant, &
!               velocity vector and co-ordinates
!     9:  etha (if modelv = 1, the dynamic viscosity
!                          2, the parameter etha_n
!                          3, the parameter etha_c
!                          4, the parameter etha_pv)
!    10:  n (if modelv = 2-4, the parameter n in the power of the model)
!    11:  lambda (if modelv = 3, the parameter lambda in the viscosity model
!                 if modelv = 4, the parameter s in the viscosity model)
!     In the case of a call by INTEGR:
!     1:  f
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   array IUSER(.) has value which is too small
!     238   Omega unequal to zero, whereas axisymmetric elements are used
!     262   ix has wrong value
!     264   jdegfd has wrong value
!     312   density is no constant
!     406   Value of icheld not allowed
!     838   Penalty parameter is not constant
!    1303   Only constants are allowed for this coefficient
!    1305   Coefficient has wrong value
!    1306   Coefficient has wrong value
!    1430   The viscosity parameter etha is non-positive
!    1915   Penalty parameter not positive
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0410' )
      call elcomm
      jtime  = 0
      jconsf = 2
      if ( itype/=406 ) then
         ncomp = ndim
      else
         ncomp = 2
      end if
      kchois = 0
      modelv = 0
      ishift = 0

!     --- Compute kchois

      if ( icheld==0 ) then

!     --- Call by elm900, all coefficients required

         ncoef = 6

      else if ( icheld>0 ) then

!     --- Call by eld900

         if ( icheld==10 .or. icheld>=20 ) then

!        --- icheld out of range

            call errint ( icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 406, 3, 0, 0 )

         end if

!        --- Set icheld equal to the new values

         ichsav = icheld

         if ( ichsav==1 ) then
            icheld = 27
         else if ( ichsav==2 ) then
            icheld = 21
         else if ( ichsav==3 ) then
            icheld = 24
         else if ( ichsav==4 ) then
            icheld = 25
         else if ( ichsav==5 ) then
            icheld = 26
         else if ( ichsav==6 ) then
            icheld = 22
         else if ( ichsav>=7 .and. ichsav<=9 ) then
            icheld = ichsav+21
         else if ( ichsav==11 ) then
            icheld = 7
         else if ( ichsav==12 ) then
            icheld = 1
         else if ( ichsav==13 ) then
            icheld = 4
         else if ( ichsav==14 ) then
            icheld = 5
         else if ( ichsav==15 ) then
            icheld = 6
         else if ( ichsav==16 ) then
            icheld = 2
         else if ( ichsav>=17 .and. ichsav<=19 ) then
            icheld = ichsav-9
         end if

         if ( icheld==6 .or. icheld==26 ) then
            ncoef = 6
         else if ( icheld==7 .or. icheld==27 ) then
            ncoef = 1
         else
            ncoef = 0
            kchois = 1
         end if

         if ( icheld==1 .or. icheld==21 ) then

            if ( ix<=0 .or. ix>ndim ) then
               call errint ( ix, 1 )
               call errint ( ndim, 2 )
               call errint ( itype, 3 )
               call errint ( ielgrp, 4 )
               call errsub ( 262, 4, 0, 0 )
            end if
            if ( jdegfd<=0 .or. jdegfd>ndim ) then
               call errint ( jdegfd, 1 )
               call errint ( ndim, 2 )
               call errsub ( 264, 2, 0, 0 )
            end if

         end if
         ncomp  = ndim
         jderiv = 1
         jind   = 1

      else if ( icheld<0 ) then

!     --- call by eli900, one (new) coefficient required

         kchois = 2
         ncoef  = 1
         if ( jdegfd<1 .or. jdegfd>ndim ) then
            call errint ( jdegfd, 1 )
            call errint ( ndim, 2 )
            call errsub ( 264, 2, 0, 0 )
         end if

      end if

!     --- Set parameters depending on the shape numbers

      n = inpelm
      m = n
      mn = m * n
      jelp = ishape
      nvert = 8
      iphi=1
      igausp = 1
      if ( itype==400 .or. itype==410 ) then

!     --- itype = 400, 410:  Cartesian co-ordinates

         jcart = 1

      else if ( itype==402 ) then

!     --- itype = 402:  Polar co-ordinates

         jcart = 3

      else if ( itype==404 ) then

!     --- itype = 404:  Axi-symmetric co-ordinates

         jcart = 2

      else

!     --- itype = 406:  Axi-symmetric co-ordinates with swirl

         jcart = 4

      end if

      if ( itype==400 ) then
         irule = 4
      else
         irule = 3
      end if
      if ( icheld>0 .and. icheld/=7 .and. icheld/=27 .or. &
           icheld<-2 ) irule = 1
      if ( irule/=1 ) jdiag=2

!     --- Fill the arrays IND, MST and coeffintpnts with information of
!         the coefficient
!         Clear array coeffintpnts

      do i = 1, 6*m
         coeffintpnts(i) = 0d0
      end do

      if ( ncoef>0 .or. icheld>0 ) then

         ipos = iuser ( ielgrp + 5 )
         if ( ipos<7 ) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 42, 5, 0, 0 )
         end if

         nuser = 0

         if ( kchois==0 ) then

!        --- Standard situation, Compute the parameters jtime, irule, jcart, &
!                                                       mconv, modelv, eps

            notmas = 1
            ishift = ishift+1
            call elusr3 ( 1, coeffintpnts, iuser, user, ishift )
            if ( ind(1)>=0 ) then
               call errint ( 6, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call errsub ( 838, 4, 0, 0 )
            end if
            eps = coeffintpnts(1)
            if ( eps<=0d0 .and. (icheld==0 .or. icheld==7 &
                 .or. icheld==27 ) ) then
               call erreal ( eps, 1 )
               call errint ( 6, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call errsub ( 1915, 4, 1, 0 )
            end if

!           --- rho

            ishift = ishift+1
            call elusr3 ( 1, coeffintpnts, iuser, user, ishift )

            mconv = inirepscal ( iuser(ipos) )
            if ( mconv>=10 ) then

!           --- mconv>0, special method for time-dependence

               mconv = mconv - 10
               jtime = 101
               thetdt = 1d0 / (tstep*theta)
               if ( ind(1)>0 ) jtime = 102

            end if

            if ( mconv<0 .or. mconv>4 ) then
               call errint ( 4, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call errint ( mconv, 5 )
               call errchr ( 'mconv', 1 )
               call errsub ( 1305, 5, 0, 1 )
            end if
            ipos = ipos + 1

!           --- omega, f1, f2, f3

            do i = 2, 2+ndim
               call elusr3 ( i, coeffintpnts(1+(i-1)*m), iuser, user, &
                             i+ishift )
            end do
            ishift = ishift+2+ndim

            if ( ind(2)==0 .and. itype==404 ) then

!           --- 238:  omega#0, whereas itype = 404

               call errint (1, ielgrp )
               call errint (2, ielem )
               call errsub ( 238, 2, 0, 0 )

            end if
            modelv = inirepscal ( iuser(ipos) )
            ipos = ipos + 1
            ishift = ishift+1
            if ( modelv<0 .or. modelv>5 .and. modelv<100 .or. &
                 modelv>102 ) then
               call errint ( 2, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call errint ( modelv, 5 )
               call errchr ( 'modelv', 1 )
               call errsub ( 1305, 5, 0, 1 )
            end if
            modelv = max ( modelv, 1 )
            if ( icheld>0 .and. icheld/=6 .and. icheld/=26 ) &
                 modelv = 0
            ishift = ishift+1
            call elusr3 ( 6, coeffintpnts(1+(6-1)*m), iuser, user, &
                          ishift )

         else if ( kchois==2 ) then

!        --- Call by INTEGR, Compute the parameter f

            ishift = ishift+1
            call elusr3 ( 1, coeffintpnts, iuser, user, ishift )

         end if

      end if

!     --- Test coefficients

      if ( ind(1)==0 ) then

!     --- rho has not been given a value, set rho equal to 1

         do i = 1, m
            coeffintpnts(i) = 1d0
         end do

      else if ( ind(1)>0 .and. icheld==0 ) then

!     --- rho not constant has not yet been implemented

         call errint ( 7, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call errsub ( 312, 4, 0, 0 )

      end if

      if ( ind(6)<=0 .and. (icheld==0 .or. icheld==6 &
           .or. icheld==26) ) then

!     --- etha is constant, test whether etha is positive

         etha = coeffintpnts(1+5*m)
         if ( etha<=0d0 ) then

!        --- etha<=0

            call erreal ( etha, 1 )
            call errint ( itype, 1 )
            call errint ( 9, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call errsub ( 1430, 4, 1, 0 )

         end if

      end if

!     --- Read extra parameters depending on modelv

      if ( modelv>1 .and. modelv<5 ) then

!     --- power law type fluid

         ishift = ishift+1
         call elusr3 ( 7, coeffintpnts(1+6*m), iuser, user, ishift )
         if ( ind(7)>0 ) then

!        --- cn is given as function instead of coefficient

            call errint ( 13, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call errchr ( 'cn', 1 )
            call errsub ( 1303, 4, 0, 1 )

         end if
         cn = coeffintpnts(1+6*m)
         if ( cn<0d0 .or. cn>5.0d0 ) then

!        --- cn out of range

            call errint ( 13, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call erreal ( cn, 1 )
            call errchr ( 'cn', 1 )
            call errsub ( 1306, 4, 1, 1 )

         end if

      end if

      if ( modelv>2 .and. modelv<5 ) then

!     --- power law type fluid with two parameters

         ishift = ishift+1
         call elusr3 ( 7, coeffintpnts(1+6*m), iuser, user, ishift )
         if ( ind(7)>0 ) then

!        --- clamb is given as function instead of coefficient

            call errint ( 14, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            if ( modelv==3 ) then
               call errchr ( 'lambda', 1 )
            else
               call errchr ( 's', 1 )
            end if
            call errsub ( 1303, 4, 0, 1 )

         end if
         clamb = coeffintpnts(1+6*m)
         if ( clamb<0d0 ) then

!        --- clamb out of range

            call errint ( 13, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call erreal ( clamb, 1 )
            if ( modelv==3 ) then
               call errchr ( 'lambda', 1 )
            else
               call errchr ( 's', 1 )
            end if
            call errsub ( 1306, 4, 1, 1 )

         end if

      end if

!     --- Test array lengths, number of nodes and number of degrees of freedom

      call prinlg ( 'iuser', ipos-1, user, iuser )
      call prinlg ( 'user', -nuser, user, iuser )

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      isubcf = 1
      if ( icheld>0 ) isubcf = 6

      do i = isubcf , ncoef
         isub = i

!        --- If ind(i)>0 then paramater i corresponds to a function

         if ( ind(i)>0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i)>0 ) goto 190
      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do

      if ( icheld>0 ) then

!     --- Compute derivative quantities

         ind(lengthmst) = 1
         if ( icheld==7 .or. icheld==27 ) then
            jderiv = 0
         else
            jderiv = 1
         end if

      else if ( icheld==-1 .or. icheld==-2 ) then

!     --- icheld = -1 or -2, do not compute derivatives

         ideriv = 0
         jderiv = 0

      else if ( icheld<-2 ) then

!     --- icheld<-2, derivatives should be computed

         jderiv = -icheld+3

      else if ( mconv>0 .or. modelv>1 .or. jtime>0 ) then

!     --- the convective terms must be taken into account and/or the
!         viscosity model depends on the gradient of the velocity and/or
!         the theta method must be applied

         ind(lengthmst) = 1
         jderiv = 1

      else

!     --- Stokes flow with Newtonian viscosity model

         ind(lengthmst) = 0

      end if

      funchk = .false.
      do i = isub, isup
         if ( ind(i)>0 .and. ind(i)<2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j)==ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
            igausp = 1
         end if
         if ( ind(i)>1000 .and. ind(i)<=2000 ) ind(lengthmst) = 1
         if ( ind(i)>10000 ) funchk = .true.
         if ( ind(i)>1000 .and. ind(i)<=2000 .or. ind(i)>10000 ) &
              igausp=1
      end do

      if ( icheld<-1 .or. icheld>0 ) ind(lengthmst) = 1
      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2

      call erclos ( 'el0410' )

      end
