      subroutine el0915 ( iuser, user, jelp, isub, isup, array, induv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.2  date 10-03-2013 Initialize nvert
!        version  1.1  date 10-03-2003 Extension with contact angle
!
!   copyright (c) 2002-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM915
!     in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     element_matrix
!     element_vector
!     initialization
!     navier_stokes_equation
!     free_boundary
!     free_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, isub, isup, induv(ndim,*)

!     array          o    Real work array in which the values of the
!                         coefficients in the integration points are stored
!                         in the sequence:
!                                2D
!                         1:     rho
!                         2:     mu
!                         3:     gamma
!                         4:     t1
!                         5:     t2
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     induv          o    Index array of size ncomp x inpelm that contains
!                         the positions of the degrees of freedom 1 to ncomp
!                         in the element
!                         Hence induv(k,i) contains the position of the k^th
!                         degree of freedom in the i^th node of the element
!                         with respect to the element vector
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELxxxx. Possible vaues:
!                          2:  Subroutine ELP611  (Quadratic line element)
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncoef, i, j, isubcf, mrule, isum, inunkp

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     isubcf         Under bound for coefficients check
!     isum           Integer variable to compute a sum
!     j              Counting variable
!     mrule          Type of integration rule as given by the user
!     ncoef          Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
!     INSTOP         Stop the program
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!
!     The sequence is:
!
!     In the case of a call by BUILD:
!
!     1:
!     2:
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!     5:  Not yet in use:  set equal to 0
!     6:  rho
!     7:  mu
!     8:  gamma
!     9:  t1
!    10:  t2
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!      74   Incorrect number of degrees of freedom for this type
!    1766   Type of integration rule incorrect
!    1771   Shape number not yet implemented
!    1786   Shape number of element in combination with dimension not available
!    2065   incorrect number of unknowns in element
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0915' )
      call elcomm
      jtrans = 0
      ideriv = 1
      jderiv = 1
      ncomp  = ndim
      ndimlc = 1
      jind = 0
      icoor = 11

!     --- Set parameters depending on the shape numbers

      n = inpelm
      m = n
      mn = m * n
      jelp = ishape

      if ( icount<n*3 ) then

!     --- Number of unknowns incorrect

         call errint ( icount, 1 )
         call errint ( n*3, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 74, 5, 0, 0 )

      end if
      isum = 0
      iseqtype = oldsols(iseqin)%iseqtype
      index4n => indoldsols(iseqtype)%index4
      do i = 1, inpelm
         inunkp = index4n(i)-isum
         if ( inunkp<ncomp ) then
            call errint ( inunkp, 1 )
            call errint ( i, 2 )
            call errint ( ncomp, 3 )
            call errint ( ielem, 4 )
            call errint ( itype, 5 )
            call errint ( ielgrp, 6 )
            call errsub ( 2065, 6, 0, 0 )
         end if
         do j = 1, ncomp
            induv(j,i) = isum+j
         end do
         isum = isum+inunkp
      end do
      if ( jelp<=0 ) then

!     --- point element

         jdercn = 0

      else if ( jelp==2 ) then

!     --- quadratic element

         jdercn = 1
         nvert = 2

      else

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end if
      if ( ndim/=2 ) then
         call errint ( jelp, 1 )
         call errint ( ndim, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1786, 4, 0, 0 )
      end if

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficient

      ipos = iuser ( ielgrp + 5 )
      if ( ipos<7 ) then
         call errint ( ielgrp+5, 1 )
         call errint ( ipos, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 42, 5, 0, 0 )
      end if

      nuser = 0

!     --- Compute the parameter irule

      ipos = ipos + 2

      mrule = inirepscal ( iuser(ipos) )
      if ( jelp==-10 ) then
         irule = 1
      else if ( mrule==0 ) then
         irule = 4
      else
         irule = max ( 1, mrule )
      end if

      ipos = ipos + 1
      if ( mrule<0 .or. mrule>40 ) then
         call errint ( mrule, 1 )
         call errint ( 3, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1766, 5, 0, 0 )
      end if

      ipos = ipos + 2

      if ( irule/=1 ) then

!     --- irule>1, Gauss rule

         igausp = 1
         iphi = 1
         jdiag = 2
         if ( jelp<=2 ) then
            m = irule - 1
         end if
         mn = m * n

      end if

      ncoef = 5
      do i = 1 , ncoef
         call elusr3 ( i, array(1+(i-1)*m), iuser, user, i )
      end do

!     --- Test array lengths, number of nodes and number of degrees of freedom

      call prinlg ( 'iuser', ipos-1, user, iuser )
      call prinlg ( 'user', -nuser, user, iuser )

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      isubcf = 1

      do i = isubcf , ncoef
         isub = i

!        --- If ind(i)>0 then paramater i corresponds to a function

         if ( ind(i)>0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i)>0 ) goto 190
      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do
      ind(lengthmst) = 3
      do i = isub, isup
         if ( ind(i)>0 .and. ind(i)<2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j)==ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
      end do

!     --- Check if matrix and vector must be filled

      if ( ierror/=0 ) call instop
      notmas = 1
      call erclos ( 'el0915' )

      end
