      subroutine el8200 ( s, xg, x0, w, Fd, Jd, Fn, Jn, p, B, usrvec, &
                          nd, imatlaw, iusrvc, iusrflg, work, xg0, &
                          ncoef, index1, nsig, ichfib, maxusr )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 07-07-2007 Extra parameters ichfib, maxusr
!        version  2.0  date 13-04-2005 Extra parameters for axi-symmetry (RvL)
!                                      extra parameter for el8300
!                                      call to el8300
!                                      call to el8300
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the stress part of the element matrix for general
!     nonlinear solid elements and store in matrix s
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imatlaw, iusrvc, iusrflg, nd, ncoef, index1(*), nsig, &
              ichfib, maxusr
      double precision s(ndim*n,ndim*n), w(m), &
                       xg(m,ndim), x0(ndim,m), work(*), usrvec(m,*), &
                       Fd(m,ndim,ndim), Fn(m,ndim,ndim), xg0(m,ndim), &
                       Jd(m), Jn(m), p(m), B(nd,ndim,n,m)

!     b              i    Array containing the B-matrix
!     fd             i    Deformation gradient F(k,j,i) = F_ij(x^k)
!     fn             i    Deformation gradient at incr. n Fn(k,j,i) = F_ij(x^k)
!     ichfib         i    Indicates which fiber subroutine has to be used
!                         See el0200
!     imatlaw        i    Type of material constitutive equation to be used
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     iusrflg        i    User flag
!     iusrvc         i    if>0, a user defined vector with iusrvec dof's
!                         present
!     jd             i    Determinant of deformation gradient
!     jn             i    Determinant of deformation gradient at incr. n
!     maxusr         i    Maximum number of degrees of freedom in usrvec
!     ncoef          i    Number of coefficients in the equation
!     nd             i    Dimension of tangent stiffness matrix
!     nsig           i    ?
!     p              i    Array containing the pressure
!     s              o    Element matrix to be filled
!     usrvec         i    User vector
!     w              i    array of length m containing the weights for
!                         integration
!     work           i    Work array
!     x0             i    coordinates of undeformed mesh
!     xg             i    coordinates of integration points
!     xg0            i    ?
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, maxcoef, ilinJ
      parameter (maxcoef=50)
      double precision Fd_wrk(ndim,ndim),coef(maxcoef), &
                       Fn_wrk(ndim,ndim), &
                       D_t(nd,nd), sigma(nsig), &
                       D_f(nd,nd), D_j(nd,nd), &
                       D(nd,nd,n), corr(ndim), corr0(ndim), &
                       usr_wrk(maxusr), corrg0(ndim)

!     coef           Local coefficient array
!     corr           coordinates of current integration point
!     corr0          coordinates in undeformed configuration
!     corrg0         ?
!     d              Tangent stiffness matrix
!     d_f            Tangent stiffness matrix stress part
!     d_j            Tangent stiffness matrix volume part
!     d_t            Tangent stiffness matrix material part
!     fd_wrk         F_delta work array
!     fn_wrk         F_n work array
!     i              Counting variable
!     ilinJ          indicate whether D_j has to be computed (for incomp. mat.)
!     j              Counting variable
!     k              Counting variable
!     maxcoef        Maximum number of coefficients
!     sigma          Stress
!     usr_wrk        Uservector work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL8201         computes stress matrix for general nonlinear solid element
!     EL8202         computes volume matrix for general nonlinear solid element
!     EL8208         Compute element matrix for nonlinear elasticity element
!                    3D case from B' * D * B
!     EL8300         fills extra stress tensor and tangential stiffness matrix
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el8200' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el8200'
         write(irefwr,1) 'm, nd, ndim, iusrvc', m, nd, ndim, iusrvc
         call prinrl1 ( x0, m, ndim, 'x0' )
         call prinrl1 ( xg, m, ndim, 'xg' )
         call prinrl1 ( xg0, m, ndim, 'xg0' )
         call prinrl1 ( usrvec, iusrvc, m, 'usrvec' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over the integration points

      do k = 1, m

!     --- clear matrices

         do i = 1, nd
            do j = 1, nd
               D_f(i,j) = 0d0
               D_t(i,j) = 0d0
               D_j(i,j) = 0d0
            end do
         end do

!        --- compute coordinates of node undeformed mesh and in
!            current integration point

         do i = 1, ndim
            corr0(i) = x0(i,k)
            corr(i) = xg(k,i)
            corrg0(i) = xg0(k,i)
         end do

!        --- compute deformation gradient in current integration point

         do j = 1, ndim
            do i = 1, ndim
               Fd_wrk(i,j) = Fd(k,j,i)
               Fn_wrk(i,j) = Fn(k,j,i)
            end do
         end do

!        --- compute user vector in current integration point

         if (iusrvc>=1) then
            do i = 1, iusrvc
               usr_wrk(i) = usrvec(k,i)
            end do
         end if

!        --- get coefficients in current integration point

         do i = 1, ncoef
            coef(i) = work(k+m*(i-1))
         end do

!        --- compute D_t-matrix and sigma

         call el8300 ( Fd_wrk, Jd(k), Fn_wrk, Jn(k), p(k), usr_wrk, &
                       coef, D_t, sigma, corr0, corr, corrg0, index1(k), &
                       imatlaw, iusrvc, iusrflg, nd, 0, 1, ichfib )

!        --- compute matrix D_f

         call el8201 (D_f, sigma, nd)

!        --- check whether D_j has to be computed for incompressible solid

         ilinJ = nint(coef(7-5))

!        --- compute matrix D_j (not for TYPE=201/202, if ilinJ = 0)

         if (((itype/=201) .and. (itype/=202)) .or. &
              (ilinJ==1)) then

!        --- compute matrix D_j

            call el8202 (D_j, sigma, nd)

         end if
         if ( debug ) then
            write(irefwr,1) 'k', k
            call prinrl1 ( D_f, nd, nd, 'D_f' )
            call prinrl1 ( D_t, nd, nd, 'D_t' )
            call prinrl1 ( D_j, nd, nd, 'D_j' )
         end if  ! ( debug )

!        --- add matrices

         do i = 1, nd
            do j = 1, nd
               D(i,j,k) = D_f(i,j) + D_t(i,j) + D_j(i,j)
            end do
         end do

      end do

!     --- assemble element matrix  B^T * D * B

      call el8208 (s, B, D, w, nd, ilinJ)

1000  call erclos ( 'el8200' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( s, n*ndim, n*ndim, 'S' )
         write(irefwr,*) 'End el8200'

      end if  ! ( debug )

      end
