      subroutine el8246 ( sigma, xg, x0, Fd, Jd, Fn, Jn, p, usrvec, &
                          imatlaw, iusrvc, iusrflg, work, &
                          ncoef, index1, nd, xg0, ichfib )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.0  date 07-07-2007 Extra parameter ichfib
!        version  3.0  date 13-04-2005 Add integer nsig for axi-symmetry (RvL)
!                                      Extension for 2D shapes (RvL)
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the stress for general nonlinear solid element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imatlaw, iusrvc, iusrflg, ncoef, index1(m), nd, ichfib
      double precision Fd(m,ndim,ndim), Jd(m), Fn(m,ndim,ndim), Jn(m), &
                       sigma(6,m), work(*), p(m), xg(m,ndim), &
                       x0(ndim,m), usrvec(m,*), xg0(m,ndim)

!     fd             i    Deformation gradient F(k,j,i) = F_ij(x^k)
!     fn             i    Deformation gradient at incr. n Fn(k,j,i) = F_ij(x^k)
!     ichfib         i    Indicates which fiber subroutine has to be used
!                         See el0200
!     imatlaw        i    Type of material constitutive equation to be used
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     iusrflg        i    User defined flag
!     iusrvc         i    if>0, a user defined vector is used
!     jd             i    Determinant of deformation gradient
!     jn             i    Determinant of deformation gradient at incr. n
!     ncoef          i    Number of coefficients in the equation
!     nd             i    Dimension of tangent stiffness matrix
!     p              i    Array containing the pressure
!                         p(1) contains the pressure in the centroid
!                         p(2-4) contains the gradient of the pressure in the
!                         centroid
!     sigma          i    parameter sigma for boundary conditions of the type
!                         u is constant
!     usrvec         i    user defined vector
!     work           i    Work array
!     x0             i    ?
!     xg             i    Array of length 27x3 containing the coordinates of
!                         the Gauss points in the reference element for a
!                         27 point Gauss rule
!     xg0            i    ?
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, maxcoef, ichoice
      parameter (maxcoef=50)
      double precision Fd_wrk(ndim,ndim), Fn_wrk(ndim,ndim), &
                       coef(maxcoef), D_t(81), corr0(ndim), corr(ndim), &
                       usr_wrk(100), corrg0(ndim)

!     coef           Element coefficients
!     corr           Array of length ndim containing the
!                    co-ordinates of the nodal points
!     corr0          coordinates in undeformed configuration
!     corrg0         ?
!     d_t            Tangential stiffness matrix
!     fd_wrk         Work array for deformation gradient
!     fn_wrk         Work array for deformation gradient
!     i              Counting variable
!     ichoice        Integer choice parameter
!     j              Counting variable
!     k              Counting variable
!     maxcoef        Maximum number of coefficients
!     usr_wrk        Work array for user vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL8300         Fills extra stress tensor and tangential stiffness matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Loop over the integration points

      do k = 1, m

!     --- compute coordinates of current integration point

         do i = 1, ndim
            corr0(i) = x0(i,k)
            corr(i) = xg(k,i)
            if (jcart==2) corrg0(i) = xg0(k,i)
         end do

!        --- compute deformation gradient in current integration point

         do j = 1, ndim
            do i = 1, ndim
               Fd_wrk(i,j) = Fd(k,j,i)
               Fn_wrk(i,j) = Fn(k,j,i)
            end do
         end do

!        --- compute user vector in current integration point

         if (iusrvc>=1) then
            do i = 1, iusrvc
               usr_wrk(i) = usrvec(k,i)
            end do
         end if

!        --- get coefficients in current integration point

         do i = 1, ncoef
            coef(i) = work(k+m*(i-1))
         end do

!        --- determine ichoice
!            ix = 0 - 2: ichoice = 1
!            ix = 3    : ichoice = 3
!            ix = 4    : ichoice = 4

         if ((ix>=0) .and. (ix<=2)) then
            ichoice = 1
         else
            ichoice = ix
         end if

!        --- compute sigma

         call el8300 (Fd_wrk, Jd(k), Fn_wrk, Jn(k), p(k), usr_wrk, &
                      coef, D_t, sigma(1,k), corr0, corr, corrg0, &
                      index1(k), imatlaw, iusrvc, iusrflg, nd, ichoice, &
                      2, ichfib)

!        --- remove hydrostatic pressure when ix = 2

         if (ix==2) then
            do i = 1, ndim
               sigma(i,k) = sigma(i,k) + p(k)
            end do
            if (jcart==2) sigma(4,k) = sigma(4,k) + p(k)
         end if

      end do

      end
