      subroutine el8310 ( Fd, Jd, Fn, Jn, p, usrvec, coef, D_t, &
                          sigma, corr0, corr, nodpnt, iusrvc, iusrflg, &
                          ichoice, isubrt, nd, ichfib )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  6.0  date 06-01-2016 Use sepmoduleelem
!        version  5.0  date 07-07-2007 Extra parameter ichfib
!        version  4.0  date 01-11-2002 extra parameters
!                                      extra parameter for elmul03, el8300
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute stress and tangent stiffness for 3D composite
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice, iusrvc, iusrflg, isubrt, nodpnt, nd, ichfib
      double precision Fd(3,3), Jd, Fn(3,3), Jn, p, coef(*), D_t(9,9), &
                       sigma(6), corr0(3), corr(3), usrvec(*)

!     coef           i    Element coefficients
!     corr           i    Integration point coordinates
!     corr0          i    Coordinates in undeformed configuration
!     d_t            o    Matrix to be filled
!     fd             i    Deformation gradient F(k,j,i) = F_ij(x^k)
!     fn             i    Deformation gradient at start of timestep
!     ichfib         i    Indicates which fiber subroutine has to be used
!                         See el0200
!     ichoice        i    Calculate tang. stifness matrix+stress(0)
!                         or just stress(1)
!     isubrt         i    Calling subroutine
!     iusrflg        i    User defined flag
!     iusrvc         i    Flag is usrvec is filled (1) or not (0)
!     jd             i    Determinant of Fd
!     jn             i    Determinant of Fn
!     nd             i    Dimension of tangent stiffness matrix
!     nodpnt         i    Nodal point number
!     p              i    Hydrostatic pressure
!     sigma          o    stress to be filled
!     usrvec         i    User vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, mtype, idx1(114)
      double precision D_tau_hat(9,9), D_psi(9,9), &
                       tau_hat(6), tau_psi(6), F(3,3), &
                       e0(3,3), e(3), fibcoef(10), &
                       psi, dpsi, ee6(6), sigfib(10), lamfib(10)
      double precision D1(9,9), D3(9,9), corrg0(ndim), &
                       lambda, lambda2, ee3(3,3), ee9(9)

!     corrg0         ?
!     d1             array of length modecl x modecl containing the values
!                    of the derivatives of the basis functions in the
!                    x-direction
!     d3             ?
!     d_psi          Total fiber tangential stiffness
!     d_tau_hat      Matrix tangential stiffness
!     dpsi           ?
!     e              Fiber directions in deformed configuration
!     e0             Fiber directions in undeformed configuration
!     ee3            ?
!     ee6            ?
!     ee9            ?
!     f              Total deformation tensor
!     fibcoef        Fiber coefficients
!     i              Counting variable
!     idx1           Indices to fill stress vector
!     j              Counting variable
!     k              Counting variable
!     lambda         ?
!     lambda2        Normalization tensor
!     lamfib         ?
!     mtype          Type of comstitutive equation for the matrix
!     psi            Array of length n x m containing the values of the
!                    modified basis functions in the integration points
!     sigfib         ?
!     tau_hat        Matrix stress
!     tau_psi          Total fiber stress
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2731         ?
!     EL8300         fills extra stress tensor and tangential stiffness matrix
!     ELMUL03        ?
!     FNFIBDIR       ?
!     FNFIBMAT       ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx1 /  6, 1,1, 1,1, 4,7, 6,6, 7,7, 9,6 , &
                   6, 2,2, 2,2, 4,4, 5,8, 7,4, 8,8 , &
                   6, 3,3, 3,3, 5,5, 6,9, 8,5, 9,9 , &
                  12, 1,4, 1,4, 2,7, 2,7, 4,1, 4,2 , &
                      5,6, 6,8, 7,1, 7,2, 8,6, 9,8 , &
                  12, 2,5, 2,5, 3,8, 3,8, 4,9, 5,2 , &
                      5,3, 6,4, 7,9, 8,2, 8,3, 9,4 , &
                  12, 1,9, 1,9, 3,6, 3,6, 4,5, 5,7 , &
                      6,1, 6,3, 7,5, 8,7, 9,1, 9,3 /
! ======================================================================
!
!     --- clear matrices

      do i = 1, 9
         do j = 1, 9
            D_tau_hat(i,j) = 0d0
            D_psi(i,j) = 0d0
         end do
      end do
      do i = 1, 6
         tau_hat(i) = 0d0
         tau_psi(i) = 0d0
      end do
      lambda2 = 0d0

!     --- get type number for the matrix constitutive equation

      mtype = nint(coef(30-5))

!     --- fill in tangential stiffness (D_tau_hat) and/or stress (tau_hat) for
!         the matrix from the selected constitutive law
!         !! hydrostatic pressure is set to zero here, to get tau !!

      call el8300 (Fd, Jd, Fn, Jn, 0d0, usrvec, coef, D_tau_hat, &
                   tau_hat, corr0, corr, corrg0, nodpnt, mtype, &
                   iusrvc, iusrflg, nd, ichoice, isubrt, ichfib)

!     --- compute F = Fd * Fn

      call elmul03 (Fd, Fn, F, 0, ndim)

!     --- loop over number of layers (coef32)

      do k = 1, nint(coef(32-5))

!     --- clear all matrices

         do i = 1, 9
            do j = 1, 9
               D1(i,j) = 0d0
               D3(i,j) = 0d0
            end do
         end do

!        --- fill in fiber material parameters
!            coef(35-45) : material coefficients (fibcoef)

         do i = 1, 10
            fibcoef(i) = coef(35-5+(i-1))
         end do

!        --- fill in fiber direction in the undeformed configuration (e0)

         call fnfibdir (e0, k, ielgrp, nodpnt, corr0, corr, iusrvc, &
                        usrvec, iusrflg, isubrt)

!        --- compute current local (fiber) coordinate system
!            lambda e = F * e0

         do i = 1, 3
            e(i) = 0d0
            do j = 1, 3
               e(i) = e(i) + F(i,j)*e0(j,1)
            end do
         end do

         lambda = sqrt(e(1)**2+e(2)**2+e(3)**2)
         lamfib(k) = lambda
         lambda2 = lambda**2
         e(1) = e(1)/lambda
         e(2) = e(2)/lambda
         e(3) = e(3)/lambda

!        --- ee = e*e'

         do i = 1, 3
            do j = 1, 3
               ee3(i,j) = e(i)*e(j)
            end do
         end do

         ee6(1) = ee3(1,1)
         ee6(2) = ee3(2,2)
         ee6(3) = ee3(3,3)
         ee6(4) = ee3(1,2)
         ee6(5) = ee3(2,3)
         ee6(6) = ee3(3,1)

         do i = 1, 6
            ee9(i) = ee6(i)
         end do
         do i = 7, 9
            ee9(i) = ee6(i-3)
         end do

!        --- compute fiber stress and tangent stiffness depending
!            on the constitutive law (local axes)

         call fnfibmat (psi, dpsi, lambda, fibcoef, nodpnt, corr0, corr, &
                        iusrvc, usrvec, iusrflg, isubrt)

!        --- compute stress update

         do i = 1, 6
            tau_psi(i) = tau_psi(i) + psi*ee6(i)
         end do
         sigfib(k) = psi

!        --- compute tangent stiffness (if required)

         if (ichoice==0) then

            call el2731 (D1, idx1, 6, ee6)

            do i = 1, 9
               do j = 1, 9
                  D3(i,j) = 2d0*ee9(i)*ee9(j)
               end do
            end do

!           --- compute tangent stiffness update

            do i = 1, 9
                 do j = 1, 9
                    D_psi(i,j) = D_psi(i,j) + &
                                 psi*D1(i,j) + &
                                 (dpsi*lambda2 - psi)*D3(i,j)
                 end do
              end do

         end if

!        --- end loop over fiber layers

      end do

!     --- compute total extra stress

!         --- compute stress to return depending on ichoice

      if (ichoice==3) then

!     --- ichoice = 3: global fiber stress

         do i = 1, 6
            sigma(i) = tau_psi(i)
         end do

      else if (ichoice==4) then

!     --- ichoice = 4: local fiber stress

         do k = 1, nint(coef(32-5))
            sigma(k) = sigfib(k)
            sigma(k+2) = lamfib(k)
         end do
         sigma(5) = 0d0
         sigma(6) = 0d0

      else

!     --- default case, return total stress

         do i = 1, 3
            sigma(i) = -p + tau_hat(i) + tau_psi(i)
         end do
         do i = 4, 6
            sigma(i) = tau_hat(i) + tau_psi(i)
         end do

      end if

      if (ichoice==0) then

!     --- compute total tangential stiffness matrix

         do i = 1, 9
            do j = 1, 9
               D_t(i,j) = D_tau_hat(i,j) + D_psi(i,j)
            end do
         end do

      end if

      end
