      subroutine elm426 ( vecloc, array )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.0  date 06-01-2016 Use sepmoduleelem
!        version  6.7  date 09-03-2013 Replace elflr7 by elflr71
!        version  6.6  date 12-12-2012 Long integers
!
!   copyright (c) 1986-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for natural boundary conditions for
!     boussinesq equation temperature part
!     Two dimensional elements
!     itype=426 :    cartesian coordinates   quadratic line element
!     itype=427 :    axi-symmetric coordinates (no swirl)
!                    quadratic line element
!     itype=428 :    polar coordinates (no swirl)
!                    quadratic line element
!     itype=438 :    axi-symmetric coordinates with swirl
!                    quadratic line element
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     boussinesq
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/celwrkf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*), array(*)

!     array          Work array for the element subroutines
!     vecloc         Work array in which all old solution vectors for the
!                    integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, j, ipphi, ipphider, ipend
      double precision qmat(12), x(6), w(3), s1(3,3), fr(3), uold(9), &
                       ugauss(9), dphidx(18), xgauss(6)
      logical funchk
      save isub, isup, ipphi, ipphider

!     dphidx         Array of length n x m x ndim  containing the
!                    derivative
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     f              array of length m containing the rhs vector
!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              Counting variable
!     ipend          Last position in array
!     ipphi          Starting address of array phi in array
!     ipphider       ?
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     j              Counting variable
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     s              Element matrix to be filled
!     ugauss         Preceding solution in Gauss points
!     uold           Array containing the previous iteration (old solution)
!     w              array of length m containing the weights for
!                    integration
!     x              x co-ordinate of point
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3002         Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                                   e            i       j
!     EL3003         Compute integrals of the shape / fr(x) phi (x) dx
!                                                   e         i
!     EL3007         Fills old solution and derivatives
!     EL3009         Fill series of old solutions in array vecloc
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELP616         Compute basis functions and weights for quadratic line
!                    element R2
!     ELUSR3         Extracts information concerning the coefficients from E%iuser
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!    The user provides the coefficients through the arrays E%iuser and USER
!    in the usual way
!    The sequence is:
!    1:  sigma
!    2:  h
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       3   Element has incorrect number of nodal points for this type
!      42   Array iuser(.) has value which is too small
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!       element matrix and element vector
!       S    =  /  sigma phi  phi  dx
!        ij     e           i    j
!       f   = /  h phi  dx
!        i    e       i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm426' )
      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         call elcomm
         if ( itype==438 ) then

!        --- itype = 438  (swirl)

            jcart = 1
            ncomp = 4

         else

!        --- itype = 426, 427, 428

            jcart = itype-425
            ncomp = 3

         end if
         ideriv = 0

         n = 3
         m = 3
         mn = 9

         ipos = E%iuser ( ielgrp + 5 )
         if ( ipos<8 ) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 42, 5, 0, 0 )
            go to 1000
         end if
         nuser = 0
         do i = 1, 2
            call elusr3 ( i, array(1+(i-1)*m), E%iuser, E%user, i )
         end do

!        --- Test array lengths, number of nodes and number of degrees
!            of freedom

         call prinlg ( 'E%iuser', ipos-1, E%user, E%iuser )
         call prinlg ( 'user', -nuser, E%user, E%iuser )

         if ( inpelm/=n ) then
            call errint ( inpelm, 1 )
            call errint ( n, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 3, 5, 0, 0 )
            go to 1000
         end if

!        --- icount is not used, only checked in this subroutine

         if ( icount/=n*ncomp ) then
            call errint ( icount, 1 )
            call errint ( n*ncomp, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 3, 5, 0, 0 )
            go to 1000
         end if
         ipphi = 7
         ipphider = ipphi+3*m
         ipend = ipphider+9
         if ( ipend>50 ) then

!        --- array wrk1 too small

            call errint ( ipend, 1 )
            call errint ( 25, 2 )
            call errchr ( 'array', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

!        --- Set ind(lengthmst)

         do i = 1, 2
            ist(i) = 0
         end do
         ind(lengthmst) = 0
         funchk = .false.
         isub = 3
         isup = 0
         jadd = 0
         if ( matrix ) then

!        --- matrix = .true.  sigma

            if ( ind(1)==0 ) then
               notmat = 1
            else if ( ind(1)<0 ) then
               jzero = 3
            else
               jzero = 2
               isub = 1
               isup = 1
            end if
            do i = 1, icount
               do j = 1, icount
                  elemmt(i,j) = 0d0
               end do
            end do
            if ( ind(1)>1000 .and. ind(1)<=2000 ) ind(lengthmst) = 1
            if ( ind(i)>10000 ) funchk = .true.
         end if
         if(vector) then

!        --- vector = true    h

            if ( ind(2)==0 ) then
               notvec = 1
            else
               jconsf = 2
               if ( ind(2)>0 ) then
                  isup = 2
                  isub = min ( isub, 2 )
               end if
            end if
            do i = 1,icount
               elemvc(i) = 0d0
            end do
            if ( ind(2)>1000 .and. ind(2)<=2000 ) ind(lengthmst) = 1
            if ( ind(i)>10000 ) funchk = .true.
         end if
         if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )

      end if

!     --- variable part

      call elp616 ( coor, x, w, dphidx, xgauss, array(ipphi), index1, &
                    qmat, work, array(ipphider) )
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) &
         call el3007 ( uold, work, array(ipphi), dphidx, m, n, &
                       ndimlc, ncomp, index1, work, ugauss, &
                       jtrans, numold, 1 )

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, array(ipphi), n )

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, E%user, array, x, xgauss, &
                          m, n, array(ipphi), 1, index1, &
                          E%iuser, ugauss, vecloc )

         end if

      end do

      if ( matrix .and. notmat==0 ) then

!     --- fill element matrix

         call el3002 ( array, s1, w, array(ipphi), m, n, n )
         do i = 1,inpelm
            do j = 1,inpelm
               elemmt(ncomp*j,i*ncomp) = s1(j,i)
            end do
         end do

      end if
      if(vector .and. notvec==0) then

!     --- fill element vector

         call el3003 ( array(1+m), fr, w, array(ipphi), m ,n )
         do i = 1, inpelm
            elemvc(ncomp*i) = fr(i)
         end do

      end if
1000  if ( ierror/=0 ) call instop
      call erclos ( 'elm426' )
      end
