      subroutine elm924
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 07-04-2004 Extension for quadratic elements
!        version  1.0  date 23-04-2003
!
!   copyright (c) 2003-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill element matrix and element vector for coupling elements of
!    type 109. Fluid and solid nodes are coupled by means of lagrange
!    multipliers.
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     element
!     element_matrix
!     element_vector
!     intersection
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i,j, inpelmstruct, inpelmfluid, ishape_curve, ishapelam, &
              index(9), ilam , igauss1, ngauss, nlam, idx1, idx2
      double precision xgauss(9), sloc(3,16), wh, w(3), xi, phistr(9), &
                       phifld(7), philam(3,3)
      logical found

!     found          If true already a possibility has been found
!     i              Counting variable
!     idx1           ?
!     idx2           ?
!     igauss1        Counter for loop over integration points
!     ilam           ?
!     index          Help array to store nodal point numbers
!     inpelmfluid    Number of points per fluid element
!     inpelmstruct   Number of points per structural element
!     ishape_curve   Shape of lambda basis functions
!     ishapelam      Shape of lambda basis functions
!     j              Counting variable
!     ngauss         ?
!     nlam           ?
!     phifld         Basis functions for fluid
!     philam         Basis functions for lagrange multipliers
!     phistr         Basis functions for structure
!     sloc           Local element matrix
!     w              Array of length m containing the weights for
!                    integration
!     wh             Help variable
!     xgauss         Array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!     xi             Local coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELMFICTLAM     Compute the integration points and basis functions for the
!                    Lagrangian multipliers
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRFINDELM1     Finds local coordinate of a coordinate (x,y) on
!                    a (curved) line
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm924' )

      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm924'

      end if

      if ( ndim.eq.2 ) then

         inpelmstruct = 3
         inpelmfluid = 3
         ishape_curve = 2
         ishapelam = 2

      else if ( ndim.eq.3 ) then

         inpelmstruct = 9
         inpelmfluid = 7
         ishape_curve = 3
         ishapelam = 6

      end if

      if ( ishapelam.eq.2 ) then

         nlam = 2
         ngauss = 2

      else if ( ishapelam.eq.6 ) then

         nlam = 3
         ngauss = 3

      end if

!     --- clear matrix and rhs

      do i = 1, icount
         do j = 1, icount
            E%mat(i,j) = 0d0
         end do
         E%vec(i) = 0d0
      end do

!     --- compute position of lagrange multipliers

      do i = 1, inpelmfluid
         index(i) = index1(i)
      end do

      call elmfictlam ( coor, index, ishape_curve, ndim, xgauss, &
                        philam, w, ishapelam, .true. )

!     --- loop over lagrange multipliers

      do ilam = 1, nlam

!     --- compute basis functions for fluid part

         do i = 1, inpelmfluid
            index(i) = index1(i)
         end do

         call prfindelm1 ( coor, index, xgauss, ilam, xi, found, &
                           phifld, ishapelam )

!        --- compute basis functions for structure part

         do i = 1, inpelmstruct
            index(i) = index1( inpelmfluid + inpelmstruct*(ilam-1) + i )
         end do

         call prfindelm1 ( coor, index, xgauss, ilam, xi, found, &
                           phistr, ishapelam )

!        --- loop over gauss points

         do igauss1 = 1, ngauss

!        --- build element matrix

            wh = w(ilam)*philam(igauss1,ilam)

!           --- structure part

            do j = 1, inpelmfluid
               sloc(igauss1,j) = wh*phifld(j)
            end do

!           --- fluid part

            do j = 1, inpelmstruct
               sloc(igauss1,j+inpelmfluid) = -wh*phistr(j)
            end do

         end do

!        --- fill element matrix

!            --- loop over gauss points

         do igauss1 = 1, ngauss

            if (ndim.eq.2) then

!           --- loop over first two fluid nodes

               do j = 1, inpelmfluid-1

                  do i = 1, ndim

                  idx1 = (inpelmfluid-1)*ndim + (igauss1-1)*ndim + i
                  idx2 = (j-1)*ndim + i
                     E%mat(idx1, idx2) = elemmt(idx1, idx2) + &
                                                         sloc(igauss1,j)
                     E%mat(idx2, idx1) = elemmt(idx2, idx1) + &
                                                         sloc(igauss1,j)

                  end do

               end do

!              --- loop over third fluid node

               j = inpelmfluid
               do i = 1, ndim

                  idx1 = (inpelmfluid-1)*ndim + (igauss1-1)*ndim + i
                  idx2 = (inpelmfluid-1)*ngauss + (j-1)*ndim + i
                  E%mat(idx1, idx2) = elemmt(idx1, idx2) + &
                                                       sloc(igauss1,j)
                  E%mat(idx2, idx1) = elemmt(idx2, idx1) + &
                                                       sloc(igauss1,j)

               end do

!              --- loop over solid nodes

               do j = inpelmfluid+1, inpelmfluid+inpelmstruct

                  do i = 1, ndim

                     idx1 = (inpelmfluid-1)*ndim + (igauss1-1)*ndim + i
                     idx2 = ngauss*ndim + (ilam-1)*inpelmstruct*ndim + &
                                                          (j-1)*ndim+i
                     E%mat(idx1, idx2) = elemmt(idx1, idx2) + &
                                                         sloc(igauss1,j)
                     E%mat(idx2, idx1) = elemmt(idx2, idx1) + &
                                                         sloc(igauss1,j)

                  end do

               end do

            else if (ndim.eq.3) then

!           --- loop over the fluid nodes

               do j = 1, inpelmfluid

                  do i = 1, ndim

                     idx1 = inpelmfluid*ndim + (igauss1-1)*ndim + i
                     idx2 = (j-1)*ndim + i
                     E%mat(idx1, idx2) = elemmt(idx1, idx2) + &
                                                         sloc(igauss1,j)
                     E%mat(idx2, idx1) = elemmt(idx2, idx1) + &
                                                         sloc(igauss1,j)

                  end do

               end do

!              --- loop over solid nodes

               do j = inpelmfluid+1, inpelmfluid+inpelmstruct

                  do i = 1, ndim

                     idx1 = inpelmfluid*ndim + (igauss1-1)*ndim + i
                     idx2 = ngauss*ndim + (ilam-1)*inpelmstruct*ndim + &
                                                          (j-1)*ndim+i
                     E%mat(idx1, idx2) = elemmt(idx1, idx2) + &
                                                         sloc(igauss1,j)
                     E%mat(idx2, idx1) = elemmt(idx2, idx1) + &
                                                         sloc(igauss1,j)

                  end do

               end do

            end if

         end do !end loop over gauss points

      end do !end loop over lagrange multipliers

      if ( debug ) write(irefwr,*) 'End elm924'
      call erclos ( 'elm924' )

      end
