      subroutine elp634 ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          phiksi, psiksi, psi, dpsi )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 10-06-2013 Use subroutines
!        version  1.1  date 14-11-2007 Extension with diagonal mass matrix
!
!   copyright (c) 2001-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for triquadratic brick
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP634 is called by several types of element subroutines
!     Preliminary version, derived from elp300
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*)
      double precision coor2d(3,*), x(27,3), w(m), dphidx(27,m,3), &
                       xgauss(m,3), phi(27,m), phiksi(27,m,3), psi(8,m), &
                       dpsi(8,m,3), psiksi(8,27,3)

!     coor2d           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dpsi           i    ?
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     phi            o    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phiksi         o    array of length 6 x 6 to store the values of dphi/dksi
!                         according to:
!                                     k
!                         dphi /dksi(x ) = phiksi(i,k)
!                             i
!     psi            o    Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     psiksi         i    Array of length 8 x m x 3 containing the values of the
!                         derivatives of the basis functions in the Gauss points
!                         in the reference element
!     w              o    array of length m containing the weights for
!                         integration
!     x             i/o   array of length 3 x 27 containing the x- y- and
!                         z-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3);
!                          i              i              i
!     xgauss         o    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indi(27,3), inds(27,3), ih, ih1, i, k, mrule, m2, isign, &
              jsign, j, iweigo, indpsi(8,3), intrule, idx(8)
      double precision weight(27), theta(3,3), s, lambda(3,3), &
                       dlamds(3,3), som, dxdksi(27,3,3), detinv(27,3,3), &
                       dtetds(3,3)
      logical check, comppsi
      character (len=18) text
      save weight, dtetds

!     check          Logical variable that is used to indicate if the Jacobian
!                    is zero (true) or not (false)
!     comppsi        if true the linear basis functions must be evaluated
!     detinv         inverse of determinant
!     dlamds
!     dtetds         d theta_i / ds in integration points
!     dxdksi         d x_i / d xsi integration points
!     i              Counting variable
!     idx            ?
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     indi           Array in which the triples indi, indj and indk are stored
!     indpsi         Array in which the triples indi, indj and indk are stored
!                    for the linear element
!     inds           Array in which the triples inds1, inds2 and inds3 are
!                    stored
!     intrule        Actual intergration rule
!     isign          Help parameters to check the sign of the Jacobian in the
!                    integration points
!     iweigo         if 1 the weights must be filled
!     j              Counting variable
!     jsign          Help parameter to compute sign of the Jacobian
!     k              Counting variable
!     lambda         Contains the one-dimensional linear basis functions in the
!                    integration points
!     m2             ?
!     mrule          Type of integration rule as given by the user
!     s
!     som            Help parameter to compute a sum
!     text           String variable containing some text
!     theta          1D basis funtion in integration points
!     weight         Help variable to compute the weights
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-diemsional array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
!     230   Jacobian of element is zero or has changing sign
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx / 1, 2, 4, 3, 5, 6, 8, 7 /
      data text / 'Triquadratic brick' /
! ======================================================================
!
      call eropen ('elp634')
      intrule = mod(irule,100)
      mrule = 0
      iweigo = mod(iweigh,10)
      if (ifirst==0 .or. ifirst==2) then

!     --- ifirst = 0  (First call) or
!         ifirst = 2, first call to store dphidx in the nodes
!                     phiksi is filled differently from the previous
!                     values, hence use another array
!                     This value of ifirst should be used in combination with
!                     ifirst = 3 and is meant for elup04 only

         comppsi = jpsi==1
         if (jcart/=1) then

!        --- Error 1769: Co-ordinate system not available

            call errorelvar ( jcart-1, 1769, text, 0 )

         end if
         if (intrule<1 .or. intrule>3) then

!        --- Error 1770: Integration rule not available

            call errorelvar ( intrule, 1770, text, 1 )
            intrule = 1
         end if
         if (intrule==1 .and. m/=27 .or. &
             intrule==2 .and. m/=8 .or. &
             intrule==3 .and. m/=27) then

!        --- Error 1424: m and integration rule do not match

            call errorelvar2 ( m, intrule, 1424, text, 0 )

         end if

!        --- Fill arrays theta and dtetds in integration points

         if ( intrule==1 ) then

!        --- intrule = 1  Newton Cotes rule

            do i = 1, 3
               do k = 1,3
                  theta(i,k) = 0d0
               end do
               theta(i,i) = 1d0
               s = (i-1) * 0.5d0
               dtetds(1,i) = 4d0 * s - 3d0
               dtetds(2,i) = -8d0 * s + 4d0
               dtetds(3,i) = 4d0 * s - 1d0
            end do
            w(1) = 1d0/6d0
            w(2) = 4d0*w(1)
            w(3) = w(1)
            mrule = 3
            if ( comppsi ) then

!           --- Compute linear basis functions

               do i = 1, 2
                  lambda(i,1) = 2 - i
                  lambda(i,2) = 0.5d0
                  lambda(i,3) = i - 1
               end do
               dlamds(1,1) = -1d0
               dlamds(1,2) = -1d0
               dlamds(1,3) = -1d0
               dlamds(2,1) =  1d0
               dlamds(2,2) =  1d0
               dlamds(2,3) =  1d0
            end if
         else if ( intrule==2 ) then

!        --- intrule = 2  Eight point Gauss rule

            do k = 1, 2
               theta(1,k) = 2d0 * (alam4(k)-0.5d0) * (alam4(k)-1d0)
               theta(2,k) = -4d0 * alam4(k) * (alam4(k)-1d0)
               theta(3,k) = 2d0 * alam4(k) * (alam4(k)-0.5d0)
               dtetds(1,k) = 4d0 * alam4(k) - 3d0
               dtetds(2,k) = -8d0 * alam4(k) + 4d0
               dtetds(3,k) = 4d0 * alam4(k) - 1d0
               w(k) = .5d0
            end do
            if ( comppsi ) then

!           --- Compute linear basis functions

               do k = 1, 2
                  lambda(1,k) = alam4(k)
                  lambda(2,k) = 1d0-alam4(k)
                  dlamds(1,k) = 1d0
                  dlamds(2,k) = -1d0
               end do
            end if
            mrule = 2
         else if ( intrule==3 ) then

!        --- intrule = 3  Twenty seven point Gauss rule

            do k = 1, 3
               theta(1,k) = 2d0 * (alam5(k)-0.5d0) * (alam5(k)-1d0)
               theta(2,k) = -4d0 * alam5(k) * (alam5(k)-1d0)
               theta(3,k) = 2d0 * alam5(k) * (alam5(k)-0.5d0)
               dtetds(1,k) = 4d0 * alam5(k) - 3d0
               dtetds(2,k) = -8d0 * alam5(k) + 4d0
               dtetds(3,k) = 4d0 * alam5(k) - 1d0
               w(k) = wgs5(k)
            end do
            mrule = 3
            if ( comppsi ) then

!           --- Compute linear basis functions

               do k = 1, 3
                  lambda(1,k) = alam5(k)
                  lambda(2,k) = 1d0-alam5(k)
                  dlamds(1,k) = 1d0
                  dlamds(2,k) = -1d0
               end do
            end if
         end if

         m2 = mrule**2

!        --- fill array indi

         do k = 1, 3
            ih = (k-1)*9
            do j = 1, 3
               ih1 = ih+(j-1)*3
               do i = 1, 3
                  indi(i+ih1,1) = i
                  indi(i+ih1,2) = j
                  indi(i+ih1,3) = k
               end do
            end do
         end do

         if ( comppsi ) then

!        --- Compute linear basis functions

            do k = 1, 2
               indpsi(1+4*(k-1),1) = 1
               indpsi(1+4*(k-1),2) = 1
               indpsi(1+4*(k-1),3) = k
               indpsi(2+4*(k-1),1) = 2
               indpsi(2+4*(k-1),2) = 1
               indpsi(2+4*(k-1),3) = k
               indpsi(3+4*(k-1),1) = 2
               indpsi(3+4*(k-1),2) = 2
               indpsi(3+4*(k-1),3) = k
               indpsi(4+4*(k-1),1) = 1
               indpsi(4+4*(k-1),2) = 2
               indpsi(4+4*(k-1),3) = k
            end do
         end if

!        --- fill array inds

         do k = 1, mrule
            ih = (k-1)*m2
            do j = 1, mrule
               ih1 = ih+(j-1)*mrule
               do i = 1, mrule
                  inds(i+ih1,1) = i
                  inds(i+ih1,2) = j
                  inds(i+ih1,3) = k
               end do
            end do
         end do

!        --- Fill array weight

         if (iweigo>0) then
            do k = 1, m
               weight(k) = w(inds(k,1))*w(inds(k,2))*w(inds(k,3))
            end do
         end if

!        --- Fill phi

         if (iphi==1) then
            do i = 1, 27
               do k = 1, m
                  phi(i,k) = theta(indi(i,1),inds(k,1))* &
                             theta(indi(i,2),inds(k,2))* &
                             theta(indi(i,3),inds(k,3))
               end do
            end do

            if ( comppsi ) then

!           --- Compute linear basis functions

               do i = 1, 8
                  do k = 1, m
                     psi(idx(i),k) = lambda(indpsi(i,1),inds(k,1))* &
                                     lambda(indpsi(i,2),inds(k,2))* &
                                     lambda(indpsi(i,3),inds(k,3))
                  end do
               end do
            end if
         end if

!        --- fill phiksi

         do i = 1, 27
            do k = 1, m
               phiksi(i,k,1) = dtetds(indi(i,1),inds(k,1))* &
                               theta(indi(i,2),inds(k,2))* &
                               theta(indi(i,3),inds(k,3))
               phiksi(i,k,2) = theta(indi(i,1),inds(k,1))* &
                               dtetds(indi(i,2),inds(k,2))* &
                               theta(indi(i,3),inds(k,3))
               phiksi(i,k,3) = theta(indi(i,1),inds(k,1))* &
                               theta(indi(i,2),inds(k,2))* &
                               dtetds(indi(i,3),inds(k,3))
            end do
         end do

!        --- fill psiksi

         do i = 1, 8
            do k = 1, m
               psiksi(i,k,1) = dlamds(indpsi(i,1),inds(k,1))* &
                               lambda(indpsi(i,2),inds(k,2))* &
                               lambda(indpsi(i,3),inds(k,3))
               psiksi(i,k,2) = lambda(indpsi(i,1),inds(k,1))* &
                               dlamds(indpsi(i,2),inds(k,2))* &
                               lambda(indpsi(i,3),inds(k,3))
               psiksi(i,k,3) = lambda(indpsi(i,1),inds(k,1))* &
                               lambda(indpsi(i,2),inds(k,2))* &
                               dlamds(indpsi(i,3),inds(k,3))
            end do
         end do
      end if

!     --- icoor>1   Fill co-ordinates of nodes in array x

      if ( icoor>=1 ) then
         do k = 1, 3
            do i = 1, 27
               if (icoor==1) x(i,k) = coor2d(k,index1(i))
               if (icoor==2) x(i,k) = coor2d(k,i)
            end do
         end do
      end if

!     --- First store dx / dksi etc. in array dxdksi

      do i = 1, m
         dxdksi(i,1,1) = ddot ( 27, x(1,1), 1, phiksi(1,i,1), 1 )
         dxdksi(i,2,1) = ddot ( 27, x(1,1), 1, phiksi(1,i,2), 1 )
         dxdksi(i,3,1) = ddot ( 27, x(1,1), 1, phiksi(1,i,3), 1 )
         dxdksi(i,1,2) = ddot ( 27, x(1,2), 1, phiksi(1,i,1), 1 )
         dxdksi(i,2,2) = ddot ( 27, x(1,2), 1, phiksi(1,i,2), 1 )
         dxdksi(i,3,2) = ddot ( 27, x(1,2), 1, phiksi(1,i,3), 1 )
         dxdksi(i,1,3) = ddot ( 27, x(1,3), 1, phiksi(1,i,1), 1 )
         dxdksi(i,2,3) = ddot ( 27, x(1,3), 1, phiksi(1,i,2), 1 )
         dxdksi(i,3,3) = ddot ( 27, x(1,3), 1, phiksi(1,i,3), 1 )
      end do

      if ( igausp==1 ) then

!     --- igausp = 1   fill gauss points

         if ( intrule==1 ) then

!        --- intrule = 1  Newton Cotes rule

            do k = 1, 3
               do i = 1, 27
                  xgauss(i,k) = x(i,k)
               end do
            end do
         else
            do i = 1, m
               xgauss(i,1) = ddot ( 27, x(1,1), 1, phi(1,i), 1 )
               xgauss(i,2) = ddot ( 27, x(1,2), 1, phi(1,i), 1 )
               xgauss(i,3) = ddot ( 27, x(1,3), 1, phi(1,i), 1 )
            end do
         end if
      end if

      if ( iweigo==1 .or. ideriv==1 ) then

!     --- ideriv = 1  Compute derivatives or iweigo = 1: weights
!                     Compute determinant in integration points
!                     Store temporarily in array w
!                     compute inverse of Jacobian

         do k = 1, m
            detinv(k,1,1) = dxdksi(k,2,2)*dxdksi(k,3,3)- &
                            dxdksi(k,3,2)*dxdksi(k,2,3)
            detinv(k,2,1) = dxdksi(k,3,2)*dxdksi(k,1,3)- &
                            dxdksi(k,1,2)*dxdksi(k,3,3)
            detinv(k,3,1) = dxdksi(k,1,2)*dxdksi(k,2,3)- &
                            dxdksi(k,2,2)*dxdksi(k,1,3)
            w(k) = dxdksi(k,1,1)*detinv(k,1,1)+dxdksi(k,2,1)* &
                   detinv(k,2,1)+dxdksi(k,3,1)*detinv(k,3,1)
         end do

         if ( ideriv==1 ) then

!        --- ideriv = 1,  compute all factors of inverse

            do k = 1,m
               detinv(k,1,2) = dxdksi(k,2,3)*dxdksi(k,3,1)- &
                               dxdksi(k,3,3)*dxdksi(k,2,1)
               detinv(k,2,2) = dxdksi(k,3,3)*dxdksi(k,1,1)- &
                               dxdksi(k,1,3)*dxdksi(k,3,1)
               detinv(k,3,2) = dxdksi(k,1,3)*dxdksi(k,2,1)- &
                               dxdksi(k,2,3)*dxdksi(k,1,1)
               detinv(k,1,3) = dxdksi(k,2,1)*dxdksi(k,3,2)- &
                               dxdksi(k,3,1)*dxdksi(k,2,2)
               detinv(k,2,3) = dxdksi(k,3,1)*dxdksi(k,1,2)- &
                               dxdksi(k,1,1)*dxdksi(k,3,2)
               detinv(k,3,3) = dxdksi(k,1,1)*dxdksi(k,2,2)- &
                               dxdksi(k,2,1)*dxdksi(k,1,2)
            end do
         end if

!        --- check sign of jacobian

         isign = nint( sign (1d0, w(1)) )
         check = .false.
         do i = 1, m
            if ( abs(w(i))<=0d0 ) check = .true.
            jsign = nint(sign(1d0, w(i)))
            if( jsign/=isign ) then
               if ( isubr==0 ) then
                  call errint ( ielem, 1 )
                  call errint ( itype, 2 )
                  call errint ( ielgrp, 3 )
                  call errsub ( 230, 3, 0, 0 )
               else
                  call errint ( ielgrp, 1 )
                  call errint ( ielem, 2 )
                  call errchr ( 'volume', 1 )
                  call errsub ( 1540, 2, 0, 1 )
               end if
               call prinin(index1,27,'node numbers')
               call prinrl2(x,27,3,'co-ordinates')
               go to 1000
            end if
         end do
         if ( check ) then
            if ( isubr==0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 105, 3, 0, 0 )
            else
               call errint ( ielgrp, 1 )
               call errint ( ielem, 2 )
               call errchr ( 'volume', 1 )
               call errsub ( 1540, 2, 0, 1 )
            end if
            call prinin(index1,27,'node numbers')
            call prinrl2(x,27,3,'co-ordinates')
            go to 1000
         end if
      end if

      if ( ideriv==1 ) then

!     --- ideriv = 1   Fill derivatives

         do i = 1, 27
            do k = 1, m
               dphidx(i,k,1) = (phiksi(i,k,1)*detinv(k,1,1)+ &
                                phiksi(i,k,2)*detinv(k,2,1)+ &
                                phiksi(i,k,3)*detinv(k,3,1))/w(k)
               dphidx(i,k,2) = (phiksi(i,k,1)*detinv(k,1,2)+ &
                                phiksi(i,k,2)*detinv(k,2,2)+ &
                                phiksi(i,k,3)*detinv(k,3,2))/w(k)
               dphidx(i,k,3) = (phiksi(i,k,1)*detinv(k,1,3)+ &
                                phiksi(i,k,2)*detinv(k,2,3)+ &
                                phiksi(i,k,3)*detinv(k,3,3))/w(k)
            end do
         end do

         do i = 1, 8
            do k = 1, m
               dpsi(idx(i),k,1) = (psiksi(i,k,1)*detinv(k,1,1)+ &
                                   psiksi(i,k,2)*detinv(k,2,1)+ &
                                   psiksi(i,k,3)*detinv(k,3,1))/w(k)
               dpsi(idx(i),k,2) = (psiksi(i,k,1)*detinv(k,1,2)+ &
                                   psiksi(i,k,2)*detinv(k,2,2)+ &
                                   psiksi(i,k,3)*detinv(k,3,2))/w(k)
               dpsi(idx(i),k,3) = (psiksi(i,k,1)*detinv(k,1,3)+ &
                                   psiksi(i,k,2)*detinv(k,2,3)+ &
                                   psiksi(i,k,3)*detinv(k,3,3))/w(k)
            end do
         end do
      end if

      if ( iweigo==1 ) then

!     --- iweigo = 1  Fill weights

         do k = 1, m
            w(k) = abs( w(k) ) * weight(k)
         end do
      end if

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w(1..m ) contains the weights
!         w(m+1 .. m+n) the diagonal of the mass matrix
!         In this case it is sufficient to use the integration rule available

         do i = 1, n
            som = 0d0
            do k = 1, m
               som = som+w(k)*phi(i,k)**2
            end do  ! k = 1, m
            w(m+i)= som
         end do  ! i = 1, n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  call erclos ( 'elp634' )

      end
