      subroutine sepelnsstrc2d ( s, phix, phi, eta, w, mcont, modelv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill stress tensor for Navier-Stokes into matrix S
!     Cartesian coordinates (2d)
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision phix(n,m,ndimlc), phi(n,m), eta(m), w(m), &
                       s(ndimlc*n,ndimlc*n)
      integer mcont, modelv

!     eta            i    Array containing the viscosity in the integration
!                         points
!     mcont          i    Defines the type of continuity equation
!                         See elm900
!     modelv         i    Defines the type of viscosity model
!                         See elm900
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phix           i    Array of length n x m x ndim  containing the
!                         derivatives of the basis functions in the sequence:
!                         d phi / dx (xg ) = phix (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = phix (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = phix (i,k,3);
!                              i        k
!                         If the element is a linear tetrahedron (n=4), then
!                         the derivatives are constant and only k = 1 is filled.
!     s             i/o   Element matrix to be filled
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double precision :: dmult(2), sw (n,n), work(m)

!     dmult          Array containing multiplication factors
!     i              Counting variable
!     j              Counting variable
!     sw             Work array in which a sub element matrix is stored
!     work           Work array to store w eta
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the stiffness matrix results in a 2 x 2 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S    |
!            |    uu      uv  |
!     S =    |                |
!            |   S       S    |
!            |    vu      vv  |
!     The submatrices are defined by:
!              T
!     S    =  S
!      xy      yx
!     S uu =  2  eta phix phix + eta phiy phiy + eta phiz phiz
!     S uv =     eta phix(j) phiy(i)
!     S vu =     eta phiy(j) phix(i)
!     S vv =  2  eta phiy phiy + eta phix phix + eta phiz phiz
!     In case of modelv = 10:
!     S uu =  eta phix phix + eta phiy phiy + eta phiz phiz = S vv
!
!    In case of compressible flow the stress term is changed from
!
!    T = eta ( grad v + grad v^T ) into
!
!    T = eta ( grad v + grad v^T ) - 2/3 eta div v I
!
!    with v the velocity
!
!    This means in the Cartesian case that the following extra terms are added:
!
!             |   phi_jx phi_ix   phi_jy phi_ix   |
!   -2/3 eta  |                                   |
!             |   phi_jx phi_iy   phi_jy phi_iy   |
!    The diagonal terms can be treated by multiplying by 4/3 (=2-2/3)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelnsstrc2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelnsstrc2d'
         write(irefwr,1) 'modelv, mcont', modelv, mcont
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( modelv==10 ) then

!     --- Pure incompressible flow, apply divergence freedom
!         simplified expression

         dmult(1) = 1d0

      else

!     --- Pure incompressible flow, apply divergence freedom

         dmult(1) = 2d0

      end if  ! ( modelv==10 )
      if ( mcont==1 .or. mcont==4 ) then

!     --- compressible flow

         dmult(1) = 4d0/3d0
         dmult(2) = -0.5d0*dmult(1)

      end if  ! ( mcont==1 .or. mcont==4 )

      if ( debug ) call prinrl ( dmult, 2, 'dmult' )

!     --- eta phix phix

      work = w*eta

      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work*phix(i,1:m,1)*phix(j,1:m,1))
         end do
      end do

!     --- Put in S_uu and S_vv

      do i = 1, n
         do j = 1, i
            s(2*i-1,2*j-1) = dmult(1)*sw(i,j)
            s(2*i,2*j) = sw(i,j)
         end do
      end do

!     --- eta phiy phiy

      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work*phix(i,1:m,2)*phix(j,1:m,2))
         end do
      end do

!     --- Add to S_uu and S_vv

      do i = 1, n
         do j = 1, i
            s(2*i-1,2*j-1) = s(2*i-1,2*j-1)+sw(i,j)
            s(2*i,2*j) = s(2*i,2*j)+dmult(1)*sw(i,j)
         end do
      end do

      if ( modelv/=10 ) then

!     --- S vu =  eta phi_jx phi_iy

         do i = 1, n
            do j = 1, n
               sw(j,i) = sum(work*phix(i,1:m,1)*phix(j,1:m,2))
            end do
         end do

!        --- Put in S_uv

         do i = 1, n
            do j = 1, i
               s(2*i,2*j-1) = sw(j,i)
               s(2*i-1,2*j) = sw(i,j)
            end do
         end do

         if ( mcont==1 .or. mcont==4 ) then

!        --- compressible flow -2/3 eta phi_ix phi_jy

            do i = 1, n
               do j = 1, n
                  sw(j,i) = sum(work*phix(i,1:m,2)*phix(j,1:m,1))
               end do
            end do

            do i = 1, n
               do j = 1, i
                  s(2*i,2*j-1) = s(2*i,2*j-1)+sw(j,i)*dmult(2)
                  s(2*i-1,2*j) = s(2*i-1,2*j)+sw(i,j)*dmult(2)
               end do
            end do

         end if  ! ( mcont==1 .or. mcont==4 )

      end if  ! ( modelv/=10 )

!     --- Fill upper half of matrix

      do i = 1, 2*n
         do j = 1, i
            s(j,i) = s(i,j)
         end do  ! j = 1, i
      end do  ! i = 1, 2*n

1000  call erclos ( 'sepelnsstrc2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelnsstrc2d'

      end if  ! ( debug )

      end subroutine sepelnsstrc2d
