      subroutine sepelnsstress ( s, phix, phi, xgauss, eta, w, mcont, modelv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill stress tensor for Navier-Stokes into matrix S
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision phix(n,m,ndimlc), phi(n,m), eta(m), w(m), &
                       s(ndimlc*n,ndimlc*n), xgauss(m,ndimlc)
      integer mcont, modelv

!     eta            i    Array containing the viscosity in the integration
!                         points
!     mcont          i    Defines the type of continuity equation
!                         See elm900
!     modelv         i    Defines the type of viscosity model
!                         See elm900
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phix           i    Array of length n x m x ndim  containing the
!                         derivatives of the basis functions in the sequence:
!                         d phi / dx (xg ) = phix (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = phix (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = phix (i,k,3);
!                              i        k
!                         If the element is a linear tetrahedron (n=4), then
!                         the derivatives are constant and only k = 1 is filled.
!     s             i/o   Element matrix to be filled
!     w              i    array of length m containing the weights for
!                         integration
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL1        Print 2d real vector
!     SEPELNSSTRAXI  Fill stress tensor for Navier-Stokes into matrix S
!                    Axisymmetric coordinates
!     SEPELNSSTRC2D  Fill stress tensor for Navier-Stokes into matrix S
!                    Cartesian coordinates (2d)
!     SEPELNSSTRC3D  Fill stress tensor for Navier-Stokes into matrix S
!                    Cartesian coordinates (3d)
!     SEPELNSSTRPOL  Fill stress tensor for Navier-Stokes into matrix S
!                    Polar coordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2984   combination of compressible flow and reduced stress term
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelnsstress' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelnsstress'
         write(irefwr,1) 'jcart, modelv, mcont', jcart, modelv, mcont
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( mcont==1 .or. mcont==4 ) then

!     --- compressible flow

         if ( modelv==10 ) then

!        --- The combination compressible flow and modelv=10 is not possible

            call errint ( mcont, 1 )
            call errint ( modelv, 2 )
            call errsub ( 2984, 2, 0, 0 )
            go to 1000

         end if  ! ( modelv==10 )

      end if  ! ( mcont==1 .or. mcont==4 )

      if ( modelv==10 ) s = 0d0   ! modelv = 10, clear matrix

      select case ( jcart )
      case (1)

!     --- Cartesian coordinates

         if ( ndimlc==2 ) then

!        --- ndim = 2, two-dimensional

            call sepelnsstrc2d ( s, phix, phi, eta, w, mcont, modelv )

         else

!        --- ndim = 3, three-dimensional

            call sepelnsstrc3d ( s, phix, phi, eta, w, mcont, modelv )

         end if  ! ( ndimlc==2 )

      case (2,4)

!     --- Axisymmetric coordinates

         call sepelnsstraxi ( s, phix, phi, xgauss, eta, w, mcont )

      case (3)

!     --- Polar coordinates

         call sepelnsstrpol ( s, phix, phi, xgauss, eta, w, mcont )

      end select  ! case ( jcart )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( s, ndimlc*n, ndimlc*n, 's' )
         write(irefwr,*) 'End sepelnsstress'

      end if  ! ( debug )
      call erclos ( 'sepelnsstress' )

      end subroutine sepelnsstress
