      subroutine el0250 ( iuser, user, jelp, jheat, igprob, istres, &
                          istrai, nd, compd, isub, isup, array, &
                          ioption )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.18 date 18-02-2015 New computation igprob
!        version  4.17 date 19-11-2014 Correction for shape 21
!
!   copyright (c) 1992-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM250 or
!     ELD250 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elasticity_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, jheat, igprob, istres, istrai, isub, isup, &
              nd
      integer ioption(10)
      logical compd

!     array     o    Real work array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                    1-21:  Information about the elasticity matrix D
!                      22:  Thickness h
!                      23:  External Force in 1-direction
!                      24:  External Force in 2-direction
!                      25:  External Force in 3-direction
!                   26-31:  Initial strain
!                   32-37:  Initial stress
!                      38:  Density rho
!                    The storage of the various components depends on the
!                    type of model to be used.
!                    For the elasticity matrix we have:
!                    Isotropic material:
!                                   1:  E
!                                   2:  nu
!                                3-21: Not used
!                    Orthotropic material:
!                                   1:  E1
!                                   2:  E2
!                                   3:  nu1
!                                   4:  nu2
!                                   5:  G2
!                                6-21: Not used
!                    User defined 3*3 elasticity matrix:
!                                   1:  d11
!                                   2:  d12
!                                   3:  d22
!                                   4:  d31
!                                   5:  d32
!                                   6:  d33
!                                7-21: Not used
!                    For the initial strain we have:
!                   26:  eps x             eps r
!                   27:  eps y             eps z
!                   28:  eps z             eps theta
!                   29:  gamma xy          gamma rz
!                   30:  gamma xz          - &
!                   31:  gamma yz          - &
!                    For the initial stress we have:
!                   32:  sigma x           sigma r
!                   33:  sigma y           sigma z
!                   34:  sigma z           sigma theta
!                   35:  tau xy            tau rz
!                   36:  tau xz
!                   37:  tau yz
!                    Each coefficient requires exact m (is number of
!                    integration points) positions
!     compd     o    Indication if the matrix D must be computed in each call
!                    (true) or if it is constant and hence has to be computed
!                    once (false)
!     igprob    o    Type of stress-strain relations:
!                    igprob = irelat + 10*inonlin + 100*ispecial
!                    Possible values:
!                    inonlin = 0
!                    irelat  1: plane stress, isotropic material
!                               or 3D stress, isotropic material (Cartesian)
!                            2: plane strain, isotropic material (Cartesian)
!                            3: axi-symmetric stress, isotropic material
!                                                               (axi-symmetric)
!                            4: plane stress, orthotropic material (Cartesian)
!                            5: plane strain, orthotropic material (Cartesian)
!                            6: axi-symmetric stress, orthotropic material
!                               (axi-symmetric)
!                            7: User defined elasticity matrix (3x3)
!                               (Cartesian)
!                            8: User defined elasticity matrix (4x4)
!                               (axi-symmetric)
!                    inonlin>0
!                    inonlin 1: Geometrical and physical nonlinear behaviour
!                            2: 1 + reduced integration compressibility term
!                            3: nonlin + variable anisotropy
!                            4: 3 + reduced integration of compressibility term
!                    irelat  0,1: 3D
!                            3: 2D axi-symmetric
!                    ispecial = 1: muscle contraction, requires ELMUSC
!                                  contact TUE/WFW for info
!     ioption   o    Integer array to with options of this element:
!                        ioption(1) = inonlin
!                        ioption(2) = ispecial
!                        ioption(3) = user coefficient 4
!                        ioption(4) = user coefficient 5
!                        ioption(5-10) ??
!     istrai    o    Indicates if initial strain must be taken into account (1)
!                    or not (0)
!     istres    o    Indicates if initial stress must be taken into account (1)
!                    or not (0)
!     isub      o    Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup      o    Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iuser     i    User array in which integer information of the
!                    coefficients is stored (See Standard Problems)
!     jelp      o    Indication of the type of basis function subroutine must
!                    be called by ELM250. Possible vaues:
!                    3:  Subroutine ELP620  (Linear triangle)
!                    5:  Subroutine ELP621  (Bilinear quadrilateral)
!     jheat     o    Integer parameter indicating if the mass matrix for the
!                    heat equation must be computed
!                    Possible values:
!                    0:  no mass matrix
!                    1:  diagonal mass matrix with constant coefficient
!                    2:  diagonal mass matrix with variable coefficient
!                   11:  full mass matrix with constant coefficient
!                   12:  full mass matrix with variable coefficient
!     nd        o    dimension of elasticity matrix d
!     user      i    User array in which real information of the
!                    coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons, ncoef, i, j, isubcf, kchois, ne, nu, list(6), &
              jgprob, ishift
      double precision e, anu, h
      logical linear, funchk, constd, allowed

!     allowed        If true it concerns an allowed possibility
!     anu            Poisson's ratio
!     constd     Inidicates whether the coefficients with respect to the
!                D-matrix are constant (true) or not (false)
!     e          Elasticity modulus
!     funchk     Indicates whether there are coefficients that must be
!                evaluated by funcc3 (true) or not (false)
!     h          Thickness of the plate
!     i          Counting variable
!     ishift     Shift in coefficient sequence number
!     isubcf     Under bound for coefficients check
!     j          Counting variable
!     jgprob     Part of igprob
!     jwcons     Indication if the weight function is constant (1) or not (2)
!     kchois     Indication what type of parameters must be computed
!                Possible values:
!                0:  Standard method, the calling subroutine is ELM250
!                    It is supposed that an element matrix must be computed
!                1:  Subroutine ELD250 calls this subroutine, only
!                    derivatives must be computed
!                2:  Subroutine ELD250 or ELI250 calls this subroutine, &
!                    a user function defined as first parameter in
!                    IUSER/USER must be filled in elemvc (icheld=9)
!                3:  Subroutine ELD250 calls this subroutine, &
!                    alpha times the gradient must be stored in ELEMVC
!                    The first 3 parameters of IUSER/USER are used
!     linear     Indicates whether the element is a linear simplex (true)
!                or not (false)
!     list       Contains a list of parameters that must be zero for 2D
!     ncoef          Number of coefficients in the equation
!     ne             Number of E coefficients
!     nu             Number of nu coefficients
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     EL0250IN       Compute the number of integration points for elements of
!                    type 250
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIREPSCAL     Replace ivalue by scalar if necessary
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     See subrutine ELM250 for a description
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!     262   ix has wrong value
!     264   jdegfd has wrong value
!     400   The value of stress-strain is incorrect
!     406   Value of icheld not allowed
!     554   Elasticity modulus E (or E1) has wrong value
!     555   Poisson ratio nu (or nu1) has wrong value
!     556   Thickness h has wrong value
!     771   Rho is non-positive
!    1449   Initial stresses have only been implemented for isotropic material
!    1450   Initial strains have only been implemented for isotropic material
!    1634   IGPROB not implemented for this shape number
!    1635   NUMOLD = 0, whereas problem is non-linear
!    1636   NUMOLD<2, whereas at least two old solutions are required
!    1637   Initial stresses and strains are not allowed for non-linear
!           problems
!    1768   Coefficient is filled that is not used
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data list / 28, 30, 31, 34, 36, 37 /
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0250' )
      call elcomm
      jconsf = 2

!     --- Compute kchois

      if ( icheld==0 ) then

!     --- Call by elm250, all coefficients required

         kchois = 0

      else if ( icheld>0 ) then

!     --- Call by eld250, all coefficients required

         kchois = 0

!        --- allow icheld = 8 (see eld250)
!            The possibilities 12-16 and 32-35 have not yet been implemented

         allowed = icheld<=8 .or. (icheld>=21.and.icheld<=27) .or. &
                   (icheld==9 .and. ndim==3) .or. &
                   (icheld>=36 .and. icheld<=37) .or. &
                   (icheld>=46 .and. icheld<=47)

         if ( .not. allowed ) then
            call errint ( icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 406, 3, 0, 0 )
         end if
         if ( icheld==1 ) then

            if ( ix<=0 .or. ix>ndim ) then
               call errint ( ix, 1 )
               call errint ( ndim, 2 )
               call errint ( itype, 3 )
               call errint ( ielgrp, 4 )
               call errsub ( 262, 4, 0, 0 )
            end if
            if ( jdegfd<=0 .or. jdegfd>ndim ) then
               call errint ( jdegfd, 1 )
               call errint ( ndim, 2 )
               call errsub ( 264, 2, 0, 0 )
            end if

         end if
         ncomp  = ndim
         jderiv = 1
         jind   = 1

      end if

      if ( kchois==0 ) then

!     --- standard situation

         ncoef = 40

      end if

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficients

      jheat = imas
      call el0250in ( iuser, jheat, igprob )
      if ( ncoef>0 ) then

         ipos = iuser ( ielgrp + 5 )
         if ( ipos<7 ) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 42, 5, 0, 0 )
         end if

         nuser = 0
         ipos = ipos+2
         ishift = 2

         if ( kchois==0 ) then

!        --- Standard situation, Compute the parameters jheat, igprob, irule
!                                                       jcart

            if ( igprob==0 .and. icheld>=8 .and. icheld<=10 ) then

!           --- In the linear case only icheld 1 to 7 are available

               call errint ( icheld, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 406, 3, 0, 0 )

            end if  ! ( igprob==0 .and. icheld>7 )

            jgprob = igprob
            ioption(2) = jgprob/100
            jgprob = jgprob - 100*ioption(2)
            ioption(1) = jgprob/10
            jgprob = jgprob - 10*ioption(1)

            if ( igprob<0 .or. &
                 ( ndim==2 .and. ioption(1)>2 ) .or. &
                 ( ioption(2)>3 ) .or. &
                 ( ndim==3 .and. jgprob/=0 .and. jgprob/=8 ) .or. &
                 ( ioption(2)==1 .and. ioption(1)==0 ) ) then

               call errint ( igprob, 1 )
               call errint ( 2, 2 )
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 400, 5, 0, 0 )
            end if
            igprob = igprob + 1
            if ( ioption(2)==2 ) igprob = igprob-200

            ipos = ipos + 1
            ishift = ishift+1

!           --- ioption(3) is passed to fnmateri as a choice variable

            ioption(3) = inirepscal ( iuser(ipos) )

!           --- ioption(4) makes only sense in the nonlinear case
!               and are merely used to switch between different nonlinear
!               methods in the testing phase of this element. Using them
!               may give inpredictable results

            ipos = ipos + 1
            ishift = ishift+1

!           --- ioption(4) is used to set special options

            ioption(4) = inirepscal ( iuser(ipos) )
            ipos = ipos + 1
            ishift = ishift+1

            if ( igprob==3 .or. igprob==6 .or. igprob== 8 &
                  .or. igprob==13 ) then
               jcart = 2
            else
               jcart = 1
            end if

         end if

      end if


!     --- Set parameters depending on the shape numbers

      mn = m * n
      jelp = ishape
      if ( ishape==7 ) then
         jelp = 4   ! elements of shape 7 are treated as 4
      else if ( ishape==17 ) then
         jelp = 13   ! elements of shape 17 are treated as 13
      end if  ! ( ishape==7 )
      iphi = 1
      if ( ndim==3 ) then

         nd = 6

!        --- NB: in the 2D non-linear case nd is always 4

      else if ( igprob<=2 .or. igprob==4 .or. igprob==5 .or. &
                igprob==7 ) then
         nd = 3
      else
         nd = 4
      end if

      if ( ioption(1)/=0 .and. .not. &
           (jelp==5 .or. jelp==14 .or. &
            jelp==13 .or. jelp==12 .or. jelp==11) ) then

         call errint ( igprob, 1 )
         call errint ( ishape, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1634, 5, 0, 0 )

      end if

      if ( jelp==3 ) then

!     --- linear element

         if ( irule==2 ) mn = 3
         linear = .true.

      else if ( jelp==4 .or. jelp==6 .or. jelp==7 ) then

!     --- quadratic triangle or quadrilateral

         linear = .false.

      else if ( jelp==5 ) then

!     --- Bilinear element

         linear = .false.

      else if ( jelp==11 ) then

!     --- linear tetrahedron

         linear = .true.

      else if ( jelp==12 .or. jelp==21 .or. jelp==22 ) then

!     --- quadratic tetrahedron

         linear = .false.

      else if ( jelp==13 ) then

!     --- trilinear brick

         linear = .false.

      else if ( jelp==14 ) then

!     --- triquadratic brick

         linear = .false.
         nvert = 8

      else

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end if

      if ( irule/=1 ) then
         jdiag = 2
         igausp = 1
      end if

      if ( linear ) then
         jwcons = min(jcart,2)
         if ( irule>2 ) jwcons = 2
         jdercn = 0
      else
         jwcons = 2
         jdercn = 1
      end if

      if ( ioption(1)/=0 .and. numold==0 ) then

         call errint ( ielem, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 1635, 3, 0, 0 )

      end if

!     --- check for specialty array

      if ( ioption(2)==1 .and. numold<=1 .and. icheld==0 ) then

         call errint ( ielem, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 1636, 3, 0, 0 )

      end if

!     --- Clear array ARRAY

      array(1:40*m) = 0d0

      do i = 1, ncoef
         call elusr3 ( i, array(1+(i-1)*m), iuser, user, i+ishift )
      end do

!     --- Test array lengths, number of nodes and number of degrees of freedom

      call prinlg ( 'iuser', ipos-1, user, iuser )
      call prinlg ( 'user', -nuser, user, iuser )

!     --- Consider the various terms separately
!         Check if E and nu are in the correct range

      if ( igprob<=3 ) then
         ne = 1
      else if ( igprob<=6 ) then
         ne = 2
      else
         ne = 0
      end if

      do i = 1, ne

         if ( ind(i)==0 ) then

!        --- ind(i)=0, hence Ei = 0

            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call erreal ( 0d0, 1 )
            call errsub ( 554, 3, 1, 0 )

         else if ( ind(i)<0 ) then

!        --- ind(1)<0, hence E is constant

            e=array(1+(i-1)*m)
            if ( e<=0d0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call erreal ( e, 1 )
               call errsub ( 554, 3, 1, 0 )
            end if

         end if

      end do

      if ( igprob<=3 ) then
         nu = 1
      else if ( igprob<=6 ) then
         nu = 3
      else
         nu = 0
      end if

      do i = 1, nu

         if ( ind(ne+i)==0 ) then

!        --- ind(ne+i)=0, hence nu(i) = 0

            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call erreal ( 0d0, 1 )
            call errsub ( 555, 3, 1, 0 )

         else if ( ind(ne+i)<0 ) then

!        --- ind(ne+i)<0, hence nu1 is constant

            anu=array(1+(ne-1+i)*m)
            if ( i==3 ) anu = min(.45d0, anu)
            if ( anu<=0d0 .or. anu>=.5d0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call erreal ( anu, 1 )
               call errsub ( 555, 3, 1, 0 )
            end if

         end if

      end do

      if ( ndim==2 .and. igprob/=3 .and. igprob/=6 .and. &
          igprob/=8 ) then

!     --- igprob = 1, 2, 4 or 5  function h  (thickness of plate)

         if ( ind(22)==0 ) then

!        --- ind(22)=0, hence h = 1

            ind(22) = -1
            array(21*m+1:22*m) = 1d0

         else if ( ind(22)<0 ) then

!        --- ind(22)<0, hence h is constant

            h = array(1+21*m)
            if ( h<=0d0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call erreal ( h, 1 )
               call errsub ( 556, 3, 1, 0 )
            end if

         end if

      else

         if ( ind(22)/=0 ) then
            call errint ( 27, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errwar ( 1768, 4, 0, 0 )
            ind(22) = 0
         end if

      end if

!     --- Set parameters istress and istrain

      if ( ndim==2 ) then

!     --- Not all initial stresses and strains should be filled

         if ( igprob==3 .or. igprob==6 ) then

!        --- axisymmetric, coefficient 33 may be non-zero

            isub = 2

         else

!        --- Cartesian case

            isub = 1

         end if

         do i = isub, 3
            if ( ind(list(i))/=0 ) then
               call errint ( list(i)+5, 1 )
               call errint ( ielem, 2 )
               call errint ( itype, 3 )
               call errint ( ielgrp, 4 )
               call errwar ( 1768, 4, 0, 0 )
               ind(list(i)) = 0
            end if
         end do

         do i = isub+3, 6
            if ( ind(list(i))/=0 ) then
               call errint ( list(i)+5, 1 )
               call errint ( ielem, 2 )
               call errint ( itype, 3 )
               call errint ( ielgrp, 4 )
               call errwar ( 1768, 4, 0, 0 )
               ind(list(i)) = 0
            end if
         end do

      end if

      istrai = 0
      istres = 0
      if ( ind(26)/=0 .or. ind(27)/=0 .or. ind(28)/=0 .or. &
           ind(29)/=0 .or. ind(30)/=0 .or. ind(31)/=0 ) istrai = 1

      if ( ind(32)/=0 .or. ind(33)/=0 .or. ind(34)/=0 .or. &
           ind(35)/=0 .or. ind(36)/=0 .or. ind(37)/=0 ) istres = 1

      if ( ioption(1)/=0 ) then
         if ( istres==1 .or. istrai==1 ) then

!        --- Initial stress and strain not allowed for non-linear problems

            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errwar ( 1637, 3, 0, 0 )
            istres = 0
            istres = 0

         end if
      end if

      if ( jheat>0 ) then

!     --- jheat>0, i.e. compute mass matrix

         if ( ind(38)==0 ) then

!        --- ind(38) = 0, set rho equal to 1

            do i = 1, m
               array(i+37*m) = 1d0
            end do
            ind(38) = -1

         else if ( ind(38)<0 .and. array(1+37*m)<0 ) then

!        --- rho negative

            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 771, 3, 0, 0 )

         else if ( ind(38)<0 .or. ind(38)==2002 ) then

            jheat = jwcons + 10 * (jheat-1)

         else

            jheat = 2 + 10 * (jheat-1)

         end if

      else

         if ( ind(38)/=0 ) then
            call errint ( 43, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errwar ( 1768, 4, 0, 0 )
            ind(38) = 0
         end if

      end if

!     --- Check if the matrices B and D must be computed in each call

      if ( igprob<=3 ) then
         ne = 2
      else if ( igprob<=6 ) then
         ne = 5
      else if ( igprob==7 ) then
         ne = 6
      else if ( igprob==8 ) then
         ne = 7
      else
         ne = 0
      end if
      constd = ind(1)<=0
      do i = 2, ne
         constd = constd .and. ind(i)<=0
      end do
      if ( constd ) then

!     --- E and nu constant
!         or  matrix = false and no initial strain
!         The D matrix has to be computed only once

         compd=.false.

      else

         compd=.true.

      end if

      if ( .not. matrix .and. istres==0 .and. istrai==0 .and. &
           icheld<=0 ) then

!     --- matrix = false   istres = 0  and  istrai = 0
!         Array B does not have to be filled

         ideriv=0

      else

         ideriv=1

      end if

      if ( ioption(1)/=0 ) then

!     --- igprob>10 : physical nonlinear behaviour has non-constant
!         D matrix, B is obsolete but needs the derivatives

         constd = .false.
         compd  = .true.
         ideriv = 1
         jderiv = 1
         jind = 1
         ncomp = ndim

      end if

!     --- Check if right-hand side must be computed

      if ( ndim==1 .and. ind(24)/=0 ) then
         call errint ( 29, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errwar ( 1768, 4, 0, 0 )
         ind(29) = 0
      end if
      if ( ndim<=2 .and. ind(25)/=0 ) then
         call errint ( 30, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errwar ( 1768, 4, 0, 0 )
         ind(30) = 0
      end if

      if ( icheld==0 .and. ind(23)==0 .and. ind(24)==0 .and. &
           ind(25)==0 .and. istrai==0 .and. istres==0 .and. &
           igprob<=10 ) then

!     --- f1 = 0, f2 = 0, f3=0;  no initial stress nor initial strain
!         and not non-linear analysis.
!         In this case no right-hand-side vector has to be computed

         notvec=1

      else

         if ( ndim==2 .and. istres==1 .and. igprob<=2 ) then

!        --- istres = 1 and isotropic material,
!            move fourth component to third position

            ind(34) = ind(35)
            ich(34) = ich(35)
            ist(34) = ist(35)
            mst(34) = mst(35)
            ind(35) = 0
            ich(35) = 0
            ist(35) = 0
            mst(35) = 0
            if ( ncoef==37 ) ncoef = 34
            do i = 1, m
               array(33*m+i) = array(34*m+i)
            end do

         else if ( istres==1 .and. igprob>3 ) then

            call errsub ( 1449, 0, 0, 0 )

         end if

         if ( ndim==2 .and. istrai==1 .and. igprob<=2 ) then

!        --- istrai = 1 and isotropic material,
!            move fourth component to third position

            ind(28) = ind(29)
            ich(28) = ich(29)
            ist(28) = ist(29)
            mst(28) = mst(29)
            ind(29) = 0
            ich(29) = 0
            ist(29) = 0
            mst(29) = 0
            if ( ncoef==31 ) ncoef = 28
            do i = 1, m
               array(27*m+i) = array(28*m+i)
            end do

         else if ( istrai==1 .and. igprob>3 ) then

            call errsub ( 1450, 0, 0, 0 )

         end if

      end if

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      if ( jcart==1 .and. irule==1 ) then
         igausp = 0
      else
         igausp = 1
      end if

      isubcf = 1

      do i = isubcf , ncoef
         isub = i

!        --- If ind(i)>0 then paramater i corresponds to a function

         if ( ind(i)>0 ) goto 180

      end do

      isub = ncoef + 1

180   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i)>0 ) goto 200
      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

200   do i = isub, isup
         ist(i) = 0
      end do
      ind(lengthmst) = 0
      funchk = .false.
      do i = isub, isup
         if ( ind(i)>0 .and. ind(i)<2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j)==ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
            igausp = 1
         end if
         if ( ind(i)>1000 .and. ind(i)<=2000 ) ind(lengthmst) = 1
         if ( ind(i)>10000 ) funchk = .true.
         if ( ind(i)>1000 .and. ind(i)<=2000 .or. ind(i)>10000) &
              igausp=1
      end do

      if ( icheld<-1 .or. icheld>0 .or. ioption(1)/=0 ) &
         ind(lengthmst) = 1

      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2

      call erclos ( 'el0250' )

      end
