      subroutine el0250in ( iuser, jheat, igprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 18-02-2015 Extra parameter igprob
!        version  2.3  date 26-05-2014 Extension for shape 22
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the number of integration points for this element group
!   using array iuser
! **********************************************************************
!
!                       KEYWORDS
!
!     elasticity_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iuser(*), jheat
      integer, intent(out) :: igprob

!     igprob         o    Type of model used
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jheat          o    Integer parameter indicating if the mass matrix for
!                         the heat equation must be computed (>0) or not ( = 0)
!                         If jheat = 1, then the coefficient for the mass matrix
!                         is constant, otherwise (jheat = 2) it is variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode=8
      integer :: ncode(nncode)
      character (len=31) :: code (nncode)
      integer mrule, massparm, jwcons, ishift, jgprob

!     code           Table with keywords corresponding to string coefficients
!     ishift         Shift in coefficient sequence number
!     jgprob         Help parameter to store igprob
!     jwcons         Indication if the weight function is constant
!                    (1) or not (2)
!     massparm       Indicates the largest value of jheat during the
!                    time integration
!     mrule          Type of integration rule as given by the user
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     nncode         Number of entries in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal, elgetintcoef


!     EL0800INM      Compute the parameter m for an element
!     ELGETINTCOEF   Give elgetintcoef the actual integer value corresponding to
!                    iuser(ipos)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Replace ivalue by scalar if necessary
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is for the Navier-Stokes equations is:
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!    1766   Type of integration rule incorrect
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'PLANE_STRESS', 'PLANE_STRAIN', &
                  'AXI_SYMMETRIC_STRESS', 'ORTHOTROPIC_PLANE_STRESS', &
                  'ORTHOTROPIC_PLANE_STRAIN', &
                  'STRATIFIED_AXI_SYMMETRIC_STRESS', &
                  'USER_DEFINED_ELASTICITY_CART', &
                  'USER_DEFINED_ELASTICITY_AXI' /
      data ncode / 13,13,20,24,24,  31,28,27 /
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0250in' )
      n = inpelm

      ipos = iuser ( ielgrp + 5 )
      if ( ipos<7 ) then
         call errint ( ielgrp+5, 1 )
         call errint ( ipos, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 42, 5, 0, 0 )
      end if
      ipos = ipos + 1
      igprob = elgetintcoef ( iuser, ipos, ishift, code, nncode, &
                              ncode, 1, 'stress_strain_relation' )

      jgprob = igprob
      igprob = mod(igprob,100)+1
      mrule = inirepscal ( iuser(ipos) )
      mrule = mod(mrule,100)
      if ( mrule<0 .or. mrule>6 ) then
         call errint ( mrule, 1 )
         call errint ( 6, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1766, 5, 0, 0 )
      end if
      if ( icheld>=11 .and. icheld<20 .and. mrule/=1 ) then
         mrule = 2
      else if ( icheld>=21 .and. icheld<30 ) then
         mrule = 0
      else if ( icheld==1 .or. icheld>=30 .and. icheld<40 .and. &
                mrule/=2 ) then
         mrule = 1
      else if ( icheld>0 .and. icheld<10 .and. mrule>2 ) then
         mrule = 1
      end if

      ipos = ipos + 1

!     --- Set parameters depending on the shape numbers

      if ( mrule>0 ) then
         irule = mrule
      else
         irule = 3
      end if
      n = inpelm
      massparm = max(jheat,massrule)

!     --- make default integration rule 2-pt Gauss: Newton-Cotes
!         integration results in very poor shear behaviour

      if ( ishape==3 ) then

!     --- linear element

         if ( icheld>0 .and. icheld<=40 ) then

!        --- Call by deriva

            if ( mrule==0 ) irule = 2

         else if ( igprob==3 .or. igprob==6 .or. igprob==8 ) then

!        --- igprob = 3, 6 or 8 axi-symmetric stress

            if ( mrule==0 ) irule = 3

         else

!        --- igprob = 1, 2, 4, 5 or 7, plane strain or stress

            if ( mrule==0 ) irule = 2

         end if

      else if ( ishape==4 .or. ishape==7 ) then

!     --- quadratic triangle

         if ( icheld>0 .and. icheld<=40 ) then

!        --- Call by deriva

            if ( mrule==0 ) irule = 1

         else if ( igprob==3 .or. igprob==6 .or. igprob==8 ) then

!        --- igprob = 3, 6 or 8 axi-symmetric stress

            if ( mrule==0 ) irule = 3

         else

!        --- igprob = 1, 2, 4, 5 or 7, plane strain or stress

            if ( mrule==0 ) irule = 2

         end if
         if ( ishape==7 ) n = inpelm-1

      else if ( ishape==5 ) then

!     --- Bilinear element

         if ( icheld==0 .or. icheld>40 ) then
            if ( mrule==0 ) irule = 3
         else
            if ( mrule==0 ) irule = 2
         end if

      else if ( ishape==6 ) then

!     --- quadratic quadrilateral

         if ( icheld>0 .and. icheld<=40 ) then

!        --- Call by deriva

            if ( mrule==0 ) irule = 1

         else

!        --- igprob = 3, 6 or 8 axi-symmetric stress

            irule = 3

         end if

      else if ( ishape==11 .or. ishape==21 ) then

!     --- linear tetrahedron or wedge

         if ( mrule==0 ) irule = 1

      else if ( ishape==12 ) then

!     --- quadratic tetrahedron

         if ( mrule==0 ) irule = 1
         if ( icheld/=0 .and. icheld<=40 ) irule = 1

      else if ( ishape==13 .or. ishape==17 ) then

!     --- trilinear brick
!         use 8 point Gauss integration if nonlinear

         if ( icheld/=0 .and. icheld<=40 ) irule = 1
         if ( ishape==17 ) n=n-1

      else if ( ishape==14 ) then

!     --- triquadratic brick
!         use 27 point Gauss integration if nonlinear

         if ( icheld/=0 .and. icheld<=40 ) irule = 1

      else if ( ishape==22 ) then

!     --- quadratic wedge

         if ( icheld/=0 .and. icheld<=40 ) irule = 1

      else

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end if

!     --- Adapt irule in case of mass matrix in order to get
!         a positive definite mass matrix

      if ( massparm==2 .and. mrule==0 ) then

!     --- Non-diagonal mass matrix
!         Integration rule must be at least 3 to avoid non-positive
!         definite mass matrices

         if ( ishape==2 ) then

!        --- Quadratic 1d, use irule = 4

            irule = 4

         else

!        --- Other cases, use irule = 3

            irule = max ( irule, 3 )

         end if

      end if

!     --- Compute the parameter m

      m = n
      if ( irule>1 ) &
         call el0800inm ( m, ishape, jwcons, irule, igausp, jdiag, &
                          iphi )

      igprob = jgprob

      call erclos ( 'el0250in' )

      end
