      subroutine el3023 ( g, matr, w, phi, dphidxi, coor2d )
! ======================================================================
!
!        programmer    Niels Driessen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 13-08-2013 Remove dphideta
!        version  1.0  date 04-07-2004
!
!   copyright (c) 2004-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute contribution to element stiffness matrix, resulting from
!   linearisation of boundary conditions for general nonlinear solids
!   (i.e. boundary integral).
!   At this moment, the subroutine has been tested only for
!   quadrilaterals and extended triangles.
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision g(m), matr(n*ndim,n*ndim), w(m), phi(n,m), &
                       dphidxi(n,m,2), coor2d(ndim,n)

!     coor2d         i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dphidxi        i    Array of length n x m containing the
!                         derivatives of the basis functions dphi_i/dxi
!     g              i    array of length m in which the values of
!                         g in the integration points are stored
!     matr          i/o   Element matrix to be filled
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision matrC(ndim,n*ndim), norm, &
                       matrE(ndim,n*ndim), p, &
                       matr_tmp(n*ndim,n*ndim), &
                       CE(n*ndim,n*ndim), t1crosst2(ndim), &
                       dxdxi(n,ndim,ndim)
      integer i, j, k, l

!     ce             Matrix for product of C and E matrices
!     dxdxi          Derivative of glocal to local coor2ddinates
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     matr_tmp       Help matrix
!     matrc          Matrix containing the shape functions
!     matre          Matrix with derivatives of coor2ddinates and shape functions
!     norm           Norm for coor2ddinate transformation
!     p              Applied pressure in the integration point
!     t1crosst2      Cross product of tangential directions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- set matrix to zero

      do i = 1, n*ndim
         do j = 1, n*ndim
            matr(i,j) = 0d0
         end do
      end do

!     --- calculate dxdxi = sum( dphidxi * coor2d )

      do i = 1, m
         do j = 1, ndim
            dxdxi(i,j,1) = 0d0
            dxdxi(i,j,2) = 0d0
            do k = 1, n
               dxdxi(i,j,1) = dxdxi(i,j,1) + dphidxi(k,i,1)*coor2d(j,k)
               dxdxi(i,j,2) = dxdxi(i,j,2) + dphidxi(k,i,2)*coor2d(j,k)
            end do
          end do
      end do

!     --- compute the integral

      do i = 1, m

!     --- clear matrices

         do j = 1, n*ndim
            do k = 1, n*ndim
               matr_tmp(j,k) = 0d0
            end do
         end do

         do j = 1, ndim
            do k = 1, n*ndim
               matrC(j,k) = 0d0
            end do
         end do

         do j = 1, ndim
            do k = 1, n*ndim
               matrE(j,k) = 0d0
            end do
         end do

!        --- get applied normal stresses in current integration point

         p = g(i)

!        --- fill in the C matrix with shape functions in current integration
!            --- point

         do j = 1, n
            do k = 1, ndim
               matrC(k,(j-1)*ndim + k) = phi(j,i)
            end do
         end do

         if (ndim==2) then

            do j = 1, n
               matrE(1,(j-1)*ndim+2) =-dphidxi(j,i,1)
               matrE(2,(j-1)*ndim+1) =dphidxi(j,i,1)
            end do

            norm =  dsqrt(dxdxi(i,1,1)**2 + dxdxi(i,2,1)**2)

         else if (ndim==3) then

!        --- fill in the E matrix with derivatives of coor2ddinates and shape
!            --- functions

            do j = 1, n

!           --- first row

               matrE(1,(j-1)*3 + 1) = 0d0
               matrE(1,(j-1)*3 + 2) = dxdxi(i,3,2)*dphidxi(j,i,1) - &
                                      dxdxi(i,3,1)*dphidxi(j,i,2)
               matrE(1,(j-1)*3 + 3) = dxdxi(i,2,1)*dphidxi(j,i,2) - &
                                      dxdxi(i,2,2)*dphidxi(j,i,1)

!              --- second row

               matrE(2,(j-1)*3 + 1) = dxdxi(i,3,1)*dphidxi(j,i,2) - &
                                      dxdxi(i,3,2)*dphidxi(j,i,1)
               matrE(2,(j-1)*3 + 2) = 0d0
               matrE(2,(j-1)*3 + 3) = dxdxi(i,1,2)*dphidxi(j,i,1) - &
                                      dxdxi(i,1,1)*dphidxi(j,i,2)

!              --- third row

               matrE(3,(j-1)*3 + 1) = dxdxi(i,2,2)*dphidxi(j,i,1) - &
                                      dxdxi(i,2,1)*dphidxi(j,i,2)
               matrE(3,(j-1)*3 + 2) = dxdxi(i,1,1)*dphidxi(j,i,2) - &
                                      dxdxi(i,1,2)*dphidxi(j,i,1)
               matrE(3,(j-1)*3 + 3) = 0d0
            end do

!           --- calculate cross-product of t1 and t2 in current integration
!               --- point

            t1crosst2(1) = dxdxi(i,2,1)*dxdxi(i,3,2) - &
                           dxdxi(i,3,1)*dxdxi(i,2,2)
            t1crosst2(2) = dxdxi(i,3,1)*dxdxi(i,1,2) - &
                           dxdxi(i,1,1)*dxdxi(i,3,2)
            t1crosst2(3) = dxdxi(i,1,1)*dxdxi(i,2,2) - &
                           dxdxi(i,2,1)*dxdxi(i,1,2)

!           --- calculate the corresponding norm for coor2ddinate transformation

            norm = dsqrt( t1crosst2(1)**2d0 + &
                          t1crosst2(2)**2d0 + &
                          t1crosst2(3)**2d0 )

         end if

!        --- calculate C^T E

         do j = 1, n*ndim
            do k = 1, n*ndim
               CE(j,k) = 0d0
               do l = 1, ndim
                  CE(j,k) = CE(j,k) + matrC(l,j)*matrE(l,k)
               end do
            end do
         end do

!        --- fill in relevant components

         do j = 1, n*ndim
            do k = 1, n*ndim
               matr_tmp(j,k) = p * CE(j,k)
            end do
         end do

!        --- add matrix using weight factors

         do j = 1, n*ndim
            do k = 1, n*ndim
               matr(j,k) = matr(j,k) + (w(i)/norm) * matr_tmp(j,k)
            end do
         end do

      end do

      end
