      subroutine el8105 ( igprob, eps, ugauss, dudx, xgauss, mdecl )
! ======================================================================
!
!        programmer     Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.3  date 10-02-1999 Include commons
!        version  1.2  date 21-01-1998 Marcel Rutten,correction for
!                                      igprob = 2, &
!
!   copyright (c) 1992-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute eps = B u in integration points
!     At this moment linear elements are assumed with one integration point
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!   elasticity_equation
!   strain
! **********************************************************************
!
!                       COMMON BLOCKS
!
      use sepmoduleelem
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer igprob, mdecl
      double precision dudx(mdecl,ncomp,ncomp), ugauss(mdecl,ncomp), &
                       xgauss(mdecl,ncomp), eps(mdecl,*)

!     dudx     i     Real m x ndim x ncomp array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j, i )
!                      i    j   k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*,*) is filled because dudx is then a constant
!                    per element
!     eps      o     Array containing the value of the strain epsilon in the
!                    integration points
!                               k
!                    epsilon  (x ) = eps(k,i)
!                            i
!     igprob   i     Type of stress-strain relations
!                    Possible values:
!                    1: plane stress, isotropic material
!                       or 3D, isotropic material (Cartesian)
!                    2: plane strain, isotropic material (cartesian)
!                    3: axi-symmetric stress, isotropic material (axi-symmetric)
!                    4: plane stress, orthotropic material (cartesian)
!                    5: plane strain, orthotropic material (cartesian)
!                    6: axi-symmetric stress, orthotropic material
!                       (axi-symmetric)
!                    7: User defined elasticity matrix (3x3) (cartesian)
!                    8: User defined elasticity matrix (4x4) (axi-symmetric)
!     mdecl    i     Number of integration points in the element
!     ugauss   i     array of length m x ncomp in which the values of
!                    u in the integration points are stored the sequence:
!                    u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                     i   j
!     xgauss   i     Array of length m x 2 containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                      i                    i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k

!    k        General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( ndim.eq.2 ) then

!     --- ndim = 2

         if ( m.eq.1 ) then

!        --- m = 1    linear triangle   or  bilinear quadrilateral

            if ( igprob.eq.3 .or. igprob.eq.6 .or. igprob.eq.8 ) then

!           --- igprob = 3  Axi-symmetric stress analysis

               eps(1,1) = dudx(1,1,1)
               eps(1,2) = dudx(1,2,2)
               eps(1,3) = ugauss(1,1)/xgauss(1,1)
               eps(1,4) = dudx(1,2,1)+dudx(1,1,2)

            else

!           --- igprob = 1, 2, 3, 4 or 7  Plane stress or plane strain

               eps(1,1) = dudx(1,1,1)
               eps(1,2) = dudx(1,2,2)
               eps(1,3) = dudx(1,2,1)+dudx(1,1,2)

            end if

         else if ( jdiag.eq.1 ) then

!        --- n # 3   and jdiag = 1    general element, Newton Cotes rule

            if ( igprob.eq.3 .or. igprob.eq.6 .or. igprob.eq.8 ) then

!           --- igprob = 3  Axi-symmetric stress analysis

               do k = 1, m
                  eps(k,1) = dudx(k,1,1)
                  eps(k,2) = dudx(k,2,2)
                  if ( xgauss(k,1).ne.0d0 ) then
                     eps(k,3) = ugauss(k,1)/xgauss(k,1)
                  else
                     eps(k,3) = dudx(k,1,1)
                  end if
                  eps(k,4) = dudx(k,1,2)+dudx(k,2,1)
               end do

            else

!           --- igprob = 1, 2, 3, 4 or 7  Plane stress or plane strain

               do k = 1, m
                  eps(k,1) = dudx(k,1,1)
                  eps(k,2) = dudx(k,2,2)
                  eps(k,3) = dudx(k,2,1)+dudx(k,1,2)
               end do

            end if

         else

!        --- n # 3   and jdiag = 2    general element

            if ( igprob.eq.3 .or. igprob.eq.6 .or. igprob.eq.8 ) then

!           --- igprob = 3  Axi-symmetric stress analysis

               do k = 1, m
                  eps(k,1) = dudx(k,1,1)
                  eps(k,2) = dudx(k,2,2)
                  eps(k,3) = ugauss(k,1)/xgauss(k,1)
                  eps(k,4) = dudx(k,1,2)+dudx(k,2,1)
               end do

            else

!           --- igprob = 1, 2, 3, 4 or 7  Plane stress or plane strain

               do k = 1, m
                  eps(k,1) = dudx(k,1,1)
                  eps(k,2) = dudx(k,2,2)
                  eps(k,3) = dudx(k,2,1)+dudx(k,1,2)
               end do

            end if

         end if
      else

!     --- ndim = 3

         if ( jdercn.eq.0 ) then

!        --- derivatives constant

            do k = 1, m
               eps(k,1) = dudx(1,1,1)
               eps(k,2) = dudx(1,2,2)
               eps(k,3) = dudx(1,3,3)
               eps(k,4) = dudx(1,1,2) + dudx(1,2,1)
               eps(k,5) = dudx(1,2,3) + dudx(1,3,2)
               eps(k,6) = dudx(1,3,1) + dudx(1,1,3)
            end do

         else

!        --- derivatives variable

            do k = 1, m
               eps(k,1) = dudx(k,1,1)
               eps(k,2) = dudx(k,2,2)
               eps(k,3) = dudx(k,3,3)
               eps(k,4) = dudx(k,1,2) + dudx(k,2,1)
               eps(k,5) = dudx(k,2,3) + dudx(k,3,2)
               eps(k,6) = dudx(k,3,1) + dudx(k,1,3)
            end do

         end if
      end if
      end
