      subroutine elri01( dphdx0, fr, detf0, w0, dphidx, dfgrad, w, &
                         ugausr, xgausr, ugauss, xgauss, &
                         ndim, jcart, nip, nnode )
! ======================================================================
!
!        programmer     Sander Gielen
!        version 1.2    date   21-01-1998 Marcel Rutten,extension to axisymmetry
!        version 1.1    date   12-11-1997 added detf0 computation
!        version 1.0    date   28-01-1997
!
!
!
!   copyright (c) 1997-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   This routine computes the deformation gradient in the center of
!   the element. For linear 3D elements (tetraeders m=4 or bricks m=8 )
!   this is done by averaging.
! **********************************************************************
!
!                       KEYWORDS
!
!     elasticity_equation
!     solid
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer nip, nnode, ndim, jcart
      double precision fr(ndim, ndim), dfgrad(nip, ndim, ndim), detf0, &
                       dphdx0(nnode,3), ugauss(nip,ndim), &
                       xgauss(nip,ndim), ugausr(2), &
                       xgausr(2), dphidx(nnode,nip,ndim), w(nip), w0

!     detf0      o    determinant of deformation gradient of RIP
!     dfgrad     i    to store the deformation gradient
!                     dfgrad(k,j,i) = f_ij(x^k)
!     dphdx0     i    derivatives of basis functions for RIP.
!     dphidx     i    Array of length n x m x ndim  containing the derivatives
!                     k: node; j: intpt; i: dir
!     fr         o    to store deformation gradient in the center
!     jcart      i    indication whether the element is a triangle with
!                     cartesian coordinates (1)
!                     or a circular triangle with cylinder symmetric
!                     coordinates (2)
!     ndim       i    Dimension of the space.
!     nip        i    number of integration points
!     nnode      i    number of nodes
!     ugausr     i    ?
!     ugauss     i    Preceding solution in Gauss points
!     w          i    array of length m containing the weights for integration
!     w0         i    ?
!     xgausr     i    ?
!     xgauss     i    array of length m x ndim containing the co-ordinates of
!                     the gauss points.  array xgauss is only filled when gauss
!                     integration is applied.
!                     xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                     zg_i = xgauss (i,3)
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer t1, t2, t3
      double precision rnip

!     rnip       double precision of nip
!     t1         counter
!     t2         counter
!     t3         counter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      rnip = dble( nip )

!     --- reset w0

      w0 = 0d0

      do t1 = 1, 2
        xgausr(t1) = 0d0
        ugausr(t1) = 0d0
      end do

!     --- reset fr

      do t1 = 1, ndim
        do t2 = 1, ndim
          fr(t1,t2) = 0d0
        end do
      end do

!     --- reset dphdx0

      do t1 = 1, nnode
        do t2 = 1, ndim
          dphdx0(t1,t2) = 0d0
        end do
      end do

!     --- do the averaging

      do t1 = 1,nip

!     --- 1: weights:

         w0 = w0 + w(t1)

!        --- 2: deformation gradient:

        do  t2 = 1,ndim
            do t3 = 1,ndim
               fr(t2,t3) = fr(t2,t3) + dfgrad(t1,t2,t3)/rnip
            end do
        end do

!       --- 3: derivatives of basis functions:

         do t2 = 1,ndim
            do t3 = 1,nnode
               dphdx0(t3,t2) = dphdx0(t3,t2) + dphidx(t3,t1,t2)/rnip
            end do
         end do

!        --- 4:  Gauss points:

          xgausr(1) = xgausr(1) + (1d0/rnip) * xgauss(t1,1)
          xgausr(2) = xgausr(2) + (1d0/rnip) * xgauss(t1,2)

!         --- 5:  Solution in Gauss points:

          ugausr(1) = ugausr(1) + (1d0/rnip) * ugauss(t1,1)
          ugausr(2) = ugausr(2) + (1d0/rnip) * ugauss(t1,2)

      end do

!     --- determinant of deformation in RI point:

      if ( ndim.eq.3 ) then

        detf0 = fr(1,1)*( fr(2,2)*fr(3,3) - fr(2,3)*fr(3,2) ) + &
                fr(1,2)*( fr(2,1)*fr(3,3) - fr(2,3)*fr(3,1) ) + &
                fr(1,3)*( fr(2,1)*fr(3,2) - fr(2,2)*fr(3,1) )

      else if ( ndim.eq.2 ) then

        detf0 = fr(1,1)*fr(2,2) - fr(1,2)*fr(2,1)

        if ( jcart.eq.2 ) then
          detf0 = detf0 * ( 1d0 + ugausr(1)/xgausr(1))
        end if

      end if

      end
