      subroutine sepelcrmt (  ndim, icount, nnode, nip, nsc, ichoice )
! ======================================================================
!
!        programmer     Sander Gielen
!        version 2.0    date   11-11-1997 completely changed for souza-
!                                         method (SG)
!        version 1.0    date   16-04-1997
!
!   copyright (c) 1997-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Assemble correction matrix for nonlinear solid analysis,
!     with enhanced convergence for (nearly) incompressible
!     problems. The additional stiffness matrix is probably non-symmetric!
!     Theory: Design of simple low order finite elements for large
!             strain analysis of nearly incompressible solids.
!             E.A. de Souza Neto, D. Peric, M. Dutko, D.R.J. Owen
!             Int. J. Solids Structures vol.33, no.20-22, pp.3277-3296
!             (1996)
!     Adapted for Total Lagrange by Sander Gielen
! **********************************************************************
!
!                       KEYWORDS
!
!    matrix
!    mechanics
!    non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulebasefn
      use sepmoduleelm
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          icount, ndim, nip, nnode, nsc, ichoice


!     blwork     i    blmatrix: blwork(i,j,k,l)
!                     i: corresponding stress/strain vector postion
!                     j: direction
!                     k: node
!                     l: integration point
!     detf       i    determinant of deformation gradient
!     detfr      i    determinant of at centre
!     dfgrad     i    deformation gradient
!     dfgrdr     i    ?
!     dphdxr     i    ?
!     dphidx     i    Array of length n x m x ndim  containing the derivatives
!                     of the basis functions in the sequence:
!                     d phi / dx (xg ) = dphidx (i,k,1);
!                          i        k
!                     d phi / dy (xg ) = dphidx (i,k,2);
!                          i        k
!                     d phi / dz (xg ) = dphidx (i,k,3);
!                          i        k
!                     If the element is a linear simplex, the
!                     derivatives are constant and only k = 1 is filled.
!     ichoice    i    choice number
!                        1: Souza method on 1st Piola-Kirchhoff stress
!                        2: Souza method on 2nd Piola-Kirchhoff stress
!     icount     i    number of entries in elemmt(icount,icount)
!     ndim       i    number of dimensions
!     nip        i    number of integration points
!     nnode      i    number of nodes
!     nsc        i    number of stress components
!     se         i    tangential material stiffness
!     w          i    weight factors for numerical integration
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer eln1, eln2, di1, di2, idx(3,3), c1, c2, c3, &
              ix1, ix2, ipn, mnode, mip
      parameter ( mnode = 8, mip = 8 )
      double precision st, tmp1, tmp2, ct, t1, t2, t3, t8, &
                       t12, t13, t14, t15, t20, t21, t22, t23
      double precision fdu(3,mnode,mip), fdur(3,mnode), ff(3,3,mip)
      double precision, pointer :: pksest(:,:), se(:,:,:), blwork(:,:,:,:), &
                                   dphidx(:,:,:), dfgrad(:,:,:), w(:), &
                                   dphdxr(:,:), dfgrdr(:,:), detfr

!     c1         counter
!     c2         counter
!     c3         counter
!     ct         correction term
!     di1        dimension counter
!     di2        dimension counter
!     eln1       element node counter
!     eln2       element node counter
!     fdu
!     fdur
!     ff
!     idx        indexing matrix
!     ipn        integration point counter
!     ix1        index counter
!     ix2        index counter
!     mip        Maximum number of integration points
!     mnode      Maximum number of nodes per element
!     st         entry stiffness
!     t1         temporary quantity
!     t12        temporary quantity
!     t13        temporary quantity
!     t14        temporary quantity
!     t15        temporary quantity
!     t2         temporary quantity
!     t20        temporary quantity
!     t21        temporary quantity
!     t22        temporary quantity
!     t23        temporary quantity
!     t3         temporary quantity
!     t8         temporary quantity
!     tmp1       temporary variable
!     tmp2       temporary variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELSGMD01  Assemble data for Souza method
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data    idx /  1, 4, 6, 4, 2, 5, 6, 5, 3 /
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      pksest => E%pksest
      se => E%Dmat
      blwork => E%bl
      dphidx => B%phider
      dfgrad => gradoldvec
      dphdxr => E%detf0(:,1,:)
      dfgrdr => E%gradred(1,:,:)
      detfr => E%detfr(1)
      w => B%w

!     --- compute auxilary data

      E%dphidx => B%phider
      E%dfgrad => dfgrad
      call sepelsgmd01 ( fdu, nip, nnode )
      E%dphidx => E%detf0
      E%dfgrad => E%gradred
      call sepelsgmd01 ( fdur, 1 , nnode )

      do ipn = 1,nip
         do eln1 = 1,nnode
            do di1 = 1,ndim
               fdu(di1,eln1,ipn) =  fdu(di1,eln1,ipn) -fdur(di1,eln1)
            end do
         end do
      end do

!     --- compute ff = F^c . F

      do ipn = 1,nip
         t1 = dfgrad(ipn,1,1)**2
         t2 = dfgrad(ipn,1,2)**2
         t3 = dfgrad(ipn,1,3)**2
         t8 = dfgrad(ipn,1,1)*dfgrad(ipn,2,1) &
             +dfgrad(ipn,1,2)*dfgrad(ipn,2,2) &
             +dfgrad(ipn,1,3)*dfgrad(ipn,2,3)
         t12 = dfgrad(ipn,1,1)*dfgrad(ipn,3,1) &
              +dfgrad(ipn,1,2)*dfgrad(ipn,3,2) &
              +dfgrad(ipn,1,3)*dfgrad(ipn,3,3)
         t13 = dfgrad(ipn,2,1)**2
         t14 = dfgrad(ipn,2,2)**2
         t15 = dfgrad(ipn,2,3)**2
         t20 = dfgrad(ipn,2,1)*dfgrad(ipn,3,1) &
              +dfgrad(ipn,2,2)*dfgrad(ipn,3,2) &
              +dfgrad(ipn,2,3)*dfgrad(ipn,3,3)
         t21 = dfgrad(ipn,3,1)**2
         t22 = dfgrad(ipn,3,2)**2
         t23 = dfgrad(ipn,3,3)**2
         ff(1,1,ipn) = t1+t2+t3
         ff(1,2,ipn) = t8
         ff(1,3,ipn) = t12
         ff(2,1,ipn) = t8
         ff(2,2,ipn) = t13+t14+t15
         ff(2,3,ipn) = t20
         ff(3,1,ipn) = t12
         ff(3,2,ipn) = t20
         ff(3,3,ipn) = t21+t22+t23

      end do

!     --- assemble the matrix

      do eln1 = 1, nnode
         do eln2 = 1, nnode

!        --- loop over degrees of freedom (=ndim) twice

            do di1 = 1, ndim
               do di2 = 1, ndim

!              --- compute the entry number in the matrix

                  ix1 = (eln1-1)*ndim + di1
                  ix2 = (eln2-1)*ndim + di2
                  st = 0d0

!                 --- NORMAL SYMMETRIC TERMS

                  if ( ix1.le.ix2 ) then

!                 --- loop over the integration point

                     do ipn = 1, nip
                        tmp1 = 0d0
                        tmp2 = 0d0

!                       --- contribution to stiffness from stress

                        if ( di1.eq.di2 ) then

                           if ( ichoice.eq.1 ) then
                              ct = (detfr/E%detf(ipn))**(1d0/3d0)
                           else
                              ct = 1d0
                           end if

                           do c1=1,ndim
                              do c2=1,ndim

                                 tmp1 = tmp1+pksest(idx(c1,c2),ipn) &
                                      *dphidx(eln1,ipn,c1) &
                                      *dphidx(eln2,ipn,c2)*ct

                              end do
                           end do
                        end if

!                       --- tangential contribution to stiffness

                        if ( ichoice.eq.1 ) then
                           ct = detfr/E%detf(ipn)
                        else
                           ct = (detfr/E%detf(ipn))**(2d0/3d0)
                        end if

                        do c1=1,nsc
                           do c2=1,nsc
                              tmp2=tmp2+ &
                                   blwork(c1,di1,eln1,ipn) &
                                   *se(c1,c2,ipn)*ct &
                                   *blwork(c2,di2,eln2,ipn)
                           end do
                        end do

!                       --- sum contribution per integration point

                        st = st + w(ipn)*(tmp1+tmp2)
                     end do

!                    --- add to matrix

                     E%mat(ix1,ix2) = E%mat(ix1,ix2) + st

!                    --- symmetry

                     if ( ix1.ne.ix2 ) then
                        E%mat(ix2,ix1) = E%mat(ix2,ix1) + st
                     end if

                  end if

!                 --- A-SYMMETRIC CORRECTION TERMS

                  st = 0d0
                  do ipn = 1, nip

                     tmp1 = 0d0
                     tmp2 = 0d0

                     if ( ichoice.eq.1 ) then
                        ct = detfr/E%detf(ipn)
                        t1 = ct**(1d0/3d0)

                        do c1 = 1,nsc
                           tmp1 = tmp1 + blwork(c1,di1,eln1,ipn) &
                                 *pksest(c1,ipn)*t1
                        end do
                        tmp1 = tmp1*fdu(di2,eln2,ipn)
                        ct = ct/3d0
                     else
                        ct = ((detfr/E%detf(ipn))**(2d0/3d0))/3d0
                     end if

                     do c1 = 1,nsc
                        do c2 = 1,ndim
                           do c3 = 1,ndim
                              tmp2=tmp2+ &
                                   blwork(c1,di1,eln1,ipn) &
                                   *se(c1,idx(c2,c3),ipn)*ct &
                                   *ff(c2,c3,ipn)
                           end do
                        end do
                     end do
                     tmp2 = tmp2*fdu(di2,eln2,ipn)

                     st = st + w(ipn)*(tmp1+tmp2)
                  end do

!                 --- add to matrix, NOTE matrix is stored transposed!

                  E%mat(ix2,ix1) = E%mat(ix2,ix1) + st

               end do
            end do

!           --- end dimension loop

         end do
      end do

!     --- end nodal loop

      end
