      subroutine sepelmshell
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element matrix for shell element
!     Old subroutine el8121
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elasticity_equation
!     shell
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k, l1, l2, ip, iq, lm, l, iqlow, iqupp, nd
      double precision :: som
      double precision, pointer :: s(:,:), w(:,:), d(:,:,:)
      double precision, pointer :: bsh(:,:,:,:,:)

!     bsh            B matrix for shell element
!     d              Dmatrix for shell element
!     i              General loop variable
!     ip             General loop variable
!     iq             General loop variable
!     iqlow          Lower bound of iq
!     iqupp          Upper bound of iq
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     l1             general loop variable
!     l2             general loop variable
!     lm             Upper bound of l2-loop
!     nd             Number of unknowns per node
!     s              Element matrix to be filled
!     som            Help parameter to compute a sum
!     w              array of length m containing the weights for quadrature
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                T              T
!     s   = B phi  D  B phi  = b   d   b
!      ij        i         j    ip  pq  qj
!
!
!              |   x      x      x      x      x  |
!              |   x      x      x      x      x  |
!     s    =   |   x      x      x      x      x  |
!      ij      |   x      x      x      x      x  |
!              |   x      x      x      x      x  |
!
!
!
!     s  (l ,l ) = sum B phi (p,l ) D(p,q) B phi (q,l )
!      ij  1  2     pq      i    1              j    2
!
!     These formulae must also be integrated by the integration rule
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmshell' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmshell'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      d => E%dmat
      bsh => E%bsh
      w => B%fibjac
      s => E%mat

      nd = 5

!     --- Matrix is symmetrical, so first lower part is stored
!         The B-matrix is not constant

      if ( E%stress_strain==0 ) then

!     --- isotropic material, most of the D-matrix = 0

         do i = 1, n
            do j = 1, i
               do l1 = 1, 5
                  if(i.eq.j) then
                     lm = l1
                  else
                     lm = 5
                  end if
                  do l2 = 1, lm
                     som = 0d0
                     do ip = 1, nd
                        if ( ip.le.2 ) then
                           iqlow = 1
                           iqupp = 2
                        else
                           iqlow = ip
                           iqupp = ip
                        end if
                        do iq = iqlow, iqupp
                           do l = 1, 2
                              do k = 1, m
                                 som = som+w(k,l)*bsh(ip,l1,i,k,l)* &
                                       d(ip,iq,k)*bsh(iq,l2,j,k,l)
                              end do
                           end do
                        end do
                     end do
                     s(5*i-5+l1,5*j-5+l2) = som
                  end do
               end do
            end do
         end do

      else

!     --- general case

         do i = 1, n
            do j = 1, i
               do l1 = 1, 5
                  if(i.eq.j) then
                     lm = l1
                  else
                     lm = 5
                  end if
                  do l2 = 1, lm
                     som = 0d0
                     do ip = 1, nd
                        do iq = 1, nd
                           do l = 1, 2
                              do k = 1, m
                                 som = som+w(k,l)*bsh(ip,l1,i,k,l)* &
                                       d(ip,iq,k)*bsh(iq,l2,j,k,l)
                              end do
                           end do
                        end do
                     end do
                     s(5*i-5+l1,5*j-5+l2) = som
                  end do
               end do
            end do
         end do

      end if

!     --- Store upper part

      do i = 1, icount
         do j = 1, i-1
            s(j,i) = s(i,j)
         end do
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelmshell'

      end if  ! ( debug )
      call erclos ( 'sepelmshell' )

      end subroutine sepelmshell
