******************************************************************************
*
*      File:  harbour.f
*
*      Contents:  Main program for the harbour test example described
*                 in the SEPRAN user's manual 8.2
!                 In this program all steps are performed, i.e.
!                 mesh generation, computation and post-processing
*
*      Usage:     Compile and link this program with the SEPRAN libraries
*                 Run this program with input: harbour.dat
*
*      version 2.0    date   31-03-93 (Adapated to new subroutines)
*
******************************************************************************
*
*
      program harbour
      use sepmoduleoldrouts
      implicit none
      integer kmesh, kprob, intmat, matr, iphase,
     +        isol, irhsd, iuser(100), iinbld(1), imag,
     +        iinvec(2)
      integer istart, ioutp, itime, ichois, iinput, jmetod,
     +        iprob, massmt, islold, ipos, jdim, jpoint, i, number,
     +        jsmoot, istop, ncntln, irotat, linopt
      double precision user(100), contln(20), rinvec(1)
      double precision rinput, xjdim, plotfm, yfact, view, base, height

!     --- Some local constants are defined in the main program and
!         tranported to the boundary funcion subroutine cfuncb, through
!         the common block hacons

      double precision pi, omega, g, angle
      common / hacons/ pi,omega,g,angle

!     --- Clear all arrays in order to prevent unwanted side effects

      kmesh = 0
      kprob = 0
      intmat= 0
      matr = 0
      isol = 0
      irhsd = 0
      iuser = 0
      iphase= 0
      imag = 0
      user = 0d0
      contln = 0d0

!     --- Set first positions of variable length arrays

      contln(1) = 20
      iuser(1)  = 100
      user(1)   = 100

!     --- Initialise constants
!         g is acceleration of gravity
!         angle is angle of incidence of incoming waves
!         omega is angular velocity of waves

      pi = 4*atan(1.0d0)
      g = 9.81d0
      angle = 270d0
      omega = 0.056d0

!     --- Start SEPRAN

      istart = 0
      irotat = 1
      ioutp = 0
      itime = 0
      call start(istart, irotat, ioutp, itime)

!     --- Mesh generation

      ichois  =  0
      call mesh ( ichois, iinput, rinput, kmesh )

!     --- Problem definition

      ichois = 0
      call probdf ( ichois, kprob, kmesh, iinput )

!     --- Define structure of matrix (symmetrical, complex profile storage)

      jmetod = 3
      call commat ( jmetod, kmesh, kprob, intmat )

!     --- Fill essential boundary conditions

      call presdf ( kmesh, kprob, isol )

!     --- Fill coefficients for the differential equation

      iprob = 1
      call filcof ( iuser, user, kprob, kmesh, iprob )

!     --- Build matrix and right-hand side (default situation)

      iinbld(1) = 0
      call build ( iinbld, matr, intmat, kmesh, kprob, irhsd,
     +             massmt, isol, islold, iuser, user )

!     --- Solve system of equations by a direct (profile) method

      ipos = 0
      call solve ( ipos, matr, isol, irhsd, intmat, kprob )

!    --- compute and print magnitude in each point

      iinvec(1) = 2
      iinvec(2) = 29
      rinvec(1) = 1
      call manvec ( iinvec, rinvec, isol, isol, imag, kmesh,
     +              kprob )
      jdim = 1
      call prinov ( imag, kmesh, kprob, jdim, 'amplitude', xjdim,
     +              jpoint )

!    --- compute and print phase in each point

      iinvec(2) = 30
      call manvec ( iinvec, rinvec, isol, isol, iphase, kmesh,
     +              kprob )
      jdim = 1
      call prinov ( imag, kmesh, kprob, jdim, 'phase', xjdim,
     +              jpoint )

!    --- plot contour lines of magnitude at level -1.00, -0.80, -0.60,
!                                                 . . . , 1.00

      do 200 i = 1, 11
         contln(i+5) = -1.00+(i-1)*0.20
200   continue

      number = 1
      ncntln = 11
      plotfm = 10
      yfact  = 1
      jsmoot = 0
      call plotc1 ( number, kmesh, kprob, imag, contln, ncntln, plotfm,
     +              yfact, jsmoot )

!    --- plot phase : contour levels -165, -135, -105,  .....  165.

      do 300 i = 1, 12
          contln(i+5) = pi*(-13.0+2*i)/12.0d0
300   continue

      number = 1
      ncntln = 12

      call plotc1 ( number, kmesh, kprob, iphase, contln, ncntln,
     +              plotfm, yfact, jsmoot )

!    --- 3d-plot of elevation:

      number = 1
      view   = 135d0
      linopt = 3
      base   = 10d0
      height = 10d0
      call plot3d ( number, kmesh, kprob, isol, base, height, view,
     +              linopt )

!     --- Stop SEPRAN

      istop = 0
      call finish ( istop )

      end



      subroutine cfuncb ( ichois, x, y, z, comval )
      implicit none
      complex * 16 comval
      integer ichois
      double precision x, y, z, depth, alpha, ak, arg

      double precision pi, omega, g, angle
      common / hacons / pi, omega, g, angle

!    --- compute boundary condition in nodal point (x, y, z) :
!        determine wave-number ak :

      depth = 15.0d0

!     angle of incoming wave :

      alpha = (pi/180.0d0)*angle
      ak = omega/sqrt(g*depth)
      arg = ak*(x*cos(alpha)+y*sin(alpha))

!     prescribed elevation :

      comval = dcmplx ( cos(arg), sin(arg) )

      end

      double complex function cfunccf ( ifunc, x, y, z )
      implicit none
      integer ifunc
      double precision x, y, z, depth, alpha, ak, arg

      double precision pi, omega, g, angle
      common / hacons / pi, omega, g, angle
      select case ( ifunc )
      case(1)

         depth = 15.0d0

!        angle of incoming wave :

         alpha = (pi/180.0d0)*angle
         ak = omega/sqrt(g*depth)
         arg = ak*(x*cos(alpha)+y*sin(alpha))

!        prescribed elevation :

         cfunccf = dcmplx ( cos(arg), sin(arg) )

      case default

!     --- Option not programmed yet

         cfunccf = 0d0
         call errchr ( 'cfunccf', 1 )
         call errint ( ifunc, 1 )
         call errsub ( 1930, 1, 0, 1 )
         call instop

      end select  ! case ( ifunc )

      end



      subroutine cfuncf ( ichois, x, y, z, comval )
      implicit none
      complex * 16 comval
      integer ichois
      double precision x, y, z, gamma, depth, ak

      double precision pi, omega, g, angle
      common / hacons / pi, omega, g, angle

      gamma = 0d0
      depth = 15.0d0
      ak = omega/sqrt(g*depth)

      if ( ichois.eq.1 ) then
          comval = dcmplx ( -ak*ak, -gamma*ak/depth )
      end if
      end
