      program heatstrs
! ======================================================================
!
!
!        programmer     Guus Segal
!        version 1.0    date   30-09-92
! ********************************************************************
!
!                         DESCRIPTION
!
!   This program computes the coupled heat equation and elasticity
!   equations
!
!   The heat equation is solved with self selecting step size
!   At fixed output times the result is outputted and the
!   stress is computed as function of the computed temperature
!
! ********************************************************************
!
!                      KEYWORDS
!
!  heat_equation
!  main_program
!  elasticity_equation
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmoduleoldrouts
      use sepmoduleconsts
      use sepmoduletime
      implicit none
! ********************************************************************
!
!                      COMMON BLOCKS
!
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer kmesh, kprob, intmat(2), isol(3),
     .        iuser(100,2), icount, itytim, iinbld(10), matr,
     .        irhsd, idummy, iprob, iposst, ichois, icheld, ix,
     .        jdegfd, ivec
      double precision user(100,2), epsusr
      data iinbld /10*0/

!    epsusr  Required accuracy of the time-solver
!    icheld  Defines type of derived quantity to be computed
!            6: stress tensor
!    ichois  Choice parameter for subroutine DERIVA (not actually used)
!    icount  General loop variable
!    idummy  Dummy parameter
!    iinbld  Contains control parameters for subroutine BUILD, see PG
!    intmat  Contains information with respect to the storage of the matrix
!            INTMAT(i,1) refers to the temperature problem
!            INTMAT(i,2) refers to the elasticity problem
!    iposst  Indicates if the matrix to be solved is positive definite (1)
!            or not (0)
!    iprob   Problem number
!    irhsd   Contains information concerning the elasticity right-hand side
!    isol    Standard SEPRAN array containing information of the solution
!            In ISOL(i,1) information concerning the displacements is stored
!            In ISOL(i,2) information concerning the temperature is stored
!            In ISOL(i,3) information concerning the stresses is stored
!    itytim  Defines the type of problem to be considered
!            1 means linear problem
!    iuser   User array in which information with respect to the coefficients
!            of the differential equation is stored
!            In IUSER(i,1) information concerning the displacements is stored
!            In IUSER(i,2) information concerning the temperature is stored
!    ivec    Choice parameter for subroutine DERIVA (not actually used)
!    ix      Choice parameter for subroutine DERIVA (not actually used)
!    jdegfd  Choice parameter for subroutine DERIVA (not actually used)
!    kmesh   Contains inforamtion with respect to the mesh
!    kprob   Contains information with respect to the problem definition
!    matr    Contains information concerning the elasticity stiffness matrix
!    user    double precision user array in which information with respect to
!            the coefficients of the differential equation is stored
!            In USER(i,1) information concerning the displacements is stored
!            In USER(i,2) information concerning the temperature is stored
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!     create  Create a SEPRAN solution vector
!     finish  Stop SEPRAN
!     outtim  Write solution to file sepcomp.out
!     sepstr  Start SEPRAN and read mesh
!     soltim  Solve time-dependent problem
! ********************************************************************
!
!                    I/O
!
!    This program reads the standard SEPRAN files.
!    It assumes that the mesh has been generated by program SEPMESH
!    with input file heatstrs.msh and output file meshoutput
!    It reads input from the standard input file (heatstrs.prb)
!    It produces one file: sepcomp.inf for program SEPPOST
!    Program SEPPOST requires the input file heatstrs.pst
! ********************************************************************
!
!                    ERROR MESSAGES
!
!   -
! ********************************************************************
!
!                    PSEUDO CODE
!
!    Perform standard SEPRAN initialisations
!    Start SEPRAN and read mesh
!    Create initial condition for the temperature equation
!    t := 0
!    Fill boundary conditions for elasticity equations
!    For tout := 0.1 (0.1) 1 do
!       Solve heat equation from t to tout
!       t := tout
!       Write solution for post processing purposes
!    Stop SEPRAN
! ======================================================================
!
!    *  Standard SEPRAN initialisations

      user(1,1)  = 100
      iuser(1,1) = 100
      user(1,2)  = 100
      iuser(1,2) = 100
      kmesh = 0
      kprob = 0
      intmat = 0
      isol = 0
      matr = 0
      irhsd = 0
      user(3,1)  = 0
      iuser(3,1) = 0
      user(3,2)  = 0
      iuser(3,2) = 0

!    *  Start SEPRAN  (read mesh etc.)

      call sepstr( kmesh, kprob, intmat )

! *   fill initial temperature in isol :

      t      = 0d0
      t0     = 0d0
      call create ( 0, kmesh, kprob, isol )

!    * fill boundary conditions for the elasticity equations

      call presdf ( kmesh, kprob, isol )

!
! *******************************************************************
!
! *   set parameters for subroutine soltim:
!
! *******************************************************************
!
      itytim = 1
      ichtim = 1
      epsusr = 1d-2

!    * read end time from the standard input file

      read ( irefre, * ) tend
!
! *******************************************************************
!
! *   compute solution for t = tend/4, tend/2, 3*tend/4 and tend
!
! *******************************************************************
!
!
      do 200 icount = 1, 4
         tout = icount*tend*.25d0
         call soltim ( itytim, ichtim, kmesh, kprob, intmat(2),
     .                 isol(2), epsusr, user(1,2), iuser(1,2) )

!     *  Fill coefficients for the elasticity problem
!        Information has to be read for the first step only

         iprob = 1
         if ( icount.eq.1 ) call filcof ( iuser(1,1), user(1,1), kprob,
     .      kmesh, iprob )

!     *  Build system of equations for elasticity problem

         iinbld(1) = 10
         iinbld(2) = 1
         iinbld(9) = 1
         iinbld(10) = 2
         call build ( iinbld, matr, intmat(1), kmesh, kprob, irhsd,
     .                idummy, isol(1), isol(1), iuser(1,1), user(1,1) )

!      *  Solve system of equations

         iposst = 1
         call solve ( iposst, matr, isol(1), irhsd, intmat(1),
     .                kprob )

!      *  Compute the stresses and store into isol(i,3)

         ichois = 201
         icheld = 6
         call deriva ( ichois, icheld, ix, jdegfd, ivec, isol(3),
     .                 kmesh, kprob, isol(1), isol(1), iuser(1,1),
     .                 user(1,1), idummy )

!     *    write solution to file sepcomp.out

         call outtim ( t, kmesh, kprob, isol )

200   continue

!    *  Close SEPRAN

      call finish(0)
      end
! **********************************************************************
! *
! *   subroutine filtim for the control of the time-dependent process
! *
! **********************************************************************
      subroutine filtim ( kmesh, kprob, intmat, isol, matrs, matrm,
     .                    irhsd, islold, user, iuser )
      use sepmoduleoldrouts
      implicit none
      double precision user(*)
      integer kmesh, kprob, intmat, isol(*), islold(*),
     .        matrs, matrm, irhsd, iuser(*)
      integer ictimen, iprob, iinbld(4)
      save ictimen
      data ictimen /0/
!
! *******************************************************************
! *
! *   User written subroutine for the incorporation of boundary
! *   conditions and the determination of the matrices M and S
! *   Example : Heat equation on the unit square .
! *
! *******************************************************************
! *
! *   Fill matrices M and S and right-hand side f
! *   Since the matrices are time-independent this is only
! *   necessary at t = 0
! *
! *******************************************************************
      if ( ictimen.eq.0 ) then

!      *  Information is only read the first step
!         The parameter ictimen is used to distinguish between first and
!         next step

         iprob = 2
         call filcof ( iuser, user, kprob, kmesh, iprob )

         iinbld(1) = 4
         iinbld(2) = 1
         iinbld(3) = 0
         iinbld(4) = 1
         call build ( iinbld, matrs, intmat, kmesh, kprob,
     .                irhsd, matrm, isol(1), islold(1), iuser, user )
         ictimen = 1

      end if
      end
