/*
 *  Copyright (c) 1991  Philips Export B.V.
 * 
 *  Philips Research Laboratories
 *  Eindhoven, The Netherlands
 *
 *  Author    : G.F.M. Haenen
 *              F.W.G.M. Wouters
 *
 *  Date      : July, 1991
 *
 *  Modified  : Kees Lemmens, June 24, 1999
 *            - fixed compiler warnings and some small bugs
 *            - global struct "usr" replaced with a local one :
 *              which makes the code a little less grubby 
 *            - introduced the zoomflags (2=in, 3=out, 1=new, 0=redraw)
 *            - fixed text outside clipping boundaries (use Xlib if angle==0)
 *
 *            - Dirk De Wachter, 28 jun 1995
 *            - Adaptation for color files
 *            - Colors are only allocated when needed
 *            - SHIFT zoom allows for different aspect ratios
 *            - Changed panning for easier positioning when SHIFT is used
 *            - new routines for GIF file production
 *
 *  Function  : translates primitives into x calls
 *              initX          : initializes X
 *              redraw         : redraws the current plot
 *              paninout       : sets another viewport 
 *              rubberband     : draws rubberband for zooming
 *
 *  Comment   : 
 *
*/

#ifdef MOTIF
#  include <Xm/Xm.h>
#else /* LESSTIF */
#  include <X11/Intrinsic.h>
#endif

#include <Mrm/MrmAppl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "plotfun.h"

#define MARGIN    10.0
#define PW        0.0
#define MAXSTACK  256                             
#define MAXCOLORS 256                             
#define ABS(x)    ((x) < 0 ? (-x) : (x))
#define SIGN(x)   ((x) < 0 ? (-1) : 1)
#define X(x)      (Dimension)(           (x + xoffset) * scale)
#define Y(y)      (Dimension)(scr.ymax - (y + yoffset) * scale)
 struct color_t
{
   unsigned short red, green, blue;
} color_t;

extern Widget graphics;
extern struct color_t colortable[MAXCOLORS];

static window_t ldim;      /* local dimension (for zooming) */

static Display  *dpy;
static Window   win;
static Pixmap   pixmap;
static GC       gc = NULL, gcxor;
static int      planes;
static struct   window_t scr;
static double   xoffset, yoffset, scale;
static unsigned long cols[MAXCOLORS];
static unsigned char color_use[MAXCOLORS/8+1];

static void initX();
void def_colortable();

void clear_colors() {
  int i;
  
  color_use[0]=0x03; /* always use back+forground color */
  for (i=1; i<MAXCOLORS/8+1; i++) color_use[i]=0;
}

void add_color(int color) {
  if (color < 0) return;
  color++;
  if (color < MAXCOLORS) color_use[color/8] |= (0x01<<(color%8));
}

int use_color(int c) {
   return (color_use[c/8] & (0x01<<(c%8)));
}

void New_Colortable(char * cfn,char flag) {
  Colormap  cmap;
  int       i, num_free=0;
  unsigned long free_pixels[MAXCOLORS];
    
  if (!cfn) {
    def_colortable();
    /* gray out "Default Colors" */
    showColorfile(NULL);
    color_status(False);
  }
  else if ((i=read_colortable(cfn)) > 0) { /* MOD KL */
     /* signal if less than two colors are read */
    showColorfile(cfn);
    /* enable "Default Colors"-Widget */;
    color_status(True);
  }
  
  /* erase previous colors */
  for (i=0; i<MAXCOLORS; i++) {
    if (cols[i]) {
      free_pixels[num_free++]=cols[i];
      cols[i]=0;
    }
  }
  dpy    = XtDisplay(graphics);
  cmap   = DefaultColormap(dpy, DefaultScreen(dpy));
  if (num_free) XFreeColors(dpy, cmap, free_pixels, num_free, 0);
  
  /* redraw */
  if (flag>0) sepplot();
}

int read_colortable(char *colfile) {
  FILE *cfp;
  int n, r;
  signed char c;
  unsigned short red, green, blue;
  
  if (access(colfile,R_OK)) return False; /* must be accessible */
  cfp = fopen(colfile,"r");
  
  for (n=1; n<MAXCOLORS; n++) {
    if ((r=fscanf(cfp,"%hu%hu%hu", &red, &green, &blue)) < 3) {
      /* one colordefinition per line 
	 this line is incomplete so skip it */
      if (r==EOF) break;
      n--;
      while ((c=getc(cfp)) != '\n') if (c == EOF) break; /* stop at 
							    end of file */
      if (c != EOF) continue;
    }
    while ((c=getc(cfp)) != '\n') if (c == EOF) break; /* skip till end 
							  of line */
      
    colortable[n].red = red<<8;
    colortable[n].green = green<<8;
    colortable[n].blue = blue<<8;
  }
   
  /* Cleanup */
  fclose(cfp);
  if (n) {
    for (r=n; r<MAXCOLORS; r++) /* set rest to grey */
      colortable[r].red=colortable[r].green=colortable[r].blue=0x8000;
  }
  
  return --n;
}

void def_colortable() {
  
  int i;
  
  /* background = white */
  colortable[0].red = 0xFFFF;
  colortable[0].green = 0xFFFF;
  colortable[0].blue = 0xFFFF;
  
  /* foreground = black */
  colortable[1].red = 0;
  colortable[1].green = 0;
  colortable[1].blue = 0;
  
  /*  The first colour set consists of 21 colours
   *  Start at 1 (internally 2)
   *  10 colours from blue to red
   *  12 colours from red to yellow
   */

  for (i = 0; i < 10; i++) {
    colortable[i+2].red = ((long)0xFFFF)*i/10;
    colortable[i+2].green = 0;
    colortable[i+2].blue = 0xFFFF-((long)0xFFFF)*i/10;
  }
  
  for (i = 0; i < 12; i++) {
    colortable[i+12].red = 0xFFFF;
    colortable[i+12].green = ((long)0xFFFF)*i/11;
    colortable[i+12].blue = 0;
  }
  
  /* The second colour set consists of 41 colours
   *  Start at 23 (internally 24)
   *  20 colours from blue to red
   *  21 colours from red to yellow
   */
  
  for (i = 0; i < 20; i++) {
    colortable[i+24].red = 0xFFFF*((float)i)/20;
    colortable[i+24].green = 0;
    colortable[i+24].blue = 0xFFFF*(1-((float)i)/20);
  }
  for (i = 0; i < 22; i++) {
    colortable[i+44].red = 0xFFFF;
    colortable[i+44].green = ((long)0xFFFF)*i/21;
    colortable[i+44].blue = 0;
  }
  
  /*
   *  The third colour set consists of 41 colours
   *  Start at 63 (internally 64)
   *  20 colours from green to yellow
   *  21 colours from yellow to red
   */
  
  for (i = 0; i < 20; i++) {
    colortable[i+66].red = 0xFFFF*((float)i)/20;
    colortable[i+66].green = 0xFFFF;
    colortable[i+66].blue = 0;
  }
  for (i = 0; i < 22; i++) {
    colortable[i+86].red = 0xFFFF;
    colortable[i+86].green = 0xFFFF*(1-((float)i)/21);
    colortable[i+86].blue = 0;
  }
  
  /*
   *  The fourth colour set consists of 41 colours
   *  Start at 104 (internally 105)
   *  41 colours from white to black
   */
  
  for (i = 0; i < 41; i++) {
    colortable[i+108].red = 0xFFFF*(1-((float)i)/40);
    colortable[i+108].green = 0xFFFF*(1-((float)i)/40);
    colortable[i+108].blue = 0xFFFF*(1-((float)i)/40);
  }
  for (i = 149; i <  MAXCOLORS; i++) { /* set rest to grey */
    colortable[i].red = 0x8000;
    colortable[i].green = 0x8000;
    colortable[i].blue = 0x8000;
  }
}

int x_init(window_t *dim, int flag)
{
   static int stackpos = -1;
   static window_t stack[MAXSTACK];
   
   /*    def_colortable(); */
   
   initX();
   
   if (flag == TRUE)  /* New plot, so use default dimensions from mainprog */
   { 
      stackpos = -1;
      ldim.xmin = dim->xmin;
      ldim.ymin = dim->ymin;
      ldim.xmax = dim->xmax;
      ldim.ymax = dim->ymax;
   }
   
   /*    printf(" x1= %f\n x2=%f\n y1=%f\n y2=%f\n\n",x1, x2, y1, y2);*/
   
   if (flag == 1 || flag == 2) /* Zoom in or first plot */
   {
      stackpos = (stackpos + 1) % MAXSTACK;
      zoom_status(stackpos);
      stack[stackpos].xmin = ldim.xmin;
      stack[stackpos].ymin = ldim.ymin;
      stack[stackpos].xmax = ldim.xmax;
      stack[stackpos].ymax = ldim.ymax;
   }
   else if(flag == 3)
   {  /* Zoom out */
      stackpos = stackpos > 0 ? stackpos - 1 : 0;
      zoom_status(stackpos);
      ldim.xmin = stack[stackpos].xmin;
      ldim.ymin = stack[stackpos].ymin;
      ldim.xmax = stack[stackpos].xmax;
      ldim.ymax = stack[stackpos].ymax;
   }

   ldim.width  = ldim.xmax - ldim.xmin;
   ldim.height = ldim.ymax - ldim.ymin;
   ldim.ratio  = ldim.height / ldim.width; 
   
   if (ldim.ratio > scr.ratio)
   {            
      scale    = (scr.height - MARGIN) / ldim.height;
      xoffset  = ((scr.width - MARGIN) / scale - ldim.width) / 2.0 - ldim.xmin;
      yoffset  = -ldim.ymin;
   }
   else
   {
      scale    = (scr.width - MARGIN) / ldim.width;
      xoffset  = -ldim.xmin;
      yoffset  = ((scr.height - MARGIN) / scale - ldim.height) / 2.0 - ldim.ymin;
   }
   
   xoffset += (MARGIN / 2.0) / scale;
   yoffset += (MARGIN / 2.0) / scale;
   
   return(stackpos+1);
}

void x_finish()
{
  redraw(0, 0, (int)scr.width, (int)scr.height);
}

void x_color(num)
     int num;
{
  XSetForeground(dpy, gc, cols[num % MAXCOLORS]);
}

void x_line(x1, y1, x2, y2)
     double x1, y1, x2, y2;
{   XGCValues val;
   val.line_width = 2;
   XChangeGC(dpy,gc,GCLineWidth,&val);
  if (clip_line(&x1, &y1, &x2, &y2, &ldim))
    XDrawLine(dpy, pixmap, gc, X(x1), Y(y1), X(x2), Y(y2));
}

void x_polyline(x, y, n)
     double x[], y[];
     int    n;
{
  int    i;
  XPoint points[MAXPOLY];
  
  if (clip_polygon(x, y, &n, &ldim))
    {
      for (i = 0; i < n; i++)
        {
	  points[i].x = X(x[i]);
	  points[i].y = Y(y[i]);
        }
      XDrawLines(dpy, pixmap, gc, points, n, CoordModeOrigin);
    }
}

void x_polyfill(x, y, n)
     double x[], y[];
     int    n;
{
  int    i;
  XPoint points[MAXPOLY];
  
  if (clip_polygon(x, y, &n, &ldim))
    {
      for (i = 0; i < n; i++)
        {
	  points[i].x = X(x[i]);
	  points[i].y = Y(y[i]);
        }
      XFillPolygon(dpy, pixmap, gc, points, n, Convex, CoordModeOrigin);
    }
}

void setNewFont(Display *dis,GC gc, double height)
{
# define NFONTS 5
   Font fonttab[NFONTS];
   XGCValues val;
   int id;
   static char *fontlist[NFONTS] ={
	"lucidasanstypewriter-10",
	"lucidasanstypewriter-12",
	"lucidasanstypewriter-14",
	"lucidasanstypewriter-18",
	"lucidasanstypewriter-24"
   };

   /* fprintf(stderr,"Font height : %f\n",height); */
   
   if(height < 0.3)
     id = 0;
   else if (height < 0.4)
     id = 1;
   else if (height < 0.5)
     id = 2;
   else if (height < 0.6)
     id = 3;
   else
     id = 4;
   
   fonttab[id] = XLoadFont(dis,fontlist[id]);
   val.font = fonttab[id];
   XChangeGC(dis,gc,GCFont,&val);
}

void x_text(x, y, height, angle, string)
     double x, y, height, angle;
     char   *string;
{
   if(angle == 0.0)
   {
      /* This routine ensures we can also draw (horizontal) text at the
       * boundaries, which is not done if the string is on top or on the
       * right and the insertion point lies on the boundary, due to the way
       * the clipping routines work.
       */
      setNewFont(dpy, gc, height);
      XDrawString(dpy, pixmap, gc, X(x), Y(y), string, strlen(string));
   }
   else
     line_text(x, y, height, angle, string);
}

void x_symbol(x, y, num)
     double x, y;
     int    num;
{
  line_symbol(x, y, num);
}

void redraw(x, y, width, height)
{
  XCopyArea(dpy, pixmap, win, gc, x, y, width, height, x, y);
}

static void paninout(double x, double y, double w, double h, int shift)
{
   double xl, yl;
   
   if (ABS(h) < 3 && ABS(w) < 3) /* PAN */
   {
      /* Shift: Changed panning to place area picked 
       in the middle of the window */
      xl = (shift?-1:1)*((double)( x - scr.width / 2) / scale);
      yl = (shift?-1:1)*((double)(scr.height / 2 - y) / scale);
      
      if (shift)
      {
	 ldim.xmin -= xl; ldim.ymin -= yl; 
	 ldim.xmax -= xl; ldim.ymax -= yl;
      }
      else
      {
	 if (ABS(xl) > ABS(yl))
	 {
	    ldim.xmin -= xl; 
	    ldim.xmax -= xl;
	 }
	 else
	 {
	    ldim.ymin -= yl; 
	    ldim.ymax -= yl;
	 }
	 x_init(&ldim,0);
      }
   }
   else if (h < 0) 
   {
      ldim.xmin = (double)(             x) / scale - xoffset;
      ldim.ymin = (double)(scr.height - y) / scale - yoffset;
      ldim.xmax = ldim.xmin + (double)w / scale;
      ldim.ymax = ldim.ymin - (double)h / scale;
      
      x_init(&ldim,2); /* ZOOM in */
   }
   else                        
     x_init(&ldim,3); /* ZOOM out */
   
   sepplot();
}

static void drawrectangle(x, y, w, h)
     double x, y, w, h;
{
  static int draw=False;
  
  if (w < 0.0)
    x += w, w = -w;
  if (h < 0.0)
    y += h, h = -h;
  
  draw = ~draw;
  if (draw) 
    XDrawRectangle(dpy, win, gcxor, (int)x, (int)y, (int)w, (int)h);
  else redraw((int)x-1, (int)y-1, (int)w+2, (int)h+2);
}

static void rubberband(w, tag, event)
     Widget w;
     int    *tag;
     XEvent *event;
{
  static double xl, yl, width, height;
  
  switch (event->type)
    {
    case ButtonPress:
      xl = (double)event->xbutton.x;
      yl = (double)event->xbutton.y;
      width = height = 0.0;
      drawrectangle(xl, yl, width, height);
      break;
    case MotionNotify:
      drawrectangle(xl, yl, width, height);
      width  = (double)(event->xbutton.x) - xl;
      height = (double)(event->xbutton.y) - yl;
      if (!(event->xbutton.state & ShiftMask)) {
	/* don't use scr.ratio if SHIFT is pressed 
	   use xbutton.button to know which one has motion */
	if (ABS(height) < scr.ratio * ABS(width))
	  height = SIGN(height) * scr.ratio * ABS(width);
	else
	  width  = SIGN(width) * ABS(height) / scr.ratio;
      }
      drawrectangle(xl, yl, width, height);
      break;
    case ButtonRelease:
      drawrectangle(xl, yl, width, height);
      paninout(xl, yl, width, height, event->xbutton.state & ShiftMask);
      break;
    }
}

static void initX()
{
  static int first = TRUE;
  static XGCValues gcv;
  static int screen;
  Window root;
  int    dum;
  unsigned int width, height;
  XColor    c;
  Colormap  cmap;
  int       i;
  unsigned long free_pixels[MAXCOLORS];
  int num_free=0;
  
   if (first)
   {
      unsigned long backpix;
      first = FALSE;
      
      XtVaGetValues(graphics,
		    /*XmNforeground, &gcv.foreground,*/
		    XmNbackground, &gcv.background, NULL);
      backpix=gcv.background; /* save it */
      gcv.background=0x1; /* white */
      gcv.foreground=0x0; /* black */
      gcv.function = GXcopy;
      gcv.line_style = LineDoubleDash;
      
      gcxor  = XtGetGC(graphics, GCForeground | GCBackground | 
		       GCFunction | GCLineStyle, &gcv);
      gcv.background=backpix; /* restore to original */
      
      dpy    = XtDisplay(graphics);
      win    = XtWindow(graphics);
      screen = DefaultScreen(dpy);
      planes = DisplayPlanes(dpy, screen);
      
      XtAddEventHandler(graphics, ButtonPressMask | ButtonMotionMask | 
			ButtonReleaseMask, FALSE, 
			(XtEventHandler) rubberband, NULL);
   }
  
   XGetGeometry(dpy, win, &root, &dum, &dum, &width, &height,
		(unsigned *) &dum,(unsigned *) &dum);
   scr.xmin  = 0.0;
   scr.ymin  = 0.0;
   scr.xmax  = scr.width  = (double)width;
   scr.ymax  = scr.height = (double)height;
   scr.ratio = scr.height / scr.width; 

   if (gc)
   {
      XFreeGC(dpy, gc);
      XFreePixmap(dpy, pixmap);
   }
  pixmap = XCreatePixmap(dpy, win, width, height, planes);
  gc     = XCreateGC(dpy, pixmap, 0, NULL);
  
  cmap   = DefaultColormap(dpy, screen);
  
  for (i=0; i< MAXCOLORS; i++) {
    if (use_color(i)) {
      if (!cols[i]) {
	c.red   = colortable[i].red;
	c.green = colortable[i].green;
	c.blue  = colortable[i].blue;
	
	cols[i] = XAllocColor(dpy, cmap, &c) ? 
	  c.pixel : BlackPixel(dpy, screen);
      }
    }
    else if (cols[i]) {
      free_pixels[num_free++]=cols[i];
      cols[i]=0;
    }
  }
  if (num_free) XFreeColors(dpy, cmap, free_pixels, num_free, 0);
  

  XSetForeground(dpy, gc, gcv.background);
  XFillRectangle(dpy, pixmap, gc, 0, 0, width, height);
  XSetForeground(dpy, gc, cols[1]); 
}


#define MAXCOL  256

int R[MAXCOLORS], G[MAXCOLORS], B[MAXCOLORS];

/***********************************************************************
 *                                                                     *
 * Name: ImgFreeImage                                Date:    07.01.93 *
 * Author: E.Chernyaev (IHEP/Protvino)               Revised:          *
 *                                                                     *
 * Function: Free image                                                *
 *                                                                     *
 * Input: theImg - pointer to the image                                *
 *                                                                     *
 ***********************************************************************/
void ImgFreeImage(theImg)
     XImage *theImg;
{
  if (theImg) {
    if (theImg->data) free(theImg->data);
    theImg->data = NULL;
    XDestroyImage(theImg);
  }
}

/***********************************************************************
 *                                                                     *
 * Name: ImgPickImage                                Date:    22.02.93 *
 * Author: E.Chernyaev (IHEP/Protvino)               Revised:          *
 * Adapted by Dirk De Wachter for usage in sepview            26.06.95 *
 *                                                                     *
 * Function: Pick image                                                *
 *                                                                     *
 * Output: width  - image width pointer                                 *
 *         height - image height pointer                                *
 *                                                                     *
 * Return: pointer to the image                                        *
 *                                                                     *
 ***********************************************************************/
XImage* ImgPickImage(width, height)
                int  *width, *height;
{
  XImage *theImg;
  
  *width=(int)scr.width;
  *height=(int)scr.height;
  theImg = XGetImage(dpy, pixmap, 0, 0, *width, *height, AllPlanes, ZPixmap);
  if (!theImg || !theImg->data) {
    ImgFreeImage(theImg); theImg = NULL;
  }
  return theImg;
}

/***********************************************************************
 *                                                                     *
 * Name: ImgPickPalette                              Date:    22.02.93 *
 * Author: E.Chernyaev (IHEP/Protvino)               Revised:          *
 *                                                                     *
 * Function: Pick palette for image                                    *
 *                                                                     *
 * Input: theDsp - pointer to display structure                        *
 *        window - window                                              *
 *                                                                     *
 * Output: Ncol  - number of colors                                    *
 *         R[]   - red components                                      *
 *         G[]   - green components                                    *
 *         B[]   - blue components                                     *
 *                                                                     *
 ***********************************************************************/
void ImgPickPalette(Ncol, R, G, B)
     int     *Ncol, *R, *G, *B;
{
  int               i, theScr, nmaps, ncolors;
  Window            theRoot, theWin;
  Colormap          cmap, *cmaps;
  XColor            colors[MAXCOL];
  XWindowAttributes theAtt;
  
  theScr  = DefaultScreen(dpy);
  theRoot = RootWindow(dpy, theScr);
  theWin  = XtWindow(graphics);
   
  if (theWin == (Window)NULL ) theWin  = theRoot;

  XGetWindowAttributes(dpy, theWin, &theAtt);

  if (theAtt.colormap && theAtt.map_installed) {
    ncolors = theAtt.visual->map_entries;
    cmap    = theAtt.colormap;
  }else{
    ncolors = DisplayCells(dpy, theScr);
    cmap    = DefaultColormap(dpy, theScr);
    cmaps   = XListInstalledColormaps(dpy, theRoot, &nmaps);
    if (nmaps != 0) {
      for (i=0; i<nmaps; i++) {
        if (cmap == cmaps[i]) break;
        if (i == nmaps-1) cmap = cmaps[0];
      }
    }
    XFree((void *) cmaps);
  }

  for (i=0; i<ncolors; i++) {
    colors[i].pixel = i;
    colors[i].pad   = 0;
  }
  XQueryColors(dpy, cmap, colors, ncolors);

  *Ncol   = ncolors;
  for (i=0; i<ncolors; i++) {
    R[i] = colors[i].red;
    G[i] = colors[i].green;
    B[i] = colors[i].blue;
  }
}
