      subroutine petsccreamats ( nrusol, nnz, onnz, nrusolglob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-07-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create space for matrix for petsc usage
!     Both serial and parallel case
! **********************************************************************
!
!                       KEYWORDS
!
!     petsc
!     stiffness_matrix
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use seppetscmod   ! defines all petsc stuff
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nrusol, nnz(*), onnz(nrusol), &
                             nrusolglob

!     nnz            i    Contains number of nonzero elements per row
!                         of the matrix
!                         In case of serial computation accumulated
!     nrusol         i    Number of free unknowns
!     nrusolglob     i    Global number of unknowns
!     onnz           i    Contains number of non-zeros per row stored
!                         on other cpu's (in sepran sense)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      integer, allocatable :: iwork(:)
      integer :: dummy_integer=0

!     i              General loop variable
!     iwork          work array to store nnz not accumulated in case
!                    of serial computation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MATCREATEAIJ   Creates a sparse parallel matrix in AIJ format
!     MATCREATESEQAIJ  Create matrix, i.e. make space for this matrix
!                      and set up petsc structure
!                      Compact row storage for sequential matrices
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'petsccreamats' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from petsccreamats'
         write(irefwr,1) 'nrusol, iseqmat', nrusol, iseqmat
         write(irefwr,1) 'nrusolglob', nrusolglob
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      
!     --- Create matrix

      if ( parallel ) then

!     --- Parallel case

         if ( debug ) then
            call prinin ( nnz, nrusol, 'nnz' )
            call prinin ( onnz, nrusol, 'onnz' )
         end if  ! ( debug )

         call MatCreateAIJ ( PETSC_COMM_WORLD, nrusol, nrusol, &
                             nrusolglob, nrusolglob, 0, nnz, 0, onnz, &
                             A(iseqmat), ierrpetsc )

         if (ierrpetsc.ne.0) then
             write(*,*) "matcreate failed with error", ierrpetsc
             stop "houdoe"
         endif

      else

!     --- Serial case

         allocate ( iwork(nrusol), stat = error )
         if ( error/=0 ) call eralloc ( error, nrusol, 'iwork' )
         if ( ierror/=0 ) go to 1000

         do i = 1, nrusol
            iwork(i) = nnz(i+1)-nnz(i)
         end do  ! i = 1, nrusol

         if ( debug ) call prinin ( iwork, nrusol, 'nnz' )

         call MatCreate(PETSC_COMM_WORLD, A(iseqmat),ierrpetsc)

         if (ierrpetsc.ne.0) then
             write(*,*) "matcreate failed with error", ierrpetsc
             stop "houdoe"
         endif

         call MatSetSizes(A(iseqmat), PETSC_DECIDE, PETSC_DECIDE, &
                          nrusol, nrusol,ierrpetsc)
         if (ierrpetsc.ne.0) then
             write(*,*) "matset sizes failed with error", ierrpetsc
             stop "houdoe"
         endif

         call MatSetFromOptions(A(iseqmat),ierrpetsc)                    
         if (ierrpetsc.ne.0) then
             write(*,*) "matsetfromoptions failed with error", ierrpetsc
             stop "houdoe"
         endif

         call MatSeqAIJSetPreallocation ( A(iseqmat), dummy_integer, iwork, ierrpetsc )
         if (ierrpetsc.ne.0) then
             write(*,*) "matprealloc failed with error", ierrpetsc
             stop "houdoe"
         endif

         deallocate ( iwork, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iwork' )

      end if  ! ( parallel )

1000  call erclos ( 'petsccreamats' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End petsccreamats'

      end if  ! ( debug )

      end subroutine petsccreamats
