      subroutine topqu0obst ( ibuffr, buffer, kmesh, iskip, iskpar, &
                              iresul, ilevel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 15-11-2012 Long integers
!        version  1.1  date 16-11-2001 Jump if no elements are found
!        version  1.0  date 20-07-2001 Copied from topqu0
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check if a mesh must be subdivided into sub elements
!    If necessary perform the subdivision and construct an output sol. array
!    The following tasks are performed:
!    If there are volume elements check whether a surface grid or a volume
!    grid is needed
!    Construct either a volume grid or construct a grid consisting of the
!    outer surface only
!    If required make a new grid consisting of the vertices of this
!    grid only
!    If required subdivide the original grid or the surface grid into
!    triangular or tetrahedral elements based on the
!    presence of the required degree of freedom.
!    Furthermore store the required degree of freedom in an output vector
!    Finally (if required, i.e. ifill=1) fill array ihelp. In ihelp it is
!    stored which new elements (elements ihelp(i)+1 up to ihelp(i+1)
!    originated from the old i'th element group. If properties for group i
!    have been given, for example the porosity, these properties can be used
!    for the new elements utilizing array ihelp
!    Remark: both output meshes consist of one type of elements only!
!    Besides that some arrays are checked (KMESH, KPROB, ISOL)
!    and some arrays are activated (COOR, new grids, new solution array, &
!    old solution array, KPROB parts f/h/p), depending on ichois
!    Warning:  subroutine TOPQU0 uses memory management subroutines
!              This means that all pointers must be recomputed after
!              this subroutine
!    If more types of elements must be used, subroutine PL0013 must be
!    incorporated
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cbuffrf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), ibuffr(*), &
              iresul(5,*), iskip, iskpar(*), ilevel
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     ilevel         i   Defines the level of elements to be plotted if
!                        kmesh(59)>0
!     iresul         o   Output array of length 5,1 if ichois=2 and otherwise of
!                        length 5,3
!                        The array is filled as follows:
!                        Pos .,1 corresponds to the "vertex" mesh array
!                                (ichois=2,3)
!                        Pos .,2 corresponds to the "triangle" mesh array
!                                (ichois=1,3)
!                        Pos .,3 corresponds to the solution array (ichois=1,3)
!                        Pos. 1,1  INDVER  sequence number with respect to
!                                          INFOR for the "vertex" mesh array
!                        Pos. 2,1  NPELM   number of points per element for
!                                          this mesh
!                        Pos. 3,1  NELEM   number of points per element for
!                                          this mesh
!                        Pos. 4,1  NEWVER  indicates if the mesh is new (1) or
!                                          is in fact the standard mesh (0)
!                        Pos. 5,1  INDLIN  sequence number with respect to INFOR
!                                          for the "line-elements" array
!     iskip          i   Indicates if element groups must be skipped (>0) or
!                        not (0)
!     iskpar         i   array of length NELGRP that is only used if ISKIP>0
!                        It indicates if element group ielgrp must be skipped
!                        iskpar(ielgrp) = 0 or not iskpar(ielgrp) = 1
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical volume, subtri, subver, vertex
      integer (kind=8) nbufol, ipelemobst, ipkelm, ipkmsc, iphelp, &
                       ipielemobst, iptria, nbufl1, ipsubm, ipline, &
                       ipelemskip, ipitriacolour, iptricol
      integer ndim, npelm, npoint, nelem, i, length, nelext, &
              ntria, neleml, kelmad, mmelemobst

!     i              Counting variable
!     ipelemobst     Starting address of array elemobst in IBUFFR
!     ipelemskip     Starting address of array elemskip in IBUFFR
!     iphelp         Starting address of array "HELP" in array IBUFFR
!     ipielemobst    Starting address of array ielemobst in IBUFFR
!     ipitriacolour  Starting address of array itriacolour in IBUFFR
!     ipkelm         Starting address of array "KELEM" in array IBUFFR
!     ipkmsc         Starting address of array KMESH part c into array IBUFFR
!     ipline         Starting address of array "line" in array IBUFFR
!     ipsubm         Starting address of array KMESHC corresponding to submesh
!                    in array IBUFFR
!     iptria         Starting address of array TRIA into array IBUFFR
!     iptricol       Starting address of array tricol into array IBUFFR
!     kelmad         memory management reference number of array KMESH part ad
!     length         Help parameter to indicate the length of arrays
!     mmelemobst     Memory management address corresponding to array elemobst
!     nbufl1         help variable to store the old value of nbuffr
!     nbufol         help variable to store the old value of nbuffr
!     ndim           Dimension of the space.
!     nelem          Number of elements
!     neleml         Number of "line"-elements
!     nelext         Number of boundary elements
!     npelm          Number of points per element
!     npoint         Number of nodal points
!     ntria          number of triangles or quadrilaterals after removal of
!                    double ones
!     subtri         Indicates if the mesh must be subdivided into triangles and
!                    also the solution copied (true) or not
!     subver         Indicates if a "vertex" mesh must be created (true) or not
!     vertex         indicates if only the vertices of the surface elements
!                    have to be filled (true) or all points (false)
!     volume         Indication if the mesh contains volume elements (true) or
!                    not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) iniget, inidbl

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI000         Copy integer array to other integer array possibly itself
!     INI060         Reactivate existing array in IBUFFR
!     INI062         reserve space for temporary arrays
!     INI066         delete temporary array
!     INI069         Reserve space at the end of IBUFFR for a semi-permanent
!                    array
!     INI081         Create permanent space in array IBUFFR
!     INIDBL         Transform pointer for IBUFFR to pointer for BUFFER
!     INIGET         Compute pointer of array in IBUFFR
!     INSTOP         Stop the program
!     MSHFILLELOBS   Fill array elemeobst for each element with information of
!                    elements in relation to the obstacle
!     TOPQU7         Compute length of array "KELEM", check if mesh contains
!                    volumes
!     TOPQU8OBST     Split volumes with obstacles in triangles
!     TOPQU9OBST     Remove double elements in volumes with obstacles
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      75   Array KMESH has not been filled
!     419   NDIM # 2
!    1382   Value of NPELM is not yet implemented for this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Perform some standard checks
!   Clear array IRESUL
!   Activate standard arrays
!   Check if the mesh contains volume elements
!   If ( volume elements are available and ichois<6 ) then
!      Create mesh of surface elements only
!      If ( vertex mesh must be created ) then
!         Create vertex mesh
!      If ( triangle mesh must be created ) then
!         Create triangle mesh
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topqu0obst' )

!     --- Check arrays KMESH, KPROB, ISOL and compute IPKPRB

      nbufol = nbuffr
      subtri = .false.
      subver = .true.
      vertex = subver .and. .not. subtri

      if ( kmesh(2)/=100 ) then
         call errint ( kmesh(2), 1 )
         call errsub ( 75, 1, 0, 0 )
      end if
      ndim = kmesh(6)
      if ( ndim<3 .or. ndim>3 ) then
         call errint ( ndim, 1 )
         call errint ( 3, 2 )
         call errint ( 3, 3 )
         call errsub ( 419, 3, 0, 0 )
         go to 1000
      end if
      npelm  = kmesh(4)
      npoint = kmesh(8)
      nelem  = kmesh(9)

!     --- Clear array IRESUL

      do i = 1, 5
         iresul(i,1) = 0
      end do

!     --- Activate standard arrays

      call ini060 ( ibuffr, -kmesh(17) )
      if ( ierror/=0 ) go to 1000

!     --- In the case of 3D it is checked whether the grid contains
!         volume elements

      call topqu7 ( kmesh, length, volume, vertex, neleml )
      if ( ierror/=0 ) go to 1000

!     --- Create array elemobst with information of the status of
!         all elements with respect to the obstacle

      kelmad = kmesh(52)
      call ini060 ( ibuffr, kmesh(kelmad+1) )
      mmelemobst = 0
      call ini081 ( ibuffr, mmelemobst, kmesh(9), 'elemobst' )
      ipielemobst = iniget(kmesh(kelmad+1))
      ipelemobst = iniget(mmelemobst)
      call mshfillelobs ( kmesh, ibuffr(ipelemobst), &
                         ibuffr(ipielemobst) )

!     --- Volume elements found, construct a mesh consisting of surface
!         elements only
!         In this case double triangles or quadrilaterals are removed
!         reserve space for array "KELEM"
!         this array is made semi-permanent

      call ini069 ( ibuffr, ipkelm, length, nbufol )
      call ini069 ( ibuffr, ipitriacolour, length/3, nbufl1 )
      if ( ierror/=0 ) go to 1000

!     --- make reservation for "line-elements" array

      if ( neleml/=0 ) &
         call ini081 ( ibuffr, iresul(5,1), 2*neleml, 'lines' )

      if ( kmesh(59)/=0 ) then

!     --- kmesh(59)#0, i.e. elements must be skipped

         call ini060 ( ibuffr, kmesh(59) )
         ipelemskip = iniget(kmesh(59))

      else

!     --- kmesh(59)=0, i.e. no elements must be skipped

         ipelemskip = 1

      end if
      if ( neleml/=0 ) then
           ipline = iniget(iresul(5,1))
      else
           ipline = 1
      end if
      if ( ierror/=0 ) go to 1000
      ipkmsc = iniget(kmesh(17))
      ipelemobst = iniget(mmelemobst)
      call topqu8obst ( kmesh, ibuffr(ipkmsc), ibuffr(ipkelm), nelem, &
                        npelm, nelext, vertex, ibuffr(ipline), neleml, &
                        iskip, iskpar, ibuffr(ipelemskip), ilevel, &
                        ibuffr(ipitriacolour), ibuffr(ipelemobst) )

!     --- remove double elements

      length = nelem*intlen + (nelem+nelext)*(npelm+1)
      if ( length==0 ) go to 900

      call ini062 ( ibuffr, length, iphelp )
      iptria = iphelp + nelem*intlen
      iptricol = iptria + (nelem+nelext)*npelm
      iphelp = inidbl(iphelp)
      if ( ierror/=0 ) go to 1000
      call topqu9obst ( ibuffr(ipkelm), nelem, npelm, ibuffr(iptria), &
                        ntria, npoint, buffer(iphelp), &
                        ibuffr(ipitriacolour), ibuffr(iptricol) )
      if ( ierror/=0 ) go to 1000

!     --- Copy array "itriangle" into "KELEM"

      call ini000 ( ibuffr(iptria), ibuffr(ipkelm), 1, 1, &
                    ntria*npelm, 0 )

!     --- If necessary copy boundary elements into "KELEM"

      if ( nelext>0 ) then
         call ini000 ( ibuffr(ipkelm+npelm*(nelem-nelext)), &
                       ibuffr(ipkelm+npelm*ntria), 1, &
                       1, nelext*npelm, 0 )
      end if

!     --- Copy array "itriaout" into "itriacolour"

      call ini000 ( ibuffr(iptricol), ibuffr(ipitriacolour), 1, 1, &
                    ntria, 0 )

!     --- If necessary copy boundary elements into "itriacolour"

      if ( nelext>0 ) then
         call ini000 ( ibuffr(ipitriacolour+nelem-nelext), &
                       ibuffr(ipitriacolour+ntria), 1, &
                       1, nelext, 0 )
         ntria = ntria + nelext
      end if

      nelem = ntria

!     --- Create a vertex mesh

      if ( npelm>4 ) then

!     --- surface mesh must be adapted, not yet implemented

         call errint ( npelm, 1 )
         call errsub ( 1382, 1, 0, 0 )

      else

!     --- Use surface mesh

         call ini081 ( ibuffr, iresul(1,1), nelem*npelm, &
                       'kmeshc_sub' )
         ipsubm = iniget(iresul(1,1))
         if ( ierror/=0 ) go to 1000
         call ini000 ( ibuffr(ipkelm), ibuffr(ipsubm), 1, 1, &
                       nelem*npelm, 0 )
         iresul(2,1) = npelm
         call ini081 ( ibuffr, iresul(5,1), nelem, &
                       'itriancolour' )
         ipsubm = iniget(iresul(5,1))
         if ( ierror/=0 ) go to 1000
         call ini000 ( ibuffr(ipitriacolour), ibuffr(ipsubm), 1, 1, &
                       nelem, 0 )

      end if

 900  iresul(3,1) = nelem
      iresul(4,1) = 1

1000  call erclos ( 'topqu0obst' )
      if ( ierror/=0 ) call instop
      call ini066 ( ibuffr, mmelemobst )

      nbuffr = nbufol

      end
