      subroutine topqu8 ( kmesh, kmeshc, ntria, npelnw, vertex, iskip, &
                          iskpar, iskipel, elemskip, ilevel, nelem, &
                          nmesha, nfaces, nmeshg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 11-09-2013 New parameter list
!        version  5.1  date 07-02-2005 Debug statements
!        version  5.0  date 17-08-2004 Extra parameters
!
!   copyright (c) 1987-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all outer triangles or quadrilaterals of a three-dimensional mesh
!     in array KMESHC
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     element
!     surface
!     volume
!     face
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kmeshc(*), ntria, npelnw, iskip, iskpar(*), &
              iskipel, elemskip(*), ilevel, nelem, nmesha(*), nfaces, &
              nmeshg(*)
      logical vertex

!     elemskip       i    Is only used if iskipel#0
!                         elemskip(ielem) = 0 means do not skip element
!                         elemskip(ielem) = 1 means skip element
!     ilevel         i    Defines the level of elements to be plotted if
!                         kmesh(59)>0
!     iskip          i    Indicates if element groups must be skipped (>0)
!                         or not (0)
!                         If <0 volumes must be skipped
!     iskipel        i    if 0 no elements must be skipped, if 1 array elemskip
!                         is used to detect which elements must be skipped
!     iskpar         i    array of length NELGRP that is only used if ISKIP>0
!                         It indicates if element group ielgrp must be skipped
!                         iskpar(ielgrp) = 0 or not iskpar(ielgrp) = 1
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshc         o    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nelem          i    Number of elements
!     nfaces         i    Number of faces
!     nmesha         i    nmesh part a
!     nmeshg         i    nmesh part g
!     npelnw         o    Number of points per element in KELEM
!     ntria          o    number of triangles or quadrilaterals after removal of
!                         double ones
!     vertex         i    indicates if only the vertices of the surface elements
!                         have to be filled (true) or all points (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, i, ielgrp, inelem, nelgrp, nelemskip, nouterfaces, &
              lpnmpf, nvertx, ishapevolm, inpelm, ishapesur, kelmg, &
              melpef, kelmaa, error
      integer, allocatable :: iskipelem(:), nmeshf(:), outerfaces(:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     inelem         Number of elements in a specific element group
!     inpelm         number of points per element
!     ishapesur      shape number of surface element
!     ishapevolm     shape number of volume element
!     iskipelem      Work array of size nelemskip containing the element
!                    numbers that must be skipped
!     kelmaa         Starting position of KMESH part AA in array KMESH
!     kelmg          Indicates if KMESH part g has been filled (>0)
!                    or not (0)
!                    If kelmg>0, kelmg is the starting address of KMESH
!                    part g in KMESH
!     lpnmpf         length of array nmeshf
!     melpef         maximum number of elements connected with one face
!     nelemskip      Number of elements that must be skipped
!     nelgrp         Number of element groups
!     nmeshf         work array of size 4*nfaces to store nmesh part f
!     nouterfaces    Number of outer faces (length of outerfaces)
!     nvertx         Number of vertices
!     outerfaces     Work array of size nfaces containing the face numbers
!                    of the outer faces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inishapesurvol, ininodeshape

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     ININODESHAPE   Returns with number of nodes in element as function of
!                    the shape number
!     INISHAPESURVOL Returns with shape number of faces of volume element
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
!     TOPELMFACES    Fill array kmeshc with surface elements corresponding to
!                    the faces stored in array outerfaces
!     TOPELMFACESSD  Fill array kmeshc with surface elements corresponding to
!                    the faces stored in array outerfaces
!                    In this case not all faces have the same number of nodes
!                    and as a consequence all quadrilaterals are subdivided
!                    into two triangles
!     TOPELMFACESVT  Fill array kmeshc with surface elements corresponding to
!                    the vertices of thefaces stored in array outerfaces
!     TOPELMSPFACES  Store all vertices of the spectral surface elements stored
!                    in the outer surfaces into an array kmeshc consisting of
!                    quadrileaterals
!     TOPFA05SKIP    creates nmesh part f, i,e. volume elements connected to
!                    faces
!                    In this case elements marked in iskipelem are skipped
!     TOPOUTERFAC    Find all outer faces of a volume by storing only those
!                    faces corresponding to one element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2421   No outer faces
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if elements must be skipped then
!        Fill array iskipelem
!     Find all volume elements that must not be skipped corresponding to faces
!     Store all face numbers that correspond to the "outer" boundary
!     Check if all these faces have the same number of nodes
!     Create array "kelem"
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topqu8' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topqu8'
         write(irefwr,*) 'Node numbers corresponding to faces'
         kelmaa = kmesh(48)-1
         call prinnmeshx ( kmesh(kelmaa+5), nmeshg )
         write(irefwr,100) 'iskip, iskipel', iskip, iskipel
100      format ( a, 1x, (10i6) )

      end if

      allocate ( iskipelem(nelem), nmeshf(4*nfaces), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nelem+4*nfaces, 'iskipelem' )
      if ( ierror/=0 ) go to 1000

      iskipelem = 0

!     --- if elements must be skipped then
!            Fill array iskipelem

      nelgrp = kmesh(5)
      inelem = nelem
      kelmg  = kmesh(21)
      nelemskip = 0

      if ( iskip>0 ) then

!     --- element groups must be skipped

         ielem  = 0
         do ielgrp = 1, nelgrp

            if ( nelgrp>1 ) inelem = kmesh(kelmg+ielgrp-1)
            if ( iskpar(ielgrp)==0 ) then

!           --- Skip this group

               do i = 1, inelem
                  iskipelem(nelemskip+i) = ielem+i
               end do
               nelemskip = nelemskip+inelem

            end if
            ielem = ielem+inelem

         end do

      end if

      if ( iskipel>0 ) then

!     --- iskipel>0, only elements with level ilevel must be used

         do ielem = 1, nelem

            if ( elemskip(ielem)/=ilevel ) then

!           --- Element must be skipped

               nelemskip = nelemskip+1
               iskipelem(nelemskip) = ielem

            end if

         end do

      end if

!     --- Find all volume elements that must not be skipped
!         corresponding to faces

      lpnmpf = 4*nfaces
      call topfa05skip ( nmesha, nmeshf, nfaces, melpef, lpnmpf, &
                         nelem, kmesh, iskipelem, nelemskip, &
                         ishapevolm )
      if ( debug ) then

!     --- Debug information

         write(irefwr,100) 'ishapevolm', ishapevolm
         write(irefwr,*) 'elements corresponding to faces'
         call prinnmeshx ( nfaces, nmeshf )

      end if

      deallocate ( iskipelem, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iskipelem' )
      if ( ierror/=0 ) go to 1000

!     --- Store all face numbers that correspond to the "outer" boundary

      allocate ( outerfaces(nfaces), stat = error )
      if ( error/=0 ) call eralloc ( error, nfaces, 'outerfaces' )
      if ( ierror/=0 ) go to 1000

      call topouterfac ( nmeshf, nfaces, nouterfaces, outerfaces )
      if ( debug ) call prinin ( outerfaces, nouterfaces, 'outerfaces' )

      deallocate ( nmeshf, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nmeshf' )

      if ( nouterfaces==0 ) then

!     --- internal error: no outer faces found

         call errsub ( 2421, 0, 0, 0 )
         go to 1000

      end if
      if ( ierror/=0 ) go to 1000

!     --- Create array kmeshc

      ishapesur = inishapesurvol ( ishapevolm )

      if ( debug ) then
         write(irefwr,*) 'ishapesur, vertex ', ishapesur, vertex
         write(irefwr,100) 'kmesh(38)', kmesh(38)
      end if

      if ( ierror/=0 ) goto 1000
      ntria = nouterfaces
      if ( vertex .and. ishapesur>3 .and. ishapesur/=5 .and. &
           ishapesur<=8 ) then

!     --- only the vertices must be stored

         if ( ishapesur==6 ) then

!        --- quadrilaterals

             nvertx = 4

         else

!        --- triangles

             nvertx = 3

         end if
         call topelmfacesvt ( nouterfaces, outerfaces, kmeshc, &
                              nmeshg, nvertx, ishapesur )
         npelnw = nvertx

      else if ( kmesh(38)>0 .and. ishapesur==5 ) then

!     --- Spectral elements, special treatment

         call topelmspfaces ( nouterfaces, outerfaces, kmeshc, &
                              nmeshg )
         inpelm = 4
         npelnw = inpelm

      else if ( ishapesur>0 ) then

!     --- All nodes must be stored

         inpelm = ininodeshape ( ishapesur )
         if ( ishapesur>8 ) inpelm = inpelm-1
         call topelmfaces ( nouterfaces, outerfaces, kmeshc, inpelm, &
                            nmeshg )
         npelnw = inpelm

      else

!     --- ishapesur = 0, volume consists of prisms, i.e. triangles and
!         quadrilaterals for the faces
!         All faces are subdivided into triangles, ntria must be adapted

         inpelm = 3
         call topelmfacessd ( nouterfaces, outerfaces, kmeshc, inpelm, &
                              nmeshg, ntria )
         npelnw = inpelm

      end if

      deallocate ( outerfaces, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'outerfaces' )

1000  call erclos ( 'topqu8' )
      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshc, ntria, inpelm, 'kmeshc' )
         write(irefwr,*) 'End topqu8'

      end if

      end
